<?php
/******************************************************************************
 *
 *  LOCAL STORE
 *
 *  local version of Config-Store
 *
 *****************************************************************************/

class LocalStore
{
  //---------------------------------------------------------------------------
  //  PRIVATE VARIABLES
  //
  private $m_configStore;
  private $m_devTree;
  private $m_rootEntityId = null;
  private $buildNummer = 0;

  //---------------------------------------------------------------------------
  //  CONSTRUCT
  //
  public function __construct(  )
  {
    $this->m_configStore = array( );
    $string = file_get_contents( DATA_PATH . "confstore/devtree.json" );
    $this->m_devTree = json_decode( $string, true );
    $this->_LoadChildren( $this->m_devTree['devtree'] );
    printf( "Loaded %d entities.\r\n", $this->Size( ) );
  }

  //---------------------------------------------------------------------------
  //  HELPERS
  //

  protected function _LoadChildren( $branch )
  {
    foreach( $branch as $entityId => $children )  {
      if( !( $this->m_rootEntityId ) )  {
        $this->m_rootEntityId = $entityId;
      }

      $this->_LoadEntity( $entityId );
      if( key_exists( 'children', $children ) && is_array( $children['children'] ) )  {
        $this->_LoadChildren( $children['children'] );
      }
    }
  }

  protected function _LoadEntity( $entityId )
  {

    if( file_exists( DATA_PATH . "confstore/" . $entityId . ".json" ) )  {
      $string = file_get_contents( DATA_PATH . "confstore/" . $entityId . ".json" );
      $entity = json_decode( $string, true );
      $this->m_configStore[ $entityId ] = $entity;

      if(  $this->buildNummer == 0 
        && $entity['class'] == "system" 
        && key_exists( 'adminUi', $entity ) 
        && key_exists( 'build', $entity['adminUi'] ) 
        )  {
        $this->buildNummer = intval( $entity['adminUi']['build'] );
      }
    }
    else  {
      printf( "NOT FOUND: entity( '%s' )\r\n", $entityId );
    }

  }

  //---------------------------------------------------------------------------
  //  METHODS
  //

  public function Size()
  {
    if( is_array( $this->m_configStore ) )  {
      return count( $this->m_configStore );
    }
    return -1;
  }

  public function Get( $entityId )
  {
    if( key_exists( $entityId, $this->m_configStore ) )  {
      return $this->m_configStore[ $entityId ];
    }
    return null;
  }

  public function GetRootId()
  {
    return $this->m_rootEntityId;
  }

  public function GetEntityIdAt( $index )
  {
    $keyList = array_keys( $this->m_configStore );
    return $keyList[ $index ];
  }

  public function GetEntityIdList()
  {
    return array_keys( $this->m_configStore );    
  }

  public function Set( $entityId, $entity )
  {
    if( key_exists( $entityId, $this->m_configStore ) )  {
      $this->m_configStore[ $entityId ] = $entity;
      $jsonString = json_encode( $entity, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE );
      file_put_contents( DATA_PATH . "confstore/" . $entityId . ".json",  $jsonString );
      return true;
    }
    return false;
  }

  public function Add( $entity, $parentEntityId )
  {
    // must have fields in entity
    if( ! key_exists( 'entid', $entity ) || ! key_exists( 'class', $entity ) ) {
      return false;
    }
    // parent has to exist
    if( ! key_exists( $parentEntityId, $this->m_configStore ) )  {
      return false;
    }
    // if entity already exists -> just update
    if( key_exists( $entity['entid'], $this->m_configStore ) )  {
      return $this->Set( $entity['entid'], $entity );
    }

    // set parent (just in case)
    $entityId = $entity['entid'];
    $entity['parent'] = $parentEntityId;

    // now add the entity into branch
    $entry = Array( 
      'class' => $entity['class'],
      'parent' => $parentEntityId,
      'name' => isset( $entity['name'] ) ? $entity['name'] : ""
    );
    $this->_AddEntityInTree( $parentEntityId, $entityId, $entry );
    // save devtree
    $jsonString = json_encode( $this->m_devTree, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE );
    file_put_contents( DATA_PATH . "confstore/devtree.json" ,  $jsonString );  

    // save entity itself
    $this->m_configStore[ $entityId ] = $entity;
    $jsonString = json_encode( $entity, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE );
    file_put_contents( DATA_PATH . "confstore/" . $entityId . ".json",  $jsonString );

    return true;    
  }

  public function HasPassword( $entityId  )
  {
    if( key_exists( $entityId, $this->m_configStore ) )  {
      return file_exists( DATA_PATH . "confstore/" . $entityId . ".pass.json" );
    }
    return false;
  }

  public function GetBuild( $entityId )
  {
    if( key_exists( $entityId, $this->m_configStore )
     && key_exists( 'adminUi', $this->m_configStore[ $entityId ] )
     && key_exists( 'build', $this->m_configStore[ $entityId ]['adminUi'] )
      ) {
      return intval(  $this->m_configStore[ $entityId ]['adminUi']['build'] );
    }

    // default: we're using b-nr of 'system.fj'
    return $this->buildNummer;
  }

  public function GetEntityClass( $entityId )
  {
    if( key_exists( $entityId, $this->m_configStore ) )  {
      return $this->m_configStore[ $entityId ]['class'];
    }
    return "";
  }

  public function AddPermissionRecord( $entity, $permission, $filter )
  {
    if( key_exists( 'permission', $entity ) )  {
        
      $permissionHolders = Array( );
      $isInstalled  = false;

      for( $i = 0; $i < count( $entity['permission'] ); $i++ )  {
        $holder = $entity['permission'][ $i ]['w'];
        $permissionHolders[ $holder ] = true;
        if( $entity['permission'][ $i ]['f'] == $filter )  {
          $isInstalled = true;
          break;
        }
      }
        
      if( !$isInstalled )  {
        foreach( $permissionHolders AS $holder => $value )  {
          array_push( $entity['permission'], Array(
            'p' => $permission,
            'w' => $holder,
            'f' => $filter
          ) );
        }
      }

      return $entity;
    }
    
  }

  public function DeleteEntity( $entityId )
  {
    if( key_exists( $entityId, $this->m_configStore ) ) {
      unset( $this->m_configStore[ $entityId ] );
      $fileName = DATA_PATH . "confstore/" . $entityId . ".json";
      if( file_exists( $fileName ) )  {
        unlink( $fileName );
      }
    }

    if( $this->_DeleteEntityFromBranch( $this->m_devTree['devtree'], $entityId ) )  {
      $jsonString = json_encode( $this->m_devTree, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE );
      file_put_contents( DATA_PATH . "confstore/devtree.json" ,  $jsonString );  
    }

  }

  protected function _DeleteEntityFromBranch( &$branch, $deleteEntityId )
  {
    foreach( $branch as $entityId => &$children )  {

      if( $entityId == $deleteEntityId )  {
        unset( $branch[ $entityId ] );
        return true;
      }

      $this->_LoadEntity( $entityId );
      if( key_exists( 'children', $children ) && is_array( $children['children'] ) 
       && $this->_DeleteEntityFromBranch( $children['children'], $deleteEntityId ) 
        ) {
        if( count( $children['children'] ) == 0 )  {
          $children['children'] = null;
        }
        return true;
      }
    }

    return false;
  }

  protected function _AddEntityInTree( $parentEntityId, $entityId, $entry )
  {
    return $this->_AddEntityInTreeRecurse( $this->m_devTree['devtree'][ $this->m_rootEntityId ]['children'], $parentEntityId, $entityId, $entry );
  }

  protected function _AddEntityInTreeRecurse( &$branch, $parentEntityId, $newEntityId, $entry )
  {
    foreach( $branch as $entityId => &$children )  {

      if( $entityId == $parentEntityId )  {
        if( ! key_exists( 'children', $children ) ) {
          $children['children'] = Array();
        }
        $children['children'][ $newEntityId ] = $entry;
        return true;
      }

      if( is_array( $children ) 
       && key_exists( 'children', $children ) 
       && is_array( $children['children'] ) 
       && $this->_AddEntityInTreeRecurse( $children['children'], $parentEntityId, $newEntityId, $entry ) 
        ) {
        return true;
      }
    }

    return false;
  }

};
?>