<?php
//-------------------------------------------------------
//  REQUIERMENTS
//-------------------------------------------------------
if( PHP_OS == "Linux" )  {
  define( "PIPE_COSCONF_ADDRESS", 'unix:///tmp/cosconf');
}
else { // Windows not supported
  LOG::error( __CLASS__, "(" . __LINE__ . ")CosConf doesn't support windows!" );
  die();
}
/****************************************************************
 * NamedPipe
 *
 * Communication between access5 server core (C++) and PHP
 *
 */
class CosConfPipe
{

  //-------------------------------------------------------
  //  MEMBER VARIABLES
  //-------------------------------------------------------
  static private $m_namedPipeHandle;
  static private $m_waitForReadTime = 60; //seconds
  static private $m_namedPipeAddress = PIPE_COSCONF_ADDRESS;

  //-------------------------------------------------------
  //  MEMBER FUNCTIONS
  //-------------------------------------------------------


  /*************************************************************
   * Send( $envelope )
   * @param envelope $envelope contains an envelope-json-string
   */
  static public function Send( $envelope )
  {
    if( !self::$m_namedPipeHandle )  {
      if( !self::Initialize() )  {
        LOG::error( __CLASS__, "(%s) can't initialise namedpipe handle!!!" , __LINE__ );
        LOG::error( __CLASS__, "(%s) get last error:(%s)" , __LINE__, error_get_last()['message'] );
        return false;
      }
    };
    
    // decode envelope for some necessary informations
    $envelope = json_decode( $envelope, true );
    $command  = $envelope['meta']['command'];
    if( !self::Write($envelope) )  {
      LOG::error( __CLASS__, "(%s) write envelope to namedpipe failed!!!", __LINE__ );
      return false;
    };
    if( !$responsedEnvelope = self::Read() )  {
      LOG::error( __CLASS__, "(%s) read envelope form namedpipe failed!!!", __LINE__ );
      return false;
    };
    
    return $responsedEnvelope;
  }


  /*************************************************************
   * IsServerRunning( )
   * @param envelope $envelope contains an envelope-json-string
   */
  public static function  IsServerRunning()
  {
    $namedPipe = self::Initialize();
    $return    = $namedPipe ? true : false;
    return $return ;
  }



  /*************************************************************
   * Read( )
   * @return envelope  return contains an evelope-json-string
   *
   */
  static private function Read()
  {
    // wait until time is over or namedPipe status is changed
    $waitToRead = Array( self::$m_namedPipeHandle );
    $null       = null;   // dummy parameter to make stream_select happy

    $readyToRead = stream_select( $waitToRead, $null, $null, self::$m_waitForReadTime );
    if( $readyToRead === 0 )  {
      // time is over and nothing is happend
      // try in few ms again
      $tries = 0;
      while( $tries >= 10 )  {
        $tries++;
        usleep(10);
        $readyToRead = stream_select( $waitToRead, $null, $null, self::$m_waitForReadTime );
        if( $readyToRead === 0 )  {
          return true;
        }
      }
      //echo __CLASS__ . ":" . __LINE__ . " time is over and nothing is happend | \$readyToRead " . var_export( $readyToRead, true ) . "<br>\n";
      return false;
    }
    elseif ( !$readyToRead ) {
      // something went worg
      //echo __CLASS__ . ":" . __LINE__ . " something went worg | \$readyToRead " . var_export( $readyToRead, true ) . "<br>\n";
      return false;
    }


    $envelope = fgets( self::$m_namedPipeHandle );
    
    //print_r( $envelope );
    //rewind( self::$m_namedPipeHandle );
    // strip leading \0
    if( $envelope[0] == "\0" )  {
      $envelope = substr( $envelope, 1 );
    }

    $envelopeArray = json_decode( $envelope, true );
    //LOG::debug( __CLASS__,"(%s) envelope from server: \"command: %s\" | status: %s | data: %s", __LINE__, $envelopeArray['meta']['command'], $envelopeArray['meta']['status'], var_export( (isset($envelopeArray['data'])?$envelopeArray['data']:false == true), true ) );
    if( !isset($envelopeArray['meta'],$envelopeArray['meta']['command']) )  {
      LOG::debug(__CLASS__, "no command in response: %s", var_export( $envelope, true ));
    }
    return $envelope;
  }

  /*************************************************************
   * Initialize( )
   *
   */
  static private function Initialize()
  {
    $tries     = 0;
    $connected = false;
    if( self::isConnected() )  {
      return self::$m_namedPipeHandle;
    }
    while( $tries < 10 && !self::isConnected() ) {
      try{
        if(self::Connect()) {
          $connected = true;
        };
        $tries++;
        LOG::debug(__CLASS__,"Pipe connecting try: %s", $tries);
        usleep(10);

      }
      catch( Exception $e ){
        LOG::error(__CLASS__,"open pipe failed!! try: %s", $tries+1);
        LOG::error( __CLASS__, "(%s) get last error:(%s)" , __LINE__, error_get_last()['message'] );
      }
    }

    if( !self::$m_namedPipeHandle ) return false;
    //$this->$m_namedPipeHandle = @fopen( m_namedPipeAddress, "r+" );
    //return $this->np_handle;
    LOG::debug(__CLASS__,"Pipe connect succeeds with %s tries", $tries);
    return self::$m_namedPipeHandle;
  }

  /*************************************************************
   * Terminate( )
   *
   */
  static private function Terminate()
  {

  }

  /*************************************************************
   * Write( $envelope )
   * @param envelope $envelope contains an envelope-json-string
   */
  static private function Write( $envelope )
  {
    $envelope = json_encode( $envelope );
    $encoding = mb_detect_encoding( $envelope, 'UTF-8' );
    //LOG::debug(__CLASS__,"envlope character encoding: %s", $encoding);
    $waitToWrite    = Array( self::$m_namedPipeHandle );
    $readyToWrite   = stream_select( $null, $waitToWrite, $null, self::$m_waitForReadTime );
    if( $readyToWrite === 0 ){

    }
    if( !$readyToWrite )  {
      LOG::debug(__CLASS__,"something went wrong on write to the pipe");
    }
    $envelopeArray  =  json_decode( $envelope, true );
    //if($envelope){print_r($envelope)."\n<br>";};
    //LOG::debug( __CLASS__, "(%s) envelope send to server: \"command: %s\" ", __LINE__, $envelopeArray['meta']['command'] );
    if( strpos($envelopeArray["meta"]["command"], "SESSION") === false )  {
       //LOG::debug( __CLASS__, "(%s) cmd: %s | fingerprint: %s ", __LINE__, var_export( $envelopeArray["meta"]["command"], true ) , var_export( $envelopeArray["meta"]["fingerprint"], true ) );
    }
   

    
    return fwrite( self::$m_namedPipeHandle, trim( $envelope ) ."\n\0" );
  }



  //-------------------------------------------------------
  //  LOW LEVEL MEMBER FUNCTIONS
  //-------------------------------------------------------

  static private function Connect( )
  {
    $tries = 0;
    if( PHP_OS == "Linux" )  {
      self::$m_namedPipeHandle = stream_socket_client(PIPE_COSCONF_ADDRESS, $errno, $errstr);
    }
    else  {
      self::$m_namedPipeHandle = @fopen( PIPE_COSCONF_ADDRESS, "r+" );
    }
    //self::$m_namedPipeHandle = @fopen( self::m_namedPipeAddress, "r+" );
    if( self::$m_namedPipeHandle )  {
      return true;
    }
    while($tries <= 10)  {
      $tries++;

    if( PHP_OS == "Linux" )  {
      self::$m_namedPipeHandle = stream_socket_client(PIPE_COSCONF_ADDRESS, $errno, $errstr);
    }
    else  {
      self::$m_namedPipeHandle =  @fopen( PIPE_COSCONF_ADDRESS, "r+" );
    }
    //self::$m_namedPipeHandle = @fopen( self::m_namedPipeAddress, "r+" );
    if( self::$m_namedPipeHandle )  {
      return true;
    }
      usleep(10);
    }
    //echo " connect pipe failed!!! " . __CLASS__  . ":" . __LINE__ . "<br> \n";
    return false;
  }

  static private function Disconnect( )
  {
    if( self::$m_namedPipeHandle === false )  return false;
    return fclose( self::$m_namedPipeHandle );
  }
  /*************************************************************
   * isConnected()
   *
   */
  static private function isConnected()
  {
    if( self::$m_namedPipeHandle )  {
      return true;
    }
    return false;
  }

  /*************************************************************
   * GetSecurityToken()
   *
   * @return string SecurityToken
   *
   */
  static public function GetSecurityToken()
  {
    $reponse = self::$m_securityToken;
    self::$m_securityToken = null;
    return $reponse;
  }

  /*************************************************************
   * SetFingerPrint( $fingerprint )
   *
   * @return string SecurityToken
   *
   */
  static public function SetFingerPrint( $fingerprint )
  {
    NamedPipe::$m_fingerprint = $fingerprint;
  }

}
?>
