<?php
/********************************************************************
 *    SESSION
 */


class SESSION
{
  /*-------------------------------------------------------
   *  CONST
   */
  const DSSESSION = "\\Admcache\\session\\";
  const TIMEOUT   = 900;    // timeout = 15 min = 900

  /*-------------------------------------------------------
   *  STATIC MEMBERS
   */
  static $status    = '';
  static $pipe      = null;
  static $sessInfo  = null;
  static $session   = null;

  /*-------------------------------------------------------
   *  GET SESSION
   */
  static function Get( )
  {
    // 1. check first browser session
    // 2. if browser session is not set try to get php stored session

    if( isset( $_COOKIE['session'] ) ) {
      SESSION::$session == $_COOKIE['session'];
      return $_COOKIE['session'];
    }
    elseif ( isset( SESSION::$session ) ) {
      return SESSION::$session;
    }
    LOG::debug( __CLASS__, "no session available!" );
    return null;
  }

  /*-------------------------------------------------------
   *  START NEW SESSION OR USE EXISTING
   */
  static function Start( $pubkey = false )
  {
    //init token & fingerprint

    if( !NamedPipe::IsServerRunning() )  {
      //server not running ==> delete session
      SESSION::UnSetCookie( 'session' );
      SESSION::UnSetCookie( 'user' );
      SESSION::UnSetCookie( 'connected' );
      SESSION::$status = "denied";

      LOG::debug( __CLASS__, "server is not running => denied" );
      return false;
    }
    else  {
      SESSION::SetCookie( 'connected', "true", time() + SESSION::TIMEOUT  );
    }

    // check browser session
    if( !isset($_COOKIE['session']) )  {
      LOG::debug( __CLASS__, "[---] No Browser Cookie( agent=%s )", isset($_SERVER['HTTP_USER_AGENT'])?substr($_SERVER['HTTP_USER_AGENT'],0,20):"no user agent" );
    }

    $sessInfo = isset($_COOKIE['session']) ? SESSION::ReadSessionFromServer( $_COOKIE['session'] ) : null;
    LOG::debug(__CLASS__, "(%s) set sessinfo => %s", __LINE__, var_export($sessInfo,true));

    // --- SESSION COOKIE NOT SET ==> New session ---
    if( !isset($_COOKIE['session']) || !is_array($sessInfo) || !key_exists('createdTime', $sessInfo) )  {

      $time = time();
      $nonce = getGUID();
      $session = SESSION::CreateSessionId( $time, null, $nonce );

      // 1. calculate session-id based on remote ip and browser info
      LOG::debug( __CLASS__, "[%s] New Session( tm=%d, agent=%s )", $session, $time, isset($_SERVER['HTTP_USER_AGENT'])?substr($_SERVER['HTTP_USER_AGENT'],0,20):"no user agent", var_export($sessInfo,true) );

      // 2. set a cookie to transport the session to browser
      //setcookie( 'session', $session, time()+SESSION::TIMEOUT );
      SESSION::SetCookie( 'session', $session, time()+SESSION::TIMEOUT );
      LOG::debug(__CLASS__,"(%s) new session set user cookie to expired",__LINE__);
      //setcookie( 'user', null, -1 );
      SESSION::UnSetCookie( 'user' );
      SESSION::$status = "created";
      $sessInfo = Array( 'createdTime' => $time, 'activity' => time(), 'ip' => SESSION::GetIpAddress(), 'type' => "admin", 'nonce' => $nonce );

      // 3. create public/private key
      if( $pubkey )  {
        $sw = microtime(true);
        $res = openssl_pkey_new( Array( 'private_key_bits' => 1024 ) );
        if( $res )  {

          openssl_pkey_export( $res, $privkey );        // Get private key
          $pubkey = openssl_pkey_get_details( $res );   // Get public key
          $pubkey = $pubkey["key"];

          $sw = microtime(true) - $sw;

          $sessInfo['privkey'] = $privkey;
          $sessInfo['pubkey']  = $pubkey;
          setrawcookie( 'pubkey', rawurlencode($pubkey) );   // it's bug in setcookie, spaces are encoded wrong.
        }
        else  { /* error */ }
      }

      // 4. write a session info to admService
      SESSION::$sessInfo = $sessInfo;
      SESSION::$session  = $session;
      SESSION::CreateSessionToServer( $session, $sessInfo );

      //SESSION::TempLog( "-------- SESSION::Start End -----------" );
      return true;
    }
    // --- Use existing session (session must be set in server) ---
    else  {

      // 1. calculate session-id based on remote ip and browser info
      $session = SESSION::CreateSessionId( $sessInfo['createdTime'], null, $sessInfo['nonce'] );

      // 2a. if calculated session matches cookie, ok
      LOG::debug( __CLASS__,"( %s ) cookie %s == %s session", __LINE__, isset($_COOKIE['session']) ? var_export($_COOKIE['session'], true) : "no cookie" , isset($session)?var_export($session,true):"no session" );
      if( $_COOKIE['session'] == $session )  {

        // if session is expired, create a new one
        if( SESSION::IsExpired( $sessInfo ) )  {

          LOG::debug( __CLASS__, "[%s] Expired( agent=%s, info=%s )", $session, isset($_SERVER['HTTP_USER_AGENT'])?substr($_SERVER['HTTP_USER_AGENT'],0,20):"no user agent", var_export($sessInfo,true) );
          // create new session
          $time = time();
          $nonce = getGUID();
          $session = SESSION::CreateSessionId( $time, null, $nonce );
          $sessInfo['createdTime'] = $time;
        }

        SESSION::$status = "ok";
        $sessInfo['activity'] = time();
        //setcookie( 'session', $session, time()+SESSION::TIMEOUT );
        SESSION::SetCookie( 'session', $session, time()+SESSION::TIMEOUT );

        if( isset($sessInfo['user']) )  {
          //setcookie( 'user', $sessInfo['user'], time()+SESSION::TIMEOUT );
          SESSION::SetCookie( 'user', $sessInfo['user'], time()+SESSION::TIMEOUT );
        }
        else  {
          //setcookie( 'user', null, -1 );
          SESSION::UnSetCookie( 'user' );
        }

        SESSION::$sessInfo = $sessInfo;
        SESSION::$session  = $session;
        SESSION::WriteSessionToServer( $session, $sessInfo );
        //SESSION::TempLog( "-------- SESSION::Start End -----------" );
        return true;
      }
      // 2b. if calculated session doesn't matches cookie, clear session
      else  {

        LOG::debug( __CLASS__, "[%s] Mismatch, clear( agent=%s, info=%s )",$session, isset($_SERVER['HTTP_USER_AGENT'])?substr($_SERVER['HTTP_USER_AGENT'],0,20):"no user agent", var_export($sessInfo,true) );
        LOG::debug( __CLASS__,"(%s) %s != %s", __LINE__,  $_COOKIE['session'], $session  );
        //setcookie( 'session', null, -1 );
        //setcookie( 'user', null, -1 );

        SESSION::UnSetCookie( 'session' );
        SESSION::UnSetCookie( 'user' );

        SESSION::$status = "denied";
        SESSION::$sessInfo = null;
        SESSION::$session  = null;
        //SESSION::TempLog( "-------- SESSION::Start End -----------" );
        return false;

      }
    }

  }

  /*-------------------------------------------------------
   *  DE-AUTHORIZE SESSION
   */
  static function DeAuthorize()
  {
    //SESSION::TempLog( "-------- SESSION::DeAuthorize -----------" );
    LOG::debug( __CLASS__, "[%s] De-Authorize()", SESSION::$session );
    SESSION::UnSetCookie( "user" );


    $sessionProxy             = new SessionProxy( SESSION::Get() );
    $sessionUnauthorizeResult = $sessionProxy->Unautorize( SESSION::Get() );
    LOG::debug( __CLASS__,"(%s)[result] sessionUnauthorizeResult: %s ",__LINE__ , var_export( $sessionUnauthorizeResult, true ) );

  }

  /*-------------------------------------------------------
   *  IS SESSION AUTH?
   */
  static function IsAuthorized( )
  {
    //SESSION::TempLog( "-------- SESSION::IsAuthorize -----------" );
    $isAuthorised = isset( SESSION::$sessInfo['user'] );
    if( $isAuthorised ){
      //SESSION::TempLog( "Session is authorized" );
    }
    else  {
      //SESSION::TempLog( "Session is not authorized!!!" );
    }
    //SESSION::TempLog( "-------- SESSION::IsAuthorize End -----------" );
    return $isAuthorised;
  }

  /*-------------------------------------------------------
   *  AUTHORIZE SESSION
   */
  static function Authorize( $user, $phrase )
  {
    //SESSION::TempLog( "-------- SESSION::Authorize -----------" );
    //SESSION::$pipe = $pipe;

    // --- server not running ==> delete session ---
    if( /*$pipe && !$pipe->IsServerRunning()*/ !NamedPipe::IsServerRunning() )  {
      LOG::debug( __CLASS__, "[%s] Authorize, Server not running( user=%s )", SESSION::$session, $user );

      SESSION::UnSetCookie( 'session' );
      SESSION::UnSetCookie( 'user' );

      SESSION::$status = "denied";
      //SESSION::TempLog( "-------- SESSION::Authorize End -----------" );
      return false;
    }
    $sessionID = SESSION::Get();
    $sessInfo = isset($sessionID) ? SESSION::ReadSessionFromServer( SESSION::Get() ) : null;

    if( is_array(SESSION::$sessInfo) && !empty($user) && isset(SESSION::$sessInfo['createdTime']) )  {

      // --- for authorize cookie must be set ---
      SESSION::SetCookie( 'session', SESSION::Get(), time()+SESSION::TIMEOUT );
      SESSION::SetCookie( 'user', $user, time()+SESSION::TIMEOUT );

      $sessInfo['user']  = $user;

      SESSION::$sessInfo = $sessInfo;
      SESSION::$session  = SESSION::Get();


      //new A5 code
      $sessionProxy = new SessionProxy( SESSION::$session );
      $sessionAuthorizeResult = $sessionProxy->Authorize( SESSION::$session, $user, $phrase);
      $sessInfo = isset( $_COOKIE['session']) ? SESSION::ReadSessionFromServer( $_COOKIE['session'] ) : null;

      if( $sessionAuthorizeResult['data'] && $sessionAuthorizeResult['meta']['status'] != "ERROR") {
        LOG::debug( __CLASS__, "[%s] Authorize, OK( user=%s, info=%s )", SESSION::$session, $user, var_export( SESSION::$sessInfo, true ) );
        return true;
      }

      LOG::debug( __CLASS__, "[%s] Authorize, Error( user=%s, info=%s )", SESSION::$session, $user, var_export( SESSION::$sessInfo, true ) );
      SESSION::UnSetCookie( 'user' );
      return false;
    }
    else  {
      LOG::debug(__CLASS__,"(%s) no session information (sessinfo) found => return false | authorize failed => excecute SESSION::Start(\$pipe)",__LINE__);
      return SESSION::Start( /*SESSION::$pipe*/ true );
    }
  }


  /*-------------------------------------------------------
   *  AUTHORIZE SESSION
   */
  static function CreateSessionId( $time, $user, $nonce )
  {
    //SESSION::TempLog( "-------- SESSION::CreateSessionId -----------" );
    $session  = isset( $_SERVER['HTTP_USER_AGENT'] ) ? $_SERVER['HTTP_USER_AGENT'] : "Keep this string secret!";
    $session .= SESSION::GetIpAddress();
    $session .= (string)$time;
    $session .= $nonce;
    // if session is associated with an user, integrate it
    if( !empty($user) )  {
      $session .= $user;
    }
    $session = "admin" . md5( $session );

    LOG::debug( __CLASS__, "[%s] CalculateID( tm=%d, agent='%s' )", $session, $time, isset($_SERVER['HTTP_USER_AGENT'])?substr($_SERVER['HTTP_USER_AGENT'],0,20):"no user agent" );
    //SESSION::TempLog( "-------- SESSION::CreateSessionId End -----------" );
    return $session;
  }

  /*-------------------------------------------------------
   *  DE-AUTHORIZE SESSION
   */
  static function Stop( $pipe )
  {
    /*SESSION::$pipe = $pipe;*/
    //SESSION::TempLog( "-------- SESSION::Stop -----------" );
    LOG::debug( __CLASS__, "[%s] Stop()", SESSION::Get() );

    // --- for stop cookie must be set ---
    if( isset( $_COOKIE['session'] ) )  {
      SESSION::UnSetCookie( 'session' );
      SESSION::UnSetCookie( 'user' );
      SESSION::$status = "";
      SESSION::DeAuthorize( /*$pipe*/ true );
    }
    //SESSION::TempLog( "-------- SESSION::Stop End -----------" );
  }


  /*-------------------------------------------------------
   *  CHECK IF SESSION HAS BEEN EXPIRED
   */
  static function IsExpired( $sessInfo=null )
  {
    //SESSION::TempLog( "-------- SESSION::IsExpired -----------" );
    $session = SESSION::Get();
    $sessionProxy = new SessionProxy( $session );
    $sessionResetExpiryResult = $sessionProxy->ResetExpiry( $session );
    LOG::debug(__CLASS__,"(%s) resetExpiry Response:  " . var_export( $sessionResetExpiryResult, true ),__LINE__);
    if( $sessionResetExpiryResult['meta']['status'] == "ERROR"  ) {
      LOG::debug( __CLASS__, "[%s] Expired( ['meta']['status'] => ERROR )", SESSION::Get() );
      //SESSION::TempLog( "-------- SESSION::IsExpired End-----------" );
      return true;
    }
    else if( isset($sessionResetExpiryResult['data']) && $sessionResetExpiryResult['data'] == true ) {
      LOG::debug( __CLASS__, "[%s] Session valid (updated)", SESSION::Get() );
      //SESSION::TempLog( "-------- SESSION::IsExpired End-----------" );
      return false;
    }
    else  {
      LOG::debug( __CLASS__, "[%s] Expired( ['data']['expired'] => true )", SESSION::Get() );
      //SESSION::TempLog( "-------- SESSION::IsExpired -----------" );
      return true;
    }
    //SESSION::TempLog( "-------- SESSION::IsExpired End-----------" );
    return true;

  }


  /*-------------------------------------------------------
   *  WRITE SESSION TO SERVER
   */
  static function CreateSessionToServer( $sessionId, $sessInfo )
  {
    $sessionProxy = new SessionProxy( $sessionId );
    $sessionCreateResult = $sessionProxy->Create( $sessionId, $sessInfo['createdTime'], $sessInfo['ip'], $_SERVER['HTTP_USER_AGENT'], true, 900, $sessInfo['nonce'] );
    
    LOG::debug( __CLASS__,"(%s)[result] sessionCreateResult: %s ",__LINE__ , var_export( $sessionCreateResult, true ) );
    
    return true;
  }


  /*-------------------------------------------------------
   *  WRITE SESSION TO SERVER
   */
  static function WriteSessionToServer( $sessionId, $sessInfo, $data = null )
  {
    //SESSION::TempLog( "-------- SESSION::WriteSessionToServer -----------" );
    $sessionProxy        = new SessionProxy( $sessionId );
    $sessionCreateResult = $sessionProxy->ResetExpiry( $sessionId );
    //LOG::debug( __CLASS__, "[%s] Session Create Result: %s", SESSION::$session, var_export($result,true) );
    if( isset( $sessionCreateResult['data'], $sessionCreateResult['data'] ) )  {
      //SESSION::TempLog( "-------- SESSION::WriteSessionToServer End-----------" );
      return $sessionCreateResult['data'];
    }
    //SESSION::TempLog( "-------- SESSION::WriteSessionToServer End-----------" );
    return false;
  }


  static function generateCallTrace()
  {
    $e = new Exception();
    $trace = explode("\n", $e->getTraceAsString());
    // reverse array to make steps line up chronologically
    $trace = array_reverse($trace);
    array_shift($trace); // remove {main}
    array_pop($trace); // remove call to this method
    $length = count($trace);
    $result = array();

    for( $i = 0; $i < $length; $i++ )  {
      $result[] = ($i + 1)  . ')' . substr($trace[$i], strpos($trace[$i], ' ')); // replace '#someNum' with '$i)', set the right ordering
    }

    return "\t" . implode("\n\t", $result);
  }

  /*-------------------------------------------------------
   *  READ SESSION from SERVER
   */
  static function ReadSessionFromServer( $sessionId )
  {
    //SESSION::TempLog( "-------- SESSION::ReadSessionFromServer -----------" );
    $sessionProxy     =  new SessionProxy( $sessionId );
    $sessionGetResult = $sessionProxy->Get($sessionId);
    LOG::debug( __CLASS__,"(%s)[result] sessionGetResult: %s ",__LINE__ , var_export( $sessionGetResult, true ) );

    //var_dump( $sessionGetResult );

    //build sessInfo
    $sessInfo = Array();
    $sessInfo['createdTime']    = $sessionGetResult['data']['createdTime'] ?? "";
    $sessInfo['expiryTime']     = $sessionGetResult['data']['expiryTime']  ?? "";
    $sessInfo['nonce']          = $sessionGetResult['data']['nonce']       ?? "";
    $sessInfo['ip']             = $sessionGetResult['data']['ipAddr']      ?? "";
    $sessInfo['type']           = isset( $sessionGetResult['data']['accountType'] ) ? $sessionGetResult['data']['accountType'] : null;
    (isset( $sessionGetResult['data']['authorizedEntityId'] ) && $sessionGetResult['data']['authorizedEntityId']) ? $sessInfo['user'] = $sessionGetResult['data']['authorizedEntityId'] : false;
    SESSION::$sessInfo = $sessInfo;
    //LOG::debug( __CLASS__,"ReadSession: (%s) \n " . SESSION::generateCallTrace(), __LINE__ );
    if( is_null( $sessionGetResult ) )  {
      //SESSION::TempLog( "sessionGetResult is null!!!" );
    }else if( empty( $sessionGetResult ) )  {
      //SESSION::TempLog( "sessionGetResult is empty!!!" );
    }
    else  {
      //SESSION::TempLog( "session: " . $sessionId  );
    }
    //SESSION::TempLog( "-------- SESSION::ReadSessionFromServer End-----------" );
    return $sessInfo;
  }


  //-------------------------------------------------------------
  // HELPERS
  //-------------------------------------------------------------
  /**
   * CreateEnvelope( $cmd, $session, $data )
   */
  static function CreateEnvelope( $cmd, $session, $data, $filter=null )
  {
    if( !$filter ){
      $filter = "*";
    }
    $env = Array(
      "meta" => Array(
        "cmd"     => "SESSION." . $cmd,
        "param"   => $filter,
        "guid"    => getGUID(),
        "session" => $session
      ),
      "data" => $data
    );
    if( $data == null )  {
      unset($env['data']);
    }
    return Array($env);
  }

  /**
   * SendEnvelope( $cmd, $session, $data )
   */
  static function SendEnvelope( $env )
  {
    //if( /*!isset( SESSION::$pipe )*/ true ){
    //  LOG::error( __CLASS__, "pipe isn't set" );
    // //LOG::error(__CLASS__,"(%s) \n " . debug_print_backtrace(), __LINE__ );
    // return false;
    //}
    //$response = SESSION::$pipe->RequestRaw( json_encode( $env ) );
    //$result = json_decode( $response, true );
    //LOG::debug(__CLASS__,"(%s) Session env result:\n " . var_export( $result, true ), __LINE__ );
    //LOG::debug(__CLASS__,"(%s) Session env result:\ncmd: \t%s \nstatus:\t%s \ndata:\t%s", __LINE__, $result[0]['meta']['cmd'], $result[0]['meta']['status'], var_export( $result[0]['data'], true ) );
    //return $result;

  }

  /**
   * GetEntid()
   *
   * @return returns a entid which is linked with the session
   */
  static function GetEntid()
  {
    $response = SESSION::ReadSessionFromServer( SESSION::Get() );
    if( isset($response['user']) ) return $response['user'];
    return false;
  }

  /******************************************************************************************************
   * SetCookie( $name, $time )
   */
  static function SetCookie( $name, $value, $time )
  {
    //SESSION::TempLog( "\tset cookie:'$name'" );
    $options = [
      'expires' => $time,
      'samesite' => "Strict",
      'secure' => true
    ];
    setcookie( $name, $value, $options );
    //setcookie( $name, $value, $time );
  }

  /******************************************************************************************************
   * SetCookie( $name, $time )
   */
  static function UnSetCookie( $name )
  {
    //SESSION::TempLog( "\tunset cookie:'$name'" );
    setcookie( $name, null, -1 );
  }

  /*******************************************************************************************************
   * static function TempLog()
   *
   * test log function for admin logoff
   */

  static function TempLog( $text )
  {
    file_put_contents( AccessLib::GetLogPath() ."\\" . str_replace(":", "", SESSION::GetIpAddress()) . "_" .  str_replace( array("/",";"), " " ,substr( (isset( $_SERVER['HTTP_USER_AGENT'] )?$_SERVER['HTTP_USER_AGENT']:"noAgent"),0,30 ) ) . "_session.log", strftime( "%d.%m.%Y - %T" ) . ": " .  $text . "\n", FILE_APPEND );
  }

  /********************************************************************************************************
   * GetIpAddress()
   *
   * prevent localhost bug  convert ipv6 localhost to ipv4 localhost
   *
   */
  static function GetIpAddress()
  {
    if( $_SERVER['REMOTE_ADDR'] == "::1" )  {
      return "127.0.0.1";
    };
    return $_SERVER['REMOTE_ADDR'];
  }

/**********************************************************************************************************
 * SetSessionId( $sessionId )
 *
 * WARNING: this function should only used by siedle device interfaces
 */

  static function SetSessionId( $sessionId )
  {
    SESSION::$session = $sessionId;
  }

}

/**********************************************************************************************************
 * shutdown()
 *
 * log last parameters before script ends
 */
function shutdown()
{
  file_put_contents( AccessLib::GetLogPath() ."\\" . str_replace(":", "", SESSION::GetIpAddress() ) . "_" . str_replace( Array("/",";"), " " ,substr( (isset( $_SERVER['HTTP_USER_AGENT'] )?$_SERVER['HTTP_USER_AGENT']:"noAgent"),0,30 ) ) . "_session.log", strftime( "%d.%m.%Y - %T" ) . ": " .  "###### SESSION SCRIPT End ########" . "\n\n", FILE_APPEND );
}



//register_shutdown_function( "shutdown" );
?>