<?php
error_reporting( E_ALL );
/********************************************************************
 *    Encapsulates the XHR Requests from server  
 */
class XHREnvelope  
{
  //-------------------------------------------------------
  //  MEMBER VARIABLES
  //-------------------------------------------------------
  protected $data     = NULL;
  protected $request  = FALSE;
  protected $cmd      = NULL;
  
  private        $exceTime;
  private        $m_mailboxProxy;
  private static $m_token = NULL;
  private static $isTokenDelivered = false;
  
  //-------------------------------------------------------
  //  CONSTRUCT
  //-------------------------------------------------------
  function __construct( $arg, $status = NULL, $data = NULL ) 
  {
    $this->m_mailboxProxy = new MailboxProxy();
    // if the Envelope is created as a copy of 
    // another it's a response
    if( $arg instanceof XHREnvelope )  {
      $this->request = FALSE;
      if( $status )  {
        if( $data )  {
          $this->data = Array( 'meta' => Array( 'guid' => $arg->GetGuid(), 'status' => $status ),
                               'data' => $data 
                             );          
        }
        else  {
          $this->data = Array( 'meta' => Array( 'guid' => $arg->GetGuid(), 'status' => $status ) );
        }
      }
      else  {
        $this->data = Array( 'meta' => Array( 'guid' => $arg->GetGuid() ) );        
      } 
    }
    // else it's a request, we copy the given data 
    else  {
      if( isset( $arg['meta']['token'] ) )  {
        self::$m_token = $arg['meta']['token'];
        
      }
      if( isset( $arg['meta']['fingerprint'] ) )  {
        NamedPipe::SetFingerprint( $arg['meta']['fingerprint'] );
      }
      $this->request = TRUE;
      $this->data    = $arg;
      $this->cmd     = preg_split( '/[.]/', $this->GetCommand() );
    }
  }
  
  //-------------------------------------------------------
  //  MEMBER FUNCTIONS
  //-------------------------------------------------------
  
  /**
   *  checks if this is request
   */
  function IsRequest( )
  {
    return $this->request;
  }
  
  /**
   *  validation
   */
  function IsValid( )
  {
    // to be valid it need to have $this->data['meta']['guid']
    if( is_array( $this->data ) )  {
      if( array_key_exists( 'meta', $this->data ) )  {
        return array_key_exists( 'guid', $this->data['meta'] );
      }
    }
    return FALSE;
  }
  
  /**
   *  returns the whole envelope as array object
   */
  function Get( )
  {
    return $this->data;
  }
  
  /**
   *  gets the command of this request
   */
  function GetCommand( )
  {
    return $this->data['meta']['cmd'];
  }

  function GetCommandSubject( )
  {
    return $this->cmd[0];
  }

  function GetCommandAction( )
  {
    return $this->cmd[1];
  }

  /**
   * gets the guid of this request
   */  
  function GetGuid( )
  {
    return $this->data['meta']['guid'];
  }
  
  /**
   * gets command parameter for this request
   */
  function GetParam( )
  {
    if( array_key_exists( 'param', $this->data['meta'] ) )  {
      return $this->data['meta']['param'];
    }
    return "";
  }
  
  /**
   * gets lang parameter for this request
   */
  function GetLang( )
  {
    if( array_key_exists( 'lang', $this->data['meta'] ) )  {
      return $this->data['meta']['lang'];
    }
    return null;
  }
  
  /**
   * gets the data associated with this request
   */
  function GetData( )
  {
    if( array_key_exists( 'data', $this->data ) )  {
      return $this->data['data'];
    } 
    return FALSE;
  }
  
  /**
   * gets the data associated with this request
   */
  function GetSession( )
  {
    if( array_key_exists( 'session', $this->data['meta'] ) )  {
      return $this->data['meta']['session'];
    } 
    return FALSE;
  }


  /**
   * gets the data associated with this request
   */
  function GetStatus( )
  {
    if( array_key_exists( 'status', $this->data['meta'] ) )  {
      return $this->data['meta']['status'];
    } 
    return FALSE;
  }

  /**
   * gets the data associated with this request
   */
  function SetStatus( $status )
  {
    $this->data['meta']['status'] = $status;
    return FALSE;
  }
  
  /**
   * gets the encoded response string
   */
  function EncodeResponse( $respData, $status=NULL, $error = NULL )
  {
    $data = Array();
    if( $status )  {
        
      // encode basic response
      $meta = Array( 'cmd' => $this->GetCommand()
                   , 'guid' => $this->GetGuid()
                   , 'status' => $status 
                   , 'exceTime' => $this->exceTime
                   );
      // add session (if set)           
      $session = $this->GetSession();
      if( $session )  {
        $meta['session'] = $session;
        // add session mbox counter.
        $messageCount = $this->m_mailboxProxy->GetSize( SESSION::Get() );
        $mbox = $messageCount["data"];
        if( !empty( $mbox ) )  {
          $meta['mbox'] = $mbox;
        }
      }

      // if reponse data has been set, add it      
      if( $respData )  {
        $data = Array( 'meta' => $meta, 'data' => $respData );          
      }
      // simple status response, no data      
      else  {
        $data = Array( 'meta' => $meta );
        if( $error )  {
          $data['meta']['error'] = $error;
        }
      }
    }
    else if( $this->GetStatus() ) {
      $data = Array( 'meta' => Array( 'cmd' => $this->GetCommand(), 'guid' => $this->GetGuid(), 'status' => $this->GetStatus() ) );        
    }
    else  {
      $data = Array( 'meta' => Array( 'cmd' => $this->GetCommand(), 'guid' => $this->GetGuid() ) );        
    }
    
    if( $securityToken = NamedPipe::GetSecurityToken() )  {
      $data['meta']['securityToken'] = $securityToken;
    }

    else if( $securityToken = XHREnvelope::GetToken() )  {
      $data['meta']['securityToken'] = $securityToken;
    }

    $res = json_encode( $data, JSON_UNESCAPED_UNICODE );
    if( $res === false )  {
      //print_r( json_last_error_msg() );      
    }
    return $res;
  }

  /***
   * GetToken() 
   */
  static public function GetToken()
  {
    return self::$m_token;
  }

  /***
   * GetToken() 
   */
  static public function SetToken( $token )
  {
    self::$m_token = $token;
  }

  /**
   * 
   */
  public function SetExecTime( $time )
  {
    $this->exceTime = $time;
  }
}

/********************************************************************
 *    Encapsulates the XHR Requests from server  
 */
class XHREnvelopeList
{
  //-------------------------------------------------------
  //  MEMBER VARIABLES
  //-------------------------------------------------------
  protected $postData = Array();
  protected $envelopes = Array();
  
  //-------------------------------------------------------
  //  CONSTRUCT
  //-------------------------------------------------------
  function __construct( $rawData ) 
  {
    if( is_string( $rawData ) )  {
      $this->Parse( $rawData );
    }
  }
    
  //-------------------------------------------------------
  //  PARSE
  //-------------------------------------------------------
  function Parse( $rawData ) 
  {
    $this->postData = json_decode( $rawData, true );

    // loop all envelopes and create env-classes
    foreach( $this->postData as $post )  {
      $this->envelopes[] = new XHREnvelope( $post );
    }  

    return sizeof( $this->envelopes );
  }
    
  //-------------------------------------------------------
  //  Getter: sizeof
  //-------------------------------------------------------
  function Size( ) 
  {
    return sizeof( $this->envelopes );
  }
    
  //-------------------------------------------------------
  //  Getter: sizeof
  //-------------------------------------------------------
  function Get( $idx = NULL ) 
  {
    if( $idx == NULL )  {
      return $this->envelopes;
    }
    else if( array_key_exists( $idx, $this->envelopes ) )  {
      return $this->envelopes[ $idx ];
    }
    return FALSE;
  }
    
};



?>