<?php
/*-----------------------------------------------------------------------------
 *  UPDATE for G.46 (> 6544)
 *
 *  1. add new reboot flag to old ipdb entries
 *
 *---------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
 *  MAIN - FUNCTION
 */
function Update6881()
{
  global $localStore;
  $accessBuild = intval( ACCESS_BUILD );

  if( FORCE || $localStore->GetBuild( 'folder.confdev' ) <= 6881 )  {  
    printf( "Start Update #6881\r\n" );
    for( $i = 0; $i < $localStore->Size( ); $i++ )  {
      $entityId = $localStore->GetEntityIdAt( $i );
      if( $localStore->GetBuild( $entityId ) <= $accessBuild )  {
        if( preg_match_all( "/admin|folder|menu|system|admui|rule/i", $entityId ) == false ) {
          Update6881RepairReferences( $entityId );
        }
      }
    }
  }
  else {
    printf( "Skip Update #6881. Server version=%d\r\n",$accessBuild );
  }

}

/*-----------------------------------------------------------------------------
 *  UPDATE IPDB
 */

//-----------------------------------------------------------------------------
//  Update6881RepairReferences(): add new references for validation framework
//
function Update6881RepairReferences( $entityId, $entity=false )
{
  global $localStore;
  if( $entity === false )  {
    $entity = $localStore->Get( $entityId );
  }

  // create empty references if it's empty
  if( isset( $entity['references'] ) === false )  {
    $entity['references'] = Array();
  }

  // check if  deviceconfig or sp is set
  if(  isset( $entity['deviceConfig'] ) === false
    && isset( $entity['sp'] ) === false ) { return false; }

  if( isset( $entity['deviceConfig'] ) ) {
    foreach( $entity['deviceConfig'] as $deviceRecord ) {
      $result = AnalyseRecord( $deviceRecord, $entityId );
      if( isset( $result ) && is_array( $result ) && empty( $result ) === false ) {
        $entity['references'] = array_merge( $entity['references'], $result );
        $entity['references'] = array_unique( $entity['references'] );
      }
    }
  }

  if( isset( $entity['sp'] ) ) {
    foreach( $entity['sp'] as $spRecord ) {
      $result = AnalyseRecord( $spRecord, $entityId );
      if( isset( $result ) && is_array( $result ) && empty( $result ) === false ) {
        $entity['references'] = array_merge( $entity['references'], $result );
        $entity['references'] = array_unique( $entity['references'] );
      }
    }
  }

  if( isset( $entity['timeline']['action']['on'] ) ) {
    foreach( $entity['timeline']['action']['on'] as $spRecord ) {
      $result = AnalyseRecord( $spRecord, $entityId );
      if( isset( $result ) && is_array( $result ) && empty( $result ) === false ) {

        $entity['references'] = array_merge( $entity['references'], $result );
        $entity['references'] = array_unique( $entity['references'] );
      }
    }
  }

  if( isset( $entity['timeline']['action']['off'] ) ) {
    foreach( $entity['timeline']['action']['off'] as $spRecord ) {
      $result = AnalyseRecord( $spRecord, $entityId );
      if( isset( $result ) && is_array( $result ) && empty( $result ) === false ) {
        $entity['references'] = array_merge( $entity['references'], $result );
        $entity['references'] = array_unique( $entity['references'] );
      }
    }
  }

  if( isset( $entity['cdrule'] ) ) {
    $entities = GetEnityIdsFromCdRule( $entity['cdrule'] );
    if( isset( $cdRules ) && is_array( $cdRules ) && empty( $cdRules ) === false ) {
      foreach( $entities as $cdRuleEntityId  )  {
        $reference = TranslateToReferences( $cdRuleEntityId );
        array_push($entity['references'], $reference );
      }
    }
  }

  $localStore->Set( $entityId, $entity );
  return $entity['references'];
}


//=============================================================================
//  HELPERS
//=============================================================================

//-----------------------------------------------------------------------------
//  TranslateToReferences(): translate state-points, entity-address & call-number to the right reference format
//
//  @$currentConfig: could be a statepoint or deviceConfigRecord
//
function TranslateToReferences( $externSource, $externSourceEntityId = false )
{

  $amountOfAddressParts = count( explode( "/", $externSource) );
  $isANumber = strpos( $externSource, "." ) === false && intval( $externSource ) !== 0;
  $isEntityAddress = $amountOfAddressParts === 2;
  $isStatepointAddress = $amountOfAddressParts === 3;
  $isEntityId = preg_match( "/\b[a-zA-Z]+\.\d+\b/", $externSource ) != false;

  if( $isANumber && $externSourceEntityId )  {
    if( $externSourceEntityId )  {
      return "call:" . $externSourceEntityId . ":" . $externSource;
    }
  }

  if( $isStatepointAddress )  {
    return "sp:" . $externSource;
  }

  if( $isEntityAddress )  {
    return "call:" . explode( "/", $externSource )[1];
  }

  if( $isEntityId )  {
    return "call:" . $externSource;
  }
  return false;
}

//-----------------------------------------------------------------------------
//  GetTarget(): description
//
function GetTarget( $currentConfig, $entityId )
{
  // check params exists && the sp isn't from this entity
  if( isset($currentConfig['param']['target'])
    && false === empty( $currentConfig['param']['target'] )
    && false === strpos( $currentConfig['param']['target'], $entityId ) ) {
    return $currentConfig['param']['target'];
  }

  return false;
}

function GetSpAddr( $currentConfig, $entityId )
{
  // check params exists && the sp isn't from this entity
  if( isset( $currentConfig['param'], $currentConfig['param']['spaddr'] )
    && false === empty( $currentConfig['param']['spaddr'] )
    && false === strpos( $currentConfig['param']['spaddr'], $entityId ) ) {
    return $currentConfig['param']['spaddr'];
  }
}

//-----------------------------------------------------------------------------
//  GetTargetEntityId(): description
//
function GetTargetEntityId( $currentConfig, $entityId )
{
  // check params exists && the sp isn't from this entity
  if( isset( $currentConfig['param'], $currentConfig['param']['targetEntityId'] )
    && false === empty( $currentConfig['param']['targetEntityId'] )
    && false === strpos( $currentConfig['param']['targetEntityId'], $entityId ) ) {
    return $currentConfig['param']['targetEntityId'];
  }

  if( isset( $currentConfig['param'], $currentConfig['param']['targetentityId'] )
    && false === empty( $currentConfig['param']['targetentityId'] )
    && false === strpos( $currentConfig['param']['targetentityId'], $entityId ) ) {
    return $currentConfig['param']['targetentityId'];
  }

  return false;
}


//-----------------------------------------------------------------------------
//  GetDest(): description
//
function GetDestination( $currentConfig, $entityId )
{
  // check params exists && the sp isn't from this entity
  if( isset($currentConfig['param']['dest'])
    && false === empty( $currentConfig['param']['dest'] )
    && false === strpos( $currentConfig['param']['dest'], $entityId ) ) {
    return $currentConfig['param']['dest'];
  }

  return false;
}

//-----------------------------------------------------------------------------
//  GetDest(): description
//
function GetDestinationEntityId( $currentConfig, $entityId )
{
  // check params exists && the sp isn't from this entity
  if( isset($currentConfig['param']['destentityId'])
    && false === empty( $currentConfig['param']['destentityId'] )
    && false === strpos( $currentConfig['param']['destentityId'], $entityId ) ) {
    return $currentConfig['param']['destentityId'];
  }

  return false;
}

//-----------------------------------------------------------------------------
//  GetAddressDoor(): description
//
function GetAddressDoor( $currentConfig, $entityId )
{
  // check params exists && the sp isn't from this entity
  if( isset($currentConfig['param']['addressDoor'])
    && false === empty( $currentConfig['param']['addressDoor'] )
    && false === strpos( $currentConfig['param']['addressDoor'], $entityId ) ) {
    return $currentConfig['param']['addressDoor'];
  }

  return false;
}

//-----------------------------------------------------------------------------
//  GetOutput(): description
//
function GetOutput( $currentConfig, $entityId )
{
  // check params exists && the sp isn't from this entity
  if( isset($currentConfig['param']['output'])
    && false === empty( $currentConfig['param']['output'] )
    && false === strpos( $currentConfig['param']['output'], $entityId ) ) {
    return $currentConfig['param']['output'];
  }

  return false;
}

//-----------------------------------------------------------------------------
//  GetAddressDoor(): description
//
function GetAddressLight( $currentConfig, $entityId )
{
  // check params exists && the sp isn't from this entity
  if( isset($currentConfig['param']['addressLight'])
    && false === empty( $currentConfig['param']['addressLight'] )
    && false === strpos( $currentConfig['param']['addressLight'], $entityId ) ) {
    return $currentConfig['param']['addressLight'];
  }

  return false;
}

//-----------------------------------------------------------------------------
//  GetTargetSp(): description
//
function GetTargetSp( $currentConfig, $entityId )
{
  // check params exists && the sp isn't from this entity
  if( isset($currentConfig['param']['targetsp'])
    && false === empty( $currentConfig['param']['targetsp'] )
    && false === strpos( $currentConfig['param']['targetsp'], $entityId ) ) {
    return $currentConfig['param']['targetsp'];
  }

  return false;
}


//-----------------------------------------------------------------------------
//  GetEnityIdsFromCdRule(): description
//
function GetEnityIdsFromCdRule( $cdRule )
{
  $entityIdList = Array();
  /*
   * example cdrule
   *
   * "cdrule": [
   * {
   *  "statPT": "system.fj\/group.1\/redir",
   *  "action": "REDIRECT"
   * },
   * {
   *  "addr": "usr:system.fj\/apartment.1",
   *  "tm1": 0,
   *  "tm2": 3600,
   *  "action": "CALLUSER"
   * },
   */

  foreach( $cdRule as $rule ) {
    if( $rule['action'] === "CALLUSER" ){
      $statePointAddress = $rule['addr'];
      $entityId = explode( "/", $statePointAddress )[1];
      array_push( $entityIdList, $entityId );
    }
  }

  return $entityIdList;
}

//-----------------------------------------------------------------------------
//  AnalyseDeviceConfig(): description
//
function AnalyseRecord( $deviceRecord, $entityId )
{
  $referenceList = [];


  $target         = GetTarget( $deviceRecord, $entityId );
  $targetEntityId = GetTargetEntityId( $deviceRecord, $entityId );
  $dest           = GetDestination( $deviceRecord, $entityId );
  $destEntityId   = GetDestinationEntityId( $deviceRecord, $entityId );
  $spAddr         = GetSpAddr( $deviceRecord, $entityId );
  $addressDoor    = GetAddressDoor( $deviceRecord, $entityId );
  $addressLight   = GetAddressLight( $deviceRecord, $entityId );
  $output         = GetOutput( $deviceRecord, $entityId );
  $tartgetSp      = GetTargetSp( $deviceRecord, $entityId );

  if( !isset( $deviceRecord['template'] ) )  {
    return false;
  }
  if( $deviceRecord['template'] === "none" ) {
    return false;
  }

  //
  //  target
  //
  if( !empty( $target ) && strpos( $target, $entityId ) === false ) {
    if( !empty( trim( $target ) ) && !empty( trim( $targetEntityId ) ) ) {
      $reference = TranslateToReferences( $target, $targetEntityId );
    }
    else {
      $reference = TranslateToReferences( $target );
    }

    if( $reference && in_array( $reference, $referenceList ) === false )  {
      array_push( $referenceList, $reference );
    }
  }

  //
  //  spAddr
  //
  if( !empty( $spAddr ) && strpos( $spAddr, $entityId ) === false ) {
    if( !empty( trim( $target ) ) && !empty( trim( $targetEntityId ) ) ) {
      $reference = TranslateToReferences( $spAddr, $targetEntityId );
    }
    else {
      $reference = TranslateToReferences( $spAddr );
    }

    if( $reference && in_array( $reference, $referenceList ) === false )  {
      array_push( $referenceList, $reference );
    }
  }

  //
  //  dest
  //
  if( !empty( $dest ) && strpos( $dest, $entityId ) === false ) {
    if( !empty( trim( $dest ) ) && !empty( trim( $destEntityId ) ) ) {
      $reference = TranslateToReferences( $dest, $destEntityId );
    }
    else {
      $reference = TranslateToReferences( $dest );
    }

    if( $reference && in_array( $reference, $referenceList ) === false )  {
      array_push( $referenceList, $reference );
    }
  }

  //
  //  addressDoor
  //
  if( !empty( $addressDoor ) && strpos( $addressDoor, $entityId ) === false ) {
    if( !empty( trim( $addressDoor ) ) ) {
      $reference = TranslateToReferences( $addressDoor, $destEntityId );
    }

    if( $reference && in_array( $reference, $referenceList ) === false )  {
      array_push( $referenceList, $reference );
    }
  }

  //
  //  addressLight
  //
  if( !empty( $addressLight ) && strpos( $addressLight, $entityId ) === false ) {
    if( !empty( trim( $addressLight ) ) ) {
      $reference = TranslateToReferences( $addressLight, $destEntityId );
    }

    if( $reference && in_array( $reference, $referenceList ) === false )  {
      array_push( $referenceList, $reference );
    }
  }

  if( !empty( $output ) && strpos( $output, $entityId ) === false ) {
    if( !empty( trim( $output ) ) ) {
      $reference = TranslateToReferences( $output );
    }

    if( $reference && in_array( $reference, $referenceList ) === false )  {
      array_push( $referenceList, $reference );
    }
  }

  //
  //  target sp
  //
  if( !empty( $tartgetSp ) && strpos( $tartgetSp, $entityId ) === false ) {
    if( !empty( trim( $tartgetSp ) ) ) {
      $reference = TranslateToReferences( $tartgetSp );
    }

    if( $reference && in_array( $reference, $referenceList ) === false )  {
      array_push( $referenceList, $reference );
    }
  }

  return $referenceList;
}