<?php
/*-----------------------------------------------------------------------------
 *  UPDATE for 6.1.0 (6981)
 *
 *  - add lastdoor/lastlight to asht
 *  - check if password exists and correct flag
 *
 *---------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
 *  MAIN - FUNCTION
 */
function Update6981()
{
  global $localStore;
  $accessBuild = intval( ACCESS_BUILD );

  if( FORCE || $localStore->GetBuild( 'folder.confdev' ) <= 6981 )  {
    printf( "Start Update #6981\r\n" );

    $entityIdList = $localStore->GetEntityIdList();
    array_push( $entityIdList, "*" );

    for( $i = 0; $i < $localStore->Size( ); $i++ )  {

      $entityId = $localStore->GetEntityIdAt( $i );

      // asc+asht problems
      if( strpos( $entityId, "asht" ) !== false 
       || strpos( $entityId, "client" ) !== false 
        ) {
        Update6981AscAsht( $entityId );
      }

      // lost permissions
      Update6981LostPermissions( $entityId, $entityIdList );
    }
  }
  else {
    printf( "Skip Update #6981. Server version=%d\r\n",$accessBuild );
  }

}

/*-----------------------------------------------------------------------------
 *  UPDATE IPDB
 */

//-----------------------------------------------------------------------------
//  Update6981AscAsht(): add statepoints "lastdoor & lastlight" if these 
//  statepoints does not exist + check password default flag
//
function Update6981AscAsht( $entityId )
{
  global $localStore;
  $entity = $localStore->Get( $entityId );

  if( isset( $entity['sp'] ) === false )  {
    $entity['sp'] = Array();
  }

  Add6981LastLightStatepoint( $entity );
  Add6981LastDoorStatepoint( $entity );
  Check6981PasswordFlag( $entity, $entityId );

  $localStore->Set( $entityId, $entity );
}


//-----------------------------------------------------------------------------
//  check if entity has password and set password flag 
// 
function Check6981PasswordFlag( &$entity, $entityId ) 
{
  global $localStore;
  if( isset( $entity['authorize'] ) ) {
    $hasPassword = $localStore->HasPassword( $entityId );
    if( $entity['authorize']['default'] === true && $hasPassword )  {
      $entity['authorize']['default'] = false;
    }
  }

}

//-----------------------------------------------------------------------------
//  AddLastLightStatepoint( $entity ): add lastlight statepoint if it's not exists
//

function Add6981LastLightStatepoint( &$entity )
{
  global $localStore;
  if( isset( $entity['sp']['lastlight'] ) === false ) {
    $entity['sp']['lastlight'] = array(
      "address"    => $localStore->GetRootId() . "/" . $entity['entid'] . "/lastlight",
      "type"       => "BOOL",
      "name"       => "lastlight",
      "persistent" => false,
      "hidden"     => true,
      "template"   => "lastlight",
      "alias"      => array(
        "target"  => null,
        "timeout" => 0
      )
    );
  }
}

//-----------------------------------------------------------------------------
//  AddLastDoorStatepoint(): add lastdoor statepoint if it's not exists
//
function Add6981LastDoorStatePoint( &$entity )
{
  global $localStore;
  if( isset( $entity['sp']['lastdoor'] ) === false ) {
    $entity['sp']['lastdoor'] = array(
      "address"    => $localStore->GetRootId() . "/" . $entity['entid'] ."/lastdoor",
      "type"       => "BOOL",
      "name"       => "lastdoor",
      "persistent" => false,
      "hidden"     => true,
      "template"   => "lastdoor",
      "alias"      => array(
        "target"  => null,
        "timeout" => 0
      )
    );
  }
}


function Update6981LostPermissions( $entityId, &$entityIdList )
{
  global $localStore;
  $entity = $localStore->Get( $entityId );
  $isModified = false;
  
  if( !( isset( $entity['permission'] ) ) )  {
    return false;
  }  

  //var_dump( $entityIdList );

  //foreach( $entity['permission'] as $index => $permRecord )  {
  for( $index = sizeof( $entity['permission'] ) -1; $index >= 0; $index-- ) {
    $permRecord = $entity['permission'][ $index ];
    if( ! in_array( $permRecord['w'], $entityIdList ) )  {
      //unset(  $entity['permission'][ $index ] );
      array_splice( $entity['permission'], $index, 1 );
      $isModified = true;
    }
  }

  if( $isModified )  {
    //printf( "#6981: correct lost permission " . $entityId . "\r\n" );
    $localStore->Set( $entityId, $entity );
  }

}