<?php
/*-----------------------------------------------------------------------------
 *  UPDATE for 6.1.0 (6986)
 *
 *  - set time parameter in grabpic explict (not using server defaults)
 *  - reformat door & light from access 5 to access 6.1
 *
 *---------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
 *  MAIN - FUNCTION
 */
function Update6986()
{
  global $localStore;
  $accessBuild = intval( ACCESS_BUILD );

  if( FORCE || $localStore->GetBuild( 'folder.confdev' ) <= 6986 )  {
    printf( "Start Update #6986\r\n" );

    $entityIdList = $localStore->GetEntityIdList();
    array_push( $entityIdList, "*" );

    for( $i = 0; $i < $localStore->Size( ); $i++ ) {
      $entityId = $localStore->GetEntityIdAt( $i );

      // update time in grabpic
      Update6986GrabpicParamTime( $entityId );

      // updates on door
      if( strpos( $entityId, "door" ) !== false ) {
        Update6986Door( $entityId );
      }

    }

  }
  else {
    printf( "Skip Update #6986. Server version=%d\r\n",$accessBuild );
  }

}

/*-----------------------------------------------------------------------------
 *  UPDATE grabpic in all entities
 */

//-----------------------------------------------------------------------------
//  add if "sp.grabpic.param.time" is missing 
//
function Update6986GrabpicParamTime( $entityId )
{
  global $localStore;
  $entity = $localStore->Get( $entityId );
  $isModified = false;

  if( ! isset( $entity['sp'] )
   || ! isset( $entity['sp']['grabpic'] ) 
    )  {
    return;
  }

  // param is missing
  if( ! isset( $entity['sp']['grabpic']['param'] ) ) {
    $entity['sp']['grabpic']['param'] = Array( 'time' => 4 );
    $isModified = true;
  }

  // time in param is missing
  else if( ! isset( $entity['sp']['grabpic']['param']['time'] ) ) {
    $entity['sp']['grabpic']['param']['time'] = 4;
    $isModified = true;
  }

  if( $isModified ) {
    $localStore->Set( $entityId, $entity );
  }
}

/*-----------------------------------------------------------------------------
 *  UPDATE door
 */

//-----------------------------------------------------------------------------
//  all corrections on door
//
function Update6986Door( $entityId )
{
  global $localStore;
  $entity = $localStore->Get( $entityId );
  $isModified = false;

  if( Update6986DoorLight( $entity ) ) {
    $isModified = true;
  }
  if( Update6986DoorOpener( $entity ) ) {
    $isModified = true;
  }
  if( Update6986DoorOpener2( $entity ) ) {
    $isModified = true;
  }  

  if( $isModified ) {
    $localStore->Set( $entityId, $entity );
  }  
}

//-----------------------------------------------------------------------------
//  door light
//
function Update6986DoorLight( &$entity )
{
  global $localStore;
  $isModified = false;
  $entityId = $entity['entid'];

  //-----------------------------------------------------------------
  //  case 1: in statepoint 'light' output param is not set
  //  to local switch contact
  //
  if( isset( $entity['sp']['light'] ) 
   && ! isset( $entity['sp']['light']['param']['output'] ) 
    ) {
    //printf( "Light param/output missing: $entityId\r\n" );

    // find sp 'light'
    foreach( $entity['deviceConfig'] as $index => $value )  {
      if( isset( $value['sp'] ) && isset( $value['uid'] ) )  {
        if( $value['sp'] == "light" ) {
          if( ! isset( $entity['sp']['light']['param'] ) )  {
            $entity['sp']['light']['param'] = Array( 'output' => $value['uid'] );
            $isModified = true;
            break;
          }
          else if( ! isset( $entity['sp']['light']['param']['output'] ) )  {
            $entity['sp']['light']['param']['output'] = $value['uid'];
            $isModified = true;
            break;
          }
        }
      }
    }
  }

  //-----------------------------------------------------------------
  //  case 2: param/output is set, but local contact (e.g. atlc_a3) 
  //  not been set to light
  //
  else if( isset( $entity['sp']['light'] ) 
        && isset( $entity['sp']['light']['param']['output'] ) 
        && strpos( $entity['sp']['light']['param']['output'], '/' ) === false
    ) {
    
    // find the configured output in devConf
    foreach( $entity['deviceConfig'] as $index => $value )  {
      
      if( ( isset( $value['uid'] ) )
       && ( $value['uid'] == $entity['sp']['light']['param']['output'] )
        ) {

        // devConf field "sp" is missing
        if( ! isset( $value['sp'] ) ) {
          $entity['deviceConfig'][ $index ]['sp'] = "light";
          $isModified = true;
        }

        // devConf in field "template" not "lightcontact" set
        if( $value['template'] != "lightcontact" ) {
          $entity['deviceConfig'][ $index ]['template'] = "lightcontact";
          $isModified = true;
        }

        // devConf field "param" is missing
        if( ! isset( $value['param'] ) ) {
          $entity['deviceConfig'][ $index ]['param'] = Array( 
            "time" => ( isset( $entity['sp']['door']['signaltime'] ) ? $entity['sp']['door']['signaltime'] : 3000 ),
            "posttrigger" => ( isset( $entity['sp']['door']['retrigger'] ) ? $entity['sp']['door']['retrigger'] : true ),
            "invert" => ( isset( $entity['sp']['door']['invert'] ) ? $entity['sp']['door']['invert'] : false ),
          );
          $isModified = true;
        }

        // devConf field ["param"]["time"] is missing
        if( ! isset( $entity['deviceConfig'][ $index ]['param']['time'] ) ) {
          $entity['deviceConfig'][ $index ]['param']['time'] = ( isset( $entity['sp']['door']['signaltime'] ) ? $entity['sp']['door']['signaltime'] : 3000 );
          $isModified = true;
        }
        // devConf field ["param"]["posttrigger"] is missing
        if( ! isset( $entity['deviceConfig'][ $index ]['param']['posttrigger'] ) ) {
          $entity['deviceConfig'][ $index ]['param']['posttrigger'] = ( isset( $entity['sp']['door']['retrigger'] ) ? $entity['sp']['door']['retrigger'] : true );
          $isModified = true;
        }
        // devConf field ["param"]["invert"] is missing
        if( ! isset( $entity['deviceConfig'][ $index ]['param']['invert'] ) ) {
          $entity['deviceConfig'][ $index ]['param']['invert'] = ( isset( $entity['sp']['door']['invert'] ) ? $entity['sp']['door']['invert'] : false );
          $isModified = true;
        }        

        if( $isModified ) {
          break;
        }
      }
    }
  }


  return $isModified;
}

//-----------------------------------------------------------------------------
//  door opener
//
function Update6986DoorOpener( &$entity )
{
  global $localStore;
  $isModified = false;
  
  global $localStore;
  $isModified = false;
  $entityId = $entity['entid'];

  //-----------------------------------------------------------------
  //  case 1: in statepoint 'light' output param is not set
  //  to local switch contact
  //
  if( isset( $entity['sp']['door'] ) 
   && ! isset( $entity['sp']['door']['param']['output'] ) 
    ) {
    //printf( "Door param/output missing: $entityId\r\n" );

    // find output used for door in devConf
    foreach( $entity['deviceConfig'] as $index => $value )  {
      if( isset( $value['sp'] ) && isset( $value['uid'] ) )  {
        if( $value['sp'] == "door" ) {
          if( ! isset( $entity['sp']['door']['param'] ) )  {
            $entity['sp']['door']['param'] = Array( 'output' => $value['uid'] );
            $isModified = true;
            break;
          }
          else if( ! isset( $entity['sp']['door']['param']['output'] ) )  {
            $entity['sp']['door']['param']['output'] = $value['uid'];
            $isModified = true;
            break;
          }
        }
      }
    }
  }

  //-----------------------------------------------------------------
  //  case 2: param/output is set, but local contact (e.g. atlc_a3) 
  //  not been set to light
  //
  else if( isset( $entity['sp']['door'] ) 
        && isset( $entity['sp']['door']['param']['output'] ) 
        && strpos( $entity['sp']['door']['param']['output'], '/' ) === false
    ) {
    
    // find the configured output in devConf
    foreach( $entity['deviceConfig'] as $index => $value )  {
      
      if( ( isset( $value['uid'] ) )
       && ( $value['uid'] == $entity['sp']['door']['param']['output'] )
        ) {

        // devConf field "sp" is missing
        if( ! isset( $value['sp'] ) ) {
          $entity['deviceConfig'][ $index ]['sp'] = "door";
          $isModified = true;
        }

        // devConf in field "template" not "openercontact" set
        if( $value['template'] != "openercontact" ) {
          $entity['deviceConfig'][ $index ]['template'] = "openercontact";
          $isModified = true;
        }

        // devConf field "param" is missing
        if( ! isset( $value['param'] ) ) {
          $entity['deviceConfig'][ $index ]['param'] = Array( 
            "time" => ( isset( $entity['sp']['door']['signaltime'] ) ? $entity['sp']['door']['signaltime'] : 3000 ),
            "posttrigger" => ( isset( $entity['sp']['door']['retrigger'] ) ? $entity['sp']['door']['retrigger'] : true ),
            "invert" => ( isset( $entity['sp']['door']['invert'] ) ? $entity['sp']['door']['invert'] : false ),
          );
          $isModified = true;
        }

        // devConf field ["param"]["time"] is missing
        if( ! isset( $entity['deviceConfig'][ $index ]['param']['time'] ) ) {
          $entity['deviceConfig'][ $index ]['param']['time'] = ( isset( $entity['sp']['door']['signaltime'] ) ? $entity['sp']['door']['signaltime'] : 3000 );
          $isModified = true;
        }
        // devConf field ["param"]["posttrigger"] is missing
        if( ! isset( $entity['deviceConfig'][ $index ]['param']['posttrigger'] ) ) {
          $entity['deviceConfig'][ $index ]['param']['posttrigger'] = ( isset( $entity['sp']['door']['retrigger'] ) ? $entity['sp']['door']['retrigger'] : true );
          $isModified = true;
        }
        // devConf field ["param"]["invert"] is missing
        if( ! isset( $entity['deviceConfig'][ $index ]['param']['invert'] ) ) {
          $entity['deviceConfig'][ $index ]['param']['invert'] = ( isset( $entity['sp']['door']['invert'] ) ? $entity['sp']['door']['invert'] : false );
          $isModified = true;
        }

        if( $isModified ) {
          break;
        }
      }
    }
  }  


  return $isModified;
}

//-----------------------------------------------------------------------------
//  door opener
//
function Update6986DoorOpener2( &$entity )
{
  global $localStore;
  $isModified = false;
  

  return $isModified;
}
