<?php
/******************************************************************************
 *    DEVICE CONFIG - TEMPLATING
 *  -------------------------------------------------------------------------
 *
 *  Inserts templates into an device-konfig file,
 *  given the entity
 *
 *****************************************************************************/

//require_once( PHPLIB_PATH."/A5_ConfigProxy.php");

/**
   * formats the xml output readable
   *
   * @param $simpleXmlObject instance of SimpleXmlObject to pretty-print
   * @return string of indented xml-elements
   */
function SimpleXMLPretty( &$simpleXmlObject )
{
  if( ! is_object($simpleXmlObject) ){
    return "";
  }

  // Format XML to save indented tree rather than one line
  $dom = new DOMDocument('1.0');
  $dom->preserveWhiteSpace = false;
  $dom->formatOutput = true;
  $dom->loadXML($simpleXmlObject->asXML());

  return $dom->saveXML();
}


class DevConfigTemplates
{
  /*-------------------------------------------------------
   *  member variables
   */
  protected $params        = Array( 'this' => null, 'parent' => null, 'system' => null, 'current' => null );
  protected $mustache      = null;
  protected $output        = null;
  protected $template      = null;
  protected $widgets       = Array();
  protected $m_configProxy = null;
  protected $m_systemProxy = null;


  /*-------------------------------------------------------
   *  constructor
   */
  function __construct( $entid )
  {
    // get the config-proxy for gathering the password
    $this->m_configProxy = new ConfigProxy(  );
    $this->m_systemProxy = new SystemProxy(  );

    $this->mustache = new Mustache_Engine;
    $this->LoadEntity( $entid );
    $widgetEngine = new WidgetTemplates( 'devconf' );
    $this->widgets = $widgetEngine->Read();

  }

  /*-------------------------------------------------------
   *  helper functions
   */
  function LoadEntity( $entid )
  {
    // load params
    // load entity
    $str = file_get_contents( CONFSTORE_PATH . $entid . '.json' );
    $this->params['this']            = json_decode( $str, true );
    $this->params['this']['entaddr'] = str_replace( ".", "", $entid );

    $passwordResponse = $this->m_configProxy->GetPassword( $entid );
    $this->params['this']['authorize']['sip']['pass'] = $passwordResponse["data"];
    $this->params['this']['password'] = $passwordResponse["data"];


    $str = file_get_contents( CONFSTORE_PATH . $this->params['this']['parent'] . '.json' );
    $this->params['parent'] = json_decode( $str, true );
    $this->params['parent']['entaddr'] = str_replace( ".", "", $this->params['parent']['entid'] );

    $str = file_get_contents( DEVTPL_PATH . '/system.defaults.json' );
    $this->params['system'] = json_decode( $str, true );
    $this->params['system'] = $this->params['system']['system'];

    //device param for templates
    $this->params['device'][$this->params['this']['class']] =  true;
    $this->params['doIgnore'] =  function( $text, $render)  {
      return $text;
    };

    //if pass empty is empty set pass to 0000
    if( !isset( $this->params['parent']['sysuser']['pass'] ) )  {
      $this->params['parent']['sysuser'] = Array(
        "pass" => "0000"
      );
    }

    $str = file_get_contents( CONFSTORE_PATH . 'admui.license.json' );
    $adminUiLicense = json_decode( $str, true );
    if( key_exists( 'ssh', $adminUiLicense ) )  {
      if( key_exists( 'sshPublicKey', $adminUiLicense['ssh'] ) )  {
        $this->params['system']['sshPublicKey'] = $adminUiLicense['ssh']['sshPublicKey'];
      }
    }

    $str = file_get_contents( CONFSTORE_PATH . 'admui.param.json' );
    $adminUiParam = json_decode( $str, true );

    // set current system time zone 
    $timeZones = $this->m_systemProxy->GetTimeZone();
    if( isset( $timeZones['data'] ) )  {
      $timeZones = $timeZones['data'];
      $adminUiParam['locale']['timezone'] = $timeZones['tzInfo'];
    }

    if( key_exists( 'locale', $adminUiParam ) )  {
      if( key_exists( 'lang', $adminUiParam['locale'] ) )  {
        $this->params['system']['lang'] = $adminUiParam['locale']['lang'];
      }
      if( key_exists( 'timezone', $adminUiParam['locale'] ) )  {
        $this->params['system']['timezone'] = $adminUiParam['locale']['timezone'];
      }
    }
    if( key_exists( 'regtimes', $adminUiParam ) )  {
      if( key_exists( 'regdelaymin', $adminUiParam['regtimes'] ) )  {
        $this->params['system']['regdelaymin'] = $adminUiParam['regtimes']['regdelaymin'];
      }
      if( key_exists( 'regdelaymax', $adminUiParam['regtimes'] ) )  {
        $this->params['system']['regdelaymax'] = $adminUiParam['regtimes']['regdelaymax'];
      }
      if( key_exists( 'regfailmin', $adminUiParam['regtimes'] ) )  {
        $this->params['system']['regfailmin'] = $adminUiParam['regtimes']['regfailmin'];
      }
      if( key_exists( 'regfailmax', $adminUiParam['regtimes'] ) )  {
        $this->params['system']['regfailmax'] = $adminUiParam['regtimes']['regfailmax'];
      }

      //------- Ringtones --------------
      $this->setRingtones();
    }

    // load device templates
    if( key_exists( 'deviceTemplate', $this->params['this'] ) )   {

      if( file_exists( DEVTPL_PATH . $this->params['this']['deviceTemplate'] . '.templates.xml'  ))  {
        $str = file_get_contents( DEVTPL_PATH . $this->params['this']['deviceTemplate'] . '.templates.xml' );
        $this->template = new SimpleXMLElement( $str );
      }

      if( file_exists( DEVTPL_PATH . $this->params['this']['deviceTemplate'] . '.output.xml'  ))  {
        $str = file_get_contents( DEVTPL_PATH . $this->params['this']['deviceTemplate'] . '.output.xml' );
        $this->output = $str;
      }
    }
    else  {
      LOG::error( "devTpl", "Device-Template not set in entity" );
    }

  }

  function append_simplexml( &$simplexml_to, &$simplexml_from )
  {
    foreach( $simplexml_from->children() as $simplexml_child )  {
      $simplexml_temp = $simplexml_to->addChild( $simplexml_child->getName(), (string) $simplexml_child );
      foreach( $simplexml_child->attributes() as $attr_key => $attr_value )  {
        $simplexml_temp->addAttribute( $attr_key, $attr_value );
      }
      $this->append_simplexml( $simplexml_temp, $simplexml_child );
    }
  }

  function append_simplexml_tag( &$simplexml_to, &$simplexml_from, $tag )
  {
    $simplexml_temp = $simplexml_to->addChild( $tag );
    $this->append_simplexml( $simplexml_temp, $simplexml_from );
  }

  /*-------------------------------------------------------
   *  writer
   */
  function Create( $file )
  {
    if( $this->template && $this->output )  {


      $this->PrepareBaseFile();
      $this->ApplyDeviceTemplates();

      // write for debug purpose
      file_put_contents( $file, SimpleXMLPretty( $this->output ) );
      return SimpleXMLPretty( $this->output );
    }
    else  {
      //var_dump( $this );
    }
    return false;
  }


  /*-------------------------------------------------------
   *  Builds the basic file (without the templates)
   */
  function PrepareBaseFile()
  {
    //$this->params['module']['avp'] = false;
    $result = $this->mustache->render( $this->output, $this->params );
    $this->output = new SimpleXMLElement( $result );
  }


  /*-------------------------------------------------------
   *  Builds the basic file (without the templates)
   */
  function ApplyDeviceTemplates()
  {
    $entity = $this->params['this'];
    // apply all deviceParams
    if( key_exists( 'deviceParam', $entity ) )  {
      foreach( $entity['deviceParam'] as $value ) {
        $this->params['current'] = $value;
        if( $this->params['current']['module'] == "KBD" )  {
          if( $this->params["current"]["trigger"] == "pos" || $this->params["current"]["trigger"] == "neg" )  {
            $this->params['module']["edgeTriggered"] = true;
          }
          else {
            $this->params['module']["edgeTriggered"] = false;
          }

        }
        if( $this->params['current']['module'] == "IO" )  {
          if( $this->params["current"]["trigger"] == "sensor" )  {
             $this->params['module']["sensor"] = true;
          }
          else {
            $this->params['module']["sensor"] = false;
          }
        }
        $this->ApplyTemplateElement( $this->params['current'] );
      }
    }

    // apply all deviceConfig
    if( key_exists( 'deviceConfig', $entity ) && is_array( $entity['deviceConfig'] ) )  {
      foreach( $entity['deviceConfig'] as $value ) {
        $this->params['current'] = $value;
        $this->ApplyTemplateElement( $this->params['current'] );
      }
    }


  }


  /*-------------------------------------------------------
   *  Builds the basic file (without the templates)
   */
  function ApplyTemplateElement( $element )
  {
    //set module for mustache
    $this->params['module'] = Array();
    $this->params['module'][ $element['module'] ] = true;
    $this->params['template'] = Array();
    $this->params['template'][ $element['template'] ] = true;

    if( $element['module'] == "AZIO" )  {
      $this->params['module']['IO'] = true;
      $this->params['module']['sensor'] = true;
    }

    if( $element['module'] == "function" )  {
      $this->params['function'] = true;
    }
    if( $this->params['current']['module'] == "KBD" )  {
      if( $this->params["current"]["trigger"] == "pos" || $this->params["current"]["trigger"] == "neg" )  {
        $this->params['module']["edgeTriggered"] = true;
      }
      else {
        $this->params['module']["edgeTriggered"] = false;
      }
    }
    if( $this->params['current']['module'] == "IO" )  {
      if( key_exists( "trigger", $this->params["current"] ) &&  $this->params["current"]["trigger"] == "sensor" )  {
          $this->params['module']["sensor"] = true;
      }
      else {
        $this->params['module']["sensor"] = false;
      }
    }
    // get the template and the associated xpath
    $template = $this->template->xpath( "//template[@name='" . $element['template'] . "']/insert" );

    // if in device-xml template the element does not exists, use global widget
    if( empty( $template ) )  {
      if( key_exists( $element['template'], $this->widgets ) )  {
        $template = $this->widgets[ $element['template'] ];
        $template = $this->mustache->render( $template->asXml(), $this->params );
        $template = simplexml_load_string($template);
        $template = $template->xpath( "//template[@name='" . $element['template'] . "']/insert" );
      }
    }

    for( $i=0; $i<count( $template ); $i++ )  {

      $tmpTemplate = $template[ $i ];
      $tplxpath = (string)$tmpTemplate['xpath'];
      $stptaddr = (string)$tmpTemplate['stptaddr'];

      // convert the tmpTemplate element to string and let mustache render it
      $str = $tmpTemplate->asXML();
      $str = $this->mustache->render( $str, $this->params );
      $result = new SimpleXMLElement( $str );

      // navigate to the desired xpath
      $insertTag = $this->output->xpath( $tplxpath );



      if( $insertTag )  {
        $insertTag = $insertTag[0];


        //check if stptaddr is set as attribute
        $foundStptaddr = false;
        if( $stptaddr )  {
          $stptaddr = $this->mustache->render( $stptaddr, $this->params );
          foreach ($insertTag->stpt as $key => $stpt) {
            if( $stpt->stptaddr == $stptaddr )  {
              $foundStptaddr = true;
              $insertTag = $stpt;
              break;
            }
          }
          if( !$foundStptaddr )  {
            $parent = $insertTag;
            $newStpt = $parent->addChild( "stpt" );
            $newStpt->addChild( "stptaddr", $stptaddr );
            $insertTag = $newStpt;
          }
        }
        $this->append_simplexml( $insertTag, $result );
      }
      else  {
        // "//varioevtcfg/atlm"
        // if <atlm> does not exists, we need to create one

        $pos = strrpos( $tplxpath, '/' );
        if( $pos )  {
          $tplxpath2 = substr( $tplxpath, 0, $pos ) ;
          $tag = substr( $tplxpath, $pos+1 ) ;
          $insertTag = $this->output->xpath( $tplxpath2 );
          if( $insertTag )  {
            $insertTag = $insertTag[0];
            $this->append_simplexml_tag( $insertTag, $result, $tag  );
          }
        }
        else  {
          LOG::error( "devTpl", "XPATH %s not found on element %s", $tplxpath, $element['template'] );
        }
      }
    }
  }

  /*****************************************************************************************************
   * setRingtones
   * 
   * set default values for xml templates or write the values from device config
   */
  function setRingtones()
  {
    // defaults
    $this->params['announcementtone'] = "Glockenspiel";
    $this->params['calltone']         = "Glockenspiel";
    $this->params['doorcalltone']     = "Suitcase_1";
    $this->params['storeycalltone']   = "Vibraphone_2";

    // check if settings are set ( funcui wideget ringtone )
    foreach( $this->params['this']['deviceConfig'] as $deviceConfigEntry )  {
      if( isset( $deviceConfigEntry['template'] ) && $deviceConfigEntry['template'] == "ringtone" )  {
        $this->params['calltone']         = $this->getRingtone( $deviceConfigEntry, "calltone" );
        $this->params['doorcalltone']     = $this->getRingtone( $deviceConfigEntry, "doorcalltone" );
        $this->params['storeycalltone']   = $this->getRingtone( $deviceConfigEntry, "storeycalltone" );
        $this->params['announcementtone'] = $this->getRingtone( $deviceConfigEntry, "announcementtone" );
        break;
      }
    }
  }

  /******************************************************************************************************
   * getRingtone()
   * return admin settings or default values;
   */
  function getRingtone( $deviceConfigEntry, $ringtone )
  {
    
    return isset( $deviceConfigEntry[ $ringtone ] ) ? $deviceConfigEntry[ $ringtone ] : $this->params[ $ringtone ];
  }




};

?>