<?php
/******************************************************************************
 *    ENTITY CREATE - TEMPLATING
 *  -------------------------------------------------------------------------
 *  
 *  creates a new entity given a template and some parameters 
 * 
 *****************************************************************************/
 
class WidgetTemplates
{
  /*-------------------------------------------------------
   *  member variables
   */  
  //protected $params   = Array( 'this' => null, 'parent' => null, 'system' => null );
  //protected $mustache = null;
  protected $result   = null;
  //protected $lang     = 'de';

  /*-------------------------------------------------------
   *  protected helpers
   */  
  function scandir( $path, $filter )
  {
    $files = scandir( $path ); 
    $len = strlen( $filter );
    foreach( $files as $key => $value ) {
      if( strlen($value) <= $len )  {
        unset( $files[$key] );
      }
      else if( strtolower( substr( $value, -$len ) ) != $filter )  {
        unset( $files[$key] );        
      }
    }
    return $files;
  }  

  /*-------------------------------------------------------
   *  constructor
   */  
  function __construct( $param ) 
  {
    if( $param == 'permdef' )  {
      $this->LoadPermdef();
    }
    else if( $param == 'devconf' )  {
      $this->LoadDevconf();
    }
    else  {
      $this->LoadEntity();
    }
  }
  
  /*-------------------------------------------------------
   *  load entity
   */  
  protected function LoadEntity()
  {
    // get all widget files for entities
    $files = $this->scandir( WIDGETTPL_PATH, ".entity.json" );

    // build global template array    
    $this->result = Array();
    foreach( $files as $key => $value )  {
      
      // load single template file
      $tpl = file_get_contents( WIDGETTPL_PATH.$value );
      $tpl = json_decode( $tpl, true );
      if( !$tpl )  {
        LOG::error( __CLASS__, "Widget '%s' - JSON-Error", $value );
        continue;
      }
      
      // add to templates
      $widgetName = explode( '.', $value );
      $widgetName = strtolower( $widgetName[0] );
      
      // all main sections should be merged together
      foreach( $tpl as $section => $content ) {
        $this->result[ $section ][ $widgetName ] = $content; 
      }      
    }
    
    // get all language definitions for widgets and rebuild dict.json
    $files = $this->scandir( WIDGETTPL_PATH, ".lang.json" );

    // build global list    
    $this->result['lang'] = Array();
    foreach( $files as $key => $value )  {
      
      // load single template file
      $lang = file_get_contents( WIDGETTPL_PATH.$value );
      $lang = json_decode( $lang, true );
      if( !$lang )  {
        LOG::error( __CLASS__, "Widget(lang) '%s' - JSON-Error", $value );
        continue;
      }
      
      // add to list
      
      // all main sections should be merged together
      foreach( $lang['de'] as $key => $value ) {
        $this->result['lang']['de'][ $key ] = $value; 
      }      
      foreach( $lang['en'] as $key => $value ) {
        $this->result['lang']['en'][ $key ] = $value; 
      }      
    }
    
    //var_dump( $this->result['lang'] );
    $dict = json_encode( $this->result['lang'], JSON_PRETTY_PRINT|JSON_UNESCAPED_UNICODE );
    if( $dict )  {
      file_put_contents( WIDGETTPL_PATH.'../lang/dict.json', $dict );
    }
    
  }
  
  /*-------------------------------------------------------
   *  load permdef
   */  
  protected function LoadPermdef()
  {
    // get all widget files for entities
    $files = $this->scandir( WIDGETTPL_PATH, ".permdef.json" );
    
    // build global template array    
    $this->result = Array();
    foreach( $files as $key => $value )  {
      
      // load single template file
      $tpl = file_get_contents( WIDGETTPL_PATH.$value );
      $tpl = json_decode( $tpl, true );
      if( !$tpl )  {
        LOG::error( __CLASS__, "Widget '%s' - JSON-Error", $value );
        continue;
      }
      
      // all main sections should be merged together
      if( key_exists( 'statPt', $tpl['permdef'] ) )  {
        foreach( $tpl['permdef']['statPt'] as $section => $content ) {
          $this->result['statPt'][ $section ] = $content; 
        }
      }      
      if( key_exists( 'callto', $tpl['permdef'] ) )  {
        foreach( $tpl['permdef']['callto'] as $section => $content ) {
          $this->result['callto'][ $section ] = $content; 
        }
      }      
    }
  }

  /*-------------------------------------------------------
   *  load device config files
   */  
  protected function LoadDevconf()
  {
    // get all widget files for entities
    $files = $this->scandir( WIDGETTPL_PATH, ".devtpl.xml" );

    // build global template array    
    $this->result = Array();
    foreach( $files as $key => $value )  {
      
      $tpl = file_get_contents( WIDGETTPL_PATH.$value );
      $tpl = new SimpleXMLElement( $tpl );
      if( !$tpl )  {
        LOG::error( __CLASS__, "Widget '%s' - JSON-Error", $value );
        continue;
      }
      $tplname = explode( '.', $value );
      $this->result[ $tplname[0] ] = $tpl;
    }
  }

    
  /*-------------------------------------------------------
   *  writer
   */  
  function Read( )
  {
    return $this->result; 
  }

  
  
       
};

?>