<?php

/***********************************************************
 * Envelope
 *
 *
 */
class Envelope{
  //-------------------------------------------------------
  //  CONSTRUCTOR
  //-------------------------------------------------------
  private $envelope;
  private $m_metaKeyLists; // only needed. optinal keys like "language" are not listed
  //-------------------------------------------------------
  //  CONSTRUCTOR
  //-------------------------------------------------------
  /********************************************************
   *  _constructor( $envelope ) / Envelope( $envelope )
   * @param $envelope  jsonString
   */

  public function __construct( ){
    //initialize variables
    $this->envelope = null;
    $this->m_metaKeyLists             = Array( "request" => Array( "command", "session", "guid" ), "response" => Array() );
    $this->m_metaKeyLists['response'] = array_merge( $this->m_metaKeyLists['request'], Array( "status" ) );

    $arguments = func_get_args();
    // arugment for an json string
    if( sizeof( $arguments ) == 1 )  {
      $this->ConvertJsonstring( $arguments[0] );
    }
    // arguments for an envelope
    else if( sizeof( $arguments ) == 4 )  {
      $this->CreateEnvlope( $arguments[0], $arguments[1], $arguments[2], $arguments[3] );
    }
    // arguments for an envelope
    else if( sizeof( $arguments ) == 5 )  {
      $this->CreateEnvlope( $arguments[0], $arguments[1], $arguments[2], $arguments[3], $arguments[4] );
    }
    else  {
      LOG::error( __CLASS__, "(%s) paramenter doesn't match. numbers of arguments: %s \n" .
      "arguments can be: Envelope( JSONstring ), Envelope( guid, command, sessionId, data ) or Envelope( guid, command, sessionId, status, data )", __LINE__, sizeof( $arguments ) );
      throw new Exception("paramenter doesn't match. numbers of arguments:" . sizeof($arguments), 1);


    }
  }
  //-------------------------------------------------------
  //  MEMBER FUNCTIONS
  //-------------------------------------------------------
  /********************************************************
   * GetMeta( $envelope = null )
   */
  static public function GetMeta( $envelope = null )
  {
    if( !is_null( $envelope ) )  {
      if( is_array( $envelope ) )  {
        return $envelope['meta'];
      }
      else if( is_string( $envelope ) )  {
        $envelope = json_decode( $envelope, true );
        return $envelope['meta'];
      }
    }
    else  {
      return false;
    }

  }
  /*********************************************************
   * GetMeta( $envelope = null )
   */
  static public function GetData( $envelope = null )
  {
    if( is_null( $envelope ) )  {
      if( is_array( $envelope ) )  {
        return $envelope['data'];
      }
      else if( is_string( $envelope ) )  {
        $envelope = json_decode( $envelope, true );
        return $envelope['data'];
      }
    }
    else  {
      return false;
    }
  }
  /**
   * Get()
   * @param bool $json if $json is true a json string will be returned
   * @return array returns the envelope array by default
   */
  public function Get( $json = false )
  {
    if( $json ) return json_encode( $this->envelope, JSON_UNESCAPED_UNICODE );
    return $this->envelope;
  }
  /*******************************************************
   * convertJsonString( $envelope )
   */

  private function ConvertJsonString( $envelope )
  {
    $envelope       = $envelope;
    $envelope       = json_decode( $envelope, true );
    $meta           = self::GetMeta( $envelope );
    $errors         = Array();
    // if envelope contains a meta['status'] field create a response envelope
    // else create a normal request envelope
    $this->CheckEnvelopeMeta($meta);


    // save the envelope
    $this->envelope = $envelope;
  }
  /**********************************************************
   * CreateEnvlope( $guid = null, $command = null, $sessionId = null, $data = null )
   */
  private function CreateEnvlope( $guid = null, $command = null, $sessionId = null, $data = null, $language = null )
  {
    if( !isset( $guid ) || is_null( $guid) )  {
      throw new Exception( "guid <br/> guid: $guid", 1 );
    }
    else if( !isset( $command ) || is_null($command) ) {
      throw new Exception("command is not defined <br/> cmd:$command", 1);
    }
    else if( !isset( $sessionId ) || is_null($sessionId) ) {
      throw new Exception("sessionId is not defined <br/> sessionId:$command", 1);
    }
    else if( !isset( $sessionId ) || is_null($sessionId) ) {
      throw new Exception("data is not defined <br/> data:$data", 1);
    }
    $this->envelope = Array(
                              "meta" => Array(
                                "guid"    => $guid,
                                "command" => $command,
                                "session" => $sessionId
                              ),
                              "data" => $data
    );
    if( isset( $language ) )  {
      $this->envelope['meta']['language'] = $language;
    }
  }
  /**********************************************************
   * CreateEnvlope( $guid = null, $command = null, $sessionId = null, $data = null )
   */
  private function CreateResponseEnvelope( $guid = null, $command = null, $sessionId = null, $data = null, $status = null )
  {
    if( !isset( $guid, $command, $sessionId, $status ) || is_null( $guid) || is_null($command) || is_null($sessionId) || is_null($status) )  {
      throw new Exception("guid, commad, session, data or status is not defined", 1);
    }
    $this->envelope = Array(
                              "meta" => Array(
                                "guid"    => $guid,
                                "command" => $command,
                                "session" => $sessionId,
                                "status"  => $status
                              ),
                              "data" => $data
    );
    if( empty( $this->envelope["data"] ) )  {
      unset($this->envelope["data"]);
    }
    return( $this->envelope );
  }
  //-------------------------------------------------------
  //  HELPERS
  //-------------------------------------------------------

  /*******************************************************
   * checkEnvelopeMeta( $meta )
   *
   * checkEnvelopeMeta check if some needed variables are
   * missing in the meta section of an envelope
   *
   * @param array $meta is the meta section of an envelope
   * @
   */
  private function CheckEnvelopeMeta( $meta )
  {
    $errorList        = array();
    $errorMessage     = "";
    $foundAllEntries  = false;
    $kindOfEnvelope   = "";

    $kindOfEnvelope = ( !key_exists( "status", $meta) ) ? "request" : "response";
    $metaKeyList    = $this->m_metaKeyLists[ $kindOfEnvelope ];
    foreach($metaKeyList as $key => $envelopMetaEntry) {
      if( !isset( $meta[$envelopMetaEntry] ) || is_null( $meta[$envelopMetaEntry] ) || !key_exists( $envelopMetaEntry, $meta ) )  {
        array_push( $errorList, $envelopMetaEntry . " is missing or null !!!" );
      }
    }
    if( !empty( $errorList ) )  {
      foreach ($errorList as $key => $error) {
        $errorMessage = $errorMessage . $error . "\n";
      }
      if( sizeof( $errorList ) > 0 )  {
        LOG::error( __CLASS__, "(%s) %s", __LINE__, $errorMessage );
        throw new Exception($errorMessage, 1);
      }
    }
  }
  

  /**********************************************************
   * ChangeEnvelop( adminEnvelope, CppEnvelope )
   */

  static public function ChangeEnvelope( &$adminEnvelope, &$cppEnvelope )
  {
    if( $cppEnvelope == null || $cppEnvelope == "" )  {
      $cppEnvelope = Array(
        "meta" => Array(
          "status" => "404 - not found"
        ),
        "data" => null,
      );
    }
    if( Envelope::IsSingleEnvelope( $adminEnvelope ) && Envelope::IsSingleEnvelope( $cppEnvelope ) )  {
     // set security token if the server send a new one
     //if( isset( $cppEnvelope['meta']['securityToken'] ) && !empty( $cppEnvelope['meta']['securityToken'] ) )  {
     //  $adminEnvelope['meta']['securityToken'] = $cppEnvelope['meta']['securityToken'];
     //}
      $adminEnvelope['meta'] = $cppEnvelope['meta'];
     return true;
    }
    else if( Envelope::IsMultipleEnvelope( $adminEnvelope ) && Envelope::IsMultipleEnvelope( $cppEnvelope )  )  {
      //TODO: implement Multienvelopes
    };

  }
  /**********************************************************
   * IsSingleEnvelope( $envelope )
   */

  static private function IsSingleEnvelope( $envelope )
  {
    if( is_array( $envelope ) && isset( $envelope['meta'] ) && key_exists( "data", $envelope ) )  {
      return true;
    }
    return false;
  }

  /**********************************************************
   * IsMultipleEnvelope( $envelope )
   */

  static private function IsMultipleEnvelope( $envelope )
  {
    if( is_array( $envelope ) && sizeof( $envelope ) > 0 && isset( $envelope[0] ) )  {
      if( is_array( $envelope[0] ) && isset( $envelope[0]['meta'], $envelope[0]['data'] ) )  {
        return true;
      }
    }
    return false;
  }
}

?>