<?php
//-------------------------------------------------------
//  REQUIERMENTS
//-------------------------------------------------------

/****************************************************************
 * NamedPipe
 *
 * Communication between access5 server core (C++) and PHP
 *
 */
class NamedPipe
{

  //-------------------------------------------------------
  //  MEMBER VARIABLES
  //-------------------------------------------------------
  static private $m_namedPipeHandle;
  static private $m_waitForReadTime = 60;//seconds
  static private $m_securityToken   = null;
  static private $m_fingerprint     = null;
  const          m_namedPipeAddress = '/tmp/access5adm';


  //-------------------------------------------------------
  //  MEMBER FUNCTIONS
  //-------------------------------------------------------


  /*************************************************************
   * Send( $envelope )
   * @param envelope $envelope contains an envelope-json-string
   */
  static public function Send( $envelope )
  {
    if( !self::$m_namedPipeHandle )  {
      if( !self::Initialize() )  {
        LOG::error( __CLASS__, "(%s) can't initialise namedpipe handle!!!" , __LINE__ );
        LOG::error( __CLASS__, "(%s) get last error:(%s)" , __LINE__, error_get_last()['message'] );
        return false;
      }
    }

    // decode envelope for some necessary informations
    $envelope = json_decode( $envelope, true );
    $command  = $envelope['meta']['command'];
    if( $token = XHREnvelope::GetToken() )  {
      if( $envelope['meta']['session'] == "dev4B274CF7-3913-0DD1-E81C-7854DC3AEE56" )  {
       self::SetFingerPrint( hash( "sha384", "Secret" . $envelope['meta']['session'] . $token ) );
      }

      $envelope['meta']['fingerprint'] = NamedPipe::$m_fingerprint;//hash( "sha384", "Secret" . $envelope['meta']['session'] . $token );
    }
    else if( self::$m_securityToken )  {

      if( $envelope['meta']['session'] == "dev4B274CF7-3913-0DD1-E81C-7854DC3AEE56" )  {
        self::SetFingerPrint( hash( "sha384", "Secret" . $envelope['meta']['session'] . self::$m_securityToken ) );
      }

      $envelope['meta']['fingerprint'] = NamedPipe::$m_fingerprint;//hash( "sha384", "Secret" . $envelope['meta']['session'] . self::$m_securityToken );
    }
    else if( self::$m_fingerprint )  {
      $envelope['meta']['fingerprint'] = self::$m_fingerprint;
    }
    else  {
      LOG::debug( __CLASS__, "no token available" );
    }

    // encode envelope for writing it to the namepipe
    $envelope = json_encode( $envelope, JSON_UNESCAPED_UNICODE );
    
    /////////////////////////////////////////////////////////////
    //                       ATTENTION                         //
    //   linux needs file end marker ( \n\0 )                  //
    /////////////////////////////////////////////////////////////
    $envelope = $envelope . "\n\0";
    //$stringHandlePipe = "HANDLE_PIPE";
    $startSend = microtime(true);

    if( !self::Write($envelope) )  {
      LOG::error( __CLASS__, "(%s) write envelope to namedpipe failed!!!", __LINE__ );
      return false;
    }
    $stopSend = microtime(true);


    // read section
    $startRead = microtime(true);
    if( !$responsedEnvelope = self::Read() )  {
      LOG::error( __CLASS__, "(%s) read envelope form namedpipe failed!!!", __LINE__ );
      return false;
    }
    $stopRead = microtime(true);
    LOG::debug( __CLASS__, "command: %s |send: %f s | read: %f s", $command, $stopSend - $startSend, $stopRead - $startRead );
    
    $responsedEnvelope = json_decode( $responsedEnvelope, true );
    
    // prepare response for admin
    if( isset( $responsedEnvelope['meta']['securityToken'] ) )  {
      self::$m_securityToken = $responsedEnvelope['meta']['securityToken'];
    }
    if( isset( $responsedEnvelope['data']['securityToken']['new'] ) )  {
      self::$m_securityToken = $responsedEnvelope['data']['securityToken']['new'];
    }
    
    $responsedEnvelope = json_encode( $responsedEnvelope );
    return $responsedEnvelope;
  }


  /*************************************************************
   * IsServerRunning( )
   * @param envelope $envelope contains an envelope-json-string
   */
  public static function  IsServerRunning()
  {
    $namedPipe = self::Initialize();
    $return    = $namedPipe ? true : false;
    return $return ;
  }



  /*************************************************************
   * Read( )
   * @return envelope  return contains an evelope-json-string
   *
   */
  static private function Read()
  {
    // wait until time is over or namedPipe status is changed
    $waitToRead = Array( self::$m_namedPipeHandle );
    $null       = null;   // dummy parameter to make stream_select happy

    $readyToRead = stream_select( $waitToRead, $null, $null, self::$m_waitForReadTime );
    if( $readyToRead === 0 )  {
      // time is over and nothing is happend
      // try in few ms again
      $tries = 0;
      while( $tries >= 10 )  {
        $tries++;
        usleep(10);
        $readyToRead = stream_select( $waitToRead, $null, $null, self::$m_waitForReadTime );
        if( $readyToRead === 0 )  {
          return true;
        }
      }
      //echo __CLASS__ . ":" . __LINE__ . " time is over and nothing is happend | \$readyToRead " . var_export( $readyToRead, true ) . "<br>\n";
      return false;
    }
    elseif ( !$readyToRead ) {
      // something went worg
      //echo __CLASS__ . ":" . __LINE__ . " something went worg | \$readyToRead " . var_export( $readyToRead, true ) . "<br>\n";
      return false;
    }

  //$envelope = socket_read( self::$m_namedPipeHandle, 1024 * 8 );+
  $envelope     = fgets( self::$m_namedPipeHandle ); 
  //$envelope = socket_read ( self::$m_namedPipeHandle, 1024, PHP_NORMAL_READ  );
  
    // $envelope = fgets( self::$m_namedPipeHandle );
	LOG::debug( __CLASS__,"envelope (%s)", $envelope );
    //rewind( self::$m_namedPipeHandle );
    // strip leading \0
    if( $envelope[0] == "\0" )  {
      $envelope = substr( $envelope, 1 );
    }

    $envelopeArray = json_decode( $envelope, true );
    LOG::debug( __CLASS__,"(%s) envelope from server: \"command: %s\" | status: %s | data: %s", __LINE__, $envelopeArray['meta']['command'], $envelopeArray['meta']['status'], var_export( (isset($envelopeArray['data'])?$envelopeArray['data']:false == true), true ) );
    if( !isset($envelopeArray['meta'],$envelopeArray['meta']['command']) )  {
      LOG::debug(__CLASS__, "no command in response: %s", var_export( $envelope, true ));
    }
    if ( empty( $envelope ) )  {
      LOG::error( __CLASS__,"response empty!!!" );
    }
    return $envelope;
  }

  /*************************************************************
   * Initialize( )
   *
   */
  static private function Initialize()
  {
    $tries     = 0;
    $connected = false;
    if( self::isConnected() )  {
      return self::$m_namedPipeHandle;
    }
    while( $tries < 10 && !self::isConnected() ) {
      try{
        if(self::Connect()) {
          $connected = true;
        }
		else  {
			$tries++;
			LOG::debug(__CLASS__,"Pipe connecting try: %s", $tries);
			usleep(5000000);
		}
      }
      catch( Exception $e ){
        LOG::error(__CLASS__,"open pipe failed!! try: %s", $tries+1);
        LOG::error( __CLASS__, "(%s) get last error:(%s)" , __LINE__, error_get_last()['message'] );
      }
    }

    if( !self::isConnected() ) return false;

	LOG::debug(__CLASS__,"Pipe connect succeeds with %s tries", $tries);
    return self::$m_namedPipeHandle;
  }

  /*************************************************************
   * Terminate( )
   *
   */
  static private function Terminate()
  {

  }

  /*************************************************************
   * Write( $envelope )
   * @param envelope $envelope contains an envelope-json-string
   */
  static private function Write( $envelope )
  {
    $encoding = mb_detect_encoding( $envelope, 'UTF-8' );
    //LOG::debug(__CLASS__,"envlope character encoding: %s", $encoding);
    $waitToWrite    = Array( self::$m_namedPipeHandle );
    $readyToWrite   = stream_select( $null, $waitToWrite, $null, self::$m_waitForReadTime );
    if( $readyToWrite === 0 ){

    }
    if( !$readyToWrite )  {
      LOG::debug(__CLASS__,"something went wrong on write to the pipe");
    }
    $envelopeArray  = json_decode( $envelope, true );



    if( fwrite( self::$m_namedPipeHandle, $envelope ."" ) == false )  {
		$errorcode = socket_last_error();
		$errormsg = socket_strerror($errorcode);
		LOG::debug(__CLASS__,"socket_write: %s(%s)", $errormsg, $errorcode );
		return false;
	}
	return true;
   // return fwrite( self::$m_namedPipeHandle, $envelope ."\0" );
  }



  //-------------------------------------------------------
  //  LOW LEVEL MEMBER FUNCTIONS
  //-------------------------------------------------------

  static private function Connect( )
  {
    $tries = 0;
    self::$m_namedPipeHandle = stream_socket_client('unix://' . self::m_namedPipeAddress, $errno, $errstr);
	  if( self::$m_namedPipeHandle == false )  {
		  //$errorcode = socket_last_error();
		  //$errormsg = socket_strerror($errorcode);
          //LOG::debug(__CLASS__,"socket_create connecting try: %s(%s)", $errormsg, $errorcode );
		  return false;
	  }


    while($tries <= 10)  {
		$tries++;
		if( self::$m_namedPipeHandle = stream_socket_client('unix://' . self::m_namedPipeAddress, $errno, $errstr) )  {
		  return true;
		}
		$errorcode = socket_last_error();
		$errormsg = socket_strerror($errorcode);
		LOG::debug(__CLASS__,"socket_connect connecting try %s: %s(%s)", $tries, $errormsg, $errorcode );

		usleep(10);
    }
	self::Disconnect();
    return false;
  }

  static private function Disconnect( )
  {
    if( self::$m_namedPipeHandle === false )  return false;
    socket_close( self::$m_namedPipeHandle );
	self::$m_namedPipeHandle = false;
  }
  /*************************************************************
   * isConnected()
   *
   */
  static private function isConnected()
  {
    if( self::$m_namedPipeHandle )  {
      return true;
    }
    return false;
  }

  /*************************************************************
   * GetSecurityToken()
   *
   * @return string SecurityToken
   *
   */
  static public function GetSecurityToken()
  {
    $reponse = self::$m_securityToken;
    self::$m_securityToken = null;
    return $reponse;
  }

  /*************************************************************
   * SetFingerPrint( $fingerprint )
   *
   * @return string SecurityToken
   *
   */
  static public function SetFingerPrint( $fingerprint )
  {
    NamedPipe::$m_fingerprint = $fingerprint;
  }

}
?>