<?php
//=============================================================================
//
//  File: $(FILE_NAME)
//  (c) 2021 by CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          $(DAY).$(MONTH).$(YEAR)
//
//  Description:
//          describe the file 
//
//=============================================================================

class AdminCommandInvoker
{
  //-----------------------------------------------------------------------------
  //  InvokeCommand(): gets the current command and executes them.
  //                   in most of the case use the corresponding proxy
  //
  public static function InvokeCommand( $envelope, $command, $iniFile )
  {
    $responseEnvelope = AdminCommandInvoker::PrepareResponse( $envelope );
    $configProxy      = new ConfigProxy();
    $presenceProxy    = new PresenceProxy();
    $licenseProxy     = new LicenseProxy();
    $adminProxy       = new AdminProxy();

    switch( $command ) {
      // config commands
      case "GET_ENTITY":      return $adminProxy->GetEntity( $envelope['data']['entityId'] );
      case "ENTITY_CREATE":   return $adminProxy->Create( $envelope['data']['createParam'], $envelope['data']['lang'] );
      case "UPDATE_ENTITY":   return $adminProxy->UpdateEntity( $envelope['data']['entity'] );
      case "UPDATE_PARTIAL_ENTITY":   return $adminProxy->UpdatePartialEntity( $envelope['data']['partialEntity'], $envelope['data']['isMoved'] );
      case "MOVE_ENTITY":     return $adminProxy->MoveEntity( $envelope['data']['moveEntityId'], $envelope['data']['toEntityId'] );
      case "DELETE_ENTITY":   return $adminProxy->DeleteEntity( $envelope['data']['entityId'] ); // obsolete!
      case "DELETE_ENTITIES": return $adminProxy->DeleteEntities( $envelope['data']['entityIdList'] );
      case "SET_PASSWORD":    return $adminProxy->SetPassword( $envelope['data']['entityId'], $envelope['data']['password'], $envelope['data']['passwordKey'] );
      case "ADD_ENTITY":      return $adminProxy->AddEntity( $envelope['data']['entity'] );

      //compartment admin
      case "STARTUP_INFO":     return $adminProxy->StartupInfo();
      case "FETCH_MAILBOX":    return $adminProxy->FetchMailbox();
      case "DEVTREE":          return $adminProxy->DevTree();
      case "GET_ENTINFO_LIST": return $adminProxy->GetEntInfoList( $envelope['data']['entityIdList'], $envelope['data']['isLastRequest'] );
      case "PRESENCE":         return $adminProxy->Presence();
      case "ADDRESSBOOK":      return $adminProxy->AddressBook( $envelope['data']['entityId'], $envelope['data']['isLastRequest'] );
      case "ADDRESSBOOK_ADDRECORD":    return $adminProxy->AddressBookAddRecord   ( $envelope['data']['addrBookEntityId'], $envelope['data']['addressRecord'] );
      case "ADDRESSBOOK_UPDATERECORD": return $adminProxy->AddressBookUpdateRecord( $envelope['data']['addrBookEntityId'], $envelope['data']['addressRecord'] );
      case "ADDRESSBOOK_DELETERECORD": return $adminProxy->AddressBookDeleteRecord( $envelope['data']['addrBookEntityId'], $envelope['data']['addressRecordId'] );
      case "LOGFILE":                  return $adminProxy->Logfile();
      case "LOGFILE_DELETE":           return $adminProxy->LogfileDelete();
      case "LOGFILE_ADDRECORD":        return $adminProxy->LogfileAddRecord( $envelope['data']['logfileRecord'] );
      case "ADMIN_MESSAGES":           return $adminProxy->AdminMessages();
      case "MESSAGES_ADDRECORD":       return $adminProxy->MessagesAddRecord( $envelope['data']['messageRecord'] ); // not used!?
      case "MARK_REBOOT_DEVICE":       return $adminProxy->MarkRebootDevice( $envelope['data']['entityId'] );
      case "REBOOT_DEVICE":            return $adminProxy->RebootDevice( $envelope['data']['entityId'] );
      case "REBOOT_DEVICE_NEEDED":     return $adminProxy->RebootDeviceNeeded( );
      case "REBOOT_DEVICE_ALL":        return $adminProxy->RebootDeviceAll( );
      case "RESET_DEVICE_PARAMS":      return $adminProxy->ResetDeviceParameters( $envelope['data']['entityId'], $envelope['data']['params'] );
      case "QRCODE_REQUEST":           return $adminProxy->QrCodeRequest( $envelope['data']['entityId'] );
      case "QRCODE_DATA":              return $adminProxy->QrCodeData( $envelope['data']['entityId'] );
      case "KEYTONES":                 return $adminProxy->Keytones( $envelope['data']['entityId'], true );

      //rules
      case "RULE_LIST":                return $adminProxy->RuleList( );
      case "RULE_SOURCE":              return $adminProxy->RuleSource( $envelope['data']['ruleId']);
      case "RULE_DEBUG":               return $adminProxy->RuleDebug( $envelope['data']['ruleId'], $envelope['data']['isDebugEnabled'] );

      // devices api 
      case "SCAN_DEVICES":             return $adminProxy->ScanDevices( $envelope['data']['type'] );
      case "CAMERA_INFO":              return $adminProxy->CamaraInfo( $envelope['data']['onvifURL'], $envelope['data']['onvifUser'], $envelope['data']['onvifPass'], $envelope['data']['entityId'] );

      //compartment presence
      case "LIST": return $presenceProxy->GetList();

      //compartment license
      case "ACTIVATE_LICENSE_FILE":    return $licenseProxy->ActivateLicenseFile(  $envelope['data']['fileName'] );
    }
  }

  //-----------------------------------------------------------------------------
  //  PrepareResponse( $envelope ): set default value of response envelope
  //
  private static function PrepareResponse( $envelope )
  {
    $responseEnvelope                   = $envelope;
    $responseEnvelope["meta"]["status"] = "200 - Ok";
    $responseEnvelope["meta"]["param"]  = $envelope["data"];
    return $responseEnvelope;
  }
}