<?php
//=============================================================================
//
//  File: AdminConfig.php
//  (c) 2021 by CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          03.02.2021
//
//  Description:
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class AdminConfig
{
  //---------------------------------------------------------------------------
  // load dhcp-options file from config/admin folder
  //
  public static function GetDhcpOptions( $iniFile )
  {
    // prequisites
    $adminConfigPath = $iniFile['root']  . "config/admin/";
    if( !is_dir( $adminConfigPath ) || !is_file( $adminConfigPath . "dhcp.json") ) {
      return "";
    }

    // load file + return it converted to array
    $result = file_get_contents( $adminConfigPath . "dhcp.json" );
    return json_decode( $result, true );
  }

  public static function Get( $iniFile, $file )
  {
    // prequisites
    $adminConfigPath = $iniFile['root']  . "config/admin/";
    if( !is_dir( $adminConfigPath ) || !is_file( $adminConfigPath . $file . ".json") ) {
      return "";
    }

    // load file + return it converted to array
    $result = file_get_contents( $adminConfigPath . $file . ".json" );
    return json_decode( $result, true );

  }

  public static function ModuleInfo( $iniFile, $moduleName )
  {
    // prequisites
    $modulePath = $iniFile['root']  . "config/modules/" . $moduleName . "/";
    if( !is_dir( $modulePath ) || !is_file( $modulePath . "meta.json" ) ) {
      return "";
    }

    // load file + return it converted to array
    $result = file_get_contents( $modulePath . "meta.json" );
    return json_decode( $result, true );

  }

  public static function ModuleGetEntity( $iniFile, $moduleName, $entityId )
  {
    // prequisites
    $modulePath = $iniFile['root']  . "config/modules/" . $moduleName . "/";
    if( !is_dir( $modulePath ) || !is_file( $modulePath . "meta.json" ) ) {
      return "";
    }

    // load file + return it converted to array
    $result = file_get_contents( $modulePath . $entityId . ".json" );
    return json_decode( $result, true );

  }  


  public static function ModuleList( $iniFile )
  {
    // prequisites
    $modulePath = $iniFile['root']  . "config/modules/";
    if( !is_dir( $modulePath ) ) {
      return "";
    }

    $result = Array();

    // list all subfolders
    $dirContent = scandir( $modulePath );
    foreach( $dirContent as $entry ) {
      if( $entry != ".." 
       && $entry != "." 
       && is_dir( $modulePath . $entry ) 
       && is_file( $modulePath . $entry . "/meta.json" ) 
        ) {
        $meta = file_get_contents( $modulePath . $entry . "/meta.json" );
        $meta = json_decode( $meta, true );
        array_push( $result, $meta );
      }
    }

    return $result;
  }    
  

}

?>