<?php
//=============================================================================
//
//  File: $(FILE_NAME)
//  (c) 2021 by CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          $(DAY).$(MONTH).$(YEAR)
//
//  Description:
//          describe the file 
//
//=============================================================================

class BackupCommandInvoker
{

  //-----------------------------------------------------------------------------
  //  InvokeCommand(): gets the current command and executes them.
  //                   in most of the case use the corresponding proxy
  //
  public static function InvokeCommand( $envelope, $command, $iniFile )
  {
    $responseEnvelope = BackupCommandInvoker::PrepareResponse( $envelope );
    switch( $command ) {

      case "STATUS":
        $backupProxy = new BackupProxy();
        $response    = $backupProxy->status();

        if( empty( $response['data'] ) ) {
          $responseEnvelope["meta"]["status"] = "503 - Ok";
        }

        return $responseEnvelope;

      case "RESTART":
        $backupProxy                        = new BackupProxy();
        $restartResult                      = $backupProxy->restart( $envelope['data']['uid'] );
        $responseEnvelope["meta"]["status"] = "503 - Ok";
        $responseEnvelope["data"]           = $restartResult['data'];
        return $responseEnvelope;

      case "REBOOT":
        $backupProxy                        = new BackupProxy();
        $restartResult                      = $backupProxy->reboot();
        $responseEnvelope["meta"]["status"] = "503 - Ok";
        $responseEnvelope["data"]           = $restartResult;
        return $responseEnvelope;

      // Create backup.
      case "CREATE":
        $backupProxy                        = new BackupProxy();
        return $backupProxy->Create( );

      // Backup status request.
      case "CREATE.STATUS":
        $backupProxy                        = new BackupProxy();
        return $backupProxy->CreateStatus( );

      // Validate backup file.
      case "INVESTIGATE":
        $backupProxy                        = new BackupProxy();
        return $backupProxy->Investigate( $envelope['data']['file'] );

      // Validate backup file.
      case "RESTORE":
        $backupProxy                        = new BackupProxy();
        return $backupProxy->Restore( $envelope['data']['file'] );
    }
  }

  //-----------------------------------------------------------------------------
  //  PrepareResponse( $envelope ): set default value of response envelope
  //
  private static function PrepareResponse( $envelope )
  {
    $responseEnvelope                   = $envelope;
    $responseEnvelope["meta"]["status"] = "200 - Ok";
    return $responseEnvelope;
  }
}