<?php
//=============================================================================
//
//  Firstinstall
//
//  Check if the admin ui should show the install wizzard. 
//  There are two criterias:
//
//  1. The file "installed.json" does NOT exist in /config/admin/
//  2. The main admin "admin.1" has the first admin password
// 
//  install-password ("402c399b-736a-497d-a55f-89927c149082") 
//  content for "admin.1.pass.json":
//  {
//    "password": "data:application\/siedle-encrypted;base64,U2FsdGVkX1\/vckWuc7dKCzY1eBgbDorAv+d99gT9RaZ2ZxZ5KwyUQRq2ydSUWzgRNdTr46sEHFu+v5fSVrkwEg=="
//  }
//  
//  standard-password ("admin")
//  content for "admin.1.pass.json":
//  {
//    "password": "data:application\/siedle-encrypted;base64,U2FsdGVkX181IrQnD6bJM+N1uente0BBl7yK2uDB8DA="
//  }
//
//=============================================================================


class FirstInstall
{
  //---------------------------------------------------------------------------
  //  data
  //
  static $firstAdminUser = "admin";
  static $firstAdminPassword = "402c399b-736a-497d-a55f-89927c149082";
  static $firstAdminFilePassword = "data:application/siedle-encrypted;base64,U2FsdGVkX1/vckWuc7dKCzY1eBgbDorAv+d99gT9RaZ2ZxZ5KwyUQRq2ydSUWzgRNdTr46sEHFu+v5fSVrkwEg==";

  //---------------------------------------------------------------------------
  //  interface
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  Check for trigger file and the first install password. 
  //  Set/Remove cookie, if needed.
  //
  public static function Manage( $iniFile )
  {
    $hasInstalledFile = file_exists( $iniFile['root'] . "config/admin/installed.json" );

    // case 1: this is new installation
    if(  ! $hasInstalledFile 
     &&  FirstInstall::CheckFirstInstallPassword( $iniFile ) 
      ) {
      SESSION::Authorize( FirstInstall::$firstAdminUser, FirstInstall::$firstAdminPassword );
      FirstInstall::SetFirstInstallCookie();
    }
    // case 2: this is an update
    else if( ! $hasInstalledFile )  {
      FirstInstall::FinishedUpdated( $iniFile );
    }
    else {
      FirstInstall::DeleteFirstInstallCookie();
    }

  }

  //---------------------------------------------------------------------------
  //  Check for secret installed password. Use 
  //  Set/Remove cookie, if needed.
  //
  public static function CheckFirstInstallPassword( $iniFile )
  {
    $admin1PasswordFile = file_get_contents( $iniFile['data'] . "data/confstore/admin.1.pass.json" );
    $admin1Password = json_decode( $admin1PasswordFile, true );
    
    if( ( key_exists( 'password', $admin1Password ) )
     && ( $admin1Password['password'] == FirstInstall::$firstAdminFilePassword )
      ) {
      return true;
    }
    return false;
  }   

  //---------------------------------------------------------------------------
  //  Check for trigger file and the first install password. 
  //  Set/Remove cookie, if needed.
  //
  public static function RebootSystem( )
  {
    $backupProxy = new BackupProxy();
    $backupProxy->reboot();
  } 

  //---------------------------------------------------------------------------
  //  Check for trigger file and the first install password. 
  //  Set/Remove cookie, if needed.
  //
  public static function Finished( $iniFile )
  {
    // build installed-file to disable wizard
    $time = time();
    $version = "\"" . $iniFile['version'] . "\"";
    $installed = "{\n  \"installed\": $time,\n  \"version\": $version\n}\n";
    file_put_contents( $iniFile['root'] . "config/admin/installed.json", $installed );

    FirstInstall::DeleteFirstInstallCookie();
  }


  //---------------------------------------------------------------------------
  //  Check for trigger file and the first install password. 
  //  Set/Remove cookie, if needed.
  //
  public static function FinishedUpdated( $iniFile )
  {
    // build installed-file to disable wizard
    $time = time();
    $installed = "{\n  \"installed\": $time,\n  \"version\": \"updated\"\n}\n";
    file_put_contents( $iniFile['root'] . "config/admin/installed.json", $installed );

    FirstInstall::DeleteFirstInstallCookie();
  }  

  //---------------------------------------------------------------------------
  //  helper functions
  //---------------------------------------------------------------------------

  public static function SetFirstInstallCookie()
  {
    $options = [
      'expires' => time() + SESSION::TIMEOUT,
      'samesite' => "Strict",
      'secure' => true
    ];
    setcookie( "firstinstall", "true", $options );
  }

  public static function DeleteFirstInstallCookie()
  {
    setcookie( "firstinstall", null, -1 );    
  }

}

?>