<?php
//=============================================================================
//
//  File: MailboxInvoker
//  (c) 2021 by CYTEL Technology AG
//
//  Editor: Peter Aidam
//          30.11.2021
//
//  Description:
//    "Long polling" implementation for mailbox messages.
//
//=============================================================================

class MailboxInvoker
{

  //-----------------------------------------------------------------------------
  //  Execute command.
  //
  public static function InvokeCommand( $envelope, $command, $iniFile )
  {
    // Prerequisites.
    $adminProxy       = new AdminProxy();

    // "MAILBOX.LONG_POLLING"
    if( $command == "LONG_POLLING" ) {

      // Poll for mailbox messages. After a time end the loop so it can be restarted.
      // nginx restricts php lifetime to 60 secs. So don't loop here too long! Otherwise
      // error "504 (Gateway Time-out)"!
      for( $i=0; $i < 50; $i++ ) {   // 50 * 0.5 sec

        // Poll for mailbox contents: Fetch mailbox from admin compartment.
        $responseEnvelope = $adminProxy->FetchMailbox();

        // If there is something in the mailbox, tell JS application.
        $mailbox = $responseEnvelope['data']['mailbox'];
        if( sizeof( $mailbox ) > 0 )  {

          // Change 'command' back to origin, so response doesn't confuse JS code.
          $responseEnvelope['meta']['command'] = "MAILBOX.LONG_POLLING";          

          return $responseEnvelope;
        }

        // If not, wait a moment and try again.
        usleep( 500000 ); // 0.5 sec

      } // for
  
      // Return to application. Script will be restarted.
      $responseEnvelope = $envelope;
      $responseEnvelope["meta"]["status"] = "408 - Request Timeout";
    }
    else  {
      $responseEnvelope = $envelope;
      $responseEnvelope["meta"]["status"] = "404 - Method not found";
    }
    
    return $responseEnvelope;
  }

}