<?php

class Support
{
  private static $m_resultPath    = "";
  private static $m_dataPath      = "";
  private static $m_confstorePath = "";
  private static $m_userDataPath  = "";
  private static $m_logPath       = "";

  //-----------------------------------------------------------------------------
  //  Build support package and return filename
  //
  public static function GetInformation( $iniFile )
  {
    set_time_limit( 10800 );    //max execution time 3hours
    
    // build path to support and result files
    Support::DefinePaths( $iniFile );

    // remove previously created support packages
    array_map( 'unlink', glob( self::$m_resultPath . "support*.zip" ) );

    // build filename
    $fileName = "support" . date( 'Y_m_d' ) . ".zip";

    // initialise zip file;
    $zip = new ZipArchive();
    if( !$zip->open( self::$m_resultPath . $fileName, ZipArchive::CREATE ) ) {
      LOG::error( __FILE__, "(%s) cannot open zip file", __LINE__ );
      return false;
    };

    //collect all necessary date from agw server
    $zip->addEmptyDir( "support" );

    self::zipFiles( self::$m_confstorePath, $zip, "support/" );
    self::zipFiles( self::$m_userDataPath,  $zip, "support/" );
    self::zipFiles( self::$m_logPath,       $zip, "support/" );

    $zip->close();

    return $fileName;
  }

  //-----------------------------------------------------------------------------
  //  prepare all paths
  //
  public static function DefinePaths( $iniFile )
  {
    self::$m_resultPath    = $iniFile['root']  . "upload/";
    self::$m_dataPath      = $iniFile['data']  . "data/";
    self::$m_confstorePath = self::$m_dataPath . "confstore";
    self::$m_userDataPath  = self::$m_dataPath . "userdata";
    self::$m_logPath       = $iniFile['log'];
  }

   /**
   *  zipFiles( $src, &$zip )
   *
   *  add recursive files and folders to support.zip
   *
   * @param $src  folder or files path
   * @param &$zip ZipArchive object
   */

  private static function zipFiles( $src, &$zip, $dest = null )
  {
    // check extension is loaded
    if( !extension_loaded( "zip" ) ) {
      LOG::error( __FILE__, "(%s) missing extension \"zip\" ", __LINE__ );
      return false;
    }

    // check source file exists
    if( !file_exists( $src ) ) {
      LOG::error( __FILE__, "(%s) missing source file or folder : %s ", __LINE__, $src );
      return false;
    }

    if( !isset( $zip ) ) {
      LOG::error( __FILE__, "(%s) missing zip file : %s ", __LINE__, $zip );
      return false;
    }
    if( !isset( $dest ) ) {
      $dest = "";
    }
    if( is_dir( $src ) ) {
      //LOG::info( __FILE__, "(%s) create zip folder : %s ", __LINE__, $dest . basename( $src ) );
      $zip->addEmptyDir( $dest . basename( $src ) );
      $files = scandir( $src );
      $dest  = $dest . basename( $src ) . "/";
      foreach( $files as $key => $file ) {

        // do not add password files
        if( strpos( $file, ".pass." ) !== false )  {
          continue;
        }

        if( $file != "." && $file != ".." ) {
          self::zipFiles( $src . "/" . $file, $zip, $dest );
        }

      }
    }

    if( is_file( $src ) ) {
      //LOG::info( __FILE__,"(%s) copy '%s' to '%s'",__LINE__, $src, $dest . basename( $src )  );
      $zip->addFile( $src, $dest . basename( $src ) );
    }
    // gather all data for the zip file from folder support
  }
}


?>