<?php
//=============================================================================
//
//  MODULE FILE
//  (c) 2022, CYTEL AG
//  
//  Manages a module file including signature
//  
//=============================================================================

class ModuleFile
{
  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  private $publicKeyFile = null;
  private $privateKeyFile = null;

  const CERT_PASSWORD = "987b3499-3906-4649-85a1-c95fd74c0c52";
  const ZIP_PASSWORD = "c95fd74c0c52";


  //-----------------------------------------------------------------
  //  CONSTRUCT
  //-----------------------------------------------------------------
  
  function __construct( $publicKeyFile, $privateKeyFile = null )
  {
    if( file_exists( $publicKeyFile ) ) {
      $this->publicKeyFile = $publicKeyFile;
    }
    if( file_exists( $privateKeyFile ) ) {
      $this->privateKeyFile = $privateKeyFile;
    }
  }

  //-----------------------------------------------------------------
  //  METHODS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  unpack and verify a module package
  //
  function Unpack( $moduleFileName, $moduleFolder )
  {
    // extract module archive
    $zip = new ZipArchive();
    $zip->open( $moduleFileName );
    $zip->setPassword( ModuleFile::ZIP_PASSWORD );
    $meta = $zip->getFromName( "meta.json" );
    $meta = json_decode( $meta, true );
    $packetId = $meta['packet-id'];

    mkdir( $moduleFolder . $packetId );
    $zip->extractTo( $moduleFolder . $packetId );    
    $zip->close();

    // check signatures
    $dirContent = scandir( $moduleFolder . $packetId );
    $dirContent = array_filter( $dirContent, function( $fileName ) {
      return $fileName != "." && $fileName != ".." && $fileName != ".signatures";
    } );

    // check all signature files
    foreach( $dirContent as $fileName ) {
      if( $fileName != "meta.json" ) {
        $result = $this->CheckSignatureFile( "$moduleFolder$packetId/$fileName", "$moduleFolder$packetId/.signatures/$fileName.sign" );
        if( $result != 0 )  {
          return false;
        }
      }
    }
    
    return $meta;
  }

  //-----------------------------------------------------------------
  //  create module package
  //
  function Create( $moduleName, $moduleFolder, $resultFolder )
  {
    // prequisites
    if( !file_exists( $moduleFolder . "meta.json" ) )  {
      echo "Error: ModuleFile:Create(), missing 'meta.json'";
      return false;
    }

    $dirContent = scandir( $moduleFolder );
    $dirContent = array_filter( $dirContent, function( $fileName ) {
      return $fileName != "." && $fileName != "..";
    } );
    if( count( $dirContent ) == 0 )  {
      echo "Error: ModuleFile:Create(), empty directory '$moduleFolder'";
      return false;
    }

    // build all signature files
    foreach( $dirContent as $fileName ) {
      $this->CreateSignatureFile( $moduleFolder . $fileName );
    }

    // delete existing result file 
    $zipFileName = $resultFolder . $moduleName . ".modpk";
    if( file_exists( $zipFileName ) ) {
      unlink( $zipFileName );
    }

    // create zip file 
    $zip = new ZipArchive();
    $zip->open( $zipFileName, ZIPARCHIVE::CREATE );
    $zip->setPassword( ModuleFile::ZIP_PASSWORD );
    foreach( $dirContent as $fileName ) {
      $zip->addFile( $moduleFolder . $fileName, $fileName );
      if( $fileName != "meta.json" )  {
        $zip->setEncryptionName( $fileName, ZipArchive::EM_AES_128, ModuleFile::ZIP_PASSWORD  );
        $zip->addFile( $moduleFolder . $fileName . ".sign", ".signatures/" . $fileName . ".sign" );
        $zip->setEncryptionName( ".signatures/" . $fileName . ".sign", ZipArchive::EM_AES_128, ModuleFile::ZIP_PASSWORD );
      }
    }
    $zip->close();

    // tidy up
    foreach( $dirContent as $fileName ) {
      unlink( $moduleFolder . $fileName . ".sign" );
    }

  }


  //-----------------------------------------------------------------
  //  HELPER
  //-----------------------------------------------------------------

  function CreateSignatureFile( $fileName )
  {
    $resultCode = 0;
    $output = null;
    $commandLine = "openssl dgst -sha256 -sign $this->privateKeyFile -out $fileName.sign -passin pass:" . ModuleFile::CERT_PASSWORD . " $fileName";
    exec( $commandLine, $output, $resultCode ); 
    return $resultCode;
  }  

  function CheckSignatureFile( $fileName, $signatureFileName )
  {
    $resultCode = 0;
    $output = null;
    $commandLine = "openssl dgst -sha256 -verify $this->publicKeyFile -signature $signatureFileName $fileName";
    exec( $commandLine, $output, $resultCode ); 
    return $resultCode;
  }    

}


?>