<?php
//=============================================================================
//
//   XHR Script for agw status page application
// 
//=============================================================================

//-----------------------------------------------------------------------------
//  INITIALIZE
//-----------------------------------------------------------------------------

// -- debug --
//ob_start( );
$queryStart = microtime( true );

// -- include --
include_once __DIR__ . "/app/php_shared/AccessLib.php";
include_once( __DIR__ . "/app/rest_commands/AdminConfig.php" );
include_once( __DIR__ . "/app/rest_commands/AdminCommandInvoker.php" );
include_once( __DIR__ . "/app/rest_commands/AdminConfigCommandInvoker.php" );
include_once( __DIR__ . "/app/rest_commands/BackupCommandInvoker.php" );
include_once( __DIR__ . "/app/rest_commands/SessionCommandInvoker.php" );
include_once( __DIR__ . "/app/rest_commands/CosconfCommandInvoker.php" );
include_once( __DIR__ . "/app/rest_commands/PermissionCommandInvoker.php" );
include_once( __DIR__ . "/app/rest_commands/Support.php" );
include_once( __DIR__ . "/app/rest_commands/SupportCommandInvoker.php" );
require_once( __DIR__ . "/app/rest_commands/FirstInstall.php" );
require_once( __DIR__ . "/app/rest_commands/FirstInstallInvoker.php" );
require_once( __DIR__ . "/app/rest_commands/MailboxInvoker.php" );


//-----------------------------------------------------------------------------
//  DEFINES / READ CONFIG
//-----------------------------------------------------------------------------

$iniFile = parse_ini_file( "../../access.ini" );
$wwwRoot = pathinfo( __FILE__ )['dirname'] . "/";

define( "LOG_PATH", $iniFile['log'] );

//-----------------------------------------------------------------------------
//  START APPLICATION
//-----------------------------------------------------------------------------

// -- decode request data --
$rawData      = file_get_contents( 'php://input' );
$envelope     = json_decode( $rawData, true );
$command      = explode( ".", $envelope['meta']['command'] );
$action       = $command[ 1 ];
$compartment  = $command[ 0 ];
$response     = "";

// Special treatment for commands like BACKUP.CREATE.STATUS!!
if( sizeof( $command ) > 2 && is_string( $command[ 2 ] ) )  {
  $action = $command[ 1 ] . '.' .  $command[ 2 ]; // like CREATE.STATUS
}

//=============================================================================
//  INIT PROXIES
//=============================================================================
$sipSbcProxy   = new SipSbcProxy();
$backupProxy   = new BackupProxy();
$configProxy   = new ConfigProxy();
$presenceProxy = new PresenceProxy();
$adminProxy    = new AdminProxy();
$backupProxy   = new BackupProxy();

// -- session management --
SESSION::Start( false );
$envelope['meta']['session'] = SESSION::Get();
$isAuthorized = SESSION::IsAuthorized();
FirstInstall::Manage( $iniFile );

//-----------------------------------------------------------------------------
//  SERVER RUNNING CHECK
//-----------------------------------------------------------------------------

$response = $envelope;

// check backup pipe before using it
// Note 24.08.2021: As a consequence of these lines the whole
// system doesn't work without Backup service!!
if( !BackupPipe::IsServerRunning( ) ) {
  $response["meta"]["status"] = "503 - Services down!";
  $options = [
    'expires' => time() + + SESSION::TIMEOUT,
    'samesite' => "Strict",
    'secure' => true
  ];
  setcookie( "connected", "false", $options );  
  exit(json_encode( $response, JSON_PRETTY_PRINT ));
}
else {
  $options = [
    'expires' => time() + + SESSION::TIMEOUT,
    'samesite' => "Strict",
    'secure' => true
  ];
  setcookie( "connected", "true", $options );  
}


$restartResult = $backupProxy->status();


// backup is rebootin services
if( $restartResult['data']['status']  === "REBOOTING" ) {
  $response["meta"]["status"] = "503 - Services restarting!";
}

// backup is restoring system
else if( $restartResult['data']['status']  === "RESTORE" ) {
  $response["meta"]["status"] = "503 - Services restore!";
}

// check pipe of admin service
else if( !NamedPipe::IsServerRunning( ) ) {
  $response["meta"]["status"] = "503 - Services down!";
}

// remark: Do NOT combine both if/else if clauses to ONE like
//  if( $restartResult['data']['status'] === "REBOOTING" ) || !NamedPipe::IsServerRunning( ) ) {
// php calles NamedPipe::IsServerRunning even if the first if clause is true!

//-----------------------------------------------------------------------------
//  REQUEST MANAGEMENT
//-----------------------------------------------------------------------------

// for security reasons don't create a generic proxy
// all command should be forbidden by default
// commands which are needed for the web ui can be whitelisted

else if( SESSION::IsAuthorized() === false
  && !($compartment == "SESSION" && $action == "LOGIN") )  {
  $envelope['meta']['status'] = "401 Unauthorized!";
  $envelope['data'] = null;
  echo json_encode($envelope);
  exit();
}
else  {

  switch ( $compartment )  {

    // ------ login/logoff -----------
    case "SESSION":  
      $response = SessionCommandInvoker::InvokeCommand( $envelope, $action, $iniFile ); 
      break;

    // ------ first install wizard -----------
    case "FIRSTINSTALL":  
      $response = FirstInstallInvoker::InvokeCommand( $envelope, $action, $iniFile ); 
      break;

    // ------ OS Config Control -----------
    case "COSCONF": 
      $response = CosconfCommandInvoker::InvokeCommand( $envelope, $action, $iniFile ); 
      break;

    // ------- Status Command -----------
    case "STATUS":
      $response = AdminCommandInvoker::InvokeCommand( $envelope, $action, $iniFile ); 
      break;

    // ------- Backup Commands -----------
    case "BACKUP":  
      $response = BackupCommandInvoker::InvokeCommand( $envelope, $action, $iniFile ); 
      break;

    // ------ Admin Config Files -----------
    case "ADMINCONFIG":   
      $response = AdminConfigCommandInvoker::InvokeCommand( $envelope, $action, $iniFile ); 
      break;

    // ------ Admin Commands -----------
    case "CONFIG":
    case "PRESENCE":
    case "LICENSE":
    case "ADMIN":
      $response = AdminCommandInvoker::InvokeCommand( $envelope, $action, $iniFile ); 
      break;

    // ------ Permission Commands -----------
    case "PERMISSION":
      $response = PermissionCommandInvoker::InvokeCommand( $envelope, $action, $iniFile ); 
      break;

    // ------ Permission Commands -----------
    case "SUPPORT":
      $response = SupportCommandInvoker::InvokeCommand( $envelope, $action, $iniFile ); 
      break;

    // ------ Mailbox Commands --------------
    case "MAILBOX":
      $response = MailboxInvoker::InvokeCommand( $envelope, $action, $iniFile );
      break;

    default:{
      LOG::error( basename( __FILE__ ), "(%d) command not found!", __LINE__ );
      break;
    }
  }
}

//-----------------------------------------------------------------------------
//  RESPONSE TO UI
//-----------------------------------------------------------------------------

echo json_encode( $response, JSON_PRETTY_PRINT );
LOG::debug( basename( __FILE__ ), "(%d) query duration: %d!", __LINE__, ( microtime( true ) - $queryStart) );
//while( @ob_end_flush( ) );
?>
