/*
*  Views2 Framework
*  walk each node recursive 
*/

!function( $ )
{
  $.walk = function( obj, func )
  {
    // loop each property in object/array
    for( var key in obj ) {

      if( obj.hasOwnProperty( key ) ) {
        if( obj[ key ] instanceof Array ) {
          $.walk( obj[ key ], func );
        }
        else if( obj[ key ] instanceof Object ) {
          $.walk( obj[ key ], func );
        }
        else {
          var result = func( key, obj[ key ] );
          if( result )  {
            obj[ key ] = result;
          }
          
        }
      }
    }
  };

}( window.jQuery );
 /*
 *  Views2 - Web Framework 
 *  create guid
 */

!function( $ ) {

  // ----------------------------------------------------
  //   plugin function
  //
  $.guidgen = function( )
  {
    var d = new Date( ).getTime( );
    var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace( /[xy]/g, function( c )  {
      var r = ( d + Math.random( ) * 16 ) % 16 | 0;
      d = Math.floor( d / 16 );
      return( c == 'x' ? r : ( r & 0x3 | 0x8 ) ).toString( 16 );
    } );
    return uuid.toLowerCase( );
  };
  $.guid = $.guidgen;

  $.uidgen = function()
  {
    var firstPart = ( Math.random() * 46656 ) | 0;
    var secondPart = ( Math.random() * 46656 ) | 0;
    firstPart = ( "000" + firstPart.toString( 36 ) ).slice( -3 );
    secondPart = ( "000" + secondPart.toString( 36 ) ).slice( -3 );
    let uuid = firstPart + secondPart;
    return uuid.toLowerCase ();
  };
  $.uuid = $.uidgen;

  
}( window.jQuery );
 /*
 *  Views2 Web Framework
 *  clone
 */

!function( $ ) {

  // ----------------------------------------------------
  //   plugin default values
  //
  $.clone = function( obj, exceptions = [] )
  {
    var copy;

    // Handle the 3 simple types, and null or undefined
    if( null == obj || "object" != typeof obj )
      return obj;
  
    // Handle Date
    if( obj instanceof Date ) {
      copy = new Date( );
      copy.setTime( obj.getTime() );
      return copy;
    }
  
    // Handle Array
    if( obj instanceof Array ) {
      copy = [ ];
      for( var i = 0, len = obj.length; i < len; i++ ) {
        copy[ i ] = $.clone( obj[ i ] );
      }
      return copy;
    }
  
    // Handle Object
    if( obj instanceof Object ) {
      copy = {};
      for( var attr in obj ) {
        if( obj.hasOwnProperty( attr ) && exceptions.indexOf( attr ) < 0 ) {
          copy[ attr ] = $.clone( obj[ attr ] );
        }
      }
      return copy;
    }    
  };

  
}( window.jQuery );
 //=============================================================================
//
//  ARRAY Helper functions
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          24.10.2019
//=============================================================================

/* ---------------------------------------------------------------------------
 *   ARRAY HELPER
 * ------------------------------------------------------------------------ */

!function( $ ){

  $.array = function()
  {
  };

  //-------------------------------------------------------
  //  returns all elements from <target> that are not
  //  in <excludeList>(s)
  //
  $.array.NotIn = function( target, excludeList, excludeList2, excludeList3 )
  {
    var result = [];
    for( var i = 0; i < target.length; i++ ) {
      if( excludeList.indexOf( target[i] ) >= 0 ) {
        continue;
      }
      else if( Array.isArray( excludeList2 )
        && excludeList2.indexOf( target[i] ) >= 0
      ) {
        continue;
      }
      else if( Array.isArray( excludeList3 )
        && excludeList3.indexOf( target[i] ) >= 0
      ) {
        continue;
      }
      else {
        result.push( target[i] );
      }
    }
    return result;
  };

  //-------------------------------------------------------
  //  returns all elements from <target> that are 
  //  in <includeList>
  //
  $.array.In = function( target, includeList )
  {
    var result = [ ];
    for( var i = 0; i < target.length; i++ ) {
      if( includeList.indexOf( target[ i ] ) >= 0 ) {
        result.push( target[ i ] );
      }
    }
    return result;
  };

  //-------------------------------------------------------
  //  returns the first elements from <target> that is
  //  found in <includeList>
  //
  $.array.FirstIn = function( target, includeList )
  {
    for( var i = 0; i < target.length; i++ ) {
      if( includeList.indexOf( target[i] ) >= 0 ) {
        return target[ i ];
      }
    }
    return null;
  };

  //-------------------------------------------------------
  //  returns is one of the elements in <includeList>
  //  is found in <target>
  //
  $.array.HasOneIn = function( target, includeList )
  {
    for( var i = 0; i < target.length; i++ ) {
      if( includeList.indexOf( target[i] ) >= 0 ) {
        return true;
      }
    }
    return false;
  };

  $.array.Has = function( target, search )
  {
    return ( target.indexOf( search ) >= 0 );
  };

  //-------------------------------------------------------
  //  returns and array from both arrays (unique)
  //
  $.array.Join = function( array1, array2 )
  {
    for( var i = 0; i < array2.length; i++ )  {
      if( array1.indexOf( array2[ i ] ) < 0 )  {
        array1.push( array2[ i ] );
      }
    }
    return array1;
  };

  
  //-------------------------------------------------------
  //  returns an array from both arrays
  //
  $.array.MutualExclude = function( array1, mutualExclude )
  {
    var mutualFound = null;
    for( var i = 0; i < array1.length; i++ )  {
      
      if( mutualFound == null )  {
        if( mutualExclude.indexOf( array1[ i ] ) >= 0 )  {
          mutualFound = array1[ i ];
        }
      }
      else if( mutualExclude.indexOf( array1[ i ] ) >= 0 )  {
        array1.splice( i, 1 );
        i--;
      }
    }
    return array1;
  };


  //-------------------------------------------------------
  //  returns and array from both arrays
  //
  $.array.Remove = function( array1, exclude )
  {
    if( Array.isArray( exclude ) )  {
      for( let i = 0; i < array1.length; i++ )  {
        if( exclude.indexOf( array1[ i ] ) >= 0 )  {
          array1.splice( i, 1 );
          i--;
        }
      }
    }
    else if( $.isFunction( exclude ) )  {
      for( let i = 0; i < array1.length; i++ ) {
        if( exclude( array1[ i ] ) === true ) {
          array1.splice( i, 1 );
          i--;
        }
      }
    }
    else if( typeof exclude == 'string' )  {
      let i = array1.indexOf( exclude );
      if( i >= 0 )  {
        array1.splice( i, 1 );
      }
    }

    return array1;
  };

  //-------------------------------------------------------
  //  add item if not in array
  //
  $.array.Add = function( array1, item )
  {
    if( array1.indexOf( item ) < 0 )  {
      array1.push( item );
    }
    return array1;
  };
  

  //-------------------------------------------------------
  //  toggle two values within an array
  //
  $.array.Toggle = function( target, toggleArray )
  {
    let currentIndex = $.array.FirstIndexIn( target, toggleArray );
    let newToggleIndex;
    let currentToggleIndex = 0;
    if( currentIndex >= 0 )  {

      // get toggle 0 or 1
      currentToggleIndex = toggleArray.indexOf( target[ currentIndex ] );
      if( currentToggleIndex > 0 ) newToggleIndex = 0;
      else newToggleIndex = 1;

      // remove existing
      target.splice( currentIndex, 1 );
    }

    target.push( toggleArray[ newToggleIndex ] );

    return toggleArray[ newToggleIndex ];
  };

}( window.jQuery );
﻿//=============================================================================
//
//  OBJECT Helper functions
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          27.09.2021
//=============================================================================

/* ---------------------------------------------------------------------------
 *   ARRAY HELPER
 * ------------------------------------------------------------------------ */

!function( $ )
{

  $.object = function()
  {
  };

  //-------------------------------------------------------
  //  returns all elements from <target> that are not
  //  in <excludeList>(s)
  //
  $.object.Has = function( target, path )
  {
    let result = null;
    let pathList = path.split( "." );
    let currentObject = object;

    for( let key of pathList ) {
      if( key in currentObject ) {
        result = currentObject = currentObject[key];
      }
    }
    return ( result != null );
  };

  $.object.GetPathValue = function( object, path )
  {
    let result = null;
    let pathList = path.split( "." );
    let currentObject = object;

    for( let key of pathList ) {
      if( key in currentObject ) {
        result = currentObject = currentObject[key];
      }
    }
    return result;
  };

  $.object.SetPathValue = function( object, path, newValue )
  {
    let result = null;
    let pathList = path.split( "." );
    let currentObject = object;

    for( let key of pathList ) {
      if( key in currentObject ) {
        result = currentObject = currentObject[key];
      }
    }

    if( result ) {
      result = newValue;
    }

  };


}( window.jQuery );
﻿//=============================================================================
//
//  CYTEL Views2 - Web Framework
//  
//  Mutation-Observer for building a destructor
//  on a DOM-ELement class
//
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class MutationApi
{
  //-------------------------------------------------------
  //  BUILD DATA
  //-------------------------------------------------------
  
  constructor( )
  {
    this.etagList = { };
    this.observer = null;
  }

  //-------------------------------------------------------
  //  WORKER
  //-------------------------------------------------------  

  Start( )
  {
    var mself = this;
    if( this.observer == null )  {
      const target = document.body;
      const options = { attributes: false, childList: true, characterData: false, subtree: true };
      this.observer = new MutationObserver( ( mutations ) => {
        mself.HandleMutations( mutations );
      } );
      this.observer.observe( target, options );
    }
  }

  Stop()
  {
    if( this.observer != null )  {
      this.observer.disconnect();
      this.observer = null;
    }
  }

  //-------------------------------------------------------
  //  OBSERVER CODE
  //-------------------------------------------------------  
  
  HandleMutations( mutations )
  {
    var removedList = [];
    mutations.forEach( ( mutation ) => {
      if( mutation.removedNodes.length > 0 )  {
        this.RecurseMutations( mutation.removedNodes, removedList );
      }
    } );  

    for( var i = 0; i < removedList.length; i++ )  {
      this.HandleRemovedETag( removedList[ i ] );
    }
  }

  RecurseMutations( nodeList, removedList )
  {
    for( var i = 0; i < nodeList.length; i++ )  {
      var node = nodeList[ i ];
      if( node.dataset  &&  'etag' in node.dataset )  {
        removedList.push( node.dataset['etag'] );
      }
      if( node.hasChildNodes( ) )  {
        this.RecurseMutations( node.childNodes, removedList );
      }
    }
  }

  HandleRemovedETag( etag )
  {
    // once each: call all registered destructors
    if( etag in this.etagList )  {
      for( var i = 0; i < this.etagList[ etag ].length; i++ )  {
        this.etagList[ etag ][ i ] ( etag );
      }
      delete this.etagList[ etag ];
    }

    // no more etags: stop
    if( this.etagList.length == 0 )  {
      this.Stop( );
    }
  }

  //-------------------------------------------------------
  //  HELPER
  //-------------------------------------------------------  

  RegisterETag( etag )
  {
    if( !( etag in this.etagList ) )  {
      this.etagList[ etag ] = [ ];
    }
  }

  //-------------------------------------------------------
  //  INTERFACE
  //-------------------------------------------------------
  
  static Instance( )
  {
    if( viewsMutationApi.instance == null )  {
      viewsMutationApi.instance = new MutationApi( );
    }
    return viewsMutationApi.instance;
  }

  static RegisterDestructor( etag, callback, context )
  {
    var instance = MutationApi.Instance() ;

    if( typeof callback == 'object' && 'destructor' in callback )  {
      instance.RegisterETag( etag );
      instance.etagList[ etag ].push( callback['destructor'].bind( context) );
      instance.Start( );
    }
    else if( callback && {}.toString.call( callback ) === '[object Function]' )  {
      instance.RegisterETag( etag );
      instance.etagList[ etag ].push( callback.bind( context ) );
      instance.Start( );
    }
  }

  static RemoveETag( etag )
  {
    MutationApi.Instance().HandleRemovedETag( etag );
  }
}


//-----------------------------------------------------------------------------
//  SINGLETON
//-----------------------------------------------------------------------------

var viewsMutationApi = function( ) {
  return this.instance;
};
//=============================================================================
//
//  CYTEL Views2 - Web Framework
//  
//  Event-Source (PubSub) working mmit mutation api
//
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class EventSource
{
  // parts taken from https://github.com/mroderick/PubSubJS
  
  //-------------------------------------------------------
  //  INIT
  //-------------------------------------------------------
  
  constructor( )
  {
    this.eventMap = { };
  }

  //-------------------------------------------------------
  //  INTERFACE
  //-------------------------------------------------------
  
  //-------------------------------------------------------
  //  register event 
  //    eventName (string):  name of the event
  //    etag (string): eTag provided by viewable
  //    func (function): callback for event function
  //
  On( eventName, etag, func )
  {
    var self = this;

    // message is not registered yet
    if( !this.eventMap.hasOwnProperty( eventName ) ) {
      this.eventMap[ eventName ] = { };
    }
    
    // forcing token as String, to allow for future expansions without breaking usage
    // and allow for easy use as key names for the 'messages' object
    this.eventMap[ eventName ][ etag ] = func;
    
    // register this event source on mutation api to automize Off()
    const destructor = function( etagParam )  {
      self.Off( etagParam );
    };
    MutationApi.RegisterDestructor( etag, destructor, this );

  }

  //-------------------------------------------------------
  //  unregister event 
  //    value (string):  
  //    - event name (deregister all callbacks for an event)
  //    - etag (derregister all event for an etag)
  //
  Off( value )
  {
    // prequisites
    var isEventName = this.eventMap.hasOwnProperty( value );
    var isETag = !isEventName && typeof value === 'string';
    var result = false;

    // clear all subscriptions for an event
    if( isEventName )  {
      delete this.eventMap[ value ];
      result = true;
    }
    else if( isETag )  {
      for( var eventName in this.eventMap )  {
        delete this.eventMap[ eventName ][ value ];
        if( Object.keys( this.eventMap[ eventName ] ).length == 0 )  {
          delete this.eventMap[ eventName ];
        }
      }
    }

    return result;
  }

  //-------------------------------------------------------
  //  publish an event
  //    eventName (string):  name of the event
  //    data (mixed): data to be send 
  //
  Publish( eventName, data, param )
  {
    var hasEventName = this.eventMap.hasOwnProperty( eventName );
    if( hasEventName )  {
      var event = this.eventMap[ eventName ];
      for( var etag in event )  {
        event[ etag ]( data, param );
      }
      return true;
    }
    return false;
  }

}


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class StaticEventSource
{
  //-------------------------------------------------------
  //  CLASS VARIABLES
  //-------------------------------------------------------

  eventMap = {};
  lastUsedId = 0;


  //-------------------------------------------------------
  //  INTERFACE
  //-------------------------------------------------------

  //-------------------------------------------------------
  //  register event 
  //    eventName (string):  name of the event
  //    func (function): callback for event function
  //    returns unique id
  //  
  On( eventName, func )
  {
    // message is not registered yet
    if( ! ( eventName in this.eventMap ) ) {
      this.eventMap[ eventName ] = { };
    }

    // create an id 
    this.lastUsedId = this.lastUsedId + 1;
    let idString = this.lastUsedId.toString();
    this.eventMap[ eventName ][ idString ] = func;

    return idString;
  }

  //-------------------------------------------------------
  //  unregister event using it's unique id
  //
  Off( id )
  {
    // check if id exists and remove
    for( let eventName in this.eventMap ) {
      if( id in this.eventMap[ eventName ]  ) {
        delete this.eventMap[ eventName ][ id ];
      }
    }
  }

  //-------------------------------------------------------
  //  publish an event
  //    eventName (string):  name of the event
  //    data (mixed): data to be send 
  //
  Publish( eventName, data, param )
  {
    // check if event name exists and loop though
    // all registered callbacks
    if( eventName in this.eventMap )  {
      for( var idString in this.eventMap[ eventName ] ) {
        this.eventMap[ eventName ][ idString ]( data, param );
      }
    }
  }

}// class
//=============================================================================
//
//  APP base class
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.01.2019 - created
//          15.02.2019 - add $ singleton
//=============================================================================

"use strict";

//-----------------------------------------------------------------------------
//  APPLICATION MAIN CLASS
//-----------------------------------------------------------------------------

class Application extends EventSource
{
  //-----------------------------------------------------------------
  //  create object
  //
  constructor()
  {
    super();
    this.applicationTitle = null;
    this.documentTitle = null;
    this.etag = $.guidgen( );
    $.metrics( 'scrollWidth' );
    $.lang.Set( $.metrics( 'browserLanguage' ) );
    console.log( "CYTEL Web-Application: Run on " + $.metrics( 'browserName' ) + ", Version " + $.metrics( 'browserVersion' ) + ", Language '" + $.lang.Get() + "'" );

    this.downloadElement = null;

  }

  //-----------------------------------------------------------------
  //  OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  override this
  //
  Run( ) 
  { 
  }

  //-----------------------------------------------------------------
  //  override this
  //
  Load( ) 
  {     
  }

  //-----------------------------------------------------------------
  //  override this
  //
  DownloadFile( filePath ) 
  {
    if( this.downloadElement == null ) {
      this.downloadElement = document.createElement( 'a' );
    }
    this.downloadElement.href = filePath;
    this.downloadElement.download = filePath.substr( filePath.lastIndexOf( '/' ) + 1 );
    this.downloadElement.click();
  }

  //-----------------------------------------------------------------
  //  BROWSER TITLE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  set titel on browser
  //
  SetApplicationTitle( applicationTitle )
  {
    // check if title has changed and change it
    let newApplicationTitle = $.lang( applicationTitle );
    if( this.applicationTitle != newApplicationTitle )  {

      // publish event
      this.applicationTitle = newApplicationTitle;
      this.Publish( "application/title", this.applicationTitle );

      // set browser title 
      if( this.documentTitle != null ) {
        $( document ).attr( "title", this.applicationTitle + " - " + this.documentTitle );
      }
      else {
        $( document ).attr( "title", this.applicationTitle );
      }
    }
  }

  //-----------------------------------------------------------------
  //  add document titel after window titel
  //
  SetDocumentTitle( documentTitle )
  {
    // prequisites
    if( !this.applicationTitle ) {
      console.warn( "try to set a document window title. Use SetApplicationTitle() in Application first." );
      return;
    }

    // check if doc-title has changed and change it
    let newDocumentTitle = $.lang( documentTitle );
    if( this.documentTitle != newDocumentTitle )  {
   
      // publish event
      this.documentTitle = newDocumentTitle;
      this.Publish( "application/doctitle", this.documentTitle );

      // set browser title 
      if( this.documentTitle != null )  {
        $( document ).attr( "title", this.applicationTitle + " - " + this.documentTitle );
      }
      else  {
        $( document ).attr( "title", this.applicationTitle );
      }
    }
  }

  //-----------------------------------------------------------------
  //  UNIT TESTS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  display unit test messages
  //
  OnDisplayUnitTestMessage( type, message )
  {    
    if( type == 0 )  {
      console.info( "UnitTest, OK:", message );
    }
    else if( type == 1 )  {
      console.warn( "UnitTest, Warning:", message );
    }
    else if( type == 2 )  {
      console.error( "UnitTest, Error:", message );
    }
  }


  //-----------------------------------------------------------------
  //  PRINTING
  //-----------------------------------------------------------------

  StartPrinting()
  {
    console.log( "CYTEL Web-Application: Start Printing..." );
    $( "body" ).append( "<div class='print-content'>Test</div>" );

    let $children = $( "body" ).children();
    for( let child of $children )  {
      this.PrintingIterator( $( child ), 0 );
    }
  }

  PrintingIterator( $child, level )
  {
    let viewable = $child.data( "viewable" );
    if( viewable )  {
      if( $child.is( ":visible" ) )  {
        let spacer = "  ";
        console.log( `${ spacer.repeat( level ) }PRINT: ${ viewable.options.controller.constructor.name }` );
      }
    }

    let $children = $child.children();
    for( let child of $children ) {
      this.PrintingIterator( $( child ), level + 1 );
    }

  }

  EndPrinting()
  {
    console.log( "CYTEL Web-Application: End Printing." );
  }

  //-----------------------------------------------------------------
  //  STATICS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  REGISTER application class
  //
  static Register( id )
  {
    if( $.app._theAppPrototype == null )  {
      $.app._theAppPrototype = this.prototype.constructor;
    }
    else  {
      console.warn( "You must not have more then one Application class!" );
    }
  }

  //-----------------------------------------------------------------
  //  SINGLETON 
  //  
  static Instance( )
  {
    return $.app._theApp;
  }

  //-----------------------------------------------------------------
  //  ENABLE UNIT TEST
  //  
  static EnableUnitTest( isEnabled )
  {
    if( isEnabled )  {
      console.log( "CYTEL Web-Application: Unit tests are enabled." );
    }
    $.app._isUnitTestEnabled = ( isEnabled ? true : false );
  }

  //-----------------------------------------------------------------
  //  ENABLE UNIT TEST
  //  
  static IsUnitTestEnabled( )
  {
    return $.app._isUnitTestEnabled || false;
  }

  static DisplayUnitTestMessage( type, message )
  {
    if( Application.IsUnitTestEnabled( ) )  {
      $.app._theApp.OnDisplayUnitTestMessage( type, message );
    }
  }

}


//-----------------------------------------------------------------------------
//  JQUERY SINGLETON
//-----------------------------------------------------------------------------

!function( $ ) {
 
  // singleton code
  $.app = function( ) {
    return $.app._theApp;
  };
  
  // init
  $.app._theApp = null;

  // create the application class
  $( document ).ready( function( )  {
    if( $.app._theAppPrototype != null )  {
      $.app._theApp = new $.app._theAppPrototype( );
      $.app._theApp.Load( );
      $.app._theApp.Run( );
    }
    else  {
      console.warn( "Missing Application object!" );
    }
  } );
  
}( window.jQuery );
/*
*  ACCESS 5.0 Web Framework
*  language system
*/

!function( $ )
{

  /* ---------------------------------------------------------------------------
   *   LANG - const values
   * ------------------------------------------------------------------------ */

  // ----------------------------------------------------
  //   lang main function
  //
  $.lang = function( subject )
  {
    if( typeof( subject ) == "string" )  {
      return $.lang._EncodeText( subject );
    }
    else if( typeof ( subject ) == "object" ) {
      return $.lang._EncodeObject( subject );
    }
    else {
      return subject;
    }
  };

  // ----------------------------------------------------
  //   SET LANGUAGE
  //
  $.lang.Set = function( id )
  {
    $.lang._intern['langId'] = id;
  };

  // ----------------------------------------------------
  //   SET LANGUAGE
  //
  $.lang.Get = function( )
  {
    return $.lang._intern['langId'];
  };


  // ----------------------------------------------------
  //   PRIVATE: encode a text
  //
  $.lang._EncodeText = function( text )
  {
    if( typeof ( text ) != 'string' )
      return text;
    
    // encode 
    var result = text;
    var matches = text.match( $.lang._intern['rxtag'] );
    var lang = $.lang._data[ $.lang._intern['langId'] ];

    for( var i = 0; matches && i < matches.length; i++ ) {
      var match = matches[ i ];
      var vname = match.replace( "{$", "" );
      vname = vname.replace( "}", "" );
      if( vname in lang )  {
        result = result.replace( match, lang[ vname ] );
      }
    }
    return result;
  };


  // ----------------------------------------------------
  //   PRIVATE: encode an object
  //
  $.lang._EncodeObject = function( subject )
  {
    if( typeof ( subject ) != 'object' )
      return subject;

    // iterate through every node (recursivly) and modify language    
    $.walk( subject, function( key, value )  {
      return $.lang._EncodeText( value );
    } );

    return subject;
  };  


  // ----------------------------------------------------
  //   lang main function
  //
  $.lang._data = { 
    de: { },
    en: { }
  };

  $.lang._intern = {
    rxtag: /\{[$][^\}]+\}/gmi,
    langId: 'de',
    loadPending: 0
  };

}( window.jQuery );
//=============================================================================
//
//  VIEWS2 - Dataset
//  
//  (c) 2019, CYTEL AG
//      Frank Jaekel
//
//=============================================================================


class Dataset
{
  //-----------------------------------------------------------------
  //  CONSTRUCTOR
  //-----------------------------------------------------------------

  constructor( options, initData )
  {
    this._data = [];
    this._keys = {};
    this._options = {};
    this._dataViews = {};
    this._dataViewsActive = {};
    this._dataViewMaxUid = 0;
    this._dataViewsTemplates = {};
    if( options )  {
      this.Initialize( options, initData );
    }

    var self = this;
    setInterval( function()  {
      self.GarbageCollectAttachedViews( );
    }, 60000 );
  }

  Attach( dataview )
  {
    if( !( dataview instanceof Dataview ) )  {
      console.error( "Dataset.Attach() must be used with Dataview" );
      return -1;
    }
    if( ! $.isFunction( dataview._DatasetEvent ) )  {
      console.error( "Dataset.Attach() class Dataview must have function '_DatasetEvent'" );
      return -1;
    }

    var uid = ++( this._dataViewMaxUid );
    this._dataViews[ uid ] = dataview;
    this._dataViewsActive[ uid ] = true;

    return uid;
  }

  Release( uid )
  {
    if( uid in this._dataViews )  {
      delete this._dataViews[ uid ];
      delete this._dataViewsActive[ uid ];
      return true;
    }
    else if( uid === null )  {
      for( uid in this._dataViews )  {
        delete this._dataViews[ uid ];
        delete this._dataViewsActive[ uid ];  
      }
      return true;
    }
    return false;
  }

  GarbageCollectAttachedViews()
  {
    for( var key in this._dataViews )  {
      if( $( '#' + this._dataViews[ key ].domElementId ).length == 0 )  {
        delete this._dataViews[ key ];
      }
    }    
  }

  Silent( uid, isSilent )
  {
    this._dataViewsActive[ uid ] = !isSilent;
    if( !isSilent )  {
      this._dataViews[ uid ]._DatasetEvent( 'create' );
    }
  }

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  Initialize( options, initData )
  {
    // init
    var defaultOptions = {
      id: 'id', 
      preview: null   
    };

    this._options =  $.extend( true, [], defaultOptions, options );   // { id: idField, preview: $.isFunction( fnPreview ) ? fnPreview : null };

    // data
    if( Array.isArray( initData ) )  {
      this.SetData( initData );
    }

  }

  SetData( initData )
  {
    // empty
    this._data = [];
    this._keys = {};

    // iterate
    for( var i = 0; i < initData.length; i++ )  {
      var record = initData[ i ];
      var id = record[ this._options['id'] ];
      var newRecord = $.lang( $.clone( record ) );
      this._data.push( newRecord );
      this._data[ i ]['_preview'] = this.BuildPreview( newRecord );
      /*if( this._options.preview )  {
        this._data[ i ]['_preview'] = this._options.preview( newRecord );
      }
      else  {
        var icon = ( 'icon' in newRecord ) ? newRecord['icon'] : null;
        var text = ( 'text' in newRecord ) ? newRecord['text'] : null;
        var description = ( 'description' in newRecord ) ? newRecord['description'] : null;
        this._data[ i ]['_preview'] = { id: id, icon: icon, text: text, description: description };
      }*/
      this._keys[ id ] = i;
    }

    // fire event 
    this._FireEvent( 'create' );
  }

  GetData()
  {
    let result = $.clone( this._data );
    for( var i = 0; i < result.length; i++ ) {
      delete result[ i ]['_preview'];
    }
    return result;
  }

  RebuildAllViews( )
  {
    this._FireEvent( 'create' );
  }

  RegisterView( name, options )
  {
    this._dataViewsTemplates[ name ] = options;
  }

  BuildPreview( record )
  {
    let id = record[ this._options['id'] ];
    let preview = {};

    if( this._options.preview ) {
      preview = this._options.preview( record );
    }
    else {
      var icon = ( 'icon' in record ) ? record['icon'] : null;
      var text = ( 'text' in record ) ? record['text'] : null;
      preview = { id: id, icon: icon, text: text /*, description: ""*/ };
    }
    return preview;
  }

  GetIdField()
  {
    return this._options['id'];
  }

  //-----------------------------------------------------------------
  //  RECORDS
  //-----------------------------------------------------------------

  BuildView( name, options )
  {
    let  template;
    if( name in this._dataViewsTemplates )  {
      template = $.clone( this._dataViewsTemplates[ name ] );
    }
    else  {
      template = { 
        filter:  null,
        sortKey: null,
        sortOrder: '+'
      };
      template = $.extend( template, options );
    }
    var createOptions = $.extend( options, template );
    var dataview = new Dataview( this, createOptions );
    dataview.datasetId = this.Attach( dataview );
    return dataview;
  }

  //-----------------------------------------------------------------
  //  RECORDS
  //-----------------------------------------------------------------

  Add( record )
  {
    var id = record[ this._options['id'] ];
    if( id in this._keys )  {
      console.warn( "Add record '"+ id + "' - already exists!" );
      return;
    }    
    var newRecord = $.lang( $.clone( record ) );
    var index = this._data.push( newRecord ) - 1;
    this._data[ index ]['_preview'] = this.BuildPreview( newRecord );
    /*if( this._options.preview )  {
      this._data[ index ]['_preview'] = this._options.preview( newRecord );
    }
    else  {
      var icon = ( 'icon' in record ) ? record['icon'] : null;
      var text = ( 'text' in record ) ? record['text'] : null;
      this._data[ index ]['_preview'] = { id: id, icon: icon, text: text, description: "" };
    }*/
    this._keys[ id ] = index;

    // fire event 
    this._FireEvent( 'add', id, newRecord );    
  }

  Delete( id )
  {
    if( !( id in this._keys ) )  {
      console.warn( "Update record '"+ id + "' - not found!" );
      return;
    }
    var oldRecord = this.Get( id );
    var index = this._keys[ id ];
    delete this._keys[ id ];
    this._data.splice( index, 1 );
    for( var i in this._keys )  {
      if( this._keys[ i ] >= index )  {
        this._keys[ i ] = this._keys[ i ] - 1;
      }
    }

    this._FireEvent( 'delete', id, oldRecord );
  }

  Update( record )
  {
    var id = record[ this._options['id'] ];
    if( !( id in this._keys ) )  {
      console.warn( "Update record '"+ id + "' - not found!" );
      return;
    }
    var index = this._keys[ id ];
    var oldRecord = this._data[ index ];
    this._data[ index ] = record;
    
    if( this._options.preview ) {
      this._data[ index ]['_preview'] = this._options.preview( record );
    }
    else {
      var icon = ( 'icon' in record ) ? record['icon'] : null;
      var text = ( 'text' in record ) ? $.lang( record['text'] ) : null;
      var description = ( 'description' in record ) ? $.lang( record['description'] ) : null;
      this._data[ index ]['_preview'] = { id: id, icon: icon, text: text, description: description };
    }

    this._FireEvent( 'update', id, oldRecord );

  }

  UpdateById( id, record )
  {
    let recordId = record[ this._options['id'] ];
    if( recordId != id )  {
      console.warn( "Dataset.Update record '" + id + "' - not found!" );
      return;
    }
    this.Update( record );
  }

  Get( id )
  {
    var index = this._keys[ id ];
    return this._data[ index ];
  }

  GetIndex( id )
  {
    if( !( id in this._keys ) ) {
      return -1;
    }
    return this._keys[ id ];
  }

  Has( id, param = null )
  {
    if( $.isFunction( id ) ) {
      for( let record of this._data )  {
        if( id( record ) === true ) {
          return true;
        }
      }
      return false;
    }
    else if( param == null )  {
      return ( id in this._keys );
    }
    else  {
      return ( this._data.findIndex( record => ( record[param] == id ) ) >= 0 );
    }
  }

  Filter( seek, fieldName )
  {
    let filtered = this._data.filter( record => ( record[ fieldName ] == seek ) );
    let result = Array.from( filtered, record => record[ this._options['id'] ] );
    return result;
  }

  Each( fnCallback )
  {
    var idField = this._options['id'];
    for( var i = 0; i < this._data.length; i++ )  {
      fnCallback( this._data[ i ][ idField ], this._data[ i ] );
    }
  }

  Find( fnCallback )
  {
    var idField = this._options['id'];
    for( var i = 0; i < this._data.length; i++ ) {
      if( fnCallback( this._data[ i ] ) === true ) {
        return this._data[ i ][ idField ];
      }
    }
    return null;
  }

  Size()  
  {
    return this._data.length;
  }

  //-----------------------------------------------------------------
  //  HELPER
  //-----------------------------------------------------------------

  _FireEvent( event, param, param2 )
  {
    for( var key in this._dataViews )  {
      if( this._dataViewsActive[ key ] )  {
        this._dataViews[ key ]._DatasetEvent( event, param, param2 );
      }
    }
  }
  
}

//=============================================================================
//
//  VIEWS2 - Dataset
//  
//  (c) 2019, CYTEL AG
//      Frank Jaekel
//
//  itemData :=  {
//    $item:  -> JjQuery-object
//    index:  -> index in list,
//    id:     -> doc-id,
//    data:  {      -> user-data
//      css:        -> css-classes set (not status)
//      status: []  -> css-status list
//    }
//  }
//=============================================================================

class Dataview 
{
  //-----------------------------------------------------------------
  //  CONSTRUCTOR / 'DESTRUCTOR'
  //-----------------------------------------------------------------

  constructor( dataset, options )
  {
    if( !( dataset instanceof Dataset ) )  {
      console.error( "A dataview must be initialized with a Dataset" );
      return;
    }

    var defaultOptions = {
      sortKey: null,
      sortOrder: 'none',
      sortOnUpdate: true,
      filter: null,
      selection: 'single',
      rebuildAlways: false,
    };

    this.length = 0;
    this.dataset = dataset;
    this.datasetId = -1;
    this.options = $.extend( true, [], defaultOptions, options );
    this.events = {};

    // data
    this.idList = [];         // array of index
    this.itemDataList = {};   // object if id's 
    this.selected = [];       // array if id's
    this.domElement = null;

    // local data
    this.localData = null;    // additional data not store in dataset
    this.localDataId = {};
    if( Array.isArray( this.options['data'] ) )  {
      this.localData = this.options['data'];
    }

    // search
    this.search = null;
    this.searchRx = null;

    this._Rebuild( );
  }

  AttachDomElement( $element )
  {
    if( $element instanceof jQuery 
     && $element.length > 0
      )  {
      if( $element[0].id == "" )  {
        console.error( "Dataview.AttachDomElement: $element must have an id tag!" );
      }
      this.domElementId = $element[ 0 ].id;
    }
    else  {
      console.warn( "Dataview.AttachDomElement: $element not a jQuery-Object!, potential memory leak!" );
    }
  }

  Options( options )
  {
    this.options = $.extend( true, [], this.options, options );
  }

  //-----------------------------------------------------------------
  //  EVENT INTERFACE
  //-----------------------------------------------------------------

  On( event, fnFunction )
  {
    this.events[ event ] = fnFunction;
  }

  Off( event )
  {
    delete this.events[ event ];
  }

  Silent( isSilent )
  {
    this.dataset.Silent( this.datasetId, isSilent );
    /*if( !isSilent )  {
      console.log( "VW: !silent #2" );
      this._FireEvent( 'rebuild' );
    }*/
  }

  Update( )
  {
    this._Rebuild( );
    //console.log( "VW: update" );
    this._FireEvent( 'rebuild' );
  }

  //-----------------------------------------------------------------
  //  DATASET Event-Function
  //-----------------------------------------------------------------

  _DatasetEvent( event, param, param2 )
  {
    if( this.options['rebuildAlways'] === true )  {
      this._Rebuild();
      //console.log( "VW: rebuildAlways" );

      this._FireEvent( 'rebuild', param );
    }
    else  {
      if( event == 'create' )  {
        this._Rebuild( );
        //console.log( "VW: create" );
        this._FireEvent( 'rebuild', param );
      }
      else if( event == 'delete' )  {
        this._DeleteRecord( param );
      }
      else if( event == 'add' )  {
        this._AddRecord( param, param2 );
      }
      else if( event == 'update' )  {
        this._UpdateRecord( param, param2 );
      }
    }

  }

  //-----------------------------------------------------------------
  //  Rebuild the data view
  //-----------------------------------------------------------------

  _Rebuild( )
  {
    var self = this;

    var fnFilter = $.isFunction( this.options.filter ) ? this.options.filter : this.DefaultFilter;
    var fnSearch = $.isFunction( this.options.search ) ? this.options.search : this.DefaultSearch;
    
    // empty all
    this.idList = [ ];
    this.itemDataList = {};
    this.localDataId = {};

    if( Array.isArray( this.localData ) )  {
      
      let idKey = this.dataset.GetIdField();
      for( let record of this.localData )  {

        let id = record[idKey];
        record['_preview'] = this.dataset.BuildPreview( record );

        if( fnFilter( id, record, self.options, self )
         && fnSearch( id, record, self.options, self )
          ) {
          this.localDataId[ id ] = record;
          self.idList.push( id );
          self.itemDataList[ id ] = {
            '$item': null,
            'id': id,
            'data': {}
          };
        }
      }
    }

    // rebuild
    this.dataset.Each( function( id, record )  {
      if( fnFilter( id, record, self.options, self ) 
       && fnSearch( id, record, self.options, self ) 
       && !( id in self.localDataId )
        )  {
        self.idList.push( id );
        self.itemDataList[ id ] = {
          '$item': null,
          'id': id,
          'data': {}
        }; 
      }
    } );   

    this.length = this.idList.length;

    if( this.options.sortOrder != 'none' )  {
      this._SortDataview( );
    }
  }

  _DeleteRecord( id, deletedRecord )
  {
    var itemData = this.itemDataList[ id ];
    delete this.itemDataList[ id ];
    $.array.Remove( this.idList, id );
    this.length = this.idList.length;

    this._FireEvent( 'delete', id, itemData );

    if( this.selected.indexOf( id ) >= 0 )  {
      $.array.Remove( this.selected, id );
      this._FireEvent( 'select', this.selected );
    }
  }

  _AddRecord( id, record )
  {
    var isAdded = false;
    var fnFilter = $.isFunction( this.options.filter ) ? this.options.filter : this.DefaultFilter;
    var fnSearch = $.isFunction( this.options.search ) ? this.options.search : this.DefaultSearch;

    if( fnFilter( id, record, this.options, this )
     && fnSearch( id, record, this.options, this ) 
      )  {
      this.idList.push( id );
      this.itemDataList[ id ] = {
        '$item': null,
        'id': id,
        data: {}
      };
      isAdded = true;
    }

    if( !isAdded )
      return;
    
    this.length = this.idList.length;

    if( this.options.sortOrder != 'none' )  {
      this._SortDataview( );
    }    
    
    //console.log( "VW: _AddRecord" );
    this._FireEvent( 'rebuild' );

  }

  _UpdateRecord( id, oldRecord )
  {
    // get new record
    var newRecord = this.dataset.Get( id );

    // if this record was filtered simulate 'add' 
    if( !( id in this.itemDataList ) )  {
      return this._AddRecord( id, newRecord );
    }

    // if the record is not longer in filter, remove
    var fnFilter = $.isFunction( this.options.filter ) ? this.options.filter : this.DefaultFilter;
    var fnSearch = $.isFunction( this.options.search ) ? this.options.search : this.DefaultSearch;
    if( !fnFilter( id, newRecord, this.options, this )
     || !fnSearch( id, newRecord, this.options, this ) 
      )  {
      let result = this._DeleteRecord( id, oldRecord );
      //console.log( "VW: _UpdateRecord #1" );
      this._FireEvent( 'rebuild' );
      return result;
    }

    // sort if required
    if( this.options.sortOnUpdate && this.options.sortOrder != 'none' )  {
      var record = this.dataset.Get( id );
      if( oldRecord[ this.options.sortKey ] != record[ this.options.sortKey ] )  {
        this._SortDataview( );
        //console.log( "VW: _UpdateRecord #2" );
        this._FireEvent( 'rebuild' );
        return;
      }
    }

    var index = this._Id2Index( id );
    this._FireEvent( 'update', id, oldRecord );
    return index;
  }


  //-----------------------------------------------------------------
  //  INTERFACE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  SEARCH data
  //
  Search( value )
  {
    // avoid unneeded rebuilds
    if( this.search === value ) {
      return;
    }

    if( value == "" )  {
      this.search = null;
    }
    else {
      this.search = value;
      this.searchRx = new RegExp( value, "gi" );
    }
    this._Rebuild( );
    //console.log( "VW: Search" );
    this._FireEvent( 'rebuild' );
  }    

  //-----------------------------------------------------------------
  //  SORT data
  //
  Sort( sortKey, sortOrder = '+' )
  {
    this.options['sortOrder'] = sortOrder;
    this.options['sortKey'] = sortKey;

    this._Rebuild();
    //console.log( "VW: Sort" );
    this._FireEvent( 'rebuild' );
  }


  //-----------------------------------------------------------------
  //  SELECT items
  //
  Select( selected )
  {
    this.ItemClearSelections( );
    if( Array.isArray( selected ) )  {
      for( var i = 0; i < selected.length; i++ )  {
        this.ItemSelect( selected[ i ] );
      }
    }
  }  

  //-----------------------------------------------------------------
  //  GET data
  //
  GetIdFromIndex( index )
  {
    return this._Index2Id( index );
  }    

  //-----------------------------------------------------------------
  //  GET data
  //
  GetIndexFromId( id )
  {
    return this._Id2Index( id );
  }    

  //-----------------------------------------------------------------
  //  GET data
  //
  Get( id )
  {
    if( ( Array.isArray( this.localData ) )
     && ( id in this.localDataId )
      ) {
      return this.localDataId[ id ];
    }
    return this.dataset.Get( id );
  }  

  //-----------------------------------------------------------------
  //  HAS item
  //
  Has( id )
  {
    if( ( Array.isArray( this.localData ) )
     && ( id in this.localDataId )
      ) {
      return true;
    }

    return this.dataset.Has( id );
  }  

  //-----------------------------------------------------------------
  //  GET data using index
  //
  GetByIndex( index )
  {
    var id = this._Index2Id( index );
    return this.Get( id );
    //return this.dataset.Get( id );
  }  

  //-----------------------------------------------------------------
  //  GET data using index
  //
  GetIdByIndex( index )
  {
    return this._Index2Id( index );
  }

  //-----------------------------------------------------------------
  //  GET item data 
  //
  GetItem( id )
  {
    return this.itemDataList[ id ];
  }  

  //-----------------------------------------------------------------
  //  GET item data using index
  //
  GetItemByIndex( index )
  {
    var id = this._Index2Id( index );
    return this.itemDataList[ id ];
  }  

  //-----------------------------------------------------------------
  //  GET Preview data
  //
  Preview( id )
  {
    // prequistes
    var record = this.dataset.Get( id );

    // return data
    if( '_preview' in record ) {
      return record['_preview'];
    }
    else {
      return id;
    }
  }  

  //-----------------------------------------------------------------
  //  GET Preview data
  //
  PreviewByIndex( index )
  {
    var id = this.idList[ index ];
    var result = this.Preview( id );
    return result; 
  }

  //-----------------------------------------------------------------
  //  Attach $item to dataview
  //
  AttachItem( index, $item, data )
  {
    var id = this._Index2Id( index );
    if( id < 0 )  {
      return null;
    }
    
    if( 'status' in this.itemDataList[ id ]['data'] )  {
      this.itemDataList[ id ]['$item'] = $item;
      this.ItemSetStatus( id, this.itemDataList[ id ]['data']['status'] );
    }
    else  {
      this.itemDataList[ id ]['$item'] = $item;
      this.itemDataList[ id ]['data'] = ( data ) ? $.clone( data ) : { };

      // get css classes from item and remove status classes
      var cssClassList = $item.attr( 'class' ).split( /\s+/ );
      var cssList = $.array.NotIn( cssClassList, $.jqc.statics.themeClassList );
      this.itemDataList[ id ]['data']['css'] = cssList.join( ' ' );
      if( data && 'status' in data )  {
        this.ItemSetStatus( id, data['status'] );
      }
      else  {
        this.itemDataList[ id ]['data']['status'] = $.array.In( cssClassList, $.jqc.statics.themeClassList );
      }
    }

    $item.data( 'itemData', this.itemDataList[ id ] );
    return this.itemDataList[ id ];
  }

  //-----------------------------------------------------------------
  //  set normalized status for item
  //
  ItemSetStatus( id, status )
  {
    if( !( id in this.itemDataList ) ) {
      //console.warn( "Dataview.ItemSetStatus(): id '" + id + "' not found" );
      return;
    }

    if( ! this.itemDataList[ id ]['$item'] )  {
      //console.error( "Dataview.ItemSetStatus(): item not atached" );
      return;
    }

    var newStatus = Array.isArray( status ) ? status : status.split( /\s+/ );

    // base status : first of       [ 'normal', 'disabled', 'readonly', 'hilight', 'red', 'green', 'yellow' ]
    // mixin status: combination of [ 'error', 'focus', 'selected', 'expanded', 'collapsed', 'no-hover', 'hidden', 'cursor', 'striked', 'underlined' ],
    var resultStatus = [];
    resultStatus.push( $.array.FirstIn( newStatus, $.jqc.statics.themeBaseStateList ) );
    var mixinStatusList = $.array.In( newStatus, $.jqc.statics.themeMixinList );

    // put all together
    $.array.Join( resultStatus, mixinStatusList );

    // be sure, that just one of [ 'expanded', 'collapsed' ] is set
    $.array.MutualExclude( resultStatus, $.jqc.statics.themeMixinGroup );

    // set hover if not in  [ 'readonly', 'disabled', 'no-hover' ]
    if( !$.array.HasOneIn( resultStatus, $.jqc.statics.themeNoHoverList ) ) {
      resultStatus.push( 'hover' );
    }

    // selected
    if( resultStatus.indexOf( 'selected' ) >= 0 )  {
      $.array.Add( this.selected, id );
    }

    // store and set class into $item
    this.itemDataList[ id ]['data']['status'] = resultStatus;
    if( this.itemDataList[ id ]['$item'] instanceof jQuery ) {
      var cssClasses = this.itemDataList[ id ]['data']['css'] + ' ' + resultStatus.join( ' ' );
      this.itemDataList[ id ]['$item'].attr( 'class', cssClasses );
    }

  }

  //-----------------------------------------------------------------
  //  add a status to item
  //
  ItemAddStatus( id, status )
  {
    if( !( id in this.itemDataList ) ) {
      //console.warn( "Dataview.ItemAddStatus(): id '" + id + "' not found" );
      return;
    }

    // prequistes
    if( ! this.itemDataList[ id ]['$item'] )  {
      //console.warn( "Dataview.ItemSetStatus(): item not atached" );
      return;
    }

    var newStatus = status.split( /\s+/ );

    // base status : first of       [ 'normal', 'disabled', 'readonly', 'hilight', 'red', 'green', 'yellow' ]
    // mixin status: combination of [ 'error', 'focus', 'selected', 'expanded', 'collapsed', 'no-hover', 'hidden', 'cursor' ],
    var resultStatus = [];
    var newBaseStatus = $.array.FirstIn( newStatus, $.jqc.statics.themeBaseStateList );
    if( newBaseStatus )  {
      resultStatus.push( newBaseStatus );
    }
    else  {
      newBaseStatus = $.array.FirstIn( this.itemDataList[ id ]['data']['status'], $.jqc.statics.themeBaseStateList );
      if( newBaseStatus )  {
        resultStatus.push( newBaseStatus );
      }
    }
    var newMixinStatusList = $.array.In( newStatus, $.jqc.statics.themeMixinList );
    var mixinStatusList = $.array.In( this.itemDataList[ id ]['data']['status'], $.jqc.statics.themeMixinList );

    // put all together
    $.array.Join( resultStatus, mixinStatusList );
    $.array.Join( resultStatus, newMixinStatusList );

    // be sure, that just one of [ 'expanded', 'collapsed' ] is set
    $.array.MutualExclude( resultStatus, $.jqc.statics.themeMixinGroup );

    // set hover if not in  [ 'readonly', 'disabled', 'no-hover' ]
    if( !$.array.HasOneIn( resultStatus, $.jqc.statics.themeNoHoverList ) ) {
      resultStatus.push( 'hover' );
    }

    // selected
    if( resultStatus.indexOf( 'selected' ) >= 0 )  {
      $.array.Add( this.selected, id );
    }

    // store and set class into $item
    this.itemDataList[ id ]['data']['status'] = resultStatus;
    if( this.itemDataList[ id ]['$item'] instanceof jQuery ) {
      var cssClasses = this.itemDataList[ id ]['data']['css'] + ' ' + resultStatus.join( ' ' );
      this.itemDataList[ id ]['$item'].attr( 'class', cssClasses );
    }

  }
  
  //-----------------------------------------------------------------
  //  remove status from item (mixin only)
  //
  ItemRemoveStatus( id, status )
  {
    // prequistes
    if( !( id in this.itemDataList ) ) {
      //console.warn( "Dataview.ItemRemoveStatus(): id '" + id + "' not found" );
      return;
    }
    if( ! this.itemDataList[ id ]['$item'] )  {
      //console.error( "Dataview.ItemSetStatus(): item not atached" );
      return;
    }

    // selected
    if( status == 'selected' )  {
      $.array.Remove( this.selected, id );
    }

    // split multiple status into array
    status = status.split( /\s+/ );

    // remove status from array
    $.array.Remove( this.itemDataList[ id ]['data']['status'], status );
    if( this.itemDataList[ id ]['$item'] instanceof jQuery ) {
      var cssClasses = this.itemDataList[ id ]['data']['css'] + ' ' + this.itemDataList[ id ]['data']['status'].join( ' ' );
      this.itemDataList[ id ]['$item'].attr( 'class', cssClasses );
    }

  }

  //-----------------------------------------------------------------
  //  toggle between one or two status 
  //
  ItemToggleStatus( id, status1, status2 )
  {
    // prequistes
    if( !( id in this.itemDataList ) ) {
      //console.warn( "Dataview.ItemToggleStatus(): id '" + id + "' not found" );
      return;
    }
    if( ! this.itemDataList[ id ]['$item'] )  {
      //console.error( "Dataview.ItemSetStatus(): item not atached" );
      return;
    }

    let resultStatus;

    // toggle between status1 <-> status2
    if( status2 )  {
      if( this.ItemHasStatus( id, status2 ) )  {
        $.array.Remove( this.itemDataList[ id ]['data']['status'], status2 );
        this.itemDataList[ id ]['data']['status'].push( status1 );
        resultStatus = status1;
      }
      else  {
        $.array.Remove( this.itemDataList[ id ]['data']['status'], status1 );
        this.itemDataList[ id ]['data']['status'].push( status2 );
        resultStatus = status2;
      }
    }
    // toggle between status1 <-> none
    else  {
      if( this.ItemHasStatus( id, status1 ) )  {
        $.array.Remove( this.itemDataList[ id ]['data']['status'], status1 );
        resultStatus = status1;
      }
      else  {
        this.itemDataList[ id ]['data']['status'].push( status1 );        
        resultStatus = "";
      }
    }

    if( this.itemDataList[ id ]['$item'] instanceof jQuery ) {
      var cssClasses = this.itemDataList[ id ]['data']['css'] + ' ' + this.itemDataList[ id ]['data']['status'].join( ' ' );
      this.itemDataList[ id ]['$item'].attr( 'class', cssClasses );
    }

    // selection
    var previousSelLength = this.selected.length;
    if( this.itemDataList[ id ]['data']['status'].indexOf( 'selected' ) >= 0 )  {
      $.array.Add( this.selected, id );
    }
    else  {
      $.array.Remove( this.selected, id );
    }
    if( previousSelLength != this.selected.length )  {
      this._FireEvent( 'select', this.selected );
    }

    return resultStatus;    
  }

  //-----------------------------------------------------------------
  //  remove status from item (mixin only)
  //
  ItemHasStatus( id, status )
  {
    // prequistes
    if( !( id in this.itemDataList ) ) {
      //console.warn( "Dataview.ItemToggleStatus(): id '" + id + "' not found" );
      return false;
    }

    return ( this.itemDataList[ id ]['data']['status'].indexOf( status ) >= 0 );
  }

  //-----------------------------------------------------------------
  //  get all selected items
  //
  ItemGetSelected(  )
  {
    return $.clone( this.selected );
  }

  // selection
  ItemToggleSelection( id )
  {
    var resultSelectStatus;

    if( this.selected.indexOf( id ) < 0 )  {
      this.ItemAddStatus( id, 'selected' );
      this._FireEvent( 'select', this.selected );
      resultSelectStatus = true;
    }
    // remove
    else  {
      this.ItemRemoveStatus( id, 'selected' );
      this._FireEvent( 'select', this.selected );
      resultSelectStatus = false;
    }

    return resultSelectStatus;
  }

  // selection
  ItemSelect( id, singleSelect )
  {
    this.ItemAddStatus( id, 'selected' );
    this._FireEvent( 'select', this.selected );

    return true;
  }  

  // selection
  ItemUnselect( id, singleSelect )
  {
    this.ItemRemoveStatus( id, 'selected' );
    this._FireEvent( 'select', this.selected );

    return true;
  }  

  ItemClearSelections( )
  {
    for( var i = this.selected.length - 1; i >= 0; i-- )  {
      this.ItemRemoveStatus( this.selected[ i ], 'selected' );
    }
    this.selected = [];
  }

  //-----------------------------------------------------------------
  //  OVERRIDABLES
  //-----------------------------------------------------------------

  DefaultFilter( id, record, options, self )
  {
    return true;
  }

  DefaultSearch( id, record, options, self )
  {
    if( self.search == null )  {
      return true;
    }
    else  {
      if( '_preview' in record && typeof ( record['_preview']['text'] ) == "string" )  {
        var preview = record['_preview'];
        return ( preview.text.search( self.searchRx ) >= 0 );
      }
      else  {
        for( let key in record )  {
          if( typeof( record[key] ) == "string" && record[ key ].search( self.searchRx) >= 0 )  {
            return true;
          }
        }
        return false;
      }
    }
  }  

  //-----------------------------------------------------------------
  //  HELPER
  //-----------------------------------------------------------------

  //---[ fire event to parent ]--------------------------------------

  _Index2Id( index )
  {
    return this.idList[ index ];
  }

  _Id2Index( id )
  {
    let index = this.idList.findIndex( ( rec ) => rec == id );
    return index;
    for( var i = 0; i < this.idList.length; i++ )  {
      if( this.idList[ i ] == id ) 
        return i;
    }
    return -1;
  }

  _FireEvent( event, param, extra )
  {
    if( event in this.events )  {
      this.events[ event ]( param, extra );
    }
    else if( 'change' in this.events )  {
      this.events['change']( event, param, extra );
    }
  }  

  //---[ sort data ]-------------------------------------------------

  _SortDataview( )
  {
    var dataSelf = this;
    var key = this.options['sortKey'];

    // prequisites
    if( !key || this.idList.length == 0 || this.options['sortOrder'] == 'none' || this.options['sortKey'] == null )  {
      return;
    } 
    // store sorting
    var isAscending = ( this.options['sortOrder'] == '+' );
    let record0 = this.GetByIndex( 0 );
    var isNumeric = ( typeof record0[ key ] == "number" );
    var isBool = ( record0[ key ] === true || record0[ key ] === false );
    var fnSort = $.isFunction( this.options.sorter ) ? this.options.sorter : null;

    // sort list OR record
    if( isAscending ) {
      this.idList.sort( function( a, b )  {
        var recordA = dataSelf.Get( a );
        var recordB = dataSelf.Get( b );
        if( fnSort )  {
          var result = fnSort( recordA, recordB );
          if( Number.isInteger( result ) )  {
            return result;
          }
        }
        else if( isNumeric || isBool )  {
          var valueA = recordA[ key ];
          var valueB = recordB[ key ];
          if( valueA < valueB )       return 1;
          else if( valueA == valueB ) return 0;
          else                        return -1;
        }
        else {
          var textA = recordA[ key ] || "";
          var textB = recordB[ key ] || "";
          return textA.localeCompare( textB, navigator.language, { numeric: true, sensitivity: 'base' } );
        }
      } );
    }
    else {
      this.idList.sort( function( b, a )  {
        var recordA = dataSelf.Get( a );
        var recordB = dataSelf.Get( b );
        if( fnSort ) {
          var result = fnSort( recordA, recordB );
          if( Number.isInteger( result ) ) {
            return result;
          }
        }
        else if( isNumeric || isBool ) {
          var valueA = recordA[ key ];
          var valueB = recordB[ key ];
          if( valueA < valueB )       return 1;
          else if( valueA == valueB ) return 0;
          else                        return -1;
        }
        else {
          var textA = recordA[ key ] || "";
          var textB = recordB[ key ] || "";
          return textA.localeCompare( textB, navigator.language, { numeric: true, sensitivity: 'base' } );
        }
      } );
    }  
    
  }

  
}
/* ----------------------------------------------------------------------------
 * 
 *   CYTEL Web Framework 
 *   Mouse Capture
 *
 * --------------------------------------------------------------------------- */

var MouseCapture = function ()
{
  // ------------------------------------------------------
  //   class variables
  // ------------------------------------------------------
    
  var options = {
    tag: "",
    start: null,
    move: null,
    stop: null,    
    cls: null
  };

  
  // ------------------------------------------------------
  //   class private functions
  // ------------------------------------------------------
    
  var _trackMouseDown = function( e )
  {
    e.preventDefault();  
    e.stopPropagation();  
    options.start( e );
    document.addEventListener( 'click', _trackClick, true );
    document.addEventListener( 'mousemove', _trackMouseMove, true );
    document.addEventListener( 'mouseup', _trackMouseUp, true );
    
  };
  
  var _trackClick = function( e )
  {
    e.preventDefault();  
    e.stopPropagation();  

    return false;
  };
  
  var _trackMouseMove = function( e )
  {
    e.preventDefault();  
    e.stopPropagation();  
    options.move( e );

    return false;
  };
  
  var _trackMouseUp = function( e )
  {
    e.preventDefault();  
    e.stopPropagation();  
    options.stop( e );

    setTimeout( function( )  {
      document.removeEventListener( 'click', _trackClick, true );
      document.removeEventListener( 'mousemove', _trackMouseMove, true );
      document.removeEventListener( 'mouseup', _trackMouseUp, true );        
    }, 10 );    
    
    return false;
  };

  // ------------------------------------------------------
  //   class public members
  // ------------------------------------------------------
    
  MouseCapture.prototype.start = function( tag, fnStart, fnMove, fnStop )  
  {
    options.tag  = tag;
    options.start = fnStart;    
    options.move  = fnMove;    
    options.stop  = fnStop;    
    options.cls = this;
    var element = null;
    if( tag instanceof jQuery ) {
      element = tag[ 0 ];
    }
    else if( tag instanceof Element ) {
      element = tag;
    }
    else {
      element = document.getElementById( tag );
    }
    if( element ) element.addEventListener( 'mousedown', _trackMouseDown, false );

  };  

  MouseCapture.prototype.stop = function( )  
  {
    window.removeEventListener( 'mousedown', _trackMouseDown, true );        
  };  
};
//=============================================================================
//
//  Viewable Component class
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.01.2019
//=============================================================================


/* ---------------------------------------------------------------------------
 *   UNITEST EVENT CLASS
 * ------------------------------------------------------------------------ */

class UnitTestEvent
{
  numError = 0;
  numWarning = 0;
  numSucceed = 0;
  timestamp = null;

  constructor()
  {
    this.timestamp = Date.now();
    this.argumentList = [];
  }

  Success( message )
  {
    this.numSucceed += 1;
    Application.DisplayUnitTestMessage( 0, message );
  }

  Warning( message )
  {
    this.numWarning += 1;
    Application.DisplayUnitTestMessage( 1, message );
  }

  Error( message )
  {
    this.numError += 1;
    Application.DisplayUnitTestMessage( 2, message );
  }

  Assert( testName, assertion )
  {
    if( !assertion )  {
      this.Error( `${ testName } assertion failed [${ ( Date.now() - this.timestamp ) / 1000 }s]` );
    }
    else {
      this.Success( `${ testName } assertion passed  [${ ( Date.now() - this.timestamp ) / 1000 }s]` );
    }
  }

  AssertObject( testName, object, objectPath, desiredValue )
  {
    let paramPath = objectPath.split( "/" );

    // check existance of parameter
    let value = object;
    for( let param of paramPath ) {
      if( typeof value == "object" && param in value ) {
        value = value[ param ];
      }
      else {
        this.Error( `${ testName } missing parameter \"${ objectPath }\" [${ ( Date.now() - this.timestamp ) / 1000 }s]` );
        return;
      }
    }

    // check desired value
    if( Array.isArray( desiredValue ) ) {
      if( desiredValue.indexOf( value ) < 0 ) {
        this.Error( `${ testName } desired parameter \"${ objectPath }\" is '${ value }' [${ ( Date.now() - this.timestamp ) / 1000 }s]` );
        return;
      }
    }
    else if( desiredValue instanceof RegExp ) {
      if( value.match( desiredValue ) === null ) {
        this.Error( `${ testName } desired parameter \"${ objectPath }\" is '${ value }' [${ ( Date.now() - this.timestamp ) / 1000 }s]` );
        return;
      }
    }
    else if( ( typeof desiredValue == "number" ) || ( typeof desiredValue == "string" ) ) {
      if( value != desiredValue ) {
        this.Error( `${ testName } desired parameter \"${ objectPath }\" is '${ value }' [${ ( Date.now() - this.timestamp ) / 1000 }s]` );
        return;
      }
    }
    else {
      this.Warning( `${ testName } value unknown type [${ ( Date.now() - this.timestamp ) / 1000 }s]` );
    }

    this.Success( `${ testName } desired parameter \"${ objectPath }\" fits [${ ( Date.now() - this.timestamp ) / 1000 }s]` );
  }


}


/* ---------------------------------------------------------------------------
 *   FORM-CONTROLLER - base class
 * ------------------------------------------------------------------------ */

class Viewable
{
  //--- member variables ---
  constructor( )  
  { 
    this.dynid = 0;
    this.etag = null;
    this.$parent = null;      // the element within the viewable is created
    this.$container = null;   // container where all elements are in ($parent or dialog/popup-frame) 
    this.$scrollbar = null;   // scrollbar if needed
    this.$controls = {};      // extra controls in container (buttons in dialogs)
    this.createOptions = null;
  }

  //---------------------------------------------------------------------------
  //  overridables
  //---------------------------------------------------------------------------
  
  //---------------------------------------------------------------------------
  // load form
  //
  Load( data )
  { 
  }

  //---------------------------------------------------------------------------
  //  receive event from parent or system
  //
  OnEvent( eventName, data )
  {
    if( eventName == 'canclose' ) {
      return !data;
    }
  }

  //---------------------------------------------------------------------------
  //  receive event from parent or system
  //
  OnValidate(  )
  {
    return 0;
  }  

  //---------------------------------------------------------------------------
  //  receive event from parent or system
  //
  OnUnitTest( param )
  {
    return true;
  }

  //---------------------------------------------------------------------------
  //  methods
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  destroy everything
  //
  Destroy( )
  {
    let controllerClass = this.$parent.data( "viewable" );
    controllerClass.destroy( );
  }


  //---------------------------------------------------------------------------
  //  send event to creator
  //  
  FireEvent( eventName, data )
  {
    if( $.isFunction( this.createOptions[ eventName ] ) )  {
      return this.createOptions[ eventName ] ( data );
    }
    return null;
  }

  //---------------------------------------------------------------------------
  //  helper function for bubble layout event
  //
  TriggerLayoutEvent()
  {
    this.$container.trigger( "event:layout-changed" );
  }  

  //---------------------------------------------------------------------------
  //  viewable options
  //
  Option( optionName, param )
  {
    // easy reset function 
    if( optionName == "reset" ) {
      this.Option( "modified", 0 );
      this.Option( "validate", false );
      this.Option( "scroll", 0 );
    }

    // "modified" setter/getter
    else if( optionName == "modified" )  {
      if( param != undefined ) {
        if( param == 0 ) {
          this.$parent.viewable( "option", "modified", param );
          this.$parent.parent().trigger( "event:modified-changed", { change: -1 } );
          this.$container.broadcast( "event:modified-reset" );
        }
        else {
          let currentModified = this.$parent.viewable( "option", "modified" );
          this.$parent.viewable( "option", "modified", currentModified + 1 );
          this.$parent.parent().trigger( "event:modified-changed", { change: 1 } );
        }
      }
      else {
        return this.$parent.viewable( "option", "modified" );
      }
    }

    // "error" setter/getter
    else if( optionName == "error" ) {
      let previousError = this.$parent.viewable( "option", "error" );

      if( Number.isInteger( param ) ) {

        this.$parent.viewable( "option", "error", param );
        if( previousError == 0 && param > 0 )  {
          this.$parent.parent().trigger( "event:error-changed", { change: 1 } );
        }
        else if( previousError > 0 && param == 0 ) {
          this.$parent.parent().trigger( "event:error-changed", { change: -1 } );
        }
      }
      else {
        return previousError;
      }
    }

    // "warning" setter/getter
    else if( optionName == "warning" ) {
      let previousWarning = this.$parent.viewable( "option", "warning" );

      if( Number.isInteger( param ) ) {
        this.$parent.viewable( "option", "warning", param );
        if( previousWarning == 0 && param > 0 )  {
          this.$parent.parent().trigger( "event:warning-changed", { change: 1 } );
        }
        else if( previousWarning > 0 && param == 0 ) {
          this.$parent.parent().trigger( "event:warning-changed", { change: -1 } );
        }
      }
      else {
        return previousWarning;
      }
    }

    // "validate" reset
    else if( optionName == "validate" ) {
      if( param != undefined ) {
        if( param === 0 || param === false ) {
          this.$container.broadcast( "event:validate-reset" );
        }
        else {
          console.warn( `Viewable.Option( ${ optionName } ): reset only, pass '0' or 'false'!` );
        }
      }
      else {
        console.warn( `Viewable.Option( ${ optionName } ): no getter, use option 'error'/'warning' to get validation result.` );
      }
    }

    // "scroll" position
    else if( optionName == "scroll" ) {
      if( param == undefined ) {
        console.warn( `Viewable.Option( ${ optionName } ): no position given.` );
        return null;
      }

      this.$scrollbar.scrollbar( "show", param, param );
    }

    // "scrollTo" selector
    else if( optionName == "scrollTo" ) {
      console.warn( `Viewable.Option( ${ optionName } ): Framework still todo.` );
      if( param == undefined ) {
        console.warn( `Viewable.Option( ${ optionName } ): no selector given.` );
        return null;
      }
    }


    return null;
  }

  //---------------------------------------------------------------------------
  //  unit test
  //
  UnitTest( param, option )
  {
    if( Application.IsUnitTestEnabled() ) {
      let unitTestEvent = new UnitTestEvent;
      unitTestEvent.argumentList = $.clone( arguments );
      this.OnUnitTest( unitTestEvent );
      this.$container.broadcast( "unittest", unitTestEvent );
    }
  }


  //---------------------------------------------------------------------------
  //  register the html layout with the class
  //
  static Register( id )
  {
    $.views.Register( id, this.prototype.constructor );
  }
}


!function( $ ) {

/* ---------------------------------------------------------------------------
 *   FORMS - main plugin
 * ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   forms default function
//
$.views = function( type, target )
{
  if( !( type in $.views._data ) ) {
    console.warn( "$.views: type", type, "not found!" );
    return;
  }

  var $element = $( target );
  if( $element instanceof jQuery )  {
    $element.html( $.views._data[ type ]['html'] );
  }
  else  {
    console.warn( "$.views: selector", target, "not found!" );
  }

};

// ----------------------------------------------------
//   Register
//
$.views.Register = function( id, prototype )
{
  if( !( id in $.views._data ) )  {
    $.views._data[ id ] = { 'builder': prototype };
  }
  else  {
    console.warn( "Duplicated View-Class '" + id +"'!" );
  }
};

// ----------------------------------------------------
//   Register
//
$.views.Has = function( id )
{
  return ( id in $.views._data );
};

// ----------------------------------------------------
//   Create a html snipplet
//
$.views.Html = function( id, data, dynid )
{
  if( !( id in $.views._tpl ) )  {
    console.warn( "VIEWS: Try to create html for '" + id + "' => not found. Did you forget to add <views::component>?" );
    return null;
  }

  var html = $.views._tpl[ id ];

  // dynid
  if( dynid !== null )  {
    html = html.replace( new RegExp( "{##}", 'g' ), dynid );
  }

  // lang
  html = $.lang( html );

  // markdown
  if( id in $.views._md )  {
    var mdData = { 'markdown' : $.views._md[ id ] };
    Mustache.escape = function( s )  { return s; };
    html = Mustache.render( html, mdData );
    Mustache.escape = Mustache.escapeHtml;
  }

  // variables
  if( data instanceof Object ) {
    html = this._EncodeData( html, data );
  }

  return html;
};

  // ----------------------------------------------------
  //   PRIVATE: mustache encode the result  
  //
  $.views._EncodeData = function( text, data )
  {
    // prequisites
    if( typeof ( text ) != 'string' )
      return text;
    
    // find {{xxx}}
    var result = text;
    var matches = text.match( /{{\s*[\w\.]+\s*}}/gmi );

    // replace matches
    for( var i = 0; matches && i < matches.length; i++ ) {
      var m = matches[ i ];
      var vname = m.substr( 2, m.length - 4 );
      if( vname in data )  {
        result = result.replace( m, data[ vname ] );
      }
    }
    return result;
  };

// ----------------------------------------------------
//   private data
//
$.views._data = { };
$.views._tpl = { };
$.views._md = { };
$.views.dynid = 0;

//=================================================================
//  VIEW PLUGIN
//=================================================================

var WfcViewable = function( element, options )
{
  this.init( element, options );
};

WfcViewable.prototype = {

  // ----------------------------------------------------
  //    class variables   
  //
  $element: null,   
  $parent: null,   
  $controls: [],
  _error: 0,
  _modified: 0,

  // define keycodes (dialog)
  KEY : {
    RETURN: 13,
    ENTER:  13,
    ESC:    27
  },

  // ----------------------------------------------------
  //   constructor directs back to class function
  //
  constructor: WfcViewable,

  // ----------------------------------------------------
  //   init-function (called by constructor)
  //
  init: function( element, options )  
  {
    var self = this;

    //-----------------------------------------------------
    //  prequistes
    //-----------------------------------------------------

    // class variables
    if( 'createDataByReference' in options && options['createDataByReference'] == true ) {
      this.options = options;
      this.options = $.extend( true, this.options, $.fn.viewable.defaults );
    }
    else {
      this.options = $.extend( true, [], $.fn.viewable.defaults, options );
    }

    this.$element = element;
    var type = this.options['view'];

    // check 
    if( !( this.$element instanceof jQuery ) ) {
      console.warn( "viewable '" + type + "' must be called on a jQuery object!" );
      return;
    }
    if( this.$element.length == 0 ) {
      console.warn( "viewable '" + type + "' no parent selector!" );
      return;
    }
    if( !( type in $.views._tpl ) )  {
      console.warn( "template '" + type + "' does not exist!" );
      return;
    }
    if( !( ( type in $.views._data ) && ( 'builder' in $.views._data[ type ] ) ) )  {
      console.warn( "viewable class for '" + type + "' does not exist!" );
      return;
    }

    //-----------------------------------------------------
    //  build
    //-----------------------------------------------------

    // eTag handling
    if( typeof MutationApi != 'undefined' )  {
      var previousETag = this.$element.attr( 'data-etag' );
      if( previousETag )  {
        MutationApi.RemoveETag( previousETag );
      }
    }

    // build new etag for data-etag attribute
    this.options['etag'] = $.guidgen( );
    this.$element.attr( 'data-etag', this.options['etag'] );
    this.$element.attr( "data-broadcast", "true" );

    // create viewable controller class 
    var controllerClass = new ( $.views._data[ type ]['builder'] )( );

    // build html code and add to $element 
    this.recalcLayout( );

    // fill data into controler class
    controllerClass.etag = this.options['etag'];
    this.options['controller'] = controllerClass;
    controllerClass.createOptions = this.options;
    controllerClass.$parent = this.$element;
    controllerClass.$container = this.$container;
    controllerClass.$controls = this.$controls;
    controllerClass.$scrollbar = null;
    controllerClass.dynid = $.views.dynid;

    // register destructor
    if( typeof MutationApi != 'undefined' ) {
      if( 'destructor' in controllerClass ) {
        MutationApi.RegisterDestructor( controllerClass.etag, controllerClass.destructor, controllerClass );
      }
    }  

    //-----------------------------------------------------
    //  initialize
    //-----------------------------------------------------

    // after rendering call form-class (async)
    setTimeout( ( ) => {

      // load controls
      controllerClass.Load( self.options );

      // install error change handler
      this.$element.on( "event:error-changed", ( ev, param ) => {

        // start
        ev.stopPropagation();
        let previousError = this.options['error'];

        // re-count errors & send event to controller class
        this.options['error'] = this.options['error'] + param['change'];
        if( this.options['error'] < 0 ) this.options['error'] = 0;
        controllerClass.OnEvent( "error", this.options['error'] );

        // send event to parent
        if( previousError == 0 && this.options['error'] > 0 )  {
          this.$element.parent().trigger( "event:error-changed", { change: 1 } );
        }
        else if( previousError > 0 && this.options['error'] == 0 ) {
          this.$element.parent().trigger( "event:error-changed", { change: -1 } );
        }

      } );

      // install warning change handler
      this.$element.on( "event:warning-changed", ( ev, param ) => {

        // start
        ev.stopPropagation();
        let previousError = this.options['warning'];

        // re-count errors & send event to controller class
        this.options['warning'] = this.options['warning'] + param['change'];
        if( this.options['warning'] < 0 ) this.options['warning'] = 0;
        controllerClass.OnEvent( "warning", this.options['warning'] );

        // send event to parent
        if( previousError == 0 && this.options['warning'] > 0 ) {
          this.$element.parent().trigger( "event:warning-changed", { change: 1 } );
        }
        else if( previousError > 0 && this.options['warning'] == 0 ) {
          this.$element.parent().trigger( "event:warning-changed", { change: -1 } );
        }

      } );

      // install modified change handler
      this.$element.on( "event:modified-changed", ( ev, param ) => {
        
        // start
        ev.stopPropagation();
        let previousModified = this.options['modified'];

        // re-count modified & send event to controller class
        this.options['modified'] = this.options['modified'] + param['change'];

        if( this.options['modified'] < 0 ) this.options['modified'] = 0;
        controllerClass.OnEvent( "modified", this.options['modified'] );

        // send event to parent
        if( previousModified == 0 && this.options['modified'] > 0 ) {
          this.$element.parent().trigger( "event:modified-changed", { change: 1 } );
        }
        else if( previousModified > 0 && this.options['modified'] == 0 ) {
          this.$element.parent().trigger( "event:modified-changed", { change: -1 } );
        }


      } );

      // install modified change handler
      this.$element.on( "event:unit-test", ( ev, param ) => {
        controllerClass.OnUnitTest( param );
        ev.stopPropagation();
        ev.preventDefault();
        ev.stopImmediatePropagation();
      } );


    }, 0 );

    // create scrollbar
    var $scroll = controllerClass.$parent.find( ".scroll-container-static" );
    if( $scroll.length == 1 )  {
      controllerClass.$scrollbar = $scroll;
      controllerClass.$scrollbar.scrollbar( { watch: true, keyboard: true } );
    }
    else  {
      $scroll = controllerClass.$parent.find( ".scroll-container" );
      if( $scroll.length == 1 )  {
        controllerClass.$scrollbar = $scroll;
        controllerClass.$scrollbar.scrollbar( { watch: true } );
      }
    }

    if( controllerClass.$scrollbar )  {
      setTimeout( () =>  {
        controllerClass.$scrollbar.scrollbar( "recalc" );
      }, 250 );
    }

  },

  // ----------------------------------------------------
  //   build html
  //
  recalcLayout: function(  )
  {
    if( this.options['type'] == 'dialog' )  {
      this.recalcLayoutDialog( );
    }
    else  {
      this.recalcLayoutStandard( );
    }
  },

  // ----------------------------------------------------
  //   standard: build html
  //
  recalcLayoutStandard: function(  )
  {
    let type = this.options['view'];
    let data = ( 'data' in this.options ) ? this.options['data'] : null;
    let html = $.views.Html( type, data, ++$.views.dynid );

    this.$element.html( html );
    this.$container = this.$element;
  },


  //-------------------------------------------------------
  //  DIALOG CODE
  //-------------------------------------------------------

  // ------------------------------------------------------
  //   dialog: build html
  //
  recalcLayoutDialog: function(  )
  {
    // prequisites
    let type = this.options['view'];
    let dynid = ++$.views.dynid ;

    // encode language
    let headerData = $.lang( this.options['header'] );
    if( !headerData )  {
      headerData = { title: "" };
    }

    // build dialog html using the template
    let dialogTemplate = '/lib/viewable/dialog/classic';
    switch( this.options['layout'] )  {
      case 'classic2':  dialogTemplate = '/lib/viewable/dialog/classic2';   break;
      case 'classic':   dialogTemplate = '/lib/viewable/dialog/classic';    break;
      case 'simple':    dialogTemplate = '/lib/viewable/dialog/simple';     break;
    }
    let $overlay = $( $.views.Html( dialogTemplate, headerData, dynid ) );
    this.$element.append( $overlay );
    if( ! Array.isArray( this.options.buttons ) )  {
      this.attachDialogOverlayEvents( $overlay );
    }

    // get dialogBox 
    this.$container = $overlay.find( '.pageDialog' );
    this.attachDialogEvents( this.$container );
    this.recalcLayoutDialogSize( this.$container );

    // layout buttons or remove if not needed
    let $footer = this.$container.find( '.dialog-footer' );
    if( $footer.length > 0 )  {
      if( !( 'buttons' in this.options ) || this.options['buttons'].length <= 0 )  {
        $footer.remove();
      }
      else  {
        this.recalcLayoutDialogButtons( $footer );
      }
    }

    // append content 
    let $content = this.$container.find( '.dialog-content' );
    if( 'style' in this.options )  {
      $content.addClass( this.options['style'] );
    }
    $content.html( $.views.Html( type, this.options, dynid ) );

  },

  recalcLayoutDialogSize: function( $box )
  {
    // ---- width & center ----
    if( typeof this.options['size'] == 'string' )  {
      $box.css( 'width', this.options['size'] );
      let outerWidth = $box.outerWidth( );
      let frameWidth = $( window ).outerWidth();
      let percent = outerWidth / frameWidth * 100;
      $box.css( 'left', ( ( 100 - percent ) / 2 ).toString() + "%" );
    }
    else if( this.options['size'] > 0 ) {
      $box.css( 'width', this.options['size'] * $.metrics( 'fontWidth' ) );
      let outerWidth = $box.outerWidth();
      let frameWidth = $( window ).outerWidth();
      let percent = outerWidth / frameWidth * 100;
      $box.css( 'left', ( ( 100 - percent ) / 2 ).toString() + "%" );
    }
    else if( this.options['size'] == -1 )  {
      $box.css( 'width', "100%" );
    }

    // ---- height & center ----
    if( typeof this.options['height'] == 'string' )  {
      $box.css( 'height', this.options['height'] );
      let outerHeight = $box.outerHeight( );
      let frameHeight = $( window ).outerHeight(); //this.$element.outerHeight( );
      let percent = outerHeight / frameHeight * 110;
      $box.css( 'top', ( ( 100 - percent ) / 2 ).toString() + "%" );
    }
    else if( this.options['height'] > 0 ) {
      $box.css( 'height', this.options['height'] * $.metrics( 'fontHeight' ) );
      let outerHeight = $box.outerHeight();
      let frameHeight = $( window ).outerHeight(); //this.$element.outerHeight( );
      let percent = outerHeight / frameHeight * 110;
      $box.css( 'top', ( ( 100 - percent ) / 2 ).toString() + "%" );
    }
    else if( this.options['height'] == -1 )  {
      $box.css( 'top', "20%" );
    }

    if( 'top' in this.options ) {
      $box.css( 'top', this.options['top'] );
    }

  },

  recalcLayoutDialogButtons: function( $footer )
  {
    for( let i = 0; i < this.options['buttons'].length; i++ )  {

      var controlData = $.clone( this.options['buttons'][ i ] );
      controlData['click'] = $.proxy( this.clickDialogButton, this );

      let $button = $( "<div>" );
      $button.button( controlData );
      this.$controls[ controlData['id'] ] = $button;
      
      $footer.append( $button );
      if( i < ( this.options['buttons'].length - 1 ) ) {
        $footer.append( $( "<s class='s'>" ) );
      }
    }
  },

  clickDialogButton( id )
  {
    let controller = this.options['controller'];    
    controller.OnEvent( 'button', id );
  },

  attachDialogEvents( $box )
  {
    var selfDialog = this;

    // click close button 
    let $header = $box.find( '.dialog-title' );
    let $closeButton = $header.find( '.close' );
    if( $closeButton.length > 0 )  {
      $closeButton.click( ( ) => { 
        let controller = selfDialog.options['controller'];
        if( controller.OnEvent( 'canclose', false ) !== false )  {
          selfDialog.destroy();
        }
      } );
    }

    // click into dialog => do nothing
    $box.click( ( ev ) =>  {
      // radio & checkbox needs click event
      if( ev.target.tagName == "LABEL" || ev.target.tagName == "INPUT" )
        return;
      ev.stopPropagation( );
      ev.preventDefault();
    } );
    $box.focus( );
    
    // close on ESC/ENTER
    $box.keyup( ( e ) =>   {
      if( e.which == this.KEY.ESC || e.which == this.KEY.ENTER )  {
        let controller = selfDialog.options['controller'];
        let eventResult = controller.OnEvent( 'canclose', e.which != this.KEY.ESC );
        if( ( typeof eventResult == 'boolean' && eventResult == true )
         || ( typeof eventResult != 'boolean' && e.which == this.KEY.ESC )
          ) {
          selfDialog.destroy();
        }
      }
    } );    
  },

  attachDialogOverlayEvents( $overlay )
  {
    var selfDialog = this;

    // click outside of dialog (modalframe) => close
    $overlay.click( ( ev ) =>  {
      let controller = selfDialog.options['controller'];
      let eventResult = controller.OnEvent( 'canclose', false );
      if( typeof eventResult == 'boolean' )  {
        if( eventResult == true )  {
          ev.stopPropagation();
          ev.preventDefault();
          selfDialog.destroy();
        }
      }
      else  {
        ev.stopPropagation();
        ev.preventDefault();
        selfDialog.destroy();
      }
    } );
  },

  //-------------------------------------------------------
  //  DESTROY
  //-------------------------------------------------------

  destroy: function( )
  {
    this.validate( [ "validate", false ] );

    this.$element.off();

    if( this.options['type'] == 'dialog' )  {
      // remove element
      let $frame = this.$element.find( '.dialog' );
      if( $frame )  {
        $frame.remove( );      
      }
    }
    else  {
      // empty element
      //this.$element.removeAttr( 'data-etag' );
    }

    // call registered destructors
    if( typeof MutationApi != 'undefined' ) {
      var previousETag = this.$element.attr( 'data-etag' );
      if( previousETag ) {
        MutationApi.RemoveETag( previousETag );
      }
      this.$element.removeAttr( 'data-etag' );
    }

  },


  // ----------------------------------------------------
  //   call viewable class function
  //
  controller: function( args )  
  {
    var controllerClass = this.options['controller'];
    if( controllerClass == null )  {
      console.warn( "controller for '" + this.options.view + "' not registered!" );
      return;
    }

    var eventName = args[ 1 ];
    if( eventName in controllerClass )  {
      if( $.isFunction( controllerClass[ eventName ] ) )  {
        return controllerClass[ eventName ]( args[ 2 ] );
      }
    }
  },

  // ----------------------------------------------------
  //   get viewable class
  //
  viewable: function( args )
  {
    var controllerClass = this.options['controller'];
    if( controllerClass == null ) {
      console.warn( "controller for '" + this.options.view + "' not registered!" );
    }
    return controllerClass;
  },

  // ----------------------------------------------------
  //   call Viewable.OnEvent( eventName, data )
  //
  event: function( args )    
  {
    var controllerClass = this.options['controller'];
    if( controllerClass != null )  {
      return controllerClass.OnEvent( args[ 1 ], args[ 2 ], args[ 3 ] );
    }
  },

  // ----------------------------------------------------
  //   option getter and setter
  //
  option: function( args )  
  {
    // setter
    if( args.length > 2 ) {
      if( args[ 1 ] == "error" )  {
        this.options['error'] = args[ 2 ];
        this.event( [ 0, "error", args[ 2 ] ] );
      }
      else if( args[1] == "warning" ) {
        this.options['warning'] = args[ 2 ];
        this.event( [ 0, "warning", args[ 2 ] ] );
      }
      else if( args[ 1 ] == "modified" )  {
        this.options['modified'] = args[ 2 ];
        this.event( [ 0, "modified", args[ 2 ] ] );
      }
      else  {
        this.options[ args[ 1 ] ] = args[ 2 ];
        this.event( [ 0, "option", args[ 1 ], args[ 2 ] ] );
      }
      return this;
    }

    // getter
    if( args[ 1 ] == "error"
     || args[ 1 ] == "warning"
     || args[ 1 ] == "modified"
      ) {
      return this.options[ args[ 1 ] ];
    }
    else {
      let userEventResult = this.event( [ 0, "option", args[ 1 ] ] );
      if( userEventResult !== undefined ) {
        return userEventResult;
      }
      else  {
        return this.options[ args[ 1 ] ];
      }
    }
  },    


  // ----------------------------------------------------
  //   option getter and setter
  //
  validate: function( args )
  {
    var controllerClass = this.options['controller'];
    if( controllerClass != null ) {
      if( ( args.length > 1 ) 
       && ( args[ 1 ] === false ) 
       && ( 'OnValidateOff' in controllerClass )
        ) {
        controllerClass.OnValidateOff();
        return 0;
      }
      return controllerClass.OnValidate( ( args.length > 1 ) ? args[ 1 ] : undefined );
    }
    return 0;
  },

  // ----------------------------------------------------
  //   option getter and setter
  //
  unittest: function( args )
  {
    var controllerClass = this.options['controller'];
    if( controllerClass != null ) {
      return controllerClass.OnUnitTest( args[ 1 ] );
    }
    return true;
  },


}; // end plugin

// ----------------------------------------------------
//   plugin main function
//
$.fn.viewable = function( option )
{
  var data    = $( this ).data( "viewable" ),
      options = ( typeof option == "object" && option );

  // remove for re-create
  if( typeof option == "object" && data )  {
    data.destroy( );
    data = null;
    $( this ).data( "viewable", null );
  }

  // if this plugin isn't installed, create one
  if( !data ) {
    var instance = new WfcViewable( this, options );
    $( this ).data( "viewable", instance );
  }

  // if called with a string, treat this as a function call
  else if( typeof option == "string" ) {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).data( "viewable", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) ) {
      return data[ option ]( arguments );
    }
    else {
      console.warn( "call to undefined function '" + option + "' in viewable '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.viewable.defaults = {
  dynid: 0,
  error: 0,
  warning: 0,
  modified: 0,
  controller: null  // class ViewController
};

}( window.jQuery );//=============================================================================
//
//  Viewable Component class
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.01.2019
//=============================================================================

//=================================================================
//  JC Control Base class
//=================================================================

!function( $ ) {
  
$.jqc = function() {};
$.jqc.statics = {
  themeClassList: ['hilight', 'red', 'green', 'gray', 'yellow', 'locked', 'selected', 'disabled', 'readonly', 'light', 'default', 'normal', 'expanded', 'collapsed', 'error', 'focus', 'hover', 'no-hover', 'hidden', 'cursor', 'diverter' ],
  themeBaseStateList: [ 'normal', 'disabled', 'readonly', 'light', 'default', 'hilight', 'red', 'green', 'yellow', 'locked' ],
  themeMixinList: [ 'error', 'warning', 'focus', 'selected', 'modified', 'expanded', 'collapsed', 'no-hover', 'hidden', 'cursor', 'striked', 'underlined', 'diverter' ],
  themeMixinGroup: [ 'expanded', 'collapsed' ],
  themeNoHoverList : [ 'readonly', 'disabled', 'no-hover' ],
  itemDataTemplate: {
    id: null,
    status: [ ],
    icon: null,
    text: null,
    type: null,
    tooltip: null,
    placeholder: null,
    disabled: false,
    $item: null
  }
};
$.jqc.dragdrop = null;

}( window.jQuery );


//=================================================================
//  Validation control class
//=================================================================

class ValidationControl
{
  //-------------------------------------------------------
  //  init class variables
  //
  constructor( )
  {
    this._value = null;
    this._error = null;
    this._errorChange = 0;
    this._warning = null;
    this._warningChange = 0;
  }

  //-------------------------------------------------------
  //  setter: value
  //
  set value( val )  { this._value = val;  }

  //-------------------------------------------------------
  //  getter: value
  //
  get value() 	    { return this._value; }

  //-------------------------------------------------------
  //  setter: error 
  //
  set error( errorValue )
  {
    // change: no error => error
    if( ( this._error == null ) && ( typeof errorValue == "string" ) ) {
      this._errorChange = 1;
      this.warning = null;
    }
    // change: error => no error
    else if( ( this._error != null ) && ( typeof errorValue != "string" ) ) {
      this._errorChange = -1;
    }

    this._error = errorValue;
  }

  //-------------------------------------------------------
  //  getter: error
  //
  get error()     { return this._error; }  

  //-------------------------------------------------------
  //  setter: warning 
  //
  set warning( warningValue )
  {
    // change: no error => error
    if( ( this._warning == null ) && ( typeof warningValue == "string" ) ) {
      this._warningChange = 1;
    }
    // change: error => no error
    else if( ( this._warning != null ) && ( typeof warningValue != "string" ) ) {
      this._warningChange = -1;
    }

    this._warning = warningValue;
  }

  //-------------------------------------------------------
  //  getter: warning
  //
  get warning() { return this._warning; }

  //-------------------------------------------------------
  //  get change of error parameter since last call
  //
  NumErrorChanged()
  {
    let result = this._errorChange;
    this._errorChange = 0;
    return result;
  }

  //-------------------------------------------------------
  //  get change of error parameter since last call
  //
  NumWarningChanged()
  {
    let result = this._warningChange;
    this._warningChange = 0;
    return result;
  }

}


//=================================================================
//  JC Control Base class
//=================================================================

class JqControl
{
  //---------------------------------------------------------------------------
  //  constructor will call init 
  constructor( $element, options )
  {
    this.$element = $element;  //$element;
    this.id = $element.prop( "id" );

    this.$element.attr( "data-broadcast", true );
    this.$error = null;
    this.options = { };
    this._items = { };
    this._selected = [ ];
    this._validationControl = new ValidationControl;
    this._isValidationEnabled = false;
    this._validationSubscriptionList = { };

    // copy 'status' to 'statusDefault'
    if( ( $.isPlainObject( options ) )
     && ( 'status' in options )
     && !( 'statusDefault' in options ) 
      ) {
      options['statusDefault'] = options['status'];
    }

    // needed default options
    if( $.isPlainObject( options ) )  {
      options['modified'] = 0;
      options['error'] = null;
      options['warning'] = null;
      if( !( 'modifiedEnabled' in options ) ) {
        options['modifiedEnabled'] = true;
      }
    }

    // initialize control
    this.OnInit( $element, options );
    this.OnDraw( );
    this.OnInitFrameworkEvents( $element );

    //console.log( "construct: ", this.options['statusDefault'] );


    // send an event to upper elements 
    this.TriggerLayoutEvent( );
  }

  //---------------------------------------------------------------------------
  //  helper function for bubble layout event
  //
  TriggerLayoutEvent( )
  {
    this.$element.trigger( "event:layout-changed" );
  }


  //---------------------------------------------------------------------------
  //  helper function for modified framework
  //
  ManageModified( isModified )
  {
    // modified counter disabled?
    if( !( this.options['modifiedEnabled'] ) ) {
      return;
    }

    // set modified
    if( isModified  && !( this.options['modified'] ) )  {
      this.$element.trigger( "event:modified-changed", { change: 1 } );
    }
    // clear modified
    else if( ! isModified && this.options['modified'] ) {
      this.$element.trigger( "event:modified-changed", { change: -1 } );
    }

    this.options['modified'] = isModified;
  }

  //---------------------------------------------------------------------------
  //  helper function for disabled overlay
  //
  ManageDisabledOverlay( isDisabled )
  {
    if( isDisabled )  {
      this.$element.find( ".disabled-overlay" ).remove();
      this.$element.append( "<div class='disabled-overlay'></div>" );
      this.$element.find( ".disabled-overlay" ).click( ( ev ) => {
        ev.preventDefault();
        ev.stopPropagation();
      } )
    }
    else {
      this.$element.find( ".disabled-overlay" ).remove();
    }
  }

    
  //---------------------------------------------------------------------------
  //  build item from template-id 
  //
  ItemCreate( templateId, options )
  {
    var $item;
    var cssClasses = '';

    if( typeof templateId != 'string' )  {
      console.warn( "JqControl::ItemCreate => 'templateId' must be string" );
      return null;
    }

    // manage status
    options._jqcItem = $.clone( $.jqc.statics.itemDataTemplate );
    this._ManageItemData( options._jqcItem, options );

    // build item from html-template    
    var html = $.views.Html( templateId, options._jqcItem );
    if( typeof html != 'string' )  {
      console.warn( "JqControl::ItemCreate => templateId='" + templateId + "' not found!" );
      return null;
    }
    $item = $( html );
    cssClasses = $item.attr( 'class' );

    // finally build css-classes
    cssClasses = this._ManageStatus( 'add', options._jqcItem, ( 'status' in options ) ? options['status'] : "", cssClasses );

    // store inside of tag
    $item.data( 'jqcItem', options._jqcItem );
    $item.attr( 'class', cssClasses );

    // tooltip
    if( options['tooltip'] )  {
      $item.attr( 'title', options['tooltip'] );
    }

    // return
    options._jqcItem.$item = $item;
    return $item;
  }

  //---------------------------------------------------------------------------
  //  remove an existing item  
  //
  ItemDelete( $item )
  {
    var itemData = this.ItemGetData( $item );
    if( itemData['id'] ) {
      delete this._items[ itemData['id'] ];
    }
    $item.remove( );  
  }

  //---------------------------------------------------------------------------
  //  modify an existing item  
  //
  ItemInitialize( $item, options )
  {
    var cssClasses = '';

    // build itemData && manage status
    cssClasses = $item.attr( 'class' );
    options._jqcItem = $.clone( $.jqc.statics.itemDataTemplate );
    options._jqcItem['id'] = $item.attr( 'id' );
    this._ManageItemData( options._jqcItem, options );
    cssClasses = this._ManageStatus( 'add', options._jqcItem, ( 'status' in options ) ? options['status'] : "", cssClasses );
    
    // store inside of tag & set classes
    $item.data( 'jqcItem', options._jqcItem );
    $item.attr( 'class', cssClasses );

    // return
    options._jqcItem.$item = $item;
    return $item;
  }


  //---------------------------------------------------------------------------
  //  get item data  
  //
  ItemGetData( $item )
  {
    return $item.data( 'jqcItem' );
  }

  //---------------------------------------------------------------------------
  //  checks if an item has a specific status
  //
  ItemHasStatus( $item, statusString )
  {
    var itemData = this.ItemGetData( $item );
    return ( itemData['status'].indexOf( statusString ) >= 0 );
  }

  //---------------------------------------------------------------------------
  //  set status, return new status string
  //  if a base status is set => new new base status is set, the old cleared
  //  if an mixin status is set => new extended status is set
  //
  ItemSetStatus( $item, statusString )
  {
    if( statusString == null && 'statusDefault' in this.options )  {
      statusString = this.options['statusDefault'];
      //console.log( "ctrl.ItemSetStatus (default): ", statusString, this );
    }
    else {
      //console.log( "ctrl.ItemSetStatus: ", statusString, this );
    }

    // get item data
    var itemData = this.ItemGetData( $item );

    // calc new css classes
    var cssClasses = $item.attr( 'class' );
    cssClasses = this._ManageStatus( 'set', itemData, statusString, cssClasses );
    $item.attr( 'class', cssClasses );

    return itemData['status'].join( ' ' );
  }

  //---------------------------------------------------------------------------
  //  add status, return new status string
  //  if a base status is set => new new base status is set, the old cleared
  //  if an mixin status is set => new extended status is added to status list
  //
  ItemAddStatus( $item, statusString )
  {    
    // get item data
    var itemData = this.ItemGetData( $item );
    if( !itemData ) {
      return statusString;
    }

    // calc new css classes
    var cssClasses = $item.attr( 'class' );
    cssClasses = this._ManageStatus( 'add', itemData, statusString, cssClasses );
    $item.attr( 'class', cssClasses );

    return itemData['status'].join( ' ' );
  }  

  //---------------------------------------------------------------------------
  //  remove status, return new status string
  //  if the current base status is removed => new base status is 'normal'
  //  if mixin status is removed => all mixin status are removed
  //
  ItemRemoveStatus( $item, statusString )
  {    // get item data
    var itemData = this.ItemGetData( $item );
    if( !itemData ) {
      return statusString;
    }

    // calc new css classes
    var cssClasses = $item.attr( 'class' );
    cssClasses = this._ManageStatus( 'remove', itemData, statusString, cssClasses );
    $item.attr( 'class', cssClasses );

    return itemData['status'].join( ' ' );
  }  

  //---------------------------------------------------------------------------
  //  toogle statue between two values]
  //  i.g. [ 'expanded', 'collapsed' ]
  //
  ItemToggleStatus( $item, statusToogleList )
  {    
    // get item data
    var itemData = this.ItemGetData( $item );
    if( !itemData ) {
      return statusString;
    }

    var result;

    // calc new css classes
    var cssClasses = $item.attr( 'class' );
    var cssClassList = cssClasses.split( /\s+/ );
    var toogleIndex0 = cssClassList.indexOf( statusToogleList[ 0 ] );
    $.array.Remove( cssClassList, statusToogleList );
    $.array.Remove( itemData['status'], statusToogleList );
    
    if( toogleIndex0 >= 0 )  {
      result = statusToogleList[ 1 ];
    }
    else  {
      result = statusToogleList[ 0 ];
    }

    itemData['status'].push( result  );
    cssClassList.push( result );
    $item.attr( 'class', cssClassList.join( ' ' ) );

    return result;
  }    

  //===========================================================================
  //  ITEM LIST FUNCTIONS
  //===========================================================================

  ListGetItem( id )
  {
    if( id in this._items )  {
      return this._items[ id ];
    }
    return null;
  }

  ListGetItemIds( )
  {
    return ( this._items ).keys( );
  }

  ListSelectItem( id, isSingle )
  {
    if( id in this._items )  {

      var itemDataSelect = this._items[ id ];
      var isSelected = ( isSingle ) ? false : this.ItemHasStatus( itemDataSelect.$item, 'selected' );

      if( isSingle )  {
        for( var itemId in this._items )  {
          var itemData = this._items[ itemId ];
          this.ItemRemoveStatus( itemData.$item, 'selected' );
        }  
      }

      if( isSelected )  {
        this.ItemRemoveStatus( itemDataSelect.$item, 'selected' );
      }
      else  {
        this.ItemAddStatus( itemDataSelect.$item, 'selected' );
      }
    }
    return isSingle ? this._selected[ 0 ] : this._selected;
  }

  ListGetSelection( isSingle = false )
  {
    return isSingle ? this._selected[ 0 ] : this._selected;
  }

  ListAddStatus( status )
  {
    for( var itemId in this._items ) {
      var itemData = this._items[itemId];
      this.ItemAddStatus( itemData.$item, status );
    }  
  }

  ListRemoveStatus( status )
  {
    for( var itemId in this._items ) {
      var itemData = this._items[itemId];
      this.ItemRemoveStatus( itemData.$item, status );
    }
  }

  ListSetStatus( status )
  {
    for( var itemId in this._items ) {
      var itemData = this._items[itemId];
      this.ItemSetStatus( itemData.$item, status );
    }
  }


  //===========================================================================
  //  ITEM LIST FUNCTIONS
  //===========================================================================

  ControlCalculateSize( size, defaultValue )
  {
    if( typeof size === 'number' ) {
      if( size > 0 ) {
        return size * $.metrics( 'fontWidth' );
      }
      else if( size == -1 )  {
        return "100%";
      }
    }
    else if( typeof( size ) == "string" )  {
      if( size.indexOf( "%" ) )  {
        return size;
      }
    }
    else  {
      return defaultValue;
    }
  }

  ControlCalculateHeight( height )
  {
    if( typeof height === 'number' )  {
      if( height > 0 ) {
        return height * $.metrics( 'fontHeight' );
      }
      else if( height == -1 )  {
        return "100%";
      }
    }
    else if( typeof( height ) == "string" )  {
      if( height.indexOf( "%" ) )  {
        return height;
      }
    }
    else  {
      return 16;
    }
  }

  ControlShowMessage( $item, error, warning, anchor, position = null )
  {
    let message = null;
    let isWarning = false;
    let isError = false;

    if( position == "none" ) {
      return this.ControlShowMessageNone( $item, error, warning, anchor );
    }    

    // check css-variables that are defined by css class "error-below"
    let cssErrorPosition = String( $item.css( "--error-position" ) ).trim( );
    if( cssErrorPosition == "below" ) {
      return this.ControlShowMessageBelow( $item, error, warning, anchor );
    }
    else if( typeof position == "string" ) {
      if( position == "below" ) {
        return this.ControlShowMessageBelow( $item, error, warning, anchor );
      }
    }

    // set error/warning flags (error has priority)
    if( typeof error == "string" && error.length > 0 ) {
      message = error;
      isWarning = false;
      isError = true;
    }
    else if( typeof warning == "string" && warning.length > 0 ) {
      message = warning;
      isWarning = true;
      isError = false;
    }

    // tabula rasa
    this.ItemRemoveStatus( this.$element, "error" );
    this.ItemRemoveStatus( this.$element, "warning" );
    if( this.$error != null )  {
      this.$error.empty();
    }

    // nothing to do
    if( !isWarning && !isError )  {
      if( this.$error != null && ! this.$error.hasClass( "message-text" ) ) {
        this.$error.remove();
      }
      return;
    }

    // try to find the error-anchor
    var $parent = this.$element.parent();
    this.$error = $parent.find( '.message-text' );
    if( typeof ( anchor ) == "string" ) {
      var $grandparent = $parent.parent();
      this.$error = $grandparent.find( anchor );
    }

    // no error tag, create one
    if( this.$error.length == 0 ) {
      if( isError ) {
        this.$error = $( "<div class='message-text error red-text' style='position:absolute'><em>" + $.lang( message ) + "</em></div>" );
        this.ItemAddStatus( this.$element, "error" );
      }
      else {
        this.$error = $( "<div class='message-text warning normal-text' style='position:absolute'><em>" + $.lang( message ) + "</em></div>" );
        this.ItemAddStatus( this.$element, "warning" );
      }
      this.$element.append( this.$error );
    }
    // set new text into error tag
    else {
      var $em = $( "<em>" + $.lang( message ) + "</em>" );
      this.$error.append( $em );
      this.$error.css( 'margin-left', + $.metrics( 'fontWidth' ) / 2 );
      if( isError ) {
        this.$error.addClass( 'error red-text' );
        this.ItemAddStatus( this.$element, "error" );
      }
      else {
        this.$error.addClass( 'warning normal-text' );
        this.ItemAddStatus( this.$element, "warning" );
      }
    }

    //if( typeof position == "string" ) {
    //  if( position == "right" ) {
    //this.$error.css( "position", "absolute" );
    let relativeX = this.$element.outerWidth() + $.metrics( 'fontWidth' ) / 2;
    this.$error.css( "left", relativeX );
    //  }
    //}

  }

  ControlShowMessageBelow( $item, error, warning, anchor )
  {
    let message = null;
    let isWarning = false;
    let isError = false;

    // set error/warning flags (error has priority)
    if( typeof error == "string" && error.length > 0 ) {
      message = error;
      isWarning = false;
      isError = true;
    }
    else if( typeof warning == "string" && warning.length > 0 ) {
      message = warning;
      isWarning = true;
      isError = false;
    }

    // tabula rasa
    this.ItemRemoveStatus( this.$element, "error" );
    this.ItemRemoveStatus( this.$element, "warning" );
    if( this.$error != null ) {
      this.$error.remove();
    }

    // nothing to do
    if( !isWarning && !isError ) {
      this.$element.css( "margin-bottom", "" );
      return;
    }

    // try to find the error-anchor
    var $parent = this.$element.parent();
    this.$error = $parent.find( '.message-text' );
    if( typeof( anchor ) == "string" ) {
      var $grandparent = $parent.parent();
      this.$error = $grandparent.find( anchor );
    }

    // no error tag, create one
    if( this.$error.length == 0 ) {
      if( isError ) {
        this.$error = $( "<div class='message-text error red-text' style='position:absolute'><em>" + $.lang( message ) + "</em></div>" );
        this.ItemAddStatus( this.$element, "error" );
      }
      else {
        this.$error = $( "<div class='message-text warning normal-text' style='position:absolute'><em>" + $.lang( message ) + "</em></div>" );
        this.ItemAddStatus( this.$element, "warning" );
      }
      this.$element.append( this.$error );
    }
    // set new text into error tag
    else {
      var $em = $( "<em>" + $.lang( message ) + "</em>" );
      this.$error.append( $em );
      this.$error.css( 'margin-left', + $.metrics( 'fontWidth' ) / 2 );
      if( isError ) {
        this.$error.addClass( 'error red-text' );
        this.ItemAddStatus( this.$element, "error" );
      }
      else {
        this.$error.addClass( 'warning normal-text' );
        this.ItemAddStatus( this.$element, "warning" );
      }
    }

    let errorHeight = this.$error.outerHeight();
    this.$element.css( "margin-bottom", errorHeight );
    this.$error.css( "position", "absolute" );
    let relativeY = this.$element.outerHeight() - $.metrics( 'fontHeight' ) / 8;
    this.$error.css( "top", relativeY );

  }

  ControlShowMessageNone( $item, error, warning, anchor )
  {
    // set error/warning flags (error has priority)
    if( typeof error == "string" ) {
      this.ItemAddStatus( this.$element, "error" );
    }
    else if( typeof warning == "string" ) {
      this.ItemAddStatus( this.$element, "warning" );
    }
    else  {
      this.ItemRemoveStatus( this.$element, "error" );
      this.ItemRemoveStatus( this.$element, "warning" );
    }

  }

  old_ControlShowMessage( $item, error, warning, anchor, position = null )
  {
    // 
    let message = null;
    let isWarning = false;
    if( typeof error == "string" && error.length > 0 )  {
      message = error;
      isWarning = false;
    }
    if( typeof warning == "string" && warning.length > 0 )  {
      message = warning;
      isWarning = true;
    }
    this.ItemRemoveStatus( this.$element, "error" );
    this.ItemRemoveStatus( this.$element, "warning" );


    var itemData = this.ItemGetData( $item );
    if( message != null )  {
        
      // create error html tag
      if( !this.$error )  {

        var $parent = this.$element.parent( );
        itemData.$error = this.$error = $parent.find( '.message-text' );
        if( typeof( anchor ) == "string" )  {
          var $grandparent = $parent.parent();
          itemData.$error = this.$error = $grandparent.find( anchor );
        }

        if( this.$error.length == 0 )  {
          if( isWarning )  {
            this.$error = $( "<div class='warning normal-text' style='position:absolute'><em>" + $.lang( this.options.error ) + "</em></div>" );
          }
          else  {
            this.$error = $( "<div class='error red-text' style='position:absolute'><em>" + $.lang( this.options.error ) + "</em></div>" );
          }
          this.$element.append( this.$error );
          this.$error.css( 'position',  'absolute' );
          let relativeX = this.$element.outerWidth( ) + $.metrics( 'fontWidth' ) / 2;
          this.$error.css( 'left', relativeX );
        }
        else  {
          var $em = $( "<em>" + $.lang( this.options.error ) + "</em>" );
          this.$error.append( $em );
          this.$error.css( 'margin-left',  + $.metrics( 'fontWidth' ) / 2 );
          if( isWarning ) {
            this.$error.removeClass( 'error red-text' );
            this.$error.addClass( 'warning normal-text' );
          }
          else {
            this.$error.removeClass( 'warning normal-text' );
            this.$error.addClass( 'error red-text' );
          }
          if( typeof position == "string" )  {
            if( position == "right" )  {
              this.$error.css( "position", "absolute" );
              let relativeX = this.$element.outerWidth() + $.metrics( 'fontWidth' ) / 2;
              this.$error.css( "left", relativeX );
            }
          }
        }
      }

      // set text
      this.$error.find( 'em' ).text( $.lang( message) );

      // add error status to element
      if( isWarning ) {
        this.$error.removeClass( 'error red-text' );
        this.ItemRemoveStatus( this.$element, "error" );
        this.ItemAddStatus( this.$element, "warning" );
      }
      else {
        this.$error.removeClass( 'warning normal-text' );
        this.ItemRemoveStatus( this.$element, "warning" );
        this.ItemAddStatus( this.$element, "error" );
      }
    }

    // remove
    else if( this.$error ) {
      this.ItemRemoveStatus( this.$element, "error" );
      this.ItemRemoveStatus( this.$element, "warning" );
      if( typeof ( anchor ) == "string" )  {
        this.$error.empty();
      }
      else if( !this.$error.hasClass( 'message-text' ) )  {
        this.$error.remove( );
      }
      else  {
        this.$error.empty( );
      }
      this.$error = null;
    }
  }



  //===========================================================================
  //  HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  calculate the status and returns css-class  
  //  if a base status is set => new new base status is set, the old cleared
  //  if an mixin status is set => new mixin status is set, othw. keep existing
  //
  _ManageStatus( term, itemData, optionsStatus, cssClasses )
  {
    var cssClassList = [];
    var optionsClassList = [];
    var cssPreviousClasses = cssClasses;

    var cssBaseStatus = null;       // base status set in css
    var optionsBaseStatus = null;   // base status set in options
    var itemBaseStatus = null;      // base status set in itemData
    var resultBaseStatus = 'normal';

    var cssMixinStatusList = [];       // mixin status set in css
    var optionsMixinStatusList = [];   // mixin status set in options
    var itemMixinStatusList = [];      // mixin status set in itemData
    var resultMixinStatusList = [];

    //---------------------------------------------------------------
    //  PREPARE
    //---------------------------------------------------------------

    // options status (if set) and 
    if( typeof optionsStatus == 'string' )  {
      optionsClassList = optionsStatus.split( /\s+/ );
      optionsBaseStatus = $.array.FirstIn( optionsClassList, $.jqc.statics.themeBaseStateList );
      optionsMixinStatusList = $.array.In( optionsClassList, $.jqc.statics.themeMixinList );
    }
    
    // split css classes (if set) and get base & mixin status
    if( typeof cssClasses == 'string' )  {
      cssClassList = cssClasses.split( /\s+/ );
      cssBaseStatus = $.array.FirstIn( cssClassList, $.jqc.statics.themeBaseStateList );
      cssMixinStatusList = $.array.In( cssClassList, $.jqc.statics.themeMixinList );
    }

    // itemData get base & mixin status
    if( Array.isArray( itemData['status'] ) )  {
      itemBaseStatus = $.array.FirstIn( itemData['status'], $.jqc.statics.themeBaseStateList );
      itemMixinStatusList = $.array.In( itemData['status'], $.jqc.statics.themeMixinList );
    }

    // filter all theme status from css class-list
    optionsClassList = $.array.NotIn( optionsClassList, $.jqc.statics.themeClassList );
    cssClassList     = $.array.NotIn( cssClassList, $.jqc.statics.themeClassList );

    //---------------------------------------------------------------
    //  RESULT
    //---------------------------------------------------------------

    // --- base status ---
    // one of [ 'normal', 'disabled', 'readonly', 'hilight', 'red', 'green', 'yellow' ]
    if( optionsBaseStatus != null )  {
      resultBaseStatus = optionsBaseStatus;
    }
    else if( cssBaseStatus != null )  {
      resultBaseStatus = cssBaseStatus;
    }
    else if( itemBaseStatus != null )  {
      resultBaseStatus = itemBaseStatus;
    }

    // --- mixin status ---
    // combination of [ 'error', 'focus', 'selected', 'expanded', 'collapsed', 'hover' ]
    if( term == 'set' )  {
      resultMixinStatusList = optionsMixinStatusList;
      cssClassList = $.array.Join( cssClassList, optionsClassList );
    }
    else if( term == 'add' )  {
      resultMixinStatusList = cssMixinStatusList;
      $.array.Join( resultMixinStatusList, itemMixinStatusList );
      $.array.Join( resultMixinStatusList, optionsMixinStatusList );
      cssClassList = $.array.Join( cssClassList, optionsClassList );
    }
    else if( term == 'remove' )  {
      resultMixinStatusList = cssMixinStatusList;
      $.array.Join( resultMixinStatusList, itemMixinStatusList );
      $.array.Remove( resultMixinStatusList, optionsMixinStatusList );
      cssClassList = $.array.Remove( cssClassList, optionsClassList );
    }

    // selection
    if( $.array.Has( resultMixinStatusList, 'selected' ) )  {
      $.array.Add( this._selected, itemData['id'] );
    }
    else  {
      $.array.Remove( this._selected, itemData['id'] );
    }

    // be sure, that just one of [ 'expanded', 'collapsed' ] is set
    $.array.MutualExclude( resultMixinStatusList, $.jqc.statics.themeMixinGroup );


    //---------------------------------------------------------------
    //  FINISH
    //---------------------------------------------------------------

    itemData['status'] = [ resultBaseStatus ];
    $.array.Join( itemData['status'], resultMixinStatusList );

    // set hover if not in  [ 'readonly', 'disabled', 'no-hover' ]
    if( !$.array.HasOneIn( itemData['status'], $.jqc.statics.themeNoHoverList ) )  {
      itemData['status'].push( 'hover' );
      itemData['disabled'] = false;
    }
    else  {
      itemData['disabled'] = true;
    }

    $.array.Join( cssClassList, itemData['status'] );
    cssClasses = cssClassList.join( ' ' );
    if( cssPreviousClasses != cssClasses )  {
      this.OnStatusChanged( $.array.In( cssClassList, $.jqc.statics.themeClassList ), itemData );
    }

    return cssClasses;
  } 
  
  //---------------------------------------------------------------------------
  //  calculate the status and returns css-class  
  //  if a base status is set => new new base status is set, the old cleared
  //  if an mixin status is set => new mixin status is set, othw. keep existing
  //
  _ManageItemData( itemData, options )
  {
    if( 'text' in options )           itemData.text = $.lang( options['text'] );
    if( 'value' in options )          itemData.value = $.lang( options['value'] );
    if( 'placeholder' in options )    itemData.placeholder = $.lang( options['placeholder'] );
    if( 'tooltip' in options )        itemData.tooltip = $.lang( options['tooltip'] );
    if( 'icon' in options )           itemData.icon = options['icon'];
    if( 'id' in options )             itemData.id = options['id'];
    if( 'index' in options )          itemData.index = options['index'];
    if( 'type' in options )           itemData.type = options['type'];
    if( 'value' in options )          itemData.value = $.lang( options['value'] );
    if( 'path' in options )           itemData.path = options['path'];
    if( 'y' in options )              itemData.y = options['y'];
    if( 'x' in options )              itemData.x = options['x'];

    if( itemData['id'] )  {
      this._items[ itemData['id'] ] = itemData;
    }

    return itemData;
  }  


  //---------------------------------------------------------------------------
  //  VALIDATION FRAMEWORK
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  helper function for validation framework
  //
  ManageValidate( value, eventName )
  {
    // preconditions
    if( !this._isValidationEnabled ) {
      return 0;
    }

    // other controls verification depends on local
    // data. So delegate the change event to the
    // remote control. 
    if( eventName == "change" )  {
      for( let key in this._validationSubscriptionList ) {
        let $subscriber = this._validationSubscriptionList[ key ];
        let ctrlClass = $subscriber.data( "ctrl" );
        if( ctrlClass instanceof JqControl ) {
          ctrlClass.OnValidate( "remote" );
        }
      }
    }

    // if control has a validation, do it
    if( $.isFunction( this.options['validate'] ) ) {
      //console.log( `ManageValidate( ${this.id}, ${ eventName} )` );

      let numPreviousError = ( this.options['error'] != null ) ? 1 : 0;
      let numPreviousWarning = ( this.options['warning'] != null ) ? 1 : 0;
      let isErrorCounterChanged = false;

      // call controls validate lambda
      this._validationControl.value = value;
      this._validationControl.error = null;
      this._validationControl.warning = null;
      this.options['validate']( this._validationControl );

      // error (overwrites a warning)
      if( this._validationControl.error != null && numPreviousError == 0 )  {
        isErrorCounterChanged = true;
        this.$element.trigger( "event:error-changed", { change: 1 } );
        if( numPreviousWarning > 0 )  {
          this.$element.trigger( "event:warning-changed", { change: -1 } );
        }
      }
      else if( this._validationControl.error == null && numPreviousError == 1 ) {
        isErrorCounterChanged = true;
        this.$element.trigger( "event:error-changed", { change: -1 } );
      }

      // warning (error must be 0)
      if( this._validationControl.warning != null && numPreviousWarning == 0 ) {
        isErrorCounterChanged = true;
        this.$element.trigger( "event:warning-changed", { change: 1 } );
      }
      else if( this._validationControl.warning == null && numPreviousWarning == 1 ) {
        isErrorCounterChanged = true;
        this.$element.trigger( "event:warning-changed", { change: -1 } );
      }

      //console.log( `CTRL: ${numError}/${numWarning}` );


      // check if error has changed & draw error
      if( this._validationControl.error != null ) {
        this.options['error'] = $.lang( this._validationControl.error );
        this.options['warning'] = null;
        this.OnDrawError();
      }
      else if( this._validationControl.warning != null )  {
        this.options['warning'] = $.lang( this._validationControl.warning );
        this.options['error'] = null;
        this.OnDrawError();
        //this.$element.trigger( "event:warning-changed", { change: warningChanged } );
      }
      else if( isErrorCounterChanged ) {
        this.options['warning'] = null;
        this.options['error'] = null;
        this.OnDrawError();
      }

      // return 1 if error or warning is set
      return ( typeof this._validationControl.error == "string" || typeof this._validationControl.warning == "string" ) ? 1 : 0;
    }

    // default 0 -> no error
    return 0;
  }


  //---------------------------------------------------------------------------
  //  turn validation On/Off
  //  respect option['validateDepends']
  //
  _ValidationEnable( )
  {
    // switch validation on 
    this._isValidationEnabled = true;

    // do we have dependant controls? 
    if( Array.isArray( this.options['validateDepends'] ) )  {

      //console.log( `_ValidationEnable( ${ this.id } )`, this.options['validateDepends'] );

      // our validation depends to other controls.
      // So set a subscription to the these controls
      for( let validationPartnerId of this.options['validateDepends'] )  {
        let $dependingValidate = $( "#" + validationPartnerId );

        // if control found, set subscription 
        if( $dependingValidate.length > 0 )  {
          let ctrlClass = $dependingValidate.data( "ctrl" );
          if( ctrlClass instanceof JqControl )  {
            ctrlClass._ValidationAddSubscription( this.$element );
          }
        }
      }
    }
  }

  _ValidationDisable( )
  {
    this._isValidationEnabled = false;
    this._validationControl.error = null;
    this._validationControl.warning = null;
    
    // reset error counter while disable validation
    if( this.options['error'] != null )  {
      this.options['error'] = null;
      this.OnDrawError();
      this.$element.trigger( "event:error-changed", { change: -1 } );
    }
    // reset warning counter while disable validation
    if( this.options['warning'] != null ) {
      this.options['warning'] = null;
      this.OnDrawError();
      this.$element.trigger( "event:warning-changed", { change: -1 } );
    }

    // validation depends list
    if( !Array.isArray( this.options['validateDepends'] ) ) {
      return;
    }

    // remove all subscritions
    for( let validationPartnerId of this.options['validateDepends'] ) {
      let $dependingValidate = $( "#" + validationPartnerId );

      // if dependant control found, add to both list
      if( $dependingValidate.length > 0 ) {
        let ctrlClass = $dependingValidate.data( "ctrl" );
        if( ctrlClass instanceof JqControl ) {
          ctrlClass._ValidationRemoveSubscription( this.$element );
        }
      }
    }    
  }

  //---------------------------------------------------------------------------
  //  remote control will call this functions to set/remove
  //  its subscription
  //
  _ValidationAddSubscription( $remoteElement )
  {
    let id = $remoteElement.prop( "id" );
    //console.log( `_ValidationAddSubscription( ${ this.id } ) => ${ id }` );
    this._validationSubscriptionList[ id ] = $remoteElement;
  }

  _ValidationRemoveSubscription( $remoteElement )
  {
    let id = $remoteElement.prop( "id" );
    //console.log( `_ValidationRemoveSubscription( ${ this.id } ) => ${ id }` );
    delete this._validationSubscriptionList[ id ];
  }

  //===========================================================================
  //  DEFAULT INTERFACE
  //===========================================================================

  // ----------------------------------------------------
  //   item: status setter / getter
  //
  item( args )
  {
    // prequistes
    if( args.length < 2 ) {
      console.warn( "JqControl::item - parameter missing!" );
      return;
    }
    if( ( ! ( 'data' in this.options ) ) || ( ! Array.isArray( this.options['data'] ) ) )  {
      console.warn( "JqControl::item - must be used with 'data'!" );
      return;
    }
    
    let item = this.ListGetItem( args[ 1 ] );
    if( item == null )  {
      console.warn( "JqControl::item - id='" + args[ 1 ] + "' not found!" );
      return;
    }

    // function
    let term = args[ 2 ];
    if( term == 'status' )  {
      if( args.length > 3 )  {
        this.ItemSetStatus( item.$item, args[ 3 ] );
        this.OnStatusChanged( item['status'], item['id'] );
        return item['status'];
      }
      return item['status'];
    }
    else if( term == 'status-add' )  {
      if( args.length > 3 ) {
        this.ItemAddStatus( item.$item, args[ 3 ] );
        this.OnStatusChanged( item['status'], item['id'] );
        return item['status'];
      }
      return item['status'];
    }
    else if( term == 'status-remove' ) {
      if( args.length > 3 ) {
        this.ItemRemoveStatus( item.$item, args[ 3 ] );
        this.OnStatusChanged( item['status'], item['id'] );
        return item['status'];
      }
      return item['status'];
    }
    else if( term == 'icon' ) {
      let $itag = item['$item'].find( "i" );
      $itag.attr( "class", args[ 3 ] );
    }
    else {
      console.warn( "JqControl::item - term='" + args[ 2 ] + "' not found!" );
    }

  }

  // ----------------------------------------------------
  //   status: status setter / getter
  //
  status( args )
  {
    // setter
    if( args.length > 2 )  {
      switch( args[ 1 ] )  {
        case 'set':  {
          this.options['status'] = this.ItemSetStatus( this.$element, args[ 2 ] );
        }
        break;
        case 'add':  {
          this.options['status'] = this.ItemAddStatus( this.$element, args[ 2 ] );
        }
        break;
        case 'remove':  {
          this.options['status'] = this.ItemRemoveStatus( this.$element, args[ 2 ] );
        }
        break;
      }
    }
    // getter
    return this.options['status'];
  }  

  // ----------------------------------------------------
  //   option: generic getter and setter
  //
  option( args )  
  {
    // setter
    if( args.length > 2 )  {
      switch( args[ 1 ] )  {
        case 'status': {
          this.options['status'] = this.ItemSetStatus( this.$element, args[ 2 ] );
        }
        break;

        case 'modified': {
          this.options['modified'] = args[ 2 ];
        }
        break;

        default: {
          this.options[ args[ 1 ] ] = args[ 2 ];
          this.OnDraw( );
        }
      }
    }
    // getter
    return this.options[ args[ 1 ] ];
  }  

  // ----------------------------------------------------
  //   destroy
  //
  destroy( args )  
  {
    this._ValidationDisable( );
    this.$element.empty( );
    this.$element.off();
  }

  // ----------------------------------------------------
  //   get class
  //
  viewable( args )  
  {
    return this;
  }

  // ----------------------------------------------------
  //   focus handling
  //
  focus( args )  
  {
    this.$element.focus();
  }

  // ----------------------------------------------------
  //   validation handling
  //
  validate( args )  
  {
    if( ( args.length > 1 && args[ 1 ] )
     || ( args.length <= 1 )
      ) {
      this._ValidationEnable( );
      return this.OnValidate( "user" );
    }
    else {
      this._ValidationDisable( );
    }
    return 0;
  }

  // ----------------------------------------------------
  //   validation query
  //
  isValidationEnabled( args )
  {
    return this._isValidationEnabled;
  }

  // ----------------------------------------------------
  //   validation query
  //
  unittest( args )
  {
    if( $.isFunction( this.options['unittest'] ) )  {
      this.options['unittest']( args[ 1 ], args[ 2 ] );
    }
    return true;
  }

  //===========================================================================
  //  modified/error framework
  //===========================================================================

  OnInitFrameworkEvents( $element, options )
  {
    this.$element.on( "event:modified-reset", ( ev ) => {
      this.ManageModified( 0 );
    } );
    this.$element.on( "event:error-reset", ( ev ) => {
      this.options['error'] = null;
      this._isValidationEnabled = false;
      this.OnDrawError();
    } );
    this.$element.on( "event:warning-reset", ( ev ) => {
      this.options['warning'] = null;
      this._isValidationEnabled = false;
      this.OnDrawError();
    } );
    this.$element.on( "event:validate-reset", ( ev ) => {
      this.validate( [ "validate", false ] );
    } );
  }

  //===========================================================================
  //  OVERRIDABLES
  //===========================================================================

  OnInit( $element, options )
  {
    console.warn( "JqControl - you must override OnInit()" );
  }
  OnDraw( )
  {
    console.warn( "JqControl - you must override OnDraw()" );
  }
  OnDrawError( )
  {
  }  
  OnStatusChanged( statusList, itemId )
  {
  }
  OnValidate( eventName )
  {
    return 0;
  }


}/*
*  Views2 - Web Framework
*  registry build on localStorage
*/

!function( $ ) {

  /* ---------------------------------------------------------------------------
   *   REGISTRY
   * ------------------------------------------------------------------------ */

  // ----------------------------------------------------
  //   functor as entry
  //
  $.registry = function( ) 
  {
    if( typeof $.registry.ns == "undefined"  )  {
      $.registry.lstore = Storages.localStorage;
      $.registry.lstoreKey = window.location.pathname + "registry";
    }

    if( arguments.length == 2 )  {
      return _setByKey( arguments[ 0 ], arguments[ 1 ] );

    }
    else if( arguments.length == 1 )  {
      return _getByKey( arguments[ 0 ] );
    }
  };

  // ----------------------------------------------------
  //   getter
  //
  function _getByKey( key )
  {
    if( $.registry.lstore.isSet( $.registry.lstoreKey, key ) )  {
      return $.registry.lstore.get( $.registry.lstoreKey, key );
    }
    return null;
  }

  // ----------------------------------------------------
  //   setter
  //
  function _setByKey( key, value )
  {
    $.registry.lstore.set( $.registry.lstoreKey, key, value );
    return value;
  }




}( window.jQuery );
/*
 *  Views - Web Framework
 *  metrics
 */

!function( $ ) {

  /* ---------------------------------------------------------------------------
   *   CHOICE - const values
   * ------------------------------------------------------------------------ */

  // ----------------------------------------------------
  //   metrics default function
  //
  $.metrics = function( type )
  {
    switch( type )  {
      
      // ====[ SCROLLBAR WIDTH ]====
      case 'scrollWidth':  {
        
        //--  check scroll bar width by create an element --
        if( $.metrics.scrollWidth > 0 )  {
          $.metrics.scrollWidth = $.metrics._getScrollWidth( );
        }

        //--  at the very beginning (when document isn't shown yet) this doesn't work --
        if( $.metrics.scrollWidth == 0 )  {
          let fontMetrics = $.metrics._getFontMetrics( );
          $.metrics.fontHeight = fontMetrics.cy;
          $.metrics.fontWidth = fontMetrics.cx;
          return $.metrics.fontHeight;
        }

        //--  return from cache --
        return $.metrics.scrollWidth;
      }

      // ====[ FONT HEIGHT ]====
      case 'fontHeight':  {
        if( $.metrics.fontHeight == 0 )  {
          let fontMetrics = $.metrics._getFontMetrics( );
          $.metrics.fontHeight = fontMetrics.cy;
          $.metrics.fontWidth = fontMetrics.cx;
        }
        return $.metrics.fontHeight;
      }

      // ====[ FONT WIDTH ]====
      case 'fontWidth':  {
        if( $.metrics.fontHeight == 0 )  {
          $.metrics._getFontMetrics( );
        }
        return $.metrics.fontWidth;
      }

      // ====[ LINE HEIGHT ]====
      case 'lineHeight': {
        if( $.metrics.fontHeight == 0 ) {
          $.metrics._getFontMetrics( );
        }
        return $.metrics.lineHeight;
      }

      // ====[ BROWSER NAME ]====
      case 'browserName':  {
        if( $.metrics.browserName.length == 0 ) {
          $.metrics._getBrowser();
        }
        return $.metrics.browserName;
      }

      // ====[ BROWSER VERSION ]====
      case 'browserVersion': {
        if( $.metrics.browserVersion == 0 ) {
          $.metrics._getBrowser();
        }
        return $.metrics.browserVersion;
      }

      case 'browserLanguage':  {
        let  browserLang = navigator.language || navigator.userLanguage;
        browserLang = browserLang.substring( 0, 2 );
        return browserLang;
      }

      // ====[ SCREEN X ]====
      case 'screen-x': {
        if( $.metrics['screen-x'] == 0 ) {
          $.metrics._getScreenMetrics();
        }
        return $.metrics['screen-x'];
      }      

      // ====[ SCREEN Y ]====
      case 'screen-y': {
        if( $.metrics['screen-y'] == 0 ) {
          $.metrics._getScreenMetrics();
        }
        return $.metrics['screen-y'];
      }  
      

      // ====[ WINDOW X ]====
      case 'window-x': {
        if( $.metrics['window-x'] == 0 ) {
          $.metrics._getScreenMetrics();
        }
        return $.metrics['window-x'];
      }      

      // ====[ WINDOW Y ]====
      case 'window-y': {
        if( $.metrics['window-y'] == 0 ) {
          $.metrics._getScreenMetrics();
        }
        return $.metrics['window-y'];
      }

      // ====[ OS WINDOW Y ]====
      case 'os-window-y': {
        if( $.metrics['os-metrics'] == null ) {
          $.metrics._getScreenMetrics();
        }
        return $.metrics['os-window-y'];
      }      

      // ====[ OS WINDOW X ]====
      case 'os-window-x': {
        if( $.metrics['os-metrics'] == null ) {
          $.metrics._getScreenMetrics();
        }
        return $.metrics['os-window-x'];
      }      

      // ====[ OS  ]====
      case 'os-metrics':  {
        if( $.metrics['os-metrics'] == null ) {
          $.metrics._getScreenMetrics();
        }
        return $.metrics['os-metrics'];
      }

      
      // ====[ NAVBAR Y ]====
      case 'os-navbar-y':  {
        if( $.metrics['os-metrics'] == null ) {
          $.metrics._getScreenMetrics();
        }
        return $.metrics['os-navbar-y'];
      }

      // ====[ STATUSBAR Y ]====
      case 'os-statusbar-y':  {
        if( $.metrics['os-metrics'] == null ) {
          $.metrics._getScreenMetrics();
        }
        return $.metrics['os-statusbar-y'];
      }      


    }
  };

  // ----------------------------------------------------
  //   private: get size of font
  //
  $.metrics._getScreenMetrics = function( )
  {
    var width  = window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth;
    var height = window.innerHeight|| document.documentElement.clientHeight|| document.body.clientHeight;

    $.metrics['screen-x'] = window.screen.availWidth;
    $.metrics['screen-y'] = window.screen.availHeight;
    $.metrics['window-x'] = width;
    $.metrics['window-y'] = height;

    if( typeof OsInterface != 'undefined' )  {
      $.metrics['os-metrics'] = JSON.parse( OsInterface.Metrics( ) )['metrics'];
      $.metrics['os-navbar-y'] = $.metrics['os-metrics']['navigationBar'] / window.devicePixelRatio;
      $.metrics['os-statusbar-y'] = $.metrics['os-metrics']['statusBar'] / window.devicePixelRatio;
      $.metrics['os-window-x'] = $.metrics['os-metrics']['screenWidthDp'] + 1;
      $.metrics['os-window-y'] = $.metrics['os-metrics']['screenHeightDp'];
        if( ( $.metrics['os-statusbar-y'] + $.metrics['os-metrics']['screenHeightDp'] ) >= $.metrics['screen-y'] )  {
        $.metrics['os-navbar-y'] = 0;
      }

    }
  };

  // ----------------------------------------------------
  //   private: get size of font
  //
  $.metrics._getFontMetrics = function( )
  {
    let h = $( "body" ).css( "font-size" );
    h = Number( h.replace( /[^-\d\.]/g, '' ) );
    let w = Math.floor( h / 2 );
    $.metrics.fontHeight = h;
    $.metrics.fontWidth = w;
    $.metrics.lineHeight = Math.floor( h * 1.2 );
    return { cx: w, cy: h };
  };

  // ----------------------------------------------------
  //   private: get size of scroll bar
  //
  $.metrics._getScrollWidth = function( )
  {
    const inner = document.createElement( 'p' );
    inner.style.width = '100%';
    inner.style.height = '200px';

    const outer = document.createElement( 'div' );
    outer.style.position = 'absolute';
    outer.style.top = '0px';
    outer.style.left = '0px';
    outer.style.visibility = 'hidden';
    outer.style.width = '200px';
    outer.style.height = '150px';
    outer.style.overflow = 'hidden';
    outer.appendChild( inner );

    document.body.appendChild( outer );
    const w1 = inner.offsetWidth;
    outer.style.overflow = 'scroll';
    var w2 = inner.offsetWidth;
    if( w1 === w2 ) w2 = outer.clientWidth;

    document.body.removeChild( outer );

    return ( w1 - w2 ) - 1;
  };

  // ----------------------------------------------------
  //   private: get size of scroll bar
  //
  $.metrics._getBrowser = function()
  {
    var ua = navigator.userAgent, tem, M = ua.match( /(opera|chrome|safari|firefox|msie|trident(?=\/))\/?\s*(\d+)/i ) || [];
    if( /trident/i.test( M[ 1 ] ) ) {
      tem = /\brv[ :]+(\d+)/g.exec( ua ) || [ ];
      $.metrics.browserName = 'IE';
      $.metrics.browserVersion = Number.parseInt( tem[ 1 ] );
    }
    else if( M[ 1 ] === 'Chrome' ) {
      tem = ua.match( /\bOPR|Edge\/(\d+)/ );
      if( tem != null ) { 
        $.metrics.browserName = 'Opera';
        $.metrics.browserVersion = Number.parseInt( tem[ 1 ] );
      }
    }
    M = M[ 2 ] ? [ M[ 1 ], M[ 2 ] ] : [ navigator.appName, navigator.appVersion, '-?' ];
    if( ( tem = ua.match( /version\/(\d+)/i ) ) != null ) { M.splice( 1, 1, tem[ 1 ] ); }
    $.metrics.browserName = M[ 0 ];
    $.metrics.browserVersion = Number.parseInt( M[ 1 ] );
    
    var userAgent = window.navigator.userAgent.toLowerCase();
    var safari = /safari/.test( userAgent );
    var ios = /iphone|ipod|ipad/.test( userAgent );
    var android = /android/.test( userAgent );

    if( ios ) {
      $.metrics.ios = true;
      if( !safari ) {
        $.metrics.webview = true;
      }
    } 
    else if( android )  {
      $.metrics.android = true;
      if( typeof OsInterface !== "undefined" )  {
        $.metrics.webview = true;
      }
    }

  };

  // ----------------------------------------------------
  //   private data
  //
  $.metrics.scrollWidth = 0;
  $.metrics.fontHeight  = 0;
  $.metrics.fontWidth   = 0;
  $.metrics.lineHeight  = 0;
  $.metrics.browserName = "";
  $.metrics.browserVersion = 0;
  $.metrics.android = false;
  $.metrics.ios = false;
  $.metrics.webview = false;
  $.metrics['screen-x'] = 0;
  $.metrics['screen-y'] = 0;
  $.metrics['window-x'] = 0;
  $.metrics['window-y'] = 0;
  $.metrics['os-window-x'] = 0;
  $.metrics['os-window-y'] = 0;
  $.metrics['os-navbar-y'] = 0;
  $.metrics['os-statusbar-y'] = 0;
  $.metrics['os-metrics'] = null;

}( window.jQuery );

//=============================================================================
//
//  CSS Animation
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          23.01.2021
//=============================================================================

class Animation
{
  constructor( element, type )
  {
    // element handling
    if( element instanceof jQuery ) {
      this.element = element[ 0 ];
    }
    else if( element instanceof Element ) {
      this.element = element;
    }
    else {
      this.element = document.getElementById( element );
    }

    // start animation and remove after 5s
    if( this.element )  {
      this.element.classList.add( type );
      setTimeout( () =>  {
        this.element.classList.remove( type );
      }, 5000 );
    }

  }
}
//=============================================================================
//
//  File: broadcast.js
//  (c) 2021 by CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          02.02.2021
//
//  Description:
//          class to send a broadcast to all decendents
//
//=============================================================================


!function( $ ) {

// ----------------------------------------------------
//   plugin main function
//
$.fn.broadcast = function( option )  
{
  // preconditions
  if( arguments.length < 1 ) {
    return this;
  }

  // should we us a custom event for that?
  let isEvent = ( arguments[ 0 ].indexOf( "event:" ) == 0 );

  // if the jquery object is allready a list, avoid find()
  if( this.length > 1 ) {
    for( let i = 0; i < this.length; i++ ) {
      
      // send custom event
      if( isEvent )  {
        this[ i ].trigger( arguments[ 0 ], arguments[ 1 ] );
      }
      // try to call objects function
      else {
        let ctrl = $( this[ i ] ).data( "ctrl" );
        if( ( ctrl instanceof JqControl )
         && ( $.isFunction( ctrl[ arguments[ 0 ] ] ) )
          ) {
          ctrl[ arguments[ 0 ] ]( arguments );
        }
        else {
          let viewable = $( this[i] ).data( "viewable" );  //var data = $( this ).data( "viewable" ),
          if( viewable &&  $.isFunction( viewable[ arguments[ 0 ] ] ) ) {
            viewable[ arguments[ 0 ] ]( arguments );
          }
        }
      }
    }
  }
  // use .find() to get the list of controls
  else  {

    // traverse dom tree
    let $found = this.find( "[data-broadcast='true']" );

    // send custom event
    if( isEvent ) {
      $found.trigger( arguments[ 0 ], arguments[ 1 ] );
    }
    // try to call objects function
    else  {
      for( let i = 0; i < $found.length; i++ ) {
        let ctrl = $( $found[ i ] ).data( "ctrl" );
        if( ( ctrl instanceof JqControl )
         && ( $.isFunction( ctrl[ arguments[ 0 ] ] ) )
          ) {
          ctrl[ arguments[ 0 ] ]( arguments );
        }
        else {
          let viewable = $( $found[ i ] ).data( "viewable" );  //var data = $( this ).data( "viewable" ),
          if( viewable && $.isFunction( viewable[ arguments[ 0 ] ] ) ) {
            viewable[ arguments[ 0 ] ]( arguments );
          }
        }
      }
    }
  }
  


  // return the object for chaining    
  return this;   
};

  
}( window.jQuery );
 ﻿/**
 * Javascript version of https://gist.github.com/jbroadway/2836900
 *
 * Slimdown - A very basic regex-based Markdown parser. Supports the
 * following elements (and can be extended via Slimdown::add_rule()):
 *
 * - Headers
 * - Links
 * - Bold
 * - Emphasis
 * - Deletions
 * - Quotes
 * - Inline code
 * - Blockquotes
 * - Ordered/unordered lists
 * - Horizontal rules
 *
 * Author: Johnny Broadway <johnny@johnnybroadway.com>
 * Website: https://gist.github.com/jbroadway/2836900
 * License: MIT
 */
class Slimdown
{

  constructor()
  {
    // Rules
    this.rules = [
      { regex: /(#+)(.*)/g, replacement: header },                                         // headers
      { regex: /!\[([^\[]+)\]\(([^\)]+)\)/g, replacement: '<i class=\'$2\' alt=\'$1\'></i>' }, // image
      { regex: /\[([^\[]+)\]\(([^\)]+)\)/g, replacement: '<span class=\'link hover\' data-id=\'$2\'>$1</span>' },        // hyperlink
      { regex: /(\*\*)(.*?)\1/g, replacement: '<strong>$2</strong>' },                  // bold
      { regex: /(\*)(.*?)\1/g, replacement: '<i>$2</i>' },                             // emphasis
      { regex: /\~\~(.*?)\~\~/g, replacement: '<del>$1</del>' },                           // del
      //{ regex: /\:\"(.*?)\"\:/g, replacement: '<q>$1</q>' },                               // quote
      //{ regex: /`(.*?)`/g, replacement: '<code>$1</code>' },                               // inline code
      //{ regex: /\n\*(.*)/g, replacement: ulList },                                         // ul lists
      //{ regex: /\n[0-9]+\.(.*)/g, replacement: olList },                                   // ol lists
      //{ regex: /\n(&gt;|\>)(.*)/g, replacement: blockquote },                              // blockquotes
      //{ regex: /\n-{5,}/g, replacement: '\n<hr />' },                                      // horizontal rule
      //{ regex: /\n([^\n]+)\n/g, replacement: para },                                       // add paragraphs
      //{ regex: /<\/ul>\s?<ul>/g, replacement: '' },                                        // fix extra ul
      //{ regex: /<\/ol>\s?<ol>/g, replacement: '' },                                        // fix extra ol
      //{ regex: /<\/blockquote><blockquote>/g, replacement: '\n' }                          // fix extra blockquote
    ];


    ////////////

    /*function para( text, line )
    {
      const trimmed = line.trim();
      if( /^<\/?(ul|ol|li|h|p|bl)/i.test( trimmed ) ) {
        return `\n${ line }\n`;
      }
      return `\n<p>${ trimmed }</p>\n`;
    }

    function ulList( text, item )
    {
      return `\n<ul>\n\t<li>${ item.trim() }</li>\n</ul>`;
    }

    function olList( text, item )
    {
      return `\n<ol>\n\t<li>${ item.trim() }</li>\n</ol>`;
    }

    function blockquote( text, tmp, item )
    {
      return `\n<blockquote>${ item.trim() }</blockquote>`;
    }*/

    function header( text, chars, content )
    {
      const level = chars.length;
      return `<h${ level }>${ content.trim() }</h${ level }>`;
    }
  }

  // Add a rule.
  AddRule( regex, replacement )
  {
    regex.global = true;
    regex.multiline = false;
    this.rules.push( {
      regex,
      replacement,
    } );
  }

  // Render some Markdown into HTML.
  Render( text )
  {
    text = `\n${ text }\n`;
    this.rules.forEach( ( rule ) => {
      text = text.replace( rule.regex, rule.replacement );
    } );

    return text.trim();
  }
}

//=============================================================================
//
//  VIEWS2 - Treeset
//  
//  (c) 2019, CYTEL AG
//      Frank Jaekel
//
//  Record-Format:
//  {
//    <id>: ->  uuid  (literal in options['id'])
//    text: ->  text
//    icon: ->  text
//    children[]: -> array of records
//    _preview:   -> { id: uuid, icon: icon, text: text, description: description };
//    _parent:    -> uuid
//  }
//  
//  data[]  -> array of records (tree-data)
//  keys{ uuid }  -> index of records
//  
//=============================================================================


class Treeset
{
  //-----------------------------------------------------------------
  //  CONSTRUCTOR
  //-----------------------------------------------------------------

  constructor( options, initData )
  {
    this._data = [];
    this._keys = {};
    this._options = {};
    this._dataViews = {};
    this._dataViewsActive = {};
    this._dataViewMaxUid = 0;
    if( options )  {
      this.Initialize( options, initData );
    }

    this._options.sorter = null;

    var self = this;
    setInterval( function()  {
      self.GarbageCollectAttachedViews( );
    }, 60000 );
  }

  //-----------------------------------------------------------------
  //  ADD NEW VIEW
  //-----------------------------------------------------------------

  AttachView( treeview )
  {
    if( !( treeview instanceof Treeview ) )  {
      console.error( "Treeset.Attach() must be used with Treeview" );
      return -1;
    }
    if( !$.isFunction( treeview._OnDataEvent ) )  {
      console.error( "Treeset.Attach() class Treeview must have function '_OnDataEvent'" );
      return -1;
    }

    // link sorter 
    if( $.isFunction( this._options.sorter ) )  {
      treeview.options.sorter = this._options.sorter;
    }

    var uid = ++( this._dataViewMaxUid );
    this._dataViews[ uid ] = treeview;
    this._dataViewsActive[ uid ] = true;

    return uid;
  }

  //-----------------------------------------------------------------
  //  RELEASE VIEW
  //-----------------------------------------------------------------

  ReleaseView( uid )
  {
    if( uid in this._dataViews )  {
      delete this._dataViews[ uid ];
      delete this._dataViewsActive[ uid ];
      return true;
    }
    else if( uid === null )  {
      for( uid in this._dataViews )  {
        delete this._dataViews[ uid ];
        delete this._dataViewsActive[ uid ];  
      }
      return true;
    }
    return false;
  }


  //-----------------------------------------------------------------
  //  clean 'disapeared' views
  //-----------------------------------------------------------------

  GarbageCollectAttachedViews()
  {
    for( var key in this._dataViews ) {
      if( $( '#' + this._dataViews[ key ].domElementId ).length == 0 ) {
        delete this._dataViews[ key ];
      }
    }
  }


  //-----------------------------------------------------------------
  //  STOP EVENTS TO VIEW
  //-----------------------------------------------------------------

  Silent( uid, isSilent )
  {
    this._dataViewsActive[ uid ] = isSilent;
    if( !isSilent ) {
      this._dataViews[ uid ]._OnDataEvent( 'create' );
    }
  }



  //-----------------------------------------------------------------
  //  HELPER
  //-----------------------------------------------------------------

  _FireEvent( event, param, param2, param3 )
  {
    for( var key in this._dataViews ) {
      if( this._dataViewsActive[ key ] ) {
        this._dataViews[ key ]._OnDataEvent( event, param, param2, param3 );
      }
    }
  }
  
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  Initialize( options, initData )
  {
    // init
    var defaultOptions = {
      id: 'id', 
      children: 'children',
      preview: null   
    };

    this._options =  $.extend( true, [], defaultOptions, options );

    // data
    if( Array.isArray( initData ) )  {
      this.SetData( initData );
    }

  }


  //-----------------------------------------------------------------
  //  SET DATA
  //-----------------------------------------------------------------

  SetData( initData )
  {
    // empty
    this._data = [];
    this._keys = {};

    this._RecurseSetData( null, initData );

    // fire event 
    this._FireEvent( 'create' );
  }

  _RecurseSetData( parentId, initData )
  {
    // iterate this branch (Array)
    if( Array.isArray( initData ) )  {
      for( var i = 0; i < initData.length; i++ ) {

        // prepare
        let record = initData[ i ];
        let newRecord = this._AddRecord( record, parentId );
        if( ! newRecord )  {
          continue;
        }
        
        let id = record[ this._options['id'] ];

        // children? start child branch.
        if( 'children' in record )  {
          newRecord['children'] = [ ];
          this._RecurseSetData( id, record['children'] );
        }

      }   
    }
    // iterate this branch (Object)
    else if( $.isPlainObject( initData ) )  {

      for( let id in initData )  {
        // prepare
        let record = initData[ id ];
        record[ this._options['id'] ] = id;
        
        let newRecord = this._AddRecord( record, parentId );
        if( !newRecord ) {
          continue;
        }

        // children? start child branch.
        if( 'children' in record ) {
          newRecord['children'] = [];
          this._RecurseSetData( id, record['children'] );
        }

      }

    }

  }

  _AddRecord( record, parentId )
  {
    let childList = null;
    let id = record[ this._options['id'] ];

    // get parent record
    if( parentId in this._keys )  {
      if( !( 'children' in this._keys[ parentId ]) )  {
        this._keys[ parentId ]['children'] = [];
      }
      childList = this._keys[ parentId ]['children'];
    }
    else  {
      childList = this._data;  // root
    }

    // key must be unique!
    if( id in this._keys ) {
      console.warn( "id '" + this._options['id'] + "' must be unique!", record );
      return null;
    }

    // clone record and do language without children (done in recursion)
    var newRecord = $.clone( record, ['children'] );
    newRecord['_parent'] = parentId;
    childList.push( newRecord );

    // build preview
    if( $.isFunction( this.OnCreatePreview ) ) {
      newRecord['_preview'] = this.OnCreatePreview( newRecord );
    }
    else {
      var icon = ( 'icon' in record ) ? record['icon'] : null;
      var text = ( 'text' in record ) ? $.lang( record['text'] ) : null;
      var description = ( 'description' in record ) ? record['description'] : null;
      newRecord['_preview'] = { id: id, icon: icon, text: text, description: description };
    }

    // store record under it's id
    this._keys[ id ] = newRecord;
    return newRecord;
  }


  //-----------------------------------------------------------------
  //  RECORDS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Add a record
  //
  Add( parentId, record )
  {
    //console.log( "Treeset.Add( -> " + parentId + " )" );
    // build & add record
    let newRecord = this._AddRecord( record, parentId );
    if( newRecord == null ) {
      console.warn( "Treeset._AddRecord() failed" );
      return;
    }

    // fire event 
    let id = newRecord[ this._options['id'] ];
    this._FireEvent( 'add', id, parentId, newRecord );    
  }

  AddBranch( parentid, branch )
  {
  }

  //-----------------------------------------------------------------
  //  delete a record
  //
  Delete( id )
  {
    // prequisites
    if( !( id in this._keys ) )  {
      console.warn( "Tree.Delete: id='" + id + "' not found!" );
      return false;
    }

    // get the record to be deleted
    let record = this._keys[ id ];

    // get parent 'children' array
    let parentChildren = [];
    if( record['_parent'] == null )  {
      parentChildren = this._data;
    }
    else  {
      let parentRecord = this._keys[ record['_parent'] ];
      parentChildren = parentRecord['children'];
    }

    // find index of record in parent children array
    let recordIndex = parentChildren.findIndex( ( record ) => record[ this._options['id'] ] == id );

    // delete record from array and keys
    parentChildren.splice( recordIndex, 1 );
    delete this._keys[ id ];

    // event to all views
    this._FireEvent( 'delete', id, record );

  }

  //-----------------------------------------------------------------
  //  delete a record and its children	
  //
  DeleteBranch( id )
  {
    this._FireEvent( 'delete-branch', id );
  }

  //-----------------------------------------------------------------
  //  update a record
  //
  Update( id, udpateRecord )
  {
    // prequisites
    if( !( id in this._keys ) ) {
      console.warn( "Tree.Update: id='" + id + "', not found!" );
      return false;
    }
    if( 'children' in udpateRecord )  {
      console.warn( "Tree.Update: id='" + id + "', children will not be copied!" );
    }

    // update the record, but don't change parent + children
    let record = this._keys[ id ];
    let newRecord = $.clone( udpateRecord, ['children'] );
    delete newRecord['_parent'];
    
    // Reference to record: $.extend() must be used to keep 
    // the record-object itself and just change the content of it,
    // because the record is stored in this._keys AND this._data
    $.extend( record, newRecord );

    // build preview
    if( $.isFunction( this.OnCreatePreview ) ) {
      record['_preview'] = this.OnCreatePreview( newRecord );
    }
    else {
      var icon = ( 'icon' in record ) ? record['icon'] : null;
      var text = ( 'text' in record ) ? $.lang( record['text'] ) : null;
      var description = ( 'description' in record ) ? record['description'] : null;
      record['_preview'] = { id: id, icon: icon, text: text, description: description };
    }    

    this._FireEvent( 'update', id );
  }

  //-----------------------------------------------------------------
  //  move a record
  //
  Move( id, toParentId )
  {
    /*console.log( "MOVE" );
    console.log( this._keys );
    console.log( this._data );*/

    // prequisites
    if( !( id in this._keys ) ) {
      console.warn( "Tree.Move: id='" + id + "', not found!" );
      return false;
    }

    // get from-parent children array
    let record = this._keys[ id ];
    let fromParentId = record['_parent'];
    let fromParentChildren = this._data;
    if( fromParentId != null )  {
      fromParentChildren = this._keys[ fromParentId ]['children'];
    }

    // get to-parent children array
    let toParentChildren = this._data;
    if( toParentId != null ) {
      let toParentRecord = this._keys[ toParentId ];
      if( !( 'children' in toParentRecord ) ) {
        toParentRecord['children'] = [];
      }
      toParentChildren = toParentRecord['children'];
    }

    // move record
    let fromRecordIndex = fromParentChildren.findIndex( ( record ) => record[ this._options['id'] ] == id );
    toParentChildren.push( fromParentChildren[ fromRecordIndex ] );
    fromParentChildren.splice( fromRecordIndex, 1 );
    record['_parent'] = toParentId;

    /*console.log( "MOVE" );
    console.log( this._keys );
    console.log( this._data );*/
    
    // send event to upper layer
    this._FireEvent( 'move', id, toParentId, fromParentId );

  }

  //-----------------------------------------------------------------
  //  get a record by id
  //
  Get( id )
  {
    return this._keys[ id ];
  }

  //-----------------------------------------------------------------
  //  get a parent id from id
  //
  GetParent( id )
  {
    return this._keys[ id ]['_parent'];
  }  

  //-----------------------------------------------------------------
  //  children?
  //
  HasChildren( id )
  {
    return ( id in this._keys 
          && 'children' in this._keys[ id ] 
          && this._keys[ id ]['children'].length > 0 
           );
  }

  //-----------------------------------------------------------------
  //  children?
  //
  Has( id )
  {
    return ( id in this._keys );
  }  

  //-----------------------------------------------------------------
  //  get all children
  //
  GetBranch( id )
  {
    var result = [];
    var data = [];

    // if root
    if( id == null || id == "" || id == "/" )  {
      data = this._data;
    }
    else  {
      data = this._keys[ id ]['children'];
      if( ! Array.isArray( data ) )  {
        data = [];
      }
    }

    for( let i = 0; i < data.length; i++ ) {
      result.push( data[ i ][ this._options['id'] ] );
    }

    return result;
  }

  //-----------------------------------------------------------------
  //  size of a branch
  //
  Size( )
  {
    return Object.keys( this._keys ).length;
  }

  //-----------------------------------------------------------------
  //  get list of keys
  //
  Keys()
  {
    return Object.keys( this._keys );
  }

  //-----------------------------------------------------------------
  //  size of a branch
  //
  Each( branchId, fnEach )
  {
    var data = [];

    // if root 
    if( branchId == null || branchId == "" || branchId == "/" ) {
      data = this._data;
    }
    else {
      data = this._keys[ branchId ]['children'];
      if( !Array.isArray( data ) ) {
        data = [];
      }
    }

    // iterate
    for( let i = 0; i < data.length; i++ ) {
      fnEach( data[ i ][ this._options['id'] ], data[ i ] );
    }

  }
  
}

//=============================================================================
//
//  VIEWS2 - Treeview
//  
//  (c) 2020, CYTEL AG
//      Frank Jaekel
//
//  itemData :=  {
//    $item:  -> JjQuery-object
//    index:  -> index in list,
//    id:     -> doc-id,
//    data:  {      -> user-data
//      css:        -> css-classes set (not status)
//      status: []  -> css-status list
//    }
//  }
//=============================================================================

class Treeview 
{
  //-----------------------------------------------------------------
  //  CONSTRUCTOR / 'DESTRUCTOR'
  //-----------------------------------------------------------------

  constructor( treeset, options )
  {
    if( !( treeset instanceof Treeset ) )  {
      console.error( "A dataview must be initialized with a Dataset" );
      return;
    }

    var defaultOptions = {
      sortKey: "text",
      sortOrder: '+',
      sortOnUpdate: false,
      filter: null,
      selection: 'single'
    };

    this.length = 0;
    this.treeset = treeset;
    this.datasetId = -1;
    this.options = $.extend( true, [], defaultOptions, options );
    this.events = {};

    // data
    this.treeList = [];       // tree of index
    this.itemDataList = {};   // object if id's 
    this.selected = [];       // array if id's
    this.domElement = null;

    // search
    this.search = null;
    this.searchRx = null;

    treeset.AttachView( this );
    this._Rebuild( );
  }

  AttachDomElement( $element )
  {
    if( $element instanceof jQuery )  {
      this.domElementId = $element[ 0 ].id;
    }
    else  {
      console.warn( "Dataview.AttachDomElement: $element not a jQuery-Object!, potential memory leak!" );
    }
  }

  Options( options )
  {
    this.options = $.extend( true, [], this.options, options );
  }

  //-----------------------------------------------------------------
  //  EVENT INTERFACE
  //-----------------------------------------------------------------

  On( event, fnFunction )
  {
    this.events[ event ] = fnFunction;
  }

  Off( event )
  {
    delete this.events[ event ];
  }

  Silent( isSilent )
  {
    this.treeset.Silent( this.datasetId, isSilent );
    if( !isSilent )  {
      this._FireEvent( 'rebuild' );
    }
  }

  Update( )
  {
    this._Rebuild( );
    this._FireEvent( 'rebuild' );
  }

  //-----------------------------------------------------------------
  //  DATA Event-Function
  //-----------------------------------------------------------------

  _OnDataEvent( event, param, param2, param3 )
  {
    //console.log( "Treeset._OnDataEvent(", event, ")" );

    if( event == 'create' )  {
      this._Rebuild( );
      this._FireEvent( 'rebuild', param );
    }
    else if( event == 'delete' )  {
      this._DeleteRecord( param );
    }
    else if( event == 'add' )  {
      //id, parentId, newRecord
      this._AddRecord( param, param2, param3 );
    }
    else if( event == 'update' )  {
      this._UpdateRecord( param );      
    }
    else if( event == 'move' ) {
      this._MoveRecord( param, param2, param3 );
    }

  }

  //-----------------------------------------------------------------
  //  Rebuild the data view
  //-----------------------------------------------------------------

  _Rebuild( )
  {
    // empty all
    this.treeList = [ ];
    this.itemDataList = {};
    this.selected = [];   

    this._RebuildRecurse( null, this.treeList  );
    this.length = this.treeList.length;

    //console.log( "View:", this.treeList );
    //console.log( "View:", this.itemDataList );

    //if( this.options.sortOrder != 'none' )  {
    //  this._SortTreeview( );
    //}
  }

  _RebuildRecurse( branchId, childList )
  {
    var fnFilter = $.isFunction( this.options.filter ) ? this.options.filter : this.DefaultFilter;
    var fnSearch = $.isFunction( this.options.search ) ? this.options.search : this.DefaultSearch;

    this.treeset.Each( branchId, ( id, record ) =>  {
      if( fnFilter( id, record, this.options, this )
       && fnSearch( id, record, this.options, this )
        ) {
        
        // push current record
        var newRecord = { 'id': id, _preview: record['_preview'], _parent: record['_parent'], 'children': [] };
        childList.push( newRecord );
        this.itemDataList[ id ] = {
          '$item': null,        // dom-element attached to this record
          'id': id,             // unique id
          'record': newRecord, 
          'data': {            // ui-status data
            'status': "",   //-> Init in AttachItem
            'css': ""       //->      "
          }
        };

        if( 'status' in this.itemDataList[ id ]['record']['_preview'] ) {
          this.itemDataList[ id ]['data']['status'] = this.itemDataList[ id ]['record']['_preview']['status'];
        }
        
        // recurse
        if( ( 'children' in record ) && ( record['children'].length > 0 ) )  {
          this._RebuildRecurse( id, newRecord['children'] );
        }
      }
    } ); 

    // sort it
    this._SortBranch( branchId, childList );
  }

  _AddRecord( id, parentId, record )
  {
    //console.log( "Treeset._AddRecord(", id, ")" );

    var fnFilter = $.isFunction( this.options.filter ) ? this.options.filter : this.DefaultFilter;
    var fnSearch = $.isFunction( this.options.search ) ? this.options.search : this.DefaultSearch;

    // add record if needed
    if( fnFilter( id, record, this.options, this )
     && fnSearch( id, record, this.options, this )
      ) {

      // build new record
      var newRecord = { 'id': id, _preview: record['_preview'], _parent: record['_parent'], 'children': [] };
      this.itemDataList[ id ] = {
        '$item': null,        // dom-element attached to this record
        'id': id,             // unique id
        'record': newRecord,
        'data': {            // ui-status data
          'status': "",   //-> Init in AttachItem
          'css': ""       //->      "
        }
      };

      // manage status (if needed)
      if( 'status' in this.itemDataList[ id ]['record']['_preview'] ) {
        this.itemDataList[ id ]['data']['status'] = this.itemDataList[id]['record']['_preview']['status'];
      }

      // push to parent
      let parentItem = this.itemDataList[ parentId ];
      if( parentItem && 'record' in parentItem && 'children' in parentItem['record'] ) {
        parentItem['record']['children'].push( newRecord );
      }

      // fire add event to ui
      this._FireEvent( "add", id, parentId );

      // sort if needed
      if( this.options['sortOrder'] && this.options['sortOrder'] != 'none' && this.options['sortKey'] != null ) {

        let itemData = this.itemDataList[id];
        let parentId = itemData['record']['_parent'];
        let parentChildren = this.treeList;   // root as default
        if( parentId != null ) {
          let parentItemData = this.itemDataList[parentId];
          parentChildren = parentItemData['record']['children'];
        }

        let sortList = this._SortBranch( parentId, parentChildren );
        if( Array.isArray( sortList ) && sortList.length > 1 ) {
          this._FireEvent( "sort", parentId, sortList );
        }

      }

    }
    else  {
      console.warn( "Treeview._AddRecord(), filtered!" );
    }

  }

  _UpdateRecord( id )
  {
    // get data 
    let itemData = this.itemDataList[ id ];
    let record = this.treeset.Get( id );

    // copy data from changed record
    itemData['record']['_preview'] = record['_preview'];
    if( 'status' in this.itemDataList[ id ]['record']['_preview'] ) {
      this.ItemSetStatus( id, this.itemDataList[ id ]['record']['_preview']['status'] );
    }

    // fire event to upper layer
    this._FireEvent( "update", id );

    // sort if needed
    if( this.options['sortOrder'] && this.options['sortOrder'] != 'none' && this.options['sortKey'] != null )  {

      let parentId = itemData['record']['_parent'];
      let parentChildren = this.treeList;   // root as default
      if( parentId != null ) {
        let parentItemData = this.itemDataList[parentId];
        parentChildren = parentItemData['record']['children'];
      }

      let sortList = this._SortBranch( parentId, parentChildren );
      if( Array.isArray( sortList ) && sortList.length > 1 ) {
        this._FireEvent( "sort", parentId, sortList );
      }

    }


  }

  _DeleteRecord( id )
  {
    // first delete in top layer: Control must be able 
    // to get the item data.
    this._FireEvent( "delete", id );

    // manage parent
    let itemData = this.itemDataList[ id ];
    let parentId = itemData['record']['_parent'];
    let parentChildren = this.treeList;   // root as default
    if( parentId != null ) {
      let parentItemData = this.itemDataList[ parentId ];
      parentChildren = parentItemData['record']['children'];
    }

    // find index position if <id> in parent children & remove
    let recordIndex = parentChildren.findIndex( ( record ) => record['id'] == id );
    parentChildren.splice( recordIndex, 1 );
    delete this.itemDataList[id];
  }


  _MoveRecord( id, idToParent, idFromParent )
  {
    // get existing children (from parent)
    let parentFromChildren = this.treeList;   // root as default
    if( idFromParent != null ) {
      parentFromChildren = this.itemDataList[ idFromParent ]['record']['children'];
    }
    let recordFromIndex = parentFromChildren.findIndex( ( record ) => record['id'] == id );

    // get existing children (to parent)
    let parentToChildren = this.treeList;   // root as default
    if( idToParent != null ) {
      let itemToData = this.itemDataList[ idToParent ];
      if( !( 'children' in itemToData['record'] ) )  {
        itemToData['record']['children'] = [];
      }
      parentToChildren = itemToData['record']['children'];
    }

    // move
    parentToChildren.push( parentFromChildren[ recordFromIndex ] );
    parentFromChildren.splice( recordFromIndex, 1 );
    let itemData = this.itemDataList[ id ];
    itemData['record']['_parent'] = idToParent;

    //console.log( this.itemDataList );

    // fire event to upper layer
    this._FireEvent( "move", id, idToParent, idFromParent );
  }

  //-----------------------------------------------------------------
  //  INTERFACE
  //-----------------------------------------------------------------

  Size()
  {
    return Object.keys( this.itemDataList ).length;
  }

  //-----------------------------------------------------------------
  //  SEARCH data
  //
  Search( value )
  {
    if( value == "" )  {
      this.search = null;
    }
    else {
      this.search = value;
      this.searchRx = new RegExp( value, "gi" );
    }
    this._Rebuild( );
    this._FireEvent( 'rebuild' );
  }    

  //-----------------------------------------------------------------
  //  SORT data
  //
  Sort( sortKey, sortOrder = '+' )
  {
    this.options['sortOrder'] = sortOrder;
    this.options['sortKey'] = sortKey;

    this._Rebuild();
    this._FireEvent( 'rebuild' );
  }


  //-----------------------------------------------------------------
  //  SELECT items
  //
  Select( selected )
  {
    this.ItemClearSelections( );
    if( Array.isArray( selected ) )  {
      for( var i = 0; i < selected.length; i++ )  {
        this.ItemSelect( selected[ i ] );
      }
    }
  }  

  //-----------------------------------------------------------------
  //  GET data
  //
  Get( id )
  {
    return this.treeset.Get( id );
  }  

  //-----------------------------------------------------------------
  //  get a parent id from id
  //
  GetParent( id )
  {
    return this.treeset.GetParent( id );
  }  


  //-----------------------------------------------------------------
  //  GET data
  //
  Has( id )
  {
    return this.treeset.Has( id );
  }  

  //-----------------------------------------------------------------
  //  GET data
  //
  HasChildren( id )
  {
    return this.treeset.HasChildren( id );
  }  


  //-----------------------------------------------------------------
  //  GET meta information
  //
  GetItem( id )
  {
    return this.itemDataList[ id ];
  }    

  //-----------------------------------------------------------------
  //  GET data
  //
  GetBranch( id )
  {
    var result = [];
    var data = [];

    // if root
    if( id == null || id == "" || id == "/" ) {
      data = this.treeList;
    }
    else {
      data = this.itemDataList[ id ]['record']['children'];
      if( !Array.isArray( data ) ) {
        data = [];
      }
    }

    for( let i = 0; i < data.length; i++ ) {
      result.push( data[ i ]['id'] );
    }

    return result;
  }  

  //-----------------------------------------------------------------
  //  get all selected items
  //
  GetSelected()
  {
    return $.clone( this.selected );
  }


  //-----------------------------------------------------------------
  //  GET Preview data
  //
  Preview( id )
  {
    // prequistes
    var record = this.treeset.Get( id );

    // return data
    if( '_preview' in record ) {
      return record['_preview'];
    }
    else {
      return id;
    }
  }  


  //-----------------------------------------------------------------
  //  GET underlaying treeset
  //
  Treeset(  )
  {
    return this.treeset;
  }

  //-----------------------------------------------------------------
  //  Attach $item to dataview
  //
  AttachItem( id, $item, $children, data )
  {
    // re-attach
    if( this.itemDataList[ id ]['$item'] != null )  {
      this.itemDataList[ id ]['$item'] = $item;
      this.itemDataList[ id ]['$children'] = $children;
      this.ItemSetStatus( id, this.itemDataList[ id ]['data']['status'] );
    }
    // attach
    else  {

      this.itemDataList[ id ]['$item'] = $item;
      this.itemDataList[ id ]['$children'] = $children;
      this.itemDataList[ id ]['data'] = $.extend( true, [], this.itemDataList[ id ]['data'], data );

      // get css classes from item and remove status classes
      var cssClassList = $item.attr( 'class' ).split( /\s+/ );
      var cssList = $.array.NotIn( cssClassList, $.jqc.statics.themeClassList );
      this.itemDataList[ id ]['data']['css'] = cssList.join( ' ' );
      if( this.itemDataList[ id ]['data']['status'] )  {
        this.ItemSetStatus( id, this.itemDataList[ id ]['data']['status'] );
      }
    }

    $item.data( 'itemData', this.itemDataList[ id ] );
    //console.log( `Attach( ${ id }`, this.itemDataList[id] );
    return this.itemDataList[ id ];
  }

  ItemIsAttached( id )
  {
    return ( this.itemDataList[ id ]['$item'] != null );
  }

  DetachItem( id, recursive = true )
  {
    // prequisites
    if( !( id in this.itemDataList ) )  {
      console.warn( "Treeview.DetachItem(): id '" + id + "' not found" );
      return;
    }

    this.itemDataList[ id ]['$item'] = null;
    this.itemDataList[ id ]['$children'] = null;

    let childList = this.itemDataList[ id ]['record']['children'];
    for( let childRecord of childList )  {
      this.DetachItem( childRecord['id'], recursive );
    }

  }

  //-----------------------------------------------------------------
  //  set normalized status for item
  //
  ItemSetStatus( id, status )
  {
    if( !( id in this.itemDataList ) ) {
      console.warn( "Treeview.ItemSetStatus(): id '" + id + "' not found" );
      return;
    }

    var newStatus = Array.isArray( status ) ? status : status.split( /\s+/ );

    // base status : first of       [ 'normal', 'disabled', 'readonly', 'hilight', 'red', 'green', 'yellow', 'locked' ]
    // mixin status: combination of [ 'error', 'focus', 'selected', 'expanded', 'collapsed', 'no-hover', 'hidden', 'cursor' ],
    var resultStatus = [];
    resultStatus.push( $.array.FirstIn( newStatus, $.jqc.statics.themeBaseStateList ) );
    var mixinStatusList = $.array.In( newStatus, $.jqc.statics.themeMixinList );

    // put all together
    $.array.Join( resultStatus, mixinStatusList );

    // be sure, that just one of [ 'expanded', 'collapsed' ] is set
    $.array.MutualExclude( resultStatus, $.jqc.statics.themeMixinGroup );

    // set hover if not in  [ 'readonly', 'disabled', 'no-hover' ]
    if( !$.array.HasOneIn( resultStatus, $.jqc.statics.themeNoHoverList ) ) {
      resultStatus.push( 'hover' );
    }

    // remove selected if disabled
    if( $.array.Has( resultStatus, "disabled" ) ) {
      $.array.Remove( resultStatus, "selected" );
      $.array.Remove( this.selected, id );
    }

    // selected
    if( resultStatus.indexOf( 'selected' ) >= 0 )  {
      $.array.Add( this.selected, id );
    }

    // store and set class into $item
    this.itemDataList[ id ]['data']['status'] = resultStatus;
    if( this.itemDataList[ id ]['$item'] instanceof jQuery ) {
      var cssClasses = this.itemDataList[ id ]['data']['css'] + ' ' + resultStatus.join( ' ' );
      this.itemDataList[ id ]['$item'].attr( 'class', cssClasses );
    }

    return resultStatus;
  }

  //-----------------------------------------------------------------
  //  add a status to item
  //
  ItemAddStatus( id, status )
  {
    // prequistes
    if( !( id in this.itemDataList ) ) {
      console.warn( "Dataview.ItemAddStatus(): id '" + id + "' not found" );
      return;
    }

    var newStatus = status.split( /\s+/ );

    // base status : first of       [ 'normal', 'disabled', 'readonly', 'hilight', 'red', 'green', 'yellow' ]
    // mixin status: combination of [ 'error', 'focus', 'selected', 'expanded', 'collapsed', 'no-hover', 'hidden', 'cursor' ],
    var resultStatus = [];
    var newBaseStatus = $.array.FirstIn( newStatus, $.jqc.statics.themeBaseStateList );
    if( newBaseStatus )  {
      resultStatus.push( newBaseStatus );
    }
    else  {
      newBaseStatus = $.array.FirstIn( this.itemDataList[ id ]['data']['status'], $.jqc.statics.themeBaseStateList );
      if( newBaseStatus )  {
        resultStatus.push( newBaseStatus );
      }
    }
    var newMixinStatusList = $.array.In( newStatus, $.jqc.statics.themeMixinList );
    var mixinStatusList = $.array.In( this.itemDataList[ id ]['data']['status'], $.jqc.statics.themeMixinList );

    // put all together
    $.array.Join( resultStatus, mixinStatusList );
    $.array.Join( resultStatus, newMixinStatusList );

    // be sure, that just one of [ 'expanded', 'collapsed' ] is set
    $.array.MutualExclude( resultStatus, $.jqc.statics.themeMixinGroup );

    // set hover if not in  [ 'readonly', 'disabled', 'no-hover' ]
    if( !$.array.HasOneIn( resultStatus, $.jqc.statics.themeNoHoverList ) ) {
      resultStatus.push( 'hover' );
    }

    // remove selected if disabled
    if( $.array.Has( resultStatus, "disabled" ) ) {
      $.array.Remove( resultStatus, "selected" );
      $.array.Remove( this.selected, id );
    }

    // selected
    if( resultStatus.indexOf( 'selected' ) >= 0 )  {
      $.array.Add( this.selected, id );
    }

    // store and set class into $item
    this.itemDataList[ id ]['data']['status'] = resultStatus;
    if( this.itemDataList[ id ]['$item'] instanceof jQuery ) {
      var cssClasses = this.itemDataList[ id ]['data']['css'] + ' ' + resultStatus.join( ' ' );
      this.itemDataList[ id ]['$item'].attr( 'class', cssClasses );
    }

    return resultStatus;
  }
  
  //-----------------------------------------------------------------
  //  remove status from item (mixin only)
  //
  ItemRemoveStatus( id, status )
  {
    // prequistes
    if( !( id in this.itemDataList ) ) {
      console.warn( "Treeview.ItemRemoveStatus(): id '" + id + "' not found" );
      return;
    }

    // selected
    if( status == 'selected' )  {
      $.array.Remove( this.selected, id );
    }

    $.array.Remove( this.itemDataList[ id ]['data']['status'], status );
    if( this.itemDataList[ id ]['$item'] instanceof jQuery ) {
      var cssClasses = this.itemDataList[ id ]['data']['css'] + ' ' + this.itemDataList[ id ]['data']['status'].join( ' ' );
      this.itemDataList[ id ]['$item'].attr( 'class', cssClasses );
    }

    return this.itemDataList[ id ]['data']['status'];
  }

  //-----------------------------------------------------------------
  //  toggle between one or two status 
  //
  ItemToggleStatus( id, status1, status2 )
  {
    // prequistes
    if( !( id in this.itemDataList ) ) {
      console.warn( "Treeview.ItemToggleStatus(): id '" + id + "' not found" );
      return;
    }

    var resultStatus;

    // toggle between status1 <-> status2
    if( status2 )  {
      if( this.ItemHasStatus( id, status2 ) )  {
        $.array.Remove( this.itemDataList[ id ]['data']['status'], status2 );
        this.itemDataList[ id ]['data']['status'].push( status1 );
        resultStatus = status1;
      }
      else  {
        $.array.Remove( this.itemDataList[ id ]['data']['status'], status1 );
        this.itemDataList[ id ]['data']['status'].push( status2 );
        resultStatus = status2;
      }
    }
    // toggle between status1 <-> none
    else  {
      if( this.ItemHasStatus( id, status1 ) )  {
        $.array.Remove( this.itemDataList[ id ]['data']['status'], status1 );
        resultStatus = status1;
      }
      else  {
        this.itemDataList[ id ]['data']['status'].push( status1 );        
        resultStatus = "";
      }
    }

    if( this.itemDataList[ id ]['$item'] instanceof jQuery ) {
      var cssClasses = this.itemDataList[ id ]['data']['css'] + ' ' + this.itemDataList[ id ]['data']['status'].join( ' ' );
      this.itemDataList[ id ]['$item'].attr( 'class', cssClasses );
    }

    // selection
    var previousSelLength = this.selected.length;
    if( this.itemDataList[ id ]['data']['status'].indexOf( 'selected' ) >= 0 )  {
      $.array.Add( this.selected, id );
    }
    else  {
      $.array.Remove( this.selected, id );
    }
    if( previousSelLength != this.selected.length )  {
      this._FireEvent( 'select', this.selected );
    }

    return resultStatus;    
  }

  //-----------------------------------------------------------------
  //  remove status from item (mixin only)
  //
  ItemHasStatus( id, status )
  {
    // prequistes
    if( !( id in this.itemDataList ) ) {
      console.warn( "Treeview.ItemHasStatus(): id '" + id + "' not found" );
      return false;
    }

    console.assert( this.itemDataList[ id ] != undefined );
    console.assert( this.itemDataList[ id ]['data'] != undefined );
    console.assert( this.itemDataList[ id ]['data']['status'] != undefined );
    return ( this.itemDataList[ id ]['data']['status'].indexOf( status ) >= 0 );
  }

  //-----------------------------------------------------------------
  //  remove status from item (mixin only)
  //
  ItemGetStatus( id, status )
  {
    // prequistes
    if( !( id in this.itemDataList ) ) {
      console.warn( "Treeview.ItemGetStatus(): id '" + id + "' not found" );
      return false;
    }

    return ( this.itemDataList[ id ]['data']['status'] );
  }  

  // selection
  ItemToggleSelection( id )
  {
    var resultSelectStatus;

    if( this.selected.indexOf( id ) < 0 )  {
      this.ItemAddStatus( id, 'selected' );
      this._FireEvent( 'select', this.selected );
      resultSelectStatus = true;
    }
    // remove
    else  {
      this.ItemRemoveStatus( id, 'selected' );
      this._FireEvent( 'select', this.selected );
      resultSelectStatus = false;
    }

    return resultSelectStatus;
  }

  // selection
  ItemSelect( id, singleSelect )
  {
    this.ItemAddStatus( id, 'selected' );
    this._FireEvent( 'select', this.selected );

    return true;
  }  

  ItemUnselect( id )
  {
    this.ItemRemoveStatus( id, 'selected' );
    this._FireEvent( 'select', this.selected );

    return true;
  }  

  ItemClearSelections( )
  {
    for( var i = this.selected.length - 1; i >= 0; i-- )  {
      this.ItemRemoveStatus( this.selected[ i ], 'selected' );
    }
  }

  //-----------------------------------------------------------------
  //  OVERRIDABLES
  //-----------------------------------------------------------------

  DefaultFilter( id, record, options, self )
  {
    return true;
  }

  DefaultSearch( id, record, options, self )
  {
    if( self.search == null )  {
      return true;
    }
    else  {
      if( '_preview' in record && typeof ( record['_preview'] ) == "string" )  {
        var preview = record['_preview'];
        return ( preview.text.search( self.searchRx ) >= 0 );
      }
      else  {
        for( let key in record )  {
          if( typeof( record[key] ) == "string" && record[ key ].search( self.searchRx) >= 0 )  {
            return true;
          }
        }
        return false;
      }
    }
  }  

  //-----------------------------------------------------------------
  //  HELPER
  //-----------------------------------------------------------------

  //---[ fire event to parent ]--------------------------------------

  _FireEvent( event, param, extra, extra2 )
  {
    if( event in this.events )  {
      this.events[ event ]( param, extra, extra2 );
    }
    else if( 'change' in this.events )  {
      this.events['change']( event, param, extra, extra2 );
    }
  }  

  //---[ sort data ]-------------------------------------------------

  _SortBranch( branchId, childList )
  {
    var key = this.options['sortKey'];

    // prequisites
    if( !key || childList.length == 0 || this.options['sortOrder'] == 'none' || this.options['sortKey'] == null )  {
      return null;
    } 

    // store sorting
    var isAscending = ( this.options['sortOrder'] == '+' );
    var fnSort = $.isFunction( this.options.sorter ) ? this.options.sorter : null;

    // sort list OR record
    if( isAscending ) {
      childList.sort( ( recordA, recordB ) => {
        if( fnSort )  {
          var result = fnSort( recordA, recordB );
          if( Number.isInteger( result ) )  {
            return result;
          }
        }
        var textA = recordA['_preview'][ key ] || "";
        var textB = recordB['_preview'][ key ] || "";
        return textA.localeCompare( textB, navigator.language, { numeric: true, sensitivity: 'base' } );
      } );
    }
    // descending
    else  {
      childList.sort( ( recordA, recordB ) => {
        if( fnSort ) {
          var result = fnSort( recordB, recordA );
          if( Number.isInteger( result ) ) {
            return result;
          }
        }
        var textA = recordA['_preview'][ key ] || "";
        var textB = recordB['_preview'][ key ] || "";
        return textB.localeCompare( textA, navigator.language, { numeric: true, sensitivity: 'base' } );
      } );

    }

    // build sort list
    let sortList = [];
    for( let child of childList )  {
      sortList.push( child['id'] );
    }

    return sortList;

  }
  
}
/* ----------------------------------------------------------------------------
 *
 *   CYTEL Web Framework
 *   Drag and Drop
 *
 * --------------------------------------------------------------------------- */


//-----------------------------------------------------------------------------
//   SOURCE
//-----------------------------------------------------------------------------

class DragSource 
{
  // ------------------------------------------------------
  //   class variables
  // ------------------------------------------------------

  constructor( dragtype = "application/uuid" )
  {
    this.options = {
      element: null,
      dragtype: dragtype,
      start: null,
      end: null,
    };
    $.jqc.dragdrop = {
      source: null,
      target: null,
      dragtype: dragtype,
      dragdata: null
    };

  }

  // ------------------------------------------------------
  //   class private functions
  // ------------------------------------------------------

  _dragStart( ev )
  {
    this.options['dragdata'] = this.options['start']( this.options['element'] );

    $.jqc.dragdrop = {
      source: this.options['element'],
      target: null,
      dragtype: this.options['dragtype'],
      dragdata: this.options['dragdata'],
    };

    ev.dataTransfer.effectAllowed = "all";
  }

  _dragEnd( ev )
  {
    this.options['end']( this.options['element'] );
  }


  // ------------------------------------------------------
  //   class public members
  // ------------------------------------------------------

  Start( element, fnStart, fnEnd )  
  {
    // element handling
    if( element instanceof jQuery ) {
      element = element[ 0 ];
    }
    /*else if( element instanceof Element ) {
      element = element;
    }*/
    else {
      element = document.getElementById( element );
    }

    // error handling
    if( element == null ) {
      console.warn( "DragSource: element '" + options['tag'] + "' not found!" );
    }

    // store
    this.options['element'] = element;
    this.options['start'] = fnStart;
    this.options['end'] = fnEnd;
    this.options['bindStart'] = this._dragStart.bind( this );
    this.options['bindEnd'] = this._dragEnd.bind( this );

    // install event handler
    if( element ) {
      element.setAttribute( "draggable", "true" );
      element.addEventListener( 'dragstart', this.options['bindStart'], true );
      element.addEventListener( 'dragend', this.options['bindEnd'], true );
    }

  }

  Stop()  
  {
    // remove event handler
    let element = this.options['element'];
    if( element ) {
      element.removeAttribute( "draggable" );
      element.removeEventListener( 'dragstart', this.options['bindStart'], true );
      element.removeEventListener( 'dragend', this.options['bindEnd'], true );
    }    
  }  

}

//=============================================================================
//
//   TARGET
//
//=============================================================================

class DropTarget
{
  // ------------------------------------------------------
  //   class variables
  // ------------------------------------------------------

  constructor( dragtype = "application/uuid" )
  {
    this.defaultOptions = {
      element: null,
      dropEffect: null,
      dragtype: dragtype,
      // -- expand --
      timeStamp: false,     // ts of dragEnter (false := already fired)
      expandTimer: 750,
      // -- event callbacks
      enter: null,
      leave: null,
      drop: null,
      expand: null
    };
  }

  // ------------------------------------------------------
  //   class private functions
  // ------------------------------------------------------

  _dragEnter( ev )
  {
    // no action if source is us
    if( $.jqc.dragdrop['source'] == this.options['param'] )  {
      return;
    }
    // dragtype must compatible
    if( $.jqc.dragdrop['dragtype'] != this.options['dragtype'] ) {
      return;
    }

    // prequisites
    ev.preventDefault();
    ev.stopImmediatePropagation();
    ev.stopPropagation();

    // set target in global drag & drop struct 
    $.jqc.dragdrop['target'] = this.options['param'];
    this.options['timeStamp'] = ev.timeStamp;

    // call function & style
    ev.currentTarget.classList.add( "dragover" );
    let action = this.options['enter']( { data: $.jqc.dragdrop['dragdata'], type: $.jqc.dragdrop['dragtype'] } , this.options['element'], $.jqc.dragdrop['source']  );

    // define drop action
    if( action === false || action === "none" )  {
      this.options['dropEffect'] = "none";
      return false;
    }
    else if( action == "copy" || action == "link" || action == "move" ) {
      this.options['dropEffect'] = action;
      ev.dataTransfer.dropEffect = action;
      return true;
    }
    else  {
      this.options['dropEffect'] = "move";
      return true;
    }

  }

  _dragOver( ev )
  {
    // no action if source is us
    if( $.jqc.dragdrop['source'] == this.options['param'] ) {
      return;
    }
    // dragtype must compatible
    if( $.jqc.dragdrop['dragtype'] != this.options['dragtype'] ) {
      return;
    }

    // expand timer
    if( this.options['timeStamp'] !== false 
     && this.options['expand'] != null 
     && ( ev.timeStamp - this.options['timeStamp'] ) > this.options['expandTimer'] 
      ) {
      this.options['expand']( this.options['param'], this.options['element'], $.jqc.dragdrop['source'] );
      this.options['timeStamp'] = false;
    }

    // repeat values from dragEnter
    ev.preventDefault();
    ev.stopImmediatePropagation();
    ev.stopPropagation();
    ev.dataTransfer.dropEffect = this.options['dropEffect'];
  }

  _dragLeave = function( ev )
  {
    // no action if source is us
    if( $.jqc.dragdrop['source'] == this.options['param'] ) {
      return;
    }

    // prequisites
    ev.preventDefault();
    ev.stopImmediatePropagation();
    ev.stopPropagation();

    // call function
    this.options['leave']( { data: $.jqc.dragdrop['dragdata'], type: $.jqc.dragdrop['dragtype'] }, this.options['element'], $.jqc.dragdrop['source'] );
    ev.currentTarget.classList.remove( "dragover" );

  };

  _drop( ev )
  {
    // no action if source is us
    if( $.jqc.dragdrop['source'] == this.options['param'] ) {
      return;
    }

    // prequisites
    ev.preventDefault();
    ev.stopImmediatePropagation();
    ev.stopPropagation();

    // call function & style
    this.options['drop']( this.options['dropEffect'], { data: $.jqc.dragdrop['dragdata'], type: $.jqc.dragdrop['dragtype'] }, this.options['element'], $.jqc.dragdrop['source'] );
    ev.currentTarget.classList.remove( "dragover" );

  }


  // ------------------------------------------------------
  //   class public members
  // ------------------------------------------------------

  Start( options, fnEnter, fnLeave, fnDrop, fnExpand = null )
  {
    // element handling
    var element = null;
    if( options['element'] instanceof jQuery ) {
      element = options['element'][ 0 ];
    }
    else if( options['element'] instanceof Element ) {
      element = options['element'];
    }
    else {
      element = document.getElementById( options['element'] );
    }

    // error handling
    if( element == null ) {
      console.warn( "DropTarget: element '" + options['element'] + "' not found!" );
    }

    // build options
    this.options = $.extend( true, [], this.defaultOptions, options );
    this.options['element'] = element;
    this.options['enter'] = fnEnter;
    this.options['leave'] = fnLeave;
    this.options['drop'] = fnDrop;
    this.options['expand'] = fnExpand;
    this.options['bindEnter'] = this._dragEnter.bind( this );
    this.options['bindOver'] = this._dragOver.bind( this );
    this.options['bindLeave'] = this._dragLeave.bind( this );
    this.options['bindDrop'] = this._drop.bind( this );

    // install event handler
    if( element ) {
      element.addEventListener( 'dragenter', this.options['bindEnter'], true );
      element.addEventListener( 'dragover', this.options['bindOver'], true );
      element.addEventListener( 'dragleave', this.options['bindLeave'], true );
      element.addEventListener( 'drop', this.options['bindDrop'], true );
    }    
  }

  Stop( )  
  {
    // remove event handler
    let element = this.options['element'];
    if( element ) {
      element.removeEventListener( 'dragenter', this.options['bindEnter'], true );
      element.removeEventListener( 'dragover', this.options['bindOver'], true );
      element.removeEventListener( 'dragleave', this.options['bindLeave'], true );
      element.removeEventListener( 'drop', this.options['bindDrop'], true );
    }        
  }    

}

//=============================================================================
//
//   DropScroll
//
//=============================================================================

class DropScroll 
{
  // ------------------------------------------------------
  //   init
  // ------------------------------------------------------

  constructor( options )
  {
    // defaults
    this.defaultOptions = {
      tag: null,
      element: null,
      scroll: null,
      zones:  []
    };

    // element handling
    var element = null;
    if( options['tag'] instanceof jQuery ) {
      element = options['tag'][ 0 ];
    }
    else if( options['tag'] instanceof Element ) {
      element = options['tag'];
    }
    else {
      element = document.getElementById( options['tag'] );
    }

    // error handling
    if( element == null ) {
      console.warn( "DropScroll: element '" + options['tag'] + "' not found!" );
    }

    // build options
    this.options = $.extend( true, [], this.defaultOptions, options );
    this.options['element'] = element;

  }

  // ------------------------------------------------------
  //   class public members
  // ------------------------------------------------------

  Start( fnEnter, fnLeave )
  {
    // store event handlers
    this.options['enter'] = fnEnter;
    this.options['leave'] = fnLeave;
    this.options['zones'] = [];

    // prequisites
    var $root = $( document.body );
    var position = $( this.options['element'] ).offset();
    var height = $( this.options['element'] ).outerHeight();
    var width = $( this.options['element'] ).outerWidth();

    // drop zone top
    var $topZone = $( "<div class='dropscroll'>" );
    this.options['zones'].push( $topZone );
    $root.append( $topZone );
    $topZone.css( "top", position.top );
    $topZone.css( "left", position.left );
    $topZone.css( "width", width );
    $topZone.attr( "id", "top" );
    //  event handler
    $topZone[ 0 ].addEventListener( 'dragenter', this._dragEnter.bind( this ), true );
    $topZone[ 0 ].addEventListener( 'dragover', this._dragOver.bind( this ), true );
    $topZone[ 0 ].addEventListener( 'dragleave', this._dragLeave.bind( this ), true );
    $topZone[ 0 ].addEventListener( 'drop', this._drop.bind( this ), true );

    // create drop zone top (high speed)
    var $topZoneSpeed = $( "<div class='dropscroll-speed'>" );
    this.options['zones'].push( $topZoneSpeed );
    $root.append( $topZoneSpeed );
    $topZoneSpeed.css( "top", position.top - $topZoneSpeed.height() );
    $topZoneSpeed.css( "left", position.left );
    $topZoneSpeed.css( "width", width );
    $topZoneSpeed.attr( "id", "top+" );
    //this.options['zones']['$topHs'] = $topZoneSpeed;
    $topZoneSpeed[ 0 ].addEventListener( 'dragenter', this._dragEnter.bind( this ), true );
    $topZoneSpeed[ 0 ].addEventListener( 'dragover', this._dragOver.bind( this ), true );
    $topZoneSpeed[ 0 ].addEventListener( 'dragleave', this._dragLeave.bind( this ), true );
    $topZoneSpeed[ 0 ].addEventListener( 'drop', this._drop.bind( this ), true );

    // create bottom zone top
    var $bottomZone = $( "<div class='dropscroll'>" );
    this.options['zones'].push( $bottomZone );
    $bottomZone.css( "top", position.top + height - $topZone.height() );
    $bottomZone.css( "left", position.left );
    $bottomZone.css( "width", width );
    $bottomZone.attr( "id", "bottom" );
    $root.append( $bottomZone );
    $bottomZone[ 0 ].addEventListener( 'dragenter', this._dragEnter.bind( this ), true );
    $bottomZone[ 0 ].addEventListener( 'dragover', this._dragOver.bind( this ), true );
    $bottomZone[ 0 ].addEventListener( 'dragleave', this._dragLeave.bind( this ), true );
    $bottomZone[ 0 ].addEventListener( 'drop', this._drop.bind( this ), true );

    // create drop zone top (hight speed)
    var $bottomZoneSpeed = $( "<div class='dropscroll-speed'>" );
    this.options['zones'].push( $bottomZoneSpeed );
    $bottomZoneSpeed.css( "top", position.top + height );
    $bottomZoneSpeed.css( "left", position.left );
    $bottomZoneSpeed.css( "width", width );
    $bottomZoneSpeed.attr( "id", "bottom+" );
    $root.append( $bottomZoneSpeed );
    $bottomZoneSpeed[ 0 ].addEventListener( 'dragenter', this._dragEnter.bind( this ), true );
    $bottomZoneSpeed[ 0 ].addEventListener( 'dragover', this._dragOver.bind( this ), true );
    $bottomZoneSpeed[ 0 ].addEventListener( 'dragleave', this._dragLeave.bind( this ), true );
    $bottomZoneSpeed[ 0 ].addEventListener( 'drop', this._drop.bind( this ), true );

  }

  Stop( )
  {
    for( let $zone of this.options['zones'] )  {
      $zone.remove();
    }

  }

  //---------------------------------------------------------------------------
  //  EVENT HANDLERS
  //---------------------------------------------------------------------------

  _dragEnter( ev )
  {
    // default is dropEffect 'deny'
    ev.preventDefault();

    if( ev.target.id == "bottom" )  {
      this.options.scroll.scrollbar( "scroll", 10 );
    }
    else if( ev.target.id == "bottom+" ) {
      this.options.scroll.scrollbar( "scroll", 20 );
    }
    else if( ev.target.id == "top" ) {
      this.options.scroll.scrollbar( "scroll", -10 );
    }
    else if( ev.target.id == "top+" ) {
      this.options.scroll.scrollbar( "scroll", -20 );
    }

    ev.dataTransfer.dropEffect = "move";
    return true;
  }

  _dragOver( ev )
  {
    // default is dropEffect 'deny'
    ev.preventDefault();

    if( ev.target.id == "bottom" ) {
      this.options.scroll.scrollbar( "scroll", 10 );
    }
    else if( ev.target.id == "bottom+" ) {
      this.options.scroll.scrollbar( "scroll", 20 );
    }
    else if( ev.target.id == "top" ) {
      this.options.scroll.scrollbar( "scroll", -10 );
    }
    else if( ev.target.id == "top+" ) {
      this.options.scroll.scrollbar( "scroll", -20 );
    }

    ev.dataTransfer.dropEffect = "move";
    return true;
  }

  _dragLeave( ev )
  {
    this.options['leave']( ev.target.id );
  }
  _drop( ev )
  {
    this.options['leave']( ev.target.id );
  }

}
//=============================================================================
//
//  CYTEL Views2 - Web Framework
//  
//  Simple state machine for start/next/finished
//
//=============================================================================

class Statemachine
{
  fnStart = null;
  fnStep = null;
  fnFinished = null;
  currentIndex = -1;
  steps = [];

  //---------------------------------------------------------------------------
  //  start state machine
  //
  constructor( steps, fnStart, fnStep, fnFinished )
  {
    if( !Array.isArray( steps ) ) {
      throw "Constructor needs array with steps!";
    }
    if( typeof fnStart !== "function" ) {
      throw "Constructor needs start function!";
    }
    if( typeof fnStep !== "function" ) {
      throw "Constructor needs step function!";
    }
    this.steps = steps;
    this.fnStep = fnStep;
    this.fnFinished = ( typeof fnFinished === "function" ) ? fnFinished : null;
    this.fnStart = ( typeof fnStart === "function" ) ? fnStart : null;;
  }

  //---------------------------------------------------------------------------
  //  start the statemachine
  //
  Append( step )
  {
    if( Array.isArray( step ) ) {
      this.steps.concat( step );
    }
    else {
      this.steps.push( step );
    }
  }

  //---------------------------------------------------------------------------
  //  start the statemachine
  //
  Start()
  {
    this.currentIndex = 0;
    var isFinished = ( this.steps.length < 2 ) || ( this.currentIndex > 1000000 );

    if( this.fnStart ) {
      this.fnStart( this.steps[ this.currentIndex ], this.currentIndex + 1, this.steps.length );
    }
    if( isFinished && this.fnFinished ) {
      this.fnFinished( this.steps[this.currentIndex], this.currentIndex + 1, this.steps.length );
    }    
    
    return !isFinished;
  }

  //---------------------------------------------------------------------------
  //  next step
  //
  Next()
  {
    if( this.currentIndex == -1 )  {
      return this.Start();
    }

    this.currentIndex++;
    var isFinished = ( this.steps.length - 2 < this.currentIndex ) || ( this.currentIndex > 1000000 );

    if( isFinished && this.fnFinished ) {
      this.fnFinished( this.steps[ this.currentIndex ], this.currentIndex + 1, this.steps.length );
    }
    else if( this.fnStep ) {
      this.fnStep( this.steps[ this.currentIndex ], this.currentIndex + 1, this.steps.length );
    }

    return !isFinished;
  }



}/*
 *  ACCESS 5.0 Web Framework
 *  SCROLLBAR
 */

!function( $ )
{

/* ---------------------------------------------------------------------------
 *   SCROLL - const values
 * ------------------------------------------------------------------------ */
  // define keycodes
  let KEY = {
    BS: 8,
    TAB: 9,
    RETURN: 13,
    ENTER: 13,
    SHIFT: 16,
    CTRL: 17,
    ALT: 18,
    SCTRL: 20,
    ESC: 27,
    SPACE: 32,
    END: 35,
    POS1: 36,
    LEFT: 37,
    UP: 38,
    RIGHT: 39,
    DOWN: 40,
    DEL: 46,
    PGDOWN: 34,
    PGUP: 33
  };

/* ---------------------------------------------------------------------------
 *   SCROLL - class definition
 * ------------------------------------------------------------------------ */

var WfcScrollbar = function( element, options )
{
  this.init( 'scrollbar', element, options );
};

WfcScrollbar.prototype = {

  // ----------------------------------------------------
  //   class member
  //
  $element:  null,          // scroll-container
  $scrollable: null,        // scrollable window
  $scrollbar: null,         // scollbar itself
  $handle: null,
  viewContainerSize: 0,     // heigh of visible window
  viewContainerSizeChanged: false, 
  viewScrollableSize: 0,    // heigh of the document
  viewCurrentScrollPos: 0,  // scroll pos in pixel
  viewScrollMax: 0,         // max. amount of pixel to scroll (view)
  scrollHandleSize: 0,      // size of scroll handle in percent
  resizeFnProxy: null,
  mouse: null,
  scrollSubscriptions: [],

  // ----------------------------------------------------
  //   constructor directs back to class function
  //
  constructor: WfcScrollbar,

  // ----------------------------------------------------
  //   init-function (called by constructor)
  //
  init: function( type, element, options )  
  {
    var self = this;

    // class variables
    this.type = type;
    this.options = $.extend( true, [], $.fn.scrollbar.defaults, options );
    this.$element = element;
    this.resizeFnProxy = $.proxy( this.recalcLayout, this );
    this.scrollSubscriptions = [];
    this.lineHight = this.options['lineHeight'] ? this.options['lineHeight'] : $.metrics( 'lineHeight' );


    //---------------------------------------------------
    //  WEBVIEW (BUILT IN)
    //---------------------------------------------------

    // on android or ios we use the built in scrollbar
    if( $.metrics.ios || $.metrics.android )  {
      if( this.$element.hasClass( 'scroll-container-static' ) )  {
        this.$element.css( 'overflow-y', 'auto' );
        return;
      }
      else if( this.$element.hasClass( 'scroll-container' ) )  {
        this.$element.css( 'overflow-y', 'auto' );
        if( this.options['elevation'] )  {
          this.$element.on( 'scroll', function( e ) {
            if( this.scrollTop > 0 )  {
              self.$element.addClass( 'scroll-elevation' );
              self.isElementElavated = true;
            }
            else if( self.isElementElavated )  {
              self.$element.removeClass( 'scroll-elevation' );
              self.isElementElavated = false;
            }
          } );
        }

        return;
      }  
    }

    //---------------------------------------------------
    //  BROWSER (CUSTOM)
    //---------------------------------------------------

    // on browsers we use custom scrolling
    if( this.$element.hasClass( 'scroll-container-static' ) )  {
      this.$scrollable = this.$element.find( '.scrollable' );
      if( this.$scrollable.length != 1 )  {
        console.error( "scrollbar must be initialized on 'scroll-container-static' that containes a 'scrollable' " );
        return;
      }
      this.options['type'] = 'static';
    }
    else if( this.$element.hasClass( 'scroll-container' ) )  {
      this.$scrollable = this.$element.find( '.scrollable' );
      if( this.$scrollable.length != 1 )  {
        console.error( "scrollbar must be initialized on scroll-container' that containes a 'scrollable' " );
        return;
      }
      if( this.options['type'] != 'static' )  {
        this.options['type'] = 'overlay';
      }
    }
    else  {
      return;
    }

    // keep time to render the view
    setTimeout( function( ) {
      self.recalcScrollRange();
    }, 100 );


    // needed for keyboard handling
    if( this.options['keyboard'] == true ) {
      this.$scrollable.attr( "tabindex", "0" );
    }

    // draw everything
    this.recalcLayout( );

    // attach event global resize event
    this.resizeTimeout = null;
    
    // wait for resize completed and call recalcScrollRange
    if( this.options['resize'] == true )  {
      window.addEventListener( 'resize', function() {        
        clearTimeout( self.resizeTimeout );
        self.resizeTimeout = setTimeout( $.proxy( self.recalcScrollRange, self ), 250 );
      } );
    }

    // watch for change layout by child elements
    this.$element.on( "event:layout-changed", this.recalcScrollRange.bind( this ) );

    // a child wants subscribe scroll events
    this.$element.on( "event:subscribe-scroll", ( ev ) => {
      ev.preventDefault();
      let index = this.scrollSubscriptions.findIndex( ( element ) => element == ev.target );
      if( index < 0 )  {
        this.scrollSubscriptions.push( ev.target );
      }
    } );
    this.$element.on( "event:unsubscribe-scroll", ( ev ) => {
      ev.preventDefault();
      let index = this.scrollSubscriptions.findIndex( ( element ) => element == ev.target );
      if( index >= 0 ) {
        this.scrollSubscriptions.splice( index, 1 );
      }
    } );


  },

  // ----------------------------------------------------
  //   language function
  //
  langEncode: function() 
  {
    this.options[ 'tooltip' ] = $.lang( this.options[ 'tooltip' ] );
  },

  // ----------------------------------------------------
  //   draw
  //
  recalcLayout: function()
  {
    var selfScroll = this;
    if( this.$scrollbar == null )  {
      var scrollClass =  'scrollbar-overlay-v';
      if( this.options['type'] == 'static' )  {
        scrollClass =  'scrollbar-static-v';
      }
      this.$scrollbar = $( "<div class='" + scrollClass + "'>" );
      this.$handle    = $( "<div class='handle'>" );
      this.$scrollbar.append( this.$handle );
      this.$element.append( this.$scrollbar );

      var $parent = this.$element.find( '.scrollable' );
      if( $parent.length > 0 )  {
        var scrollWidth = this.$scrollbar.outerWidth( );
        var cssMarginRight = parseInt( $parent.css( 'margin-right' ) ) + scrollWidth; //$.metrics( 'scrollWidth' );
        if( this.options['type'] == 'static' )  {
          $parent.css( 'margin-right', cssMarginRight );
        }
        else  {
          $parent.hover( function( e ) {
            selfScroll.$handle.css( 'opacity', '0.5' );
          },
          function( e )  {
            var to = e.toElement || e.relatedTarget;
            if( to == selfScroll.$scrollbar[ 0 ] || to == selfScroll.$handle[ 0 ] ) {
              return;
            }
            selfScroll.$handle.css( 'opacity', '0' );
          } );
        }

        // check if mouse is inside while creating
        var element = $parent[ 0 ];
        if( element.parentNode.querySelector( ":hover" ) == element ) {
          selfScroll.$handle.css( 'opacity', '0.5' );
        } 

      }

      this.attachEvents( );
      if( this.options['keyboard'] == true ) {
        this.$scrollable.attr( "tabindex", "0" );
        this.attachKeyboardEvents();
      }
      else {
        this.$scrollable.attr( "tabindex", "" );
      }

    }

  },

  // ----------------------------------------------------
  //   events
  //
  recalcScrollRange: function( )
  {
    // prequisites
    if( this.$element.is( ':hidden' ) )  {
      return;
    }
    if( this.$scrollable == null )  {
      return;
    }

    // 1st check if sizes has been changed, no: do nothing
    var elementSize = this.$element.height( );
    var scrollableSize = this.$scrollable.outerHeight( );
    if( this.viewContainerSize == elementSize &&  this.viewScrollableSize == scrollableSize )  {
      return;
    }

    // remember values for next call
    this.viewContainerSize = elementSize;
    this.viewContainerSizeChanged = true;
    this.viewScrollableSize = scrollableSize;

    // recalc 
    this.scrollHandleSize = Math.round( this.viewContainerSize * 1000 / this.viewScrollableSize ) / 10;
    if( this.scrollHandleSize < 10 )  this.scrollHandleSize = 10;
    this.scrollRange = 100 - this.scrollHandleSize;
    this.viewScrollMax = this.viewScrollableSize - this.viewContainerSize;
    if( this.viewScrollMax < 0 )  {
      this.viewScrollMax = 0;
    }

    // manage ceil
    if( this.viewCurrentScrollPos > this.viewScrollMax ) {
      this.viewCurrentScrollPos = this.viewScrollMax;
    }

    // show/hide scroll handle
    if( this.scrollHandleSize < 100.0 )  {
      this.$handle.css( 'height', this.scrollHandleSize + '%' );
      this.$handle.css( 'display', '' );
      this.$handle.css( 'top', '0%' );     
      this.doScroll( 0 );
    }
    else  {
      this.viewCurrentScrollPos = 0;
      this.$scrollable.css( 'top', this.viewCurrentScrollPos );
      this.$handle.css( 'display', 'none' );
      if( this.options['scroll'] )  {
        this.viewContainerSizeChanged = false;
        this.options['scroll']( null );
      }
    }
  },    

  // ----------------------------------------------------
  //   events
  //
  attachEvents: function( )
  {
    // prequisites
    var self = this;
    this.$element.off( );

    // wheel event      
    var domElement = this.$element[ 0 ];
    domElement.addEventListener( 'wheel', function( e ) {
      
      var scrollAmount = e.deltaY;
      switch( e.deltaMode )  {
        case 0: scrollAmount = scrollAmount / 2;                break;    // DOM_DELTA_PIXEL
        case 1: scrollAmount = scrollAmount * this.lineHight;   break;    // DOM_DELTA_LINE
        case 3: scrollAmount = scrollAmount / 4;                break;    // DOM_DELTA_PAGE
      }
      self.doScroll( scrollAmount );
      e.stopPropagation( );
    }, { passive: true } );

    
    // track scrollbar handle
    this.mouseStartY = 0;      
    this.mouseCurrent = 0;
    this.mouse = new MouseCapture;
    this.mouse.start( this.$handle,
      // mouse down
      function( e )  {
        self.mouseStartY = e.clientY;
        self.mouseCurrent = self.viewCurrentScrollPos;
      },
      // move mouse
      function( e )  {
        var mouseDy = e.clientY - self.mouseStartY;
        var scaleFactor = self.viewScrollableSize / self.viewContainerSize;
        self.scrollTo( self.mouseCurrent + mouseDy * scaleFactor );
      },
      // mouse up
      function( e )  { }
    );
  },

  attachKeyboardEvents: function()
  {
    var lineHight = $.metrics( 'lineHeight' );

    // keyboard event
    this.$element.on( "keydown", ( ev ) => {

      if( document.activeElement == this.$scrollable[0] ) {
        if( ev.which == KEY.POS1 ) {
          this.scrollTo( 0 );
        }
        else if( ev.which == KEY.END ) {
          this.scrollTo( 32536 );
        }
        else if( ev.which == KEY.UP ) {
          this.doScroll( -lineHight );
        }
        else if( ev.which == KEY.DOWN ) {
          this.doScroll( lineHight );
        }
        else if( ev.which == KEY.PGDOWN ) {
          let amount = this.$element.height() - 2 * lineHight;
          this.doScroll( amount );
        }
        else if( ev.which == KEY.PGUP ) {
          let amount = this.$element.height() - 2 * lineHight;
          this.doScroll( -amount );
        }
        ev.stopPropagation();
      }
    } );
  },

  // ----------------------------------------------------
  //   make the actual scrolling
  //
  scrollTo: function( position )
  {
    // prequisites
    if( position < 0 ) {
      position = 0;
    }
    else if( position > this.viewScrollMax ) {
      position = this.viewScrollMax;
    }

    var isPositionChanged = ( position != this.viewCurrentScrollPos );
    this.viewCurrentScrollPos = position;
    
    // calc the position of scroll handle in percent
    var scrollHandlePosition = this.viewCurrentScrollPos * 100 / this.viewScrollableSize;
     
    // set css for view and scroll handle
    clearTimeout( this.timerId );
    this.timerId = setTimeout( () => {
      this.$scrollable.css( 'top', -( this.viewCurrentScrollPos ) );
    }, 0 );
    this.$handle.css( 'top', scrollHandlePosition + '%' );

    // scroll event to owner
    if( this.options['scroll'] && ( isPositionChanged || this.viewContainerSizeChanged ) )  {
      // reset update flag
      this.viewContainerSizeChanged = false;
      
      var scrollInfo = {
        pos: this.viewCurrentScrollPos,
        height: this.viewContainerSize,
        firstIndex: this.options['lineMode'] ? Math.floor( this.viewCurrentScrollPos / this.lineHight ) : 0,
        lastIndex: this.options['lineMode'] ? Math.floor( ( this.viewCurrentScrollPos + this.viewContainerSize ) / this.lineHight ) : 0
      };
      this.options['scroll']( scrollInfo );
    }

    // send scroll events to subscribers
    for( let subscriber of this.scrollSubscriptions )  {
      $( subscriber ).trigger( "event:scroll", { "scrollPos": this.viewCurrentScrollPos } );
    }
  },

  // ----------------------------------------------------
  //   scroll an spefific amout (e.g. "-10")
  //
  doScroll: function( delta )
  {
    this.scrollTo( this.viewCurrentScrollPos + delta );
  },


  // ----------------------------------------------------
  //   make sure, that y1 - y2 is visible
  //
  show: function( args )
  {    
    var y1 = args[ 1 ];
    var y2 = args[ 2 ];
    if( y2 > this.viewCurrentScrollPos + this.viewContainerSize )  {
      this.scrollTo( y2 - this.viewContainerSize );
    }
    else if( y1 < this.viewCurrentScrollPos )  {
      this.scrollTo( y1 );
    }
  },

  // ----------------------------------------------------
  //   make sure, that y1 - y2 is visible
  //
  scroll: function( args )
  {
    var delta = args[ 1 ];
    this.doScroll( delta );
  },  

  // ----------------------------------------------------
  //   recalc scrolling after event from parent
  //
  recalc: function( args )
  {    
    this.recalcScrollRange( );
  },

  // ----------------------------------------------------
  //   destroy scrollbar
  //
  destroy: function( args )
  {
    this.$element.empty();
  },

  flush: function( args )
  {
    this.$handle.css( 'opacity', '0.5' );
    setTimeout( () => {
      this.$handle.css( 'opacity', '0' );
    }, 1000 );

  },

  // ----------------------------------------------------
  //   option getter and setter
  //
  option: function( args )  
  {
    // setter
    if( args.length > 2 ) {
      this.options[ args[ 1 ] ] = args[ 2 ];
      this.recalcLayout();
    }
    // getter
    return this.options[ args[ 1 ] ];
  },

};

/* ---------------------------------------------------------------------------
 *   SCROLLBAR - plugin definition
 * ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   plugin main function
//
$.fn.scrollbar = function( option )  
{
  var data    = $( this ).data( "ctrl" ),
      options = ( typeof option == "object" && option );

  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data ) {
    var instance = new WfcScrollbar( this, options );
    $( this ).data( "ctrl", instance );
  }

  // if called with a string, treat this as a function call
  else if( typeof option == "string" ) {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy( );
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) ) {
      return data[ option ]( arguments );
    }
    else {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.scrollbar.defaults = {
  tooltip: null,
  resize: true,
  watch: false,
  keyboard: false,
  type: 'overlay',  // [ 'overlay' | 'static ]
  scrollClass: 'scrollbar-static-v',
  elevation: true,
  lineHeight: null,
  lineMode: true,
  // callback
  scroll: null
};


}( window.jQuery );

/*
 *  ACCESS 5.0 Web Framework
 *  SPLITTER
 */

!function( $ )
{

/* ---------------------------------------------------------------------------
 *   MENU - class definition
 * ------------------------------------------------------------------------ */

class WfcSplitter extends JqControl
{
  // ----------------------------------------------------
  //   init-function (called by constructor)
  //
  OnInit( $element, options )  
  {
    // options
    this.options = $.extend( true, [], $.fn.splitter.defaults, options );

    // class variables
    this.$element = $element;
    this.$splitter = null;
    this.isHorizontal = true;
    this.isOpposite = false;
    this.mouseStartPos = 0;
    this.elementSize = 0;
    this.minSize = 0;

  }
  
  // ----------------------------------------------------
  //   draw (called by constructor && options )
  //
  OnDraw( )
  {
    var self = this;
    let storedWidth = 0;
    let optionWidth = 0;

    // --- get values from registry and options ---
    if( this.options['regkey'] )  {
      storedWidth = $.registry( this.options.regkey );
    }
    if( !Number.isNaN( this.options['size'] ) )  {
      optionWidth = this.options['size'] * $.metrics( 'fontWidth' );
    }

    // --- set values to control ---
    if( storedWidth > 0 )  {
      this.$element.width( storedWidth );
    }
    else if( optionWidth > 0 )  {
      this.$element.width( optionWidth );
    }

    // --- build  ---
    this.$element.addClass( 'splitter-parent' );
    if( this.options['position'] == 'right' )  {
      this.$splitter = this.$element.append( "<div class='splitter-h-r'></div>" ).find( ".splitter-h-r" );
      this.isHorizontal = true;
      this.isOpposite = false;
      this.minSize = ( this.options['minSize'] * $.metrics( 'fontWidth' ) );
      this.maxSize = ( this.options['maxSize'] * $.metrics( 'fontWidth' ) );
    }
    else if( this.options['position'] == 'left' )  {
      this.$splitter = this.$element.append( "<div class='splitter-h-l'></div>" ).find( ".splitter-h-l" );
      this.isHorizontal = true;
      this.isOpposite = true;
      this.minSize = ( this.options['minSize'] * $.metrics( 'fontWidth' ) );
      this.maxSize = ( this.options['maxSize'] * $.metrics( 'fontWidth' ) );
    }
    else if( this.options['position'] == 'top' )  {
      this.$splitter = this.$element.append( "<div class='splitter-v-t'></div>" ).find( ".splitter-v-t" );
      this.isHorizontal = false;
      this.isOpposite = false;
      this.minSize = ( this.options['minSize'] * $.metrics( 'fontHeight' ) );
      this.maxSize = ( this.options['maxSize'] * $.metrics( 'fontHeight' ) );
    }
    else if( this.options['position'] == 'bottom' )  {
      this.$splitter = this.$element.append( "<div class='splitter-v-b'></div>" ).find( ".splitter-v-b" );
      this.isHorizontal = false;
      this.isOpposite = true;
      this.minSize = ( this.options['minSize'] * $.metrics( 'fontHeight' ) );
      this.maxSize = ( this.options['maxSize'] * $.metrics( 'fontHeight' ) );
    }

    // --- mouse event  ---
    if( this.isHorizontal )  {

      this.mouseStartPos = 0;      
      this.mouse = new MouseCapture;
      this.mouse.start( this.$splitter,
        // mouse down
        function( e )  {
          self.mouseStartPos = e.clientX;
          self.elementSize = self.$element.outerWidth( );
        },
        // move mouse
        function( e )  {
          var mouseDx = e.clientX - self.mouseStartPos;
          var mouseWidth = ( self.isOpposite == true ) ? ( self.elementSize - mouseDx ) : ( self.elementSize + mouseDx );
          if( mouseWidth > self.minSize )  {
            self.$element.width( mouseWidth );
          }
        },
        // mouse up
        function( e )  {
          // store within registry
          if( $.isFunction( $.registry ) && typeof self.options.regkey == "string" )  {
            $.registry( self.options.regkey, self.$element.outerWidth( ) );
          }           
        }
      );      
    }
  }

}


/* ---------------------------------------------------------------------------
 *   SPLITTER - plugin definition
 * ------------------------------------------------------------------------ */


// ----------------------------------------------------
//   plugin main function
//
$.fn.splitter = function( option )  
{
  var data    = $( this ).data( "ctrl" ),
      options = ( typeof option == "object" && option );

  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy( );
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data ) {
    var instance = new WfcSplitter( this, options );
    $( this ).data( "ctrl", instance );
  }

  // if called with a string, treat this as a function call
  else if( typeof option == "string" ) {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) ) {
      return data[ option ]( arguments );
    }
    else {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.splitter.defaults = {
  position: 'right',
  regkey: null,
  minSize: 4,
  size: 16,
  maxSize: 0
};


}( window.jQuery );
/*
 *  ACCESS 5.0 Web Framework
 *  metrics
 */

!function( $ ) {

  /* ---------------------------------------------------------------------------
   *   CHOICE - const values
   * ------------------------------------------------------------------------ */

  // ----------------------------------------------------
  //   metrics default function
  //
  $.notifier = function( options )
  {
    createNotification( ( fn ) => {
      fn( options );  
    });
  };

  function createNotification( fnResult )
  {
    // first let's check browser support
    if( !( 'Notification' in window ) ) {
      throw new Error( 'Browser doesn\'t support notifications' );
    }
    // now let's look whether permission is already granted or not
    switch( Notification.permission ) {
      case 'granted': {
        fnResult( innerCreateNotification );
        return innerCreateNotification;
      }
      case 'denied': {
        break;
      }
      case 'default': {
        Notification.requestPermission( ( permission ) =>  {
          if( permission === 'granted' ) {
            fnResult( innerCreateNotification );
            return innerCreateNotification;
          }
        } );
      }
    }

    function innerCreateNotification( notificationProps )
    {
      var notification = new Notification( notificationProps.title, notificationProps );
      notification.onclick = function( event )  {
        event.preventDefault();
      };

      if( Number.isInteger( notificationProps.closeTimeout ) ) {
        setTimeout( () =>  {
          notification.close();
        }, notificationProps.closeTimeout );
      }
    }
  
  }


}( window.jQuery );
/*
 *  ACCESS 5.0 Web Framework
 *  MENU
 */

!function( $ )
{

/* ---------------------------------------------------------------------------
 *   MENU - class definition
 * ------------------------------------------------------------------------ */

class WfcTreemenu extends JqControl
{
  // ----------------------------------------------------
  //   init-function (called by constructor)
  //
  OnInit( $element, options )  
  {
    // class variables
    this.$element = $element;
    this.$error = null;
    this.options = $.extend( true, [], $.fn.treemenu.defaults, options );

    // language
    this.EncodeLang( );

  }

  // ----------------------------------------------------
  //   language
  //
  EncodeLang( )
  {
    this.options['tooltip'] = $.lang( this.options['tooltip'] );
    this.options['text'] = $.lang( this.options['text'] );
    this.options['error'] = $.lang( this.options['error'] );
  }
  
  // ----------------------------------------------------
  //   draw (called by constructor && options )
  //
  OnDraw( )
  {
    this.$element.empty( );
    this.$error = null;

    // set sizes
    if( 'size' in this.options ) {
      let size = this.ControlCalculateSize( this.options['size'] );
      this.$element.css( 'width', size );
    }
    if( 'height' in this.options ) {
      if( this.options['height'] == "fit" ) {
        this.FitControlHeight();
      }
      else if( this.options['height'] > 0 ) {
        this.$element.css( 'height', this.lineHeight * this.options['height'] + 3 );
      }
      else {
        let height = this.ControlCalculateHeight( this.options['height'] );
        this.$element.css( 'height', height );  //this.options['height'] );
      }
    }

    // width (size in character or percent)
    //this.$element.css( 'width', this.ControlCalculateSize( this.options['size'] ) );
    this.$element.addClass( "normal" );

    if( this.options['border'] == true ) {
      this.$element.addClass( 'control-border' );
    }

    //this.$element.addClass( this.options['style'] );
    //if( this.options.size > 0 )  {
    //  this.$element.css( 'width', this.options.size * $.metrics( 'fontWidth' ) );
    //}
    this.$menu = $( "<div class='treemenu'></div>" );
    this.$element.append( this.$menu );
    switch( this.options['type'] )  {
      case 'accordion' : 
        this.OnDrawAccordion( this.$menu, this.options.data );
        break;
    }

  }

  // ----------------------------------------------------
  //   accordion
  //
  OnDrawAccordion( $menu, menuData )
  {
    var selfMenu = this;
    buildRecursive( $menu, menuData, 0, '' );

    //---------------------------------------------------
    //  local function, recurse the menu structure
    //
    function buildRecursive( $parent, data, level, path )  {
      
      for( var i = 0; i < data.length; i++ ) {

        //-----------------------------------------------
        //  prequisites
        //-----------------------------------------------

        // --- get data & normalize it ---
        var item = data[ i ];
        var levelClass = "l-" + level;
        item['path'] = path + '/' + item['id'];
        if( !( 'tooltip' in item ) )  {
          item['tooltip'] = "";
        }
        
        //-----------------------------------------------
        //  build html
        //-----------------------------------------------

        // --- draw separator ---
        switch( item['type'] )  {
          case 'separator': 
            var $item = selfMenu.ItemCreate( "/lib/treemenu/accordion/separator", item );
            $item.addClass( 'separator ' + levelClass );
            break;
          
          case 'head': 
            item['status'] = item['status'] ? item['status'] + " hilight" : 'hilight';
            var $item = selfMenu.ItemCreate( "/lib/treemenu/accordion/head", item );            
            $item.addClass( 'head ' + levelClass );
            break;

          case 'subitem':
            item['status'] = item['status'] ? item['status'] + " light" : 'light';
            var $item = selfMenu.ItemCreate( "/lib/treemenu/accordion/subitem", item );
            $item.addClass( 'subitem ' + levelClass );
            break;

          case 'group':
            item['status'] = item['status'] ? item['status'] + " normal expanded" : 'normal expanded';
            var $item = selfMenu.ItemCreate( "/lib/treemenu/accordion/item", item );
            $item.addClass( 'group ' + levelClass );
            break;

          default: // 'item':  
            item['status'] = item['status'] ? item['status'] + " normal" : 'normal';
            var $item = selfMenu.ItemCreate( "/lib/treemenu/accordion/item", item );
            $item.addClass( item['type'] + " " + levelClass );
          break;
        }
        
        // manage current selection
        if( selfMenu.ItemHasStatus( $item, 'selected' ) )  {
          selfMenu.options['selected'] = item['id'];
        }
        else if( selfMenu.options['selected'] == item['id'] )  {
          selfMenu.ItemAddStatus( $item, "selected" );
        }

        // append and click handler
        $parent.append( $item );
        $item.click( function( e )  {
          var $item = $( e.currentTarget );
          var itemData = selfMenu.ItemGetData( $item );
          selfMenu.OnClickItem( $item, itemData, e );
        } );
    
        //-----------------------------------------------
        //  draw submenu
        //-----------------------------------------------

        if( 'children' in item ) {
          var $group = $( $.views.Html( "/lib/treemenu/accordion/group-container", item ) );
          $group.addClass( 'group-container ' + levelClass );
          if( 'status' in item && item['status'].indexOf( 'collapsed' ) >= 0 )  {
            $group.addClass( 'hidden' );
          }
          $group.append( buildRecursive( $group, item['children'], level + 1, item['path'] ) );
          $parent.append( $group );
        }
      
      }//for

      return $parent;
    
    }//function
  
  } 

  // ----------------------------------------------------
  //   event: OnClickItem
  //
  OnClickItem( $item, itemData, e )
  {
    if( itemData['disabled'] != true  )  {
      
      // group 
      if( itemData['type'] == 'group' )  {
        var status = this.ItemToggleStatus( $item, ['collapsed', 'expanded'] );
        var $groupContainer = $item.next( '[data-group-id="' + itemData['id'] + '"]' );
        if( $groupContainer.length )  {
          if( status == 'expanded' ) 
            $groupContainer.removeClass( 'hidden' );
          else 
            $groupContainer.addClass( 'hidden' );
        }
      }
      // event to ui 
      if( $.isFunction( this.options['click'] ) )  {

        // if parent allows to change selection 
        if( this.options['click']( itemData['id'], itemData['path'], itemData, e ) != false )  {
          this.SetSelection( itemData['id'] );
        }
      }
    }
}

  // ----------------------------------------------------
  //   selection
  //
  SetSelection( docId )
  {
    // single selection
    if( typeof docId == 'string' )  {
      // remove selection from old
      var itemData = this.ListGetItem( this.options['selected'] );
      if( itemData )  {
        this.ItemRemoveStatus( itemData.$item, 'selected' );
      }
      // set selection to new
      var itemData = this.ListGetItem( docId );
      if( itemData ) {
        this.ItemAddStatus( itemData.$item, 'selected' );
        this.options['selected'] = docId;
      }
    }
  }


  // ----------------------------------------------------
  //   option getter and setter
  //
  option( args )  
  {
    // setter
    if( args.length > 2 ) {
      if( args[ 1 ] == 'selected' ) {
        this.SetSelection( args[ 2 ] );
      }
      else {
        this.options[ args[ 1 ] ] = $.clone( args[ 2 ] );
        this.OnDraw( );
      }
    }
    // getter
    return this.options[ args[ 1 ] ];
  }

}

/* ---------------------------------------------------------------------------
  *   MENU - plugin definition
  * ------------------------------------------------------------------------ */


// ----------------------------------------------------
//   plugin main function
//
$.fn.treemenu = function( option )  
{
  var data    = $( this ).data( "ctrl" ),
      options = ( typeof option == "object" && option );

  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data ) {
    var instance = new WfcTreemenu( this, options );
    $( this ).data( "ctrl", instance );
  }

  // if called with a string, treat this as a function call
  else if( typeof option == "string" ) {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) ) {
      return data[ option ]( arguments );
    }
    else {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.treemenu.defaults = {
  size: -1,
  border: false,
  data: [ ],
  type: '',
  status: "normal",
  statusDefault: "normal",
  selection: 'single',  // [ 'single', 'none', 'multiple' ]
  selected: null,
  click: null
};


}( window.jQuery );
/*
 *  Views 2 - Web Framework
 *  BREADCRUMPS
 */

!function( $ )
{

/* ---------------------------------------------------------------------------
 *   BREADCRUMPS - class definition
 * ------------------------------------------------------------------------ */

class WfcBreadcrumps extends JqControl
{
  // ----------------------------------------------------
  //   init-function (called by constructor)
  //
  OnInit( $element, options )  
  {
    // class variables
    this.$element = $element;
    this.$error = null;
    this.options = $.extend( true, [], $.fn.breadcrumps.defaults, options );
    this.EncodeLang( );
  }

  // ----------------------------------------------------
  //   encode texts 
  //
  EncodeLang()
  {
    this.options['text'] = $.lang( this.options['text'] );
  }

  // ----------------------------------------------------
  //   draw (called by constructor && options )
  //
  OnDraw( )
  {
    var selfCtrl = this;
    var hasHome = false;

    // if given a path and item Data callback, prepare data struct
    if( 'path' in this.options && $.isFunction( this.options.itemData ) )  {
      var pathList = this.options['path'].split( '/' );
      this.options['data'] = [ ];
      for( let i = 0; i <pathList.length; i++ )  {
        if( pathList[ i ].length > 0 )  {
          let itemData = this.options.itemData( pathList[ i ] );
          this.options['data'].push( itemData );
        }
      }
    }

    this.$container = $( $.views.Html( "/lib/breadcrumps/container" ) );
    this.$element.empty();
    this.$element.append( this.$container );

    // width (size in character or percent)
    this.$element.css( 'width', this.ControlCalculateSize( this.options['size'] ) );

    // first add home-button (if applied)
    if( this.options['home'] )  {
      
      // build
      var options = { icon: this.options['home'] };
      var $itemHome = this.ItemCreate( "/lib/breadcrumps/home", options );
      this.$container.append( $itemHome );
      hasHome = true;

      // click handlers
      $itemHome.click( function ( e )  {
        let $item = $( this );
        let itemData = selfCtrl.ItemGetData( $item );
        selfCtrl.OnClick( $item, itemData, e );  
      } );

    }

    // then iterate through the data
    var path = "";
    for( let i = 0; i < this.options.data.length; i++ )  {
      var itemData = this.options.data[ i ];
      path = path + '/' + itemData['id'];
      itemData['path'] = path;
      if( hasHome || i > 0 )  {
        var $chevron = $( $.views.Html( "/lib/breadcrumps/chevron" ) );
        this.$container.append( $chevron );
      }

      // create item
      var $item = this.ItemCreate( "/lib/breadcrumps/button", itemData );
      this.$container.append( $item );

      // click handlers
      $item.click( function ( e )  {
        var $item = $( this );
        var itemData = selfCtrl.ItemGetData( $item );
        selfCtrl.OnClick( $item, itemData, e );  
      } );

    }

  }

  // ----------------------------------------------------
  //   event handler
  //
  OnClick( $item, itemData, e )
  {
    if( ( itemData['disabled'] != true  )
     && ( $.isFunction( this.options['click'] ) ) 
      )  {
      this.options['click']( itemData['id'], itemData['path'], itemData, e );
    }
  }


  // ----------------------------------------------------
  //   selection
  //
  SetSelection( docId )
  {
  }


}

/* ---------------------------------------------------------------------------
  *   MENU - plugin definition
  * ------------------------------------------------------------------------ */


// ----------------------------------------------------
//   plugin main function
//
$.fn.breadcrumps = function( option )  
{
  var data = $( this ).data( "ctrl" ),
  options  = ( typeof option == "object" && option );

  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data ) {
    var cTab = new WfcBreadcrumps( this, options );
    $( this ).data( "ctrl", cTab );
  }

  // if called with a string, treat this as a function call
  else if( typeof option == "string" ) {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }    
    // call function in class
    else if( $.isFunction( data[ option ] ) )  {
      return data[ option ]( arguments );
    }
    else  {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.breadcrumps.defaults = {
  home: null,
  size: -1,
  data: [ ],
  status: "normal",
  statusDefault: "normal",
  click: null,
  itemData: null
};


}( window.jQuery );
 /*
 *  ACCESS 5.0 Web Framework
 *  button 
 */

!function( $ ) {

 
/* ---------------------------------------------------------------------------
  *   BUTTON - class definition
  * ------------------------------------------------------------------------ */

class WfcButton extends JqControl
{
  // ----------------------------------------------------
  //   constructor (will be called by base class)
  //
  OnInit( $element, options )
  {
    // class variables
    this.options  = $.extend( true, [], $.fn.button.defaults, options );

    // get button text from html, if not set
    if( !this.options['text'] )  {
      this.options['text'] = this.$element.text( );
    }

    this.$indicator = null;

    // language
    this.EncodeLang();

  }

  // ----------------------------------------------------
  //   encode texts 
  //
  EncodeLang()
  {
    this.options['text'] = $.lang( this.options['text'] );
    this.options['error'] = $.lang( this.options['error'] );
    this.options['warning'] = $.lang( this.options['warning'] );
    this.options['tooltip'] = $.lang( this.options['tooltip'] );
  }

  // ----------------------------------------------------
  //   recalc layout after change
  //
  OnDraw( )  
  {
    // check
    if( this.$element.length == 0 )  {
      console.warn( "Try to initialize a control, html-tag not found! Abort." );
      return;
    }

    // distinguish button types
    if( this.options['type'] == "icon" )  {
      this.OnDrawTypeIcon();
    }
    else if( ( this.options['type'] == "indicator" ) || ( 'indicator' in this.options ) ) {
      this.OnDrawTypeIndicator();
    }
    else  {
      this.OnDrawTypeStandard();
    }

    this.OnDrawError();

    // add event handler
    this.$element.off();
    this.$element.click( ( e ) =>  {
      let itemData = this.ItemGetData( this.$element );
      this.OnClick( this.$element, itemData, e );
      e.preventDefault();
      e.stopPropagation();
    } );

  }

  // ----------------------------------------------------
  //   draw standard button
  //
  OnDrawTypeStandard()
  {
    // normalize css and status
    this.$element.addClass( 'button' );
    this.ItemInitialize( this.$element, this.options );
    var itemData = this.ItemGetData( this.$element );

    //  width + height
    this.$element.css( 'width', this.ControlCalculateSize( this.options['size'] ) );
    if( 'height' in this.options ) {
      this.$element.css( 'height', this.ControlCalculateHeight( this.options['height'] * 1.5 ) );
    }

    // border
    if( this.options['border'] !== false ) {
      this.$element.addClass( "control-border" );
    }

    // -- tooltip --
    if( ! itemData )  {
      console.log( "hey???" );
    }
    if( typeof itemData['tooltip'] == "string" ) {
      this.$element.attr( 'title', this.options['tooltip'] );
    }
    else {
      this.$element.removeAttr( 'title' );
    }

    // do we need to add a dropdown icon?
    let isDropDownIcon = this.options['dropdownicon'] === true;

    if( itemData['text'] && itemData['icon'] ) {
      let htmlTemplate = '/lib/button/iconText' + ( isDropDownIcon ? "-dd" : "" );
      let innerHtml = $.views.Html( htmlTemplate, itemData );
      this.$element.html( innerHtml );
    }
    else if( itemData['icon'] ) {
      let htmlTemplate = '/lib/button/icononly' + ( isDropDownIcon ? "-dd" : "" );
      let innerHtml = $.views.Html( htmlTemplate, itemData );
      this.$element.html( innerHtml );
    }
    else {
      let htmlTemplate = '/lib/button/textonly' + ( isDropDownIcon ? "-dd" : "" );
      let innerHtml = $.views.Html( htmlTemplate, itemData );
      this.$element.html( innerHtml );
    }

  }//OnDrawTypeStandard

  // ----------------------------------------------------
  //   draw standard button
  //
  OnDrawTypeIcon()
  {
    // normalize css and status
    this.$element.addClass( 'button' );
    this.ItemInitialize( this.$element, this.options );
    var itemData = this.ItemGetData( this.$element );

    //  width + height
    this.$element.css( 'width', this.ControlCalculateSize( this.options['size'] ) );
    if( 'height' in this.options )  {
      this.$element.css( 'height', this.ControlCalculateHeight( this.options['height'] * 1.5 ) );
    }

    // border
    if( this.options['border'] !== false )  {
      this.$element.addClass( "control-border" );
    }

    // -- tooltip --
    if( typeof itemData['tooltip'] == "string" ) {
      this.$element.attr( 'title', this.options['tooltip'] );
    }
    else {
      this.$element.removeAttr( 'title' );
    }
    
    // do we need to add a dropdown icon?
    let isDropDownIcon = this.options['dropdownicon'] === true;

    // special button type "icon" only
    let htmlTemplate = '/lib/button/icon' + ( isDropDownIcon ? "-dd" : "" );
    let innerHtml = $.views.Html( htmlTemplate, itemData );
    this.$element.html( innerHtml );

    if( 'height' in this.options ) {
      let height = this.ControlCalculateHeight( this.options['height'] * 1.5 );
      this.$element.css( 'font-size', height );
      this.$element.css( 'line-heigth', height );
    }

  }//OnDrawTypeIcon

  // ----------------------------------------------------
  //   draw status button
  //
  OnDrawTypeIndicator()
  {
    // normalize css and status
    this.$element.addClass( 'button-indicator' );
    let innerHtml = $.views.Html( "/lib/button/indicator", this.options );
    this.$element.html( innerHtml );
    
    this.ItemInitialize( this.$element, this.options );
    var itemData = this.ItemGetData( this.$element );

    //  width + height
    this.$element.css( 'width', this.ControlCalculateSize( this.options['size'] ) );
    if( 'height' in this.options ) {
      this.$element.css( 'height', this.ControlCalculateHeight( this.options['height'] * 1.5 ) );
    }

    // border
    if( this.options['border'] !== false ) {
      this.$element.addClass( "control-border" );
    }

    // -- tooltip --
    if( typeof itemData['tooltip'] == "string" ) {
      this.$element.attr( 'title', this.options['tooltip'] );
    }
    else {
      this.$element.removeAttr( 'title' );
    }

    // --- status indicator ---
    if( !( 'indicator' in this.options ) ) {
      this.options['indicator'] = "default";
    }

    this.$indicator = this.$element.find( ".indicator" );
    this.$indicator.addClass( this.options['indicator'] );


  }//OnDrawTypeStatus

  // ----------------------------------------------------
  //   additional: error
  //
  OnDrawError()
  {
    //if( this.options.error ) {
      return this.ControlShowMessage( this.$element, this.options.error, this.options.warning, this.options.errorAnchor, "none" );
    //}
  }

  // ----------------------------------------------------
  //   after ("option","status", ... ) check if indicator
  //   need follow
  //
  IndicatorFollowStatus( status )
  {
    // preconditions
    if( status == null && 'statusDefault' in this.options ) {
      status = this.options['statusDefault'];
    }
    else if( status == null )  {
      return;
    }

    // indicator status should follow specific states
    if( status == "disabled" )  {
      this.$indicator.attr( "class", "indicator disabled" );
    }
    else if( status.indexOf( "selected" ) >= 0 )  {
      this.$indicator.attr( "class", "indicator selected" );
    }
    else  {
      this.$indicator.attr( "class", "indicator " + this.options['indicator'] );
    }

  }

  // ----------------------------------------------------
  //   event handler
  //
  OnClick( $item, itemData, e )
  {
    if( ( itemData['disabled'] != true  )
     && ( $.isFunction( this.options['click'] ) ) 
      )  {
      this.options['click']( itemData['id'], itemData, e );
    }
  }


  // ----------------------------------------------------
  //   option: generic getter and setter
  //
  option( args )  
  {
    // setter
    if( args.length > 2 ) {
      switch( args[ 1 ] ) {
        case 'status': {
          this.options['status'] = this.ItemSetStatus( this.$element, args[ 2 ] );
          if( this.$indicator )  {
            this.IndicatorFollowStatus( args[ 2 ] );
          }
        }
        break;

        case 'modified': {
          this.options['modified'] = args[ 2 ];
        }
        break;

        case 'error': {
          this.options[ args[ 1 ] ] = args[ 2 ];
          this.OnDraw();
        }
        break;

        default: {
          this.options[ args[ 1 ] ] = args[ 2 ];
          this.OnDraw();
        }
      }
    }
    // getter
    return this.options[ args[ 1 ] ];
  }


}

   
/* ---------------------------------------------------------------------------
 *   BUTTON - plugin definition
 * ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   plugin main function
//
$.fn.button = function( option )  
{
  var data    = $( this ).data( "ctrl" ),            
      options = ( typeof option == "object" && option );  
  
  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data )  {
    var instance = new WfcButton( this, options ); 
    $( this ).data( "ctrl", instance );
  }
  
  // if called with a string, treat this as a function call
  else if( typeof option == "string" )  {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) )  {
      return data[ option ]( arguments );
    }
    else  {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;   
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.button.defaults = {
  size: 0,
  //height: 1.5,
  //border: true,
  type: "standard",   // "icon" 
  dropdownicon: false,
  text: null,
  error: null,
  warning: null,
  tooltip: null,
  status: "normal",
  icon: null,
  click: $.noop
};

  
}( window.jQuery );
  /*
 *  ACCESS 5.0 Web Framework
 *  edit control
 */

!function( $ ) {

/* ---------------------------------------------------------------------------
*   EDIT - const values
* ------------------------------------------------------------------------ */

// define keycodes
var KEY = {
  BS:     8,
  TAB:    9,
  RETURN: 13,
  ENTER:  13,
  SHIFT:  16,
  CTRL:   17,
  ALT:    18,
  SCTRL:  20,
  ESC:    27,
  SPACE:  32,
  END:    35,
  POS1:   36,
  LEFT:   37,
  UP:     38,
  RIGHT:  39,
  DOWN:   40,
  DEL:    46,
  F1:    112,
  F2:    113,
  F3:    114,
  F4:    115,
  F5:    116,
  F6:    117,
  F7:    118,
  F8:    119,
  F9:    120,
  F10:   121,
  F11:   122,
  F12:   123,
};

/* ---------------------------------------------------------------------------
*   EDIT - class definition
* ------------------------------------------------------------------------ */

class WfcEdit extends JqControl  
{
  // ----------------------------------------------------
  //   first create control
  //
  OnInit( $element, options )
  {
    // class member variable
    this.$input = null;
    this.$icon = null;
    this.$error = null;

    // class variables
    this.options = $.extend( true, [], $.fn.edit.defaults, options );
    this.$element.addClass( 'edit v-align-middle no-hover' );

    // language
    this.EncodeLang( );

  }

  // ----------------------------------------------------
  //   language
  //
  EncodeLang( )
  {
    this.options['tooltip'] = $.lang( this.options['tooltip'] );
    this.options['placeholder'] = $.lang( this.options['placeholder'] );
    this.options['error'] = $.lang( this.options['error'] );
  }

  // ----------------------------------------------------
  //   recalc layout after change
  //
  OnDraw( )
  {
    // tabula rasa
    this.$element.empty( );
    this.$error = null;

    if( ( $.isFunction( $.registry ) )
     && ( typeof this.options.regkey == "string" )
      ) {
      let regvalue = $.registry( this.options.regkey );
      if( regvalue && this.options.value == null && regvalue.length > 0 ) {
        this.options.value = regvalue;
      }
    }

    // normalize css and status
    this.ItemInitialize( this.$element, this.options );
    if( this.options['border'] == true ) {
      this.$element.addClass( 'control-border' );
    }

    // width (size in character or percent)
    this.$element.css( 'width', this.ControlCalculateSize( this.options['size'] ) );

    // get normalized item data
    var itemData = this.ItemGetData( this.$element );

    // append input & set value
    if( this.options['password'] == true )  {
      this.$input = $( "<input type='password' spellcheck='false' autocomplete='new-password'>" );
    }
    else  {
      let uid = $.uuid();
      this.$input = $( `<input type='text' id='${ uid }' spellcheck='false' autocomplete='${ uid }'>` );
    }
    this.$element.append( this.$input );
    this.$input.val( itemData['value'] );

    // placeholder
    if( itemData['placeholder'] )  {
      this.$input.attr( 'placeholder', itemData['placeholder'] );
    }

    // icon
    if( itemData['icon'] )  {
      this.$icon = $( "<i class='" +  itemData['icon'] + "' >" );
      this.$element.append( this.$icon );
    }
    else  {
      this.$icon = null;
    }

    // macros & event-handlers
    this._handleMacroMode( 'focusout' );
    this._attachEvents( );

    // error overlay?
    this.OnDrawError( );
    this.OnStatusChanged( itemData['status'] );

  }

  // ----------------------------------------------------
  //   additional: error
  //
  OnDrawError( )
  {
    return this.ControlShowMessage( this.$element, this.options.error, this.options.warning, this.options.errorAnchor );
  }
  
  // ----------------------------------------------------
  //   called by base class if status has been changed
  //
  OnStatusChanged( statusList )
  {
    if( this.$input == null )  {
      return;
    }

    // spacial handling for default status
    if( statusList == null && ( typeof this.options['statusDefault'] == "string" ) ) {
      statusList = this.options['statusDefault'];
    }

    if( statusList.indexOf( 'disabled') >= 0 )  {
      this.$input.prop( 'readonly', null );
      this.$input.prop( 'disabled', 'disabled' );
      this.$input.addClass( 'disabled' );
      this.$input.removeClass( 'readonly' );
    }
    else if( statusList.indexOf( 'readonly') >= 0 )  {
      this.$input.prop( 'readonly', 'readonly' );
      this.$input.prop( 'disabled', null );
      this.$input.addClass( 'readonly' );
      this.$input.removeClass( 'disabled' );
    }
    else  {
      this.$input.prop( 'readonly', null );
      this.$input.prop( 'disabled', null );
      this.$input.removeClass( 'readonly' );
      this.$input.removeClass( 'disabled' );
    }
  }

  // ----------------------------------------------------
  //   set focus to the edit control
  //
  SetFocus( )
  {
    if( this.$input == null ) {
      return;
    }
    this.$input.focus();
  }

  // ----------------------------------------------------
  //   overridable: Validate 
  //
  OnValidate( eventName )
  {
    // do validation (delegate to upper layer)
    let value = this.$input.val();
    return this.ManageValidate( value, eventName );
  }


  // ----------------------------------------------------
  //   attach/detach the keyboard change event
  //
  _attachEvents( )
  {
    var self = this;

    // key up event for filter/verify
    this.$input.keydown( function ( e )  {
      
      // enable F5 reload browser
      if( e.which == KEY.F5 )  {
        location.reload();
      }

      if( e.which != KEY.BS
       && e.which != KEY.TAB
       && e.which != KEY.RETURN
       && e.which != KEY.ENTER
       && e.which != KEY.SHIFT
       && e.which != KEY.CTRL
       && e.which != KEY.ALT
       && e.which != KEY.SCTRL
       && e.which != KEY.ESC
       && e.which != KEY.END
       && e.which != KEY.POS1
       && e.which != KEY.LEFT
       && e.which != KEY.RIGHT
       && e.which != KEY.UP
       && e.which != KEY.DOWN
       && e.which != KEY.DEL
        ) {
        return self._handleFilter( e.key );
      }
    } );

    // key up event for filter/verify
    this.$input.keyup( function( e ) {

      // enter & return
      if( e.which == KEY.ENTER
       || e.which == KEY.RETURN
        ) {
        document.activeElement.blur();
      }

      // special characters
      if( e.which != KEY.SHIFT
       && e.which != KEY.CTRL
       && e.which != KEY.SCTRL
       && e.which != KEY.ALT
       && e.which != KEY.ESC
       && e.which != KEY.POS1
       && e.which != KEY.END
       && e.which != KEY.LEFT
       && e.which != KEY.RIGHT
       && e.which != KEY.UP
       && e.which != KEY.DOWN
        ) {
        let value = $( this ).val();
        self._handleValueChanged(  value, 'change' );

        // delegate change event to upper layer
        self.ManageValidate( value, "change" );
        self.ManageModified( 1 );
      }

    } );


    // clipboard
    this.$input.on( 'paste', function ( e ) {
      let pasteData = e.originalEvent.clipboardData.getData( 'text' );
      let pasteData2 = pasteData;
      if( self.options.filterRegEx )  {
        pasteData2 = pasteData.replace( self.options.filterRegEx, "" );
      }
      if( pasteData != pasteData2 ) {
        var element = this;
        setTimeout( function( ) {
          $( element ).val( pasteData2 );
        }, 150 );
        return false;
      }
    } );
    
    // focusout for validation and ctrl+shift      
    this.$input.focusout( function( e )  {
      self.options.value = $( this ).val();
      self.options.focus = false;
      if( self.options.trimResult == true ) {
        self.options.value = self.options.value.trim();
        if( !( 'macros' in self.options ) ) {
          $( this ).val( self.options.value );
        }
      }
      self._handleMacroMode( "focusout" );
      self._handleValueChanged( self.options.value, 'focusout' );
      self.option( [ null, 'changed' ] );   // refresh

      if( $.isFunction( self.options.result ) )  {
        var escapedValue = self._value( self.options.value, false );
        self.options.result( escapedValue );
      }
      self.$element.removeClass( 'focus' );

    });
    
    // focusin for validation and ctrl+shift      
    this.$input.focusin( function(e)  {
      if( 'spellcheck' in  self.$element[ 0 ] )  {
        self.$input[ 0 ].spellcheck = false;
      }
      
      self.options.focus = true;
      self._handleMacroMode( "focusin" );
      self._handleValueChanged( $( this ).val(), 'focusin' );

      self.$element.addClass( 'focus' );
    } );

    if( this.$icon )  {
      this.$icon.on( 'click', $.proxy( this._iconClick, this ) );
    }
    
  }

  // ----------------------------------------------------
  //   handle clicked icon
  //
  _iconClick( )
  {
    if( $.isFunction( this.options.click ) )  {
      this.options.value = this.$input.val( );
      this.options.click( this.options.value );
    }
  }

  // ----------------------------------------------------
  //   in password mode switch input-type
  //
  _handleFilter( char )
  {
    var isAllowed = true;

    if( this.options.filter != null )  {
      if( this.options.filter.allow != null ) {
        isAllowed = ( this.options.filter.allow.indexOf( char ) >= 0 );
      }
      if( this.options.filter.deny != null ) {
        var isDenied = ( this.options.filter.deny.indexOf( char ) >= 0 );
        if( isDenied )  {
          isAllowed = false;
        }
      }
    }

    return isAllowed;  // result
  }

      
  // ----------------------------------------------------
  //   recalc layout after change
  //
  _handleValueChanged( value, event )  
  {
    if( this.$element.prop( 'disabled') || this.$element.prop( 'readonly' ) )
      return; 
    if( value == this.options.init )  {
      return;
    }

    // -- error marker --
    //var iconHandled = false; 
    if( $.isFunction( this.options.verify ) )  {
      
      //var focusEvent = ( event=='focusin' || event=='focusout' );  
      if( ( this.options.verifyEvent == event ) 
        || ( event=='focusout' ) 
        || ( this.options.verifyEvent=='focus' && event=='focusin' )
        )  {    
        var error = this.options.verify( value );
        if( typeof ( error ) == "string" )  {
          this.options.error = $.lang( error );
          this.OnDrawError( );
        }
        else  {
          this.options.error = null;
          this.OnDrawError( );
        }
      }
    } 
    
    if( event != 'focusout'  &&  event != 'focusin'  &&  $.isFunction( this.options.change ) )  {
      this.options.value = value;
      var escapedValue = this._value( this.options.value, false );
      this.options.change( escapedValue );
    }

    if( ( $.isFunction( $.registry ) )
     && ( typeof this.options.regkey == "string" )
      ) {
      $.registry( this.options.regkey, this.options.value );
    }
    
  }
  
  // ----------------------------------------------------
  //   macro mode
  //
  _handleMacroMode( event )
  {
    // if there's a macro mode
    if( 'macros' in this.options ) {
      
      // in focus out event the value will be set back to the 
      // original values. Here we scan for the key and replace
      // them with the source macros
      var value = this.$input.val();
      if( event == 'focusout' ) {
        for( var key in this.options.macros ) {
          value = value.replace( key, this.options.macros[ key ] );
        }
      }
      else  {
        value = this.options['value'];
      }

      this.$input.val( value );
      //this.OnDraw();
      return this.options.value;
    }
  }

  // ----------------------------------------------------
  //   escape characters 
  //
  _value( value, isEscaped )
  {
    var replaceList = this.options.escapeChar;
    if( replaceList == null )
      return value;
      
    // setter
    if( !isEscaped ) {
      let result = value;
      for( let i = 0; i < replaceList.length; i++ )  {
        let pattern = replaceList[ i ][ 0 ];
        let rx = new RegExp( pattern.replace( /\\/g, '\\\\' ), 'g' );
        result = result.replace( rx, replaceList[ i ][ 1 ] );
      }
      return result;
    }
    else  {
      let result = value;
      for( let i = 0; i < replaceList.length; i++ )  {
        let pattern = replaceList[ i ][ 1 ];
        let rx = new RegExp( pattern.replace( /\\/g, '\\\\' ), 'g' );
        result = result.replace( rx, replaceList[ i ][ 0 ] );
      }
      return result;
    }
  }


  // ----------------------------------------------------
  //   option getter and setter
  //
  option( args )  
  {
    // setter
    if( args.length > 2 )  {
      switch( args[ 1 ] )  {
        case 'value' : {
          this.options.value = this._value( args[ 2 ], true );
          this.OnDraw();
        }break;
        
        case 'error': {
          this.options[ args[ 1 ] ] = $.lang( args[ 2 ] );
          this.OnDrawError();
        } break;

        case 'modified': {
          this.options[ args[ 1 ] ] = args[ 2 ] ;
        } break;

        case 'status':  {
          let desiredStatus = args[ 2 ];
          if( desiredStatus == null )  {
            desiredStatus = this.options['statusDefault'];
          }
          this.options[ args[ 1 ] ] = desiredStatus;
          this.OnDraw();
        } break;

        default:  {
          this.options[ args[ 1 ] ] = args[ 2 ];
          this.OnDraw();
        }break;
      }
      
    }
    // getter (value)
    else if( args[ 1 ] == 'value' ) {
      if( 'macros' in this.options )  {
        return this.options.value;
      }
      return this.$input.val();
    }
    // getter
    return this.options[ args[ 1 ] ];
  }
  
  // ----------------------------------------------------
  //   focus can't be set by container. 
  //
  focus( args )  
  {
    // if focus is set in result() handler of control it'll 
    // be overwritten. Set a timeout as solution.
    setTimeout( () => { 
      this.SetFocus(); 
    }, 0 );
  }
  
}

/* ---------------------------------------------------------------------------
*   EDIT - plugin definition
* ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   plugin main function
//
$.fn.edit = function ( option )  {

  var data    = $( this ).data( "ctrl" ),            
      options = ( typeof option == "object" && option );  
  
  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy( );
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data )  {
    var instance = new WfcEdit( this, options ); 
    $( this ).data( "ctrl", instance );
  }
  
  // if called with a string, treat this as a function call
  else if( typeof option == "string" )  {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy( );
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) ) {
      return data[ option ]( arguments );
    }
    else {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;   
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.edit.defaults = {
  size: 20,
  border: true,
  value: null,
  icon: null,
  tooltip: null,
  placeholder: null,
  password: false,
  status: "normal",
  statusDefault: "normal",
  regkey: null,
  verifyEvent: 'focusout', //'change',
  verify: null,
  error: null,
  errorAnchor: null,
  trimResult: true,
  escapeChar: null,
  /*escapeChar: [ 
    [ '\\', '\\x5C' ],
    [ '\"', "\\x22" ],
    [ '>', "\\x3E" ],
    [ '<', "\\x3C" ],
  ],*/
  //macros: { '$pi' : "3.14" },
  filter: null,
  filterRegEx: null,
  //filter: { allow: null, deny: '\"\<\>\'\\' },
  //filterRegEx: /[\"\<\>\\']/g,
  trackChange: false,
  focus: false, 
  result: null,
  change: null,
  click: null,
  validate: null
};


}( window.jQuery );
 /*
 *  ACCESS 5.0 Web Framework
 *  choice (radio oder droplist)
 */

!function( $ ) {

/* ---------------------------------------------------------------------------
*   CHOICE - const values
* ------------------------------------------------------------------------ */



/* ---------------------------------------------------------------------------
*   CHOICE - class definition
* ------------------------------------------------------------------------ */

//var WfcChoice = function( element, options ) {
//  this.init( element, options );
//};
//WfcChoice.prototype = {

class WfcChoice extends JqControl  
{
  // ----------------------------------------------------
  //   first create control
  //
  OnInit( $element, options )
  {
    // class member variable
    this.fontSize = { cx: $.metrics( 'fontWidth' ), cy: $.metrics( 'fontHeight' ) };
    this.select2 = { };

    // class variables
    this.options = $.extend( true, [], $.fn.choice.defaults, options );
    this.options['statusDefault'] = ( this.options['statusDefault'] || "" ) + " no-hover";
    this.EncodeLang( );
    
  }    
      
  // ----------------------------------------------------
  //   encode texts 
  //
  EncodeLang()
  {
    this.options['text'] = $.lang( this.options['text'] );
    if( Array.isArray( this.options['data'] ) )  {
      this.options['data'] = $.lang( this.options['data'] );
    }
  }
  
  // ----------------------------------------------------
  //   recalc layout after change
  //
  OnDraw( )  
  {
    if( this.options['type'] == 'radio' )  {
      return this.OnDrawRadiogroup( );
    }
    else if( this.options['type'] == 'check' && 'data' in this.options )  {
      return this.OnDrawCheckgroup( );
    }
    else if( this.options['type'] == 'check' )  {
      return this.OnDrawCheckbox( );
    }
  }


  // ----------------------------------------------------
  //   recalc if type is checkbox
  //
  OnDrawCheckbox( )
  {
    var self = this;

    // tabula rasa
    this.$element.empty( );
    this.$element.addClass( 'checkbox no-hover' );
    this.$error = null;

    // get normalized item data
    this.ItemInitialize( this.$element, this.options );
    //var itemData = this.ItemGetData( this.$element );      

    // normalize css and status, prequisites
    this.$fset = $( $.views.Html( '/lib/choice/checkgroup', this.options ) );
    this.$element.append( this.$fset );

    var id = this.$element.attr( 'id' );
    var idname = id + "_GRP";

    // build dom        
    this.options['ctrlid']   = idname;
    this.options['ctrlname'] = idname;
    var $item = $( $.views.Html( '/lib/choice/checkbox', this.options ) );
    var $input = this.ItemInitialize( $item, this.options );      
    this.$fset.first().append( $input );

    if( this.options['value'] == true )  {
      $input.prop( 'checked', true );
      this.options.value = true;
    }
    else  {
      this.options.value = false;
    }

    // common status
    if( this.options['status'] == "readonly" || this.options['status'] == "disabled" )  {
      $input.attr( 'disabled', true );        
    }
    
    // handle error overlay below
    this.OnDrawError( );

    $input.change( function( ) {
      self.options['value'] = $( this ).prop( 'checked' );
      self._HandleValueChanged( self.options['value'], "change" );
      if( $.isFunction( self.options['result'] ) )  {
        self.options['result']( self.options.value );
      }
    });

  }

  // ----------------------------------------------------
  //   recalc if type is checkbox
  //
  OnDrawCheckgroup( )
  {
    var self = this;

    // tabula rasa
    this.$element.empty( );
    this.$element.addClass( 'checkgroup no-hover' );
    this.$error = null;

    // normalize css and status, prequisites
    this.ItemInitialize( this.$element, this.options );
    var idctrl = this.$element.attr( 'id' );
    var idgroup = id + "_GRP";

    // add container to element
    this.$fset = $( $.views.Html( '/lib/choice/checkgroup', this.options ) );
    this.$element.append( this.$fset );
    this.options.value = { };

    // items
    for( var i = 0; i < this.options['data'].length; i++ )  {
      
      // build dom        
      var id = this.options['data'][ i ]['id'];
      this.options['data'][ i ]['ctrlid']   = idctrl + "_"+ i.toString( );
      var $item = $( $.views.Html( '/lib/choice/checkbox', this.options['data'][ i ] ) );
      var $input = this.ItemInitialize( $item, this.options['data'][ i ] );
      if( i > 0 ) {
        $input.addClass( 'checkgroup-top-spacer' );
      }
      
      // status of input tag
      if( this.options['data'][ i ]['value'] || this.options['data'][ i ]['status'] == 'checked' )  {
        $input.prop( 'checked', true );
        this.options.value[ id ] = true;
      }
      else  {
        this.options.value[ id ] = false;
      }
      if( this.options['data'][ i ]['status'] == "disabled" 
       || this.options['status'] == "disabled" 
       || this.options['status'] == "readonly" 
        ) {
        $input.attr( 'disabled', true );        
      }
      
      // append to <fieldset>
      this.$fset.first( ).append( $input );

      $input.change( function( ) {
        var isCheck = $( this ).prop( 'checked' );
        var name = $( this ).prop( 'name' );
        self.options.value[ name ] = isCheck;
        self._HandleValueChanged( self.options['value'], "change" );
        if( $.isFunction( self.options['result'] ) )  {
          self.options['result']( self.options.value );
        }
      } );        
              
    }
    
    // handle error overlay below
    this.OnDrawError( );
    
  }


  // ----------------------------------------------------
  //   recalc if type is droplist
  //
  OnDrawRadiogroup( )
  {
    var self = this;

    // tabula rasa
    this.$element.empty( );
    this.$element.addClass( 'radiogroup no-hover' );
    this.$error = null;

    // width (size in character or percent)
    this.$element.css( 'width', this.ControlCalculateSize( this.options['size'] ) );


    // normalize css and status, prequisites
    this.ItemInitialize( this.$element, this.options );
    var id = this.$element.attr( 'id' );
    var idgroup = id + "_GRP";
    let isHorizontal = ( this.options['layout'] == 'horizontal' );
    var itemLayout = isHorizontal ? '/lib/choice/radioitem-v' : '/lib/choice/radioitem';

    // add container to element
    this.$fset = $( $.views.Html( '/lib/choice/radiogroup', this.options ) );
    this.$element.append( this.$fset );

    // items
    for( var i = 0; i < this.options['data'].length; i++ )  {
      
      // build dom        
      this.options['data'][ i ]['ctrlid']   = id + "_"+ i.toString( );
      this.options['data'][ i ]['groupid']  = idgroup;
      var $item = $( $.views.Html( itemLayout, this.options['data'][ i ] ) );
      var $input = this.ItemInitialize( $item, this.options['data'][ i ] );
      if( i > 0 && !isHorizontal )  {
        $input.addClass( 'radiogroup-top-spacer' );
      }
      // status of input tag
      if( this.options['data'][ i ]['id'] == this.options['value'] )  {
        $input.prop( 'checked', true );
      }
      if( this.options['data'][ i ]['status'] == "disabled" 
       || this.options['status'] == "disabled" 
       || this.options['status'] == "readonly" 
        ) {
        $input.attr( 'disabled', true );   
      }
      
      // append to <fieldset>
      this.$fset.first( ).append( $input );
              
    }
    
    // handle error overlay below
    this.OnDrawError( );
    
    // select item requested in 'value'
    $( 'input[type=radio][name=' + idgroup + ']', this.$element ).change( function( ) {
      self.options['value'] = $( this ).val();
      self._HandleValueChanged( self.options['value'], "change" );
      if( $.isFunction( self.options['result'] ))  {
        self.options['result']( self.options['value'] );          
      }
    });
  }

  // ----------------------------------------------------
  //   additional: error
  //
  OnDrawError( )
  {
    return this.ControlShowMessage( this.$element, this.options.error, this.options.warning, this.options.errorAnchor );
  }    
  
  // ----------------------------------------------------
  //   called by base class if status has been changed
  //
  OnStatusChanged( statusList, itemData )
  {
    //console.log( "EDIT: status changed", statusList, this );
    if( this.$input == null )  {
      return;
    }

    if( statusList.indexOf( 'disabled') >= 0 )  {
      this.$input.prop( 'readonly', null );
      this.$input.prop( 'disabled', 'disabled' );
      this.$input.addClass( 'disabled' );
      this.$input.removeClass( 'readonly' );
    }
    else if( statusList.indexOf( 'readonly') >= 0 )  {
      this.$input.prop( 'readonly', 'readonly' );
      this.$input.prop( 'disabled', null );
      this.$input.addClass( 'readonly' );
      this.$input.removeClass( 'disabled' );
    }
    else  {
      this.$input.prop( 'readonly', null );
      this.$input.prop( 'disabled', null );
      this.$input.removeClass( 'readonly' );
      this.$input.removeClass( 'disabled' );
    }
  }           

  // ----------------------------------------------------
  //   validate
  //
  OnValidate( eventName )
  {
    return this.ManageValidate( this.options['value'], eventName );
  }  
  
  // ----------------------------------------------------
  //   helper: Value changed
  //
  _HandleValueChanged( value, eventName )
  {
    // delegate change event to upper layer
    this.ManageValidate( value );
    this.ManageModified( 1 );
  }

  // ----------------------------------------------------
  //   option getter and setter
  //
  option( args )  
  {
    // setter
    if( args.length > 2 )  {
      switch( args[ 1 ] )  {
        case 'status': {
          if( ( typeof args[ 2 ] == "string" )
           && ( args[ 2 ].indexOf( "disabled" ) >= 0 )
            ) {
            this.ManageDisabledOverlay( true );
          }
          else if( args[2] == null 
            && ( 'statusDefault' in this.options )
            && ( this.options['statusDefault'].indexOf( "disabled" ) >= 0 )
            ) {
            this.ManageDisabledOverlay( true );
          }
          else {
            this.ManageDisabledOverlay( false );
            this.options['status'] = this.ItemSetStatus( this.$element, args[ 2 ] );
          }
        }
        break;

        case 'value': {
          this.options['value'] = $.lang( args[ 2 ] );
          this.OnDraw( );
        }
        break;

        case 'data': {
          this.options['data'] = $.clone( args[ 2 ] );
          this.EncodeLang();
          this.OnDraw();
        }
        break;

        case 'error': {
          this.options['error'] = $.lang( args[ 2 ] );
          this.OnDrawError( );
        }
        break;

        default: 
          this.options[ args[1] ] = $.lang( args[ 2 ] );
          this.OnDraw( );
      }
    }
    // getter
    return this.options[ args[1] ];
  }

 
}


/* ---------------------------------------------------------------------------
*   CHOICE - plugin definition
* ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   plugin main function
//
$.fn.choice = function ( option )  {

  var data    = $( this ).data( "ctrl" ),            
      options = ( typeof option == "object" && option );  
  
  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    $( this ).off( );
    $( this ).data( "ctrl", null );
  }
  
  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data )  {
    var instance = new WfcChoice( this, options ); 
    $( this ).data( "ctrl", instance );
  }
  
  // if called with a string, treat this as a function call
  else if( typeof option == "string" )  {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) ) {
      return data[ option ]( arguments );
    }
    else {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;   
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.choice.defaults = {
  type: 'radio',
  layout: 'vertical',
  value: '',
  text: '',
  status: "normal",
  result: $.noop,
  error: null,
  errorAnchor: null

};


}( window.jQuery );
/*
 *  Views 2 - Web Framework
 *  LISTBOX
 */

!function( $ )
{

  // define keycodes
  var KEY = {
    BS:     8,
    TAB:    9,
    RETURN: 13,
    ENTER:  13,
    SHIFT:  16,
    CTRL:   17,
    ALT:    18,
    SCTRL:  20,
    ESC:    27,
    SPACE:  32,
    END:    35,
    POS1:   36,
    LEFT:   37,
    UP:     38,
    RIGHT:  39,
    DOWN:   40,
    DEL:    46
  };

/* ---------------------------------------------------------------------------
 *   LISTBOX - class definition
 * ------------------------------------------------------------------------ */

class WfcListbox extends JqControl
{
  // ----------------------------------------------------
  //   init-function (called by constructor)
  //
  OnInit( $element, options )  
  {
    // class variables
    this.options = $.extend( true, [], $.fn.listbox.defaults, options );
    this.options['statusDefault'] = ( this.options['statusDefault'] || "" ) + " no-hover";

    this.EncodeLang();

    this.isSharedDataset = false;
    this.selected = [ ];
    this.selectIndex = -1;
    this.$droplist = null;
    this.$scroll = null;
    this.$search = null;
    this.$input = null;
    this.searchString = ""; 
    this.isSelectionSet = false;
    this.isPopupOpen = false;
    this.metrics = {
      itemHeight: $.metrics( 'lineHeight' ),      // height of one item
      listHeigth: 0,                              // height of complete item-list
      searchHeight: 0
    };
    this.fireResultOnChange = false;
    this.index2Id = [];
    this.pinnedAsLastItem = "";

    //  Dataset/View - Handling
    let dataViewOptions = {};
    if( 'sort' in this.options )        dataViewOptions['sortOrder'] = this.options['sort'];
    if( 'sortKey' in this.options )     dataViewOptions['sortKey'] = this.options['sortKey'];
    if( 'dataPinned' in this.options )  dataViewOptions['pinned'] = this.options['dataPinned'];

    // a dataset/dataview needs an id, so if the element doesn't have one create a guid
    if( $element[0].id == "" )  {
      $element.attr( "id", $.guidgen() );
    }

    if( Array.isArray( this.options.data ) )  {
      this.dataset = new Dataset( { id: this.options.dataId }, this.options.data );
      this.dataview = this.dataset.BuildView( 'default', dataViewOptions );
      this.dataview.Silent( true );
      this.isSharedDataset = false;
    }
    else if( this.options.data instanceof Dataset ) {
      this.dataset = this.options.data;
      this.dataview = this.dataset.BuildView( 'default', dataViewOptions );
      this.dataview.Silent( true );
      this.isSharedDataset = true;
    }
    else if( this.options.data instanceof Dataview )  {
      this.dataview = this.options.data;
      if( 'sortOrder' in this.options || 'dataPinned' in this.options )  {
        this.dataview.Options( dataViewOptions );
      }
      this.isSharedDataset = true;
    }
    // IMPORTANT: for garbage collection of removed DOM-Elements
    this.dataview.AttachDomElement( this.$element );

    // language
    this.EncodeLang();
  }


  // ----------------------------------------------------
  //   language
  //
  EncodeLang( )
  {
    this.options['text'] = $.lang( this.options['text'] );
    this.options['placeholder'] = $.lang( this.options['placeholder'] );
    this.options['searchPlaceholder'] = $.lang( this.options['searchPlaceholder'] );
    this.options['error'] = $.lang( this.options['error'] );
  }


  // ----------------------------------------------------
  //   draw listbox 
  //
  OnDraw( )
  {
    switch( this.options['type'] )  {
      case 'listbox':  {   
        return this.OnDrawListbox( );  
      }
      case 'droplist':  {   
        return this.OnDrawDroplist( );  
      }      
      default:  {
        console.warn( "LISTBOX: unknown listbox type'" + this.options['type'] + "'" );
      }
      break;
    }
  }

  // ----------------------------------------------------
  //   draw listbox 
  //
  OnDrawListbox( )
  {
    var selfCtrl = this;
    var heightLines = 0;

    //--------------------------------------------
    //  fire result on every click
    //--------------------------------------------

    this.fireResultOnChange = true;

    //--------------------------------------------
    //  DRAW
    //--------------------------------------------

    // build container
    this.$element.empty();
    this.$error = null;

    this.$container = $( $.views.Html( "/lib/listbox/container" ) );
    this.$element.append( this.$container );
    this.$element.addClass( 'listbox normal no-hover' ) ;
    if( this.options['border'] == true ) {
      this.$element.addClass( "control-border" );
    }
    if( this.options['focus'] == true ) {
      this.$element.prop( 'tabindex', '-1' );
    }

    // width (size in character or percent)
    this.$element.css( 'width', this.ControlCalculateSize( this.options['size'] ) );

    // attach frame 
    this.ItemInitialize( this.$element, { } );

    // get inner (scrollable) tag & draw data
    this.$content = this.$element.find( '.scrollable' ); 
    this.metrics['itemHeight'] =  this._CalculateItemHeight( this.$content );
    this.metrics['searchHeight'] = this._CalculateSearchHeight( this.$content );

    // search bar
    this.OnDrawSearch();

    // set height (will be corrected in OnDrawSearch & popup code)
    if( Number.isInteger( this.options['height'] ) )  {
      if( this.options['anchor'] )  {
        heightLines = ( this.dataview.length < this.options['height'] ) ? this.dataview.length : this.options['height'];
        this.$element.css( 'height', heightLines * this.metrics['itemHeight'] + this.metrics['searchHeight'] );
      }
      else if( this.options['height'] == -1 )  {
        this.$element.css( 'height', "100%" );
      }
      else  {
        heightLines = this.options['height'];
        this.$element.css( 'height', heightLines * this.metrics['itemHeight'] + this.metrics['searchHeight'] );        
      }
    }
    // just set the height with given percent
    else if( this.options['height'].includes( '%' ) )  {
      this.$element.css( 'height', this.options['height']  );
    }

    // scrollbar
    this.$scroll = this.$element.find( ".scroll-container" );
    if( this.$scroll.length == 1 ) {
      this.$scroll.scrollbar( { resize: false } );
      this.$scroll.css( 'height', heightLines * this.metrics['itemHeight'] );
    }

    // disabled? 
    if( this.options['status'] == "disabled" && !( 'anchor' in this.options ) ) {
      this.ManageDisabledOverlay( true );
    }
    
    //--------------------------------------------
    //  DATA-HANDLER
    //--------------------------------------------

    this.dataview.On( 'rebuild', function( ) {
      selfCtrl.OnDrawData( );
    } );
    this.dataview.On( 'delete', function( ) {
      selfCtrl.OnDrawData();
    } );


    //-----------------------------------------------------
    //  popup code
    if( this.options['anchor'] ) {
      
      this.fireResultOnChange = false;

      this.$anchor = $( this.options['anchor'] );
      if( this.$anchor.length > 0 ) {
        this.$element.addClass( 'hidden' );
        this.$element.addClass( 'no-focus' );
        this.$element.css( 'position', 'fixed' );
        this.$element.css( 'z-index', '101' );
        //this.$element.detach().appendTo( this.$anchor );
      }
      else {
        console.error( "Anchor" + this.options['anchor'] + " not found" );
      }
    }
    else  {
      // emit a 'rebuild' event
      this.dataview.Silent( false );
    }

    // handle error overlay below
    this.OnDrawError();

    //--------------------------------------------
    //  EVENTS
    //--------------------------------------------

    // focus-out handling
    this.$element.focusout( function( e )  {
      if( !this.contains( e.relatedTarget ) )  {
        if( selfCtrl.isPopupOpen )  {
          selfCtrl._HideDroplist( );
        } 
        selfCtrl.OnClickClearSearch( );
        selfCtrl.OnLeaveFocus();
      }
    } );    
    // focus-in handling
    this.$element.focusin( function( e )  {
      if( !this.contains( e.relatedTarget ) )  {
        selfCtrl.OnEnterFocus( );
      }
    } );
    // keyboard handling
    this.$element.on( 'keydown', function ( e )  {
      selfCtrl.OnKeydown( e.which, e );
    } );    
    
    
  }

  // ----------------------------------------------------
  //   draw drop list
  //
  OnDrawDroplist( )
  {
    var selfCtrl = this;

    this.fireResultOnChange = false;

    //--------------------------------------------
    //  PREQUISITES
    //--------------------------------------------

    if( this.options['selection'] != 'single' 
     && this.options['selection'] != 'singleplus' 
      )  {
      console.warn( 'DROPLIST: selection must be \'single\' or \'singleplus\'' );
      this.options['selection'] = 'single';
    }

    //--------------------------------------------
    //  DRAW 
    //--------------------------------------------

    // container
    this.$element.empty();
    this.$error = null;

    this.$container = $( $.views.Html( "/lib/listbox/droplist/container", this.options ) );
    this.$element.append( this.$container );
    this.$element.addClass( 'droplist no-hover' ) ;
    this.$element.prop( 'tabindex', '0' ) ;   
    this.ItemInitialize( this.$element, this.options );
    this.$droplist = this.$element.find( '.popup' );
    this.$head = this.$element.find( '.head' );
    this.$head.css( 'height', this.$element.innerHeight( ) ) ;
    

    // get inner (scrollable) tag & draw data
    this.$content = this.$element.find( '.scrollable' ); 

    this.metrics['itemHeight'] = this._CalculateItemHeight( this.$content );
    this.metrics['searchHeight'] = this._CalculateSearchHeight( this.$content );

    // width (size in character or percent)
    this.$element.css( 'width', this.ControlCalculateSize( this.options['size'] ) );

    // search bar
    this.OnDrawSearch( );
    
    // create scrollbar
    this.$scroll = this.$element.find( ".scroll-container" );
    if( this.$scroll.length == 1 )  {
      this.$scroll.scrollbar( { resize: false } );       
    } 
    this.$scroll.css( 'height', this.options['height']  * this.metrics['itemHeight'] );


    // show collapsed state
    this._HideDroplist();
    this._UpdateDroplistHead();

    // disabled? 
    if( this.options['status'] == "disabled" )  {
      this.ManageDisabledOverlay( true );
    }

    //--------------------------------------------
    //  DATA-HANDLER
    //--------------------------------------------

    this.dataview.On( 'rebuild', function( ) {
      selfCtrl.OnDrawData( );
      selfCtrl._UpdateDroplistHead();
    } );
    this.dataview.On( 'delete', function( ) {
      selfCtrl.OnDrawData();
      selfCtrl._UpdateDroplistHead();
    } );

    // handle error overlay below
    this.OnDrawError();

    //--------------------------------------------
    //  EVENT-HANDLER
    //--------------------------------------------

    // show
    this.$head.click( function( e )  {
      selfCtrl._ShowDroplist( );
      selfCtrl._InitDroplist( );
      selfCtrl.$element.addClass( 'focus' );
      e.stopPropagation();
    } );
    // focus handling
    this.$element.focusout( function( e )  {
      if( !this.contains( e.relatedTarget ) ) {
        if( selfCtrl.isPopupOpen )  {
          selfCtrl._HideDroplist( );
          //selfCtrl._FireResult( );
          selfCtrl._DeInitDroplist( );
        }
        selfCtrl.OnClickClearSearch();
        selfCtrl.OnLeaveFocus();
      }
    } );    
    this.$element.focusin( function( e )  {
      selfCtrl.OnEnterFocus( );
      if( !selfCtrl.isPopupOpen ) {
        selfCtrl._ShowDroplist( );
        selfCtrl._InitDroplist( );
      }
    } );    
    // keyboard handling
    this.$element.on( 'keydown', function ( e )  {
      selfCtrl.OnKeydown( e.which, e );
    } );        
  }  

  // ----------------------------------------------------
  //   click item (droplist) event handler
  //  
  OnDrawData( )
  {
    //console.log( "OnDrawData" );

    // prepare
    this.$content.empty( );
    this.metrics['listHeight'] = 0;
    let $item = null;
    this.index2Id = [];

    if( this.dataview.length <= 0 )  {
      this.dataview.ItemClearSelections();
    }

    // draw pinned items
    if( this.options['dataPinned'].length > 0 )  {
      for( let pinnedIndex = 0; pinnedIndex < this.dataview.length; pinnedIndex++ ) {
        let record = this.dataview.GetByIndex( pinnedIndex );
        if( this.options['dataPinned'].indexOf( record['_preview']['id'] ) >= 0 ) {
          $item = this.OnDrawItem( record, pinnedIndex );
        }
      }
    }

    // draw diverter if needed
    if( $item )  {
      let itemData = $item.data( "itemData" );
      this.dataview.ItemAddStatus( itemData.id, "diverter" );
      this.pinnedAsLastItem = itemData.id;
    }

    // draw all other items 
    for( let index = 0; index < this.dataview.length; index++ )  {
      // get rec & skip pinned
      let record = this.dataview.GetByIndex( index );
      if( this.options['dataPinned'].indexOf( record['_preview']['id'] ) >= 0 )  {
        continue;
      }
      this.OnDrawItem( record, index );
    }

    // placeholder
    if( this.dataview.length <= 0 )  {
      this.OnDrawPlaceholder( );
    }

    // select & scroll
    if( !this.isSelectionSet )  {
      this.dataview.Select( this.options.selected );
      this.isSelectionSet = true;
    }
    if( this.$scroll )  {
      this.$scroll.scrollbar( "recalc" );
    }    

    if( this.dataview.length == 0 )  {
      this._FireResult();
    }

  }

  // ----------------------------------------------------
  //   draw item and set handler
  //
  OnDrawItem( record, index )
  {
    var selfCtrl = this;
    let isCustomDraw = false;
    let $item = null;
    let preview = record['_preview'];

    if( $.isFunction( this.options['render'] ) ) {
      preview = this.options['render']( record, preview );
    }

    if( $.isFunction( this.options.drawItem ) ) {
      isCustomDraw = true;
      $item = $( $.views.Html( "/lib/listbox/customitem", preview ) );
      let $content = this.options.drawItem( record, $item );
      $item.html( $content );
    }
    else {
      $item = $( $.views.Html( "/lib/listbox/item", preview ) );
    }
    let status = ( 'status' in record ) ? record['status'] + ' normal hover' : 'normal hover';
    let itemData = this.dataview.AttachItem( index, $item, { status: status } );
    this.$content.append( $item );
    itemData.y = this.metrics['listHeight'];
    this.index2Id.push( itemData['id'] );

    this.metrics['itemHeight'] = $item.outerHeight();
    this.metrics['listHeight'] = this.metrics['listHeight'] + this.metrics['itemHeight'];

    // disable icon (if not set)
    if( !isCustomDraw && !preview['icon'] ) {
      $item.find( "i" ).css( 'display', 'none' );
    }

    $item.click( function( e ) {
      itemData = $( this ).data( 'itemData' );
      selfCtrl.OnClickItem( itemData, e.shiftKey );
      e.stopPropagation();
      if( !e.shiftKey ) {
        selfCtrl._HideDroplist();
      }
    } );

    $item.dblclick( function( e ) {
      itemData = $( this ).data( 'itemData' );
      selfCtrl.OnDblClickItem( itemData, e.shiftKey );
      e.stopPropagation();
    } );

    return $item;
  }

  // ----------------------------------------------------
  //   search input
  //
  OnDrawSearch( )
  {
    var selfCtrl = this;
    
    if( this.options.search === true )  {
      
      var $element = null;
      if( this.$droplist )  {
        $element = this.$droplist;
      }
      else  {
        $element = this.$element;
      }

      // build search
      this.$search = $( $.views.Html( "/lib/listbox/search", this.options ) );
      $element.prepend( this.$search );
      this.metrics['searchHeight'] = this.$search.outerHeight( );

      // input field & button
      this.$input = this.$search.find( 'input' );
      this.$searchButton = this.$search.find( 'i' );

      // otherwise a parent with attribut 'tabindex' would get 
      // the click-event and get the focus
      this.$input.click( function( e ) {
        e.stopPropagation();
      } );

      // click event
      this.$searchButton.click( function( e ) {
        selfCtrl.OnClickClearSearch( );
      } );

      // keyboard handling
      this.$element.on( 'keyup', function ( e )  {
        selfCtrl.OnKeyup( e );
      } );        

    }
  }

  // ----------------------------------------------------
  //   called if dataview is empty
  //
  OnDrawPlaceholder()
  {
    if( ( this.options['type'] == "listbox" )
     && ( typeof this.options['placeholder'] == "string" )
      ) {
      var $placeholderOverlay = $( $.views.Html( "/lib/listbox/placeholder", this.options ) );
      let controlHeight = this.$element.innerHeight();
      $placeholderOverlay.css( "height", controlHeight );
      this.$content.append( $placeholderOverlay );
    }
  }

  // ----------------------------------------------------
  //   click item event handler
  //
  OnClickItem( itemData, isShift )
  {
    //console.log( "OnClickItem");
    if( !this.dataview.ItemHasStatus( itemData.id, 'disabled' ) )  {
      this._SelectItem( itemData['id'], isShift );
      this._FireResult();
    }
  }

  // ----------------------------------------------------
  //   double click item event handler
  //
  OnDblClickItem( itemData, isShift )
  {
    if( !this.dataview.ItemHasStatus( itemData.id, 'disabled' ) ) {
      this._SelectItem( itemData['id'], isShift );
      if( $.isFunction( this.options['dblclick'] ) ) {
        this.options['dblclick']( itemData['id'] );
      }
    }
  }

  // ----------------------------------------------------
  //   enter focus event handler
  //
  OnEnterFocus( )
  {
    if( this.$scroll )  {
      this.$scroll.scrollbar( "recalc" );
    }
    this.$element.addClass( 'focus' );
  }

  // ----------------------------------------------------
  //   enter focus event handler
  //
  OnLeaveFocus( )
  {
    this.$element.removeClass( 'focus' );
  }

  // ----------------------------------------------------
  //   keyboard handler
  //
  OnKeydown( keycode, e )
  {
    let selected = this.dataview.ItemGetSelected();
    let selectedIndex = this.index2Id.findIndex( ( elem ) => selected[0] == elem );
  
    if( keycode == KEY.UP )  {
      if( selectedIndex > 0 )  {
        var newIndex = selectedIndex - 1;
        let newSelectedId = this.index2Id[ newIndex ];
        var itemData = this.dataview.GetItem( newSelectedId );
        this._SelectItem( itemData.id, false, true );
        var itemHeight = itemData.$item.outerHeight( );
        this.$scroll.scrollbar( "show", itemData.y, itemData.y + itemHeight );
        e.preventDefault( );
        if( this.fireResultOnChange ) {
          this._FireResult();
        }
      }
    }
    else if( keycode == KEY.DOWN )  {
      if( selectedIndex < ( this.dataview.length - 1 ) )  {
        var newIndex = selectedIndex + 1;
        let newSelectedId = this.index2Id[ newIndex ];
        var itemData = this.dataview.GetItem( newSelectedId );
        this._SelectItem( itemData.id, false, true );
        var itemHeight = itemData.$item.outerHeight( );
        this.$scroll.scrollbar( "show", itemData.y, itemData.y + itemHeight );
        e.preventDefault( );
        if( this.fireResultOnChange ) {
          this._FireResult();
        }
      }
    }
    else if( keycode == KEY.END ) {
      let newSelectedId = this.index2Id[ this.index2Id.length-1 ];
      var itemData = this.dataview.GetItem( newSelectedId );
      this._SelectItem( itemData.id, false, true );
      var itemHeight = itemData.$item.outerHeight();
      this.$scroll.scrollbar( "show", itemData.y, itemData.y + itemHeight );
      e.preventDefault();
      if( this.fireResultOnChange ) {
        this._FireResult();
      }
    }
    else if( keycode == KEY.POS1 ) {
      let newSelectedId = this.index2Id[ 0 ];
      var itemData = this.dataview.GetItem( newSelectedId );
      this._SelectItem( itemData.id, false, true );
      var itemHeight = itemData.$item.outerHeight();
      this.$scroll.scrollbar( "show", itemData.y, itemData.y + itemHeight );
      e.preventDefault();
      if( this.fireResultOnChange ) {
        this._FireResult();
      }
    }
    else if( keycode == KEY.ENTER || keycode == KEY.RETURN )  {
      this._FireResult( );
      this._HideDroplist( );
      this._DeInitDroplist( );
    }
    else if( keycode == KEY.ESC )  {
      if( this.searchString.length > 0 )  { //this.$input.is( ":focus") )  {
        this.OnClickClearSearch( );
      }
      else  {
        this._HideDroplist( );
        this._DeInitDroplist( );
        this._UpdateDroplistHead( );
      }
    }
    else if( this.$input )  {
      if( !this.$input.is( ":focus") )  {
        this.$input.focus();
      }
    }

  }

  // ----------------------------------------------------
  //   keyboard: search edit
  //
  OnKeyup( e )
  {
    if( this.$input ) {
      var newSearch = this.$input.val( );
      if( newSearch != this.searchString )  {
        this.searchString = newSearch; 
        this.dataview.Search( this.searchString );
        var $icon = this.$search.find( 'i' );
        if( newSearch.length == 0 )  {
          $icon.attr( 'class', this.options.seachIcon );
        }
        else  {
          $icon.attr( 'class', this.options.seachCloseIcon );
        }      
      }
    }
  }

  OnClickClearSearch( e )
  {
    if( this.$input ) {
      this.$input.val( '' );
      this.searchString = "";
      this.dataview.Search( null );
      var $icon = this.$search.find( 'i' );
      $icon.attr( 'class', this.options.seachIcon );
    }
  }

  // ----------------------------------------------------
  //   helper: close popup
  //
  _HideDroplist( )
  {
    if( this.$droplist )  {
      this.$droplist.css( 'display', 'none' );
      this._UpdateDroplistHead( );
      this.isPopupOpen = false;
      this.$element.blur( );
      this.$droplist.trigger( "event:unsubscribe-scroll" );
    }
    else if( this.$anchor )  {
      this.$element.addClass( 'hidden' );
      this.isPopupOpen = false;
      this.$element.blur( );
      this.$element.trigger( "event:unsubscribe-scroll" );
    }
  }

  // ----------------------------------------------------
  //   helper: close popup
  //
  _ShowDroplist( )
  {
    // do nothing if popup is already open
    if( this.isPopupOpen )
      return;

    // flush scrollbar to indicate the list is longer
    this.$scroll.scrollbar( "flush" );

    if( this.$droplist )  {
      this.$droplist.css( 'display', '' );
      let position = this.$element.offset();
      let elementHeight = this.$element.outerHeight();
      let elementWidth = this.$element.width();
      this.$droplist.css( "top", position.top + elementHeight );
      this.$droplist.css( "left", position.left );
      this.$droplist.css( "width", elementWidth );      
      this.isPopupOpen = true;

      this.$droplist.trigger( "event:subscribe-scroll" );
      this.$droplist.on( "event:scroll", ( ev, param ) => {
        this._HideDroplist();
      });

    }
    else if( this.$anchor )  {
      let anchorPos = this.$anchor.offset();
      let anchorWidth = this.$anchor.outerWidth();
      this.$element.removeClass( 'hidden' );
      let anchorHeight = this.$anchor.outerHeight();
      let elementWidth = this.$element.outerWidth();
      this.$element.css( "top", anchorPos.top + anchorHeight - 1 );
      if( 'position' in this.options && this.options['position'] == "bottomleft" )  {
        this.$element.css( "left", anchorPos.left );
      }
      else {
        this.$element.css( "left", anchorPos.left + anchorWidth - elementWidth );
      }

      this.$element.focus( );
      this.isPopupOpen = true;

      this.$element.trigger( "event:subscribe-scroll" );
      this.$element.on( "event:scroll", ( ev, param ) => {
        this._HideDroplist();
      } );

    }
  }

  // ----------------------------------------------------
  //   helper: init popup
  //
  _InitDroplist( )
  {
    //console.log( "InitDropList")

    if( this.$droplist )  {
      this.dataview.Update( );
      var height = ( this.dataview.length < this.options['height'] ) ? this.dataview.length : this.options['height'];
      this.$droplist.css( 'height', height * this.metrics['itemHeight'] + this.metrics['searchHeight'] );
      this.$scroll.css( 'height', height * this.metrics['itemHeight'] );
      if( this.$scroll )  {
        this.$scroll.scrollbar( "recalc" );
      }
    }
    else if( this.$anchor )  {
      this.dataview.Update( );
      var height = ( this.dataview.length < this.options['height'] ) ? this.dataview.length : this.options['height'];
      this.$element.css( 'height', height * this.metrics['itemHeight'] + this.metrics['searchHeight'] );
      if( this.$scroll )  {
        this.$scroll.scrollbar( "recalc" );
      }
    }    
    this.isSelectionSet = false;
  }

  _DeInitDroplist( )
  {
    if( this.$droplist || this.$anchor )  {
      this.dataview.Silent( true );
    }
    //this.isSelectionSet = false;
  }


  // ----------------------------------------------------
  //   helper: update droplist head
  //
  _UpdateDroplistHead( )
  {
    // get itemData for display head
    if( this.isPopupOpen )  {
      var selected = this.dataview.ItemGetSelected( );
    }
    else  {
      var selected = this.options.selected;
    }
    var record = this.dataview.Get( selected[ 0 ] );

    // icon
    if( record )  {
      var $icon = this.$element.find( '.icon' );
      if( record['_preview']['icon'] != null ) {
        $icon.attr( 'class', 'icon hover ' + record['_preview']['icon'] );
        $icon.css( 'display', "" );
      }
      else {
        $icon.attr( 'class', "" );
        $icon.css( 'display', 'none' );
      }
      // text
      var $text = this.$element.find( '.text' );
      $text.removeClass( 'disabled-text' );
      if( selected.length > 1 )  {
        $text.text( record['_preview']['text'] + " + " + ( selected.length - 1 ).toString( ) );
      }
      else  {
        $text.text( record['_preview']['text'] );    
      }
    }

    else if( ( this.options.selected.length > 0 ) && ( 'dataDeletedItem' in this.options ) ) {
      var $text = this.$element.find( '.text' );
      $text.removeClass( 'disabled-text' );
      $text.text( $.lang( this.options['dataDeletedItem']['text'] ) );

      var $icon = this.$element.find( '.icon' );
      $icon.attr( 'class', 'icon hover ' + this.options['dataDeletedItem']['icon'] );
      $icon.css( 'display', "" );

    }

    // no data -> placeholder
    else if( this.options['placeholder'] )  {
      var $icon = this.$element.find( '.icon' );
      $icon.css( 'display', 'none' );
      var $text = this.$element.find( '.text' );
      $text.addClass( 'disabled-text' );
      $text.text( $.lang( this.options['placeholder'] ) );
    }
  }


  // ----------------------------------------------------
  //   helper: select an item
  //
  _SelectItem( id, isShiftKey, forceSingle )
  {
    if( forceSingle )  {
      this.dataview.ItemClearSelections( );
      this.dataview.ItemSelect( id );
    }
    else if( ( this.options['selection'] == 'singleplus' && isShiftKey ) 
          || ( this.options['selection'] == 'multi' ) 
      ) {
      this.dataview.ItemToggleSelection( id, false );
    }
    else  {
      this.dataview.ItemClearSelections( );
      this.dataview.ItemSelect( id );
    }    

    // flush scrollbar to indicate
    this.$scroll.scrollbar( "flush" );

    if( this.options['type'] == 'droplist' )  {
      this._UpdateDroplistHead( );
    }

    //console.log( "_Select", id, this.dataview.ItemGetSelected() );
    this.selectIndex = this.dataview.GetIndexFromId( id );
  }

  // ----------------------------------------------------
  //   helper: select an item
  //
  _FireResult(  )
  {
    this.options.selected = this.dataview.ItemGetSelected( );
    this.options['value'] = this.options.selected;
    if( $.isFunction( this.options['result'] ) )  {
      this.options['result']( this.options.selected );
    }

    // delegate modified & validation
    this.ManageValidate( this.options['value'], "change" );
    this.ManageModified( 1 );

    //console.log( "_FireResult", this.options.selected );
  } 

  // ----------------------------------------------------
  //   test height for an item
  //  
  _CalculateItemHeight( $content )
  {
    var preview = { id: '1', icon: 'mdi mdi-home', text: 'TEST' };
    var $item = $( $.views.Html( "/lib/listbox/item", preview ) );
    $content.append( $item );
    var height = $item.outerHeight( );
    $item.remove();
    return height;
  }

  // ----------------------------------------------------
  //   test height for search bar
  //  
  _CalculateSearchHeight( $content )
  {
    if( this.options['search'] === true )  {
      var preview = { searchPlaceholder: '1', seachIcon: 'mdi mdi-home' };
      var $item = $( $.views.Html( "/lib/listbox/search", preview ) );
      $content.append( $item );
      var height = $item.outerHeight();
      $item.remove();
      return height;
    }
    return 0;
  }

  // ----------------------------------------------------
  //   additional: error
  //
  OnDrawError()
  {
    let errorPosition = this.options.errorPosition || "right";
    return this.ControlShowMessage( this.$element, this.options.error, this.options.warning, this.options.errorAnchor, errorPosition );
  }    

  // ----------------------------------------------------
  //   validate
  //
  OnValidate( eventName )
  {
    return this.ManageValidate( this.options['value'], eventName );
  }  


  // ----------------------------------------------------
  //   option: generic getter and setter
  //
  option( args )  
  {
    // setter
    if( args.length > 2 )  {
      switch( args[ 1 ] )  {
        case 'status': {
          if( 'anchor' in this.options ) {
            // do nothing
          }
          else if( ( typeof args[2] == "string" )
           && ( args[2].indexOf( "disabled" ) >= 0 )
            ) {
            this.ManageDisabledOverlay( true );
          }
          else if( args[2] == null
           && ( 'statusDefault' in this.options )
           && ( this.options['statusDefault'].indexOf( "disabled" ) >= 0 )
            ) {
            this.ManageDisabledOverlay( true );
          }
          else {
            this.ManageDisabledOverlay( false );
            this.options['status'] = this.ItemSetStatus( this.$element, args[2] );
          }
        }
        break;

        case 'data': {
          if( this.isSharedDataset )  {
            console.warn( "Listbox::option( 'data' ) - data must be updated in dataset!" );
          }
          else if( Array.isArray( args[ 2 ] ) ) {
            this.dataset.SetData( args[ 2 ] );
            //this.dataview.ItemClearSelections();
            this.OnDraw( );
            this.ManageValidate( args[ 2 ], "change" );
          }
          else  {
            console.warn( "Listbox::option( 'data' ) - given data must be array!" );
          }
        }
        break;

        case 'selected':  {
          this.isSelectionSet = false;
          if( typeof args[ 2 ] == "string" ) {
            this.options[ args[ 1 ] ] = [ args[ 2 ] ];
          }
          else {
            this.options[ args[ 1 ] ] = args[ 2 ];
          }
          this.options['value'] = this.options['selected'];
          this.dataview.Select( this.options['selected'] );
          this._SelectItem( this.options['selected'][ 0 ] );

          if( ! this.dataview.Has( this.options['selected'][ 0 ] ) ) {
            this.options['selected'] = [];
          }
        }
        break;
        
        case 'error': {
          this.options['error'] = $.lang( args[2] );
          this.OnDrawError();
        }
        break;
        
        default: {
          this.options[ args[ 1 ] ] = args[ 2 ];
          this.OnDraw( );
        }
      }
    }

    // getter
    if( args[ 1 ] == "data" )  {
      return this.dataset.GetData( );
    }
    if( args[ 1 ] == "dataset" ) {
      return this.dataset;
    }
    else  {
      return this.options[ args[ 1 ] ];
    }    
  }  


  // ----------------------------------------------------
  //   interface: show popup
  //  
  focus()
  {
    if( this.$search )  {
      this.$search.find( "input" ).focus();
    }
    else {
      this.$element.focus();
    }
  }

  // ----------------------------------------------------
  //   interface: show popup
  //  
  show( )
  {
    this._ShowDroplist( );
    this._InitDroplist( );
  }

  // ----------------------------------------------------
  //   interface: hide popup
  //  
  hide( )
  {
    this._HideDroplist( );
    this._DeInitDroplist( );
  }

}

/* ---------------------------------------------------------------------------
*   LISTBOX - plugin definition
* -------------------------------------------------------------------------- */


// ----------------------------------------------------
//   plugin main function
//
$.fn.listbox = function( option )  
{
  var data    = $( this ).data( "ctrl" ),
      options = ( typeof option == "object" && option );

  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.listbox( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }
  
  // if this plugin isn't installed, create one
  if( !data ) {
    var instance = new WfcListbox( this, options );
    $( this ).data( "ctrl", instance );
  }

  // if called with a string, treat this as a function call
  else if( typeof option == "string" ) {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) )  {
      return data[ option ]( arguments );
    }
    else  {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.listbox.defaults = {
  type: 'listbox',
  size: 25,
  height: 5,
  border: true,
  focus: true,
  status: "normal",
  selection: 'single',
  selected: [ ],
  selectedIndex: -1,
  data: [ ],
  dataId: 'id',
  dataPinned: [],
  sort: 'none',
  search: false,
  searchPlaceholder: "option 'searchPlaceholder'",
  seachIcon: 'mdi mdi-magnify',
  seachCloseIcon: 'mdi mdi-close',
  result: $.noop,
  dblclick: $.noop,
  drawItem: null,
  icon: null,
  placeholder: 'please select'
};


}( window.jQuery );
/*
 *  Views 2 - Web Framework
 *  BREADCRUMPS
 */

!function( $ )
{

/* ---------------------------------------------------------------------------
 *   BREADCRUMPS - class definition
 * ------------------------------------------------------------------------ */

class WfcToast extends JqControl
{
  // ----------------------------------------------------
  //   init-function (called by constructor)
  //
  OnInit( $element, options )  
  {
    // class variables
    this.$element = $element;
    this.timer1 = null;
    this.$toast = null;
    this.$error = null;
    this.transition = "none";
    this.options = $.extend( true, [], $.fn.toast.defaults, options );
    this.EncodeLang( );
  }

  // ----------------------------------------------------
  //   encode texts 
  //
  EncodeLang()
  {
    this.options['text'] = $.lang( this.options['text'] );
  }

  // ----------------------------------------------------
  //   draw (called by constructor && options )
  //
  OnDraw()
  {
    // prequisites
    this.destroy();

    // convert markdown text
    let slimdown = new Slimdown;
    this.options['text'] = slimdown.Render( this.options['text'] );
    //console.log( this.options['text'] );

    // create toast html
    this.$toast = this.ItemCreate( '/lib/toast/container', this.options );
    this.$element.append( this.$toast );
    this.ItemAddStatus( this.$toast, "no-hover" );

    // add click handler to link
    this.$links = this.$toast.find( ".link" );
    this.$links.click( ( ev ) => {
      if( $.isFunction( this.options['click'] ) ) {
        this.options['click']( ev.target.dataset['id'] );
      }
    } );

    // add close handler to link
    this.$closeButton = this.$toast.find( ".close-button" );
    this.$closeButton.on( "click", ( ev ) => {
      if( $.isFunction( this.options['close'] ) ) {
        if( this.options['close']( ) !== false )  {
          this.destroy();
        }
      }
      else {
        this.destroy();
      }
    } );


    // add transition if needed
    if( 'transition' in this.options ) {
      if( this.options['transition'] == "h" || this.options['transition'] == "hight" ) {
        this.$toast.addClass( 'toast-transition-height' );
        this.transition = "height";
      }
      else {
        this.$toast.addClass( 'toast-transition-width' );
        this.transition = "width";
      }
    }

    // --- transition height ---
    if( this.transition == "height" ) {
      this.$toast.css( 'width', this.ControlCalculateSize( this.options['size'] ) );

      // start css transition (async)
      setTimeout( () => {
        this.$toast.css( 'height', this.ControlCalculateHeight( this.options['height'] ) );
        this.$toast.css( 'margin-block-start', "1em" );
      }, 0 );

      // transition must not take longer tthen 250ms, trigger layout event after
      setTimeout( () => {
        this.TriggerLayoutEvent();
      }, 250 );


      // time 0 or < 0, stay forever
      if( this.options['time'] > 0 ) {
        this.timer1 = setTimeout( () => {
          this.$toast.css( 'height', "" );
          this.$toast.css( 'margin-block-start', "" );
          // after transition trigger layout event
          setTimeout( () => {
            this.TriggerLayoutEvent();
          }, 250 );

        }, this.options['time'] );
      }
    }

    // --- transition width ---
    else if( this.transition == "width" ) {
      this.$toast.css( 'height', this.ControlCalculateHeight( this.options['height'] ) );
      setTimeout( () => {
        this.$toast.css( 'width', this.ControlCalculateSize( this.options['size'] ) );
      }, 0 );

      // transition must not take longer tthen 250ms, trigger layout event after
      setTimeout( () => {
        this.TriggerLayoutEvent();
      }, 250 );


      // time 0 or < 0, stay forever
      if( this.options['time'] > 0 ) {
        this.timer1 = setTimeout( () => {
          this.$toast.css( 'width', "" );
          // after transition trigger layout event
          setTimeout( () => {
            this.TriggerLayoutEvent();
          }, 250 );
        }, this.options['time'] );
      }
    }

    // --- no transition ---
    else {
      this.$toast.css( 'width', this.ControlCalculateSize( this.options['size'] ) );
      this.$toast.css( 'height', this.ControlCalculateHeight( this.options['height'] ) );

      // inform upper layer about recalc document height
      this.TriggerLayoutEvent();

      // time 0 or < 0, stay forever
      if( this.options['time'] > 0 ) {
        this.timer1 = setTimeout( () => {
          this.$toast.css( 'width', "" );
          this.$toast.css( 'height', "" );
          this.TriggerLayoutEvent();
        }, this.options['time'] );
      }

    }


  }//draw

  destroy()
  {
    this.$element.empty( );
    clearTimeout( this.timer1 );

    // inform upper layer about recalc document height
    this.TriggerLayoutEvent();

  }

  option( args )
  {
    // do nothing
  }

}

/* ---------------------------------------------------------------------------
 *   MENU - plugin definition
 * ------------------------------------------------------------------------ */


// ----------------------------------------------------
//   plugin main function
//
$.fn.toast = function( option )  
{
  var data    = $( this ).data( "ctrl" ),
      options = ( typeof option == "object" && option );

  // remove for re-create
  if( typeof option == "object" && data )  {
    data.destroy( );
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data ) {
    var instance = new WfcToast( this, options );
    $( this ).data( "ctrl", instance );
  }

  // if called with a string, treat this as a function call
  else if( typeof option == "string" ) {
    // destroy outside of class
    if( option == "destroy" )  {
      data.destroy( );
      $( this ).off();
      $( this ).data( "ctrl", null );  
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) )  {
      return data[ option ]( arguments );
    }
    else  {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.toast.defaults = {
  size: -1,
  height: 3,
  status: "normal",
  statusDefault: "normal",
  position: "br",
  anchor: null,
  text: "",
  icon: null,
  click: null,
  close: null,
  time: 5000
};


}( window.jQuery );
/*
 *  CYTEL Viewable Framework
 *  TABS
 */

!function( $ )
{

/* ---------------------------------------------------------------------------
 *   BREADCRUMPS - class definition
 * ------------------------------------------------------------------------ */

class WfcTabs extends JqControl
{
  // ----------------------------------------------------
  //   init-function (called by constructor)
  //
  OnInit( $element, options )  
  {
    // class variables
    this.$element = $element;
    this.$container = null;

    this.options = $.extend( true, [], $.fn.tabs.defaults, options );
    this.EncodeLang( );
  }

  // ----------------------------------------------------
  //   encode texts 
  //
  EncodeLang()
  {
    this.options['text'] = $.lang( this.options['text'] );
    this.options['data'] = $.lang( this.options['data'] );
  }

  // ----------------------------------------------------
  //   draw (called by constructor && options )
  //
  OnDraw( )
  {
    // prequisites
    this.$element.empty();
    let item0 = null;
    let data = this.options['data'];
    this.scrollerLeftOffset = 0;
    this.scrollPosition = 0;

    // build container
    this.$container = $( $.views.Html( '/lib/tabs/container' ) );
    this.$element.append( this.$container );
    this.$element.css( "position", "relative" );

    // build items
    for( var i = 0; i < data.length; i++ ) {
      let item = data[ i ];
      if( i == 0 )  {
        item0 = item;
      }
      if( !( 'icon' in item ) )  {
        item['icon'] = "";
      }
      this.DrawItem( item );
    }

    // default selection is #0
    if( this.options['selected'] == null && item0 ) {
      this.SelectId( item0['id'], false );
    }

    // build scroll button left + right
    this.$scrollerLeft = this.ItemCreate( "/lib/tabs/scroller-left", {} );
    this.$element.append( this.$scrollerLeft );
    this.$scrollerRight = this.ItemCreate( "/lib/tabs/scroller-right", {} );
    this.$element.append( this.$scrollerRight );
    
    // show/hide scrollers
    this.ManageScrollersVisible();
    $( window ).resize( () => {
      this.ManageScrollersVisible();
    } );

    // scrollers click events
    this.$scrollerLeft.click( () => {
      this.OnScrollLeft();
    } );
    this.$scrollerRight.click( () => {
      this.OnScrollRight();
    } );

  }

  // ----------------------------------------------------
  //   draw (called by constructor && options )
  //
  DrawItem( item )
  {
    var selfCtrl = this;

    let htmlTemplate = "/lib/tabs/header";
    if( 'status' in item && item['status'].indexOf( 'sticky' ) >= 0 )  {
      htmlTemplate = "/lib/tabs/header-sticky";
    }

    // build
    var $item = this.ItemCreate( htmlTemplate, item );
    this.$container.append( $item );

    // attach event
    $item.click( ( e ) =>  {
      var $item = $( e.currentTarget );
      var itemData = selfCtrl.ItemGetData( $item );
      selfCtrl.SelectId( itemData['id'] );
      this.ItemScrollEnsureVisible( itemData['id'] );
    } );

    var $close = $item.find( ".close-button" );
    $close.click( ( e ) => {

      // get item
      var $item = $( e.currentTarget ).parents( '.header' );
      var itemData = selfCtrl.ItemGetData( $item );

      // check if close is allowed
      if( $.isFunction( this.options['event'] ) ) {
        if( this.options['event']( 'canclose', itemData['id'] ) !== true ) {
          console.info( `Close tab "${itemData.id}" prohibited` );
          return;
        }
      }

      // event to upper layer
      if( $.isFunction( this.options['close'] ) ) {
        this.options['close']( itemData['id'] );
      }

    } );

  }

  // ----------------------------------------------------
  //   check if scrollers need to be shown or not
  //
  ManageScrollersVisible()
  {
    // we can't use outerWidth, because it's a trunkated 
    // flex box -> calculate in _GetContentWidth.
    let contentWidth = this._GetContentWidth();

    // if container smaller than content, show scrollers
    if(  contentWidth > this.$container.outerWidth() )  {
      // show buttons
      this.$scrollerLeft.css( "display", "" );
      this.$scrollerRight.css( "display", "" );
      // move all items to right
      this.scrollerLeftOffset = this.$scrollerLeft.outerWidth();
      this.$container.css( "left", this.scrollerLeftOffset + this.scrollPosition );
    }
    else  {
      // reset scroll position
      this.scrollerLeftOffset = 0;
      this.scrollPosition = 0;
      // hide buttons
      this.$scrollerLeft.css( "display", "none" );
      this.$scrollerRight.css( "display", "none" );
      this.$container.css( "left", this.scrollerLeftOffset + this.scrollPosition );
    }
  }

  ItemScrollEnsureVisible( id )
  {
    let visibleItem = this.ListGetItem( id );
    if( visibleItem ) {

      let containerUsableWidth = this.$container.outerWidth() - this.scrollerLeftOffset * 2;
      let childPos = visibleItem.$item.offset();
      let parentPos = visibleItem.$item.parent().offset();
      let itemPos = childPos.left - parentPos.left + this.scrollPosition;
      let itemWidth = visibleItem.$item.outerWidth();

      if( itemPos < 0 )  {
        this.scrollPosition -= itemPos;
      }
      if( ( itemPos + itemWidth ) > containerUsableWidth )  {
        this.scrollPosition = ( containerUsableWidth - ( itemPos + itemWidth ) );
      }

      this.$container.css( "left", this.scrollerLeftOffset + this.scrollPosition );

    }
  }

  // ----------------------------------------------------
  //   scroll to left
  //
  OnScrollLeft()
  {
    // prepare variables
    let containerUsableWidth = this.$container.outerWidth() - this.scrollerLeftOffset * 2;
    let itemWidth = $( this.$container.children()[0] ).outerWidth();
    let itemWidthRest = containerUsableWidth % itemWidth;

    if( this.scrollPosition % itemWidth == 0 ) {
      this.scrollPosition += ( itemWidth );
    }
    else {
      this.scrollPosition += itemWidth - itemWidthRest;
    }

    //this.ItemSetStatus( this.$scrollerRight, "normal" );

    if( this.scrollPosition >= 0 )  {
      this.scrollPosition = 0;
      //this.ItemSetStatus( this.$scrollerLeft, "disabled" );
    }

    this.$container.css( "left", this.scrollerLeftOffset + this.scrollPosition );

  }

  // ----------------------------------------------------
  //   scroll to right
  //
  OnScrollRight()
  {
    // prepare variables
    let contentWidth = this._GetContentWidth();
    let containerUsableWidth = this.$container.outerWidth() - this.scrollerLeftOffset * 2;
    let itemWidth = $( this.$container.children()[ 0 ] ).outerWidth();
    let itemWidthRest = containerUsableWidth % itemWidth;

    // 
    if( this.scrollPosition % itemWidth == 0 )  {
      this.scrollPosition -= ( itemWidth - itemWidthRest );
    }
    else  {
      this.scrollPosition -= itemWidth;
    }

    //this.ItemSetStatus( this.$scrollerLeft, "normal" );
    if( this.scrollPosition <= -( contentWidth - containerUsableWidth ) ) {
      this.scrollPosition = -( contentWidth - containerUsableWidth );
      //this.ItemSetStatus( this.$scrollerRight, "disabled" );
    }

    this.$container.css( "left", this.scrollerLeftOffset + this.scrollPosition );

  }

  // ----------------------------------------------------
  //   scroll to right
  //
  _GetContentWidth()
  {
    let contentWidth = 0;

    // calculate the used with of content area by 
    // getting the last element's right position
    // relative to it's parent ($container)
    let $children = this.$container.children();
    let length = $children.length;
    if( length > 0 ) {
      let $item = $( $children[ length - 1] );
      var childPos = $item.offset();
      var parentPos = $item.parent().offset();
      contentWidth = childPos.left - parentPos.left + $item.outerWidth();
    }

    return contentWidth;
  }

  // ----------------------------------------------------
  //   close button clicked
  //
  CloseItem( closeItem )
  {
    if( $.isFunction( this.options['event'] ) ) {
      if( this.options['event']( 'canclose', closeItem['id'] ) != true ) {
        console.info( "Close tab prohibited" );
        return;
      }
    }

    // get index into data
    let indexToClose = this.options['data'].findIndex( record => ( record['id'] == closeItem['id'] ) );
    if( closeItem['id'] == this.options['selected'] )  {
      if( indexToClose > 0 )  {
        let newSelectId = this.options['data'][ indexToClose - 1 ]['id'];
        this.SelectId( newSelectId );
      }
    }

    // remove 
    this.options['data'].splice( indexToClose, 1 );
    this.ItemDelete( closeItem.$item );
    this.ManageScrollersVisible( );

  }

  // ----------------------------------------------------
  //   click to select
  //
  SelectId( selectedId, isEventNeeded = true )
  {
    if( this.options['selected'] == selectedId )
      return;

    // as if we may change selection
    let currentSelectItem = this.ListGetItem( this.options['selected'] );
    if( currentSelectItem && $.isFunction( this.options['event'] ) )  {
      if( this.options['event']( 'canunselect', currentSelectItem['id'] ) != true )  {
        console.info( "Selection prohibited" );
        return;
      }
    }

    // change selection
    let newSelectItem = this.ListGetItem( selectedId );
    if( newSelectItem ) {
      this.ListRemoveStatus( "selected" );
      this.ItemAddStatus( newSelectItem.$item, "selected" );
      this.options['selected'] = selectedId;

      this.ItemScrollEnsureVisible( selectedId );

      // event to upper layer
      if( isEventNeeded ) {
        this.options['click']( newSelectItem['id'] );
      }
    }

  }

  // ----------------------------------------------------
  //   jQuery methods
  // ----------------------------------------------------

  item( args )
  {
    // prequistes
    if( args.length < 2 ) {
      console.warn( "TabsControl::item - parameter missing!" );
      return;
    }

    // special change text of tab
    let term = args[2];
    if( term == "text" ) {

      let item = this.ListGetItem( args[ 1 ] );
      if( item == null ) {
        console.warn( "TabsControl::item - id='" + args[ 1 ] + "' not found!" );
        return;
      }

      let $text = item.$item.find( ".tab-text" );
      $text.text( $.lang( args[ 3 ] ) );

    }
    else {
      super.item( args );
    }
  }

  option( args )
  {
    if( args[ 1 ] == 'selected' ) {
      if( args.length > 2 )  {
        this.SelectId( args[ 2 ] );
      }
      return this.options['selected'];
    }

    // call default implementation
    return super.option( args );
  }

  remove( args )
  {
    let id = args[ 1 ];
    let item = this.ListGetItem( id );
    this.CloseItem( item );
  }

  create( args )
  {
    let item = args[ 1 ];
    if( !( 'icon' in item ) ) {
      item['icon'] = "";
    }
    if( !( 'text' in item ) ) {
      item['text'] = "";
    }

    this.options['data'].push( item );
    this.DrawItem( item );
    this.ManageScrollersVisible();
  }

  exists( args )
  {
    let id = args[1];
    let item = this.ListGetItem( id );
    return ( item != null );
  }

  destroy( )
  {
    this.$element.empty( );
  }

}

/* ---------------------------------------------------------------------------
 *   MENU - plugin definition
 * ------------------------------------------------------------------------ */


// ----------------------------------------------------
//   plugin main function
//
$.fn.tabs = function( option )  
{
  var data    = $( this ).data( "ctrl" ),
      options = ( typeof option == "object" && option );

  // remove for re-create
  if( typeof option == "object" && data )  {
    data.destroy( );
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data ) {
    var instance = new WfcTabs( this, options );
    $( this ).data( "ctrl", instance );
  }

  // if called with a string, treat this as a function call
  else if( typeof option == "string" ) {
    // destroy outside of class
    if( option == "destroy" )  {
      data.destroy( );
      $( this ).off();
      $( this ).data( "ctrl", null );  
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) )  {
      return data[ option ]( arguments );
    }
    else  {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.tabs.defaults = {
  size: -1,
  data: [],
  selected: null,
  selectedIndex: -1,
  click: $.noop,
  close: $.noop,
  event: null
};


}( window.jQuery );
 /*
 *  CYTEL Viewable Framework
 *  MULTI  
 */

!function( $ ) {

 
/* ---------------------------------------------------------------------------
 *   MULTI - class definition
 * ------------------------------------------------------------------------ */

class WfcMulti extends JqControl
{
  // ----------------------------------------------------
  //   constructor (will be called by base class)
  //
  OnInit( $element, options )
  {
    // class variables
    this.options = $.extend( true, [], $.fn.multi.defaults, options );
    this.$element.addClass( 'multi' );
    this.$container = null;

    // language
    this.EncodeLang();

  }

  // ----------------------------------------------------
  //   encode texts 
  //
  EncodeLang()
  {
  }

  // ----------------------------------------------------
  //   recalc layout after change
  //
  OnDraw( )  
  {
    this.$element.empty();
    this.$error = null;

    if( 'size' in this.options )  {
      let width = this.ControlCalculateSize( this.options['size'] );
      this.$element.css( 'width', width );
    }
    if( 'height' in this.options ) {
      let height = this.ControlCalculateSize( this.options['height'] );
      this.$element.css( 'height', height );
    }

    this.$container = $( $.views.Html( "/lib/multi/container" ) );
    this.$element.append( this.$container );

    this.BuildData( );

  }

  // ----------------------------------------------------
  //   while init: work data[] array
  //
  BuildData( )
  {
    let data = this.options['data'];
    for( let i = 0; i < data.length; i++ )  {
      
      var item = data[ i ];
      // destroy if one exists
      if( '$item' in item )  {
        item['$item'].destroy( );
        delete item['$item'];
      }
      this.CreateViewable( item, null );

      if( i == 0 )  {
        this.ActivateViewable( item['id'] );
      }

    }
  }

  // ----------------------------------------------------
  //   create a viewable 
  //
  CreateViewable( item, createOptions )
  {
    // get the viewable container template
    let scrollTemplate = this.options.templates['scroll']; 
    switch( this.options['type'] )  {
      case 'scroll':        scrollTemplate = this.options.templates['scroll'];         break;
      case 'scroll-static': scrollTemplate = this.options.templates['scroll-static'];  break;
      case 'blank':         scrollTemplate = this.options.templates['blank'];          break;
    }

    // create viewable item inside
    let $viewableContainer = $( $.views.Html( scrollTemplate ) );
    this.$container.append( $viewableContainer );
    $viewableContainer.css( 'display', 'none' );

    // remember
    item['$item'] = $viewableContainer;
    item['$viewable'] = $viewableContainer;
    item['$scrollbar'] = null;

    // make create options
    // make create options
    if( !createOptions )  {
      createOptions = {};
    }    
    createOptions['view'] = ( 'view' in createOptions ) ? createOptions['view'] : item['template'];

    // create the from inside of the scrollable
    let $scrollable = $viewableContainer.find( ".scrollable" );
    if( $scrollable.length == 0 )  {
      $viewableContainer.viewable( createOptions );
    }
    else  {
      $scrollable.viewable( createOptions );
      let $scrollContainer = $scrollable.parent( );
      $scrollContainer.scrollbar( { } );
      item['$viewable'] = $scrollable;
      item['$scrollbar'] = $scrollContainer;
    }

  }

  // ----------------------------------------------------
  //   activate a viewable 
  //
  ActivateViewable( id )
  {
    let currentSelected = this.FindItem( this.options['selected'] );
    if( currentSelected )  {
      currentSelected['$item'].css( 'display', 'none' );
      currentSelected['$item'].broadcast( "event:hide" );
    }

    let item = this.FindItem( id );
    if( item )  {
      item['$item'].css( 'display', "" );
      if( item['$scrollbar'] )  {
        item['$scrollbar'].scrollbar( 'recalc' );
      }
      this.options['selected'] = item['id'];
      item['$item'].broadcast( "event:show" );
    }
    else {
      console.warn( "Multi::Show - id='" + id + "' not found!" );
    }

    // inform upper layers
    this.TriggerLayoutEvent();    

  }


  // ----------------------------------------------------
  //   activate a viewable 
  //
  DestroyViewable( id )
  {
    let item = this.FindItem( id );
    if( item ) {
      // remove from data array
      item['$viewable'].viewable( "destroy" );
      item['$item'].remove( );

      // do not set the current selection, in remove() tis will be corrected
      // this.options['selected'] = item['id'];
  
      // remove from data array
      for( let i = 0; i < this.options['data'].length; i++ ) {
        if( this.options['data'][ i ]['id'] == id ) {
          ( this.options['data'] ).splice( i, 1 );
          break;
        }
      }
    }
    else  {
      console.warn( "Multi::Destroy - id='" + id + "' not found!" );
    }

    // inform upper layers
    this.TriggerLayoutEvent();    

  }  

  // ----------------------------------------------------
  //   helper: find in data[] array
  //
  FindItem( id )
  {
    for( let i = 0; i < this.options['data'].length; i++ ) {
      if( this.options['data'][ i ]['id'] == id )  {
        return this.options['data'][ i ];
      }
    }
    return null;
  }


  //-------------------------------------------------------
  //  interface
  //-------------------------------------------------------

  create( args )
  {
    let item = args[ 1 ];
    let createOptions = args[ 2 ];
    if( this.FindItem( item['id'] ) )  {
      this.ActivateViewable( item['id'] );
    }
    else  {
      this.options['data'].push( item );
      this.CreateViewable( item, createOptions );
      this.ActivateViewable( item['id'] );
    }
  }

  viewable( args )
  {
    let id = args[ 1 ];
    let item = this.FindItem( id );
    if( item && '$viewable' in item )  {
      return item['$viewable'].viewable( "viewable" );
    }
    
    return null;
  }

  remove( args )
  {
    let id = args[ 1 ];

    // find previous 
    let idToActivate = null;
    let indexToClose = this.options['data'].findIndex( item => ( item['id'] == id ) );
    if( indexToClose > 0 )  {
      idToActivate = this.options['data'][ indexToClose - 1 ]['id'];
    }
    else if( this.options['data'].length > 1 )  {
      idToActivate = this.options['data'][ 0 ]['id'];
    }
    if( idToActivate )  {
      this.ActivateViewable( idToActivate );
    }
    this.DestroyViewable( id );
  }

  show( args )
  {
    let id = args[ 1 ];
    this.ActivateViewable( id );
  }

}

   
/* ---------------------------------------------------------------------------
 *   CHOICE - plugin definition
 * ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   plugin main function
//
$.fn.multi = function( option )  
{
  var data    = $( this ).data( "ctrl" ),            
      options = ( typeof option == "object" && option );  
  
  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data )  {
    var ctrl = new WfcMulti( this, options ); 
    $( this ).data( "ctrl", ctrl );
  }
  
  // if called with a string, treat this as a function call
  else if( typeof option == "string" )  {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) )  {
      return data[ option ]( arguments );
    }
    else  {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;   
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.multi.defaults = {
  //size: -1,
  //height: -1,
  type: 'blank',
  status: "normal",
  statusDefault: "normal",
  templates: {
    'container': "/lib/multi/container",
    'blank': "/lib/multi/blank",
    'scroll': "/lib/multi/scroll",
    'scroll-static': "/lib/multi/scroll-static"
  },
  data: [],
  selected: "",
  // events
  hidden: $.noop,
  shown: $.noop
};

  
}( window.jQuery );
  /*
 *  ACCESS 5.0 Web Framework
 *  groupbox
 */

!function( $ ) {

 
/* ---------------------------------------------------------------------------
  *   GROUPBOX - class definition
  * ------------------------------------------------------------------------ */

class WfcGroupbox extends JqControl
{
  // ----------------------------------------------------
  //   constructor (will be called by base class)
  //
  OnInit( $element, options )
  {
    // class variables
    this.options  = $.extend( true, [], $.fn.groupbox.defaults, options );
    this.$element.addClass( 'groupbox' );

    // language
    this.EncodeLang();

  }

  // ----------------------------------------------------
  //   encode texts 
  //
  EncodeLang()
  {
    this.options['text'] = $.lang( this.options['text'] );
  }

  // ----------------------------------------------------
  //   recalc layout after change
  //
  OnDraw( )  
  {
    var controlSelf = this;

    // normalize css and status
    this.ItemInitialize( this.$element, this.options );

    // prequisites
    this.$label = null;
    let $children = this.$element.children( );

    // move the content (except label) to content-container
    if( $children.length > 0 )  {
      this.$label = $( $children[ 0 ] );
      if( this.$label[ 0 ].tagName == "H3" )  {
        $children.splice( 0, 1 );
      }

      // check if this groupbox is re-created
      this.$container = null; 
      if( $( $children[0] ).hasClass( 'content' ) )  {
        this.$container = $( $children[0] );
      }
      // new created
      if( this.$container == null )  {
        this.$container = $( "<div class='content'></div>" ).insertAfter( this.$label );
        this.$container.append( $children );
      }

      if( this.options['type'] == 'stretch' ) {
        this.$element.addClass( "v-layout" );
        this.$container.addClass( "v-layout-stretch" );
      }
    }


    // remove previous label, to create it new
    if( this.$label )  {
      this.options['text'] = this.$label.html();
      this.$label.remove();
    }

    // create label & register click handler
    let innerHtml = $.views.Html( '/lib/groupbox/label', this.options );
    this.$label = $( innerHtml );
    this.$element.prepend( this.$label );
    this.$icon = this.$label.find( "i" );
    this.$action = this.$label.find( ".groupbox-action" );
    this.$action.click( ( e ) =>  {
      controlSelf.OnClick( e );
    } );

    // inform upper layers
    this.TriggerLayoutEvent();
  }

  // ----------------------------------------------------
  //   event handler
  //
  OnClick( e )
  {
    this.$element.toggleClass( 'groupbox-collapsed' );
    if( this.$element.hasClass( 'groupbox-collapsed') )  {
      this.$container.css( 'display', 'none' );
      this.$icon.removeClass( 'mdi-rotate-180' );
      this.$icon.addClass( 'mdi-rotate-90' );
      this.$container.broadcast( "event:hide");
    }
    else  {
      this.$icon.removeClass( 'mdi-rotate-90' );
      this.$icon.addClass( 'mdi-rotate-180' );
      this.$container.css( 'display', "" );
      this.$container.broadcast( "event:show" );
    }

    // inform upper layers
    this.TriggerLayoutEvent( );
  }

  // ----------------------------------------------------
  //   show disabled overlay
  //
  OnShowDisabled( )
  {
    this.$container.append( "<div class='disabled-overlay'></div>" );
  }

  // ----------------------------------------------------
  //   show disabled overlay
  //
  OnHideDisabled()
  {
    this.$container.find( ".disabled-overlay" ).remove();
  }

  // ----------------------------------------------------
  //   option: generic getter and setter
  //
  option( args )  
  {
    // setter
    if( args.length > 2 ) {
      switch( args[ 1 ]  ) {
        case 'status': {
          // intentionally do nothing
          /*if( args[ 2 ] == "disabled" )  {
            this.OnShowDisabled();
          }
          else  {
            this.OnHideDisabled();
          }*/
        }
        break;

        default: {
          this.options[ args[ 1 ] ] = args[ 2 ];
          this.OnDraw();
        }
      }
    }
    // getter
    return this.options[args[1]];
  }  

  destroy()
  {
  }

}

   
/* ---------------------------------------------------------------------------
 *   GROUPBOX - plugin definition
 * ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   plugin main function
//
$.fn.groupbox = function( option )  
{
  // implicit iterator for multi-create
  if( this.length > 1 )  {
    for( let i = 0; i < this.length; i++ )  {
      stub( $( this[ i ], arguments ) );
    }
  }
  // create a single groupbox
  else  {
    stub( this, arguments );
  }

  // local function 
  function stub( element, args )  {
    var data    = $( element ).data( "ctrl" ),            
    options = ( typeof option == "object" && option );  

    // remove for re-create
    if( typeof option == "object" && data ) {
      data.destroy();
      data = null;
      $( this ).off();
      $( this ).data( "ctrl", null );
    }

    // if this plugin isn't installed, create one
    if( !data )  {
      var ctrl = new WfcGroupbox( element, options ); 
      $( element ).data( "ctrl", ctrl );
    }

    // if called with a string, treat this as a function call
    else if( typeof option == "string" )  {
      if( $.isFunction( data[ option ] ) )  {
        return data[ option ]( args );
      }
      else  {
        console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
        return null;
      }
    }
  }

  // return the object for chaining    
  return this;   
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.groupbox.defaults = {
  size: 0,
  type: 'fix',
  text: null,
  tooltip: null,
  status: "normal no-hover",
  statusDefault: "normal no-hover",
  click: $.noop
};

  
}( window.jQuery );
 /*
 *  ACCESS 5.0 Web Framework
 *  CONTEXT MENU
 */

!function ($) {

/* ---------------------------------------------------------------------------
 *   MENU - const values
 * ------------------------------------------------------------------------ */


/* ---------------------------------------------------------------------------
 *   MENU - class definition
 * ------------------------------------------------------------------------ */

class WfcPopupMenu extends JqControl
{
  // ----------------------------------------------------
  //   constructor (will be called by base class)
  //
  OnInit( $element, options )
  {
    // class variables
    this.options = $.extend( true, [], $.fn.popupmenu.defaults, options );

    if( ! ( 'anchor' in this.options ) ) {
      console.warn( "popupmenu: 'anchor' needed!" );
    }
    // encode all language 
    this.EncodeLang( );

  }

  // ----------------------------------------------------
  //   event functions
  //
  EncodeLang( )
  {
    this.options['data'] = $.lang( this.options['data'] );
  }

  // ----------------------------------------------------
  //   recalc layout after change
  //
  OnDraw()  
  {
    this.$element.empty();
    this.$element.addClass( 'popupmenu themeNormal' );

    // do NOT call this.ItemInitialize (container should not have status)

    // loop through items
    for( var itemData of this.options.data ) {
      this.OnDrawItem( itemData );
    }

    // focus & loose focus event
    this.$element.attr( "tabindex", "0" );
    this.$element.focus();
    this.$element.on( "blur", this.OnHide.bind( this ) );

    // position
    this.CalculatePosition();

  }

  //----------------------------------------------------
  //  draw an item
  //
  OnDrawItem( itemData )
  {
    var $item = null;
    if( itemData['id'][0] == "-" )  {
      $item = $( $.views.Html( "/lib/popupmenu/separator", itemData ) );
      this.ItemInitialize( $item, itemData );

    }
    else {
      $item = $( $.views.Html( "/lib/popupmenu/item", itemData ) );
      this.ItemInitialize( $item, itemData );
      $item.on( "click", ( ev ) => {
        var evItemData = this.ItemGetData( $( ev.currentTarget ) );
        this._FireEvent( evItemData['id'] );
      });
    }

    this.$element.append( $item );

  }

  //----------------------------------------------------
  //  calculate the menu pos & display
  //
  CalculatePosition()
  {
    // check if anchor is accessible
    var $anchor = $( this.options['anchor'] );
    if( $anchor.length != 1 ) {
      return;
    }

    if( this.options['position'] == "below-left" ) {
      var anchorPos = $anchor.offset();
      var anchorHeight = $anchor.outerHeight();
      this.$element.css( "top", anchorPos.top + anchorHeight - 1 );
      this.$element.css( "left", anchorPos.left );
    }
    else if( this.options['position'] == "below-right" ) {
      var anchorPos = $anchor.offset();
      var anchorHeight = $anchor.outerHeight();
      var anchorWidth = $anchor.outerWidth();
      var elementWidth = this.$element.outerWidth();
      this.$element.css( "top", anchorPos.top + anchorHeight - 1 );
      this.$element.css( "left", anchorPos.left - ( elementWidth - anchorWidth ) );
    }
    else if( this.options['position'] == "beside-right" ) {
      var anchorPos = $anchor.offset();
      var anchorWidth = $anchor.outerWidth();
      this.$element.css( "top", anchorPos.top  );
      this.$element.css( "left", anchorPos.left + anchorWidth - 1 );
    }
    else if( this.options['position'] == "beside-left" ) {
      var anchorPos = $anchor.offset();
      var elementWidth = this.$element.outerWidth();
      this.$element.css( "top", anchorPos.top );
      this.$element.css( "left", anchorPos.left - elementWidth + 1 );
    }
  }

  OnHide()
  {
    this.$element.empty();
    this.$element.attr( "class", "" );
    this.$element.attr( "tabindex", "" );
  }

  _FireEvent( id )
  {
    if( $.isFunction( this.options['click'] ) )  {
      this.options['click']( id );
    }
    this.$element.blur();
  }



  // ----------------------------------------------------
  //   option getter and setter
  //
  option( args ) 
  {
    // setter
    if( args.length > 2 ) {
      this.options[ args[ 1 ] ] = args[ 2 ];
      this.OnDraw( );
    }
    
    // getter
    return this.options[ args[ 1 ] ];
  }

};

/* ---------------------------------------------------------------------------
 *   MENU - plugin definition
 * ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   plugin main function
//
$.fn.popupmenu = function( option ) 
{
  let data    = $( this ).data( "ctrl" );
  let options = ( typeof option == "object" && option );

  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // if this plugin isn't installed, create one
  if( !data ) {
    var instance = new WfcPopupMenu( this, options );
    $( this ).data( "ctrl", instance );
  }

  // if called with a string, treat this as a function call
  else if( typeof option == "string" )  {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy( );
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) ) {
      return data[ option ]( arguments );
    }
    else {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.popupmenu.defaults = {
  data: [],
  opacity: 0,
  size: 16,
  status: "normal no-hover",
  position: 'below-left',
  click: null
};


}(window.jQuery);
//=============================================================================
//
//  File: toolbar.js
//  (c) 2021 by CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.10.2020
//
//  Description:
//          toolbar with text/icon on left and buttons on the right
//
//=============================================================================


!function( $ ) {
 
//-----------------------------------------------------------------------------
//  TOOLBAR - class definition
//-----------------------------------------------------------------------------

class WfcToolbar extends JqControl
{
  // ----------------------------------------------------
  //   constructor (will be called by base class)
  //
  OnInit( $element, options )
  {
    // class variables
    this.options  = $.extend( true, [], $.fn.toolbar.defaults, options );
    this.$element.addClass( 'toolbar' );

    // language
    this.EncodeLang();

  }

  // ----------------------------------------------------
  //   encode texts 
  //
  EncodeLang()
  {
    this.options['text'] = $.lang( this.options['text'] );
    this.options['tooltip'] = $.lang( this.options['tooltip'] );
    this.options['data'] = $.lang( this.options['data'] );
  }

  //-------------------------------------------------------
  //  recalc layout after change
  //
  OnDraw( )  
  {
    this.$element.addClass( 'toolbar' );
    if( this.options['closeable'] === true ) {
      this.$container = $( $.views.Html( "/lib/toolbar-closeable", this.options ) );
      this.$closeButton = this.$container.find( ".close-button" );
      this.$closeButton.on( "click", ( ev ) => {
        if( $.isFunction( this.options['close'] ) ) {
          this.options['close']( );
        }
      } );
    }
    else {
      this.$container = $( $.views.Html( "/lib/toolbar", this.options ) );
    }
    this.$element.append( this.$container );

    this.OnDrawButtons( );
  }

  //-------------------------------------------------------
  //  redraw buttons
  //
  OnDrawButtons( )
  {
    var controlSelf = this;
    var $buttons = this.$element.find( ".buttons" );
    $buttons.empty();

    for( let item of this.options.data ) {

      let $item = null;

      // display: text + icon
      if( 'type' in item && item['type'] == "icon" )  {
        $item = this.ItemCreate( "/lib/toolbar/icon", item );
      }
      else if( item['text']  &&  item['icon'] )  {
        $item = this.ItemCreate( "/lib/toolbar/button", item );
      }
      // display: icon only
      else if( item['icon'] ) {
        $item = this.ItemCreate( "/lib/toolbar/button-icon", item );
      }
      // display: text only
      else if( item['text'] )  {
        $item = this.ItemCreate( "/lib/toolbar/button-text", item );
      }
      else  {
        continue;
      }
     
      $buttons.append( $item );

      $item.click( function( e )  {
        let $item = $( e.currentTarget );
        let itemData = controlSelf.ItemGetData( $item );
        controlSelf.OnClickItem( $item, itemData, e );
      } );

    }
  }

  //-------------------------------------------------------
  //  button click handler
  //
  OnClickItem( $item, itemData, e )
  {
    if( itemData['disabled'] != true ) {
      if( $.isFunction( this.options['click'] ) ) {
        this.options['click']( itemData['id'], itemData['path'], itemData, e );
      }
    }
  }

  //-------------------------------------------------------
  //  option
  //
  OptionIconStatus( icon )
  {
    this.options['iconStatus'] = icon;
    this.$icon = this.$container.find( ".icon-status > i" );
    this.$icon.attr( "class", icon );
  }

  //-------------------------------------------------------
  //  option
  //
  option( args )
  {
    // do itentionally northing
    if( args[ 1 ] == "text" )  {
      this.options['text'] = $.lang( args[ 2 ] );
      this.$title = this.$container.find( ".toolbar-title" );
      this.$title.text( this.options['text'] );
    }

    if( args[ 1 ] == "iconStatus") {
      this.OptionIconStatus( args[ 2 ] );
    }

    if( args[ 1 ] == "data" ) {
      this.options['data'] = args[ 2 ];
      this.OnDrawButtons( );
    }

  }

}

   
//-----------------------------------------------------------------------------
//  TOOLBAR - plugin definition
//-----------------------------------------------------------------------------

//---------------------------------------------------------
//   plugin main function
//
$.fn.toolbar = function( option )  
{
  var data    = $( this ).data( "ctrl" ),            
      options = ( typeof option == "object" && option );  

  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy( );
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data )  {
    var instance = new WfcToolbar( this, options ); 
    $( this ).data( "ctrl", instance );
  }

  // if called with a string, treat this as a function call
  else if( typeof option == "string" )  {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy( );
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) ) {
      return data[ option ]( arguments );
    }
    else {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;   
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.toolbar.defaults = {
  size: 0,
  text: null,
  tooltip: null,
  icon: null,
  iconStatus: null, // like "mdi mdi-circle red-text"
  closeable: false,
  status: "normal",
  statusDefault: "normal",
  data: [],
  close: null,  // event
};

  
}( window.jQuery );
 /*
 *  Views 2 - Web Framework
 *  GRID
 */

!function( $ )
{
  // define keycodes
  var KEY = {
    BS:     8,
    TAB:    9,
    RETURN: 13,
    ENTER:  13,
    SHIFT:  16,
    CTRL:   17,
    ALT:    18,
    SCTRL:  20,
    ESC:    27,
    SPACE:  32,
    END:    35,
    POS1:   36,
    LEFT:   37,
    UP:     38,
    RIGHT:  39,
    DOWN:   40,
    DEL:    46
  };

/* ---------------------------------------------------------------------------
 *   GRID - class definition
 * ------------------------------------------------------------------------ */

class WfcGrid extends JqControl
{
  // ----------------------------------------------------
  //   init-function (called by constructor)
  //
  OnInit( $element, options )  
  {
    // class variables (views)
    this.$cssId = $.uidgen();
    this.$element = $element;
    this.$container = null;
    this.$header = null;
    this.controlHeight = 0;
    this.scrollInfo = null;

    // copy options
    this.options = $.extend( true, [], $.fn.grid.defaults, options );
    this.options['statusDefault'] = ( this.options['statusDefault'] || "" ) + " no-hover";
    this.EncodeLang();

    // class variables (logic)
    this.columnOrder = [];
    this.isSharedDataset = false;
    this.isSelectionSet = false;
    this.selected = [];
    this.selectIndex = -1;
    this.dataset = null;
    this.dataview = null;
    this.scrollWindow = { top: 0, bottom: 0 };

    //  Dataset/View - Handling
    if( Array.isArray( this.options.data ) ) {
      this.dataset = new Dataset( { id: this.options.dataId }, this.options.data );
      this.dataview = this.dataset.BuildView( 'default', { sortKey: this.options['sort'], sortOrder: this.options['sortOrder'], filter: this.options['dataFilter'] } );
      this.dataview.Silent( true );
      this.isSharedDataset = false;
    }
    else if( this.options.data instanceof Dataview ) {
      this.dataview = this.options.data;
      this.dataview.Options( { sortKey: this.options['sort'], sortOrder: this.options['sortOrder'], filter: this.options['dataFilter'] } );
      this.isSharedDataset = true;
    }
    else if( this.options.data instanceof Dataset ) {
      this.dataset = this.options.data;
      this.dataview = this.dataset.BuildView( 'default', { sortKey: this.options['sort'], sortOrder: this.options['sortOrder'], filter: this.options['dataFilter'] } );
      this.isSharedDataset = true;
    }
    // IMPORTANT: for garbage collection of removed DOM-Elements
    this.dataview.AttachDomElement( this.$element );

    // data filter
    if( $.isFunction( this.options['dataFilter'] ) )  {
      this.dataview.Options( { filter: this.options['dataFilter'] } );
    }

    // encode header
    this.EncodeLang();

  }

  // ----------------------------------------------------
  //   encode texts 
  //
  EncodeLang()
  {
    this.options['header'] = $.lang( this.options['header'] );
  }

  // ----------------------------------------------------
  //    MAIN
  // ----------------------------------------------------

  // ----------------------------------------------------
  //   draw (called by constructor && options )
  //
  OnDraw( )
  {
    // prequisites
    var selfCtrl = this;
    this.destroy() ;

    let id = this.$element.attr( 'id' );
    if( !id )  {
      console.warn( "Grid: Missing 'id' on html-tag. Abort!" );
      return;
    }

    // build container 
    this.$element.empty();
    this.$error = null;
    this.ItemInitialize( this.$element, { status: "normal no-hover" } );

    // attach frame 
    this.$element.addClass( 'grid normal no-hover' );
    if( this.options['border'] == true ) {
      this.$element.addClass( "control-border" );
    }
    if( this.options['focus'] == true )  {
      this.$element.prop( 'tabindex', '-1' );
    }

    this.$container = $( $.views.Html( "/lib/grid/container" ) );
    this.$element.append( this.$container );
    this.$header = this.$container.find( '.header' );
    this.$content = this.$container.find( '.scrollable' );
    this.lineHeight = this._CalculateItemHeight( this.$container );

    // set sizes
    if( 'size' in this.options )  {
      let size = this.ControlCalculateSize( this.options['size'] );
      this.$element.css( 'width', size );
    }
    if( 'height' in this.options ) {
      if( this.options['height'] == "fit" )  {
        this.FitControlHeight();
      }
      else if( this.options['height'] > 0 )  {
        this.$element.css( 'height', this.lineHeight * this.options['height'] + 3 );
      }
      else  {
        let height = this.ControlCalculateHeight( this.options['height'] );
        this.$element.css( 'height', height );  //this.options['height'] );
      }
    }

    // check if header is hidden
    if( this.options['headerHidden'] ) {
      this.$header.addClass( "hidden" );
    }

    // build header & css class
    if( 'header' in this.options )  {
      this.OnDrawHeader( );
    }

    // recalc line height after adding header (result differs for some reason)
    this.lineHeight = this._CalculateItemHeight( this.$container );

    // set all callback functions from options
    this.OnDrawManageCallbacks();

    // data handeler
    this.dataview.On( 'rebuild', function( )  {
      selfCtrl.OnDrawData();
    } );
    this.dataview.On( 'delete', ( id, itemData ) => {
      selfCtrl.OnDrawData();
    } );
    this.dataview.On( 'select', ( selectedList ) => {
      //selfCtrl.OnDrawData();
    } );
    this.dataview.On( 'update', ( id ) => {
      selfCtrl.OnDrawData( );
    } );


    // make scrollbar
    this.$scroll = this.$element.find( ".scroll-container" );
    if( this.$scroll.length == 1 ) {
      this.$scroll.scrollbar( { 
        resize: true, 
        type: this.options['scrollbar'],
        lineHeight: this.lineHeight,
        scroll: this.OnScrollWindow.bind( this )
      } );
    }

    // emit a 'rebuild' event
    this.dataview.Silent( false );

    // focus-out handling
    this.$element.focusout( function( e )  {
      if( !this.contains( e.relatedTarget ) )  {
        selfCtrl.OnLeaveFocus();
      }
    } );    
    // focus-in handling
    this.$element.focusin( function( e )  {
      if( !this.contains( e.relatedTarget ) )  {
        selfCtrl.OnEnterFocus( );
      }
    } );    
    // keyboard handling
    this.$element.on( 'keydown', function ( e )  {
      selfCtrl.OnKeydown( e.which, e );
    } );    

    // viewable broadcast, emitted by optional & multi
    this.$element.on( 'event:show', ( ev ) => {
      if( this.$scroll ) {
        this.$scroll.scrollbar( "recalc" );
      }
      let selectedList = this.dataview.ItemGetSelected( );
      this._ScrollToItem( selectedList[ 0 ] );
      ev.stopPropagation( );
    } );    

  }

  // ----------------------------------------------------
  //   ensure option "renderStatus" and "renderCell"
  //
  OnDrawManageCallbacks()
  {
    // cell draw may be overridden (custom draw)
    const drawCell = ( columnId, record ) => {
      return {
        text: record[columnId],
        icon: ""
      };
    };
    // if external randering: to language translation
    const drawCellExtern = ( columnId, record ) => {
      let result = this.options['renderCell']( columnId, record );
      if( typeof result['text'] === 'string' ) {
        result['text'] = $.lang( result['text'] );
      }
      return result;
    };
    let fnDrawCell = drawCell;
    if( $.isFunction( this.options['renderCell'] ) ) {
      fnDrawCell = drawCellExtern;
    }

    // row status may be overridden (custom status)
    const rowStatus = ( columnId, record ) => {
      return ( 'status' in record ) ? record['status'] + ' normal' : 'normal';
    };
    let fnRowStatus = rowStatus;
    if( $.isFunction( this.options['renderStatus'] ) ) {
      fnRowStatus = this.options['renderStatus'];
    }

    this.rendering = {
      cell: fnDrawCell,
      status: fnRowStatus
    };
  }

  // ----------------------------------------------------
  //    HEADER
  // ----------------------------------------------------

  // ----------------------------------------------------
  //   draw header
  //
  OnDrawHeader( )
  {
    var selfControl = this;

    // get <style> tag from template
    this.$style = this.$element.find( "style" );
    if( this.$style.length <= 0 )  {
      console.warn( "Missing <style> tag in html-template! Abort." );
      return;
    }

    // remember the order of the columns to find left/right neighbor
    for( let headerId in this.options["header"] ) {
      this.columnOrder.push( headerId );
    }

    // load from registry
    if( $.isFunction( $.registry ) && typeof this.options.regkey == "string" ) {
      let storedValue = $.registry( this.options.regkey );
      for( let headerId in storedValue )  {
        if( headerId in this.options["header"] )  {
          this.options["header"][ headerId ]['cssWidth'] = storedValue[ headerId ];
        }
      }
    }

    // calculate 
    this.BuildStyles( this.$style );

    // create header
    let lastIndex = Object.keys( this.options["header"] ).length - 1;
    let currentIndex = 0;
    for( let headerId in this.options["header"] ) {
      let headerItem = this.options["header"][ headerId ];
      let $item = null;
      if( headerItem['text'].length && headerItem['icon'] )  {
        $item = $( $.views.Html( '/lib/grid/header-item', headerItem ) );
      }
      else if( headerItem['text'] )  {
        $item = $( $.views.Html( '/lib/grid/header-item-text', headerItem ) );
      }
      else {
        $item = $( $.views.Html( '/lib/grid/header-item-icon', headerItem ) );
      }
      headerItem['$item'] = $item;
      $item.data( 'itemData', headerItem );

      if( headerItem['resizeable'] )  {
        this.OnDrawSplitter( $item, headerItem );
      }
      if( headerItem['sortable'] ) {
        $item.addClass( "cell-hover" );
        $item.on( 'click', function( e ) {
          var itemData = $( this ).data( 'itemData' );
          selfControl.OnClickHeader( itemData );
        } );
      }
      if( currentIndex < lastIndex )  {
        $item.addClass( "diverter" );
        currentIndex = currentIndex + 1;
      }
      if( headerId == this.options['sort'] )  {
        let $i = $item.find( ".cell-sorter > i" );
        $i.removeClass( "mdi-none" );
        if( this.options['sortOrder'] == '+' )  {
          $i.addClass( this.options['sortIconAsc'] );
        }
        else {
          $i.addClass( this.options['sortIconDesc'] );
        }
      }
      this.$header.append( $item );
    }

    // static scrollbar add element at the end
    if( this.options['scrollbar'] == 'static' )  {
      let $placeholder = $( $.views.Html( '/lib/grid/header-scrollbar', { } ) );
      this.$header.append( $placeholder );
    }

    // remember actual width of items after push to screen
    for( let headerId in this.options["header"] ) {
      let headerItem = this.options["header"][headerId];
      headerItem['cssWidth'] = headerItem['$item'].outerWidth( );
    }

    // remember height for scrolling
    this.headerHeight = this.$header.outerHeight( );
    this.lineHeight = this.headerHeight;

  }

  // ----------------------------------------------------
  //   splitter & resize
  //
  OnDrawSplitter( $item, headerItem )
  {
    var selfControl = this;

    let $splitter = $( $.views.Html( '/lib/grid/header-splitter', headerItem ) );
    $item.append( $splitter );

    headerItem['mouse'] = new MouseCapture;
    headerItem['mouse'].start( $splitter,
      // --- mouse down ---
      function( e )
      {
        // get current item
        let headerId = this.tag.prop( "id" );
        let header = selfControl.options["header"][ headerId ];
        header.mouseStartPos = e.clientX;
        header.elementSize = header.$item.outerWidth( );
        
        // get neighbor item
        let myIndex = selfControl.columnOrder.indexOf( headerId );
        let neightborId = selfControl.columnOrder[ myIndex + 1 ];
        var neighborItem = selfControl.options["header"][ neightborId ];
        neighborItem.elementSize = neighborItem.$item.outerWidth( );
        header.neighborItem = neighborItem;
      
      },
      // --- move mouse ---
      function( e )
      {
        // current item
        let headerId = this.tag.prop( "id" );
        let header = selfControl.options["header"][ headerId ];
        if( !header.mouseStartPos ) return;
        var mouseDx = e.clientX - header.mouseStartPos;
        var newWidth = ( header.elementSize + mouseDx );
        header['cssWidth'] = newWidth;

        // neighbor item
        var neighborWidth = ( header.neighborItem.elementSize - mouseDx );
        header.neighborItem['cssWidth'] = neighborWidth;

        selfControl.BuildStyles( selfControl.$style );
      },
      // --- mouse up (clean up) ----
      function( e )
      {
        // current item
        let headerId = this.tag.prop( "id" );
        let header = selfControl.options["header"][ headerId ];
        delete header.mouseStartPos;
        delete header.neighborItem;
        delete header.elementSize;

        // store widthin registry
        if( $.isFunction( $.registry ) && typeof selfControl.options.regkey == "string" ) {
          let storedValue = {};
          for( let headerId in selfControl.options["header"] ) {
            let header = selfControl.options["header"][ headerId ];
            storedValue[ headerId ] = header['cssWidth'];
          }
          $.registry( selfControl.options.regkey, storedValue );
        }

      }
    );    
  }

  // ----------------------------------------------------
  //   build styles
  //
  BuildStyles( $style )
  {
    // build row styles
    let styleTag = "";
    for( let headerId in this.options["header"] ) {
      
      // prepare
      let headerItem = this.options["header"][ headerId ];
      headerItem['cssColumnId'] = headerId + "-" + this.$cssId;
      headerItem['id'] = headerId;
      styleTag = styleTag + "." + headerItem['cssColumnId'] + "{";

      // width in characters. We need to shrink one pixel for borders
      let size = this.ControlCalculateSize( Math.abs( headerItem['size'] ) ) - 1;

      // sizable?
      let flexGrow = headerItem['resizeable'] ? 1 : 0;
      styleTag = styleTag + " flex-grow: " + flexGrow + ";";

      // width
      // function called after actual sizes are calculated
      if( 'cssWidth' in headerItem )  {
        if( flexGrow > 0 ) {
          styleTag = styleTag + " width: " + headerItem['cssWidth'] + "%;";
          styleTag = styleTag + " min-width: 10px;";
          styleTag = styleTag + " flex-shrink: 1;";
        }
        else {
          styleTag = styleTag + " width: " + ( headerItem['cssWidth'] - 1 ) + "px;";
          styleTag = styleTag + " flex-shrink: 0;";
        }
      }
      // function called first time
      else  {
        if( flexGrow > 0 ) {
          styleTag = styleTag + " width: " + size * 10 + "%;";
          styleTag = styleTag + " min-width: 10px;";
          styleTag = styleTag + " flex-shrink: 1;";
        }
        else {
          styleTag = styleTag + " width: " + size + "px;";
          styleTag = styleTag + " flex-shrink: 0;";
        }
      }

      // alignment
      if( headerItem['align'] == "right" )  {
        styleTag = styleTag + " justify-content: flex-end; ";
      }
      else if( headerItem['align'] == "center" ) {
        styleTag = styleTag + " justify-content: center; ";
      }

      styleTag = styleTag + "}\n";
    }

    // update style tag
    $style.html( styleTag );

  }

  // ----------------------------------------------------
  //    DATA
  // ----------------------------------------------------

  // ----------------------------------------------------
  //   draw data (using a data-window + scrollInfo)
  //
  OnDrawDataWindow()
  {
    this.options['dataDrawWindow'] = true;

    //console.time( "OnDrawDataWindow" );
    var selfCtrl = this;
    this.$content.empty();
    this.$contentWindow = $( "<div class='data-window'></div>" );
    this.$content.append( this.$contentWindow );

    var firstIndex = this.scrollInfo ? this.scrollInfo.firstIndex : 0;
    var lastIndex = this.scrollInfo ? this.scrollInfo.lastIndex : -1;
    if( lastIndex < 0 ) {
      lastIndex = this.dataview.length - 1;
      var height = lastIndex - firstIndex;
      var scrollContainerHeight = Math.floor( this.$scroll.outerHeight() / this.lineHeight );
      if( height > scrollContainerHeight ) {
        lastIndex = firstIndex + scrollContainerHeight;
      }
    }
    this.$content.css( "height", this.dataview.length * this.lineHeight );
    this.$contentWindow.css( "top", firstIndex * this.lineHeight );
    this.options['selected'] = this.dataview.ItemGetSelected();

    //console.log( "draw-window", firstIndex, lastIndex, this.dataview.length );

    // cell draw may be overridden (custom draw)
    function drawCell( columnId, record ) {
      return {
        text: record[columnId],
        icon: ""
      };
    }
    function drawCellExtern( columnId, record ) {
      let result = selfCtrl.options['renderCell']( columnId, record );
      if( typeof result['text'] === 'string' ) {
        result['text'] = $.lang( result['text'] );
      }
      return result;
    }
    let fnDrawCell = drawCell;
    if( $.isFunction( this.options['renderCell'] ) ) {
      fnDrawCell = drawCellExtern;
    }

    // row status may be overridden (custom status)
    function rowStatus( columnId, record ) {
      let result = ( 'status' in record ) ? record['status'] + ' normal' : 'normal';
      return result;
    }
    let fnRowStatus = rowStatus;
    if( $.isFunction( this.options['renderStatus'] ) ) {
      fnRowStatus = this.options['renderStatus'];
    }

    // no selection, no hover ;-)
    let isCellHover = this.options['focus'] == "cell";
    let rowHoverClass = ( this.options['selection'] == "none" || isCellHover ) ? " no-hover" : " hover";
    let listHeight = 0;

    // then iterate through the data
    for( var i = firstIndex; i <= lastIndex; i++ ) {

      // create item and attach it to dataview
      let record = this.dataview.GetByIndex( i );
      if( !record ) {
        continue;
      }

      let recordId = record[this.options['dataId']];
      let $row = $( $.views.Html( '/lib/grid/data-row', record ) );

      // create row 
      let status = fnRowStatus( recordId, record );
      status = status + rowHoverClass;
      let itemData = this.dataview.AttachItem( i, $row, { status: status } );
      if( this.dataview.ItemHasStatus( recordId, "selected" ) ) {
        status += " selected";
      }
      this.dataview.ItemSetStatus( recordId, status );

      this.$contentWindow.append( $row );

      // position for scrolling
      itemData.y = listHeight;
      listHeight = listHeight + $row.outerHeight();

      // iterate through cells & append 
      for( let headerId in this.options["header"] ) {
        let headerItem = this.options["header"][headerId];
        let item = fnDrawCell( headerId, record );
        item['cssColumnId'] = headerItem['cssColumnId'];
        let $cell = null;

        // jump out for inplace controls
        let inplaceControl = headerItem['inplace'] || null;
        if( inplaceControl == "onoff" && typeof item.text == "boolean" ) {
          $cell = this.OnDrawInplaceOnoff( item, headerId, record, recordId );
        }
        else if( inplaceControl == "check" && typeof item.text == "boolean" ) {
          $cell = this.OnDrawInplaceCheck( item, headerId, record, recordId );
        }
        else if( inplaceControl == "link" ) {
          $cell = this.OnDrawInplaceLink( item, headerId, record, recordId );
        }
        // standard grid cell
        else {
          if( ( typeof item['text'] == "string" && item['text'].length > 0 )
            && ( typeof item['icon'] == "string" && item['icon'].length > 0 )
          ) {
            $cell = $( $.views.Html( '/lib/grid/data-cell', item ) );
          }
          else if( typeof item['text'] == "string" && item['text'].length > 0 ) {
            $cell = $( $.views.Html( '/lib/grid/data-cell-text', item ) );
          }
          else if( typeof item['icon'] == "string" && item['icon'].length > 0 ) {
            $cell = $( $.views.Html( '/lib/grid/data-cell-icon', item ) );
          }
          else {
            $cell = $( $.views.Html( '/lib/grid/data-cell-empty', item ) );
          }
          if( isCellHover )
            $cell.addClass( "hover" );
        }

        if( 'tooltip' in item ) {
          $cell.attr( "title", item['tooltip'] );
        }

        $row.append( $cell );
      }

      // row selection
      $row.click( function( ev ) {
        var itemData = $( ev.currentTarget ).data( 'itemData' );
        selfCtrl.OnClickItem( itemData, ev.shiftKey );
        ev.stopPropagation();
      } );

      // row selection
      $row.dblclick( function( ev ) {
        var itemData = $( ev.currentTarget ).data( 'itemData' );
        selfCtrl.OnDblClickItem( itemData, ev.shiftKey );
        ev.stopPropagation();
      } );


    }//for


    // resize control & recalc scrollbar
    this.FitControlHeight();
    if( this.$scroll ) {
      this.$scroll.scrollbar( "recalc" );
    }

    // finally (re-)do the validation
    this.ManageValidate( this.dataview, "rebuild" );

    //console.timeEnd( "OnDrawDataWindow" );
  }
  
  // ----------------------------------------------------
  //   draw data
  //
  OnDrawData()
  {
    if( this.dataview.length > this.options['dataDrawWindowThreshold'] )  {
      return this.OnDrawDataWindow();
    }
    this.options['dataDrawWindow'] = false;

    this.$content.empty();

    // no selection, no hover ;-)
    let isCellHover = this.options['focus'] == "cell";
    let rowHoverClass = ( this.options['selection'] == "none" || isCellHover ) ? " no-hover" : " hover";

    this.$content.css( "height", this.dataview.length * this.lineHeight );
    if( this.$scroll ) {
      this.$scroll.scrollbar( "recalc" );
    }
    this.FitControlHeight();

    if( this.dataview.length == 0 )  {
      this.OnDrawPlaceholder( );
    }
    else {
      this.OnDrawRows( 0, this.dataview.length, isCellHover, rowHoverClass );
    }


    // finally (re-)do the validation
    this.ManageValidate( this.dataview, "rebuild" );

  }

  OnDrawPlaceholder()
  {
    if( this.options['placeholderIcon'] && ! this.options['placeholder'] ) {
      let $row = $( $.views.Html( '/lib/grid/data-placeholder-icon-only', { placeholder: "", placeholderIcon: $.lang( this.options['placeholderIcon'] ) } ) );
      this.$content.append( $row );
      $row.css( "height", this.$element.outerHeight() - this.lineHeight );
    }
    else if( this.options['placeholderIcon'] ) {
      let $row = $( $.views.Html( '/lib/grid/data-placeholder-icon', { placeholder: $.lang( this.options['placeholder'] ), placeholderIcon: $.lang( this.options['placeholderIcon'] ) } ) );
      this.$content.append( $row );
      $row.css( "height", this.$element.outerHeight() - this.lineHeight );
    }
    else {
      let $row = $( $.views.Html( '/lib/grid/data-placeholder', { placeholder: $.lang( this.options['placeholder'] ) } ) );
      this.$content.append( $row );
      $row.css( "height", this.$element.outerHeight() - this.lineHeight );
    }
  }

  //-----------------------------------------------------------------
  //  draw an amount of rows
  //
  OnDrawRows( rowStart, size, isCellHover, rowHoverClass )
  {
    // then iterate through the data
    for( var i = rowStart; i < ( rowStart + size ); i++ ) {

      // create item and attach it to dataview
      let record = this.dataview.GetByIndex( i );
      if( !record ) {
        if( i > this.dataview.length )  {
          return false;
        }
        continue;
      }

      let recordId = record[this.options['dataId']];
      let $row = $( $.views.Html( '/lib/grid/data-row', record ) );

      // create row
      let status = this.rendering.status( recordId, record );
      status = status + rowHoverClass;
      this.dataview.AttachItem( i, $row, { status: status } );
      this.dataview.ItemSetStatus( recordId, status );
      this.$content.append( $row );


      // iterate through cells & append 
      for( let headerId in this.options["header"] ) {

        // preapre cell data
        let headerItem = this.options["header"][headerId];
        let item = this.rendering.cell( headerId, record );
        item['cssColumnId'] = headerItem['cssColumnId'];

        // create cell & append to row
        let $cell = this.OnDrawCell( record, recordId, item, headerItem, headerId, isCellHover );
        $row.append( $cell );
      }

      // row selection
      $row.click( ( ev ) => {
        var itemData = $( ev.currentTarget ).data( 'itemData' );
        this.OnClickItem( itemData, ev.shiftKey );
        ev.stopPropagation();
      } );

      // row selection
      $row.dblclick( ( ev ) => {
        var itemData = $( ev.currentTarget ).data( 'itemData' );
        this.OnDblClickItem( itemData, ev.shiftKey );
        ev.stopPropagation();
      } );


    }//for

    return true;
  }

  //-----------------------------------------------------------------
  //  draw a cell
  //
  OnDrawCell( record, recordId, item, headerItem, headerId, isCellHover )
  {
    let $cell = null;

    // jump out for inplace controls
    let inplaceControl = headerItem['inplace'] || null;
    if( inplaceControl == "onoff" && typeof item.text == "boolean" ) {
      $cell = this.OnDrawInplaceOnoff( item, headerId, record, recordId );
    }
    else if( inplaceControl == "check" && typeof item.text == "boolean" ) {
      $cell = this.OnDrawInplaceCheck( item, headerId, record, recordId );
    }
    else if( inplaceControl == "link" ) {
      $cell = this.OnDrawInplaceLink( item, headerId, record, recordId );
    }
    // standard grid cell
    else {
      if( ( typeof item['text'] == "string" && item['text'].length > 0 )
        && ( typeof item['icon'] == "string" && item['icon'].length > 0 )
      ) {
        $cell = $( $.views.Html( '/lib/grid/data-cell', item ) );
      }
      else if( typeof item['text'] == "string" && item['text'].length > 0 ) {
        $cell = $( $.views.Html( '/lib/grid/data-cell-text', item ) );
      }
      else if( typeof item['icon'] == "string" && item['icon'].length > 0 ) {
        $cell = $( $.views.Html( '/lib/grid/data-cell-icon', item ) );
      }
      else {
        $cell = $( $.views.Html( '/lib/grid/data-cell-empty', item ) );
      }
      if( isCellHover )
        $cell.addClass( "hover" );
    }

    if( 'tooltip' in item ) {
      $cell.attr( "title", item['tooltip'] );
    }

    return $cell;
  }

  OnDrawInplaceOnoff( item, headerId, record, recordId )
  {
    // build cell
    if( item.text === true || item.text === "true" )  {
      item.icon = "mdi mdi-check-bold green-text";
    }
    else {
      item.icon = "mdi mdi-close-thick red-text";
    }
    let $cell = $( $.views.Html( '/lib/grid/inplace-onoff', item ) );

    // cell disabled status
    let isDisabled = false;
    if( 'status' in item )  {
      $cell.addClass( item['status'] );
      if( item['status'].indexOf( "disabled" ) >= 0 )  {
        $cell.removeClass( "grid-inplace-onoff" );
        isDisabled = true;
      }
    }
  
    // call inplace edit function
    if( !isDisabled  &&  $.isFunction( this.options['inplaceEdit'] ) )  {
      $cell.on( 'click', ( ev ) => {
        let changedRecord = this.options['inplaceEdit']( headerId, record );
        if( typeof changedRecord == "object" )  {
          this.dataset.Update( changedRecord );
        }
        ev.stopPropagation();
      } );

    }
    return $cell;
  }

  OnDrawInplaceCheck( item, headerId, record, recordId )
  {
    item['icon2'] = "";
    if( 'icon' in item ) {
      item['icon2'] = item['icon'];
    }

    // build cell
    if( item.text === true || item.text === "true" )  {
      item.icon = "mdi mdi-checkbox-marked-outline";
    }
    else if( item.text == null ) {
      item.icon = "";
    }
    else {
      item.icon = "mdi mdi-checkbox-blank-outline";
    }
    let $cell = $( $.views.Html( '/lib/grid/inplace-onoff', item ) );

    // cell disabled status
    let isDisabled = false;
    if( 'status' in item )  {
      $cell.addClass( item['status'] );
      if( item['status'].indexOf( "disabled" ) >= 0 )  {
        $cell.removeClass( "grid-inplace-onoff" );
        isDisabled = true;
      }
    }

    // call inplace edit function
    if( !isDisabled  && $.isFunction( this.options['inplaceEdit'] ) )  {
      $cell.on( 'click', ( ev ) => {
        let changedRecord = this.options['inplaceEdit']( headerId, record );
        if( typeof changedRecord == "object" )  {
          this.dataset.Update( changedRecord );
        }
        ev.stopPropagation();
      } );

    }
    return $cell;
  }  

  OnDrawInplaceLink( item, headerId, record, recordId )
  {
    let $cell = null;
    let isCellHover = this.options['focus'] == "cell";

    if( ( typeof item['text'] == "string" && item['text'].length > 0 ) 
     && ( typeof item['icon'] == "string" && item['icon'].length > 0 )  
      ) {
     $cell = $( $.views.Html( '/lib/grid/data-cell', item ) );
    }
    else if( typeof item['text'] == "string" && item['text'].length > 0 ) {
      $cell = $( $.views.Html( '/lib/grid/data-cell-text', item ) );
    }
    else if( typeof item['icon'] == "string" && item['icon'].length > 0 ) {
      $cell = $( $.views.Html( '/lib/grid/data-cell-icon', item ) );
    }
    else  {
      $cell = $( $.views.Html( '/lib/grid/data-cell-empty', item ) );
    }
    if( isCellHover ) {
      $cell.addClass( "hover" );
    }
    
    $cell.addClass( "grid-inplace-link" );

    if( $.isFunction( this.options['inplaceLink'] ) )  {
      $cell.on( 'click', ( ev ) => {
        this.options['inplaceLink']( headerId, record );
      } );
    }


    return $cell;
  }  

  FitControlHeight()
  {
    if( this.options['height'] != "fit" ) {
      return;
    }
    
    let length = ( this.dataview.length + 1 );
    if( this.options['heightMin'] > 0 && length < this.options['heightMin'] )  {
      length = this.options['heightMin'];
    }
    if( this.options['heightMax'] > 0 && length > this.options['heightMax'] ) {
      length = this.options['heightMax'];
    }

    if( length != this.controlHeight )  {
      this.controlHeight = length;
      this.$element.css( 'height', this.lineHeight * length + 3 );
      this.TriggerLayoutEvent();
    }
  }

  // ----------------------------------------------------
  //    EVENTS
  // ----------------------------------------------------

  // ----------------------------------------------------
  //   scrollbar changed window
  //
  OnScrollWindow( scrollInfo )
  {
    this.scrollInfo = scrollInfo;
    if( this.options['dataDrawWindow'] )
    this.OnDrawDataWindow();
  }

  // ----------------------------------------------------
  //   click item event handler
  //
  OnClickItem( itemData, isShift )
  {
    if( !this.dataview.ItemHasStatus( itemData.id, 'disabled' ) ) {
      this._SelectItem( itemData['id'], isShift );
    }
  }  

  // ----------------------------------------------------
  //   click item event handler
  //
  OnDblClickItem( itemData, isShift )
  {
    if( !this.dataview.ItemHasStatus( itemData.id, 'disabled' ) ) {
      this._FireDblClick( itemData['id'] );
    }
  }


  // ----------------------------------------------------
  //   click item event handler
  //
  OnClickHeader( itemData )
  {
    let $item = itemData['$item'];
    let headerId = itemData['id'];

    // clear previous selection
    let currentHeader = this.options['header'][ this.options['sort'] ];
    if( currentHeader )  {
      let $i = currentHeader['$item'].find( ".cell-sorter > i" );
      $i.removeClass( this.options['sortIconAsc'] );
      $i.removeClass( this.options['sortIconDesc'] );
      $i.addClass( "mdi-none" );
    }

    // just change sort order
    if( this.options['sort'] == headerId )  {
      this.options['sortOrder'] = ( this.options['sortOrder'] == '+' ) ? '-' : '+';
    }
    else  {
      this.options['sort'] = headerId;
    }

    // append style
    let $i = $item.find( ".cell-sorter > i" );
    $i.removeClass( "mdi-none" );
    if( this.options['sortOrder'] == '+' )  {
      $i.addClass( this.options['sortIconAsc'] );
    }
    else  {
      $i.addClass( this.options['sortIconDesc'] );
    }

    // do actual sorting
    this.dataview.Sort( this.options['sort'], this.options['sortOrder'] );

  }


  // ----------------------------------------------------
  //   enter focus event handler
  //
  OnEnterFocus( )
  {
    if( this.$scroll )  {
      this.$scroll.scrollbar( "recalc" );
    }
    this.$element.addClass( 'focus' );
  }

  // ----------------------------------------------------
  //   enter focus event handler
  //
  OnLeaveFocus( )
  {
    this.$element.removeClass( 'focus' );
  }

  // ----------------------------------------------------
  //   keyboard handler
  //
  OnKeydown( keycode, e )
  {
    if( keycode == KEY.UP )  {
      if( this.selectIndex > 0 )  {
        var newIndex = this.selectIndex - 1;
        var itemData = this.dataview.GetItemByIndex( newIndex );
        this._SelectItem( itemData.id, false, true );
        
        var itemPos = newIndex * this.lineHeight;
        var itemHeight = this.lineHeight;
        this.$scroll.scrollbar( "show", itemPos, itemPos + itemHeight );

        e.preventDefault( );
      }
    }
    else if( keycode == KEY.DOWN )  {
      if( this.selectIndex < ( this.dataview.length - 1 ) )  {
        var newIndex = this.selectIndex + 1;
        var itemData = this.dataview.GetItemByIndex( newIndex );
        this._SelectItem( itemData.id, false, true );

        var itemPos = newIndex * this.lineHeight;
        var itemHeight = this.lineHeight;
        this.$scroll.scrollbar( "show", itemPos, itemPos + itemHeight );

        e.preventDefault( );
      }
    }
    else if( keycode == KEY.POS1 )  {
      var newIndex = 0;
      var itemData = this.dataview.GetItemByIndex( newIndex );
      this._SelectItem( itemData.id, false, true );

      var itemPos = newIndex * this.lineHeight;
      var itemHeight = this.lineHeight;
      this.$scroll.scrollbar( "show", itemPos, itemPos + itemHeight );

      e.preventDefault();
    }
    else if( keycode == KEY.END ) {
      var newIndex = this.dataview.length - 1 ;
      var itemData = this.dataview.GetItemByIndex( newIndex );
      this._SelectItem( itemData.id, false, true );

      var itemPos = newIndex * this.lineHeight;
      var itemHeight = this.lineHeight;
      this.$scroll.scrollbar( "show", itemPos, itemPos + itemHeight );

      e.preventDefault();
    }

    /*else if( keycode == KEY.ENTER || keycode == KEY.RETURN )  {
      this._FireResult( );
      this._HideDroplist( );
      this._DeInitDroplist( );
    }
    else if( keycode == KEY.ESC )  {
      if( ( 'searchString' in this ) && ( this.searchString.length > 0 ) ) {
        this.OnClickClearSearch( );
      }
      else  {
        this._HideDroplist( );
        this._DeInitDroplist( );
        this._UpdateDroplistHead( );
      }
    }
    else if( this.$input )  {
      if( !this.$input.is( ":focus") )  {
        this.selectIndex = -1;
        this.$input.focus();
      }
    }*/

  }  

  // ----------------------------------------------------
  //   keyboard handler
  //
  OnSearch( newSearch )
  {
    if( newSearch != this.searchString ) {
      this.searchString = newSearch;
      this.dataview.Search( this.searchString );
    }
  }

  // ----------------------------------------------------
  //    HELPER
  // ----------------------------------------------------

  // ----------------------------------------------------
  //   helper: select an item
  //
  _SelectItem( id, isShiftKey, forceSingle )
  {
    // just click item
    if( this.options['selection'] == 'click' )  {
      this._FireClick( id );  
      return;
    }
    
    // no selection or click
    if( this.options['selection'] == 'none' )  {
      return;
    }

    if( forceSingle ) {
      this.dataview.ItemClearSelections();
      this.dataview.ItemSelect( id );
    }
    else if( ( this.options['selection'] == 'singleplus' && isShiftKey )
          || ( this.options['selection'] == 'multi' )
           ) {
      this.dataview.ItemToggleSelection( id, false );
    }
    else {
      this.dataview.ItemClearSelections();
      this.dataview.ItemSelect( id );
    }

    this.selectIndex = this.dataview.GetIndexFromId( id );

    this._FireResult( );

  }

  // ----------------------------------------------------
  //   selection
  //
  SetSelection( idList )
  {
    // no selection
    if( this.options['selection'] == 'none'
     || this.options['selection'] == 'click'
    ) {
      return;
    }

    //  multiple items given
    if( Array.isArray( idList ) && idList.length > 0 ) {

      // multiselect?
      if( this.options['selection'] == 'multi'
       || this.options['selection'] == 'singleplus'
        ) {
        for( let i = 0; i < idList.length; i++ ) {
          this.dataview.ItemSelect( idList[ i ] );
        }
      }
      // single: select id[ 0 ]
      else {
        this.dataview.ItemClearSelections();
        this.dataview.ItemSelect( idList[0] );
      }

      // scroll to first item
      if( idList.length > 0 ) {
        this._ScrollToItem( idList[ 0 ] );
      }
    }
    //  single item given
    else if( typeof( idList ) == "string" ) {
      this.dataview.ItemClearSelections();
      this.dataview.ItemSelect( idList );
      this._ScrollToItem( idList );
    }
    //  no item given
    else  {
      this.dataview.ItemClearSelections();
    }

  }

  // ----------------------------------------------------
  //   overridable: Validate 
  //
  OnValidate( eventName )
  {
    // do validation (delegate to upper layer)
    return this.ManageValidate( this.dataview, eventName );
  }

  // ----------------------------------------------------
  //   additional: error
  //
  OnDrawError()
  {
    return this.ControlShowMessage( this.$element, this.options.error, this.options.warning, this.options.errorAnchor );
  }

  // ----------------------------------------------------
  //   test height for an item
  //  
  _CalculateItemHeight( $content )
  {
    var preview = { id: '1', icon: 'mdi mdi-home', text: 'TEST' };
    var $row = $( $.views.Html( "/lib/grid/test", preview ) );
    $content.append( $row );
    var height = $row.outerHeight();
    $row.remove();
    return height;
  }

  // ----------------------------------------------------
  //   helper: send result event
  //
  _FireResult()
  {
    this.options.selected = this.dataview.ItemGetSelected();
    this.options['value'] = this.options.selected;
    if( $.isFunction( this.options['result'] ) ) {
      this.options['result']( this.options.selected );
    }
  } 

  // ----------------------------------------------------
  //   helper: send click event
  //
  _FireClick( id )
  {
    if( $.isFunction( this.options['click'] ) ) {
      this.options['click']( id );
    }
  } 

  // ----------------------------------------------------
  //   helper: send click event
  //
  _FireDblClick( id )
  {
    if( $.isFunction( this.options['dblclick'] ) ) {
      this.options['dblclick']( id );
    }
  } 

  // ----------------------------------------------------
  //   helper: select an item
  //
  _ScrollToItem( id )
  {
    var index = this.dataview.GetIndexFromId( id );
    var indexPos = index * this.lineHeight;
    this.$scroll.scrollbar( "show", indexPos, indexPos + this.lineHeight );
    /*let itemData = this.dataview.GetItem( id );
    if( itemData && ( '$item' in itemData ) ) {
      let itemHeight = itemData.$item.outerHeight();
      this.$scroll.scrollbar( "show", itemData.y, itemData.y + itemHeight );
    }*/
  }  

  //---------------------------------------------------------------------------
  //  INTERFACE functions
  //---------------------------------------------------------------------------

  // ----------------------------------------------------
  //   interface: $tag.grid( "item", itemId, "update", newRecord );
  //  
  IF_ItemUpdate( itemId, newRecord )
  {
    let record = this.dataset.Get( itemId );
    record = $.extend( record, newRecord );
    this.dataset.UpdateById( itemId, record );
  }

  // ----------------------------------------------------
  //   interface: $tag.grid( "item", itemId, "get" );
  //  
  IF_ItemGet( itemId )
  {
    return $.clone( this.dataset.Get( itemId ) );
  }

  // ----------------------------------------------------
  //   interface: $tag.tree( "item", itemId, "selected", true );
  //  
  IF_ItemSetSelect( itemId, param )
  {
    if( param === true || param === undefined ) {
      this.dataview.ItemSelect( itemId, ( this.options['selection'] == "single" ) );
    }
    else if( param === false ) {
      this.dataview.ItemUnselect( itemId );
    }
    else {
      this.dataview.ItemToggleSelection( itemId );
    }
    this._FireResult();
  }

  // ----------------------------------------------------
  //   interface: let iosSelected = $tag.tree( "item", itemId, "selected" );
  //  
  IF_ItemGetSelect( itemId )
  {
    return this.treeview.ItemHasStatus( itemId, 'selected' );
  }  

  //-----------------------------------------------------------------
  //  interface: $tag.grid( "item", itemId, "status", "red" );
  //  
  IF_ItemSetStatus( itemId, status )
  {
    this.dataview.ItemSetStatus( itemId, status );
  }

  //-----------------------------------------------------------------
  //  interface: $tag.grid( "item", itemId, "status+", "warning" );
  //  
  IF_ItemAddStatus( itemId, status )
  {
    this.dataview.ItemAddStatus( itemId, status );
  }

  //-----------------------------------------------------------------
  //  interface: $tag.grid( "item", itemId, "status-", "warning" );
  //  
  IF_ItemRemoveStatus( itemId, status )
  {
    this.dataview.ItemRemoveStatus( itemId, status );
  }  

  //-----------------------------------------------------------------
  //  interface: let status = $tag.grid( "item", itemId, "status" );
  //  
  IF_ItemGetStatus( itemId, status )
  {
    this.dataview.ItemSetStatus( itemId, status );
  }

  // ----------------------------------------------------
  //   destruct
  //
  destroy()
  {
    this.$element.empty( );
  }

  // ----------------------------------------------------
  //   Interface: item option getter and setter
  //
  item( args )
  {
    // prequistes
    if( args.length < 2 ) {
      console.warn( "Tree('item') - parameter missing!" );
      return;
    }

    // check itemId
    let itemId = args[1];
    if( !this.dataview.Has( itemId ) ) {
      console.warn( "Tree('item') - id='" + itemId + "' not found!" );
      return;
    }

    // setter
    let command = args[2];
    switch( command ) {

      case "update": {
        this.IF_ItemUpdate( itemId, args[3] );
      } break;

      case "get": {
        return this.IF_ItemGet( itemId );
      }

      case "select": {
        this.IF_ItemSetSelect( itemId, args[3] );
      } break;

      case "selected": {
        return this.IF_ItemGetSelect( itemId );
      }

      case "status+": {
        if( args.length >= 4 )
          this.IF_ItemAddStatus( itemId, args[3] );
        else
          return this.IF_ItemGetStatus( itemId );
      } break;

      case "status-": {
        if( args.length >= 4 )
          this.IF_ItemRemoveStatus( itemId, args[3] );
        else
          return this.IF_ItemGetStatus( itemId );
      } break;

      case "status": {
        if( args.length >= 4 )
          this.IF_ItemSetStatus( itemId, args[ 3 ] );
        else
          return this.IF_ItemGetStatus( itemId );
      } break;
    }
  }

  //-------------------------------------------------------
  //   option: generic getter and setter
  //
  option( args )  
  {
    // setter
    if( args.length > 2 ) {
      switch( args[ 1 ] ) {
        case 'status': {
          if( ( typeof args[2] == "string" )
           && ( args[2].indexOf( "disabled" ) >= 0 )
            ) {
            this.ManageDisabledOverlay( true );
          }
          else if( args[2] == null
            && ( 'statusDefault' in this.options )
            && ( this.options['statusDefault'].indexOf( "disabled" ) >= 0 )
            ) {
            this.ManageDisabledOverlay( true );
          }
          else {
            this.ManageDisabledOverlay( false );
            this.options['status'] = this.ItemSetStatus( this.$element, args[2] );
          }
        }
        break;

        case 'data': {
          if( this.isSharedDataset ) {
            console.warn( "Listbox::option( 'data' ) - data must be updated in dataset!" )
          }
          else if( Array.isArray( args[ 2 ] ) ) {
            this.dataset.SetData( args[ 2 ] );
            this.dataview.Update( );
          }
          else {
            console.warn( "Listbox::option( 'data' ) - given data must be array!" )
          }
        }
        break;

        case 'selected': {
          this.SetSelection( args[ 2 ] );
        } 
        break;

        case 'placeholderIcon':
        case 'placeholder': {
          this.options[ args[ 1 ] ] = args[ 2 ];
          this.OnDrawData();
        } 
        break;

        case 'search':  {
          this.OnSearch( args[ 2 ] );
        }
        break;

        default: {
          this.options[ args[ 1 ] ] = args[ 2 ];
          this.OnDraw();
        }
      }
    }

    // getter
    if( args[ 1 ] == "data" )  {
      return this.dataset.GetData( );
    }
    if( args[ 1 ] == "dataset" ) {
      return this.dataset;
    }
    if( args[ 1 ] == "selected" ) {
      return this.dataview.ItemGetSelected();
    }
    else  {
      return this.options[ args[ 1 ] ];
    }
  }  

};

/* ---------------------------------------------------------------------------
 *   MENU - plugin definition
 * ------------------------------------------------------------------------ */


// ----------------------------------------------------
//   plugin main function
//
$.fn.grid = function( option )  
{
  var data    = $( this ).data( "ctrl" ),
      options = ( typeof option == "object" && option );

  // remove for re-create
  if( typeof option == "object" && data )  {
    data.destroy( );
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data ) {
    var instance = new WfcGrid( this, options );
    $( this ).data( "ctrl", instance );
  }

  // if called with a string, treat this as a function call
  else if( typeof option == "string" ) {
    // destroy outside of class
    if( option == "destroy" )  {
      data.destroy( );
      $( this ).off();
      $( this ).data( "ctrl", null );  
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) )  {
      return data[ option ]( arguments );
    }
    else  {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.grid.defaults = {
  size: -1,
  height: 3,
  heightMin: 0,
  heightMax: 0,
  border: true,
  focus: true,
  status: 'normal no-hover',
  selection: 'single',
  selected: [],
  selectedIndex: -1,
  placeholder: "",
  placeholderIcon: null,
  data: [],
  dataId: 'id',
  dataFilter: null,
  dataDrawWindow: true,
  dataDrawWindowThreshold: 500,
  sort: null,
  sortOrder: '+',
  sortIconAsc: "mdi-chevron-down",
  sortIconDesc: "mdi-chevron-up",
  headerHidden: false,
  header: {},
  scrollbar: 'static', // 'overlay', 'static'
  renderCell: null,
  renderStatus: null,
  result: null,
  click: null
};


}( window.jQuery );
 /*
 *  ACCESS 5.0 Web Framework
 *  range 
 */

!function( $ ) {

 
/* ---------------------------------------------------------------------------
  *   BUTTON - class definition
  * ------------------------------------------------------------------------ */

class WfcRange extends JqControl
{
  // ----------------------------------------------------
  //   constructor (will be called by base class)
  //
  OnInit( $element, options )
  {
    // class variables
    this.options  = $.extend( true, [], $.fn.range.defaults, options );
    this.options['statusDefault'] = ( this.options['statusDefault'] || "" ) + " no-hover";
    this.$element.addClass( 'range' );

    this.$handle = null;
    this.$ltrack = null;
    this.$rtrack = null;
    this.$ttip = null;

    this.mouse = null;
    this.rangeX = 0;
    this.trackWidth = 0;

    // language
    this.EncodeLang();

  }

  // ----------------------------------------------------
  //   encode texts 
  //
  EncodeLang()
  {
    this.options['text'] = $.lang( this.options['text'] );
    this.options['tooltip'] = $.lang( this.options['tooltip'] );    
  }

  // ----------------------------------------------------
  //   recalc layout after change
  //
  OnDraw( )  
  {
    var selfCtrl = this;
    this._DetachEvents( );

    let html = $.views.Html( "/lib/range", { } );
    this.$element.html( html );

    // attach frame 
    this.ItemInitialize( this.$element, { status: "normal no-hover" } );


    // width (size in character or percent)
    this.$element.css( 'width', this.ControlCalculateSize( this.options['size'] ) );
    this.$handle = this.$element.find( '.range-handle' );
    this.$ltrack = this.$element.find( '.range-active' );
    this.$rtrack = this.$element.find( '.range-nonactive' );
    this.$ttip = this.$element.find( '.range-tooltip' );

    if( this.options['valueRange'] <= 0 )  {
      this.$handle.css( "display", "none" );
      this.$ltrack.css( "display", "none" );
      this.$ttip.css( "opacity", "0" );
    }

    // get metrics
    this.trackWidth = this.$element.width();
    this.rangeX = this.trackWidth - this.$handle.width();

    // move handle to the desired position
    if( this.options['valueRange'] > 0 ) {
      this._SetHandle( this.options['value'] );
      this._AttachEvents();
    }
  }

  // ----------------------------------------------------
  //   install event handlers
  //
  _AttachEvents( )  
  {
    if( this.options.status == "readonly" || this.options.status == "disabled" )
      return;

    var self = this;
    var offs = this.$element.offset();
    var correction = 0;

    // click on track -> move there      
    this.$element.click( function( e )  {
      self._MoveHandle( e.clientX - offs.left );
      self._HandleValueChanged( self.options.value, "change" );
      if( $.isFunction( self.options.result ) ) {
        self.options.result( self.options.value );
      }
    } );

    this.mouse = new MouseCapture;
    this.mouse.start( this.$handle,
      // mouse down
      function( e )  {
        self.trackWidth = self.$element.width();
        correction = e.clientX - offs.left - self.$handle.position().left;
        self.rangeX = self.trackWidth - self.$handle.width();
      },
      // move mouse
      function( e )  {
        self._MoveHandle( e.clientX - offs.left - correction );
      },
      // mouse up
      function( e )  {
        self._HandleValueChanged( self.options.value, "change" );
      }
    );
  } 

  // ----------------------------------------------------
  //   free event handlers
  //
  _DetachEvents( )
  {
    this.$element.off();
    if( this.mouse ) {
      this.mouse.stop();
      delete this.mouse;
      this.mouse = null;
    }
  }  

  // ----------------------------------------------------
  //   value changed
  //
  _HandleValueChanged( value, event )  
  {
    // delegate change event to upper layer
    this.ManageValidate( value, event );
    this.ManageModified( 1 );

    // fire result event
    if( $.isFunction( this.options['result'] ) )  {
      this.options.result( this.options.value );
    }

  }


  // ----------------------------------------------------
  //   move handle by mouse
  //
  _MoveHandle( clientX )  
  {
    if( clientX < 0 ) clientX = 0;
    if( clientX > this.rangeX ) clientX = this.rangeX;
    let value = Math.round( clientX * this.options['valueRange'] / this.rangeX );
    
    // min-value
    if( value < this.options['valueMin'] ) {
      value = this.options['valueMin'];
    }
    // max-value
    if( value > this.options['valueMax'] ) {
      value = this.options['valueMax'];
    }
    
    this._SetHandle( value );
  }

  // ----------------------------------------------------
  //   set the handle to value
  //
  _SetHandle( value )
  {
    this.options['value'] = value;

    var clientX = Math.floor( value * this.rangeX / this.options['valueRange'] );
    this.$handle.css( 'left', clientX );
    clientX = Math.round( clientX + this.$handle.width() / 2 );
    this.$ltrack.css( 'width', clientX );
    let tooltip = value.toString();
    if( $.isFunction( this.options.renderTooltip ) ) {
      tooltip = $.lang( this.options.renderTooltip( value ) );
    }
    this.$ttip.css( 'left', clientX )
              .text( tooltip );
  }

  // ----------------------------------------------------
  //   destroy to detach events
  //
  destroy( )
  {
    this._DetachEvents( );
    this.$element.empty();
  }

  // ----------------------------------------------------
  //   option getter and setter
  //
  option( args )  
  {
    // setter
    if( args.length > 2 ) {
      switch( args[1] ) {
        case 'status': {
          if( args[2] == "disabled" ) {
            this.ManageDisabledOverlay( true );
          }
          else {
            this.ManageDisabledOverlay( false );
            this.options['status'] = this.ItemSetStatus( this.$element, args[2] );
          }
        }
        break;

        case 'error': {
          this.options['error'] = $.lang( args[2] );
          this.OnDrawError();
        }
        break;

        default:  {
          this.options[args[1]] = $.lang( args[2] );
          this.OnDraw();
        }
      }
    }

    // getter
    return this.options[ args[ 1 ] ];
  }

  // ----------------------------------------------------
  //   validate
  //
  OnValidate( eventName )
  {
    return this.ManageValidate( this.options['value'], eventName );
  }    

}

   
/* ---------------------------------------------------------------------------
 *   BUTTON - plugin definition
 * ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   plugin main function
//
$.fn.range = function( option )  
{
  var data    = $( this ).data( "ctrl" ),            
      options = ( typeof option == "object" && option );  
  
  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data )  {
    var instance = new WfcRange( this, options ); 
    $( this ).data( "ctrl", instance );
  }
  
  // if called with a string, treat this as a function call
  else if( typeof option == "string" )  {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) )  {
      return data[ option ]( arguments );
    }
    else  {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;   
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.range.defaults = {
  size: 0,
  status: "normal",
  value: 0,
  valueRange: 100,
  valueMin: 0,
  valueMax: 100,
  valueSteps: 1,
  result: $.noop
};

  
}( window.jQuery );
  /*
 *  ACCESS 5.0 Web Framework
 *  optional 
 */

!function( $ ) {

 
/* ---------------------------------------------------------------------------
  *   OPTIONAL - class definition
  * ------------------------------------------------------------------------ */

class WfcOptional extends JqControl
{
  // ----------------------------------------------------
  //   constructor (will be called by base class)
  //
  OnInit( $element, options )
  {
    // class variables
    this.options = $.extend( true, [], $.fn.optional.defaults, options );
    this.$element.addClass( 'optional' );
  }


  // ----------------------------------------------------
  //   recalc layout after change
  //
  OnDraw( )  
  {
  }

  // ----------------------------------------------------
  //   methods show/hide/radio
  //
  show( args )
  {
    let dataId = args[ 1 ];
    let $optionalElement = this.$element.find( "[data-id='" + dataId + "']" );
    if( $optionalElement.length > 0 )  {
      $optionalElement.attr( 'data-status', "normal" );
      $optionalElement.broadcast( "event:show" );
    }
    
    // inform upper layers
    this.TriggerLayoutEvent();    
  }

  hide( args )
  {
    let dataId = args[ 1 ];
    let $optionalElement = this.$element.find( "[data-id='" + dataId + "']" );
    if( $optionalElement.length > 0 ) {
      $optionalElement.attr( 'data-status', "hidden" );
      $optionalElement.broadcast( "event:hide" );
    }

    // inform upper layers
    this.TriggerLayoutEvent();    

  }

  radio( args )
  {
    let dataId = args[1];
    let $optionalElementList = this.$element.find( "[data-id]" );
    if( $optionalElementList.length > 0 ) {
      for( let i = 0; i < $optionalElementList.length; i++ )  {
        let $optionalElement = $( $optionalElementList[ i ] );
        if( $optionalElement.attr( 'data-id') == dataId )  {
          $optionalElement.attr( 'data-status', "normal" );
          $optionalElement.broadcast( "event:show" );
        }
        else  {
          $optionalElement.attr( 'data-status', "hidden" );
          $optionalElement.broadcast( "event:hide" );
        }
      }
    }

    // inform upper layers
    this.TriggerLayoutEvent();    

  }

  option( args )
  {
    // do nothing
  }

}

   
/* ---------------------------------------------------------------------------
 *   OPTIONAL - plugin definition
 * ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   plugin main function
//
$.fn.optional = function( option )  
{
  var data    = $( this ).data( "ctrl" ),            
      options = ( typeof option == "object" && option );  
  
  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data )  {
    var instance = new WfcOptional( this, options ); 
    $( this ).data( "ctrl", instance );
  }
  
  // if called with a string, treat this as a function call
  else if( typeof option == "string" )  {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) )  {
      return data[ option ]( arguments );
    }
    else  {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;   
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.optional.defaults = {
  size: 0
};

  
}( window.jQuery );
  /*
 *  CYTEL Viewable Framework
 *  MULTI  
 */

!function( $ ) {

 
/* ---------------------------------------------------------------------------
 *   MULTI - class definition
 * ------------------------------------------------------------------------ */

class WfcAccordion extends JqControl
{
  // ----------------------------------------------------
  //   constructor (will be called by base class)
  //
  OnInit( $element, options )
  {
    // class variables
    this.options = $.extend( true, [], $.fn.accordion.defaults, options );
    this.$element.addClass( 'accordion' );
    this.$container = null;

    // language
    this.EncodeLang();

  }

  // ----------------------------------------------------
  //   encode texts 
  //
  EncodeLang()
  {
    this.options['data'] = $.lang( this.options['data'] );
  }

  // ----------------------------------------------------
  //   recalc layout after change
  //
  OnDraw( )  
  {
    // control sizes
    if( 'size' in this.options )  {
      let width = this.ControlCalculateSize( this.options['size'] );
      this.$element.css( 'width', width );
    }
    if( 'height' in this.options ) {
      let height = this.ControlCalculateSize( this.options['height'] );
      this.$element.css( 'height', height );
    }
    if( this.options['border'] == true ) {
      this.$element.addClass( 'control-border' );
    }

    // standard class
    this.$element.addClass( "accordion normal" );

    // find all children that are items and modify them
    let $itemList = this.$element.find( ".accordion-item" );

    // HTML: build expanded list 
    for( let itemElement of $itemList ) {
      let $item = $( itemElement );
      let isCollapsed = $item.hasClass( "collapsed" );
      if( ! isCollapsed )  {
        $.array.Add( this.options['expanded'], $item.attr( "data-id" ) );
      }
    }
    //console.log( "HTML:", this.options['expanded'] );

    // DATA: build expanded list 
    for( let dataItem of this.options['data'] )  {
      if( ( 'status' in dataItem ) && ( 'id' in dataItem ) )  {
        if( dataItem['status'].indexOf( "collapsed" ) >= 0 )  {
          $.array.Remove( this.options['expanded'], dataItem['id'] );
        }
        else {
          $.array.Add( this.options['expanded'], dataItem['id'] );
        }
      }
    }
    //console.log( "DATA:", this.options['expanded'] );

    // REGISTRY: read expanded list
    this.ReadStatusFromRegistry();
    //console.log( "REG:", this.options['expanded'] );

    for( let itemElement of $itemList )  {
      this.OnDrawItem( $( itemElement ) );
    }
  }

  // ----------------------------------------------------
  //   draw: modify an item
  //
  OnDrawItem( $item )
  {
    // get 
    let id = $item.attr( "data-id" );
    let type = $item.attr( "data-type" ) || "flex";

    // prepare item data (read from html tags)
    let isExpanded = $.array.Has( this.options['expanded'], id );
    let isSelected = $item.hasClass( "selected" );
    let itemData = this.options['data'].find( ( element ) => element['id'] == id );

    // get item data from html code
    let $h3 = $item.find( ">:first-child" );

    // find item in options.data, create if not exists
    if( itemData == null ) {
      itemData = {
        id: id,
        text: $h3.text(),
        type: type,
        status: ( isExpanded ? "expanded" : "collapsed" ) + ( isSelected ? " selected" : "" ),
        height: 16
      };
      this.options['data'].push( itemData );
      itemData = this.options['data'].find( ( element ) => element['id'] == id );
    }
    else {
      itemData['type'] = itemData['type'] || type;
      itemData['text'] = itemData['text'] || $h3.text();
      itemData['status'] = ( isExpanded ? "expanded" : "collapsed" ) + ( isSelected ? " selected" : "" );
      itemData['height'] = itemData['height'] || 16;
    }

    // create new html layout
    let $itemContainer = $( $.views.Html( "/lib/accordion/item-container", itemData ) );

    // move content 
    $h3.remove();
    $item.removeClass( "expanded selected" );
    let $content = $itemContainer.closest( ".content" );
    let $children = $item.children();
    $content.append( $children );

    // attach to container
    $itemContainer.prependTo( $item );
    let $head = $itemContainer.closest( ".head" );

    // no splitter? remove it
    let $splitter = $itemContainer.closest( ".splitter" );
    if( itemData["type"].indexOf( "fix" ) < 0 )  {
      $splitter.remove();
    }
    else  {
      this.ItemAttachSplitter( itemData, $splitter );
    }

    // set resulting html tags to itemData
    itemData['$container'] = $itemContainer;
    itemData['$item'] = $item;
    itemData['$head'] = $head;
    itemData['$content'] = $content;
    itemData['$splitter'] = $splitter;
    itemData['minHeight'] = this.ControlCalculateHeight( this.options['minHeight'] ) + itemData['$head'].outerHeight();
    if( typeof itemData['height'] != "string" ) {
      itemData['height'] = ( this.ControlCalculateHeight( itemData['height'] ) + itemData['$head'].outerHeight() ).toString();
    }

    this.ItemManageStatus( itemData );
    this.ItemAttachEvents( itemData );

  }

  //-----------------------------------------------------------------
  //   draw: splitter
  //
  ItemAttachSplitter( itemData, $splitter )
  {
    itemData['mouse'] = new MouseCapture;
    itemData['mouse'].start( $splitter,
      // --- mouse down ---
      ( e ) => {
        itemData['mouseStartPos'] = e.clientY;
        itemData['mouseElementSize'] = itemData['$item'].outerHeight();
      },
      // --- move mouse ---
      ( e ) => {
        var mouseDy = e.clientY - itemData['mouseStartPos'];
        var mouseHeight = itemData['mouseElementSize'] + mouseDy;
        if( mouseHeight >= itemData['minHeight'] ) {
          itemData['$item'].height( mouseHeight );
        }
      },
      // --- mouse up ---
      ( e ) => {
        itemData['height'] = ( itemData['$item'].height() ).toString();
        this.$element.broadcast( "event:layout-changed" );
        this.SaveStatusToRegistry( );
      }
    );

  }

  //-----------------------------------------------------------------
  //   draw: event
  //
  ItemAttachEvents( itemData )
  {
    itemData['$head'].click( () => {
      let itemStatusList = itemData['status'].split( " " );
      let isExpanded = ( itemStatusList.indexOf( "expanded" ) >= 0 );
      if( isExpanded )  {
        $.array.Remove( this.options['expanded'], itemData['id'] );
        let index = itemStatusList.indexOf( "expanded" );
        itemStatusList[ index ] = "collapsed";
      }
      else  {
        $.array.Add( this.options['expanded'], itemData['id'] );
        let index = itemStatusList.indexOf( "collapsed" );
        itemStatusList[ index ] = "expanded";
      }
      itemData['status'] = itemStatusList.join( " " );
      this.ItemManageStatus( itemData );
      this.SaveStatusToRegistry();
      this.$element.broadcast( "event:layout-changed" );
    } );
  }

  //-----------------------------------------------------------------
  //   status
  //
  ItemManageStatus( itemData )
  {
    let itemStatusList = itemData['status'].split( " " );
    let itemTypeList = itemData['type'].split( " " );

    // overwrite expanded status from expanded-list
    if( this.options['expanded'].indexOf( itemData['id'] ) >= 0 )  {
      $.array.Remove( itemStatusList, "collapsed" );
      $.array.Add( itemStatusList, "expanded" );
      itemData['status'] = itemStatusList.join( " " );
    }

    // get some informations
    let isExpanded = ( itemStatusList.indexOf( "expanded" ) >= 0 );// || (  );
    let isSelected = ( itemStatusList.indexOf( "selected" ) >= 0 );
    let isFix = ( itemTypeList.indexOf( "fix" ) >= 0 );
    let $icon = itemData['$head'].find( "i" );
    $icon.removeClass( this.options.chevronIcons["expanded"] );
    $icon.removeClass( this.options.chevronIcons["collapsed"] );

    // set status to container
    if( isExpanded && isFix ) {
      itemData['$item'].addClass( "expanded-fix" );
      itemData['$item'].removeClass( "expanded-flex collapsed" );
      itemData['$item'].css( "height", itemData['height'] );
      itemData['$content'].css( "display", "" );
      itemData['$splitter'].css( "display", "" );
      $icon.addClass( this.options.chevronIcons["expanded"] );
      $.array.Add( this.options['expanded'], itemData['id'] );
    }
    else if( isExpanded ) {
      itemData['$item'].addClass( "expanded-flex" );
      itemData['$item'].removeClass( "expanded-fix collapsed" );
      itemData['$item'].css( "height", "" );
      itemData['$content'].css( "display", "" );
      itemData['$splitter'].css( "display", "none" );
      $icon.addClass( this.options.chevronIcons["expanded"] );
      $.array.Add( this.options['expanded'], itemData['id'] );
    }
    else {
      itemData['$item'].addClass( "collapsed" );
      itemData['$item'].removeClass( "expanded-fix expanded-flex" );
      itemData['$item'].css( "height", itemData['$head'].outerHeight() );
      itemData['$content'].css( "display", "none" );
      itemData['$splitter'].css( "display", "none" );
      $icon.addClass( this.options.chevronIcons["collapsed"] );
      $.array.Remove( this.options['expanded'], itemData['id'] );
    }

    // set status to head
    if( isSelected ) {
      itemData['$head'].removeClass( "default" );
      itemData['$head'].addClass( "selected" );
    }
    else {
      itemData['$head'].removeClass( "selected" );
      itemData['$head'].addClass( "default" );
    }
  }

  //-----------------------------------------------------------------
  //   Registry, save status
  //
  SaveStatusToRegistry( )
  {
    // rebuild expanded list
    this.options['expanded'] = [];
    let sizesList = {};
    for( let item of this.options['data'] )  {
      if( item['status'].indexOf( "expanded" ) >= 0 )  {
        this.options['expanded'].push( item['id'] );
      }
      sizesList[ item['id'] ] = item['height'];
    }

    // save registry
    if( this.options['regkey'] != null ) {
      $.registry( this.options['regkey'], { expanded: this.options['expanded'], sizes: sizesList } );
    }

  }

  //-----------------------------------------------------------------
  //   Registry, save status
  //
  ReadStatusFromRegistry()
  {
    // read
    let regStatus = $.registry( this.options['regkey'] );
    if( ! regStatus )
      return;

    this.options['expanded'] = regStatus['expanded'];

    // rebuild expanded list
    for( let item of this.options['data'] ) {
      item['height'] = ( regStatus['sizes'][ item['id'] ] ).toString() || this.options['minHeight'];
    }


  }

  //-----------------------------------------------------------------
  //   Interface: set selected
  //
  SetSelection( selectedList )
  {
  }

  //-----------------------------------------------------------------
  //   Interface: get selected
  //
  GetSelected(  )
  {
    return this.options['selected'];
  }

  //-----------------------------------------------------------------
  //   Interface: set selected
  //
  SetExpanded( selectedList )
  {
  }

  //-----------------------------------------------------------------
  //   Interface: set selected
  //
  GetExpanded(  )
  {
    return this.options['expanded'];
  }


  //-----------------------------------------------------------------
  //  INTERFACE
  //-----------------------------------------------------------------


  //-----------------------------------------------------------------
  //   Interface: option getter and setter
  //
  option( args )  
  {
    // ---- setter ----
    if( args.length > 2 ) {
      switch( args[ 1 ] ) {

        case 'status': {
          if( args[ 2 ] == "disabled" ) {
            this.ManageDisabledOverlay( true );
          }
          else {
            this.ManageDisabledOverlay( false );
          }
        }break;

        case 'selected': {
          this.SetSelection( args[ 2 ] );
        } break;

        case 'expanded': {
          this.SetExpanded( args[ 2 ] );
        } break;

      }
      return null;
    }

    // getter
    switch( args[ 1 ] ) {

      case 'selected': {
        return this.GetSelected();
      }

      case 'expanded': {
        return this.GetExpanded();
      }

      default: {
        return this.options[ args[ 1 ] ];
      }
    }

  }// option


}

   
/* ---------------------------------------------------------------------------
 *   ACCORDION - plugin definition
 * ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   plugin main function
//
$.fn.accordion = function( option )  
{
  var data    = $( this ).data( "ctrl" ),            
      options = ( typeof option == "object" && option );  
  
  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data )  {
    var ctrl = new WfcAccordion( this, options );
    $( this ).data( "ctrl", ctrl );
  }
  
  // if called with a string, treat this as a function call
  else if( typeof option == "string" )  {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) )  {
      return data[ option ]( arguments );
    }
    else  {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;   
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.accordion.defaults = {
  size: -1,
  height: -1,
  border: false,
  minHeight: 3,
  status: "normal",
  regkey: null,
  data: [],
  selected: [],
  expanded: [],
  chevronIcons: {
    expanded: "mdi mdi-chevron-down",
    collapsed: "mdi mdi-chevron-right"
  },
  // events
  hidden: $.noop,
  shown: $.noop
};

  
}( window.jQuery );
 /*
 *  ACCESS 5.0 Web Framework
 *  badge 
 */

!function( $ ) {

 
/* ---------------------------------------------------------------------------
  *   BADGE - class definition
  * ------------------------------------------------------------------------ */

class WfcBadge extends JqControl
{
  //-----------------------------------------------------------------
  //   constructor (will be called by base class)
  //
  OnInit( $element, options )
  {
    // class variables
    this.options  = $.extend( true, [], $.fn.badge.defaults, options );

    // language
    this.EncodeLang();

  }

  //-----------------------------------------------------------------
  //   encode texts
  //
  EncodeLang()
  {
    this.options['text'] = $.lang( this.options['text'] );
  }

  //-----------------------------------------------------------------
  //   recalc layout after change
  //
  OnDraw( )  
  {
    // preconditions
    if( ! this.options['anchor'] ) {
      console.warn( "Badge: you need to set an anchor!" );
      return;
    }

    this.$anchor = $( this.options['anchor'] );
    if( this.$anchor.length == 0 )  {
      console.warn( "Badge: given anchor not found!" );
      return;
    }

    // re-create
    if( this.$badge )  {
      this.$badge.remove();
    }

    // if text length > 1 charcter, modify to "!"
    if( this.options['text'] && this.options['text'].length > 1 )  {
      this.options['text'] = "!";
    }

    // get style="" attribute
    let styleAttribute = this.$element.attr( "style" );


    // modifiy anchor and attach batch
    this.$anchor.css( "position", "relative" );

    // text or icon? 
    if( this.options['text'] )  {
      
      this.$badge = $( $.views.Html( '/lib/badge/text', this.options ) );
     
      // copy style attribute to new elemnt
      let badgeStyle = this.$badge.attr( "style" ) || "";
      badgeStyle = badgeStyle + " " + styleAttribute;
      this.$badge.attr( "style", badgeStyle );

      // append badge to anchor
      this.$badge.appendTo( this.$anchor );
    }
    else if( this.options['icon'] ) {

      this.$badge = $( $.views.Html( '/lib/badge/icon', this.options ) );

      // copy style attribute to new elemnt
      let badgeStyle = this.$badge.attr( "style" ) || "";
      badgeStyle = badgeStyle + " " + styleAttribute;
      this.$badge.attr( "style", badgeStyle );

      // append badge to anchor
      this.$badge.appendTo( this.$anchor );
    }


  }//OnDraw


  //-----------------------------------------------------------------
  //  INTERFACE
  //-----------------------------------------------------------------

  option( args )
  {
    // override default handler   
    if( args[ 1 ] == "text" || args[ 1 ] == "icon" )  {
      this.options[ args[ 1 ] ] = args[ 2 ];
      this.OnDraw( );
    }
  
  }//option

}

   
/* ---------------------------------------------------------------------------
 *   BADGE - plugin definition
 * ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   plugin main function
//
$.fn.badge = function( option )  
{
  var data    = $( this ).data( "ctrl" ),            
      options = ( typeof option == "object" && option );  
  
  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data )  {
    var instance = new WfcBadge( this, options ); 
    $( this ).data( "ctrl", instance );
  }
  
  // if called with a string, treat this as a function call
  else if( typeof option == "string" )  {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) )  {
      return data[ option ]( arguments );
    }
    else  {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;   
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.badge.defaults = {
  size: 0,
  anchor: null,
  text: null,
  icon: null,
  tooltip: null,
  status: "normal",
  click: $.noop
};

  
}( window.jQuery );
//=============================================================================
//
//  File: tree.js
//  (c) 2021 by CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          12.12.2020
//
//  Description:
//          Tree-Control using a Treeset / Treeview
//
//=============================================================================


!function( $ )
{
  // define keycodes
  var KEY = {
    BS: 8,
    TAB: 9,
    RETURN: 13,
    ENTER: 13,
    SHIFT: 16,
    CTRL: 17,
    ALT: 18,
    SCTRL: 20,
    ESC: 27,
    SPACE: 32,
    END: 35,
    POS1: 36,
    LEFT: 37,
    UP: 38,
    RIGHT: 39,
    DOWN: 40,
    DEL: 46
  };

//-----------------------------------------------------------------------------
//  CLASS DEFINITION
//-----------------------------------------------------------------------------

class WfcTree extends JqControl
{
  // ----------------------------------------------------
  //   init-function (called by constructor)
  //
  OnInit( $element, options )  
  {
    // initialize class variables
    this.$element = $element;
    this.$error = null;
    this.$scroll = null;
    this.$scrollable = null;
    this.dropscroll = null;
    this.options = $.extend( true, [], $.fn.tree.defaults, options );

    // special for expanded option
    if( this.options['expanded'] == "*" )  {
      this.isExpandAllRequired = true;
      this.options['expanded'] = [];
    }
    else if( ! Array.isArray( this.options['expanded'] ) ) {
      this.options['expanded'] = [];
    }
    else  {
      this.isExpandAllRequired = false;
    }

    // read registry
    if( this.options['regkey'] != null )  {
      let expanded = $.registry( this.options['regkey'] );
      if( Array.isArray( expanded ) )  {
        this.options['expanded'] = expanded;
      }
    }

    // data 
    if( this.options.data instanceof Treeset ) {
      this.treeset = this.options.data;
      this.treeview = new Treeview( this.treeset );
      this.treeview.AttachDomElement( this.$element );
      this.treeview.Options( { sortOrder: this.options.sort } );
      this.isSharedDataset = true;
    }
    else  {
      console.warn( "TREE: element 'data' must be instance if Treeset!" );
    }

  }
  
  //-------------------------------------------------------
  //  DRAWING
  //-------------------------------------------------------

  // ----------------------------------------------------
  //   draw (called by constructor && options )
  //
  OnDraw( )
  {
    // reset
    this.$element.empty( );
    this.$error = null;

    // build container and setup main element
    this.$element.addClass( 'tree normal no-hover' );
    if( this.options['border'] == true )  {
      this.$element.addClass( 'control-border' );
    }
    if( this.options['focus'] == true )  {
      this.$element.prop( 'tabindex', '-1' );
    }

    this.$container = $( $.views.Html( "/lib/tree/container" ) );
    this.$element.append( this.$container );
    this.$content = this.$container.find( '.scrollable' );

    // container-event: focus-in/out 
    this.$element.focusout( ( e ) => {
      if( !e.currentTarget.contains( e.relatedTarget ) ) {
        this.OnLeaveFocus();
      }
    } );
    this.$element.focusin( ( e ) => {
      if( !e.currentTarget.contains( e.relatedTarget ) ) {
        this.OnEnterFocus();
      }
    } );

    // sizes from options
    this.ManageControlSizes();

    // get events from data interface
    this.treeview.On( "change", ( event, param, extra, extra2 ) => {
      switch( event )  {
        case "rebuild": return this.OnDataRebuild( param, extra );
        case "add":     return this.OnDataAdd( param, extra );
        case "update":  return this.OnDataUpdate( param, extra );
        case "delete":  return this.OnDataDelete( param, extra );
        case "move":    return this.OnDataMove( param, extra, extra2 );
        case "sort":    return this.OnDataSort( param, extra );
      }
    } );
    this.treeview.Update();

    // manage scrollbar
    this.$scroll = this.$element.find( ".scroll-container" );
    this.$scrollable = this.$element.find( ".scrollable" );
    if( this.$scroll.length == 1 ) {
      this.$scroll.scrollbar( { resize: true, type: this.options['scrollbar'] } );
      this.$scroll.scrollbar( "recalc" );
    }
    this.$element.on( "event:layout-changed", () => {
      this.$scroll.scrollbar( "recalc" );
    });

  }


  // ----------------------------------------------------
  //   Draw: Size-Handling
  //
  ManageControlSizes()
  {
    // calculate control width (size in character or percent)
    this.$element.css( 'width', this.ControlCalculateSize( this.options['size'] ) );

    // calculate control height (size in lines or percent)
    if( 'height' in this.options ) {
      if( this.options['height'] > 0 ) {
        let lineHeight = this._CalculateItemHeight( this.$content );
        this.$element.css( 'height', lineHeight * this.options['height'] );
      }
      else {
        let height = this.ControlCalculateHeight( this.options['height'] );
        this.$element.css( 'height', height );
      }
    }    
  }

  // ----------------------------------------------------
  //   draw (branch) including root
  //
  OnDrawBranch( branch, $parent, level )
  {
    for( let i = 0; i < branch.length; i++ ) {
      let recordId = branch[ i ];
      let record = this.treeview.Get( recordId );
      this.OnDrawItem( recordId, record, $parent, level );
    }
  }

  // ----------------------------------------------------
  //   draw (item)
  //
  OnDrawItem( id, record, $parent, level )
  {
    // if custom rendering
    let preview = record._preview;
    if( $.isFunction( this.options['renderRow'] ) ) {
      preview = this.options['renderRow']( id, preview );
    }

    // build item from template
    let $item = $( $.views.Html( '/lib/tree/item', preview ) );
    let $children = $item.closest( ".children" );
    let $row = $item.closest( ".row" );
    let $chevron = $row.find( ".chevron" );

    // add indentation level & data-element
    $row.find( ".level" ).addClass( "l" + level );
    $row.data( 'itemId', id );
    $children.find( ".child-line" ).addClass( "c" + level );

    if( ( this.options['inplaceButton'] ) && ( 'inplaceButton' in preview ) && preview['inplaceButton'] ) {
      let $inplaceButton = $item.find( ".inplace-button" );
      for( let button of preview['inplaceButton'] ) {
        $inplaceButton.append( $( "<s class='s'></s>" ) );
        let $icon = $( "<i class='" + button['icon'] + "' title='" + button['tooltip'] + "'></i>" );
        $icon.on( "click", ( ev ) => {
          this.options.inplaceButton( id, button['id'] );
          ev.stopPropagation();
        } );
        $inplaceButton.append( $icon );
      }
    }

    // attach to treeview & events 
    this.treeview.AttachItem( id, $row, $children, { 'level': level } );
    this.OnDrawItemAttachEvents( $row );

    // drag & drop
    this.AttachDragAndDropEvents( id );

    // selection
    let isSelected = $.array.Has( this.options['selected'], id );
    if( isSelected )  {
      this.treeview.ItemSelect( id );
    }

    // children & chevron
    if( this.treeview.HasChildren( id ) )  {

      // chevron expand/collapse
      this.AttachChevronEvents( $row );

      // expanded / collapsed status
      let isExpanded = ( $.array.Has( this.options['expanded'], id ) || this.isExpandAllRequired == true );
      if( isExpanded )  {
        $.array.Add( this.options['expanded'], id );
        this.treeview.ItemAddStatus( id, "expanded" );
        $chevron.addClass( "mdi-chevron-down" ).removeClass( "mdi-spacer" );
        let branch = this.treeview.GetBranch( id );
        $children.removeClass( 'hidden' );
        this.OnDrawBranch( branch, $children, level + 1 );
      }
      else  {
        $.array.Remove( this.options['expanded'], id );
        this.treeview.ItemAddStatus( id, "collapsed" );
        $children.addClass( 'hidden' );
        $chevron.addClass( "mdi-chevron-right" ).removeClass( "mdi-spacer" );
      }
    }
    else  {
      $chevron.addClass( "mdi mdi-spacer" );
      $.array.Remove( this.options['expanded'], id );
    }
    
    // create row
    $parent.append( $item );

  }

  //-------------------------------------------------------
  //  item events: click left/right
  //
  OnDrawItemAttachEvents( $row )
  {
    // install click handler
    $row.click( ( ev ) =>  {
      var itemId = $( ev.currentTarget ).data( 'itemId' );
      this.OnClickItem( itemId, ev.shiftKey );
      ev.stopPropagation();
    } );

    // right click ("menu" event handler)
    if( $.isFunction( this.options['menu'] ) ) {
      $row.contextmenu( ( ev ) =>  {
        var itemId = $( ev.currentTarget ).data( 'itemId' );
        this.options['menu']( itemId, { x: ev.clientX, y: ev.clientY } );
        ev.stopPropagation();
        ev.preventDefault();
      } );
    }

  }

  //-------------------------------------------------------
  //  enable events: expand item
  //
  AttachChevronEvents( $row )
  {
    let $chevron = $row.find( ".chevron" );
    $chevron.on( "click", ( ev ) => {
      let $item = $( ev.currentTarget ).parents( ".row" );
      let itemId = $item.data( 'itemId' );
      this.OnClickChevron( itemId, ev );
      ev.stopPropagation();
    } );
  }

  //-------------------------------------------------------
  //  disable events: expand item
  //
  DetachChevronEvents( $row )
  {
    let $chevron = $row.find( ".chevron" );
    $chevron.off( "click" );
  }

  // ----------------------------------------------------
  //   item drag & drop
  //
  AttachDragAndDropEvents( itemId )
  {
    // prequisites
    if( this.treeview.ItemHasStatus( itemId, "disabled" ) ) {
      this.DetachDragAndDropEvents( itemId );
    }

    let itemData = this.treeview.GetItem( itemId );
    let $row = itemData['$item'];
    if( $row == null ) {
      return;
    }

    // don't create twice
    if( $row.data( 'dragSource' ) instanceof DragSource )  {
      return;
    }

    // create drag & drop classes & start
    let dragsource = new DragSource( this.options['dragtype'] );
    var droptarget = new DropTarget( this.options['dragtype'] );
    $row.data( 'dragSource', dragsource );
    $row.data( 'dropTarget', droptarget );

    // ------------------------------------------
    //  drag source 
    // ------------------------------------------
    dragsource.Start(
      $row,     // item
      ( $item ) => {   // start
        // start drop scroller
        this.AttachDropscroll();

        this.$container.find( ".hover" ).removeClass( "hover" ).addClass( "no-hover hover-locked" );
        let rowItemid = $row.data( 'itemId' );

        // manage selection
        if( ( this.options['selection'] == 'singleplus' )
         || ( this.options['selection'] == 'multi' )
          ) {
          this.treeview.ItemSelect( rowItemid, false );
        }
        else {
          this.treeview.ItemClearSelections();
          this.treeview.ItemSelect( rowItemid );
        }

        let dragdata = this.GetSelected();
        return dragdata;
      },
      ( $item ) => {  // stop
        this.DetachDropscroll();
        this.$container.find( ".hover-locked" ).removeClass( "no-hover hover-locked" ).addClass( "hover" );
      }
    );

    // ------------------------------------------
    //  drop target
    // ------------------------------------------
    droptarget.Start( {
      element: $row,
      param: itemId,
      expandTimer: 600
    },
      ( dragdata, element, elementSource ) => {   // enter
        let itemId = $( element ).data( 'itemId' );
        let isDisabled = this.treeview.ItemHasStatus( itemId, "disabled" );
        if( !isDisabled ) {
          return this.options['dragover'](
            { data: dragdata.data, type: dragdata.type, element: elementSource },
            { data: itemId, type: this.options['dragtype'], element: element }
          );
        }
        else {
          return false;
        }
      },
      ( dragdata, element, elementSource ) => { },  // leave
      ( action, dragdata, element, elementSource ) => {   // drop
        return this.options['drop'](
          action,
          { data: dragdata.data, type: dragdata.type, element: elementSource },
          { data: itemId, type: this.options['dragtype'], element: element }
        );

      },
      ( dragdata, element, elementSource ) => {   // expand
        let itemId = $( element ).data( 'itemId' );
        //console.log( "Expand", itemId );
        this.Expand( itemId );
      }
    );
  

  }

  // ----------------------------------------------------
  //   remove drag source / target 
  //
  DetachDragAndDropEvents( itemId )
  {
    let itemData = this.treeview.GetItem( itemId );
    let $row = itemData['$item'];
    if( $row == null )  {
      return;
    }

    let dragSource = $row.data( 'dragSource' );
    if( dragSource instanceof DragSource )  {
      dragSource.Stop( );
      $row.removeData( 'dragSource' );
    }
    let dropTarget = $row.data( 'dropTarget' ); 
    if( dropTarget instanceof DropTarget ) {
      dropTarget.Stop();
      $row.removeData( 'dropTarget' );
    }
  }


  // ----------------------------------------------------
  //   Draw: build dropscroller while drag
  //
  AttachDropscroll()
  {
    // preconditions
    if( this.dropscroll != null ) {
      return;
    }

    // create dropscroller
    this.dropscroll = new DropScroll( {
      tag: this.$element,
      scroll: this.$scroll
    } );
    this.dropscroll.Start(
      ( id ) => { },  // enter
      ( id ) => { }   // leave
    );
  }

  // ----------------------------------------------------
  //   Draw: disable drop scroller after drop
  //
  DetachDropscroll()
  {
    // destroy dropscroller
    if( this.dropscroll != null ) {
      this.dropscroll.Stop();
      this.dropscroll = null;
    }
  }

  //-------------------------------------------------------
  //  DRAW HELPER
  //-------------------------------------------------------

  ManageChevron( itemId )
  {
    if( itemId == null )  {
      return;
    }

    let itemData = this.treeview.GetItem( itemId );
    let hasChildren = this.treeview.HasChildren( itemId );

    // visible?
    if( itemData['$item'] != null )  {

      let $row = itemData['$item'];
      let $chevron = $row.find( ".chevron" );
      let isExpanded = ( $.array.Has( this.options['expanded'], itemId ) || this.isExpandAllRequired == true );

      if( hasChildren )  {
        if( isExpanded ) {
          $chevron.removeClass( "mdi-spacer" );
          $chevron.addClass( "chevron mdi-chevron-down" );
          this.Expand( itemId );
        }
        else {
          $chevron.removeClass( "mdi-spacer" );
          $chevron.addClass( "chevron mdi-chevron-right" );
          this.Collapse( itemId );
        }
        this.DetachChevronEvents( $row );
        this.AttachChevronEvents( $row );
      }
      else  {
        $chevron.removeClass( "chevron mdi-chevron-right mdi-chevron-down" );
        $chevron.addClass( "mdi-spacer" );
        this.DetachChevronEvents( $row );
      }
    }

  }

  ManageLevel( id, itemLevel )
  {
    let item = this.treeview.GetItem( id );
    if( item['$item'] == null )  {
      return;
    }

    let $level = item['$item'].find( ".level" );
    let $line = item['$children'].find( ".child-line" );
    $level.removeClass( "l0 l1 l2 l3 l4 l5 l6 l7 l8 l9 l10 l11 l12 l13 l14 l15 l16 l17 l18 l19 l20" );
    $line.removeClass( "c0 c1 c2 c3 c4 c5 c6 c7 c8 c9 c10 c11 c12 c13 c14 c15 c16 c17 c18 c19 c20" );
    $level.addClass( "l" + itemLevel );
    $line.addClass( "c" + itemLevel );
    item['data']['level'] = itemLevel;

    for( let i = 0; i < item['record']['children'].length; i++ ) {
      let childId = item['record']['children'][ i ]['id'];
      this.ManageLevel( childId, itemLevel + 1 );
    }

  }

  //-------------------------------------------------------
  //  DATA EVENTS
  //-------------------------------------------------------
  
  // ----------------------------------------------------
  //   rebuild
  //
  OnDataRebuild()
  {
    // prequisites
    if( typeof this.options['dataRoot'] == "string" )  {
      if( ! this.treeview.Has( this.options['dataRoot'] ) ) {
        return;
      }
    }

    // draw branch from root
    let branch = this.treeview.GetBranch( this.options['dataRoot'] );
    this.OnDrawBranch( branch, this.$content, 0 );

    // if during creation expanded is "*"
    if( branch.length > 0 )  {
      this.isExpandAllRequired = false;
    }

    // clean up deleted items from expanded list
    if( this.treeview.Size( ) > 0 )  {
      $.array.Remove( this.options['expanded'], ( id ) => !this.treeview.Has( id ) );
    }

    // manage scrollbar
    if( this.$scroll && this.$scroll.length == 1 ) {
      this.$scroll.scrollbar( "recalc" );
    }

  }

  // ----------------------------------------------------
  //   add record
  //
  OnDataAdd( id, parentId )
  {
    // root as default (parent may be null)
    let $children = this.$content;
    let level = 0;
    let isParentExpanded = true;
    let wasParentExpanded = false;

    // get $children element
    if( this.treeview.Has( parentId ) )  {
      let parentItem = this.treeview.GetItem( parentId );
      $children = parentItem['$children'];
      level = parentItem['data']['level'];

      // flag to check if this parent is newly expanded
      wasParentExpanded = ( $.array.Has( this.options['expanded'], parentId ) );

      // chevron & auto expand
      this.ManageChevron( parentId );
      if( this.options['autoExpandOnAdd'] == true ) {
        this.ExpandTo( id );
      }

      // get expanded status
      isParentExpanded = ( $.array.Has( this.options['expanded'], parentId ) || this.isExpandAllRequired == true );

    }

    // if element is visible: draw row
    if( isParentExpanded && wasParentExpanded ) {
      let record = this.treeview.Get( id );
      this.OnDrawItem( id, record, $children, level + 1 );

      // manage scrollbar
      if( this.$scroll && this.$scroll.length == 1 ) {
        this.$scroll.scrollbar( "recalc" );
      }

      // flash once after update
      if( this.options['animation'].indexOf( "add" ) >= 0
       || this.options['animation'].indexOf( "all" ) >= 0
        ) {
        let item = this.treeview.GetItem( id );
        let $row = item['$item'].closest( ".row" );
        new Animation( $row, "flash" );
      }      
    }
  }

  // ----------------------------------------------------
  //   delete record
  //
  OnDataDelete( id )
  {
    // HINT: the item is not (yet) deleted in treeview 

    // remove drag & drop 
    this.DetachDragAndDropEvents( id );

    // remove item
    let item = this.treeview.GetItem( id );
    if( item['$item'] != null )  {
      item['$item'].remove();
    }

    // remove chevron if last element (not at root)
    if( this.treeview.Has( item['record']['_parent'] ) )  {

      // chevron & auto expand
      this.ManageChevron( item['record']['_parent'] );

      // flash once after update
      if( this.options['animation'].indexOf( "remove" ) >= 0
       || this.options['animation'].indexOf( "all" ) >= 0
        ) {
        let parentItem = this.treeview.GetItem( item['record']['_parent'] );
        let $row = parentItem['$item'];
        new Animation( $row, "flash" );
      }    
    }

    // manage scrollbar
    if( this.$scroll && this.$scroll.length == 1 ) {
      this.$scroll.scrollbar( "recalc" );
    }

    return true;
  }

  // ----------------------------------------------------
  //   update record
  //
  OnDataUpdate( id )
  {
    // change $item (if needed)
    let item = this.treeview.GetItem( id );
    if( item['$item'] != null ) {

      // if custom rendering
      let preview = item.record._preview;
      if( $.isFunction( this.options['renderRow'] ) )  {
        preview = this.options['renderRow']( id, preview );
      }

      // build a new item content and apply
      let $itemContent = $( $.views.Html( '/lib/tree/item-content', preview ) );
      let $row = item['$item'].closest( ".row" );
      $row.empty();
      $row.append( $itemContent );

      // inplace buttons
      if( ( this.options['inplaceButton'] ) && ( 'inplaceButton' in preview ) && preview['inplaceButton'] ) {
        let $inplaceButton = item.$item.find( ".inplace-button" );
        for( let button of preview['inplaceButton'] ) {
          $inplaceButton.append( $( "<s class='s'></s>" ) );
          let $icon = $( "<i class='" + button['icon'] + "' title='" + button['tooltip'] + "'></i>" );
          $icon.on( "click", ( ev ) => {
            this.options.inplaceButton( id, button['id'] );
            ev.stopPropagation();
          } );
          $inplaceButton.append( $icon );
        }
      }

      // add indentation level & data-element
      $row.find( ".level" ).addClass( "l" + item.data.level );
      $row.data( 'itemId', id );

      // manage chevron
      if( item['record']['children'].length > 0 )  {
        this.ManageChevron( id );
      }

      // flash once after update
      if( this.options['animation'].indexOf( "update") >= 0 
       || this.options['animation'].indexOf( "all" ) >= 0 
        ) {
        new Animation( $row, "flash" );
      }
    }
  }

  // ----------------------------------------------------
  //   move record
  //
  OnDataMove( id, idToParent, idFromParent )
  {
    // get item 
    let item = this.treeview.GetItem( id );

    // get to parent $children 
    let $toChildren = this.$content;
    let itemToParent = null;
    let itemLevel = 0;
    if( this.treeview.Has( idToParent ) )  {
      itemToParent = this.treeview.GetItem( idToParent );
      itemLevel = itemToParent['data']['level'] + 1;
      $toChildren = itemToParent['$children'];
    }
    
    // move it
    item['$item'].appendTo( $toChildren );
    item['$children'].appendTo( $toChildren );

    // set level (recursive)
    this.ManageLevel( id, itemLevel );

    // chevron
    if( this.treeview.Has( idToParent ) ) {
      this.ManageChevron( idToParent );
    }
    this.ManageChevron( idFromParent );

    // flash once after update
    if( this.options['animation'].indexOf( "move" ) >= 0
     || this.options['animation'].indexOf( "all" ) >= 0
      ) {
      if( itemToParent != null ) {
        new Animation( itemToParent['$item'], "flash" );
      }
      new Animation( item['$item'], "flash" );
    }

    // manage scrollbar
    if( this.$scroll && this.$scroll.length == 1 ) {
      this.$scroll.scrollbar( "recalc" );
    }

  }

  // ----------------------------------------------------
  //   sort children 
  //
  OnDataSort( idParent, sortList )
  {
    // children visible? if not, do nothing.
    if( this.treeview.ItemHasStatus( idParent, "expanded" ) || idParent == null ) {

      // get $children element (root as default)
      let $children = this.$content;
      if( idParent != null )  {
        let itemData = this.treeview.GetItem( idParent );
        $children = itemData['$children'];
      }

      // detach all items from $children
      for( let id of sortList )  {
        let itemData = this.treeview.GetItem( id );
        itemData['$item'].detach();
        itemData['$children'].detach();
      }

      // re-attach items in new order
      for( let id of sortList ) {
        let itemData = this.treeview.GetItem( id );
        itemData['$item'].appendTo( $children );
        itemData['$children'].appendTo( $children );
      }

    }
  }

  //-------------------------------------------------------
  //  UI EVENTS
  //-------------------------------------------------------

  OnEnterFocus()
  {
    this.$element.addClass( "focus" );
  }

  OnLeaveFocus()
  {
    this.$element.removeClass( "focus" );
  }

  // ----------------------------------------------------
  //   event: OnClickChevron
  //
  OnClickChevron( itemId, ev )
  {
    // already expanded? 
    if( this.treeview.ItemHasStatus( itemId, "expanded" ) ) {
      this.Collapse( itemId );
    }
    else  {
      this.Expand( itemId );
    } 
  }
  

  // ----------------------------------------------------
  //   event: OnClickItem
  //
  OnClickItem( itemId, isShiftKey )
  {
    if( !this.treeview.ItemHasStatus( itemId, 'disabled' ) ) {
      this._SelectItem( itemId, isShiftKey );
    }   
  }

  // ----------------------------------------------------
  //    HELPER
  // ----------------------------------------------------

  // ----------------------------------------------------
  //   helper: select an item
  //
  _SelectItem( id, isShiftKey, forceSingle )
  {
    // just click item
    if( this.options['selection'] == 'click' ) {
      this._FireClick( id );
      return;
    }

    // no selection or click
    if( this.options['selection'] == 'none' ) {
      return;
    }

    if( forceSingle ) {
      this.treeview.ItemClearSelections();
      this.treeview.ItemSelect( id );
    }
    else if( ( this.options['selection'] == 'singleplus' && isShiftKey )
     || ( this.options['selection'] == 'multi' )
    ) {
      this.treeview.ItemToggleSelection( id, false );
    }
    else {
      this.treeview.ItemClearSelections();
      this.treeview.ItemSelect( id );
    }

    // finished
    this._FireResult();

  }

  // ----------------------------------------------------
  //   helper: send result event
  //
  _FireResult()
  {
    this.options['selected'] = this.treeview.GetSelected( );
    this.options['value'] = this.options['selected'];
    if( $.isFunction( this.options['result'] ) ) {
      this.options['result']( this.options['selected'] );
    }
  }

  // ----------------------------------------------------
  //   helper: send click event
  //
  _FireClick( id )
  {
    if( $.isFunction( this.options['click'] ) ) {
      this.options['click']( id );
    }
  }   

  // ----------------------------------------------------
  //   helper: select an item
  //
  _ScrollToItem( id )
  {
    if( !this.treeview.Has( id ) ) {
      return;
    }
    let itemData = this.treeview.GetItem( id );
    let $item = itemData['$item'];
    if( $item == null ) { // tree starts at other root node      
      return;
    }
    //let itemPos = $item.position();
    let itemHeight = $item.outerHeight();

    var itemOffset = $item.offset();
    var scrollOffset = this.$scrollable.offset();
    var itemYPos = itemOffset.top - scrollOffset.top;
    this.$scroll.scrollbar( "show", itemYPos, itemYPos + itemHeight );
    //this.$scroll.scrollbar( "show", itemPos.top, itemPos.top + itemHeight );
  }


  //-------------------------------------------------------
  //  METHODS
  //-------------------------------------------------------

  // ----------------------------------------------------
  //   selection
  //
  SetSelection( idList )
  {
    // no selection
    if( this.options['selection'] == 'none'
     || this.options['selection'] == 'click' 
      ) {
      return;
    }

    //  multiple items given
    if( Array.isArray( idList ) )  {

      // multiselect?
      if( this.options['selection'] == 'multi'
       || this.options['selection'] == 'singleplus'
        ) {
        for( let id of idList.length ) {
          this.treeview.ItemSelect( id );
          this.ExpandTo( id );
        }
      }
      // single: select id[ 0 ]
      else {
        this.treeview.ItemClearSelections();
        this.treeview.ItemSelect( idList[ 0 ] );
        this.ExpandTo( idList[ 0 ] );
      }

      // scroll to first item
      if( idList.length > 0 )  {
        this._ScrollToItem( idList[ 0 ] );
      }
    }

    //  clear selection
    else if( idList == null ) {
      this.treeview.ItemClearSelections();
    }

    //  single item given
    else  {
      this.treeview.ItemClearSelections();
      this.treeview.ItemSelect( idList );
      this.ExpandTo( idList );
      this._ScrollToItem( idList );
    }

  }

  // ----------------------------------------------------
  //   get selection
  //
  GetSelected(  )
  {
    return this.treeview.GetSelected( );
  }


  // ----------------------------------------------------
  //   expanded
  //
  SetExpanded( expandedList )
  {
    // expand/collapse all
    if( expandedList == "*" )  {
      return this.ExpandAll();
    }
    else if( expandedList == "" || expandedList === null )  {
      return this.CollapseAll();
    }

    // set expanded items explicitly
    if( Array.isArray( expandedList ) ) {
      for( let i = 0; i < expandedList.length; i++ ) {
        this.ExpandTo( expandedList[ i ] );
      }
    }
    else {
      this.ExpandTo( expandedList );
    }

  }

  // ----------------------------------------------------
  //  Interface: Expand a branch
  //
  ExpandTo( itemId )
  {
    // iterate to the root
    let expandList = [];
    while( itemId != null )  {
      expandList.push( itemId );
      itemId = this.treeview.GetParent( itemId );
    }

    // expand from root to itemId
    for( let i = expandList.length -1; i >= 0; i-- )  {
      this.Expand( expandList[ i ] );
    }

  }

  // ----------------------------------------------------
  //  Interface: Expand all items
  //
  ExpandAll(  )
  {
    // draw branch from root
    this._ExpandAllBranch( null);
  }

  _ExpandAllBranch( parentId )
  {
    // get all children
    let branch = this.treeview.GetBranch( parentId );

    // loop through children and do all expandables
    for( let i = 0; i < branch.length; i++ )  {
      let itemId = branch[ i ];
      if( this.treeview.HasChildren( itemId ) )  {
        this.Expand( itemId );
        this._ExpandAllBranch( itemId );
      }
    }
  }

  // ----------------------------------------------------
  //  Interface: Expand a branch
  //
  Expand( itemId )
  {
    // already expanded? 
    if( !this.treeview.HasChildren( itemId )
      || this.treeview.ItemHasStatus( itemId, "expanded" )
      ) {
      return;
    }

    // get item data
    let itemData = this.treeview.GetItem( itemId );
    let $item = itemData['$item'];
    if( $item == null )  {  // tree starts at other root node
      return;
    }
    let $children = itemData['$children'];
    let $chevron = $item.find( ".chevron" );

    // expand
    $children.empty();
    $children.append( $( $.views.Html( '/lib/tree/children-content', {} ) ) );
    $children.removeClass( 'hidden' );
    this.treeview.ItemRemoveStatus( itemId, "collapsed" );
    this.treeview.ItemAddStatus( itemId, "expanded" );

    // manage chevron
    $chevron.removeClass( "mdi-chevron-right" );
    $chevron.addClass( "mdi-chevron-down" );

    // draw branch
    let branch = this.treeview.GetBranch( itemId );
    this.OnDrawBranch( branch, $children, itemData['data']['level'] + 1 );

    // put into expanded array
    $.array.Add( this.options['expanded'], itemId );
    if( this.options['regkey'] != null )  {
      $.registry( this.options['regkey'], this.options['expanded'] );
    }

    // manage level (for indent & lines)
    this.ManageLevel( itemId, itemData['data']['level'] );

    // potentially size changed -> recalc scrolling
    if( this.$scroll ) {
      this.$scroll.scrollbar( "recalc" );
    }

  }

  // ----------------------------------------------------
  //   Collapse a branch
  //
  Collapse( itemId )
  {
    // already collapsed? 
    if( !( this.treeview.HasChildren( itemId ) )                // no children -> collapsed
     || !( this.treeview.ItemIsAttached( itemId ) )             // item is not attached -> collapsed
     || ( this.treeview.ItemHasStatus( itemId, "collapsed" ) )  // item -> collapsed
      ) {
      return;
    }

    // get item data
    let itemData = this.treeview.GetItem( itemId );
    let $item = itemData['$item'];
    if( $item == null ) {  // tree starts at other root node
      return;
    }
    let $children = itemData['$children'];
    let $chevron = $item.find( ".chevron" );

    // manage css classes
    $children.addClass( 'hidden' );
    this.treeview.ItemRemoveStatus( itemId, "expanded" );
    this.treeview.ItemAddStatus( itemId, "collapsed" );
    // manage chevron
    $chevron.removeClass( "mdi-chevron-down" );
    $chevron.addClass( "mdi-chevron-right" );

    // remove from expanded array
    $.array.Remove( this.options['expanded'], itemId );
    if( this.options['regkey'] != null ) {
      $.registry( this.options['regkey'], this.options['expanded'] );
    }

    // remove (recursivly) collapsed dom elements
    for( let childRecord of itemData['record']['children'] )  {
      this.treeview.DetachItem( childRecord['id'], true );
    }
    $children.empty();

    // potentially size changed -> recalc scrolling
    if( this.$scroll ) {
      this.$scroll.scrollbar( "recalc" );
    }
  }  

  // ----------------------------------------------------
  //  Interface: Expand all items
  //
  CollapseAll()
  {
    this._CollapseAllBranch( null );
  }

  _CollapseAllBranch( parentId )
  {
    // get all children
    let branch = this.treeview.GetBranch( parentId );

    // loop through children and do all expandables
    for( let i = 0; i < branch.length; i++ ) {
      let itemId = branch[ i ];
      if( this.treeview.HasChildren( itemId ) ) {
        this.Collapse( itemId );
        this._CollapseAllBranch( itemId );
      }
    }
  }

  // ----------------------------------------------------
  //   test height for an item
  //  
  _CalculateItemHeight( $content )
  {
    var preview = { id: '1', icon: 'mdi mdi-home', text: 'TEST' };
    var $item = $( $.views.Html( "/lib/tree/test", preview ) );
    $content.append( $item );
    var height = $item.outerHeight();
    $item.remove();
    return height;
  }

  // ----------------------------------------------------
  //   interface: $tag.tree( "item", itemId, "redraw" );
  //  
  IF_ItemRedraw( itemId )
  {
    // simulate a update event as from data-layer
    this.OnDataUpdate( itemId );
  }

  // ----------------------------------------------------
  //   interface: $tag.tree( "item", itemId, "update", newRecord );
  //  
  IF_ItemUpdate( itemId, newRecord )
  {
    this.treeset.Update( itemId, newRecord );
  }

  // ----------------------------------------------------
  //   interface: $tag.tree( "item", itemId, "get" );
  //  
  IF_ItemGet( itemId )
  {
    this.treeset.Get( itemId );
  }

  // ----------------------------------------------------
  //   interface: $tag.tree( "item", itemId, "selected", true );
  //  
  IF_ItemSetSelect( itemId, param )
  {
    if( param === true || param === undefined ) {
      this.ExpandTo( itemId );
      this.treeview.ItemSelect( itemId, ( this.options['selection'] == "single" ) );
    }
    else if( param === false ) {
      this.treeview.ItemUnselect( itemId );
    }
    else {
      this.treeview.ItemToggleSelection( itemId );
    }
  }

  // ----------------------------------------------------
  //   interface: let iosSelected = $tag.tree( "item", itemId, "selected" );
  //  
  IF_ItemGetSelect( itemId )
  {
    return this.treeview.ItemHasStatus( itemId, 'selected' );
  }  

  // ----------------------------------------------------
  //   interface: $tag.tree( "item", itemId, "expanded", true );
  //  
  IF_ItemSetExpanded( itemId, param )
  {
    if( param === true ) {
      this.ExpandTo( itemId );
    }
    else if( param === false ) {
      this.Collapse( itemId );
    }
    else {
      if( this.treeview.ItemHasStatus( itemId, "expanded" ) )  {
        this.Collapse( itemId );
      }
      else  {
        this.ExpandTo( itemId );
      }
    }
  }

  // ----------------------------------------------------
  //   interface: $tag.tree( "item", itemId, "expanded", true );
  //  
  IF_ItemGetExpanded( itemId )
  {
    return this.treeview.ItemHasStatus( itemId, 'expanded' );
  }

  // ----------------------------------------------------
  //   interface: $tag.tree( "item", itemId, "status", "red" );
  //  
  IF_ItemSetStatus( itemId, status )
  {
    let resultStatus = this.treeview.ItemSetStatus( itemId, status );
    if( resultStatus.indexOf( "disabled" ) >= 0 ) {
      this.DetachDragAndDropEvents( itemId );
    }
    else  {
      this.AttachDragAndDropEvents( itemId );
    }
  }

  // ----------------------------------------------------
  //   interface: $tag.tree( "item", itemId, "status+", "red" );
  //  
  IF_ItemAddStatus( itemId, status )
  {
    let resultStatus = this.treeview.ItemAddStatus( itemId, status );
    if( resultStatus.indexOf( "disabled" ) >= 0 ) {
      this.DetachDragAndDropEvents( itemId );
    }
    else {
      this.AttachDragAndDropEvents( itemId );
    }
  }

  // ----------------------------------------------------
  //   interface: $tag.tree( "item", itemId, "status-", "red" );
  //  
  IF_ItemRemoveStatus( itemId, status )
  {
    let resultStatus = this.treeview.ItemRemoveStatus( itemId, status );
    if( resultStatus.indexOf( "disabled" ) >= 0 ) {
      this.DetachDragAndDropEvents( itemId );
    }
    else {
      this.AttachDragAndDropEvents( itemId );
    }
  }

  // ----------------------------------------------------
  //   interface: let status = $tag.tree( "item", itemId, "status" );
  //  
  IF_ItemGetStatus( itemId )
  {
    return this.treeview.ItemGetStatus( itemId );
  }

  // ----------------------------------------------------
  //   Interface: item option getter and setter
  //
  item( args )
  {
    // prequistes
    if( args.length < 2 ) {
      console.warn( "Tree('item') - parameter missing!" );
      return;
    }

    // check itemId
    let itemId = args[ 1 ];
    if( !this.treeview.Has( itemId ) ) {
      console.warn( "Tree('item') - id='" + itemId + "' not found!" );
      return;
    }

    // setter
    let command = args[ 2 ];
    switch( command ) {
      
      case "redraw": {
        this.IF_ItemRedraw( itemId );
      } break;

      case "update": {
        this.IF_ItemUpdate( itemId, args[ 3 ] );
      } break;

      case "get": {
        this.IF_ItemGet( itemId );
      } break;

      case "select":  {
        this.IF_ItemSetSelect( itemId, args[ 3 ] ); 
      }break;

      case "selected": {
        return this.IF_ItemGetSelect( itemId );
      }
      
      case "expand": {
        this.IF_ItemSetExpanded( itemId, true );
      } break;
      
      case "expanded": {
        return this.IF_ItemGetExpanded( itemId );
      }
      
      case "collapse": {
        this.IF_ItemSetExpanded( itemId, false );
      } break;
      
      case "collapsed": {
        return !this.IF_ItemGetExpanded( itemId );
      }
      
      case "status+": {
        if( args.length >= 4 )
          this.IF_ItemAddStatus( itemId, args[3] );
        else
          return this.IF_ItemGetStatus( itemId );
      }break;
      
      case "status-": {
        if( args.length >= 4 )
          this.IF_ItemRemoveStatus( itemId, args[3] );
        else
          return this.IF_ItemGetStatus( itemId );
      } break;
      
      case "status": {
        if( args.length >= 4 )
          this.IF_ItemSetStatus( itemId, args[ 3 ] );
        else 
          return this.IF_ItemGetStatus( itemId );
      } break;
    }

  }

  // ----------------------------------------------------
  //   Interface: option getter and setter
  //
  option( args )  
  {
    // ---- setter ----
    if( args.length > 2 ) {
      switch( args[ 1 ] )  {

        case 'status': {
          if( args[2] == "disabled" ) {
            this.ManageDisabledOverlay( true );
          }
          else {
            this.ManageDisabledOverlay( false );
            //this.options['status'] = this.ItemSetStatus( this.$element, args[ 2 ] );
          }
        }
        break;

        case 'selected': {
          this.SetSelection( args[ 2 ] ); 
        }break;

        case 'expanded': {
          this.SetExpanded( args[ 2 ] ); 
        }break;

        case 'collapsed': {
          this.SetExpanded( args[2] );
        } break;

      }
      return null;
    }
    
    // getter
    switch( args[ 1 ] )  {
     
      case 'selected': {
        return this.GetSelected( );
      }
      
      default:  {
        return this.options[ args[ 1 ] ];
      }
    }
  
  }


}

/* ---------------------------------------------------------------------------
  *   MENU - plugin definition
  * ------------------------------------------------------------------------ */


// ----------------------------------------------------
//   plugin main function
//
$.fn.tree = function( option )  
{
  var data    = $( this ).data( "ctrl" ),
      options = ( typeof option == "object" && option );

  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data ) {
    var instance = new WfcTree( this, options );
    $( this ).data( "ctrl", instance );
  }

  // if called with a string, treat this as a function call
  else if( typeof option == "string" ) {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) ) {
      return data[ option ]( arguments );
    }
    else {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.tree.defaults = {
  size: -1,
  height: 10,
  border: true,
  focus: true,
  status: "normal",
  statusDefault: "normal",  
  data: null,
  dataRoot: null,
  expanded: [ ],
  regkey: null,
  autoExpandOnAdd: true,
  autoExpandOnMove: true,
  animation: "all",     // ["add" "update" "remove" "move" "all"]
  type: '',
  dragtype: 'id',
  selection: 'single',  // [ 'single', 'singleplus', 'none', 'multiple' ]
  selected: [ ],
  click: null,
  dragover: null,
  drop: null,
  menu: null,
  renderRow: null,
  inplaceButton: null
};


}( window.jQuery );
 /*
 *  ACCESS 5.0 Web Framework
 *  qrcode 
 */

!function( $ ) {

 
/* ---------------------------------------------------------------------------
  *   BUTTON - class definition
  * ------------------------------------------------------------------------ */

class WfcQrCode extends JqControl
{
  // ----------------------------------------------------
  //   constructor (will be called by base class)
  //
  OnInit( $element, options )
  {
    // class variables
    this.options  = $.extend( true, [], $.fn.button.defaults, options );

    // get button text from html, if not set
    if( !this.options['text'] )  {
      this.options['text'] = this.$element.text( );
    }
    if( ! ( 'size' in this.options ) || this.options['size'] <= 0 )  {
      this.options['size'] = 32;
    }

    // language
    this.EncodeLang();

  }

  // ----------------------------------------------------
  //   encode texts 
  //
  EncodeLang()
  {
    this.options['text'] = $.lang( this.options['text'] );
  }

  // ----------------------------------------------------
  //   recalc layout after change
  //
  OnDraw( )  
  {
    this.$element.empty();

    // normalize css and status
    this.$element.addClass( 'qrcode normal' );

    //  width + height
    this.$element.css( 'width', this.ControlCalculateSize( this.options['size'] ) + 2 );
    this.$element.css( 'height', this.ControlCalculateSize( this.options['size'] ) + 2 );

    // border
    if( this.options['border'] !== false ) {
      this.$element.addClass( "control-border" );
    }

    // create element
    this.$element.html( $.views.Html( "/lib/qrcode" ) );

    // create qr code
    this.$codeCanvas = this.$element.find( ".code" );
    let element = kjua( { 
      render: 'image', 
      text: this.options['text'],
      size: this.$codeCanvas.width(),
      ecLevel: 'H'
    } );
    this.$codeCanvas.append( $( element ) );

  }

  option( args )
  {
    // setter
    if( args.length > 2 ) {
      if( args[ 1 ] == "text" ) {
        this.options['text'] = args[ 2 ];
        this.OnDraw();
      }
    }

    // getter
    return this.options[ args[ 1 ] ];
  }



}

   
/* ---------------------------------------------------------------------------
 *   BUTTON - plugin definition
 * ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   plugin main function
//
$.fn.qrcode = function( option )  
{
  var data    = $( this ).data( "ctrl" ),            
      options = ( typeof option == "object" && option );  
  
  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data )  {
    var instance = new WfcQrCode( this, options ); 
    $( this ).data( "ctrl", instance );
  }
  
  // if called with a string, treat this as a function call
  else if( typeof option == "string" )  {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) )  {
      return data[ option ]( arguments );
    }
    else  {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;   
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.qrcode.defaults = {
  size: 0,
  //height: 1.5,
  //border: true,
  text: null,
  tooltip: null,
  status: "normal"
};

  
}( window.jQuery );
  /*
 *  ACCESS 5.0 Web Framework
 *  description line
 */

!function( $ ) {

 
/* ---------------------------------------------------------------------------
  *   DESCRIPTION LINE - class definition
  * ------------------------------------------------------------------------ */

class WfcDescriptionLine extends JqControl
{
  // ----------------------------------------------------
  //   constructor (will be called by base class)
  //
  OnInit( $element, options )
  {
    // class variables
    this.options  = $.extend( true, [], $.fn.descriptionline.defaults, options );
    this.options['statusDefault'] = ( this.options['statusDefault'] || "" ) + " no-hover";
    this.$element.addClass( 'description-line readonly' );


    // language
    this.EncodeLang();

  }

  // ----------------------------------------------------
  //   encode texts 
  //
  EncodeLang()
  {
    this.options['placeholder'] = $.lang( this.options['placeholder'] );
    this.options['data'] = $.lang( this.options['data'] );
  }

  // ----------------------------------------------------
  //   recalc layout after change
  //
  OnDraw( )  
  {
    // tidy up
    this.$element.empty();
    this.$element.off();

    // click on element
    if( $.isFunction( this.options['click'] ) )  {
      this.$element.on( "click", () => {
        this.options['click']( this.options['data'] );
      } );
    }
    else if( this.options['clickexpand'] === true )  {
      this.$element.on( "click", () => {
        this._ToggleExpanded();
      } );
    }

    // size
    this.ManageControlSizes();

    // copy option-name into css-style name
    if( Array.isArray( this.options.data ) )  {
      for( let item of this.options.data )  {
        if( 'posticonColor' in item )  {
          item['color'] = item['posticonColor'];
        }
      }
    }

    // draw
    if( this.options.data.length < 2 )  {
      this.OnDrawSingleLine();
    }
    else {
      this.OnDrawMultiLine();
    }

  }

  // ----------------------------------------------------
  //   draw single line
  //
  OnDrawSingleLine()
  {
    // container
    let html = $.views.Html( "/lib/descriptionline/container-single", {} );
    this.$element.html( html );
    this.ItemInitialize( this.$element, { status: "readonly no-hover" } );

    // line
    this.$container = this.$element.find( ".container" );

    // content
    if( Array.isArray( this.options['data'] ) && this.options['data'].length >= 1 ) {
      if( !( 'tooltip' in this.options['data'][ 0 ] ) ) {
        this.options['data'][ 0 ]['tooltip'] = "";
      }
      this.OnDrawLineContent( this.options['data'][ 0 ] );
    }

  }

  // ----------------------------------------------------
  //   draw multi line
  //
  OnDrawMultiLine()
  {
    // container
    if( this.options['expandable'] ) {
      let html = $.views.Html( "/lib/descriptionline/container-multi", { "chevronIcon": "" } );
      this.$element.html( html );
    }
    else {
      let html = $.views.Html( "/lib/descriptionline/container-single", { } );
      this.$element.html( html );
    }


    this.ItemInitialize( this.$element, { status: this.options['status'] } );
    this.ChevronStatus();

    // chevron event
    this.$chevron = this.$element.find( ".chevron > i" );
    if( this.$chevron.length > 0 )  {
      this.$chevron.on( "click", ( ev ) => {
        this._ToggleExpanded();
        ev.stopPropagation();
      } );
    }

    // line
    this.$container = this.$element.find( ".container" );

    // content
    if( this.ItemHasStatus( this.$element, "collapsed" ) )  {
      this.OnDrawPlaceholder();
    }
    else {
      if( Array.isArray( this.options['data'] ) && this.options['data'].length > 1 ) {
        for( let item of this.options['data'] ) {
          if( !( 'tooltip' in item ) ) {
            item['tooltip'] = "";
          }
          this.OnDrawLineContent( item );
        }
      }
    }

  }

  // ----------------------------------------------------
  //   set expanded / collapsed status
  //
  ChevronStatus()
  {
    let isExpanded = this.ItemHasStatus( this.$element, "expanded" );

    let $icon = this.$element.find( ".chevron > i" );
    $icon.removeClass( this.options['chevron']['collapsed'] )
    $icon.removeClass( this.options['chevron']['expanded'] )

    if( isExpanded ) {
      $icon.addClass( this.options['chevron']['expanded'] );
      this.options['expand']( );
    }
    else {
      $icon.addClass( this.options['chevron']['collapsed'] );
      this.options['collapse']( );
    }

  }

  // ----------------------------------------------------
  //   draw inner of container (one line)
  //
  OnDrawPlaceholder(  )
  {
    let $line = $.views.Html( "/lib/descriptionline/line", { text: this.options['placeholder'] } );
    this.$container.append( $line );
  }

  // ----------------------------------------------------
  //   draw inner of container (one line)
  //
  OnDrawLineContent( content )
  {
    let $line = $.views.Html( "/lib/descriptionline/line", content );
    this.$container.append( $line );
  }


  // ----------------------------------------------------
  //   Draw: Size-Handling
  //
  ManageControlSizes()
  {
    // calculate control width (size in character or percent)
    this.$element.css( 'width', this.ControlCalculateSize( this.options['size'] ) );

    // calculate control height (size in lines or percent)
    if( 'height' in this.options ) {
      if( this.options['height'] > 0 ) {
        let lineHeight = this._CalculateItemHeight( this.$content );
        this.$element.css( 'height', lineHeight * this.options['height'] );
      }
      else {
        let height = this.ControlCalculateHeight( this.options['height'] );
        this.$element.css( 'height', height );
      }
    }
  }

  // ----------------------------------------------------
  //   install event handlers
  //
  _ToggleExpanded( )  
  {
    this.ItemToggleStatus( this.$element, [ "collapsed", "expanded" ] );
    this.OnDraw();
  } 




  // ----------------------------------------------------
  //   destroy to detach events
  //
  destroy( )
  {
    this._DetachEvents( );
    this.$element.empty();
  }

  // ----------------------------------------------------
  //   option getter and setter
  //
  option( args )  
  {
    // setter
    if( args.length > 2 ) {
      switch( args[1] ) {
        case 'status': {
          if( args[2] == "disabled" ) {
            this.ManageDisabledOverlay( true );
          }
          else {
            this.ManageDisabledOverlay( false );
            this.options['status'] = this.ItemSetStatus( this.$element, args[2] );
          }
        }
        break;

        case 'error': {
          this.options['error'] = $.lang( args[2] );
          this.OnDrawError();
        }
        break;

        default:  {
          this.options[args[1]] = $.lang( args[2] );
          this.OnDraw();
        }
      }
    }
  }

  // ----------------------------------------------------
  //   validate
  //
  OnValidate( eventName )
  {
    return this.ManageValidate( this.options['value'], eventName );
  }    

};

   
/* ---------------------------------------------------------------------------
 *   BUTTON - plugin definition
 * ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   plugin main function
//
$.fn.descriptionline = function( option )
{
  var data    = $( this ).data( "ctrl" ),            
      options = ( typeof option == "object" && option );  
  
  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // if this plugin isn't installed, create one
  if( !data )  {
    var instance = new WfcDescriptionLine( this, options );
    $( this ).data( "ctrl", instance );
  }
  
  // if called with a string, treat this as a function call
  else if( typeof option == "string" )  {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) )  {
      return data[ option ]( arguments );
    }
    else  {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;   
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.descriptionline.defaults = {
  size: 0,
  status: "readonly expanded",
  placeholder: "more info",
  data: null,
  clickexpand: true,
  expandable: true,
  chevron: {
    collapsed: "mdi mdi-chevron-right",
    expanded: "mdi mdi-chevron-down",
  },
  // events
  result: $.noop,
  click: null,
  collapse: $.noop,
  expand: $.noop,
};

  
}( window.jQuery );
 /*
 *  ACCESS 5.0 Web Framework
 *  svg 
 */

!function( $ ) {

 
/* ---------------------------------------------------------------------------
  *   SVG - class definition
  * ------------------------------------------------------------------------ */

class WfcSvg extends JqControl
{
  //-----------------------------------------------------------------
  //   constructor (will be called by base class)
  //
  OnInit( $element, options )
  {
    // class variables
    this.options  = $.extend( true, [], $.fn.svg.defaults, options );

    this.options.data = {};
    this.$container = $( $.views.Html( '/lib/svg/container' ) );
    this.$element.append( this.$container );

    this.LoadFile();
  }

  //-----------------------------------------------------------------
  //   dummy: draw will be done after LoadFile()
  //
  OnDraw()
  {
  }

  //-----------------------------------------------------------------
  //   load the svg file
  //
  LoadFile()
  {
    var selfCtrl = this;

    // build request data
    let request = new XMLHttpRequest();

    // response handler function
    function responseHandler() {
      selfCtrl.LoadFileCompleted( this.responseText );
    }

    // Perform xhr-request
    request.addEventListener( "load", responseHandler );
    request.addEventListener( "error", this.LoadFileXhrEventError.bind( this ) );
    request.open( "GET", this.options['url'] );
    request.overrideMimeType( "text/plain; charset=x-user-defined" );
    request.send();
  }

  //-----------------------------------------------------------------
  //   display loaded file
  //
  LoadFileCompleted( data )
  {
    this.options.data = {};

    // build svg element
    let $svg = $( data );

    // scan all selectables and replace id
    this.$selectables = $svg.find( ".svg-selectable" );
    for( let element of this.$selectables )  {
      let $item = $( element );
      let id = $item.attr( "id" );
      if( id ) {
        id = id.replace( "{##}", "" );
        this.options.data[ id ] = { id: $.uidgen(), key: id, '$item': $item };
        $( element ).attr( "id", this.options.data[ id ]['id'] );

        if( $.isFunction( this.options['click'] ) ) {
          $item.data( "itemData", this.options.data[ id ] );
          $item.on( "click", ( ev ) => {
            this._ClickItem( this.options.data[ id ] );
          } );
        }
      }
    }

    // append svg to element
    this.$svgContent = this.$element.find( ".svg-content" );
    this.$svgContent.append( $svg );

    if( $.isFunction( this.options['load'] ) ) {
      let keys = Object.keys( this.options['data'] );
      this.options['load']( keys );
    }

    // form height might be changed
    this.TriggerLayoutEvent();
  }

  LoadFileXhrEventError( event )
  {
    this.$container.empty();
  }

  //-----------------------------------------------------------------
  //  HELPER
  //-----------------------------------------------------------------

  _ClickItem( itemData )
  {
    this.options['click']( itemData['key'] );
  }

  _Select( id )
  {
    if( id in this.options.data ) {
      this.options.data[ id ]['$item'].addClass( "svg-selected" );
      this.options['selected'] = id;
    }
    else {
      this.options['selected'] = "";
    }
  }

  _ClearAllSelection()
  {
    for( let key in this.options.data )  {
      this.options.data[ key ]['$item'].removeClass( "svg-selected" );
    }
  }

  //-----------------------------------------------------------------
  //  INTERFACE
  //-----------------------------------------------------------------

  option( args )
  {
    if( args[ 1 ] == "selected" )  {
      if( args.length > 2 ) {
        this._ClearAllSelection();
        this._Select( args[ 2 ] );
      }
      else {
        return this.options['selected'];
      }
    }

    else if( args[ 1 ] == "data" ) {
      return this.options['data'];
    }
  }//option

}

   
/* ---------------------------------------------------------------------------
 *   SVG - plugin definition
 * ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   plugin main function
//
$.fn.svg = function( option )  
{
  var data    = $( this ).data( "ctrl" ),            
      options = ( typeof option == "object" && option );  
  
  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data )  {
    var instance = new WfcSvg( this, options ); 
    $( this ).data( "ctrl", instance );
  }
  
  // if called with a string, treat this as a function call
  else if( typeof option == "string" )  {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) )  {
      return data[ option ]( arguments );
    }
    else  {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;   
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.svg.defaults = {
  size: 0,
  anchor: null,
  text: null,
  icon: null,
  data: {},
  tooltip: null,
  status: "normal",
  click: null,
  load: null
};

  
}( window.jQuery );
 /*
 *  ACCESS 5.0 Web Framework
 *  button 
 */

!function( $ ) {

 
  /* ---------------------------------------------------------------------------
   *   FILEUPLOAD - class definition
   * ------------------------------------------------------------------------ */
 
class WfcFileUpload extends JqControl
{   



  // ----------------------------------------------------
  //   init-function (called by constructor)
  //
  OnInit( element, options )  
  {
    // class variables
    this.options  = $.extend( true, [], $.fn.fileupload.defaults, options );
    this.$element = element;

    this.idInput = "";
    this.$input = null;
    this.$container = null;
    this.$label = null;
    this.$progress = null;
    this.xhrRequest = null;
    this.fontSize = { cx: 0, cy: 0 };

    // metrics
    this.fontSize.cx = $.metrics( 'fontWidth' );
    this.fontSize.cy = $.metrics( 'fontHeight' );

    // draw everything
    this.EncodeLanguage();
    this.OnDraw( );
           
  }
   
  // ----------------------------------------------------
  //   language function
  //
  EncodeLanguage() 
  {
    this.options['text'] = $.lang( this.options['text'] );
    this.options['tooltip'] = $.lang( this.options['tooltip'] );
  }

  // ----------------------------------------------------
  //   recalc layout after change
  //
  OnDraw( )  
  {
    this.$element.empty();
    
    this.idInput = this.$element.attr( 'id' ) + "_INPUT"; 
    var idLabel = this.$element.attr( 'id' ) + "_LABEL"; 

    this.options['id'] = this.$element.attr( 'id' );
    this.options['ctrlid'] = this.idInput;
    this.options['ctrlname'] = idLabel;

    this.$container = $( $.views.Html( "/lib/fileupload/container", this.options ) );
    this.$element.append( this.$container );
    this.$element.addClass( "fileupload normal" );

    this.$input = this.$element.find( "input" );
    this.$label = this.$element.find( "label" );

    this.ManageControlSizes();
    
    switch( this.options['status'] )  {
      case 'transfer':
        this.recalcStatusTransfer();
        break;

      case 'finished':
        this.recalcStatusFinished();
        break;

      case 'error':
        this.recalcStatusError();
        break;

      default:
        this.recalcStatusAuto();
        break;
    }

    // this event can't be deleted for some reason
    this.$input.on( 'change', $.proxy( this._onFileChoosen, this ) );

  }

  // ----------------------------------------------------
  //   Draw: Size-Handling
  //
  ManageControlSizes()
  {
    // calculate control width (size in character or percent)
    this.$element.css( 'width', this.ControlCalculateSize( this.options['size'] ) );

    // calculate control height (size in lines or percent)
    if( 'height' in this.options ) {
      if( this.options['height'] > 0 ) {
        let lineHeight = this.fontSize.cy;
        this.$element.css( 'height', lineHeight * this.options['height'] );
      }
      else {
        this.$element.css( 'height', this.fontSize.cy );
      }
    }
  }


  recalcStatusAuto( )
  {
    this.$label.empty( );
    this.$label.append(
      $( "<i class='" + this.options['icon'] + "'>" )
    );
    this.$label.append( "<s>" );
    this.$label.append(
      $( "<span>" + this.options['text'] + "</span>" )
    );

    // attach events
    this.$input.prop( 'disabled', false );
    this.$container.on( 'dragover', $.proxy( this._onFileDragOver, this ) );
    this.$container.on( 'dragleave', $.proxy( this._onFileDragLeave, this ) );
    this.$container.on( 'drop', $.proxy( this._onFileDrop, this ) );
  }

  recalcStatusFinished()
  {
    var selfUpload = this;

    this.$label.empty();
    this.$label.append(
      $( "<i class='green-text mdi mdi-check'>" )
    );
    this.$label.append( "<s>" );
    this.$label.append(
      $( "<span class='finished'>" + this.options['value'] + "</span>" )
    );

    if( this.$progress )  {
      delete this.$progress;
      this.$progress = null;
    }

    this.$container.on( 'click', function( event )  {
      selfUpload.recalcStatusAuto( );
    } );
    if( this.options['finishedTimeout'] > 0 )  {
      setTimeout( function( ) {
        selfUpload.recalcStatusAuto();
      }, this.options['finishedTimeout'] );
    }
  }

  recalcStatusError()
  {
    var selfUpload = this;

    this.$input.prop( 'disabled', false );
    this.$label.empty();
    this.$label.append(
      $( "<i class='color-red mdi mdi-alert-circle-outline'>" )
    );
    this.$label.append( "<s>" );
    this.$label.append(
      $( "<span class='color-red'>" + this.options['value'] + "</span>" )
    );

    if( this.$progress ) {
      delete this.$progress;
      this.$progress = null;
    }

    this.$container.on( 'click', function( event )  {
      selfUpload.recalcStatusAuto( );
    } );

  }

  recalcStatusTransfer()
  {
    // attach events
    this.$input.prop( 'disabled', true );
    this.$container.off( );

    this.$label.empty();
    this.$label.append(
      $( "<i class='color-blue mdi mdi-progress-upload'>" )
    );
    this.$label.append( "<s>" );
    let $span = $( "<span class='h-layout'>" );

    this.$progress = $( "<div class='h-layout-fix background-grey'>&nbsp;</div>" );
    let $text = $( "<div class='overlay color-blue'>" + this.options['value'] + "</div>" );
    $span.append( this.$progress );
    $span.append( $text );
    this.$label.append( $span );

  }

  setStatusTransferProgress( transfered )
  {
    this.options['transfered'] = transfered;
    if( this.$progress != null )  {
      this.$progress.css( 'width', transfered + "%" );
    }
  }
  
  // ----------------------------------------------------
  //   event: on file choosen
  //
  _onFileChoosen( ev )
  {
    var files = ev.currentTarget.files;
    var file = files[ 0 ];
    this._startFileUpload( file );
  }
  
  _onFileDragOver( ev )
  {
    ev.preventDefault( );
    ev.stopPropagation( );    
  }

  _onFileDragLeave( ev )
  {
    ev.preventDefault( );
    ev.stopPropagation( );    
  }

  _onFileDrop( ev )
  {
    ev.preventDefault( );
    ev.stopPropagation( );   

    let evOriginal = ev.originalEvent;
    if( evOriginal.dataTransfer.items && evOriginal.dataTransfer.items.length > 0 )  {
      if( evOriginal.dataTransfer.items[ 0 ].kind === 'file' ) {
        var file = evOriginal.dataTransfer.items[ 0 ].getAsFile( );
        this._startFileUpload( file );
      }
    } 
    else {
      if( evOriginal.dataTransfer.files.length > 0  )  {
        this._startFileUpload( file[ 0 ] );
      }
    }    
  }

  // ----------------------------------------------------
  //   helper: start file upload
  //
  _startFileUpload( file )
  {
    var selfUpload = this;
    
    this.options['value'] = file.name;

    if( this.xhrRequest != null )  {
      this.xhrRequest.upload.removeEventListener( 'progress', OnUploadProgress );
      this.xhrRequest.removeEventListener( 'load', OnUploadFinished );
      this.xhrRequest.removeEventListener( 'error', OnUploadError );
      this.xhrRequest.removeEventListener( 'abort', OnUploadError );
      delete this.xhrRequest;
      this.xhrRequest = null;
    }

    this.xhrRequest = new XMLHttpRequest();
    this.xhrRequest.open( this.options['method'], this.options['url'], true );
    this.xhrRequest.setRequestHeader( "X-File-Name", file.name );
    this.xhrRequest.upload.addEventListener( 'progress', OnUploadProgress, false );
    this.xhrRequest.onload = OnUploadFinished;
    this.xhrRequest.addEventListener( 'error', OnUploadError, false );
    this.xhrRequest.addEventListener( 'abort', OnUploadError, false );
    this.xhrRequest.send( file );
    
    this.options['status'] = "transfer";
    this.recalcStatusTransfer( );

    if( $.isFunction( this.options.start ) ) {
      this.options.start( this.options['status'] );
    }

    function OnUploadProgress( evt )
    {
      selfUpload.options.transferedBytes = evt.loaded;
      selfUpload.options.transferedTotal = evt.total;
      var percent = evt.loaded / evt.total * 100;
      selfUpload.setStatusTransferProgress( percent );
    }

    function OnUploadFinished( evt )
    {
      document.getElementById( selfUpload.idInput ).value = null;
      if( selfUpload.xhrRequest.status == 200 )  {
        selfUpload.options.status = "finished";
        selfUpload.recalcStatusFinished();
        if( $.isFunction( selfUpload.options.result ) ) {
          setTimeout( function()  {
            var responseHeaders = selfUpload.xhrRequest.getAllResponseHeaders();
            var headers = {};
            responseHeaders = responseHeaders.split( "\r\n" );
            for( var line of responseHeaders ) {
              line = line.split( ":" );
              if( line.length < 2 ) continue;
              line[ 0 ] = line[ 0 ].toLowerCase();
              line[ 0 ] = line[ 0 ].trim();
              line[ 1 ] = line[ 1 ].trim();
              headers[ line[ 0 ] ] = line[ 1 ];
            }
            selfUpload.options.result( true, selfUpload.options.value, headers );
          }, 0 );
        }
      }
      else  {
        selfUpload.options.status = "error";
        selfUpload.recalcStatusError();
        if( $.isFunction( selfUpload.options.result ) ) {
          setTimeout( function()  {
            selfUpload.options.result( false, selfUpload.options.value, {} );
          }, 0 );
        }
      }
    }      

    function OnUploadError( evt )
    {
      document.getElementById( selfUpload.idInput ).value = null;
      selfUpload.options.status = "error";
      selfUpload.recalcStatusError( );
      if( $.isFunction( selfUpload.options.result ) ) {
        setTimeout( function()  {
          selfUpload.options.result( false, selfUpload.options.value );
        }, 0 );
      }
    }      

  }

  // ----------------------------------------------------
  //   option getter and setter
  //
  option( args )  
  {
    // setter
    if( args.length > 2 )  {
      switch( args[ 1 ] )  {
        default: 
          this.options[ args[ 1 ] ] = args[ 2 ];
          this.OnDraw();
      }
    }
    // getter
    return this.options[ args[ 1 ] ];
  }
    
}//class
 
 
/* ---------------------------------------------------------------------------
 *   FILEUPLOAD - plugin definition
 * ------------------------------------------------------------------------ */
 
// ----------------------------------------------------
//   plugin main function
//
$.fn.fileupload = function ( option )  {

  var data = $( this ).data( "ctrl" ),
      options = ( typeof option == "object" && option );  
  
  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data )  {
    var instance = new WfcFileUpload( this, options ); 
    $( this ).data( "ctrl", instance );
  }
  
  // if called with a string, treat this as a function call
  else if( typeof option == "string" )  {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) ) {
      return data[ option ]( arguments );
    }
    else {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;   
};
 
// ----------------------------------------------------
//   plugin default values
//
$.fn.fileupload.defaults = {
  size: 35,
  height: 3,
  text: '&lt;title&gt;',
  tooltip: '',
  status: 'auto',   // 'error', 'disabled', 'readonly', 'transfer', 'finished'
  icon: "mdi mdi-cloud-upload-outline",
  url: "upload.php",
  method: "PUT",
  value: "",
  transfered: 30,
  transferedBytes: 0,
  transferedTotal: 0,
  finishedTimeout: 5000,
  result: null,
  start: null
};
 
   
}( window.jQuery );
 /* ----------------------------------------------------------------------------
 * 
 *   CYTEL Web Framework 
 *   Time line control
 *
 * --------------------------------------------------------------------------- */

!function ( $ ) {

/* ---------------------------------------------------------------------------
 *   TIMELINE - class definition
 * ------------------------------------------------------------------------ */

var WfrmTimeline = function( element, options ) {
  this.init( element, options );
};

WfrmTimeline.prototype =
{
  /*---------------------------------------------------------------
   *   class variables
   */
  $container: null,
  $boxes: null,
  $svg: null,
  $ticks: null,
  $popdown: null,
  mouse: null,
  mouseCurrentToggle: -1,
  mouseCurrentStartElement: null,
  isMouseActive: false,
  isPopupActive: false,
  popupCurrentDataIdx: -1, 
  dynid: 0,
  id2tm:  {
    'TL0' : [ 0, 0], 'TL1' : [ 0, 15], 'TL2' : [ 0, 30], 'TL3' : [ 0, 45],  //  0:00 ..  0:45
    'TL4' : [ 1, 0], 'TL5' : [ 1, 15], 'TL6' : [ 1, 30], 'TL7' : [ 1, 45],  //  1:00 ..  1:45
    'TL8' : [ 2, 0], 'TL9' : [ 2, 15], 'TL10': [ 2, 30], 'TL11': [ 2, 45],  //  2:00 ..  2:45
    'TL12': [ 3, 0], 'TL13': [ 3, 15], 'TL14': [ 3, 30], 'TL15': [ 3, 45],  //  3:00 ..  3:45
    'TL16': [ 4, 0], 'TL17': [ 4, 15], 'TL18': [ 4, 30], 'TL19': [ 4, 45],  //  4:00 ..  4:45
    'TL20': [ 5, 0], 'TL21': [ 5, 15], 'TL22': [ 5, 30], 'TL23': [ 5, 45],  //  5:00 ..  5:45
    'TL24': [ 6, 0], 'TL25': [ 6, 15], 'TL26': [ 6, 30], 'TL27': [ 6, 45],  //  6:00 ..  6:45
    'TL28': [ 7, 0], 'TL29': [ 7, 15], 'TL30': [ 7, 30], 'TL31': [ 7, 45],  //  7:00 ..  7:45
    'TL32': [ 8, 0], 'TL33': [ 8, 15], 'TL34': [ 8, 30], 'TL35': [ 8, 45],  //  8:00 ..  8:45
    'TL36': [ 9, 0], 'TL37': [ 9, 15], 'TL38': [ 9, 30], 'TL39': [ 9, 45],  //  9:00 ..  9:45
    'TL40': [10, 0], 'TL41': [10, 15], 'TL42': [10, 30], 'TL43': [10, 45],  // 10:00 .. 10:45
    'TL44': [11, 0], 'TL45': [11, 15], 'TL46': [11, 30], 'TL47': [11, 45],  // 11:00 .. 11:45
    'TL48': [12, 0], 'TL49': [12, 15], 'TL50': [12, 30], 'TL51': [12, 45],  // 12:00 .. 12:45
    'TL52': [13, 0], 'TL53': [13, 15], 'TL54': [13, 30], 'TL55': [13, 45],  // 13:00 .. 13:45
    'TL56': [14, 0], 'TL57': [14, 15], 'TL58': [14, 30], 'TL59': [14, 45],  // 14:00 .. 14:45
    'TL60': [15, 0], 'TL61': [15, 15], 'TL62': [15, 30], 'TL63': [15, 45],  // 15:00 .. 15:45
    'TL64': [16, 0], 'TL65': [16, 15], 'TL66': [16, 30], 'TL67': [16, 45],  // 16:00 .. 16:45
    'TL68': [17, 0], 'TL69': [17, 15], 'TL70': [17, 30], 'TL71': [17, 45],  // 17:00 .. 17:45
    'TL72': [18, 0], 'TL73': [18, 15], 'TL74': [18, 30], 'TL75': [18, 45],  // 18:00 .. 18:45
    'TL76': [19, 0], 'TL77': [19, 15], 'TL78': [19, 30], 'TL79': [19, 45],  // 19:00 .. 19:45
    'TL80': [20, 0], 'TL81': [20, 15], 'TL82': [20, 30], 'TL83': [20, 45],  // 20:00 .. 20:45
    'TL84': [21, 0], 'TL85': [21, 15], 'TL86': [21, 30], 'TL87': [21, 45],  // 21:00 .. 21:45
    'TL88': [22, 0], 'TL89': [22, 15], 'TL90': [22, 30], 'TL91': [22, 45],  // 22:00 .. 22:45
    'TL92': [23, 0], 'TL93': [23, 15], 'TL94': [23, 30], 'TL95': [23, 45]   // 23:00 .. 23:45
  },
  htmlPopdown:
    "<div class='tl_popdown' tabindex='-1'>\r\n" +
    "<table><tbody>" +
    "  <tr>" + 
    "    <td><label>{{label_start}}</label></td><td><input id='{{id_start}}' type='text'></td>\r\n" +
    "  </tr>" +
    "  <tr>" +
    "    <td><label>{{label_stop}}</label></td><td><input id='{{id_stop}}' type='text'></td>\r\n" +
    "  </tr>" +
    "</tbody></table>" +
    "</div>",

  /*---------------------------------------------------------------
   *   constructor directs back to class function
   */
  constructor: WfrmTimeline,

  /*---------------------------------------------------------------
   *   init-function (called by constructor)
   */
  init: function ( $container, options )
  {
    this.$container = $container;
    this.options = $.extend( true, [], $.fn.timeline.defaults, options );
    this.langEncode( );

    // create click handler for close on click outside 
    this.outsideClickHandler = $.proxy( function ( e ) {
      this.hidePopdown( false );
    }, this );

    this._normalizeData( );
    this.recalcLayout( );

  },

  // ----------------------------------------------------
  //   language function
  //
  langEncode: function() 
  {
    this.options['tooltip'] = $.lang( this.options['tooltip'] );
    this.options['label']['start'] = $.lang( this.options['label']['start'] );
    this.options['label']['stop'] = $.lang( this.options['label']['stop'] );
  },



  /*---------------------------------------------------------------
   *   all event handlers
   */
  attachEvents: function( )
  {
    var self = this;

    // context menu
    this.$container.on( 'contextmenu', function ( e ) {
      var pressedElement = document.elementFromPoint( e.clientX, e.clientY );
      if( pressedElement && self.onContextMenu( pressedElement ) == true ) {
        return false;
      }
    } );

    this.mouse = new MouseCapture;
    this.mouse.start( this.$boxes,
      // mouse down
      function( e )  {
        // left mouse key
        if( e.buttons & 0x01 ) { 
          self.mouseCurrentStartElement = document.elementFromPoint( e.clientX, e.clientY );
          if( self.mouseCurrentStartElement ) {
            self.mouseCurrentToggle = self.onToggleElement( self.mouseCurrentStartElement, -1 );
            self.isMouseActive = true;
            if( self.$popdown ) {
              self.$popdown.blur();
            }            
          }
        }
      },

      // move mouse
      function( e )  {
        // left mouse key
        if( e.buttons & 0x01 ) { 
          var element = document.elementFromPoint( e.clientX, e.clientY );
          if( element ) {
            self.onSetElementRange( self.mouseCurrentStartElement, element, self.mouseCurrentToggle );
          }
        }
      },

      // mouse up
      function ( e ) {
        // if left mouse key has been pressed
        if( self.isMouseActive == true ) {
          self.isMouseActive = false;
          self._getValuesFromControl();
          if( typeof self.options.result === "function" ) {
            self.options.result( self.options.value, self.$container );
          }
        }
      }
    );

  },

  /*---------------------------------------------------------------
   *   all event handlers
   */
  detachEvents: function( )
  {
    // disable click events
    if( this.$boxes instanceof jQuery ) {
      this.$boxes.off();
    }

    // disable mouse capturing
    if( this.mouse ) {
      this.mouse.stop( );
      delete this.mouse;
      this.mouse = null;
    }
  },


  /*---------------------------------------------------------------
   *   Event handler: context menu
   */
  onContextMenu: function( element )
  {
    // turn element on (if not yet)
    this.onToggleElement( element, 1 );
    this._getValuesFromControl( );

    // just if an element has been clicked
    if( typeof ( element.id ) == "string" && element.id.length > 0 ) {
      var startTime = this._id2Time( element.id );
      var stopTime = [ 0, 0 ];
      var dataIdx = this._findRangeInData( startTime );
      if( dataIdx < 0 )  {
        startTime[ 1 ] += 15;
        dataIdx = this._findRangeInData( startTime );
      }
      if( dataIdx >= 0 ) {
        startTime = this._parseTime( this.options.value[ dataIdx ]['start'] );
        stopTime = this._parseTime( this.options.value[ dataIdx ]['stop'] );
        this.showPopdown( element, startTime, stopTime, dataIdx );
      }
    }

    return true;
  },

  /*---------------------------------------------------------------
   *   helper: hide popdown and remove global event handler
   */
  hidePopdown: function( isSaveRequested )
  {
    if( ! this.$popdown )  {
      return;
    }

    // save if needed
    if( isSaveRequested ) {
      var startTime = this.$edStart.timeEntry( "getTime" );
      var stopTime = this.$edStop.timeEntry( "getTime" );
      if( this.popupCurrentDataIdx >= 0 )  {
        var startTm = this._buildTime( startTime.getHours( ), startTime.getMinutes( ) );
        var stopTm = this._buildTime( stopTime.getHours( ), stopTime.getMinutes( ) );
        this.options.value[ this.popupCurrentDataIdx ]['start'] = startTm;
        this.options.value[ this.popupCurrentDataIdx ]['stop'] = stopTm;
        this._normalizeData( );
        this._setValuesFromData( );
        if( typeof this.options.result === "function" ) {
          this.options.result( this.options.value, this.$container );
        }
      }
    }

    // remove from screen
    this.$popdown.remove();
    this.$popdown = null;

    // remove class "disbox" to enable  hover
    var elements = this.$boxes[ 0 ].childNodes;
    for( var i = 0; i < elements.length; i++ )  {
      var $rect = $( elements[ i ] );
      $rect.addClass( "box" );
      $rect.removeClass( "disbox" );
    }

    $( 'html' ).off( 'click', '', this.outsideClickHandler );
    $( 'html' ).off( 'contextmenu', '', this.outsideClickHandler );

  },

  /*---------------------------------------------------------------
   *   helper: show popdown and set all callbacks
   */
  showPopdown: function ( element, startTime, stopTime, dataIdx )
  {
    if( this.$popdown ) {
      this.hidePopdown( false );
    }

    var self = this;
    this.popupCurrentDataIdx = dataIdx;

    // get position of clicked element
    var offsetElement = $( element ).offset();
    var hightElement = this.$svg.height();

    // prepare html from template
    var htmlPopdown = this.htmlPopdown;
    this.dynid = ++( $.fn.timeline.defaults.id );
    if( $.fn.timeline.defaults.id > 100 ) $.fn.timeline.defaults.id = 0;
    htmlPopdown = htmlPopdown.replace( "{{label_start}}", this.options['label']['start'] );
    htmlPopdown = htmlPopdown.replace( "{{label_stop}}", this.options['label']['stop'] );
    htmlPopdown = htmlPopdown.replace( "{{id_start}}", "IDTL_START_" + this.dynid );
    htmlPopdown = htmlPopdown.replace( "{{id_stop}}", "IDTL_STOP_" + this.dynid );
    this.$popdown = $( htmlPopdown );
    this.$popdown.css( "top", offsetElement.top + hightElement / 2 );
    this.$popdown.css( "left", offsetElement.left );
    this.$popdown.click( function ( e ) {
      e.preventDefault();
      e.stopPropagation();
      return true;
    } );


    // show on screen
    this.$container.append( this.$popdown );
    this.$popdown.focus();
    this.$popdown.focusout( ( ev ) =>  {
      if( !$( ev.relatedTarget ).hasClass( "is-timeEntry" ) ) {
        this.hidePopdown( false );
      }
    } );

    // disable mouse overs on controls (no selection while popup)
    var elements = this.$boxes[0].childNodes;
    for( var i = 0; i < elements.length; i++ ) {
      var $rect = $( elements[ i ] );
      $rect.addClass( "disbox" );
      $rect.removeClass( "box" );
    }

    // manage two time edit controls
    this.$edStart = $( "#IDTL_START_" + this.dynid );
    this.$edStop = $( "#IDTL_STOP_" + this.dynid );
    this.$edStart.val( this._buildTime( startTime[ 0 ], startTime[ 1 ] ) );
    this.$edStop.val( this._buildTime( stopTime[0], stopTime[1] ) );
    this.$edStart.timeEntry( this.options.timeFormat );
    this.$edStop.timeEntry( this.options.timeFormat );

    // change focus on <start> => minValue on <stop>
    // change focus on <start> => minValue on <stop>
    this.$edStart.focusout( function( ev )  {
      var value = $( this ).timeEntry( "getTime" );
      self.$edStop.timeEntry( "option", "minTime", value );
      if( $( ev.relatedTarget ).attr( "id" ) != self.$edStop.attr( "id" ) ) {
        self.hidePopdown( false );
      }
    } );
    this.$edStop.focusout( function( ev )  {
      if( $( ev.relatedTarget ).attr( "id" ) != self.$edStart.attr( "id" ) ) {
        self.hidePopdown( false );
      }
    } );


    // <esc> or <enter>
    this.$edStart.keyup( function ( e ) {
      if( e.which == 13 ) {
        self.hidePopdown( true );
      }
      else if( e.which == 27 ) {
        self.hidePopdown( false );
      }
    } );

    // <esc> or <enter>
    this.$edStop.keyup( function ( e ) {
      if( e.which == 13 ) {
        self.hidePopdown( true );
      }
      else if( e.which == 27 )  {
        self.hidePopdown( false );
      }
    } );

    // click somewhere outside
    $( 'html' ).on( 'click', this.outsideClickHandler );
    $( 'html' ).on( 'contextmenu', this.outsideClickHandler );

  },


  /*---------------------------------------------------------------
   *   Event handler: toggle a value
   */
  onToggleElement: function ( element, toggle )
  {
    var id = element.id;
    //var tm = this.id2tm[id];
    var $element = $( element );

    if( toggle < 0 ) {
      toggle = $element.hasClass( 'active' ) ? 0 : 1;
    }

    if( toggle == 0 ) {
      $element.removeClass( 'active' );
    }
    else if( toggle == 1 ) {
      $element.addClass( 'active' );
    }

    return toggle;
  },

  
  /*---------------------------------------------------------------
   *   event handler: set elements from...to
   */
  onSetElementRange: function( startElement, endElement, value )
  {
    var elements = this.$boxes[ 0 ].childNodes; //.children
    if( elements == null )
      return;

    var startIndex = -1;
    var endIndex = -1;

    // get the range of elements
    for( i = 0; i < elements.length; i++ ) {
      if( elements[ i ] == startElement ) {
        startIndex = i;
      }
      else if( elements[i] == endElement ) {
        endIndex = i;
      }
    }

    // may be from right to left
    if( startIndex > endIndex ) {
      var temp = endIndex;
      endIndex = startIndex;
      startIndex = temp;
    }

    if( startIndex >= 0 && endIndex >= 0 )  {
      for( i = startIndex; i <= endIndex; i++ )  {
        var $element = $( elements[ i ] );
        if( value == 0 ) {
          $element.removeClass( 'active' );
        }
        else if( value == 1 ) {
          $element.addClass( 'active' );
        }
      }
    }

  },

  /*---------------------------------------------------------------
   *  recalc layout (legend)
   */
  drawLegend: function()
  {
    if( this.options['style'] == "de" ) {
      let $html = $( $.views.Html( "/lib/timeline/legend-de", {} ) );
      this.$container.append( $html );
    }
    else {
      let $html = $( $.views.Html( "/lib/timeline/legend-en", {} ) );
      this.$container.append( $html );
    }
  },

  /*---------------------------------------------------------------
   *  recalc layout after change 
   */
  recalcLayout: function ()
  {
    // draw legend
    if( this.options['type'] == "legend" ) {
      return this.drawLegend();
    }

    this.detachEvents();

    // build inline svg & groups.
    this.$svg = $( document.createElementNS( "http://www.w3.org/2000/svg", "svg" ) ).attr( {
      class: 'tl',
      width: '100%'
    } );
    this.$boxes = $( document.createElementNS( "http://www.w3.org/2000/svg", "g" ) );
    this.$ticks = $( document.createElementNS( "http://www.w3.org/2000/svg", "g" ) );

    // boxes and ticks
    for( var i = 0; i < 96; i += 4 ) {      
      // 1st box
      var id = "TL" + i.toString();
      var $box = this._createBox( i, id );
      var $tick = this._createTick( i + 1, '100%', '75%' );
      this.$boxes.append( $box );
      this.$ticks.append( $tick );

      // 2nd box
      var id = "TL" + (i+1).toString();
      var $box = this._createBox( i + 1, id );
      var $tick = this._createTick( i + 2, '100%', '50%' );
      this.$boxes.append( $box );
      this.$ticks.append( $tick );

      // 3rd box
      var id = "TL" + (i+2).toString();
      var $box = this._createBox( i + 2, id );
      var $tick = this._createTick( i + 3, '100%', '75%' );
      this.$boxes.append( $box );
      this.$ticks.append( $tick );

      // 4th box
      var id = "TL" + (i+3).toString();
      var $box = this._createBox( i + 3, id );
      this.$boxes.append( $box );
      // don't paint the last tick 
      if( i < 92 ) {
        var $tick = this._createTick( i + 4, '100%', '0%' );
        this.$ticks.append( $tick );
      }
    }

    // control width
    var tickWidth = parseFloat( this.options.tickWidth ) * 100;
    var ctrlWidth = tickWidth * 96;
    ctrlWidth = ctrlWidth / 100 + 0.1;
    ctrlWidth = ctrlWidth.toString() + "em";


    // finish and show 
    this.$svg.append( this.$boxes );
    this.$svg.append( this.$ticks );
    this.$container.empty()
                   .append( this.$svg )
                   .css( 'width', ctrlWidth )
                   ;

    // -- tooltip --
    if( typeof this.options.tooltip == "string" ) {
      this.$container.attr( 'title', this.options.tooltip );
    }
    else {
      this.$container.removeAttr( 'title' );
    }

    // value
    this._setValuesFromData();

    // events
    this.attachEvents();

  },

  /*---------------------------------------------------------------
   *  Helper: check for overlapping data and merge
   */
  _normalizeData: function( )
  {
    // an array of minutes per day
    var minutes = [];
    for( var i = 0; i < 1440; i++ ) {
      minutes[i] = 0;
    }

    // set all minutes given the data
    for( dataIdx = 0; dataIdx < this.options.value.length; dataIdx++ ) {

      // current element
      var dataElement = this.options.value[ dataIdx ];

      // convert start and end to minutes
      var startMinutes = this._parseTime( dataElement['start'] );
      startMinutes = startMinutes[ 0 ] * 60 + startMinutes[ 1 ];
      var endMinutes= this._parseTime( dataElement['stop'] );
      endMinutes = endMinutes[ 0 ] * 60 + endMinutes[ 1 ];

      // set the minutes
      for( var i = startMinutes; i < endMinutes; i++ ) {
        minutes[ i ] = 1;
      }
    }

    // convert minute array to event list
    var data = [ ];
    var isActive = false;
    var startMinute = 0;
    var stopMinute = 0;
    for( var i = 0; i < 1440; i++ )  {
      if( !isActive && minutes[ i ] == 1 ) {
        isActive = true;
        startMinute = i;
      }
      else if( isActive && minutes[ i ] == 0 ) {
        isActive = false;
        stopMinute = i;

        var startHours = Math.floor( startMinute / 60 );
        startMinute = startMinute - startHours * 60;

        var stopHours = Math.floor( stopMinute / 60 );
        stopMinute = stopMinute - stopHours * 60;
        data.push( { 'start': this._buildTime( startHours, startMinute ), 'stop': this._buildTime( stopHours, stopMinute ) } );
      }
      else if( minutes[ i ] == 1  && i == 1439 )  {
        isActive = false;
        stopMinute = i;

        var startHours = Math.floor(startMinute / 60);
        startMinute = startMinute - startHours * 60;

        var stopHours = Math.floor(stopMinute / 60);
        stopMinute = stopMinute - stopHours * 60;
        data.push( { 'start': this._buildTime( startHours, startMinute ), 'stop': this._buildTime( stopHours, stopMinute ) } );
      }
    }

    // set data back
    this.options.value = data;
  },

  /*---------------------------------------------------------------
   *  Helper: set all values
   */
  _setValuesFromData: function( )
  {
    // preparation
    var elements = this.$boxes[ 0 ].childNodes;
    if( elements == null )
      return;

    var data = this.options.value;

    // check if there's something to da
    if( !Array.isArray( data ) || data.length <= 0 )  {
      return;
    }

    // reset control
    for( i = 0; i < elements.length; i++ )  {
      $( elements[i] ).removeClass( 'active' );
    }

    // go through all data elements
    for( dataIdx = 0; dataIdx < data.length; dataIdx++ )  {

      var dataElement = data[ dataIdx ];
      var startTime = this._parseTime( dataElement['start'] );
      var endTime   = this._parseTime( dataElement['stop'] );
      var startId = "TL" + ( startTime[0] * 4 + startTime[2] ).toString();
      // if the endtime is not exactly we need to mark this quarter-hour as used
      if( endTime[3] == false ) {
        endTime[ 2 ] ++;
      }
      var endId = "TL" + ( endTime[ 0 ] * 4 + endTime[ 2 ] ).toString();

      var isStarted = false;
      for( i = 0; i < elements.length; i++ )  {
        if( elements[ i ].id == startId ) {
          isStarted = true;
          $( elements[ i ] ).addClass( 'active' );
        }
        else if( elements[i].id == endId ) {
          break;
        }
        else if( isStarted )  {
          $( elements[i] ).addClass( 'active' );
        }
      }

    }

  },

  /*---------------------------------------------------------------
   *  Helper: get & convert all values
   */
  _getValuesFromControl: function( )
  {
    // preparation
    var elements = this.$boxes[ 0 ].childNodes;
    if( elements == null )
      return;

    // usually all times are in 15min time grid. so we read out all
    // selected cells to normalize the data. Here we build an 
    // exception list. e.G. 10:00 is really 10:05 (start)
    // or 13:57 is 14:00 (stop).
    var exceptions = [];
    for( var i = 0; i < this.options.value.length; i++ )  {
      var element = this.options.value[i];
      var startTm = this._parseTime( element['start'] );
      var stopTm = this._parseTime( element['stop'] );
      if( !startTm[ 3 ] ) {
        var hours1 = startTm[ 0 ];
        var hours2 = startTm[ 0 ];
        var minutes1 = startTm[ 1 ];
        var minutes2 = startTm[ 2 ] * 15;
        exceptions.push( [ hours2, minutes2, hours1, minutes1 ] );
      }
      if( !stopTm[ 3 ] ) {
        var hours1 = stopTm[ 0 ];
        var hours2 = stopTm[ 0 ];
        var minutes1 = stopTm[ 1 ];
        var minutes2 = stopTm[ 2 ] * 15 + 15;
        if( minutes2 >= 60 )  {
          hours2++;
          minutes2 = 0;
        }
        exceptions.push( [ hours2, minutes2, hours1, minutes1 ] );
      }
    }

    // will be use locally below
    function checkForException( tm )  {
      for( var i = 0; i < exceptions.length; i++ ) {
        var element = exceptions[ i ];
        if( tm[0] == element[0] && tm[1] == element[1] )  {
          return( [ element[ 2 ], element[ 3 ] ] );
        }
      }
      return tm;
    }

    this.options.value = [];


    // go through all elements
    var start = -1;
    for( i = 0; i < elements.length; i++ ) {
      
      var $element = $( elements[ i ] );
      var isChecked = $element.hasClass( 'active' ) ? true : false;
      
      // start of time sequence
      if( start < 0 && isChecked ) {
        start = i;
      }
      // endof time sequence
      else if( start >= 0 && !isChecked ) {
        // convert from ID to time
        var startTm = this.id2tm[ elements[ start ].id ];
        startTm = checkForException( startTm );
        var endTm = this.id2tm[ elements[ i ].id ];
        endTm = checkForException( endTm );
        // push it to the result value
        this.options.value.push( { start: this._buildTime( startTm[ 0 ], startTm[ 1 ] ), stop: this._buildTime( endTm[ 0 ], endTm[ 1 ] ) } );
        start = -1;
      }
      
      if( i == 95 && isChecked )  {
        if( start == -1 )  {
          start = 95;
        }
        // convert from ID to time
        var startTm = this.id2tm[ elements[ start ].id ];
        startTm = checkForException( startTm );
        // push it to the result value
        this.options.value.push( { start: this._buildTime( startTm[ 0 ], startTm[ 1 ] ), stop: "24:00" } );
        start = -1;
      }
      //else if (i == 95 && isChecked )  {
      //  this.options.value.push( { start: "00:00", stop: "24:00" } );
      //  start = -1;
      //}
    }

  },

  /*---------------------------------------------------------------
   *  Helper: find a selected range
   */
  _findRangeInData: function( tm )
  {
    tm = tm[ 0 ] + tm[ 1 ] / 60;

    for( var i = 0; i < this.options.value.length; i++ ) {
      var value = this.options.value[ i ];
      var startTm = this._parseTime( value['start'] );
      var stopTm = this._parseTime( value['stop'] );
      startTm = startTm[ 0 ] + ( startTm[ 1 ] / 60 );
      stopTm = stopTm[ 0 ] + ( stopTm[ 1 ] / 60 );
      if( tm >= startTm  &&  tm < stopTm )  {
        return i;
      }
    }
    return -1;
  },


  /*---------------------------------------------------------------
   *  Helper: convert control ID to value
   */
  _id2Time: function ( id )
  {
    var result = [];
    var lookup = this.id2tm[id];
    result.push( lookup[ 0 ] );
    result.push( lookup[ 1 ] );
    return result;
  },

  /*---------------------------------------------------------------
   *  Helper: create a single box given the index
   */
  _createBox: function ( index, id )
  {
    var tickWidth = parseFloat( this.options.tickWidth ) * 100;
    var x = index * tickWidth;
    x = x / 100;
    x = x.toString() + "em";
    return $( document.createElementNS( "http://www.w3.org/2000/svg", "rect" ) ).attr( {
      'id': id,
      'x': x,
      'width': this.options.tickWidth,
      'height': '100%',
      'shape-rendering': 'crispEdges',
      'class': 'box'
    } );
  },

  /*---------------------------------------------------------------
   *  Helper: create a single tick given the index
   */
  _createTick: function ( index, y1, y2 )
  {
    var tickWidth = parseFloat( this.options.tickWidth ) * 100;
    var x = index * tickWidth;
    x = x / 100;
    x = x.toString() + "em";
    return $( document.createElementNS( "http://www.w3.org/2000/svg", "line" ) ).attr( {
      'x1': x,
      'y1': y1,
      'x2': x,
      'y2': y2,
      'shape-rendering': 'crispEdges',
      'class': 'tick'
    } );
  },

  /*---------------------------------------------------------------
   *  Helper: parse a time string "08:15"
   *  returns: [0]: hour
   *           [1]: minute
   *           [2]: the quarter (0:=[0..14], 1:=[15..30], etc.)
   *           [3]: isExactly ( minute one of [0,15,30,45] )
   */
  _parseTime: function ( tmString )
  {
    var tmArray = tmString.split( ":" );
    if( Array.isArray( tmArray ) && tmArray.length >= 2 )  {
      var result = [ 0, 0, 0 ];
      result[ 0 ] = parseInt( tmArray[ 0 ] );
      result[ 1 ] = parseInt( tmArray[ 1 ] );

      if( result[ 1 ] >= 0 && result[ 1 ] <= 14 ) {
        result[ 2 ] = 0;
      }
      else if( result[ 1 ] >= 15 && result[ 1 ] <= 29 ) {
        result[ 2 ] = 1;
      }
      else if( result[ 1 ] >= 30 && result[ 1 ] <= 44 ) {
        result[ 2 ] = 2;
      }
      else if( result[ 1 ] >= 45 && result[1] <= 59 ) {
        result[ 2 ] = 3;
      }
      result[ 3 ] = ( result[ 1 ] == 0 || result[ 1 ] == 15 || result[ 1 ] == 30 || result[ 1 ] == 45 );
      return result;
    }
    return null;
  },


  /*---------------------------------------------------------------
   *  Helper: parse a time string "08:15"
   */
  _buildTime: function ( tmHour, tmMinute )
  {
    var h = "00" + tmHour;
    h = h.substr( h.length - 2 );
    var m = "00" + tmMinute;
    m = m.substr( m.length - 2 );
    return ( h + ":" + m );
  },

  // ----------------------------------------------------
  //   option getter and setter
  //
  option: function ( args )
  {
    // setter
    if( args.length > 2 ) {
      this.options[ args[ 1 ] ] = args[ 2 ];
      this._normalizeData( );
      this.recalcLayout( );
    }
    // getter
    return this.options[ args[ 1 ] ];
  },


};

/* ---------------------------------------------------------------------------
 *   TIMELINE - plugin definition
 * ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   plugin main function
//
$.fn.timeline = function ( option )
{
  var data = $( this ).data( "timeline" ),
      options = ( typeof option == "object" && option );

  // if this plugin isn't installed, create one
  if( !data ) {
    var ctrl = new WfrmTimeline( this, options );
    $( this ).data( "timeline", ctrl );
  }

    // if called with a string, treat this as a function call
  else if( typeof option == "string" ) {
    // pass the complete list of arguments to function 
    return data[ option ]( arguments );
  }

  // return the object for chaining    
  return this;
};


// ----------------------------------------------------
//   plugin default values
//
$.fn.timeline.defaults = {
  tickWidth: "0.375em",
  value: [],
  result: null,
  id: 0,
  tooltip: null,
  label: {
    start: "X:",
    stop: "Y:"
  },
  timeFormat: {
    show24Hours: true,
    separator: ':',
    ampmPrefix: '',
    ampmNames: ['AM', 'PM'],
    spinnerImage: ''
  }
};


}( window.jQuery );
 /*
 *  ACCESS 5.0 Web Framework
 *  button 
 */

!function( $ ) {

 
/* ---------------------------------------------------------------------------
  *   BUTTON - class definition
  * ------------------------------------------------------------------------ */

class WfcWizzard extends JqControl
{
  // ----------------------------------------------------
  //   constructor (will be called by base class)
  //
  OnInit( $element, options )
  {
    // class variables
    this.options  = $.extend( true, [], $.fn.wizzard.defaults, options );

    // get button text from html, if not set
    if( !this.options['text'] )  {
      this.options['text'] = this.$element.text( );
    }

    this.$indicator = null;

    // data
    this.DataInitialize();

    // language
    this.EncodeLang();

  }

  // ----------------------------------------------------
  //   encode texts 
  //
  EncodeLang()
  {
  }

  // ----------------------------------------------------
  //   recalc layout after change
  //
  OnDraw( )  
  {
    // check
    if( this.$element.length == 0 )  {
      console.warn( "Try to initialize a control, html-tag not found! Abort." );
      return;
    }

    // reset
    this.$element.empty();

    // sizes: calculate control width (size in character or percent)
    this.$element.css( 'width', this.ControlCalculateSize( this.options['size'] ) );

    if( this.options.border === true ) {
      this.$element.addClass( "control-border" );
    }
    this.$element.addClass( "normal" );

    // sizes: calculate control height
    if( 'height' in this.options ) {
      let height = this.ControlCalculateHeight( this.options['height'] );
      this.$element.css( 'height', height );
    }

    // basic border around
    this.$element.append( $( $.views.Html( "/lib/wizzard/head", {} ) ) );
    this.$content = this.$element.find( ".wizzard-sidebar" );

    // draw data
    if( Array.isArray( this.options.data ) )  {

      var index = 0;
      for( let itemData of this.options.data )  {
        this.OnDrawItem( itemData, index++ );
      }
    }

  }

  // ----------------------------------------------------
  //   item
  //
  OnDrawItem( itemData, index )
  {
    itemData = $.lang( itemData );

    // numeric icon
    itemData['icon'] = this.options.iconTemplate;
    if( index < 9 ) {
      itemData['icon'] = itemData['icon'].replace( "{{index}}", index + 1 );
    }
    else {
      itemData['icon'] = this.options.iconTemplate2;
    }

    // build item html code
    var $item = null;
    if( itemData['checked'] ) {
      $item = $( $.views.Html( "/lib/wizzard/item-checked", itemData ) );
    }
    else {
      $item = $( $.views.Html( "/lib/wizzard/item", itemData ) );
    }

    // status item
    if( 'status' in itemData ) {
      $item.addClass( itemData['status'] );
      if( itemData['status'] == "normal" ) {
        $item.addClass( "hover" );
        $item.data( "itemData", itemData );
        $item.on( 'click', ( ev ) => {
          var clickItemData = $( ev.currentTarget ).data( "itemData" );
          this.CommandSelect( clickItemData['id'] );
        } );
      }
    }
    else {
      $item.addClass( "disabled" );
    }

    // selection
    if( this.options.selected == itemData['id'] ) {
      $item.addClass( "selected" );
    }

    // append
    this.$content.append( $item );
  }

  //---------------------------------------------------------------------------
  //  first time: add status and checked into data
  //  
  DataInitialize()
  {
    this.firstItemId = this.options.data[ 0 ]['id'];
    for( var itemData of this.options.data ) {
      itemData['checked'] = false;
      itemData['status'] = ( itemData['id'] == this.firstItemId ) ? "normal" : "disabled";
    }
    this.options.selected = this.firstItemId;
  }

  //===========================================================================
  //  HELPER
  //===========================================================================

  _FireResult( id )
  {
    this.options.result( id );
  }

  //===========================================================================
  //  COMMAND
  //===========================================================================

  // ----------------------------------------------------
  //   select
  //
  CommandSelect( id )
  {
    this.options.selected = id;
    this._FireResult( this.options.selected );
    this.OnDraw();
  }

  // ----------------------------------------------------
  //   next
  //
  CommandNext()
  {
    // check current item if not yet checked
    for( var itemData of this.options.data ) {
      if( !( itemData['checked'] ) && this.options.selected == itemData['id'] ) {
        itemData['checked'] = true;
        itemData['status'] = "normal";
        break;
      }
    }

    // go to next selected item
    var isFound = false;
    for( var itemData of this.options.data ) {
      if( itemData['id'] == this.options.selected ) {
        isFound = true;
      }
      else if( isFound ) {
        this.options.selected = itemData['id'];
        itemData['status'] = "normal";
        this._FireResult( this.options.selected );
        break;
      }
    }

    this.OnDraw();
  }

  // ----------------------------------------------------
  //   previous
  //
  CommandPrevious()
  {
    // go to previous selected item
    for( var index = 0; index < this.options.data.length; index++ ) {
      var itemData = this.options.data[index];
      if( itemData['id'] == this.options.selected ) {
        if( index > 0 ) {
          this.options.selected = this.options.data[index - 1]['id'];
          this._FireResult( this.options.selected );
        }
      }
    }

    this.OnDraw();
  }

  // ----------------------------------------------------
  //   option: generic getter and setter
  //
  option( args )  
  {
    // commands
    switch( args[ 1 ] ) {
      case 'next': {
        return this.CommandNext();
      }

      case 'previous': {
        return this.CommandPrevious();
      }
    }

    // setter
    if( args.length > 2 ) {
      switch( args[ 1 ] ) {

        case 'status': {
          this.options['status'] = this.ItemSetStatus( this.$element, args[ 2 ] );
        }
        break;

        case 'selected': {
          this.options.selected = args[ 2 ];
          this.CommandSelect( this.options.selected );
        }
        break;


        default: {
          this.options[ args[ 1 ] ] = args[ 2 ];
          this.OnDraw();
        }
      }
    }

    // getter
    return this.options[ args[ 1 ] ];
  }


}

   
/* ---------------------------------------------------------------------------
 *   WIZZARD - plugin definition
 * ------------------------------------------------------------------------ */

// ----------------------------------------------------
//   plugin main function
//
$.fn.wizzard = function( option )  
{
  var data    = $( this ).data( "ctrl" ),            
      options = ( typeof option == "object" && option );  
  
  // remove for re-create
  if( typeof option == "object" && data ) {
    data.destroy();
    data = null;
    $( this ).data( "ctrl", null );
  }

  // error case: control need to be initialized with an object before
  // you can call option like $something.ctrl( "option", "status" )
  if( !data && ( typeof option != "object" ) ) {
    return this;
  }

  // if this plugin isn't installed, create one
  if( !data )  {
    var instance = new WfcWizzard( this, options ); 
    $( this ).data( "ctrl", instance );
  }
  
  // if called with a string, treat this as a function call
  else if( typeof option == "string" )  {
    // destroy outside of class
    if( option == "destroy" ) {
      data.destroy();
      $( this ).off();
      $( this ).data( "ctrl", null );
      return null;
    }
    // call function in class
    else if( $.isFunction( data[ option ] ) )  {
      return data[ option ]( arguments );
    }
    else  {
      console.warn( "call to undefined function '" + option + "' in class '" + data.constructor.name + "'" );
      return null;
    }
  }

  // return the object for chaining    
  return this;   
};

// ----------------------------------------------------
//   plugin default values
//
$.fn.wizzard.defaults = {
  size: 0,
  height: 10,
  border: false,
  selected: null,
  data: [],
  error: null,
  warning: null,
  tooltip: null,
  status: "normal",
  iconTemplate: "mdi mdi-numeric-{{index}}-circle-outline",
  iconTemplate2:"mdi mdi-numeric-9-plus-circle-outline",
  result: $.noop
};


  
}( window.jQuery );
 //=============================================================================
//  LABEL including Help-Topic
//=============================================================================

class HelpTopic extends HTMLElement 
{
  //---------------------------------------------------------------------------
  //  VARIABLES
  //---------------------------------------------------------------------------

  elementLabel = null;
  elementContent = null;
  topicId = null;

  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  constructor()
  {
    super();

    // find label tag for click handler
    this.elementLabel = this.querySelector( "label" );
    if( !this.elementLabel ) {
      return;
    }

    // a "data-help" attribute need to be given
    this.topicId = this.getAttribute( "data-help" );
    if( !this.topicId ) {
      return;
    }

    // prepare label tag
    this.elementLabel.classList.add( "help-label" );
    this.elementLabel.addEventListener( "click", this.Show.bind( this ) );
    
    // get content tag, or create one
    this.elementContent = this.querySelector( ".help-content" );
    if( !this.elementContent )  {
      this.elementContent = document.createElement( 'div' );
      this.elementContent.classList.add( "help-content" );
      this.elementContent.innerHTML = "<div class='help-text'></div><div class='help-close'><i class='mdi mdi-close'></i></div>";
      this.appendChild( this.elementContent );
    }

    let closeButton = this.elementContent.querySelector( ".help-close" );
    if( closeButton )  {
      closeButton.addEventListener( "click", ( ev ) => {
        this.elementContent.blur();
      });
    }

    this.elementContent.setAttribute( 'tabindex', 0 );
    this.elementContent.addEventListener( "blur", this.Hide.bind( this ) );

  }

  //---------------------------------------------------------------------------
  //  SHOW TOPIC
  //---------------------------------------------------------------------------

  Show( ev )
  {
    this.elementContent.classList.add( "shown" );
    this.elementContent.focus();
    
    var elementHelpText = this.elementContent.querySelector( ".help-text" );
    if( elementHelpText ) {
      Help.QueryTopic( this.topicId, ( text ) => {
        elementHelpText.innerHTML = text;
      } );
    }

    // use jquery (custom-events don't work)
    this.dispatchEvent( new CustomEvent( 'event:layout-changed', { bubbles: true } ) );

  }

  Hide()
  {
    this.elementContent.classList.remove( "shown" );

    // use jquery (custom-events don't work)
    this.dispatchEvent( new CustomEvent( 'event:layout-changed', { bubbles: true } ) );

  }
}

// ---- global register -------------------------------------------------------
customElements.define( 'help-topic', HelpTopic );


//---------------------------------------------------------------------------
//  HELP LOADER
//---------------------------------------------------------------------------

class Help
{
  //---------------------------------------------------------------------------
  //  SINGLETON
  //---------------------------------------------------------------------------

  static _instance = null;
  constructor()
  {
    Help._instance = this;
  }

  //---------------------------------------------------------------------------
  //  CUSTOM OVERRIDABLE
  //---------------------------------------------------------------------------

  OnLoadTopic( topicId, resultCallback )
  {
    resultCallback( "<b>Error loading</b> '" + topicId + "': Function 'OnLoadTopic' need to be overwritten!" );
  }

  //---------------------------------------------------------------------------
  //  STATIC INTERFACE TO WC
  //---------------------------------------------------------------------------

  static QueryTopic( topicId, resultCallback )
  {
    if( Help._instance == null )  {
      resultCallback( "<b>Error loading</b> '" + topicId + "': Help not initialized!" );
    }
    else {
      Help._instance.OnLoadTopic( topicId, resultCallback );
    }
  }
}
$.extend( true, $.lang._data, {  } );
//=============================================================================
//
//  SIMPLE document view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.01.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class PageMain extends Viewable
{
  //-----------------------------------------------------------------
  //  CLASS MEMBERS
  //-----------------------------------------------------------------

  static _registeredForms = {};
  static _instance = null;
  static _showEntidAfterCreate = null;
  
  selectedEntid = null;
  localInfo = {};
  /* example
    apartment.1: {
      title: "new Title", // null
      modified: true/false
      error: true/false
    }
   */

  //-------------------------------------------------------
  //  INIT
  //-------------------------------------------------------

  constructor()
  {
    super();
    PageMain._instance = this;
  }

  //-------------------------------------------------------
  //  LOAD
  //
  Load( data )
  {
    //  login page use larger font, reset
    $( 'html' ).css( 'font-size', '1.0em' );

    //-----------------------------------------------------
    //  main layout viewables
    //-----------------------------------------------------

    $( '#ID_HEADER' ).viewable( { view: '/part/header' } );
    $( '#ID_SIDEBAR' ).viewable( { 
      view: '/part/sidebar',
      click: this.ControlSidebarClick.bind( this ),
      entityMove: this.ControlSidebarHasMoved.bind( this )
    } );
    $( "#ID_SIDEBAR" ).splitter( { position: 'right', regkey: 'sidebar-splitter', size: 40 } );


    //-----------------------------------------------------
    //  views & tabs
    //-----------------------------------------------------

    $( "#ID_TABS" ).tabs( {
      size: -1,
      data: [
        { id: "dashboard", text: FormDashboard.documentTitle, template: FormDashboard.viewableTemplate, status: "sticky" }
      ],
      click: this.ControlTabsClick.bind( this ),
      close: this.ControlTabsClose.bind( this )
    } );
    $( "#ID_MULTI" ).multi( { size: -1 } );


    //-----------------------------------------------------
    //  data handling
    //-----------------------------------------------------

    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, ( entinfo ) => {
      this.DataEntinfoUpdate( entinfo );
    } );

    Data.Instance().On( "ENTINFO/DELETE/*", this.etag, ( entinfo ) => {
      this.DataEntinfoDelete( entinfo );
    } );

    Verify.Instance().On( "VERIFY/ERRORSTATUS/*", this.etag, ( verifyInfo ) => {
      this.DataVerifyError( verifyInfo );
    } );


    //-----------------------------------------------------
    //  show dashboard
    //-----------------------------------------------------

    $( "#ID_MULTI" ).multi( "create", {
      'id': "dashboard",
      'template': FormDashboard.viewableTemplate,
      'status': "sticky"
    }, {
      'entid': "dashboard",
      'entinfo': null
    } );

    // we need to give sidebar a few moments to initialize
    setTimeout( () => {
      this.selectedEntid = "dashboard";     // and remember previous shown form
      $( '#ID_SIDEBAR' ).viewable( "event", "selected", "dashboard" );  // send event to sidebar
      this._SetApplicationDocumentTitle( "dashboard" );
    }, 10 );

  }

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  click on a tab
  //
  ControlTabsClick( entid )
  {
    // fire "OnHide" event to previous shown form
    let formClass = $( "#ID_MULTI" ).multi( "viewable", this.selectedEntid );
    if( ( formClass instanceof Viewable ) && ( 'OnTabVisible' in formClass ) ) {
      formClass.OnTabVisible( false, this.selectedEntid );
    }

    // bring selected form to front
    $( "#ID_MULTI" ).multi( "show", entid );
    let entinfo = Data.Instance().Entinfo( entid );
    this._SetApplicationDocumentTitle( entid );

    // fire "OnShow" event to form
    formClass = $( "#ID_MULTI" ).multi( "viewable", entid );
    if( ( formClass instanceof Viewable ) && ( 'OnTabVisible' in formClass ) ) {
      formClass.OnTabVisible( true, entid );
    }

    // and remember previous shown form
    this.selectedEntid = entid;

    // send event to sidebar
    $( '#ID_SIDEBAR' ).viewable( "event", "selected", entid );

  }

  //-----------------------------------------------------------------
  //  close a tab
  //
  ControlTabsClose( id )
  {
    this._CloseEntity( id );
  }


  //-----------------------------------------------------------------
  //  sidebar clicked on entity
  //
  ControlSidebarClick( id )
  {
    PageMain.ShowEntity( id );
  }

  //-----------------------------------------------------------------
  //  sidebar moved an item to another parent
  //
  ControlSidebarHasMoved( moveData )
  {
    // 1st. get entid
    let entinfo = Data.Instance().Entinfo( moveData['entid'] );
    if( entinfo ) {
      if( !FormFactory.MoveEntity( entinfo['class'], moveData['entid'], moveData['toParent'] ) )  {
        Data.Instance().MoveEntity( moveData['entid'], moveData['toParent'] );
      }
    }
  }


  //-----------------------------------------------------------------
  //  DATA LAYER
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  entity has been changed
  //
  DataEntinfoUpdate( entinfo )
  {
    if( PageMain._showEntidAfterCreate == entinfo['entid'] )  {
      PageMain._showEntidAfterCreate = null;
      setTimeout( () => {
        PageMain.ShowEntity( entinfo['entid'] );
      }, 500 );
    }
  }

  //-----------------------------------------------------------------
  //  entity has been deleted, close form
  //
  DataEntinfoDelete( entid )
  {
    PageMain.CloseEntity( entid );
  }

  //-----------------------------------------------------------------
  //  verify (error) information has been changed
  //
  DataVerifyError( verifyInfo )
  {
    //console.log( "PageMain(): Verify Error-Status", verifyInfo );
    this._DocumentError( verifyInfo['entid'], verifyInfo['error'] );
  }

  //-----------------------------------------------------------------
  //  HELPER
  //-----------------------------------------------------------------

  _HelperLocalInfoChange( entid, localInfo )
  {
    $( '#ID_SIDEBAR' ).viewable( "event", "localinfo", {
        'entid': entid,
        'info': localInfo 
      }
    );  // send event to sidebar

  }

  //=================================================================
  //  SINGLETON INTERFACE FUNCTIONS
  //=================================================================

  _FocusSearch()
  {
    $( '#ID_HEADER' ).viewable( "option", "search" );
  }

  //-----------------------------------------------------------------
  //  remove entity from tab and multi
  //
  _CloseEntity( entid )
  {
    if( ( entid in this.localInfo ) && this.localInfo['error'] == false ) {
      delete this.localInfo[ entid ];
      this._HelperLocalInfoChange( entid, null );
    }

    // if tab already closed, skip
    if( ! $( "#ID_TABS" ).tabs( "exists", entid ) ) {
      return;
    }

    // remove form + tab
    $( "#ID_MULTI" ).multi( "remove", entid );
    $( "#ID_TABS" ).tabs( "remove", entid );
 
    // tab change to previous item, get and active it
    let currrentSelction = $( "#ID_TABS" ).tabs( "option", "selected" );
    $( "#ID_MULTI" ).multi( "show", currrentSelction );
  }

  //-----------------------------------------------------------------
  //  show entity in tab and multi
  //
  _ShowEntity( entid, registeredForm, entinfo = null )
  {
    // ---- bring existing form to front ----
    if( $( "#ID_TABS" ).tabs( "exists", entid ) )  {

      $( "#ID_TABS" ).tabs( "option", "selected", entid );
      // this will raise a click event to tab control
      // there multi is brought to fron
    }

    // ---- create form ----
    else  {
      // create tab
      $( "#ID_TABS" ).tabs( "create", { 
        text: entinfo ? entinfo['name'] : registeredForm['text'], 
        id: entid
      } );
      // create viewable
      $( "#ID_MULTI" ).multi( "create", { 
        'id': entid,
        'template': registeredForm['template'],
        'status': registeredForm['sticky'] ? "sticky" : ""
      }, 
      { 
        'entid': entid,
        'entinfo': entinfo
      } );

      // this will raise a click event to tab control
      // there multi is brought to fron
      $( "#ID_TABS" ).tabs( "option", "selected", entid );

    }//else

  }

 //-----------------------------------------------------------------
  //  show permission dialog
  //  
  _ShowPermission( entid )
  {
    if( ( entid in this.localInfo ) && ( this.localInfo[ entid ]['modified'] ) ) {
      this._ShowEntity( entid );
      let formClass = $( "#ID_MULTI" ).multi( "viewable", this.selectedEntid );
      if( 'OnEvent' in formClass ) {
        formClass.OnEvent( "toastSaveFirst" );
      }
    }
    else {
      PermissionsDialog.Show( { 'entid': entid } );
    }
  }


  //-----------------------------------------------------------------
  //  show entity in tab and multi
  //
  _SetApplicationDocumentTitle( entid )
  {
    let entinfo = Data.Instance().Entinfo( entid );
    if( !entinfo ) {
      let general = CONST.sidebarGeneral.find( ( rec ) => rec['id'] == entid );
      if( general ) {
        Application.Instance().SetDocumentTitle( general['text'] );
      }
    }
    else {
      Application.Instance().SetDocumentTitle( entinfo['name'] );
    }
  }

  //-----------------------------------------------------------------
  //  set document modified indicator in tab
  //
  _DocumentModified( entid, isModified )
  {
    if( !( entid in this.localInfo ) ) {
      this.localInfo[ entid ] = { title: null, modified: false, error: false };
    }
    this.localInfo[ entid ]['modified'] = isModified;
    if( !isModified ) {
      delete this.localInfo[ entid ]['title'];
    }
    // inform sidebar
    this._HelperLocalInfoChange( entid, this.localInfo[ entid ] );


    if( isModified ) {
      $( "#ID_TABS" ).tabs( "item", entid, "status-add", "modified" );
    }
    else {
      $( "#ID_TABS" ).tabs( "item", entid, "status-remove", "modified" );
    }
  }

  //-----------------------------------------------------------------
  //  set document modified indicator in tab
  //
  _DocumentTitle( entid, text )
  {
    if( !( entid in this.localInfo ) ) {
      this.localInfo[ entid ] = { title: null, modified: false, error: false };
    }
    this.localInfo[ entid ]['title'] = text;
    // inform sidebar
    this._HelperLocalInfoChange( entid, this.localInfo[ entid ] );

    $( "#ID_TABS" ).tabs( "item", entid, "text", text );
    if( $( "#ID_TABS" ).tabs( "option", "selected" ) == entid )  {
      Application.Instance().SetDocumentTitle( text );
    }
  }

  //-----------------------------------------------------------------
  //  set document modified indicator in tab
  //
  _DocumentError( entid, isError )
  {
    if( !( entid in this.localInfo ) ) {
      this.localInfo[ entid ] = { title: null, modified: false, error: false };
    }

    this.localInfo[ entid ]['error'] = isError;
    // inform sidebar
    this._HelperLocalInfoChange( entid, this.localInfo[ entid ] );

  }

  //=================================================================
  //  STATIC INTERFACE
  //=================================================================

  //-----------------------------------------------------------------
  //  close interface to forms
  //
  static FocusSearch( )
  {
    if( PageMain._instance == null ) {
      return;
    }
    PageMain._instance._FocusSearch( );
  }


  //-----------------------------------------------------------------
  //  close interface to forms
  //
  static DocumentTitle( entid, text )
  {
    if( PageMain._instance == null ) {
      return;
    }
    PageMain._instance._DocumentTitle( entid, text );
  }

  //-----------------------------------------------------------------
  //  close interface to forms
  //
  static DocumentModified( entid, isModified )
  {
    if( PageMain._instance == null ) {
      return;
    }
    PageMain._instance._DocumentModified( entid, isModified );
  }

  //-----------------------------------------------------------------
  //  error interface 
  //
  static DocumentError( entid, isModified )
  {
    if( PageMain._instance == null ) {
      return;
    }
    PageMain._instance._DocumentError( entid, isModified );
  }  

  //-----------------------------------------------------------------
  //  close interface to forms
  //
  static CloseEntity( entid )
  {
    if( PageMain._instance == null ) {
      return;
    }
    PageMain._instance._CloseEntity( entid );
  }

  //-----------------------------------------------------------------
  //  open interface to application
  //
  static ShowEntity( entid )
  {
    if( PageMain._instance == null ) {
      return;
    }

    // symbolic entities
    if( entid == "admui.basicparam" )  {
      entid = "basicparam";
    }

    // 1st. get entid
    let entinfo = Data.Instance().Entinfo( entid );
    if( entinfo )  {
      let registeredForm = PageMain._registeredForms[ entinfo['class'] ];
      if( registeredForm ) {
        return PageMain._instance._ShowEntity( entid, registeredForm, entinfo );
      }
    }
    else  {
      for( let entityClass in PageMain._registeredForms ) {
        let registeredForm = PageMain._registeredForms[ entityClass ];
        if( registeredForm['predefinedEntid'] == entid )  {
          return PageMain._instance._ShowEntity( entid, registeredForm, null );
        }
      }
    }

    console.warn( `PageMain.ShowEntity: No form for '${ entid }' found!` );
  }

  //-----------------------------------------------------------------
  //  open interface to application
  //
  static ShowPermission( entid )
  {
    if( PageMain._instance == null ) {
      return;
    }
    PageMain._instance._ShowPermission( entid );
  }


  //-----------------------------------------------------------------
  //  open interface to application
  //
  static CreateEntity( parentEntityId )
  {
    FormNewDialog.Show( {
      result: ( result ) => {
        Data.Instance().CreateEntity(
          result['formId'],
          result['deviceTemplate'],
          parentEntityId,
          ( resultEntid ) => {
            PageMain._showEntidAfterCreate = resultEntid;
          }
        );
      }
    } );
  }

  

  //-----------------------------------------------------------------
  //  Show MessageBox before an entity together with all its children
  //  is going to be deleted.
  //
  static DeleteEntity( entityId )
  {
    // Make sure entity admin.1 is not inside a folder to be deleted!
    {    
      // Get an array of all entInfos to be deleted.
      let entInfoList = [];
      Data.Instance().TreeData().BranchEntInfoList( entityId, entInfoList )

      // Look if admin.1 is inside the array.
      for( let info of entInfoList ) {

        // If found, tell the user he can't delete the folder.
        if( info["entid"] == "admin.1" )  {

          MessageBox.Show( {
            text: "{$const_messagebox_delete_refused}",
            layout: "fatal"
          } );
          return; 
        }
      } // for
    }

    MessageBox.Show( {
      text: "{$const_messagebox_delete_text}",
      oktext: "{$const_messagebox_delete_label}",
      layout: "alert",
      result: ( isOk ) => {
        if( isOk ) {
          Data.Instance().DeleteEntity( entityId );
          PageMain.CloseEntity( entityId );
        }
      }
    } );
  }


  //-------------------------------------------------------
  //  register a form for tabs 
  //
  static RegisterForm( entityClass, template, title, predefinedEntid = null )
  {
    PageMain._registeredForms[ entityClass ] = {
      'template': template,
      'text': title,
      'predefinedEntid': predefinedEntid
    };
    //console.log( "Registered Form :", PageMain._registeredForms[ entityClass ] );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

PageMain.Register( '/pages/main' );
//=============================================================================
//
//  SIMPLE document view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.01.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class LoginPage extends Viewable
{
  //-------------------------------------------------------
  //  DESTROY
  //
  destructor()
  {
  }

  //-------------------------------------------------------
  //  LOAD
  //
  Load( data )
  {
    var self = this;

    // --- prepare global ---
    $( 'html' ).css( 'font-size', '1.1em' );
    Application.Instance().SetDocumentTitle( "{$document_title_login}" );

    // --- create controls ---
    $( '#ID_LOGIN_USER' ).edit( {
      size: 30,
      placeholder: "",
      regkey: "lastlogin",
      password: false
    } );

    $( '#ID_LOGIN_PASSWORD' ).edit( {
      size: 30,
      placeholder: "",
      password: true
    } );
    
    // if login name has been changed set focus on password
    let storedValue = $( '#ID_LOGIN_USER' ).edit( "option", "value" );
    if( storedValue && storedValue.length > 0 )  {
      $( '#ID_LOGIN_PASSWORD' ).edit( "focus" );
    }
    else {
      $( '#ID_LOGIN_USER' ).edit( "focus" );
    }

    $( '#ID_LOGIN_LANG' ).listbox( {
      type: 'droplist',
      size: 30,
      height: 5,
      selected: [ AccessApplication.Instance( ).Language( ) ],
      placeholder: "{$const_common_empty_placeholder}",
      data: [
        { id: 'de', text: $.lang( '{$login_lang_german}' ) },
        { id: 'en', text: $.lang( '{$login_lang_english}' ) }
      ],
      result: ( selected ) => {
        this.OnSelectLanguage( selected[ 0 ] );
      }
    } );

    $( '#ID_LOGIN' ).click( ( e ) => {
      this.Login();
    } );

    // --- keyboard handler: return ---
    $( document ).keypress( ( e ) => {
      if( e.which == 13 || e.which == 10 ) {
        e.preventDefault();
        self.Login();
      }
    } );

    Data.Instance().On( "SESSION.LOGIN", this.etag, ( response ) =>  {
      self.OnXhrLoginResponse( response );
    } );
  
  }// load



  //-------------------------------------------------------
  //  LANGUAGE
  //
  OnSelectLanguage( language )
  {
    let cookiePath = (window.location.pathname).slice( 0, -1 );
    Cookies.set( 'lang', language, { path: cookiePath } );
    Storages.localStorage.set( 'lang', language );
    window.location.reload();
  }

  //-------------------------------------------------------
  //  LOGIN
  //
  Login( )
  {
    let user     = $( '#ID_LOGIN_USER' )    .edit( "option", "value" );
    let password = $( '#ID_LOGIN_PASSWORD' ).edit( "option", "value" );
    //Data.Instance().XhrRequest( "SESSION.LOGIN", { 'user': user, 'password': password } );

    new XhrRequest( "SESSION.LOGIN", { 'user': user, 'password': password }, this.OnXhrLoginResponse.bind( this ) );

  }


  //-------------------------------------------------------
  //  XHR-RESPONSE LOGIN
  //
  OnXhrLoginResponse( response )
  {
    let responseCode = parseInt( response['meta']['status'] );
    if( responseCode == 200 )  {
      // reload window for refresh the session-id cookie
      window.location.reload();
    }
    else  {
      $( '#ID_LOGIN_PASSWORD' ).edit( "option", "value", "" );
      $( '#ID_LOGIN_USER' ).edit( "option", "error", " " );
      $( '#ID_LOGIN_PASSWORD' ).edit( "option", "error", " " );
      $( '#ID_LOGIN_PASSWORD' ).edit( "focus" );
    }

  }// login response
}


//=============================================================================
//  BUILDER
//=============================================================================

LoginPage.Register( '/pages/login' );
//=============================================================================
//
//  SIMPLE document view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.01.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class RestartPage extends Viewable
{
  //-------------------------------------------------------
  //  DESTROY
  //
  destructor()
  {
  }

  //-------------------------------------------------------
  //  LOAD
  //
  Load( data )
  {
    var self = this;

    // --- prepare global ---
    $( 'html' ).css( 'font-size', '1.1em' );
    Application.Instance().SetDocumentTitle( "{$restart_title}" );

  }// load

}


//=============================================================================
//  BUILDER
//=============================================================================

RestartPage.Register( '/pages/restart' );
//=============================================================================
//
//  SIMPLE document view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.01.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class InstallPage extends Viewable
{
  //===========================================================================
  //  INIT
  //===========================================================================

  Load( data )
  {
    $( 'html' ).css( 'font-size', '1.1em' );
    Application.Instance().SetApplicationTitle( "{$install_header_title}" );
    Data.Instance().LoadAdminConfig();

    new XhrRequest( "FIRSTINSTALL.STARTED", { }, ( response ) => {
      //console.log( "FIRSTINSTALL.STARTED", response );
    } );

    // REM: The timezone database was taken from this project, https://github.com/vvo/tzdb 
    //      after an update of IANA database this file need to be updated manually!
    $.getJSON( "tools/raw-time-zones.json", this.DataTimezonesResponse.bind( this ) );


    this.cosconfData = null;

    this.resultData = { };
    this.resultData[ WizzardPageWelcome.id ] = {};
    this.resultData[ WizzardPageLicense.id ] = {};
    this.resultData[ WizzardPageNetwork.id ] = {};
    this.resultData[ WizzardPageTimezone.id ] = {};
    this.resultData[ WizzardPagePassword.id ] = {};
    this.resultData[ WizzardPageResult.id ] = {};
    this.currentPage = WizzardPageWelcome.id;

    $( "#ID_OPTIONAL" + this.dynid ).optional( {} );

    /*$( "#ID_SSS" ).click( ( ev ) => {
      new XhrRequest( "FIRSTINSTALL.FINISHED", {}, ( response ) => {} );
    } );*/

    $( "#ID_WIZZARD_SIDEBAR" ).wizzard( {
      size: -1,
      height: -1,
      data: [
        { id: WizzardPageWelcome.id, text: WizzardPageWelcome.sidebarText },
        { id: WizzardPageLicense.id, text: WizzardPageLicense.sidebarText },
        { id: WizzardPageNetwork.id, text: WizzardPageNetwork.sidebarText },
        { id: WizzardPageTimezone.id, text: WizzardPageTimezone.sidebarText },
        { id: WizzardPagePassword.id, text: WizzardPagePassword.sidebarText },
        { id: WizzardPageResult.id, text: WizzardPageResult.sidebarText },
      ],
      selected: "welcome",      
      result: this.ControlSidebarResult.bind( this )      
    });

    WizzardPageWelcome.Build( "#ID_WIZZARD_PAGE", {
      command: this.CommandPageButtons.bind( this ),
      result: this.ControlPageResult.bind( this ),
      data: this.resultData['welcome']
    });

    Data.Instance().On( "COSCONF.READ", this.etag, this.DataResponseCosconfRead.bind( this ) );
    Data.Instance().On( "DATA/UPDATE/admui.basicparam", this.etag, this.DataResponseEntity.bind( this ) );

    this.DataRequest();
  
  }// load


  //===========================================================================
  //  DATA
  //===========================================================================

  DataRequest()
  {
    Data.Instance().XhrRequest( "COSCONF.READ" );
    Data.Instance().RequestEntity( "admui.basicparam" );
  }

  DataResponseCosconfRead( response )
  {
    this.cosconfData = response['data'];
    this.resultData[ WizzardPageNetwork.id ]['interfaces'] = response['data']['interfaces'];
    this.resultData[ WizzardPageNetwork.id ]['ntp'] = response['data']['ntp'];
    this.resultData[ WizzardPageTimezone.id ]['timezone'] = response['data']['timezone'];
  }

  //-----------------------------------------------------------------------------
  //  response cosconf read
  //
  DataTimezonesResponse( data )
  {
    var tzList = [];
    for( var record of data ) {
      tzList.push( {
        id: record['name'],
        text: record['rawFormat']
      } );
    }
    this.resultData[ WizzardPageTimezone.id ]['tz-list'] = tzList;    
  }  

  DataResponseEntity( entity )
  {
    //console.log( "BASICPARAM", entity );
    this.basicparam = $.clone( entity );
  }

  //===========================================================================
  //  CONTROL
  //===========================================================================

  ControlSidebarResult( id )
  {
    // get result from current page
    $( "#ID_WIZZARD_PAGE" ).viewable( "option", "save", this.resultData[ this.currentPage ] );

    // create new page
    switch( id ) {
      case WizzardPageWelcome.id: 
        WizzardPageWelcome.Build( "#ID_WIZZARD_PAGE", {
          command: this.CommandPageButtons.bind( this ),
          result: this.ControlPageResult.bind( this ),
          data: this.resultData[ WizzardPageWelcome.id ]
        } );
        break;

      case WizzardPageLicense.id: 
        WizzardPageLicense.Build( "#ID_WIZZARD_PAGE", {
          command: this.CommandPageButtons.bind( this ),
          result: this.ControlPageResult.bind( this ),
          data: this.resultData[ WizzardPageLicense.id ]
        } );
        break;

      case WizzardPageNetwork.id:
        WizzardPageNetwork.Build( "#ID_WIZZARD_PAGE", {
          command: this.CommandPageButtons.bind( this ),
          result: this.ControlPageResult.bind( this ),
          data: this.resultData[ WizzardPageNetwork.id ]
        } );
        break;

      case WizzardPageTimezone.id:
        WizzardPageTimezone.Build( "#ID_WIZZARD_PAGE", {
          command: this.CommandPageButtons.bind( this ),
          result: this.ControlPageResult.bind( this ),
          data: this.resultData[ WizzardPageTimezone.id ]
        } );
        break;

      case WizzardPagePassword.id:
        WizzardPagePassword.Build( "#ID_WIZZARD_PAGE", {
          command: this.CommandPageButtons.bind( this ),
          result: this.ControlPageResult.bind( this ),
          data: this.resultData[ WizzardPagePassword.id ]
        } );
        break;

      case WizzardPageResult.id:
        WizzardPageResult.Build( "#ID_WIZZARD_PAGE", {
          command: this.CommandPageButtons.bind( this ),
          result: this.ControlPageResult.bind( this ),
          data: this.resultData[ WizzardPageResult.id ]
        } );
        break;
    }

    // store current page
    this.currentPage = id;
  }

  ControlPageResult( data )
  {
    this.resultData[ data['id'] ] = data;
    if( data['id'] == WizzardPageResult.id ) {
      this.CommandFinish();
    }
  }

  //===========================================================================
  //  COMMAND
  //===========================================================================

  //---------------------------------------------------------------------------
  //  event from wizard page
  //
  CommandPageButtons( command )
  {
    if( command == "next" ) {
      $( "#ID_WIZZARD_SIDEBAR" ).wizzard( "option", "next" );
    }
    else if( command == "previous" )  {
      $( "#ID_WIZZARD_SIDEBAR" ).wizzard( "option", "previous" );
    }
  }

  //---------------------------------------------------------------------------
  //  result page: finish everything
  //
  CommandFinish()
  {
    //console.log( "FINISH:", this.resultData );
    let numberOfStepsFinished = 0;

    // ---- parepare cosconf ----
    this.cosconfData['interfaces'] = this.resultData[ WizzardPageNetwork.id ]['interfaces'];
    this.cosconfData['timezone'] = this.resultData[ WizzardPageTimezone.id ]['timezone'];
    
    new XhrRequest( "COSCONF.WRITE", this.cosconfData, ( response ) => {
      numberOfStepsFinished++;
      this.CommandFinishXhrResult( numberOfStepsFinished );
    } );

    // ---- prepare basicparam ----
    if( !( 'sip' in this.basicparam ) ) {
      this.entity['sip'] = {};
    }
    this.basicparam['sip']['network'] = {};

    // all interfaces are enabled (if configured)
    for( let macAddr in this.cosconfData['interfaces'] ) {
      var netInterface = this.cosconfData['interfaces'][ macAddr ];

      this.basicparam['sip']['network'][ macAddr ] = {
        name:         netInterface['name'],
        ipV4:         netInterface['configured'],
        ipV4Address:  netInterface['configured'] ? netInterface['ipv4'] : "",
        ipV6:         false,
        ipV6Address:  "",
        client:       netInterface['configured'],
        telephone:    netInterface['configured'],
        port:         5060
      };
    }
    
    new XhrRequest( "ADMIN.UPDATE_ENTITY", { 'entity': this.basicparam }, ( response ) => {
      //console.log( "WRITTEN basicparam:", response );
      numberOfStepsFinished++;
      this.CommandFinishXhrResult( numberOfStepsFinished );
    } );

    // ---- password ----
    var password = this.resultData[ WizzardPagePassword.id ]['password'];
    new XhrRequest( "ADMIN.SET_PASSWORD", { 'entityId': "admin.1", 'password': password, 'passwordKey': "password" }, ( response ) => {
      //console.log( "WRITTEN password:", response );
      numberOfStepsFinished++;
      this.CommandFinishXhrResult( numberOfStepsFinished );
    } );
  
    // reboot

    //console.log( "WRITE basicparam:", this.basicparam );

  }

  //---------------------------------------------------------------------------
  //  result page: finish everything
  //
  CommandFinishXhrResult( numberOfStepsFinished )
  {
    if( numberOfStepsFinished == 3 )  {
      new XhrRequest( "FIRSTINSTALL.FINISHED", {}, ( response ) => {
        AccessApplication.Instance().Command( "exit" );
      } );
    }
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

InstallPage.Register( '/pages/install' );
//=============================================================================
//
//  Wizzard sidebar view
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          18.10.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class InstallWizzardSidebar extends Viewable
{
  static template = "/pages/install/sidebar";
  static iconTemplate = "mdi mdi-numeric-{{index}}-circle-outline";
  static iconTemplate2 = "mdi mdi-numeric-9-plus-circle-outline";

  //===========================================================================
  //  INIT
  //===========================================================================

  //---------------------------------------------------------------------------
  //  first load viewable
  // 
  Load( options )
  {
    this.data = options['data'];
    this.selected = options['selected'];

    this.DataInitialize();
    this.Draw();

  }// load

  //---------------------------------------------------------------------------
  //  main event function from upper layer
  // 
  OnEvent( eventName, param, data )
  {
    if( eventName == "option" && param == "next" )  {
      this.CommandNext();
    }
    else if( eventName == "option" && param == "previous" ) {
      this.CommandPrevious();
    }
    else if( eventName == "option" && param == "selected" ) {
      // setter
      if( data ) {
        this.CommandSelect( data );
      }
      // getter
      else {
        return this.selected;
      }
    }
  }

  //---------------------------------------------------------------------------
  //  (re)draw sidebar
  // 
  Draw( )
  {
    var $sidebarContent = this.$container.find( ".wizzard-sidebar" );
    $sidebarContent.empty();

    for( let index = 0; index < this.data.length; index++ ) {
    
      var itemData = $.lang( this.data[ index ] );
    
      // numeric icon
      itemData['icon'] = InstallWizzardSidebar.iconTemplate;
      if( index < 9 ) {
        itemData['icon'] = itemData['icon'].replace( "{{index}}", index + 1 );
      }
      else {
        itemData['icon'] = InstallWizzardSidebar.iconTemplate2;
      }

      // build item html code
      var $item = null;
      if( itemData['checked'] )  {
        $item = $( $.views.Html( "/pages/install/sidebar/item-checked", itemData ) );
      }
      else {
        $item = $( $.views.Html( "/pages/install/sidebar/item", itemData ) );
      }

      // status item
      if( 'status' in itemData )  {
        $item.addClass( itemData['status'] );
        if( itemData['status'] == "normal" ) {
          $item.addClass( "hover" );
          $item.data( "itemData", itemData );
          $item.on( 'click', ( ev ) => {
            var clickItemData = $( ev.currentTarget ).data( "itemData" );
            this.CommandSelect( clickItemData['id'] );
          });
        }
      }
      else {
        $item.addClass( "disabled" );
      }

      // selection
      if( this.selected == itemData['id'] )  {
        $item.addClass( "selected" );
      }

      // append
      $sidebarContent.append( $item );

    }
  }



  //===========================================================================
  //  COMMAND
  //===========================================================================

  CommandSelect( id )
  {
    this.selected = id;
    this.FireEventSelected( this.selected );
    this.Draw();
  }

  CommandNext()
  {
    // check current item if not yet checked
    for( var itemData of this.data ) {
      if( !( itemData['checked'] ) && this.selected == itemData['id'] ) {
        itemData['checked'] = true;
        itemData['status'] = "normal";
        break;
      }
    }

    // go to next selected item
    var isFound = false;
    for( var itemData of this.data ) {
      if( itemData['id'] == this.selected ) {
        isFound = true;
      }
      else if( isFound )  {
        this.selected = itemData['id'];
        itemData['status'] = "normal";
        this.FireEventSelected( this.selected );
        break;
      }
    }

    this.Draw();
  }

  CommandPrevious()
  {
    // go to previous selected item
    for( var index = 0; index < this.data.length; index++ ) {
      var itemData = this.data[ index ];
      if( itemData['id'] == this.selected )  {
        if( index > 0 )  {
          this.selected = this.data[ index - 1 ]['id'];
          this.FireEventSelected( this.selected );
        }
      }
    }

    this.Draw();
  }

  //===========================================================================
  //  EVENT upper layer
  //===========================================================================

  FireEventSelected( id )
  {
    this.FireEvent( "result", id );
  }

  //===========================================================================
  //  DATA
  //===========================================================================

  //---------------------------------------------------------------------------
  //  first time: add status and checked into data
  //  
  DataInitialize()
  {
    this.firstItemId = this.data[ 0 ]['id'];
    for( var itemData of this.data ) {
      itemData['checked'] = false;
      itemData['status'] = ( itemData['id'] == this.firstItemId ) ? "normal" : "disabled";
    }
  }

  //===========================================================================
  //  BUILDER
  //===========================================================================

  static Build( selector, options )
  {
    options['view'] = InstallWizzardSidebar.template;
    $( selector ).viewable( options );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

InstallWizzardSidebar.Register( InstallWizzardSidebar.template );
//=============================================================================
//
//  Wizzard Page
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          16.10.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WizzardPageWelcome extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATIC (CONFIG)
  //---------------------------------------------------------------------------

  static id = "welcome";
  static template = "/pages/install/welcome";
  static sidebarText = "{$wizpage_welcome_sidebar}";

  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( data )
  {
    $( "#ID_LANGUAGE" + this.dynid ).click( () => {
      this.CommandLanguage( );
    } );

    
    $( "#ID_NEXT" + this.dynid ).click( () => {
      this.FireCommand( "next" );
    });
  
  }// load


  //---------------------------------------------------------------------------
  //  event interface to upper layer
  //---------------------------------------------------------------------------

  FireCommand( command )
  {
    this.FireEvent( "command", command );
  }

  //---------------------------------------------------------------------------
  //  COMMAND
  //---------------------------------------------------------------------------

  CommandLanguage()
  {
    let cookiePath = ( window.location.pathname ).slice( 0, -1 );
    var currentLanguage = Cookies.get( 'lang' );
    if( currentLanguage == "en" )  {
      Cookies.set( 'lang', "de", { path: cookiePath } );
      Storages.localStorage.set( 'lang', "de" );
    }
    else {
      Cookies.set( 'lang', "en", { path: cookiePath } );
      Storages.localStorage.set( 'lang', "en" );
    }
    window.location.reload();    
  }

  //===========================================================================
  //  BUILDER
  //===========================================================================

  static Build( selector, options )
  {
    options['view'] = WizzardPageWelcome.template;
    $( selector ).viewable( options );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

WizzardPageWelcome.Register( WizzardPageWelcome.template );
//=============================================================================
//
//  Wizzard Page
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          18.10.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WizzardPageLicense extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATIC (CONFIG)
  //---------------------------------------------------------------------------

  static id = "license";
  static template = "/pages/install/license";
  static sidebarText = "{$wizpage_license_sidebar}";

  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createOptions )
  {
    this.isLicenseAccepted = ( 'isLicenseAccepted' in createOptions.data ) ? createOptions.data['isLicenseAccepted'] : false;

    $( "#ID_PREVIOUS" + this.dynid ).click( () => {
      this.ControlClickPrevious( "previous" );
    } );
    $( "#ID_NEXT" + this.dynid ).click( () => {
      this.ControlClickNext();
    });
    $( "#ID_CHECK" + this.dynid ).choice( {
      size: 20,
      type: "check",
      value: this.isLicenseAccepted,
      text: "{$wizpage_license_check}",
      result: this.ControlAcceptLicenseCheck.bind( this )
    });

    var eulaFile = "img/eula-en.txt";
    if( AccessApplication.Instance().Language() == "de" )  {
      eulaFile = "img/eula-de.txt";
    }

    // load eula.text
    $.ajax( {
      url: eulaFile,
      success: ( data ) => {
        $( "#ID_LICENSE_TEXT" + this.dynid ).text( data );
      }
    } );
  
    this.FormManageLicenseAccepted();

  }// load


  //---------------------------------------------------------------------------
  //  FORM
  //---------------------------------------------------------------------------

  FormManageLicenseAccepted()
  {
    if( this.isLicenseAccepted )  {
      $( "#ID_NEXT" + this.dynid ).removeClass( "disabled" );
      $( "#ID_NEXT" + this.dynid ).addClass( "default" );
      $( "#ID_NEXT" + this.dynid ).addClass( "hover" );
    }
    else {
      $( "#ID_NEXT" + this.dynid ).removeClass( "default" );
      $( "#ID_NEXT" + this.dynid ).removeClass( "hover" );
      $( "#ID_NEXT" + this.dynid ).addClass( "disabled" );
    }
  }

  //---------------------------------------------------------------------------
  //  CONTROL
  //---------------------------------------------------------------------------

  ControlAcceptLicenseCheck( isChecked )
  {
    this.isLicenseAccepted = isChecked;
    this.FormManageLicenseAccepted();
  }

  ControlClickNext()
  {
    if( this.isLicenseAccepted )  {
      this.FireEvent( "result", { id: WizzardPageLicense.id, 'isLicenseAccepted': this.isLicenseAccepted } );
      this.FireCommand( "next" );
    }
  }

  ControlClickPrevious()
  {
    this.FireEvent( "result", { id: WizzardPageLicense.id, 'isLicenseAccepted': this.isLicenseAccepted } );
    this.FireCommand( "previous" );
  }

  //---------------------------------------------------------------------------
  //  event interface to upper layer
  //---------------------------------------------------------------------------

  FireCommand( command )
  {
    this.FireEvent( "command", command );
  }

  //===========================================================================
  //  BUILDER
  //===========================================================================

  static Build( selector, options )
  {
    options['view'] = WizzardPageLicense.template;
    $( selector ).viewable( options );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

WizzardPageLicense.Register( WizzardPageLicense.template );
//=============================================================================
//
//  Wizzard Page
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          18.10.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WizzardPageNetwork extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATIC (CONFIG)
  //---------------------------------------------------------------------------

  static id = "network";
  static template = "/pages/install/network";
  static sidebarText = "{$wizpage_network_sidebar}";

  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createOptions )
  {
    this.data = createOptions['data'];
    this.data['id'] = WizzardPageNetwork.id;

    this.interfaceList = [];
    this.currentStep = 0;
    this.numberOfSteps = 0;

    // get first network interface data
    if( ( 'interfaces' in this.data ) && ( $.isPlainObject( this.data['interfaces'] ) ) ) {
      this.interfaceList = Object.keys( this.data['interfaces'] );
      this.numberOfSteps = this.interfaceList.length + 1;
    }

    // build page 1 of network 
    WizzardNetworkInterface.Build( "#ID_WIDGET" + this.dynid, {
      macaddr: this.interfaceList[ this.currentStep ],
      data: this.data['interfaces'][ this.interfaceList[ this.currentStep ] ],
      step: this.currentStep,
      numsteps: this.numberOfSteps,
    } );

    // next/prev buttons
    $( "#ID_PREVIOUS" + this.dynid ).click( () => {
      this.ControlClickPrevious( );
    } );
    $( "#ID_NEXT" + this.dynid ).click( () => {
      this.ControlClickNext();
    });
  
    this.FormEnableNextButton( true );

  }// load


  //---------------------------------------------------------------------------
  //  FORM
  //---------------------------------------------------------------------------

  FormEnableNextButton( isEnabled )
  {
    if( isEnabled )  {
      $( "#ID_NEXT" + this.dynid ).removeClass( "disabled" );
      $( "#ID_NEXT" + this.dynid ).addClass( "default" );
      $( "#ID_NEXT" + this.dynid ).addClass( "hover" );
    }
    else {
      $( "#ID_NEXT" + this.dynid ).removeClass( "default" );
      $( "#ID_NEXT" + this.dynid ).removeClass( "hover" );
      $( "#ID_NEXT" + this.dynid ).addClass( "disabled" );
    }
  }

  //---------------------------------------------------------------------------
  //  CONTROL
  //---------------------------------------------------------------------------

  ControlClickNext()
  {
    // save current step
    let numErr = $( "#ID_WIDGET" + this.dynid ).viewable( "validate" );
    if( numErr )  {
      return;
    }

    // widget should save data
    $( "#ID_WIDGET" + this.dynid ).viewable( "option", "save", this.data );

    // next step
    this.currentStep++;

    // dns step
    if( this.currentStep == ( this.numberOfSteps - 1 ) ) {
      WizzardNetworkDns.Build( "#ID_WIDGET" + this.dynid, {
        data: this.data,
        step: this.currentStep,
        numsteps: this.numberOfSteps,
      } );
    }
    // nic step
    else if( this.currentStep < this.numberOfSteps - 1 )  {
      WizzardNetworkInterface.Build( "#ID_WIDGET" + this.dynid, {
        macaddr: this.interfaceList[ this.currentStep ],
        data: this.data['interfaces'][ this.interfaceList[ this.currentStep ] ],
        step: this.currentStep,
        numsteps: this.numberOfSteps,
      } );
    }
    // exit
    else {
      this.FireEvent( "result", this.data );
      this.FireCommand( "next" );
    }
  }

  ControlClickPrevious()
  {
    this.currentStep--;

    if( this.currentStep >= 0 ) {
      WizzardNetworkInterface.Build( "#ID_WIDGET" + this.dynid, {
        macaddr: this.interfaceList[ this.currentStep ],
        data: this.data['interfaces'][ this.interfaceList[ this.currentStep ] ],
        step: this.currentStep,
        numsteps: this.numberOfSteps,
      } );
    }
    else {
      this.data['id'] = WizzardPageNetwork.id;
      this.FireEvent( "result", this.data );
      this.FireCommand( "previous" );
    }
  }

  //---------------------------------------------------------------------------
  //  event interface to upper layer
  //---------------------------------------------------------------------------

  FireCommand( command )
  {
    this.FireEvent( "command", command );
  }

  //===========================================================================
  //  BUILDER
  //===========================================================================

  static Build( selector, options )
  {
    options['view'] = WizzardPageNetwork.template;
    $( selector ).viewable( options );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

WizzardPageNetwork.Register( WizzardPageNetwork.template );
//=============================================================================
//
//  Wizzard Page
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          18.10.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WizzardNetworkInterface extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATIC (CONFIG)
  //---------------------------------------------------------------------------

  static template = "/pages/install/networkInterface";

  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createOptions )
  {
    this.data = createOptions['data'];
    this.macaddr = createOptions['macaddr'];
    this.currentType = this.data['type'] || "noConfig";
    //console.log( "net copy:", this.macaddr, this.data );

    // optionals
    $( "#ID_OPTIONAL" + this.dynid ).optional( {} );

    // name of interface
    var interfaceName = $.lang( "{$wizpage_nic_name}" );
    interfaceName = interfaceName.replace( "{{name}}", createOptions['data']['displayname'] || createOptions['data']['name'] );
    $( '#ID_NETCARD_NAME' + this.dynid ).text( interfaceName );

    // steps
    var stepText = $.lang( "{$wizpage_nic_steps}" );
    stepText = stepText.replace( "{{step}}", createOptions['step'] + 1 );
    stepText = stepText.replace( "{{numsteps}}", createOptions['numsteps'] );
    $( "#ID_NETCARD_STEP" + this.dynid ).text( stepText );

    // type
    $( '#ID_NETCARD_TYPE' + this.dynid ).choice( {
      size: CONST.size.L,
      type: 'radio',
      data: [
        { id: 'static', text: "{$netcard_static}" },
        { id: 'dhcp', text: "{$netcard_dhcp}" },
        { id: 'noConfig', text: "{$netcard_no_config}" }
      ],
      value: this.currentType,
      result: this.ControlNetcardTypeResult.bind( this )
    } );

    $( '#ID_NETCARD_IP' + this.dynid ).edit( {
      size: CONST.size.L,
      status: "normal",
      value: this.data['ipv4'],
      validate: this.ValidateNetcardIp.bind( this ),
      validateDepends: ["ID_NETCARD_DHCP_POOL_START" + this.dynid, "ID_NETCARD_DHCP_POOL_END" + this.dynid],
    } );

    $( '#ID_NETCARD_NET_MASK' + this.dynid ).edit( {
      size: CONST.size.L,
      status: "normal",
      value: this.data['ipv4mask'],
      validate: Validate.IsNetmask,
    } );

    $( '#ID_NETCARD_GATEWAY' + this.dynid ).edit( {
      size: CONST.size.L,
      status: "normal",
      value: this.data['gateway'],
      validate: this.ValidateGateway.bind( this ),
    } );

    // dhcp controls
    var isDhcpServer = ( 'dhcp-server' in this.data ) && ( ['enable'] in this.data['dhcp-server'] ) && this.data['dhcp-server']['enable'];
    $( '#ID_NETCARD_DHCP_ACTIV' + this.dynid ).choice( {
      type: 'check',
      status: "normal",
      text: '{$netcard_is_dhcp_active}',
      value: isDhcpServer,
      result: this.ControlDhcpServerResult.bind( this ),
    } );

    var dhcpPoolStart = ( ( 'dhcp-server' in this.data ) && ( ['ip-start'] in this.data['dhcp-server'] ) ) ? this.data['dhcp-server']['ip-start'] : "";
    $( '#ID_NETCARD_DHCP_POOL_START' + this.dynid ).edit( {
      size: CONST.size.L,
      status: "normal",
      value: dhcpPoolStart,
      validate: this.ValidateDhcpIpPoolStart.bind( this ),
      validateDepends: ["ID_NETCARD_IP" + this.dynid, "ID_NETCARD_NET_MASK" + this.dynid, "ID_NETCARD_DHCP_POOL_END" + this.dynid]
    } );

    var dhcpPoolEnd = ( ( 'dhcp-server' in this.data ) && ( ['ip-end'] in this.data['dhcp-server'] ) ) ? this.data['dhcp-server']['ip-end'] : "";
    $( '#ID_NETCARD_DHCP_POOL_END' + this.dynid ).edit( {
      size: CONST.size.L,
      status: "normal",
      value: dhcpPoolEnd,
      validate: this.ValidateDhcpIpPoolEnd.bind( this ),
      validateDepends: ["ID_NETCARD_IP" + this.dynid, "ID_NETCARD_NET_MASK" + this.dynid, "ID_NETCARD_DHCP_POOL_START" + this.dynid]
    } );


    // switch to modes
    switch( this.currentType )  {
      case "static":    this.ModeStatic();    break;
      case "dhcp":      this.ModeDhcp();      break;
      case "noConfig":  this.ModeNoConfig();  break;
    }

  }// load

  //---------------------------------------------------------------------------
  //  event function from upper layer
  //  
  OnEvent( eventName, param, data )
  {
    if( eventName == "option" && param == "save" ) {
      this.FormSave( data );
    }
  }

  //---------------------------------------------------------------------------
  //  FORM
  //---------------------------------------------------------------------------

  FormSave( cosconfData )
  {
    this.FormReadControls();
    cosconfData['interfaces'][ this.macaddr ] = this.data;
  }

  //---------------------------------------------------------------------------
  //  read controls (dispatcher)
  //
  FormReadControls()
  {
    if( this.mode == "modeDhcp" ) {
      return this.FormReadControlsDhcp();
    }
    else if( this.mode == "modeNoConfig" ) {
      return this.FormReadControlsNoConfig();
    }
    else if( this.mode == "modeStatic" ) {
      return this.FormReadControlsStatic();
    }
    else if( this.mode == "modeStaticDhcpServer" ) {
      return this.FormReadControlsStaticDhcpServer();
    }
    else {
      return;
    }
  }

  //---------------------------------------------------------------------------
  //  read controls (dhcp mode)
  //
  FormReadControlsDhcp()
  {
    // no dhcp-server
    delete this.data['dhcp-server'];
    this.data['configured'] = true;
    this.data['type'] = "dhcp";
  }

  //---------------------------------------------------------------------------
  //  read controls (mode no config)
  //
  FormReadControlsNoConfig()
  {
    // no dhcp-server
    delete this.data['dhcp-server'];
    this.data['configured'] = false;
  }

  //---------------------------------------------------------------------------
  //  read controls (static )
  //
  FormReadControlsStatic()
  {
    // no dhcp-server
    delete this.data['dhcp-server'];
    this.data['configured'] = true;
    this.data['type'] = "static";
    this.data['ipv4'] = $( '#ID_NETCARD_IP' + this.dynid ).edit( "option", "value" );
    this.data['ipv4mask'] = $( '#ID_NETCARD_NET_MASK' + this.dynid ).edit( "option", "value" );
    this.data['gateway'] = $( '#ID_NETCARD_GATEWAY' + this.dynid ).edit( "option", "value" );
  }

  //---------------------------------------------------------------------------
  //  read controls (static )
  //
  FormReadControlsStaticDhcpServer()
  {
    // no dhcp-server
    this.data['configured'] = true;
    this.data['type'] = "static";
    this.data['ipv4'] = $( '#ID_NETCARD_IP' + this.dynid ).edit( "option", "value" );
    this.data['ipv4mask'] = $( '#ID_NETCARD_NET_MASK' + this.dynid ).edit( "option", "value" );
    this.data['gateway'] = $( '#ID_NETCARD_GATEWAY' + this.dynid ).edit( "option", "value" );
    this.data['dhcp-server'] = {};
    this.data['dhcp-server']['enable'] = true;
    this.data['dhcp-server']['ip-start'] = $( '#ID_NETCARD_DHCP_POOL_START' + this.dynid ).edit( "option", "value" );
    this.data['dhcp-server']['ip-end'] = $( '#ID_NETCARD_DHCP_POOL_END' + this.dynid ).edit( "option", "value" );
    this.data['dhcp-server']['options'] = DhcpOptionsDialog.BuildDhcpDefaultOptions( this.data );

  }
  

  //---------------------------------------------------------------------------
  //  CONTROL
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  type result from control
  //
  ControlNetcardTypeResult( result )
  {
    // switch to modes
    this.currentType = result;
    switch( this.currentType ) {
      case "static":    this.ModeStatic();    break;
      case "dhcp":      this.ModeDhcp();      break;
      case "noConfig":  this.ModeNoConfig();  break;
    }
  }

  //---------------------------------------------------------------------------
  //  enable/disable dhcp server result
  //
  ControlDhcpServerResult( isEnabled )
  {
    if( isEnabled )  {
      this.ModeStaticDhcpServer();
    }
    else {
      this.ModeStatic();
    }
  }

  //---------------------------------------------------------------------------
  //  MODES
  //---------------------------------------------------------------------------

  ModeStatic()
  {
    if( this.mode == "modeStatic" ) 
      return;
    
    if( $( '#ID_NETCARD_DHCP_ACTIV' + this.dynid ).choice( "option", "value" ) )  {
      return this.ModeStaticDhcpServer();
    }

    this.mode = "modeStatic";

    $( "#ID_NETCARD_IP" + this.dynid ).edit( "option", "status", "normal" );
    $( "#ID_NETCARD_NET_MASK" + this.dynid ).edit( "option", "status", "normal" );
    $( "#ID_NETCARD_GATEWAY" + this.dynid ).edit( "option", "status", "normal" );

    // show/hide all controls
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_IP_SETTINGS" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_DHCP_ACTIVE_SETTINGS" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_DHCP_SETTINGS" );

    // disable validate for not needed controls
    $( "#ID_NETCARD_DHCP_POOL_START" + this.dynid ).edit( "validate", false );
    $( "#ID_NETCARD_DHCP_POOL_END" + this.dynid ).edit( "validate", false );

    // focus
    $( "#ID_NETCARD_IP" + this.dynid ).edit( "focus" );

  }

  ModeStaticDhcpServer()
  {
    if( this.mode == "modeStaticDhcpServer" )
      return;
    this.mode = "modeStaticDhcpServer";

    $( "#ID_NETCARD_IP" + this.dynid ).edit( "option", "status", "normal" );
    $( "#ID_NETCARD_NET_MASK" + this.dynid ).edit( "option", "status", "normal" );
    $( "#ID_NETCARD_GATEWAY" + this.dynid ).edit( "option", "status", "normal" );

    // show/hide all controls
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_IP_SETTINGS" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_DHCP_ACTIVE_SETTINGS" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_DHCP_SETTINGS" );

    $( "#ID_NETCARD_DHCP_POOL_START" + this.dynid ).edit( "focus" );

  }

  ModeDhcp()
  {
    if( this.mode == "modeDhcp" )
      return;
    this.mode = "modeDhcp";

    $( "#ID_NETCARD_IP" + this.dynid ).edit( "option", "status", "readonly" );
    $( "#ID_NETCARD_NET_MASK" + this.dynid ).edit( "option", "status", "readonly" );
    $( "#ID_NETCARD_GATEWAY" + this.dynid ).edit( "option", "status", "readonly" );

    // show/hide all controls
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_IP_SETTINGS" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_DHCP_ACTIVE_SETTINGS" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_DHCP_SETTINGS" );

    // disable validate for not needed controls
    $( "#ID_NETCARD_IP" + this.dynid ).edit( "validate", false );
    $( "#ID_NETCARD_NET_MASK" + this.dynid ).edit( "validate", false );
    $( "#ID_NETCARD_GATEWAY" + this.dynid ).edit( "validate", false );
    $( "#ID_NETCARD_DHCP_POOL_START" + this.dynid ).edit( "validate", false );
    $( "#ID_NETCARD_DHCP_POOL_END" + this.dynid ).edit( "validate", false );

  }

  ModeNoConfig()
  {
    if( this.mode == "modeNoConfig" )
      return;
    this.mode = "modeNoConfig";

    // hide all controls
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_IP_SETTINGS" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_DHCP_ACTIVE_SETTINGS" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_DHCP_SETTINGS" );

    // disable validate for not needed controls
    $( "#ID_NETCARD_IP" + this.dynid ).edit( "validate", false );
    $( "#ID_NETCARD_NET_MASK" + this.dynid ).edit( "validate", false );
    $( "#ID_NETCARD_GATEWAY" + this.dynid ).edit( "validate", false );
    $( "#ID_NETCARD_DHCP_POOL_START" + this.dynid ).edit( "validate", false );
    $( "#ID_NETCARD_DHCP_POOL_END" + this.dynid ).edit( "validate", false );

  }


  //---------------------------------------------------------------------------
  //  CONTROL
  //---------------------------------------------------------------------------


  //===========================================================================
  //  VALIDATE
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  validate event from upper layer
  //
  OnValidate()
  {
    return this.Validate();
  }

  //-----------------------------------------------------------------------------
  //  perform validation
  //
  Validate()
  {
    var numErr = 0;
    if( this.mode == "modeStatic" )  {
      numErr += $( "#ID_NETCARD_IP" + this.dynid ).edit( "validate" );
      numErr += $( "#ID_NETCARD_NET_MASK" + this.dynid ).edit( "validate" );
      numErr += $( "#ID_NETCARD_GATEWAY" + this.dynid ).edit( "validate" );
    }
    else if( this.mode == "modeStaticDhcpServer" ) {
      numErr += $( "#ID_NETCARD_IP" + this.dynid ).edit( "validate" );
      numErr += $( "#ID_NETCARD_NET_MASK" + this.dynid ).edit( "validate" );
      numErr += $( "#ID_NETCARD_GATEWAY" + this.dynid ).edit( "validate" );
      numErr += $( "#ID_NETCARD_DHCP_POOL_START" + this.dynid ).edit( "validate" );
      numErr += $( "#ID_NETCARD_DHCP_POOL_END" + this.dynid ).edit( "validate" );
    }
    else { // "modeDhcp" || "modeNoConfig" 
      numErr = 0;
    }
    return numErr;
  }

  //-----------------------------------------------------------------------------
  // IP-Address: Validate Ip and scope if dhcp-server is set
  //
  ValidateNetcardIp( validateData )
  {
    let ipNetcard = new IpAddress( validateData.value );
    let ipPoolStart = new IpAddress( $( "#ID_NETCARD_DHCP_POOL_START" + this.dynid ).edit( "option", "value" ) );
    let ipPoolEnd = new IpAddress( $( "#ID_NETCARD_DHCP_POOL_END" + this.dynid ).edit( "option", "value" ) );
    let isDhcpServerActive = $( '#ID_NETCARD_DHCP_ACTIV' + this.dynid ).choice( "option", "value" );

    // simple ip address validation
    if( !Validate.IsIpAddress( validateData.value ) ) {
      validateData.error = "{$const_error_novalidip}";
      return false;
    }

    // error if netcard ip is in range of dhcp scope
    if( ( isDhcpServerActive )
     && ( ipPoolStart.IsValid() )
     && ( ipPoolEnd.IsValid() )
     && ( ipNetcard.AsNumber() >= ipPoolStart.AsNumber() )
     && ( ipNetcard.AsNumber() <= ipPoolEnd.AsNumber() )
    ) {
      validateData.error = "{$netcard_pool_error_ip}";
      return false;
    }

    // no error
    validateData.error = null;
    return true;
  }

  //-----------------------------------------------------------------------------
  // DHCP server: IP pool start address
  //
  ValidateDhcpIpPoolStart( validateData )
  {
    // prepare
    let ipv4 = $( '#ID_NETCARD_IP' + this.dynid ).edit( "option", "value" );
    let ipv4mask = $( '#ID_NETCARD_NET_MASK' + this.dynid ).edit( "option", "value" );

    // validation
    if( !Validate.IsIpAddress( validateData.value ) ) {
      validateData.error = "{$const_error_novalidip}";
      return false;   // error ip
    }
    if( !Validate.IsIpInSubnet( ipv4, ipv4mask, validateData.value ) ) {
      validateData.error = "{$dhcp_scope_error_range}";
      return false;   // error subnet
    }

    // no error
    validateData.error = null;
    return true;
  }

  //-----------------------------------------------------------------------------
  // DHCP server: IP pool end address
  //
  ValidateDhcpIpPoolEnd( validateData )
  {
    // prepare
    let ipv4 = $( '#ID_NETCARD_IP' + this.dynid ).edit( "option", "value" );
    let ipv4mask = $( '#ID_NETCARD_NET_MASK' + this.dynid ).edit( "option", "value" );
    let ipPoolStartString = $( "#ID_NETCARD_DHCP_POOL_START" + this.dynid ).edit( "option", "value" );
    let ipPoolStart = new IpAddress( ipPoolStartString );
    let ipPoolEnd = new IpAddress( validateData.value );

    // validation
    if( !Validate.IsIpAddress( validateData.value ) ) {
      validateData.error = "{$const_error_novalidip}";
      return false;   // error ip
    }
    if( !Validate.IsIpInSubnet( ipv4, ipv4mask, validateData.value ) ) {
      validateData.error = "{$dhcp_scope_error_range}";
      return false;   // error subnet
    }
    if( ipPoolEnd.AsNumber() <= ipPoolStart.AsNumber() ) {
      validateData.error = "{$netcard_pool_error_range}";
      return false;   // error ip-pool range
    }

    // no error
    validateData.error = null;
    return true;
  }

  //-----------------------------------------------------------------------------
  // Gateway: Validate Ip-Address but might be empty. 
  //
  ValidateGateway( validateData )
  {
    if( validateData.value == false ) {
      return validateData.error = null;
    }
    else {
      return Validate.IsIpAddress( validateData );
    }
  }


  //===========================================================================
  //  BUILDER
  //===========================================================================

  static Build( selector, options )
  {
    options['view'] = WizzardNetworkInterface.template;
    $( selector ).viewable( options );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

WizzardNetworkInterface.Register( WizzardNetworkInterface.template );
//=============================================================================
//
//  Wizzard Page
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          18.10.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WizzardNetworkDns extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATIC (CONFIG)
  //---------------------------------------------------------------------------

  static template = "/pages/install/networkDns";

  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createOptions )
  {
    this.data = createOptions['data'];
    this.dnsMode = null;

    // steps
    var stepText = $.lang( "{$wizpage_nic_steps}" );
    stepText = stepText.replace( "{{step}}", createOptions['step'] + 1 );
    stepText = stepText.replace( "{{numsteps}}", createOptions['numsteps'] );
    $( "#ID_NETCARD_STEP" + this.dynid ).text( stepText );

    // edit
    $( "#ID_DNS_SERVER" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "normal",
    } );

    this.FormFillDnsServer();

  }// load

  //---------------------------------------------------------------------------
  //  event function from upper layer
  //  
  OnEvent( eventName, param, data )
  {
    if( eventName == "option" && param == "save" ) {
      this.FormSave( data );
    }
  }

  //---------------------------------------------------------------------------
  //  DATA
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  save data into cosconf
  //
  FormSave( cosconfData )
  {
    var dnsEntry = $( "#ID_DNS_SERVER" + this.dynid ).edit( "option", "value" );

    if( ( dnsEntry.length > 0 ) 
     && ( this.dnsMode == "manual" || this.dnsMode == null ) 
      ) {
      for( var macAddr in cosconfData['interfaces'] ) {
        var netInterface = cosconfData['interfaces'][ macAddr ];
        netInterface['dns'] = dnsEntry;
      }
    }
    else if( ( dnsEntry.length == 0 )
     && ( this.dnsMode == "manual" || this.dnsMode == null )
      ) {
      for( var macAddr in cosconfData['interfaces'] ) {
        var netInterface = cosconfData['interfaces'][macAddr];
        delete netInterface['dns'];
      }
    }
  }

  //---------------------------------------------------------------------------
  //  read data and fill control
  //
  FormFillDnsServer()
  {
    var dnsServer = "";

    // first check if there is a interface using dhcp
    for( var macAddr in this.data['interfaces'] ) {
      var netInterface = this.data['interfaces'][ macAddr ];
      if( netInterface['type'] == "dhcp" && ( 'supersede' in netInterface ) ) {
        dnsServer = netInterface['supersede']['6'][ 0 ];
        $( "#ID_DNS_SERVER" + this.dynid ).edit( "option", "value", dnsServer );
        $( "#ID_DNS_SERVER" + this.dynid ).edit( "option", "status", "readonly" );
        $( "#ID_DNS_SERVER_HINT" + this.dynid ).text( $.lang( "{$netcard_hint_dns_server_supersede}" ) );
        this.dnsMode = "supersede";
        return;
      }
      if( netInterface['type'] == "dhcp" && ( 'dhcp-options' in netInterface ) )  {
        dnsServer = netInterface['dhcp-options']['6'][ 0 ];
        $( "#ID_DNS_SERVER" + this.dynid ).edit( "option", "value", dnsServer );
        $( "#ID_DNS_SERVER" + this.dynid ).edit( "option", "status", "readonly" );
        $( "#ID_DNS_SERVER_HINT" + this.dynid ).text( $.lang( "{$netcard_hint_dns_server_dhcp}" ) );
        this.dnsMode = "dhcp";
        return;
      }
      if( 'dns' in netInterface )  {
        if( Array.isArray( netInterface['dns'] ) ) {
          dnsServer = netInterface['dns'][ 0 ];
        }
        else {
          dnsServer = netInterface['dns'];
        }
        $( "#ID_DNS_SERVER" + this.dynid ).edit( "option", "value", dnsServer );
        $( "#ID_DNS_SERVER" + this.dynid ).edit( "option", "status", "normal" );
        $( "#ID_DNS_SERVER_HINT" + this.dynid ).text( $.lang( "{$netcard_hint_dns_server_opt}" ) );
        this.dnsMode = "manual";
        return;
      }
    }

  }

  //---------------------------------------------------------------------------
  //  COMMAND
  //---------------------------------------------------------------------------
  
  CommandNext()
  {
    return false;
  }

  CommandPrevious()
  {
    return false;
  }


  //---------------------------------------------------------------------------
  //  CONTROL
  //---------------------------------------------------------------------------


  //---------------------------------------------------------------------------
  //  event interface to upper layer
  //---------------------------------------------------------------------------

  FireCommand( command )
  {
    this.FireEvent( "command", command );
  }



  //===========================================================================
  //  BUILDER
  //===========================================================================

  static Build( selector, options )
  {
    options['view'] = WizzardNetworkDns.template;
    $( selector ).viewable( options );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

WizzardNetworkDns.Register( WizzardNetworkDns.template );
//=============================================================================
//
//  Wizzard Page
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          18.10.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WizzardPageTimezone extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATIC (CONFIG)
  //---------------------------------------------------------------------------

  static id = "timezone";
  static template = "/pages/install/timezone";
  static sidebarText = "{$wizpage_timezone_sidebar}";

  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createOptions )
  {
    this.data = createOptions['data'];
    this.data['id'] = WizzardPageTimezone.id; //['tz-list']
    console.log( "tz-load", this.data );


    this.ntpZoneListNew = new Dataset( { id: "id" } );
    this.ntpZoneListNew.SetData( this.data['tz-list'] );
    $( "#ID_ZONE_NEW" + this.dynid ).listbox( {
      size: CONST.size.XL,
      height: 10,
      type: 'droplist',
      placeholder: "{$const_common_empty_placeholder}",
      searchPlaceholder: "{$const_common_search_placeholder}",
      selected: [ this.data['timezone']['id'] ],
      search: true,
      data: this.ntpZoneListNew,
      validate: this.ValidateZone.bind( this )
    } );

    $( "#ID_PREVIOUS" + this.dynid ).click( () => {
      this.ControlClickPrevious( "previous" );
    } );
    $( "#ID_NEXT" + this.dynid ).click( () => {
      this.ControlClickNext();
    });
  
    this.FormEnableNextButton( true );

  }// load

  //---------------------------------------------------------------------------
  //  viewable event function
  //
  OnEvent( eventName, param, data )
  {
    if( eventName == "error" )  {
      this.FormEnableNextButton( param == 0 );
    }
  }

  //---------------------------------------------------------------------------
  //  FORM
  //---------------------------------------------------------------------------

  FormEnableNextButton( isEnabled )
  {
    if( isEnabled )  {
      $( "#ID_NEXT" + this.dynid ).removeClass( "disabled" );
      $( "#ID_NEXT" + this.dynid ).addClass( "default" );
      $( "#ID_NEXT" + this.dynid ).addClass( "hover" );
    }
    else {
      $( "#ID_NEXT" + this.dynid ).removeClass( "default" );
      $( "#ID_NEXT" + this.dynid ).removeClass( "hover" );
      $( "#ID_NEXT" + this.dynid ).addClass( "disabled" );
    }
  }

  //---------------------------------------------------------------------------
  //  read before next
  //
  FormReadControls()
  {
    let newZoneId = $( "#ID_ZONE_NEW" + this.dynid ).listbox( "option", "selected" )[ 0 ];
    var newZone = this.ntpZoneListNew.Get( newZoneId );
    this.data['timezone']['id'] = newZone['id'];
    this.data['timezone']['name'] = newZone['text'];
  }

  //---------------------------------------------------------------------------
  //  CONTROL
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  click next 
  //
  ControlClickNext()
  {
    // save current step
    if( this.Validate() > 0 ) {
      return;
    }

    this.FormReadControls();
    this.FireEvent( "result", this.data );
    this.FireCommand( "next" );
  }

  //---------------------------------------------------------------------------
  //  click back
  //
  ControlClickPrevious()
  {
    this.FormReadControls();
    this.FireEvent( "result", this.data );
    this.FireCommand( "previous" );
  }

  //---------------------------------------------------------------------------
  //  new region selected
  //
  ControlRegionSelected( selected )
  {
    $( "#ID_ZONE" + this.dynid ).listbox( "option", "selected", [] );
    this.FormBuildTimezones( selected[0] );
  }

  //---------------------------------------------------------------------------
  //  VALIDATE
  //---------------------------------------------------------------------------

  Validate()
  {
    var numErr = 0;
    numErr += $( "#ID_ZONE" + this.dynid ).listbox( "validate" );
    return numErr;
  }

  ValidateZone( validateData )
  {
    if( !Array.isArray( validateData.value ) || validateData.value.length <= 0 )  {
      validateData.error = "{$wizpage_timezone_errorempty}";
      return 1;
    }
    return 0;
  }

  //---------------------------------------------------------------------------
  //  event interface to upper layer
  //---------------------------------------------------------------------------

  FireCommand( command )
  {
    this.FireEvent( "command", command );
  }

  //---------------------------------------------------------------------------
  //  find text to timezone id
  //
  HelperFindTimezone( tzId )
  {
    for( let region in this.data['tz-list'] ) {
      var regionData = this.data['tz-list'][ region ];
      for( let zone in regionData ) {
        if( regionData[ zone ] instanceof Object ) {
          for( let subzone in regionData[ zone ] ) {
            if( regionData[ zone ][ subzone ] == tzId ) {
              return { region: region, zone: subzone };
            }
          }
        }
        else if( regionData[ zone ] == tzId ) {
          return { region: region, zone: zone };
        }
      }
    }

    return { region: "", zone: tzId };
  }

  //===========================================================================
  //  BUILDER
  //===========================================================================

  static Build( selector, options )
  {
    options['view'] = WizzardPageTimezone.template;
    $( selector ).viewable( options );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

WizzardPageTimezone.Register( WizzardPageTimezone.template );
//=============================================================================
//
//  Wizzard Page
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          18.10.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WizzardPagePassword extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATIC (CONFIG)
  //---------------------------------------------------------------------------

  static id = "password";
  static template = "/pages/install/password";
  static sidebarText = "{$wizpage_password_sidebar}";

  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createOptions )
  {
    this.data = createOptions['data'];
    this.data['id'] = WizzardPagePassword.id;

    $( "#ID_ACCOUNT" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly",
      value: "admin"
    } );

    $( "#ID_PASSWORD1" + this.dynid ).edit( {
      size: CONST.size.L,
      password: true,
      value: "",
      validateDepends: ["ID_PASSWORD2" + this.dynid],
      validate: this.ValidateEdit1.bind( this )
    } );

    $( "#ID_PASSWORD2" + this.dynid ).edit( {
      size: CONST.size.L,
      password: true,
      value: "",
      validateDepends: ["ID_PASSWORD1" + this.dynid],
      validate: this.ValidateEdit2.bind( this )
    } );

    $( "#ID_PREVIOUS" + this.dynid ).click( () => {
      this.ControlClickPrevious( "previous" );
    } );
    $( "#ID_NEXT" + this.dynid ).click( () => {
      this.ControlClickNext();
    });
  
    this.FormEnableNextButton( true );

  }// load

  //---------------------------------------------------------------------------
  //  viewable event function
  //
  OnEvent( eventName, param, data )
  {
    if( eventName == "error" ) {
      this.FormEnableNextButton( param == 0 );
    }
  }


  //---------------------------------------------------------------------------
  //  FORM
  //---------------------------------------------------------------------------

  FormEnableNextButton( isEnabled )
  {
    if( isEnabled )  {
      $( "#ID_NEXT" + this.dynid ).removeClass( "disabled" );
      $( "#ID_NEXT" + this.dynid ).addClass( "default" );
      $( "#ID_NEXT" + this.dynid ).addClass( "hover" );
    }
    else {
      $( "#ID_NEXT" + this.dynid ).removeClass( "default" );
      $( "#ID_NEXT" + this.dynid ).removeClass( "hover" );
      $( "#ID_NEXT" + this.dynid ).addClass( "disabled" );
    }
  }

  FormReadControls()
  {
    this.data['password'] = $( "#ID_PASSWORD1" + this.dynid ).edit( "option", "value" );
  }

  //---------------------------------------------------------------------------
  //  CONTROL
  //---------------------------------------------------------------------------

  ControlClickNext()
  {
    // save current step
    if( this.Validate() > 0 ) {
      return;
    }

    this.FormReadControls();
    this.FireEvent( "result", this.data );
    this.FireCommand( "next" );
  }

  ControlClickPrevious()
  {
    this.FormReadControls();
    this.FireEvent( "result", this.data );
    this.FireCommand( "previous" );
  }

  //---------------------------------------------------------------------------
  //  event interface to upper layer
  //---------------------------------------------------------------------------

  FireCommand( command )
  {
    this.FireEvent( "command", command );
  }

  //---------------------------------------------------------------------------
  //  VALIDATE
  //---------------------------------------------------------------------------

  Validate()
  {
    var numErr = 0;
    numErr += $( "#ID_PASSWORD1" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_PASSWORD2" + this.dynid ).edit( "validate" );
    return numErr;
  }

  //-----------------------------------------------------------------
  //  validate password repeat
  //
  ValidateEdit1( validateControl )
  {
    let password1 = $( "#ID_PASSWORD1" + this.dynid ).edit( "option", "value" );

    if( password1.length == 0 ) {
      validateControl.error = "{$change_password_error_empty}";
      return 1;
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  validate password repeat
  //
  ValidateEdit2( validateControl )
  {
    let password1 = $( "#ID_PASSWORD1" + this.dynid ).edit( "option", "value" );
    let password2 = $( "#ID_PASSWORD2" + this.dynid ).edit( "option", "value" );

    if( password1.length == 0 && password2.length == 0 ) {
      return 0;
    }

    if( password1 != password2 ) {
      validateControl.error = "{$change_password_error_different}";
      return 1;
    }

    return 0;
  }

  //===========================================================================
  //  BUILDER
  //===========================================================================

  static Build( selector, options )
  {
    options['view'] = WizzardPagePassword.template;
    $( selector ).viewable( options );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

WizzardPagePassword.Register( WizzardPagePassword.template );
//=============================================================================
//
//  Wizzard Page
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          18.10.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WizzardPageResult extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATIC (CONFIG)
  //---------------------------------------------------------------------------

  static id = "result";
  static template = "/pages/install/result";
  static sidebarText = "{$wizpage_result_sidebar}";

  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createOptions )
  {
    $( "#ID_FINISH" + this.dynid ).click( () => {
      this.ControlClickNext();
    });
  
  }// load


  //---------------------------------------------------------------------------
  //  FORM
  //---------------------------------------------------------------------------


  //---------------------------------------------------------------------------
  //  CONTROL
  //---------------------------------------------------------------------------

  ControlClickNext()
  {
    $( "#ID_FINISH" + this.dynid ).attr( "class", "button disabled" );
    this.FireEvent( "result", { id: WizzardPageResult.id } );
    this.FireCommand( "next" );
  }

  ControlClickPrevious()
  {
    this.FireEvent( "result", { id: WizzardPageResult.id } );
    this.FireCommand( "previous" );
  }

  //---------------------------------------------------------------------------
  //  event interface to upper layer
  //---------------------------------------------------------------------------

  FireCommand( command )
  {
    this.FireEvent( "command", command );
  }

  //===========================================================================
  //  BUILDER
  //===========================================================================

  static Build( selector, options )
  {
    options['view'] = WizzardPageResult.template;
    $( selector ).viewable( options );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

WizzardPageResult.Register( WizzardPageResult.template );
//=============================================================================
//
//  Base Form Class
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          05.05.2021
//=============================================================================

class FormBase extends Viewable
{
  //=============================================================================
  //  CLASS MEMBER VARIABLES
  //=============================================================================

  $toast = null;
  $toolbar = null;
  entinfo = null;
  entity = null;
  _formbaseIsSaveable = false;
  isDeviceRestartNeeded = false;
  isRestarting = false;
  isSystemForm = false;
  disabledFolders = [ "folder.unconfdev" ];

  //=============================================================================
  //  STATIC FUNCTIONS
  //=============================================================================

  static MinimumNeededLicenses()
  {
    return null;
  }

  //=============================================================================
  //  ABSTRACT FUNCTIONS
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  Load( data ): entry point while create viewable
  //
  Load( data )
  {
    throw new Error( "Load must be overridden!" );
  }

  //-----------------------------------------------------------------------------
  //  OnEvent( eventName, data ): main event handler from viewable
  //
  OnEvent( eventName, data )
  {
    //console.log( "OnEvent", eventName, data );
    
    // modified handling
    if( eventName == "modified" ) {
      
      // only if a save-button is present
      if( this._formbaseIsSaveable )  {
       
        // modfied-event is sent after error, so check error as well
        if( this.Option( "error" ) == 0 ) {
          if( data > 0 ) this.StatusModified();
          else {
            this.StatusIdle();
          }
        }
        else {
          this.StatusError();
        }

        if( !this.isSystemForm )  {
          PageMain.DocumentModified( this.createOptions['entid'], data > 0 );
        }
      }
    }
    // error handling (data := num error)
    else if( eventName == "error" && this.Option( "modified" ) > 0 ) {
      
      // only if a save-button is present
      if( this._formbaseIsSaveable )  {
        if( data > 0 ) this.StatusError();
        else this.StatusModified();
      }
    
    }
    // message "save first"
    else if( eventName == "toastSaveFirst" && this.Option( "modified" ) > 0 ) {
      this.ToastShowSaveFirst( "#ID_TOAST" + this.dynid );
    }

  }


  //=============================================================================
  //  REBOOT
  //=============================================================================  

  DataMarkEntityForReboot()
  {
    if( ( this.isDeviceRestartNeeded )
     && ( 'adminUi' in this.entity )
     && ( 'macaddr' in this.entity['adminUi'] )
     && ( typeof this.entity['adminUi']['macaddr'] == "string" )
     && this.entity['adminUi']['macaddr'].match( /((?:[a-zA-Z0-9]{2}[:-]){5}[a-zA-Z0-9]{2})/ )
      ) {
      Data.Instance().MarkEntityForReboot( this.entity['entid'] );
    }
  }

  //---------------------------------------------------------------------------
  //  update description lines if some names has been changed
  //
  DataEntinfoUpdate( entinfo, changedList )
  {
    //console.log( "ENTINFO-UPD", entinfo, changedList );
    if( entinfo['entid'] != this.entinfo['entid'] ) {
      return;
    }

    //if( entinfo['entid'] == this.entinfo['entid'] ) {
    this.PermissionCheck( entinfo );
    //}

    // ignore updates of userparam => ['name', 'entid', 'userparam']
    if( Array.isArray( changedList ) && changedList[ 2 ] == "userparam" )  {
      return;
    }

    if( this.Option( "modified" ) && this.status != "statusSaveRequest" )  {
      this.ToastShowModifyConflict( "#ID_TOAST" + this.dynid );
    }
    else {
      this.DataRequest();
    }
  }

  //=============================================================================
  //  COMMANDS predefined
  //=============================================================================

  CommandUndo() { }
  CommandSave() { }
  CommandLists() { }
  
  //-------------------------------------------------------
  //  reboot this device
  //
  CommandDeviceRestart()
  { 
    Data.Instance().Reboot().RebootDevice( this.entinfo['entid'] );
  }

  //-------------------------------------------------------
  //  show permission dialog
  //
  CommandPermissions() 
  { 
    PermissionsDialog.Show( { entid: this.entinfo['entid'] } );
  }

  //-------------------------------------------------------
  //  show help for current page
  //
  CommandDelete()
  {
    if( this.entinfo == null )  {
      console.warn( "You must set entinfo!" );
      return;
    }

    if( this.isSystemForm )  {
      console.warn( "A systemform can't be deleted!" );
      return;
    }

    // delete after alert-box
    PageMain.DeleteEntity( this.entinfo['entid'] );

  }

  //-------------------------------------------------------
  //  show help for current page
  //
  CommandIncompleteValidate()
  {
    // fill data into controls and do 1st validation
    if( this.Validate() == 0 ) {
      // if no errors occur leave the status modified it as it is
      if( this.status !== "statusModified" ) {
        this.StatusIdle();
      }
      else  {
        this.StatusModified();
      }
    }
    else {
      this.StatusError();
    }
    
    this.ToastDestroy();
  }


  //=============================================================================
  //  VALIDATION
  //=============================================================================
  
  //-----------------------------------------------------------------------------
  //  permission check for own statepoints
  //
  PermissionCheck( entinfo )
  {
  }

  //-----------------------------------------------------------------------------
  //  Validate(): do validation
  //
  Validate()
  {
    throw new Error( "Validate must be overridden!" );
  }

  //-----------------------------------------------------------------------------
  //  Validate(): do validation
  //
  ValidateOff()
  {
    throw new Error( "ValidateOff must be overridden!" );
  }

  //-----------------------------------------------------------------------------
  //  OnValidate(): description
  //
  OnValidate()
  {
    return super.OnValidate();
  }


  //===========================================================================
  //  STATUS MACHINE
  //===========================================================================

  //---------------------------------------------------------------------------
  //  entered in DataRequest
  //
  StatusLoading()
  {
    this.status = "statusLoading";
    //console.log( this.status );

    // disable all controls while loading
    this.$container.broadcast( "option", "status", "disabled" );

    // prevent commands while loading
    this.FormToolbarButtonStatus( "ID_SAVE", "disabled" );
    this.FormToolbarButtonStatus( "ID_UNDO", "disabled" );
    this.FormToolbarButtonStatus( "ID_PERMISSIONS", "disabled" );
    this.FormToolbarButtonStatus( "ID_LISTS", "disabled" );
    this.FormToolbarButtonStatus( "ID_DEVICERESTART", "disabled" );
    this.FormToolbarButtonStatus( "ID_DELETE", "disabled" );
  }

  //-----------------------------------------------------------------------------
  //  entered in DataResponseEntity
  //
  StatusLoadFinished()
  {
    let isOutsideReloading = ( this.status != "statusLoading" && this.status != "statusSaveRequest" );
    let isModifyConflict = ( this.status == "statusModified" || this.status == "statusError" ) && ( this.Option( "modified" ) > 0 );
    let isIncomplete = ( this.entity && ( 'adminUi' in this.entity ) && ( 'incomplete' in this.entity['adminUi'] ) && ( this.entity['adminUi']['incomplete'] ) );

    this.status = "statusLoadFinished";
    //console.log( this.status );

    // not modified -> clear flag
    this.isDeviceRestartNeeded = Verify.Instance().IsDeviceRestartNeeded( this.entinfo['entid'] );

    // fill controls has been called short before, so we need to 
    // put this code into the timeout queue, because viewable creation 
    // also uses setTimeout.
    setTimeout( () =>  {

      this.ToastDestroy();

      if( this.entity && this.disabledFolders.indexOf( this.entity['parent'] ) >= 0 )  {
        this.FormSetDisabled( true );
      }
      else {
        this.FormSetDisabled( false );
      }

      // show modified conflict and do not change status
      if( isModifyConflict ) {
        this.ToastShowModifyConflict( "#ID_TOAST" + this.dynid );
      }
      // after creation there are some field that need to be filled
      else if( isIncomplete )  {

        // reset modified & error counter for all controls
        this.$container.broadcast( "option", "status", null );
        this.Option( "modified", 0 );

        // show toast and go to idle
        this.ToastShowIncomplete( "#ID_TOAST" + this.dynid );
        this.StatusIdle();

      }
      else {
        // scroll to top & show toast (if desired)
        this.Option( "scroll", 0 );
        if( isOutsideReloading ) {
          this.ToastShowReloaded( "#ID_TOAST" + this.dynid );
        }

        // reset modified & error counter for all controls
        this.$container.broadcast( "option", "status", null );
        this.Option( "modified", 0 );

        // fill data into controls and do 1st validation
        if( this.Validate() == 0 ) {
          this.StatusIdle();
        }
        else {
          this.StatusError();
        }
      }

      // finally perform permission check
      this.PermissionCheck( this.entinfo );
      
    }, 10 );

  }

  //-----------------------------------------------------------------------------
  //  entered from StatusLoadFinished and OnEvent modified == 0
  //
  StatusIdle()
  {
    this.status = "statusIdle";
    //console.log( this.status );
    
    if( this.$toolbar ) {
      this.FormToolbarButtonStatus( "ID_SAVE",          "disabled" );
      this.FormToolbarButtonStatus( "ID_UNDO",          "disabled" );
      this.FormToolbarButtonStatus( "ID_PERMISSIONS",   null );
      this.FormToolbarButtonStatus( "ID_LISTS",         null );
      this.FormToolbarButtonStatus( "ID_DEVICERESTART", this.isDeviceRestartNeeded ? "yellow" : null );
      this.FormToolbarButtonStatus( "ID_DELETE",        null );
    }

    // status idle = no validation error, so turn validation off
    this.ValidateOff();

    // manage disabled status
    if( this.isFormDisabled )  {
      this.FormSetDisabled( this.isFormDisabled );
    }
  }

  //-----------------------------------------------------------------------------
  //  entered from OnEvent modified > 0
  //
  StatusModified()
  {
    this.status = "statusModified";
    //console.log( this.status );

    if( this.$toolbar ) {
      this.FormToolbarButtonStatus( "ID_SAVE", null );
      this.FormToolbarButtonStatus( "ID_UNDO", null );
      this.FormToolbarButtonStatus( "ID_PERMISSIONS", "disabled" );
      this.FormToolbarButtonStatus( "ID_LISTS", "disabled" );
      this.FormToolbarButtonStatus( "ID_DEVICERESTART", "disabled" );
      this.FormToolbarButtonStatus( "ID_DELETE", null );
    }
  }

  //-----------------------------------------------------------------------------
  //  entered from OnEvent error > 0
  //
  StatusError()
  {
    this.status = "statusError";
    //console.log( this.status );
    let isModified = ( this.Option( "modified" ) > 0 );

    if( this.$toolbar ) {
      this.FormToolbarButtonStatus( "ID_SAVE",        "disabled" );
      this.FormToolbarButtonStatus( "ID_UNDO",        isModified ? "normal" : "disabled" );
      this.FormToolbarButtonStatus( "ID_PERMISSIONS", isModified ? "disabled" : "normal" );
      this.FormToolbarButtonStatus( "ID_LISTS",       isModified ? "disabled" : "normal" );
      this.FormToolbarButtonStatus( "ID_DEVICERESTART", isModified ? "disabled" : this.isDeviceRestartNeeded ? "yellow" : null );
      this.FormToolbarButtonStatus( "ID_DELETE",      null );

    }
  }

  //-----------------------------------------------------------------------------
  //  entered from CommandSave
  //
  StatusSaveRequest()
  {
    this.status = "statusSaveRequest";
    //console.log( this.status );

    // reset controls and modified counter
    this.$container.broadcast( "option", "status", null );
    this.Option( "modified", 0 );

    if( this.$toolbar ) {
      this.FormToolbarButtonStatus( "ID_SAVE", "disabled" );
      this.FormToolbarButtonStatus( "ID_UNDO", "disabled" );
      this.FormToolbarButtonStatus( "ID_PERMISSIONS", "disabled" );
      this.FormToolbarButtonStatus( "ID_LISTS", "disabled" );
      this.FormToolbarButtonStatus( "ID_DEVICERESTART", "disabled" );
      this.FormToolbarButtonStatus( "ID_DELETE", "disabled" );
    }
  }

  //-----------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    throw new Error( "DataRequest must be overridden!" );
  }

  //-----------------------------------------------------------------------------
  //  DataReloadAll: reload complete form after save
  //
  DataReloadAll()
  {
    // server may need a while to save the entity
    setTimeout( () => {
      this.status = "statusInitialized";
      this.DataRequest();
    }, 100 );
  }

  //-----------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity()
  {
    throw new Error( "DataResponseEntity must be overridden!" );
  }

  //-----------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    throw new Error( "DataSaveRequest must be overridden!" );
  }

  //---------------------------------------------------------------------------
  //  a reference has been changed, so revalidate the form
  //  
  DataRevalidate()
  {
    if( this.Option( "modified" ) <= 0 ) {
      setTimeout( () => {
        if( this.Validate() > 0 )  {
          PageMain.DocumentError( this.entinfo['entid'], true );
        }
      }, 250 );
    }
  }  
    
  //-----------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataPrepareEntityAfterLoad( entityInput )
  {
    let entity = $.clone( entityInput );

    if( 'deviceConfig' in entity )  {
      for( let deviceRecord of entity['deviceConfig'] )  {
        if( 'label' in deviceRecord ) {
          deviceRecord['label'] = deviceRecord['label'].replace( entity['name'], "{{name}}" );
        }
      }
    }

    if( 'sp' in entity )  {
      for( let key in entity['sp'] ) {
        let statepoint = entity['sp'][ key ];
        if( 'label' in statepoint )  {
          statepoint['label'] = statepoint['label'].replace( entity['name'], "{{name}}" );
        }
      }
    }

    if( 'rules' in entity )  {
      if( Array.isArray( entity['rules'] ) ) {
        delete entity['rules'];
      }
    }

    return entity;
  }

  //-----------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataPrepareEntityForSave( entityInput )
  {
    let entity = $.clone( entityInput );

    // fullfill name macro
    if( 'deviceConfig' in entity ) {
      for( let deviceRecord of entity['deviceConfig'] ) {
        if( 'label' in deviceRecord ) {
          deviceRecord['label'] = deviceRecord['label'].replace( "{{name}}", entity['name']  );
        }
      }
    }

    if( 'sp' in entity ) {
      for( let key in entity['sp'] ) {
        let statepoint = entity['sp'][key];
        if( 'label' in statepoint ) {
          statepoint['label'] = statepoint['label'].replace( "{{name}}", entity['name'] );
        }
      }
    }

    // references are recreated
    entity['references'] = [];

    // remove parts of entity that should not be saved
    if( 'userparam' in entity && entity['userparam']['reset'] === true )  {
      let resetParam = entity['userparam'];
      entity['userparam'] = [];
      Data.Instance().ResetDeviceParameters( entity['entid'], resetParam['unpairing'], resetParam['calldetails'], resetParam['grabpic'] );
    }
    else {
      delete entity['userparam'];
    }

    // remove first time incomplete flag
    if( ( 'adminUi' in entity ) 
     && ( 'incomplete' in entity['adminUi'] ) 
      ) {
      delete entity['adminUi']['incomplete'];
    }

    // remove empty rules
    if( 'rules' in entity ) {
      if( Array.isArray( entity['rules'] ) || Object.keys( entity['rules'] ).length == 0 ) {
        entity['rules']['size'] = 0;
      }
    }    

    return entity;
  }

  //-----------------------------------------------------------------------------
  //  add references to 
  //
  DataAddReferences( entity, referenceToAdd )
  {
    if( Array.isArray( referenceToAdd ) ) {
      $.array.Join( entity['references'], referenceToAdd );
    }
  }

  //-----------------------------------------------------------------------------
  //  remove all local references, just keep external
  //
  DataFilterReferencesForSave( entity )
  {
    let regex = new RegExp( "\\b" + entity['entid'] + "\\b", "i" );
    let references = entity['references'].filter( ( rec ) => {
      if( typeof rec != "string" )  {
        console.warn( "Widget error: A reference must be a string!", rec );
        return false;
      }
      return ! rec.match( regex );
    } );

    entity['references'] = references;
  }

  //---------------------------------------------------------------------------
  //  status icon update
  //
  DataPresenceUpdate( mailboxMessage  )
  {
    // set icon for presence value
    let icon = 'mdi mdi-circle grey-text';
    if( mailboxMessage['presenceValue'] in CONST.presence ) {
      icon = CONST.presence[ mailboxMessage['presenceValue'] ];
    }
    else {
      icon = 'mdi mdi-circle grey-text';
    }

    // restart icon need to modify?
    if( mailboxMessage['presenceValue'] == "DEVREQUEST" 
     || mailboxMessage['presenceValue'] == "DEVREBOOT" 
     || mailboxMessage['presenceValue'] == "DEVSTART" 
     || mailboxMessage['presenceValue'] == "DEVREADY"
      ) {
      this.FormUpdateRestart( true );
      this.isRestarting = true;
    }
    else {
      this.FormUpdateRestart( false );
      this.isRestarting = false;
    }

    //console.log( mailboxMessage['presenceValue'] );
    this.$toolbar.toolbar( "option", "iconStatus", icon );
  }

  //=============================================================================
  //  CONTROLS READ/WRITE
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  update restart button
  //
  FormUpdateRestart( isRestarting )
  {
    // no restart command in toolbar
    if( !( 'ID_DEVICERESTART' in this._formbaseToolbarStatus ) 
     || ( this._formbaseToolbarStatus['ID_DEVICERESTART'] == "hidden" )
      ) {
      return;
    }

    let icon = "";
    if( isRestarting ) {
      icon = "mdi mdi-spin mdi-rotate-right";
      this.$toolbar.toolbar( "item", "ID_DEVICERESTART", "status", "disabled" );
    }
    else {
      if( this.Option( "modified") > 0 )  {
        this.FormToolbarButtonStatus( "ID_DEVICERESTART", "disabled" );
      }
      else {
        this.FormToolbarButtonStatus( "ID_DEVICERESTART", null );
      }
      icon = "mdi mdi-restart";
    }
    this.$toolbar.toolbar( "item", "ID_DEVICERESTART", "icon", icon );
  
  }

  //-----------------------------------------------------------------------------
  //  load toolbar
  //
  FormLoadToolbar( selector, options )
  {
    // remember the "normal" status 
    if( options['data'] ) {
      this._formbaseToolbarStatus = { 'ID_DEVICERESTART' : "hidden" };
      for( let record of options['data'] )  {
        this._formbaseToolbarStatus[ record['id'] ] = record['status'] || "normal";
        if( record['id'] == "ID_SAVE" )  {
          this._formbaseIsSaveable = true;
        }
      }
    }

    // status icon
    if( this.entinfo && Data.Instance().PresenceData().Has( this.entinfo['entid'] ) )  {
      let presence = Data.Instance().PresenceData().Get( this.entinfo['entid'] );

      // Translate device status -> LED color.
      if( presence['presenceValue'] in CONST.presence ) {
        options['iconStatus'] = CONST.presence[ presence['presenceValue'] ];
      }
      else {
        options['iconStatus'] = 'mdi mdi-circle grey-text';
      }

      // restart icon need to modify?
      if( presence['presenceValue'] == "DEVREQUEST"
       || presence['presenceValue'] == "DEVREBOOT"
       || presence['presenceValue'] == "DEVSTART"
       || presence['presenceValue'] == "DEVREADY"
        ) {
        this.isRestarting = true;
        let restartRecord = options['data'].find( ( rec ) => rec['id'] == "ID_DEVICERESTART" );
        if( restartRecord ) {
          restartRecord['status'] = "disabled";
        }
      }

      Data.Instance().On( "PRESENCE/UPDATE/" + this.entinfo['entid'], this.etag, this.DataPresenceUpdate.bind( this ) );

    }

    this.$toolbar = $( selector ).toolbar( options );

    // 
    $( selector ).contextmenu( ( ev ) => {
      if( ev.ctrlKey == true && ev.altKey == true )  {
        alert( this.entinfo['entid'] );
      }
    } );
  }

  //-----------------------------------------------------------------------------
  //  change status of toolbar button
  //
  FormToolbarButtonStatus( id, status, isForced = false )
  {
    if( !( id in this._formbaseToolbarStatus ) 
     || ( this._formbaseToolbarStatus[id] == "hidden" && ! isForced )
      ) {
      return;
    }

    if( status == "hidden" )  {
      this._formbaseToolbarStatus[ id ] = "hidden";
    }

    let resultStatus = status;
    if( status == null )  {
      resultStatus = this._formbaseToolbarStatus[ id ];
    }
    this.$toolbar.toolbar( "item", id, "status", resultStatus );
  }

  //-----------------------------------------------------------------------------
  //  change title
  //
  FormTitle( text )
  {
    this.$toolbar.toolbar( "option", "text", text );
    PageMain.DocumentTitle( this.createOptions['entid'], text );
  }
 
  //-----------------------------------------------------------------------------
  //  overridables
  //
  FormSetDisabled( isDisabled )
  {
    this.isFormDisabled = isDisabled;
    if( this.isFormDisabled )  {
      this.$container.broadcast( "option", "status", "disabled" );
      this.ToastShowUnkonfigured( "#ID_TOAST" + this.dynid );
    }
    else {
      this.$container.broadcast( "option", "status", null );
      this.ToastDestroy();
    }
  }
  

  //=============================================================================
  //  HELPERS
  //=============================================================================

  ToastDestroy()
  {
    if( this.$toast ) {
      this.$toast.toast( "destroy" );
    }
  }

  ToastShowUnkonfigured( selector )
  {
    this.$toast = $( selector ).toast( {
      text: "{$formbase_unconfdev_notice}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h",
      click: ( id ) => {
        this.CommandUndo();
      },
    } );
  }

  ToastShowModifyConflict( selector )
  {
    this.$toast = $( selector ).toast( {
      text: "{$formbase_modify_conflict_notice}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h",
      click: ( id ) => {
        this.CommandUndo();
      },
    } );
  }

  ToastShowReloaded( selector )
  {
    this.$toast = $( selector ).toast( {
      text: "{$formbase_reloaded_notice}",
      icon: "mdi mdi-information",
      status: "yellow",
      height: 3,
      size: -1,
      time: 5000,
      transition: "h"
    } );
  }

  ToastShowSaveFirst( selector )
  {
    this.$toast = $( selector ).toast( {
      text: "{$formbase_savefirst_notice}",
      icon: "mdi mdi-information",
      status: "yellow",
      height: 3,
      size: -1,
      time: 5000,
      transition: "h"
    } );
  }  

  ToastShowIncomplete( selector )
  {
    this.$toast = $( selector ).toast( {
      text: "{$formbase_incomplete_notice}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h",
      click: ( id ) => {
        setTimeout( ( ) => { 
          this.CommandIncompleteValidate(); 
        }, 0 );
      },
    } );
  }
  
}﻿//=============================================================================
//
//  Form Factory
//  (c) 2021 CYTEL Technology AG
//
//  Editor: frank Jaekel
//          23.07.2021
//
//  Description:  Dispatcher for all available and
//                creatable forms
//
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class FormFactory
{
  //=================================================================
  //  STATIC VARIABLES / REGISTER
  //=================================================================

  static registeredForms = {};
  static menuCreate = [];
  static lockedEntityId = [
    "admin.1",
    "folder.confdev",
    "folder.unconfdev"
  ];

  static noPermissionEntityClasses = [
    "tkgw",
    "trunk",
    "admin",
    "agw"
  ];


  //=================================================================
  //  METHODS
  //=================================================================

  //-----------------------------------------------------------------
  //  register form in form factory
  //  
  static Register( classPointer )
  {
    // prequisites
    let className = classPointer.toString().split( '(' || /s+/ )[ 0 ].split( ' ' || /s+/ )[ 1 ];
    if( !( 'template' in classPointer ) ) {
      console.warn( `${ className }: missing static variable "template"` );
      return;
    }
    if( !( 'formId' in classPointer ) ) {
      console.warn( `${ className }: missing static variable "formId"` );
      return;
    }
    if( !( 'icon' in classPointer ) ) {
      console.warn( `${ className }: missing static variable "icon"` );
      return;
    }
    if( !( 'title' in classPointer ) ) {
      console.warn( `${ className }: missing static variable "title"` );
      return;
    }
    if( !( 'creatable' in classPointer ) ) {
      console.warn( `${ className }: missing static variable "creatable"` );
      return;
    }
    if( !( 'container' in classPointer ) ) {
      console.warn( `${ className }: missing static variable "container"` );
      return;
    }


    // get data
    let formId = classPointer['formId'];

    // register
    FormFactory.registeredForms[ formId ] = classPointer;

    // register on tabs interface
    let formInfo = FormFactory.FormInfo( formId );
    PageMain.RegisterForm( formId, formInfo['template'], formInfo['title'] );

    // add this into menu "create"
    if( 'createTemplate' in classPointer )  {
      FormFactory.UpdateMenuCreate( classPointer, formId );
    }
  }

  //-----------------------------------------------------------------
  //  get the viewable template for a form
  //  
  static UpdateMenuCreate( classPointer, formId )
  {
    for( let createRecord of classPointer['createTemplate'] ) {

      // flat menu
      let id = "/" + classPointer['formId'];
      if( 'deviceTemplate' in createRecord && createRecord['deviceTemplate'] )  {
        id += ( "/" + createRecord['deviceTemplate'] );
      }

      // get needed licenses
      var licenses = classPointer['MinimumNeededLicenses']();

      FormFactory.menuCreate.push( {
        id: id,
        text: createRecord['text'],
        formId: classPointer['formId'],
        icon: classPointer['icon'],
        deviceTemplate: createRecord['deviceTemplate'] || null,
        licenses: licenses
      } );

    }//for

    //console.log( FormFactory.menuCreate );

  }

  //-----------------------------------------------------------------
  //  get form informations for different purposes
  //  
  static FormInfo( formId )
  {
    if( formId in FormFactory.registeredForms ) {
      let info = {};
      info['template']  = this.registeredForms[ formId ]['template'];
      info['icon']      = this.registeredForms[ formId ]['icon'];
      info['title']     = this.registeredForms[ formId ]['title'];
      info['creatable'] = this.registeredForms[ formId ]['creatable'];
      info['container'] = this.registeredForms[ formId ]['container'];
      return info;
    }

    return null;
  }

  //-----------------------------------------------------------------
  //  get form informations for different purposes
  //  
  static IsLockedForm( entityId )
  {
    return ( FormFactory.lockedEntityId.indexOf( entityId ) >= 0 );
  }

  static IsPermission( entityClass )
  {
    return ( FormFactory.noPermissionEntityClasses.indexOf( entityClass ) < 0 );    
  }

  //-----------------------------------------------------------------
  //  get form informations for different purposes
  //  
  static IsContainer( formId )
  {
    if( formId in FormFactory.registeredForms ) {
      return this.registeredForms[ formId ]['container'];
    }
    return false;
  }

  //-----------------------------------------------------------------
  //  Called when a tree-item is dropped on another.
  //  
  static MoveEntity( formId, entityId, newParent )
  {
    // Call given form's MoveEntity static function - if it exists.
    if( formId in FormFactory.registeredForms && ( 'MoveEntity' in this.registeredForms[ formId ] ) ) {
      this.registeredForms[ formId ]['MoveEntity']( entityId, newParent );
      return true;
    }

    // Given form doesn't have MoveEntity function.
    return false;
  }

  //=================================================================
  //  create / delete
  //=================================================================

  static MenuCreate()
  {
    return $.clone( FormFactory.menuCreate );
  }


}
﻿//=============================================================================
//
//  Form attach helper
//  (c) 2022 CYTEL Technology AG
//
//  Editor: frank Jaekel
//          12.05.2022
//
//  Description:  Helper for attach video cam to an entity
//
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class FormAttachHelper
{
  static realDoorEntityClasses = [ "door" ];

  //---------------------------------------------------------------------------
  //  while attach an IP cam to a phone the phone becomes an virtual door. 
  //
  static PrepareVirtualEntrance( entity )
  {
    var isChanged = false;

    // quite simple: virtual door means NOT a real door. 
    if( ! ( FormAttachHelper.realDoorEntityClasses.includes( entity['class'] ) ) ) {
      isChanged = true;

      // add entrance property set
      entity['entrance'] = {
        "enabled": true,
        "door": null,
        "door2": null,
        "light": null
      };

      // remove videourl, must no be used on entrance
      WidgetDeviceConfigFactory.RemoveFunctionFromEntity( "videourl", entity );

    }

    return isChanged;
  }


  //---------------------------------------------------------------------------
  //  after detaching an IP cam the entity is not a virtual door. 
  //  Check if the entity is a real door and skip if so. 
  //
  static RemoveVirtualEntrance( entity )
  {
    var isChanged = false;

    // quite simple: virtual door means NOT a real door. 
    if( ! ( FormAttachHelper.realDoorEntityClasses.includes( entity['class'] ) )
     && ( 'entrance' in entity ) 
      ) {
      isChanged = true;
      delete entity['entrance'];
      entity['video']['camera'] = null;
    }

    return isChanged;
  }


  //---------------------------------------------------------------------------
  //  prepare a camera as attached device (exclusive for an parent)
  //
  static PrepareAttachedCamera( cameraEntity )
  {
    var isChanged = false;

    if( cameraEntity['class'] == "ipcam" && !( 'backup' in entid ) ) {

      cameraEntity['backup'] = {};
      cameraEntity['backup']['sp'] = cameraEntity['sp'];
      cameraEntity['backup']['deviceConfig'] = cameraEntity['deviceConfig'];
      cameraEntity['backup']['permission'] = cameraEntity['permission'];

      delete cameraEntity['sp'];
      delete cameraEntity['deviceConfig'];
      delete cameraEntity['permission'];    

      isChanged = true;
    }

    return isChanged;
  }

  //---------------------------------------------------------------------------
  //  prepare a camera as attached device (exclusive for an parent)
  //
  static RestoreAttachedCamera( cameraEntity )
  {
    var isChanged = false;

    if( cameraEntity['class'] == "ipcam" && 'backup' in cameraEntity ) {

      cameraEntity['sp'] = cameraEntity['backup']['sp'] || {};
      cameraEntity['deviceConfig'] = cameraEntity['backup']['deviceConfig'] || [];
      cameraEntity['permission'] = cameraEntity['backup']['permission'] || [];
  
      if( !( 'camera' in cameraEntity['sp'] ) ) {
        cameraEntity['sp']['camera'] = {
          label: cameraEntity['name'],
          address: Data.Instance().RootEntid() + "/" + cameraEntity['entid'] + "/camera",
          type: "CAMERA",
          name: "camera",
          template: "camera"
        };
      }
  
      if( cameraEntity['deviceConfig'].findIndex( ( rec ) => rec['template'] == "camera" ) < 0 ) {
        cameraEntity['deviceConfig'].push( { 
          module: "function",
          template: "camera",
          sp: "camera"
        } );
      }
      
      if( cameraEntity['permission'].findIndex( ( rec ) => rec['f'] == "camera" ) < 0 ) {
        cameraEntity['permission'].push( {
          p: "rw",
          w: "*",
          f: "camera"
        } );
      }
  
      delete cameraEntity['backup'];
      isChanged = true;
    }

    return isChanged;
  }

  //---------------------------------------------------------------------------
  //  attach camera to an entity
  //  
  static AttachCameraToEntity( entity, cameraEntityId )
  {
    var isChanged = false;

    if( cameraEntityId != null )  {
      if( !( 'video' in entity ) ) {
        entity['video'] = {
          receive: false,
          send: false,
          camera: null
        };
      }
      entity['video']['camera'] = cameraEntityId;
      isChanged = true;
    }

    return isChanged;
  }

  //---------------------------------------------------------------------------
  //  attach camera to an entity
  //  
  static DetachCameraFromEntity( entity, cameraEntityId = null )
  {
    var isChanged = false;

    if( 'video' in entity ) {
      entity['video']['camera'] = null;
      isChanged = true;
    }

    return isChanged;
  }

}
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          28.05.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormDashboard extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static viewableTemplate = "/forms/system/dashboard";  // vw template
  static documentIcon = "mdi mdi-gauge";                // icon for toolbar
  static documentTitle = "{$const_form_dashboard}";     // title that is shown tab

  //-----------------------------------------------------------------
  //  LOAD
  //
  Load( data )
  {
    $( '.groupbox' ).groupbox();

    //-----------------------------------------------------
    //  commands 
    //-----------------------------------------------------

    $( '#ID_TOOLBAR' + this.dynid ).toolbar( {
      size: -1,
      icon: FormDashboard.documentIcon,
      text: FormDashboard.documentTitle,
      closeable: false,
      data: [
        { id: 'ID_EXIT', icon: 'mdi mdi-exit-to-app', text: '{$const_menu_exit}', status: 'normal' },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_menu_help}', status: 'hilight' }
      ],
      click: ( id ) => {
        if( id == 'ID_HELP' )  {
          AccessApplication.Instance().Help( 'dashboard' );
        }
        else if( id == 'ID_EXIT' ) {
          AccessApplication.Instance().Command( 'exit' );
        }
        else  {
          AccessApplication.Instance().Command( id );
        }
      }
    } );

    $( "#ID_OPTIONAL" + this.dynid ).optional( {} );

    //-----------------------------------------------------
    //  controls
    //-----------------------------------------------------

    $( '#ID_SUPPORTINFO' + this.dynid ).click( () => {
      AccessApplication.Instance().Command( 'support-info' );
    } );
    $( '#ID_EXIT' + this.dynid ).click( () => {
      AccessApplication.Instance().Command( 'exit' );
    } );
    $( '#ID_DEVICE_RESTART_ALL' + this.dynid ).click( () => {
      Data.Instance().Reboot().RebootDeviceAll();
    } );
    $( '#ID_DEVICE_RESTART_NEEDED' + this.dynid ).click( () => {
      // Current method of disabling button only showns it as disabled, but
      // it can be clicked anyway. So check back if there is sth to be restarted.
      if( Verify.Instance().IsAnyDeviceRestartNeeded( ) )  {
        Data.Instance().Reboot().RebootDeviceNeeded();
      }
    } );
    $( '#ID_RESTART_SERVICE' + this.dynid ).click( () => {
      AccessApplication.Instance().Command( 'restart-service' );
    } );
    $( '#ID_REBOOT' + this.dynid ).click( () => {
      AccessApplication.Instance().Command( 'reboot' );
    } );


    //-----------------------------------------------------
    //  box 'messages'
    //-----------------------------------------------------

    $( "#ID_MESSAGE_GRID" + this.dynid ).grid( {
      size: -1,
      height: 13,
      selection: 'click',
      data: Verify.Instance().ErrorDataset(),
      placeholder: "{$dashboard_grid_placeholder}",
      focus: false,
      header: {
        'type': { text: "", size: 4, align: "center", resizeable: false, sortable: true, icon: "mdi mdi-checkbox-multiple-blank-circle-outline" },
        'entid': { text: "{$dashboard_grid_column_entid}", size: 15, align: "left", resizeable: true, sortable: true, inplace: "link" },
        'message': { text: "{$dashboard_grid_column_msgid}", size: 60, align: "left", resizeable: true, sortable: true },
      },
      renderCell: this.ControlMessagegridRenderCell.bind( this ),
      inplaceLink: ( columnId, record ) => {
        PageMain.ShowEntity( record['entid'] );
      }
    } );

    //-----------------------------------------------------
    //  box 'info'
    //-----------------------------------------------------

    const infoBoxWidth = 38;
    const downloadingText = "..";

    $( "#ID_HARDWAREID" + this.dynid ).edit( {
      size: infoBoxWidth,
      status: "readonly",
      border: false,
      focus: false,
      value: downloadingText
    });

    $( "#ID_VERSION" + this.dynid ).edit( {
      size: infoBoxWidth,
      status: "readonly",
      border: false,
      focus: false,
      value: downloadingText
    } );

    $( "#ID_NUMBERDEVICES" + this.dynid ).edit( {
      size: infoBoxWidth,
      status: "readonly",
      border: false,
      focus: false,
      value: downloadingText
    } );

    $( "#ID_DEMOTIME" + this.dynid ).edit( {
      size: infoBoxWidth,
      status: "readonly",
      border: false,
      focus: false,
      value: downloadingText
    } );

    $( "#ID_SUPPORTCONTRACT" + this.dynid ).edit( {
      size: infoBoxWidth,
      status: "readonly",
      border: false,
      focus: false,
      value: downloadingText
    } );
        

    $( "#ID_SUSSTATUS" + this.dynid ).edit( {
      size: infoBoxWidth,
      status: "readonly",
      border: false,
      focus: false,
      value: downloadingText
    } );    

    //-----------------------------------------------------
    //  data 
    //-----------------------------------------------------

    Data.Instance().On( "DATA/STARTUP_FINISHED/", this.etag, this.DataStartupFinished.bind( this ) );

    // Subscribe "device restart" (reboot) mailbox events.
    this.deviceRestartEventId = Mailbox.Instance().On( "DEVICE_RESTART", this.MailboxEventDeviceReboot.bind( this ) );

    // Subscribe event for admin message DEVRESTART_NEEDED exist/non-exist.
    Verify.Instance().On( "VERIFY/DEVRESTART_NEEDED", this.etag, this.VerifyEventDeviceRestartNeeded.bind( this ) );    

    // Subscribe event for response on reboot-devices request.
    Data.Instance().On( "REBOOT_DEVICES/STARTED/", this.etag, this.RebootDeviceResponseEvent.bind( this ) );

    // subscript the license
    //Data.Instance().On( "ENTINFO/UPDATE/menu.licence", this.etag, this.DataEntinfoUpdateLicense.bind( this ) );

    Data.Instance().On( "SUS/STATUS", this.etag, this.DataSusStatusEvent.bind( this ) );

    Data.Instance().On( "DATA/UPDATE/menu.licence", this.etag, this.DataResponseEntityLicense.bind( this ) );

  }//load


  //-----------------------------------------------------------------
  //  DATA
  //-----------------------------------------------------------------
  
  //-----------------------------------------------------------------
  //  After startup is finished
  //  - do first verify
  //  - send request for HW-ID, licenses etc.
  //
  DataStartupFinished()
  {
    Verify.Instance().Start();
    Verify.Instance().All();

    //  HW-ID, licenses etc.
    this.XhrRequestStartupInfo();
  }

  //-----------------------------------------------------------------
  //  menu.license has been changed, reload
  //
  DataEntinfoUpdateLicense( entinfo )  
  {
    Data.Instance().RequestEntity( "menu.licence" );
  }

  //-----------------------------------------------------------------
  //  the device (= APT license) cout has been changed
  //
  DataResponseEntityLicense( param )  
  {
    this.FormFillNumberDevices( param['licence']['APR']['engagedLicenses'] );

    // support contract
    var timeStr = Time.FormatFromEndOfSupport( param['licence']['endofsupport'] );
    $( '#ID_SUPPORTCONTRACT' + this.dynid ).edit( "option", "value", timeStr );

    if( !param['licence']['isDemo']  &&  !param['licence']['isVersionValid'] )  {
      this.ToastShowInvalidVersion( "#ID_TOAST" + this.dynid );
    }    
    else if( ! param['licence']['isFinallyOk'] ) {
      this.ToastShowInvalid( "#ID_TOAST" + this.dynid );
    }
    else {
      this.ToastHide();
    }

  }

  //-----------------------------------------------------------------
  //  sus status event received by mailbox
  //
  DataSusStatusEvent( status )  
  {
    this.FormFillSusStatus( status );
    if( status == "ERROR")  {
      this.ToastShowSusError( "#ID_TOAST" + this.dynid );
    }
    else {
      this.ToastHide();
    }
  }

  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------  

  FormFillNumberDevices( numDevices )
  {
    $( "#ID_NUMBERDEVICES" + this.dynid ).edit( "option", "value", numDevices );
  }

  FormFillSusStatus( status )
  {
    // sus status
    var stat = status || "NOTUSED";
    var susStatus = "";
    switch( stat ) {
      case 'NOTUSED': susStatus = "{$dashboard_susinfo_notused}"; break;
      case 'CONNECTING': susStatus = "{$dashboard_susinfo_connecting}"; break;
      case 'OK': susStatus = "{$dashboard_susinfo_online}"; break;
      case 'ERROR': susStatus = "{$dashboard_susinfo_error}"; break;
    }
    $( "#ID_SUSSTATUS" + this.dynid ).edit( "option", "value", susStatus );
  }

  //-----------------------------------------------------------------
  //  MAILBOX
  //-----------------------------------------------------------------  

  //-----------------------------------------------------------------
  //  Reboot of devices (all devices or needed devices) has
  //  "started" or "ended".
  //  
  MailboxEventDeviceReboot( messageElementList )
  {
    // Iterate thru mailbox element list.
    for( let id in messageElementList )  {
      let messageElement = messageElementList[ id ];

      if( messageElement["status"] == "started" )  {

        // Disable controls.
        $( "#ID_DEVICE_RESTART_ALL" + this.dynid ).removeClass( "normal" );
        $( "#ID_DEVICE_RESTART_ALL" + this.dynid ).removeClass( "hover" );
        $( "#ID_DEVICE_RESTART_ALL" + this.dynid ).addClass( "disabled" );
        $( "#ID_DEVICE_RESTART_NEEDED" + this.dynid ).removeClass( "normal" );
        $( "#ID_DEVICE_RESTART_NEEDED" + this.dynid ).removeClass( "hover" );
        $( "#ID_DEVICE_RESTART_NEEDED" + this.dynid ).addClass( "disabled" );

        break;
      }
      else if( messageElement["status"] == "ended" )  {

        // Enable controls.
        $( "#ID_DEVICE_RESTART_ALL" + this.dynid ).removeClass( "disabled" );
        $( "#ID_DEVICE_RESTART_ALL" + this.dynid ).addClass( "normal" );
        $( "#ID_DEVICE_RESTART_ALL" + this.dynid ).addClass( "hover" );

        // Enable "If Needed" only if there is sth needed.        
        this.HelperEnableRestartIfNeeded( Verify.Instance().IsAnyDeviceRestartNeeded( ) );

        break;
      }

    } // for
  }

  //-----------------------------------------------------------------
  // Event: At least one DEVRESTART_NEEDED record in 
  // admin messages - or not.
  //
  VerifyEventDeviceRestartNeeded( isDeviceRestartNeeded )
  {
    this.HelperEnableRestartIfNeeded( isDeviceRestartNeeded );
  }


  //-----------------------------------------------------------------
  //  REBOOT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  XHR response: Reboot of devices (all devices or needed devices) has
  //  started or failed. This event is published after response on 
  //  RebootDeviceNeeded or RebootDeviceAll. 
  //
  RebootDeviceResponseEvent( status )
  {
    // status == "started" is not used. Use Mailbox event DEVICE_RESTART instead.

    // Server didn't find any devices to boot. So no "ended" Mailbox event can be expected!
    if ( status == "nothing_to_do" )
    {
      // Show some visible reaction. Disable controls for some seconds and then enable again.

      // Disable controls.
      $( "#ID_DEVICE_RESTART_ALL" + this.dynid ).removeClass( "normal" );
      $( "#ID_DEVICE_RESTART_ALL" + this.dynid ).removeClass( "hover" );
      $( "#ID_DEVICE_RESTART_ALL" + this.dynid ).addClass( "disabled" );
      $( "#ID_DEVICE_RESTART_NEEDED" + this.dynid ).removeClass( "normal" );
      $( "#ID_DEVICE_RESTART_NEEDED" + this.dynid ).removeClass( "hover" );
      $( "#ID_DEVICE_RESTART_NEEDED" + this.dynid ).addClass( "disabled" );

      setTimeout( () => {
        // Enable controls.
        $( "#ID_DEVICE_RESTART_ALL" + this.dynid ).removeClass( "disabled" );
        $( "#ID_DEVICE_RESTART_ALL" + this.dynid ).addClass( "normal" );
        $( "#ID_DEVICE_RESTART_ALL" + this.dynid ).addClass( "hover" );

        // Enable "If Needed" only if there is sth needed.
        this.HelperEnableRestartIfNeeded( Verify.Instance().IsAnyDeviceRestartNeeded( ) );

      }, 5000 ); // 5 secs

    }
  }


 
  //-----------------------------------------------------------------
  //  CONTROL
  //-----------------------------------------------------------------

  ControlMessagegridRenderCell( columnId, record ) 
  {
    let icon = "";
    let text = "";
    if( columnId == "type" ) {
      if( record['msgid'] == "DEVRESTART_NEEDED" ) {
        icon = "mdi mdi-restart yellow-text";
      }
      else {
        icon = "mdi mdi-alert-outline red-text";
      }
    }
    else if( columnId == "entid" ) {
      let entinfo = Data.Instance().Entinfo( record['entid'] );
      if( !entinfo )  {
        console.warn( `Entinfo for ${ record.entid } not found!` );
      }
      else {
        text = entinfo['name'];
      }
    }
    else if( columnId == "message" ) {
      text = $.lang( record['message'] );
    }
    else {
      text = record[columnId];
    }
    return { icon: icon, text: text };

  }


  //-----------------------------------------------------------------
  //  XHR REQUEST/RESPONSE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // XHR request for HW-ID + version + license info.
  //
  XhrRequestStartupInfo( )
  {
    let xhrRequest = new XhrRequest( "ADMIN.STARTUP_INFO", null, ( response ) => {
      this.XhrResponse( response );
    } );
  }

  //-----------------------------------------------------------------
  // XHR response handling
  //
  XhrResponse( response )
  {
    let command      = response['meta']['command'];
    let responseCode = parseInt( response['meta']['status'] );

    // If error, we are done here.
    if( responseCode != 200 ) {
      return;
    }

    //-- ADMIN.STARTUP_INFO ---------------------------------
    if( command == "ADMIN.STARTUP_INFO" )  {

      // Show some info.
      $( "#ID_HARDWAREID"    + this.dynid ).edit( "option", "value", response['data']['numid'] );
      $( "#ID_VERSION"       + this.dynid ).edit( "option", "value", response['data']['version'] );

      this.FormFillNumberDevices( response['data']['numDevices'] );
      this.FormFillSusStatus( response['data']['susStatus'] );
      Data.Instance().SetSusStatus( response['data']['susStatus'] );
      Data.Instance().SetVersion( response['data']['version'] );

      // optionals
      if( response['data']['isDemo'] ) {
        $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_DEMOTIME" );
        $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_SUPPORTCONTRACT" );
      }
      else {
        $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_DEMOTIME" );
        $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_SUPPORTCONTRACT" );
      }

      // support contract
      var timeStr = Time.FormatFromEndOfSupport( response['data']['endofsupport'] );
      $( '#ID_SUPPORTCONTRACT' + this.dynid ).edit( "option", "value", timeStr );

      // demo period
      let demoExpiryTime = "-";
      if( response['data']['isDemo'] && typeof response['data']['demoExpiryTime'] == "number" && response['data']['demoExpiryTime'] > 0 )  {
        demoExpiryTime = Time.FormatFromUnix( response['data']['demoExpiryTime'] );
      }
        //
      if( response['data']['isNetworkConfigured'] === false )  {
        this.ToastShowBasicparamNoNetwork( "#ID_TOAST" + this.dynid );
      }
      else if( response['data']['isDemo'] && response['data']['isExpired'] )  {
        this.ToastShowExpired( "#ID_TOAST" + this.dynid );
        $( "#ID_DEMOTIME" + this.dynid ).edit( "option", "value", $.lang( "{$dashboard_info_warning_expired}" ) + " " + demoExpiryTime );
        $( "#ID_DEMOTIME" + this.dynid ).edit( "option", "status", "red" );
      }
      else {
        $( "#ID_DEMOTIME" + this.dynid ).edit( "option", "value", demoExpiryTime );
        $( "#ID_DEMOTIME" + this.dynid ).edit( "option", "status", "readonly" );  
      }
    }

  }


  //=============================================================================
  //  TOAST
  //=============================================================================

  ToastShowBasicparamNoNetwork( selector )
  {
    this.$toast = $( selector ).toast( {
      text: "{$dashboard_toast_nonetworkconfigured}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h",
      click: ( id ) => {
        PageMain.ShowEntity( "admui.basicparam" );
      }
    } );
  }

  ToastShowExpired( selector )
  {
    this.$toast = $( selector ).toast( {
      text: "{$license_toast_expired}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }

  ToastShowInvalid( selector )
  {
    this.$toast = $( selector ).toast( {
      text: "{$license_toast_invalid}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }

  ToastShowInvalidVersion( selector )
  {
    this.$toast = $( selector ).toast( {
      text: "{$license_toast_invalid_version}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }

  ToastShowSusError( selector )
  {
    this.$toast = $( selector ).toast( {
      text: "{$const_toast_suserror}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }

  ToastHide()
  {
    if( this.$toast )  {
      this.$toast.toast( "destroy" );
    }
  }

  //=============================================================================
  //  HELPER
  //=============================================================================

  //-----------------------------------------------------------------
  // ID_DEVICE_RESTART_NEEDED to be enabled or disabled?
  //
  HelperEnableRestartIfNeeded( isDeviceRestartNeeded )
  {
    if( isDeviceRestartNeeded )  {
      // This method of disabling button only showns it as disabled! So
      // when clicked, check back if there is sth to be restarted.
      $( "#ID_DEVICE_RESTART_NEEDED" + this.dynid ).removeClass( "disabled" );
      $( "#ID_DEVICE_RESTART_NEEDED" + this.dynid ).addClass( "normal" );
      $( "#ID_DEVICE_RESTART_NEEDED" + this.dynid ).addClass( "hover" );
    }
    else  {
      $( "#ID_DEVICE_RESTART_NEEDED" + this.dynid ).removeClass( "normal" );
      $( "#ID_DEVICE_RESTART_NEEDED" + this.dynid ).removeClass( "hover" );
      $( "#ID_DEVICE_RESTART_NEEDED" + this.dynid ).addClass( "disabled" );
    }
  }

}


//-----------------------------------------------------------------------------
//  BUILDER
//-----------------------------------------------------------------------------

FormDashboard.Register( FormDashboard.viewableTemplate );
PageMain.RegisterForm( "dashboard", FormDashboard.viewableTemplate, FormDashboard.documentTitle, "dashboard", true );
//=============================================================================
//
//  Presence Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.06.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormPresence extends FormBase
{
  //---------------------------------------------------------------------------
  //  STATIC
  //---------------------------------------------------------------------------

  static viewableTemplate = "/forms/system/presence";  // vw template
  static documentIcon = "mdi mdi-traffic-light";     // icon for toolbar & doc
  static documentTitle = "{$presence_document_title}";  // default title if no entinfo
  static helpFile = "presence";
  
  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createData )
  {
    //-----------------------------------------------------
    //  controls
    //-----------------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: FormPresence.documentIcon,
      text: FormPresence.documentTitle,
      data: [
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );

    // filter controls
    $( "#ID_FILTER_CHECK_ONLINE" + this.dynid ).choice( {
      size: CONST.size.M,
      type: "check",
      text: "{$presence_filter_check_online}",
      result: this.ControlFilterCheckOnline.bind( this )
    });

    $( "#ID_FILTER_CHECK_OFFLINE" + this.dynid ).choice( {
      size: CONST.size.M,
      type: "check",
      text: "{$presence_filter_check_offline}",
      result: this.ControlFilterCheckOffline.bind( this )
    } );

    // Default values for Online/Offline checkboxes. Do this before grid is loaded,
    // because the check values are used in grid's dataFilter.
    $( "#ID_FILTER_CHECK_ONLINE" + this.dynid ).choice( "option", "value", true );
    $( "#ID_FILTER_CHECK_OFFLINE" + this.dynid ).choice( "option", "value", true );     

    $( "#ID_FILTER_SEARCH" + this.dynid ).edit( {
      size: CONST.size.L,
      icon: 'mdi mdi-magnify',
      click: this.ControlSearchClick.bind( this ),
      result: this.ControlSearchResult.bind( this )
    } );

    // Custom dataview based on presenceData.
    let presenceDataset = Data.Instance().PresenceData();
    this.dataview = presenceDataset.BuildView( 'default', {
      search: ( id, record, options, self  ) => { 
        if( self.search == null ) {
          return true;
        }
        else {
          // Do the search only on the columns as specified here.  
          if( typeof( record['name'] ) == "string" && record['name'].search( self.searchRx ) >= 0 )  {
            return true;
          }
          if( typeof( record['callnumber'] ) == "string" && record['callnumber'].search( self.searchRx ) >= 0 )  {
            return true;
          }          
          if( typeof( record['ipAddr'] ) == "string" && record['ipAddr'].search( self.searchRx ) >= 0 )  {
            return true;
          }
          return false;
        }
      },
    } );

    // grid
    $( '#ID_PRESENCE_GRID' + this.dynid ).grid( {
      size: -1,   // same as "100%"
      height: 25,
      selection: 'click',
      data: this.dataview,
      focus: false,
      dataId: 'callnumber',
      scrollbar: 'static', // 'overlay', 'static'
      regkey: 'presence',
      sort: 'name',
      sortOrder: '+',
      header: {
        'status':         { text: "", size: 4,    align: "center",  resizeable: false,   sortable: false, icon: "mdi mdi-checkbox-multiple-blank-circle-outline" },
        'name':           { text: "{$presence_header_object_name}", size: -10,  align: "left",  resizeable: true, sortable: true, inplace: "link" },
        'callnumber':     { text: "{$presence_header_callnumber}",  size: -6,   align: "left",  resizeable: true, icon: "mdi mdi-account", sortable: true },
        'presenceValue':  { text: "{$presence_header_status}",      size: -6,   align: "left",  resizeable: true, sortable: true },        
        'ipAddr':         { text: "{$presence_header_ip}",          size: -6,   align: "left",  resizeable: true, sortable: true  },
        'lastAction':     { text: "{$presence_header_last_action}",   size: -6,  align: "left",  resizeable: true, sortable: true }, 
        'remotePhoneNo':  { text: "{$presence_header_remote_number}", size: -10, align: "left",  resizeable: true, sortable: true }
      },
      renderCell: ( columnId, record ) => {
        let icon = "";
        let text = "";
        if( columnId == 'status' )  {
          // Translate device status -> LED color.
          if( record['presenceValue'] in CONST.presence ) {
            icon = CONST.presence[ record['presenceValue'] ];
          }
          else {
            icon = 'mdi mdi-circle grey-text';
          }
        }
        else if( columnId == 'callnumber' ) {
          // Don't show icon if callnumber is empty.
          if( typeof( record[ columnId ] ) == "string" )  {          
            icon = "mdi mdi-account";
            text = record[ columnId ];
          }
        }
        else if( columnId == 'presenceValue' ) {
          // Translate device status -> status text.          
          switch( record['presenceValue'] )  {

            case  'OFFLINE':        text = "{$presence_status_offline}";   break;

            case  'DEVREQUEST':
            case  'DEVREBOOT':
            case  'DEVSTART':       text = "{$presence_status_devstart}";  break;
            case  'DEVREADY':       text = "{$presence_status_devready}";  break;

            case  'TRYING':         text = "{$presence_status_trying}";    break;  // susapp

            case  'IDLE':           text = "{$presence_status_idle}" ;     break;
            
            case  'ERROR':          text = "{$presence_status_error}" ;     break;

            case  'DND':            text = "{$presence_status_dnd}";        break;
            case  'CALL':           text = "{$presence_status_call}";       break;
            case  'RING':           text = "{$presence_status_ring}";       break;
            case  'DISCONNECTING':  text = "{$presence_status_disc}";       break;
            case  'BUSY':           text = "{$presence_status_busy}";       break;
            default:                text = "{$presence_status_offline}";    break;
          }
        }
        else if( columnId == 'lastAction' ) {
          // Format "last changed" time for display.
          if( record['lastAction'] > 0 )  {
            text = Time.FormatFromUnix( record['lastAction'] );
          }
          else  {
            text = "-";
          }
        }
        else if( columnId == 'ipAddr' )  {
          // Show "-" instead of "" on certain columns for a better reading of grid.
          if( typeof record['ipAddr'] != "string" || record['ipAddr'] == "" )  {
            text = "-";
          }
          else  {
            text = record[ columnId ];
          }
        }
        else if( columnId == 'remotePhoneNo' ) {

          // in case of error we show info in field remote phone nr
          if( record['presenceValue'] == "ERROR" )  {
            text = record['info'];
          }
          // Show "-" instead of "" on certain columns for a better reading of grid.          
          else if( typeof record['remotePhoneNo'] != "string" || record['remotePhoneNo'] == "" )  {
            text = "-";
          }
          else  {
            text = record[ columnId ];
          }
        }
        else  {
          text = record[ columnId ];
        }

        return { 
          'text': ( typeof text == "string" ) ? text : "",
          'icon': icon
        };
      },
      renderStatus: ( idRecord, record ) => {
        return "normal";
        // TODO: if( record['presenceValue'] == "OFFLINE" ) return "red" ?
      },
      drawItem: ( record, $container ) => {
        return self.CustomListDrawItem( record, $container );
      },
      dataFilter: ( id, record ) => { 

        // Prerequisites
        let isCheckedOnline = $( "#ID_FILTER_CHECK_ONLINE" + this.dynid ).choice( "option", "value" );
        let isCheckedOffline = $( "#ID_FILTER_CHECK_OFFLINE" + this.dynid ).choice( "option", "value" );
        let isDeviceOnline = true;

        // Is device online or offline by defintion?
        switch( record['presenceValue'] )  {
          case  'OFFLINE':
          case  '':              isDeviceOnline = false; break;          
          case  'DEVSTART':
          case  'DEVREADY':
          case  'TRYING':
          case  'DND':
          case  'IDLE':
          case  'RING':
          case  'DISCONNECTING':
          case  'BUSY':
          default:               isDeviceOnline = true; break;
        }

        // Now decide if record is to be shown or not.
        if( isCheckedOnline && isDeviceOnline )  {
          return true;
        }
        if( isCheckedOffline && !isDeviceOnline )  {
          return true;
        }        
        return false;
      },
      inplaceLink: ( columnId, record ) => {
        PageMain.ShowEntity( record['entid'] );
      }
    } );

  }//load



  //---------------------------------------------------------------------------
  //  CONTROL
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  close form (close button on toolbar)
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( "presence" );
  }

  //---------------------------------------------------------------------------
  //  toolbar button
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId ) {
    }
  }

  //---------------------------------------------------------------------------
  //  checkbox "filter online"
  //
  ControlFilterCheckOnline( value )
  {
    // Just trigger a grid refresh. Grid's dataFilter will apply the change.
    this.dataview.Update();
  }

  //---------------------------------------------------------------------------
  //  checkbox "filter offline"
  //
  ControlFilterCheckOffline( value )
  {
    // Just trigger a grid refresh. Grid's dataFilter will apply the change.
    this.dataview.Update();
  }

  //---------------------------------------------------------------------------
  //  edit "search": Icon clicked.
  //
  ControlSearchClick( value )
  {
    // If there is a string in the control, the 'X' icon must habe been clicked.
    // -> Delete the filter.
    if( value.length > 0 ) {
      $( '#ID_FILTER_SEARCH' + this.dynid ).edit( "option", "value", "" );
      $( '#ID_FILTER_SEARCH' + this.dynid ).edit( "option", "icon", "mdi mdi-magnify" );
      $( '#ID_PRESENCE_GRID' + this.dynid ).grid( "option", "search", "" );
    }    
  }  

  //---------------------------------------------------------------------------
  //  edit "search": Search string was entered.
  //
  ControlSearchResult( result )
  {
    // If there is a search string, show the 'X' icon.
    if( result.length > 0 )  {
      $( '#ID_FILTER_SEARCH' + this.dynid ).edit( "option", "icon", "mdi mdi-close" );
    }

    // If not, show the magnifying glass icon again.
    else  {
      $( '#ID_FILTER_SEARCH' + this.dynid ).edit( "option", "icon", "mdi mdi-magnify" );
    }

    // Do the search filter.
    $( '#ID_PRESENCE_GRID' + this.dynid ).grid( "option", "search", result );
  }


  //---------------------------------------------------------------------------
  //  CONTROL
  //---------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  StatusModified(): need to overwrite base class
  //
  StatusModified()
  {
  }

  
}


//=============================================================================
//  REGISTER
//=============================================================================

FormPresence.Register( FormPresence.viewableTemplate );
PageMain.RegisterForm( "presence", FormPresence.viewableTemplate, FormPresence.documentTitle, "presence" );
//=============================================================================
//
//  Basic Parameter Form
//  (c) 2021 CYTEL Technology AG
//
//  Form for the one-and-only entity "admui.basicparam".
//
//  Editor: Peter Aidam
//          13.09.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormBasicparam extends FormBase
{
  //---------------------------------------------------------------------------
  //  STATIC
  //---------------------------------------------------------------------------

  static viewableTemplate = "/forms/system/basicparam";   // vw template
  static documentIcon = "mdi mdi-cog";                    // icon for toolbar & doc
  static documentTitle = "{$basicparam_document_title}";  // title
  static helpFile = "basicparam";
  
  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createData )
  {

    //-------------------------------------------
    //  data
    //-------------------------------------------

    // Note: This form is not started from the device tree, so entinfo is not part of createData!
    this.entinfo = Data.Instance().Entinfo( "admui.basicparam" );
    this.entity = null;
    this.networkData = null;
    this.gridDataset = new Dataset( { id: "id" } );
    this.status = "statusInitialized";
    this.nextProfileId = 1;
    this.selectedProfile = null;

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();
    $( '#ID_OPTIONAL' + this.dynid ).optional( { } );

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: FormBasicparam.documentIcon,
      text: FormBasicparam.documentTitle,      
      data: [
        { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
        { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
        { id: 'ID_RESTART_SERVICES', icon: 'mdi mdi-power', text: '{$const_menu_restart_service_short}', status: 'red' },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );


    //-------------------------------------------
    //  Group "general"
    //-------------------------------------------

    // the netcards
    $( "#ID_SYSLANG" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "droplist",
      data: CONST.syslang,
      selected: [ "en" ]
    } );

    // the netcards
    $( "#ID_NETCARD_GRID" + this.dynid ).grid( {
      size: CONST.size.XL,
      height: 6,
      selection: 'none',
      data: this.gridDataset,
      dataId: 'id',
      sort: "text",
      scrollbar: 'overlay', // 'overlay', 'static'
      header: {
        'displayname': { text: "{$basicparam_netcardgrid_header_name}", size: 35 },
        'ipV4': { text: "{$basicparam_netcardgrid_header_ipv4}", size: 10, inplace: "check", align: "left" },
        //'ipV6': { text: "{$basicparam_netcardgrid_header_ipv6}", size: 10, inplace: "check", align: "left" },
        'client': { text: "{$basicparam_netcardgrid_header_client}", size: 10, inplace: "check", align: "left" },
        'telephone': { text: "{$basicparam_netcardgrid_header_phone}", size: 10, inplace: "check", align: "left" }
      },
      inplaceEdit: this.ControlGridInplace.bind( this ),
      renderCell: this.ControlGridRenderCell.bind( this )
    } );


    //-------------------------------------------
    //  Group "times (door)"
    //-------------------------------------------

    $( "#ID_DOOR_RING_TIMEOUT" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      filter: { allow: "0123456789" },
      validate: this.ValidateDoorRingTimeout.bind( this )
    } );

    $( "#ID_DOOR_CALL_TIMEOUT" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      filter: { allow: "0123456789" },
      validate: this.ValidateDoorcallTimeout.bind( this )
    } );

    $( "#ID_LATEVIDEO_TIMEOUT" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      filter: { allow: "0123456789" },
      validate: this.ValidateLatevideoTimeout.bind( this )
    } );


    //-------------------------------------------
    //  Group "times (general)"
    //-------------------------------------------

    $( "#ID_RING_TIMEOUT" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      filter: { allow: "0123456789" },
      validate: this.ValidateRingTimeout.bind( this )
    } );

    $( "#ID_CALL_TIMEOUT" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      filter: { allow: "0123456789" },
      validate: this.ValidateCallTimeout.bind( this )
    } );


    //-------------------------------------------
    //  Group "times (lists)"
    //-------------------------------------------

    $( "#ID_LISTS_TIME_GRABPIC" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      filter: { allow: "0123456789" },
      validate: this.ValidateGrabpicTime.bind( this )
    } );

    $( "#ID_LISTS_TIME_CDR" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      filter: { allow: "0123456789" },
      validate: this.ValidateCdrTime.bind( this )
    } );

    $( "#ID_LISTS_TIME_VOICEMAIL" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      filter: { allow: "0123456789" },
      validate: this.ValidateVoicemailTime.bind( this )
    } );

    //-------------------------------------------
    //  Device outage
    //-------------------------------------------

    if( Data.IsFeatureEnabled( "module.deviceoutage" ) )  {
      $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_DEVICEOUTAGE" );    
    }
    else {
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_DEVICEOUTAGE" );    
    }

    $( "#ID_DEVICEOUTAGE_TIME" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      filter: { allow: "0123456789" },
      validate: this.ValidateOutageTime.bind( this )
    } );    



    //-------------------------------------------
    //  Start 
    //-------------------------------------------

    Data.Instance().On( "COSCONF.READ", this.etag, this.DataResponseCosconfRead.bind( this ) );
    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Fill entity into controls
  //
  FormFillControls()
  {
    // Group "general".
    $( "#ID_SYSLANG" + this.dynid ).listbox( "option", "selected", this.entity['lang']['syslang'] );

    // Group "times (door)".
    $( "#ID_DOOR_RING_TIMEOUT" + this.dynid ).edit( "option", "value", this.entity['timeouts']['doorRingTimeout'].toString() );
    let doorCallTimeout = parseInt( this.entity['timeouts']['doorCallTimeout'] ) / 60;
    $( "#ID_DOOR_CALL_TIMEOUT" + this.dynid ).edit( "option", "value", doorCallTimeout );
    $( "#ID_LATEVIDEO_TIMEOUT" + this.dynid ).edit( "option", "value", this.entity['timeouts']['lateVideoTimeout'].toString() );

    // Group "times (general)".
    $( "#ID_RING_TIMEOUT" + this.dynid ).edit( "option", "value", this.entity['timeouts']['ringTimeout'].toString() );
    let callTimeout = parseInt( this.entity['timeouts']['callTimeout'] ) / 60;
    $( "#ID_CALL_TIMEOUT" + this.dynid ).edit( "option", "value", callTimeout );
    
    // Group "times (lists)".
    $( "#ID_LISTS_TIME_GRABPIC" + this.dynid ).edit( "option", "value", this.entity['listsTime']['grabpic'].toString() );
    $( "#ID_LISTS_TIME_CDR" + this.dynid ).edit( "option", "value", this.entity['listsTime']['cdr'].toString() );
    $( "#ID_LISTS_TIME_VOICEMAIL" + this.dynid ).edit( "option", "value", this.entity['listsTime']['voicemail'].toString() );        

    // Group "outage alarm"
    var outageTime = 60;
    var statepoint = null;
    if( ( 'features' in this.entity ) && ( 'deviceoutage' in this.entity['features'] ) ) {
      outageTime = parseInt( this.entity['features']['deviceoutage']['time'] );
      statepoint = this.entity['features']['deviceoutage']['statepoint'];
    }    
    outageTime = parseInt( outageTime / 60 );
    $( "#ID_DEVICEOUTAGE_TIME" + this.dynid ).edit( "option", "value", outageTime.toString() );
    
    ControlStatepointList.Build( $( "#ID_DEVICEOUTAGE_CONTACT" + this.dynid ), {
      inquirer: "*",
      allowedTypes: [ "BOOL" ]
    } );     
    if( statepoint ) {
      $( "#ID_DEVICEOUTAGE_CONTACT" + this.dynid ).viewable( "option", "selected", [ statepoint ] );
    }

  }


  //---------------------------------------------------------------------------
  //  Fill entity into controls
  //
  FormFillNetworkList()
  {
    let gridData = [];

    if( this.entity && this.networkData )  {

      let basicParamInterfaces = {};
      if( ( 'sip' in this.entity ) && ( 'network' in this.entity['sip'] ) ) {
        basicParamInterfaces = this.entity['sip']['network'];
      }

      for( let macAddr in this.networkData['interfaces'] )  {
        
        let interfaceData = this.networkData['interfaces'][ macAddr ];
        let newRecord = { 
          id: macAddr,
          name: interfaceData['name'],
          displayname: interfaceData['displayname'] || interfaceData['name'],
          ipV4: false,
          ipV4Address: interfaceData['ipv4'],
          ipV6: false,
          ipV6Address: null,
          client: false,
          telephone: false,
          port: 5060
        };

        if( macAddr in basicParamInterfaces )  {
          let paramRecord = basicParamInterfaces[ macAddr ];
          newRecord['ipV4'] = paramRecord['ipV4'];
          newRecord['ipV6'] = paramRecord['ipV6'];
          newRecord['client'] = paramRecord['client'];
          newRecord['telephone'] = paramRecord['telephone'];
        }
        gridData.push( newRecord );
      }
    }

    this.gridDataset.SetData( gridData );
  }

  //---------------------------------------------------------------------------
  //  Read values from controls and write back to entity.
  //
  FormReadControls()
  {
    // Group "general".
    this.entity['lang']['syslang'] = $( "#ID_SYSLANG" + this.dynid ).listbox( "option", "selected" )[ 0 ];
    
    // Group "times (door)".
    this.entity['timeouts']['doorRingTimeout'] = parseInt( $( "#ID_DOOR_RING_TIMEOUT" + this.dynid ).edit( "option", "value" ) );
    this.entity['timeouts']['doorCallTimeout'] = parseInt( $( "#ID_DOOR_CALL_TIMEOUT" + this.dynid ).edit( "option", "value" ) ) * 60;
    this.entity['timeouts']['lateVideoTimeout'] =  parseInt( $( "#ID_LATEVIDEO_TIMEOUT" + this.dynid ).edit( "option", "value" ) );

    // Group "times (general)".
    this.entity['timeouts']['ringTimeout'] = parseInt( $( "#ID_RING_TIMEOUT" + this.dynid ).edit( "option", "value" ) );
    this.entity['timeouts']['callTimeout'] = parseInt( $( "#ID_CALL_TIMEOUT" + this.dynid ).edit( "option", "value" ) ) * 60;
    
    // Group "times (lists)".
    this.entity['listsTime']['grabpic']   = parseInt( $( "#ID_LISTS_TIME_GRABPIC" + this.dynid ).edit( "option", "value" ) );
    this.entity['listsTime']['cdr']       = parseInt( $( "#ID_LISTS_TIME_CDR" + this.dynid ).edit( "option", "value" ) );
    this.entity['listsTime']['voicemail'] = parseInt( $( "#ID_LISTS_TIME_VOICEMAIL" + this.dynid ).edit( "option", "value" ) );

    // device outage
    if( !( 'features' in this.entity ) )  this.entity['features'] = {};
    if( !( 'deviceoutage' in this.entity['features'] ) )  this.entity['features']['deviceoutage'] = {};
    
    if( Data.IsFeatureEnabled( "module.deviceoutage" ) )  {
      var outageTime = parseInt( $( "#ID_DEVICEOUTAGE_TIME" + this.dynid ).edit( "option", "value" ) );
      var statepoint = $( "#ID_DEVICEOUTAGE_CONTACT" + this.dynid ).viewable( "option", "selected" )[ 0 ];
      this.entity['features']['deviceoutage']['time'] = outageTime * 60;
      this.entity['features']['deviceoutage']['statepoint'] = statepoint;
      if( !( 'sp' in this.entity ) ) {
        this.entity['sp'] = {};
      }
      this.entity['sp']['deviceoutage'] = {
        name: "deviceoutage",
        type: "BOOL",
        address: Data.Instance().RootEntid() + "/admui.basicparam/deviceoutage",
        hidden: true,
        alias: {
          target: statepoint,
          timeout: null
        }
      };
    }
    else {
      if( 'features' in this.entity ) {
        delete this.entity['features']['deviceoutage'];
      }
      if( 'sp' in this.entity ) {
        delete this.entity['sp']['deviceoutage'];
      }
    }

  }

  //---------------------------------------------------------------------------
  //  Read values from network list
  //
  FormReadNetworkList()
  {
    if( !( 'sip' in this.entity ) ) {
      this.entity['sip'] = {};
    }
    this.entity['sip']['network'] = {};

    let gridData = this.gridDataset.GetData();
    for( let record of gridData )  {
      this.entity['sip']['network'][ record['id'] ] = {
        name: record['name'],
        ipV4: record['ipV4'],
        ipV4Address: record['ipV4Address'],
        ipV6: record['ipV6'],
        ipV6Address: record['ipV6Address'],
        client: record['client'],
        telephone: record['telephone'],
        port: record['port']
      };
    }

  }


  //===========================================================================
  //  TOOLBAR CONTROL
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close form (close button on toolbar)
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( "callnumberplan" );
  }

  //---------------------------------------------------------------------------
  //  toolbar button
  //
  ControlToolbarClick( id )
  {
    switch( id )  {
      case "ID_UNDO":               this.CommandUndo();  break;
      case "ID_SAVE":               this.CommandSave();  break;
      case "ID_RESTART_SERVICES":   AccessApplication.Instance().Command( 'restart-service' ); break;
      case "ID_HELP":               break;
    }   
  }

  //---------------------------------------------------------------------------
  //  checkbox in grid changed
  //
  ControlGridInplace( columnId, record, param )
  {
    this.Option( "modified", 1 );
    switch( columnId ) {
      case "ipV4": return this.ControlGridInplaceIPv4( record, param );
      case "client": return this.ControlGridInplaceClient( record, param );
      case "telephone": return this.ControlGridInplacePhone( record, param );
    }
    return record;
  }

  //---------------------------------------------------------------------------
  //  checkbox IPv4 in grid changed
  //
  ControlGridInplaceIPv4( record, param )
  {
    record['ipV4'] = !( record['ipV4'] );
    record['client'] = record['ipV4'] ? true : false;
    record['telephone'] = record['ipV4'] ? true : false;
    return record;
  }

  //---------------------------------------------------------------------------
  //  checkbox IPv4 in grid changed
  //
  ControlGridInplaceClient( record, param )
  { 
    record['client'] = !( record['client'] );
    return record;
  }

  //---------------------------------------------------------------------------
  //  checkbox IPv4 in grid changed
  //
  ControlGridInplacePhone( record, param )
  { 
    record['telephone'] = !( record['telephone'] );
    return record;
  }

  //---------------------------------------------------------------------------
  //  render cell to disable client / telephone
  //
  ControlGridRenderCell( columnId, record )
  {
    let item = {
      text: record[ columnId ],
      icon: null
    };

    if( ( columnId == "client" && record['ipV4'] === false )
     || ( columnId == "telephone" && record['ipV4'] === false )
      ) {
      item['status'] = "disabled";
    }

    return item;
  }


  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Validate & save
  //
  CommandSave()
  {
    // Do validation.
    let numErr = this.Validate();

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.FormReadNetworkList();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  CommandUndo(): undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }


  //===========================================================================
  //  DATA
  //===========================================================================

  //---------------------------------------------------------------------------
  //  XHR read request.
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( "admui.basicparam" );
    Data.Instance().XhrRequest( "COSCONF.READ" );
    this.StatusLoading();
  }

  //---------------------------------------------------------------------------
  //  Response on XHR request.
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy
    this.entity = $.clone( entity );

    // Fill controls and enter load finished to do 1st validation
    this.FormFillControls();
    this.FormFillNetworkList();

    this.StatusLoadFinished();

  }

  //---------------------------------------------------------------------------
  //  Response on XHR COSCONFF.READ
  //
  DataResponseCosconfRead( response )
  {
    this.networkData = response['data'];
    this.FormFillNetworkList();

  }

  //---------------------------------------------------------------------------
  //  XHR write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    //this.UnitTest( );

    // create a copy for save: name & label-handling + reference reset.
    let entity = this.DataPrepareEntityForSave( this.entity );    

    // save the entity
    Data.Instance().SavePartialEntity( entity );

    // base-function: reload entity after save
    this.DataReloadAll();
    this.ToastRebootRequired();
  }


  //===========================================================================
  //  VALIDATION
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_DOOR_RING_TIMEOUT" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_DOOR_CALL_TIMEOUT" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_LATEVIDEO_TIMEOUT" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_RING_TIMEOUT" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_CALL_TIMEOUT" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_LISTS_TIME_GRABPIC" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_LISTS_TIME_CDR" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_LISTS_TIME_VOICEMAIL" + this.dynid ).edit( "validate" );
    return numErr;
  }

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  validate handler door ring
  //
  ValidateDoorRingTimeout( validateControl )
  {
    let intValue = parseInt( validateControl.value );
    if( isNaN( intValue ) || intValue < 5 || intValue > 60 ) {
      validateControl.error = "{$basicparam_error_door_ring_timeout}";
      return 1;
    }
    return 0;
  }

  //-----------------------------------------------------------------------------
  //  validate handler door call
  //
  ValidateDoorcallTimeout( validateControl )
  {
    let intValue = parseInt( validateControl.value );
    if( isNaN( intValue ) || intValue < 1 || intValue > 1440 ) {
      validateControl.error = "{$basicparam_error_door_call_timeout}";
      return 1;
    }
    return 0;
  }
  //-----------------------------------------------------------------------------
  //  validate handler late video
  //
  ValidateLatevideoTimeout( validateControl )
  {
    let intValue = parseInt( validateControl.value );
    if( isNaN( intValue ) || intValue < 0 || intValue > 15 ) {
      validateControl.error = "{$basicparam_error_lateviedeo_timeout}";
      return 1;
    }
    return 0;
  }

  //-----------------------------------------------------------------------------
  //  validate handler door ring
  //
  ValidateRingTimeout( validateControl )
  {
    let intValue = parseInt( validateControl.value );
    if( isNaN( intValue ) || intValue < 5 || intValue > 120 ) {
      validateControl.error = "{$basicparam_error_ring_timeout}";
      return 1;
    }
    return 0;
  }

  //-----------------------------------------------------------------------------
  //  validate handler door ring
  //
  ValidateCallTimeout( validateControl )
  {
    let intValue = parseInt( validateControl.value );
    if( isNaN( intValue ) || intValue < 10 || intValue > 1440 ) {
      validateControl.error = "{$basicparam_error_call_timeout}";
      return 1;
    }
    return 0;
  }

  //-----------------------------------------------------------------------------
  //  validate handler door ring
  //
  ValidateGrabpicTime( validateControl )
  {
    let intValue = parseInt( validateControl.value );
    if( isNaN( intValue ) || intValue < 1 || intValue > 120 ) {
      validateControl.error = "{$basicparam_error_grabpic_time}";
      return 1;
    }
    return 0;
  }

  //-----------------------------------------------------------------------------
  //  validate handler door ring
  //
  ValidateCdrTime( validateControl )
  {
    let intValue = parseInt( validateControl.value );
    if( isNaN( intValue ) || intValue < 1 || intValue > 120 ) {
      validateControl.error = "{$basicparam_error_cdr_time}";
      return 1;
    }
    return 0;
  }

  //-----------------------------------------------------------------------------
  //  validate handler door ring
  //
  ValidateVoicemailTime( validateControl )
  {
    let intValue = parseInt( validateControl.value );
    if( isNaN( intValue ) || intValue < 1 || intValue > 120 ) {
      validateControl.error = "{$basicparam_error_voicemail_time}";
      return 1;
    }
    return 0;
  }

  //-----------------------------------------------------------------------------
  //  validate handler door ring
  //
  ValidateOutageTime( validateControl )
  {
    let intValue = parseInt( validateControl.value );
    if( isNaN( intValue ) || intValue < 1 || intValue > 10 ) {
      validateControl.error = "{$basicparam_error_outage_time}";
      return 1;
    }
    return 0;
  }  

  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  OnUnitTest( testController )
  {
    testController.AssertObject( "Entity class", this.entity, "class", "admui" );
  }  

  //===========================================================================
  //  TOAST
  //===========================================================================

  //---------------------------------------------------------------------------
  //  ApplyStageRebootRequired(): description
  //
  ToastRebootRequired()
  {
    $( "#ID_TOAST" + this.dynid ).toast( {
      text: "{$network_toast_unsaved_notice}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h",
      click: ( id ) => {
        if( id == "ID_SAVE_UNSAVED" )  {
          AccessApplication.Instance().Command( 'restart-service' );
        }
      },
    } );
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormBasicparam.Register( FormBasicparam.viewableTemplate );
PageMain.RegisterForm( "basicparam", FormBasicparam.viewableTemplate, FormBasicparam.documentTitle, "basicparam" );
//=============================================================================
//
//  Logfile Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          19.07.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormLogfile extends FormBase
{
  //---------------------------------------------------------------------------
  //  STATIC
  //---------------------------------------------------------------------------

  static viewableTemplate = "/forms/system/logfile";   // vw template
  static documentIcon = "mdi mdi-clipboard-outline";   // icon for toolbar & doc
  static documentTitle = "{$logfile_document_title}";  // title
  static helpFile = "logfile";
  
  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createData )
  {
    //-----------------------------------------------------
    //  controls
    //-----------------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    $( '#ID_TOOLBAR' + this.dynid ).toolbar( {
      size: -1,
      icon: FormLogfile.documentIcon,
      text: FormLogfile.documentTitle,
      data: [
        { id: 'ID_REFRESH', icon: 'mdi mdi-refresh', status: "normal", tooltip: "{$const_form_tooltip_refresh}" },
        { id: 'ID_DELETE', icon: 'mdi mdi-delete', status: "red", tooltip: "{$const_form_tooltip_delete}" },        
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );

    // grid
    $( '#ID_LOGFILE_GRID' + this.dynid ).grid( {
      size: -1,   // same as "100%"
      height: 25,
      selection: 'click',
      data: [],
      dataId: 'guid',
      scrollbar: 'static', // 'overlay', 'static'
      regkey: 'logfile',
      sort: 'created',
      sortOrder: '+',
      header: {
        'type':      { text: "", size: 4,    align: "center",  resizeable: false,   sortable: true, icon: "mdi mdi-checkbox-multiple-blank-circle-outline" },    
        'created':   { text: "{$logfile_header_created}", size: -4,  align: "left",  resizeable: true, sortable: true },
        'name':      { text: "{$logfile_header_entid}",   size: -6,  align: "left",  resizeable: true, sortable: true, inplace: "link" },
        'message':   { text: "{$logfile_header_message}", size: -12, align: "left",  resizeable: true, sortable: true },        
        'admin':     { text: "{$logfile_header_admin}",   size: -6,  align: "left",  resizeable: true, sortable: true  }
      },
      renderCell: ( columnId, record ) => {
        let icon = "";
        let text = "";
        if( columnId == 'type' )  {
          // Translate record type -> LED.

          // Up to now there is only one type: 'info'!
          icon = 'mdi mdi-information-outline blue-text';
        }
        else if( columnId == 'created' ) {

          // Format "created" time for display.
          if( record['created'] > 0 )  {
            text = Time.FormatFromUnix( record['created'] );
          }
          else  {
            text = "-";
          }
        }
        else  {
          text = record[ columnId ];
        }

        return { 
          'text': ( typeof text == "string" ) ? text : "",
          'icon': icon
        };
      },
      //renderStatus: ( idRecord, record ) => {
      //  return "normal";
      //},
      //drawItem: ( record, $container ) => {
      //  return self.CustomListDrawItem( record, $container );
      //},
      inplaceLink: ( columnId, record ) => {
        PageMain.ShowEntity( record['entid'] );
      }

    } );

    //-----------------------------------------------------
    //  Start
    //-----------------------------------------------------    

    Data.Instance().On( "LOGFILE/LIST/", this.etag, this.DataResponseLogfile.bind( this ) );
    Data.Instance().On( "LOGFILE/LIST/DELETE/", this.etag, this.DataResponseLogfileDelete.bind( this ) );
    this.DataRequest();

  }//load



  //---------------------------------------------------------------------------
  //  CONTROL
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  close form (close button on toolbar)
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( "logfile" );
  }

  //---------------------------------------------------------------------------
  //  toolbar button
  //
  ControlToolbarClick( id )
  {
    switch( id )  {
      case "ID_REFRESH":  this.DataRequest();   break;
      case "ID_DELETE":   this.DeleteRequest(); break;
      case "ID_HELP":     break;
    }   
  }


  //---------------------------------------------------------------------------
  //  DATA
  //---------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  Do xhr request to get the logfile.
  //
  DataRequest()
  {
    Data.Instance().Logfile().ListRequest( );
    //this.StatusLoading();
  }

  //-----------------------------------------------------------------------------
  //  Do xhr request to delete the logfile.
  //
  DeleteRequest()  
  {
    Data.Instance().Logfile().DeleteRequest( );
  }


  //-----------------------------------------------------------------------------
  //  XHR response: logfile.
  //  
  DataResponseLogfile( logfile )
  {
    $( '#ID_LOGFILE_GRID' + this.dynid ).grid( "option", "data", logfile );
  }

  //-----------------------------------------------------------------------------
  //  XHR response: logfile deleted.
  //    
  DataResponseLogfileDelete( )
  {
    $( '#ID_LOGFILE_GRID' + this.dynid ).grid( "option", "data", [] );
  }


  //---------------------------------------------------------------------------
  //  STATUS
  //---------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  StatusModified(): need to overwrite base class
  //
  StatusModified()
  {
  }

  
}


//=============================================================================
//  REGISTER
//=============================================================================

FormLogfile.Register( FormLogfile.viewableTemplate );
PageMain.RegisterForm( "logfile", FormLogfile.viewableTemplate, FormLogfile.documentTitle, "logfile" );
//=============================================================================
//
//  Callnumber Plan Form
//  (c) 2021 CYTEL Technology AG
//
//  Form for the one-and-only entity "admui.callprofile".
//
//  Editor: Peter Aidam
//          22.07.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormCallnumberPlan extends FormBase
{
  //---------------------------------------------------------------------------
  //  STATIC
  //---------------------------------------------------------------------------

  static viewableTemplate = "/forms/system/callnumberplan";   // vw template
  static documentIcon = "mdi mdi-phone-lock";                 // icon for toolbar & doc
  static documentTitle = "{$callnumberplan_document_title}";  // title
  static helpFile = "callnumberplan";
  
  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createData )
  {

    //-------------------------------------------
    //  data
    //-------------------------------------------

    // Note: This form is not started from the device tree, so entinfo is not part of createData!
    this.entinfo = Data.Instance().Entinfo( "admui.callprofile" );
    this.entity = {};
    this.status = "statusInitialized";
    this.nextProfileId = 1;
    this.selectedProfile = null;

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: FormCallnumberPlan.documentIcon,
      text: FormCallnumberPlan.documentTitle,      
      data: [
        { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
        { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );

    // Profiles grid.
    $( '#ID_PROFILES' + this.dynid ).grid( {
      size: CONST.size.XL,
      height: 6, //"fit",
      //heightMin: 6,
      //heightMax: 10,
      selection: 'single',
      data: [],
      dataId: 'id',
      scrollbar: 'overlay', // 'overlay', 'static'
      focus: "cell",
      sort: 'name',
      sortOrder: '+',
      headerHidden: true,
      header: {
        'name': { text: "{$callnumberplan_profile_header_name}", size: CONST.size.XL }
      },
      result: this.ControlProfileGridResult.bind( this )
    } );

    // Profiles buttons
    $( '#ID_PROFILE_ADD' + this.dynid ).button( {
      size: CONST.size.M,
      status: 'normal',
      text: "{$callnumberplan_buttonlabel_add}",
      click: () => {
        this.ControlProfileAddClick();
      }
    } );
    $( '#ID_PROFILE_EDIT' + this.dynid ).button( {
      size: CONST.size.M,
      text:"{$callnumberplan_buttonlabel_edit}",
      status: 'disabled',
      click: () => {
        this.ControlProfileEditClick();
      }
    } );
    $( '#ID_PROFILE_DELETE' + this.dynid ).button( {
      size: CONST.size.XS,
      status: 'disabled',
      icon: 'mdi mdi-trash-can-outline',
      click: () => {
        this.ControlProfileDeleteClick();
      }
    } );

    // Permissions data.
    this.permissionsDataset = new Dataset( { id: 'id' } );
    this.permissionsDataset.SetData( [] );    
    this.permissionsDataview = this.permissionsDataset.BuildView( 'default', {
      selection: 'single',
      sorter: function( recordA, recordB )  {
        if( recordA["t"].length < recordB["t"].length )
          return 1;               // bigger, return positive
        if( recordA["t"].length > recordB["t"].length )
          return -1;              // smaller, return negative
        return null; // use default sorter (compare field 'sortKey')
      }
    } );

    // Permissions grid.
    $( '#ID_PERMISSIONS' + this.dynid ).grid( {
      size: CONST.size.XL,
      height: 6,
      selection: 'single',
      data: this.permissionsDataview,
      dataId: 'id',
      scrollbar: 'overlay',
      sort: 't',
      sortOrder: '+',
      headerHidden: true,
      header: {
        't': { text: "{$callnumberplan_permission_header_rule}", size: CONST.size.XL - 10 },
        'p': { text: "{$callnumberplan_permission_header_allowed}", size: 10, align: "left" }
      },
      renderCell: ( columnId, record ) => {
        let icon = "";
        let text = "";

        if( columnId == 't' ) {
          text = record[ columnId ];
        }
        else if( columnId == 'p' )  {
          icon = record[ columnId ] ? "mdi mdi-check-bold green-text" : "mdi mdi-close-thick red-text";
        }
        return { 
          'text': ( typeof text == "string" ) ? text : "",
          'icon': icon
        };
      },
      result: this.ControlPermissionGridResult.bind( this )      
    } );

    // Permissions buttons
    $( '#ID_PERMISSION_ADD' + this.dynid ).button( {
      size: CONST.size.M,
      status: 'normal',
      text: "{$callnumberplan_buttonlabel_add}",
      click: () => {
        this.ControlPermissionAddClick();
      }
    } );
    $( '#ID_PERMISSION_EDIT' + this.dynid ).button( {
      size: CONST.size.M,
      text:"{$callnumberplan_buttonlabel_edit}",
      status: 'disabled',
      click: () => {
        this.ControlPermissionEditClick();
      }
    } );
    $( '#ID_PERMISSION_DELETE' + this.dynid ).button( {
      size: CONST.size.XS,
      status: 'disabled',
      icon: 'mdi mdi-trash-can-outline',
      click: () => {
        this.ControlPermissionDeleteClick();
      }
    } );
 

    //-------------------------------------------
    //  Start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    this.DataRequest();   

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Fill entity into controls
  //
  FormFillControls()
  {
    let profileGridDataset = [];
    this.nextProfileId = this.entity["callprofile"]["maxprofile"];

    // Translate entity's profile object into datset array for grid.
    for( let profileId in this.entity["callprofile"] )  {

      let profileElement = this.entity["callprofile"][ profileId ];

      // Skip "maxprofile" property.
      if( typeof profileElement != "object")  {
        continue;
      }

      // Prepare record.
      let record = profileElement;
      record["id"] = profileId;

      // Add to grid data.
      profileGridDataset.push( record );
   
    } // for

    // Data
    $( "#ID_PROFILES" + this.dynid ).grid( "option", "data", profileGridDataset );

    // 1st selection
    if( profileGridDataset.length > 0  )  {
      let profileId = profileGridDataset[ 0 ]["id"];
      $( "#ID_PROFILES" + this.dynid ).grid( "option", "selected", [ profileId ] );
      this.FormFillPermissionGrid( profileId );
    }
  }

  //---------------------------------------------------------------------------
  //  Fill permissions grid with a given profile's rules.
  //  
  FormFillPermissionGrid( profileId )
  {
    // Get profile record from profileId.
    var profileDataset = $( "#ID_PROFILES" + this.dynid ).grid( "option", "dataset" );
    let profileRecord = profileDataset.Get( profileId ) ;

    let permissionsGridData = [];
    
    // Translate profile's permission array into datset array for grid. 
    // Grid needs ID for each record!
    if( Array.isArray( profileRecord["permission"] ) )  {
      for( let permissionRecord of profileRecord["permission"] )  {

        // Compose element for permission grid dataset.
        let permissionElement = $.clone( permissionRecord );
        permissionElement["id"] = "id" + $.uuid();

        permissionsGridData.push( permissionElement );
      } // for
    }

    // Data
    this.permissionsDataset.SetData( permissionsGridData );

    // 1st selection
    if( permissionsGridData.length > 0  )  {
      let firstRecord = this.permissionsDataview.GetByIndex( 0 );
      $( "#ID_PERMISSIONS" + this.dynid ).grid( "option", "selected", [ firstRecord["id"] ] );
    }

    // Ensure saving permissions on correct profile.
    this.selectedProfile = profileId;

    this.FormManageProfileGridButtons();
  }

  //---------------------------------------------------------------------------
  //  Read values from controls and write back to entity.
  //
  FormReadControls()
  {
    // Read profile array back from profile grid control.
    let profileGridDataset = $( "#ID_PROFILES" + this.dynid ).grid( "option", "data" );

    // Rebuild entity's callprofile object.
    this.entity["callprofile"] = {};
    this.entity["callprofile"]["maxprofile"] = this.nextProfileId;
    for( let profileRecord of profileGridDataset ) {

      // Remove ids from the permission elements.
      let permissionArray = profileRecord["permission"];

      let profileElement = {};
      profileElement["name"]       = profileRecord["name"];
      profileElement["permission"] = permissionArray;

      this.entity["callprofile"][ profileRecord["id"] ] = profileElement;
    } // for

  }

  //---------------------------------------------------------------------------
  //  Enable or disable profile grid's edit/delete buttons.
  //
  FormManageProfileGridButtons()
  {
    let selected = $( "#ID_PROFILES" + this.dynid ).grid( "option", "selected" ); 
    if( selected.length == 0 )  {
      $( "#ID_PROFILE_EDIT" + this.dynid ).button( "option", "status", "disabled" );
      $( "#ID_PROFILE_DELETE" + this.dynid ).button( "option", "status", "disabled" );
    }
    else if( selected == "prfstd" )  {
      // Standard profile may be edited but not deleted.
      $( "#ID_PROFILE_EDIT" + this.dynid ).button( "option", "status", "normal" );
      $( "#ID_PROFILE_DELETE" + this.dynid ).button( "option", "status", "disabled" );
    }
    else {
      $( "#ID_PROFILE_EDIT" + this.dynid ).button( "option", "status", "normal" );
      $( "#ID_PROFILE_DELETE" + this.dynid ).button( "option", "status", "normal" );
    }
  }

  //---------------------------------------------------------------------------
  //  Enable or disable permission grid's edit/delete buttons.
  //
  FormManagePermissionGridButtons()
  {
    let selected = $( "#ID_PERMISSIONS" + this.dynid ).grid( "option", "selected" ); 
    if( selected.length == 0 )  {
      $( "#ID_PERMISSION_EDIT" + this.dynid ).button( "option", "status", "disabled" );
      $( "#ID_PERMISSION_DELETE" + this.dynid ).button( "option", "status", "disabled" );
    }
    else {
      $( "#ID_PERMISSION_EDIT" + this.dynid ).button( "option", "status", "normal" );
      $( "#ID_PERMISSION_DELETE" + this.dynid ).button( "option", "status", "normal" );
    }
  }


  //===========================================================================
  //  TOOLBAR CONTROL
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close form (close button on toolbar)
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( "callnumberplan" );
  }

  //---------------------------------------------------------------------------
  //  toolbar button
  //
  ControlToolbarClick( id )
  {
    switch( id )  {
      case "ID_UNDO":   this.CommandUndo();  break;
      case "ID_SAVE":   this.CommandSave();  break;
      case "ID_HELP":   break;
    }   
  }


  //===========================================================================
  //  GRID CONTROL
  //===========================================================================

  //---------------------------------------------------------------------------
  //  A profile was selected. Fill permissions grid with selected profile's
  //  rules.
  //  
  ControlProfileGridResult( profileIdArray )
  {
    let profileId = profileIdArray[ 0 ];
    this.FormFillPermissionGrid( profileId );
  }

  //---------------------------------------------------------------------------
  //  A permission was selected.
  //  
  ControlPermissionGridResult( permissionId )
  {
    this.FormManagePermissionGridButtons();
  }  


  //===========================================================================
  //  BUTTON CONTROL
  //===========================================================================

  ControlProfileAddClick( )
  {
    // Create new profile record.
    let record = {
      id: "prf" + this.nextProfileId,
      name: "",
      permission: []
    };
    this.nextProfileId++;

    // Dialog.
    NameCallProfile.Show( {
      value: record,
      result: ( resultData ) => {
        var dataset = $( "#ID_PROFILES" + this.dynid ).grid( "option", "dataset" );
        dataset.Add( resultData );
        this.Option( "modified", 1 );
        $( "#ID_PERMISSIONS" + this.dynid ).grid( "option", "dataset", [] );
      }
    } );
  }
  
  ControlProfileEditClick( )
  {
    // Get the profile record to be modified.    
    let selected = $( "#ID_PROFILES" + this.dynid ).grid( "option", "selected" );
    var dataset = $( "#ID_PROFILES" + this.dynid ).grid( "option", "dataset" );
    let record = dataset.Get( selected[ 0 ] ) ;
    
    // Dialog.
    NameCallProfile.Show( {
      value: record,
      result: ( resultData ) => {
        dataset.Update( resultData );
        this.Option( "modified", 1 );
      }
    } );
  }

  ControlProfileDeleteClick( )
  {
    // Get the profile record to be deleted.
    let selected = $( "#ID_PROFILES" + this.dynid ).grid( "option", "selected" );
    var profileGridDataset = $( "#ID_PROFILES" + this.dynid ).grid( "option", "dataset" );
    let len = profileGridDataset.length;
    profileGridDataset.Delete( selected[ 0 ] );

    len = profileGridDataset.length;

    this.Option( "modified", 1 );

    // Selection
    var profileData = $( "#ID_PROFILES" + this.dynid ).grid( "option", "data" );
    if( profileData.length > 0  )  {
      let profileId = profileData[ 0 ]["id"];
      $( "#ID_PROFILES" + this.dynid ).grid( "option", "selected", [ profileId ] );
      this.FormFillPermissionGrid( profileId );
    }
    else  {
      $( "#ID_PERMISSIONS" + this.dynid ).grid( "option", "dataset", [] );
    }
  }

  ControlPermissionAddClick( )
  {
    // Create new permission record.
    let record = {
      id: "id" + $.uuid(),
      t: "",
      p: true
    };

    // Dialog.
    PermissionCallProfile.Show( {
      value: record,      
      result: ( resultData ) => {
        this.permissionsDataset.Add( resultData );

        // Write profile's permissions back into profile grid's dataset.
        this.ApplyPermissions();

        // Modified flag.
        this.Option( "modified", 1 );
      }
    } );
  }
  
  ControlPermissionEditClick( )
  {
    // Get the permission record to be modified.
    let selected = $( "#ID_PERMISSIONS" + this.dynid ).grid( "option", "selected" );
    let permissionRecord = this.permissionsDataset.Get( selected[ 0 ] );     
    
    // Dialog.    
    PermissionCallProfile.Show( {
      value: permissionRecord,
      result: ( resultData ) => {
        this.permissionsDataset.Update( resultData );

        // Write profile's permissions back into profile grid's dataset.
        this.ApplyPermissions();

        // Modified flag.
        this.Option( "modified", 1 );
      }
    } );
  }

  ControlPermissionDeleteClick( )
  {
    // Get the permission record to be deleted.
    let selected = $( "#ID_PERMISSIONS" + this.dynid ).grid( "option", "selected" );
    this.permissionsDataset.Delete( selected[ 0 ] );

    // Write profile's permissions back into profile grid's dataset.
    this.ApplyPermissions();

    // Modified flag.
     this.Option( "modified", 1 );    
  }

  //---------------------------------------------------------------------------
  //  Helper function for button controls: Get permissions from permission grid
  //  and apply them to selected profile in profile grid.
  //
  ApplyPermissions( )
  {
    // Precondition.
    if( typeof this.selectedProfile != "string" )  {
      return;
    }

    // Get the selected profile record.
    var profileDataset = $( "#ID_PROFILES" + this.dynid ).grid( "option", "dataset" );
    let profileRecord = profileDataset.Get( this.selectedProfile );

    // Rebuild profile's permission array.
    profileRecord["permission"] = [];
    let permissionData = this.permissionsDataset.GetData( );
    for ( let record of permissionData )  {

      let permissionElement = {};
      permissionElement["t"] = record["t"];
      permissionElement["p"] = record["p"];

      profileRecord["permission"].push( permissionElement );
    } // for  
    profileDataset.Update( profileRecord );
  }


  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Validate & save
  //
  CommandSave()
  {
    // Do validation.
    let numErr = this.Validate();

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  CommandUndo(): undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }


  //===========================================================================
  //  DATA
  //===========================================================================

  //---------------------------------------------------------------------------
  //  XHR read request.
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( "admui.callprofile" );
    this.StatusLoading();
  }

  //---------------------------------------------------------------------------
  //  Response on XHR request.
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy
    this.entity = $.clone( entity );

    // while developing the unit test uncomment this:
    // this.UnitTest();

    // Fill controls and enter load finished to do 1st validation
    this.FormFillControls();
    this.StatusLoadFinished();

    setTimeout( () =>  {
      this.FormManageProfileGridButtons( );
      this.FormManagePermissionGridButtons( );
    }, 10 );
  }

  //---------------------------------------------------------------------------
  //  XHR write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest( );

    // create a copy for save: name & label-handling + reference reset.
    let entity = this.DataPrepareEntityForSave( this.entity );    

    // save the entity
    Data.Instance().SavePartialEntity( entity );

    // base-function: reload entity after save
    this.DataReloadAll();
  }


  //===========================================================================
  //  VALIDATION
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  Validate()
  {
    let numErr = 0;
    // No validation at this point, it's done by the add/edit dialogs.    
    return numErr;
  }

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  OnUnitTest( testController )
  {
    testController.AssertObject( "Entity class", this.entity, "class", "admui" );
  }  

}


//=============================================================================
//  REGISTER
//=============================================================================

FormCallnumberPlan.Register( FormCallnumberPlan.viewableTemplate );
PageMain.RegisterForm( "callnumberplan", FormCallnumberPlan.viewableTemplate, FormCallnumberPlan.documentTitle, "callnumberplan" );
﻿//=============================================================================
//
//  Backup/Restore Form
//  (c) 2021 CYTEL Technology AG
//
//  Operating the Backup service.
//
//  Editor: Frank Jaekel
//          29.07.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormBackup extends FormBase
{
  //---------------------------------------------------------------------------
  //  STATIC
  //---------------------------------------------------------------------------

  static viewableTemplate = "/forms/system/backup";   // vw template
  static documentIcon = "mdi mdi-archive";         // icon for toolbar & doc
  static documentTitle = "{$backup_document_title}";  // title
  static helpFile = "backup";
  
  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.uploadedFile = null;  // Vorläufig, stattdessen vom Server abrufbar machen!
    this.timerId = 0;

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: FormBackup.documentIcon,
      text: FormBackup.documentTitle,      
      data: [
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
    } );

    $( "#ID_BACKUP" + this.dynid ).button( {
      size: CONST.size.L,
      text: "{$backup_backup_button}",
      click: () => {
        this.ControlCreateButtonClick();
      }
    });

    $( "#ID_RESTORE_UPLOAD" + this.dynid ).fileupload( {
      height: 4,
      size: CONST.size.XL,
      url: "uploadBackup.php",
      text: "{$backup_restore_upload}",
      result: this.ControlUploadResult.bind( this )
    } );

    $( "#ID_RESTORE_FILE" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: "readonly",
      placeholder: "{$backup_restore_file_placeholder}"
    } );

    $( "#ID_RESTORE_BUTTON" + this.dynid ).button( {
      size: CONST.size.L,
      status: "disabled",      
      text: "{$backup_restore_button}",
      click: () => {
        this.ControlRestoreButtonClick();
      }      
    } );



    //-------------------------------------------
    //  Start 
    //-------------------------------------------

    this.StatusIdle();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Fill entity into controls
  //
  FormFillControls()
  {
  }  


  //===========================================================================
  //  CONTROL
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close form (close button on toolbar)
  //
  ControlToolbarClose()
  {
  }

  //---------------------------------------------------------------------------
  //  toolbar button
  //
  ControlToolbarClick( id )
  {
    switch( id )  {
    }   
  }


  //===========================================================================
  //  UPLOAD CONTROL
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Upload control reports on success or failure.
  //  
  ControlUploadResult( success, filename )
  {
    if( success )  {

      // Vorläufig! 
      this.uploadedFile = filename;

      // Now tell server to check if a valid backup file was uploaded.
      // Note: On server file was renamed to "access.backup", to avoid trouble with Umlauts!
      new XhrRequest( "BACKUP.INVESTIGATE", { 'file': "access.backup"/*filename*/ }, ( response ) => {
        this.XhrResponse( response );
      } );
    }
    else  {
      
      // Show error info. Note: Reason for failure may be inacceptable filename or extension.
      this.ToastShowError( "#ID_TOAST" + this.dynid, "{$backup_restore_file_failed}", filename );
      $( "#ID_RESTORE_FILE" + this.dynid ).edit( "option", "placeholder", "{$backup_restore_file_placeholder}" );      
    }
  }


  //===========================================================================
  //  BUTTON CONTROL
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Start create server backup file.
  //  
  ControlCreateButtonClick( )
  {
    new XhrRequest( "BACKUP.CREATE", null, ( response ) => {
      this.XhrResponse( response );
    } );
   
  }

  //---------------------------------------------------------------------------
  //  Restore server from a previously uploaded .backup file.
  //    
  ControlRestoreButtonClick( )
  {
    // Note: On server file was renamed to "access.backup", to avoid trouble with Umlauts!
    new XhrRequest( "BACKUP.RESTORE", { 'file': "access.backup"/*this.uploadedFile*/ }, ( response ) => {
      this.XhrResponse( response );
    } );

    // First visible response on Restore click: Disable button.
    $( "#ID_RESTORE_BUTTON" + this.dynid ).button( "option", "status", "disabled" );    
  }


  //===========================================================================
  //  VALIDATION
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  Validate()
  {
    return 0;
  }

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }


  //===========================================================================
  //  XHR
  //===========================================================================

  XhrResponse( response )
  {
    let command      = response['meta']['command'];
    let responseCode = parseInt( response['meta']['status'] );

    // We are done if server was not available.
    if( responseCode == 503 ) {
      return;
    }

    //-- BACKUP.CREATE --------------------------------------
    if( command == "BACKUP.CREATE" )  {

      // Start polling for creation status.
      setTimeout( () => {
        new XhrRequest( "BACKUP.CREATE.STATUS", null, ( response ) => {
          this.XhrResponse( response );
        } );
      }, 10 );  // <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

    }

    //-- BACKUP.CREATE.STATUS -------------------------------
    else if( command == "BACKUP.CREATE.STATUS" )  {

      if( response['data']['status'] == "WORKING" )  {
        // (Still) waiting for server to finish backup file.

        // Continue polling for creation status.
        setTimeout( () => {
          new XhrRequest( "BACKUP.CREATE.STATUS", null, ( response ) => {
            this.XhrResponse( response );
          } );
        }, 10 );
      }
      else if( response['data']['status'] == "FINISHED" )  {

        // Now download the file from server's backup directory.
        var fileName = response['data']['file'];
        var filePath = "download.php?file=" + fileName;
        Application.Instance().DownloadFile( filePath );

      }

    }

    //-- BACKUP.INVESTIGATE ---------------------------------
    else if( command == "BACKUP.INVESTIGATE" )  {

      if( response['data']['valid'] )  {

        // Server says uploaded file is a valid backup.
        // So show uploaded file's name + enable Restore button.
        $( "#ID_RESTORE_FILE" + this.dynid ).edit( "option", "placeholder", this.uploadedFile );
        $( "#ID_RESTORE_BUTTON" + this.dynid ).button( "option", "status", "red" );

      }
      else  {
        // Server says uploaded file is NOT a valid backup. 
        // So show error info.
        this.ToastShowError( "#ID_TOAST" + this.dynid, "{$backup_restore_file_corrupted}", this.uploadedFile );
        $( "#ID_RESTORE_FILE" + this.dynid ).edit( "option", "placeholder", "{$backup_restore_file_placeholder}" );
      }

    }

    //-- BACKUP.RESTORE -------------------------------------
    else if( command == "BACKUP.RESTORE" )  {
      // Give the user some visible response on his restore request.
      if( response['meta']['status'].search( "200" ) >= 0  )  {  // like "200 - OK"
        $( "#ID_RESTORE_BUTTON" + this.dynid ).button( "option", "text", "{$backup_restore_button_exec}" );
      }
    }

  }


  //===========================================================================
  //  TOAST
  //===========================================================================

  ToastShowError( selector, messageLangId, filename )
  {
    let errorText = $.lang( messageLangId );
    errorText = errorText.replace( "{file}", filename );

    this.$toast = $( selector ).toast( {
      text: errorText,
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormBackup.Register( FormBackup.viewableTemplate );
PageMain.RegisterForm( "backup", FormBackup.viewableTemplate, FormBackup.documentTitle, "backup" );
//=============================================================================
//
//  Logfile Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          19.07.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormSearch extends FormBase
{
  //---------------------------------------------------------------------------
  //  STATIC
  //---------------------------------------------------------------------------

  static viewableTemplate = "/forms/system/searchForm";   // vw template
  static documentIcon = "mdi mdi-magnify";   // icon for toolbar & doc
  static documentTitle = "{$search_document_title}";  // title
  static helpFile = "search";
  
  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createData )
  {
    //-----------------------------------------------------
    //  controls
    //-----------------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    $( '#ID_TOOLBAR' + this.dynid ).toolbar( {
      size: -1,
      icon: FormSearch.documentIcon,
      text: FormSearch.documentTitle,
      data: [
        //{ id: 'ID_REFRESH', icon: 'mdi mdi-refresh', status: "normal", tooltip: "{$const_form_tooltip_refresh}" },
        //{ id: 'ID_DELETE', icon: 'mdi mdi-delete', status: "red", tooltip: "{$const_form_tooltip_delete}" },        
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );

    // grid
    $( '#ID_SEARCH_GRID' + this.dynid ).grid( {
      size: -1,   // same as "100%"
      height: 25,
      selection: 'click',
      data: Search.Instance().Dataset(),
      dataId: 'guid',
      scrollbar: 'static', // 'overlay', 'static'
      regkey: 'search',
      //sort: 'name',
      //sortOrder: '+',
      header: {
        'type':      { text: "", size: 4,    align: "center",  resizeable: false,   sortable: true, icon: "mdi mdi-checkbox-multiple-blank-circle-outline" },    
        'name':      { text: "{$search_header_entid}",   size: -6,  align: "left",  resizeable: true, sortable: true, inplace: "link" },
        'message':   { text: "{$search_header_message}", size: -22, align: "left",  resizeable: true, sortable: true },        
      },
      renderCell: this.ControlGridRenderCell.bind( this ),
      inplaceLink: ( columnId, record ) => {
        PageMain.ShowEntity( record['entid'] );
      }

    } );

    //-----------------------------------------------------
    //  Start
    //-----------------------------------------------------    


  }//load



  //---------------------------------------------------------------------------
  //  CONTROL
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  render cells on grid
  //
  ControlGridRenderCell( columnId, record )
  {
    let icon = "";
    let text = "";

    if( columnId == 'type' ) {
      let entinfo = Data.Instance().Entinfo( record['entid'] );
      icon = entinfo['_preview']['icon'] || "mdi mdi-help";
    }
    else if( columnId == 'name' ) {
      let entinfo = Data.Instance().Entinfo( record['entid'] );
      text = entinfo['name'];
    }
    else if( columnId == 'message' ) {
      text = $.lang( "{$search_result_message}" ) + " ";
      let isFirst = true;
      for( let item of record['message'] ) {
        if( !isFirst ) { text += ", "; }
        text += $.lang( item );
        isFirst = false;
      }
    }
    else {
      text = record[ columnId ];
    }

    return {
      'text': ( typeof text == "string" ) ? text : "",
      'icon': icon
    };
  }

  //---------------------------------------------------------------------------
  //  close form (close button on toolbar)
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( "logfile" );
  }

  //---------------------------------------------------------------------------
  //  toolbar button
  //
  ControlToolbarClick( id )
  {
    switch( id )  {
      case "ID_HELP":     break;
    }   
  }

  //---------------------------------------------------------------------------
  //  DATA
  //---------------------------------------------------------------------------



  
}


//=============================================================================
//  REGISTER
//=============================================================================

FormSearch.Register( FormSearch.viewableTemplate );
PageMain.RegisterForm( "search", FormSearch.viewableTemplate, FormSearch.documentTitle, "search" );
//=============================================================================
//
//  License Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          19.07.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormLicenses extends FormBase
{
  //---------------------------------------------------------------------------
  //  STATIC
  //---------------------------------------------------------------------------

  static viewableTemplate = "/forms/system/licenses";   // vw template
  static documentIcon = "mdi mdi-license";              // icon for toolbar & doc
  static documentTitle = "{$licenses_document_title}";  // title
  static helpFile = "licenses";
  
  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createData )
  {
    //-----------------------------------------------------
    //  controls
    //-----------------------------------------------------

    let toolbarData = [
      { id: 'ID_EXPORT', icon: 'mdi mdi-download', text: "{$licenses_download_label}", status: "normal", tooltip: "{$licenses_download_tooltip}" },        
      //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
    ];

    // groupboxes
    $( '.groupbox' ).groupbox();

    $( "#ID_OPTIONAL" + this.dynid ).optional( {} );

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: FormLicenses.documentIcon,
      text: FormLicenses.documentTitle,
      data: toolbarData,
      click: this.ControlToolbarClick.bind( this ),
    } );


    $( '#ID_DEMOPERIOD' + this.dynid ).edit( { 
      size: CONST.size.L,
      value: "",
      status: "readonly"
    } );

    $( '#ID_SUPPORTCONTRACT' + this.dynid ).edit( { 
      size: CONST.size.L,
      value: "",
      status: "readonly"
    } );    


    $( "#ID_UPLOAD" + this.dynid ).fileupload( {
      height: 4,
      size: CONST.size.XL,
      url: "upload.php",
      text: "{$license_upload}",
      result: this.ControlUploadResult.bind( this )
    } );    

    // grid
    $( '#ID_LICENSES_GRID' + this.dynid ).grid( {
      size: CONST.size.XXL,   // same as "100%"
      height: 12,
      selection: 'click',
      data: [],
      dataId: 'type',
      scrollbar: 'overlay', // 'overlay', 'static'
      regkey: 'licenses',
      sort: 'type',
      sortOrder: '+',
      header: {
        'type':       { text: "{$license_gridheader_type}", size: 16,  align: "left",  resizeable: true, sortable: true },
        'description': { text: "{$license_gridheader_description}", size: 40, align: "left",  resizeable: true, sortable: true },        
        'available':   { text: "{$license_gridheader_available}",   size: 12,  align: "left",  resizeable: false, sortable: true  },
        'engaged':    { text: "{$license_gridheader_engaged}",   size: 12,  align: "left",  resizeable: false, sortable: true },
      },
    } );

    //-----------------------------------------------------
    //  Start
    //-----------------------------------------------------    

    Data.Instance().On( "ENTINFO/UPDATE/menu.licence*", this.etag, this.DataEntinfoUpdateLicense.bind( this ) );
    Data.Instance().On( "DATA/UPDATE/menu.licence", this.etag, this.DataResponseEntity.bind( this ) );
    this.DataRequest();

  }//load


  //---------------------------------------------------------------------------
  //  FORM
  //---------------------------------------------------------------------------

  FormFillControls( )
  {
    if( this.entity['licence']['isDemo'] )  {
      $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_DEMOPERIOD" );
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_SUPPORTCONTRACT" );
      let time = Time.FormatFromUnix( this.entity['licence']['demoExpiryTime'] );
      $( '#ID_DEMOPERIOD' + this.dynid ).edit( "option", "value", time );
    }
    else  {
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_DEMOPERIOD" );
      $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_SUPPORTCONTRACT" );
      $( '#ID_DEMOPERIOD' + this.dynid ).edit( "option", "value", "-" );      
      var timeStr = Time.FormatFromEndOfSupport( this.entity['licence']['endofsupport'] );
      $( '#ID_SUPPORTCONTRACT' + this.dynid ).edit( "option", "value", timeStr );
    }

    this.FormFillLicenseList();

    if( this.entity['licence']['isExpired'] )  {
      this.ToastShowExpired( "#ID_TOAST" + this.dynid );
      let time = Time.FormatFromUnix( this.entity['licence']['demoExpiryTime'] );
      $( '#ID_DEMOPERIOD' + this.dynid ).edit( "option", "value", $.lang( "{$dashboard_info_warning_expired} " + time ) );
      $( '#ID_DEMOPERIOD' + this.dynid ).edit( "option", "status", "red" );
    }
    else if( !this.entity['licence']['isDemo']  &&  !this.entity['licence']['isVersionValid'] )  {
      this.ToastShowInvalidVersion( "#ID_TOAST" + this.dynid );
    }    
    else if( !this.entity['licence']['isFinallyOk'] )  {
      this.ToastShowInvalid( "#ID_TOAST" + this.dynid );
    }
    else {
      this.ToastHide();
    }
  }

  FormFillLicenseList( )
  {
    let licenseList = [];
    let entityLicense = this.entity['licence'];

    for( let key in entityLicense )  {

      let value = entityLicense[ key ];
      let type = null;
      let description = "";
      let available = value['availabeLicenses'];
      let engaged = value['engagedLicenses'];
      let status = "normal";
      if( engaged > available )  {
        status = "red";
      }

      // ---- APR License ----
      if( key == "APR") {
        type = "APR 670-0 ";
        description = $.lang( "{$license_description_apr}" );
      }
      // ---- ALFP License ----
      else if( key == "ALFP") {
        type = "ALFP 270-0";
        description = $.lang( "{$license_description_alfp}" );
      }
      // ---- ALSA License ----
      else if( key == "ALSA") {
        type = "ALSA 270-0";
        description = $.lang( "{$license_description_alsa}" );
      }
      // ---- ALT License ----
      else if( key == "ALT") {
        type = "ALT 270-0";
        description = $.lang( "{$license_description_alt}" );
      }      
      // ---- ALT License ----
      else if( key == "ASC") {
        type = "ASC 170-0";
        description = $.lang( "{$license_description_asc}" );
      }    
      // ---- ALHT License ----
      else if( key == "ASHT") {
        type = "ASHT 170-0";
        description = $.lang( "{$license_description_asht}" );
      }    
      // ---- ALHT License ----
      else if( key == "KNXAnbindung") {
        type = "ALKNX 270-0";
        description = $.lang( "{$license_description_knx}" );
      }   
      // ---- ALHT License ----
      else if( key == "SIPAudio") {
        type = "ALFA 270-0";
        description = $.lang( "{$license_description_sipaudio}" );
      }       
      // ---- ALFC License ----
      else if( key == "ALFC" ) {
        type = "ALFC 270-0";
        description = $.lang( "{$license_description_alfc}" );
      }       
      // ---- ALFM License ----
      else if( key == "ALFM" ) {
        type = "ALFM 270-0";
        description = $.lang( "{$license_description_alfm}" );
      }       
      else {
        type = null;
      }
      
      if( typeof type == "string" ) {
        licenseList.push( {
          type: type, 
          description: description, 
          available: available.toString(),
          engaged: engaged.toString(),
          status: status
        } );
      }

    }

    $( '#ID_LICENSES_GRID' + this.dynid ).grid( "option", "data", licenseList );
    
  }


  //---------------------------------------------------------------------------
  //  CONTROL
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  toolbar button
  //
  ControlToolbarClick( id )
  {
    switch( id )  {
      case "ID_EXPORT":   this.CommandExport(); break;
    }   
  }


  //===========================================================================
  //  UPLOAD CONTROL
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Upload control reports on success or failure.
  //  
  ControlUploadResult( success, filename )
  {
    if( success )  {

      // Vorläufig! 
      this.uploadedFile = filename;

      // Now tell server to activate the uploaded license file.
      new XhrRequest( "LICENSE.ACTIVATE_LICENSE_FILE", { 'fileName': filename }, ( response ) => {
        this.XhrResponse( response );
      } );
    }
    else  {
      // Show error info. Note: Reason for failure may be inacceptable filename or extension.
      this.ToastShowUploadFailed( "#ID_TOAST" + this.dynid, filename );
    }
  }
  

  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Export licenses
  //
  CommandExport()
  {
    // Download license overview xml file.
    Application.Instance().DownloadFile( "exportLicense.php?file=lic.xml" );
  }


  //---------------------------------------------------------------------------
  //  DATA
  //---------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  Do xhr request to get the logfile.
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( "menu.licence" );
    this.StatusLoading();
  }

  //-----------------------------------------------------------------------------
  //  XHR response: license.
  //  
  DataResponseEntity( entity )
  {
    this.entity = entity;

    this.$container.broadcast( "option", "status", null );
    this.FormFillControls();
    this.StatusIdle();
  }

  //-----------------------------------------------------------------------------
  //  license has been changed
  //
  DataEntinfoUpdateLicense( entinfo )
  {
    this.DataRequest();
  }


  //---------------------------------------------------------------------------
  //  VALIDATE
  //---------------------------------------------------------------------------
  
  Validate()
  {
    return 0;
  }

  ValidateOff()
  {
    return 0;
  }


  //===========================================================================
  //  XHR
  //===========================================================================

  XhrResponse( response )
  {
    let command      = response['meta']['command'];
    let responseCode = parseInt( response['meta']['status'] );

    // If error, we are done here.
    if( responseCode != 200 ) {
      return;
    }

    //-- LICENSE.ACTIVATE_LICENSE_FILE ----------------------
    if( command == "LICENSE.ACTIVATE_LICENSE_FILE" )  {

      console.log("ACTIVATE_LICENSE_FILE response", response );

      // OK or not?
      if( response['data']['isFinallyOk'] )  {

        this.ToastShowUploadSucceeded( "#ID_TOAST" + this.dynid, this.uploadedFile );

        // Reload menu.license. 
        this.DataRequest();
      }
      else if( response['data']['isVersionValid'] == false ) {
        this.ToastShowUploadInvalidVersion( "#ID_TOAST" + this.dynid, this.uploadedFile );
      }
      else  {
        this.ToastShowUploadInvalid( "#ID_TOAST" + this.dynid, this.uploadedFile );
      }

    } // ACTIVATE_LICENSE_FILE
  }


  //---------------------------------------------------------------------------
  //  TOAST
  //---------------------------------------------------------------------------

  ToastShowExpired( selector )
  {
    this.$toast = $( selector ).toast( {
      text: "{$license_toast_expired}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }

  ToastShowInvalid( selector )
  {
    this.$toast = $( selector ).toast( {
      text: "{$license_toast_invalid}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }

  ToastShowInvalidVersion( selector )
  {
    this.$toast = $( selector ).toast( {
      text: "{$license_toast_invalid_version}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }


  ToastShowUploadSucceeded( selector, filename )
  {
    let errorPlaceholder = $.lang( "{$license_file_upload_succeeded}" );
    errorPlaceholder = errorPlaceholder.replace( "{file}", filename );

    this.$toast = $( selector ).toast( {
      text: errorPlaceholder,
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }

  ToastShowUploadFailed( selector, filename )
  {
    let errorPlaceholder = $.lang( "{$license_file_upload_failed}" );
    errorPlaceholder = errorPlaceholder.replace( "{file}", filename );

    this.$toast = $( selector ).toast( {
      text: errorPlaceholder,
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }

  ToastShowUploadInvalid( selector, filename )
  {
    let errorPlaceholder = $.lang( "{$license_file_upload_invalid}" );
    errorPlaceholder = errorPlaceholder.replace( "{file}", filename );

    this.$toast = $( selector ).toast( {
      text: errorPlaceholder,
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }

  ToastShowUploadInvalidVersion( selector, filename )
  {
    let errorPlaceholder = $.lang( "{$license_file_upload_invalidversion}" );
    errorPlaceholder = errorPlaceholder.replace( "{file}", filename );

    this.$toast = $( selector ).toast( {
      text: errorPlaceholder,
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }  

  ToastHide()
  {
    if( this.$toast ) {
      this.$toast.toast( "destroy" );
    }
  }
  
}


//=============================================================================
//  REGISTER
//=============================================================================

FormLicenses.Register( FormLicenses.viewableTemplate );
PageMain.RegisterForm( "licenses", FormLicenses.viewableTemplate, FormLicenses.documentTitle, "licenses" );
//=============================================================================
//
//  Modules List Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          19.07.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormRules extends FormBase
{
  //---------------------------------------------------------------------------
  //  STATIC
  //---------------------------------------------------------------------------

  static viewableTemplate = "/forms/system/rules";    // vw template
  static documentIcon = "mdi mdi-puzzle";             // icon for toolbar & doc
  static documentTitle = "{$rules_document_title}";   // title
  
  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    var langId = AccessApplication.Instance().Language();

    this.status = "statusInitialized";
    this.entinfo = Data.Instance().Entinfo( "folder.rules" );

    //-----------------------------------------------------
    //  controls
    //-----------------------------------------------------

    let toolbarData = [
      //{ id: 'ID_EXPORT', icon: 'mdi mdi-download', text: "{$licenses_download_label}", status: "normal", tooltip: "{$licenses_download_tooltip}" },        
      //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
    ];

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: FormRules.documentIcon,
      text: FormRules.documentTitle,
      data: toolbarData,
      click: this.ControlToolbarClick.bind( this ),
    } );

    $( "#ID_UPLOAD" + this.dynid ).fileupload( {
      height: 4,
      size: CONST.size.XL,
      url: "uploadModule.php",
      text: "{$rule_upload_text}",
      result: this.ControlUploadResult.bind( this ),
      start: this.ControlUploadStart.bind( this ),
    } );    

    // grid
    $( '#ID_RULES_GRID' + this.dynid ).grid( {
      size: CONST.size.XXL,
      height: 10,
      selection: 'single',
      data: [],
      dataId: 'id',
      scrollbar: 'overlay', // 'overlay', 'static'
      regkey: 'rulegrid',
      sort: 'text',
      sortOrder: '+',
      header: {
        'text': { text: "{$rule_gridheader_description}", size: 25, align: "left",  resizeable: true, sortable: true },        
        'type': { text: "{$rule_gridheader_type}", size: 20, align: "left", resizeable: false, sortable: true },
        'version':   { text: "{$rule_gridheader_version}",   size: 12,  align: "left",  resizeable: false, sortable: true  },
        'license':   { text: "Lizenzen",   size: 10,  align: "left",  resizeable: false, sortable: true  }
      },
      renderCell: this.ControlGridRenderCell.bind( this ),          
      result: this.ControlRulesSelect.bind( this ),
      renderStatus: this.ControlGridRenderStatus.bind( this )
    } );

    $( '#ID_RULE_DELETE' + this.dynid ).button( {
      size: CONST.size.M,
      text: "{$rule_button_delete}",
      status: "disabled",
      click: this.ControlRuleDeleteClick.bind( this )
    } );

    //-----------------------------------------------------
    //  Start
    //-----------------------------------------------------    

    Data.Instance().On( "MODULES/UPDATED/", this.etag, this.DataModulesUpdated.bind( this ) );
    this.DataRequest();

  }//load


  //---------------------------------------------------------------------------
  //  FORM
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  fill rule list into grid
  //
  FormFillControls( )
  {
    var version = Data.Instance().GetVersion( true );
    var moduleList = Data.Instance().RuleList();
    var ruleList = [];
    
    // used module list
    for( var rule of moduleList ) {
      ruleList.push( {
        id: rule['id'],
        entid: rule['entid'],
        icon: rule['icon'] || 'mdi mdi-puzzle-outline',
        text: rule['label'],
        type: "rule",
        version: rule['version'],
        active: true,
        license: rule['license']['ALFM'] ? rule['license']['ALFM'] : 0,
        error: rule['error'],
        'packet-minversion': rule['packet-minversion']
      } );
    }

    var widgetList = WidgetDeviceConfigFactory.WidgetList( "function" );
    for( var widget of widgetList ) {
      ruleList.push( {
        id: widget['id'],
        entid: null,
        icon: widget['icon'] || 'mdi mdi-puzzle-outline',
        text: widget['text'],
        type: "widget",
        version: version, //rule['version'],
        active: true,
        license: 0,
        error: false
      } );
    }

    $( '#ID_RULES_GRID' + this.dynid ).grid( "option", "data", ruleList );

  }

  
  //---------------------------------------------------------------------------
  //  CONTROL
  //---------------------------------------------------------------------------

  ControlGridRenderCell( columnId, record )
  {
    var icon = "";
    var text = "";

    if( columnId == 'text' ) {
      icon = record['icon'] || 'mdi mdi-puzzle-outline';
      text = record[ columnId ];
    }
    else if( columnId == 'created' ) {
      if( record['created'] > 0 ) {
        text = Time.FormatFromUnix( record['created'] );
      }
      else {
        text = "-";
      }
    }
    else if( columnId == 'type' ) {
      text = ( record['type'] == "rule" ) ? "{$rule_type_module}" : "{$rule_type_core}";
    }
    else {
      text = record[ columnId ].toString();
    }

    if( record['error'] == "license" ) {
      return {
        'text': ( typeof text == "string" ) ? text : "",
        'icon': icon,
        'tooltip': $.lang( "{$rule_error_update}" )
      };
    }
    else if( record['error'] == "version" ) {
      var versionNeeded = record['packet-minversion'];
      var tooltip = $.lang( "{$rule_error_version}" );
      tooltip = tooltip.replace( "{{version}}", versionNeeded );
      return {
        'text': ( typeof text == "string" ) ? text : "",
        'icon': icon,
        'tooltip': tooltip
      };
    }

    return {
      'text': ( typeof text == "string" ) ? text : "",
      'icon': icon
    };
  }

  //---------------------------------------------------------------------------
  //  error status of control
  //
  ControlGridRenderStatus( id, record )
  {
    if( record['error'] !== false ) {
      return "red striked";
    }
  }


  //---------------------------------------------------------------------------
  //  delete a rule
  //
  ControlRuleDeleteClick( id )  
  {
    var selected = $( '#ID_RULES_GRID' + this.dynid ).grid( "option", "selected" );
    var gridDataset = $( '#ID_RULES_GRID' + this.dynid ).grid( "option", "dataset" );
    var selectedRecord = gridDataset.Get( selected[0] );
    if( selectedRecord && selectedRecord['type'] == "rule" ) {
      var modDel = new ModuleDeleter( ( isOk, param ) => {
        if( isOk && param == "restart:service" ) {
          setTimeout( () => {
            this.ToastRebootRequired();
            AccessApplication.Instance().Command( 'restart-service' );
          }, 100 );
        }
      } );
      modDel.Start( selectedRecord['entid'] );
      
    }
  }

  //---------------------------------------------------------------------------
  //  a rule/widget is selected
  //
  ControlRulesSelect( selected )
  {
    if( selected && selected.length )  {
      var gridDataset = $( '#ID_RULES_GRID' + this.dynid ).grid( "option", "dataset" );
      var selectedRecord = gridDataset.Get( selected[ 0 ] );
      if( selectedRecord && selectedRecord['type'] == "rule" )  {
        $( '#ID_RULE_DELETE' + this.dynid ).button( "option", "status", "normal" );
        return;
      }
    }

    $( '#ID_RULE_DELETE' + this.dynid ).button( "option", "status", "disabled" );

  }

  
  //---------------------------------------------------------------------------
  //  toolbar button
  //
  ControlToolbarClick( id )
  {
  }

  //---------------------------------------------------------------------------
  //  Upload control starts
  //  
  ControlUploadStart( status )
  {
    this.moduleLoader = new ModuleLoader( ( isOk, param ) => {
      if( isOk && param == "restart:service" ) {
        setTimeout( () => {
          this.ToastRebootRequired();
          AccessApplication.Instance().Command( 'restart-service' );
        }, 100 );
      }
    } );
  }

  //---------------------------------------------------------------------------
  //  Upload control reports on success or failure.
  //  
  ControlUploadResult( success, filename, responseHeaders )
  {
    // get x-access-module-info header
    var xModuleInfo = responseHeaders['x-access-module-info'];
    xModuleInfo = xModuleInfo.split( ";" );
    var moduleInfo = {};
    for( var line of xModuleInfo ) {
      if( line == "" )  continue;
      line = line.split( "=" );
      line[ 0 ] = line[ 0 ].trim();
      line[ 1 ] = line[ 1 ].trim();
      moduleInfo[ line[ 0 ] ] = line[ 1 ];
    }

    // if this header is not available, use filename instead
    if( !( 'packet-id' in moduleInfo ) ) {
      moduleInfo['packet-id'] = filename.split( "." )[ 0 ];
    }

    this.moduleLoader.Start( moduleInfo['packet-id'] );
  }  

  //---------------------------------------------------------------------------
  //  MODE
  //---------------------------------------------------------------------------

  /*ModeError()
  {
    this.mode = "modeError";

    $( "#ID_UPLOAD" + this.dynid ).fileupload( "option", "status", "disabled" );
    $( '#ID_RULES_GRID' + this.dynid ).grid(  "option", "status", "disabled" );
  }*/

  //---------------------------------------------------------------------------
  //  DATA
  //---------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  Do xhr request to get the logfile.
  //
  DataRequest()
  {
    this.StatusLoading();
    this.FormFillControls();
    this.StatusLoadFinished();
    this.StatusIdle();

    /*var serverVersion = Data.Instance().GetVersion();
    if( serverVersion.substr( 0, 1 ) != "7" ) {
      this.ToastShowVersion6x();
      this.ModeError();
    }*/
  }

  //-----------------------------------------------------------------------------
  //  XHR response: license.
  //  
  DataModulesUpdated( )
  {
    this.FormFillControls();
  }


  //---------------------------------------------------------------------------
  //  VALIDATE
  //---------------------------------------------------------------------------
  
  Validate()
  {
    return 0;
  }

  ValidateOff()
  {
    return 0;
  }


  //===========================================================================
  //  XHR
  //===========================================================================



  //---------------------------------------------------------------------------
  //  TOAST
  //---------------------------------------------------------------------------

  ToastShowExpired( selector )
  {
    this.$toast = $( selector ).toast( {
      text: "{$license_toast_expired}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }

  ToastShowInvalid( selector )
  {
    this.$toast = $( selector ).toast( {
      text: "{$license_toast_invalid}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }

  ToastShowUploadSucceeded( selector, filename )
  {
    let errorPlaceholder = $.lang( "{$license_file_upload_succeeded}" );
    errorPlaceholder = errorPlaceholder.replace( "{file}", filename );

    this.$toast = $( selector ).toast( {
      text: errorPlaceholder,
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }

  ToastShowUploadFailed( selector, filename )
  {
    let errorPlaceholder = $.lang( "{$license_file_upload_failed}" );
    errorPlaceholder = errorPlaceholder.replace( "{file}", filename );

    this.$toast = $( selector ).toast( {
      text: errorPlaceholder,
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }

  ToastShowUploadInvalid( selector, filename )
  {
    let errorPlaceholder = $.lang( "{$license_file_upload_invalid}" );
    errorPlaceholder = errorPlaceholder.replace( "{file}", filename );

    this.$toast = $( selector ).toast( {
      text: errorPlaceholder,
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }

  ToastHide()
  {
    if( this.$toast ) {
      this.$toast.toast( "destroy" );
    }
  }

  ToastRebootRequired()
  {
    $( "#ID_TOAST" + this.dynid ).toast( {
      text: "{$network_toast_unsaved_notice}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h",
      click: ( id ) =>
      {
        if( id == "ID_SAVE_UNSAVED" ) {
          AccessApplication.Instance().Command( 'restart-service' );
        }
      },
    } );
  }  
  
}


//=============================================================================
//  REGISTER
//=============================================================================

FormRules.Register( FormRules.viewableTemplate );
PageMain.RegisterForm( "rules", FormRules.viewableTemplate, FormRules.documentTitle, "rules" );
//=============================================================================
//
//  Systemtime Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          28.09.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormSystemtime extends FormBase
{
  //---------------------------------------------------------------------------
  //  STATIC
  //---------------------------------------------------------------------------

  static viewableTemplate = "/forms/system/systemtime";   // vw template
  static documentIcon = "mdi mdi-clock-outline";         // icon for toolbar & doc
  static documentTitle = "{$systemtime_document_title}";  // title
  static helpFile = "systemtime";
  
  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = { entid: "systemtime" };
    this.cosconfData = null;
    this.cosconfSelectedZone = null;
    this.tzData = null;
    this.timerId = 0;
    this.isSystemForm = true;

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();
    $( "#ID_OPTIONAL" + this.dynid ).optional( {} );

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: FormSystemtime.documentIcon,
      text: FormSystemtime.documentTitle,      
      data: [
        { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
        { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
        //{ id: 'ID_RESTART_SERVICES', icon: 'mdi mdi-power', text: '{$const_menu_restart_service_short}', status: 'red' },
        { id: 'ID_REBOOT', icon: 'mdi mdi-desktop-tower-monitor', text: '{$const_menu_restart_service_short}', status: 'red' },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
    } );

    $( "#ID_SRVTIME" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly",
      value: "wird geladen..."
    });

    // --- time ---
    this.timeOffset = Data.Instance().TimeOffset();
    this.timeOffset = ( Math.abs( this.timeOffset ) == 1 ) ? 0 : this.timeOffset;
    this.browserTimeZone = moment.tz.guess();

    var localTime = moment.tz( new Date(), "utc" );
    var localTimeString = moment.tz( localTime, this.browserTimeZone ).format( "L" ) + "  " + moment.tz( localTime, this.browserTimeZone ).format( "LTS" );

    $( "#ID_CLIENTTIME" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly",
      value: localTimeString
    } );

    /*this.ntpRegionList = new Dataset( { id: "id" } );
    $( "#ID_REGION" + this.dynid ).listbox( {
      size: CONST.size.L,
      height: 7,
      type: 'droplist',
      selected: [],
      sort: "+",
      sortKey: "id",
      placeholder: "{$const_common_empty_placeholder}",
      data: this.ntpRegionList,
      result: this.ControlRegionSelected.bind( this )
    } );

    this.ntpZoneList = new Dataset( { id: "id" } );
    $( "#ID_ZONE" + this.dynid ).listbox( {
      size: CONST.size.L,
      height: 7,
      type: 'droplist',
      placeholder: "{$const_common_empty_placeholder}",
      searchPlaceholder: "{$const_common_search_placeholder}",
      selected: [],
      search: true,
      sort: "+",
      sortKey: "id",
      data: this.ntpZoneList,
      validate: this.ValidateZone.bind( this ),
      result: this.ControlZoneSelected.bind( this )
    } );*/

    this.ntpZoneListNew = new Dataset( { id: "id" } );
    $( "#ID_ZONE_NEW" + this.dynid ).listbox( {
      size: CONST.size.XL,
      height: 10,
      type: 'droplist',
      placeholder: "{$const_common_empty_placeholder}",
      searchPlaceholder: "{$const_common_search_placeholder}",
      selected: [],
      search: true,
      data: this.ntpZoneListNew,
      validate: this.ValidateZone.bind( this ),
      result: this.ControlZoneSelectedNew.bind( this )
    } );

    $( "#ID_TIMEMODE" + this.dynid ).choice( {
      size: CONST.size.XL,
      type: "radio",
      value: "automatic",
      data: [
        { id: "automatic", text: "{$systemtime_timemode_automatic}" },
        { id: "manual", text: "{$systemtime_timemode_manual}" },
      ],
      result: this.ControlTimemodeResult.bind( this )
    });

    $( "#ID_TIME_SEND" + this.dynid ).button( {
      size: CONST.size.L,
      status: "normal",
      text: "{$systemtime_timesend_button}",
      click: this.ControlTimesendClick.bind( this )
    } );
    


    this.ntpList = new Dataset( { id: "id" } );
    $( "#ID_NTP_LIST" + this.dynid ).grid( {
      size: CONST.size.XL,
      height: 7,
      selected: [],
      data: this.ntpList,
      scrollbar: "overlay",
      headerHidden: true,
      header: {
        text: { text: "", size: CONST.size.L }
      },
      result: this.ControlNtpListSelect.bind( this ),
      dblclick: this.ControlNtpEditClick.bind( this )
    } );

    $( "#ID_NTP_ADD" + this.dynid ).button( {
      size: CONST.size.M,
      status: "normal",
      text: "{$systemtime_button_add}",
      click: this.ControlNtpAddClick.bind( this )
    } );

    $( "#ID_NTP_EDIT" + this.dynid ).button( {
      size: CONST.size.M,
      status: "disabled",
      text: "{$systemtime_button_edit}",
      click: this.ControlNtpEditClick.bind( this )
    } );

    $( "#ID_NTP_DELETE" + this.dynid ).button( {
      size: CONST.size.XS,
      status: "disabled",
      icon: "mdi mdi-delete",
      click: this.ControlNtpDeleteClick.bind( this )
    } );

    $( "#ID_NTP_UP" + this.dynid ).button( {
      size: CONST.size.XS,
      status: "disabled",
      icon: "mdi mdi-chevron-up",
      click: this.ControlNtpUpClick.bind( this )
    } );

    $( "#ID_NTP_DOWN" + this.dynid ).button( {
      size: CONST.size.XS,
      status: "disabled",
      icon: "mdi mdi-chevron-down",
      click: this.ControlNtpDownClick.bind( this )
    } );

    //-------------------------------------------
    //  Start 
    //-------------------------------------------

    Data.Instance().On( "COSCONF.READ", this.etag, this.DataCosconfReadResponse.bind( this ) );
    Data.Instance().On( "COSCONF.WRITE", this.etag, this.DataCosconfWriteResponse.bind( this ) );

    // request data
    this.xhrResponseCount = 0;
    this.DataRequest();
    this.StatusLoading();


    // update time once a second
    clearInterval( this.timerId );
    this.timerId = setInterval( () => {
      this.FormUpdateTimes();
    }, 1000 );


  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Fill entity into controls
  //
  FormReadControls()
  {
    // set new zone
    //let newZoneId = $( "#ID_ZONE" + this.dynid ).listbox( "option", "selected" )[ 0 ];
    //var newZone = this.ntpZoneList.Get( newZoneId );

    let newZoneId = $( "#ID_ZONE_NEW" + this.dynid ).listbox( "option", "selected" )[ 0 ];
    this.cosconfData['timezone']['id'] = newZoneId;

    // manual mode, no ntp
    if( this.mode == "modeTimeManual" )  {
      this.cosconfData['ntp']['pool'] = [];
    }
    // set new ntp pool
    else {
      var ntpData = this.ntpList.GetData();
      var ntpArray = [];
      for( let rec of ntpData ) {
        ntpArray.push( rec['text'] );
      }
      this.cosconfData['ntp'] = {};
      this.cosconfData['ntp']['pool'] = ntpArray;
    }

  }

  //---------------------------------------------------------------------------
  //  Fill entity into controls
  //
  FormFillControls()
  {
    this.FormUpdateTimes();
    this.FormBuildNtpList();
    this.FormFillCosconfState();

    if( this.ntpList.Size() == 0 )  {
      $( "#ID_TIMEMODE" + this.dynid ).choice( "option", "value", "manual" );
      this.ModeTimeManual();
    }
    else {
      $( "#ID_TIMEMODE" + this.dynid ).choice( "option", "value", "automatic" );
      this.ModeTimeAutomatic();
    }

    // ntp status
    if( ( 'ntp' in this.cosconfData ) && ( 'statistic' in this.cosconfData['ntp'] ) )  {
      this.FormFillNtpStatus();
    }

    var zoneDescription = this.HelperGetZoneDescriptionNew( this.cosconfSelectedZone );
    if( this.cosconfSelectedZone )  {
      $( "#ID_SRVTIME_POST" + this.dynid ).text( zoneDescription );
      $( "#ID_ZONE_NEW" + this.dynid ).listbox( "option", "selected", [ this.cosconfSelectedZone ] );
    }
    else {
      $( "#ID_SRVTIME_POST" + this.dynid ).text( "" );
      $( "#ID_ZONE_NEW" + this.dynid ).listbox( "option", "selected", [] );
    }

    zoneDescription = this.HelperGetZoneDescriptionNew( this.browserTimeZone );
    $( "#ID_CLIENTTIME_POST" + this.dynid ).text( zoneDescription );
    
  }  

  //---------------------------------------------------------------------------
  //  manage toast for reboot
  //
  FormFillCosconfState()
  {
    switch( this.cosconfData['status'] )  {
      case "REBOOT-REQUIRED":
      case "REBOOT_REQUIRED":
        this.ToastRebootRequired();
        break;

      default:
        this.ToastDestroy();

    }
  }

  //---------------------------------------------------------------------------
  //  show ntp statistics
  //  
  FormFillNtpStatus()
  {
    /* clksource: 6
    ntpip: "192.168.0.10"
    polling: 128
    status: "SYNC"
    stratum: "3"
    tmcorrect: 107.426 */
    var statistic = this.cosconfData['ntp']['statistic'];

    var ln1 = "";
    if( 'status' in statistic )  {
      switch( statistic['status'] ) {
        case "SYNC":    
          ln1 = $.lang( "{$systemtime_ntpstat_status_sync}");     
          ln1 = ln1.replace( "{{ntpip}}", statistic['ntpip'] );
          ln1 = ln1.replace( "{{stratum}}", statistic['stratum'] );
          break;

        case "UNSYNC":  ln1 = $.lang( "{$systemtime_ntpstat_status_unsync}");   break;
        case "UNKNOWN":
        default:        ln1 = $.lang( "{$systemtime_ntpstat_status_unknown}");  break;
      }
    }
    $( "#ID_NTP_STATISTIC" + this.dynid ).text( ln1 );

    var ln2 = $.lang( "{$systemtime_ntpstat_info_updates}" );
    ln2 = ln2.replace( "{{polling}}", statistic['polling'] );
    ln2 = ln2.replace( "{{tmcorrect}}", statistic['tmcorrect'] );
    $( "#ID_NTP_STATISTIC2" + this.dynid ).text( ln2 );


  }

  //---------------------------------------------------------------------------
  //  Fill times into controls (called by interval timer)
  //
  FormUpdateTimes()
  {
    var localTime = moment.tz( new Date(), "utc" );
    var localTimeString = moment.tz( localTime, this.browserTimeZone ).format( "L" ) + "  " + moment.tz( localTime, this.browserTimeZone ).format( "LTS" );
    $( "#ID_CLIENTTIME" + this.dynid ).edit( "option", "value", localTimeString );

    if( this.cosconfSelectedZone )  {
      var serverTime = moment.tz( new Date(), "utc" ).subtract( this.timeOffset, 's' );
      var serverTimeString = moment.tz( serverTime, this.cosconfSelectedZone ).format( "L" ) + "  " + moment.tz( serverTime, this.cosconfSelectedZone ).format( "LTS" );
      $( "#ID_SRVTIME" + this.dynid ).edit( "option", "value", serverTimeString );
    }
    else {
      $( "#ID_SRVTIME" + this.dynid ).edit( "option", "value", "-" );
    }

  }


  //-----------------------------------------------------------------------------
  //  build ntp list dataset
  //
  FormBuildNtpList()
  {
    if( !this.cosconfData ) {
      return;
    }

    this.ntpList.SetData( [] );
    var ntpList = this.cosconfData['ntp']['pool'];
    if( ntpList )  {
      for( let entry of ntpList ) {
        this.ntpList.Add( {
          id: $.uuid(),
          text: entry
        } );
      }
    }
  }

  //===========================================================================
  //  CONTROL
  //===========================================================================

  //---------------------------------------------------------------------------
  //  toolbar button
  //
  ControlToolbarClick( id )
  {
    switch( id )  {
      case "ID_UNDO":             this.CommandUndo(); break;
      case "ID_SAVE":             this.CommandSave(); break;
      case 'ID_RESTART_SERVICES': this.CommandRestartServices();  break;
      case 'ID_REBOOT':           this.CommandReboot();  break;
    }
  }

  //---------------------------------------------------------------------------
  //  edit ntp server
  //
  ControlNtpEditClick()
  {
    var selected = $( "#ID_NTP_LIST" + this.dynid ).grid( "option", "selected" );
    if( selected.length == 0 ) {
      return;
    }

    var ntpRecord = this.ntpList.Get( selected[ 0 ] );

    NtpEntryDialog.Show( {
      entry: ntpRecord,
      result: ( record ) => {
        this.Option( "modified", 1 );
        if( this.ntpList.Has( record['id'] ) ) {
          this.ntpList.Update( record );
        }
        else {
          this.ntpList.Add( record );
        }
      }
    });

  }

  //---------------------------------------------------------------------------
  //  add ntp server
  //
  ControlNtpAddClick()
  {
    NtpEntryDialog.Show( {
      entry: null,
      result: ( record ) => {
        this.ntpList.Add( record );
        this.Option( "modified", 1 );
      }
    } );
  }

  //---------------------------------------------------------------------------
  //  delete a ntp server
  //
  ControlNtpDeleteClick()
  {
    var selected = $( "#ID_NTP_LIST" + this.dynid ).grid( "option", "selected" );
    if( selected.length == 0 ) {
      return;
    }

    this.ntpList.Delete( selected[ 0 ] );
    this.Option( "modified", 1 );
  }

  //---------------------------------------------------------------------------
  //  move ntp server up
  //
  ControlNtpUpClick()
  {
    var selected = $( "#ID_NTP_LIST" + this.dynid ).grid( "option", "selected" );
    if( selected.length == 0 ) {
      return;
    }
    var ntpIndex = this.ntpList.GetIndex( selected[ 0 ] );
    if( ntpIndex <= 0 ) {
      return;
    }

    var ntpData = this.ntpList.GetData();
    var swapRecord = ntpData[ ntpIndex - 1 ];
    ntpData[ ntpIndex - 1 ] = ntpData[ ntpIndex ];
    ntpData[ ntpIndex ] = swapRecord;

    this.ntpList.SetData( ntpData );
    this.ControlNtpListSelect();
    this.Option( "modified", 1 );

  }  

  //---------------------------------------------------------------------------
  //  move ntp server up
  //
  ControlNtpDownClick()
  {
    var selected = $( "#ID_NTP_LIST" + this.dynid ).grid( "option", "selected" );
    if( selected.length == 0 ) {
      return;
    }
    var ntpIndex = this.ntpList.GetIndex( selected[ 0 ] );
    if( ntpIndex < 0 || ntpIndex >= ( this.ntpList.Size() - 1 ) ) {
      return;
    }

    var ntpData = this.ntpList.GetData();
    var swapRecord = ntpData[ ntpIndex + 1 ];
    ntpData[ ntpIndex + 1 ] = ntpData[ ntpIndex ];
    ntpData[ ntpIndex ] = swapRecord;

    this.ntpList.SetData( ntpData );
    this.ControlNtpListSelect();
    this.Option( "modified", 1 );

  }

  //---------------------------------------------------------------------------
  //  new zone selected
  //
  ControlZoneSelectedNew( selected )
  {
    if( !selected || selected.length == 0 ) {
      $( "#ID_ZONE_NEW" + this.dynid ).listbox( "option", "selected", [ this.cosconfSelectedZone ] );
      return;
    }

    let zoneData = this.ntpZoneListNew.Get( selected[ 0 ] );
    this.cosconfSelectedZone = zoneData['id'];
    let zoneDescription = this.HelperGetZoneDescriptionNew( zoneData['id'] );
    $( "#ID_SRVTIME_POST" + this.dynid ).text( zoneDescription );
  }

  //---------------------------------------------------------------------------
  //  a ntp server is selected
  //
  ControlNtpListSelect( )
  {
    var selected = $( "#ID_NTP_LIST" + this.dynid ).grid( "option", "selected" );
    if( selected.length == 0 )
      return;

    var recordIndex = this.ntpList.GetIndex( selected[ 0 ] );

    $( "#ID_NTP_EDIT" + this.dynid ).button( "option", "status", "normal" );
    $( "#ID_NTP_DELETE" + this.dynid ).button( "option", "status", "normal" );

    if( recordIndex == 0 )  {
      $( "#ID_NTP_UP" + this.dynid ).button( "option", "status", "disabled" );
      $( "#ID_NTP_DOWN" + this.dynid ).button( "option", "status", "normal" );
    }
    else if( recordIndex < ( this.ntpList.Size() - 1 ) ) {
      $( "#ID_NTP_UP" + this.dynid ).button( "option", "status", "normal" );
      $( "#ID_NTP_DOWN" + this.dynid ).button( "option", "status", "normal" );
    }
    else {
      $( "#ID_NTP_UP" + this.dynid ).button( "option", "status", "normal" );
      $( "#ID_NTP_DOWN" + this.dynid ).button( "option", "status", "disabled" );
    }

  }

  //---------------------------------------------------------------------------
  //  timemode changed
  //
  ControlTimemodeResult( result )
  {
    if( result == "manual" )  {
      this.ModeTimeManual();
    }
    else {
      this.ModeTimeAutomatic();
    }
  }

  //---------------------------------------------------------------------------
  //  send current browser time to server
  //
  ControlTimesendClick()
  {
    let currentTime = moment().unix();
    new XhrRequest( "COSCONF.SET-TIME", { 'time': currentTime }, ( response ) => {
      
      if( 'time' in response['data'] )  {
        Data._timeDifference = moment( new Date() ).unix() - response['data']['time'];
        this.timeOffset = Data._timeDifference;
        this.timeOffset = ( Math.abs( this.timeOffset ) == 1 ) ? 0 : this.timeOffset;
      }
      
    } );
  }


  //===========================================================================
  //  MODE
  //===========================================================================

  //---------------------------------------------------------------------------
  // timemode auto
  //
  ModeTimeAutomatic()
  {
    this.mode = "modeTimeAutomatic";
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTION_NTP" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTION_MANUAL" );
  }

  //---------------------------------------------------------------------------
  // timemode auto
  //
  ModeTimeManual()
  {
    this.mode = "modeTimeManual";
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTION_NTP" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTION_MANUAL" );
  }


  //===========================================================================
  //  COMMAND
  //===========================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = this.Validate();

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  CommandUndo(): undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }

  //-----------------------------------------------------------------------------
  //  restarts the cosconf and access service 
  //
  CommandRestartServices()
  {
    AccessApplication.Instance().Command( 'restart-service' );
  }

  CommandReboot()
  {
    AccessApplication.Instance().Command( 'reboot' );
  }


  //===========================================================================
  //  DATA
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  do xhr read requests
  //
  DataRequest()
  {
    Data.Instance().XhrRequest( "COSCONF.READ" );
    
    // REM: The timezone database was taken from this project, https://github.com/vvo/tzdb 
    //      after an update of IANA database this file need to be updated manually!
    $.getJSON( "tools/raw-time-zones.json", this.DataTimezonesResponse.bind( this ) );

    this.StatusLoading();
  }

  //-----------------------------------------------------------------------------
  //  response cosconf read
  //
  DataTimezonesResponse( data )
  {
    var tzList = [];
    for( var record of data ) {
      tzList.push( {
        id: record['name'],
        text: record['rawFormat']
      } );
    }
    this.ntpZoneListNew.SetData( tzList );

    this.xhrResponseCount++;
    if( this.xhrResponseCount >= 2 ) {
      this.StatusLoadFinished();
      this.FormFillControls();
    }
  }

  //-----------------------------------------------------------------------------
  //  response cosconf read
  //
  DataCosconfReadResponse( response )
  {
    this.xhrResponseCount++;
    this.cosconfData = response['data'];
    this.cosconfSelectedZone = this.cosconfData['timezone']['id'];

    if( this.xhrResponseCount >= 2 ) {
      this.StatusLoadFinished();
      this.FormFillControls();
    }
  }

  //-----------------------------------------------------------------------------
  //  write finished
  //
  DataCosconfWriteResponse( response )
  {
  }

  //-----------------------------------------------------------------------------
  //  save to cosconf
  //
  DataSaveRequest()
  {
    // send cosconf-write
    Data.Instance().XhrRequest( "COSCONF.WRITE", this.cosconfData );
    this.StatusSaveRequest();

    // base-function: reload entity after save
    this.xhrResponseCount = 0;
    this.DataReloadAll();
  }


  //===========================================================================
  //  HELPER
  //===========================================================================

  HelperGetZoneDescriptionNew( zoneId )
  {
    if( !zoneId ) return;

    let zone = moment.tz( new Date(), zoneId );
    let zoneDst = zone.isDST() ? " (DST)" : "";
    return ( zone.format( 'Z z' ) + zoneDst + ", " + zoneId );
  }


  //-----------------------------------------------------------------------------
  //  find text to timezone id
  //
  HelperFindTimezone( tzId )
  {
    for( let region in this.tzData ) {
      var regionData = this.tzData[region];
      for( let zone in regionData ) {
        if( regionData[zone] instanceof Object ) {
          for( let subzone in regionData[zone] ) {
            if( regionData[zone][subzone] == tzId ) {
              return { region: region, zone: subzone };
            }
          }
        }
        else if( regionData[zone] == tzId ) {
          return { region: region, zone: zone };
        }
      }
    }

    return { region: "", zone: tzId };
  }


  //===========================================================================
  //  VALIDATION
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  Validate()
  {
    return 0;
  }

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  validate zone list
  //
  ValidateZone( validateData )
  {
    if( !Array.isArray( validateData.value ) || validateData.value.length <= 0 ) {
      validateData.error = "{$wizpage_timezone_errorempty}";
    }
  }

  //===========================================================================
  //  TOASTS
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  ApplyStageRebootRequired(): description
  //
  ToastRebootRequired()
  {
    $( "#ID_TOAST" + this.dynid ).toast( {
      text: "{$systemtime_toast_unsaved_notice}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h",
      click: ( id ) => {
        this.CommandReboot();
      },
    } );
  }



}


//=============================================================================
//  REGISTER
//=============================================================================

FormSystemtime.Register( FormSystemtime.viewableTemplate );
PageMain.RegisterForm( "systemtime", FormSystemtime.viewableTemplate, FormSystemtime.documentTitle, "systemtime" );
//=============================================================================
//
//  NETWORK Form
//  (c) 2021 by CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          03.10.2020
//
//  Description:
//          Form for network settings
//
//=============================================================================

'use strict';

//=============================================================================
//  CLASS
//=============================================================================

class NetworkForm extends Viewable
{
  //-------------------------------------------------------
  //  LOAD
  //
  Load( data )
  {
    //=============================================================================
    //  DATA
    //=============================================================================
    this.entity             = null;
    this.isModified         = false;
    this.$netcardGroupList  = {};
    this.$netcardWidgetList = {};
    this.currentState       = "EXPERT";
    this.shouldReboot       = false;
    this.isAgw              = typeof AdminConfigHelper !== "undefined";

    //=============================================================================
    //  CONTROLS
    //=============================================================================
    $( '.groupbox' ).groupbox();

    $( '#ID_TOOLBAR' + this.dynid ).toolbar( {
      size: -1,
      icon: 'mdi mdi-lan',
      text: '{$const_menu_network}',
      data: [
        { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_menu_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'disabled' },
        { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_menu_save}', tooltip: "{$const_form_tooltip_save}", status: 'disabled' },
        { id: 'ID_RESTART_SERVICES', icon: 'mdi mdi-power', text: '{$const_menu_restart_service_short}', status: 'red' },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_menu_help}', status: 'hilight' }
      ],
      click: ( id ) => {
        switch( id ) {
          case 'ID_UNDO':
            this.CommandUndo();
            break;
          case 'ID_SAVE':
            this.CommandSave( false);
            break;
          case 'ID_RESTART_SERVICES':
            this.CommandRestartServices();
            break;
        }
      }
    } );

    //=============================================================================
    //  XHR-REQUESTS & EVENTS
    //=============================================================================
    Data.Instance().On( "COSCONF.READ", this.etag, this.OnNetworkList.bind( this ) );
    Data.Instance().On( "COSCONF.WRITE", this.etag, this.OnNetworkSettingsSaved.bind( this ) );
    Data.Instance().On( "COSCONF.RESYNC", this.etag, () => { Data.Instance().XhrRequest( "COSCONF.READ" ); } );
    Data.Instance().On( "BACKUP.RESTART", this.etag, () => { $( '#ID_BODY' ).viewable( { view: "/pages/restart" } ); } );
    Data.Instance().XhrRequest( "COSCONF.READ" );

    //=============================================================================
    //  AGW Special Function don't delete it!!!!!!!!!!!!!!
    //=============================================================================

    if( this.isAgw ) {
      Data.Instance().GetEntity( "config.agw" );

      // read config.agw to change ip addresses
      this.agwConfigHelper = new AdminConfigHelper;
      Data.Instance().On( "DATA/UPDATE/config.agw", this.etag, this.OnDataUpdate.bind( this ) );
    }


  }//load

  //-----------------------------------------------------------------------------
  //  OnDataUpdate(): handles the response from GetEntity()
  //
  OnDataUpdate( entity )
  {
    //this.agwConfigHelper.SetAgwConfig( entity );
    //console.log( "AGW-Config", entity );
  }

  //-----------------------------------------------------------------------------
  //  OnNetworkList(): handles the response from cosconf =>
  //                   1. create widgets for the interfaces
  //                   2. create the dns box
  //
  OnNetworkList( response )
  {
    if( $.isPlainObject( response[ 'data' ] ) ) {
      
      // copy data 
      this.networkList = $.clone( response[ 'data' ] );

      // dhcp-options need host name, so add it to the interface
      for( let index in this.networkList['interfaces'] )  {
        let netInterface = this.networkList['interfaces'][ index ];
        netInterface['hostname'] = this.networkList['system']['hostname'];
      }

      //=============================================================================
      //  AGW Special Function don't delete it!!!!!!!!!!!!!!
      //=============================================================================

      if( this.isAgw ) {
        // put to agwConfig helper
        this.agwConfigHelper.SetCosconfData( this.networkList );
      }

      // draw form
      this.DrawNetworkList( this.networkList );
      this.DrawDnsBox( this.networkList );
      this.ChangeState( response[ 'data' ]['status'] );

      // reset form
      this.Option( "reset" );
      this.Option( "scroll", 0 );
    }
  }

  //-----------------------------------------------------------------------------
  //  OnNetworkSettingsSaved(): handles responses to save network settings
  //
  OnNetworkSettingsSaved( response )
  {
    Data.Instance().XhrRequest( "COSCONF.READ" );
    if( this.shouldReboot === true )  {
      this.shouldReboot = false;
      Data.ClearAuthorized( );
      Data.ClearConnected( );
      $( '#ID_BODY' ).viewable( { view: "/pages/restart" } );
      Application.Instance().CommandRestartService();
    }
  }

  //-----------------------------------------------------------------------------
  //  DrawNetworkList(): draw a widget for each interface
  //
  DrawNetworkList( netlist )
  {
    let $anchor              = $( '#ID_NETCARD_ANCHOR' + this.dynid );
    let currentNetworkValues = "interfaces";
    let backupNetworkValues  = "actual";
    let backupNetcard        = false;

    if( netlist[ 'status' ] === "EXPERT" ) {
      currentNetworkValues = "interfaces";
      backupNetworkValues  = "actual";
    }
    // get all mac addresses and sort the array
    let macList = Object.keys( netlist[ 'interfaces' ] );
    macList.sort( ( a, b ) => netlist[ 'interfaces' ][ a ][ 'name' ].localeCompare( netlist[ 'interfaces' ][ b ][ 'name' ] ) );

    // prepare the group boxes for each interface
    for( let i = macList.length - 1; i >= 0; i-- ) {

      // prepare
      let macAddress = macList[ i ];
      let netCard    = netlist[ currentNetworkValues ][ macAddress ];

      if( backupNetworkValues in netlist ) {
        backupNetcard = netlist[ backupNetworkValues ][ macAddress ];
      }

      this.NormaliseName( netCard, backupNetcard, netlist['status'] );

      // set default name if no name is set
      if( !netCard['name'] ) {
        netCard['name']        = netlist['interfaces'][ macAddress ]['name'] || "interface " + i;
        netCard['displayname'] = netlist['interfaces'][ macAddress ]['displayname'] || netCard['name'];
        netCard['dhcp-server'] = netlist['interfaces'][ macAddress ]['dhcp-server'];
        netCard['macAddress']  = macAddress;
        netCard['prepend']     = netlist['interfaces'][ macAddress ]['prepend'] || [];
        netCard['supersede']   = netlist['interfaces'][ macAddress ]['supersede'] || [];
      }

      if( !netCard['ipv4'] && backupNetworkValues ) {
        netCard['ipv4']     = backupNetcard['ipv4'];
        netCard['ipv4mask'] = backupNetcard['ipv4mask'];
        netCard['gateway']  = backupNetcard['gateway'];
      }

      if( !netCard['dhcp-server'] && backupNetcard ) {
        netCard['dhcp-server'] = backupNetcard['dhcp-server'];
      }

      // create groupbox tag as anchor for widget (create and replace)
      let tempTitleString = "<div></div>";

      if( this.$netcardGroupList[ macAddress ] === undefined ) {
        let $group = $( tempTitleString ).insertAfter( $anchor );
        this.$netcardGroupList[ macAddress ] = $group;
      }
      else {
        this.$netcardGroupList[ macAddress ].html( tempTitleString );
      }

      // create netcard widget
      let $group = this.$netcardGroupList[ macAddress ];

      let $widgetNetcard = $group.viewable( { 
        view: '/forms/system/network/widgets/controls/netcard',
        data: netCard, 
        agwConfigHelper: this.agwConfigHelper, 
        result: ( value ) => {
          this.NetCardChanged( value );
        }
      } );
      this.$netcardWidgetList[ macAddress ] = $widgetNetcard;

    }//for

  }

  //-----------------------------------------------------------------------------
  //  DrawDnsBox(): draws/creates a dns widget box
  //
  DrawDnsBox( response )
  {
    let $anchor = $( '#ID_DNS_ANCHOR' + this.dynid );

    if( this.$dnsContent === undefined ) {
      let $dnsTitle = $( "<div class='groupbox'><h3>" + $.lang( "{$network_dns_box}" ) + "</h3></div>" ).insertAfter( $anchor );
      $dnsTitle.groupbox();
      this.$dnsContent = $dnsTitle.find( '.content' );
    }

    this.$dnsContent.viewable( { view: '/forms/system/network/widgets/controls/dns_config', data: response } );
  }

  //=============================================================================
  //  COMMANDS
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  CommandUndo(): undo current form changes. ( all Widgets will be reset too )
  //
  CommandUndo()
  {
    Data.Instance().XhrRequest( "COSCONF.READ" );
  }

  //-----------------------------------------------------------------------------
  //  CommandSave(): handles the on click save actions
  //
  CommandSave( )
  {
    // prequisites
    let numberOfErrors = 0;

    // apply change dns
    let dnsConfig = this.$dnsContent.viewable( "option", "data" );
    numberOfErrors += this.$dnsContent.viewable( "validate" );
    // apply changes for each interface
    for( let netcardMacAddress in this.$netcardWidgetList ) {
      let $currentNetcard         = this.$netcardWidgetList[ netcardMacAddress ];
      let currentNetcardData      = $currentNetcard.viewable( "option", "data" );
      if( dnsConfig['interfaces'][ netcardMacAddress ][ 'supersede' ] && dnsConfig['interfaces'][ netcardMacAddress ][ 'supersede' ]['6'] )  {
        if( currentNetcardData['type'] === "static" )  {
          currentNetcardData['dns'] = dnsConfig['interfaces'][ netcardMacAddress ][ 'supersede' ]['6'].split( " " );
        }
        else  {
          if( !currentNetcardData['supersede'] )  {
            currentNetcardData['supersede'] = dnsConfig['interfaces'][ netcardMacAddress ][ 'supersede'];
          }
          currentNetcardData['supersede']['6'] = dnsConfig['interfaces'][ netcardMacAddress ][ 'supersede' ]['6'];
        }
      }

      if( !dnsConfig['interfaces'][ netcardMacAddress ][ 'supersede' ] || !dnsConfig['interfaces'][ netcardMacAddress ][ 'supersede' ]['6'] )  {
        delete currentNetcardData['supersede'];
        delete currentNetcardData['dns'];
      }

      numberOfErrors += $currentNetcard.viewable( "validate" );
      this.ApplyChanges( netcardMacAddress, currentNetcardData );
    }

    // after validation succeed write it to cosconf
    if( numberOfErrors === 0 ) {

      // write netork settings
      Data.Instance().XhrRequest( "COSCONF.WRITE", this.networkList );

      // write config.agw to server
      if( this.isAgw ) {
        this.agwConfigHelper.SetCosconfData( this.networkList );
        let configAgwEntity = this.agwConfigHelper.BuildConfig();
        // in case of agw save the entity
        // for the case it is run on access ignore it
        if( configAgwEntity )  {
          Data.Instance().SaveEntity( configAgwEntity );
        }
      }


      // reset: not modified, no error
      this.Option( "reset" );

    }
  }

  //-----------------------------------------------------------------------------
  //  OnEvent(): event handler cwf viewables
  //
  OnEvent( eventName, data )
  {
    // modified handling (data := num modified)
    if( eventName == "modified" ) {
      $( '#ID_TOOLBAR' + this.dynid ).toolbar( "item", "ID_SAVE", "status", ( data > 0 ) ? "green" : "disabled" );
      $( '#ID_TOOLBAR' + this.dynid ).toolbar( "item", "ID_UNDO", "status", ( data > 0 ) ? "normal" : "disabled" );
      //AgwApplication.Instance().Modified( "ID_NETWORK", ( data > 0 ) );

    }
    // error handling (data := num error)
    else if( eventName == "error" && this.Option( "modified" ) > 0 ) {
      $( '#ID_TOOLBAR' + this.dynid ).toolbar( "item", "ID_SAVE", "status", ( data > 0 ) ? "disabled" : "green" );
    }

  }


  //-----------------------------------------------------------------
  //  HIDE FORM (TAB-CHANGE)
  //
  OnTabChangeRequested()
  {
    // prevent change tab if modified
    if( this.Option( "modified" ) > 0 ) {
      $( '#ID_TOAST' + this.dynid ).toast( {
        text: "{$const_toast_savefirst}",
        icon: "mdi mdi-alert-octagon",
        status: "yellow",
        height: 3,
        size: -1,
        time: 3000,
        transition: "h"
      } );
      return false;
    }

    return true;
  }

  //-----------------------------------------------------------------------------
  //  ApplyChanges(): apply changes from netcard widgets
  //
  ApplyChanges( macAddress, netcard )
  {
    // interface options
    this.networkList[ 'interfaces' ][ macAddress ][ 'ipv4' ]        = netcard[ 'ipv4' ];
    this.networkList[ 'interfaces' ][ macAddress ][ 'ipv4mask' ]    = netcard[ 'ipv4mask' ];
    this.networkList[ 'interfaces' ][ macAddress ][ 'gateway' ]     = netcard[ 'gateway' ];
    this.networkList[ 'interfaces' ][ macAddress ][ 'type' ]        = netcard[ 'type' ];
    this.networkList[ 'interfaces' ][ macAddress ][ 'name' ]        = netcard[ 'name' ];
    this.networkList[ 'interfaces' ][ macAddress ][ 'displayname' ] = netcard[ 'displayname' ];
    this.networkList[ 'interfaces' ][ macAddress ][ 'dns' ]         = netcard[ 'dns' ];
    this.networkList[ 'interfaces' ][ macAddress ][ 'configured' ]  = netcard[ 'configured' ];
    this.networkList[ 'interfaces' ][ macAddress ][ 'dnsMode' ]     = netcard[ 'dnsMode' ];
    this.networkList[ 'interfaces' ][ macAddress ][ 'supersede' ]   = netcard[ 'supersede' ];



    // dhcp options
    this.networkList[ 'interfaces' ][ macAddress ][ 'dhcp-server' ]               = netcard[ 'dhcp-server' ];
    this.networkList[ 'interfaces' ][ macAddress ][ 'dhcp-server' ][ 'enable' ]   = netcard[ 'dhcp-server' ][ 'enable' ] || false;
    this.networkList[ 'interfaces' ][ macAddress ][ 'dhcp-server' ][ 'ip-start' ] = netcard[ 'dhcp-server' ][ 'ip-start' ];
    this.networkList[ 'interfaces' ][ macAddress ][ 'dhcp-server' ][ 'ip-end' ]   = netcard[ 'dhcp-server' ][ 'ip-end' ];

    //dns options
    //get droplist results are always an array with length 1
    const cleanEmpty = obj => {
      if (Array.isArray(obj)) {
        return obj
          .map(v => (v && typeof v === 'object') ? cleanEmpty(v) : v)
          .filter(v => !(v == null || v.length === 0));
      } else {
        return Object.entries(obj)
                     .map(([k, v]) => [k, v && typeof v === 'object' ? cleanEmpty(v) : v])
                     .reduce((a, [k, v]) => (v == null || v === "" ? a : (a[k]=v, a)), {});
      }
    };

    this.networkList[ 'interfaces' ][ macAddress ] = cleanEmpty( this.networkList[ 'interfaces' ][ macAddress ] );
  }

  //-----------------------------------------------------------------------------
  //  CommandRestartServices(): restarts the cosconf service and agw service
  //
  CommandRestartServices()
  {
    //Data.ClearAuthorized();
    //Data.ClearConnected();
    AccessApplication.Instance().Command( 'restart-service' );
    //AgwApplication.Instance().OnCommand( 'ID_RESTART_SERVICE' );
  }

  //-----------------------------------------------------------------------------
  //  NormaliseCosConfData(): get data from cosconf and normalise
  //
  NormaliseName( netcard, actualNetcardValues, status )
  {
  }

  //-----------------------------------------------------------------------------
  //  ChangeState( state ): change the stage of the form
  //                        stage changes will be triggered from cosconf responses
  //
  ChangeState( stage )
  {
    this.stage = stage;
    switch( this.stage ) {
      case "EXPERT":
        this.ApplyStageExpert();
        break;
      case "SYNC":
        this.ApplyStageSync();
        break;
      case "REBOOT-REQUIRED":
      case "REBOOT_REQUIRED":
        this.ApplyStageRebootRequired();
        break;
      default:
        this.ApplyDefaultState();
    }
  }

  //-----------------------------------------------------------------------------
  //  ApplyStageExpert(): applies the stage Expertmode
  //                      all input fields will be readonly
  //                      a permanent toast informs the user that the expert-mode is active
  //

  ApplyStageExpert()
  {
    setTimeout( () => { 
      this.$container.broadcast( "option", "status", "disabled" ); 
    }, 0 );

    $( "#ID_TOAST" + this.dynid ).toast( {
      text: "{$network_toast_expert_mode_notice}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h",
      click: ( id ) => {
        this.OnClick( id );
      },
    } );
  }

  //-----------------------------------------------------------------------------
  //  ApplyStageSync(): description
  //
  ApplyStageSync()
  {
  }

  //-----------------------------------------------------------------------------
  //  ApplyStageRebootRequired(): description
  //
  ApplyStageRebootRequired()
  {
    $( "#ID_TOAST" + this.dynid ).toast( {
      text: "{$network_toast_unsaved_notice}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h",
      click: ( id ) => {
        this.OnClick( id );
      },
    } );
  }

  //-----------------------------------------------------------------------------
  //  OnClick(): handles all click events from the network form widgets not included
  //
  OnClick( id )
  {
    switch( id ) {
      case "ID_RESYNC":
        //refresh data
        Data.Instance().XhrRequest( "COSCONF.RESYNC" );
        //this.CommandSave();
        break;
      case "ID_SAVE_UNSAVED":
        //this.CommandSave( );
        this.CommandRestartServices();
        break;
      default:
        break;
    }
  }

  //-----------------------------------------------------------------------------
  //  ApplyDefaultState(): set up all default states
  //                       currently only the reboot toast would be removed
  //
  ApplyDefaultState()
  {
    $( "#ID_TOAST" + this.dynid ).empty();
  }

  NetCardChanged( netcard )
  {
    for( const mac in this.networkList['interfaces'] ) {
      if( this.networkList['interfaces'][ mac ]['name']  === netcard['name'] )  {
        this.networkList['interfaces'][ mac ] = netcard;
      }
    }
    this.DrawDnsBox( this.networkList );
  }
}



//=============================================================================
//  BUILDER
//=============================================================================

NetworkForm.Register( '/forms/network/network' );
PageMain.RegisterForm( "network", "/forms/network/network", "{$const_menu_network}", "network" );//=============================================================================
//
//  WIDGET dns_config
//  (c) 2020 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          03.02.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================
class WidgetDnsConfig extends Viewable
{
  //-------------------------------------------------------
  //  LOAD
  //
  Load( options )
  {
    //this.dnsList = []
    this.dnsList = [];
    this.dhcpOfferDns = [];
    this.isSupersede = false;
    this.areAllNotConfigured = true;
    this.isDhcpOptionPossible = false;

    // read dns entries
    if( "interfaces" in options['data'] )  {
      for( let macAddress in options['data']['interfaces'] ) {

        // get dns entries from dhcp offer
        if( options['data']['interfaces'][ macAddress ]['configured'] && options['data']['interfaces'][ macAddress ]['type'] !== "static"  )  {
          this.isDhcpOptionPossible = true;
        }

        if( options['data']['interfaces'][ macAddress ]['supersede'] && options['data']['interfaces'][ macAddress ]['supersede']['6'] ) {
          this.isSupersede = true;
        }

        // read dhcp options
        this.ReadDhcpOptions( options['data']['interfaces'][ macAddress ]['dhcp-options'] );

        //  read values for manual dns entries
        if( options['data']['interfaces'][ macAddress ]['supersede'] && options['data']['interfaces'][ macAddress ]['supersede']['6'] )  {
          this.ReadSupersedeEntries( options['data']['interfaces'][ macAddress ]['supersede']['6'] );
        }

        //read dns
        this.ReadSupersedeEntries( options['data']['interfaces'][ macAddress ]['dns'] );
      }
    }

    //$( '#ID_TITLE' + this.dynid ).text( $.lang( "{$dns_config_server}" ) );

    $( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( {
      size: CONST.size.XL,
      height: 5,
      type: 'listbox',
      selection: 'single',
      search: false,
      placeholder: "{$dns_config_dns_noconfig}",
      modifiedEnabled: false,
      //validate: this.ValidateHelperDnsBox.bind (this ),
      data: this.isSupersede ? this.dnsList : this.dhcpOfferDns,
      result: ( selected ) => {
        this.ShowButtons( selected );
      },
    } );

    $( '#ID_DNS_CONFIG_MODUS' + this.dynid ).choice( {
      size: CONST.size.L,
      type: "check",
      status: "normal",
      value: !this.isSupersede,
      text: "{$dns_config_modus}",
      result: this.HandleDnsModusChanged.bind( this ),
    } );

    $( '#ID_DNS_CONFIG_ADD' + this.dynid ).button( {
      size: CONST.size.M,
      status: 'normal',
      text: "Neu",
      click: () => {
        this.OnClickNew();
      }
    } );

    $( '#ID_DNS_CONFIG_EDIT' + this.dynid ).button( {
      size: CONST.size.M,
      text: "{$dns_config_edit}",
      //icon: 'mdi mdi-pencil',
      status: 'disabled',
      click: () => {
        this.OnClickEdit();
      }
    } );-

    $( '#ID_DNS_CONFIG_DELETE' + this.dynid ).button( {
      size: CONST.size.XS,
      status: 'disabled',
      icon: 'mdi mdi-trash-can-outline',
      click: () => {
        this.OnClickDelete();
      }
    } );

    $( '#ID_DNS_CONFIG_UP' + this.dynid ).button( {
      size: CONST.size.XS,
      status: 'disabled',
      icon: 'mdi mdi-chevron-up',
      click: () => {
        this.OnClickUp();
      }
    } );

    $( '#ID_DNS_CONFIG_DOWN' + this.dynid ).button( {
      size: CONST.size.XS,
      status: 'disabled',
      icon: 'mdi mdi-chevron-down',
      click: () => {
        this.OnClickDown();
      }
    } );


    if( !this.isDhcpOptionPossible )  {
      $( '#ID_DNS_CONFIG_MODUS' + this.dynid ).choice( "option", "value", false );
      $( '#ID_DNS_CONFIG_MODUS' + this.dynid ).choice( "option", "status", "disabled" );
      $( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( "option", "data", this.dnsList );
      this.isSupersede = true;
    }
    else {
      this.isSupersede = !$( '#ID_DNS_CONFIG_MODUS' + this.dynid ).choice( "option", "value" );
      $( '#ID_DNS_CONFIG_MODUS' + this.dynid ).choice( "option", "status", "normal" );
    }

    if( this.isSupersede )  {
      this.WriteSupersede();
    }
    this.ShowButtons(  $( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( "option", "selected" ) );
  }


  //=============================================================================
  //  Button Events
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  OnClickNew(): open the ip change dialog with empty fields
  //
  OnClickNew()
  {
    let options = {
      data: {},
      result: ( resultData ) => {
        if( resultData != false ) this.WorkEditRecord( resultData );
      }
    };
    ChangeIpAddressDialog.Show( options );
  }

  //-----------------------------------------------------------------------------
  //  OnClickEdit(): sends a DNS-Entry to the dialog
  //
  OnClickEdit()
  {
    ChangeIpAddressDialog.Show( {
      data: {
        ipAddress: $( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( "option", "selected" )[0]
      },
      result: ( resultData ) => {
        if( resultData != false ) this.WorkEditRecord( resultData );
      }
    } );
  }

  //-----------------------------------------------------------------------------
  //  OnClickDelete(): just delete a existing DNS-Entry
  //
  OnClickDelete()
  {
    let selectedDnsEntry = $( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( "option", "selected" );
    if( Array.isArray( selectedDnsEntry ) && selectedDnsEntry.length > 0 ) {
      this.DeleteDnsEntry( selectedDnsEntry );
      this.Option( "modified", 1 );
    }
  }

  //-----------------------------------------------------------------------------
  //  DeleteDnsEntry(): deltes an entry from dns list
  //
  DeleteDnsEntry( selectedDnsEntry )
  {
    for( const dnsEntry in selectedDnsEntry ) {
      let currentDnsEntry = selectedDnsEntry[ dnsEntry ];

      for( let dnsEntryNumber in this.dnsList ) {
        if( this.dnsList[ dnsEntryNumber ][ "id" ] == currentDnsEntry ) {
          this.dnsList.splice( dnsEntryNumber, 1 );
        }
      }

      this.WriteSupersede();
    }
    $( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( "option", "data", this.dnsList );
    if( this.dnsList.length === 0 )  {
      this.ShowButtons( [] );
    }

  }

  //-----------------------------------------------------------------------------
  //  WorkEditRecord(): process dialog result data => change or create a dns entry
  //
  WorkEditRecord( result )
  {
    // prerequisites
    let isNewIpAddress = false;
    let currentDnsList = this.isSupersede ? this.dnsList : this.dhcpOfferDns;

    // change existing ip address
    if( "ipAddress" in result
      && result['ipAddress']
      && currentDnsList ) {

      for( let dnsNumber in currentDnsList ) {
        if( currentDnsList[dnsNumber]['id'] == result['ipAddress'] ) {
          currentDnsList[dnsNumber]['id']   = result['newIpAddress'];
          currentDnsList[dnsNumber]['text'] = result['newIpAddress'];
        }
      }
    }
    else {
      isNewIpAddress = true;
    }

    // add new ip to dns list
    if( "newIpAddress" in result
      && isNewIpAddress
      && currentDnsList ) {
      currentDnsList.splice( 0, 0, { id: result['newIpAddress'], text: result['newIpAddress'] } );
    }

    // finish
    $( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( "option", "data", currentDnsList );
    this.dnsList = currentDnsList;

    if( isNewIpAddress )  {
      $( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( "option", "selected", [] );
      this.ShowButtons( [] );
    }
    else  {
      $( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( "option", "selected", [ ""+result['newIpAddress'] ] );
      this.ShowButtons( [ ""+result['newIpAddress'] ] );
    }

    // store the dns entries in data
    if( this.isSupersede )  {
      for( let netcardMacAddress in  this.createOptions['data']['interfaces'])  {
        if( !this.createOptions['data']['interfaces'][ netcardMacAddress ]['supersede'] )  {
          this.createOptions['data']['interfaces'][ netcardMacAddress ]['supersede'] = { "6": "" };
        }
        this.createOptions['data']['interfaces'][ netcardMacAddress ]['supersede']['6'] = currentDnsList.map( value => value.id ).join( " " );
      }
    }


    // set modified flag 
    this.Option( "modified", 1 );

  }


  //-----------------------------------------------------------------------------
  //  OnClick(): Control Button Handler
  //
  OnClickUp()
  {
    this.MoveListEntry();
  }

  //-----------------------------------------------------------------------------
  //  OnClick(): Control Button Handler
  //
  OnClickDown()
  {
    this.MoveListEntry( false );
  }

  MoveListEntry( moveUp = true )
  {
    let currentSelection = $( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( "option", "selected" );
    let currentData      = (this.isSupersede) ? this.dnsList : this.dhcpOfferDns;
    let step             = ( moveUp ) ? -1 : 1;

    //necessary data check
    if( !Array.isArray( currentSelection ) || currentSelection.length === 0 )  {
      console.error( "DNS-BOX: nothing is selected!" );
      return 0;
    }

    if( !Array.isArray( currentData ) || currentData.length === 0 )  {
      console.error( "DNS-BOX: has no data" );
      return 0;
    }

    // sort selection as they are stored in data
    currentSelection = currentSelection.sort( ( a, b ) => {
      let GetIdA =  (element) =>  element.id === a;
      let GetIdB =  (element) =>  element.id === b;
      return currentData.findIndex( GetIdA ) - currentData.findIndex( GetIdB );
    }  );

    // in case of "move down" we need to move first the last selected item
    if( moveUp === false ) {
      currentSelection = currentSelection.reverse();
    }

    let border = ( moveUp ) ? 0 : currentData.length - 1;


    // find position and move the entry one position upwards
    for( let entry in currentSelection  )  {
      let index     = currentData.findIndex(  o =>  o['id'] === currentSelection[ entry ] );
      let ipDataset = currentData.find(  o =>  o['id'] === currentSelection[ entry ] );
      if( ( moveUp ) ? index > 0 + entry : index < border - entry )  {
        currentData.splice( index, 1 );
        currentData.splice( index + step, 0, ipDataset );
      }
    }
    // apply changes
    if( this.isSupersede ) {
      this.dnsList = currentData;
    }
    else {
      this.dhcpOfferDns = currentData;
    }
    $( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( "option", "data", currentData );


    for( let macAddress in this.createOptions['data']['interfaces'] ) {
      if( !this.createOptions['data']['interfaces'][ macAddress ]['supersede'] )  {
        this.createOptions['data']['interfaces'][ macAddress ]['supersede'] = {};
      }
      if( !this.createOptions['data']['interfaces'][ macAddress ]['supersede']['6'] )  {
        this.createOptions['data']['interfaces'][ macAddress ]['supersede']['6'] = "";
      }
      this.createOptions['data']['interfaces'][ macAddress ]['supersede']['6'] = currentData.map( value => value.id ).join( " " );
    }

    setTimeout( () => { 
      $( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( "option", "selected", currentSelection.reverse() ); 
    }, 0 );
    this.ShowButtons( currentSelection.reverse() );

    this.Option( "modified", 1 );
  }

  //-----------------------------------------------------------------------------
  //  ShowButtons( selected ): disables/enables buttons based on selected values
  //
  ShowButtons( selected )
  {
    let indexTable = [];
    let minDnsIndex = 0;
    let maxDnsIndex = 0;
    for( let dnsEntry in selected )  {
      indexTable.push( this.dnsList.findIndex( ( dnsObject ) => dnsObject['id'] === selected[ dnsEntry ]  ));
    }

    minDnsIndex = Math.min( ...indexTable );
    maxDnsIndex = Math.max( ...indexTable );

    let isReadonly = false;

    if( indexTable.length > 0 ) {
      indexTable.forEach( ( index ) => {
        if( this.dnsList[index]['status'] === "readonly" )  {
          isReadonly = true;
        }
      } );
    }

    //normal case
    if( this.isSupersede )  {
      $( '#ID_DNS_CONFIG_ADD'    + this.dynid ).button( "option", "status", "normal" );
    }
    else  {
      $( '#ID_DNS_CONFIG_ADD'    + this.dynid ).button( "option", "status", "disabled" );
    }

    if( isReadonly !== true && Array.isArray( selected ) && selected.length > 0 )  {
      $( '#ID_DNS_CONFIG_DELETE' + this.dynid ).button( "option", "status", "normal" );
      $( '#ID_DNS_CONFIG_EDIT'   + this.dynid ).button( "option", "status", "normal" );
      $( '#ID_DNS_CONFIG_UP'     + this.dynid ).button( "option", "status", "normal" );
      $( '#ID_DNS_CONFIG_DOWN'   + this.dynid ).button( "option", "status", "normal" );
    }
    else  {
      $( '#ID_DNS_CONFIG_DELETE' + this.dynid ).button( "option", "status", "disabled" );
      $( '#ID_DNS_CONFIG_EDIT'   + this.dynid ).button( "option", "status", "disabled" );
      $( '#ID_DNS_CONFIG_UP'     + this.dynid ).button( "option", "status", "disabled" );
      $( '#ID_DNS_CONFIG_DOWN'   + this.dynid ).button( "option", "status", "disabled" );
    }

    //special case frist entry (position = 0)
    if( minDnsIndex === 0 && maxDnsIndex - minDnsIndex <= 1 )  {
      $( '#ID_DNS_CONFIG_UP'     + this.dynid ).button( "option", "status", "disabled" );
    }

    //special case multi select
    if( selected.length > 1 )  {
      $( '#ID_DNS_CONFIG_EDIT'   + this.dynid ).button( "option", "status", "disabled" );
      if( isReadonly )  {
        $( '#ID_DNS_CONFIG_UP'     + this.dynid ).button( "option", "status", "normal" );
        $( '#ID_DNS_CONFIG_DOWN'   + this.dynid ).button( "option", "status", "normal" );
      }
    }

    //special case last entry
    if( maxDnsIndex === this.dnsList.length - 1 && maxDnsIndex - minDnsIndex <= 1 )  {
      $( '#ID_DNS_CONFIG_DOWN'     + this.dynid ).button( "option", "status", "disabled" );
    }

  }

  //-----------------------------------------------------------------------------
  //  WriteSupersede(): save the changed supersede settings
  //
  WriteSupersede()
  {
    for( let macAddress in this.createOptions['data']['interfaces'] ) {
      if( !this.createOptions['data']['interfaces'][ macAddress ]['supersede'] )  {
        this.createOptions['data']['interfaces'][ macAddress ]['supersede'] = {};
      }
      if( !this.createOptions['data']['interfaces'][ macAddress ]['supersede']['6'] )  {
        this.createOptions['data']['interfaces'][ macAddress ]['supersede']['6'] = "";
      }
      this.createOptions['data']['interfaces'][ macAddress ]['supersede']['6'] = this.dnsList.map( value => value.id ).join( " " );
    }
  }

  //-------------------------------------------------------
  //  helper for validate dhcp scope
  //
  ValidateHelperDnsBox( ev )
  {
    if( $( '#ID_DNS_CONFIG_MODUS' + this.dynid ).choice('option', 'value') )  {
      return ev.error = null;
    }
    else if ( !$( '#ID_DNS_CONFIG_MODUS' + this.dynid ).choice('option', 'value') && this.dnsList.length !== 0 ) {
      return ev.error = null;
    }
    ev.error = "{$dns_config_supersede_list_empty}";
  }

  //-----------------------------------------------------------------------------
  //  OnValidate(): validate dns box settings
  //
  OnValidate()
  {
    let countOfError = 0;
    countOfError += $( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( "validate" );
    return countOfError;
  }

  //-----------------------------------------------------------------------------
  //  ReadDhcpOptions(): read dns entries from dhcp-options
  //
  ReadDhcpOptions( dhcpOptions )
  {
    if( !dhcpOptions || !dhcpOptions['6'] ) return;

    for( let entry in  dhcpOptions["6"] ) {
      let currentValue = dhcpOptions["6"][ entry ];
      if( this.dhcpOfferDns.length === 0 || this.dhcpOfferDns.some( (dnsEntry) => dnsEntry.id === currentValue ) === false  )  {
        this.dhcpOfferDns.push( {
          id: currentValue,
          text: `${currentValue} (dhcp)`,
          status: "locked",
        } );
      }

      if( this.dhcpOfferDns.length === 0 || this.dhcpOfferDns.some( ( element ) => { element != currentValue; } ) )  {
        this.dhcpOfferDns.push( currentValue );
      }
    }
  }

  //-----------------------------------------------------------------------------
  //  ReadSupersedeEntries(): reads the dns entries from supersede properties
  //
  //  Supersede entries are located at
  //    - options['interfaces'][macAddress]['dns']
  //    - options['interfaces'][macAddress]['supersede']['6']
  //
  ReadSupersedeEntries( supersede )
  {

    if( !supersede )  return;
    this.isSupersede = true;
    // if dns entry is just a string convert it to an array ( normalise cosconf value )
    if( typeof supersede === "string" ) supersede = supersede.split( " " );

    for( let dnsNumber in supersede ) {
      let currentDnsEntry = supersede[dnsNumber];
      // if dns entry is just a string convert it to an array ( normalise cosconf value )
      if( typeof currentDnsEntry === "string" ) currentDnsEntry = currentDnsEntry.split( " " );
      currentDnsEntry.forEach( ( value ) => {
        if( !Array.isArray( value ) && currentDnsEntry && this.dnsList.some( (dnsEntry) => dnsEntry.id === value ) === false ) {
          this.dnsList.push( {
            id: value,
            text: value,
            status: "normal",
          } );
        }
      } );
    }
  }

  //-----------------------------------------------------------------------------
  //  HandleDnsModusChanged(): handles enabling/disabling dns mode "use dns from dhcp options"
  //
  HandleDnsModusChanged( value )
  {
    this.isSupersede = !value;
    if( this.isSupersede )  {
      $( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( "option", "data", this.dnsList );
      for( const mac in this.createOptions['data']['interfaces'] ) {
        this.createOptions['data']['interfaces'][ mac ]['supersede'] = { "6": ""  };
        delete this.createOptions['data']['interfaces'][ mac ]['prepend'];
      }
    }
    else {
      for( const mac in this.createOptions['data']['interfaces'] ) {
        delete this.createOptions['data']['interfaces'][ mac ]['supersede'];
        delete this.createOptions['data']['interfaces'][ mac ]['dns'];
      }
      //$( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( "option", "data", this.dhcpOfferDns );
      $( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( "option", "data", [{ id:"unknown", text: "{$dns_config_dhcp_dns_unknown}", status: "locked" }] );
    }
    this.ShowButtons( $( '#ID_DNS_CONFIG_BOX' + this.dynid ).listbox( "option", "selected" ) );
  }
}




//=============================================================================
//  BUILDER
//=============================================================================

WidgetDnsConfig.Register( '/forms/system/network/widgets/controls/dns_config' );
﻿//=============================================================================
//
//  WIDGET netcard
//  (c) 2020 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.11.2020
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WidgetNetcard extends Viewable
{
  //-----------------------------------------------------------------------------
  //  Load(): on loading the form initialise the viewable
  //          - create all controls
  //          - define envent subscribtions
  //          - etc.
  Load( options )
  {
    this.isDhcpIsActive  = false;
    this.dhcpPoolStart   = "";
    this.dhcpPoolEnd     = "";
    this.isConfigured    = options.data['configured'];
    this.agwConfigHelper = options.agwConfigHelper;
    this.isAgw           = typeof AdminConfigHelper !== "undefined";

    let dhcpMode       = false;
    let self           = this;

    dhcpMode = options['data']['type'];

    // default values
    this.PrepareResponseValues( options );

    //=============================================================================
    //  CONTROLS
    //=============================================================================

    // groupbox: name and status icon
    this.$parent.groupbox( { } );
    $( "#ID_HEADLINE_NAME" + this.dynid ).text( options['data']['displayname'] || options['data']['name'] );
    if( options['data']['configured'] === true  )  {
      this.ShowInterfaceStatus( options['data']['status'].toLowerCase() );
    }
    else  {
      this.ShowInterfaceStatus( "unconfigured" );
    }

    //common ip setting controls
    $( '#ID_NETCARD_NAME' + this.dynid ).edit( {
      size: CONST.size.L,
      status: "normal",
      value: options['data']['displayname'] || options['data']['name'] || "",
      result: ( value ) => {
        options['data']['displayname'] = value;
      }
    } );

    $( '#ID_NETCARD_SYSTEM_NAME' + this.dynid ).text( options['data']['name'] );

    $( '#ID_NETCARD_TYPE' + this.dynid ).choice( {
      size: CONST.size.L,
      type: 'radio',
      data: [
        { id: 'static',   text: "{$netcard_static}" },
        { id: 'dhcp',     text: "{$netcard_dhcp}" },
        { id: 'noConfig', text: "{$netcard_no_config}"}
      ],
      value: !this.isConfigured ? "noConfig" : options['data']['type'] || "static",
      result: ( value ) => {
        options['data']['type'] = value;
        if( value === "dhcp" )  {
          $( '#ID_NETCARD_DHCP_ACTIV' + this.dynid ).choice( "option", "value", false );
          options['data']['dhcp-server']['enable'] = false;
          this.isDhcpIsActive = false;
        }
        else {
          this.isDhcpIsActive = false;
          $( '#ID_NETCARD_IP' + this.dynid ).edit( "option", "value", "" );
          $( '#ID_NETCARD_GATEWAY' + this.dynid ).edit( "option", "value", "" );
          $( '#ID_NETCARD_NET_MASK' + this.dynid ).edit( "option", "value", "" );
          $( '#ID_NETCARD_DHCP_ACTIV' + this.dynid ).edit( "option", "value", false );
          $( '#ID_NETCARD_DHCP_POOL_START' + this.dynid ).edit( "option", "value", "" );
          $( '#ID_NETCARD_DHCP_POOL_END' + this.dynid ).edit( "option", "value", "" );

          options['data']['ipv4']     = "";
          options['data']['ipv4mask'] = "";
          options['data']['gateway']  = "";
          options.data['dhcp-server'] = {
            "enable": false,
            "ip-start": "",
            "ip-end": "",
            "leases": [],
            "lease-time": [],
            "options": {},
          };
        }
        if( value === "noConfig" )  {
          options['data']['configured'] = false;
          this.isConfigured = false;
          $( '#ID_NETCARD_DHCP_ACTIV' + this.dynid ).choice( "option", "value", false );
          $( '#ID_NETCARD_IP' + this.dynid ).edit( "validate", false );
          $( "#ID_NETCARD_NET_MASK" + this.dynid ).edit( "validate", false );
          $( '#ID_NETCARD_GATEWAY' + this.dynid ).edit( "validate", false );
          $( '#ID_NETCARD_DHCP_POOL_START' + this.dynid ).edit( "validate", false );
          $( '#ID_NETCARD_DHCP_POOL_END' + this.dynid ).edit( "validate", false );
        }
        else {
          options['data']['configured'] = true;
          this.isConfigured = true;
        }
        //this.ShowDhcpOption( value === "static" );
        //this.ShowDhcpCheckBox( this.isDhcpIsActive );
        this.FireEvent( "result", this.createOptions['data'] );
        this.FormChanged();
      }
    } );

    $( '#ID_NETCARD_IP' + this.dynid ).edit( {
      size: CONST.size.L,
      status: "normal",
      validate: this.ValidateNetcardIp.bind( this ),
      validateDepends: [ "ID_NETCARD_DHCP_POOL_START" + this.dynid, "ID_NETCARD_DHCP_POOL_END" + this.dynid ],
      value: this.isConfigured ? options['data']['ipv4'] || "" : "",
      result: ( value ) => {
        options['data']['ipv4'] = value;
      },
      change: ( value ) =>  {
        if( Validate.IsIpAddress( value ) && ( options['data']['ipv4'] != value ) )  {
          this.OnNetcardIpChanged( value, options['data'] );
        }
      }
    } );

    $( '#ID_NETCARD_NET_MASK' + this.dynid ).edit( {
      size: CONST.size.L,
      status: "normal",
      validate: Validate.IsNetmask,
      value: this.isConfigured ? options['data']['ipv4mask'] || "" : "",
      result: ( value ) => {
        options['data']['ipv4mask'] = value;
      },
      change: ( value ) => {
        if( Validate.IsIpAddress( value ) && ( options['data']['ipv4'] != value ) ) {
          this.OnNetcardMaskChanged( value, options['data'] );
        }
      }
    } );

    $( '#ID_NETCARD_GATEWAY' + this.dynid ).edit( {
      size: CONST.size.L,
      status: "normal",
      validate: this.ValidateGateway.bind( this ),
      value: this.isConfigured ? options['data']['gateway'] || "" : "",
      result: ( value ) => {
        options['data']['gateway'] = value;
      },
      change: ( value ) => {
        if( ( Validate.IsIpAddress( value ) || ( value == "" ) ) 
         && ( options['data']['gateway'] != value ) 
          ) {
          this.OnNetcardGatewayChanged( value, options['data'] );
        }
      }
    } );

    // dhcp controls
    $( '#ID_NETCARD_DHCP_ACTIV' + this.dynid ).choice( {
      type: 'check',
      status: "normal",
      text: '{$netcard_is_dhcp_active}',
      value: this.isDhcpIsActive,
      result: ( value ) => {
        this.isDhcpIsActive = value;
        //this.ShowDhcpCheckBox( value );
        if( this.isDhcpIsActive )  {
          this.OnDhcpServerChanged( options['data'], this.isDhcpIsActive );
        }
        this.FormChanged();
        options['data']['dhcp-server']['enable'] = value;
      }
    } );

    $( '#ID_NETCARD_DHCP_POOL_START' + this.dynid ).edit( {
      size: CONST.size.L,
      status: "normal",
      value: this.dhcpPoolStart,
      validate: this.ValidateDhcpIpPoolStart.bind( this ),
      validateDepends: [ "ID_NETCARD_IP" + this.dynid, "ID_NETCARD_NET_MASK" + this.dynid, "ID_NETCARD_DHCP_POOL_END" + this.dynid ],
      result: ( value ) => {
        options['data']['dhcp-server']['ip-start'] = value;
      }
    } );

    $( '#ID_NETCARD_DHCP_POOL_END' + this.dynid ).edit( {
      size: CONST.size.L,
      status: "normal",
      value: this.dhcpPoolEnd,
      validate: this.ValidateDhcpIpPoolEnd.bind( this ),
      validateDepends: [ "ID_NETCARD_IP" + this.dynid, "ID_NETCARD_NET_MASK" + this.dynid, "ID_NETCARD_DHCP_POOL_START" + this.dynid ],
      result: ( value ) => {
        options['data']['dhcp-server']['ip-end'] = value;
      }
    } );

    $( '#ID_NETCARD_BUTTON_OPTIONS' + this.dynid ).button( {
      size: CONST.size.M,
      text: '{$netcard_button_options}',
      status: 'normal',
      click: () => {
        DhcpOptionsDialog.Show( {
          data: options['data'],
          result: ( data ) => {
            if( data )  {
              options['data']['dhcp-server'] = data['dhcp-server'];
              $( '#ID_NETCARD_DHCP_POOL_START' + this.dynid ).edit( "option", "value", data['dhcp-server']['ip-start'] );
              $( '#ID_NETCARD_DHCP_POOL_END' + this.dynid ).edit( "option", "value", data['dhcp-server']['ip-end'] );
              this.Option( "modified", 1 );
            }
          }
        } );
      }
    } );

    //=============================================================================
    //  CONFIGURE OPTIONALS
    //=============================================================================
    // activate the optional controls
    $( '#ID_OPTIONAL_DHCP' + this.dynid ).optional( {} );
    $( '#ID_OPTIONAL_DHCP_ACTIVE' + this.dynid ).optional( {} );

    // hide warning on dhcp options
    $( "#ID_NETCARD_BUTTON_OPTIONS_HINT" + this.dynid ).css( "display", "none" );

    //this.ShowDhcpCheckBox( this.isDhcpIsActive );
    //this.ShowDhcpOption( dhcpMode === "static" );

    this.FormChanged();

  }//load

  //-----------------------------------------------------------------------------
  //  control validation handlers
  //-----------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  // IP-Address: Validate Ip and scope if dhcp-server is set
  //
  ValidateNetcardIp( validateData )
  {
    let ipNetcard = new IpAddress( validateData.value );
    let ipPoolStart = new IpAddress( $( "#ID_NETCARD_DHCP_POOL_START" + this.dynid ).edit( "option", "value" ) );
    let ipPoolEnd = new IpAddress( $( "#ID_NETCARD_DHCP_POOL_END" + this.dynid ).edit( "option", "value" ) );
    let isDhcpServerActive = $( '#ID_NETCARD_DHCP_ACTIV' + this.dynid ).choice( "option", "value" );

    // "unconfigured" mode -> no error
    if( this.isConfigured == false ) {
      validateData.error = null;
      return true;
    }

    // simple ip address validation
    if( ! Validate.IsIpAddress( validateData.value ) )  {
      validateData.error = "{$const_error_novalidip}";
      return false;
    }

    //=============================================================================
    //  AGW Special Function don't delete it!!!!!!!!!!!!!!
    //=============================================================================

    if( this.isAgw ) {
      if( this.agwConfigHelper.GetServerIp() == validateData.value ) {
        validateData.error = "{$netcard_ip_as_access}";
        return false;
      }
    }

    // error if netcard ip is in range of dhcp scope
    if( ( isDhcpServerActive )
     && ( ipPoolStart.IsValid() )
     && ( ipPoolEnd.IsValid() )
     && ( ipNetcard.AsNumber() >= ipPoolStart.AsNumber() )
     && ( ipNetcard.AsNumber() <= ipPoolEnd.AsNumber() )
      ) {
      validateData.error = "{$netcard_pool_error_ip}";
      return false;
    }

    // no error
    validateData.error = null;
    return true;
  }

  //-----------------------------------------------------------------------------
  // Gateway: Validate Ip-Address but might be empty. 
  //
  ValidateGateway( validateData )
  {
    if( validateData.value == false ) {
      return validateData.error = null;
    }
    else {
      return Validate.IsIpAddress( validateData );
    }
  }

  //-----------------------------------------------------------------------------
  // DHCP server: IP pool start address
  //
  ValidateDhcpIpPoolStart( validateData )
  {
    // prepare
    let ipv4 = $( '#ID_NETCARD_IP' + this.dynid ).edit( "option", "value" );
    let ipv4mask = $( '#ID_NETCARD_NET_MASK' + this.dynid ).edit( "option", "value" );
    
    // validation
    if( ! Validate.IsIpAddress( validateData.value ) ) {
      validateData.error = "{$const_error_novalidip}";
      return false;   // error ip
    }
    if( ! Validate.IsIpInSubnet( ipv4, ipv4mask, validateData.value ) ) {
      validateData.error = "{$dhcp_scope_error_range}";
      return false;   // error subnet
    }

    // no error
    validateData.error = null;
    return true;
  }

  //-----------------------------------------------------------------------------
  // DHCP server: IP pool end address
  //
  ValidateDhcpIpPoolEnd( validateData )
  {
    // prepare
    let ipv4 = $( '#ID_NETCARD_IP' + this.dynid ).edit( "option", "value" );
    let ipv4mask = $( '#ID_NETCARD_NET_MASK' + this.dynid ).edit( "option", "value" );
    let ipPoolStartString = $( "#ID_NETCARD_DHCP_POOL_START" + this.dynid ).edit( "option", "value" );
    let ipPoolStart = new IpAddress( ipPoolStartString );
    let ipPoolEnd = new IpAddress( validateData.value );

    // validation
    if( !Validate.IsIpAddress( validateData.value ) ) {
      validateData.error = "{$const_error_novalidip}";
      return false;   // error ip
    }
    if( !Validate.IsIpInSubnet( ipv4, ipv4mask, validateData.value ) ) {
      validateData.error = "{$dhcp_scope_error_range}";
      return false;   // error subnet
    }
    if( ipPoolEnd.AsNumber() <= ipPoolStart.AsNumber() )  {
      validateData.error = "{$netcard_pool_error_range}";
      return false;   // error ip-pool range
    }

    // no error
    validateData.error = null;
    return true;
  }

  //-----------------------------------------------------------------------------
  //  viewable class events
  //-----------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  OnEvent(): collect all events from controls
  //
  OnEvent( eventName, data )
  {
  }

  //-----------------------------------------------------------------------------
  //  helper functions
  //-----------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  ShowInterfaceStatus( status ): manages the led indicator before network
  //  status: ["down", "up", "unconfigured" ]
  //
  ShowInterfaceStatus( status )
  {
    if( !this.isConfigured )  {
      status = "unconfigured";
    }

    $( "#ID_HEADLINE_STATUS" + this.dynid ).toggleClass( ['red-text', 'green-text', 'yellow-text'], false );
    switch( status )  {
      case "down":          $( "#ID_HEADLINE_STATUS" + this.dynid ).addClass( "red-text" ); break;
      case "up":            $( "#ID_HEADLINE_STATUS" + this.dynid ).addClass( "green-text" ); break;
      case "unconfigured":  $( "#ID_HEADLINE_STATUS" + this.dynid ).addClass( "yellow-text" ); break;
      default:              $( "#ID_HEADLINE_STATUS" + this.dynid ).addClass( "yellow-text" ); break;
    }
  }

  //-----------------------------------------------------------------------------
  //  ShowDhcpOptions(isShown): show or hide the options of dhcp configuration
  //                            is the network interface type static => show checkbox
  //                            is type dhcp => hide checkbox
  //
  ShowDhcpOption( ishown )
  {
    if( ishown === true ) {
      //show dchp settings
      $( '#ID_OPTIONAL_DHCP' + this.dynid ).optional( "show", "ID_OPTIONAL_DHCP_SETTINGS" );
      if( $( "#ID_NETCARD_IP" + this.dynid ).edit( "isValidationEnabled" ) )  {
        $( "#ID_NETCARD_IP" + this.dynid ).edit( "validate" );
        $( "#ID_NETCARD_DHCP_POOL_START" + this.dynid ).edit( "validate" );
        $( "#ID_NETCARD_DHCP_POOL_END" + this.dynid ).edit( "validate" );
      }      
    }
    else {
      //hide dchp settings & switch off validation for disabled controls
      $( '#ID_OPTIONAL_DHCP' + this.dynid ).optional( "hide", "ID_OPTIONAL_DHCP_SETTINGS" );
      
      // re-validate (error ip is in range of ip-pool)
      if( $( "#ID_NETCARD_IP" + this.dynid ).edit( "isValidationEnabled" ) ) {
        $( "#ID_NETCARD_IP" + this.dynid ).edit( "validate" );
        $( "#ID_NETCARD_NET_MASK" + this.dynid ).edit( "validate" );
      }
      $( "#ID_NETCARD_DHCP_POOL_START" + this.dynid ).edit( "validate", false );
      $( "#ID_NETCARD_DHCP_POOL_END" + this.dynid ).edit( "validate", false );
    }
  }

  //-----------------------------------------------------------------------------
  //  ShowDhcpOptions(isShown): show or hide the options of dhcp configuration
  //
  ShowDhcpCheckBox( ishown )
  {
    //trigger the optional framework to show or hide these settings.
    if( ishown === true ) {
      $( '#ID_OPTIONAL_DHCP_ACTIVE' + this.dynid ).optional( "show", "ID_OPTIONAL_DHCP_ACTIVE_SETTINGS" );
    }
    else {
      $( '#ID_OPTIONAL_DHCP_ACTIVE' + this.dynid ).optional( "hide", "ID_OPTIONAL_DHCP_ACTIVE_SETTINGS" );
    }
  }

  //-----------------------------------------------------------------------------
  //  EnableIpSettings(): enable or disable ip settings
  //
  EnableIpSettings( isShown )
  {
    if( isShown )  {
      $( "#FORM_STATUS_GROUP_DHCP" + this.dynid ).broadcast( "option", "status", "normal" );
    }
    else  {
      $( "#FORM_STATUS_GROUP_DHCP" + this.dynid ).broadcast( "option", "status", "readonly" );
    }
  }

  //-----------------------------------------------------------------------------
  //  FormChanged(): central handler for from changes
  //
  FormChanged()
  {
    let isDhcpCheckBoxShown = false;
    let isDhcpOptionShown   = false;

    isDhcpCheckBoxShown = $( '#ID_NETCARD_TYPE' + this.dynid ).choice( "option", "value" ) === "static" ;
    isDhcpOptionShown   = $( '#ID_NETCARD_DHCP_ACTIV' + this.dynid ).choice( "option", "value" );


    this.ShowDhcpOption( isDhcpOptionShown );
    this.EnableIpSettings( isDhcpCheckBoxShown );
    this.ShowDhcpCheckBox( isDhcpCheckBoxShown );
  }

  //-----------------------------------------------------------------------------
  //  OnValidate(): entry point of enable a viewables validation
  //
  OnValidate( )
  {
    let numberOfErrors = 0;

    if( $( '#ID_NETCARD_TYPE' + this.dynid ).edit( "option", "value" ) === "static"   )  {
      numberOfErrors += $( '#ID_NETCARD_IP' + this.dynid ).edit( "validate" );
      numberOfErrors += $( '#ID_NETCARD_NET_MASK' + this.dynid ).edit( "validate" );
      numberOfErrors += $( '#ID_NETCARD_GATEWAY' + this.dynid ).edit( "validate" );
    }

    if( $( "#ID_NETCARD_DHCP_ACTIV" + this.dynid ).choice( "option", "value" ) === true )  {
      numberOfErrors += $( '#ID_NETCARD_DHCP_POOL_START' + this.dynid ).edit( "validate" );
      numberOfErrors += $( '#ID_NETCARD_DHCP_POOL_END'   + this.dynid ).edit( "validate" );
    }

    return numberOfErrors;
  }

  //-----------------------------------------------------------------------------
  //  after enabling dhcp-server a set of default options must be created
  //
  OnDhcpServerChanged( interfaceData, isActive )
  {
    // dhcp-server switched on, create default options
    if( isActive )  {
      
      // build options
      let defaultOptions = DhcpOptionsDialog.BuildDhcpDefaultOptions( interfaceData );
      interfaceData['dhcp-server']['options'] = defaultOptions;
      
      // display warning "dhcp options changed"
      $( "#ID_NETCARD_BUTTON_OPTIONS_HINT" + this.dynid ).css( "display", "" );
      new Animation( $( '#ID_NETCARD_BUTTON_OPTIONS' + this.dynid ), "flash" );
    }

    // dhcp-server switched OFF, delete dhcp-options
    else  {
      $( "#ID_NETCARD_BUTTON_OPTIONS_HINT" + this.dynid ).css( "display", "none" );
      delete interfaceData['dhcp-server'];
    }

  }

  //-----------------------------------------------------------------------------
  //  if ip-address of netcard is changed we need to adapt the dhcp-options
  //
  OnNetcardIpChanged( value, interfaceData )
  {
    if( $( '#ID_NETCARD_DHCP_ACTIV' + this.dynid ).choice( "option", "value" ) == true )  {
      
      // re-build options, need to copy since this is the changed-handler from edit
      interfaceData['ipv4'] = value;
      interfaceData['ipv4mask'] = $( "#ID_NETCARD_NET_MASK" + this.dynid ).edit( "option", "value" );;
      interfaceData['gateway'] = $( "#ID_NETCARD_GATEWAY" + this.dynid ).edit( "option", "value" );
      interfaceData['dhcp-server']['options'] = {};
      let defaultOptions = DhcpOptionsDialog.BuildDhcpDefaultOptions( interfaceData );
      interfaceData['dhcp-server']['options'] = defaultOptions;

      // display warning "dhcp options changed"
      $( "#ID_NETCARD_BUTTON_OPTIONS_HINT" + this.dynid ).css( "display", "" );
      new Animation( $( '#ID_NETCARD_BUTTON_OPTIONS' + this.dynid ), "flash" );
    }
  }

  //-----------------------------------------------------------------------------
  //  if ip-mask of netcard is changed we need to adapt the dhcp-options
  //
  OnNetcardMaskChanged( value, interfaceData )
  {
    if( $( '#ID_NETCARD_DHCP_ACTIV' + this.dynid ).choice( "option", "value" ) == true ) {

      // re-build options, need to copy since this is the changed-handler from edit
      interfaceData['ipv4'] = $( "#ID_NETCARD_IP" + this.dynid ).edit( "option", "value" );
      interfaceData['ipv4mask'] = value;
      interfaceData['gateway'] = $( "#ID_NETCARD_GATEWAY" + this.dynid ).edit( "option", "value" );
      interfaceData['dhcp-server']['options'] = {};
      let defaultOptions = DhcpOptionsDialog.BuildDhcpDefaultOptions( interfaceData );
      interfaceData['dhcp-server']['options'] = defaultOptions;

      // display warning "dhcp options changed"
      $( "#ID_NETCARD_BUTTON_OPTIONS_HINT" + this.dynid ).css( "display", "" );
      new Animation( $( '#ID_NETCARD_BUTTON_OPTIONS' + this.dynid ), "flash" );
    }
  }

  //-----------------------------------------------------------------------------
  //  if gateway of netcard is changed we need to adapt the dhcp-options
  //
  OnNetcardGatewayChanged( value, interfaceData )
  {
    if( $( '#ID_NETCARD_DHCP_ACTIV' + this.dynid ).choice( "option", "value" ) == true ) {

      // re-build options, need to copy since this is the changed-handler from edit
      interfaceData['ipv4'] = $( "#ID_NETCARD_IP" + this.dynid ).edit( "option", "value" );
      interfaceData['ipv4mask'] = $( "#ID_NETCARD_NET_MASK" + this.dynid ).edit( "option", "value" );;
      interfaceData['gateway'] = value;
      interfaceData['dhcp-server']['options'] = {};
      let defaultOptions = DhcpOptionsDialog.BuildDhcpDefaultOptions( interfaceData );
      interfaceData['dhcp-server']['options'] = defaultOptions;

      // display warning "dhcp options changed"
      $( "#ID_NETCARD_BUTTON_OPTIONS_HINT" + this.dynid ).css( "display", "" );
      new Animation( $( '#ID_NETCARD_BUTTON_OPTIONS' + this.dynid ), "flash" );
    }
  }

  //-----------------------------------------------------------------------------
  //  PrepareResponseValues(): processes the response data to get the 
  //  needed values for the controls. 
  //
  PrepareResponseValues( options )
  {
    options.data = {
      "ipv4":        options.data['ipv4']          || "",
      "dns":         options.data['dns']           || "",
      "ipv4mask":    options.data['ipv4mask']      || "",
      "status":      options.data['status']        || "DOWN",
      "name":        options.data['name']          || "",
      "displayname": options.data['displayname']   || options.data['name'] || "",
      "configured":  options.data['configured'] != null ? options.data['configured'] : true,
      "type":        !options.data['configured'] ? "noConfig" : options.data['type'] || "static",
      "gateway":     options.data['gateway']       || "",
      "dhcp-server": options.data['dhcp-server']   || {},
      "hostname":    options.data['hostname']      || "",      // needed by dhcp options
      "prepend":     options['data']['prepend']    || {},
      "supersede":   options['data']['supersede']  || {},
    };

    if( options.data['dhcp-server'] )  {
      options.data['dhcp-server'] = {
        "ip-start":   options.data['dhcp-server']['ip-start']   || "",
        "ip-end":     options.data['dhcp-server']['ip-end']     || "",
        "lease-time": options.data['dhcp-server']['lease-time'] || [],
        "options":    options.data['dhcp-server']['options']    || {},
        "enable":     options.data['dhcp-server']['enable']     || false,
        "leases":     options.data['dhcp-server']['leases']     || []
      };
    }

    //=============================================================================
    //  dhcp values
    //=============================================================================

    // store complex structure in local variables for easier reading 
    if( "dhcp-server" in options['data'] ) {

      if( "enable" in options['data']['dhcp-server'] && options['data']['type'] === "static" ) {
        this.isDhcpIsActive = options['data']['dhcp-server']['enable'];
      }
      else {
        this.isDhcpIsActive = false;
      }

      if( "ip-start" in options['data']['dhcp-server'] ) {
        this.dhcpPoolStart = options['data']['dhcp-server']['ip-start'];
      }

      if( "ip-end" in options['data']['dhcp-server'] ) {
        this.dhcpPoolEnd = options['data']['dhcp-server']['ip-end'];
      }
    }
    else  {
      options['data']['dhcp-server'] = {
        "enable": false,
        "ip-start": "",
        "ip-end": "",
        "leases": [],
        "lease-time": [],
        "options": {},
      };
    }
  }
}


//=============================================================================
//  BUILDER 
//=============================================================================

WidgetNetcard.Register( '/forms/system/network/widgets/controls/netcard' );
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          03.02.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class ChangeIpAddressDialog extends Viewable
{
  //-------------------------------------------------------
  //  REMOVE
  //
  destructor()
  {
    if( this.verifyTimer )  {
      clearInterval( this.verifyTimer );
    } 
  }
  
  //-------------------------------------------------------
  //  DIALOG LOAD
  //
  Load( options )
  {
    // variables
    var self = this;
    this.isSavePressed = false;
    this.isEditMode = false;
    this.verifyTimer = null;

    // edit mode?
    if( 'data' in options && Object.keys( options['data'] ).length !== 0 )  {
      $( '#ID_TITLE' + this.dynid ).text( $.lang( "{$change_ip_dialog_title_edit}" ) );
      this.isEditMode = true;
    }
    else  {
      $( '#ID_TITLE' + this.dynid ).text( $.lang( "{$change_ip_dialog_title_new}" ) );
      this.createOptions.data = {
        ipAddress:        "",
      };
    }

    $( '#ID_DNS_IP_ADDRESS' + this.dynid ).edit( {
      size: CONST.size.L,
      status: "normal",
      value: options.data['ipAddress'],
      errorAnchor: ".dialog-error",
      validate: Validate.IsIpAddress
    } );


    $( '#ID_DNS_IP_ADDRESS' + this.dynid ).edit( "focus" );

  }//
  // load


  //-------------------------------------------------------
  //  DIALOG EVENTS 
  //  
  OnEvent( eventName, data )
  {
    switch( eventName ) {
      // buttons
      case 'button': {
        if( this.OnSave( ) == true && data == "IDOK" )  {
          this.Destroy();
          this.FireEvent( 'result', this.createOptions.data );
        }
        else if( data == "IDCANCEL" )  {
          this.Destroy();
          this.FireEvent( 'result', false );
        }
      }
      break;

      // keyboard: ENTER (true) and ESC (false)
      case 'canclose': {
        if( data == true )  {   // <ENTER>
          if( this.OnSave() == true ) {
            this.FireEvent( 'result', this.createOptions.data );
            return true;
          }
          else  {
            return false;
          }
        }
        else  { // <ESC>
          this.FireEvent( 'result', false );
          return true;
        }
      }
      break;
    }

    return true;
  }

  //-------------------------------------------------------
  //  SAVE
  //
  OnSave()
  {
    // enable verify timer
    var self = this;
    if( !this.isSavePressed )  {
      this.verifyTimer = setInterval(() => {
        self.VerifyForm( );
      }, 1000 );
      this.isSavePressed = true;
    }

    // verify
    if( !this.VerifyForm() )  {
      return false;
    }

    let ipAddress = $( '#ID_DNS_IP_ADDRESS' + this.dynid ).edit( "option", "value" );

    
    this.createOptions.data['newIpAddress'] = ipAddress;
    return true;
  }

  //-----------------------------------------------------------------------------
  //  VerifyForm(): verifies complete user input
  //
  VerifyForm( )
  {
    // we do not very the controls until <ok> is pressed
    if( !this.isSavePressed )  {
      return true;
    }

    let isError = false;


    let registrar = $( '#ID_DNS_IP_ADDRESS' + this.dynid ).edit( "option", "value" );

    // --- test #1: ip address ---
    if( !isError )  {
      if( !Validate.IsIpAddress( registrar ) ) {
        isError = true;
        $( '#ID_DNS_IP_ADDRESS' + this.dynid ).edit( "option", "error", "{$const_error_novalidip}" );
        $( '#ID_DNS_IP_ADDRESS' + this.dynid ).edit( "focus" );
      }
      else {
        $( '#ID_DNS_IP_ADDRESS' + this.dynid ).edit( "option", "error", null );
      }
    }

    // enable/disable <OK>
    if( isError )   {
      this.$controls['IDOK'].button( 'option', 'status', 'disabled' );
    }
    else  {
      this.$controls['IDOK'].button( 'option', 'status', 'default' );
    }

    // result
    return !isError;
  }

  //-------------------------------------------------------
  //  STATIC 
  //-------------------------------------------------------

  static Show( options )
  {
    var createdata = $.extend( true, [], {
      view: '/forms/system/network/dialog/changeIpAddress',
      type: 'dialog',
      layout: 'classic',
      size: "28em",
      height: "16em",
      buttons: [
        { id: 'IDCANCEL', size: 13, status: 'normal', text: '{$const_dialog_button_cancel}' },
        { id: 'IDOK', size: 13, status: 'hilight', text: '{$const_dialog_button_apply}' }
      ]
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

ChangeIpAddressDialog.Register( '/forms/system/network/dialog/changeIpAddress' );
//=============================================================================
//
//  File: dhcpOptions.js
//  (c) 2021 by CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          01.02.2021
//
//  Description:
//          DHCP-Options (monster-)dialog
//
//  Lease/Reservation dataset, record format:  
//    {
//      macaddr: "",
//      ipv4: "",
//      status: "fix",
//      expires: 0,
//      hostname: "",
//    }
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class DhcpOptionsDialog extends Viewable
{
  //-------------------------------------------------------
  //  Load(): Initialize dialog content
  //
  Load( options )
  {
    // copy the data given by upper layer
    this.data = $.clone( this.createOptions['data'] );

    // error mode prevent change of section
    this.isError = false;

    // declaration of data 
    this.dhcpLeasesDataset = new Dataset( { id: 'macaddr' }, [] );

    // dialog parts
    this.dhcpOptionCurrentEdit = null;
    //console.log( "this.dhcpOptionCurrentEdit", this.dhcpOptionCurrentEdit );

    this.dhcpReserveCurrentEdit = null;

    // initialize the dialog sections
    this.LoadDialogFrame( this.createOptions );
    this.LoadSectionGeneral( this.createOptions );
    this.LoadSectionOptions( this.createOptions );
    this.LoadSectionLeaseList( this.createOptions );
    this.LoadSectionReservationList( this.createOptions );

    // get dhcp-options from folder "admin/config"
    this.dhcpOptionList = Data.Instance().AdminConfig( "dhcp" );
    if( ! this.dhcpOptionList ) {
      Data.Instance().On( "ADMINCONFIG.DHCP_OPTIONS", this.etag, this.OnXhrDhcpOptions.bind( this ) );
      Data.Instance().XhrRequest( "ADMINCONFIG.DHCP_OPTIONS" );
    }
    else {
      this.dhcpOptionList = this.dhcpOptionList['dhcp-options'];
      this.FormFillDhcpOptionList( );
    }

    // --- fill data ---
    this.FillData( );

    // start with "general"
    this.SelectOption( "ID_GENERAL" );

  }

  //-------------------------------------------------------
  //  OnEvent(): get central modified/error messages
  //
  OnEvent( eventName, param )
  {
    if( eventName == "error" )  {
      this.OnErrorStatusChanged( param );
    }
    else if( eventName == "modified" && param == 0 )  {
      this.$controls['IDAPPLY'].button( "option", "status", "disabled" );
    }
    else if( eventName == "modified" && param > 0 && this.dhcpOptionCurrentEdit == null && this.dhcpReserveCurrentEdit == null && this.isNewReservation == false ) {
      this.$controls['IDAPPLY'].button( "option", "status", null );
    }
    else if( eventName == "button" && param == "IDAPPLY" )  {
      this.OnDialogApply();
    }
    else if( eventName == "button" && param == "IDCANCEL" ) {
      this.OnDialogCancel();
    }
  }

  //-------------------------------------------------------
  //  enable/disable all save buttons
  //
  OnErrorStatusChanged( numError )
  {
    //console.log( "error status: ", numError );
    if( numError > 0 )  {
      $( "#ID_RESERVATION_APPLY" + this.dynid ).button( "option", "status", "disabled" );
      $( "#ID_DHCP_OPTIONEDIT_APPLY" + this.dynid ).button( "option", "status", "disabled" );
      this.$controls['IDAPPLY'].button( "option", "status", "disabled" );
    }
    else  {
      $( "#ID_RESERVATION_APPLY" + this.dynid ).button( "option", "status", "normal" );
      $( "#ID_DHCP_OPTIONEDIT_APPLY" + this.dynid ).button( "option", "status", "normal" );
      if( this.Option( "modified" ) == 0 )  {
        this.$controls['IDAPPLY'].button( "option", "status", null );
      }
    }

  }

  //-------------------------------------------------------
  //  LoadDialogFrame(): Initialize sidebar + header
  //
  LoadDialogFrame( options )
  {
    // --- general dialog controls ---
    $( "#ID_SIDEBAR" + this.dynid ).treemenu( {
      type: 'accordion',
      data: [
        { type: 'item', text: "{$dhcp_menu_general}", id: "ID_GENERAL" },
        { type: 'item', text: "{$dhcp_menu_options}", id: "ID_OPTIONS" },
        { type: 'item', text: "{$dhcp_menu_leases}", id: "ID_LEASES" },
        { type: 'item', text: "{$dhcp_menu_reservations}", id: "ID_RESERVATIONS" }
      ],
      click: this.OnSidebarClick.bind( this )
    } );
    $( "#ID_CONTENT" + this.dynid ).optional( {} );
  }


  //-------------------------------------------------------
  //  SECTION "GENERAL"
  //-------------------------------------------------------

  //-------------------------------------------------------
  //  LoadSectionGeneral(): "general" in sidebar
  //
  LoadSectionGeneral( options )
  {
    // --- controls "general" ---
    $( "#ID_IPADDRESS" + this.dynid ).edit( {
      size: 30, status: "readonly",
      //validate: Validate.IsIpAddress
    } );
    $( "#ID_SUBNETMASK" + this.dynid ).edit( {
      size: 30, status: "readonly",
      //validate: Validate.IsNetmask
    } );
    $( "#ID_SCOPE_START" + this.dynid ).edit( {
      size: 30,
      //validateDepends: [ "ID_SUBNETMASK" + this.dynid, "ID_IPADDRESS" + this.dynid ],
      validate: this.ValidateHelperDhcpScope.bind( this )

    } );
    $( "#ID_SCOPE_END" + this.dynid ).edit( {
      size: 30,
      //validateDepends: [ "ID_SUBNETMASK" + this.dynid, "ID_IPADDRESS" + this.dynid ],
      validate: this.ValidateHelperDhcpScope.bind( this )
    } );
  }

  //-------------------------------------------------------
  //  helper for validate dhcp scope
  //
  ValidateHelperDhcpScope( ev )
  {
    let subnetMask = $( "#ID_SUBNETMASK" + this.dynid ).edit( "option", "value" );
    let ipAddress = $( "#ID_IPADDRESS" + this.dynid ).edit( "option", "value" );
    if( Validate.IsIpInSubnet( ipAddress, subnetMask, ev.value ) ) {
      return ev.error = null;
    }
    ev.error = "{$dhcp_scope_error_range}";
  }


  //-------------------------------------------------------
  //  validate new/changed reservation
  //
  ValidateSectionGeneral(  )
  {
    let numError = 0;
    numError = numError + $( "#ID_IPADDRESS" + this.dynid ).edit( "validate" );
    numError = numError + $( "#ID_SUBNETMASK" + this.dynid ).edit( "validate" );
    numError = numError + $( "#ID_SCOPE_START" + this.dynid ).edit( "validate" );
    numError = numError + $( "#ID_SCOPE_END" + this.dynid ).edit( "validate" );
    return numError;
  }

  //-------------------------------------------------------
  //  SECTION "DHCP OPTIONS"
  //-------------------------------------------------------

  //-------------------------------------------------------
  //  LoadSectionOptions(): "dhcp-options" in sidebar
  //
  LoadSectionOptions( options )
  {
    // grid: list of all dhcp options
    $( "#ID_DHCP_OPTIONLIST" + this.dynid ).grid( {
      size: -1,
      height: 11,
      scrollbar: 'overlay',
      selection: "single",
      header: {
        text: { text: "{$dhcp_optionheader_option}", size: 23 },
        value: { text: "{$dhcp_optionheader_value}", size: 45 },
        active: { text: "{$dhcp_optionheader_active}", size: 8 }
      },
      renderCell: ( id, record ) => {
        let text = record[id];
        // convert bool
        if( id == "active" ) {
          text = record[id] ? "{$dhcp_option_active_yes}" : "{$dhcp_option_active_no}";
        }
        // combine array
        else if( Array.isArray( text ) ) {
          text = text.join( ", " );
        }
        return { text: text };
      },
      data: [],
      result: this.OnDhcpOptionSelect.bind( this )
    } );

    // edit control section
    $( "#ID_DHCP_OPTIONENABLE" + this.dynid ).choice( {
      size: 30, type: "check", status: "normal", text: "{$dhcp_option_enabled_check}",
      validateDepends: [ "ID_DHCP_OPTIONVALUE" + this.dynid ],
      result: this.OnDhcpOptionEnableChange.bind( this )
    } );
    $( "#ID_DHCP_OPTIONNAME" + this.dynid ).edit( {
      size: 30, status: "readonly"
    } );
    $( "#ID_DHCP_OPTIONVALUE" + this.dynid ).edit( {
      size: 30, status: "normal",
      validateDepends: [ "ID_DHCP_OPTIONENABLE" + this.dynid ],
      validate: this.ValidateHelperDhcpOption.bind( this )
    } );
    // buttons
    $( "#ID_DHCP_OPTIONVALUE_AUTO" + this.dynid ).button( {
      size: 4, icon: "mdi mdi-auto-fix", status: "normal", tooltip: "{$dhcp_option_autovalue_tooltip}",
      click: this.OnDhcpOptionAutoClick.bind( this )
    } );
    $( "#ID_DHCP_OPTIONEDIT_CANCEL" + this.dynid ).button( {
      size: 4, icon: "mdi mdi-undo", status: "normal", tooltip: "{$const_dialog_button_cancel}",
      click: this.OnDhcpOptionCancelClick.bind( this )
    } );
    $( "#ID_DHCP_OPTIONEDIT_APPLY" + this.dynid ).button( {
      size: 4, icon: "mdi mdi-check", status: "normal", tooltip: "{$const_dialog_button_apply}",
      click: this.OnDhcpOptionApplyClick.bind( this )
    } );
    
    // group of controls for ".broadcast"
    this.$optionControlGroup = $( 
          "#ID_DHCP_OPTIONENABLE" + this.dynid
      + ", #ID_DHCP_OPTIONNAME" + this.dynid
      + ", #ID_DHCP_OPTIONVALUE" + this.dynid
      + ", #ID_DHCP_OPTIONVALUE_AUTO" + this.dynid
      + ", #ID_DHCP_OPTIONEDIT_CANCEL" + this.dynid
      + ", #ID_DHCP_OPTIONEDIT_APPLY" + this.dynid
    );

    // disable controls at the beginning
    this.OnDhcpOptionSelect( null );
  }

  //-------------------------------------------------------
  //  helper for validate dhcp option
  //
  ValidateHelperDhcpOption( ev )
  {
    //console.log( "ValidateHelperDhcpOption", ev );

    // option not active? no error!
    if( $( "#ID_DHCP_OPTIONENABLE" + this.dynid ).choice( "option", "value" ) == false )  {
      ev.error = null;
    }

    // common for all: must not be empty!
    else if( ev.value == "" ) {
      ev.error = "{$const_error_empty}";
    }

    // verify ipmask
    else if( this.dhcpOptionCurrentEdit['type'] == "ipmask" ) {
      Validate.IsNetmask( ev );
    }

    // verify ipaddr
    else if( this.dhcpOptionCurrentEdit['type'] == "ipaddr" ) {
      Validate.IsIpAddress( ev );
    }

    // verify array of ip
    else if( this.dhcpOptionCurrentEdit['type'] == "*ipaddr" ) {

      // iterate through the list and stop at first error
      let newValueList = ev.value.split( " " );
      for( let currentValue of newValueList ) {
        if( !Validate.IsIpAddress( currentValue ) ) {
          ev.error = "{$const_error_novalidip}";
          return;
        }
      }
      // all ip's are verified
      ev.error = null;
    }

    // unknown type: no error 
    else  {
      // default
      ev.error = null;

    }

  }  

  //-------------------------------------------------------
  // event when selection on dhcp grid changes
  //
  OnDhcpOptionSelect( selectedList )
  {
    if( typeof selected == "string" || Array.isArray( selectedList ) ) {
      let selected = selectedList[0];
      let dhcpOption = this.dhcpGridList.find( ( element ) => element['id'] == selected );
      this.dhcpOptionCurrentEdit = dhcpOption;
      //console.log( "this.dhcpOptionCurrentEdit", this.dhcpOptionCurrentEdit );

      this.$controls['IDAPPLY'].button( "option", "status", "disabled" );

      // convert array to string
      let editValue = dhcpOption['value'];
      if( Array.isArray( editValue ) ) {
        editValue = editValue.join( ", " );
      }

      // enable controls and set values
      this.$optionControlGroup.broadcast( "option", "status", null );
      $( "#ID_DHCP_OPTIONENABLE" + this.dynid ).choice( "option", "value", dhcpOption['active'] );
      $( "#ID_DHCP_OPTIONNAME" + this.dynid ).edit( "option", "value", dhcpOption['text'] );
      $( "#ID_DHCP_OPTIONVALUE" + this.dynid ).edit( "option", "value", editValue );

      // set filter on edit control
      if( dhcpOption['type'] == "ipaddr" ) {
        $( "#ID_DHCP_OPTIONVALUE" + this.dynid ).edit( "option", "filter", { allow: "0123456789.", deny: null } );
      }
      else if( dhcpOption['type'] == "*ipaddr" ) {
        $( "#ID_DHCP_OPTIONVALUE" + this.dynid ).edit( "option", "filter", { allow: " ,0123456789.", deny: null } );
      }
      else if( dhcpOption['type'] == "host" ) {
        $( "#ID_DHCP_OPTIONVALUE" + this.dynid ).edit( "option", "filter", { allow: "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-", deny: null } );
      }
      else if( dhcpOption['type'] == "integer" ) {
        $( "#ID_DHCP_OPTIONVALUE" + this.dynid ).edit( "option", "filter", { allow: "0123456789", deny: null } );
      }
      else {
        $( "#ID_DHCP_OPTIONVALUE" + this.dynid ).edit( "option", "filter", null );
      }
    }
    else {
      $( "#ID_DHCP_OPTIONVALUE" + this.dynid ).edit( "option", "change", null );
      $( "#ID_DHCP_OPTIONLIST" + this.dynid ).grid( "option", "selected", [] );
      this.$optionControlGroup.broadcast( "option", "value", "" );
      this.$optionControlGroup.broadcast( "option", "status", "disabled" );
      this.dhcpOptionCurrentEdit = null;
      //console.log( "this.dhcpOptionCurrentEdit", this.dhcpOptionCurrentEdit );

      this.$controls['IDAPPLY'].button( "option", "status", null );

    }

    // reset error counter
    //this.Option( "reset" );  
    this.Option( "error", 0 );

  }

  //-------------------------------------------------------
  //  start validate dhcp option
  //
  ValidateDhcpOption()
  {
    let numError = 0;
    numError = numError + $( "#ID_DHCP_OPTIONENABLE" + this.dynid ).choice( "validate" );
    numError = numError + $( "#ID_DHCP_OPTIONVALUE" + this.dynid ).edit( "validate" );
    return numError;
  }

  //-------------------------------------------------------
  //  switch dhcp option on/off
  //
  OnDhcpOptionEnableChange( isEnabled )
  {
    if( isEnabled ) {
      $( "#ID_DHCP_OPTIONVALUE" + this.dynid ).edit( "focus" );
    }
    else {
      $( "#ID_DHCP_OPTIONVALUE" + this.dynid ).edit( "option", "value", null );
      $( "#ID_DHCP_OPTIONVALUE" + this.dynid ).edit( "validate", false );
    }
  }

  //-------------------------------------------------------
  // set the dhcp option value to default
  //
  OnDhcpOptionAutoClick()
  {
    // set value to the given default
    let defaultValue = this.dhcpOptionCurrentEdit['default'];
    defaultValue = defaultValue.replace( "{ip}", this.data['ipv4'] );
    defaultValue = defaultValue.replace( "{mask}", this.data['ipv4mask'] );
    defaultValue = defaultValue.replace( "{hostname}", this.data['hostname'] );
    defaultValue = defaultValue.replace( "{router}", this.data['gateway'] );
    $( "#ID_DHCP_OPTIONVALUE" + this.dynid ).edit( "option", "value", defaultValue );

    // clear error while validate
    $( "#ID_DHCP_OPTIONVALUE" + this.dynid ).edit( "validate" );
  }

  //-------------------------------------------------------
  //  if selection changes or "apply" is clicked
  //
  ApplyDhcpOptionAuto()
  {
    return this.OnDhcpOptionApplyClick( );
  }

  //-------------------------------------------------------
  // cancel edit and discard changes
  //
  OnDhcpOptionCancelClick()
  {
    this.OnDhcpOptionSelect( null );
    this.Option( "error", 0 );
    //this.Option( "reset" );
  }
  
  //-------------------------------------------------------
  // (try to) save changes to dhcp option list
  //
  OnDhcpOptionApplyClick()
  {
    let isOptionEnabled = $( "#ID_DHCP_OPTIONENABLE" + this.dynid ).choice( "option", "value" );

    // remove value and write back (dhcpOptionCurrentEdit is a reference into dhcpGridList)
    if( !isOptionEnabled ) {
      if( this.dhcpOptionCurrentEdit !== null )  {
        this.dhcpOptionCurrentEdit['active'] = false;
        this.dhcpOptionCurrentEdit['value'] = "";
      }

      $( "#ID_DHCP_OPTIONLIST" + this.dynid ).grid( "option", "data", this.dhcpGridList );

      this.Option( "modified", 1 );
      return true;
    }

    // validate dhcp option + write back to list
    if( this.ValidateDhcpOption( ) == 0 ) {

      if( this.dhcpOptionCurrentEdit !== null ) {
        this.dhcpOptionCurrentEdit['active'] = true;
      }
      let newValue = $( "#ID_DHCP_OPTIONVALUE" + this.dynid ).edit( "option", "value" );

      // if type (like 'int', 'string', 'ip') starts with * it's a list
      if( this.dhcpOptionCurrentEdit['type'].indexOf( "*" ) == 0 ) {
        newValue = newValue.split( "," ).map( item => item.trim() );
      }
      this.dhcpOptionCurrentEdit['value'] = newValue;

      // write changed grid list back & clear edit control group
      $( "#ID_DHCP_OPTIONLIST" + this.dynid ).grid( "option", "data", this.dhcpGridList );
      this.OnDhcpOptionSelect( null );

      this.Option( "modified", 1 );
      return true;
    }

    return false;
  }

  //-------------------------------------------------------
  //  LEASE LIST
  //-------------------------------------------------------

  //-------------------------------------------------------
  //  LoadSectionLeaseList(): "leases" in sidebar
  //
  LoadSectionLeaseList( options )
  {
    // --- controls "leases" ---
    $( "#ID_DHCP_LEASELIST" + this.dynid ).grid( {
      size: -1,
      height: 12,
      scrollbar: 'overlay',
      data: this.dhcpLeasesDataset,
      dataId: "macaddr",
      dataFilter: ( id, record ) => {
        return ( record['type'] == "dynamic" );
      },
      header: {
        macaddr: { text: "{$dhcp_leaseheader_macaddr}", size: 18, resizeable: true, sortable: true },
        ipv4: { text: "{$dhcp_leaseheader_ipaddr}", size: 18, resizeable: true, sortable: true },
        hostname: { text: "{$dhcp_leaseheader_hostname}", size: 24, resizeable: true, sortable: true },
        expires: { text: "{$dhcp_leaseheader_date}", size: 16, align: "left", resizeable: false, sortable: true }
      },
      renderCell: ( id, record ) => {
        let text = record[ id ];
        // convert date/time
        if( id == "expires" ) {
          if( parseInt( text ) > 0 )  {
            text = Time.FormatFromUnix( text );
          }
          else  {
            text = "";
          }
        }
        return { text: text };
      },      
      renderStatus: ( id, record ) => {
        if( record['status'] == "deleted" ) {
          return "red striked";
        }
        return "normal";
      },
      result: this.OnDhcpLeasesSelect.bind( this )
    } );  
    
    $( "#ID_DHCP_LEASE_DELETE" + this.dynid ).button( { 
      size: 15, 
      text: "{$dhcp_lease_delete}",
      status: "normal",
      click: this.OnDhcpLeaseDeleteClick.bind( this )
    } );
    $( "#ID_DHCP_LEASE_RESERVE" + this.dynid ).button( { 
      size: 15, 
      text: "{$dhcp_lease_reserve}" ,
      status: "normal",
      click: this.OnDhcpLeaseReserveClick.bind( this )
    } );
    this.$leaseControlGroup = $( "#ID_DHCP_LEASE_DELETE" + this.dynid
      + ", #ID_DHCP_LEASE_RESERVE" + this.dynid
    );
    this.OnDhcpLeasesSelect( null );

  }  

  //-------------------------------------------------------
  // event when selection on lease grid changes
  //
  OnDhcpLeasesSelect( selected )
  {
    // something is selected
    if( selected && selected.length > 0 ) {
      // enable controls
      this.$leaseControlGroup.broadcast( "option", "status", null );

      // manage text for do/undo
      let record = $( "#ID_DHCP_LEASELIST" + this.dynid ).grid( "item", selected[0], "get" );
      if( record['status'] == "deleted" ) {
        $( "#ID_DHCP_LEASE_DELETE" + this.dynid ).button( "option", "text", "{$dhcp_lease_delete_undo}" );
      }
      else {
        $( "#ID_DHCP_LEASE_DELETE" + this.dynid ).button( "option", "text", "{$dhcp_lease_delete}" );
      }
    }
    // nothing selected
    else {
      // disable controls & default button text
      $( "#ID_DHCP_LEASE_DELETE" + this.dynid ).button( "option", "text", "{$dhcp_lease_delete}" );
      this.$leaseControlGroup.broadcast( "option", "status", "disabled" );
    }
  }

  //-------------------------------------------------------
  // event "delete lease" clicked
  //
  OnDhcpLeaseDeleteClick()
  {
    // get selected record and invert deleted status
    let selected = $( "#ID_DHCP_LEASELIST" + this.dynid ).grid( "option", "selected" );
    let record = $( "#ID_DHCP_LEASELIST" + this.dynid ).grid( "item", selected[0], "get" );
    record['status'] = ( record['status'] == "deleted" ) ? "active" : "deleted";

    // update record & clear selected status
    $( "#ID_DHCP_LEASELIST" + this.dynid ).grid( "item", selected, "update", record );
    $( "#ID_DHCP_LEASELIST" + this.dynid ).grid( "item", selected, "select", false );

    this.Option( "modified", 1 );

    // after clicking "reserve" the record was in status deleted, undo must remove
    // the reserved record as well. Therefore we search for the mac-address and
    // remove it. 
    if( record['status'] == "active" )  {
      var reservedRecord = this.dhcpLeasesDataset.Get( "*" + record['macaddr'] );
      console.log( reservedRecord );
      this.dhcpLeasesDataset.Delete( "*" + record['macaddr'] );
    }
    
    //console.log( "OnDhcpLeaseDeleteClick" );
    //this.DataLeaseDump();
  }

  //-------------------------------------------------------
  // event "lease as reservation" clicked
  //
  OnDhcpLeaseReserveClick()
  {
    // get selected record and invert deleted status
    let selected = $( "#ID_DHCP_LEASELIST" + this.dynid ).grid( "option", "selected" );
    let record = $( "#ID_DHCP_LEASELIST" + this.dynid ).grid( "item", selected[0], "get" );
    //record['type'] = "fix";
    //record['status'] = "new";

    record['status'] = "deleted";
    $( "#ID_DHCP_LEASELIST" + this.dynid ).grid( "item", selected, "update", record );

    // create new reservation
    let newRecord = {
      type: "fix",
      status: "new",
      macaddr: "*" + record['macaddr'],
      ipv4: record['ipv4'],
      expires: 0
    };
    this.dhcpLeasesDataset.Add( newRecord );

    // update record & clear selected status
    //$( "#ID_DHCP_LEASELIST" + this.dynid ).grid( "item", selected, "update", record );
    $( "#ID_DHCP_LEASELIST" + this.dynid ).grid( "option", "selected", null );
    this.OnDhcpLeasesSelect( null );

    this.Option( "modified", 1 );

    //console.log( "OnDhcpLeaseReserveClick" );
    //this.DataLeaseDump();
  }


  //-------------------------------------------------------
  //  RESERVATION LIST
  //-------------------------------------------------------

  //-------------------------------------------------------
  //  LoadSectionReservationList(): "reservations" in sidebar
  //
  LoadSectionReservationList( options )
  {
    // --- controls "reservations" ---
    $( "#ID_DHCP_RESERVATIONLIST" + this.dynid ).grid( {
      size: -1,
      height: 12,
      scrollbar: 'overlay',
      data: this.dhcpLeasesDataset,
      dataId: "macaddr",
      dataFilter: ( id, record ) => {
        return ( record['type'] == "fix" );
      },
      renderStatus: ( id, record ) =>  {
        if( record['status'] == "new" ) {
          return "green";
        }
        else if( record['status'] == "fix-deleted" ) {
          return "red striked";
        }
        return "normal";
      },
      renderCell: ( field, record ) => {
        let text = record[ field ];
        if( field == "status" )  {
          switch( text )  {
            case 'active':  text = $.lang( "{$dhcp_lease_status_active}" );   break;
            case 'deleted': text = $.lang( "{$dhcp_lease_status_deleted}" );  break;
            case 'new':     text = $.lang( "{$dhcp_lease_status_new}" );      break;
            case 'edit':    text = $.lang( "{$dhcp_lease_status_edit}" );     break;
            default: text = "";
          }
        }
        if( field == "macaddr" && text[0] == "*" )  {
          text = text.substr( 1 );
        }
        return { text: text };
      },
      header: {
        macaddr: { text: "{$dhcp_leaseheader_macaddr}", size: 18, resizeable: true, sortable: true },
        ipv4: { text: "{$dhcp_leaseheader_ipaddr}", size: 45, resizeable: true, sortable: true },
        status: { text: "Status", size: 10, resizeable: false, sortable: true }
      },
      result: this.OnDhcpReservationSelect.bind( this )
    } );

    $( "#ID_DHCP_RESERVATION_NEW" + this.dynid ).button( {
      size: 12,
      status: "normal",
      text: "{$dhcp_reserve_new}",
      click: this.OnDhcpReserveNew.bind( this )
    } );
    $( "#ID_DHCP_RESERVATION_DELETE" + this.dynid ).button( {
      size: 12,
      status: "normal",
      text: "{$dhcp_reserve_delete}",
      click: this.OnDhcpReserveDelete.bind( this )
    } );

    $( "#ID_RESERVATION_MAC" + this.dynid ).edit( {
      size: 30,
      status: "normal",
      //filter: { allow: "abcdefABCDEF0123456789:-", deny: null } ,
      result: ( value ) => {
        var newValue = "";
        for( var i = 0; i < value.length; i++ )  {
          switch( value[ i ] ) {
            case 'a':
            case 'A': newValue += "A";  break;
            case 'b':
            case 'B': newValue += "B";  break;
            case 'c':
            case 'C': newValue += "C";  break;
            case 'd':
            case 'D': newValue += "D";  break;
            case 'e':
            case 'E': newValue += "E";  break;
            case 'f':
            case 'F': newValue += "F";  break;
            case ':':
            case '-': newValue += ":";  break;
            case '0':case '1':case '2':case '3':case '4':
            case '5':case '6':case '7':case '8':case '9':
              newValue += value[ i ];  
              break;
          }
        }
        if( newValue != value ) {
          $( "#ID_RESERVATION_MAC" + this.dynid ).edit( "option", "value", newValue );
        }
      },
      validate: this.ValidateHelperReservationMac.bind( this )
    } );
    $( "#ID_RESERVATION_IP" + this.dynid ).edit( {
      size: 30,
      status: "normal",
      filter: { allow: "0123456789.", deny: null } ,
      validate: this.ValidateHelperReservationIp.bind( this )
    } );
    $( "#ID_RESERVATION_CANCEL" + this.dynid ).button( {
      size: 4, icon: "mdi mdi-undo", status: "normal", tooltip: "{$dhcp_reserve_cancel_tooltip}",
      click: this.OnDhcpReservationCancelClick.bind( this )
    } );
    $( "#ID_RESERVATION_APPLY" + this.dynid ).button( {
      size: 4, icon: "mdi mdi-check", status: "normal", tooltip: "{$dhcp_reserve_apply_tooltip}",
      click: this.OnDhcpReservationApplyClick.bind( this )
    } );

    // build control group for enable/disable
    this.$reservationControlGroup = $( "#ID_RESERVATION_MAC" + this.dynid
      + ", #ID_RESERVATION_IP" + this.dynid
      + ", #ID_RESERVATION_CANCEL" + this.dynid
      + ", #ID_RESERVATION_APPLY" + this.dynid
    );

    // disable controls at the beginning
    this.OnDhcpReservationSelect( null );

  }  

  //-------------------------------------------------------
  //  validate helper for mac address
  //
  ValidateHelperReservationMac( ev )
  {
    // check mac itself and duplicated mac
    if( !Validate.IsMacAddress( ev.value ) ) {
      ev.error = "{$const_error_novalidmac}";
      return false;
    }
    if( this.isNewReservation && ( this.dhcpLeasesDataset.Has( ev.value ) || this.dhcpLeasesDataset.Has( "*" + ev.value ) ) ) {
      ev.error = "{$dhcp_reserve_error_duplicatemac}";
      return false;
    }

    // no error
    ev.error = null;
    return true;
  }

  //-------------------------------------------------------
  //  validate helper for mac address
  //
  ValidateHelperReservationIp( ev )
  {
    // check ip address format
    if( !Validate.IsIpAddress( ev.value ) ) {
      ev.error = "{$const_error_novalidip}";
      return false;
    }
    // ip must not be duplicated
    if( this.isNewReservation && this.dhcpLeasesDataset.Has( ev.value, "ipv4" ) ) {
      ev.error = "{$dhcp_reserve_error_duplicateip}";
      return false;
    }
    // ip must in ip scope
    if( ! Validate.IsIpInSubnet( this.data['ipv4'], this.data['ipv4mask'], ev.value ) ) {
      ev.error = "{$dhcp_reserve_error_ipnotinrange}";
      return false;
    }
    // ip is same as netcard's ip
    if( ev.value == this.data['ipv4'] ) {
      ev.error = "{$dhcp_reserve_error_ipnetcard}";
      return false;
    }
    // ip duplicated
    if( !this.isNewReservation                            // in ..
     && this.dhcpReserveCurrentEdit != null               // ..edit mode ..
     && this.dhcpReserveCurrentEdit['ipv4'] != ev.value   // ..ip changed? 
     && this.dhcpLeasesDataset.Has( ev.value, "ipv4" )    // ..to an existing value?
     ) {
      ev.error = "{$dhcp_reserve_error_duplicateip}";
      return false;
    }

    // no error
    ev.error = null;
    return true;
  }

  //-------------------------------------------------------
  //  event when selection on reservation grid changes
  //
  OnDhcpReservationSelect( selected )
  {
    // select -> no new reservation
    this.isNewReservation = false;
    this.isMacAddressWithStar = false;

    $( "#ID_DHCP_RESERVATION_DELETE" + this.dynid ).button( "option", "text", "{$dhcp_reserve_delete}" );
    
    // something is selected: fill data & enable controls
    if( selected && selected.length > 0 ) {
      $( "#ID_DHCP_RESERVATION_DELETE" + this.dynid ).button( "option", "status", null );

      let record = $( "#ID_DHCP_RESERVATIONLIST" + this.dynid ).grid( "item", selected[ 0 ], "get" );
      this.dhcpReserveCurrentEdit = record;

      if( record['status'] == "fix-deleted" )  {
        $( "#ID_DHCP_RESERVATION_DELETE" + this.dynid ).button( "option", "text", "{$dhcp_reserve_delete_undo}" );
      }
      else  {
        this.$reservationControlGroup.broadcast( "option", "status", null );
        let macaddr = record['macaddr'];
        if( macaddr[0] == "*" )  {
          macaddr = macaddr.substr( 1 );
          this.isMacAddressWithStar = true;
        }
        $( "#ID_RESERVATION_MAC" + this.dynid ).edit( "option", "value", macaddr );
        $( "#ID_RESERVATION_IP" + this.dynid ).edit( "option", "value", record['ipv4'] );
        $( "#ID_RESERVATION_IP" + this.dynid ).edit( "focus" );
      }

    }
    // no selection, disable and remove value
    else {
      this.$reservationControlGroup.broadcast( "option", "value", null );
      this.$reservationControlGroup.broadcast( "option", "error", null );
      this.$reservationControlGroup.broadcast( "option", "status", "disabled" );
      $( "#ID_DHCP_RESERVATION_DELETE" + this.dynid ).button( "option", "status", "disabled" );
      $( "#ID_DHCP_RESERVATION_NEW" + this.dynid ).button( "option", "status", null );
      this.dhcpReserveCurrentEdit = null;

      $( "#ID_DHCP_RESERVATIONLIST" + this.dynid ).grid( "option", "selected", null );
    }

    // clear error & validate mode
    //this.Option( "reset" );
    //this.Option( "error", 0 );
    //this.Option( "validate", 0 );
  }

  //-------------------------------------------------------
  //  create a new reservation 
  //
  OnDhcpReserveNew()
  {
    // empty controls & unselect in grid
    this.OnDhcpReservationSelect( null );
    this.OnDhcpReservationCancelClick();
    this.isNewReservation = true;
    this.dhcpReserveCurrentEdit = null;

    // enable controls & set focus
    this.$reservationControlGroup.broadcast( "option", "status", "normal" );
    $( "#ID_RESERVATION_MAC" + this.dynid ).edit( "focus" );

    //console.log( "OnDhcpReserveNew" );
    //this.DataLeaseDump();

  }

  //-------------------------------------------------------
  //  delete selected reservation from grid
  //
  OnDhcpReserveDelete()
  {
    // toggle deleted status
    let selected = $( "#ID_DHCP_RESERVATIONLIST" + this.dynid ).grid( "option", "selected" );
    let record = $( "#ID_DHCP_RESERVATIONLIST" + this.dynid ).grid( "item", selected[ 0 ], "get" );
    if( record['status'] == "fix-deleted" )  {
      record['status'] = "fix";
    }
    else {
      record['status'] = "fix-deleted";
    }

    // write back
    $( "#ID_DHCP_RESERVATIONLIST" + this.dynid ).grid( "item", selected[ 0 ], "update", record );

    // a new reservation might have been a lease before
    // in this case: restore the lease
    if( record['macaddr'].substr( 0, 1 ) == "*" )  {
      var mac = record['macaddr'].substring( 1 );
      var leaseRecord = this.dhcpLeasesDataset.Get( mac );
      if( leaseRecord['status'] == "deleted" ) {
        leaseRecord['status'] = "active";
        this.dhcpLeasesDataset.Update( leaseRecord );
        this.dhcpLeasesDataset.Delete( record['macaddr'] );
      }
    }
    
    // clear selection
    this.OnDhcpReservationSelect( null );

    // 
    this.Option( "modified", 1 );

    //console.log( "OnDhcpReserveDelete" );
    //this.DataLeaseDump();

  }

  //-------------------------------------------------------
  //  cancel input 
  //
  OnDhcpReservationCancelClick()
  {
    // clear controls & 
    $( "#ID_DHCP_RESERVATIONLIST" + this.dynid ).grid( "option", "selected", null );
    this.OnDhcpReservationSelect( null );
  }

  //-------------------------------------------------------
  //  save input 
  //
  OnDhcpReservationApplyClick()
  {
    // start validation
    let numError = 0;
    numError = numError + $( "#ID_RESERVATION_MAC" + this.dynid ).edit( "validate" );
    numError = numError + $( "#ID_RESERVATION_IP" + this.dynid ).edit( "validate" );

    if( numError == 0 ) {

      // get values
      let macAddr = $( "#ID_RESERVATION_MAC" + this.dynid ).edit( "option", "value" );
      let ipAddr = $( "#ID_RESERVATION_IP" + this.dynid ).edit( "option", "value" );

      if( this.isMacAddressWithStar )  {
        macAddr = "*" + macAddr;
      }

      // create new reservation
      if( this.isNewReservation ) {
        let newRecord = {
          type: "fix",
          status: "new",
          macaddr: macAddr,
          ipv4: ipAddr,
          expires: 0,
          hostname: "",
        };
        this.dhcpLeasesDataset.Add( newRecord );
      }
      // modify existing reservation
      else {

        let selected = $( "#ID_DHCP_RESERVATIONLIST" + this.dynid ).grid( "option", "selected" );
        let record = $( "#ID_DHCP_RESERVATIONLIST" + this.dynid ).grid( "item", selected[ 0 ], "get" );
        record['macaddr'] = macAddr;
        record['ipv4'] = ipAddr;

        $( "#ID_DHCP_RESERVATIONLIST" + this.dynid ).grid( "item", selected[ 0 ], "update", record );
      }      

      // clear selection        
      $( "#ID_DHCP_RESERVATIONLIST" + this.dynid ).grid( "option", "selected", null );
      this.OnDhcpReservationSelect( null );      
    }

    this.Option( "modified", 1 );

    //console.log( "OnDhcpReservationApplyClick" );
    //this.DataLeaseDump();

  }


  //-------------------------------------------------------
  //  DATA
  //-------------------------------------------------------

  DataLeaseDump()
  {
    var data = this.dhcpLeasesDataset.GetData();
    console.log( data );
  }

  //-------------------------------------------------------
  //  set data into controls (-> this.data)
  //
  FillData()
  {
    // section "general"
    $( "#ID_IPADDRESS" + this.dynid ).edit( "option", "value", this.data['ipv4'] );
    $( "#ID_SUBNETMASK" + this.dynid ).edit( "option", "value", this.data['ipv4mask'] );
    $( "#ID_SCOPE_START" + this.dynid ).edit( "option", "value", this.data['dhcp-server']['ip-start'] );
    $( "#ID_SCOPE_END" + this.dynid ).edit( "option", "value", this.data['dhcp-server']['ip-end'] );

    // Hint: dhcp options will be filled in OnXhrDhcpOptions()

    //console.log( this.data['dhcp-server']['leases'] );

    // shared dataset for section "leases" & "reservations"
    // first all leases
    for( let leaseRecord of this.data['dhcp-server']['leases'] ) {
      if( !( 'macaddr' in leaseRecord ) || !( leaseRecord['macaddr'] ) )  {
        continue;
      }
      if( leaseRecord['type'] == "dynamic" )  {
        this.dhcpLeasesDataset.Add( leaseRecord );
      }
    }

    // 2nd reservations, add * if duplicated entry
    for( let leaseRecord of this.data['dhcp-server']['leases'] ) {
      if( !( 'macaddr' in leaseRecord ) || !( leaseRecord['macaddr'] ) ) {
        continue;
      }
      if( leaseRecord['type'] == "fix" ) {
        if( this.dhcpLeasesDataset.Has( leaseRecord['macaddr'] ) )  {
          leaseRecord['macaddr'] = "*" + leaseRecord['macaddr'];
        }
        this.dhcpLeasesDataset.Add( leaseRecord );
      }
    }
    //console.log( "FillData" );
    //this.DataLeaseDump();
    
  }

  //-------------------------------------------------------
  //  build result data
  //
  FillResultData()
  {
    // Hint: this.data is already cloned.
    this.data['dhcp-server']['ip-start'] = $( "#ID_SCOPE_START" + this.dynid ).edit( "option", "value" );
    this.data['dhcp-server']['ip-end'] = $( "#ID_SCOPE_END" + this.dynid ).edit( "option", "value" );
    
    // leases & reservations
    this.data['dhcp-server']['leases'] = this.dhcpLeasesDataset.GetData( );
    for( let lease of this.data['dhcp-server']['leases'] ) {
      if( lease['macaddr'][0] == "*" )  {
        lease['macaddr'] = lease['macaddr'].substr( 1 );
      }
    }
    //console.log( this.data['dhcp-server']['leases'] );

    // encode dhcp options as needed by cosconf
    let optionList = { };
    let changedData = $( "#ID_DHCP_OPTIONLIST" + this.dynid ).grid( "option", "data" );
    for( const option of changedData )  {
      if( option['active'] )  {
        optionList[ option['id'] ] = option['value'];
      }
    }
    this.data['dhcp-server']['options'] = optionList;

    return this.data;
  }


  //-------------------------------------------------------
  //  result of xhr request
  //
  OnXhrDhcpOptions( result )
  {
    // store the resulting list from server config
    this.dhcpOptionList = result['data']['dhcp-options'];
    this.FormFillDhcpOptionList();
  }

  //-------------------------------------------------------
  //  dhcp options will be filled by a config file
  //
  FormFillDhcpOptionList()
  {
    // the dhcp-options that is currently edited
    this.dhcpOptionCurrentEdit = null;
    //console.log( "this.dhcpOptionCurrentEdit", this.dhcpOptionCurrentEdit );

    // build dhcp grid data
    this.dhcpGridList = [];
    for( let dhcpOption of this.dhcpOptionList ) {

      const dhcpValue = this.data['dhcp-server']['options'][dhcpOption['option']];
      let record = {
        id: dhcpOption['option'],
        text: dhcpOption['text'],
        value: ( dhcpValue == undefined ) ? "" : dhcpValue,
        active: ( dhcpValue != undefined ),
        type: dhcpOption['type'],
        default: dhcpOption['default']
      };
      this.dhcpGridList.push( record );
    }

    // set the data into grid control
    $( "#ID_DHCP_OPTIONLIST" + this.dynid ).grid( "option", "data", this.dhcpGridList );

  }


  //-------------------------------------------------------
  // OnSidebarClick(): choose option on sidebar
  //
  OnSidebarClick( id )
  {
    // if there are errors, cancel change of page
    if( this.Option( "error" ) > 0 )  {
      return false;
    }

    // if there are unsaved edited dhcp options, apply. 
    if( this.Option( "modified" ) > 0 )  {
      if( this.ApplyDhcpOptionAuto( ) != true )  {
        return false;
      }
    }

    // what is to be verified?
    let currentSelection = $( "#ID_SIDEBAR" + this.dynid ).treemenu( "option", "selected" );
    
    // validate ip & mask 
    if( currentSelection == "ID_GENERAL" )  {
      if( this.ValidateSectionGeneral( ) > 0 )  {
        return false;
      }
    }

    // in all other cases cancel input and change page
    this.OnDhcpReservationCancelClick();
    this.OnDhcpOptionCancelClick();

    // disable verifikation und continue
    //this.Option( "reset" );
    /*this.Option( "error", 0 );
    this.Option( "validate", 0 );*/
    this.SelectOption( id );

    // change is allowed
    return true;
  }

  //-------------------------------------------------------
  // OnDialogApply(): save everything
  //
  OnDialogApply( )
  {
    if( this.ValidateSectionGeneral( ) > 0 )  {
      return false;
    }
    this.FillResultData();
    this.FireEvent( "result", this.data );
    this.Destroy();
  }

  //-------------------------------------------------------
  // OnDialogApply(): save everything
  //
  OnDialogCancel( )  
  {
    this.FireEvent( "result", null );
    this.Destroy();
  }  

  //-------------------------------------------------------
  //  HELPER FUNCTIONS
  //-------------------------------------------------------

  //-------------------------------------------------------
  // SelectOption(): show part of dialog and select
  //
  SelectOption( id )
  {
    $( "#ID_SIDEBAR" + this.dynid ).treemenu( "option", "selected", id );
    $( "#ID_CONTENT" + this.dynid ).optional( "radio", id );
  }


  //-------------------------------------------------------
  //  STATIC 
  //-------------------------------------------------------

  //-------------------------------------------------------
  //  Show the dialog
  //
  static Show( options )
  {
    let createdata = $.extend( true, [], {
      view: '/dialog/dhcpOptions',
      type: 'dialog',
      layout: 'classic2',
      size: "60rem",
      height: -1,
      top: CONST.dlgTop.XL,
      buttons: [
        { id: 'IDCANCEL', size: 15, status: 'normal', text: '{$const_dialog_button_cancel}' },
        { id: 'IDAPPLY', size: 15, status: 'hilight', text: '{$const_dialog_button_apply}' }
      ]
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }  

  //-------------------------------------------------------
  //  build default options into interface data
  //
  static BuildDhcpDefaultOptions( interfaceData )
  {
    if( !( 'dhcp-server' in interfaceData )
     || !( 'options' in interfaceData['dhcp-server'] )
     || $.isEmptyObject( interfaceData['dhcp-server']['options'] )
      ) {
      
      let dhcpDefaultOptions = Data.Instance().AdminConfig( 'dhcp' );

      let resultOptions = {};
      for( let record of dhcpDefaultOptions['dhcp-options'] )  {

        // write if required
        if( record['active'] == true ) {

          let defaultValue = record['default'];
          defaultValue = defaultValue.replace( "{ip}", interfaceData['ipv4'] || "" );
          defaultValue = defaultValue.replace( "{mask}", interfaceData['ipv4mask'] || "" );
          defaultValue = defaultValue.replace( "{hostname}", interfaceData['hostname'] || "" );
          defaultValue = defaultValue.replace( "{router}", interfaceData['gateway'] || "" );

          // result need to be an array
          if( defaultValue == "" ) {
            record['active'] == false;
          }
          else if( record['type'].charAt( 0 ) == '*' )  {
            resultOptions[ record['option'] ] = [ defaultValue ];
          }
          else if( record['type'] == "integer" ){
            resultOptions[ record['option'] ] = parseInt( defaultValue );
          }
          else  {
            resultOptions[ record['option'] ] = defaultValue;
          }
        
        }//if
      }//for

      return resultOptions;
    }//if

    // no change needed
    return null;
  }

}

//=============================================================================
//  BUILDER
//=============================================================================

DhcpOptionsDialog.Register( '/dialog/dhcpOptions' );
//=============================================================================
//
//  Logfile Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          19.07.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormRuleDebug extends FormBase
{
  //---------------------------------------------------------------------------
  //  STATIC
  //---------------------------------------------------------------------------

  static viewableTemplate = "/forms/system/ruleDebug";   // vw template
  static documentIcon = "mdi mdi-flash-red-eye";   // icon for toolbar & doc
  static documentTitle = "{$ruledbg_title}";  // title

  //---------------------------------------------------------------------------
  //  CLASS DATA
  //---------------------------------------------------------------------------

  ruleInstanceList = [];
  ruleSource = null;
  ruleId = null;
  ruleSequenceList = null;

  destructor()
  {
    RuleDebugger.Instance().RequestRuleDebug( "*", false );
  }

  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createData )
  {
    //-----------------------------------------------------
    //  controls
    //-----------------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    this.ruleSequenceList = new Dataset( { id: "id" } );

    // create controls
    $( '#ID_TOOLBAR' + this.dynid ).toolbar( {
      size: -1,
      icon: FormRuleDebug.documentIcon,
      text: FormRuleDebug.documentTitle,
      //iconStatus: "mdi mdi-circle red-blink",
      data: [ 
        { id: 'ID_REFRESH', icon: 'mdi mdi-refresh', status: "normal", text: "{$const_form_label_refresh}", tooltip: "{$const_form_tooltip_refresh}" },
      ],
      click: this.ControlToolbarClick.bind( this )
    } );


    RuleSourceControl.Build( $( "#ID_RULESSOURCE" + this.dynid ), {
      width: -1,
      //height: 10
    } );
    
    
    $( '#ID_RULESLIST' + this.dynid ).listbox( {
      size: CONST.size.XL,
      height: 7,
      result: this.ControlRulesListSelect.bind( this )
    } );

    $( '#ID_RULEWATCH' + this.dynid ).button( {
      size: CONST.size.M,
      text: "{$ruledbg_label_watch}",
      status: "disabled",
      click:  this.ControlClickRulewatch.bind( this )
    } );

    $( '#ID_RULE_NOWATCH' + this.dynid ).button( {
      size: CONST.size.S,
      status: "disabled",
      icon: "mdi mdi-eye-off",
      click: this.ControlClickRuleNowatch.bind( this )
    } );    

    $( "#ID_RULE_SEQUENCES" + this.dynid ).grid( {
      width: "100%",
      height: -1,
      border: false,
      headerHidden: true,
      scrollbar: 'overlay',
      header: {
        text: { text: "", size: CONST.size.XL - 4 },
        warning: { text: "", size: 4 }
      }, 
      renderCell: ( id, rec ) => {
        if( id == "text" ) return rec;
        return {};
      },
      data: this.ruleSequenceList,
      result: this.ControlRuleSequenceResult.bind( this )      
    } );

    $( "#ID_SEQUENCES_DELETE" + this.dynid ).click( this.ControlSequenceEmpty.bind( this ) );
    
    //-----------------------------------------------------
    //  Start
    //-----------------------------------------------------    

    RuleDebugger.Instance().RequestRuleInstanceList( ( response ) => {
      for( var item of response['instances'] ) {
        this.ruleInstanceList.push( {
          id: item,
          text: item
        });
      }
      $( '#ID_RULESLIST' + this.dynid ).listbox( "option", "data", this.ruleInstanceList );
    } );

    RuleDebugger.Instance().RequestRuleDebug( "*", false );
    RuleDebugger.Instance().On( "RULE/SEQUENCE", this.etag, this.RuleSequenceUpdate.bind( this ) );
    RuleDebugger.Instance().On( "RULE/STOP_DEBUG", this.etag, this.RuleStopDebug.bind( this ) );

    this.ModeNormal();

  }//load

  
  //---------------------------------------------------------------------------
  //  FORM
  //---------------------------------------------------------------------------

  FormShowSource()
  {
    if( this.ruleId ) {
      $( "#ID_RULESSOURCE" + this.dynid ).viewable( "option", "script", this.ruleSource['script'] );
      if( this.ruleSource['error'] && this.ruleSource['error'].length ) {
        $( "#ID_RULESSOURCE" + this.dynid ).viewable( "option", "scripterror", this.ruleSource['error'] );
      }
    }
    else {
      $( "#ID_RULESSOURCE" + this.dynid ).viewable( "option", "script", null );
    }

  }

  //---------------------------------------------------------------------------
  //  CONTROL
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  a rule of the rule instance list is selected
  //
  ControlRulesListSelect( select )  
  {
    RuleDebugger.Instance().RequestRuleDebug( "*", false );

    if( select && select.length > 0 )  {
      RuleDebugger.Instance().RequestRuleSource( select[ 0 ], ( response ) => {
        this.ruleSource = response;
        this.ruleId = select[ 0 ];
        this.FormShowSource();
        this.ModeSelected();
      } );

    }
    else {
      this.ruleSource = null;
      this.ruleId = null;
      this.FormShowSource();
      this.ModeNormal();
    }
  }

  //---------------------------------------------------------------------------
  //  watch a rule should be turned on
  //
  ControlClickRulewatch()
  {
    this.ModeWatch();
    if( this.ruleId ) {
      RuleDebugger.Instance().RequestRuleDebug( this.ruleId, true );
    }
  }

  //---------------------------------------------------------------------------
  //  watch a rule should be turned off
  //
  ControlClickRuleNowatch()
  {
    this.ModeSelected();
    RuleDebugger.Instance().RequestRuleDebug( "*", false );
  }

  //---------------------------------------------------------------------------
  //  empty sequ list
  //
  ControlSequenceEmpty()
  {
    this.ruleSequenceList.SetData( [] );
    $( "#ID_RULESSOURCE" + this.dynid ).viewable( "option", "sequence", null );
  }

  //---------------------------------------------------------------------------
  //  a sequence should be shown 
  //
  ControlRuleSequenceResult( selected )
  {
    var sequence = this.ruleSequenceList.Get( selected[ 0 ] );
    $( "#ID_RULESSOURCE" + this.dynid ).viewable( "option", "sequence", sequence['seq'] );
  }

  //---------------------------------------------------------------------------
  //  toolbar button
  //
  ControlToolbarClick( id )
  {
    if( id == "ID_REFRESH" )  {
      
    }
  }


  //---------------------------------------------------------------------------
  //  MODE
  //---------------------------------------------------------------------------

  ModeNormal()
  {
    if( this.mode == "modeNormal" ) return;

    $( '#ID_TOOLBAR' + this.dynid ).toolbar( "option", "iconStatus", "" );

    this.mode = "modeNormal";
    $( '#ID_RULEWATCH' + this.dynid ).button( "option", "status", "disabled" );
    $( '#ID_RULEWATCH' + this.dynid ).button( "option", "icon", "" );
    $( '#ID_RULE_NOWATCH' + this.dynid ).button( "option", "status", "disabled" );
  }

  ModeSelected()
  {
    if( this.mode == "modeSelected" ) return;

    $( '#ID_TOOLBAR' + this.dynid ).toolbar( "option", "iconStatus", "" );

    this.mode = "modeSelected";
    $( '#ID_RULEWATCH' + this.dynid ).button( "option", "status", "normal" );
    $( '#ID_RULEWATCH' + this.dynid ).button( "option", "icon", "" );
    $( '#ID_RULE_NOWATCH' + this.dynid ).button( "option", "status", "disabled" );
  }

  ModeWatch()
  {
    if( this.mode == "modeWatch" ) return;

    $( '#ID_TOOLBAR' + this.dynid ).toolbar( "option", "iconStatus", "mdi mdi-circle red-blink" );

    this.mode = "modeWatch";
    $( '#ID_RULEWATCH' + this.dynid ).button( "option", "status", "disabled" );
    $( '#ID_RULEWATCH' + this.dynid ).button( "option", "icon", "mdi mdi-autorenew mdi-spin" );
    $( '#ID_RULE_NOWATCH' + this.dynid ).button( "option", "status", "normal" );
  }

  //---------------------------------------------------------------------------
  //  DATA
  //---------------------------------------------------------------------------

  RuleSequenceUpdate( data )
  {
    var id = this.ruleSequenceList.Size();
    var localTime = moment.tz( new Date(), "utc" );
    var localTimeString = moment.tz( localTime, this.browserTimeZone ).format( "LTS" );
    if( data['sequences'] && data['sequences'].length )  {
      var lineNr = data['sequences'][ 0 ]['line'];
      var sourceLine = this.ruleSource['script'][ lineNr ];
      var match = sourceLine.match( /\(.*\)/gm );
      localTimeString += ": " + match[ 0 ];
    }

    this.ruleSequenceList.Add( {
      id: id,
      text: localTimeString,
      icon: "mdi mdi-flash",
      seq: data['sequences']
    } );

    $( "#ID_RULE_SEQUENCES" + this.dynid ).grid( "option", "selected", [ id ] );
    $( "#ID_RULESSOURCE" + this.dynid ).viewable( "option", "sequence", data['sequences'] );      
  }

  RuleStopDebug( data )
  {
    this.ModeSelected( );
  }
  
}


//=============================================================================
//  REGISTER
//=============================================================================

FormRuleDebug.Register( FormRuleDebug.viewableTemplate );
PageMain.RegisterForm( "ruledebug", FormRuleDebug.viewableTemplate, FormRuleDebug.documentTitle, "ruledebug" );
//=============================================================================
//
//  Logfile Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          19.07.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class RuleSourceControl extends Viewable 
{
  //---------------------------------------------------------------------------
  //  STATIC
  //---------------------------------------------------------------------------

  static template = "/forms/system/ruleDebugSource";   // vw template

  //---------------------------------------------------------------------------
  //  CLASS DATA
  //---------------------------------------------------------------------------

  ruleId = null;
  ruleSource = null;
  ruleSequenceList = null;
  options = {
    width: -1,
    height: "fit",
    data: [],
    script: null,
    sequence: null
  };
  
  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  load function
  //
  Load( createData )
  {
    // merge object
    Object.assign( this.options, createData );

    // control height
    if( this.options['height'] != "fit" ) {
      var height = "100%";
      if( this.options['height'] > 0 ) {
        height = this.options['height'] * $.metrics( "fontHeight" );
      }
      this.$container.css( "height", height );
    }
    
    // control width
    var width = "100%";
    if( this.options['width'] > 0 ) {
      width = this.options['width'] * $.metrics( "fontWidth" );
    }
    this.$container.css( "width", width );

    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );    

    // json control
    JsonControl.Build( $( "#ID_JSONCONTROL" + this.dynid ), {
      width: "100%",
      height: 10
    } );

    // draw control
    this.Draw();
    
  }//load

  //---------------------------------------------------------------------------
  //  viewable event function
  //
  OnEvent( eventName, param, data )
  {
    if( eventName == "option" ) {
      return this.EventOption( param, data );
    }

    return false;
  }

  //---------------------------------------------------------------------------
  //  viewable event function
  //
  EventOption( param, data )
  {
    // setter
    if( data !== undefined ) {
      this.options[ param ] = data;
      this.DataEncode( param, data );
      this.Draw();
    }

    return this.options[ param ];

  }

  //---------------------------------------------------------------------------
  //  DATA
  //---------------------------------------------------------------------------

  DataEncode( param, data )
  {
    // reset script
    if( param == "script" ) {
      this.options.data = [];
      if( !data ) return;

      for( var index = 0; index < data.length; index++ ) {
        this.options.data.push( {
          lineNr: index,
          line: data[ index ],
          enabled: true,
          parameter: null,
          error: null
        } );
      }
    }
    // set error
    else if( param == "scripterror" ) {
      for( var err of data ) {
        var scrLine = this.options.data.find( ( rec ) => rec['lineNr'] == err['line'] );
        if( scrLine ) {
          scrLine['error'] = err['error'];
        }
      }
    }
    // set error
    else if( param == "sequence" ) {
      if( !data ) {
        for( var line of this.options.data ) {
          line['parameter'] = null;
          line['enabled'] = true;
        }
      }
      else {
        for( var line of this.options.data ) {
          line['parameter'] = null;
          line['enabled'] = false;
          line['error'] = null;
        }

        for( var seq of data ) {
          var scrLine = this.options.data.find( ( rec ) => rec['lineNr'] == ( seq['line'] ) );
          if( scrLine ) {
            scrLine['parameter'] = seq['parameter'];
            scrLine['enabled'] = true;
            if( seq['term'] == "e" ) {
              scrLine['error']= seq['parameter']['error'];
            }
          }
        }
      }
    }

  }

  //---------------------------------------------------------------------------
  //  INTERNALS
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  main draw dispatcher
  //
  Draw()
  {
    if( this.options['script'] ) {
      this.DrawSource();
    }
    else {
      this.DrawEmpty();
    }
  }

  //---------------------------------------------------------------------------
  //  draw only source
  //
  DrawEmpty()
  {
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_SOURCE" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_PARAMETER" );    
  }

  //---------------------------------------------------------------------------
  //  draw only source
  //
  DrawSource()
  {
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_SOURCE" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_PARAMETER" );    

    var $ruleSource = this.$container.find( ".rule-src-container" );
    $ruleSource.empty();

    var html = "";
    for( var lineRecord of this.options.data ) {
      
      // build html
      html = "<div class='src-line-nr'><span>" + String( parseInt( lineRecord['lineNr'] ) + 1 ).padStart( 2, " " ) + "</span></div>";
      var line = this.HelperSyntaxHilight( lineRecord['line'], lineRecord['enabled'] );
      var classList = "src-line";
      if( lineRecord['error'] ) {
        classList += " src-error";
      }
      if( lineRecord['parameter'] ) {
        classList += " hover";
      }
      if( lineRecord['error'] ) {
        html += `<div class='${ classList }' title="${ lineRecord['error'] }">${ line }</div>`;
      }
      else {
        html += `<div class='${ classList }'>${ line }</div>`;
      }
      html += "</div>";

      // create line and append
      var $line = $( `<div data-lineno="${ lineRecord['lineNr'] }">` );
      $line.html( html );

      // show parameter
      if( lineRecord['enabled'] ) {
        $line.click( ( ev ) => {
          var lineNo = ev.currentTarget.dataset["lineno"];
          this.ControlClickLine( lineNo );
        } );
      }
      $ruleSource.append( $line );
    }

  }


  ControlClickLine( lineNr )
  {
    var record = this.options.data[ lineNr ];
    this.$container.find( ".src-line-selected" ).removeClass( "src-line-selected" );

    if( record['parameter'] ) {
      this.$container.find( `[data-lineno="${ lineNr }"]` ).find( ".src-line" ).addClass( "src-line-selected" );  
      $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_PARAMETER" );    
      $( "#ID_JSONCONTROL" + this.dynid ).viewable( "option", "data", record['parameter'] );  
    }
    else {
      $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_PARAMETER" );    
    }
  }

  //---------------------------------------------------------------------------
  //  HELPER
  //---------------------------------------------------------------------------

  HelperSyntaxHilight( line, isEnabled = true )
  {
    if( isEnabled ) {
      line = line.replaceAll( /IF\(|IF-NOT\(|EVENT\(|ACTION\(|ELSE\(|ELIF\(|ELIF-NOT\(|AND\(\)|OR\(\)/gm, "<span class=\"keyword\">$&</span>" );
      line = line.replaceAll( /\'([a-zA-Z0-9]|\\|[^\\'])*\'/gm, "<span class=\"parameter\">$&</span>" );
      line = line.replaceAll( /\{([a-zA-Z0-9]|\\|[^\\'])*\}/gm, "<span class=\"parameter\">$&</span>" );
      line = line.replaceAll( /\d+.\d+/gm, "<span class=\"parameter\">$&</span>" );      
      line = line.replaceAll( /true|false/gm, "<span class=\"parameter\">$&</span>" );
    }
    else {
      line = line.replaceAll( /IF\(|IF-NOT\(|EVENT\(|ACTION\(|ELSE\(|ELIF\(|ELIF-NOT\(|AND\(\)|OR\(\)/gm, "<span class=\"keyword-disabled\">$&</span>" );
      line = line.replaceAll( /\'([a-zA-Z0-9]|\\|[^\\'])*\'/gm, "<span class=\"parameter-disabled\">$&</span>" );
      line = line.replaceAll( /\{([a-zA-Z0-9]|\\|[^\\'])*\}/gm, "<span class=\"parameter-disabled\">$&</span>" );
      line = line.replaceAll( /\d+.\d+/gm, "<span class=\"parameter-disabled\">$&</span>" );      
      line = line.replaceAll( /true|false/gm, "<span class=\"parameter-disabled\">$&</span>" );
    }
    return line;
  }
  
  //---------------------------------------------------------------------------
  //  STATIC
  //---------------------------------------------------------------------------

  static Build( $element, createOptions )
  {
    createOptions['view'] = RuleSourceControl.template;
    $element.viewable( createOptions );
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

RuleSourceControl.Register( RuleSourceControl.template );
//=============================================================================
//
//  Modules List Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          19.07.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormApi extends FormBase
{
  //---------------------------------------------------------------------------
  //  STATIC
  //---------------------------------------------------------------------------

  static viewableTemplate = "/forms/system/api";    // vw template
  static documentIcon = "mdi mdi-link-variant-plus";             // icon for toolbar & doc
  static documentTitle = "{$api_document_title}";   // title
  
  //---------------------------------------------------------------------------
  //  INIT
  //---------------------------------------------------------------------------

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = { entid: "dummy" };   // no $.clone(), createData is cloned!
    this.entityOnvif = null;
    this.modes = {
      onvif: true
    };

    //-----------------------------------------------------
    //  controls
    //-----------------------------------------------------

    let toolbarData = [
      { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
      { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
      { id: 'ID_RESTART_SERVICES', icon: 'mdi mdi-power', text: '{$const_menu_restart_service_short}', status: 'red' },
    ];

    // groupboxes
    $( '.groupbox' ).groupbox();
    $( "#ID_OPTIONAL" + this.dynid ).optional( { } );    

    // ---- ONVIF -----

    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: FormApi.documentIcon,
      text: FormApi.documentTitle,
      data: toolbarData,
      click: this.ControlToolbarClick.bind( this ),
    } );

    $( "#ID_ONVIF_ENABLE" + this.dynid ).choice( {
      size: CONST.size.L,
      type: "check",
      text: "{$api_label_onvif_enable_text}",
      result: this.ControlOnvifEnable.bind( this )
    } );

    $( "#ID_ONVIF_USER" + this.dynid ).edit( {
      size: CONST.size.L,
      validate: Validate.NotEmptyString,
      filter: { allow: "0123456789.abcdefghijklmnopqrstuvwxyz_ABCDEFGHIJKLMNOPQRSTUVWXYZ" }
    } );

    // if module is not installed, don't show onvif
    var entidOnvif = Data.Instance().Entinfo( "module.onvif" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( entidOnvif ? "show" : "hide", "ID_OPTIONAL_ONVIF" );


    //-----------------------------------------------------
    //  Start
    //-----------------------------------------------------    

    Data.Instance().On( "DATA/UPDATE/module.onvif", this.etag, this.DataResponseEntity.bind( this ) );

    this.DataRequest();

  }//load


  //---------------------------------------------------------------------------
  //  FORM
  //---------------------------------------------------------------------------

  FormFillControls()
  {
    if( this.entityOnvif ) {

      this.modes['onvif'] = this.entityOnvif['authorize']['onvif']['enabled'];
        
      $( "#ID_ONVIF_ENABLE" + this.dynid ).choice( "option", "value", this.modes['onvif'] );
      $( "#ID_ONVIF_USER" + this.dynid ).edit( "option", "value", this.entityOnvif['authorize']['onvif']['account'] );
      ControlPassword.Build( $( "#ID_ONVIF_PASSWORD" + this.dynid ), {
        entity: this.entityOnvif,
      } );      

    }
  }
  
  //---------------------------------------------------------------------------
  //  Read values from controls and write back to entity.
  //
  FormReadControls()
  {
    if( this.entityOnvif ) {
      this.entityOnvif['authorize']['onvif']['enabled'] = $( "#ID_ONVIF_ENABLE" + this.dynid ).choice( "option", "value" );
      this.entityOnvif['authorize']['onvif']['account'] = $( "#ID_ONVIF_USER" + this.dynid ).edit( "option", "value" );
      $( "#ID_ONVIF_PASSWORD" + this.dynid ).viewable( "option", "save", this.entityOnvif );      
    }
  }


  //---------------------------------------------------------------------------
  //  CONTROL
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  check: onvif enabled
  //
  ControlOnvifEnable( isEnabled )  
  {
    this.modes['onvif'] = isEnabled;
    this.ModeOnvifEnabled( isEnabled );
  }

  //---------------------------------------------------------------------------
  //  toolbar button
  //
  ControlToolbarClick( id )
  {
    switch( id )  {
      case "ID_UNDO":               this.CommandUndo();  break;
      case "ID_SAVE":               this.CommandSave();  break;
      case "ID_RESTART_SERVICES":   AccessApplication.Instance().Command( 'restart-service' ); break;
    }   
  }

  //---------------------------------------------------------------------------
  //  MODES
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  called after load has finished 
  //
  ModeManageAll()
  {
    this.ModeOnvifEnabled( this.modes['onvif'] );
  }

  //---------------------------------------------------------------------------
  //  new mode after onvif en-/disabled
  //
  ModeOnvifEnabled( isEnabled )
  {
    if( isEnabled ) {
      $( "#ID_ONVIF_USER" + this.dynid ).edit( "option", "status", "normal" );
      $( "#ID_ONVIF_PASSWORD" + this.dynid ).viewable( "option", "status", "normal" );
    }
    else {
      $( "#ID_ONVIF_USER" + this.dynid ).edit( "option", "status", "disabled" );
      $( "#ID_ONVIF_PASSWORD" + this.dynid ).viewable( "option", "status", "disabled" );
      
      // validate off
      $( "#ID_ONVIF_USER" + this.dynid ).edit( "validate", false );
      $( "#ID_ONVIF_PASSWORD" + this.dynid ).viewable( "validate", false );  
    }
  }

  //---------------------------------------------------------------------------
  //  DATA
  //---------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  Do xhr request to get the logfile.
  //
  DataRequest()
  {
    this.numberOfLoadEntities = 1;

    Data.Instance().RequestEntity( "module.onvif" );
    this.StatusLoading();

  }

  //---------------------------------------------------------------------------
  //  Response on XHR request.
  //
  DataResponseEntity( entity )
  {
    this.numberOfLoadEntities --;

    // we need to create a copy
    if( entity['entid'] == "module.onvif" ) {
      this.entityOnvif = $.clone( entity );
    }

    // Fill controls and enter load finished to do 1st validation
    if( this.numberOfLoadEntities == 0 )  {
      this.FormFillControls();
      this.StatusLoadFinished();
      this.StatusIdle();
      setTimeout( () => {
        this.ModeManageAll();
      }, 50 );
      
    }

  }

  //---------------------------------------------------------------------------
  //  XHR write request(s)
  //
  DataSaveRequest()
  {
    // create a copy for save: name & label-handling + reference reset.
    let entityOnvif = this.DataPrepareEntityForSave( this.entityOnvif );    

    // save the entity
    Data.Instance().SavePartialEntity( entityOnvif );

    // base-function: reload entity after save
    this.DataReloadAll();
    //this.ToastRebootRequired();
  }  

  //-----------------------------------------------------------------------------
  //  Message from data-layer: Module list updated. 
  //  
  DataModulesUpdated( )
  {
    this.FormFillControls();
  }

  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Validate & save
  //
  CommandSave()
  {
    // Do validation.
    let numErr = this.Validate();

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  CommandUndo(): undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }

  //---------------------------------------------------------------------------
  //  MODES
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  VALIDATE
  //---------------------------------------------------------------------------
  
  Validate()
  {
    var numErr = 0;
    if( this.modes['onvif'] ) {
      numErr += $( "#ID_ONVIF_USER" + this.dynid ).edit( "validate" );
      numErr += $( "#ID_ONVIF_PASSWORD" + this.dynid ).viewable( "validate" );
    }
    return numErr;
  }

  ValidateOff()
  {
    $( "#ID_ONVIF_USER" + this.dynid ).edit( "validate", false );
    $( "#ID_ONVIF_PASSWORD" + this.dynid ).viewable( "validate", false );
    return 0;
  }


  //===========================================================================
  //  XHR
  //===========================================================================



  //---------------------------------------------------------------------------
  //  TOAST
  //---------------------------------------------------------------------------


  
}


//=============================================================================
//  REGISTER
//=============================================================================

FormApi.Register( FormApi.viewableTemplate );
PageMain.RegisterForm( "api", FormApi.viewableTemplate, FormApi.documentTitle, "api" );
//=============================================================================
//
//  Folder Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.06.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormFolder extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/folder";        // vw template
  static formId     = "folder";                       // entity class
  static icon       = "mdi mdi-folder-outline";       // icon for toolbar & doc
  static title      = "{$folder_document_title}";     // default title if not yet loaded
  static creatable  = true;                           // self explaining
  static container  = true;                           // only folder can create subitems
  static createTemplate = [                           // menu entries in "add" command
    { text: "{$folder_create_type}", deviceTemplate: null, folder: null }
  ];
  helpFile = "folder";
  
  //-----------------------------------------------------------------
  //  might be overwritten for special entities
  //
  IsDeleteAllowed( entinfo )
  {
    if( ( this.entinfo['entid'] == "folder.unconfdev" ) || ( this.entinfo['entid'] == "folder.confdev" ) ) {
      return false;
    }
    return true;
  }

  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};
    let isReadOnlyForm = false;
    let toolbarData = [
      { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
      { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
      { id: 'ID_ADD', icon: 'mdi mdi-plus-circle', text: '{$const_form_label_add}', tooltip: "{$const_form_tooltip_add}", status: 'normal' },
      { id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', tooltip: "{$const_form_tooltip_permission}", status: 'normal' },
      { id: 'ID_DELETE', icon: 'mdi mdi-delete', tooltip: "{$const_form_tooltip_delete}", status: "red" },
      //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
    ];

    // special handling for system folders
    if( ( this.entinfo['entid'] == "folder.unconfdev" ) || ( this.entinfo['entid'] == "folder.confdev" ) ) {
      isReadOnlyForm = true;
      toolbarData[ 0 ]['status'] = "disabled";  // ID_UNDO
      toolbarData[ 1 ]['status'] = "disabled";  // ID_SAVE
      toolbarData[ 4 ]['status'] = "disabled";  // ID_DELETE
    }
    if( this.entinfo['entid'] == "folder.unconfdev" ) {
      toolbarData[ 2 ]['status'] = "disabled";  // ID_ADD
      toolbarData[ 3 ]['status'] = "disabled";  // ID_PERMISSIONS
    }


    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: toolbarData,
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );


    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this ),
      readonly: isReadOnlyForm
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: ( isReadOnlyForm ) ? "readonly" : "normal",
    } );

    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );
    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  fill controls from entity
  //
  FormFillControls()
  {
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );
  }

  //---------------------------------------------------------------------------
  //  read controls
  //
  FormReadControls()
  {
    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );
  }


  //===========================================================================
  //  CONTROL
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;
    this.FormTitle( this.entity['name'] );
  }


  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":         this.CommandUndo();         break;
      case "ID_SAVE":         this.CommandSave();         break;
      case "ID_ADD":          this.CommandAdd();          break;
      case "ID_PERMISSIONS":  this.CommandPermissions();  break;
      case "ID_DELETE":       this.CommandDelete();       break;
    }
  }

  
  //=============================================================================
  //  COMMANDS
  //=============================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }

  //-----------------------------------------------------------------------------
  //  add element into this folder
  //
  CommandAdd()
  {
    PageMain.CreateEntity( this.entinfo['entid'] );
  }

  
  //===========================================================================
  //  DATA
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //-----------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();
  }


  //-----------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest();

    // save modified entity (don't change status manually!)
    let entity = this.DataPrepareEntityForSave( this.entity );
    Data.Instance().SavePartialEntity( entity );
  }

  //=============================================================================
  //  VALIDATION
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  Validate(): do validation
  //
  Validate()
  {
    return 0;
  }

  //-----------------------------------------------------------------------------
  //  Validate(): do validation
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormFolder.Register( FormFolder.template );
FormFactory.Register( FormFolder );
//=============================================================================
//
//  Apartment Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.06.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormApartment extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/apartment";     // vw template
  static formId     = "apartment";                    // entity class
  static icon       = "mdi mdi-bell-ring-outline";       // icon for toolbar & doc
  static title      = "{$apartment_document_title}";  // default title if not yet loaded
  static creatable  = true;                           // self explaining
  static container  = false;                          // only folder can create subitems
  static createTemplate = [                           // menu entries in "add" command
    { text: "{$apartment_create_type_afs}",   deviceTemplate: "AFS",        folder: "{$const_category_new_indoor}" },
    { text: "{$apartment_create_type_afsv}",  deviceTemplate: "AFSV",       folder: "{$const_category_new_indoor}"  },
    { text: "{$apartment_create_type_ahf}",   deviceTemplate: "AHF_870-0",  folder: "{$const_category_new_indoor}"  },
    { text: "{$apartment_create_type_ahfv}",  deviceTemplate: "AHFV_870-0", folder: "{$const_category_new_indoor}"  },
    { text: "{$apartment_create_type_aht}",   deviceTemplate: "AHT_870-0",  folder: "{$const_category_new_indoor}"  },
    { text: "{$apartment_create_type_ahtv}",  deviceTemplate: "AHTV_870-0", folder: "{$const_category_new_indoor}"  }
  ];
  helpFile   = "";


  //=============================================================================
  //  STATIC FUNCTIONS
  //=============================================================================

  static MinimumNeededLicenses()
  {
    return { "APR": 1 };
  }

  //-----------------------------------------------------------------
  //  might be overwritten for special entities
  //
  IsDeleteAllowed( entinfo )
  {
    return true;
  }


  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: [
        { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
        { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
        { id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', tooltip: "{$const_form_tooltip_permission}", status: 'normal' },
        { id: 'ID_LISTS', icon: 'mdi mdi-format-list-bulleted', tooltip: "{$const_form_tooltip_lists}", status: "normal" },
        { id: 'ID_DEVICERESTART', icon: 'mdi mdi-restart', tooltip: "{$const_form_tooltip_devicerestart}", status: "normal" },
        { id: 'ID_DELETE', icon: 'mdi mdi-delete', tooltip: "{$const_form_tooltip_delete}", status: "red" },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );

    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this )
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: "normal",
    } );

    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly",
    } );

    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    Data.Instance().On( "MODULES/UPDATED/", this.etag, this.DataModulesUpdated.bind( this ) );
    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );
    
    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  read entity into all controls
  //
  FormFillControls()
  {
    // on all other devives then AFSV/AFS show lists dialog
    if( this.entity['deviceTemplate'] == "AFSV" || this.entity['deviceTemplate'] == "AFS" )  {
      this.FormToolbarButtonStatus( "ID_LISTS", "hidden" );
    }

    // standard controls
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );
    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( "option", "value", this.entity['deviceTemplate'] );

    // entity controls

    // address book
    ControlAddressBook.Build( $( "#ID_ADDRBOOK" + this.dynid ), {
      entity: this.entity
    } );

    // call profile
    ControlCallProfile.Build( $( "#ID_CALLPROFILE" + this.dynid ), {
      entity: this.entity
    } );

    // mac
    ControlMacAddress.Build( $( "#ID_MACADDR" + this.dynid ), {
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // call number
    ControlCallNumber.Build( $( "#ID_CALLNUMBER" + this.dynid ), {
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this ),
      result: this.ControlEntityCallnumber.bind( this )
    } );

    // mount type
    ControlMountType.Build( $( "#ID_MOUNTTYPE" + this.dynid ), {
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // volume deviceConfig
    ControlDeviceVolume.Build( $( "#ID_VOLUME" + this.dynid ), { 
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // call waiting
    ControlCallWaiting.Build( $( "#ID_CALLWAITING" + this.dynid ), {
      entity: this.entity
    } );

    // clock
    ControlShowClock.Build( $( "#ID_SHOWCLOCK" + this.dynid ), {
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // ringtone
    ControlRingtone.Build( $( "#ID_RINGTONE" + this.dynid ), {
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );
    

    // group controls
    
    // KBD deviceConfig
    ControlDeviceConfigGroup.Build( $( "#ID_DEVGROUP_KBD" + this.dynid ), {
      entity: this.entity,
      module: ["KBD", "LED"],
      result: this.ControlEntityResult.bind( this ),
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // IO deviceConfig
    ControlDeviceConfigGroup.Build( $( "#ID_DEVGROUP_IO" + this.dynid ), {
      entity: this.entity,
      module: ["IO", "AZIO"],
      result: this.ControlEntityResult.bind( this ),
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // function deviceConfig
    ControlDeviceFunctionGroup.Build( $( "#ID_DEVGROUP_FUNCTION" + this.dynid ), {
      entity: this.entity,
      module: [ "function" ],
      result: this.ControlEntityResult.bind( this ),
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

  }

  //---------------------------------------------------------------------------
  //  read control values
  //
  FormReadControls()
  {
    // get value from standard controls
    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );

    // all "entity-controls" save directly into entity
    $( "#ID_ADDRBOOK" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_CALLPROFILE" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_VOLUME" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_SHOWCLOCK" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_MOUNTTYPE" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_CALLNUMBER" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_CALLWAITING" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_MACADDR" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_RINGTONE" + this.dynid ).viewable( "option", "save", this.entity );

    // short before save all widgets are called to update itself
    $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "save", this.entity );

  }


  //===========================================================================
  //  CONTROL HANDLER
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":           this.CommandUndo();           break;
      case "ID_SAVE":           this.CommandSave();           break;
      case "ID_PERMISSIONS":    this.CommandPermissions();    break;
      case "ID_LISTS":          this.CommandLists();          break;
      case "ID_DEVICERESTART":  this.CommandDeviceRestart();  break;
      case "ID_DELETE":         this.CommandDelete();         break;
    }
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;

    this.FormTitle( this.entity['name'] );
    $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "option", "rebuild", this.entity );
    $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "option", "rebuild", this.entity );
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );

    // change name need to restart
    this.ControlRestartNeeded();
  }

  //---------------------------------------------------------------------------
  //  callnumber has been changed
  //  
  ControlEntityCallnumber( result )
  {
    if( ( 'phone' in this.entity ) && ( 'callnumber' in this.entity['phone'] ) ) {
      this.entity['phone']['callnumber'] = value;
      $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "option", "rebuild", this.entity );
      $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "option", "rebuild", this.entity );
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );      
    }

  }

  //---------------------------------------------------------------------------
  //  device config has been changed.
  //
  ControlEntityResult( entity )
  {
    this.entity = entity;

    // exception: master version of entity has been changed, so update all
    // widgets that does not have a save handler
    $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "option", "rebuild", this.entity );
    $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "option", "rebuild", this.entity );
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }

  //---------------------------------------------------------------------------
  //  device config says: We need to restart the device
  //
  ControlRestartNeeded()
  {
    this.isDeviceRestartNeeded = true;
  }

  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = this.Validate( );

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  CommandUndo(): undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }

  //-------------------------------------------------------
  //  show device lists dialog
  //
  CommandLists() 
  { 
    DeviceListsDialog.Show( { entid: this.entinfo['entid'], addressBook: this.entity['addrBook'] } );
  }



  //===========================================================================
  //  DATA
  //===========================================================================

  //---------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //---------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

    // set help file
    switch( this.entity['deviceTemplate'] )  {
      case "AFS":         this.helpFile = "afs";    break;
      case "AFSV":        this.helpFile = "afsv";   break;
      case "AHF_870-0":   this.helpFile = "ahf";    break;
      case "AHFV_870-0":  this.helpFile = "ahfv";   break;
      case "AHT_870-0":   this.helpFile = "aht";    break;
      case "AHTV_870-0":  this.helpFile = "ahtv";   break;
    }

    // while developing the unit test uncomment this:
    // this.UnitTest();

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();

  }

  //---------------------------------------------------------------------------
  //  update description lines if some names has been changed
  //
  DataEntinfoUpdate( entinfo, changedList )
  {
    // other entity has been changed
    if( entinfo['entid'] != this.entinfo['entid'] )  {
      $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "option", "rebuild", this.entity );
      $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "option", "rebuild", this.entity );    
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
    }
    else {
      super.DataEntinfoUpdate( entinfo, changedList );
    }
  }

  //---------------------------------------------------------------------------
  //  module list has been changed
  //
  DataModulesUpdated()
  {
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }

  //---------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest( );

    // create a copy for save: name & label-handling + reference reset.
    let entity = this.DataPrepareEntityForSave( this.entity );

    // add all references
    this.DataAddReferences( entity, $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_CALLPROFILE" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_ADDRBOOK" + this.dynid ).viewable( "option", "references" ) );

    // finally filter all internal references
    this.DataFilterReferencesForSave( entity );

    // save the entity
    Data.Instance().SavePartialEntity( entity );

    // if controls told us to restart device, add message
    if( this.isDeviceRestartNeeded )  {
      this.DataMarkEntityForReboot();
    }

    // base-function: reload entity after save
    this.DataReloadAll();
    
  }

  //===========================================================================
  //  VALIDATION
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_CALLNUMBER" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_MACADDR" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_ADDRBOOK" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_CALLPROFILE" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_MOUNTTYPE" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_VOLUME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_SHOWCLOCK" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_RINGTONE" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_CALLWAITING" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "validate" );

    return numErr;
  }

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  permission check for own statepoints
  //
  PermissionCheck( entinfo )
  {
    $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "event", "permission-check", entinfo );
    $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "event", "permission-check", entinfo );
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "event", "permission-check", entinfo );
  }


  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  OnUnitTest( testController )
  {
    testController.Assert( "uuid > 32", this.entity['uuid'].length > 32 );
    testController.AssertObject( "Entity class", this.entity, "class", "apartment" );

    // Hint: UnitTest() will call all widgets OnUnitTest() automatically
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormApartment.Register( FormApartment.template );
FormFactory.Register( FormApartment );//=============================================================================
//
//  TK Phone Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          15.07.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormTkphone extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/tkphone";     // vw template
  static formId     = "tkphone";                    // entity class
  static icon       = "mdi mdi-phone-classic";      // icon for toolbar & doc
  static title      = "{$tkphone_document_title}";  // default title if not yet loaded
  static creatable  = true;                         // self explaining
  static container  = false;                        // only folder can create subitems
  static createTemplate = [                         // menu entries in "add" command
    { text: "{$tkphone_create_type}", deviceTemplate: null, folder: null }
  ];
  helpFile = "tkphone";

  //=============================================================================
  //  STATIC FUNCTIONS
  //=============================================================================

  static MinimumNeededLicenses()
  {
    return { "APR": 1 };
  }

  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: [
        { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
        { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
        { id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', status: 'normal', tooltip: "{$const_form_tooltip_permission}" },
        { id: 'ID_LISTS', icon: 'mdi mdi-format-list-bulleted', status: "hidden", tooltip: "{$const_form_tooltip_lists}" },
        { id: 'ID_DEVICERESTART', icon: 'mdi mdi-restart', status: "hidden", tooltip: "{$const_form_tooltip_devicerestart}" },
        { id: 'ID_DELETE', icon: 'mdi mdi-delete', status: "red", tooltip: "{$const_form_tooltip_delete}" },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );


    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this )
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: "normal",
    } );

    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly",
    } );

    ControlEntityList.Build( $( "#ID_GATEWAY" + this.dynid ), {
      allowedClass: ["tkgw", "trunk"],
    } );    
   

    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    Data.Instance().On( "MODULES/UPDATED/", this.etag, this.DataModulesUpdated.bind( this ) );
    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );
    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  read entity into all controls
  //
  FormFillControls()
  {
    // standard controls
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );
    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( "option", "value", "{$tkphone_document_title}" );

    // address book
    ControlAddressBook.Build( $( "#ID_ADDRBOOK" + this.dynid ), {
      entity: this.entity
    } );

    // call number
    ControlCallNumber.Build( $( "#ID_CALLNUMBER" + this.dynid ), {
      entity: this.entity
    } );

    // extern call number
    ControlExternCallNumber.Build( $( "#ID_EXTERN_CALLNUMBER" + this.dynid ), {
      entity: this.entity
    } );

    // gateway
    $( "#ID_GATEWAY" + this.dynid ).viewable( "option", "selected", [ this.entity["phone"]["provider"] ] );    

    // call profile
    ControlCallProfile.Build( $( "#ID_CALLPROFILE" + this.dynid ), {
      entity: this.entity,    
    } );    

    // DTMF door/light
    ControlDtmfDoorAndLight.Build( $( "#ID_DTMF" + this.dynid ), {
      entity: this.entity,
    } );

    // function deviceConfig
    ControlDeviceFunctionGroup.Build( $( "#ID_DEVGROUP_FUNCTION" + this.dynid ), {
      entity: this.entity,
      module: ["function"],
      result: this.ControlEntityResult.bind( this )
    } );

  }

  //---------------------------------------------------------------------------
  //  Read control values back into entity
  //
  FormReadControls()
  {
    // all other controls write directly into entity
    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );
    this.entity["phone"]["provider"] = $( "#ID_GATEWAY" + this.dynid ).viewable( "option", "selected" )[0];

    // Tell widget controls to save directly into entity.
    $( "#ID_CALLNUMBER" +  this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_EXTERN_CALLNUMBER" +  this.dynid ).viewable( "option", "save", this.entity );  
    $( "#ID_ADDRBOOK"    + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_CALLPROFILE" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_DTMF" + this.dynid ).viewable( "option", "save", this.entity );    


    // short before save all widgets are called to update itself
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "save", this.entity );

  }

  //===========================================================================
  //  CONTROL HANDLER
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":           this.CommandUndo();           break;
      case "ID_SAVE":           this.CommandSave();           break;
      case "ID_PERMISSIONS":    this.CommandPermissions();    break;
      case "ID_LISTS":          this.CommandLists();          break;
      case "ID_DEVICERESTART":  this.CommandDeviceRestart();  break;
      case "ID_DELETE":         this.CommandDelete();         break;
    }
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;
    this.FormTitle( this.entity['name'] );
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }

  //---------------------------------------------------------------------------
  //  device config has been changed.
  //
  ControlEntityResult( entity )
  {
    this.entity = entity;
  }


  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = this.Validate();

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  CommandUndo(): undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }


  //===========================================================================
  //  DATA
  //===========================================================================

  //---------------------------------------------------------------------------
  //  do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //-----------------------------------------------------------------------------
  //  response from xhr
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

    // while developing the unit test uncomment this:
    // this.UnitTest();

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();
  
  }

  //-----------------------------------------------------------------------------
  //  some entinfo has been changed
  //
  DataEntinfoUpdate( entinfo, changedList )
  {
    // other entity has been changed
    if( entinfo['entid'] != this.entinfo['entid'] )  {
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
    }
    else {
      super.DataEntinfoUpdate( entinfo, changedList );
    }
  }

  //---------------------------------------------------------------------------
  //  module list has been changed
  //
  DataModulesUpdated()
  {
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }      

  //-----------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest();

    // save modified entity (don't change status manually!)
    let entity = this.DataPrepareEntityForSave( this.entity );

    // add all references
    this.DataAddReferences( entity, $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_CALLPROFILE" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_ADDRBOOK" + this.dynid ).viewable( "option", "references" ) );

    // finally filter all internal references
    this.DataFilterReferencesForSave( entity );
    Data.Instance().SavePartialEntity( entity );

    // base-function: reload entity after save
    this.DataReloadAll();

  }

  //===========================================================================
  //  VALIDATION
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_CALLNUMBER" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_EXTERN_CALLNUMBER" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_ADDRBOOK" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_GATEWAY" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_CALLPROFILE" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DTMF" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "validate" );

    return numErr;
  }


  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  permission check for own statepoints
  //
  PermissionCheck( entinfo )
  {
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "event", "permission-check", entinfo );
  }
  
}


//=============================================================================
//  REGISTER
//=============================================================================

FormTkphone.Register( FormTkphone.template );
FormFactory.Register( FormTkphone );
//=============================================================================
//
//  Apartment Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.06.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormAvp extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/avp";     // vw template
  static formId     = "avp";                    // entity class
  static icon       = "adi adi-siedle-avp";     // icon for toolbar & doc
  static title      = "{$avp_document_title}";  // default title if no entinfo
  static creatable  = true;                     // self explaining
  static container  = false;                    // only folder can create subitems
  static createTemplate = [                     // menu entries in "add" command
    { text: "{$avp_create_type}", deviceTemplate: "AVP_870-0", folder: "{$const_category_new_indoor}" },
  ];
  helpFile = "avp";

  //=============================================================================
  //  STATIC FUNCTIONS
  //=============================================================================

  static MinimumNeededLicenses()
  {
    return { "APR": 1 };
  }
    
  //-----------------------------------------------------------------
  //  might be overwritten for special entities
  //
  IsDeleteAllowed( entinfo )
  {
    return true;
  }
  

  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: [
        { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
        { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
        { id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', tooltip: "{$const_form_tooltip_permission}", status: 'normal' },
        { id: 'ID_LISTS', icon: 'mdi mdi-format-list-bulleted', tooltip: "{$const_form_tooltip_lists}", status: "normal" },
        { id: 'ID_DEVICERESTART', icon: 'mdi mdi-restart', tooltip: "{$const_form_tooltip_devicerestart}", status: "normal" },
        { id: 'ID_DELETE', icon: 'mdi mdi-delete', tooltip: "{$const_form_tooltip_delete}", status: "red" },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );

    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this )
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: "normal",
    } );

    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly",
    } );

    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    Data.Instance().On( "MODULES/UPDATED/", this.etag, this.DataModulesUpdated.bind( this ) );
    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );
    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  read entity into all controls
  //
  FormFillControls()
  {
    // standard controls
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );
    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( "option", "value", this.entity['deviceTemplate'] );

    // address book
    ControlAddressBook.Build( $( "#ID_ADDRBOOK" + this.dynid ), {
      entity: this.entity
    } );

    // call profile
    ControlCallProfile.Build( $( "#ID_CALLPROFILE" + this.dynid ), {
      entity: this.entity
    } );

    // mac
    ControlMacAddress.Build( $( "#ID_MACADDR" + this.dynid ), {
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // call number
    ControlCallNumber.Build( $( "#ID_CALLNUMBER" + this.dynid ), {
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // mount type
    ControlMountType.Build( $( "#ID_MOUNTTYPE" + this.dynid ), {
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // volume deviceConfig
    ControlDeviceVolume.Build( $( "#ID_VOLUME" + this.dynid ), { 
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // call waiting
    ControlCallWaiting.Build( $( "#ID_CALLWAITING" + this.dynid ), {
      entity: this.entity
    } );

    // clock
    ControlShowClock.Build( $( "#ID_SHOWCLOCK" + this.dynid ), {
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // ringtone
    ControlRingtone.Build( $( "#ID_RINGTONE" + this.dynid ), {
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // KBD deviceConfig
    ControlDeviceConfigGroup.Build( $( "#ID_DEVGROUP_KBD" + this.dynid ), {
      entity: this.entity,
      module: ["KBD", "TILE"],
      result: this.ControlGenericResult.bind( this ),
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // IO deviceConfig
    ControlDeviceConfigGroup.Build( $( "#ID_DEVGROUP_IO" + this.dynid ), {
      entity: this.entity,
      module: ["IO", "AZIO"],
      result: this.ControlGenericResult.bind( this ),
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // function deviceConfig
    ControlDeviceFunctionGroup.Build( $( "#ID_DEVGROUP_FUNCTION" + this.dynid ), {
      entity: this.entity,
      module: ["function"],
      result: this.ControlGenericResult.bind( this ),
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

  }

  //---------------------------------------------------------------------------
  //  read control values
  //
  FormReadControls()
  {
    // all other controls write directly into entity
    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );

    // all "entity-controls" save directly into entity
    $( "#ID_ADDRBOOK" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_CALLPROFILE" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_VOLUME" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_SHOWCLOCK" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_MOUNTTYPE" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_CALLNUMBER" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_CALLWAITING" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_MACADDR" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_RINGTONE" + this.dynid ).viewable( "option", "save", this.entity );

    // short before save all widgets are called to update itself
    $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "save", this.entity );

  }


  //===========================================================================
  //  CONTROL HANDLER
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":           this.CommandUndo();           break;
      case "ID_SAVE":           this.CommandSave();           break;
      case "ID_PERMISSIONS":    this.CommandPermissions();    break;
      case "ID_LISTS":          this.CommandLists();          break;
      case "ID_DEVICERESTART":  this.CommandDeviceRestart();  break;
      case "ID_DELETE":         this.CommandDelete();         break;
    }
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;

    this.FormTitle( this.entity['name'] );
    $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "option", "rebuild", this.entity );
    $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "option", "rebuild", this.entity );
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
   
    this.ControlRestartNeeded();    
  }

  //---------------------------------------------------------------------------
  //  device config has been changed.
  //
  ControlGenericResult( entity )
  {
    this.entity = entity;
  }

  //---------------------------------------------------------------------------
  //  device config says: We need to restart the device
  //
  ControlRestartNeeded()
  {
    this.isDeviceRestartNeeded = true;
  }


  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  CommandSave():  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = this.Validate();

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  CommandUndo(): undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }

  //-------------------------------------------------------
  //  show device lists dialog
  //
  CommandLists() 
  { 
    DeviceListsDialog.Show( { entid: this.entinfo['entid'], addressBook: this.entity['addrBook'] } );
  }

  
  //===========================================================================
  //  DATA
  //===========================================================================

  //---------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //---------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

    // while developing the unit test uncomment this:
    // this.UnitTest();

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();

  }

  //---------------------------------------------------------------------------
  //  update description lines if some names has been changed
  //
  DataEntinfoUpdate( entinfo, changedList )
  {
    // other entity has been changed
    if( entinfo['entid'] != this.entinfo['entid'] )  {
      $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "option", "rebuild", this.entity );
      $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "option", "rebuild", this.entity );
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity ); 
    }
    else {
      super.DataEntinfoUpdate( entinfo, changedList );
    }
  }

  //---------------------------------------------------------------------------
  //  module list has been changed
  //
  DataModulesUpdated()
  {
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }

  //---------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest( );

    // create a copy for save: name & label-handling + reference reset.
    let entity = this.DataPrepareEntityForSave( this.entity );

    // add all references
    this.DataAddReferences( entity, $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_CALLPROFILE" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_ADDRBOOK" + this.dynid ).viewable( "option", "references" ) );

    // finally filter all internal references
    this.DataFilterReferencesForSave( entity );

    // save the entity
    Data.Instance().SavePartialEntity( entity );

    // if controls told us to restart device, add message
    if( this.isDeviceRestartNeeded )  {
      this.DataMarkEntityForReboot();
    }

    // base-function: reload entity after save
    this.DataReloadAll();
    
  }
  

  //===========================================================================
  //  VALIDATION
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_CALLNUMBER" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_MACADDR" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_ADDRBOOK" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_CALLPROFILE" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_MOUNTTYPE" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_VOLUME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_SHOWCLOCK" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_CALLWAITING" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_RINGTONE" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "validate" );
    return numErr;
  }

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }
  
  //-----------------------------------------------------------------------------
  //  permission check for own statepoints
  //
  PermissionCheck( entinfo )
  {
    $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "event", "permission-check", entinfo );
    $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "event", "permission-check", entinfo );
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "event", "permission-check", entinfo );
  }

  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  OnUnitTest( testController )
  {
    testController.Assert( "uuid > 32", this.entity['uuid'].length > 32 );
    testController.AssertObject( "Entity class", this.entity, "class", "avp" );

    // Hint: UnitTest() will call all widgets OnUnitTest() automatically
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormAvp.Register( FormAvp.template );
FormFactory.Register( FormAvp );
//PageMain.RegisterForm( "avp", FormAvp.template, FormAvp.title );
//=============================================================================
//
//  Door Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          28.07.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormDoor extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/door";          // vw template
  static formId     = "door";                         // entity class
  static icon       = "mdi mdi-key";                  // icon for toolbar & doc
  static title      = "{$door_document_title}";       // default title if not yet loaded
  static creatable  = true;                           // self explaining
  static container  = false;                          // only folder can create subitems
  static createTemplate = [                           // menu entries in "add" command
    { text: "{$door_create_type}", deviceTemplate: "ATLC_670-0" }
  ];
  helpFile   = "";
  
  //=============================================================================
  //  STATIC FUNCTIONS
  //=============================================================================

  static MinimumNeededLicenses()
  {
    return { "APR": 1 };
  }
    
  //-----------------------------------------------------------------
  //  might be overwritten for special entities
  //
  IsDeleteAllowed( entinfo )
  {
    return true;
  }


  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: [
        { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
        { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
        { id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', tooltip: "{$const_form_tooltip_permission}", status: 'normal' },
        { id: 'ID_LISTS', icon: 'mdi mdi-format-list-bulleted', tooltip: "{$const_form_tooltip_lists}", status: "hidden" },
        { id: 'ID_DEVICERESTART', icon: 'mdi mdi-restart', tooltip: "{$const_form_tooltip_devicerestart}", status: "normal" },
        { id: 'ID_DELETE', icon: 'mdi mdi-delete', tooltip: "{$const_form_tooltip_delete}", status: "red" },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );

    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this )
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: "normal",
    } );

    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly",
    } );

    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    Data.Instance().On( "MODULES/UPDATED/", this.etag, this.DataModulesUpdated.bind( this ) );
    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );
    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  read entity into all controls
  //
  FormFillControls()
  {
    // standard controls
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );
    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( "option", "value", this.entity['deviceTemplate'] );

    // entity controls

    // call number
    ControlCallNumber.Build( $( "#ID_CALLNUMBER" + this.dynid ), {
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // mac
    ControlMacAddress.Build( $( "#ID_MACADDR" + this.dynid ), {
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // call profile
    ControlCallProfile.Build( $( "#ID_CALLPROFILE" + this.dynid ), {
      entity: this.entity
    } );

    // door camera
    ControlDoorCamera.Build( $( "#ID_DOORCAMERA" + this.dynid ), {
      entity: this.entity
    } );

    // volume deviceConfig
    ControlDeviceVolume.Build( $( "#ID_VOLUME" + this.dynid ), { 
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    ControlKeyTone.Build( $( "#ID_KEYTONES" + this.dynid ), {
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    ControlZam.Build( $( "#ID_ZAM" + this.dynid ), {
      entity: this.entity,
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );    

    

    // group controls
    
    // KBD deviceConfig
    ControlDeviceConfigGroup.Build( $( "#ID_DEVGROUP_KBD" + this.dynid ), {
      entity: this.entity,
      module: ["ATLM", "BTM", "BRMA" ],
      result: this.ControlEntityResult.bind( this ),
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // IO deviceConfig
    ControlDeviceConfigGroup.Build( $( "#ID_DEVGROUP_IO" + this.dynid ), {
      entity: this.entity,
      module: ["ATLC", "ATLCE1", "ATLCE2", "ATLCE3", "ATLCE4"],
      result: this.ControlEntityResult.bind( this ),
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // function deviceConfig
    ControlDeviceFunctionGroup.Build( $( "#ID_DEVGROUP_FUNCTION" + this.dynid ), {
      entity: this.entity,
      module: [ "function" ],
      result: this.ControlEntityResult.bind( this ),
      restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

  }

  //---------------------------------------------------------------------------
  //  read control values
  //
  FormReadControls()
  {
    // get value from standard controls
    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );

    // all "entity-controls" save directly into entity
    $( "#ID_VOLUME" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_KEYTONES" + this.dynid ).viewable( "option", "save", this.entity );    
    $( "#ID_CALLNUMBER" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_CALLPROFILE" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_DOORCAMERA" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_MACADDR" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_ZAM" + this.dynid ).viewable( "option", "save", this.entity );

    // short before save all widgets are called to update itself
    $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "save", this.entity );

  }


  //===========================================================================
  //  CONTROL HANDLER
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":           this.CommandUndo();           break;
      case "ID_SAVE":           this.CommandSave();           break;
      case "ID_PERMISSIONS":    this.CommandPermissions();    break;
      case "ID_LISTS":          this.CommandLists();          break;
      case "ID_DEVICERESTART":  this.CommandDeviceRestart();  break;
      case "ID_DELETE":         this.CommandDelete();         break;
    }
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;

    this.FormTitle( this.entity['name'] );
    $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "option", "rebuild", this.entity );
    $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "option", "rebuild", this.entity );
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );

    this.ControlRestartNeeded();
  }

  //---------------------------------------------------------------------------
  //  device config has been changed.
  //
  ControlEntityResult( entity )
  {
    this.entity = entity;

    // exception: master version of entity has been changed, so update all
    // widgets that does not have a save handler
    $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "option", "rebuild", this.entity );
    $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "option", "rebuild", this.entity );
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }

  //---------------------------------------------------------------------------
  //  device config says: We need to restart the device
  //
  ControlRestartNeeded()
  {
    this.isDeviceRestartNeeded = true;
  }

  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = this.Validate();

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  CommandUndo(): undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }



  //===========================================================================
  //  DATA
  //===========================================================================

  //---------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //---------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

    // set help file
    switch( this.entity['deviceTemplate'] )  {
      case "AFS":         this.helpFile = "afs";    break;
      case "AFSV":        this.helpFile = "afsv";   break;
      case "AHF_870-0":   this.helpFile = "ahf";    break;
      case "AHFV_870-0":  this.helpFile = "ahfv";   break;
      case "AHT_870-0":   this.helpFile = "aht";    break;
      case "AHTV_870-0":  this.helpFile = "ahtv";   break;
    }

    // while developing the unit test uncomment this:
    // this.UnitTest();

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();

  }

  //---------------------------------------------------------------------------
  //  update description lines if some names has been changed
  //
  DataEntinfoUpdate( entinfo, changedList )
  {
    // other entity has been changed
    if( entinfo['entid'] != this.entinfo['entid'] )  {
      $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "option", "rebuild", this.entity );
      $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "option", "rebuild", this.entity );
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
    }
    else {
      super.DataEntinfoUpdate( entinfo, changedList );
    }
  }

  //---------------------------------------------------------------------------
  //  module list has been changed
  //
  DataModulesUpdated()
  {
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }

  //---------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest( );

    // create a copy for save: name & label-handling + reference reset.
    let entity = this.DataPrepareEntityForSave( this.entity );

    // add all references
    this.DataAddReferences( entity, $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_CALLPROFILE" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_DOORCAMERA" + this.dynid ).viewable( "option", "references" ) );

    // finally filter all internal references
    this.DataFilterReferencesForSave( entity );

    // save modified entity
    Data.Instance().SavePartialEntity( entity );

    // if controls told us to restart device, add message
    if( this.isDeviceRestartNeeded )  {
      this.DataMarkEntityForReboot();
    }

    // base-function: reload entity after save
    this.DataReloadAll();

  }


  //===========================================================================
  //  VALIDATION
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_CALLNUMBER" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_MACADDR" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_CALLPROFILE" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DOORCAMERA" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_VOLUME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_KEYTONES" + this.dynid ).viewable( "validate" );
    if( numErr == NaN )  {
      console.error( "errnum NaN!" );
    }
    return numErr;
  }

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }


  //-----------------------------------------------------------------------------
  //  permission check for own statepoints
  //
  PermissionCheck( entinfo )
  {
    $( "#ID_DEVGROUP_KBD" + this.dynid ).viewable( "event", "permission-check", entinfo );
    $( "#ID_DEVGROUP_IO" + this.dynid ).viewable( "event", "permission-check", entinfo );
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "event", "permission-check", entinfo );
  }
  

  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  OnUnitTest( testController )
  {
    testController.Assert( "uuid > 32", this.entity['uuid'].length > 32 );
    testController.AssertObject( "Entity class", this.entity, "class", "door" );

    // Hint: UnitTest() will call all widgets OnUnitTest() automatically
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormDoor.Register( FormDoor.template );
FormFactory.Register( FormDoor );//=============================================================================
//
//  TK Gateway Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          28.07.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormTkgw extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/tkgw";        // vw template
  static formId     = "tkgw";                       // entity class
  static icon       = "mdi mdi-phone-voip";         // icon for toolbar & doc
  static title      = "{$tkgw_document_title}";     // default title if not yet loaded
  static creatable  = true;                         // self explaining
  static container  = false;                        // only folder can create subitems
  static createTemplate = [                         // menu entries in "add" command
    { text: "{$tkgw_create_type}", deviceTemplate: null, folder: null }
  ];
  helpFile = "tkgw";

  //=============================================================================
  //  STATIC FUNCTIONS
  //=============================================================================

  static MinimumNeededLicenses()
  {
    return { "ALT": 1 };
  }

  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};
    this.modeIncomingCalls = null; // "modeRejectIncoming" | "modeForwardIncoming"
     

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: [
        { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },        
        { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
        { id: 'ID_DELETE', icon: 'mdi mdi-delete', status: "red", tooltip: "{$const_form_tooltip_delete}" },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );

    //-------------------------------------------
    //  Group "general"
    //-------------------------------------------

    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this )
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: "normal"
    } );

    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly"
    } );

    // Incoming call handling: checkbox + optional dropdown.
    $( "#ID_INCOMING" + this.dynid ).choice( {
      size: CONST.size.L,
      text: "{$tkgw_incoming_forward}",
      type: "check",
      result: this.ControlForwardResult.bind( this )      
      //value:
    } );

    // "DTMF only for calls door => gateway".
    $( "#ID_DTMF_FROM_DOOR" + this.dynid ).choice( {
      size: CONST.size.L,
      text: "{$formbase_value_dtmf_from_door_only}",
      type: "check"
    } );

    //-------------------------------------------
    //  Group "gateway"
    //-------------------------------------------

    $( "#ID_REGISTRAR" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "normal",
      validate: Validate.IsIpOrHost
    } );

    $( "#ID_ACCOUNT" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      validate: this.ValidateEditAccount.bind( this )         
    } );

    $( "#ID_FRAMESIZE" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "droplist",
      selected: ['AUTO'],
      data: CONST.framesizes
    } );

    $( "#ID_TRANSPORT" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "droplist",
      selected: ['UDP'],
      data: [
        { id: "UDP", text: "{$tkgw_transport_udp}" },
        { id: "TCP", text: "{$tkgw_transport_tcp}" },
        { id: "TLS", text: "{$tkgw_transport_tls}" }
      ]
    } );

    $( "#ID_PORT" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      filter: { allow: "0123456789" },      
      validate: this.ValidateEditPort.bind( this )      
    } );


    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  read entity into all controls
  //
  FormFillControls()
  {
    // Group "general".
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );
    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( "option", "value", "{$tkgw_document_title}" );

    ControlCallPrefix.Build( $( "#ID_PREFIX" + this.dynid ), {
      entity: this.entity,
    } );    

    // Incoming call forwarding.
    if( typeof this.entity['phone']['incomingCalls'] == "string" && this.entity['phone']['incomingCalls'].length > 0 )  {
      $( "#ID_INCOMING" + this.dynid ).choice( "option", "value", true );
      this.ModeForwardIncoming( );
    }
    else  {
      this.ModeRejectIncoming( );
    }

    // DTMF door/light
    ControlDtmfDoorAndLight.Build( $( "#ID_DTMF" + this.dynid ), {
      entity: this.entity,
    } );
    $( "#ID_DTMF_FROM_DOOR" + this.dynid ).choice( "option", "value", this.entity['phone']['dtmfOnlyOutgoing'] );
 
    // Group "gateway".
    $( "#ID_REGISTRAR" + this.dynid ).edit( "option", "value", this.entity['tkgw']['registrar'] );
    $( "#ID_ACCOUNT" + this.dynid ).edit( "option", "value", this.entity['tkgw']['account'] );

    ControlPassword.Build( $( "#ID_PASSWORD" + this.dynid ), {
      entity: this.entity,
    } );

    // Framesize
    let framesize = this.entity['phone']['framesize'];
    let framesizeStringValue = framesize.toString();
    $( "#ID_FRAMESIZE" + this.dynid ).listbox ( "option", "selected", [ framesizeStringValue ] );

    // Transport
    let transport = "UDP";
    switch( this.entity['tkgw']['transport'] )  {
      case "tcp": transport = "TCP"; break;
      case "tls": transport = "TLS";
    }
    $( "#ID_TRANSPORT" + this.dynid ).listbox ( "option", "selected", [ transport ] );

    // Port number
    let port = this.entity['tkgw']['port'];
    if( typeof port != "number" || port <= 0 )  {
      port = 5060;
    }
    $( "#ID_PORT" + this.dynid ).edit( "option", "value", port );
  }

  //---------------------------------------------------------------------------
  //  Read control values back into entity
  //
  FormReadControls()
  {
    // Tell widget controls to save directly into entity.
    // All other controls write directly into entity.

    // Group "general".
    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );

    $( "#ID_PREFIX" + this.dynid ).viewable( "option", "save", this.entity );

    // Incoming call forwarding.
    if( $( "#ID_INCOMING" + this.dynid ).choice( "option", "value" ) == true ) {
      let forwardDestination = $( "#ID_INCOMING_FORWARD_TO" + this.dynid ).viewable( "option", "selected" )[ 0 ];
      if( typeof forwardDestination == "string" )  {
        this.entity['phone']['incomingCalls'] = forwardDestination;
      }
      else  {
        this.entity['phone']['incomingCalls'] = "";
      }
    }
    else  {
      this.entity['phone']['incomingCalls'] = "";
    }    

    // DTMF
    $( "#ID_DTMF" + this.dynid ).viewable( "option", "save", this.entity );
    this.entity['phone']['dtmfOnlyOutgoing'] = $( "#ID_DTMF_FROM_DOOR" + this.dynid ).choice( "option", "value" );


    // Group "gateway".
    this.entity['tkgw']['registrar'] = $( "#ID_REGISTRAR" + this.dynid ).edit( "option", "value" );
    this.entity['tkgw']['account'] = $( "#ID_ACCOUNT" + this.dynid ).edit( "option", "value" );

    $( "#ID_PASSWORD" + this.dynid ).viewable( "option", "save", this.entity );

    this.entity['phone']['framesize'] = parseInt( $( "#ID_FRAMESIZE" + this.dynid ).listbox( "option", "selected" )[ 0 ] );
    if( typeof this.entity['phone']['framesize'] != "number" )  {
      this.entity['phone']['framesize'] = 0; // default "Auto"
    }
 
    let transport = $( "#ID_TRANSPORT" + this.dynid ).listbox( "option", "selected" )[ 0 ];
    switch( transport )  {
      case "TCP": this.entity['tkgw']['transport'] = "tcp"; break;
      case "TLS": this.entity['tkgw']['transport'] = "tls"; break;
      default: this.entity['tkgw']['transport'] = "udp";
    }

    this.entity['tkgw']['port'] = parseInt( $( "#ID_PORT" + this.dynid ).edit( "option", "value" ) );
  }


  //===========================================================================
  //  CONTROL HANDLER
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":           this.CommandUndo();           break;
      case "ID_SAVE":           this.CommandSave();           break;
      case "ID_DELETE":         this.CommandDelete();         break;
    }
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;
    this.FormTitle( this.entity['name'] );
  }

  //---------------------------------------------------------------------------
  //  device config has been changed.
  //
  ControlEntityResult( entity )
  {
    this.entity = entity;
  }

  //---------------------------------------------------------------------------
  //  Call forwarding switched on or off.
  //    
  ControlForwardResult( result )
  {
    if( result )  {
      this.ModeForwardIncoming( );
    }
    else  {
      this.ModeRejectIncoming( );
    }
  }

  //---------------------------------------------------------------------------
  //  A call forward target has been selected from dropdown.
  //  
  ControlForwardTargetResult( result )
  {
  }


  //===========================================================================
  //  BUTTON etc. CONTROL
  //===========================================================================


  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = this.Validate();

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  CommandUndo(): undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }


  //===========================================================================
  //  DATA
  //===========================================================================

  //---------------------------------------------------------------------------
  //  do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //-----------------------------------------------------------------------------
  //  response from xhr
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

    // while developing the unit test uncomment this:
    // this.UnitTest();

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();
  
  }

  //-----------------------------------------------------------------------------
  //  some entinfo has been changed
  //
  DataEntinfoUpdate( entinfo, changedList )
  {
    // this entity has been changed
    if( entinfo['entid'] == this.entinfo['entid'] )  {
      super.DataEntinfoUpdate( entinfo, changedList );
    }
  }

  //-----------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest();

    // save modified entity (don't change status manually!)
    let entity = this.DataPrepareEntityForSave( this.entity );
    Data.Instance().SavePartialEntity( entity );

    // base-function: reload entity after save
    this.DataReloadAll();

  }


  //===========================================================================
  //  MODES
  //===========================================================================

  //---------------------------------------------------------------------------
  //  "Forward incoming calls" is unchecked.
  //
  ModeRejectIncoming()
  {
    if( this.modeIncomingCalls == "modeRejectIncoming" )  return;
    this.modeIncomingCalls = "modeRejectIncoming";

    //console.log("ModeRejectIncoming");

    // Must be built even if hidden, because otherwise "validate false" crashes.
    ControlCalltargetList.Build( $( "#ID_INCOMING_FORWARD_TO" + this.dynid ), {
      size: CONST.size.L,
      result: this.ControlForwardTargetResult.bind( this ),
      forbiddenEntityId: this.entity['entid'],
      addressBook: null
    } );

    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INCOMING_FORWARD_TO" );
    $( "#ID_INCOMING_FORWARD_TO" + this.dynid ).viewable( "validate", false );
  }

  //---------------------------------------------------------------------------
  //  "Forward incoming calls" is checked.
  //
  ModeForwardIncoming()
  {
    if( this.modeIncomingCalls == "modeForwardIncoming" )  return;
    this.modeIncomingCalls = "modeForwardIncoming";

    //console.log("ModeForwardIncoming");
        
    let destEntityId = this.entity['phone']['incomingCalls'];
    let select = [];
    if( typeof destEntityId == "string" && destEntityId.length > 0 )  {
      select = [ destEntityId ];
    }

    ControlCalltargetList.Build( $( "#ID_INCOMING_FORWARD_TO" + this.dynid ), {
      size: CONST.size.L,
      result: this.ControlForwardTargetResult.bind( this ),
      forbiddenEntityId: this.entity['entid'],
      addressBook: null,
      selected: select
    } );

    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_INCOMING_FORWARD_TO" );
  }


  //===========================================================================
  //  VALIDATION
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_PREFIX" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_PASSWORD" + this.dynid ).viewable( "validate" );
    if( this.modeIncomingCalls == "modeForwardIncoming" )  {
      numErr += $( "#ID_INCOMING_FORWARD_TO" + this.dynid ).viewable( "validate" );
    }
    numErr += $( "#ID_DTMF" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_REGISTRAR" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_ACCOUNT" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_PORT" + this.dynid ).edit( "validate" );

    return numErr;
  }

  //-----------------------------------------------------------------------------
  //  account need to be validated
  //     
  ValidateEditAccount( validateControl )
  {
    if( validateControl.value.length == 0 )  {
      validateControl.error = "{$const_error_empty}";
      return 1;
    }
    validateControl.error = null;
    return 0;
  }

  //-----------------------------------------------------------------------------
  //  Validate port number: Must not be empty or 0. 
  //  
  ValidateEditPort( validateControl )
  {
    if( typeof validateControl.value == "string" ) {
      let port = parseInt( validateControl.value );
      if( typeof port == "number" && port > 0 )  {
        validateControl.error = null;
        return 0;
      }
    }
    validateControl.error = "{$tkgw_error_port_empty}";
    return 1;
  }


  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

  
}


//=============================================================================
//  REGISTER
//=============================================================================

FormTkgw.Register( FormTkgw.template );
FormFactory.Register( FormTkgw );
//=============================================================================
//
//  SIP-Trunk Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          28.07.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormTrunk extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/trunk";       // vw template
  static formId     = "trunk";                      // entity class
  static icon       = "mdi mdi-phone-settings";     // icon for toolbar & doc
  static title      = "{$trunk_document_title}";    // default title if not yet loaded
  static creatable  = true;                         // self explaining
  static container  = false;                        // only folder can create subitems
  static createTemplate = [                         // menu entries in "add" command
    { text: "{$trunk_create_type}", deviceTemplate: null, folder: null }
  ];
  helpFile = "trunk";


  //=============================================================================
  //  STATIC FUNCTIONS
  //=============================================================================

  static MinimumNeededLicenses()
  {
    return { "ALT": 1 };
  }

  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};

    this.currentLicensedLines = 0;

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: [
        { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },        
        { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
        { id: 'ID_DELETE', icon: 'mdi mdi-delete', status: "red", tooltip: "{$const_form_tooltip_delete}" },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );

    //-------------------------------------------
    //  Group "general"
    //-------------------------------------------

    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this )
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: "normal",
    } );

    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly",
    } );

    // Note: For whatever reason old admin allows *# for remote prefix, but not prefix!    
    $( "#ID_REMOTE_PREFIX" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      filter: { allow: "0123456789*#" }
    } );
    
    $( "#ID_LINES" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      filter: { allow: "0123456789" },
      validate: this.ValidateLines.bind( this )
    } );

    $( "#ID_DTMF_FROM_DOOR" + this.dynid ).choice( {
      size: CONST.size.L,
      text: "{$formbase_value_dtmf_from_door_only}",
      type: "check"
    } );

    //-------------------------------------------
    //  Group "trunk"
    //-------------------------------------------

    $( "#ID_IPADDRESS" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "normal",
      validate: Validate.IsIpOrHost
    } );

    $( "#ID_PORT" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      value: "5060",
      filter: { allow: "0123456789" },      
      validate: this.ValidateEditPort.bind( this )      
    } );

    $( "#ID_FRAMESIZE" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "droplist",
      selected: ['AUTO'],
      data: CONST.framesizes
    } );


    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );
    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  read entity into all controls
  //
  FormFillControls()
  {
    // Group "general".  
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );
    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( "option", "value", "{$trunk_document_title}" );

    ControlCallPrefix.Build( $( "#ID_PREFIX" + this.dynid ), {
      entity: this.entity,
    } );

    $( "#ID_REMOTE_PREFIX" + this.dynid ).edit( "option", "value", this.entity['phone']['fromprefix'] );

    this.currentLicensedLines = parseInt( this.entity['phone']['lines'] );
    $( "#ID_LINES" + this.dynid ).edit( "option", "value", this.entity['phone']['lines'].toString() );

    // DTMF door/light
    ControlDtmfDoorAndLight.Build( $( "#ID_DTMF" + this.dynid ), {
      entity: this.entity,
    } );
    $( "#ID_DTMF_FROM_DOOR" + this.dynid ).choice( "option", "value", this.entity['phone']['dtmfOnlyOutgoing'] );      

    // Group "sip trunk".
    $( "#ID_IPADDRESS" + this.dynid ).edit( "option", "value", this.entity['trunk']['ipaddress'] );    
    if( parseInt( this.entity['trunk']['port'] ) > 0 ) {
      $( "#ID_PORT" + this.dynid ).edit( "option", "value", this.entity['trunk']['port'] );    
    }
    else {
      $( "#ID_PORT" + this.dynid ).edit( "option", "value", "5060" );
    }

    let framesize = this.entity['phone']['framesize'];
    let framesizeStringValue = framesize.toString();
    $( "#ID_FRAMESIZE" + this.dynid ).listbox ( "option", "selected", [ framesizeStringValue ] );    
  }

  //---------------------------------------------------------------------------
  //  Read control values back into entity
  //
  FormReadControls()
  {
    // Tell widget controls to save directly into entity.
    // All other controls write directly into entity.

    // Group "general".
    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );

    $( "#ID_PREFIX" + this.dynid ).viewable( "option", "save", this.entity );

    this.entity['phone']['fromprefix'] = $( "#ID_REMOTE_PREFIX" + this.dynid ).edit( "option", "value" );
    this.entity['phone']['lines'] = parseInt( $( "#ID_LINES" + this.dynid ).edit( "option", "value" ) );

    // DTMF
    $( "#ID_DTMF" + this.dynid ).viewable( "option", "save", this.entity );
    this.entity['phone']['dtmfOnlyOutgoing'] = $( "#ID_DTMF_FROM_DOOR" + this.dynid ).choice( "option", "value" );

    // Group "sip trunk".
    this.entity['trunk']['ipaddress'] = $( "#ID_IPADDRESS" + this.dynid ).edit( "option", "value" );
    this.entity['trunk']['port'] = parseInt( $( "#ID_PORT" + this.dynid ).edit( "option", "value" ) );

    this.entity['phone']['framesize'] = parseInt( $( "#ID_FRAMESIZE" + this.dynid ).listbox( "option", "selected" )[ 0 ] );
    if( typeof this.entity['phone']['framesize'] != "number" )  {
      this.entity['phone']['framesize'] = 0; // default "Auto"
    }    
  }

  //===========================================================================
  //  CONTROL HANDLER
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":           this.CommandUndo();           break;      
      case "ID_SAVE":           this.CommandSave();           break;
      case "ID_DELETE":         this.CommandDelete();         break;
    }
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;
    this.FormTitle( this.entity['name'] );
  }

  //---------------------------------------------------------------------------
  //  device config has been changed.
  //
  ControlEntityResult( entity )
  {
    this.entity = entity;
  }


  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = this.Validate();

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  CommandUndo(): undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }


  //===========================================================================
  //  DATA
  //===========================================================================

  //---------------------------------------------------------------------------
  //  do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //-----------------------------------------------------------------------------
  //  response from xhr
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

    // while developing the unit test uncomment this:
    // this.UnitTest();

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();
  
  }

  //-----------------------------------------------------------------------------
  //  some entinfo has been changed
  //
  DataEntinfoUpdate( entinfo, changedList )
  {
    // this entity has been changed
    if( entinfo['entid'] == this.entinfo['entid'] )  {
      super.DataEntinfoUpdate( entinfo, changedList );
    }    
  }

  //-----------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest();

    // save modified entity (don't change status manually!)
    let entity = this.DataPrepareEntityForSave( this.entity );
    Data.Instance().SavePartialEntity( entity );
    
    // base-function: reload entity after save
    this.DataReloadAll();

  }

  //===========================================================================
  //  VALIDATION
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  check if enough licenses are available 
  //
  ValidateLines( validateControl )  
  {
    var licenseInfo = Data.Instance().Licences();
    var freeLicenes = licenseInfo['ALT']['availabeLicenses'] - licenseInfo['ALT']['engagedLicenses'];
    var additionalNeededLines = parseInt( validateControl.value ) - this.currentLicensedLines;

    if( additionalNeededLines > freeLicenes )  {
      validateControl.error = "{$trunk_error_license}";
      return 1;
    }
    
    return 0;
  }

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_PREFIX" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DTMF" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_IPADDRESS" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_PORT" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_LINES" + this.dynid ).edit( "validate" );

    return numErr;
  }


  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  Validate port number: Must not be empty or 0. 
  //  
  ValidateEditPort( validateControl )
  {
    if( typeof validateControl.value == "string" ) {
      let port = parseInt( validateControl.value );
      if( typeof port == "number" && port > 0 )  {
        validateControl.error = null;
        return 0;
      }
    }
    validateControl.error = "{$tkgw_error_port_empty}";
    return 1;
  }  

  
}


//=============================================================================
//  REGISTER
//=============================================================================

FormTrunk.Register( FormTrunk.template );
FormFactory.Register( FormTrunk );
//=============================================================================
//
//  Apartment Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.06.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormGroup extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/group";         // vw template
  static formId     = "group";                        // entity class
  static icon       = "mdi mdi-account-multiple";     // icon for toolbar & doc
  static title      = "{$group_document_title}";      // default title if not yet loaded
  static creatable  = true;                           // self explaining
  static container  = false;                          // only folder can create subitems
  static createTemplate = [                           // menu entries in "add" command
    { text: "{$group_create_type}",       deviceTemplate: null,        folder: null },
  ];
  helpFile   = "group";
  
  //-----------------------------------------------------------------
  //  might be overwritten for special entities
  //
  IsDeleteAllowed( entinfo )
  {
    return true;
  }


  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};
    this.memberDataset = new Dataset( { id: "entid" } );
    this.references = [];

    this.callTargetDataset = Data.Instance().CalltargetDataset();
    this.callTargetDataview = this.callTargetDataset.BuildView( 'default', {
      rebuildAlways: true,
      sortOrder: "+",
      sortKey: "text",
      filter: this.ControlCallTargetDataviewFilter.bind( this )
    } );

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: [
        { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
        { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
        { id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', tooltip: "{$const_form_tooltip_permission}", status: 'normal' },
        { id: 'ID_DELETE', icon: 'mdi mdi-delete', tooltip: "{$const_form_tooltip_delete}", status: "red" },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );

    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this )
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: "normal",
    } );

    $( "#ID_CALLREJECT" + this.dynid ).choice( {
      size: CONST.size.XL,
      type: "check",
      text: "{$group_callreject_check}"
    } );

    $( "#ID_MISSEDCALL" + this.dynid ).choice( {
      size: CONST.size.XL,
      type: "check",
      text: "{$group_missedcall_check}"
    } );

    $( "#ID_VOICEMAIL" + this.dynid ).choice( {
      size: CONST.size.XL,
      type: "check",
      text: "{$group_voicemail_check}"
    } );

    $( "#ID_MEMBERLIST" + this.dynid ).grid( {
      size: CONST.size.XL,
      height: 7,
      data: this.memberDataset,
      placeholder: "{$group_member_placeholder}",
      scrollbar: 'overlay', // 'overlay', 'static'
      header: {
        name: { text: "{$group_member_header_name}", size: 45 },
        callnumber: { text: "{$group_member_header_callnumber}", size: 23 }
      },
      result: this.ControlMemberlistGridResult.bind( this ),
      renderCell: ( columnId, record ) => {
        if( columnId == "name" ) {
          return {
            text: record['name'],
            icon: record['icon']
          };
        }
        return { text: record[ columnId ] };
      },
      validate: this.ValidateMemberList.bind( this )
    } );

    $( "#ID_MEMBERLIST_ADD" + this.dynid ).button( {
      size: CONST.size.M,
      status: "normal",
      text: "{$group_member_button_add}",
      tooltip: "{$group_member_button_tooltip}",
      click: () => {
        this.callTargetDataview.Update();
        $( '#ID_MEMBERLIST_POPUP' + this.dynid ).listbox( "show" );
      }
    } );

    $( '#ID_MEMBERLIST_POPUP' + this.dynid ).listbox( {
      size: 40,
      height: 5,
      type: 'listbox',
      search: true,
      focus: true,
      status: "normal",
      searchPlaceholder: "{$const_common_search_placeholder}",
      anchor: "#ID_MEMBERLIST_ADD" + this.dynid,
      position: "bottomleft",
      data: this.callTargetDataview,
      result: this.ControlMemberPopupResult.bind( this )
    } );

    $( "#ID_MEMBERLIST_DELETE" + this.dynid ).button( {
      size: 4,
      icon: "mdi mdi-delete",
      tooltip: "{$const_form_tooltip_delete}",
      click: this.ControlMemberDeleteClick.bind( this )
    } );

    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    Data.Instance().On( "MODULES/UPDATED/", this.etag, this.DataModulesUpdated.bind( this ) );
    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );
    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  read entity into all controls
  //
  FormFillControls()
  {
    // standard controls
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );
    $( "#ID_CALLREJECT" + this.dynid ).choice( "option", "value", this.entity['phone']['rejectGlobal'] );
    $( "#ID_MISSEDCALL" + this.dynid ).choice( "option", "value", this.entity['phone']['saveAcceptedAsMissed'] );
    $( "#ID_VOICEMAIL" + this.dynid ).choice( "option", "value", this.entity['phone']['privateVoicemail'] );

    // entity controls

    // call number
    ControlCallNumber.Build( $( "#ID_CALLNUMBER" + this.dynid ), {
      entity: this.entity,
      //restartNeeded: this.ControlRestartNeeded.bind( this ),
      result: this.ControlEntityCallnumber.bind( this )
    } );

    // group controls
    
    // function deviceConfig
    ControlDeviceFunctionGroup.Build( $( "#ID_DEVGROUP_FUNCTION" + this.dynid ), {
      entity: this.entity,
      module: [ "function" ],
      result: this.ControlEntityResult.bind( this ),
      //restartNeeded: this.ControlRestartNeeded.bind( this )
    } );

    // group members
    this.FormFillMemberList();

  }

  //---------------------------------------------------------------------------
  //  read control values
  //
  FormReadControls()
  {
    // get value from standard controls
    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );
    this.entity['phone']['rejectGlobal'] = $( "#ID_CALLREJECT" + this.dynid ).choice( "option", "value" );
    this.entity['phone']['saveAcceptedAsMissed'] = $( "#ID_MISSEDCALL" + this.dynid ).choice( "option", "value" );
    this.entity['phone']['privateVoicemail'] = $( "#ID_VOICEMAIL" + this.dynid ).choice( "option", "value" );

    // all "entity-controls" save directly into entity
    $( "#ID_CALLNUMBER" + this.dynid ).viewable( "option", "save", this.entity );

    // short before save all widgets are called to update itself
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "save", this.entity );

    // member list
    this.FormReadMemberList();

  }

  //---------------------------------------------------------------------------
  //  convert cdrule into dataset
  //
  FormFillMemberList()
  {
    let isError = false;
    let list = [];

    if( 'cdrule' in this.entity ) {
      for( let rule of this.entity['cdrule'] ) {

        if( rule['action'] != "CALLUSER" ) {
          continue;
        }

        let addr = rule['addr'].split( "/" )[ 1 ];
        let addrEntinfo = Data.Instance().Entinfo( addr );
        if( !addrEntinfo )  {
          addrEntinfo = {
            'name': "{$const_deleted_item}",
            'class': "none",
            'callnumber': "-"
          };
        }

        let record = {};
        record['entid'] = addr;
        record['name'] = addrEntinfo['name'];
        record['icon'] = CONST_CLASS_ICO[ addrEntinfo['class'] ];
        record['callnumber'] = addrEntinfo['callnumber'];
        if( addrEntinfo['class'] == "none" )  {
          record['status'] = "error";
          isError = true;
        }

        list.push( record );
      }
    }

    this.memberDataset.SetData( list );

    return isError;
  }

  //---------------------------------------------------------------------------
  //  convert cdrule into dataset
  //
  FormReadMemberList()
  {
    let data = this.memberDataset.GetData();
    let result = [];
    this.references = [];

    if( ( 'sp' in this.entity ) && ( 'redir' in this.entity['sp'] ) ) {
      result.push( {
        'statPT': this.entity['sp']['redir']['address'],
        'action': "REDIRECT"
      } );
    }

    for( let record of data ) {
      let target = "usr:" + Data.Instance().RootEntid() + "/" + record['entid'];
      result.push( {
        'addr': target,
        'tm1': 0,
        'tm2': 3600,
        'action': "CALLUSER"
      });

      let ref = "call:" + record['entid'];
      this.references.push( ref );
    }

    this.entity['cdrule'] = result;
  }


  //===========================================================================
  //  CONTROL HANDLER
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":           this.CommandUndo();           break;
      case "ID_SAVE":           this.CommandSave();           break;
      case "ID_PERMISSIONS":    this.CommandPermissions();    break;
      case "ID_DELETE":         this.CommandDelete();         break;
    }
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;

    this.FormTitle( this.entity['name'] );
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );

  }

  //---------------------------------------------------------------------------
  //  callnumber has been changed
  //  
  ControlEntityCallnumber( result )
  {
    if( ( 'phone' in this.entity ) && ( 'callnumber' in this.entity['phone'] ) ) {
      this.entity['phone']['callnumber'] = value;
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );      
    }

  }

  //---------------------------------------------------------------------------
  //  device config has been changed.
  //
  ControlEntityResult( entity )
  {
    this.entity = entity;

    // exception: master version of entity has been changed, so update all
    // widgets that does not have a save handler
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }

  //---------------------------------------------------------------------------
  //  popup "add member"
  //
  ControlMemberPopupResult( resultList )
  {
    for( let entid of resultList ) {

      let addrEntinfo = Data.Instance().Entinfo( entid );

      let record = {};
      record['entid'] = entid;
      record['name'] = addrEntinfo['name'];
      record['icon'] = CONST_CLASS_ICO[ addrEntinfo['class'] ];
      record['callnumber'] = addrEntinfo['callnumber'];
      this.memberDataset.Add( record );
    }
  }

  //---------------------------------------------------------------------------
  //  grid: selected
  //
  ControlMemberlistGridResult( result )
  {
    if( result.length > 0 )  {
      $( "#ID_MEMBERLIST_DELETE" + this.dynid ).button( "option", "status", "normal" );
    }
    else {
      $( "#ID_MEMBERLIST_DELETE" + this.dynid ).button( "option", "status", "disabled" );
    }
  }

  //---------------------------------------------------------------------------
  //  button delete
  //
  ControlMemberDeleteClick( )
  {
    let selected = $( "#ID_MEMBERLIST" + this.dynid ).grid( "option", "selected" );
    this.memberDataset.Delete( selected[ 0 ] );
    this.ControlMemberlistGridResult( [] );
    this.Option( "modified", 1 );
  }

  //---------------------------------------------------------------------------
  //  filter in add member dataview the already existing entries
  //
  ControlCallTargetDataviewFilter( entid, record )
  {
    let isAllowed = true;
    
    if( isAllowed && entid == this.entinfo['entid'] )  {
      isAllowed = false;
    }
    if( isAllowed && this.memberDataset.Has( entid ) )  {
      isAllowed = false;
    }
    if( isAllowed && record['class'] == "addrBook" ) {
      isAllowed = false;
    }
    if( isAllowed && record['class'] == "group" ) {
      isAllowed = false;
    }

    return isAllowed;
  }

  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = this.Validate();

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  CommandUndo(): undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }

  //===========================================================================
  //  DATA
  //===========================================================================

  //---------------------------------------------------------------------------
  //  a reference has been changed, so revalidate the form
  //  
  DataRevalidate()
  {
    // re-create data
    if( this.Option( "modified" ) <= 0 ) {
      this.FormFillMemberList();
    }

    // call base class (for verify)
    super.DataRevalidate();
  }
  //---------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //---------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

   
    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();

  }



  //---------------------------------------------------------------------------
  //  update description lines if some names has been changed
  //
  DataEntinfoUpdate( entinfo, changedList )
  {
    // other entity has been changed
    if( entinfo['entid'] != this.entinfo['entid'] )  {
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
    }
    else {
      super.DataEntinfoUpdate( entinfo, changedList );
    }
  }

  //---------------------------------------------------------------------------
  //  module list has been changed
  //
  DataModulesUpdated()
  {
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }  

  //---------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest( );

    // create a copy for save: name & label-handling + reference reset.
    let entity = this.DataPrepareEntityForSave( this.entity );

    // add all references
    this.DataAddReferences( entity, $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, this.references );

    // finally filter all internal references
    this.DataFilterReferencesForSave( entity );

    // save the entity
    Data.Instance().SavePartialEntity( entity );

  }

  //===========================================================================
  //  VALIDATION
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_CALLNUMBER" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_MEMBERLIST" + this.dynid ).grid( "validate" );
    numErr += $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "validate" );

    return numErr;
  }

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateMemberList( validateControl )
  {
    let isError = false;
    let dataview = validateControl.value;

    for( let index = 0; index < dataview.length; index++ ) {

      let record = dataview.GetByIndex( index );
      let recordId = dataview.GetIdByIndex( index );
      let entinfo = Data.Instance().Entinfo( record['entid'] );
      if( !entinfo ) {
        validateControl.error = " ";
        dataview.ItemAddStatus( recordId, "error" );
        isError = true;
      }
      else {
        dataview.ItemRemoveStatus( recordId, "error" );
      }
    }

    return isError ? 1 : 0;
  }

  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  OnUnitTest( testController )
  {
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormGroup.Register( FormGroup.template );
FormFactory.Register( FormGroup );//=============================================================================
//
//  SIP Audio Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          08.09.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormSipaudio extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/sipaudio";      // vw template
  static formId     = "sipaudio";                     // entity class
  static icon       = "mdi mdi-deskphone";    // icon for toolbar & doc
  static title      = "{$sipaudio_document_title}";   // default title if not yet loaded
  static creatable  = true;                           // self explaining
  static container  = false;                          // only folder can create subitems
  static createTemplate = [                           // menu entries in "add" command
    { text: "{$sipaudio_create_type}",  deviceTemplate: null, folder: null }
  ];
  helpFile   = "";
  
  //=============================================================================
  //  STATIC FUNCTIONS
  //=============================================================================

  static MinimumNeededLicenses()
  {
    return { "APR": 1, "SIPAudio": 1 };
  }

  //-----------------------------------------------------------------
  //  might be overwritten for special entities
  //
  IsDeleteAllowed( entinfo )
  {
    return true;
  }


  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: [
        { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
        { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
        { id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', tooltip: "{$const_form_tooltip_permission}", status: 'normal' },
        { id: 'ID_LISTS', icon: 'mdi mdi-format-list-bulleted', tooltip: "{$const_form_tooltip_lists}", status: "normal" },
        { id: 'ID_DELETE', icon: 'mdi mdi-delete', tooltip: "{$const_form_tooltip_delete}", status: "red" },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );

    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this )
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: "normal",
    } );

    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly",
    } );

    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    Data.Instance().On( "MODULES/UPDATED/", this.etag, this.DataModulesUpdated.bind( this ) );
    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );
    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  read entity into all controls
  //
  FormFillControls()
  {
    // standard controls
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );
    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( "option", "value", "{$sipaudio_document_title}" );

    // entity controls

    // call number
    ControlCallNumber.Build( $( "#ID_CALLNUMBER" + this.dynid ), {
      entity: this.entity,
      result: this.ControlEntityCallnumber.bind( this )
    } );

    // address book
    ControlAddressBook.Build( $( "#ID_ADDRBOOK" + this.dynid ), {
      entity: this.entity
    } );

    // call profile
    ControlCallProfile.Build( $( "#ID_CALLPROFILE" + this.dynid ), {
      entity: this.entity
    } );

    // Password
    ControlPassword.Build( $( "#ID_PASSWORD" + this.dynid ), {
      entity: this.entity,
    } );

    // call waiting
    ControlCallWaiting.Build( $( "#ID_CALLWAITING" + this.dynid ), {
      entity: this.entity
    } );

    // DTMF door/light
    ControlDtmfDoorAndLight.Build( $( "#ID_DTMF" + this.dynid ), {
      entity: this.entity,
    } );

    // group controls

    // function deviceConfig
    ControlDeviceFunctionGroup.Build( $( "#ID_DEVGROUP_FUNCTION" + this.dynid ), {
      entity: this.entity,
      module: [ "function" ],
      result: this.ControlEntityResult.bind( this )
    } );

  }

  //---------------------------------------------------------------------------
  //  read control values
  //
  FormReadControls()
  {
    // Tell widget controls to save directly into entity.
    // All other controls write directly into entity.

    // Group "general".

    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );

    $( "#ID_CALLNUMBER" + this.dynid ).viewable( "option", "save", this.entity );    
    $( "#ID_ADDRBOOK" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_CALLPROFILE" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_PASSWORD" + this.dynid ).viewable( "option", "save", this.entity );    
    $( "#ID_CALLWAITING" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_DTMF" + this.dynid ).viewable( "option", "save", this.entity );    

    // Group "functions".

    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "save", this.entity );

  }


  //===========================================================================
  //  CONTROL HANDLER
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":           this.CommandUndo();           break;
      case "ID_SAVE":           this.CommandSave();           break;
      case "ID_PERMISSIONS":    this.CommandPermissions();    break;
      case "ID_LISTS":          this.CommandLists();          break;
      case "ID_DELETE":         this.CommandDelete();         break;
    }
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;

    this.FormTitle( this.entity['name'] );
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );

  }

  //---------------------------------------------------------------------------
  //  callnumber has been changed
  //  
  ControlEntityCallnumber( result )
  {
    if( ( 'phone' in this.entity ) && ( 'callnumber' in this.entity['phone'] ) ) {
      this.entity['phone']['callnumber'] = value;
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );      
    }

  }

  //---------------------------------------------------------------------------
  //  device config has been changed.
  //
  ControlEntityResult( entity )
  {
    this.entity = entity;

    // exception: master version of entity has been changed, so update all
    // widgets that does not have a save handler
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }


  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = this.Validate( );

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  CommandUndo(): undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }

  //-------------------------------------------------------
  //  show device lists dialog
  //
  CommandLists() 
  { 
    DeviceListsDialog.Show( { entid: this.entinfo['entid'], addressBook: this.entity['addrBook'] } );
  }



  //===========================================================================
  //  DATA
  //===========================================================================

  //---------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //---------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

    // set help file
    this.helpFile = "sipaudio";

    // while developing the unit test uncomment this:
    // this.UnitTest();

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();

  }

  //---------------------------------------------------------------------------
  //  update description lines if some names has been changed
  //
  DataEntinfoUpdate( entinfo, changedList )
  {
    // other entity has been changed
    if( entinfo['entid'] != this.entinfo['entid'] )  {
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
    }
    else {
      super.DataEntinfoUpdate( entinfo, changedList );
    }
  }

  //---------------------------------------------------------------------------
  //  module list has been changed
  //
  DataModulesUpdated()
  {
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }    

  //---------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest( );

    // create a copy for save: name & label-handling + reference reset.
    let entity = this.DataPrepareEntityForSave( this.entity );

    // add all references
    this.DataAddReferences( entity, $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_CALLPROFILE" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_ADDRBOOK" + this.dynid ).viewable( "option", "references" ) );

    // finally filter all internal references
    this.DataFilterReferencesForSave( entity );

    // save the entity
    Data.Instance().SavePartialEntity( entity );

    // base-function: reload entity after save
    this.DataReloadAll();

  }

  //===========================================================================
  //  VALIDATION
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_CALLNUMBER" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_ADDRBOOK" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_CALLPROFILE" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_PASSWORD" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_CALLWAITING" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DTMF" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "validate" );

    return numErr;
  }

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  permission check for own statepoints
  //
  PermissionCheck( entinfo )
  {
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "event", "permission-check", entinfo );
  }


  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  OnUnitTest( testController )
  {
    testController.Assert( "uuid > 32", this.entity['uuid'].length > 32 );
    testController.AssertObject( "Entity class", this.entity, "class", "apartment" );

    // Hint: UnitTest() will call all widgets OnUnitTest() automatically
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormSipaudio.Register( FormSipaudio.template );
FormFactory.Register( FormSipaudio );
//=============================================================================
//
//  Sus App Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          09.09.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormSusapp extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/susapp";        // vw template
  static formId     = "susapp";                       // entity class
  static icon       = "mdi mdi-cellphone";    // icon for toolbar & doc
  static title      = "{$susapp_document_title}";     // default title if not yet loaded
  static creatable  = true;                           // self explaining
  static container  = false;                          // only folder can create subitems
  static createTemplate = [                           // menu entries in "add" command
    { text: "{$susapp_create_type}",  deviceTemplate: null, folder: null }
  ];
  helpFile   = "";

  static storefinderUrl = "https://www.siedle.com/siedle-server-storefinder";
  
  //=============================================================================
  //  STATIC FUNCTIONS
  //=============================================================================

  static MinimumNeededLicenses()
  {
    return { "APR": 1, "ALSA": 1 };
  }

  //-----------------------------------------------------------------
  //  might be overwritten for special entities
  //
  IsDeleteAllowed( entinfo )
  {
    return true;
  }


  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.isTabVisible = false;
    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};
    this.qrData = null;

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();
    $( "#ID_OPTIONAL" + this.dynid ).optional( {} );

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: [
        { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
        { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
        { id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', tooltip: "{$const_form_tooltip_permission}", status: 'normal' },
        { id: 'ID_LISTS', icon: 'mdi mdi-format-list-bulleted', tooltip: "{$const_form_tooltip_lists}", status: "normal" },
        { id: 'ID_DELETE', icon: 'mdi mdi-delete', tooltip: "{$const_form_tooltip_delete}", status: "red" },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );

    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      size: CONST.size.L + 5,
      result: this.ControlNameResult.bind( this )
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: "normal",
    } );

    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( {
      size: CONST.size.L + 5,
      status: "readonly",
    } );

    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_QRCODE" );
    $( "#ID_QRCODE" + this.dynid ).qrcode( {
      size: CONST.size.L + 5,
      border: true, //false,
      text: "" 
    } );
    $( "#ID_PRINT_QRCODE" + this.dynid ).qrcode( {
      size: 23,
      border: true, //false,
      text: ""
    } );

    $( "#ID_PRINT_QRLINK" + this.dynid ).qrcode( {
      size: 12.7,
      border: true, //false,
      text: FormSusapp.storefinderUrl
    } );
  

    $( "#ID_RESET_PARAM" + this.dynid ).button( {
      size: CONST.size.XS,
      icon: "mdi mdi-wrench",
      status: "normal",
      click: this.ControlSusReset.bind( this )
    } );

    $( "#ID_RESET_PARAM2" + this.dynid ).button( {
      size: CONST.size.XS,
      icon: "mdi mdi-wrench",
      status: "normal",
      click: this.ControlSusReset.bind( this )
    } );

    $( "#ID_QRCODE_PRINT" + this.dynid ).button( {
      size: CONST.size.XS,
      icon: "mdi mdi-printer",
      status: "normal",
      click: this.ControlSusPrint.bind( this )
    } );

    $( "#ID_QRVALID" + this.dynid ).edit( {
      size: CONST.size.L + 5,
      status: "readonly"
    } );        

    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_STATUS" );
    $( "#ID_SUS_STATUS" + this.dynid ).edit( {
      size: CONST.size.L + 5,
      status: "readonly",
    } );        

    $( "#ID_SUS_STATUS_EXPIRED" + this.dynid ).edit( {
      size: CONST.size.L + 5,
      status: "readonly",
      value: "{$susapp_susstatus_expired}",
      icon: "mdi mdi-alert yellow-text"
    } );        

    $( "#ID_RESET_PARAM_EXPIRED" + this.dynid ).button( {
      size: CONST.size.XS,
      icon: "mdi mdi-refresh",
      status: "normal",
      click: this.ControlSusRefreshExpiredQrcode.bind( this )
    } );    

    // sus error toast
    if( Data.Instance().GetSusStatus() == "ERROR" )  {
      setTimeout( () => {
        this.ToastShowSusError( "#ID_TOAST" + this.dynid );
      }, 250 );
    }

    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    Data.Instance().On( "MODULES/UPDATED/", this.etag, this.DataModulesUpdated.bind( this ) );
    Data.Instance().On( "SUS/STATUS", this.etag, this.DataSusStatusEvent.bind( this ) );

    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );
    this.DataRequest();

  }//load

  OnTabVisible( isVisible, entid )
  {
    this.isTabVisible = isVisible;
    if( isVisible && this.mode == "modeQr" ) {
      this.FormFillQrCode( this.qrData );
    }
  }


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  read entity into all controls
  //
  FormFillControls()
  {
    // standard controls
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_PRINT_NAME" + this.dynid ).text( this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );
    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( "option", "value", "{$susapp_document_title}" );

    // entity controls

    // call number
    ControlCallNumber.Build( $( "#ID_CALLNUMBER" + this.dynid ), {
      entity: this.entity,
      result: this.ControlEntityCallnumber.bind( this )
    } );

    if( this.entity['sus']['isScanned'] )  {
      this.ModeScanned();
    }
    else {
      this.ModeRequestQr();  //ModeRequestQr();
    }    

    // function deviceConfig
    ControlDeviceFunctionGroup.Build( $( "#ID_DEVGROUP_FUNCTION" + this.dynid ), {
      entity: this.entity,
      module: [ "function" ],
      result: this.ControlEntityResult.bind( this )
    } );

  }

  //---------------------------------------------------------------------------
  //  read entity into all controls
  //
  FormFillQrCode( qrKeys )
  {
    if( qrKeys != null )  {
      var qrcode = '{"susUrl": ' + '"' 
                 + qrKeys['susUrl'] + '"' 
                 + ',"endpointTransferSecret": "' 
                 + qrKeys['endpointTransferSecret'] 
                 + '","endpointSetupKey": "' 
                 + qrKeys['endpointSetupKey'] 
                 + '", "expirationDate": "' 
                 + qrKeys.expirationDate + '"}'
                 ;
      $( "#ID_QRCODE" + this.dynid ).qrcode( "option", "text", qrcode );
      $( "#ID_PRINT_QRCODE" + this.dynid ).qrcode( "option", "text", qrcode );
      var date = moment.utc( qrKeys['expirationDate'] ); 
      var localTime = moment.tz( date, moment.tz.guess() );
      $( "#ID_QRVALID" + this.dynid ).edit( "option", "value", localTime.format( "L LTS" ) );
      $( "#ID_PRINT_EXPIRY" + this.dynid ).text( localTime.format( "L" ) );    

    }
    else {
      $( "#ID_QRCODE" + this.dynid ).html( "<i class='mdi mdi-close'></i>" );
    }
  }

  //---------------------------------------------------------------------------
  //  read control values
  //
  FormReadControls()
  {
    // Group "general".
    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );

    $( "#ID_CALLNUMBER" + this.dynid ).viewable( "option", "save", this.entity );    

    // Group "functions".
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "save", this.entity );

  }


  //===========================================================================
  //  MODES
  //===========================================================================

  ModeScanned()
  {
    if( this.mode == "modeScanned" ) {
      return;
    }
    this.mode = "modeScanned";

    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_QRCODE" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_STATUS" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_STATUS_EXPIRED" );    
    $( "#ID_SUS_STATUS" + this.dynid ).edit( "option", "value", "{$susapp_susstatus_scanned}" );
    $( "#ID_SUS_STATUS" + this.dynid ).edit( "option", "icon", "mdi mdi-check-bold green-text" );
    $( "#ID_RESET_PARAM2" + this.dynid ).edit( "option", "status", "normal" );

    // --- print ---
    this.$container.find( ".sus-screen" ).removeClass( "no-print" );
    this.$container.find( ".sus-print" ).addClass( "hidden" );

  }

  ModeRequestQr()
  {
    if( this.mode == "modeRequestQr" ) {
      return;
    }
    this.mode = "modeRequestQr";

    
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_QRCODE" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_STATUS" );    
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_STATUS_EXPIRED" );    
    $( "#ID_SUS_STATUS" + this.dynid ).edit( "option", "value", "{$susapp_susstatus_request_qr}" );
    $( "#ID_SUS_STATUS" + this.dynid ).edit( "option", "icon", "" );
    $( "#ID_RESET_PARAM2" + this.dynid ).edit( "option", "status", "disabled" );
  }

  ModeQrExpired()
  {
    if( this.mode == "modeQrExpired" ) {
      return;
    }
    this.mode = "modeQrExpired";
    
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_QRCODE" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_STATUS" );    
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_STATUS_EXPIRED" );    
    $( "#ID_SUS_STATUS" + this.dynid ).edit( "option", "value", "{$susapp_susstatus_request_qr}" );
    $( "#ID_SUS_STATUS" + this.dynid ).edit( "option", "icon", "" );
    $( "#ID_RESET_PARAM2" + this.dynid ).edit( "option", "status", "disabled" );
  }  

  ModeQr()
  {
    if( this.mode == "modeQr" ) {
      return;
    }
    this.mode = "modeQr";

    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_QRCODE" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_STATUS" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_STATUS_EXPIRED" );    

    // --- print ---
    this.$container.find( ".sus-screen" ).addClass( "no-print" );
    this.$container.find( ".sus-print" ).removeClass( "hidden" );

    $( "#ID_PRINT_QRLINK" + this.dynid ).qrcode( "option", "text", FormSusapp.storefinderUrl );
    
  }  

  //===========================================================================
  //  CONTROL HANDLER
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":           this.CommandUndo();           break;
      case "ID_SAVE":           this.CommandSave();           break;
      case "ID_PERMISSIONS":    this.CommandPermissions();    break;
      case "ID_LISTS":          this.CommandLists();          break;
      case "ID_DELETE":         this.CommandDelete();         break;
    }
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;

    this.FormTitle( this.entity['name'] );
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
    $( "#ID_PRINT_NAME" + this.dynid ).text( result );

  }

  //---------------------------------------------------------------------------
  //  callnumber has been changed
  //  
  ControlEntityCallnumber( result )
  {
    if( ( 'phone' in this.entity ) && ( 'callnumber' in this.entity['phone'] ) ) {
      this.entity['phone']['callnumber'] = value;
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );      
    }

  }

  //---------------------------------------------------------------------------
  //  device config has been changed.
  //
  ControlEntityResult( entity )
  {
    this.entity = entity;

    // exception: master version of entity has been changed, so update all
    // widgets that does not have a save handler
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }

  //---------------------------------------------------------------------------
  //  sus parameters should be deleted
  //  
  ControlSusReset( )
  {
    MacAddressDialog.Show( { 
      param: {
        entid: this.entity['entid'],
        mode: "sus",
        macaddr: this.entity['adminUi']['macaddr'] || "",
        reset: false,
      },
      result: this.ControlDialogResult.bind( this ) 
    } );
  }

  //---------------------------------------------------------------------------
  //  expired qr code should be refreshed
  //  
  ControlSusRefreshExpiredQrcode( )
  {
    this.DataResetSusQrCode();
  }  

  //---------------------------------------------------------------------------
  //  print qr-code
  //  
  ControlSusPrint()
  {
    window.print();
  }

  //---------------------------------------------------------------------------
  //  sus parameters should be deleted
  //   
  ControlDialogResult( result )
  {
    if( result['reset']['calldetails'] || result['reset']['grabpic'] ) {
      Data.Instance().ResetDeviceParameters( this.entinfo['entid'], false, result['reset']['calldetails'], result['reset']['grabpic'] );
    }

    if( result['reset']['sus'] )  {
      this.DataResetSusQrCode();
    }
  }

  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = this.Validate( );

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  CommandUndo(): undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }

  //-------------------------------------------------------
  //  show device lists dialog
  //
  CommandLists() 
  { 
    DeviceListsDialog.Show( { 
      entid: this.entinfo['entid'], 
      //addressBook: this.entity['addrBook'],
      lists: {
        doorcontact: true
      }
    } );
  }



  //===========================================================================
  //  DATA
  //===========================================================================

  //---------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //---------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

    // set help file
    this.helpFile = "susapp";

    // while developing the unit test uncomment this:
    // this.UnitTest();

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();

    // get QR code if not yet scanned
    if( ! this.entity['sus']['isScanned'] )  {
      this.DataRequestSusQrCode();
    }

    this.StatusLoadFinished();

  }

  //---------------------------------------------------------------------------
  //  get QR code if not yet scanned
  //
  DataRequestSusQrCode( )
  {
    this.ModeRequestQr();
    
    // QR Code Abfrage.
    new XhrRequest( "ADMIN.QRCODE_DATA", { 'entityId': this.entinfo['entid'] }, ( response ) => {

      let command      = response['meta']['command'];
      let responseCode = parseInt( response['meta']['status'] );
  
      // If error, we are done here.
      if( responseCode != 200 ) {
        return;
      }
  
      if( command == "ADMIN.QRCODE_DATA" )  {
        if( response['data'] )  {
          this.ModeQr();
          this.qrData = response['data'];
          this.FormFillQrCode( response['data'] );
        }
        else {
          this.ModeQrExpired();
        }
      }
    } );
  }

  //---------------------------------------------------------------------------
  //  Request to create a new QR code.
  //
  DataResetSusQrCode( )
  {
    this.ModeRequestQr();

    // QR Code Abfrage.
    new XhrRequest( "ADMIN.QRCODE_REQUEST", { 'entityId': this.entinfo['entid'] }, ( response ) => {
      // Now its waiting for an entInfo event after server has set the entity's "isScanned" flag to false.
      // Then requests complete entity, and on the response uses ADMIN.QRCODE_DATA to get the new code.

      // Callstack after entInfo event:
      //Data.Instance().RequestEntity
      //this.DataRequest
      //FormBase.DataEntinfoUpdate
      //this.DataEntinfoUpdate      
    } );
  }


  //---------------------------------------------------------------------------
  //  update description lines if some names has been changed
  //
  DataEntinfoUpdate( entinfo, changedList )
  {
    // other entity has been changed
    if( entinfo['entid'] != this.entinfo['entid'] )  {
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
    }
    else {
      super.DataEntinfoUpdate( entinfo, changedList );
    }
  }

  //---------------------------------------------------------------------------
  //  module list has been changed
  //
  DataModulesUpdated()
  {
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }    


  //-----------------------------------------------------------------
  //  sus status event received by mailbox
  //
  DataSusStatusEvent( status )  
  {
    if( status == "ERROR" ) {
      this.ToastShowSusError( "#ID_TOAST" + this.dynid );
    }
    else {
      this.ToastHide();
    }
  }


  //---------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest( );

    // create a copy for save: name & label-handling + reference reset.
    let entity = this.DataPrepareEntityForSave( this.entity );

    // add all references
    this.DataAddReferences( entity, $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "references" ) );

    // finally filter all internal references
    this.DataFilterReferencesForSave( entity );

    // save the entity
    Data.Instance().SavePartialEntity( entity );

    // base-function: reload entity after save
    this.DataReloadAll();

  }

  //===========================================================================
  //  VALIDATION
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_CALLNUMBER" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "validate" );

    return numErr;
  }

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  permission check for own statepoints
  //
  PermissionCheck( entinfo )
  {
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "event", "permission-check", entinfo );
  }


  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  OnUnitTest( testController )
  {
    testController.Assert( "uuid > 32", this.entity['uuid'].length > 32 );
    testController.AssertObject( "Entity class", this.entity, "class", "apartment" );

    // Hint: UnitTest() will call all widgets OnUnitTest() automatically
  }

  //===========================================================================
  //  TOAST
  //===========================================================================

  ToastShowSusError( selector )
  {
    this.$toast = $( selector ).toast( {
      text: "{$const_toast_suserror}",
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
      transition: "h"
    } );
  }  

  ToastHide()
  {
    if( this.$toast ) {
      this.$toast.toast( "destroy" );
    }
  }  

}


//=============================================================================
//  REGISTER
//=============================================================================

FormSusapp.Register( FormSusapp.template );
FormFactory.Register( FormSusapp );
//=============================================================================
//
//  ASC Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          09.09.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormClient extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/client";        // vw template
  static formId     = "client";                       // entity class
  static icon       = "adi adi-siedle-asc";    // icon for toolbar & doc
  static title      = "{$client_document_title}";     // default title if not yet loaded
  static creatable  = true;                           // self explaining
  static container  = false;                          // only folder can create subitems
  static createTemplate = [                           // menu entries in "add" command
    { text: "{$client_create_type}",  deviceTemplate: null, folder: null }
  ];
  helpFile   = "";

  //=============================================================================
  //  STATIC FUNCTIONS
  //=============================================================================

  static MinimumNeededLicenses()
  {
    return { "APR": 1, "ASC": 1 };
  }  

  //===========================================================================
  //  MOVE 
  //===========================================================================

  //-----------------------------------------------------------------
  //  - Get the entity to be moved from server.
  //  - Change the entity's parent and resort it according to ASC/ASHT
  //    structure rules.
  //
  static MoveEntity( entityId, newParentEntityId )
  {
    // Prerequisites.
    let oldParentEntityId = Data.Instance().Entinfo( entityId )["parent"];    
    let oldParentClass = Data.Instance().Entinfo( oldParentEntityId )["class"];
    let newParentClass = Data.Instance().Entinfo( newParentEntityId )["class"];

    // If client is autonomous and becomes non-autonomous.
    if( (oldParentClass == "folder" && newParentClass != "folder" ) 

    // OR If client is non-autonomous and becomes autonomous.    
    || ( oldParentClass != "folder" && newParentClass == "folder" ) )  {

      // Note: It's an intention not to to use Data.RequestEntity here because we don't
      // want the result to be saved (it will be modfied right away) or any events published.

      // Get the entity, then do the move and resort process.
      let xhrRequest = new XhrRequest( "ADMIN.GET_ENTITY", { 'entityId': entityId }, ( response ) => {

        let command      = response['meta']['command'];
        let responseCode = parseInt( response['meta']['status'] );
    
        // If error, we are done here.
        if( responseCode != 200 ) {
          return;
        }
    
        //-- GET-------------------------------------------------
        if( command == "ADMIN.GET_ENTITY" )  {

          // Do the move and resort process.
          FormClient.MoveEntityCallback( response['data'], newParentEntityId )
    
        } // GET_ENTITY

      } );
    }

    // Just move from folder to another.
    else if( oldParentClass == "folder" && newParentClass == "folder" )  {
      Data.Instance().MoveEntity( entityId, newParentEntityId );
    }
  }

  //-----------------------------------------------------------------
  //  To be called after full client entity has arrived after XHR request.
  //  Change parent and resort entity.
  //  
  static MoveEntityCallback( movedClientEntity, newParentEntityId )
  {
    let parentClass = Data.Instance().Entinfo( newParentEntityId )["class"];

    // Change parent.
    movedClientEntity["parent"] = newParentEntityId;

    // If client is autonomous and becomes non-autonomous.
    if( parentClass != "folder" )  {

      // Create a backup object and store some propertysets inside it for later use.
      movedClientEntity["backup"] = {};
      movedClientEntity["backup"]["phone"]  = movedClientEntity["phone"];
      movedClientEntity["backup"]["cdrule"] = movedClientEntity["cdrule"];
      movedClientEntity["backup"]["sp"]     = movedClientEntity["sp"];
      movedClientEntity["backup"]["sip"]    = movedClientEntity["authorize"]["sip"];

      // Then delete those propertysets from their original positions.
      delete movedClientEntity["phone"];
      delete movedClientEntity["cdrule"];
      delete movedClientEntity["sp"];
      delete movedClientEntity["authorize"]["sip"];
    }

    // If client is non-autonomous and becomes autonomous.
    else  {

      // Copy some propertysets back from backup object.
      if( typeof movedClientEntity["backup"] == "object" )  {
        movedClientEntity["phone"]  = movedClientEntity["backup"]["phone"];
        movedClientEntity["cdrule"] = movedClientEntity["backup"]["cdrule"];
        movedClientEntity["sp"]     = movedClientEntity["backup"]["sp"];
        movedClientEntity["authorize"]["sip"]    = movedClientEntity["backup"]["sip"];
      }

      // Delete backup object.
      delete movedClientEntity["backup"];
    }

    // Finally save entity.
    Data.Instance().SaveEntity( movedClientEntity );

  }

  
  //-----------------------------------------------------------------
  //  might be overwritten for special entities
  //
  IsDeleteAllowed( entinfo )
  {
    return true;
  }


  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};
    this.mode = null; // "modeAutonomous" | "modeNonAutonomous"

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );    

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: [
        { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
        { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
        { id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', tooltip: "{$const_form_tooltip_permission}", status: 'normal' },
        { id: 'ID_LISTS', icon: 'mdi mdi-format-list-bulleted', tooltip: "{$const_form_tooltip_lists}", status: "normal" },
        { id: 'ID_DELETE', icon: 'mdi mdi-delete', tooltip: "{$const_form_tooltip_delete}", status: "red" },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );

    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this )
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: "normal"
    } );

    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly"
    } );


    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    Data.Instance().On( "MODULES/UPDATED/", this.etag, this.DataModulesUpdated.bind( this ) );
    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );
    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  read entity into all controls
  //
  FormFillControls()
  {
    // standard controls
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );
    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( "option", "value", "{$client_device_type}" );

    // Find if autonmous or not.
    let parentEntity = Data.Instance().Entinfo( this.entity["parent"] );
    let parentClass = parentEntity["class"];
    if( parentClass != "folder")  {
      this.ModeNonAutonomous();
    }
    else  {
      this.ModeAutonomous();
    }

    // call number
    ControlCallNumber.Build( $( "#ID_CALLNUMBER" + this.dynid ), {
      entity: this.entity,
      result: this.ControlEntityCallnumber.bind( this )
    } );

    // address book
    ControlAddressBook.Build( $( "#ID_ADDRBOOK" + this.dynid ), {
      entity: this.entity
    } );

    // call profile
    ControlCallProfile.Build( $( "#ID_CALLPROFILE" + this.dynid ), {
      entity: this.entity
    } );

    // Account
    ControlAccount.Build( $( "#ID_USERNAME" + this.dynid ), {
      entity: this.entity
    } );

    // Password
    ControlPassword.Build( $( "#ID_PASSWORD" + this.dynid ), {
      entity: this.entity,
    } );

    // Call waiting
    ControlCallWaiting.Build( $( "#ID_CALLWAITING" + this.dynid ), {
      entity: this.entity
    } );

    // group controls

    // function deviceConfig
    ControlDeviceFunctionGroup.Build( $( "#ID_DEVGROUP_FUNCTION" + this.dynid ), {
      entity: this.entity,
      module: [ "function" ],
      result: this.ControlEntityResult.bind( this )
    } );

  }

  //---------------------------------------------------------------------------
  //  read control values
  //
  FormReadControls()
  {
    // Tell widget controls to save directly into entity.
    // All other controls write directly into entity.

    // Group "general".

    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );

    if( this.mode == "modeAutonomous" )  {
      $( "#ID_CALLNUMBER" + this.dynid ).viewable( "option", "save", this.entity );    
      $( "#ID_ADDRBOOK" + this.dynid ).viewable( "option", "save", this.entity );
      $( "#ID_CALLPROFILE" + this.dynid ).viewable( "option", "save", this.entity );
    }

    $( "#ID_USERNAME" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_PASSWORD" + this.dynid ).viewable( "option", "save", this.entity );

    if( this.mode == "modeAutonomous" )  {
      $( "#ID_CALLWAITING" + this.dynid ).viewable( "option", "save", this.entity );
    }

    // Group "functions".

    if( this.mode == "modeAutonomous" )  {
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "save", this.entity );
    }

  }


  //===========================================================================
  //  CONTROL HANDLER
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":           this.CommandUndo();           break;
      case "ID_SAVE":           this.CommandSave();           break;
      case "ID_PERMISSIONS":    this.CommandPermissions();    break;
      case "ID_LISTS":          this.CommandLists();          break;
      case "ID_DELETE":         this.CommandDelete();         break;
    }
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;

    this.FormTitle( this.entity['name'] );

    if( this.mode == "modeAutonomous" )  {
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
    }
  }

  //---------------------------------------------------------------------------
  //  callnumber has been changed
  //  
  ControlEntityCallnumber( result )
  {
    if( ( 'phone' in this.entity ) && ( 'callnumber' in this.entity['phone'] ) ) {
      this.entity['phone']['callnumber'] = value;
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );      
    }

  }

  //---------------------------------------------------------------------------
  //  device config has been changed.
  //
  ControlEntityResult( entity )
  {
    this.entity = entity;

    // exception: master version of entity has been changed, so update all
    // widgets that does not have a save handler
    if( this.mode == "modeAutonomous" )  {
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
    }
  }


  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = this.Validate( );

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  CommandUndo(): undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }

  //-------------------------------------------------------
  //  show device lists dialog
  //
  CommandLists() 
  { 
    DeviceListsDialog.Show( { entid: this.entinfo['entid'], addressBook: this.entity['addrBook'] } );
  }



  //===========================================================================
  //  DATA
  //===========================================================================

  //---------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //---------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

    // set help file
    this.helpFile = "client";    // "asc"?

    // while developing the unit test uncomment this:
    // this.UnitTest();

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();

  }

  //---------------------------------------------------------------------------
  //  update description lines if some names has been changed
  //
  DataEntinfoUpdate( entinfo, changedList )
  {
    // other entity has been changed
    if( entinfo['entid'] != this.entinfo['entid'] )  {
      if( this.mode == "modeAutonomous" )  {
        $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
      }
    }
    else {
      super.DataEntinfoUpdate( entinfo, changedList );
    }
  }

  //---------------------------------------------------------------------------
  //  module list has been changed
  //
  DataModulesUpdated()
  {
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }

  //---------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest( );

    // create a copy for save: name & label-handling + reference reset.
    let entity = this.DataPrepareEntityForSave( this.entity );

    // add all references
    if( this.mode == "modeAutonomous" )  {
      this.DataAddReferences( entity, $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "references" ) );
      this.DataAddReferences( entity, $( "#ID_CALLPROFILE" + this.dynid ).viewable( "option", "references" ) );
      this.DataAddReferences( entity, $( "#ID_ADDRBOOK" + this.dynid ).viewable( "option", "references" ) );
    }

    // finally filter all internal references
    this.DataFilterReferencesForSave( entity );

    // save the entity
    Data.Instance().SavePartialEntity( entity );

    // base-function: reload entity after save
    this.DataReloadAll();

  }


 //===========================================================================
  //  MODES
  //===========================================================================

  //---------------------------------------------------------------------------
  //  "Forward incoming calls" is unchecked.
  //
  ModeAutonomous()
  {
    if( this.mode == "modeAutonomous" ) return;
    this.mode = "modeAutonomous";

    // call number
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_CALLNUMBER" );

    // address book
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_ADDRBOOK" );

    // call profile
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_CALLPROFILE" );

    // Call waiting
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_CALLWAITING" );

    // function deviceConfig
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_DEVGROUP_FUNCTION" );
  }

  //---------------------------------------------------------------------------
  //  "Forward incoming calls" is checked.
  //
  ModeNonAutonomous()
  {
    if( this.mode == "modeNonAutonomous" ) return;
    this.mode = "modeNonAutonomous";

    // Controls must be built even if hidden, because otherwise "validate false" crashes.    

    // call number
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_CALLNUMBER" );
    //$( "#ID_CALLNUMBER" + this.dynid ).viewable( "validate", false );

    // address book
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_ADDRBOOK" );
    //$( "#ID_ADDRBOOK" + this.dynid ).viewable( "validate", false );

    // call profile
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_CALLPROFILE" );
    //$( "#ID_CALLPROFILE" + this.dynid ).viewable( "validate", false );


    // Call waiting
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_CALLWAITING" );
    //$( "#ID_CALLWAITING" + this.dynid ).viewable( "validate", false );

    // function deviceConfig
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_DEVGROUP_FUNCTION" );
    //$( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "validate", false );
  }



  //===========================================================================
  //  VALIDATION
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );
    if( this.mode == "modeAutonomous" )  {
      numErr += $( "#ID_CALLNUMBER" + this.dynid ).viewable( "validate" );
      numErr += $( "#ID_ADDRBOOK" + this.dynid ).viewable( "validate" );
      numErr += $( "#ID_CALLPROFILE" + this.dynid ).viewable( "validate" );
    }
    numErr += $( "#ID_USERNAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_PASSWORD" + this.dynid ).viewable( "validate" );
    if( this.mode == "modeAutonomous" )  {
      numErr += $( "#ID_CALLWAITING" + this.dynid ).viewable( "validate" );
      numErr += $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "validate" );
    }

    return numErr;
  }

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  permission check for own statepoints
  //
  PermissionCheck( entinfo )
  {
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "event", "permission-check", entinfo );
  }

  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  OnUnitTest( testController )
  {
    testController.Assert( "uuid > 32", this.entity['uuid'].length > 32 );
    testController.AssertObject( "Entity class", this.entity, "class", "apartment" );

    // Hint: UnitTest() will call all widgets OnUnitTest() automatically
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormClient.Register( FormClient.template );
FormFactory.Register( FormClient );
//=============================================================================
//
//  ASHT Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          23.09.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormAsht extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/asht";          // vw template
  static formId     = "asht";                         // entity class
  static icon       = "mdi mdi-trello";             // icon for toolbar & doc
  static title      = "{$asht_document_title}";       // default title if not yet loaded
  static creatable  = true;                           // self explaining
  static container  = false;                          // only folder can create subitems
  static createTemplate = [                           // menu entries in "add" command
    { text: "{$asht_create_type}",  deviceTemplate: null, folder: null }
  ];
  helpFile   = "";

  //=============================================================================
  //  STATIC FUNCTIONS
  //=============================================================================

  static MinimumNeededLicenses()
  {
    return { "APR": 1, "ASHT" : 1 };
  }  

  //===========================================================================
  //  MOVE 
  //===========================================================================

  //-----------------------------------------------------------------
  //  - Get the entity to be moved from server.
  //  - Change the entity's parent and resort it according to ASC/ASHT
  //    structure rules.
  //
  static MoveEntity( entityId, newParentEntityId )
  {
    // Prerequisites.
    let oldParentEntityId = Data.Instance().Entinfo( entityId )["parent"];    
    let oldParentClass = Data.Instance().Entinfo( oldParentEntityId )["class"];
    let newParentClass = Data.Instance().Entinfo( newParentEntityId )["class"];

    // If client is autonomous and becomes non-autonomous.
    if( (oldParentClass == "folder" && newParentClass != "folder" ) 

    // OR If client is non-autonomous and becomes autonomous.    
    || ( oldParentClass != "folder" && newParentClass == "folder" ) )  {

      // Note: It's an intention not to to use Data.RequestEntity here because we don't
      // want the result to be saved (it will be modfied right away) or any events published.

      // Get the entity, then do the move and resort process.
      let xhrRequest = new XhrRequest( "ADMIN.GET_ENTITY", { 'entityId': entityId }, ( response ) => {

        let command      = response['meta']['command'];
        let responseCode = parseInt( response['meta']['status'] );

        // If error, we are done here.
        if( responseCode != 200 ) {
          return;
        }
    
        //-- GET-------------------------------------------------
        if( command == "ADMIN.GET_ENTITY" )  {

          // Do the move and resort process.
          FormClient.MoveEntityCallback( response['data'], newParentEntityId )
    
        } // GET_ENTITY

      } );
    }

    // Just move from folder to another.
    else if( oldParentClass == "folder" && newParentClass == "folder" )  {
      Data.Instance().MoveEntity( entityId, newParentEntityId );
    }
  }

  //-----------------------------------------------------------------
  //  To be called after full client entity has arrived after XHR request.
  //  Change parent and resort entity.
  //  
  static MoveEntityCallback( movedClientEntity, newParentEntityId )
  {
    let parentClass = Data.Instance().Entinfo( newParentEntityId )["class"];

    // Change parent.
    movedClientEntity["parent"] = newParentEntityId;

    // If client is autonomous and becomes non-autonomous.
    if( parentClass != "folder" )  {

      // Create a backup object and store some propertysets inside it for later use.
      movedClientEntity["backup"] = {};
      movedClientEntity["backup"]["phone"]  = movedClientEntity["phone"];
      movedClientEntity["backup"]["cdrule"] = movedClientEntity["cdrule"];
      movedClientEntity["backup"]["sp"]     = movedClientEntity["sp"];
      movedClientEntity["backup"]["sip"]    = movedClientEntity["authorize"]["sip"];

      // Then delete those propertysets from their original positions.
      delete movedClientEntity["phone"];
      delete movedClientEntity["cdrule"];
      delete movedClientEntity["sp"];
      delete movedClientEntity["authorize"]["sip"];
    }

    // If client is non-autonomous and becomes autonomous.
    else  {

      // Copy some propertysets back from backup object.
      if( typeof movedClientEntity["backup"] == "object" )  {
        movedClientEntity["phone"]  = movedClientEntity["backup"]["phone"];
        movedClientEntity["cdrule"] = movedClientEntity["backup"]["cdrule"];
        movedClientEntity["sp"]     = movedClientEntity["backup"]["sp"];
        movedClientEntity["authorize"]["sip"]    = movedClientEntity["backup"]["sip"];
      }

      // Delete backup object.
      delete movedClientEntity["backup"];
    }

    // Finally save entity.
    Data.Instance().SaveEntity( movedClientEntity );

  }

  
  //-----------------------------------------------------------------
  //  might be overwritten for special entities
  //
  IsDeleteAllowed( entinfo )
  {
    return true;
  }


  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};
    this.mode = null; // "modeAutonomous" | "modeNonAutonomous"

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );    

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: [
        { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
        { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
        { id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', tooltip: "{$const_form_tooltip_permission}", status: 'normal' },
        { id: 'ID_LISTS', icon: 'mdi mdi-format-list-bulleted', tooltip: "{$const_form_tooltip_lists}", status: "normal" },
        { id: 'ID_DELETE', icon: 'mdi mdi-delete', tooltip: "{$const_form_tooltip_delete}", status: "red" },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );

    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this )
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: "normal"
    } );

    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly"
    } );


    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    Data.Instance().On( "MODULES/UPDATED/", this.etag, this.DataModulesUpdated.bind( this ) );
    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );
    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  read entity into all controls
  //
  FormFillControls()
  {
    // standard controls
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );
    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( "option", "value", "{$asht_device_type}" );

    // Find if autonmous or not.
    let parentEntity = Data.Instance().Entinfo( this.entity["parent"] );
    let parentClass = parentEntity["class"];
    if( parentClass != "folder")  {
      this.ModeNonAutonomous();
    }
    else  {
      this.ModeAutonomous();
    }

    // call number
    ControlCallNumber.Build( $( "#ID_CALLNUMBER" + this.dynid ), {
      entity: this.entity,
      result: this.ControlEntityCallnumber.bind( this )
    } );

    // address book
    ControlAddressBook.Build( $( "#ID_ADDRBOOK" + this.dynid ), {
      entity: this.entity
    } );

    // call profile
    ControlCallProfile.Build( $( "#ID_CALLPROFILE" + this.dynid ), {
      entity: this.entity
    } );

    // Account
    ControlAccount.Build( $( "#ID_USERNAME" + this.dynid ), {
      entity: this.entity
    } );

    // Password
    ControlPassword.Build( $( "#ID_PASSWORD" + this.dynid ), {
      entity: this.entity,
    } );

    // Call waiting
    ControlCallWaiting.Build( $( "#ID_CALLWAITING" + this.dynid ), {
      entity: this.entity
    } );

    // group controls

    // function deviceConfig
    ControlDeviceFunctionGroup.Build( $( "#ID_DEVGROUP_FUNCTION" + this.dynid ), {
      entity: this.entity,
      module: [ "function" ],
      result: this.ControlEntityResult.bind( this )
    } );

  }

  //---------------------------------------------------------------------------
  //  read control values
  //
  FormReadControls()
  {
    // Tell widget controls to save directly into entity.
    // All other controls write directly into entity.

    // Group "general".

    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );

    if( this.mode == "modeAutonomous" )  {
      $( "#ID_CALLNUMBER" + this.dynid ).viewable( "option", "save", this.entity );    
      $( "#ID_ADDRBOOK" + this.dynid ).viewable( "option", "save", this.entity );
      $( "#ID_CALLPROFILE" + this.dynid ).viewable( "option", "save", this.entity );
    }

    $( "#ID_USERNAME" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_PASSWORD" + this.dynid ).viewable( "option", "save", this.entity );

    if( this.mode == "modeAutonomous" )  {
      $( "#ID_CALLWAITING" + this.dynid ).viewable( "option", "save", this.entity );
    }

    // Group "functions".

    if( this.mode == "modeAutonomous" )  {
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "save", this.entity );
    }

  }


  //===========================================================================
  //  CONTROL HANDLER
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":           this.CommandUndo();           break;
      case "ID_SAVE":           this.CommandSave();           break;
      case "ID_PERMISSIONS":    this.CommandPermissions();    break;
      case "ID_LISTS":          this.CommandLists();          break;
      case "ID_DELETE":         this.CommandDelete();         break;
    }
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;

    this.FormTitle( this.entity['name'] );

    if( this.mode == "modeAutonomous" )  {
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
    }
  }

  //---------------------------------------------------------------------------
  //  callnumber has been changed
  //  
  ControlEntityCallnumber( result )
  {
    if( ( 'phone' in this.entity ) && ( 'callnumber' in this.entity['phone'] ) ) {
      this.entity['phone']['callnumber'] = value;
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );      
    }

  }

  //---------------------------------------------------------------------------
  //  device config has been changed.
  //
  ControlEntityResult( entity )
  {
    this.entity = entity;

    // exception: master version of entity has been changed, so update all
    // widgets that does not have a save handler
    if( this.mode == "modeAutonomous" )  {
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
    }
  }


  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = this.Validate( );

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  CommandUndo(): undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }

  //-------------------------------------------------------
  //  show device lists dialog
  //
  CommandLists() 
  { 
    DeviceListsDialog.Show( { entid: this.entinfo['entid'], addressBook: this.entity['addrBook'] } );
  }



  //===========================================================================
  //  DATA
  //===========================================================================

  //---------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //---------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

    // set help file
    this.helpFile = "asht";

    // while developing the unit test uncomment this:
    // this.UnitTest();

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();

  }

  //---------------------------------------------------------------------------
  //  update description lines if some names has been changed
  //
  DataEntinfoUpdate( entinfo, changedList )
  {
    // other entity has been changed
    if( entinfo['entid'] != this.entinfo['entid'] )  {
      if( this.mode == "modeAutonomous" )  {
        $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
      }
    }
    else {
      super.DataEntinfoUpdate( entinfo, changedList );
    }
  }

  //---------------------------------------------------------------------------
  //  module list has been changed
  //
  DataModulesUpdated()
  {
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }

  //---------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest( );

    // create a copy for save: name & label-handling + reference reset.
    let entity = this.DataPrepareEntityForSave( this.entity );

    // add all references
    if( this.mode == "modeAutonomous" )  {
      this.DataAddReferences( entity, $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "references" ) );
      this.DataAddReferences( entity, $( "#ID_CALLPROFILE" + this.dynid ).viewable( "option", "references" ) );
      this.DataAddReferences( entity, $( "#ID_ADDRBOOK" + this.dynid ).viewable( "option", "references" ) );
    }

    // finally filter all internal references
    this.DataFilterReferencesForSave( entity );

    // save the entity
    Data.Instance().SavePartialEntity( entity );

    // base-function: reload entity after save
    this.DataReloadAll();

  }


 //===========================================================================
  //  MODES
  //===========================================================================

  //---------------------------------------------------------------------------
  //  "Forward incoming calls" is unchecked.
  //
  ModeAutonomous()
  {
    if( this.mode == "modeAutonomous" ) return;
    this.mode = "modeAutonomous";

    // call number
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_CALLNUMBER" );

    // address book
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_ADDRBOOK" );

    // call profile
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_CALLPROFILE" );

    // Call waiting
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_CALLWAITING" );

    // function deviceConfig
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_DEVGROUP_FUNCTION" );
  }

  //---------------------------------------------------------------------------
  //  "Forward incoming calls" is checked.
  //
  ModeNonAutonomous()
  {
    if( this.mode == "modeNonAutonomous" ) return;
    this.mode = "modeNonAutonomous";

    // Controls must be built even if hidden, because otherwise "validate false" crashes.    

    // call number
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_CALLNUMBER" );
    //$( "#ID_CALLNUMBER" + this.dynid ).viewable( "validate", false );

    // address book
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_ADDRBOOK" );
    //$( "#ID_ADDRBOOK" + this.dynid ).viewable( "validate", false );

    // call profile
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_CALLPROFILE" );
    //$( "#ID_CALLPROFILE" + this.dynid ).viewable( "validate", false );


    // Call waiting
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_CALLWAITING" );
    //$( "#ID_CALLWAITING" + this.dynid ).viewable( "validate", false );

    // function deviceConfig
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_DEVGROUP_FUNCTION" );
    //$( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "validate", false );
  }



  //===========================================================================
  //  VALIDATION
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );
    if( this.mode == "modeAutonomous" )  {
      numErr += $( "#ID_CALLNUMBER" + this.dynid ).viewable( "validate" );
      numErr += $( "#ID_ADDRBOOK" + this.dynid ).viewable( "validate" );
      numErr += $( "#ID_CALLPROFILE" + this.dynid ).viewable( "validate" );
    }
    numErr += $( "#ID_USERNAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_PASSWORD" + this.dynid ).viewable( "validate" );
    if( this.mode == "modeAutonomous" )  {
      numErr += $( "#ID_CALLWAITING" + this.dynid ).viewable( "validate" );
      numErr += $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "validate" );
    }

    return numErr;
  }

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  permission check for own statepoints
  //
  PermissionCheck( entinfo )
  {
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "event", "permission-check", entinfo );
  }


  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  OnUnitTest( testController )
  {
    testController.Assert( "uuid > 32", this.entity['uuid'].length > 32 );
    testController.AssertObject( "Entity class", this.entity, "class", "apartment" );

    // Hint: UnitTest() will call all widgets OnUnitTest() automatically
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormAsht.Register( FormAsht.template );
FormFactory.Register( FormAsht );
//=============================================================================
//
//  "Jung Panel" Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          13.09.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormJungpanel extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/jungpanel";     // vw template
  static formId     = "jungpanel";                    // entity class
  static icon       = "mdi mdi-tablet";    // icon for toolbar & doc
  static title      = "{$jungpanel_document_title}";  // default title if not yet loaded
  static creatable  = true;                           // self explaining
  static container  = false;                          // only folder can create subitems
  static createTemplate = [                           // menu entries in "add" command
  { 
    text: "{$jungpanel_create_type}",  deviceTemplate: "ALFP_270-0", folder: null }
  ];
  helpFile   = "";

  //=============================================================================
  //  STATIC FUNCTIONS
  //=============================================================================

  static MinimumNeededLicenses()
  {
    return { "APR": 1, "ALFP": 1 };
  }

  //-----------------------------------------------------------------
  //  might be overwritten for special entities
  //
  IsDeleteAllowed( entinfo )
  {
    return true;
  }


  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: [
        { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
        { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
        { id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', tooltip: "{$const_form_tooltip_permission}", status: 'normal' },
        { id: 'ID_LISTS', icon: 'mdi mdi-format-list-bulleted', tooltip: "{$const_form_tooltip_lists}", status: "normal" },
        { id: 'ID_DELETE', icon: 'mdi mdi-delete', tooltip: "{$const_form_tooltip_delete}", status: "red" },
        //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
      ],
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );

    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this )
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: "normal",
    } );

    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly",
    } );

    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    Data.Instance().On( "MODULES/UPDATED/", this.etag, this.DataModulesUpdated.bind( this ) );
    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );
    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  read entity into all controls
  //
  FormFillControls()
  {
    // standard controls
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );

    // older admin did not set the string.
    this.entity['deviceTemplate'] = FormJungpanel.createTemplate[ 0 ]['deviceTemplate'];
    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( "option", "value", this.entity['deviceTemplate'] );

    // entity controls

    // address book
    ControlAddressBook.Build( $( "#ID_ADDRBOOK" + this.dynid ), {
      entity: this.entity
    } );

    // call profile
    ControlCallProfile.Build( $( "#ID_CALLPROFILE" + this.dynid ), {
      entity: this.entity
    } );

    // mac
    ControlMacAddress.Build( $( "#ID_MACADDR" + this.dynid ), {
      entity: this.entity
    } );

    // call number
    ControlCallNumber.Build( $( "#ID_CALLNUMBER" + this.dynid ), {
      entity: this.entity,
      result: this.ControlEntityCallnumber.bind( this )
    } );

    // call waiting
    ControlCallWaiting.Build( $( "#ID_CALLWAITING" + this.dynid ), {
      entity: this.entity
    } );

    // group controls

    // function deviceConfig
    ControlDeviceFunctionGroup.Build( $( "#ID_DEVGROUP_FUNCTION" + this.dynid ), {
      entity: this.entity,
      module: [ "function" ],
      result: this.ControlEntityResult.bind( this )
    } );

  }

  //---------------------------------------------------------------------------
  //  read control values
  //
  FormReadControls()
  {
    // get value from standard controls
    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );

    // all "entity-controls" save directly into entity
    $( "#ID_ADDRBOOK" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_CALLPROFILE" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_CALLNUMBER" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_CALLWAITING" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_MACADDR" + this.dynid ).viewable( "option", "save", this.entity );

    // short before save all widgets are called to update itself
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "save", this.entity );

  }


  //===========================================================================
  //  CONTROL HANDLER
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":           this.CommandUndo();           break;
      case "ID_SAVE":           this.CommandSave();           break;
      case "ID_PERMISSIONS":    this.CommandPermissions();    break;
      case "ID_LISTS":          this.CommandLists();          break;
      case "ID_DELETE":         this.CommandDelete();         break;
    }
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;

    this.FormTitle( this.entity['name'] );
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }

  //---------------------------------------------------------------------------
  //  callnumber has been changed
  //  
  ControlEntityCallnumber( result )
  {
    if( ( 'phone' in this.entity ) && ( 'callnumber' in this.entity['phone'] ) ) {
      this.entity['phone']['callnumber'] = value;
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );      
    }

  }

  //---------------------------------------------------------------------------
  //  device config has been changed.
  //
  ControlEntityResult( entity )
  {
    this.entity = entity;

    // exception: master version of entity has been changed, so update all
    // widgets that does not have a save handler
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }


  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = this.Validate( );

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  CommandUndo(): undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }

  //-------------------------------------------------------
  //  show device lists dialog
  //
  CommandLists() 
  { 
    DeviceListsDialog.Show( { entid: this.entinfo['entid'], addressBook: this.entity['addrBook'] } );
  }



  //===========================================================================
  //  DATA
  //===========================================================================

  //---------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //---------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );
    this.entity['deviceTemplate'] = 

    // set help file
    this.helpFile = "jungpanel";      

    // while developing the unit test uncomment this:
    // this.UnitTest();

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();

  }

  //---------------------------------------------------------------------------
  //  update description lines if some names has been changed
  //
  DataEntinfoUpdate( entinfo, changedList )
  { 
    // other entity has been changed
    if( entinfo['entid'] != this.entinfo['entid'] )  {
      $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
    }
    else {
      super.DataEntinfoUpdate( entinfo, changedList );
    }
  }


  //---------------------------------------------------------------------------
  //  module list has been changed
  //
  DataModulesUpdated()
  {
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }  


  //---------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest( );

    // create a copy for save: name & label-handling + reference reset.
    let entity = this.DataPrepareEntityForSave( this.entity );

    // add all references
    this.DataAddReferences( entity, $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_CALLPROFILE" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_ADDRBOOK" + this.dynid ).viewable( "option", "references" ) );

    // finally filter all internal references
    this.DataFilterReferencesForSave( entity );

    // save the entity
    Data.Instance().SavePartialEntity( entity );

    // base-function: reload entity after save
    this.DataReloadAll();

  }

  //===========================================================================
  //  VALIDATION
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_CALLNUMBER" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_MACADDR" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_ADDRBOOK" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_CALLPROFILE" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_CALLWAITING" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "validate" );

    return numErr;
  }

  //-----------------------------------------------------------------------------
  //  validate all controls once and switch on permanent validation 
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  permission check for own statepoints
  //
  PermissionCheck( entinfo )
  {
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "event", "permission-check", entinfo );
  }


  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  OnUnitTest( testController )
  {
    testController.Assert( "uuid > 32", this.entity['uuid'].length > 32 );
    testController.AssertObject( "Entity class", this.entity, "class", "jungpanel" );

    // Hint: UnitTest() will call all widgets OnUnitTest() automatically
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormJungpanel.Register( FormJungpanel.template );
FormFactory.Register( FormJungpanel );//=============================================================================
//
//  Addressbook Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          15.09.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormAddressBook extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/addrbook";      // vw template
  static formId     = "addrBook";                     // entity class
  static icon       = "mdi mdi-book-open-variant";    // icon for toolbar & doc
  static title      = "{$addrbook_document_title}";   // default title if not yet loaded
  static creatable  = true;                           // self explaining
  static container  = false;                          // only folder can create subitems
  static createTemplate = [                           // menu entries in "add" command
    { text: "{$addrbook_create_type}", deviceTemplate: null, folder: null }
  ];
  helpFile = "addrbook";
  
  //-----------------------------------------------------------------
  //  might be overwritten for special entities
  //
  IsDeleteAllowed( entinfo )
  {
    return true;
  }

  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};
    let isReadOnlyForm = false;
    let toolbarData = [
      { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
      { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
      { id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', tooltip: "{$const_form_tooltip_permission}", status: 'normal' },
      { id: 'ID_DELETE', icon: 'mdi mdi-delete', tooltip: "{$const_form_tooltip_delete}", status: "red" },
      //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
    ];

    this.addressBookDataset = new Dataset( { id: "id" } );
 
    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: toolbarData,
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );


    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this ),
      readonly: isReadOnlyForm
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: ( isReadOnlyForm ) ? "readonly" : "normal",
    } );

    $( "#ID_ADDRBOOK_GRID" + this.dynid ).grid( {
      size: CONST.size.XL + CONST.size.LBL,  //89.3, //24 + 65,
      height: 13,
      data: this.addressBookDataset,
      scrollbar: 'overlay', // 'overlay', 'static'
      sort: 'label',
      sortOrder: '+',
      header: {
        label: { text: "{$addrbook_header_name}", size: 25, resizeable: true, sortable: true },
        callnumber: { text: "{$addrbook_header_callnr}", size: 20, resizeable: true, sortable: true },
        comment: { text: "{$addrbook_header_comment}", size: 45, sortable: true }
      },
      result: this.ControlAddrbookGridSelect.bind( this ),
      dblclick: this.ControlEditClick.bind( this ),
      renderCell: ( columnId, record ) =>  {
        if( columnId == "name" ) {
          return {
            text: record['name'],
            icon: record['icon']
          };
        }
        return { text: record[columnId] };
      },
      //validate: this.ValidateMemberList.bind( this )
    } );

    $( "#ID_ADDRBOOK_ADD" + this.dynid ).button( {
      size: CONST.size.M,
      text: "{$addrbook_button_add}",
      status: "normal",
      click: this.ControlAddClick.bind( this )
    } );

    $( "#ID_ADDRBOOK_EDIT" + this.dynid ).button( {
      size: CONST.size.M,
      text: "{$addrbook_button_edit}",
      status: "disabled",
      click: this.ControlEditClick.bind( this )
    } );

    $( "#ID_ADDRBOOK_DELETE" + this.dynid ).button( {
      size: 4,
      icon: "mdi mdi-delete",
      status: "disabled",
      click: this.ControlDeleteClick.bind( this )
    } );

    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    Data.Instance().On( "ADDRESSBOOK/UPDATE/" + this.entinfo['entid'], this.etag, this.DataAddressbookHasUpdated.bind( this ) );
    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );

    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  fill controls from entity
  //
  FormFillControls()
  {
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );
  }

  //---------------------------------------------------------------------------
  //  read controls
  //
  FormReadControls()
  {
    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );
  }

  
  //===========================================================================
  //  CONTROL
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;
    this.FormTitle( this.entity['name'] );
  }


  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":         this.CommandUndo();         break;
      case "ID_SAVE":         this.CommandSave();         break;
      case "ID_ADD":          this.CommandAdd();          break;
      case "ID_PERMISSIONS":  this.CommandPermissions();  break;
      case "ID_DELETE":       this.CommandDelete();       break;
    }
  }

  //---------------------------------------------------------------------------
  //  selection changed
  //
  ControlAddrbookGridSelect( selected )
  {
    if( selected.length == 0 )  {
      $( "#ID_ADDRBOOK_EDIT" + this.dynid ).button( "option", "status", "disabled" );
      $( "#ID_ADDRBOOK_DELETE" + this.dynid ).button( "option", "status", "disabled" );
    }
    else {
      $( "#ID_ADDRBOOK_EDIT" + this.dynid ).button( "option", "status", "normal" );
      $( "#ID_ADDRBOOK_DELETE" + this.dynid ).button( "option", "status", "normal" );
    }
  }

  //---------------------------------------------------------------------------
  //  edit button
  //
  ControlEditClick( )
  {
    let selected = $( "#ID_ADDRBOOK_GRID" + this.dynid ).grid( "option", "selected" );
    if( selected.length > 0 ) {
      let addressEntry = this.addressBookDataset.Get( selected[ 0 ] );
      AddressEntryDialog.Show( { 
        entry: addressEntry,
        result: ( entry ) => {
          this.DataEntryEdit( entry );
        }
      } );
    }
  }

  //---------------------------------------------------------------------------
  //  add button
  //
  ControlAddClick()
  {
    AddressEntryDialog.Show( { 
      result: ( entry ) => {
        this.DataEntryAdd( entry );
      }
    } );
  }

  ControlDeleteClick()
  {
    let selected = $( "#ID_ADDRBOOK_GRID" + this.dynid ).grid( "option", "selected" );
    if( selected.length > 0 ) {
      this.DataEntryDelete( selected[ 0 ] );
    }
  }


  
  //=============================================================================
  //  COMMANDS
  //=============================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }

  //-----------------------------------------------------------------------------
  //  add element into this folder
  //
  CommandAdd()
  {
    PageMain.CreateEntity( this.entinfo['entid'] );
  }

  
  //===========================================================================
  //  DATA
  //===========================================================================


  //---------------------------------------------------------------------------
  //  button: add new entry to address book
  //
  DataEntryAdd( entry )
  {
    this.addressBookDataset.Add( entry );
    Data.Instance().Addressbook().AddRecord( this.entinfo['entid'], entry );
  }

  //---------------------------------------------------------------------------
  //  button: edit entry in address book
  //
  DataEntryEdit( entry )
  {
    this.addressBookDataset.Update( entry );

    // Server's table compartment doesn't accept unknown fields on table record.
    delete entry["_preview"];
    
    Data.Instance().Addressbook().UpdateRecord( this.entinfo['entid'], entry );
  }

  //---------------------------------------------------------------------------
  //  button: delete entry from address book
  //
  DataEntryDelete( id )
  {
    this.addressBookDataset.Delete( id );
    this.ControlAddrbookGridSelect( [] );
    Data.Instance().Addressbook().DeleteRecord( this.entinfo['entid'], id );
  }

  //-----------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.DataRequestAddressbook();
    this.StatusLoading();
  }

  //-----------------------------------------------------------------------------
  //  get address book table
  //
  DataRequestAddressbook()
  {
    Data.Instance().Addressbook().AddressbookRequest( this.entinfo['entid'], ( result ) => {
      this.addressBookDataset.SetData( result );
      this.ControlAddrbookGridSelect( [] );
    } );

  }

  //-----------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();
  }

  //-----------------------------------------------------------------------------
  //  addressbook has been changed -> reload
  //
  DataAddressbookHasUpdated()
  {
    this.DataRequestAddressbook();
  }


  //-----------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest();

    // save modified entity (don't change status manually!)
    let entity = this.DataPrepareEntityForSave( this.entity );
    Data.Instance().SavePartialEntity( entity );

    // base-function: reload entity after save
    this.DataReloadAll();

  }

  //=============================================================================
  //  VALIDATION
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  Validate(): do validation
  //
  Validate()
  {
    return 0;
  }

  //-----------------------------------------------------------------------------
  //  Validate(): do validation
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormAddressBook.Register( FormAddressBook.template );
FormFactory.Register( FormAddressBook );
//=============================================================================
//
//  Folder Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.06.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormAdmin extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/admin";        // vw template
  static formId     = "admin";                       // entity class
  static icon       = "adi adi-siedle-admin";       // icon for toolbar & doc
  static title      = "{$admin_document_title}";     // default title if not yet loaded
  static creatable  = true;                           // self explaining
  static container  = false;                           // only folder can create subitems
  static createTemplate = [                           // menu entries in "add" command
    { text: "{$admin_create_type}", deviceTemplate: null, folder: null }
  ];
  helpFile = "admin";
  
  //-----------------------------------------------------------------
  //  might be overwritten for special entities
  //
  IsDeleteAllowed( entinfo )
  {
    if( this.entinfo['entid'] == "admin.1" ) {
      return false;
    }
    return true;
  }

  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};
    this.isMainAdmin = ( this.entinfo['entid'] == "admin.1" );
    let toolbarData = [
      { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
      { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
      //{ id: 'ID_ADD', icon: 'mdi mdi-plus-circle', text: '{$const_form_label_add}', tooltip: "{$const_form_tooltip_add}", status: 'normal' },
      //{ id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', tooltip: "{$const_form_tooltip_permission}", status: 'normal' },
      { id: 'ID_DELETE', icon: 'mdi mdi-delete', tooltip: "{$const_form_tooltip_delete}", status: this.isMainAdmin ? "disabled" : "red" },
      //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
    ];

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: toolbarData,
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );


    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this )
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
    } );

    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );
    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  fill controls from entity
  //
  FormFillControls()
  {
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );

    // Account
    ControlAccount.Build( $( "#ID_USERNAME" + this.dynid ), {
      entity: this.entity,
      readonly: this.isMainAdmin
    } );

    // Password
    ControlPassword.Build( $( "#ID_PASSWORD" + this.dynid ), {
      entity: this.entity,
    } );

  }

  //---------------------------------------------------------------------------
  //  read controls
  //
  FormReadControls()
  {
    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );
    $( "#ID_USERNAME" + this.dynid ).viewable( "option", "save", this.entity );
    $( "#ID_PASSWORD" + this.dynid ).viewable( "option", "save", this.entity );
  }


  //===========================================================================
  //  CONTROL
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;
    this.FormTitle( this.entity['name'] );
  }


  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":         this.CommandUndo();         break;
      case "ID_SAVE":         this.CommandSave();         break;
      case "ID_DELETE":       this.CommandDelete();       break;
    }
  }

  
  //=============================================================================
  //  COMMANDS
  //=============================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = this.Validate();

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }


  
  //===========================================================================
  //  DATA
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //-----------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

    // if default admin the password ist already set
    if( this.isMainAdmin )  {
      this.entity['authorize']['default'] = false;
    }



    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();
  }


  //-----------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest();

    // save modified entity (don't change status manually!)
    let entity = this.DataPrepareEntityForSave( this.entity );
    Data.Instance().SavePartialEntity( entity );

    // base-function: reload entity after save
    this.DataReloadAll();

  }

  //=============================================================================
  //  VALIDATION
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  Validate(): do validation
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_USERNAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_PASSWORD" + this.dynid ).viewable( "validate" );
    return numErr;
  }

  //-----------------------------------------------------------------------------
  //  Validate(): do validation
  //
  ValidateOff()
  {
    $( "#ID_NAME" + this.dynid ).viewable( "validate", false );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate", false );
    $( "#ID_USERNAME" + this.dynid ).viewable( "validate", false );
    $( "#ID_PASSWORD" + this.dynid ).viewable( "validate", false );
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormAdmin.Register( FormAdmin.template );
FormFactory.Register( FormAdmin );
//=============================================================================
//
//  KNXGW Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.09.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormKnxGw extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/knxgw";         // vw template
  static formId     = "knxgw";                        // entity class
  static icon       = "mdi mdi-lock-pattern";       // icon for toolbar & doc
  static title      = "{$knxgw_document_title}";   // default title if not yet loaded
  static creatable  = true;                           // self explaining
  static container  = false;                          // only folder can create subitems
  static createTemplate = [                           // menu entries in "add" command
    { text: "{$knxgw_create_type}", deviceTemplate: null, folder: null }
  ];
  helpFile = "knxgw";
  
  //=============================================================================
  //  STATIC FUNCTIONS
  //=============================================================================

  static MinimumNeededLicenses()
  {
    return { "KNXAnbindung": 1 };
  }

  //-----------------------------------------------------------------
  //  might be overwritten for special entities
  //
  IsDeleteAllowed( entinfo )
  {
    return true;
  }

  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};
    let isReadOnlyForm = false;
    let toolbarData = [
      { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
      { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
      { id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', tooltip: "{$const_form_tooltip_permission}", status: 'normal' },
      { id: 'ID_DELETE', icon: 'mdi mdi-delete', tooltip: "{$const_form_tooltip_delete}", status: "red" },
      //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' },
    ];

    this.knxAddrDataset = new Dataset( { id: "id" } );
    this.currentSavedIpAddress = this.entinfo['ip'];

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: toolbarData,
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );


    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this ),
      readonly: isReadOnlyForm
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: ( isReadOnlyForm ) ? "readonly" : "normal",
    } );

    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly",
      value: "{$knxgw_document_title}"
    } );      

    $( "#ID_IPADDR" + this.dynid ).edit( {
      size: CONST.size.L,
      status: ( isReadOnlyForm ) ? "readonly" : "normal",
      validate: Validate.IsIpOrHost
    } );

    $( "#ID_IPADDR_FIND" + this.dynid ).button( {
      size: 4,
      icon: "mdi mdi-magnify",
      status: "normal",
      click: this.ControlButtonIpFindClick.bind( this )
    });


    $( "#ID_PORT" + this.dynid ).edit( {
      size: CONST.size.M,
      status: ( isReadOnlyForm ) ? "readonly" : "normal",
      validate: Validate.IsIpPort
    } );

    $( "#ID_KNX_GRID" + this.dynid ).grid( {
      size: CONST.size.XL , //+ CONST.size.LBL,  //89.3, //24 + 65,
      height: 9,
      data: this.knxAddrDataset,
      scrollbar: 'overlay', // 'overlay', 'static'
      sort: 'label',
      sortOrder: '+',
      regkey: 'knxgw',
      header: {
        label: { text: "{$knxgw_header_name}", size: 35, resizeable: true, sortable: true },
        send: { text: "{$knxgw_header_addr_send}", size: 19, resizeable: true, sortable: true },
        receive: { text: "{$knxgw_header_addr_receive}", size: 19, sortable: true },
        error: { text: "", size: 3.5 }
      },
      renderCell: ( columnId, record ) => {
        if( columnId == "error" ) {
          return { icon: ( record['error'] ? "mdi mdi-security yellow-text" : "" ) };
        }
        else {
          return { text: record[ columnId ] };
        }
      },
      result: this.ControlKnxGridSelect.bind( this ),
      dblclick: this.ControlEditClick.bind( this )
    } );

    $( "#ID_KNX_ADD" + this.dynid ).button( {
      size: CONST.size.M,
      text: "{$knxgw_button_add}",
      status: "normal",
      click: this.ControlAddClick.bind( this )
    } );

    $( "#ID_KNX_EDIT" + this.dynid ).button( {
      size: CONST.size.M,
      text: "{$knxgw_button_edit}",
      status: "disabled",
      click: this.ControlEditClick.bind( this )
    } );

    $( "#ID_KNX_DELETE" + this.dynid ).button( {
      size: 4,
      icon: "mdi mdi-delete",
      status: "disabled",
      click: this.ControlDeleteClick.bind( this )
    } );

    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );

    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  fill controls from entity
  //
  FormFillControls()
  {
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );
    $( "#ID_IPADDR" + this.dynid ).edit(  "option", "value", this.entity['knxgw']['ipaddress'] );
    $( "#ID_PORT" + this.dynid ).edit(  "option", "value", this.entity['knxgw']['ipport'] );

    this.FormFillKnxGrid();
  }

  //---------------------------------------------------------------------------
  //  fill grid dataset
  //
  FormFillKnxGrid()
  {
    let list = [];
    let statepointList = this.entity['sp'];
    let permission = this.entity['permission'];


    for( let spName in statepointList )  {
      if( spName.substr( 0, 4) == "knx_" )  {

        var filteredPermissions = permission.filter( ( rec ) => rec['f'] == spName );
        var isPermissionSet = ( filteredPermissions.length >= 2 );
        let sp = statepointList[ spName ];
        
        let newRecord = {
          id: spName,
          label: sp['label'],
          send: sp['knxdef'][ 0 ]['send'],
          receive: ( sp['knxdef'][ 0 ]['receive'] ) ? sp['knxdef'][ 0 ]['receive'] : "",
          error: ( ! isPermissionSet )
        };
        list.push( newRecord );
      }
    }

    this.knxAddrDataset.SetData( list );
  }

  //---------------------------------------------------------------------------
  //  read controls
  //
  FormReadControls()
  {
    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );
    this.entity['knxgw']['ipaddress'] = $( "#ID_IPADDR" + this.dynid ).edit(  "option", "value" );
    this.entity['knxgw']['ipport'] = $( "#ID_PORT" + this.dynid ).edit(  "option", "value");
    // future use: this.entity['knxgw']['mcaddress'] = "224.0.23.12";
    // future use: this.entity['knxgw']['physicaladdress'] = "";

    this.FormReadKnxGrid();
    this.FormBuildPermissions();
  }

  //---------------------------------------------------------------------------
  //  fill grid dataset
  //
  FormReadKnxGrid()
  {
    let statepointList = {};

    // copy all NON-KNX statepoints into new list
    let oldStatepointList = this.entity['sp'];
    for( let spName in oldStatepointList )  {
      if( spName.substr( 0, 4 ) != "knx_" )  {
        statepointList[ spName ] = oldStatepointList[ spName ];
      }
    }

    // create new statepoint list
    let listData = this.knxAddrDataset.GetData();
    for( let record of listData )  {
      let spName = record['id'];
      statepointList[ spName ] = {
        'name': spName,
        'label': record['label'],
        'type': "BOOL",
        'hidden': false,
        'HasDsr': true,
        'gateway': this.entinfo['entid'],
        'address': Data.Instance().RootEntid() + "/" + this.entinfo['entid'] + "/" + spName,
        'default': false,
        'template': "",
        'knxdef': [ {
          'send': record['send'],
          'receive': ( record['receive'].length > 0 ) ? record['receive'] : null,
          'dpt': "1.000"
        } ]
      };
    }

    // switch to new sp-list
    this.entity['sp'] = statepointList;

  }  

  //---------------------------------------------------------------------------
  //  after creating all statepoints, manage permissions
  //
  FormBuildPermissions()
  {
    // reference to permission list
    let permissionList = this.entity['permission'];

    // build the list of knx-statepoints
    let knxListData = this.knxAddrDataset.GetData();
    let spList = [];
    for( let record of knxListData )  {
      spList.push( record['id'] );
    }

    // remove deleted permissions
    for( let index = permissionList.length - 1; index >= 0 ; index-- )  {
      let perm = permissionList[ index ];
      if( ( perm['f'].substr( 0, 4 ) == "knx_" )
       && ( spList.indexOf( perm['f'] ) < 0 )
        ) {
        permissionList.splice( index, 1 );
      }
    }

    // add new permissions
    for( let spName of spList ) {
      if( permissionList.findIndex( ( rec ) => rec['f'] == spName ) < 0 )  {
        permissionList.push( {
          'p': "rw",
          'w': "*",
          'f': spName
        } );
      }
    }

    // sort permission list to have default at top
    permissionList.sort( ( recordA, recordB ) => {
      if( recordA['f'] == "*" ) return -1;
      if( recordA['f'] < recordB['f'] ) return -1;
      if( recordA['f'] == recordB['f'] ) return 0;
      if( recordA['f'] > recordB['f'] ) return 1;
    } );

  }
  
  //===========================================================================
  //  CONTROL
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;
    this.FormTitle( this.entity['name'] );
  }


  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":         this.CommandUndo();         break;
      case "ID_SAVE":         this.CommandSave();         break;
      case "ID_ADD":          this.CommandAdd();          break;
      case "ID_PERMISSIONS":  this.CommandPermissions();  break;
      case "ID_DELETE":       this.CommandDelete();       break;
    }
  }

  //---------------------------------------------------------------------------
  //  selection changed
  //
  ControlKnxGridSelect( selected )
  {
    if( selected.length == 0 )  {
      $( "#ID_KNX_EDIT" + this.dynid ).button( "option", "status", "disabled" );
      $( "#ID_KNX_DELETE" + this.dynid ).button( "option", "status", "disabled" );
    }
    else {
      $( "#ID_KNX_EDIT" + this.dynid ).button( "option", "status", "normal" );
      $( "#ID_KNX_DELETE" + this.dynid ).button( "option", "status", "normal" );
    }
  }

  //---------------------------------------------------------------------------
  //  edit button
  //
  ControlEditClick( )
  {
    let selected = $( "#ID_KNX_GRID" + this.dynid ).grid( "option", "selected" );
    if( selected.length > 0 ) {
      let knxEntry = this.knxAddrDataset.Get( selected[ 0 ] );
      KnxEntryDialog.Show( { 
        entry: knxEntry,
        result: ( entry ) => {
          this.DataEntryEdit( entry );
          this.Option( "modified", 1 );
        }
      } );
    }
  }

  //---------------------------------------------------------------------------
  //  add button
  //
  ControlAddClick()
  {
    KnxEntryDialog.Show( { 
      result: ( entry ) => {
        this.DataEntryAdd( entry );
        this.Option( "modified", 1 );
      }
    } );
  }

  ControlDeleteClick()
  {
    let selected = $( "#ID_KNX_GRID" + this.dynid ).grid( "option", "selected" );
    if( selected.length > 0 ) {
      this.DataEntryDelete( selected[ 0 ] );
      this.Option( "modified", 1 );
    }
  }

  //---------------------------------------------------------------------------
  //  find camera button clicked
  //
  ControlButtonIpFindClick()
  {
    var ipaddress = $( "#ID_IPADDR" + this.dynid ).edit(  "option", "value" );
    var ipport = $( "#ID_PORT" + this.dynid ).edit(  "option", "value" );

    ScandeviceDialog.Show( { 
      deviceType: "KNX",  
      current: ipaddress + ":" + ipport,
      result: ( data ) => {
        if( !data['ip'] )
          return;
        
        ipaddress = data['ip'];
        if( ipaddress.indexOf( ':') < 0 )  {
          ipaddress += ":3671";
        }

        // check if something has been changed
        if( ipaddress != this.currentSavedIpAddress ) {
          this.currentSavedIpAddress = ipaddress;
          var ip = ipaddress.split( ":" )[ 0 ];
          var port = ipaddress.split( ":" )[ 1 ];
          $( "#ID_IPADDR" + this.dynid ).edit(  "option", "value", ip );
          $( "#ID_PORT" + this.dynid ).edit(  "option", "value", port );
          this.Option( "modified", 1 );
        }
      }
    } );

  }
  
  //=============================================================================
  //  COMMANDS
  //=============================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = this.Validate();

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }
  

  //===========================================================================
  //  DATA
  //===========================================================================

  //---------------------------------------------------------------------------
  //  button: add new entry to address book
  //
  DataEntryAdd( entry )
  {
    this.knxAddrDataset.Add( entry );
  }

  //---------------------------------------------------------------------------
  //  button: edit entry in address book
  //
  DataEntryEdit( entry )
  {
    this.knxAddrDataset.Update( entry );
  }

  //---------------------------------------------------------------------------
  //  button: delete entry from address book
  //
  DataEntryDelete( id )
  {
    this.knxAddrDataset.Delete( id );
    this.ControlKnxGridSelect( [] );
  }

  //-----------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //-----------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();
  }

  //-----------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest();

    // save modified entity (don't change status manually!)
    let entity = this.DataPrepareEntityForSave( this.entity );

    // here permissions are not handled by widgets, so don't save
    delete entity['permission'];

    // save it
    Data.Instance().SavePartialEntity( entity );

    // base-function: reload entity after save
    this.DataReloadAll();

  }

  //-----------------------------------------------------------------------------
  //  verifier detected permission change
  //  
  DataRevalidate()
  {
    if( this.Option( "modified" ) <= 0 ) {
      setTimeout( () => {

        // do validation
        if( this.Validate() > 0 )  {
          PageMain.DocumentError( this.entinfo['entid'], true );
        }
        // reload knx-sps (perm-check)
        this.FormFillKnxGrid();

      }, 250 );
    }

  }

  //=============================================================================
  //  VALIDATION
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  Validate(): do validation
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_IPADDR" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_PORT" + this.dynid ).edit( "validate" );
    return numErr;
  }

  //-----------------------------------------------------------------------------
  //  Validate(): do validation
  //
  ValidateOff()
  {
    $( "#ID_NAME" + this.dynid ).viewable( "validate", false );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate", false );
    $( "#ID_IPADDR" + this.dynid ).edit( "validate", false );
    $( "#ID_PORT" + this.dynid ).edit( "validate", false );
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormKnxGw.Register( FormKnxGw.template );
FormFactory.Register( FormKnxGw );
//=============================================================================
//
//  Timeline Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.06.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormTimeline extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/timeline";        // vw template
  static formId     = "timeline";                       // entity class
  static icon       = "mdi mdi-calendar-clock";       // icon for toolbar & doc
  static title      = "{$timeline_document_title}";     // default title if not yet loaded
  static creatable  = true;                           // self explaining
  static container  = false;                           // only folder can create subitems
  static createTemplate = [                           // menu entries in "add" command
    { text: "{$timeline_create_type}", deviceTemplate: null, folder: null }
  ];
  helpFile = "timeline";
  
  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};
    let isReadOnlyForm = false;
    let toolbarData = [
      { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
      { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
      { id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', tooltip: "{$const_form_tooltip_permission}", status: 'normal' },
      { id: 'ID_DELETE', icon: 'mdi mdi-delete', tooltip: "{$const_form_tooltip_delete}", status: "red" },
      //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
    ];

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: toolbarData,
      click: this.ControlToolbarClick.bind( this ),
    } );


    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this ),
      readonly: isReadOnlyForm
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: ( isReadOnlyForm ) ? "readonly" : "normal",
    } );

    $( "#ID_TL_LEGEND" + this.dynid ).timeline( {
      type: "legend",
      style: AccessApplication.Instance().Language()
    } );

    const tlLabel = {
      start: "{$timeline_detail_start}",
      stop: "{$timeline_detail_end}"
    };

    $( "#ID_TL_MONDAY" + this.dynid    ).timeline( { label: tlLabel, result: this.ControlTimelineResult.bind( this ) } );
    $( "#ID_TL_TUESDAY" + this.dynid   ).timeline( { label: tlLabel, result: this.ControlTimelineResult.bind( this ) } );
    $( "#ID_TL_WEDNESDAY" + this.dynid ).timeline( { label: tlLabel, result: this.ControlTimelineResult.bind( this ) } );
    $( "#ID_TL_THURSDAY" + this.dynid  ).timeline( { label: tlLabel, result: this.ControlTimelineResult.bind( this ) } );
    $( "#ID_TL_FRIDAY" + this.dynid    ).timeline( { label: tlLabel, result: this.ControlTimelineResult.bind( this ) } );
    $( "#ID_TL_SATURDAY" + this.dynid  ).timeline( { label: tlLabel, result: this.ControlTimelineResult.bind( this ) } );
    $( "#ID_TL_SUNDAY" + this.dynid    ).timeline( { label: tlLabel, result: this.ControlTimelineResult.bind( this ) } );


    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/" + this.entinfo['entid'], this.etag, this.DataEntinfoUpdate.bind( this ) );

    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );
    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  fill controls from entity
  //
  FormFillControls()
  {
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );

    ControlLabel.Build( $( "#ID_LABEL_STATUS" + this.dynid ), {
      text: "{$timeline_status_label}",
      label: this.entity['sp']['timeline']['label'],
      entityName: this.entity['name'],
      functionName: "{$aui_widget_timeline}",
      permissionCheck: { entinfo: this.entinfo, spname: "timeline" }
    } );

    ControlLabel.Build( $( "#ID_LABEL_AUTOMATIC" + this.dynid ), {
      text: "{$timeline_automatic_label}",
      label: this.entity['sp']['timeline2']['label'],
      entityName: this.entity['name'],
      functionName: "{$aui_widget_automatic}",
      permissionCheck: { entinfo: this.entinfo, spname: "timeline2" }
    } );
    
    $( "#ID_TL_MONDAY" + this.dynid ).timeline( "option", "value", this.entity['timeline']['mon'] );
    $( "#ID_TL_TUESDAY" + this.dynid ).timeline( "option", "value", this.entity['timeline']['tue'] );
    $( "#ID_TL_WEDNESDAY" + this.dynid ).timeline( "option", "value", this.entity['timeline']['wed'] );
    $( "#ID_TL_THURSDAY" + this.dynid ).timeline( "option", "value", this.entity['timeline']['thu'] );
    $( "#ID_TL_FRIDAY" + this.dynid ).timeline( "option", "value", this.entity['timeline']['fri'] );
    $( "#ID_TL_SATURDAY" + this.dynid ).timeline( "option", "value", this.entity['timeline']['sat'] );
    $( "#ID_TL_SUNDAY" + this.dynid ).timeline( "option", "value", this.entity['timeline']['sun'] );

    ControlCommandGroup.Build( $( "#ID_COMMAND_ON" + this.dynid ), {
      text: "{$timeline_action_on}",
      entity: this.entity,
      module: [ "TIMELINE" ],
      commandPath: "timeline.action.on",
      uidPrefix: "on_",
      result: this.ControlEntityResult.bind( this )
    } );    

    ControlCommandGroup.Build( $( "#ID_COMMAND_OFF" + this.dynid ), {
      text: "{$timeline_action_off}",
      entity: this.entity,
      module: [ "TIMELINE" ],
      commandPath: "timeline.action.off",
      uidPrefix: "off",
      result: this.ControlEntityResult.bind( this )
    } );        
  }

  //---------------------------------------------------------------------------
  //  read controls
  //
  FormReadControls()
  {
    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );

    this.entity['sp']['timeline']['label'] = $( "#ID_LABEL_STATUS" + this.dynid ).viewable( "option", "value" );
    this.entity['sp']['timeline2']['label'] = $( "#ID_LABEL_AUTOMATIC" + this.dynid ).viewable( "option", "value" );

    this.entity['timeline']['mon'] = $( "#ID_TL_MONDAY" + this.dynid ).timeline( "option", "value" );
    this.entity['timeline']['tue'] = $( "#ID_TL_TUESDAY" + this.dynid ).timeline( "option", "value" );
    this.entity['timeline']['wed'] = $( "#ID_TL_WEDNESDAY" + this.dynid ).timeline( "option", "value" );
    this.entity['timeline']['thu'] = $( "#ID_TL_THURSDAY" + this.dynid ).timeline( "option", "value" );
    this.entity['timeline']['fri'] = $( "#ID_TL_FRIDAY" + this.dynid ).timeline( "option", "value" );
    this.entity['timeline']['sat'] = $( "#ID_TL_SATURDAY" + this.dynid ).timeline( "option", "value" );
    this.entity['timeline']['sun'] = $( "#ID_TL_SUNDAY" + this.dynid ).timeline( "option", "value" );
  }


  //===========================================================================
  //  CONTROL
  //===========================================================================

  ControlTimelineResult( result )
  {
    this.Option( "modified", 1 );
  }
  

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;
    this.FormTitle( this.entity['name'] );
  }

  //---------------------------------------------------------------------------
  //  device config has been changed.
  //
  ControlEntityResult( entity )
  {
    this.entity = entity;

    $( "#ID_COMMAND_ON" + this.dynid ).viewable( "option", "rebuild", this.entity );
    $( "#ID_COMMAND_OFF" + this.dynid ).viewable( "option", "rebuild", this.entity );

    if( this.Option( "error") >  0 )  {
      $( "#ID_COMMAND_ON" + this.dynid ).viewable( "validate" );
      $( "#ID_COMMAND_OFF" + this.dynid ).viewable( "validate" );  
    }
  }

  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":         this.CommandUndo();         break;
      case "ID_SAVE":         this.CommandSave();         break;
      case "ID_PERMISSIONS":  this.CommandPermissions();  break;
      case "ID_DELETE":       this.CommandDelete();       break;
    }
  }

  
  //=============================================================================
  //  COMMANDS
  //=============================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.StatusSaveRequest();
      this.FormReadControls();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }

  
  
  //===========================================================================
  //  DATA
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //-----------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );
    
    // create 'uid' & 'in' (needed for DeviceKeyDialog)
    for( let command of this.entity['timeline']['action']['on'] )  {
      if( !( 'uid' in command ) ) {
        command['uid'] = "on_" + $.uuid();
        command['in'] = { sysid: 1 };
      }
    }
    for( let command of this.entity['timeline']['action']['off'] )  {
      if( !( 'uid' in command ) ) {
        command['uid'] = "off" + $.uuid();
        command['in'] = { sysid: 1 };
      }
    }    

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();
  }

  //-----------------------------------------------------------------------------
  //  this object has been changed
  //
  DataEntinfoUpdate( entinfo, changedList )
  {
    this.PermissionCheck( entinfo );
  }


  //-----------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest();

    // save modified entity (don't change status manually!)
    let entity = this.DataPrepareEntityForSave( this.entity );
    
    // here permissions are not handled by widgets, so don't save
    delete entity['permission'];

    // add all references
    this.DataAddReferences( entity, $( "#ID_COMMAND_ON" + this.dynid ).viewable( "option", "references" ) );
    this.DataAddReferences( entity, $( "#ID_COMMAND_OFF" + this.dynid ).viewable( "option", "references" ) );

    // finally filter all internal references
    this.DataFilterReferencesForSave( entity );
    
    // request save
    Data.Instance().SavePartialEntity( entity );

    // base-function: reload entity after save
    this.DataReloadAll();

  }

  //=============================================================================
  //  VALIDATION
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  do permission check for non-hidden statepoints
  //
  PermissionCheck( entinfo )
  {
    $( "#ID_LABEL_STATUS" + this.dynid ).viewable( "event", "permission-check", entinfo );
    $( "#ID_LABEL_AUTOMATIC" + this.dynid ).viewable( "event", "permission-check", entinfo );
  }

  //-----------------------------------------------------------------------------
  //  Validate(): do validation
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_COMMAND_ON" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_COMMAND_OFF" + this.dynid ).viewable( "validate" );
    return numErr;
  }

  //-----------------------------------------------------------------------------
  //  Validate(): do validation
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormTimeline.Register( FormTimeline.template );
FormFactory.Register( FormTimeline );
//=============================================================================
//
//  Folder Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.06.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormAgw extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/agw";        // vw template
  static formId     = "agw";                       // entity class
  static icon       = "mdi mdi-swap-horizontal-circle-outline";       // icon for toolbar & doc
  static title      = "{$agw_document_title}";        // default title if not yet loaded
  static creatable  = false;                          // self explaining
  static container  = false;                          // only folder can create subitems
  static createTemplate = [                           // menu entries in "add" command
  ];
  helpFile = "agw";
  
  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};
    let isReadOnlyForm = false;
    let toolbarData = [
      { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
      { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
      //{ id: 'ID_ADD', icon: 'mdi mdi-plus-circle', text: '{$const_form_label_add}', tooltip: "{$const_form_tooltip_add}", status: 'normal' },
      //{ id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', tooltip: "{$const_form_tooltip_permission}", status: 'normal' },
      { id: 'ID_DELETE', icon: 'mdi mdi-delete', tooltip: "{$const_form_tooltip_delete}", status: "red" },
      //{ id: 'ID_HELP', icon: 'mdi mdi-help-circle', text: '{$const_form_label_help}', tooltip: "{$const_form_tooltip_help}", status: 'hilight' }
    ];

    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: toolbarData,
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );


    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this ),
      readonly: isReadOnlyForm
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: ( isReadOnlyForm ) ? "readonly" : "normal",
    } );

    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );
    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );
    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  fill controls from entity
  //
  FormFillControls()
  {
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );
  }

  //---------------------------------------------------------------------------
  //  read controls
  //
  FormReadControls()
  {
    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );
  }


  //===========================================================================
  //  CONTROL
  //===========================================================================

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;
    this.FormTitle( this.entity['name'] );
  }


  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":         this.CommandUndo();         break;
      case "ID_SAVE":         this.CommandSave();         break;
      case "ID_ADD":          this.CommandAdd();          break;
      case "ID_PERMISSIONS":  this.CommandPermissions();  break;
      case "ID_DELETE":       this.CommandDelete();       break;
    }
  }

  
  //=============================================================================
  //  COMMANDS
  //=============================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }

  //-----------------------------------------------------------------------------
  //  add element into this folder
  //
  CommandAdd()
  {
    PageMain.CreateEntity( this.entinfo['entid'] );
  }

  
  //===========================================================================
  //  DATA
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //-----------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity( entity )
  {
    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();
  }


  //-----------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest();

    // save modified entity (don't change status manually!)
    let entity = this.DataPrepareEntityForSave( this.entity );
    Data.Instance().SavePartialEntity( entity );
  }

  //=============================================================================
  //  VALIDATION
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  Validate(): do validation
  //
  Validate()
  {
    return 0;
  }

  //-----------------------------------------------------------------------------
  //  Validate(): do validation
  //
  ValidateOff()
  {
    // shortcut to a broadcast to all child controls
    this.Option( "validate", false );
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormAgw.Register( FormAgw.template );
FormFactory.Register( FormAgw );
//=============================================================================
//
//  KNXGW Form
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.09.2021
//
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormIpCam extends FormBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template   = "/forms/general/ipcam";         // vw template
  static formId     = "ipcam";                        // entity class
  static icon       = "mdi mdi-video-outline";      // icon for toolbar & doc
  static title      = "{$ipcam_document_title}";      // default title if not yet loaded
  static creatable  = true;                           // self explaining
  static container  = false;                          // only folder can create subitems
  static createTemplate = [                           // menu entries in "add" command
    { text: "{$ipcam_create_type}", deviceTemplate: null, folder: null }
  ];

  static maxResX = 1280;
  static maxResY = 720;
  static allowedEncoding = "h264";
  
  
  //=============================================================================
  //  STATIC FUNCTIONS
  //=============================================================================

  static MinimumNeededLicenses()
  {
    return { "ALFC": 1 };
  }

  //===========================================================================
  //  MOVE 
  //===========================================================================

  //-----------------------------------------------------------------
  //  move stub, called by framework
  //
  static MoveEntity( cameraEntityId, newParentEntityId )
  {
    // prequisites.
    let oldParentEntityId = Data.Instance().Entinfo( cameraEntityId )["parent"];
    //let oldParentClass = Data.Instance().Entinfo( oldParentEntityId )["class"];
    //let newParentClass = Data.Instance().Entinfo( newParentEntityId )["class"];

    // do move
    Data.Instance().MoveEntity( cameraEntityId, newParentEntityId );

    // load the three involved entities and call DoMove() after it. 
    var entityList = {
      cameraEntityId: null,
      oldParentEntityId: null,
      newParentEntityId: null
    };
    Data.Instance().GetEntityListDirect( [ cameraEntityId, oldParentEntityId, newParentEntityId ], ( result, isLastLoad ) => {
      entityList[ result['entid'] ] = result;
      if( isLastLoad ) {
        FormIpCam.DoMove( entityList, cameraEntityId, oldParentEntityId, newParentEntityId );
      }
    } );
    
  }

  //-----------------------------------------------------------------
  //  after loading the three entities, do actual move
  //  
  static DoMove( entityList, cameraEntityId, oldParentEntityId, newParentEntityId )
  {
    console.log( `Move ${cameraEntityId} from ${oldParentEntityId} => ${newParentEntityId}` );
    
    // get a direkt link to the three entitites
    var cameraEntity = entityList[ cameraEntityId ];
    var isCameraChanged = false;
    var oldParentEntity = entityList[ oldParentEntityId ];
    var isOldParentChanged = false;
    var newParentEntity = entityList[ newParentEntityId ];
    var isNewParentChanged = false;

    // check all data is loaded
    if( ! cameraEntity || ! oldParentEntity || ! newParentEntity ) {
      console.error( "Move, entity not loaded:", entityList );
      return;
    }

    //---------------------------------------------------------------
    //  cleanup 
    //---------------------------------------------------------------

    // remove virtual entrance if camera was attached to a phone
    if( FormAttachHelper.RemoveVirtualEntrance( oldParentEntity ) ) {
      isOldParentChanged = true;
    }

    // detach the camera from previous entity
    if( FormAttachHelper.DetachCameraFromEntity( oldParentEntity ) ) {
      isOldParentChanged = true;
    }      

    // restore all sections that had been moved to 'backup'
    if( FormAttachHelper.RestoreAttachedCamera( cameraEntity ) ) {
      isCameraChanged = true;
    }      



    //---------------------------------------------------------------
    //  prepare
    //---------------------------------------------------------------

    // some time the move takes longer then the load, so corrent
    // parent in camera entity
    cameraEntity['parent'] = newParentEntityId;

    // porters camera: if camera should be attached to client
    if( newParentEntity['class'] == "client" )  {
      if( FormAttachHelper.AttachCameraToEntity( newParentEntity, cameraEntityId ) ) {
        isNewParentChanged = true;
      }      
      if( FormAttachHelper.PrepareAttachedCamera( cameraEntity ) ) {
        isCameraChanged = true;
      }      
    }

    // virtual entrance: if camera should be attached to a phone
    if( newParentEntity['class'] == "sipaudio" )  {
      if( FormAttachHelper.AttachCameraToEntity( newParentEntity, cameraEntityId ) ) {
        isNewParentChanged = true;
      }      
      if( FormAttachHelper.PrepareVirtualEntrance( newParentEntity ) ) {
        isNewParentChanged = true;
      }
      if( FormAttachHelper.PrepareAttachedCamera( cameraEntity ) ) {
        isCameraChanged = true;
      }      
    }

    // door + ipcam: a camera should be attached to a door station
    if( newParentEntity['class'] == "door" )  {
      if( FormAttachHelper.AttachCameraToEntity( newParentEntity, cameraEntityId ) ) {
        isNewParentChanged = true;
      }      
      if( FormAttachHelper.PrepareAttachedCamera( cameraEntity ) ) {
        isCameraChanged = true;
      }      
    }

    if( isOldParentChanged )  {
      Data.Instance().SaveEntity( oldParentEntity );
    }
    if( isNewParentChanged )  {
      Data.Instance().SaveEntity( newParentEntity );
    }
    if( isCameraChanged )  {
      Data.Instance().SaveEntity( cameraEntity );
    }

  }


  //-----------------------------------------------------------------
  //  might be overwritten for special entities
  //
  IsDeleteAllowed( entinfo )
  {
    return true;
  }

  //===========================================================================
  //  INIT
  //===========================================================================

  Load( createData )
  {
    //-------------------------------------------
    //  data
    //-------------------------------------------

    this.status = "statusInitialized";
    this.entinfo = createData['entinfo'];   // no $.clone(), createData is cloned!
    this.entity = {};
    this.password = null;
    let isReadOnlyForm = false;
    let toolbarData = [
      { id: 'ID_UNDO', icon: 'mdi mdi-undo', text: '{$const_form_label_undo}', tooltip: "{$const_form_tooltip_undo}", status: 'normal' },
      { id: 'ID_SAVE', icon: 'mdi mdi-content-save', text: '{$const_form_label_save}', tooltip: "{$const_form_tooltip_save}", status: 'green' },
      { id: 'ID_PERMISSIONS', icon: 'mdi mdi-security', tooltip: "{$const_form_tooltip_permission}", status: 'normal' },
      { id: 'ID_DELETE', icon: 'mdi mdi-delete', tooltip: "{$const_form_tooltip_delete}", status: "red" }
    ];

    this.timeoutOnvivChanged = null;
 
    //-------------------------------------------
    //  controls
    //-------------------------------------------

    // groupboxes
    $( '.groupbox' ).groupbox();
    $( "#ID_OPTIONAL" + this.dynid ).optional( {} );

    // create controls
    this.FormLoadToolbar( '#ID_TOOLBAR' + this.dynid, {
      size: -1,
      icon: this.entinfo['_preview']['icon'],
      text: this.entinfo['_preview']['text'],
      data: toolbarData,
      click: this.ControlToolbarClick.bind( this ),
      close: this.ControlToolbarClose.bind( this )
    } );


    ControlEntityName.Build( $( "#ID_NAME" + this.dynid ), {
      result: this.ControlNameResult.bind( this ),
      readonly: isReadOnlyForm
    } );

    $( "#ID_DESCRIPTION" + this.dynid ).edit( {
      size: CONST.size.XL,
      status: ( isReadOnlyForm ) ? "readonly" : "normal",
    } );

    $( "#ID_DEVICE_TYPE" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly",
      value: "{$ipcam_document_title}"
    } );    

    $( "#ID_IPADDR" + this.dynid ).edit( {
      size: CONST.size.L,
      status: ( isReadOnlyForm ) ? "readonly" : "normal",
      validate: Validate.IsIpOrHost,
      //errorAnchor: ".error-text",
      change: ( value ) => {
        this.StatusOnvifChanged( "ipaddr", value );
      }      
    } );

    $( "#ID_IPADDR_FIND" + this.dynid ).button( {
      size: 4,
      icon: "mdi mdi-magnify",
      status: "normal",
      click: this.ControlButtonIpFindClick.bind( this )
    });

    $( "#ID_PORT" + this.dynid ).edit( {
      size: CONST.size.M,
      value: "80",
      status: ( isReadOnlyForm ) ? "readonly" : "normal",
      validate: Validate.IsIpPort,
      change: ( value ) => {
        this.StatusOnvifChanged( "ipport", value );
      }
    } );

    $( "#ID_ONVIFHTTPS" + this.dynid ).choice( {
      size: CONST.size.L,
      type: "check",
      text: "{$ipcam_check_onvifhttps}",
      value: false,
      status: ( isReadOnlyForm ) ? "readonly" : "normal"
    } );        

    $( "#ID_USEAUTH" + this.dynid ).choice( {
      size: CONST.size.L,
      type: "check",
      text: "{$ipcam_check_auth}",
      value: true,
      status: ( isReadOnlyForm ) ? "readonly" : "normal"
    } );    

    $( "#ID_USERNAME" + this.dynid ).edit( {
      size: CONST.size.L,
      status: ( isReadOnlyForm ) ? "readonly" : "normal",
      validate: Validate.NotEmptyString,
      change: ( value ) => {
        this.StatusOnvifChanged( "user", value );
      }
    } );    

    this.streamsDataset = new Dataset( { id: 'id' } );
    $( "#ID_STREAMS" + this.dynid ).grid( {
      size: CONST.size.XL,
      height: 5,
      data: this.streamsDataset,
      scrollbar: "overlay",
      selection: "single",
      inplaceEdit: this.ControlGridInplace.bind( this ),
      header: {
        name: { text: "{$ipcam_grid_name}", size: 35, resizeable: false },
        resultion: { text: "{$ipcam_grid_resultion}", size: 15, resizeable: true },
        enabled: { text: "", icon: "mdi mdi-check-bold", size: 5, resizeable: false, align: "left", inplace: "onoff" }
      },
      renderCell: ( columnId, record ) =>  {
        if( columnId == "resultion" ) {
          return {
            text: record['resultion'] + " (" + record['codec'] + ")"
          };
        }
        else if( columnId == "enabled" && record['forbidden'] )  {
          return { text: null, icon: "mdi mdi-close-thick" };
        }
        return { text: record[ columnId ] };
      },
      renderStatus: ( columnId, record ) =>  {
        if( record['forbidden'] )  {
          return "disabled"; 
        }
        return "normal";
      }
    } );    

    $( "#ID_CAMERA_INFO" + this.dynid ).button( {
      size: CONST.size.XS,
      icon: "mdi mdi-refresh",
      status: "normal",
      click: this.ControlCameraInfoClick.bind( this )
    });



    //-------------------------------------------
    //  start 
    //-------------------------------------------

    Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    Data.Instance().On( "ENTINFO/UPDATE/*", this.etag, this.DataEntinfoUpdate.bind( this ) );

    Verify.Instance().On( "VERIFY/REVALIDATE/" + this.entinfo['entid'], this.etag, this.DataRevalidate.bind( this ) );
    Data.Instance().On( "VIDEO_INFO/UPDATE", this.etag, this.DataResponseCameraInfo.bind( this ) );

    this.DataRequest();

  }//load


  //===========================================================================
  //  FORM HELPERS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  fill controls from entity
  //
  FormFillControls()
  {
    $( "#ID_NAME" + this.dynid ).viewable( "option", "value", this.entity['name'] );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value", this.entity['comment'] );
    $( "#ID_IPADDR" + this.dynid ).edit(  "option", "value", this.entity['ipcam']['ipaddress'] );
    $( "#ID_PORT" + this.dynid ).edit(  "option", "value", this.entity['ipcam']['ipport'] );
    $( "#ID_USERNAME" + this.dynid ).edit(  "option", "value", this.entity['ipcam']['user'] );

    // no camera found to work without authorization
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_USE_AUTH" );

    // Password
    ControlPassword.Build( $( "#ID_PASSWORD" + this.dynid ), {
      entity: this.entity,
      result: ( password ) => {
        this.Option( "modified", 1 );
        this.StatusOnvifChanged( "password", password );
      }
    } );

    // show streams of cameraInfo
    this.FormFillCameraInfo( );
    $( "#ID_CAMERAINFO_ERROR" + this.dynid ).text( "" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_CAMERAINFO_ERROR" );

    // attached camera?    
    if( !( 'deviceConfig' in this.entity )  )  {
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_FUNCTIONS" );
      this.FormToolbarButtonStatus( "ID_PERMISSIONS", "hidden" );
    }
    else {
      $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_FUNCTIONS" );
      this.FormToolbarButtonStatus( "ID_PERMISSIONS", "normal", true ); // force (unhide)
    }
    
    // build up functions regardless if seen or not
    ControlDeviceFunctionGroup.Build( $( "#ID_DEVGROUP_FUNCTION" + this.dynid ), {
      entity: this.entity,
      module: [ "function" ],
      result: this.ControlEntityResult.bind( this )
    } );

  }

  //---------------------------------------------------------------------------
  //  fill controls from entity
  //  
  FormFillCameraInfo( )
  {
    $( "#ID_CAMERA_INFO" + this.dynid ).button( "option", "status", null );

    var data = [];
    for( var stream of this.entity['ipcam']['streams'] ) {

      var isForbidden = false;
      if( stream['resx'] > FormIpCam.maxResX 
       || stream['resy'] > FormIpCam.maxResY
       || stream['encoder'] != FormIpCam.allowedEncoding
        ) {
        isForbidden = true;
      }

      data.push( {
        id: stream['id'],
        name: stream['name'],
        resultion: stream['resx'] + " x " + stream['resy'],
        codec: stream['encoder'],
        enabled: stream['enabled'],
        forbidden: isForbidden
      } );
    }    
    this.streamsDataset.SetData( data );
  }

  //---------------------------------------------------------------------------
  //  camerinfo responded an error
  //  
  FormShowCameraInfoError( reponseCode, responseText )
  {
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_CAMERAINFO_ERROR" );
    $( "#ID_CAMERAINFO_ERROR" + this.dynid ).text( responseText );  
    $( "#ID_STREAMS" + this.dynid ).grid( "option", "placeholder", "{$ipcam_streams_placeholder}" );
    $( "#ID_STREAMS" + this.dynid ).grid( "option", "placeholderIcon", null );
    $( "#ID_CAMERA_INFO" + this.dynid ).button( "option", "status", null );

    this.streamsDataset.SetData( [] );    
  }

  //---------------------------------------------------------------------------
  //  while loading the camera info
  //  
  FormShowLoadingCameraInfo( )
  {
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_CAMERAINFO_ERROR" );

    $( "#ID_CAMERA_INFO" + this.dynid ).button( "option", "status", "disabled" );
    $( "#ID_CAMERAINFO_ERROR" + this.dynid ).text( "" );
    $( "#ID_STREAMS" + this.dynid ).grid( "option", "placeholder", null );
    $( "#ID_STREAMS" + this.dynid ).grid( "option", "placeholderIcon", "mdi mdi-loading mdi-spin" );

    // spinner will be shown automatically if there are is no data
    this.streamsDataset.SetData( [] );
  }


  //---------------------------------------------------------------------------
  //  read controls
  //
  FormReadControls()
  {
    this.entity['name'] = $( "#ID_NAME" + this.dynid ).viewable( "option", "value" );
    this.entity['comment'] = $( "#ID_DESCRIPTION" + this.dynid ).edit( "option", "value" );
    this.entity['ipcam']['ipaddress'] = $( "#ID_IPADDR" + this.dynid ).edit(  "option", "value" );
    this.entity['ipcam']['ipport'] = $( "#ID_PORT" + this.dynid ).edit(  "option", "value");
    this.entity['ipcam']['user'] = $( "#ID_USERNAME" + this.dynid ).edit(  "option", "value");

    $( "#ID_PASSWORD" + this.dynid ).viewable( "option", "save", this.entity );

    this.FormReadCameraInfo();
  }


  //---------------------------------------------------------------------------
  //  read controls
  //
  FormReadCameraInfo()
  {
    for( var stream of this.entity['ipcam']['streams'] ) {
      var uiRecord = this.streamsDataset.Get( stream['id'] );
      stream['enabled'] = uiRecord['enabled'];
    }
  }

  //===========================================================================
  //  CONTROL
  //===========================================================================

  //---------------------------------------------------------------------------
  //  device config has been changed.
  //
  ControlEntityResult( entity )
  {
    this.entity = entity;

    // exception: master version of entity has been changed, so update all
    // widgets that does not have a save handler
    $( "#ID_DEVGROUP_FUNCTION" + this.dynid ).viewable( "option", "rebuild", this.entity );
  }

  //---------------------------------------------------------------------------
  //  refresh camera info clicked
  //
  ControlCameraInfoClick( )  
  {
    // this function is called by timeout and by the reload button.
    // So clear timeout if user changed something and then presses
    // the button. 
    clearTimeout( this.timeoutOnvivChanged );

    var ipaddress = $( "#ID_IPADDR" + this.dynid ).edit(  "option", "value" );
    var ipport = $( "#ID_PORT" + this.dynid ).edit(  "option", "value" );    
    var user = $( "#ID_USERNAME" + this.dynid ).edit( "option", "value" );
    var password = $( "#ID_PASSWORD" + this.dynid ).viewable( "option", "value" );

    this.StatusCameraInfoLoading();
    this.DataRequestCameraInfo( ipaddress + ":" + ipport, user, password );

    this.FormShowLoadingCameraInfo();
  }

  //---------------------------------------------------------------------------
  //  login dialog return result
  //
  /*ControlLoginDialogResult( result )
  {
    this.Option( "modified", 1 );

    this.password = result['password'];
    this.entity['ipcam']['caminfo'] = result['data']['media'];
    this.entity['ipcam']['streams'] = [];
    delete this.entity['ipcam']['password'];// = this.password;

    for( var key in result['data']['profiles'] )  {
      var profile = result['data']['profiles'][ key ];
      var record = {
        name:         profile['name'],
        encoder:      ( profile['video']['encoding'] ).toLowerCase(),
        framerate:    parseInt( profile['video']['framerate'] ),
        resx:         parseInt( profile['video']['width'] ),
        resy:         parseInt( profile['video']['height'] ),
        encoderparam: profile['video']['h264profile'],
        rtsp:         profile['video']['rtsp']
      };
      this.entity['ipcam']['streams'].push( record );
    }

  }*/

  //---------------------------------------------------------------------------
  //  find camera button clicked
  //
  ControlButtonIpFindClick()
  {
    var ipaddress = $( "#ID_IPADDR" + this.dynid ).edit(  "option", "value" );
    var ipport = $( "#ID_PORT" + this.dynid ).edit(  "option", "value" );

    ScandeviceDialog.Show( { 
      deviceType: "IPCAM",  
      current: ipaddress + ":" + ipport,
      result: ( data ) => {
        //console.log( data );
        var url = new URL( data['url'] );
        var ipaddress = url.host;
        if( ! url.port ) {
          ipaddress += ":80";
        }

        // check if something has been changed
        if( ipaddress != this.currentSavedIpAddress ) {
          
          this.currentSavedIpAddress = ipaddress;
          var ip = ipaddress.split( ":" )[ 0 ];
          var port = ipaddress.split( ":" )[ 1 ];
          $( "#ID_IPADDR" + this.dynid ).edit(  "option", "value", ip );
          $( "#ID_PORT" + this.dynid ).edit(  "option", "value", port );
          this.Option( "modified", 1 );

          // timeout needed because we're inside of this dialog
          // dialog need to be finished before show a new one.
          setTimeout( ()=> {
            this.StatusOnvifChanged( "ipaddress,ipport" );
          }, 0 );          
        }
      }
    } );

  }

  //---------------------------------------------------------------------------
  //  close this document
  //
  ControlToolbarClose()
  {
    PageMain.CloseEntity( this.entinfo['entid'] );
  }

  //---------------------------------------------------------------------------
  //  name has been changed
  //
  ControlNameResult( result )
  {
    this.entity['name'] = result;
    this.FormTitle( this.entity['name'] );
  }


  //---------------------------------------------------------------------------
  //  toolbar click handler
  //
  ControlToolbarClick( commandId )
  {
    switch( commandId )  {
      case "ID_UNDO":         this.CommandUndo();         break;
      case "ID_SAVE":         this.CommandSave();         break;
      case "ID_ADD":          this.CommandAdd();          break;
      case "ID_PERMISSIONS":  this.CommandPermissions();  break;
      case "ID_DELETE":       this.CommandDelete();       break;
    }
  }

  //---------------------------------------------------------------------------
  //  checkbox in grid changed
  //
  ControlGridInplace( columnId, record, param )
  {
    this.Option( "modified", 1 );
    record['enabled'] = record['enabled'] ? false : true;
    return record;
  }

  //=============================================================================
  //  COMMANDS
  //=============================================================================

  //---------------------------------------------------------------------------
  //  save, validate before
  //
  CommandSave()
  {
    // start validation
    let numErr = this.Validate();

    // we save only if no error is in form.
    if( numErr == 0 ) {
      this.FormReadControls();
      this.StatusSaveRequest();
      this.DataSaveRequest();
    }
  }

  //-------------------------------------------------------
  //  undo, disable validation
  //
  CommandUndo()
  {
    // clean form & load data
    this.ToastDestroy();
    this.DataRequest();
  }
  

  //===========================================================================
  //  DATA
  //===========================================================================

  //-----------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    //console.log( "ipcam: data-request")
    Data.Instance().RequestEntity( this.entinfo['entid'] );
    this.StatusLoading();
  }

  //-----------------------------------------------------------------------------
  //  DataResponseEntity(): do xhr request(s)
  //
  DataResponseEntity( entity )
  {
    //console.log( "ipcam: data-response" );

    // we need to create a copy + include {{name}} in label fields
    this.entity = this.DataPrepareEntityAfterLoad( entity );

    // get ip address to check StatusIpChanged
    this.currentSavedIpAddress = this.entity['ipcam']['ipaddress'] + ":" + this.entity['ipcam']['ipport'];
    this.currentSavedUser = this.entity['ipcam']['user'];

    // fill controls and enter load finished to do 1st validation
    this.FormTitle( this.entity['name'] );
    this.FormFillControls();
    this.StatusLoadFinished();
  }

  //-----------------------------------------------------------------------------
  //  something has changed
  //
  DataEntinfoUpdate( entinfo, changedList )
  {
    if( entinfo['entid'] != this.entinfo['entid'] ) {
      return;
    }

    // ['entid', 'ipcam']
    if( Array.isArray( changedList ) && changedList.length == 2 && changedList[ 1 ] == "ipcam" ) {

      // Get the entity, to update streams
      new XhrRequest( "ADMIN.GET_ENTITY", { 'entityId': entinfo['entid'] }, ( response ) => {

        // If error, we are done here.
        let responseCode = parseInt( response['meta']['status'] );
        if( responseCode != 200 ) {
          return;
        }

        // reload stream info
        var entity = response['data'];
        this.entity['ipcam']['streams'] = entity['ipcam']['streams'];
        this.FormFillCameraInfo();

      } );
    }
    else {
      super.DataEntinfoUpdate( entinfo, changedList );
    }
  } 

  //-----------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // do unit test (viewable member will broadcast the test)
    this.UnitTest();

    // save modified entity (don't change status manually!)
    let entity = this.DataPrepareEntityForSave( this.entity );

    // here permissions are not handled by widgets, so don't save
    delete entity['permission'];
    console.log( "SAVE: ", entity );

    // save it
    Data.Instance().SavePartialEntity( entity );

    // base-function: reload entity after save
    this.DataReloadAll();

    // store password
    if( this.password )  {
      Data.Instance().SetEntityPassword( this.entity['entid'], this.password, "password" );
    }

  }

  //-----------------------------------------------------------------------------
  //  get video info for getting the streams
  //  
  DataRequestCameraInfo( ipaddressPort, user, password )
  {
    this.requestedIpAddress = ipaddressPort;

    var url = "http://" + ipaddressPort + "/onvif/device_service";
    Data.Instance().CameraInfo( url, user, password, this.entity['entid'] );
    this.scanTimeoutId = setTimeout( () => {
      this.FormShowCameraInfoError( -3, "no response" );
    }, 20000 );

  }

  //-----------------------------------------------------------------------------
  //  video info 
  //  
  DataResponseCameraInfo( response )
  {
    var url = new URL( response['url'] );
    var ipaddress = url.host;
    if( !url.port ) {
      ipaddress += ":80";
    }

    if( this.requestedIpAddress == ipaddress )  {
      
      // clear timeouts
      clearTimeout( this.timeoutOnvivChanged );
      clearTimeout( this.scanTimeoutId );

      this.timeoutOnvivChanged = null;
      this.scanTimeoutId = null;

      //console.log( response );

      if( response['error'] == true )  {
        this.FormShowCameraInfoError( response['errorcode'], response['errordescription'] );
      }
      else {
        this.entity['ipcam']['streams'] = [];
        for( var profileKey in response['data']['profiles'] ) {
          var profile = response['data']['profiles'][ profileKey ];
          this.entity['ipcam']['streams'].push( {
            id: profile['video']['id'],
            enabled: profile['video']['enabled'],
            name: profile['video']['name'],
            encoder: ( profile['video']['encoding'] ).toLowerCase(),
            framerate: parseInt( profile['video']['framerate'] ),
            resx: parseInt( profile['video']['width'] ),
            resy: parseInt( profile['video']['height'] ),
            encoderparam: profile['video']['h264profile']
          } );
        }
        this.FormFillCameraInfo();
      }
    }
  }

  //-----------------------------------------------------------------------------
  //  verifier detected permission change
  //  
  DataRevalidate()
  {
    if( this.Option( "modified" ) <= 0 ) {
      setTimeout( () => {
        // do validation
        if( this.Validate() > 0 )  {
          PageMain.DocumentError( this.entinfo['entid'], true );
        }
      }, 250 );
    }

  }

  //=============================================================================
  //  STATUS
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  change of password/user/ip or port 
  //
  StatusOnvifChanged( hint, data )
  {
    this.status = "statusOnvifChanged";
    clearTimeout( this.timeoutOnvivChanged );
    this.timeoutOnvivChanged = setTimeout( () => {
      this.ControlCameraInfoClick();
    }, 3000 );
  }

  StatusCameraInfoLoading( hint, data )
  {
    this.status = "statusCameraInfoLoading";

  }  

  //=============================================================================
  //  VALIDATION
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  Validate(): do validation
  //
  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_NAME" + this.dynid ).viewable( "validate" );
    numErr += $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_IPADDR" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_PORT" + this.dynid ).edit( "validate" );
    return numErr;
  }

  //-----------------------------------------------------------------------------
  //  Validate(): do validation
  //
  ValidateOff()
  {
    $( "#ID_NAME" + this.dynid ).viewable( "validate", false );
    $( "#ID_DESCRIPTION" + this.dynid ).edit( "validate", false );
    $( "#ID_IPADDR" + this.dynid ).edit( "validate", false );
    $( "#ID_PORT" + this.dynid ).edit( "validate", false );
  }

}


//=============================================================================
//  REGISTER
//=============================================================================

FormIpCam.Register( FormIpCam.template );
FormFactory.Register( FormIpCam );
﻿//=============================================================================
//
//  AGW Constants
//  (c) 2020 CYTEL Technology AG
//
//  Editor: Frank Jäkel
//          09.07.2020
//=============================================================================

var CONST = {
  presence: {
    'OFFLINE': 'mdi mdi-circle grey-text',
    
    'DEVREQUEST': 'mdi mdi-circle yellow-text',
    'DEVREBOOT': 'mdi mdi-circle yellow-text',
    'DEVSTART': 'mdi mdi-circle yellow-text',
    'DEVREADY': 'mdi mdi-circle yellow-text',
    'TRYING': 'mdi mdi-circle yellow-text',

    'IDLE': 'mdi mdi-circle green-text',

    'ERROR': 'mdi mdi-circle red-text',

    'CALL': 'mdi mdi-circle red-text',
    'DND': 'mdi mdi-circle red-text',
    'RING': 'mdi mdi-circle red-text',
    'DISCONNECTING': 'mdi mdi-circle red-text',
    'BUSY': 'mdi mdi-circle red-text',
  },
  sidebarGeneral: [
    { text: '{$const_form_dashboard}', icon: 'mdi mdi-gauge', id: "dashboard" },
    { text: '{$const_form_presence}', icon: 'mdi mdi-traffic-light', id: "presence" },
    { text: '{$const_form_basicparam}', icon: 'mdi mdi-cog', id: "basicparam" },
    { text: '{$const_form_backup}', icon: 'mdi mdi-archive', id: "backup" },
    { text: '{$const_form_rules}', icon: 'mdi mdi-puzzle', id: "rules" },    
    { text: '{$const_form_systemtime}', icon: 'mdi mdi-clock-outline', id: "systemtime" },
    { text: '{$const_form_network}', icon: 'mdi mdi-lan', id: "network" },
    { text: '{$const_form_logfile}', icon: 'mdi mdi-clipboard-outline', id: "logfile" },
    { text: '{$const_form_callnumberplan}', icon: 'mdi mdi-phone-lock', id: "callnumberplan" },
    { text: '{$const_form_licenses}', icon: 'mdi mdi-license', id: "licenses" },
  ],
  size: {
    XS:   4,
    S:    7,
    M:   13,
    L:   29,
    XL:  65,
    XXL: 80,
    LBL: 24.3
  },
  dlgSize: {
    M: "28em"
  },
  dlgTop: {
    S: "30%",
    M: "25%",
    L: "20%",
    XL: "15%"
  },
  moduleGroups: {
    "*indoor": [
      "apartment",
      "asht",
      "client",
      "tkphone",
      "sipaudio",
      "jungpanel",
      "avp"
    ],
    "*app": [
      "susapp"
    ]
  },
  syslang: [
    { id: "da", text: "Dansk" },
    { id: "de", text: "Deutsch" },
    { id: "en", text: "English" },
    { id: "es", text: "Español" },
    { id: "fr", text: "Français" },
    { id: "it", text: "Italiano" },
    { id: "nl", text: "Nederlands" },
    { id: "pl", text: "Polski" },
    { id: "ru", text: "Pусский" },
    { id: "sv", text: "Svenska" }
  ],
  switchTime:[
    { id: '0',   text: "{$const_time_manual}" },
    { id: '400', text: "0,4 s." },
    { id: '1000', text: "1 s." },
    { id: '2000', text: "2 s." },
    { id: '3000', text: "3 s." },
    { id: '4000', text: "4 s." },
    { id: '5000', text: "5 s." },
    { id: '6000', text: "6 s." },
    { id: '7000', text: "7 s." },
    { id: '8000', text: "8 s." },
    { id: '9000', text: "9 s." },
    { id: '10000', text: "10 s." },
    { id: '11000', text: "11 s." },
    { id: '12000', text: "12 s." },
    { id: '13000', text: "13 s." },
    { id: '14000', text: "14 s." },
    { id: '15000', text: "15 s." },
    { id: '16000', text: "16 s." },
    { id: '17000', text: "17 s." },
    { id: '18000', text: "18 s." },
    { id: '19000', text: "19 s." },
    { id: '20000', text: "20 s." },
    { id: '21000', text: "21 s." },
    { id: '22000', text: "22 s." },
    { id: '23000', text: "23 s." },
    { id: '24000', text: "24 s." },
    { id: '25000', text: "25 s." },
    { id: '26000', text: "26 s." },
    { id: '27000', text: "27 s." },
    { id: '28000', text: "28 s." },
    { id: '29000', text: "29 s." },
    { id: '30000', text: "30 s." },
    { id: '31000', text: "31 s." },
    { id: '32000', text: "32 s." },
    { id: '33000', text: "33 s." },
    { id: '34000', text: "34 s." },
    { id: '35000', text: "35 s." },
    { id: '36000', text: "36 s." },
    { id: '37000', text: "37 s." },
    { id: '38000', text: "38 s." },
    { id: '39000', text: "39 s." },
    { id: '40000', text: "40 s." },
    { id: '41000', text: "41 s." },
    { id: '42000', text: "42 s." },
    { id: '43000', text: "43 s." },
    { id: '44000', text: "44 s." },
    { id: '45000', text: "45 s." },
    { id: '46000', text: "46 s." },
    { id: '47000', text: "47 s." },
    { id: '48000', text: "48 s." },
    { id: '49000', text: "49 s." },
    { id: '50000', text: "50 s." },
    { id: '51000', text: "51 s." },
    { id: '52000', text: "52 s." },
    { id: '53000', text: "53 s." },
    { id: '54000', text: "54 s." },
    { id: '55000', text: "55 s." },
    { id: '56000', text: "56 s." },
    { id: '57000', text: "57 s." },
    { id: '58000', text: "58 s." },
    { id: '59000', text: "59 s." },
    { id: '60000', text: "1 min." },
    { id: '120000', text: "2 min." },
    { id: '180000', text: "3 min." },
    { id: '240000', text: "4 min." },
    { id: '300000', text: "5 min." },
    { id: '360000', text: "6 min." },
    { id: '420000', text: "7 min." },
    { id: '480000', text: "8 min." },
    { id: '540000', text: "9 min." },
    { id: '600000', text: "10 min." },
    { id: '900000', text: "15 min." },
    { id: '1200000', text: "20 min." },
    { id: '1500000', text: "25 min." },
    { id: '1800000', text: "30 min." },
    { id: '2100000', text: "35 min." },
    { id: '2400000', text: "40 min." },
    { id: '2700000', text: "45 min." },
    { id: '3000000', text: "50 min." },
    { id: '3300000', text: "55 min." },
    { id: '3600000', text: "1 h." },
    { id: '7200000', text: "2 h." }
    ],
  framesizes:[
    { id: '0',  text: "{$const_framesize_auto}" },
    { id: '20', text: "{$const_framesize_ms20}" },
    { id: '40', text: "{$const_framesize_ms40}" }
  ]

};

// Entity class -> icon
var CONST_CLASS_ICO = {
  folder:          'mdi mdi-folder-outline',
  group:           'mdi mdi-account-multiple',
  apartment:       'mdi mdi-bell-ring-outline',
  sipaudio:        'mdi mdi-deskphone',
  door:            'mdi mdi-key',
  addrBook:        'mdi mdi-book-open-variant',
  client:          'adi adi-siedle-asc',
  asht:            'mdi mdi-trello',
  avp:             'adi adi-siedle-avp',  
  admin:           'adi adi-siedle-admin', 
  app:             'mdi mdi-cellphone',
  knxgw:           'mdi mdi-lock-pattern',
  tkgw:            'mdi mdi-phone-voip',
  trunk:           'mdi mdi-phone-settings',
  tkphone:         'mdi mdi-phone-classic',
  timeline:        'mdi mdi-calendar-clock',
  jungpanel:       'mdi mdi-tablet',
  susapp:          'mdi mdi-cellphone',
  agw:             'mdi mdi-swap-horizontal-circle-outline',
  rule:            'mdi mdi-code-braces',
  dashboard:       'mdi mdi-gauge',
  basicparameters: 'mdi mdi-cog',
  networksettings: 'mdi mdi-lan',
  callprofile:     'mdi mdi-phone-lock',
  log:             'mdi mdi-clipboard-outline',
  ipcam:           'mdi mdi-video-outline',
  none:            'mdi mdi-help'
};//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          28.05.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class MessageBox extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATICS
  //---------------------------------------------------------------------------

  static template = "/dialog/messageBox";

  //---------------------------------------------------------------------------
  //  LOAD
  //
  Load( createOptions )
  {
    let icon = "mdi mdi-help blue-text";

    if( 'icon' in createOptions )  {
      icon = createOptions['icon'];
    }

    $( "#ID_ICON" + this.dynid ).attr( "class", icon );
    $( "#ID_TEXT" + this.dynid ).text( $.lang( createOptions['text'] ) );

  }//load

  //---------------------------------------------------------------------------
  //  VIEWABLE EVENTS
  //  
  OnEvent( eventName, data )
  {
    switch( eventName ) {
      // buttons
      case 'button':
      case 'canclose': {
        if( data == 'IDOK' || data === true  )  {
          this.CommandOk( );
        }
        else {
          this.CommandCancel();
        }
      }
      break;
    }

    return true;
  }//onevent

  
  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Button IDOK or <enter>
  // 
  CommandOk()
  {
    this.FireEvent( 'result', true );
    this.Destroy();
  }

  //---------------------------------------------------------------------------
  //  Button IDCANCEL or <esc>
  // 
  CommandCancel()
  {
    this.Destroy();
  }

  //===========================================================================
  //  STATIC 
  //===========================================================================

  static Show( options )
  {
    let buttons = [];
    let okText = null;

    if( 'oktext' in options )  {
      okText = $.lang( options['oktext'] );
    }

    // info box
    if( options['layout'] == "info" )  {
      if( !okText ) okText = '{$messagebox_ok}';
      buttons = [
        { id: 'IDOK', size: 13, status: 'hilight', text: okText }
      ];
      delete options['layout'];
      options['icon'] = "mdi mdi-information-variant blue-text";
    }

    // info box
    if( options['layout'] == "ok" ) {
      if( !okText ) okText = '{$messagebox_ok}';
      buttons = [
        { id: 'IDCANCEL', size: 13, status: 'normal', text: '{$messagebox_cancel}' },
        { id: 'IDOK', size: 13, status: 'hilight', text: okText }
      ];
      delete options['layout'];
      options['icon'] = "mdi mdi-alert blue-text";
    }

    // alert box
    else if( options['layout'] == "alert" ) {
      if( !okText ) okText = '{$messagebox_ok}';
      buttons = [
        { id: 'IDCANCEL', size: 13, status: 'normal', text: '{$messagebox_cancel}' },
        { id: 'IDOK', size: 13, status: 'red', text: okText }
      ];
      delete options.layout;
      options['icon'] = "mdi mdi-alert blue-text";
    }

    // fatal box
    else if( options['layout'] == "fatal" ) {
      if( !okText ) okText = '{$messagebox_cancel}';
      buttons = [
        { id: 'IDOK', size: 13, status: 'red', text: okText }
      ];
      delete options.layout;
      options['icon'] = "mdi mdi-alert blue-text";
    }


    var createdata = $.extend( true, [], {
      view: MessageBox.template,
      type: 'dialog',
      layout: 'classic2',
      size: "25em",
      height: "10em",
      buttons: buttons
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }
}


//=============================================================================
//  BUILDER
//=============================================================================

MessageBox.Register( MessageBox.template );
//=============================================================================
//
//  MODULE INSTALL 
//  (c) 2022 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          14.04.2022
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class ProgressDialog extends Viewable
{
  //---------------------------------------------------------------------------
  //  VARIABLES
  //---------------------------------------------------------------------------

  static template = "/dialog/progressDialog";  
  messageLines = [];
  isInitialized = false;

  //---------------------------------------------------------------------------
  //  LOAD
  //
  Load( createData )
  {
    this.isInitialized = true;
    this.FormFill();
  }//load

  //---------------------------------------------------------------------------
  //  viewable event function
  //
  OnEvent( eventName, data, param )
  {
    //console.log( eventName, data, param );
    if( eventName == "option" && data == "message" ) {
      this.DataChangeMessageLine( param );
      if( this.isInitialized ) {
        this.FormFill();
      }
    }
    return false;
  }


  //---------------------------------------------------------------------------
  //  FORM
  //---------------------------------------------------------------------------

  FormFill( line )
  {
    $( "#ID_CONTENT" + this.dynid ).empty();
    
    for( var line of this.messageLines ) {
      if( line['finished'] < 0 ) {
        line['icon'] = "mdi mdi-close-circle red-text";
      }
      else {
        line['icon'] = line['finished'] ? "mdi mdi-check green-text" : "mdi mdi-autorenew mdi-spin";
      }
      line['text'] = $.lang( line['text'] );

      var $line = $( $.views.Html( "/dialog/progressDialog/line", line ) );
      $( "#ID_CONTENT" + this.dynid ).append( $line );
    }
  }


  //---------------------------------------------------------------------------
  //  DATA
  //---------------------------------------------------------------------------

  DataChangeMessageLine( param )
  {
    var line = this.messageLines.find( ( rec ) => rec['id'] == param['id'] );
    if( line )  {
      line['finished'] = param['finished'];
      line['text'] = param['text'];
    }
    else {
      this.messageLines.push( param )  ;
    }
  }

  //---------------------------------------------------------------------------
  //  STATIC
  //---------------------------------------------------------------------------

  static Show( options )
  {
    var createData = $.extend( true, [], {
      view: ProgressDialog.template,
      size: "27em",
      height: -1,
      type: 'dialog',
      layout: 'simple'
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createData );
  }

  static Hide()
  {
    $( '#ID_BODY_DIALOG' ).viewable( "destroy" );
  }

  static Message( id, isFinished, text )
  {
    $( '#ID_BODY_DIALOG' ).viewable( "option", "message", { id: id, finished: isFinished, text: text } );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

ProgressDialog.Register( ProgressDialog.template );
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          28.05.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class ChangePasswordDialog extends Viewable
{
  //-------------------------------------------------------
  //  DIALOG LOAD
  //
  Load( options )
  {
    this.isSavePressed = false;

    if( 'text' in options )  {
      $( '#ID_TITLE' + this.dynid ).text( $.lang( options['text'] ) );
    }

    // controls
    $( '#ID_PASSWORD' + this.dynid ).edit( {
      size: 30,
      status: "normal",
      password: true,
      //errorAnchor: ".password-error",
      placeholder: "{$password_placeholder_password}"
    } );

    $( '#ID_PASSWORD_REPEATED' + this.dynid ).edit( {
      size: 30,
      status: "normal",
      password: true,
      //errorAnchor: ".password-error",
      placeholder: "{$password_placeholder_repeated}",
      change: $.proxy( this.VerifyForm, this )
    } );

    $( '#ID_PASSWORD' + this.dynid ).edit( "focus" );

  }//load


  //-------------------------------------------------------
  //  DIALOG EVENTS 
  //  
  OnEvent( eventName, data )
  {
    switch( eventName ) {
      // buttons
      case 'button': {
        if( this.OnSave( ) == true && data == "IDOK" )  {
          let password = $( '#ID_PASSWORD' + this.dynid ).edit( "option", "value" );
          this.Destroy();
          this.FireEvent( 'result', password );
        }
        else if( data == "IDCANCEL" )  {
          this.Destroy();
        }
      }
      break;

      // keyboard: ENTER (true) and ESC (false)
      case 'canclose': {
        if( data == true )  {   // <ENTER>
          if( this.OnSave() == true ) {
            let password = $( '#ID_PASSWORD' + this.dynid ).edit( "option", "value" );
            this.FireEvent( 'result', password );
            return true;
          }
          else  {
            return false;
          }
        }
        else  { // <ESC>
          return true;
        }
      }
    }

    return true;
  }

  //-------------------------------------------------------
  //  SAVE
  //
  OnSave()
  {
    // enable verify
    this.isSavePressed = true;
    return this.VerifyForm();
  }

  //-------------------------------------------------------
  //  VERIFY FORM
  //
  VerifyForm( )
  {
    // we do not very the controls until <ok> is pressed
    if( !this.isSavePressed )  {
      return true;
    }

    let password1 = $( '#ID_PASSWORD' + this.dynid ).edit( "option", "value" );
    let password2 = $( '#ID_PASSWORD_REPEATED' + this.dynid ).edit( "option", "value" );
    let isOk = false;

    // error password must no be empty
    if( password1.length == 0 )  {
      $( '#ID_PASSWORD' + this.dynid ).edit( "option", "error", "{$change_password_error_empty}" );
      $( '#ID_PASSWORD' + this.dynid ).edit( "focus" );
      isOk = false;
    }
    // error password and repeat are not identical
    else if( password1 != password2 ) {
      $( '#ID_PASSWORD' + this.dynid ).edit( "option", "error", null );
      $( '#ID_PASSWORD_REPEATED' + this.dynid ).edit( "option", "error", "{$change_password_error_different}" );
      $( '#ID_PASSWORD_REPEATED' + this.dynid ).edit( "focus" );
      isOk = false;
    }
    else  {
      $( '#ID_PASSWORD' + this.dynid ).edit( "option", "error", null );
      $( '#ID_PASSWORD_REPEATED' + this.dynid ).edit( "option", "error", null );
      isOk = true;
    }

    // enable/disable <OK>
    if( !isOk )   {
      this.$controls['IDOK'].button( 'option', 'status', 'disabled' );
    }
    else  {
      this.$controls['IDOK'].button( 'option', 'status', 'default' );
    }

    // result
    return isOk;
  }


  //-------------------------------------------------------
  //  CLEAR ERROR (after modification)
  //
  ClearError( value )
  {
    $( '#ID_PASSWORD'          + this.dynid ).edit( "option", "error", null );
    $( '#ID_PASSWORD_REPEATED' + this.dynid ).edit( "option", "error", null );
  }


  //-------------------------------------------------------
  //  STATIC 
  //-------------------------------------------------------

  static Show( options )
  {
    var createdata = $.extend( true, [], {
      view: '/dialog/changePassword',
      type: 'dialog',
      layout: 'classic2',
      height: -1,
      size: CONST.dlgSize.M,
      top: CONST.dlgTop.S,
      buttons: [
        { id: 'IDCANCEL', size: 13, status: 'normal', text: '{$const_dialog_button_cancel}' },
        { id: 'IDOK', size: 13, status: 'hilight', text: '{$const_dialog_button_save}' }
      ]
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

ChangePasswordDialog.Register( '/dialog/changePassword' );
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          28.05.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class RebootAccessDialog extends Viewable
{
  //-------------------------------------------------------
  //  LOAD
  //
  Load( data )
  {
    $( "#ID_REBOOT_TEXT" + this.dynid ).text( $.lang( data['text'] ) );
    
  }//load

  //-------------------------------------------------------
  //  VIEWABLE EVENTS 
  //  
  OnEvent( eventName, data )
  {
    switch( eventName ) {
      // buttons
      case 'button': {
        if( data == "IDOK" )  {
          this.FireEvent( 'result', true );
        }
        this.Destroy();
      }
      break;

      // keyboard: ENTER and ESC
      case 'canclose': {
        if( data == true ) {
          this.FireEvent( 'result', true );
        }
      }
      break;
    }

    return true;
  }


  //-------------------------------------------------------
  //  STATIC 
  //-------------------------------------------------------

  static Show( options )
  {
    var text = "{$reboot_agw_really_restart_agw}";
    if( 'layout' in options && options['layout'] == "reboot" )  {
      text = "{$reboot_agw_reboot}";
    }

    var createdata = $.extend( true, [], {
      view: '/dialog/rebootAccess',
      type: 'dialog',
      layout: 'classic2',
      size: "25em",
      height: "10em",
      text: text,
      buttons: [
        { id: 'IDCANCEL', size: 14, status: 'normal', text: '{$reboot_agw_cancel}' },
        { id: 'IDOK', size: 14, status: 'red', text: '{$reboot_agw_ok}' }
      ]
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

RebootAccessDialog.Register( '/dialog/rebootAccess' );
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          28.05.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class FormNewDialog extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATICS
  //---------------------------------------------------------------------------

  static template = "/dialog/formNew";

  //---------------------------------------------------------------------------
  //  LOAD
  //
  Load( createOptions )
  {
    /*this.data = FormFactory.MenuCreate();
    console.log( this.data );
    this.data[10]['status'] = "disabled";
    this.data[10]['text'] += " {$formnew_no_license}";*/

    Data.Instance().On( "DATA/UPDATE/menu.licence", this.etag, this.DataResponseEntityLicense.bind( this ) );


    // create listbox with search
    $( "#ID_LIST" + this.dynid ).listbox( {
      size: -1,
      height: 9,
      search: true,
      sort: "+",
      sortKey: "text",
      data: [], //this.data,
      searchPlaceholder: "{$const_common_search_placeholder}",
      result: this.ControlListResult.bind( this ),
      dblclick: this.ControlListDblclick.bind( this )
    } );
    $( "#ID_LIST" + this.dynid ).listbox( "focus" );

    this.$controls["IDOK"].button( "option", "status", "disabled" );

    // fill data
    this.FormFillList();

  }//load

  //---------------------------------------------------------------------------
  //  VIEWABLE EVENTS
  //  
  OnEvent( eventName, data )
  {
    switch( eventName ) {
      // buttons
      case 'button':
      case 'canclose': {
        if( data == 'IDOK' || data === true  )  {
          this.CommandOk( );
        }
        else {
          this.CommandCancel();
        }
      }
      break;
    }

    return true;
  }//onevent


  //===========================================================================
  //  FORM
  //===========================================================================

  FormFillList()
  {
    // get data
    this.data = FormFactory.MenuCreate();
    var licenseInformation = Data.Instance().Licences();

    for( var record of this.data ) {
      if( 'licenses' in record && record['licenses'] != null )  {
        for( var licenseTag in record['licenses'] ) {
          // if this admin is used in a previous version server will
          // not show a new license (e.g. ip-cam).
          // this is done for branch video2 (v6.2.x)
          if( ! ( licenseTag in licenseInformation ) )
            continue;
            
          var freeLicenes = licenseInformation[ licenseTag ]['availabeLicenses'] - licenseInformation[ licenseTag ]['engagedLicenses'];
          if( freeLicenes < 0 ) freeLicenes = 0;
          if( record['licenses'][ licenseTag ] > freeLicenes ) {
            record['status'] = "disabled";
            record['text'] += " {$formnew_no_license}";
            break;
          }
        }
      }
    }

    $( "#ID_LIST" + this.dynid ).listbox( "option", "data", this.data );
    $( "#ID_LIST" + this.dynid ).listbox( "focus" );
  }

  
  //===========================================================================
  //  DATA
  //===========================================================================

  DataResponseEntityLicense()
  {
    this.FormFillList();
  }

  //===========================================================================
  //  CONTROL RESPONSE
  //===========================================================================

  ControlListResult( result )
  {
    if( result.length )  {
      this.$controls["IDOK"].button( "option", "status", "default" );
    }
    else {
      this.$controls["IDOK"].button( "option", "status", "disabled" );
    }
  }

  ControlListDblclick( result )
  {
    let record = this.data.find( ( rec ) => rec['id'] == result );
    if( record )  {
      this.FireEvent( 'result', record );
    }
    this.Destroy();
  }

  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Button IDOK or <enter>
  // 
  CommandOk()
  {
    let result = $( "#ID_LIST" + this.dynid ).listbox( "option", "value" );
    if( result.length > 0 )  {
      let record = this.data.find( ( rec ) => rec['id'] == result[ 0 ] );
      this.FireEvent( 'result', record );
    }
    this.Destroy();
  }

  //---------------------------------------------------------------------------
  //  Button IDCANCEL or <esc>
  // 
  CommandCancel()
  {
    this.Destroy();
  }

  //===========================================================================
  //  STATIC 
  //===========================================================================

  static Show( options )
  {
    let createdata = $.extend( true, [], {
      view: FormNewDialog.template,
      type: 'dialog',
      layout: 'classic2',
      size: "30em",
      height: -1,
      top: CONST.dlgTop.L,
      buttons: [
        { id: 'IDCANCEL', size: 13, status: 'normal', text: '{$const_dialog_button_cancel}' },
        { id: 'IDOK', size: 13, status: 'hilight', text: "{$const_dialog_button_ok}" }
      ],
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }
}


//=============================================================================
//  BUILDER
//=============================================================================

FormNewDialog.Register( FormNewDialog.template );
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          18.08.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class DeviceListsDialog extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATICS
  //---------------------------------------------------------------------------

  static template = "/dialog/deviceLists";

  //---------------------------------------------------------------------------
  //  MEMBER VARIABLES
  //---------------------------------------------------------------------------

  isContactOnlyDoor = false;
  contactAddressbook = null;
  addressBookData = null;

  //---------------------------------------------------------------------------
  //  LOAD
  //
  Load( createOptions )
  {
    // if an addressbook is needed, load it.
    if( 'addressBook' in createOptions && createOptions['addressBook'] != "none" ) {
      this.contactAddressbook = createOptions['addressBook'];
      Data.Instance().Addressbook().AddressbookRequest( this.contactAddressbook, ( result ) =>  {
        if( result )  {
          this.addressBookData = result;
          this.FormContactAddAddressbookData( result );
        }
      } );

    }

    // build desired lists
    var listData = [];
    if( 'lists' in createOptions )  {
      if( 'contact' in createOptions['lists'] && createOptions['lists']['contact'] ){
        listData.push( { id: "contact", icon: "mdi mdi-account-multiple-outline", text: "{$devicelist_category_contact}" } );
      }
      if( 'doorcontact' in createOptions['lists'] && createOptions['lists']['doorcontact'] ) {
        this.isContactOnlyDoor = true;
        listData.push( { id: "contact", icon: "mdi mdi-account-multiple-outline", text: "{$devicelist_category_contact}" } );
      }
      if( 'camera' in createOptions['lists'] && createOptions['lists']['camera'] ){
        listData.push( { id: "camera", icon: "mdi mdi-video", text: "{$devicelist_category_camera}" } );
      }
      if( 'switch' in createOptions['lists'] && createOptions['lists']['switch'] ){
        listData.push( { id: "switch", icon: "mdi mdi-flag", text: "{$devicelist_category_switch}" } );
      }
    }
    else {
      listData =  [
        { id: "contact", icon: "mdi mdi-account-multiple-outline", text: "{$devicelist_category_contact}" },
        { id: "camera", icon: "mdi mdi-video", text: "{$devicelist_category_camera}" },
        { id: "switch", icon: "mdi mdi-flag", text: "{$devicelist_category_switch}" },
      ];
    }

    // create listbox for category
    $( "#ID_CATEGORY" + this.dynid ).listbox( {
      size: -1,
      height: 4,
      selected: [ "contact" ],
      data: listData,
      result: this.ControlCategoryResult.bind( this )
    } );

    // show contact list first & set focus to category
    this.FormFillContactList();
    $( "#ID_CATEGORY" + this.dynid ).listbox( "focus" );

  }//load

  //---------------------------------------------------------------------------
  //  VIEWABLE EVENTS
  //  
  OnEvent( eventName, data )
  {
    switch( eventName ) {
      // buttons
      case 'button':
      case 'canclose': {
        if( data == 'IDCLOSE' || data === true  )  {
          this.CommandClose( );
        }
      }
      break;
    }

    return true;
  }//onevent


  //===========================================================================
  //  CONTROL RESPONSE
  //===========================================================================

  ControlCategoryResult( selected )
  {
    switch( selected[ 0 ] ) {
      case "contact": this.FormFillContactList();   break;
      case "camera":  this.FormFillCameraList();    break;
      case "switch":  this.FormFillSwitchList();    break;
    }
  }


  //===========================================================================
  //  FORM FUNCTIONS
  //===========================================================================

  //-----------------------------------------------------------------
  //  put contact dataset into list
  //  
  FormFillContactList()
  {
    // if we need to fill the user's addressbook
    if( this.contactAddressbook )  {
      return this.FormFillContactListAddressbook();
    }
    else {
      return this.FormFillContactListStandard();
    }
  }

  //-----------------------------------------------------------------
  //  put contact dataset + addressbook into list
  //  
  FormFillContactListAddressbook()
  {
    // get shared dataset and create a view for filter
    var spDataset = Data.Instance().StatepointDataset();
    var spDataview = spDataset.BuildView( 'default', {
      rebuildAlways: true,
      //sortOrder: "+",
      //sortKey: "label",
      inquirer: this.createOptions['entid'],
      filter: this.FormContactListFilter.bind( this )
    } );

    // create a new dataset
    this.dataset = new Dataset( { 
      id: "address",
      preview: ( record ) => {
        return { id: record['address'], text: record['label'], icon: record['icon'], status: "normal" };
      }
    } );

    // copy all records from databiew into new dataset
    for( let index = 0; index < spDataview.length; index++ ) {
      var record = spDataview.GetByIndex( index );
      this.dataset.Add( record );
    }

    // (re-)create listbox
    $( "#ID_LIST" + this.dynid ).listbox( {
      size: -1,
      height: 7,
      search: true,
      sort: "+",
      sortKey: "label",
      data: this.dataset,
      searchPlaceholder: "{$const_common_search_placeholder}"
    } );

    if( this.addressBookData ) {
      this.FormContactAddAddressbookData( );
    }

  }

  //-----------------------------------------------------------------
  //  add addressbook data into list
  //
  FormContactAddAddressbookData( )
  {
    for( var record of this.addressBookData )  {
      var newRecord = {
        address: record['id'],
        class: "addrBook",
        entid: null,
        icon: CONST_CLASS_ICO['addrBook'],
        label: record['label'],
        name: null,
        type: null
      };
      this.dataset.Add( newRecord );
    }
  }

  //-----------------------------------------------------------------
  //  put contact dataset into list
  //  
  FormFillContactListStandard()
  {
    // get shared dataset and create a view
    this.dataset = Data.Instance().StatepointDataset();
    let dataview = this.dataset.BuildView( 'default', {
      rebuildAlways: true,
      sortOrder: "+",
      sortKey: "label",
      inquirer: this.createOptions['entid'],
      filter: this.FormContactListFilter.bind( this )
    } );

    // (re-)create listbox
    $( "#ID_LIST" + this.dynid ).listbox( {
      size: -1,
      height: 7,
      search: true,
      sort: "+",
      sortKey: "label",
      data: dataview,
      searchPlaceholder: "{$const_common_search_placeholder}"
    } );

  }


  //-----------------------------------------------------------------
  //  filter function for contact list
  //
  FormContactListFilter( id, record, options, dataviewClass )
  {
    let isAllowed = false;

    // ---- filter for name ----
    isAllowed = ( record['name'] == "presence" );

    // ----  sus: only door ----
    if( isAllowed && this.isContactOnlyDoor ) {
      isAllowed = record['class'] == "door";
    }

    // ----  permissions ----
    if( isAllowed && options['inquirer'] != "*" )  {
      isAllowed = ( Data.Instance().PermissionList().IsAllowed( options['inquirer'], record['address'], "r" ) );
    }

    // result
    return isAllowed;    
  }  

  //-----------------------------------------------------------------
  //  put camera dataset into list
  //  
  FormFillCameraList()
  {
    // get shared dataset and create a view
    this.dataset = Data.Instance().StatepointDataset();
    let dataview = this.dataset.BuildView( 'default', { 
      rebuildAlways: false,
      sortOrder: "+",
      sortKey: "label",
      inquirer: this.createOptions['entid'],
      filter: this.FormCameraListFilter.bind( this )
    } );

    // (re-)create listbox
    $( "#ID_LIST" + this.dynid ).listbox( {
      size: -1,
      height: 7,
      search: true,
      sort: "+",
      sortKey: "label",
      data: dataview,
      searchPlaceholder: "{$const_common_search_placeholder}"
    } );

  }


  //-----------------------------------------------------------------
  //  filter function called by dataview
  //
  FormCameraListFilter( id, record, options, dataviewClass )
  {
    let isAllowed = false;

    // ---- filter for name ----
    isAllowed = ( record['name'] == "camera" );

    // ----  permissions ----
    if( isAllowed && options['inquirer'] != "*" )  {
      isAllowed = ( Data.Instance().PermissionList().IsAllowed( options['inquirer'], record['address'], "r" ) );
    }

    // result
    return isAllowed;
  }  

  //-----------------------------------------------------------------
  //  put switches dataset into list
  //  
  FormFillSwitchList()
  {
    // get shared dataset and create a view
    this.dataset = Data.Instance().StatepointDataset();
    let dataview = this.dataset.BuildView( 'default', { 
      rebuildAlways: true,
      sortOrder: "+",
      sortKey: "label",
      inquirer: this.createOptions['entid'],
      filter: this.FormSwitchListFilter.bind( this )
    } );

    // (re-)create listbox
    $( "#ID_LIST" + this.dynid ).listbox( {
      size: -1,
      height: 7,
      search: true,
      sort: "+",
      sortKey: "label",
      data: dataview,
      searchPlaceholder: "{$const_common_search_placeholder}"
    } );
  }

  //-----------------------------------------------------------------
  //  filter function called by dataview
  //
  FormSwitchListFilter( id, record, options, dataviewClass )
  {
    let isAllowed = false;

    // ---- filter for name ----
    isAllowed = ( ( record['name'] != "camera" ) && ( record['type'] == "BOOL" ) );

    // ----  permissions ----
    if( isAllowed && options['inquirer'] != "*" )  {
      isAllowed = ( Data.Instance().PermissionList().IsAllowed( options['inquirer'], record['address'], "w" ) );
    }

    // result
    return isAllowed;
  }    

  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Button IDCLOSE or <esc>
  // 
  CommandClose()
  {
    this.Destroy();
  }


  //===========================================================================
  //  STATIC 
  //===========================================================================

  static Show( options )
  {
    let createdata = $.extend( true, [], {
      view: DeviceListsDialog.template,
      type: 'dialog',
      layout: 'classic2',
      size: "30em",
      height: -1,
      top: CONST.dlgTop.L,
      buttons: [
        { id: 'IDCLOSE', size: 13, status: 'hilight', text: "{$const_dialog_button_close}" }
      ],
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

DeviceListsDialog.Register( DeviceListsDialog.template );
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          28.05.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class PermissionsDialog extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATICS
  //---------------------------------------------------------------------------

  static template = "/dialog/permissions";


  //---------------------------------------------------------------------------
  //  LOAD
  //
  Load( createOptions )
  {
    // mother of everything    
    this.entinfo = Data.Instance().Entinfo( createOptions['entid'] );

    // datasets
    this.permholderDataset = new Dataset( { id: "id" } );
    this.permholderDataview = this.permholderDataset.BuildView( 'default', { } );
    this.rightDataset = new Dataset( { id: "id" } );

    // dataset/-view used by <add> button
    this.entityDataset = Data.Instance().EntityDataset();
    let entityDataview = this.entityDataset.BuildView( 'default', { 
      rebuildAlways: true,
      sortOrder: "+",
      sortKey: "text",
      filter: this.ControlEntityDataviewFilter.bind( this )
    } );

    // quick data
    this.permissionList = [];

    // title
    let title = $.lang( "{$permissions_dialog_title}" );
    title = $.lang( title.replace( "{{name}}", this.entinfo['name'] ) );
    $( "#ID_TITLE" + this.dynid ).text( title );

    // create listbox for right holders
    $( "#ID_RIGHTHOLDER" + this.dynid ).listbox( {
      size: -1,
      height: 4,
      data: this.permholderDataview,
      placeholder: "{$permissions_who_placeholder}",
      result: this.ControlPermholderListResult.bind( this )
    } );

    // add right holder
    $( "#ID_ADD" + this.dynid ).button( {
      size: CONST.size.M,
      status: 'normal',
      text: "{$permissions_button_add}",
      click: () => { 
        $( '#ID_POPUP' + this.dynid ).listbox( "show" );
        //this.Option( "modified", 1 );
      }
    } );
    $( '#ID_POPUP' + this.dynid ).listbox( {
      size: 40,
      height: 5,
      type: 'listbox',
      search: true,
      status: "normal",
      searchPlaceholder: "{$const_common_search_placeholder}",
      anchor: "#ID_ADD" + this.dynid,
      position: "bottomright",
      data: entityDataview,
      result:  this.ControlAddPopupResult.bind( this )
    } );    



    // delete right holder
    $( "#ID_DELETE" + this.dynid ).button( {
      size: CONST.size.M,
      status: 'disabled',
      text: "{$permissions_button_delete}",
      click: this.ControlDeleteClick.bind( this )
    } );

    // the rights
    $( "#ID_RIGHTS" + this.dynid ).grid( {
      size: -1,
      height: 10,
      selection: 'none',
      data: this.rightDataset,
      dataId: 'id',
      sort: "text",
      scrollbar: 'overlay', // 'overlay', 'static'
      header: {
        'text':  { text: "{$permissions_rightgrid_text}", size: -40 },
        'read':  { text: "{$permissions_rightgrid_read}", size: 10, inplace: "check", align: "left" },
        'write': { text: "{$permissions_rightgrid_write}", size: 10, inplace: "check", align: "left" },
        'cr': { text: "{$permissions_rightgrid_cr}", size: 10, inplace: "check", align: "left" }
      },
      inplaceEdit: this.ControlGridRightsInplaceEdit.bind( this ),
      renderCell: this.ControlGridRightsRenderCell.bind( this )
    } );        

    // data initialisation
    let dispPermission = DevicePermissions.DisplayedPermission( this.entinfo );
    this.isInheritPermholder = dispPermission['inherit'];
    this.displayedPermissions = dispPermission['list'];

    // get first perm holder and display right list
    //this.FormRebuild();

    // load entity
    //Data.Instance().On( "DATA/UPDATE/" + this.entinfo['entid'], this.etag, this.DataResponseEntity.bind( this ) );
    this.DataRequest();

  }//load

  //---------------------------------------------------------------------------
  //  VIEWABLE EVENTS
  //  
  OnEvent( eventName, data )
  {
    //console.log( eventName, data );
    switch( eventName ) {
      // buttons
      case 'button':
      case 'canclose': {
        if( data == 'IDCANCEL' || data === false  )  {
          this.CommandCancel( );
        }
        else if( data == 'IDOK' || data === true  )  {
          this.CommandSave( );
        }
      }
      break;

      case 'modified': {
        this.StatusModified();        
      }
    }

    return true;
  }//onevent


  //===========================================================================
  //  FORM FUNCTIONS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  fill the list of permholders of this entity
  //
  FormRebuild( permHolder = null )
  {
    this.currentPermHolder = ( permHolder == null ) ? this.DataGetFirstExplictPermholder() : permHolder;
    
    // fill grids
    this.FormFillPermholderList();
    this.FormFillRightList( this.currentPermHolder );

    // select initial permholder
    $( "#ID_RIGHTHOLDER" + this.dynid ).listbox( "option", "selected", [ this.currentPermHolder ] );
    this.FormManagePermholderButtons();

  }

  //---------------------------------------------------------------------------
  //  fill the list of permholders of this entity
  //
  FormFillPermholderList()
  {
    // default "*" is supported
    let permHolder = { '*': true };
    for( let perm of this.permissionList )  {
      permHolder[ perm['w'] ] = true;
    }

    let permholderData = [];
    for( let entityId in permHolder )  {
      
      // skip if not folder
      if( entityId == "*" && !this.isInheritPermholder )  {
        continue;
      }

      let record = null;
      if( entityId == "*" )  {
        record = {};
        record['id'] = entityId;
        record['text'] = "{$permissions_inherit_objects}";
        record['icon'] = "mdi mdi-layers";
      }
      else {
        let entinfo = Data.Instance().Entinfo( entityId );
        if( entinfo )  {
          record = {};
          record['id'] = entityId;
          record['text'] = entinfo['name'];
          record['icon'] = CONST_CLASS_ICO[ entinfo['class'] ];
        }
        else {
          console.log( "Permission holder does not exist. Remove (", entityId, "), current =", this.entinfo['entid'] );
          this.FormRemoveHolderFromRightList( entityId );
        }
      }
      if( record ) {
        permholderData.push( record );
      }
    }

    // put into dataset
    this.permholderDataset.SetData( permholderData );

  }

  //---------------------------------------------------------------------------
  //  enable / disable delete buttons on permholder list
  //
  FormManagePermholderButtons()
  {
    let selected = $( "#ID_RIGHTHOLDER" + this.dynid ).listbox( "option", "selected" );
    if( ( selected.length > 0  )
     && ( selected[ 0 ] != "*" )
      ) {
      $( "#ID_DELETE" + this.dynid ).button( "option", "status", "normal" );
    }
    else {
      $( "#ID_DELETE" + this.dynid ).button( "option", "status", "disabled" );
    }
  }

  //---------------------------------------------------------------------------
  //  build the list of permissions that need to be shown
  //
  FormFillRightList( permWho )
  {
    let list = [];

    // if no permholder is selected just show permission list.
    // just informative: don't show inplace controls (fields := null)
    if( permWho == "*" && !this.isInheritPermholder )  {
      for( let permName in this.displayedPermissions )  {
        let permDef = this.displayedPermissions[ permName ];
        let newRecord = {
          id: permName,
          who: permWho,
          text: permDef['label'],
          read: null,
          write: null,
          cr: null,
          holder: null
        };      
        list.push( newRecord );  
      }
    }
    // show editable list for selected permission holder
    else {
      for( let permName in this.displayedPermissions )  {
        let permDef = this.displayedPermissions[ permName ];
        let permRecord = this.HelperFindPermissionRecord( permWho, permName );
        let newRecord = {
          id: permName,
          who: permWho,
          text: permDef['label'],
          read: ( permRecord['p'].indexOf( "r" ) >= 0 ),
          write: ( permRecord['p'].indexOf( "w" ) >= 0 ),
          cr: permRecord['cr'],
          holder: permRecord['holder']
        };      
        list.push( newRecord );  
      }
    }

    //console.log( "FILL", list );
    this.rightDataset.SetData( list );
  }

  //---------------------------------------------------------------------------
  //  save the current right list into entity's permission
  //  
  FormReadRightList( permWho )
  {
    //console.log( "+ FormReadRightList", permWho, this.permissionList );

    // get a copy of data
    let data = this.rightDataset.GetData();
    for( let perm of data )  {

      // if 'cr' is set the permission should inherit from parent, so remove
      if( perm['cr'] == true )  {
        let index = this.permissionList.findIndex( ( rec ) => rec['w'] == permWho && rec['f'] == perm['id'] );
        if( index >= 0 )  {
          this.permissionList.splice( index, 1 );
        }
      }
      // local permission record in this entity
      else {

        let index = this.permissionList.findIndex( ( rec ) => rec['w'] == permWho && rec['f'] == perm['id'] );

        // modify exiting permission
        if( index >= 0 )  {
          let right = perm['read'] == true ? "r" : "";
          if( perm['write'] == true ) {
            right += "w";
          }
          this.permissionList[ index ]['p'] = right;
        }

        // add new permission
        else {
          let right = perm['read'] == true ? "r" : "";
          if( perm['write'] == true ) {
            right += "w";
          }
          this.permissionList.push( {
            p: right,
            w: permWho,
            f: perm['id']
          });
        }
      }
      
    }

    //console.log( "- FormReadRightList", permWho, this.permissionList );

  }

  //---------------------------------------------------------------------------
  //  save the current right list into entity's permission
  //  
  FormRemoveHolderFromRightList( permWho )
  {
    let newPermissionList = this.permissionList.filter( ( rec ) => rec['w'] != permWho );
    this.permissionList = newPermissionList;

    this.Option( "modified", 1 );
  }

  //---------------------------------------------------------------------------
  //  read all unsaved controls
  //  
  FormReadControls()
  {
    this.FormReadRightList( this.currentPermHolder );
    this.entity['permission'] = this.permissionList;
  }


  //===========================================================================
  //  DATA FUNCTIONS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  DataRequest(): do xhr read request(s)
  //
  DataRequest()
  {
    this.StatusLoading();

    // we need to create a copy 
    this.entity = {};
    this.entity['entid'] = this.entinfo['entid'];
    this.entity['name'] = this.entinfo['name'];
    this.entity['permission'] = $.clone( this.entinfo['permission'] );
    this.permissionList = this.entity['permission'];
    if( !Array.isArray( this.permissionList ) ) {
      this.permissionList = [];
    }

    this.StatusLoadFinished();
    this.FormRebuild();
  }

  //---------------------------------------------------------------------------
  //  get the first permholder to select
  //
  DataGetFirstExplictPermholder()
  {
    for( let perm of this.permissionList )  {
      if( perm['w'] != "*" )  {
        return perm['w'];
      }
    }
    return "*";
  }

  //---------------------------------------------------------------------------
  //  DataSaveRequest(): do xhr write request(s)
  //
  DataSaveRequest()
  {
    // save the entity
    let entity = {};
    entity['entid'] = this.entity['entid'];
    entity['permission'] = this.entity['permission'];

    // write partial entity, notMoved, noLogEntry
    Data.Instance().SavePartialEntity( entity, false, false );

  }

  //===========================================================================
  //  CONTROL
  //===========================================================================

  //---------------------------------------------------------------------------
  //  render cell, manage disabled if cr & tooltip for inherit rights
  //
  ControlGridRightsRenderCell( columnId, record )
  {
    let item = {
      text: record[ columnId ],
      icon: null
    };
    if( ( columnId == "read" && record['cr'] == true )
     || ( columnId == "write" && record['cr'] == true )
      ) {
      item['status'] = "disabled";
    }
    if( columnId == "cr" && record['cr'] == true )  {
      let holderInfo = Data.Instance().Entinfo( record['holder'] );
      if( holderInfo )  {
        let tooltip = $.lang( "{$permissions_rightgrid_cr_tooltip}" );
        tooltip = tooltip.replace( "{{name}}", holderInfo['name'] );
        tooltip = $.lang( tooltip );
        item['tooltip'] = tooltip;
        item['icon'] = "mdi mdi-information-outline";
      }
    }        
    return item;        
  }

  //---------------------------------------------------------------------------
  //  checkboxes are clicked
  //
  ControlGridRightsInplaceEdit( columnId, record, param )
  {
    this.Option( "modified", 1 );

    switch( columnId )  {
      case "read":  return this.ControlGridRightsInplaceRead( record, param );
      case "write": return this.ControlGridRightsInplaceWrite( record, param );
      case "cr":    return this.ControlGridRightsInplaceCR( record, param );
    }
    return record;
  }
  
  //---------------------------------------------------------------------------
  //  change record if read
  //
  ControlGridRightsInplaceRead( record, param )
  {
    record['read'] = !( record['read'] );
    if( record['read'] == false )  {
      record['write'] = false;
    }
    return record;
  }

  //---------------------------------------------------------------------------
  //  change record if write
  //
  ControlGridRightsInplaceWrite( record, param )
  {
    record['write'] = !( record['write'] );
    if( record['write'] == true )  {
      record['read'] = true;
    }
    return record;
  }

  //---------------------------------------------------------------------------
  //  change record for cr, set read/write & holder
  //  
  ControlGridRightsInplaceCR( record, param )
  {
    record['cr'] = !( record['cr'] );
    let permRecord = this.HelperFindPermissionRecord( record['who'], record['id'], record['cr'] );
    record['read'] = ( permRecord['p'].indexOf( "r" ) >= 0 );
    record['write'] = ( permRecord['p'].indexOf( "w" ) >= 0 );
    record['holder'] = permRecord['holder'];
    return record;
  }

  //---------------------------------------------------------------------------
  //  permholder changed
  //    
  ControlPermholderListResult( result )  
  {
    if( ( result.length > 0 ) && ( result[ 0 ] != this.currentPermHolder ) ) {
      // save changes into permissions
      this.FormReadRightList( this.currentPermHolder );
      // read new
      this.currentPermHolder = result[ 0 ];
      this.FormFillRightList( this.currentPermHolder );
    }
    
    this.FormManagePermholderButtons();
  }

  //---------------------------------------------------------------------------
  //  current permholder should be deleted
  //      
  ControlDeleteClick()
  {
    if( this.currentPermHolder != "*" ) {
      this.FormRemoveHolderFromRightList( this.currentPermHolder );
      this.FormRebuild();
    }
  }  

  //---------------------------------------------------------------------------
  //  add new permission holder
  //     
  ControlAddPopupResult( result )  
  {
    //console.log( "ControlAddPopupResult", result );
    if( result.length > 0 )  {

      // save changes into permissions
      this.FormReadRightList( this.currentPermHolder );

      for( let permName in this.displayedPermissions )  {

        // get the permission the perm hold previously had
        let inquirerEntityId = result[ 0 ];
        let targetStatpoint = Data.Instance().RootEntid() + "/" + this.entinfo['entid'] + "/" + permName;
        let permRecord = Data.Instance().PermissionList().Get( inquirerEntityId, targetStatpoint );
        if( !permRecord ) {
          permRecord = { p: "deny" };
        }

        // add permission record
        this.permissionList.push( {
          p: permRecord['p'],
          w: result[ 0 ],
          f: permName
        } );
      }      

      this.Option( "modified", 1 );

      // rebuild dialog and select new perm holder
      this.FormRebuild( result[ 0 ] );
    }
  }

  //---------------------------------------------------------------------------
  //  filter for entity list
  //       
  ControlEntityDataviewFilter( id, record, options, dataviewClass )
  {
    let isAllowed = true;

    // check for allowed classes
    if( record['class'] == "admin" )  {
      isAllowed = false;
    }

    // self is forbidden
    if( record['id'] == this.entinfo['entid'] ) {
      isAllowed = false;
    }

    // already in permholder list?
    if( isAllowed ) {
      isAllowed = !( this.permholderDataset.Has( id ) );
    }

    // check if child of folder "project"
    if( isAllowed ) {
      isAllowed = Data.Instance().IsChild( id, "folder.confdev" );
    }

    return isAllowed;
  }


  //===========================================================================
  //  HELPER
  //===========================================================================

  //---------------------------------------------------------------------------
  //  find a permission record in permission list
  //
  HelperFindPermissionRecord( permWho, spName, isParentRequest = false )
  {
    // try to find the permission record in local entity
    if( !isParentRequest )  {
      let permissionList = this.permissionList;
      let index = permissionList.findIndex( ( rec ) => ( rec['w'] == permWho ) && ( rec['f'] == spName ) );
      if( index >= 0 )  {
        let result = $.clone( permissionList[ index ] );
        result['holder'] = this.entinfo['entid'];
        result['cr'] = false;
        return result;
      }
    }

    //console.log( "HelperFindPermissionRecord:", permWho, spName, isParentRequest );


    // dialog should show the permission that are inherited
    if( permWho == "*" )  {
      let inquirerEntityId = this.entinfo['parent'];
      let targetStatpoint = Data.Instance().RootEntid() + "/" + this.entinfo['entid'] + "/" + spName;
      let permRecord = Data.Instance().PermissionList().Get( inquirerEntityId, targetStatpoint );
      if( permRecord ) {
        let result = permRecord;
        result['cr'] = true;
        return result;
      }
    }
    // dialog should show the permission that are explicitly inherited.
    else {
      let targetStatpoint = Data.Instance().RootEntid() + "/" + this.entinfo['entid'] + "/" + spName;
      let permRecord = Data.Instance().PermissionList().Get( permWho, targetStatpoint );
      if( permRecord ) {
        let result = permRecord;
        result['cr'] = true;
        return result;
      }
    }


    return { w: "*", f: "*", p: "deny", cr: false };

  }


  //===========================================================================
  //  CONTROL RESPONSE
  //===========================================================================


  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Button IDCANCEL or <esc>
  // 
  CommandCancel()
  {
    this.Destroy();
  }

  //---------------------------------------------------------------------------
  //  Button IDCANCEL or <esc>
  // 
  CommandSave()
  {
    this.FormReadControls();
    this.DataSaveRequest();

    this.Destroy();
  }

  //===========================================================================
  //  STATUS MACHINE
  //===========================================================================

  //---------------------------------------------------------------------------
  //  entered in DataRequest
  //
  StatusLoading()
  {
    //console.log( "statusLoading" );
    this.status = "statusLoading";

    $( "#ID_RIGHTHOLDER" + this.dynid ).listbox( "option", "status", "disabled" );
    $( "#ID_ADD" + this.dynid ).listbox( "option", "status", "disabled" );
    $( "#ID_DELETE" + this.dynid ).listbox( "option", "status", "disabled" );
    $( "#ID_RIGHTS" + this.dynid ).listbox( "option", "status", "disabled" );
    this.$controls["IDOK"].button( "option", "status", "disabled" );
  }

  //-----------------------------------------------------------------------------
  //  entered in DataResponseEntity
  //
  StatusLoadFinished()
  {
    //console.log( "statusLoadFinished" );
    this.status = "statusLoadFinished";

    $( "#ID_RIGHTHOLDER" + this.dynid ).listbox( "option", "status", null );
    $( "#ID_ADD" + this.dynid ).listbox( "option", "status", null );
    $( "#ID_DELETE" + this.dynid ).listbox( "option", "status", null );
    $( "#ID_RIGHTS" + this.dynid ).listbox( "option", "status", null );

    this.StatusIdle();
  }

  //-----------------------------------------------------------------------------
  //  entered from StatusLoadFinished and OnEvent modified == 0
  //
  StatusIdle()
  {
    //console.log( "statusIdle" );
    this.status = "statusIdle";
  }

  //-----------------------------------------------------------------------------
  //  entered from OnEvent modified > 0
  //
  StatusModified()
  {
    //console.log( "statusModified" );
    this.status = "statusModified";
    this.$controls["IDOK"].button( "option", "status", "default" );
  }


  //===========================================================================
  //  STATIC 
  //===========================================================================

  static Show( options )
  {
    let createdata = $.extend( true, [], {
      view: PermissionsDialog.template,
      type: 'dialog',
      layout: 'classic2',
      size: "47em",
      height: -1, //"36em",
      top: CONST.dlgTop.XL,
      buttons: [
        { id: 'IDCANCEL', size: 13, status: 'normal', text: '{$const_dialog_button_cancel}' },
        { id: 'IDOK', size: 13, status: 'default', text: "{$const_dialog_button_save}" }
      ],
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }
}


//=============================================================================
//  BUILDER
//=============================================================================

PermissionsDialog.Register( PermissionsDialog.template );
﻿//=============================================================================
//
//  callProfile name dialog
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          28.07.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class NameCallProfile extends Viewable
{
  //-------------------------------------------------------
  //  LOAD 
  //
  Load( options )
  {
    // edit
    if( typeof options['value'] == "object" )  {
      this.data = $.clone( options['value'] );
    }
    // new
    else {
      this.data = {
        id: "id" + $.uuid(),
        name: "",
        permission: []
      };
    }

    // initialize controls
    $( '#ID_EDIT_NAME' + this.dynid ).edit( {
      size: 35,
      value: this.data['name'],
      validate: this.ValidateNotEmptyString.bind( this )
    } );

    $( '#ID_EDIT_NAME' + this.dynid ).edit( "focus" );
    
    this.StatusIdle();
  }//load

  //-------------------------------------------------------
  //  VIEWABLE EVENTS 
  //  
  OnEvent( eventName, data )
  {
    switch( eventName )  {

      // buttons
      case 'button': {
        if( ( data == "IDCANCEL" )
         || ( data == "IDOK" && this.CommandSave() === true )
          ) {
          this.Destroy();
        }
      }
      break;

      // keyboard: ENTER (true) and ESC (false)
      case 'canclose': {
        if( data === true ) {   // <ENTER>
          return ( this.CommandSave() === true );
        }
      }
      break;
    
      // error
      case "error": {
        if( this.Option( "modified" ) > 0 ) {
          if( data > 0 ) this.StatusError();
          else this.StatusModified();
        }
      }
      break;

      // modified
      case "modified": {
        if( this.Option( "error" ) == 0 ) {
          if( data > 0 ) this.StatusModified();
          else this.StatusIdle();
        }
        else {
          this.StatusError();
        }
      }
        break;
    }

    return true;
  }

  //-------------------------------------------------------
  //  COMMANDS
  //-------------------------------------------------------

  CommandSave()
  {
    this.Validate();
    let errNum = this.Option( "error" );
    if( errNum == 0 && this.FormReadControls() ) {
      this.FireEvent( 'result', this.data );
    }
    return ( errNum == 0 );
  }

  //-------------------------------------------------------
  //  FORM 
  //-------------------------------------------------------

  FormReadControls()
  {
    this.data['name'] = $( '#ID_EDIT_NAME' + this.dynid ).edit( "option", "value" );
    return true;
  }

  //-------------------------------------------------------
  //  VALIDATE
  //-------------------------------------------------------

  Validate()
  {
    let numErr = 0;
    numErr += $( '#ID_EDIT_NAME' + this.dynid ).edit( "validate" );
    return numErr;
  }

  ValidateNotEmptyString( validateControl )
  {
    if( typeof validateControl.value != "string" ) {
      validateControl.error = "{$const_error_nostring}";
    }
    else if( validateControl.value.length == 0 ) {
      validateControl.error = "{$name_callprofile_error_empty}";
    }
    else {
      validateControl.error = null;
    }
  }


  //-----------------------------------------------------------------
  //  STATUS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  called after loadValidate, if there is no error
  //
  StatusIdle()
  {
    this.status = "statusIdle";
    this.$controls["IDOK"].button( "option", "status", "disabled" );
  }

  //-----------------------------------------------------------------
  //  called after loadValidate, if there is no error
  //
  StatusModified()
  {
    this.status = "statusModified";
    this.$controls["IDOK"].button( "option", "status", "default" );
  }

  //-----------------------------------------------------------------
  //  if error, disable OK
  //
  StatusError()
  {
    this.status = "statusError";
    this.$controls["IDOK"].button( "option", "status", "disabled" );
  }


  //-------------------------------------------------------
  //  STATIC 
  //-------------------------------------------------------

  static Show( options )
  {
    var createdata = $.extend( true, [], {
      view: '/dialog/nameCallProfile',
      type: 'dialog', 
      layout: 'classic2',
      size: CONST.dlgSize.M,
      top: CONST.dlgTop.M,
      height: -1,
      buttons: [
        { id: 'IDCANCEL', size: 12, status: 'normal',  text: '{$const_dialog_button_cancel}' },
        { id: 'IDOK',     size: 12, status: 'hilight', text: '{$const_dialog_button_save}' }
      ]
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }
}


//=============================================================================
//  BUILDER
//=============================================================================

NameCallProfile.Register( '/dialog/nameCallProfile' );
﻿//=============================================================================
//
//  callProfile permission dialog
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          29.07.2021  as a copy of ListSelectDialog of cwf
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class PermissionCallProfile extends Viewable
{
  //-------------------------------------------------------
  //  LOAD 
  //
  Load( options )
  {
    // edit
    if( typeof options['value'] == "object" )  {
      this.data = $.clone( options['value'] );
    }
    // new
    else {
      this.data = {
        id: "id" + $.uuid(),
        t: "",
        p: true  // allowed
      };
    }

    // initialize controls
    $( '#ID_EDIT_RULE' + this.dynid ).edit( {
      size: 35,
      value: this.data['t'],
      filter: { allow: "0123456789*", deny: null },
      validate: this.ValidateEditRule.bind( this )
    } );

    $( '#ID_CHECK_FORBID' + this.dynid ).choice( {
      size: 35,
      type: "check",
      text: "{$permission_callprofile_label_forbid}",
      value: !this.data['p']
    } );

    $( '#ID_EDIT_RULE' + this.dynid ).edit( "focus" );
    

    this.StatusIdle();
  }//load

  //-------------------------------------------------------
  //  VIEWABLE EVENTS 
  //  
  OnEvent( eventName, data )
  {
    switch( eventName )  {

      // buttons
      case 'button': {
        if( ( data == "IDCANCEL" )
         || ( data == "IDOK" && this.CommandSave() === true )
          ) {
          this.Destroy();
        }
      }
      break;

      // keyboard: ENTER (true) and ESC (false)
      case 'canclose': {
        if( data === true ) {   // <ENTER>
          return ( this.CommandSave() === true );
        }
      }
      break;
    
      // error
      case "error": {
        if( this.Option( "modified" ) > 0 ) {
          if( data > 0 ) this.StatusError();
          else this.StatusModified();
        }
      }
      break;

      // modified
      case "modified": {
        if( this.Option( "error" ) == 0 ) {
          if( data > 0 ) this.StatusModified();
          else this.StatusIdle();
        }
        else {
          this.StatusError();
        }
      }
      break;
    }

    return true;
  }

  //-------------------------------------------------------
  //  COMMANDS
  //-------------------------------------------------------

  CommandSave()
  {
    this.Validate();
    let errNum = this.Option( "error" );
    if( errNum == 0 && this.FormReadControls() ) {
      this.FireEvent( 'result', this.data );
    }
    return ( errNum == 0 );
  }

  //-------------------------------------------------------
  //  FORM 
  //-------------------------------------------------------

  FormReadControls()
  {
    this.data['t'] = $( '#ID_EDIT_RULE' + this.dynid ).edit( "option", "value" );
    this.data['p'] = ! $( '#ID_CHECK_FORBID' + this.dynid ).edit( "option", "value" );
    return true;
  }

  //-------------------------------------------------------
  //  VALIDATE
  //-------------------------------------------------------

  Validate()
  {
    let numErr = 0;
    numErr += $( '#ID_EDIT_RULE' + this.dynid ).edit( "validate" );
    return numErr;
  }

  ValidateEditRule( validateControl )
  {
    if( typeof validateControl.value != "string" ) {
      validateControl.error = "{$const_error_nostring}";
    }
    else if( validateControl.value.length == 0 ) {
      validateControl.error = "{$permission_callprofile_error_empty}";
    }
    else if( validateControl.value.indexOf("*") >= 0 && validateControl.value.indexOf("*") != validateControl.value.length-1 )  {
      validateControl.error = "{$permission_callprofile_error_wildcard_pos}";
    }
    else {
      validateControl.error = null;
    }
  }


  //-----------------------------------------------------------------
  //  STATUS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  called after loadValidate, if there is no error
  //
  StatusIdle()
  {
    this.status = "statusIdle";
    this.$controls["IDOK"].button( "option", "status", "disabled" );
  }

  //-----------------------------------------------------------------
  //  called after loadValidate, if there is no error
  //
  StatusModified()
  {
    this.status = "statusModified";
    this.$controls["IDOK"].button( "option", "status", "default" );
  }

  //-----------------------------------------------------------------
  //  if error, disable OK
  //
  StatusError()
  {
    this.status = "statusError";
    this.$controls["IDOK"].button( "option", "status", "disabled" );
  }


  //-------------------------------------------------------
  //  STATIC 
  //-------------------------------------------------------

  static Show( options )
  {
    var createdata = $.extend( true, [], {
      view: '/dialog/permissionCallProfile',
      type: 'dialog', 
      layout: 'classic2',
      //size: "30em",
      //height: "16em",
      size: CONST.dlgSize.M,
      top: CONST.dlgTop.M,
      height: -1,
      buttons: [
        { id: 'IDCANCEL', size: 12, status: 'normal',  text: '{$const_dialog_button_cancel}' },
        { id: 'IDOK',     size: 12, status: 'hilight', text: '{$const_dialog_button_save}' }
      ]
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }
}


//=============================================================================
//  BUILDER 
//=============================================================================

PermissionCallProfile.Register( '/dialog/permissionCallProfile' );
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          28.05.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class AddressEntryDialog extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATICS
  //---------------------------------------------------------------------------

  static template = "/dialog/addressEntry";

  //---------------------------------------------------------------------------
  //  LOAD
  //
  Load( createOptions )
  {
    this.entry = createOptions['entry'] || {
      id: $.guidgen(),
      label: "",
      callnumber: "",
      comment: "",
      type: null
    };

    // initialize controls
    $( "#ID_LABEL" + this.dynid ).edit( {
      size: CONST.size.L,
      value: this.entry['label'],
      validate: Validate.NotEmptyString
    } );

    $( "#ID_CALLNUMBER" + this.dynid ).edit( {
      size: CONST.size.L,
      value: this.entry['callnumber'],
      filter: { allow: "0123456789*-()", deny: null },
      result: this.ControlCallnumberFilter.bind( this ),
      validate: this.ValidateCallnumber.bind( this ) //Validate.NotEmptyString
    } );

    $( "#ID_COMMENT" + this.dynid ).edit( {
      size: CONST.size.L,
      value: this.entry['comment'],
    } );

    $( "#ID_LABEL" + this.dynid ).edit( "focus" );

  }//load

  //---------------------------------------------------------------------------
  //  VIEWABLE EVENTS
  //  
  OnEvent( eventName, data )
  {
    switch( eventName ) {
      // buttons
      case 'button':
      case 'canclose': {
        if( data == 'IDOK' || data === true  )  {
          return this.CommandOk( );
        }
        else {
          this.CommandCancel();
        }
      }
      break;

      case 'modified': {
        this.$controls['IDOK'].button( 'option', 'status', ( data > 0 ) ? "hilight" : "disabled" );
      }
      break;
    }

    return true;
  }//onevent


  //===========================================================================
  //  FORM
  //===========================================================================

  FormReadControls()
  {
    this.entry['label'] = $( "#ID_LABEL" + this.dynid ).edit( "option", "value" );
    var callnumber = $( "#ID_CALLNUMBER" + this.dynid ).edit( "option", "value" );
    callnumber = callnumber.replace( /[^1234567890*()-]/gm, "" );
    this.entry['callnumber'] = callnumber;
    this.entry['comment'] = $( "#ID_COMMENT" + this.dynid ).edit( "option", "value" );
  }


  //===========================================================================
  //  VALIDATE
  //===========================================================================

  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_LABEL" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_CALLNUMBER" + this.dynid ).edit( "validate" );
    return numErr;
  }

  ValidateCallnumber( validateControl )
  {
    // error must not be an empty string
    if( Validate.NotEmptyString( validateControl ) == false ) {
      return false;
    }

    // check if callnumber is internal
    var callnumberInfo = Data.Instance().CalltargetDataset().CallnumberInfo( validateControl.value );
    if( $.isPlainObject( callnumberInfo ) 
     || Array.isArray( callnumberInfo ) 
      ) {
      validateControl.error = "{$address_error_nointernal}";
      return false;
    }

    // no error
    validateControl.error = null;
  }


  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Button IDOK or <enter>
  // 
  CommandOk()
  {
    if( this.Validate() == 0 )  {
      this.FormReadControls();
      this.FireEvent( 'result', this.entry );
      this.Destroy();
      return true;
    }
    return false;
  }

  //---------------------------------------------------------------------------
  //  Button IDCANCEL or <esc>
  // 
  CommandCancel()
  {
    this.Destroy();
  }

  //---------------------------------------------------------------------------
  //  replace special characters that can't be filtered
  //
  ControlCallnumberFilter( value )
  {
    let newValue = value;
    newValue = newValue.replace( "^", "" );
    newValue = newValue.replace( "`", "" );
    newValue = newValue.replace( "´", "" );
    if( newValue.length != value.length ) {
      $( "#ID_CALLNUMBER" + this.dynid ).edit( "option", "value", newValue );
    }

  }

  //===========================================================================
  //  STATIC 
  //===========================================================================

  static Show( options )
  {
    let createdata = $.extend( true, [], {
      view: AddressEntryDialog.template,
      type: 'dialog',
      layout: 'classic2',
      height: -1,
      size: CONST.dlgSize.M,
      top: CONST.dlgTop.S,
      buttons: [
        { id: 'IDCANCEL', size: 13, status: 'normal', text: "{$const_dialog_button_cancel}" },
        { id: 'IDOK', size: 13, status: 'disabled', text: "{$const_dialog_button_save}" },
      ],
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

AddressEntryDialog.Register( AddressEntryDialog.template );
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          28.05.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class KnxEntryDialog extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATICS
  //---------------------------------------------------------------------------

  static template = "/dialog/knxEntry";

  //---------------------------------------------------------------------------
  //  LOAD
  //
  Load( createOptions )
  {
    this.entry = createOptions['entry'] || {
      id: "knx_" + $.guidgen(),
      label: "",
      send: "",
      receive: ""
    };

    // initialize controls
    $( "#ID_LABEL" + this.dynid ).edit( {
      size: CONST.size.L,
      value: this.entry['label'],
      validate: Validate.NotEmptyString
    } );

    $( "#ID_GRPADDR_SEND" + this.dynid ).edit( {
      size: CONST.size.L,
      value: this.entry['send'],
      filter: { allow: "0123456789/", deny: null },
      validate: Validate.NotEmptyString
    } );

    $( "#ID_GRPADDR_RECV" + this.dynid ).edit( {
      size: CONST.size.L,
      value: this.entry['receive'],
      filter: { allow: "0123456789/", deny: null },
      //validate: Validate.NotEmptyString
    } );

    $( "#ID_LABEL" + this.dynid ).edit( "focus" );

  }//load

  //---------------------------------------------------------------------------
  //  VIEWABLE EVENTS
  //  
  OnEvent( eventName, data )
  {
    switch( eventName ) {
      // buttons
      case 'button':
      case 'canclose': {
        if( data == 'IDOK' || data === true  )  {
          this.CommandOk( );
        }
        else {
          this.CommandCancel();
        }
      }
      break;

      case 'modified': {
        this.$controls['IDOK'].button( 'option', 'status', ( data > 0 ) ? "hilight" : "disabled" );
      }
      break;
    }

    return true;
  }//onevent


  //===========================================================================
  //  FORM
  //===========================================================================

  FormReadControls()
  {
    this.entry['label'] = $( "#ID_LABEL" + this.dynid ).edit( "option", "value" );
    this.entry['send'] = $( "#ID_GRPADDR_SEND" + this.dynid ).edit( "option", "value" );
    this.entry['receive'] = $( "#ID_GRPADDR_RECV" + this.dynid ).edit( "option", "value" );
  }


  //===========================================================================
  //  VALIDATE
  //===========================================================================

  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_LABEL" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_GRPADDR_SEND" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_GRPADDR_RECV" + this.dynid ).edit( "validate" );
    return numErr;
  }


  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Button IDOK or <enter>
  // 
  CommandOk()
  {
    if( this.Validate() == 0 )  {
      this.FormReadControls();
      this.FireEvent( 'result', this.entry );
      this.Destroy();
    }
  }

  //---------------------------------------------------------------------------
  //  Button IDCANCEL or <esc>
  // 
  CommandCancel()
  {
    this.Destroy();
  }

  //===========================================================================
  //  STATIC 
  //===========================================================================

  static Show( options )
  {
    let createdata = $.extend( true, [], {
      view: KnxEntryDialog.template,
      type: 'dialog',
      layout: 'classic2',
      size: "30em",
      height: -1,
      top: CONST.dlgTop.L,
      buttons: [
        { id: 'IDCANCEL', size: 13, status: 'normal', text: "{$const_dialog_button_cancel}" },
        { id: 'IDOK', size: 13, status: 'disabled', text: "{$const_dialog_button_apply}" },
      ],
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

KnxEntryDialog.Register( KnxEntryDialog.template );
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          28.05.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class NtpEntryDialog extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATICS
  //---------------------------------------------------------------------------

  static template = "/dialog/ntpEntry";

  //---------------------------------------------------------------------------
  //  LOAD
  //
  Load( createOptions )
  {
    this.entry = createOptions['entry'] || {
      id: $.uuid(),
      text: ""
    };

    // initialize controls
    $( "#ID_IPADDRESS" + this.dynid ).edit( {
      size: CONST.size.L,
      value: this.entry['text'],
      validate: Validate.NotEmptyString
    } );
    $( "#ID_IPADDRESS" + this.dynid ).edit( "focus" );

  }//load

  //---------------------------------------------------------------------------
  //  VIEWABLE EVENTS
  //  
  OnEvent( eventName, data )
  {
    switch( eventName ) {
      // buttons
      case 'button':
      case 'canclose': {
        if( data == 'IDOK' || data === true  )  {
          this.CommandOk( );
        }
        else {
          this.CommandCancel();
        }
      }
      break;

      case 'modified': {
        this.$controls['IDOK'].button( 'option', 'status', ( data > 0 ) ? "hilight" : "disabled" );
      }
      break;
    }

    return true;
  }//onevent


  //===========================================================================
  //  FORM
  //===========================================================================

  FormReadControls()
  {
    this.entry['text'] = $( "#ID_IPADDRESS" + this.dynid ).edit( "option", "value" );
  }


  //===========================================================================
  //  VALIDATE
  //===========================================================================

  Validate()
  {
    let numErr = 0;
    return numErr;
  }


  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Button IDOK or <enter>
  // 
  CommandOk()
  {
    if( this.Validate() == 0 )  {
      this.FormReadControls();
      this.FireEvent( 'result', this.entry );
      this.Destroy();
    }
  }

  //---------------------------------------------------------------------------
  //  Button IDCANCEL or <esc>
  // 
  CommandCancel()
  {
    this.Destroy();
  }


  //===========================================================================
  //  STATIC 
  //===========================================================================

  static Show( options )
  {
    let createdata = $.extend( true, [], {
      view: NtpEntryDialog.template,
      type: 'dialog',
      layout: 'classic2',
      //size: "30em",
      //height: "16em",
      size: CONST.dlgSize.M,
      top: CONST.dlgTop.M,
      height: -1,
      buttons: [
        { id: 'IDCANCEL', size: 13, status: 'normal', text: "{$const_dialog_button_cancel}" },
        { id: 'IDOK', size: 13, status: 'disabled', text: "{$const_dialog_button_save}" },
      ],
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

NtpEntryDialog.Register( NtpEntryDialog.template );
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          28.05.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class MacAddressDialog extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATICS
  //---------------------------------------------------------------------------

  static template = "/dialog/macAddr";

  //---------------------------------------------------------------------------
  //  LOAD
  //
  Load( createOptions )
  {
    this.param = createOptions['param'];

    let presence = Data.Instance().PresenceData().Get( this.param['entid'] );
    this.isOffline = ( presence ) ? ( presence['presenceValue'] == "OFFLINE" ) : ( true );

    let resetData = [];

    $( "#ID_OPTIONAL" + this.dynid ).optional( { } );

    if( this.param['mode'] == "sus" )  {
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_MACADDR" );
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_USERPARAM" );
      $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_SUSREG" );
      resetData = [
        { id: 'calldetails', text: "{$macdialog_calllist_check}", value: false },
        { id: 'grabpic', text: "{$macdialog_grabpic_check}", value: false }
      ];
    }
    else {
      $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_MACADDR" );
      $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_USERPARAM" );
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_SUSREG" );
      resetData = [
        { id: 'unpairing', text: "{$macdialog_userparam_check}", value: false }, 
        { id: 'calldetails', text: "{$macdialog_calllist_check}", value: false },
        { id: 'grabpic', text: "{$macdialog_grabpic_check}", value: false }
      ];
    }

    // initialize controls
    $( "#ID_MACADDRESS" + this.dynid ).edit( {
      size: CONST.size.L,
      value: this.param['macaddr'],
      validate: this.ValidateMacAddess.bind( this ),
      result: ( value ) => {
        var newValue = "";
        for( var i = 0; i < value.length; i++ )  {
          switch( value[ i ] ) {
            case 'a':
            case 'A': newValue += "A";  break;
            case 'b':
            case 'B': newValue += "B";  break;
            case 'c':
            case 'C': newValue += "C";  break;
            case 'd':
            case 'D': newValue += "D";  break;
            case 'e':
            case 'E': newValue += "E";  break;
            case 'f':
            case 'F': newValue += "F";  break;
            case ':':
            case '-': newValue += ":";  break;
            case '0':case '1':case '2':case '3':case '4':
            case '5':case '6':case '7':case '8':case '9':
              newValue += value[ i ];  
              break;
          }
        }
        if( newValue != value ) {
          $( "#ID_MACADDRESS" + this.dynid ).edit( "option", "value", newValue );
        }
      },                  
      //result: ( value ) => {
      //  value = value.replaceAll( /-/gi,':' );
      //  $( "#ID_MACADDRESS" + this.dynid ).edit( "option", "value", value );
      //}
    } );

    if( !this.isOffline )  {
      $( "#ID_MACADDRESS" + this.dynid ).edit( "option", "status", "readonly" );
      $( "#ID_MACADDRESS" + this.dynid ).edit( "option", "warning", "{$macdialog_warning_online}" );
    }
    else {
      $( "#ID_MACADDRESS" + this.dynid ).edit( "focus" );
    }

    // initialize controls
    $( "#ID_SUSREG" + this.dynid ).choice( {
      size: CONST.size.L,
      type: "check",
      text: "{$macdialog_susreg_check}"
    } );

    // initialize controls
    $( "#ID_USERPARAM" + this.dynid ).choice( {
      size: CONST.size.L,
      type: "check",
      data: resetData,
    } );



  }//load

  //---------------------------------------------------------------------------
  //  VIEWABLE EVENTS
  //  
  OnEvent( eventName, data )
  {
    switch( eventName ) {
      // buttons
      case 'button':
      case 'canclose': {
        if( data == 'IDOK' || data === true  )  {
          this.CommandOk( );
        }
        else {
          this.CommandCancel();
        }
      }
      break;

      case 'modified': {
        this.$controls['IDOK'].button( 'option', 'status', ( data > 0 ) ? "hilight" : "disabled" );
      }
      break;
    }

    return true;
  }//onevent


  //===========================================================================
  //  FORM
  //===========================================================================

  FormReadControls()
  {
    var macaddr = $( "#ID_MACADDRESS" + this.dynid ).edit( "option", "value" );
    macaddr = macaddr.replaceAll( /-/gm, `:` );
    this.param['macaddr'] = macaddr;

    //this.param['macaddr'] = $( "#ID_MACADDRESS" + this.dynid ).edit( "option", "value" );
    this.param['reset'] = $( "#ID_USERPARAM" + this.dynid ).choice( "option", "value" );
    if( this.param['mode'] == "sus" ) {
      this.param['reset']['sus'] = $( "#ID_SUSREG" + this.dynid ).choice( "option", "value" );
    }
  }


  //===========================================================================
  //  VALIDATE
  //===========================================================================

  Validate()
  {
    let numErr = 0;
    if( this.isOffline )  {
      numErr += $( "#ID_MACADDRESS" + this.dynid ).edit( "validate" );
    }
    return numErr;
  }

  //-----------------------------------------------------------------
  //  validate the mac-address edit
  //
  ValidateMacAddess( validateControl )  
  {
    // empty mac-addresses are allowed
    if( validateControl.value.length == 0 )  {
      return 0;
    }

    if( ! Validate.IsMacAddress( validateControl.value ) )  {
      validateControl.error = "{$const_error_novalidmac}";
      return 1;
    }

    var macInfo = Data.Instance().MacAddressInfo( validateControl.value );
    if( macInfo && macInfo['entid'] != this.param['entid'] )  {
      validateControl.error = "{$macaddr_error_exists}";
      return 1;
    }
    return 0;
  }  

  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Button IDOK or <enter>
  // 
  CommandOk()
  {
    if( this.Validate() == 0 )  {
      this.FormReadControls();
      this.FireEvent( 'result', this.param );
      this.Destroy();
    }
  }

  //---------------------------------------------------------------------------
  //  Button IDCANCEL or <esc>
  // 
  CommandCancel()
  {
    this.Destroy();
  }


  //===========================================================================
  //  STATIC 
  //===========================================================================

  static Show( options )
  {
    let createdata = $.extend( true, [], {
      view: MacAddressDialog.template,
      type: 'dialog',
      layout: 'classic2',
      size: "30em",
      top: CONST.dlgTop.M,
      height: -1,
      buttons: [
        { id: 'IDCANCEL', size: 13, status: 'normal', text: "{$const_dialog_button_cancel}" },
        { id: 'IDOK', size: 13, status: 'disabled', text: "{$const_dialog_button_apply}" },
      ],
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

MacAddressDialog.Register( MacAddressDialog.template );
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          28.05.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class DeviceKeysDialog extends Viewable
{
  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  load dialog
  //
  Load( createData )
  {
    // ---- data ----
    this.entity = $.clone( createData['data'] );
    this.options = $.clone( createData['options'] );
    this.mode = "triggerSingle";
    this.trigger = "single";
    this.isLedTriggerPresent = false;
    this.currentShownWidget = null;
    if( 'deviceConfig' in createData['options'] ) {
      this.deviceConfig = createData['options']['deviceConfig'];
    }
    else {
      this.deviceConfig = this.entity['deviceConfig'];
    }

    // ---- form ----
    // Get title. On AVP the title of the button will be created on function name,
    // so we need to find firstDeviceConfigEntry to create a title like >Tile "Call"<.
    // In other cases the title will be created by the type and number of 
    // the key, like >Module 1, Key 2<
    let firstDeviceConfigRecord = this.deviceConfig.find( record => record['uid'] == this.options['uid'] );
    let keyTitle = DeviceKeyInfo.GetTitle( this.options['uid'], this.options['keyNumberIndex'], this.entity['deviceTemplate'], firstDeviceConfigRecord );
    $( '#ID_TITLE' + this.dynid ).text( keyTitle );

    // build list of usable widgets
    this.FormLoadSidebar( firstDeviceConfigRecord );
    this.FormFillSidebar( firstDeviceConfigRecord );

    // which trigger modes should be displayed? That depends on device (AHTx, ATLC, ..) 
    // and on the key type, wich can be indentified by key-uid.
    let allowedTriggers = DeviceKeyInfo.GetAllowedTriggers( this.options['uid'], this.entity['deviceTemplate'] );
    this.isLedTriggerPresent = ( allowedTriggers != null ) ? ( allowedTriggers.indexOf( "led" ) >= 0 ) : false;
    this.FormLoadTriggerButtons( allowedTriggers );

    // build the map which triggers are used
    this.triggerMap = {
      "null": false,
      "single": this.HelperIsTriggerUsed( this.options['uid'], "single" ),
      "double": this.HelperIsTriggerUsed( this.options['uid'], "double" ),
      "pos": this.HelperIsTriggerUsed( this.options['uid'], "pos" ),
      "neg": this.HelperIsTriggerUsed( this.options['uid'], "neg" ),
      "led": this.HelperIsTriggerUsed( this.options['uid'], "led" )
    };

    // ---- set first mode (first used trigger) ----
    if( allowedTriggers == null )  {
      this.ModeTriggerNull();
    }
    else if( this.triggerMap['single'] == true ) {
      this.ModeTriggerSingle();
    }
    else if( this.triggerMap['double'] == true ) {
      this.ModeTriggerDouble();
    }
    else if( this.triggerMap['pos'] == true ) {
      this.ModeTriggerKeyPos();
    }
    else if( this.triggerMap['neg'] == true ) {
      this.ModeTriggerKeyNeg();
    }
    else if( this.triggerMap['led'] == true ) {
      this.ModeTriggerLed();
    }
    else {
      // nothing used: change to single/double 
      this.ModeChangeTriggerPosNeg( true );
      this.ModeTriggerSingle();
    }

    // ---- subscribe to special bubble for tile label edit ----
    this.$container.on( "tile-label", ( ev, data ) => {
      ev.stopPropagation();
      this.FormUpdateTileLabel( data );
    } );


    // ---- 1. status: LoadValidation ----
    this.StatusIdle();


  }//load

  //-----------------------------------------------------------------
  //  load device key trigger buttons (single-click, ..., led)
  //  
  FormLoadTriggerButtons( allowedTriggers )
  {
    const keySize = 6;
    const keyHeight = 1.05;

    // indoor devices show key(s) and led
    $( "#ID_OPTIONAL" + this.dynid ).optional( {} );
    if( allowedTriggers == null || allowedTriggers.indexOf( "single" ) < 0 ) {
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTION_KEYSINGLE" );
    }
    if( allowedTriggers == null || allowedTriggers.indexOf( "double" ) < 0 ) {
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTION_KEYDOUBLE" );
    }
    if( allowedTriggers == null || allowedTriggers.indexOf( "pos" ) < 0 ) {
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTION_KEYPOS" );
    }
    if( allowedTriggers == null || allowedTriggers.indexOf( "neg" ) < 0 ) {
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTION_KEYNEG" );
    }
    if( allowedTriggers == null || allowedTriggers.indexOf( "led" ) < 0 ) {
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTION_LED" );
    }

    // create controls
    $( '#ID_BUTTON_KEYSINGLE' + this.dynid ).button( {
      size: keySize,
      height: keyHeight,
      status: 'normal',
      indicator: "normal selected",
      tooltip: "{$devkey_tooltip_keysingle}",
      icon: 'mdi mdi-gesture-tap',
      //click: this.ModeTriggerSingle.bind( this )
      click: () => {
        this.ControlTriggerSelect( "single" );
      }
    } );

    $( '#ID_BUTTON_LED' + this.dynid ).button( {
      size: keySize,
      height: keyHeight,
      status: 'normal',
      indicator: "normal",
      tooltip: "{$devkey_tooltip_led}",
      icon: 'mdi mdi-led-on',
      //click: this.ModeTriggerLed.bind( this )
      click: () =>  {
        this.ControlTriggerSelect( "led" );
      }

    } );

    $( '#ID_BUTTON_KEYDOUBLE' + this.dynid ).button( {
      size: keySize,
      height: keyHeight,
      status: "normal",
      indicator: "normal",
      tooltip: "{$devkey_tooltip_keydouble}",
      icon: 'mdi mdi-gesture-double-tap',
      //click: this.ModeTriggerDouble.bind( this )
      click: () =>  {
        this.ControlTriggerSelect( "double" );
      }
    } );

    $( '#ID_BUTTON_KEYPOS' + this.dynid ).button( {
      size: keySize,
      height: keyHeight,
      status: 'normal',
      indicator: "normal",
      tooltip: "{$devkey_tooltip_keypos}",
      icon: 'mdi mdi-format-vertical-align-bottom',
      //click: this.ModeTriggerKeyPos.bind( this )
      click: () =>  {
        this.ControlTriggerSelect( "pos" );
      }
    } );

    $( '#ID_BUTTON_KEYNEG' + this.dynid ).button( {
      size: keySize,
      height: keyHeight,
      status: 'normal',
      indicator: "normal",
      tooltip: "{$devkey_tooltip_keyneg}",
      icon: 'mdi mdi-format-vertical-align-top',
      //click: this.ModeTriggerKeyNeg.bind( this )
      click: () =>  {
        this.ControlTriggerSelect( "neg" );
      }
    } );

  }// FormLoadKeyButtons


  //-----------------------------------------------------------------
  //  load widget list into sidebar
  //  
  FormLoadSidebar( deviceRecord )
  {
    $( "#ID_SIDEBAR" + this.dynid ).listbox( {
      size: -1,
      height: 15,
      border: false,
      focus: false,
      type: 'listbox',
      result: this.ControlSidebarResult.bind( this )
    } );
  }

  //-----------------------------------------------------------------
  //  load widget list into sidebar
  //  
  FormFillSidebar( deviceRecord )
  {
    //console.log( deviceRecord );
    let widgetList = WidgetDeviceConfigFactory.AllowedWidgetList( deviceRecord, this.entity );
    let isReadOnly = WidgetDeviceConfigFactory.IsReadOnly( deviceRecord );
    if( isReadOnly )  {
      for( let widget of widgetList ) {
        widget['status'] = "disabled";
      }
    }
    $( "#ID_SIDEBAR" + this.dynid ).listbox( "option", "data", widgetList );

  }

  //-----------------------------------------------------------------
  //  set the status of all trigger buttons
  //  
  FormFillTriggerButtons( selectedTrigger )
  {
    // build map of trigger buttons status
    this.triggerMap = {
      "null": false,
      "single": this.HelperIsTriggerUsed( this.options['uid'], "single" ),
      "double": this.HelperIsTriggerUsed( this.options['uid'], "double" ),
      "pos": this.HelperIsTriggerUsed( this.options['uid'], "pos" ),
      "neg": this.HelperIsTriggerUsed( this.options['uid'], "neg" ),
      "led": this.HelperIsTriggerUsed( this.options['uid'], "led" )
    };

    $( "#ID_BUTTON_KEYSINGLE" + this.dynid ).button( "option", "indicator", this.triggerMap['single'] ? "yellow" : "normal" );
    $( "#ID_BUTTON_KEYDOUBLE" + this.dynid ).button( "option", "indicator", this.triggerMap['double'] ? "yellow" : "normal" );
    $( "#ID_BUTTON_KEYPOS" + this.dynid ).button( "option", "indicator", this.triggerMap['pos'] ? "yellow" : "normal" );
    $( "#ID_BUTTON_KEYNEG" + this.dynid ).button( "option", "indicator", this.triggerMap['neg'] ? "yellow" : "normal" );
    $( "#ID_BUTTON_LED" + this.dynid ).button( "option", "indicator", this.triggerMap['led'] ? "yellow" : "normal" );

    $( "#ID_BUTTON_KEYSINGLE" + this.dynid ).button( "option", "status", ( selectedTrigger == "single" ) ? "selected" : "normal" );
    $( "#ID_BUTTON_KEYDOUBLE" + this.dynid ).button( "option", "status", ( selectedTrigger == "double" ) ? "selected" : "normal" );
    $( "#ID_BUTTON_KEYPOS" + this.dynid ).button( "option", "status", ( selectedTrigger == "pos" ) ? "selected" : "normal" );
    $( "#ID_BUTTON_KEYNEG" + this.dynid ).button( "option", "status", ( selectedTrigger == "neg" ) ? "selected" : "normal" );
    $( "#ID_BUTTON_LED" + this.dynid ).button( "option", "status", ( selectedTrigger == "led" ) ? "selected" : "normal" );
   
  }

  //-----------------------------------------------------------------
  //  show or hide error on trigger
  //  
  FormShowErrorOnTrigger( trigger, isError )
  {
    switch( trigger )  {
      case "single":  $( "#ID_BUTTON_KEYSINGLE" + this.dynid ).button( "option", "error", isError ? "" : null );  break;
      case "double":  $( "#ID_BUTTON_KEYDOUBLE" + this.dynid ).button( "option", "error", isError ? "" : null );  break;
      case "pos":     $( "#ID_BUTTON_KEYPOS" + this.dynid ).button( "option", "error", isError ? "" : null );     break;
      case "neg":     $( "#ID_BUTTON_KEYNEG" + this.dynid ).button( "option", "error", isError ? "" : null );     break;
      case "led":     $( "#ID_BUTTON_LED" + this.dynid ).button( "option", "error", isError ? "" : null );        break;
    }
  }

  //-----------------------------------------------------------------
  //  load widget, use factory
  //  
  FormLoadWidget( widgetTemplate, uid, trigger, entity )
  {
    // reset error counter on exiting 
    if( this.currentShownWidget )  {
      $( '#ID_WIDGET_CONTENT' + this.dynid ).viewable( "validate", false );
    }

    if( ! WidgetDeviceConfigFactory.BuildUI( $( '#ID_WIDGET_CONTENT' + this.dynid ), widgetTemplate, uid, trigger, entity ) )  {
      widgetTemplate = "none";
      WidgetDeviceConfigFactory.BuildUI( $( '#ID_WIDGET_CONTENT' + this.dynid ), widgetTemplate, uid, trigger, entity );
    }

    setTimeout( () => {
      $( '#ID_WIDGET_CONTENT' + this.dynid ).viewable( "validate" );
    }, 25 );

    this.currentShownWidget = widgetTemplate;

  }

  //-----------------------------------------------------------------
  //  read out widget data into entity
  //  
  FormReadWidget( )
  {
    // check for viewable plugin
    let plugin = $( '#ID_WIDGET_CONTENT' + this.dynid ).data( "viewable" );
    if( !plugin ) {
      return;
    }

    // get controller class and call "Save" with the correct scope
    let controllerClass = plugin['viewable']();
    if( "Save" in controllerClass )  {
      let functionSave = controllerClass['Save'].bind( controllerClass );
      let result = functionSave();
      if( $.isPlainObject( result ) )
        this.entity = result;
    }
  
  }

  //-----------------------------------------------------------------
  //  on avp a changed label should be show in title bar
  //
  FormUpdateTileLabel( label )
  {
    // clone device record and update the (not yet saved) label to update title
    let firstDeviceConfigRecord = this.deviceConfig.find( record => record['uid'] == this.options['uid'] );
    firstDeviceConfigRecord['label'] = label;
    let keyTitle = DeviceKeyInfo.GetTitle( this.options['uid'], this.options['keyNumberIndex'], this.entity['deviceTemplate'], firstDeviceConfigRecord );
    
    // set control
    $( '#ID_TITLE' + this.dynid ).text( keyTitle );
  }

  
  //-----------------------------------------------------------------
  //  OVERRIDABLE EVENTS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  DIALOG EVENTS
  //  
  OnEvent( eventName, data )
  {
    switch( eventName ) {
      
      // buttons
      case 'button': {
        if( data == "IDCANCEL" ) {
          this.Destroy();
        }
        else if( data == "IDOK" && this.CommandSave( ) == true )  {
          this.Destroy();
        }
      }
      break;

      // keyboard: ENTER (true) and ESC (false)
      case 'canclose': {
        if( data == true )  {   // <ENTER>
          if( this.CommandSave() == true ) {
            return true;
          }
        }
      }
      break;

      case "error": {
        if( this.Option( "modified" ) > 0 ) {
          if( data > 0 ) this.StatusError();
          else this.StatusModified();
        }
        this.FormShowErrorOnTrigger( this.trigger, data > 0 );
      }
      break;

      case "modified": {
        if( this.Option( "error" ) == 0 ) {
          if( data > 0 ) this.StatusModified();
          else this.StatusIdle();
        }
        else {
          this.StatusError();
        }
      }
      break;

      case "tile-label": {
        this.FormUpdateTileLabel( data );
      }
      break;

    }

    return true;
  }

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  change widget in sidebar
  //
  ControlTriggerSelect( trigger )
  {
    if( trigger == this.trigger ) {
      return;
    }

    if( !this.HelperNeedChangePosNeg( trigger ) ) {
      $( "#ID_WIDGET_CONTENT" + this.dynid ).viewable( "validate" );
      if( this.Option( "error" ) > 0 ) {
        return;
      }
    }

    switch( trigger )  {
      case null:      this.ModeTriggerNull();     break;
      case "single":  this.ModeTriggerSingle();   break;
      case "double":  this.ModeTriggerDouble();   break;
      case "pos":     this.ModeTriggerKeyPos();   break;
      case "neg":     this.ModeTriggerKeyNeg();   break;
      case "led":     this.ModeTriggerLed();      break;
    }
  }


  //-----------------------------------------------------------------
  //  change widget in sidebar
  //
  ControlSidebarResult( selected  )
  {
    if( selected.length == 0 )
      return;

    let widgetId = selected[ 0 ];
    
    // 1. remove 
    WidgetDeviceConfigFactory.RemoveFromEntity( this.options['uid'], this.trigger, this.entity );

    // 2. create new widget
    this.FormLoadWidget( widgetId, this.options['uid'], this.trigger, this.entity );

    // 3. check LED widget (linked to a key?)
    if( this.isLedTriggerPresent )  {
      let ledDeviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.options['uid'], "led" );
      if( ! WidgetDeviceConfigFactory.UpdateLinked( ledDeviceRecord, this.entity ) ) {
        WidgetDeviceConfigFactory.RemoveFromEntity( this.options['uid'], "led", this.entity );
      }
      this.FormFillTriggerButtons( this.trigger );
    }

    // 4. modified
    this.Option( "modified", 1 );

  }


  //-----------------------------------------------------------------
  //  COMMANDS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  save: returns true if no error
  //
  CommandSave()
  {
    this.Validate();
    let errNum = this.Option( "error" );
    if( errNum == 0 ) {

      // read widget
      this.FormReadWidget();

      // update linked led widget
      if( this.isLedTriggerPresent ) {
        let ledDeviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.options['uid'], "led" );
        if( ! WidgetDeviceConfigFactory.UpdateLinked( ledDeviceRecord, this.entity ) ) {
          WidgetDeviceConfigFactory.RemoveFromEntity( this.options['uid'], "led", this.entity );
        }
      }

      this.FireEvent( 'result', this.entity );
    }
    return ( errNum == 0 );
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  validate current shown widget
  //
  Validate( )
  {
    let numErr = $( "#ID_WIDGET_CONTENT" + this.dynid ).viewable( "validate" );
    switch( this.trigger ) {
      case "single":  $( "#ID_BUTTON_KEYSINGLE" + this.dynid ).button( "option", "error", ( numErr > 0 ) ? "" : null ); break;
      case "double":  $( "#ID_BUTTON_KEYDOUBLE" + this.dynid ).button( "option", "error", ( numErr > 0 ) ? "" : null ); break;
      case "pos":     $( "#ID_BUTTON_KEYPOS" + this.dynid ).button( "option", "error", ( numErr > 0 ) ? "" : null );    break;
      case "neg":     $( "#ID_BUTTON_KEYNEG" + this.dynid ).button( "option", "error", ( numErr > 0 ) ? "" : null );    break;
      case "led":     $( "#ID_BUTTON_LED" + this.dynid ).button( "option", "error", ( numErr > 0 ) ? "" : null );       break;
    }
    return numErr;
  }


  //-----------------------------------------------------------------
  //  STATUS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  called after loadValidate, if there is no error
  //
  StatusIdle()
  {
    this.status = "statusIdle";
    this.$controls["IDOK"].button( "option", "status", "disabled" );
  }

  //-----------------------------------------------------------------
  //  called after loadValidate, if there is no error
  //
  StatusModified()
  {
    this.status = "statusModified";
    this.$controls[ "IDOK" ].button( "option", "status", "hilight" );
  }

  //-----------------------------------------------------------------
  //  if error, disable OK
  //
  StatusError()
  {
    this.status = "statusError";
    this.$controls["IDOK"].button( "option", "status", "disabled" );
  }

  //-----------------------------------------------------------------
  //  MODES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  no different trigger (e.g. IO)
  //
  ModeTriggerNull()
  {
    // ---- data ----
    this.mode = "triggerNull";
    this.trigger = null;  // everything else then single/double/pos/neg || led

    let deviceConfigIndex = this.HelperIndexOfDeviceConfig( this.options['uid'], null );
    let deviceConfigRecord = this.deviceConfig[ deviceConfigIndex ];
    let template = deviceConfigRecord['template'] || "none";

    // ---- load widget ----
    this.FormReadWidget( );
    this.FormFillSidebar( deviceConfigRecord, template );
    this.FormLoadWidget( template, this.options['uid'], null, this.entity );

    // ---- sidebar ----
    $( "#ID_SIDEBAR" + this.dynid ).treemenu( "option", "selected", template );

    // manage status of trigger buttons & load status
    this.FormFillTriggerButtons( null );

  }


  //-----------------------------------------------------------------
  //  mode single click selected
  //
  ModeTriggerSingle()
  {
    // change to "single/double" scheme (from "pos/neg")
    if( this.HelperNeedChangePosNeg( "single" ) ) {
      this.ModeChangeTriggerPosNeg( true );
    }
    else {
      // save just if not trigger changed 
      this.FormReadWidget();
    }

    // ---- data ----
    this.mode = "triggerSingle";
    this.trigger = "single";

    let deviceConfigIndex = this.HelperIndexOfDeviceConfig( this.options['uid'], "single" );
    let deviceConfigRecord = this.deviceConfig[ deviceConfigIndex ];
    let template = deviceConfigRecord['template'] || "none";

    // ---- load widget ----
    this.FormFillSidebar( deviceConfigRecord, template );
    this.FormLoadWidget( template, this.options['uid'], "single", this.entity );

    // ---- sidebar ----
    $( "#ID_SIDEBAR" + this.dynid ).treemenu( "option", "selected", template );

    // manage status of trigger buttons
    this.FormFillTriggerButtons( "single" );

  }

  //-----------------------------------------------------------------
  //  mode double click selected
  //
  ModeTriggerDouble()
  {
    // change to "single/double" scheme (from "pos/neg")
    if( this.HelperNeedChangePosNeg( "double" ) ) {
      this.ModeChangeTriggerPosNeg( true );
    }
    else {
      // save just if not trigger changed 
      this.FormReadWidget();
    }

    // ---- save ----
    // before changing the trigger save existing
    //this.FormReadWidget();
    //console.log( this.deviceConfig );

    // ---- data ----
    this.mode = "triggerDouble";
    this.trigger = "double";

    let deviceConfigIndex = this.HelperIndexOfDeviceConfig( this.options['uid'], "double" );
    let deviceConfigRecord = this.deviceConfig[ deviceConfigIndex ];
    let template = deviceConfigRecord['template'] || "none";

    // ---- load widget ----
    this.FormFillSidebar( deviceConfigRecord, template );
    this.FormLoadWidget( template, this.options['uid'], "double", this.entity );

    // ---- sidebar ----
    $( "#ID_SIDEBAR" + this.dynid ).treemenu( "option", "selected", deviceConfigRecord['template'] || "none" );

    // manage status of trigger buttons
    this.FormFillTriggerButtons( "double" );

  }

  //-----------------------------------------------------------------
  //  mode keydown selected
  //
  ModeTriggerKeyPos()
  {
    // change to "single/double" scheme (to "pos/neg")
    if( this.HelperNeedChangePosNeg( "pos" ) ) {
      this.ModeChangeTriggerPosNeg( false );
    }
    else {
      // save just if not trigger changed 
      this.FormReadWidget();
    }

    // ---- save ----
    // before changing the trigger save existing
    //this.FormReadWidget();

    // ---- data ----
    this.mode = "triggerPos";
    this.trigger = "pos";

    let deviceConfigIndex = this.HelperIndexOfDeviceConfig( this.options['uid'], "pos" );
    let deviceConfigRecord = this.deviceConfig[ deviceConfigIndex ];
    let template = deviceConfigRecord['template'] || "none";

    // ---- load widget ----
    this.FormFillSidebar( deviceConfigRecord, template );
    this.FormLoadWidget( template, this.options['uid'], "pos", this.entity );

    // ---- sidebar ----
    $( "#ID_SIDEBAR" + this.dynid ).treemenu( "option", "selected", deviceConfigRecord['template'] || "none" );

    // manage status of trigger buttons
    this.FormFillTriggerButtons( "pos" );

  }

  //-----------------------------------------------------------------
  //  mode keyup selected
  //
  ModeTriggerKeyNeg()
  {
    // change to "single/double" scheme (to "pos/neg")
    if( this.HelperNeedChangePosNeg( "neg" ) ) {
      this.ModeChangeTriggerPosNeg( false );
    }
    else {
      // save just if not trigger changed 
      this.FormReadWidget();
    }

    // ---- save ----
    // before changing the trigger save existing
    //this.FormReadWidget();

    // ---- data ----
    this.mode = "triggerKeyup";
    this.trigger = "neg";

    let deviceConfigIndex = this.HelperIndexOfDeviceConfig( this.options['uid'], "neg" );
    let deviceConfigRecord = this.deviceConfig[ deviceConfigIndex ];
    let template = deviceConfigRecord['template'] || "none";

    // ---- load widget ----
    this.FormFillSidebar( deviceConfigRecord, template );
    this.FormLoadWidget( template, this.options['uid'], "neg", this.entity );

    // ---- sidebar ----
    $( "#ID_SIDEBAR" + this.dynid ).treemenu( "option", "selected", deviceConfigRecord['template'] || "none" );

    // manage status of trigger buttons
    this.FormFillTriggerButtons( "neg" );

  }

  //-----------------------------------------------------------------
  //  mode led selected
  //
  ModeTriggerLed()
  {
    // ---- save ----
    // before changing the trigger save existing
    this.FormReadWidget();

    // ---- data ----
    this.mode = "triggerLed";
    this.trigger = "led";

    let deviceConfigIndex = this.HelperIndexOfDeviceConfig( this.options['uid'], "led" );
    let deviceConfigRecord = this.deviceConfig[ deviceConfigIndex ];
    let template = deviceConfigRecord['template'] || "none";

    // ---- load widget ----
    this.FormFillSidebar( deviceConfigRecord, template );
    this.FormLoadWidget( template, this.options['uid'], "led", this.entity );

    // ---- sidebar ----
    $( "#ID_SIDEBAR" + this.dynid ).treemenu( "option", "selected", deviceConfigRecord['template'] || "none" );

    // manage status of trigger buttons
    this.FormFillTriggerButtons( "led" );

  }

  //-----------------------------------------------------------------
  //  change trigger single/double <=> pos/neg
  //
  ModeChangeTriggerPosNeg( isSingle )
  {
    let deviceConfigIndex = -1;
    let deviceConfigRecord = null;

    // becomes "single/double"
    if( isSingle ) {

      // widgets are now "none" -> no error
      this.FormShowErrorOnTrigger( "pos", false );
      this.FormShowErrorOnTrigger( "neg", false );

      deviceConfigIndex = this.HelperIndexOfDeviceConfig( this.options['uid'], "pos" );
      if( deviceConfigIndex >= 0 ) {
        WidgetDeviceConfigFactory.RemoveFromEntity( this.options['uid'], "pos", this.entity );
        deviceConfigRecord = this.deviceConfig[ deviceConfigIndex ];
        deviceConfigRecord['trigger'] = "single";
      }

      deviceConfigIndex = this.HelperIndexOfDeviceConfig( this.options['uid'], "neg" );
      if( deviceConfigIndex >= 0 ) {
        WidgetDeviceConfigFactory.RemoveFromEntity( this.options['uid'], "neg", this.entity );
        deviceConfigRecord = this.deviceConfig[ deviceConfigIndex ];
        deviceConfigRecord['trigger'] = "double";
      }
    }
    // becomes "pos/neg"
    else {

      // widgets are now "none" -> no error
      this.FormShowErrorOnTrigger( "single", false );
      this.FormShowErrorOnTrigger( "double", false );

      deviceConfigIndex = this.HelperIndexOfDeviceConfig( this.options['uid'], "single" );
      if( deviceConfigIndex >= 0 ) {
        WidgetDeviceConfigFactory.RemoveFromEntity( this.options['uid'], "single", this.entity );
        deviceConfigRecord = this.deviceConfig[ deviceConfigIndex ];
        deviceConfigRecord['trigger'] = "pos";
      }

      deviceConfigIndex = this.HelperIndexOfDeviceConfig( this.options['uid'], "double" );
      if( deviceConfigIndex >= 0 ) {
        WidgetDeviceConfigFactory.RemoveFromEntity( this.options['uid'], "double", this.entity );
        deviceConfigRecord = this.deviceConfig[ deviceConfigIndex ];
        deviceConfigRecord['trigger'] = "neg";
      }

    }
  }//change trigger pos/neg

  
  //-----------------------------------------------------------------
  //  HELPER
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  check if current trigger is pos/neg
  //
  HelperNeedChangePosNeg( desiredTrigger )
  {
    let isTriggerPosNeg = ( this.HelperIndexOfDeviceConfig( this.options['uid'], "pos" ) >= 0 );
    let isDesiredTriggerPosNeg = ( desiredTrigger == "pos" || desiredTrigger == "neg" );
  
    return isTriggerPosNeg != isDesiredTriggerPosNeg;

    //return( this.HelperIndexOfDeviceConfig( this.options['uid'], "pos" ) >= 0 );
  }

  //-----------------------------------------------------------------
  //  find index of device record given uid and trigger (maybe null)
  //
  HelperIndexOfDeviceConfig( uid, trigger )
  {
    if( trigger == null )  {
      let index = this.deviceConfig.findIndex( ( record ) => ( record['uid'] == uid ) );
      return index;
    }
    else {
      let index = this.deviceConfig.findIndex( ( record ) => ( record['uid'] == uid  && record['trigger'] == trigger ) );
      return index;
    }
  }

  //-----------------------------------------------------------------
  //  check if trigger exists and is NOT "none"
  //
  HelperIsTriggerUsed( uid, trigger )
  {
    let template = "none";
    let index = this.HelperIndexOfDeviceConfig( uid, trigger );
    if( index >= 0 ) {
      let deviceConfigRecord = this.deviceConfig[ index ];
      template = deviceConfigRecord['template'] || "none";
    }

    return ( index >= 0 && template != "none" );
  }


  //-----------------------------------------------------------------
  //  STATIC 
  //-----------------------------------------------------------------

  static Show( options )
  {
    var createdata = $.extend( true, [], {
      view: '/dialog/deviceKeys',
      type: 'dialog',
      layout: 'classic2',
      size: "50rem",
      height: -1,
      top: CONST.dlgTop.L,
      buttons: [
        { id: 'IDCANCEL', size: 13, status: 'normal', text: '{$const_dialog_button_cancel}' },
        { id: 'IDOK', size: 13, status: 'hilight', text: '{$const_dialog_button_apply}' }
      ]
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

DeviceKeysDialog.Register( '/dialog/deviceKeys' );
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          28.05.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class DeviceFunctionDialog extends Viewable
{
  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  load dialog
  //
  Load( createData )
  {
    // ---- data ----
    this.entity = $.clone( createData['data'] );
    this.options = $.clone( createData['options'] );

    // set titel
    let keyTitle = WidgetDeviceConfigFactory.WidgetName( this.options['functionId'] );
    let icon = WidgetDeviceConfigFactory.WidgetIcon( this.options['functionId'] );
    $( '#ID_TITLE' + this.dynid ).text( keyTitle );
    $( '#ID_ICON' + this.dynid ).attr( "class", icon );

    // load widget
    this.FormLoadWidget( this.options['functionId'], "", "", this.entity );

    // wait until UI is created, then do load-validation
    setTimeout( () => {
      if( this.Validate() == 0 )  {
        this.ValidateOff();
      }  
    }, 100 );

    this.StatusIdle();
  }//load

  
  //-----------------------------------------------------------------
  //  load widget, use factory
  //  
  FormLoadWidget( widgetTemplate, uid, trigger, entity )
  {
    WidgetDeviceConfigFactory.BuildUI( $( '#ID_WIDGET_CONTENT' + this.dynid ), widgetTemplate, uid, trigger, entity );
  }

  //-----------------------------------------------------------------
  //  read out widget data into entity
  //  
  FormReadWidget( )
  {
    // check for viewable plugin
    let plugin = $( '#ID_WIDGET_CONTENT' + this.dynid ).data( "viewable" );
    if( !plugin ) {
      return false;
    }

    // get controller class and call "Save" with the correct scope
    let controllerClass = plugin['viewable']();
    if( "Save" in controllerClass )  {
      let functionSave = controllerClass['Save'].bind( controllerClass );
      let result = functionSave();
      if( $.isPlainObject( result ) ) {
        this.entity = result;
        return true;
      }
    }
  
    return false;
  }

  //-----------------------------------------------------------------
  //  on avp a changed label should be show in title bar
  //
  FormUpdateTileLabel( label )
  {
    // clone device record and update the (not yet saved) label to update title
    let firstDeviceConfigRecord = this.entity['deviceConfig'].find( record => record['uid'] == this.options['uid'] );
    firstDeviceConfigRecord['label'] = label;
    let keyTitle = DeviceKeyInfo.GetTitle( this.options['uid'], this.options['keyNumberIndex'], this.entity['deviceTemplate'], firstDeviceConfigRecord );
    
    // set control
    $( '#ID_TITLE' + this.dynid ).text( keyTitle );
  }

  
  //-----------------------------------------------------------------
  //  OVERRIDABLE EVENTS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  DIALOG EVENTS
  //  
  OnEvent( eventName, data )
  {
    switch( eventName ) {
      
      // buttons
      case 'button': {
        if( ( data == "IDCANCEL" )
         || ( data == "IDOK" && this.CommandSave() == true )
        ) {
          this.Destroy();
        }
      }
      break;

      // keyboard: ENTER (true) and ESC (false)
      case 'canclose': {
        if( data == true )  {   // <ENTER>
          if( this.CommandSave() == true ) {
            return true;
          }
        }
      }
      break;

      case "error": {
        if( this.Option( "modified" ) > 0 ) {
          if( data > 0 ) this.StatusError();
          else this.StatusModified();
        }
      }
      break;

      case "modified": {
        if( this.Option( "error" ) == 0 ) {
          if( data > 0 ) this.StatusModified();
          else this.StatusIdle();
        }
        else {
          this.StatusError();
        }
      }
      break;

      case "tile-label": {
        this.FormUpdateTileLabel( data );
      }
      break;

     
    }

    return true;
  }

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------


  //-----------------------------------------------------------------
  //  COMMANDS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  save: returns true if no error
  //
  CommandSave()
  {
    this.Validate();
    let errNum = this.Option( "error" );
    if( errNum == 0 && this.FormReadWidget( ) ) {
      this.FireEvent( 'result', this.entity );
    }
    return ( errNum == 0 );
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  validate current shown widget
  //
  Validate( )
  {
    return $( "#ID_WIDGET_CONTENT" + this.dynid ).viewable( "validate" );
  }

  //-----------------------------------------------------------------
  //  validate current shown widget
  //
  ValidateOff( )
  {
    $( "#ID_WIDGET_CONTENT" + this.dynid ).viewable( "validate", false );
  }



  //-----------------------------------------------------------------
  //  STATUS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  called after loadValidate, if there is no error
  //
  StatusIdle()
  {
    this.status = "statusIdle";
    this.$controls["IDOK"].button( "option", "status", "disabled" );
  }

  //-----------------------------------------------------------------
  //  called after loadValidate, if there is no error
  //
  StatusModified()
  {
    this.status = "statusModified";
    this.$controls[ "IDOK" ].button( "option", "status", "hilight" );
  }

  //-----------------------------------------------------------------
  //  if error, disable OK
  //
  StatusError()
  {
    this.status = "statusError";
    this.$controls["IDOK"].button( "option", "status", "disabled" );
  }

  //-----------------------------------------------------------------
  //  HELPER
  //-----------------------------------------------------------------



  //-----------------------------------------------------------------
  //  STATIC 
  //-----------------------------------------------------------------

  static Show( options )
  {
    let widgetHeight = WidgetDeviceConfigFactory.WidgetUiHeight( options['options']['functionId'] );
    if( widgetHeight < 9 )  {
      widgetHeight = 9;
    }

    var createdata = $.extend( true, [], {
      view: '/dialog/deviceFunction',
      type: 'dialog',
      layout: 'classic2',
      size: "29rem",
      height: -1, //( widgetHeight + 8 ) + "rem",
      top: "20%",
      buttons: [
        { id: 'IDCANCEL', size: 13, status: 'normal', text: '{$const_dialog_button_cancel}' },
        { id: 'IDOK', size: 13, status: 'hilight', text: '{$const_dialog_button_apply}' }
      ]
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

DeviceFunctionDialog.Register( '/dialog/deviceFunction' );
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          28.05.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class DeviceRuleDialog extends Viewable
{
  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  load dialog
  //
  Load( createData )
  {
    // ---- data ----
    this.entity = $.clone( createData['data'] );
    this.options = $.clone( createData['options'] );
    this.rule = Data.Instance().Rule( createData['options']['ruleId'] );

    // set titel
    let keyTitle = this.rule['label'] || "Rule Config";
    let icon = this.rule['icon'] || "mdi mdi-puzzle";
    $( '#ID_TITLE' + this.dynid ).text( keyTitle );
    $( '#ID_ICON' + this.dynid ).attr( "class", icon );

    // load rule UI
    this.FormLoadRuleUI( this.options['ruleId'], this.entity );

    // wait until UI is created, then do load-validation
    setTimeout( () => {
      if( this.Validate() == 0 )  {
        this.ValidateOff();
      }  
    }, 100 );

    this.StatusIdle();
  }//load

  
  //-----------------------------------------------------------------
  //  load widget, use factory
  //  
  FormLoadRuleUI( ruleId, entity )
  {
    return WidgetRuleGenericUI.Build( $( '#ID_WIDGET_CONTENT' + this.dynid ), { ruleId: ruleId, entity: entity } );
  }

  //-----------------------------------------------------------------
  //  read out widget data into entity
  //  
  FormReadRuleUI( )
  {
    $( '#ID_WIDGET_CONTENT' + this.dynid ).viewable( "option", "save", this.entity );
    return true;
  }


  
  //-----------------------------------------------------------------
  //  OVERRIDABLE EVENTS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  DIALOG EVENTS
  //  
  OnEvent( eventName, data )
  {
    switch( eventName ) {
      
      // buttons
      case 'button': {
        if( ( data == "IDCANCEL" )
         || ( data == "IDOK" && this.CommandSave() == true )
        ) {
          this.Destroy();
        }
      }
      break;

      // keyboard: ENTER (true) and ESC (false)
      case 'canclose': {
        if( data == true )  {   // <ENTER>
          if( this.CommandSave() == true ) {
            return true;
          }
        }
      }
      break;

      case "error": {
        if( this.Option( "modified" ) > 0 ) {
          if( data > 0 ) this.StatusError();
          else this.StatusModified();
        }
      }
      break;

      case "modified": {
        if( this.Option( "error" ) == 0 ) {
          if( data > 0 ) this.StatusModified();
          else this.StatusIdle();
        }
        else {
          this.StatusError();
        }
      }
      break;

    }

    return true;
  }

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------


  //-----------------------------------------------------------------
  //  COMMANDS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  save: returns true if no error
  //
  CommandSave()
  {
    this.Validate();
    let errNum = this.Option( "error" );
    if( errNum == 0 && this.FormReadRuleUI( ) ) {
      this.FireEvent( 'result', this.entity );
    }
    return ( errNum == 0 );
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  validate current shown widget
  //
  Validate( )
  {
    return $( '#ID_WIDGET_CONTENT' + this.dynid ).viewable( "validate" );
  }

  //-----------------------------------------------------------------
  //  validate current shown widget
  //
  ValidateOff( )
  {
    $( '#ID_WIDGET_CONTENT' + this.dynid ).viewable( "validate", false );
  }



  //-----------------------------------------------------------------
  //  STATUS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  called after loadValidate, if there is no error
  //
  StatusIdle()
  {
    this.status = "statusIdle";
    this.$controls["IDOK"].button( "option", "status", "disabled" );
  }

  //-----------------------------------------------------------------
  //  called after loadValidate, if there is no error
  //
  StatusModified()
  {
    this.status = "statusModified";
    this.$controls[ "IDOK" ].button( "option", "status", "hilight" );
  }

  //-----------------------------------------------------------------
  //  if error, disable OK
  //
  StatusError()
  {
    this.status = "statusError";
    this.$controls["IDOK"].button( "option", "status", "disabled" );
  }

  //-----------------------------------------------------------------
  //  HELPER
  //-----------------------------------------------------------------



  //-----------------------------------------------------------------
  //  STATIC 
  //-----------------------------------------------------------------

  static Show( options )
  {
    var createdata = $.extend( true, [], {
      view: '/dialog/deviceRule',
      type: 'dialog',
      layout: 'classic2',
      size: "29rem",
      height: -1, //( widgetHeight + 8 ) + "rem",
      top: "20%",
      buttons: [
        { id: 'IDCANCEL', size: 13, status: 'normal', text: '{$const_dialog_button_cancel}' },
        { id: 'IDOK', size: 13, status: 'hilight', text: '{$const_dialog_button_apply}' }
      ]
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

DeviceRuleDialog.Register( '/dialog/deviceRule' );
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          28.05.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class ChangeKeytoneDialog extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATICS
  //---------------------------------------------------------------------------

  static template = "/dialog/changeKeytoneDialog";

  //---------------------------------------------------------------------------
  //  LOAD
  //
  Load( createOptions )
  {
    this.entity = createOptions['param']['entity']
    this.entry  = createOptions['entry'] || {
      id:            createOptions['param']['guid'] ?? "none",
      oldCallNumber: createOptions['param']['callnumber'] ?? "",
    };

    // initialize controls
    $( "#ID_CALL_TARGET" + this.dynid ).edit( {
      size: CONST.size.L,
      value: this.entry['oldCallNumber'],
      filter: { allow: "0123456789*-()", deny: null },
      validate: Validate.NotEmptyString
    } );

    $( "#ID_KEYTONE_LIST" + this.dynid ).listbox( {
      size: CONST.size.L,
      validate: this.ValidateKeyTone.bind(this),
      selected: [ this.entry['id'] ],
      placeholder: "",
      type: "droplist"
    } );

    Data.Instance().Keytones().KeytoneRequest( ( param ) => {
      this.keytoneList = param;
      this.CreateListDataFromEntity( param );
    } );

    $( "#ID_CALL_:_" + this.dynid ).edit( "focus" );

    this.preventFirstTimeValidate = false;
  }//load

  //---------------------------------------------------------------------------
  //  VIEWABLE EVENTS
  //  
  OnEvent( eventName, data )
  {
    switch( eventName ) {
      // buttons
      case 'button':
      case 'canclose': {
        if( data == 'IDOK' || data === true  )  {
          this.CommandOk( );
        }
        else {
          this.CommandCancel();
        }
      }
      break;

      case 'modified': {
        this.$controls['IDOK'].button( 'option', 'status', ( data > 0 ) ? "hilight" : "disabled" );
      }
      break;
    }

    return true;
  }//onevent


  //===========================================================================
  //  FORM
  //===========================================================================

  FormReadControls()
  {
    let selected = $( "#ID_KEYTONE_LIST" + this.dynid ).listbox( "option", "selected" );
    this.entry['callnumber'] = $( "#ID_CALL_TARGET" + this.dynid ).edit( "option", "value" );
    this.entry['id'] = selected[0];
  }


  //===========================================================================
  //  VALIDATE
  //===========================================================================

  Validate()
  {
    let numErr = 0;
    numErr += $( "#ID_CALL_TARGET" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_KEYTONE_LIST" + this.dynid ).listbox( "validate" );
    return numErr;
  }


  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Button IDOK or <enter>
  // 
  CommandOk()
  {
    if( this.Validate() == 0 )  {
      this.FormReadControls();
      this.FireEvent( 'result', this.entry );
      this.Destroy();
    }
  }

  //---------------------------------------------------------------------------
  //  Button IDCANCEL or <esc>
  // 
  CommandCancel()
  {
    this.Destroy();
  }

  //---------------------------------------------------------------------------
  //  replace special characters that can't be filtered
  //
  ControlCallnumberFilter( value )
  {
    let newValue = value;
    newValue = newValue.replace( "^", "" );
    newValue = newValue.replace( "`", "" );
    newValue = newValue.replace( "´", "" );
    if( newValue.length != value.length ) {
      $( "#ID_KEYTONE_LIST" + this.dynid ).edit( "option", "value", newValue );
    }

  }

  //===========================================================================
  //  STATIC 
  //===========================================================================

  static Show( options )
  {
    let createdata = $.extend( true, [], {
      view:    ChangeKeytoneDialog.template,
      type:    'dialog',
      layout:  'classic2',
      height:  -1,
      size:    CONST.dlgSize.M,
      top:     CONST.dlgTop.S,
      buttons: [
        { id: 'IDCANCEL', size: 13, status: 'normal', text: "{$const_dialog_button_cancel}" },
        { id: 'IDOK', size: 13, status: 'disabled', text: "{$const_dialog_button_save}" },
      ],
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }


  //-----------------------------------------------------------------------------
  //  CreateListDataFromEntity(): description
  //
  CreateListDataFromEntity( keytones )
  {
    let listboxData =  [];
    //let keytones = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, "tlckeytone" );

    for( let keytone of keytones )  {
      let keytoneGuid = keytones[ "guid" ];
      listboxData.push( {
        id: keytone['guid'],
        text: keytone['label'],
        icon: "mdi mdi-music"
      } )
    }
    $( "#ID_KEYTONE_LIST" + this.dynid ).listbox( "option", "data", listboxData );
  }

  ValidateKeyTone( validateControl )
  {
    let selected = $("#ID_KEYTONE_LIST" + this.dynid ).listbox( "option", "selected" );
    if( selected.length === 0 || selected[0] === false || selected[0] === "none" )  {
      validateControl.error = "{$change_keytone_dialog_tone_must_selected}"
      return 1
    }
    return 0;
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

ChangeKeytoneDialog.Register( ChangeKeytoneDialog.template );
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          18.08.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class ScandeviceDialog extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATICS
  //---------------------------------------------------------------------------

  static template = "/dialog/scandevice";

  //---------------------------------------------------------------------------
  //  LOAD
  //
  Load( createOptions )
  {
    this.mode = "modeUnknown";
    this.entry = null;
    this.currentDevice = createOptions['current'] || null;

    // text while searching
    var placeholderText = "{$scandevice_status_search_unknown}";
    if( createOptions['deviceType'] == "KNX" ) {
      placeholderText = "{$scandevice_status_searchknx}";
      this.mode = "modeKnx";
    }
    else if( createOptions['deviceType'] == "IPCAM" ) {
      placeholderText = "{$scandevice_status_searchipcam}";
      this.mode = "modeIpCam";
    }

    this.isOnlyUnconfigured = true;
    $( "#ID_CHECK" + this.dynid ).choice( {
      type: "check",
      size: CONST.size.L,
      value: this.isOnlyUnconfigured,
      text: "{$scandevice_check_showunconfigured}",
      result: this.ControlCheckResult.bind( this )
    } );
    

    // build up grid for results
    this.gridDataset = new Dataset( { id: "ipaddress" } );
    $( "#ID_LIST" + this.dynid ).grid( {
      size: -1,
      height: 8,
      data: this.gridDataset,
      dataFilter: this.ControlGridDataviewFilter.bind( this ),
      scrollbar: 'overlay',
      selection: "single",
      placeholder: placeholderText,
      placeholderIcon: "mdi mdi-autorenew mdi-spin",
      header: {
        name: { text: "{$scandevice_grid_name}", size: 40 },
        ipaddress: { text: "{$scandevice_grid_ip}", size: 25 }
      },
      result: ( id ) => {
        this.StatusModified();
      },
      renderStatus: ( id, record ) => {
        if( record['configured'] == "configured" )  return "disabled";
        return "normal";
      },
      renderCell: ( id, record ) => {
        if( id == "name" ) {
          var icon = ( record['configured'] != "new" ) ? "mdi mdi-check-bold" : "mdi mdi-star";
          return { icon: icon, text: record['name'] };
        }
        else {
          return { text: record[ id ] };
        }
      },
      });


    Data.Instance().On( "SCANDEVICE/UPDATE", this.etag, this.DataResponseScandevice.bind( this ) );
    this.DataRequest();

    this.scanTimeoutId = setTimeout( () => {
      this.FormFillNotFound();
    }, 20000 );

  }//load

  //---------------------------------------------------------------------------
  //  VIEWABLE EVENTS
  //  
  OnEvent( eventName, data )
  {
    switch( eventName ) {
      
      // buttons
      case 'button': {
        if( ( data == "IDCANCEL" )
         || ( data == "IDOK" && this.CommandSave() === true )
          ) {
          this.Destroy();
        }
      }
      break;

      // keyboard: ENTER (true) and ESC (false)
      case 'canclose': {
        if( data === true ) {   // <ENTER>
          return ( this.CommandSave() === true );
        }
      }
      break;
          
    }

    return true;
  }//onevent


  //===========================================================================
  //  DATA
  //===========================================================================
  
  DataRequest()
  {
    if( this.mode == "modeKnx" ) {
      Data.Instance().ScanDevice( "KNX" );
    }
    else if( this.mode == "modeIpCam" ) {
      Data.Instance().ScanDevice( "IPCAM" );
    }
  }

  DataResponseScandevice( response )
  {
    clearTimeout( this.scanTimeoutId );

    console.log( response );

    $( "#ID_LIST" + this.dynid ).grid( "option", "placeholderIcon", null );
    $( "#ID_LIST" + this.dynid ).grid( "option", "placeholder", "{$scandevice_status_notfound}" );

    if( response['devicetype'] == "KNX" ) {
      this.FormFillKnxDeviceList( response['data'] );
    }
    else if( response['devicetype'] == "IPCAM" ) {
      this.FormFillIpCamList( response['data'] );
    }
    else {
      this.FormFillNotFound();
    }
  }

  //===========================================================================
  //  FORM FUNCTIONS
  //===========================================================================

  //-----------------------------------------------------------------
  //  put received knx devices
  //  
  FormFillKnxDeviceList( deviceList )
  {
    var list = [];
    for( var key in deviceList ) {

      var ipaddr = deviceList[ key ]['ip']
      var entityId = Data.Instance().EntityIdFromIp( ipaddr );
      var isConfigured = ( entityId != null );
      var isCurrent = ( this.currentDevice == ipaddr );
      var status = isCurrent ? "current" : isConfigured ? "configured" : "new";
      list.push( {
        name: deviceList[ key ]['name'],
        ipaddress: ipaddr,
        configured: status,
        data: deviceList[ key ]
      } );   
    }
    this.gridDataset.SetData( list );
    this.gridDataset.RebuildAllViews();      

  }

  //-----------------------------------------------------------------
  //  put received knx devices
  //  
  FormFillIpCamList( deviceList )
  {
    var list = [];
    for( var record of deviceList ) {

      // filter out some error records
      if( !( 'url' in record ) )
        continue;

      // multiple ip's are possible
      var ipList = [];
      for( var urlRecord of record['url'] ) {
        var url = new URL( urlRecord );
        var ipaddr = url.host;
        if( !url.port )  {
          ipaddr += ":80";
        }
        // filter optional https
        if( url.protocol == "https:" ) {
          continue;
        }
        // filter unusable default ip address
        if( url.host.substr( 0, 7 ) == "169.254" ) {
          continue;
        }

        // if not ipv6, add to list
        if( ipaddr.substring( 0, 1 ) != "[" ) {

          var entityId = Data.Instance().EntityIdFromIp( ipaddr );
          var isConfigured = ( entityId != null );
          var isCurrent = ( this.currentDevice == ipaddr );
          var status = isCurrent ? "current" : isConfigured ? "configured" : "new";
          ipList.push( {
            ip: ipaddr,
            current: isCurrent,
            status: status,
            url: urlRecord,
            entityId: entityId
          } );
        }
      }

      for( var index = 0; index < ipList.length; index++ )  {

        var ipaddrRecord = ipList[ index ];
        var name = record['name'];
        if( ipList.length > 1 )  {
          name += " (IP #" + ( index + 1 ) + ")";
        }

        var data = record;
        data['url'] = ipaddrRecord['url'];
        delete data['location'];

        list.push( {
          name: name,
          ipaddress: ipaddrRecord['ip'],
          configured: ipaddrRecord['status'],
          data: data
        } );        
      }

    }

    this.gridDataset.SetData( list );
    this.gridDataset.RebuildAllViews();      

  }

  //-----------------------------------------------------------------
  //  no devices had been found
  //  
  FormFillNotFound()
  {
    $( "#ID_LIST" + this.dynid ).grid( "option", "placeholderIcon", null );
    $( "#ID_LIST" + this.dynid ).grid( "option", "placeholder", "{$scandevice_status_notfound}" );
  }  

  FormReadControls()
  {
    let selected = $( "#ID_LIST" + this.dynid ).grid( "option", "selected" );
    let record = $( "#ID_LIST" + this.dynid ).grid( "item", selected[ 0 ], "get" );
    this.entry = record['data'];
  }

  //===========================================================================
  //  CONTROL
  //===========================================================================

  ControlCheckResult( value )
  {
    if( value != this.isOnlyUnconfigured )  {
      this.isOnlyUnconfigured = value;
      this.gridDataset.RebuildAllViews();      
    }
  }

  ControlGridDataviewFilter( id, record, options, dataviewClass )
  {
    if( ! this.isOnlyUnconfigured ) {
      return true;
    }
    return record['configured'] != "configured";
  }

  //===========================================================================
  //  STATUS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  called after loadValidate, if there is no error
  //
  StatusIdle()
  {
    this.status = "statusIdle";
    this.$controls["IDOK"].button( "option", "status", "disabled" );
  }

  //---------------------------------------------------------------------------
  //  called after loadValidate, if there is no error
  //
  StatusModified()
  {
    this.status = "statusModified";
    this.$controls["IDOK"].button( "option", "status", "default" );
  }

  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Button IDOK or <enter>
  // 
  CommandSave()
  {
    this.FormReadControls();
    this.FireEvent( 'result', this.entry );
    this.Destroy();
  }

  //---------------------------------------------------------------------------
  //  Button IDCANCEL or <esc>
  // 
  CommandCancel()
  {
    this.Destroy();
  }


  //===========================================================================
  //  STATIC 
  //===========================================================================

  static Show( options )
  {
    let createdata = $.extend( true, [], {
      view: ScandeviceDialog.template,
      type: 'dialog',
      layout: 'classic2',
      size: "35em",
      height: -1,
      top: CONST.dlgTop.L,
      buttons: [
        { id: 'IDCANCEL', size: 13, status: 'normal', text: "{$const_dialog_button_cancel}" },
        { id: 'IDOK', size: 13, status: 'disabled', text: "{$const_dialog_button_apply}" }
      ],
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

ScandeviceDialog.Register( ScandeviceDialog.template );
//=============================================================================
//
//  PAGE HEADER view
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          18.08.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class IpCamCheckDialog extends Viewable
{
  //---------------------------------------------------------------------------
  //  STATICS
  //---------------------------------------------------------------------------

  static template = "/dialog/ipcamCheck";

  //---------------------------------------------------------------------------
  //  LOAD
  //
  Load( createOptions )
  {
    this.entry = createOptions['data'];

    $( "#ID_IPADDRESS" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly",
      value: this.entry['ipaddress'],
    } );      

    $( "#ID_USERNAME" + this.dynid ).edit( {
      size: CONST.size.L,
      value: this.entry['user'],
      change: this.ControlEditChanged.bind( this )
    } );      

    $( "#ID_PASSWORD" + this.dynid ).edit( {
      size: CONST.size.L,
      value: "",
      password: true,
      change: this.ControlEditChanged.bind( this )
    } );      

    Data.Instance().On( "VIDEO_INFO/UPDATE", this.etag, this.DataResponseVideoInfo.bind( this ) );

    if( !this.entry['user'] || this.entry['user'] == "" ) {
      $( "#ID_USERNAME" + this.dynid ).edit( "focus" );
    }
    else {
      $( "#ID_PASSWORD" + this.dynid ).edit( "focus" );
    }
 

  }//load

  //---------------------------------------------------------------------------
  //  VIEWABLE EVENTS
  //  
  OnEvent( eventName, data )
  {
    switch( eventName ) {
      
      // buttons
      case 'button': {
        if( ( data == "IDCANCEL" )
         || ( data == "IDOK" && this.CommandOk() === true )
          ) {
          this.Destroy();
        }
      }
      break;

      // keyboard: ENTER (true) and ESC (false)
      case 'canclose': {
        if( data === true ) {   // <ENTER>
          return ( this.CommandOk() === true );
        }
      }
      break;

      case 'modified': {
        this.StatusModified();
      }
      break;

      default:
        console.log( "OnEvent", eventName, data );
          
    }

    return true;
  }//onevent


  //===========================================================================
  //  DATA
  //===========================================================================
  
  //---------------------------------------------------------------------------
  //  get VideoInfo from server
  //
  DataRequestVideoInfo( ipaddressPort, user, password )
  {
    // change status
    this.StatusCameraInfo();
    this.requestedIpAddress = ipaddressPort;

    var url = "http://" + ipaddressPort + "/onvif/device_service";
    Data.Instance().CameraInfo( url, user, password );
    this.scanTimeoutId = setTimeout( () => {
      this.FormShowError( -3, "no response" );
    }, 20000 );
  }

  //---------------------------------------------------------------------------
  //  server reponds video info
  //
  DataResponseVideoInfo( response )
  {
    // get ipaddress from response
    var url = new URL( response['url'] );
    var ipaddress = url.host;
    if( !url.port ) {
      ipaddress += ":80";
    }

    // if the reponse is for us (reponses are broadcasts)
    if( this.requestedIpAddress == ipaddress )  {
      clearTimeout( this.scanTimeoutId );

      if( response['error'] )  {
        this.FormShowError( response['errorcode'], response['errordescription'] );
      }
      else {
        this.FormReadControls( );
        this.entry['data'] = response['data'];
        this.FireEvent( 'result', this.entry );
        this.Destroy();
      }
    }


  }

  //===========================================================================
  //  FORM FUNCTIONS
  //===========================================================================

  FormReadControls( )
  {
    this.entry['user'] = $( "#ID_USERNAME" + this.dynid ).edit( "option", "value" );
    this.entry['password'] = $( "#ID_PASSWORD" + this.dynid ).edit( "option", "value" );
  }

  FormShowError( errorCode, description )
  {
    $( "#ID_RESPONSE" + this.dynid ).text( description );
  }  


  //===========================================================================
  //  CONTROL
  //===========================================================================

  ControlEditChanged( )
  {
    $( "#ID_RESPONSE" + this.dynid ).text( "" );
    this.StatusModified();
  }


  //===========================================================================
  //  STATUS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  called after loadValidate, if there is no error
  //
  StatusIdle()
  {
    this.status = "statusIdle";
    this.$controls["IDOK"].button( "option", "status", "disabled" );
  }

  //---------------------------------------------------------------------------
  //  called after loadValidate, if there is no error
  //
  StatusModified()
  {
    this.status = "statusModified";
    this.$controls["IDOK"].button( "option", "status", "default" );

  }

  //---------------------------------------------------------------------------
  //  called after CameraInfo is requested
  //
  StatusError()
  {
    this.status = "statusError";
    this.$controls["IDOK"].button( "option", "status", "disabled" );
  }  

  //---------------------------------------------------------------------------
  //  called after CameraInfo is requested
  //
  StatusCameraInfo()
  {
    this.status = "statusCameraInfo";
    this.$controls["IDOK"].button( "option", "status", "disabled" );
  }  

  //===========================================================================
  //  COMMANDS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  Button IDOK or <enter>
  // 
  CommandOk()
  {
    this.FormReadControls();
    this.DataRequestVideoInfo( this.entry['ipaddress'], this.entry['user'], this.entry['password'] );

  }

  //---------------------------------------------------------------------------
  //  Button IDCANCEL or <esc>
  // 
  CommandCancel()
  {
    this.Destroy();
  }


  //===========================================================================
  //  STATIC 
  //===========================================================================

  static Show( options )
  {
    let createdata = $.extend( true, [], {
      view: IpCamCheckDialog.template,
      type: 'dialog',
      layout: 'classic2',
      size: "27em",
      height: -1,// 19,
      top: CONST.dlgTop.L,
      buttons: [
        { id: 'IDCANCEL', size: 13, status: 'normal', text: "{$const_dialog_button_cancel}" },
        { id: 'IDOK', size: 13, status: 'disabled', text: "{$ipcam_button_login}" }
      ],
    }, options );
    $( '#ID_BODY_DIALOG' ).viewable( createdata );
  }

}


//=============================================================================
//  BUILDER
//=============================================================================

IpCamCheckDialog.Register( IpCamCheckDialog.template );
﻿//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.01.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class HeaderBar extends Viewable
{
  //-------------------------------------------------------
  //  LOAD 
  //
  Load( data )
  {
    //-------------------------------------------
    //  Data
    //-------------------------------------------

    this.searchString = "";


    // --- build controls ---
    $( "#ID_SSS" ).click( this.OnClickSSSLogo.bind( this ) );

    $( "#ID_HEADER_SEARCH" ).edit( {
      size: 33,
      status: "default",
      border: false,
      focus: true,
      icon: "mdi mdi-magnify",
      placeholder: "{$page_header_search_placeholder}",
      result: this.ControlSearchResult.bind( this ),
      click: this.ControlSearchClick.bind( this )
    } );

    $( "#ID_HEADER_ACCOUNT" ).button( {
      size: 5,
      height: 1.1,
      border: false,
      type: "icon",
      icon: "mdi mdi-account-circle",
      click: this.ControlAccountClick.bind( this )
    } );

    $( "#ID_HEADER_ACCOUNT_BADGE" ).badge( {
      anchor: "#ID_HEADER_ACCOUNT",
      icon: "mdi mdi-menu-down",
    } );

    $( "#ID_HEADER_STATUS" ).button( {
      size: 5,
      height: 1.1,
      border: false,
      type: "icon",
      tooltip: "{$page_header_warning_websocket}",
      icon: "mdi mdi-none"
    } );

    Verify.Instance().On( "VERIFY/ERRORSTATUS/*", this.etag, this.DataVerifyErrorstatus.bind( this ) );

  }//load

  //-------------------------------------------------------
  //  viewable event function
  //
  OnEvent( eventName, param )
  {
    if( eventName == "option" && param == "search" )  {
      $( "#ID_HEADER_SEARCH" ).edit( "focus" );
    }
    if( eventName == "option" && param == "close-search" )  {
      $( "#ID_HEADER_SEARCH" ).edit( "option", "value", "" );
      $( '#ID_HEADER_SEARCH' ).edit( "option", "icon", "mdi mdi-magnify" );
    }
  }

  //---------------------------------------------------------------------------
  //  DATA
  //---------------------------------------------------------------------------

  DataVerifyErrorstatus( param )
  {
    switch( param.numErr )  {
      case 0: $( "#ID_HEADER_ACCOUNT_BADGE" ).badge( "option", "icon", "mdi mdi-menu-down" );  break;
      case 1: $( "#ID_HEADER_ACCOUNT_BADGE" ).badge( "option", "icon", "red-text mdi mdi-numeric-1-circle" );  break;
      case 2: $( "#ID_HEADER_ACCOUNT_BADGE" ).badge( "option", "icon", "red-text mdi mdi-numeric-2-circle" );  break;
      case 3: $( "#ID_HEADER_ACCOUNT_BADGE" ).badge( "option", "icon", "red-text mdi mdi-numeric-3-circle" );  break;
      case 4: $( "#ID_HEADER_ACCOUNT_BADGE" ).badge( "option", "icon", "red-text mdi mdi-numeric-4-circle" );  break;
      case 5: $( "#ID_HEADER_ACCOUNT_BADGE" ).badge( "option", "icon", "red-text mdi mdi-numeric-5-circle" );  break;
      case 6: $( "#ID_HEADER_ACCOUNT_BADGE" ).badge( "option", "icon", "red-text mdi mdi-numeric-6-circle" );  break;
      case 7: $( "#ID_HEADER_ACCOUNT_BADGE" ).badge( "option", "icon", "red-text mdi mdi-numeric-7-circle" );  break;
      case 8: $( "#ID_HEADER_ACCOUNT_BADGE" ).badge( "option", "icon", "red-text mdi mdi-numeric-8-circle" );  break;
      case 9: $( "#ID_HEADER_ACCOUNT_BADGE" ).badge( "option", "icon", "red-text mdi mdi-numeric-9-circle" );  break;
      default: $( "#ID_HEADER_ACCOUNT_BADGE" ).badge( "option", "icon", "red-text mdi mdi-numeric-9-plus-circle" );  break;
    }
  }
  
  //-------------------------------------------------------
  //  click on Siedle logo
  //
  OnClickSSSLogo()
  {
    //Data.Instance().CameraInfo( "http://192.168.100.133:8000/onvif/device_service", "admin", "Abcd1234" );
    // Data.Instance().CameraInfo( "http://192.168.0.102:8000/onvif/device_service", "admin", "Abcd1234" );
    //ScandeviceDialog.Show( { deviceType: "IPCAM", current: "192.168.100.117:820" } );
  }

  //-------------------------------------------------------
  //  entered text in search bar
  //
  ControlSearchResult( value )
  {
    if( value.length > 0 ) {

      // If search value was changed, (re)start search.
      if ( value != this.searchString )  {

        $( '#ID_HEADER_SEARCH' ).edit( "option", "icon", "mdi mdi-close" );
        Search.Instance().Start( value );
        PageMain.ShowEntity( "search" );
        this.searchString = value;
      }
    }
    else {
      // If search string was removed, reset the control and the form,
      // but don't close the form(?).
      $( '#ID_HEADER_SEARCH' ).edit( "option", "value", "" );
      $( '#ID_HEADER_SEARCH' ).edit( "option", "icon", "mdi mdi-magnify" );
      this.searchString = "";
      Search.Instance().Stop( );
      Search.Instance().Clear( );
    }

  }//controlsearchresult

  //-------------------------------------------------------
  //  Search control's icon was clicked.
  //
  ControlSearchClick( value )
  {
    // If X icon was clicked, close everything and reset control and icon.
    if( $( '#ID_HEADER_SEARCH' ).edit( "option", "icon" ) == "mdi mdi-close" ) {
      $( '#ID_HEADER_SEARCH' ).edit( "option", "value", "" );
      $( '#ID_HEADER_SEARCH' ).edit( "option", "icon", "mdi mdi-magnify" );
      Search.Instance().Stop( );
      PageMain.CloseEntity( "search" );
      this.searchString = "";
    }
  
  }//controlsearchclick

  //-------------------------------------------------------
  //  Account icon was clicked.
  //
  ControlAccountClick( )
  {
    $( "#ID_HEADER_ACCOUNT_MENU" ).popupmenu( {
      anchor: "#ID_HEADER_ACCOUNT",
      position: "below-right",
      data: [
        { id: "ID_MENU_DASHBOARD", icon: "mdi mdi-gauge", text: "{$page_menu_dashboard}" },
        { id: "ID_MENU_SETTINGS", icon: "adi adi-siedle-admin", text: "{$page_menu_settings}" },
        { id: "-" },
        { id: "ID_MENU_EXIT", icon: "mdi mdi-exit-to-app", text: "{$page_menu_exit}" },
      ],
      click: ( id ) => {
        if( id == "ID_MENU_DASHBOARD" ) {
          PageMain.ShowEntity( "dashboard" );
        }
        else if( id == "ID_MENU_SETTINGS") {
          var user = ConnectStatus.AuthorizedEntid();
          PageMain.ShowEntity( user );
        }
        else if( id == "ID_MENU_EXIT" ) {
          AccessApplication.Instance().Command( 'exit' );
        }
      }
    });
  }

}


//=============================================================================
//  BUILDER 
//=============================================================================

HeaderBar.Register( '/part/header' );
﻿//=============================================================================
//
//  SIDEBAR TREE view
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          28.04.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class SidebarTree extends Viewable
{
  //=================================================================
  //  member variables
  //=================================================================

  localInfo = {};

  //=================================================================
  //  init
  //=================================================================

  //-----------------------------------------------------------------
  //  called by viewable during load
  //
  Load( data )
  {

    //--- 'General' tree ---
    // TODO: {$...} don't work here!?
    this.generalTree = new Treeset( { id: 'id' } );
    this.generalTree.SetData( CONST.sidebarGeneral );


    //--- CONTROLS ---    
    $( "#ID_ACCORDION" + this.dynid ).accordion( {
      size: -1, //50,
      height: -1, //75,
      border: false,   // <- HINT: set to 'false' in sidebar
      regkey: "sidebar-acc",
      data: [
        { id: 'ACCORDION_GENERAL', text: '{$accordion_section_general}', type: 'fix', status: "expanded", height: 17 },
        { id: 'ACCORDION_DEVTREE', text: '{$accordion_section_devtree}', type: 'flex' },
      ]
    });

    $( '#ID_GENERAL_TREE' + this.dynid ).tree( {
      size: -1,
      height: -1,
      border: false,
      focus: false,
      data: this.generalTree,
      selection: 'single',
      sort: "none",
      result: this.ControlTreeClick.bind( this )
    } );

    $( '#ID_DEVTREE' + this.dynid ).tree( {
      size: -1,
      height: -1,
      border: false,
      data: Data.Instance().TreeData(),
      dataRoot: "folder.devices",
      selection: 'single',
      sort: "+",  
      regkey: "ID_DEVTREE",
      result: this.ControlTreeClick.bind( this ),
      inplaceButton: this.ControlTreeInplaceButton.bind( this ),
      //--- drag & drop ---
      dragtype: "application/entid",
      dragover: this.ControlTreeDragover.bind( this ),
      drop: this.ControlTreeDrop.bind( this ),
      renderRow: this.ControlTreeRenderRow.bind( this )
    } );

    Data.Instance().On( "MODULES/UPDATE-SIDEBAR", this.etag, this.OnModulesUpdateSidebar.bind( this ) );
    //Data.Instance().On( "MODULES/DELETE-SIDEBAR", this.etag, this.OnModulesDeleteSidebar.bind( this ) );

  }//load


  //=================================================================
  //  VIEWABLE EVENTS
  //=================================================================

  //-----------------------------------------------------------------
  //  event function viewable
  //
  OnEvent( eventName, param )
  {
    if( eventName == "selected" )  {
      if( this.generalTree.Has( param ) )  {
        $( '#ID_GENERAL_TREE' + this.dynid ).tree( "option", "selected", param );
        $( '#ID_DEVTREE' + this.dynid ).tree( "option", "selected", null );
      }
      else if( Data.Instance().TreeData().Has( param ) ) {
        $( '#ID_DEVTREE' + this.dynid ).tree( "option", "selected", param );
        $( '#ID_GENERAL_TREE' + this.dynid ).tree( "option", "selected", null );
      }
      else {
        $( '#ID_DEVTREE' + this.dynid ).tree( "option", "selected", null );
        $( '#ID_GENERAL_TREE' + this.dynid ).tree( "option", "selected", null );
      }
    }
    else if( eventName == "localinfo" )  {
      this.localInfo[ param['entid'] ] = param.info;
      this.OnEventLocalInfo( param );
    }
  }

  //-----------------------------------------------------------------
  //  called by pagemain if a local info has been changed
  //
  OnEventLocalInfo( localInfo )
  {
    // localInfo.info: { title: null, modified: false, error: false };
    let localEntid = localInfo['entid'];

    if( localInfo.info == null )  {
      delete this.localInfo[ localEntid ];
      $( '#ID_DEVTREE' + this.dynid ).tree( "item", localEntid, "status+", "normal" );
      $( '#ID_DEVTREE' + this.dynid ).tree( "item", localEntid, "redraw" );
    }
    else {
      if( localInfo.info && ( localInfo.info['modified'] || localInfo.info['error'] ) ) {
        $( '#ID_DEVTREE' + this.dynid ).tree( "item", localEntid, "status+", "red" );
      }
      else {
        $( '#ID_DEVTREE' + this.dynid ).tree( "item", localEntid, "status+", "normal" );
      }

      if( localInfo.info['title'] )  {
        $( '#ID_DEVTREE' + this.dynid ).tree( "item", localEntid, "redraw" );
      }
    }

  }

  //-----------------------------------------------------------------
  //  module api requires to add a sidebar entry
  //
  OnModulesUpdateSidebar( sidebar )
  {
    CONST.sidebarGeneral.push( sidebar );
    this.generalTree = new Treeset( { id: 'id' } );
    this.generalTree.SetData( CONST.sidebarGeneral );

    $( '#ID_GENERAL_TREE' + this.dynid ).tree( {
      size: -1,
      height: -1,
      border: false,
      focus: false,
      data: this.generalTree,
      selection: 'single',
      sort: "none",
      result: this.ControlTreeClick.bind( this )
    } );
  }

  //-----------------------------------------------------------------
  //  module api requires to delete a sidebar entry
  //
  OnModulesDeleteSidebar( sidebarId )
  {

  }

  //=================================================================
  //  TREE CONTROL ACTIONS
  //=================================================================

  ControlTreeRenderRow( id, record )
  {
    if( ( id in this.localInfo ) && ( this.localInfo[ id ]['title'] ) )  {
      let overwriteRecord = $.clone( record );
      overwriteRecord['text'] = $.lang( this.localInfo[ id ]['title'] );
      return overwriteRecord;
    }
    return record;
  }

  //-----------------------------------------------------------------
  //  
  //
  ControlTreeClick( selected )
  {
    this.FireEvent( "click", selected[ 0 ] );
  }  

  ControlTreeInplaceButton( id, command )
  {
    if( command == "delete" )  {
      PageMain.DeleteEntity( id );
    }
    else if( command == "add" )  {
      PageMain.CreateEntity( id );
    }
    else if( command == "permission" ) {
      PageMain.ShowPermission( id );
      //PermissionsDialog.Show( { entid: id } );
    }
    //console.log( "inplace-command", id, command );
  }

  ControlTreeDragover( dragdata, dropdata )
  {
    // No multiselect!
    if( !dragdata || !dragdata.data || dragdata.data.length != 1 )  {
      return "none";
    }

    let dragEntityId = dragdata.data[ 0 ];
    let dropEntityId = dropdata.data;       

    // Ask treeData if dragged item may be dropped.
    if( Data.Instance().TreeData().IsDropAllowed( dragEntityId, dropEntityId ) == true )  {      
      return "link";
    }
    return "none";  //"move";
  }  

  ControlTreeDrop( action, dragdata, dropdata )
  {
    if( action != "link" )  {
      return 0;
    }

    // No multiselect!
    if( dragdata.data.length != 1 )  {
      return 0;
    }

    // do the actual move
    let moveEntityId = dragdata.data[ 0 ];
    let toEntityId = dropdata.data;

    // fire event to upper layer
    this.FireEvent( "entityMove", { 'entid': moveEntityId, 'toParent': toEntityId } );

  }

}


//=============================================================================
//  BUILDER 
//=============================================================================

SidebarTree.Register( '/part/sidebar' );
//=============================================================================
//
//  File: validate.js
//  (c) 2021 by CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          04.02.2021
//
//  Description:
//          toolbox for the most common verifications
//
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class Validate
{
  //-----------------------------------------------------------------
  //  check for ip is in range
  //
  static NotEmptyString( data )
  {
    // used in control validation framework
    if( data instanceof ValidationControl ) {
      if( typeof data.value != "string" )  {
        data.error = "{$const_error_nostring}";
        return false;
      }
      else if( data.value.length == 0 ) {
        data.error = "{$const_error_notemptystring}";
        return false;
      }
      else {
        data.error = null;
        return true;
      }
    }
    return false;
  }

  //-----------------------------------------------------------------
  //  check for valid ip v4 address
  //
  static IsIpAddress( data )
  {
    // used in control validation framework
    if( data instanceof ValidationControl )  {
      if( /^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/.test( data.value ) ) {
        data.error = null;
        return true;
      }
      else if( data.value.length == 0 )  {
        data.error = "{$const_error_ipempty}";
        return false;
      }
      else {
        data.error = "{$const_error_novalidip}";
        return false;
      }
    }

    // used as bool true/false
    else  {
      if( /^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/.test( data ) ) {
        return ( true );
      }
      return ( false );
    }
  }    

  //-----------------------------------------------------------------
  //  check for valid ip v4 address
  //
  static IsIpOrHost( data )
  {
    const regex = /^([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])(\.([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]{0,61}[a-zA-Z0-9]))*$/g;

    // used in control validation framework
    if( data instanceof ValidationControl )  {
      if( regex.test( data.value ) ) {
        data.error = null;
        return true;
      }
      else if( data.value.length == 0 )  {
        data.error = "{$const_error_ipempty}";
        return false;
      }
      else {
        data.error = "{$const_error_novalidiporhost}";
        return false;
      }
    }

    // used as bool true/false
    else  {
      if( /^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/.test( data ) ) {
        return ( true );
      }
      return ( false );
    }
  }      

  //-----------------------------------------------------------------
  //  check for valid ip v4 address
  //
  static IsIpPort( data )
  {
    let port = parseInt( data.value );
    
    // used in control validation framework
    if( data instanceof ValidationControl )  {
      if( port > 0 && port <= 65535 )  {
        data.error = null;
        return true;
      }
      else if( data.value.length == 0 )  {
        data.error = "{$const_error_portempty}";
        return false;
      }
      else {
        data.error = "{$const_error_novalidport}";
        return false;
      }
    }

    // used as bool true/false
    else  {
      if( port > 0 && port <= 65535 )  {
        return ( true );
      }
      return ( false );
    }
  }    

  //-----------------------------------------------------------------
  //  check for valid netmask
  //
  static IsNetmask( data )
  {
    // used in control validation framework
    if( data instanceof ValidationControl ) {

      // split value against the four dots
      let mask = data.value.match( /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/ );
      if( !mask )  {
        data.error = "{$const_error_novalidmask}";
        return false;
      }

      // convert to number
      let maskValue = ( +mask[1] << 24 ) + ( +mask[2] << 16 ) + ( +mask[3] << 8 ) + ( +mask[4] );
      if( ( maskValue & ( ~maskValue >> 1 ) ) != 0 ) {
        data.error = "{$const_error_novalidmask}";
        return false;
      }

      // nice :-)
      data.error = null;
      return true;
    }

    // used as bool true/false
    else  {

      let mask = data.match( /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/ );
      if( mask == null ) {
        return false;
      }

      let maskValue = ( +mask[1] << 24 ) + ( +mask[2] << 16 ) + ( +mask[3] << 8 ) + ( +mask[4] );
      return ( ( maskValue & ( ~maskValue >> 1 ) ) == 0 );
    }
  }

  //-----------------------------------------------------------------
  //  check for valid ip mac address
  //
  static IsMacAddress( data )
  {
    // used in control validation framework
    if( data instanceof ValidationControl ) {
      if( /^([0-9A-F]{2}[:-]){5}([0-9A-F]{2})$/.test( data.value ) ) {
        data.error = null;
        return true;
      }
      else {
        data.error = "{$const_error_novalidmac}";
        return false;
      }
    }
    else {
      if( /^([0-9A-F]{2}[:-]){5}([0-9A-F]{2})$/.test( data ) )  {
        return true;
      }
      return false;
    }
  }

  //-----------------------------------------------------------------
  //  check for ip is in range
  //
  static IsIpInSubnet( ipNetCard, ipMask, ipAddress )
  {
    // preconditions
    let net = ipNetCard.match( /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/ );
    let mask = ipMask.match( /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/ );
    let ip = ipAddress.match( /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/ );
    if( !net || !mask || !ip )  {
      return null;
    }

    // convert to long values
    let netValue = ( +net[ 1 ] << 24 ) + ( +net[ 2 ] << 16 ) + ( +net[ 3 ] << 8 ) + ( +net[ 4 ] );
    let maskValue = ( +mask[ 1 ] << 24 ) + ( +mask[ 2 ] << 16 ) + ( +mask[ 3 ] << 8 ) + ( +mask[ 4 ] );
    let ipValue = ( +ip[ 1 ] << 24 ) + ( +ip[ 2 ] << 16 ) + ( +ip[ 3 ] << 8 ) + ( +ip[ 4 ] );

    // check ip agains network + mask 
    return ( ( netValue & maskValue ) == ( ipValue & maskValue ) );
  }

  
}
﻿//=============================================================================
//
//  WIDGET BASE CLASS
//  (c) 2020 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigBase extends Viewable
{
  static avpIcons = {
    videoformat:  "avp avp-videoformat",
    zustandsanzeige:  "avp avp-zustandsanzeige",
    sprachen: "avp avp-sprachen",
    systeminfo: "avp avp-systeminfo",
    tuer: "avp avp-tuer",
    tuer_oeffnen: "avp avp-tuer_oeffnen",
    tuerrufbilder:  "avp avp-tuerrufbilder",
    benutzer_nicht_stoeren: "avp avp-benutzer_nicht_stoeren",
    anrufliste: "avp avp-anrufliste",
    audio:  "avp avp-audio",
    concierge:  "avp avp-concierge",
    design: "avp avp-design",
    display:  "avp avp-display",
    display_reinigen: "avp avp-display_reinigen",
    eco:  "avp avp-eco",
    eco_einstellungen:  "avp avp-eco_einstellungen",
    einstellungen:  "avp avp-einstellungen",
    home: "avp avp-home",
    kameras:  "avp avp-kameras",
    kontakt_extern: "avp avp-kontakt_extern",
    kontakt_gruppe: "avp avp-kontakt_gruppe",
    kontakt_intern: "avp avp-kontakt_intern",
    kontakte: "avp avp-kontakte",
    licht:  "avp avp-licht",
    mikrofon_aus: "avp avp-mikrofon_aus",
    postmelder: "avp avp-postmelder",
    reset:  "avp avp-reset",
    rufton_aus: "avp avp-rufton_aus",
    rufumleitung: "avp avp-rufumleitung",
    rufumleitung_zeitlich:  "avp avp-rufumleitung_zeitlich",
    schalten: "avp avp-schalten",
    schalter: "avp avp-schalter"
  };

  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //static allowedModules = "*";
  //static allowedModules = ["KBD", "IO", "TILE", "AZIO", "ATLC", "ATLCE1", "ATLCE2", "ATLCE3", "BRMA", "BTM", "ATLM"];
  static allowedModules = []; // allowed nowhere, everywhere:  "*"
  static widgetIcon = "mdi mdi-head-question-outline";
  static widgetNameShort = "${widget_base_short}";
  static widgetNameLong = "${widget_base_long}";
  static widgetId = "base";

  //-----------------------------------------------------------------
  //  STATIC OVERRIDABLES
  //-----------------------------------------------------------------

  static Remove( uid, trigger, entity )
  {
    var classname = this.toString().split( '(' || /s+/ )[0].split( ' ' || /s+/ )[1];
    console.error( `${ classname } You must override 'Remove' in Widget!` );
  }

  static Build( $element, createData )
  {
    var classname = this.toString().split( '(' || /s+/ )[0].split( ' ' || /s+/ )[1];
    console.error( `${ classname } You must override 'Build' in Widget!` );
  }

  static Validate( validateControl, uid, trigger, entity )
  {
    var classname = this.toString().split( '(' || /s+/ )[0].split( ' ' || /s+/ )[1];
    console.error( `${ classname } You must override 'Validate' in Widget!` );
    return 0;
  }

  static References( uid, trigger, entity )
  {
    var classname = this.toString().split( '(' || /s+/ )[0].split( ' ' || /s+/ )[1];
    console.error( `${ classname } You must override 'References' in Widget!` );

    // no external ressources are used
    return null;
    // example: return a used statepoint and a call to a device
    //return [ "sp:server.fj/folder.confdev/dnd", "call:server.fj/apartment.1:101" ];
  }

  //-----------------------------------------------------------------
  //  HELPER: REMOVE WIDGET 
  //-----------------------------------------------------------------

  static HelperRemove( uid, trigger, entity )
  {
    // get the desired record
    let deviceConfigIndex = -1;
    let deviceConfigList =  entity['deviceConfig'];

    if( trigger ) {
      deviceConfigIndex = deviceConfigList.findIndex( ( record ) => ( record['uid'] == uid && record['trigger'] == trigger ) );
    }
    else if( entity['class'] == "timeline" ) {
      let uidClass = uid.substr( 0, 3 );
      if( uidClass == "on_" ) {
        deviceConfigList = entity['timeline']['action']['on'];
        deviceConfigIndex = entity['timeline']['action']['on'].findIndex( ( rec ) => ( rec['uid'] == uid ) );
      }
      else if( uidClass == "off" ) {
        deviceConfigList = entity['timeline']['action']['off'];
        deviceConfigIndex = entity['timeline']['action']['off'].findIndex( ( rec ) => ( rec['uid'] == uid ) );
      }
    }
    else {
      deviceConfigIndex = deviceConfigList.findIndex( ( record ) => ( record['uid'] == uid ) );
    }
    
    if( deviceConfigIndex < 0 ) {
      return;
    }


    let deviceConfigRecord = deviceConfigList[ deviceConfigIndex ];

    // create new record (copy mandantory fields)
    let newConfigRecord = { 
      'template': "none"
    };  
    if( 'module' in deviceConfigRecord )        newConfigRecord['module']   = deviceConfigRecord['module'];
    if( 'modid' in deviceConfigRecord )         newConfigRecord['modid']    = deviceConfigRecord['modid'];
    if( 'uid' in deviceConfigRecord )           newConfigRecord['uid']      = deviceConfigRecord['uid'];
    if( 'abbr' in deviceConfigRecord )          newConfigRecord['abbr']     = deviceConfigRecord['abbr'];
    if( 'trigger' in deviceConfigRecord )       newConfigRecord['trigger']  = deviceConfigRecord['trigger'];
    if( 'opener' in deviceConfigRecord )        newConfigRecord['opener']   = deviceConfigRecord['opener'];
    if( 'in' in deviceConfigRecord )            newConfigRecord['in']       = deviceConfigRecord['in'];
    if( 'led' in deviceConfigRecord )           newConfigRecord['led']      = deviceConfigRecord['led'];
    if( 'out' in deviceConfigRecord )           newConfigRecord['out']      = deviceConfigRecord['out'];
    if( 'actionid' in deviceConfigRecord )      newConfigRecord['actionid'] = deviceConfigRecord['actionid'];
    if( 'stateid' in deviceConfigRecord )       newConfigRecord['stateid']  = deviceConfigRecord['stateid'];
    if( 'buscontroller' in deviceConfigRecord ) newConfigRecord['buscontroller']  = deviceConfigRecord['buscontroller'];

    // copy new record
    deviceConfigList[ deviceConfigIndex ] = newConfigRecord;
  }

  static HelperRemoveStatepoint( spName, entity )
  {
    if( "sp" in entity && spName in entity['sp'] )  {
      delete entity['sp'][ spName ];
    }
  }

  //------ permissions ----------------------------------------------

  static HelperRemovePermission( spName, entity  )
  {
    if( "permission" in entity )  {
      entity['permission'] = entity['permission'].filter( ( item ) => item['f'] != spName );
    }
  }

  static HelperAddDefaultPermission( spName, entity )
  {
    if( !( "permission" in entity ) ) {
      entity['permission'] = [];
    }

    // check if already exists
    let foundIndex = entity['permission'].findIndex( ( item ) => ( item['f'] == spName ) && ( item['w'] == "*" ) );
    if( foundIndex >= 0 )    
      return;

    // add default permission record
    let defaultPermission = {
      "p": "rw", 
      "w": "*", 
      "f": spName
    };
    entity['permission'].push( defaultPermission );
  }

  //------ device record --------------------------------------------

  static HelperGetDeviceRecord( entity, uid, trigger )
  {
    if( typeof trigger == "string" )  {
      let deviceRecordIndex = entity['deviceConfig'].findIndex( ( rec ) => ( rec['uid'] == uid && rec['trigger'] == trigger ) );
      if( deviceRecordIndex >= 0 )  {
        return entity['deviceConfig'][ deviceRecordIndex ];
      }
    }
    else if( entity['class'] == "timeline" ) {
      let uidClass = uid.substr( 0, 3 );
      if( uidClass == "on_" ) {
        let deviceRecordIndex = entity['timeline']['action']['on'].findIndex( ( rec ) => ( rec['uid'] == uid ) );
        if( deviceRecordIndex >= 0 )  {
          return entity['timeline']['action']['on'][ deviceRecordIndex ];
        }
      }
      else if( uidClass == "off" ) {
        let deviceRecordIndex = entity['timeline']['action']['off'].findIndex( ( rec ) => ( rec['uid'] == uid ) );
        if( deviceRecordIndex >= 0 )  {
          return entity['timeline']['action']['off'][ deviceRecordIndex ];
        }
      }
    }
    else  {
      let deviceRecordIndex = entity['deviceConfig'].findIndex( ( rec ) => ( rec['uid'] == uid ) );
      if( deviceRecordIndex >= 0 ) {
        return entity['deviceConfig'][ deviceRecordIndex ];
      }
    }

    return null;
  }

  //------ device function ------------------------------------------

  static HelperGetDeviceFunction( entity, functionId )
  {
    let deviceRecordIndex = entity['deviceConfig'].findIndex( ( rec ) => ( rec['template'] == functionId ) );
    if( deviceRecordIndex >= 0 ) {
      return entity['deviceConfig'][ deviceRecordIndex ];
    }
  }

  static HelperGetDeviceFunctionIndex( entity, functionId )
  {
    return entity['deviceConfig'].findIndex( ( rec ) => ( rec['template'] == functionId ) );
  }

  static HelperAddDeviceFunction( entity, functionId )
  {
    if( !( 'deviceConfig' in entity ) ) {
      entity['deviceConfig'] = [];
    }
    
    let deviceRecordIndex = entity['deviceConfig'].findIndex( ( rec ) => ( rec['template'] == functionId ) );
    if( deviceRecordIndex < 0 ) {
      let newDeviceFunction = {
        'module': "function",
        'template': functionId,
        'sp': null
      };
      entity['deviceConfig'].push( newDeviceFunction );   
      deviceRecordIndex = ( entity['deviceConfig'].length - 1 );
    }
    return entity['deviceConfig'][ deviceRecordIndex ];
  }

  static HelperRemoveDeviceFunction( entity, functionId )
  {
    let deviceRecordIndex = entity['deviceConfig'].findIndex( ( rec ) => ( rec['template'] == functionId ) );
    if( deviceRecordIndex >= 0 ) {
      entity['deviceConfig'].splice( deviceRecordIndex, 1 );
    }
  }

  static HelperIsCommandMode( deviceRecord )
  {
    return ( deviceRecord['module'] == "TIMELINE" );
  }


  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  create the tile icon & text selector
  //
  FormLoadTileControls( selector, deviceRecord, defaultIcon, defaultLabel )
  {
    // prepare
    let $element = $( selector );
    if( $element.length == 0 )  return;

    // add 
    let $avpControls = $( $.views.Html( "/widget/deviceConfigBase/avpTile", {}, this.dynid ) );
    $element.append( $avpControls );

    // prepare droplist content for avp icons
    let dropListData = [];
    for( let key in WidgetDeviceConfigBase.avpIcons )  {
      dropListData.push( {
        id: key,
        icon: WidgetDeviceConfigBase.avpIcons[ key ],
        text: ""
      });
    }

    // icon droplist
    let icon = deviceRecord['icon'] || defaultIcon;
    if( icon == "none" )  icon = defaultIcon;

    $( "#ID_AVP_TILE_ICON" + this.dynid ).listbox( {
      size: 6,  // not using CONST.size.S,
      height: 5,
      selected: [ icon ],
      type: "droplist",
      data: dropListData,
      placeholder: "{$const_common_empty_placeholder}",
    } );

    // label edit
    let label = deviceRecord['label'] || $.lang( defaultLabel );
    if( label == "{$aui_lang_tile}" ) label = $.lang( defaultLabel );

    $( "#ID_AVP_TILE_EDIT" + this.dynid ).edit( {
      size: CONST.size.L,
      value: label,
      result: this.ControlAvpTileResult.bind( this )
    });

    // set first label
    this.ControlAvpTileResult( label );

  }

  //-----------------------------------------------------------------
  //  read tile controls and save directly into device record
  //
  FormReadTileControls( selector, deviceRecord )
  {
    let icon = $( "#ID_AVP_TILE_ICON" + this.dynid ).listbox( "option", "selected" );
    let label = $( "#ID_AVP_TILE_EDIT" + this.dynid ).edit( "option", "value" );
    deviceRecord['icon'] = icon[ 0 ];
    deviceRecord['label'] = label;
  }


  //-----------------------------------------------------------------
  //  bubble event to parent about the changed label
  //
  ControlAvpTileResult( result )
  {
    this.$parent.trigger( "tile-label", result );
  }


  //-----------------------------------------------------------------
  //  UNIT TEST HELPER
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  bubble event to parent about the changed label
  //
  UnitTestCheckObject( testName, testController, paramPathString, object, desiredValue, timestamp = null )
  {
    let paramPath = paramPathString.split( "/" );
    
    // check existance of parameter
    let valueObject = object;
    for( let param of paramPath )  {
      if( typeof valueObject == "object" && param in valueObject )  {
        valueObject = valueObject[ param ];
      }
      else  {
        if( timestamp ) testController.Error( `${ testName } - failed, Missing Parameter \"${ paramPathString }\" [${ ( Date.now() - timestamp ) / 1000 }s]` );
        else testController.Error( `${ testName } - failed, Missing Parameter \"${ paramPathString }\"` );
        return;
      }
    }

    // check desired value
    if( typeof desiredValue == "string" )  {
      if( valueObject != desiredValue )  {
        if( timestamp ) testController.Error( `${ testName } - failed, Desired Parameter \"${ paramPathString }\" is '${ valueObject }' [${ ( Date.now() - timestamp ) / 1000 }s]` );
        else testController.Error( `${ testName } - failed, Desired Parameter \"${ paramPathString }\" is '${ valueObject }'` );
        return;
      }
    }
    else if( Array.isArray( desiredValue ) ) {
      if( desiredValue.indexOf( valueObject ) < 0 )  {
        if( timestamp ) testController.Error( `${ testName } - failed, Desired Parameter \"${ paramPathString }\" is '${ valueObject }' [${ ( Date.now() - timestamp ) / 1000 }s]` );
        else testController.Error( `${ testName } - failed, Desired Parameter \"${ paramPathString }\" is '${ valueObject }'` );
        return;
      }
    }
    else if( desiredValue instanceof RegExp )  {
      if( valueObject.match( desiredValue ) === null ) {
        if( timestamp ) testController.Error( `${ testName } - failed, Desired Parameter \"${ paramPathString }\" is '${ valueObject }' [${ ( Date.now() - timestamp ) / 1000 }s]` );
        else testController.Error( `${ testName } - failed, Desired Parameter \"${ paramPathString }\" is '${ valueObject }'` );
      }
    }

    if( timestamp ) testController.Success( `${ testName } - ok, Desired Parameter \"${ paramPathString }\" [${ ( Date.now() - timestamp ) / 1000 }s]` );
    else testController.Success( `${ testName } - ok, Desired Parameter \"${ paramPathString }\"` );

  }


}
﻿//=============================================================================
//
//  Device Config Widgets Factory
//  (c) 2019 CYTEL Technology AG
//
//  Editor: frank Jaekel
//          09.06.2021
//
//  Description:  Dispatcher for all available widgets
//                used by deviceConfig
//
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigFactory
{
  //-----------------------------------------------------------------
  //  STATIC VARIABLES / REGISTER
  //-----------------------------------------------------------------

  static registeredWidgets = {};

  static Register( classPointer )
  {
    let className = classPointer.toString().split( '(' || /s+/ )[0].split( ' ' || /s+/ )[1];
    if( !( 'allowedModules' in classPointer ) ) {
      console.warn( `${ className }: missing static variable "allowedModules"` );
      return;
    }
    if( !( 'allowedTypes' in classPointer ) ) {
      console.warn( `${ className }: missing static variable "allowedTypes"` );
      return;
    }
    if( !( 'widgetIcon' in classPointer ) ) {
      console.warn( `${ className }: missing static variable "functionIcon"` );
      return;
    }
    if( !( 'widgetNameShort' in classPointer ) ) {
      console.warn( `${ className }: missing static variable "nameShort"` );
      return;
    }
    if( !( 'widgetNameLong' in classPointer ) ) {
      console.warn( `${ className }: missing static variable "nameLong"` );
      return;
    }
    if( !( 'widgetId' in classPointer ) ) {
      console.warn( `${ className }: missing static variable "widgetId"` );
      return;
    }

    let widgetId = this._HelperGetWidgetId( classPointer );
    this.registeredWidgets[ widgetId ] = classPointer;

  }

  //-----------------------------------------------------------------
  //  METHODS FOR KEY DIALOG
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  for door-contact, check if read only
  //
  static IsReadOnly( deviceRecordOrModule, entity )
  {
    if( 'template' in deviceRecordOrModule ) {
      let classPointer = this.registeredWidgets[ deviceRecordOrModule['template'] ];
      return WidgetDeviceConfigFactory._HelperIsReadOnly( classPointer );
    }
    return false;
  }

  //-----------------------------------------------------------------
  //  Build list for key- or function-dialog in correct order
  //
  static AllowedWidgetList( deviceRecordOrModule, entity )
  {
    let resultList = [];
    let entityClass = entity['class'];

    // prepare filter
    let module = "*";
    let isFunction = false;
    let isLed = false;
    let isEntrance = ( 'entrance' in entity && entity['entrance'] );
    let configRecordType = "*";

    // filter for functions
    if( typeof deviceRecordOrModule == "string" ) {
      module = deviceRecordOrModule;
      isFunction = true;
    }
    // filter for keys
    else {
      module = deviceRecordOrModule['module'];
      // build list of usable widgets
      if( 'in' in deviceRecordOrModule ) configRecordType = "in";
      if( 'out' in deviceRecordOrModule ) configRecordType = "out";
      if( 'led' in deviceRecordOrModule ) configRecordType = "led";
      isFunction = false;
      isLed = ( configRecordType == "led" );
    }

    // iterate through all registered device widgets and use filter
    for( let classId in this.registeredWidgets ) {
      let classPointer = this.registeredWidgets[ classId ];
      let isAllowed = false;
      if( 'IsUseable' in classPointer ) {
        isAllowed = this._HelperIsUsable( classPointer, isFunction, isLed, entity, module, configRecordType );
      }
      else {
        isAllowed =  ( this._HelperIsUsableByModule( classPointer, module )
                    && this._HelperIsUsableByType( classPointer, configRecordType )
                    && ( this._HelperIsUsableByEntityClass( classPointer, entityClass )
                      || this._HelperIsUsableByEntrance( classPointer, entity )
                       )
                     );
      }

      if( isAllowed ) {
        let result = {
          type: "widget",
          id: this._HelperGetWidgetId( classPointer ),
          text: this._HelperGetWidgetName( classPointer, false ),
          icon: this._HelperGetWidgetIcon( classPointer )
        };
        if( WidgetDeviceConfigFactory._HelperIsReadOnly( classPointer ) ) {
          result['status'] = "disabled";
        }
        resultList.push( result );
      }
    }

    // sort the result ("none" to top)
    resultList.sort( ( recordA, recordB ) =>
    {
      if( recordA['id'] == "none" )
        return -1;
      if( recordB['id'] == "none" )
        return 1;

      return recordA['text'].localeCompare( recordB['text'], navigator.language, { numeric: true, sensitivity: 'base' } );
    } );

    return resultList;
  }

  static WidgetList( deviceRecordOrModule )
  {
    var resultList = [];
    for( let classId in this.registeredWidgets ) {
      let classPointer = this.registeredWidgets[ classId ];
      let isAllowed = false;
      if( 'isFunction' in classPointer ) {
        isAllowed = classPointer['isFunction'];
      }
      if( 'isModule' in classPointer ) {
        isAllowed = !classPointer['isModule'];
      }

      if( isAllowed ) {
        let result = {
          type: "widget",
          id: this._HelperGetWidgetId( classPointer ),
          text: this._HelperGetWidgetName( classPointer, false ),
          icon: this._HelperGetWidgetIcon( classPointer )
        };
        resultList.push( result );
      }
    }

    return resultList;
  }

  //-----------------------------------------------------------------
  //  Build list for key- or function-dialog in correct order
  //
  static WidgetIcon( widgetId )
  {
    if( widgetId in this.registeredWidgets ) {
      let classPointer = this.registeredWidgets[ widgetId ];
      return this._HelperGetWidgetIcon( classPointer );
    }
    return "mdi mdi-help";
  }

  //-----------------------------------------------------------------
  //  Build list for key- or function-dialog in correct order
  //
  static WidgetName( widgetId )
  {
    if( widgetId in this.registeredWidgets ) {
      let classPointer = this.registeredWidgets[widgetId];
      return this._HelperGetWidgetName( classPointer, false );
    }
    return "Unbekannt";
  }

  //-----------------------------------------------------------------
  //  Build list for key- or function-dialog in correct order
  //
  static WidgetUiHeight( widgetId )
  {
    if( widgetId in this.registeredWidgets ) {
      let classPointer = this.registeredWidgets[ widgetId ];
      if( 'widgetUiHeight' in classPointer )  {
        return classPointer['widgetUiHeight'];
      }      
    }
    return 0;
  }

  //-----------------------------------------------------------------
  //  Build list for key- or function-dialog in correct order
  //
  static WidgetIsLineControl( widgetId )
  {
    if( widgetId in this.registeredWidgets ) {
      let classPointer = this.registeredWidgets[widgetId];
      if( 'widgetIsLineControl' in classPointer ) {
        return classPointer['widgetIsLineControl'];
      }
    }
    return false;
  }  

  //-----------------------------------------------------------------
  //  remove widget from entity
  //
  static RemoveFromEntity( uid, trigger, entity )
  {
    // find device record in entity
    let deviceRecord = null;
    if( trigger ) {
      deviceRecord = entity['deviceConfig'].find( ( record ) => ( record['uid'] == uid && record['trigger'] == trigger ) );
    }
    else if( entity['class'] == "timeline" ) {
      let uidClass = uid.substr( 0, 3 );
      if( uidClass == "on_" ) {
        deviceRecord = entity['timeline']['action']['on'].find( ( rec ) => ( rec['uid'] == uid ) );
      }
      else if( uidClass == "off" ) {
        deviceRecord = entity['timeline']['action']['off'].find( ( rec ) => ( rec['uid'] == uid ) );
      }

    }
    else {
      deviceRecord = entity['deviceConfig'].find( ( record ) => ( record['uid'] == uid  ) );
    }

    if( deviceRecord )  {

      // get record and the used template
      let template = deviceRecord['template'] || "";

      // check if we've got a registered template
      if( !( template in this.registeredWidgets ) ) {
        console.warn( `Widget template ${ template } not found!` );
        return false;
      }

      // no matter what, the avp-label and -icon need to be removed 
      delete deviceRecord['label'];
      delete deviceRecord['icon'];

      // call widget remote function
      let classPointer = this.registeredWidgets[ template ];
      return classPointer['Remove']( uid, trigger, entity );

    }

  }//remove

  //-----------------------------------------------------------------
  //  remove widget from entity
  //
  static RemoveFunctionFromEntity( functionId, entity )
  {
    // find device record in entity
    let index = entity['deviceConfig'].findIndex( ( record ) => ( record['template'] == functionId ) );
    if( index >= 0 ) {

      // check if we've got a registered template
      if( !( functionId in this.registeredWidgets ) ) {
        console.warn( `Widget template ${ functionId } not found!` );
        return false;
      }

      // call widget remote function
      let classPointer = this.registeredWidgets[ functionId ];
      return classPointer['Remove']( null, null, entity );

    }

  }//remove

  //-----------------------------------------------------------------
  //  build the widget user interface given a device key
  //  and the trigger into an html-element
  //
  static BuildUI( $element, widgetTemplate, uid, trigger, entity, resultHandler )
  {
    // check if we've got a registered template
    if( !( widgetTemplate in this.registeredWidgets ) ) {
      console.warn( `Widget template ${ widgetTemplate } not found!` );
      return false;
    }

    // create the widget in $element

    // Hint: usualy all createData will be cloned, in this case we want 
    // to give a reference to our entity. This is the option
    // 'createDataByReference' for.
    let classPointer = this.registeredWidgets[ widgetTemplate ];
    let createData = {
      'createDataByReference': true,
      'options': {
        'uid': uid,
        'trigger': trigger,
        'result': resultHandler
      },
      'entity': entity
    };
    classPointer['Build']( $element, createData );
    return true;
  }

  //-----------------------------------------------------------------
  //  get number of errors in this widget without loading
  //
  static Validate( widgetTemplate, uid, trigger, entity )
  {
    // check if we've got a registered template
    if( !( widgetTemplate in this.registeredWidgets ) ) {
      console.warn( `Validate: Widget template ${ widgetTemplate } not found!` );
      return 0;
    }

    // check if a Validate function exists
    if( !( 'Validate' in this.registeredWidgets[ widgetTemplate ] ) ) {
      console.warn( `No 'Validate()' in widget template '${ widgetTemplate }'!` );
      return 0;
    }

    let validateControl = new ValidationControl;
    let classPointer = this.registeredWidgets[ widgetTemplate ];
    return classPointer['Validate']( validateControl, uid, trigger, entity );

  }

  //-----------------------------------------------------------------
  //  get the local added statepoint (for permission check)
  //
  static LocalStatepointName( widgetTemplate, uid, trigger, entity )
  {
    // check if we've got a registered template
    if( !( widgetTemplate in this.registeredWidgets ) ) {
      console.warn( `LocalStatepointName: Widget template ${ widgetTemplate } not found!` );
      return null;
    }

    // check if a Validate function exists
    if( !( 'LocalStatepointName' in this.registeredWidgets[ widgetTemplate ] ) ) {
      return null;
    }

    let classPointer = this.registeredWidgets[ widgetTemplate ];
    return classPointer['LocalStatepointName']( uid, trigger, entity );

  }

  //-----------------------------------------------------------------
  //  RESSOURCE METHODS 
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  get used ressource list for entity (by key uid)
  //
  static ReferencesByKey( uid, entity, objectPath = null )
  {
    // prequisites
    let deviceConfig = entity['deviceConfig'] || null;
    if( objectPath ) {
      deviceConfig = $.object.GetPathValue( entity, objectPath );
    }

    if( !deviceConfig )
      return [];

    // get needed stuff
    let result = [];
    let deviceRecordList = deviceConfig.filter( ( rec ) => rec['uid'] == uid );

    // loop through all triggers of the device key ('uid')
    for( let record of deviceRecordList )  {
      if( ( 'template' in record )
       && ( record['template'] in this.registeredWidgets ) 
        ) {
        // pointer stored in Register()
        let classPointer = this.registeredWidgets[ record['template'] ];

        // some device records does not have a trigger (AFAIK only "atlc*")
        let trigger = record['trigger'] || null;

        // call static References function in widgets 
        // implementation and join the result uniquely
        let ressources = classPointer['References']( uid, trigger, entity );
        if( Array.isArray( ressources ) ) {
          $.array.Join( result, ressources );
        }
      }
    }

    return result;
  }


  //-----------------------------------------------------------------
  //  get used ressource list for entity
  //
  static ReferencesByFunction( funcTemplate, entity, objectPath = null )
  {
    // prequisites
    let deviceConfig = entity['deviceConfig'] || null;
    if( objectPath )  {
      deviceConfig = $.object.GetPathValue( entity, objectPath );
    }
    if( !deviceConfig )
      return [];

    // get needed stuff
    let result = [];
    let deviceRecordList = deviceConfig.filter( ( rec ) => rec['template'] == funcTemplate );

    // loop through all triggers of the device key ('uid')
    for( let record of deviceRecordList ) {
      if( ( 'template' in record )
       && ( record['template'] in this.registeredWidgets )
       ) {
        // pointer stored in Register()
        let classPointer = this.registeredWidgets[ record['template'] ];

        // call static References function in widgets 
        // implementation and join the result uniquely
        let ressources = classPointer['References']( funcTemplate, null, entity );
        if( Array.isArray( ressources ) ) {
          $.array.Join( result, ressources );
        }
      }
    }

    return result;
  }

  //-----------------------------------------------------------------
  //  DESCRIPTION LINE(s)
  //-----------------------------------------------------------------

  static SingleDescriptionLine( keyRecord, entity )
  {
    let classPointer = this.registeredWidgets[ keyRecord['template'] ];
    if( classPointer && ( 'DescriptionLine' in classPointer ) ) {
      return classPointer['DescriptionLine']( keyRecord, entity );
    }
    return null;
  }

  static DescriptionLine( keyGroup, entity )
  {
    // preconditions
    if( ( ! Array.isArray( keyGroup ) ) || ( keyGroup.length == 0 ) ) {
      return [];
    }


    let isLinkedLedPresent = false;
    let resultDescriptionLineData = [];

    // check if all triggers are "none"
    let isEmptyKeyGroup = true;
    for( let keyRecord of keyGroup ) {
      let classId = keyRecord['template'];
      if( classId != "none" ) {
        isEmptyKeyGroup = false;
        break;
      }
    }

    // if all triggers are empty we show a special message
    if( isEmptyKeyGroup )  {
      resultDescriptionLineData.push( {
        icon: null,
        tooltip: null,
        text: "{$devkeygrp_no_action}"
      } );
      return resultDescriptionLineData;
    }

    // build description line for set triggers
    for( let keyRecord of keyGroup )  {

      // prepare
      let classId = keyRecord['template'];
      let descriptionLine = {};

      // skip this trigger if action is "none"
      if( classId == "none" ) {
        continue;
      }

      // get description for trigger icon and tooltip
      let triggerDescription = DeviceKeyInfo.GetTriggerDescription( keyRecord, entity['deviceTemplate'] );
      descriptionLine['icon'] = triggerDescription['icon'] || "mdi mdi-help-circle-outline";
      descriptionLine['tooltip'] = triggerDescription['text'] || "";

      // error case: template with no UI widget (should no happen)
      if( !( classId in this.registeredWidgets ) )  {
        descriptionLine['text'] = `Function "${classId}" not found!`;
      }

      // ask widget for the description line
      else {

        // call DescriptionLine() in widget
        let classPointer = this.registeredWidgets[ classId ];
        if( 'DescriptionLine' in classPointer )  {
          let description = classPointer['DescriptionLine']( keyRecord, entity );
          if( description )  {
            descriptionLine['text'] = description['text'];
            if( description['icon'] ) descriptionLine['icon'] = description['icon'];
            if( description['color'] ) descriptionLine['iconcolor'] = description['color'];
            if( description['linked'] )  {
              isLinkedLedPresent = true;
            }
          }
        }
        // or as default: use widget name
        else {
          descriptionLine['text'] = WidgetDeviceConfigFactory._HelperGetWidgetName( classPointer, false );
        }
      }

      // put line into array
      resultDescriptionLineData.push( descriptionLine );

    }

    // if led is linked and only one trigger is used, make one line
    if( isLinkedLedPresent && resultDescriptionLineData.length == 2 )  {
      let ledDescription = resultDescriptionLineData[ 1 ];
      let postfix = " " + $.lang( "{$device_config_description_linkedled}" );
      resultDescriptionLineData[ 0 ]['text'] += postfix;
      resultDescriptionLineData[ 0 ]['posticon'] = ledDescription['icon'];
      resultDescriptionLineData[ 0 ]['posticoncolor'] = ledDescription['iconcolor'];
      resultDescriptionLineData.splice( 1, 1 );
    }

    // finish
    return resultDescriptionLineData;
  }

  //-----------------------------------------------------------------
  //  UPDATE LINKED LED
  //-----------------------------------------------------------------

  static UpdateLinked( deviceRecord, entity )
  {
    let widgetId = deviceRecord['template'] || "";
    if( widgetId in this.registeredWidgets ) {
      let classPointer = this.registeredWidgets[ widgetId ];
      if( 'UpdateLinked' in classPointer )  {
        return classPointer['UpdateLinked']( deviceRecord, entity );
      }
    }
    
    return false;
  }

  //-----------------------------------------------------------------
  //  REBUILD AFTER ENTITY CHANGE
  //-----------------------------------------------------------------  

  static UpdateEntity( deviceRecord, entity )
  {
    let widgetId = deviceRecord['template'] || "";
    if( widgetId in this.registeredWidgets ) {
      let classPointer = this.registeredWidgets[ widgetId ];
      if( 'UpdateEntity' in classPointer )  {
        return classPointer['UpdateEntity']( deviceRecord, entity );
      }
    }
    
    return false;
  }

  //-----------------------------------------------------------------
  //  HELPER
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  return id like "rule", "call", "switch"
  //
  static _HelperGetWidgetId( classPointer )
  {
    return classPointer['widgetId'];
  }

  static _HelperGetWidgetName( classPointer, isShort )
  {
    if( isShort ) return $.lang( classPointer['widgetNameShort'] );
    else          return $.lang( classPointer['widgetNameLong'] );
  }

  static _HelperGetWidgetIcon( classPointer )
  {
    return classPointer['widgetIcon'];
  }

  static _HelperIsUsableByModule( classPointer, keyModule )
  {
    return ( ( classPointer['allowedModules'] == "*" )
          || ( classPointer['allowedModules'].indexOf( keyModule ) >= 0 )
           );
  }

  static _HelperIsReadOnly( classPointer )
  {
    if( 'IsReadOnly' in classPointer ) {
      return classPointer['IsReadOnly']( );
    }
    return false;
  }

  static _HelperIsUsableByType( classPointer, keyType )
  {
    return ( ( classPointer['allowedTypes'] == "*" )
          || ( classPointer['allowedTypes'].indexOf( keyType ) >= 0 )
           );
  }

  static _HelperIsUsable( classPointer, isFunction, isLed, entity, module, type )
  {
    if( 'IsUseable' in classPointer ) {
      return classPointer['IsUseable']( entity, isFunction, isLed, module, type );
    }
    return false;
  }

  static _HelperIsUsableByEntityClass( classPointer, entityClass )
  {
    return ( entityClass == "*"
          || !( 'allowedClasses' in classPointer )
          || ( classPointer['allowedClasses'] == "*" )
          || ( classPointer['allowedClasses'].indexOf( entityClass ) >= 0 )
           );
  }

  static _HelperIsUsableByEntrance( classPointer, entity )
  {
    let isEntrance = ( 'entrance' in entity && entity['entrance'] );
    if( !isEntrance )
      return false;

    if( 'allowedOnEntrance' in classPointer ) {
      if( classPointer['allowedOnEntrance'] == true ) {
        return true;
      }
      else {
        return false;
      }
    }
    return false;
  }


}//class
﻿//=============================================================================
//
//  Device Key Info
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          09.06.2021
//
//  Description:  Gets informations about the key
//
//    1. Builds a name given the uid + the key index
//       Example:
//         uid: "key1":  "Taste 1"
//         uid: "brma2_1": "Modul 2, Taste 13" (13 is the serial number)
//
//    2. Gets the info if other triggers then 'single' is possible
//       Example: On AHTx/AHFx 
//
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class DeviceKeyInfo
{

  //-----------------------------------------------------------------------------
  //  Build the name
  //
  static GetTitle( uid, keyIndex, deviceTemplate, deviceConfigRecord = null )
  {
    // avp: must be first uid "key1" is re-used
    if( deviceTemplate == "AVP_870-0" )  {
      return DeviceKeyInfo._GetAVPTitle( uid, keyIndex, deviceTemplate, deviceConfigRecord );
    }

    if( !uid )  {
      return "?";
    }

    // timeline
    if( uid.substr( 0, 3 ) == "on_" ) {
      return $.lang( "{$keytitle_timeline_on}" );
    }
    if( uid.substr( 0, 3 ) == "off" ) {
      return $.lang( "{$keytitle_timeline_off}" );
    }


    // apartment
    if( uid.substr( 0, 3 ) == "key" )  {
      return DeviceKeyInfo._GetApartmentTitle( uid, keyIndex, deviceTemplate, deviceConfigRecord );
    }

    // apartment i/o
    if( uid.substr( 0, 4 ) == "ahtf" ) {
      return DeviceKeyInfo._GetApartmentIOTitle( uid, keyIndex, deviceTemplate, deviceConfigRecord );
    }

    // brma
    if( uid.substr( 0, 4 ) == "brma" ) {
      return DeviceKeyInfo._GetBrmaTitle( uid, keyIndex, deviceTemplate, deviceConfigRecord );
    }

    // btm
    if( uid.substr( 0, 3 ) == "btm" ) {
      return DeviceKeyInfo._GetBtmTitle( uid, keyIndex, deviceTemplate, deviceConfigRecord );
    }

    // atlc(e)
    if( uid.substr( 0, 4 ) == "atlc" ) {
      return DeviceKeyInfo._GetAtlcTitle( uid, keyIndex, deviceTemplate, deviceConfigRecord );
    }

    // atlm
    if( uid.substr( 0, 4 ) == "atlm" ) {
      return DeviceKeyInfo._GetAtlmTitle( uid, keyIndex, deviceTemplate, deviceConfigRecord );
    }


  }//get title

  //-----------------------------------------------------------------------------
  //  Build the svg preview
  //
  static GetSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord = null, deviceConfig = null )
  {
    if( !uid ) {
      return "?";
    }

    // avp: must be first (uid "key1" is re-used)
    if( deviceTemplate == "AVP_870-0" ) {
      return DeviceKeyInfo._GetAVPSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord, deviceConfig );
    }

    // apartment
    if( uid.substr( 0, 3 ) == "key" ) {
      return DeviceKeyInfo._GetApartmentSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord, deviceConfig );
    }

    // apartment i/o
    if( uid.substr( 0, 4 ) == "ahtf" ) {
      if( deviceTemplate.substr( 0, 3 ) == "AFS" ) {
        return DeviceKeyInfo._GetApartmentAfsxIOSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord, deviceConfig );
      }
      return DeviceKeyInfo._GetApartmentIOSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord, deviceConfig );
    }

    // brma
    if( uid.substr( 0, 4 ) == "brma" ) {
      return DeviceKeyInfo._GetBrmaSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord, deviceConfig );
    }

    // btm
    if( uid.substr( 0, 3 ) == "btm" ) {
      return DeviceKeyInfo._GetBtmSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord, deviceConfig );
    }

    // atlc(e)
    if( uid.substr( 0, 4 ) == "atlc" ) {
      return DeviceKeyInfo._GetAtlcSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord, deviceConfig );
    }

    // atlm
    if( uid.substr( 0, 4 ) == "atlm" ) {
      return DeviceKeyInfo._GetAtlmSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord );
    }


  }//get title

  //-----------------------------------------------------------------------------
  //  Get key trigger mode (single/multi/led)
  //
  static GetAllowedTriggers( uid, deviceTemplate )
  {
    let uidKey = uid.substr( 0, 3 );
    let deviceTemplatePrefix = deviceTemplate.substr( 0, 3 );

    // avp
    /*if( uidKey == "til" )  {
      return ["single"];
    }*/
    // old Siedle devices 
    if( uidKey == "key" && ( deviceTemplatePrefix == "AHT" || deviceTemplatePrefix == "AHF" ) )  {
      return [ "single", "double", "pos", "neg", "led" ];
    }
    // new Siedle devices 
    if( uidKey == "key" && deviceTemplatePrefix == "AFS" ) {
      return [ "single", "led" ];
    }

    // in all other cases "null" for "not important"
    return null;
  }

  //-----------------------------------------------------------------------------
  //  Get key trigger icon and tooltip text (used by description line)
  //
  static GetTriggerDescription( deviceKeyRecord, deviceTemplate )
  {
    // prequisites
    let icon = "mdi mdi-help-circle-outline";
    let text = "";
    let trigger = deviceKeyRecord['trigger'] || null;
    let module = deviceKeyRecord['module'] || null;

    // for avp
    if( module == "TILE" )  {
      text = $.lang( "{$triggername_tile}" );
      icon = "avp avp-" + deviceKeyRecord['icon'];
    }

    // some atlc i/o don't have trigger
    else if( trigger == null ) {
      if( 'in' in deviceKeyRecord ) {
        text = $.lang( "{$triggername_input_generic}" );
        icon = "mdi mdi-current-dc";
      }
      else if( 'out' in deviceKeyRecord ) {
        text = $.lang( "{$triggername_output_generic}" );
        icon = "mdi mdi-electric-switch";
      }
    }

    // all other cases 
    else {
      switch( trigger )  {
        case 'sensor':  text = $.lang( "{$triggername_input_generic}" );  icon = "mdi mdi-current-dc";              break;
        case 'callkey': text = $.lang( "{$triggername_callkey}" );  icon = "mdi mdi-gesture-tap";                   break;
        case "single":  text = $.lang( "{$triggername_single}" );   icon = "mdi mdi-gesture-tap";                   break;
        case "double":  text = $.lang( "{$triggername_double}" );   icon = "mdi mdi-gesture-double-tap";            break;
        case "pos":     text = $.lang( "{$triggername_pos}" );      icon = "mdi mdi-format-vertical-align-bottom";  break;
        case "neg":     text = $.lang( "{$triggername_neg}" );      icon = "mdi mdi-format-vertical-align-top";     break;
        case "led":     text = $.lang( "{$triggername_led}" );      icon = "mdi mdi-circle-medium";                 break;
      }
    }

    // result
    return {
      text: text,
      icon: icon
    };
  }

  //-----------------------------------------------------------------------------
  //  PRIVATE FUNCTIONS
  //-----------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  Build name for door, atlce
  //
  static _GetBtmTitle( uid, keyIndex, deviceTemplate, deviceConfigRecord )
  {
    switch( uid )  {
      case "atlce1_e1": return $.lang( "{$keytitle_atlce1_e1}" );
      case "atlce1_e2": return $.lang( "{$keytitle_atlce1_e2}" );
      case "atlce1_a1": return $.lang( "{$keytitle_atlce1_a1}" );
      case "atlce1_a2": return $.lang( "{$keytitle_atlce1_a2}" );
      case "atlce1_a3": return $.lang( "{$keytitle_atlce1_a3}" );
      case "atlce1_a4": return $.lang( "{$keytitle_atlce1_a4}" );
      
      case "atlce2_e1": return $.lang( "{$keytitle_atlce2_e1}" );
      case "atlce2_e2": return $.lang( "{$keytitle_atlce2_e2}" );
      case "atlce2_a1": return $.lang( "{$keytitle_atlce2_a1}" );
      case "atlce2_a2": return $.lang( "{$keytitle_atlce2_a2}" );
      case "atlce2_a3": return $.lang( "{$keytitle_atlce2_a3}" );
      case "atlce2_a4": return $.lang( "{$keytitle_atlce2_a4}" );
 
      case "atlce3_e1": return $.lang( "{$keytitle_atlce3_e1}" );
      case "atlce3_e2": return $.lang( "{$keytitle_atlce3_e2}" );
      case "atlce3_a1": return $.lang( "{$keytitle_atlce3_a1}" );
      case "atlce3_a2": return $.lang( "{$keytitle_atlce3_a2}" );
      case "atlce3_a3": return $.lang( "{$keytitle_atlce3_a3}" );
      case "atlce3_a4": return $.lang( "{$keytitle_atlce3_a4}" );
    }
  }

  //-----------------------------------------------------------------------------
  //  Build preview for door
  //
  static _GetBtmSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord, deviceConfig )
  {
    let svgFile = null;
    let svgKey = null;

    let btmKeys = deviceConfig.filter( ( rec ) => ( rec['modid'] == deviceConfigRecord['modid'] ) && ( rec['module'] == "BTM" ) );
    switch( btmKeys.length ) {
      case 1: svgFile = "btm650-1.svg"; break;
      case 2: svgFile = "btm650-2.svg"; break;
      case 3: svgFile = "btm650-3.svg"; break;
      case 4: svgFile = "btm650-4.svg"; break;
    }

    let keyNumber = parseInt( uid.substr( 5, 1 ) );
    svgKey = "IDSVG_BTM" + keyNumber;

    if( !svgKey || !svgFile ) {
      return null;
    }

    return {
      file: svgFile,
      key: svgKey
    };
  }

  //-----------------------------------------------------------------------------
  //  Build name for door, btm
  //
  static _GetBtmTitle( uid, keyIndex, deviceTemplate, deviceConfigRecord )
  {
    let module = parseInt( uid.substr( 3, 2 ) );
    let result = $.lang( "{$keytitle_btm}" );
    result = result.replace( "{module}", module );
    result = result.replace( "{key}", keyIndex );
    return result;
  }

  //-----------------------------------------------------------------------------
  //  Build name for door, brma
  //
  static _GetBrmaTitle( uid, keyIndex, deviceTemplate, deviceConfigRecord )
  {
    let module = parseInt( uid.substr( 4, 2 ) );
    let result = $.lang( "{$keytitle_brma}" );
    result = result.replace( "{module}", module );
    result = result.replace( "{key}", keyIndex );
    return result;
  }

  //-----------------------------------------------------------------------------
  //  Build preview for door
  //
  static _GetBrmaSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord, deviceConfig )
  {
    let svgFile = "brma.svg";
    let svgKey = null;

    let keyNumber = parseInt( uid.substr( 6, 2 ) );
    svgKey = "IDSVG_BRMA" + keyNumber;

    if( !svgKey || !svgFile ) {
      return null;
    }

    return {
      file: svgFile,
      key: svgKey
    };
  }

  //-----------------------------------------------------------------------------
  //  Build name for Apartment
  //
  static _GetApartmentIOTitle( uid, keyIndex, deviceTemplate, deviceConfigRecord )
  {
    switch( uid )  {
      case "ahtf_a1": return $.lang( "{$keytitle_ahtf_a1}" );
      case "ahtf_e1": return $.lang( "{$keytitle_ahtf_e1}" );
      case "ahtf_a2": return $.lang( "{$keytitle_ahtf_a2}" );
      case "ahtf_e2": return $.lang( "{$keytitle_ahtf_e2}" );
    }
  }

  //-----------------------------------------------------------------------------
  //  Build preview for Apartment
  //
  static _GetApartmentIOSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord, deviceConfig )
  {
    let svgFile = "rj45-alt.svg";
    let svgKey = null;

    let isAzio = ( deviceConfig.findIndex( ( rec ) => rec['uid'] == "ahtf_a2" ) >= 0 );
    if( isAzio )  {
      svgFile = "rj45-azio.svg";
    }
    switch( uid ) {
      case "ahtf_a1": svgKey = "IDSVG_A1";  break;
      case "ahtf_e1": svgKey = "IDSVG_E1";  break;
      case "ahtf_a2": svgKey = "IDSVG_A2";  break;
      case "ahtf_e2": svgKey = "IDSVG_E2";  break;
    }

    if( !svgKey )  {
      return null;
    }

    return {
      file: svgFile,
      key: svgKey
    };
  }

  //-----------------------------------------------------------------------------
  //  Build preview for Apartment
  //
  static _GetApartmentAfsxIOSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord, deviceConfig )
  {
    let svgFile = "afsx-clamp.svg";
    let svgKey = null;

    switch( uid ) {
      case "ahtf_e1": svgKey = "IDSVG_E1"; break;
    }

    return {
      file: svgFile,
      key: svgKey
    };
  }



  //-----------------------------------------------------------------------------
  //  Build name for Apartment
  //
  static _GetApartmentTitle( uid, keyIndex, deviceTemplate, deviceConfigRecord )
  {
    if( deviceTemplate == "AFSV" ) {
      switch( uid ) {
        case "key1": return $.lang( "{$keytitle_afsv_key1}" );
        case "key2": return $.lang( "{$keytitle_afsv_key2}" );
        case "key3": return $.lang( "{$keytitle_afsv_key3}" );
        case "key4": return $.lang( "{$keytitle_afsv_key4}" );
      }
    }

    else if( deviceTemplate == "AFS" ) {
      switch( uid ) {
        case "key1": return $.lang( "{$keytitle_afs_key1}" );
        case "key2": return $.lang( "{$keytitle_afs_key2}" );
        case "key3": return $.lang( "{$keytitle_afs_key3}" );
      }
    }

    else {
      let key = parseInt( uid.substr( 3, 2 ) );
      let result = $.lang( "{$keytitle_apartment_key}" );
      result = result.replace( "{key}", key );
      return result;
    }
  }//apartment

  //-----------------------------------------------------------------------------
  //  Build name for Apartment
  //
  static _GetApartmentSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord )
  {
    let svgFile = null;
    let svgKey = "";

    switch( deviceTemplate ) {
      case "AHFV_870-0":
      case "AHF_870-0":   svgFile = "ahf870.svg"; break;
      case "AHT_870-0":
      case "AHTV_870-0":  svgFile = "aht870.svg"; break;
      case "AFSV":        svgFile = "afsv.svg";   break;
      case "AFS":         svgFile = "afs.svg";    break;
    }

    let key = parseInt( uid.substr( 3, 2 ) );
    key = key.toString( ).toUpperCase( );
    svgKey = "IDSVG_KEY" + key;
    return {
      file: svgFile,
      key: svgKey
    };

  }//apartment


  //-----------------------------------------------------------------------------
  //  Build name for AVP
  //
  static _GetAVPTitle( uid, keyIndex, deviceTemplate, deviceConfigRecord )
  {
    if( uid.substr( 0, 4 ) == "ahtf" )  {
      return DeviceKeyInfo._GetApartmentIOTitle( uid, keyIndex, deviceTemplate );
    }

    if( uid.substr( 0, 4 ) == "tile" ) {
      let result = $.lang( "{$keytitle_avp_tile}" );
      let widgetName = $.lang( deviceConfigRecord['label'] );
      result = result.replace( "{name}", widgetName );
      return result;

    }
    
    if( uid == "key1" ) {
      return $.lang( "{$keytitle_avp_key1}" );
    }

  }

  //-----------------------------------------------------------------------------
  //  Build name for AVP
  //
  static _GetAVPSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord, deviceConfig )
  {
    if( uid.substr( 0, 4 ) == "ahtf" ) {
      return DeviceKeyInfo._GetApartmentIOSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord, deviceConfig );
    }

    let svgFile = "avp870-alt.svg";
    let svgKey = "";

    if( uid == "key1" )  {
      svgKey = "IDSVG_KEY1";
    }
    else {
      svgKey = "IDSVG_TILE";
    }

    return {
      file: svgFile,
      key: svgKey
    };

  }

  //-----------------------------------------------------------------------------
  //  Build name for ATLC
  //
  static _GetAtlcTitle( uid, keyIndex, deviceTemplate, deviceConfigRecord )
  {
    switch( uid )  {
      case "atlc_e":      return $.lang( "{$keytitle_atlc_e}"    );
      case "atlc_a1":     return $.lang( "{$keytitle_atlc_a1}"   );
      case "atlc_a2":     return $.lang( "{$keytitle_atlc_a2}"   );
      case "atlc_a1-toemode": return $.lang( "{$keytitle_atlc_a1_toemode}" );
      case "atlc_a3":     return $.lang( "{$keytitle_atlc_a3}"   );
      case "atlce1_e1":   return $.lang( "{$keytitle_atlce1_e1}" ); 
      case "atlce1_e2":   return $.lang( "{$keytitle_atlce1_e2}" );
      case "atlce1_a1":   return $.lang( "{$keytitle_atlce1_a1}" );
      case "atlce1_a2":   return $.lang( "{$keytitle_atlce1_a2}" );
      case "atlce1_a3":   return $.lang( "{$keytitle_atlce1_a3}" );
      case "atlce1_a4":   return $.lang( "{$keytitle_atlce1_a4}" );
      case "atlce2_e1":   return $.lang( "{$keytitle_atlce2_e1}" );
      case "atlce2_e2":   return $.lang( "{$keytitle_atlce2_e2}" );
      case "atlce2_a1":   return $.lang( "{$keytitle_atlce2_a1}" );
      case "atlce2_a2":   return $.lang( "{$keytitle_atlce2_a2}" );
      case "atlce2_a3":   return $.lang( "{$keytitle_atlce2_a3}" );
      case "atlce2_a4":   return $.lang( "{$keytitle_atlce2_a4}" );
      case "atlce3_e1":   return $.lang( "{$keytitle_atlce3_e1}" );
      case "atlce3_e2":   return $.lang( "{$keytitle_atlce3_e2}" );
      case "atlce3_a1":   return $.lang( "{$keytitle_atlce3_a1}" );
      case "atlce3_a2":   return $.lang( "{$keytitle_atlce3_a2}" );
      case "atlce3_a3":   return $.lang( "{$keytitle_atlce3_a3}" );
      case "atlce3_a4":   return $.lang( "{$keytitle_atlce3_a4}" );
    }
  }

  //-----------------------------------------------------------------------------
  //  Build preview for ATLC
  //
  static _GetAtlcSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord, deviceConfig )
  {
    let svgFile = null;
    let svgKey = "";

    if( uid == "atlc_a3" )  {
      svgFile = "atlcng.svg";
      svgKey = "IDSVG_ATLC_A3";
    }
    else if( uid == "atlc_e" ) {
      svgFile = "atlc.svg";
      svgKey = "IDSVG_ATLC_E";
    }
    else if( uid == "atlc_a1" || uid == "atlc_a1-toemode" ) {
      svgFile = "atlc.svg";
      svgKey = "IDSVG_ATLC_A1";
    }
    else if( uid == "atlc_a2" ) {
      svgFile = "atlc.svg";
      svgKey = "IDSVG_ATLC_A2";
    }
    else if( uid.substr( 0, 5 ) == "atlce" )  {
      svgFile = "atlce.svg";
      let key = uid.substr( 7, 2 );
      key = key.toUpperCase();
      svgKey = "IDSVG_ATLCE_" + key;
    }

    return {
      file: svgFile,
      key: svgKey
    };
  }

  //-----------------------------------------------------------------------------
  //  Build name for ATLC
  //
  static _GetAtlmTitle( uid, keyIndex, deviceTemplate, deviceConfigRecord )
  {
    if( uid == "atlm_l" ) {
      return $.lang( "{$keytitle_atlm_l}" );
    }
  }

  static _GetAtlmSvgPreview( uid, keyIndex, deviceTemplate, deviceConfigRecord, deviceConfig )
  {
    let svgFile = "atlm670.svg";
    let svgKey = "";

    if( uid == "atlm_l" ) {
      svgKey = "IDSVG_ATLM_L";
    }

    return {
      file: svgFile,
      key: svgKey
    };
  }


}//class
﻿//=============================================================================
//
//  Device Permissions
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          21.08.2021
//
//  Description:  defines permissions
//
//
//=============================================================================

class DevicePermissions
{
  //===========================================================================
  //  DATA
  //===========================================================================
  static permDefinition =
  {
    "camera": {
      "label": "{$permission_label_camera}",
      "show": ["folder"]
    },
    "addrbook": {
      "label": "{$permission_label_addrbook}",
      "show": ["folder"]
    },
    "dnd": {
      "label": "{$permission_label_dnd}",
      "show": ["folder"]
    },
    "door": {
      "label": "{$permission_label_door}",
      "show": ["folder"]
    },
    "doormatic": {
      "label": "{$permission_label_doormatic}",
      "show": ["folder"]
    },
    "light": {
      "label": "{$permission_label_light}",
      "show": ["folder"]
    },
    "presence": {
      "label": "{$permission_label_presence}",
      "show": ["folder"]
    },
    "redir": {
      "label": "{$permission_label_redir}",
      "show": ["folder"]
    },
    "door2": {
      "label": "{$permission_label_door2}",
      "show": []
    },
    "videoprivacy": {
      "label": "{$permission_label_videoprivacy}",
      "show": []
    }

  };

  //===========================================================================
  //  INTERFACE FUNCTIONS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  get the list of displayed permissions for a given sp-list
  //
  static DisplayedPermission( entinfo )
  {
    let resultList = {};
    let isInheritPermholder = false;

    // first go though all statepoints
    for( let spName in entinfo['sp'] ) {

      // prepare data
      let sp = entinfo['sp'][ spName ];
      let isSpHidden = sp['hidden'] || false;
      let permDef = DevicePermissions.Definition( spName );
      let spLabel = sp['label'] || null;
      if( spLabel ) spLabel = "\"" + spLabel + "\"";
      let record = { show: [], label: spLabel };
      if( permDef ) {
        if( 'label' in permDef && ( typeof permDef['label'] == "string" ) ) {
          record['label'] = permDef['label'];
        }
        record['show'] = permDef['show'];
      }

      // skip (don't display) if sp is hidden or no label is present
      if( isSpHidden || record['label'] == null ) {
        continue;
      }

      // add
      resultList[ spName ] = record;
    }

    // second check the permissions that should be showed even if no sp is present
    for( let permName in DevicePermissions.permDefinition ) {
      let permDef = DevicePermissions.permDefinition[ permName ];
      if( permDef['show'].indexOf( entinfo['class'] ) >= 0 ) {
        isInheritPermholder = true;
        if( !( permName in resultList ) ) {
          resultList[ permName ] = permDef;
        }
      }
    }

    // results
    return {
      'inherit': isInheritPermholder,
      'list':  resultList
    };
  }


  //---------------------------------------------------------------------------
  //  get definition for a given permission (perm['f'])
  //
  static Definition( seek )
  {
    for( let permName in DevicePermissions.permDefinition ) {

      // if permdef is wildcard we check the first characters
      let isWildcard = ( permName.substr( -1, 1 ) == "*" );
      let permSeek = permName;
      if( isWildcard ) {
        permSeek = permSeek.substr( 0, permSeek.length - 1 );
      }

      // check current permdef
      if( ( permName == seek )
       || ( isWildcard && ( seek.indexOf( permSeek ) == 0 ) )
      ) {
        return DevicePermissions.permDefinition[ permName ];
      }
    }
  }


}﻿//=============================================================================
//
//  Device Config Widgets Factory
//  (c) 2019 CYTEL Technology AG
//
//  Editor: frank Jaekel
//          09.06.2021
//
//  Description:  Dispatcher for all available widgets
//                used by deviceConfig
//
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetRuleConfigFactory
{
  //-----------------------------------------------------------------
  //  STATIC VARIABLES / REGISTER
  //-----------------------------------------------------------------


  static AllowedRuleList( entity )
  {
    var result = [];
    var moduleList = Data.Instance().RuleList();
    for( var module of moduleList ) {
      if( module['allowed'].includes( entity['class'] ) ) {
        var record = {
          type: "rule",
          id: module['id'],
          icon: module['icon'],
          text: module['label']
        };
        result.push( record );
      }
    }

    return result;
  }

  //-----------------------------------------------------------------
  //  remove widget from entity
  //
  static RemoveFromEntity( ruleId, entity )
  {
    WidgetRuleGenericUI.Remove( ruleId, entity );    
  }//remove

  //-----------------------------------------------------------------
  //  get number of errors in this widget without loading
  //
  static LocalStatepointName( ruleId, entity )
  {
    return WidgetRuleGenericUI.LocalStatepointName( ruleId, entity );
  }

  //-----------------------------------------------------------------
  //  get number of errors in this widget without loading
  //
  static LocalStatepointName( ruleId, entity )
  {
    return WidgetRuleGenericUI.LocalStatepointName( ruleId, entity );
  }

  //-----------------------------------------------------------------
  //  get number of errors in this widget without loading
  //
  static Validate( ruleId, entity )
  {
    return WidgetRuleGenericUI.Validate( ruleId, entity );
  }


  //-----------------------------------------------------------------
  //  get the references used by this rule
  //
  static References( ruleId, entity )
  {
    return WidgetRuleGenericUI.References( ruleId, entity );
  }  


}//class
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigEmpty extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules = []; //["KBD", "IO", "TILE", "AZIO", "ATLC", "ATLCE1", "ATLCE2", "ATLCE3", "BRMA", "BTM", "ATLM"];
  static allowedTypes = []; //[ "in", "out" ];
  static widgetIcon = "mdi mdi-paw";
  static widgetNameShort = "{$widget_empty_short}";
  static widgetNameLong = "{$widget_empty_long}";
  static widgetId = "empty";
  static dictMacro = "{$aui_widget_empty}";

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( "empty", entity );
    WidgetDeviceConfigBase.HelperRemovePermission( "empty", entity );
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/empty";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    // example: a statepoint in another entity is used and
    // a call to another device is performed. 
    return [ "sp:server.fj/folder.confdev/dnd", "call:server.fj/apartment.1" ];

    // no external ressources:
    // return null;
  }

  //-----------------------------------------------------------------
  //  validate the widget
  //
  static Validate( uid, trigger, entity )
  {
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: "Empty",
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity = createData['entity'];
    this.uid = createData['options']['uid'];
    this.trigger = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );
    
    // console.log( `${ this.constructor.name }( ${ this.uid }, ${ this.trigger } )`, this.deviceRecord );

    // create controls
    $( '#ID_CHECK_OPTION1' + this.dynid ).choice( {
      type: 'check',
      text: "{$widget_empty_check_option1}"
    } );

    $( '#ID_CHECK_OPTION2' + this.dynid ).choice( {
      type: 'check',
      text: "{$widget_empty_check_option2}"
    } );

    // fill data
    this.FormFillControls();

  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidate()
  {
    let numError = 0;
    numError = numError + $( '#ID_CHECK_OPTION1' + this.dynid ).choice( "validate" );
    numError = numError + $( '#ID_CHECK_OPTION2' + this.dynid ).choice( "validate" );
    return numError;
  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidateOff()
  {
    $( '#ID_CHECK_OPTION1' + this.dynid ).choice( "validate", false );
    $( '#ID_CHECK_OPTION2' + this.dynid ).choice( "validate", false );
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save( )
  {
    // ---- widget parameters ----

    // standard: save param into device record
    this.deviceRecord['template'] = "empty";
    this.deviceRecord['sp'] = "empty";
    this.deviceRecord['param'] = {};
    this.deviceRecord['param']['option1'] = $( '#ID_CHECK_OPTION1' + this.dynid ).choice( "option", "value" );
    this.deviceRecord['param']['option2'] = $( '#ID_CHECK_OPTION2' + this.dynid ).choice( "option", "value" );
 
    // ---- widget statepoint ----

    // (re-)create statepoint "empty"
    // The statepoint should always be recreated to remove 
    // params that are not longer used. 
    let spEmpty = {
      name: "empty",
      address: `/${ Data.Instance().RootEntid() }/${ this.entity['entid'] }/empty`,
      type: "BOOL",
      template: "empty",
      default: false,
      persistent: false,
      hidden: false,
      label: "{$aui_widget_newpic}"
    };

    // write safely into entity
    if( !( "sp" in this.entity ) )  {
      this.entity['sp'] = {};
    }
    this.entity['sp']["empty"] = spEmpty;

    // return our copy of entity
    return this.entity;
  }


  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  FormFillControls()
  {
    // define default values
    let option1 = true;
    let option2 = false;

    // red param from entity
    if( 'param' in this.deviceRecord )  {
      option1 = this.deviceRecord['param']['option1'] || option1;
      option2 = this.deviceRecord['param']['option2'] || option2;
    }

    // fill into controls
    $( '#ID_CHECK_OPTION1' + this.dynid ).choice( "option", "value", option1 );
    $( '#ID_CHECK_OPTION2' + this.dynid ).choice( "option", "value", option2 );
  }



}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigEmpty.Register( "/widget/deviceConfig/empty" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigEmpty );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigNone extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules = "*";
  static allowedTypes = "*";
  static widgetIcon = "mdi mdi-close";
  static widgetNameShort = "{$widget_none_short}";
  static widgetNameLong = "{$widget_none_long}";
  static widgetId = "none";

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything from entity
  //
  static Remove( uid, trigger, entity )
  {
    return true;
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/none";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    // no external ressources
    return null;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: $.lang( "{$widget_none_description}" ),
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigNone.Register( "/widget/deviceConfig/none" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigNone );
﻿//=============================================================================
//
//  CONTROL CALLTARGET IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlCalltargetList extends Viewable
{
  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  dataset = null;
  isInitialized = false;

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    if( ( 'forbiddenEntityId' in createOptions )&& !Array.isArray( createOptions['forbiddenEntityId'] ) ) {
      createOptions['forbiddenEntityId'] = [ createOptions['forbiddenEntityId'] ];
    }

    // get shared dataset and create a view
    this.dataset = Data.Instance().CalltargetDataset();
    let dataview = this.dataset.BuildView( 'default', { 
      rebuildAlways: true,
      sortOrder: "+",
      sortKey: "text",
      addressBook: ( 'addressBook' in createOptions ) ? createOptions['addressBook'] : null,
      allowedClass: ( 'allowedClass' in createOptions ) ? createOptions['allowedClass'] : null,
      forbiddenClass: ( 'forbiddenClass' in createOptions ) ? createOptions['forbiddenClass'] : null,
      forbiddenEntityId: ( 'forbiddenEntityId' in createOptions ) ? createOptions['forbiddenEntityId'] : null,
      filter: this.ControlDataviewFilter.bind( this )
    } );

    this.pinned = [];
    if( ( 'pinnedEntityId' in createOptions ) && ( typeof createOptions['pinnedEntityId'] == "string" ) ) {
      // loop though all and mark the pinned
      for( let index = 0; index < dataview.length; index++ ) {
        let record = dataview.GetByIndex( index );
        if( record['id'] == createOptions['pinnedEntityId'] ) {
          this.pinned.push( record['id'] );
        }
      }
    }

    // create listbox using the created view
    $( "#ID_DROPLIST" + this.dynid ).listbox( {
      size: ( 'size' in createOptions ) ? createOptions['size'] : CONST.size.L,
      height: ( 'height' in createOptions ) ? createOptions['height'] : 5,
      type: 'droplist',
      selection: 'single',
      selected: createOptions['selected'],
      placeholder: ( 'placeholder' in createOptions ) ? createOptions['placeholder'] : "{$const_common_empty_placeholder}",
      dataDeletedItem: { icon: "mdi mdi-help", text: "{$const_deleted_item}" },
      search: true,
      searchPlaceholder: "{$const_common_search_placeholder}",
      data: dataview,
      dataPinned: this.pinned,
      validate: this.ValidateDroplist.bind( this ),
      result: this.ControlDroplistResult.bind( this )
    } );

    this.isInitialized = true;
  }

  //-----------------------------------------------------------------
  //  VIEWABLE OVERRIDABLE
  //-----------------------------------------------------------------

  OnEvent( eventName, param, data )
  {
    // This event can be called before load has been called.
    // Viewable calles load from a timer. 
    if( !this.isInitialized )
      return;

    // all initialized: set option
    if( eventName == "option" && param == "selected" ) {
      if( data )  {
        $( "#ID_DROPLIST" + this.dynid ).listbox( "option", "selected", data );
      }
      else  {
        return $( "#ID_DROPLIST" + this.dynid ).listbox( "option", "selected" );
      }
    }
  }


  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  filter function called by dataview
  //
  ControlDataviewFilter( id, record, options, dataviewClass )
  {
    let isAllowed;

    // addrbook: 
    if( record['class'] == "addrBook" ) {
      isAllowed = ( options['addressBook'] == record['entid'] );  // same addressbook
    }
    // not addrbook: some classes are forbidden
    else if( options['forbiddenClass'] !== null ) {
      isAllowed = ( options['forbiddenClass'].indexOf( record['class'] ) < 0 ); // NOT in forbidden
    }
    // not addrbook: only some classes are allowed
    else if( options['allowedClass'] !== null ) {
      isAllowed = ( options['allowedClass'].indexOf( record['class'] ) >= 0 );  // IS in allowed
    }
    // no filter
    else {
      isAllowed = true;
    }

    // finally check all forbidden entityIds
    if( isAllowed && ( 'forbiddenEntityId' in options ) && Array.isArray( options['forbiddenEntityId'] ) ) {
      isAllowed = ( options['forbiddenEntityId'].indexOf( record['entid'] ) < 0 );
    }

    // result
    return isAllowed;
  }

  //-----------------------------------------------------------------
  //  result callback from droplist 
  //
  ControlDroplistResult( selected )
  {
    if( selected.length > 0 )  {
      let record = this.dataset.Get( selected[ 0 ] );
      this.createOptions['selectedCallnumber'] = record['callnumber'];
      this.FireEvent( "result", record );
    }
    else {
      this.FireEvent( "result", null );
    }

  }


  //-----------------------------------------------------------------
  //  VALIATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    if( param === false ) {
      return $( "#ID_DROPLIST" + this.dynid ).listbox( "validate", false );
    }
    return $( "#ID_DROPLIST" + this.dynid ).listbox( "validate" );
  }
  
  //-----------------------------------------------------------------
  //  validation handler for droplist
  //
  ValidateDroplist( validateControl )
  {
    let selected = $( "#ID_DROPLIST" + this.dynid ).listbox( "option", "selected" );
    let numErr = ControlCalltargetList.Validate( validateControl, selected );

    // special: check if callnumber has been changed. 
    // needed for widgets that stores callnumber in entity beside entity-id
    if( ( numErr == 0 ) && ( typeof this.createOptions['selectedCallnumber'] == "string" ) ) {
      let entinfo = Data.Instance().Entinfo( selected[ 0 ] );
      if( entinfo['callnumber'] != this.createOptions['selectedCallnumber'] )  {
        validateControl.error = "{$control_calltarget_callnumber_changed}";
        numErr = 1;
        $( "#ID_DROPLIST" + this.dynid ).listbox( "option", "selected", [] );
      }
    }
    return numErr;
  }
  
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = "/widget/control/calltargetList";
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    if( Array.isArray( selected ) && selected.length > 0 ) {
      if( !( Data.Instance().CalltargetDataset().Has( selected[ 0 ] ) ) ) {
        validateControl.error = "{$control_calltarget_error_deleted}";
        return 1;
      }
    }
    
    if( Array.isArray( selected ) && selected.length == 0 ) {
      validateControl.error = "{$control_calltarget_error_empty}";
      return 1;
    }

    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlCalltargetList.Register( "/widget/control/calltargetList" );
﻿//=============================================================================
//
//  CONTROL CALLTARGET IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlStatepointList extends Viewable
{
  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  dataset = null;
  isInitialized = false;
  localData = null;
  permission = "w";

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    this.pinned = [];
    this.inquirer = createOptions['inquirer'] || "";

    this.permission = ( 'permission' in createOptions ) ? createOptions['permission'] : "w";

    // create local data (if desired)
    if( 'entity' in createOptions )  {

      this.localData = [];
      let localName = createOptions['entity']['name'];
      let isPinned = ( createOptions['pinnedEntityId'] == createOptions['entity']['entid'] );

      // our statepoints we're creating all locally
      if( !( 'forbiddenEntityId' in createOptions ) ) {
        createOptions['forbiddenEntityId'] = [];
      }
      createOptions['forbiddenEntityId'].push( createOptions['entity']['entid'] );

      for( let spName in createOptions['entity']['sp'] )  {
        
        // get record and replace unsaved name in label 
        let spRecord = createOptions['entity']['sp'][ spName ];
        let label = $.lang( spRecord['label'] ) || null;
        if( label == null || spRecord['hidden'] == true )  {
          continue;
        }

        // replace {{name}} macro
        label = label.replace( "{{name}}", localName );

        // create a local record and push it to list
        let newSpInfoRecord = {
          'address': spRecord['address'],
          'name': spName,
          'type': spRecord['type'],
          'entid': createOptions['entity']['entid'],
          'class': createOptions['entity']['class'],
          'label': label,
          'icon': CONST_CLASS_ICO[ createOptions['entity']['class'] ],
          'local': true
        };
        this.localData.push( newSpInfoRecord );

        // a local entry might be pinnded as well
        if( isPinned )  {
          this.pinned.push( spRecord['address'] );
        }
      }
    }
    else {
      this.data = null; 
    }

    if( 'noneRecord' in createOptions && createOptions['noneRecord'] == true ) {

      if( !Array.isArray( this.localData ) )  {
        this.localData = [];
      }

      // create a local record and push it to list
      let noneSpInfoRecord = {
        'address': "none",
        'type': "NONE",
        'entid': "none",
        'class': "none",
        'label': "-",
        'icon': "mdi mdi-blank",
        'local': true
      };
      this.localData.push( noneSpInfoRecord );
      this.pinned.push( noneSpInfoRecord['address'] );

      if( !( 'allowedTypes' in createOptions ) || ! Array.isArray( createOptions['allowedTypes'] ) ) {
        createOptions['allowedTypes'] = [];
      }
      createOptions['allowedTypes'].push( "NONE" );
    }
    
    // get shared dataset and create a view
    this.dataset = Data.Instance().StatepointDataset();
    let dataview = this.dataset.BuildView( 'default', { 
      rebuildAlways: true,
      sortOrder: "+",
      sortKey: "label",
      inquirer: createOptions['inquirer'],
      forbiddenTypes: createOptions['forbiddenTypes'],
      allowedTypes: createOptions['allowedTypes'],
      forbiddenNames: createOptions['forbiddenNames'],
      allowedNames: createOptions['allowedNames'],
      forbiddenEntityId: createOptions['forbiddenEntityId'],
      permission: ( 'permission' in createOptions ) ? createOptions['permission'] : "w",
      filter: this.ControlDataviewFilter.bind( this ),
      data: this.localData
    } );

    // get all pinned items from shared dataset
    if( ( 'pinnedEntityId' in createOptions ) && ( typeof createOptions['pinnedEntityId'] == "string" ) ) {

      // if this is an entityId, then add a trailing backslash
      let statePointeAddressPattern = /\b.+\/[a-zA-Z]+\.\d+\/[-_a-zA-Z0-9]+\b/g;
      if( createOptions['pinnedEntityId'].match( statePointeAddressPattern ) === null )  {
        createOptions['pinnedEntityId'] += "/";
      }
      
      // loop though all and mark the pinned
      for( let index = 0; index < dataview.length; index++ ) {
        let record = dataview.GetByIndex( index );
        let pinnedSearch = createOptions['pinnedEntityId'] + "/";
        if( record['address'].indexOf( pinnedSearch ) >= 0 )  {
          this.pinned.push( record['address'] );
        }
      }
    }

    // create listbox using the created view
    $( "#ID_DROPLIST" + this.dynid ).listbox( {
      size: ( 'size' in createOptions ) ? createOptions['size'] : CONST.size.L,
      height: ( 'height' in createOptions ) ? createOptions['height'] : 5,
      status: ( 'status' in createOptions ) ? createOptions['status'] : "normal",
      type: 'droplist',
      selection: 'single',
      selected: createOptions['selected'],
      placeholder: ( 'placeholder' in createOptions ) ? createOptions['placeholder'] : "{$const_common_empty_placeholder}",
      dataDeletedItem: { icon: "mdi mdi-help", text: "{$const_deleted_item}" },
      search: true,
      searchPlaceholder: "{$const_common_search_placeholder}",
      data: dataview,
      dataPinned: this.pinned,
      validate: this.ValidateDroplist.bind( this ),
      result: this.ControlDroplistResult.bind( this )
    } );

  }

  //-----------------------------------------------------------------
  //  VIEWABLE OVERRIDABLE
  //-----------------------------------------------------------------

  OnEvent( eventName, param, data )
  {
    // all initialized: set option
    if( eventName == "option" && param == "selected" )  {
      if( data )  {
        $( "#ID_DROPLIST" + this.dynid ).listbox( "option", "value", data );
      }
      else  {
        return $( "#ID_DROPLIST" + this.dynid ).listbox( "option", "value" );
      }
    }
    // all initialized: set option
    if( eventName == "option" && param == "status" ) {
      if( data ) {
        $( "#ID_DROPLIST" + this.dynid ).listbox( "option", "status", data );
      }
      else {
        return $( "#ID_DROPLIST" + this.dynid ).listbox( "option", "status" );
      }
    }

  }
 

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  filter function called by dataview
  //
  ControlDataviewFilter( id, record, options, dataviewClass )
  {
    let isAllowed = false;

    // ---- filter for type ----
    
    // negative filter first
    if( 'forbiddenTypes' in options ) {
      isAllowed = ( options['forbiddenTypes'].indexOf( record['type'] ) < 0 );
      //if( !isAllowed ) console.log( "- Filter, forbiddenTypes", record['address'] );
    }

    // positive filter second
    else if( 'allowedTypes' in options )  {
      isAllowed = ( options['allowedTypes'].indexOf( record['type'] ) >= 0 );
      //if( !isAllowed ) console.log( "- Filter, allowedTypes", record['address'] );
    }

    // ---- filter for name ----

    // negative filter first
    if( isAllowed && ( 'forbiddenNames' in options ) ) {
      isAllowed = ( options['forbiddenNames'].indexOf( record['name'] ) < 0 );
      //if( !isAllowed ) console.log( "- Filter, forbiddenNames", record['address'] );
    }

    // positive filter second
    else if( isAllowed && 'allowedNames' in options ) {
      isAllowed = ( options['allowedNames'].indexOf( record['name'] ) >= 0 );
      //if( !isAllowed ) console.log( "- Filter, allowedNames", record['address'] );
    }

    // ---- filter for entid ----

    // negative filter only
    if( isAllowed && ( 'forbiddenEntityId' in options ) && record['local'] !== true ) {
      isAllowed = ( options['forbiddenEntityId'].indexOf( record['entid'] ) < 0 );
      //if( !isAllowed ) console.log( "- Filter, forbiddenEntityId", record['address'] );
    }


    // ---- finally permissions ----
    
    if( isAllowed && options['inquirer'] != "*" && record['local'] !== true )  {
      let permission = "w";
      if( 'permission' in options && options['permission'] == "r" )  {
        permission = "r";
      }
      isAllowed = ( Data.Instance().PermissionList().IsAllowed( options['inquirer'], record['address'], permission ) );
      //if( !isAllowed ) console.log( "- Filter, permissions \"", permission, "\"", record['address'] );
    }

    // result
    return isAllowed;
  }

  //-----------------------------------------------------------------
  //  result callback from droplist 
  //
  ControlDroplistResult( selected )
  {
    if( selected.length >= 0 )  {
      if( this.dataset.Has( selected[0]) ) {
        let record = this.dataset.Get( selected[ 0 ] );
        this.FireEvent( "result", record );
      }
      else if( this.localData ) {
        let record = this.localData.find( ( rec ) => rec['address'] == selected[ 0 ] );
        this.FireEvent( "result", record );
      }
      else {
        this.FireEvent( "result", null );
      }
    }
    else {
      this.FireEvent( "result", null );
    }

  }


  //-----------------------------------------------------------------
  //  VALIATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    if( param === false )  {
      return $( "#ID_DROPLIST" + this.dynid ).listbox( "validate", false );
    }
    return $( "#ID_DROPLIST" + this.dynid ).listbox( "validate" );
  }
  
  //-----------------------------------------------------------------
  //  validation handler for droplist
  //
  ValidateDroplist( validateControl )
  {
    let selected = $( "#ID_DROPLIST" + this.dynid ).listbox( "option", "selected" );
    
    if( Array.isArray( selected ) && selected.length > 0 ) {

      let isLocalData = false;

      // check if selection is in global or local data
      if( !( Data.Instance().StatepointDataset().Has( selected[0] ) ) ) {
        if( this.localData != null ) {
          let record = this.localData.find( ( rec ) => rec['address'] == selected[0] );
          if( !record ) {
            validateControl.error = "{$const_error_statepoint_deleted}";
            return 1;
          }
          else {
            isLocalData = true;
          }
        }
        else {
          validateControl.error = "{$const_error_statepoint_deleted}";
          return 1;
        }
      }

      // check if we've got permission to use
      let permissions = Data.Instance().PermissionList();
      if( ! ( isLocalData ) 
       && !( permissions.IsAllowed( this.inquirer, selected[0], this.permission ) ) 
        ) {
        validateControl.error = "{$const_error_statepoint_permission}";
        return 1;
      }

    }

    // no selection
    if( Array.isArray( selected ) && selected.length == 0 ) {
      validateControl.error = "{$control_statepoint_error_empty}";
      return 1;
    }

    return 0;

  }
  
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = "/widget/control/statepointList";
    $element.viewable( createOptions );
  }


}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlStatepointList.Register( "/widget/control/statepointList" );
﻿//=============================================================================
//
//  CONTROL ENTITY LIST IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlEntityList extends Viewable
{
  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  dataset = null;
  isInitialized = false;

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // get shared dataset and create a view
    this.dataset = Data.Instance().EntityDataset();
    let dataview = this.dataset.BuildView( 'default', { 
      rebuildAlways: true,
      sortOrder: "+",
      sortKey: "text",
      allowedClass: createOptions['allowedClass'],
      filter: this.ControlDataviewFilter.bind( this )
    } );

    this.pinned = [];
    if( ( 'pinnedEntityId' in createOptions ) && ( typeof createOptions['pinnedEntityId'] == "string" ) ) {

      // loop though all and mark the pinned
      for( let index = 0; index < dataview.length; index++ ) {
        let record = dataview.GetByIndex( index );
        if( record['id'] == createOptions['pinnedEntityId'] ) {
          this.pinned.push( record['id'] );
        }
      }

    }

    // create listbox using the created view
    $( "#ID_DROPLIST" + this.dynid ).listbox( {
      size: ( 'size' in createOptions ) ? createOptions['size'] : CONST.size.L,
      height: ( 'height' in createOptions ) ? createOptions['height'] : 5,
      type: 'droplist',
      selection: 'single',
      selected: createOptions['selected'],
      placeholder: ( 'placeholder' in createOptions ) ? createOptions['placeholder'] : "{$const_common_empty_placeholder}",
      dataDeletedItem: { icon: "mdi mdi-help", text: "{$const_deleted_item}" },
      //search: true,
      //searchPlaceholder: "{$const_common_search_placeholder}",
      data: dataview,
      dataPinned: this.pinned,
      validate: this.ValidateDroplist.bind( this ),
      result: this.ControlDroplistResult.bind( this )
    } );

    this.isInitialized = true;
  }

  //-----------------------------------------------------------------
  //  VIEWABLE OVERRIDABLE
  //-----------------------------------------------------------------

  OnEvent( eventName, param, data )
  {
    // This event can be called before load has been called.
    // Viewable calles load from a timer. 
    if( !this.isInitialized )
      return;

    // all initialized: set option
    if( eventName == "option" && param == "selected" ) {
      if( data )  {
        $( "#ID_DROPLIST" + this.dynid ).listbox( "option", "selected", data );
      }
      else {
        return $( "#ID_DROPLIST" + this.dynid ).listbox( "option", "selected" );
      }
    }
  }


  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  filter function called by dataview
  //
  ControlDataviewFilter( id, record, options, dataviewClass )
  {
    let isAllowed;

    // filter only allowed classes
    if( 'allowedClass' in options )  {
      isAllowed = ( options['allowedClass'].indexOf( record['class'] ) >= 0 );
    }
    // no filter
    else {
      isAllowed = true;
    }

    // result
    return isAllowed;
  }

  //-----------------------------------------------------------------
  //  result callback from droplist 
  //
  ControlDroplistResult( selected )
  {
    if( selected.length >= 0 )  {
      let record = this.dataset.Get( selected[ 0 ] );
      this.FireEvent( "result", record );
    }
    else {
      this.FireEvent( "result", null );
    }

  }


  //-----------------------------------------------------------------
  //  VALIATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    if( param === false ) {
      return $( "#ID_DROPLIST" + this.dynid ).listbox( "validate", false );
    }
    return $( "#ID_DROPLIST" + this.dynid ).listbox( "validate" );
  }
  
  //-----------------------------------------------------------------
  //  validation handler for droplist
  //
  ValidateDroplist( validateControl )
  {
    let selected = $( "#ID_DROPLIST" + this.dynid ).listbox( "option", "selected" );
    return ControlEntityList.Validate( validateControl, selected );
  }
  
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = "/widget/control/entityList";
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    if( Array.isArray( selected ) && selected.length > 0 ) {
      if( !( Data.Instance().EntityDataset().Has( selected[ 0 ] ) ) ) {
        validateControl.error = "{$control_entitylist_error_deleted}";
        return 1;
      }
    }

    if( Array.isArray( selected ) && selected.length == 0 ) {
      validateControl.error = "{$control_entitylist_error_empty}";
      return 1;
    }

    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlEntityList.Register( "/widget/control/entityList" );
﻿//=============================================================================
//
//  CONTROL CALL PROFILE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlEntityName extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/entityName";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  isInitialized = false;
  name = null;
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    let controlStatus = createOptions['readonly'] ? "readonly" : "normal";
    let size = createOptions['size'] || CONST.size.L;
    this.$editCtrl = this.$container.find( ".edit" );
    this.$editCtrl.edit( {
      size: size,
      status: controlStatus,
      value: ( this.name ) ? this.name : "",
      validate: Validate.NotEmptyString,
      result: this.ControlEditResult.bind( this )
    } );


    this.isInitialized = true;
  }

  OnEvent( eventName, data, param )  
  {
    // implement standard ( "option", "value" .. ) handler
    if( eventName == "option" && data == "value" ) {
      if( param ) {
        this.name = param;
        if( this.isInitialized )  {
          return this.$editCtrl.edit( "option", "value", param );
        }
      }
      else {
        return this.$editCtrl.edit( "option", "value" );
      }
    }
  }

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------

  ControlEditResult( value )
  {
    value = value.trim();
    this.FireEvent( "result", value );
  }

  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    if( param === false ) {
      return this.$editCtrl.edit( "validate", false );
    }
    return this.$editCtrl.edit( "validate" );
  }

  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlEntityName.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlEntityName.Register( ControlEntityName.template );
﻿//=============================================================================
//
//  CONTROL CALL PROFILE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlLabel extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/label";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  isInitialized = false;
  label = null;
  entityName = null;
  functionName = null;
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // label of statepoint
    this.label = createOptions['label'];
    this.entityName = createOptions['entityName'];
    this.functionName = createOptions['functionName'];

    // label fields are translated by server into the desired
    // language using "dict.json". All these language strings
    // are starting with "aui_widget..".
    if( this.functionName && this.functionName.substring( 0, 5 ) != "{$aui" )  {
      console.warn( "Label macro $FUNCTION need to be a dict.json string!" );
    }

    let controlStatus = createOptions['readonly'] ? "readonly" : "normal";
    this.$editCtrl = this.$container.find( ".edit" );
    this.$editCtrl.edit( {
      size: CONST.size.L,
      status: controlStatus,
      validate: Validate.NotEmptyString
    } );

    if( 'text' in createOptions )  {
      this.$labelCtrl = this.$container.find( "label" );
      this.$labelCtrl.text( $.lang( createOptions['text'] ) );
    }

    // permission check for statepoint
    if( 'permissionCheck' in createOptions )  {
      this.$permissionCheck = this.$container.find( ".permissionCheck" );
      ControlPermissionCheck.Build( this.$permissionCheck, { spname: createOptions['permissionCheck']['spname'] } );
      this.$permissionCheck.viewable( "event", "permission-check", createOptions['permissionCheck']['entinfo'] );
    }

    // fill control
    this.FormFillLabel( createOptions['label'] );

  }

  //-----------------------------------------------------------------
  //  control event handler
  //
  OnEvent( eventName, data, param )  
  {
    // implement standard ( "option", "value" .. ) handler
    if( eventName == "option" && data == "value" ) {
      if( param )  {
        this.FormFillLabel( param );
      }
      return this.FormReadLabel( );
    }
    else if( eventName == "permission-check" ) {
      this.$permissionCheck.viewable( "event", "permission-check", data );
    }
  }

  //-----------------------------------------------------------------
  //  FORM HELPER
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  prepare label edit control
  //
  FormFillLabel( label )
  {
    this.label = label;
    //this.label = this.label.replace( this.entityName, "$NAME" );
    this.label = this.label.replace( "{{name}}", "$NAME" );
    if( this.functionName ) {
      this.label = this.label.replace( this.functionName, "$FUNCTION" );
      this.$editCtrl.edit( "option", "macros", {
        "$NAME": this.entityName,
        "$FUNCTION": $.lang( this.functionName )
      } );
    }
    else {
      this.$editCtrl.edit( "option", "macros", {
        "$NAME": this.entityName
      } );
    }

    this.$editCtrl.edit( "option", "value", this.label );
  }

  //-----------------------------------------------------------------
  //  read edit and build final label
  //
  FormReadLabel()
  {
    // read label and replace macros
    let label = this.$editCtrl.edit( "option", "value" );
    label.trim();
    //label = label.replace( "$NAME", this.entityName );
    label = label.replace( "$NAME", "{{name}}" );
    label = label.replace( "$FUNCTION", this.functionName );
    return label;
  }

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------

  ControlEditResult( value )
  {
    // prepare label and replace macros
    let label = value.trim();
    label = label.replace( "$NAME", this.entityName );
    label = label.replace( "$FUNCTION", this.functionName );

    this.FireEvent( "result", label );
  }

  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    if( param === false ) {
      return this.$editCtrl.edit( "validate", false );
    }
    return this.$editCtrl.edit( "validate" );
  }

  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlLabel.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlLabel.Register( ControlLabel.template );
//=============================================================================
//
//  CONTROL CALL PROFILE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlVideoUrlAddress extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/videoUrlAddress";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  isInitialized = false;
  entity = null;
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    let controlStatus = "readonly";
    this.$root = this.$container.find( ".videoUrl" );
    this.isInitialized = true;
    if( createOptions && Array.isArray( createOptions['data'] ) )  {
      for( let ip of createOptions['data'] )  {
        let tempLine = this.CreateUrlLine( ip );
        this.$root.append( tempLine );
        this.$root.append( document.createElement("br") );
      }
    }
  }

  OnEvent( eventName, data, param )  
  {
    //no option needed!

    // implement standard ( "option", "value" .. ) handler
    // if( eventName == "option" && data == "value" ) {
    //   if( param )  {
    //     return this.$editCtrl.edit( "option", "value", param );
    //   }
    //   return this.$editCtrl.edit( "option", "value" );
    // }
  }

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------

  ControlEditResult( value )
  {
    value = value.trim();
    this.FireEvent( "result", value );
  }

  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    // no validation because user can't change anything

    // if( param === false ) {
    //   return this.$editCtrl.edit( "validate", false );
    // }
    // return this.$editCtrl.edit( "validate" );
  }

  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlVideoUrlAddress.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    return 0;
  }

  //=============================================================================
  //  DYNAMIC HTML
  //=============================================================================
  CreateUrlLine( ip )
  {
    let $line = document.createElement( "div" );
    let $input = document.createElement( "div" );
    let $space = document.createElement( "s" );
    let $copyButton = document.createElement( "div" );
    let $toast = this.createOptions['toast'];

    let httpguid =  this.createOptions['entity']['video']['httpGuid'] ?? false;
    let httpFormat = this.createOptions['entity']['video']['httpFormat'] ?? "mjpeg";
    let templateVideo = `http://${ip}:8080/${httpguid}/video.mjpeg`;
    let templateImage = `http://${ip}:8080/${httpguid}/image.jpg`;

    if( !httpguid )  {
      console.error( "httpguid doesn't exists!" );
      return;
    }

    $line.className = "h-layout";
    $space.className = "s";
    this.$root.append( $line );
    $line.append( $input );
    $line.append( $copyButton );

    $($input).edit( {
      size: CONST.size.L,
      status: "readonly",
      value: httpFormat === "mjpeg" ? templateVideo : templateImage,
    } );

    $( $copyButton ).button( {
      icon: "mdi mdi-content-copy",
      click: () => {
        let url = $( $input ).edit( "option", "value" );
        navigator.clipboard.writeText( url );
      },
    } );
    $line.append( $input );
    $line.append( $space );
    $line.append( $copyButton );
    return $line;
  }
}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlVideoUrlAddress.Register( ControlVideoUrlAddress.template );
﻿//=============================================================================
//
//  CONTROL CALL PROFILE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlPermissionCheck extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/permissionCheck";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  $span = null;
  spname = null;

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    this.$span = this.$container.find( ".permIcon" );
    this.spname = createOptions['spname'];
  }

  OnEvent( eventName, data, param )  
  {
    if( eventName == "permission-check" ) {
      this.FormPermissionCheck( data );
    }
  }

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------

  FormPermissionCheck( entinfo )
  {
    if( !this.$span )  {
      setTimeout( () => {
        this.FormDoCheck( entinfo );
      }, 250 );
    }
    else {
      this.FormDoCheck( entinfo );
    }
  }

  FormDoCheck( entinfo )
  {
    if( !entinfo || !this.spname ) {
      return;
    }

    // check if a (2nd) permission record for this sp exists
    if( this.spname in entinfo['sp'] ) {
      let isHidden = entinfo['sp']['hidden'] || false;
      if( isHidden ) {
        this.$span.addClass( "hidden" );
      }
      else {
        let filteredPermissions = entinfo['permission'].filter( ( rec ) => rec['f'] == this.spname );
        if( filteredPermissions.length < 2 )  {
          this.$span.removeClass( "hidden" );
        }
        else {
          this.$span.addClass( "hidden" );
        }
      }
    }

  }

  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    return 0;
  }

  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Check( spname, entinfo )
  {
    if( ( 'sp' in entinfo ) && ( spname in entinfo['sp'] ) && ( 'permission' in entinfo ) ) {
      let isHidden = ( 'hidden' in entinfo['sp'][ spname ]  ) ? entinfo['sp'][ spname ]['hidden'] : false;
      if( isHidden ) {
        return false;
      }
      else {
        let filteredPermissions = entinfo['permission'].filter( ( rec ) => ( rec['f'] == spname && rec['w'] != "*" ) );
        return ( filteredPermissions.length == 0 );
      }
    }
    return false;
  }

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlPermissionCheck.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlPermissionCheck.Register( ControlPermissionCheck.template );
﻿//=============================================================================
//
//  CONTROL CALLTARGET IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlAddressbookChoose extends Viewable
{
  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  dataset = null;
  isInitialized = false;
  addrbkEntid = null;
  lastResult = null;

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    //---------------------------------------------------------------
    //  data
    //---------------------------------------------------------------

    this.isInitialized = false;
    this.addrbkEntid = createOptions['addrBook'];
    this.anchor = createOptions['anchor'];
    this.dataset = new Dataset( {
      id: "id",
      preview: ( record ) => {
        return {
          icon: "mdi mdi-book-open",
          text: record['label']
        };
      }
    } ); 

    //---------------------------------------------------------------
    //  init controls
    //---------------------------------------------------------------

    $( this.anchor ).button( {
      size: CONST.size.XS,
      icon: "mdi mdi-book-open",
      click: () => {
        this.FormShowListbox();
      }
    } );

    //setTimeout( () => {
    // create listbox using the created view
    /*$( "#ID_DROPLIST" + this.dynid ).listbox( {
      size: CONST.size.L,
      height: 5,
      type: 'listbox',
      anchor: anchor,
      position: "bottomright",
      data: this.dataset,
      selection: 'single',
      placeholder: ( 'placeholder' in createOptions ) ? createOptions['placeholder'] : "{$const_common_empty_placeholder}",
      searchPlaceholder: "{$const_common_search_placeholder}",
      search: true,
      result: this.ControlDroplistResult.bind( this )
    } );*/

    //}, 500 );


    //---------------------------------------------------------------
    //  init controls
    //---------------------------------------------------------------

    // get addressbook data
    this.DataRequest();

  }

  //-----------------------------------------------------------------
  //  VIEWABLE OVERRIDABLE
  //-----------------------------------------------------------------

  OnEvent( eventName, param, data )
  {
    // query result from outside
    if( eventName == "option" && param == "result" ) {
      return this.lastResult;
    }
  }

  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  FormShowListbox()
  {
    if( this.isInitialized )  {
      $( "#ID_DROPLIST" + this.dynid ).listbox( "show" );
    }
    else {
      $( "#ID_DROPLIST" + this.dynid ).listbox( {
        size: CONST.size.L,
        height: 5,
        type: 'listbox',
        anchor: this.anchor,
        position: "bottomright",
        data: this.dataset,
        selection: 'single',
        placeholder: "{$const_common_empty_placeholder}",
        searchPlaceholder: "{$const_common_search_placeholder}",
        search: true,
        result: this.ControlDroplistResult.bind( this )
      } );
      $( "#ID_DROPLIST" + this.dynid ).listbox( "show" );
      this.isInitialized = true;
    }
  }

  //-----------------------------------------------------------------
  //  DATA
  //-----------------------------------------------------------------

  DataRequest()
  {
    Data.Instance().Addressbook().AddressbookRequest( this.addrbkEntid, ( result ) => {
      this.dataset.SetData( result );
    } );    
  }

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------


  //-----------------------------------------------------------------
  //  result callback from droplist 
  //
  ControlDroplistResult( selected )
  {
    if( selected.length > 0 )  {
      this.lastResult = this.dataset.Get( selected[ 0 ] );
    }
    else {
      this.lastResult = null;
    }

    this.FireEvent( "result", this.lastResult );
  }


  //-----------------------------------------------------------------
  //  VALIATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    return 0;
  }
  
  
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = "/widget/control/addrbkchoose";
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlAddressbookChoose.Register( "/widget/control/addrbkchoose" );
﻿//=============================================================================
//
//  PAGE HEADER view
//  (c) 2019 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          26.01.2019
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class JsonControl extends Viewable
{
  //-----------------------------------------------------------------
  //  LOAD 
  //
  Load( options )
  {
    // --- build control ---
    if( 'height' in options ) {
      this.$container.find( ".scroll-container" ).css( "height", $.metrics( 'lineHeight' ) * options['height'] );
    }
    if( 'width' in options ) {
      this.$container.css( "width", $.metrics( 'fontWidth' ) * options['width'] );
    }

    // --- data ---
    if( 'data' in options )  {
      this.OnEventData( options['data'] );
    }

  }//load


  //-----------------------------------------------------------------
  //  EVENT FUNCTION 
  //  
  OnEvent( msg, param, data )
  {
    //console.log( msg, param, data );
    if( msg == "option" && param == "data" )  {
      this.OnEventData( data );
    }
  }

  //-----------------------------------------------------------------
  //  EVENT: Set new json data
  //  
  OnEventData( data )
  {
    this.json = data;
    if( $.isPlainObject( this.json ) ) {
      this.json = JSON.stringify( this.json, undefined, 2 );
    }
    $( "#ID_JSON" + this.dynid ).html( this.HelperSyntaxHighlight( this.json ) );
    this.$scrollbar.scrollbar( "recalc" );
  }


  //-----------------------------------------------------------------
  //  HELPER
  //-----------------------------------------------------------------

  HelperSyntaxHighlight( json ) 
  {
    if( !json )   return "";

    json = json.replace( /&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;' );
    return json.replace( /("(\\u[a-zA-Z0-9]{4}|\\[^u]|[^\\"])*"(\s*:)?|\b(true|false|null)\b|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?)/g, function( match ) {
      var cls = 'number';
      if( /^"/.test( match ) ) {
        if( /:$/.test( match ) ) {
          cls = 'key';
        } 
        else {
          cls = 'string';
        }
      } 
      else if( /true|false/.test( match ) ) {
        cls = 'boolean';
      } 
      else if( /null/.test( match ) ) {
        cls = 'null';
      }
      return '<span class="' + cls + '">' + match + '</span>';
    } );
  }

  //-----------------------------------------------------------------
  //  STATIC BUILDER
  //-----------------------------------------------------------------

  static Build( selector, options )
  {
    if( ! $.isPlainObject( options ) ) {
      options = {};
    }
    options['view'] = '/widget/jsoncontrol';
    $( selector ).viewable( options );
  }

}


//=============================================================================
//  BUILDER 
//=============================================================================

JsonControl.Register( '/widget/jsoncontrol' );
﻿//=============================================================================
//
//  CONTROL MAC ADDRESS IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlMacAddress extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/macAddress";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    this.entity = createOptions['entity'];
    let mac = this.entity['adminUi']['macaddr'] || "";
    this.isNew = ( mac.length == 0 );
    this.isUserparamReset = false;
    this.resetParam = null;

    $( "#ID_MACADDR" + this.dynid ).edit( {
      size: CONST.size.L,
      status: this.isNew ? "normal" : "readonly",
      value: mac,
      validate: this.ValidateMacAddess.bind( this ),
      result: ( value ) => {
        var newValue = "";
        for( var i = 0; i < value.length; i++ )  {
          switch( value[ i ] ) {
            case 'a':
            case 'A': newValue += "A";  break;
            case 'b':
            case 'B': newValue += "B";  break;
            case 'c':
            case 'C': newValue += "C";  break;
            case 'd':
            case 'D': newValue += "D";  break;
            case 'e':
            case 'E': newValue += "E";  break;
            case 'f':
            case 'F': newValue += "F";  break;
            case ':':
            case '-': newValue += ":";  break;
            case '0':case '1':case '2':case '3':case '4':
            case '5':case '6':case '7':case '8':case '9':
              newValue += value[ i ];  
              break;
          }
        }
        if( newValue != value ) {
          $( "#ID_MACADDR" + this.dynid ).edit( "option", "value", newValue );
        }
      },      
      //result: ( value ) => {
      //  value = value.replaceAll( /-/gi,':' );
      //  $( "#ID_MACADDR" + this.dynid ).edit( "option", "value", value );
      //}
    } );

    $( "#ID_MACADDR_EDIT" + this.dynid ).button( {
      size: CONST.size.XS,
      icon: "mdi mdi-wrench",
      click: this.ControlButtonClick.bind( this )
    } );

    if( this.isNew )  {
      $( "#ID_MACADDR_EDIT" + this.dynid ).css( "display", "none" );
    }

    this.isInitialized = true;
  }


  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // send DEVRESTART_NEEDED
    if( eventName == "modified" && param > 0 )  {
      this.FireEvent( "restartNeeded" );
    }
        
    // save
    if( eventName == "option" && param == "save" ) {
      return this.FormSave( data );
    }

    return false;
  }

  //-----------------------------------------------------------------
  //  FORM FUNCTIONS
  //-----------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    var macaddr = $( "#ID_MACADDR" + this.dynid ).edit( "option", "value" );
    macaddr = macaddr.replaceAll( /-/gi,':' );
    entity['adminUi']['macaddr'] = macaddr; //$( "#ID_MACADDR" + this.dynid ).edit( "option", "value" );

    if( this.isUserparamReset )  {
      entity['userparam'] = this.resetParam;
      entity['userparam']['reset'] = true;
    }
    return true;
  }

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------

  ControlButtonClick( value )
  {
    MacAddressDialog.Show( { 
      param: {
        entid: this.entity['entid'],
        macaddr: this.entity['adminUi']['macaddr'] || "",
        reset: false,
      },
      result: this.ControlDialogResult.bind( this )      
    });
  }

  ControlDialogResult( data )
  {
    // reset part of dialog
    this.isUserparamReset = false;
    this.resetParam = data['reset'];
    for( var key in data['reset'] ) {
      if( data['reset'][ key ] )  {
        this.isUserparamReset = true;
        break;
      }
    }
    // mac address part
    $( "#ID_MACADDR" + this.dynid ).edit( "option", "value", data['macaddr'] );
    this.Option( "modified", 1 );

  }

  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    if( param === false ) {
      $( "#ID_MACADDR" + this.dynid ).edit( "validate", false );
      return 0;
    }
    else  {
      var numErr = 0;
      if( this.isNew )  {
        numErr += $( "#ID_MACADDR" + this.dynid ).edit( "validate" );
      }
      return numErr;
    }
  }

  //-----------------------------------------------------------------
  //  validate the mac-address edit
  //
  ValidateMacAddess( validateControl )  
  {
    // empty mac-addresses are allowed
    if( validateControl.value.length == 0 )  {
      return 0;
    }

    if( ! Validate.IsMacAddress( validateControl.value ) )  {
      validateControl.error = "{$const_error_novalidmac}";
      return 1;
    }

    var macInfo = Data.Instance().MacAddressInfo( validateControl.value );
    if( macInfo && macInfo['entid'] != this.entity['entid'] )  {
      validateControl.error = "{$macaddr_error_exists}";
      return 1;
    }
    return 0;
  }

  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlMacAddress.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlMacAddress.Register( ControlMacAddress.template );
﻿//=============================================================================
//
//  CONTROL CALL NUMBER IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlCallWaiting extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/callWaiting";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------
 
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    let entity = createOptions['entity'];
    let isCallWaiting = false;

    // save only if a phone property set
    if( ( 'phone' in entity ) && ( 'callWaitingLines' in entity['phone'] ) ) {
      isCallWaiting = ( parseInt( entity['phone']['callWaitingLines'] ) > 0 );
    }

    $( "#ID_CALLWAITING" + this.dynid ).choice( {
      size: CONST.size.L,
      text: "{$callwaiting_value}",
      type: "check",
      value: isCallWaiting
    } );

  }

  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // save
    if( eventName == "option" && param == "save" ) {
      return this.FormSave( data );
    }

    return false;
  }

  //-----------------------------------------------------------------
  //  FORM FUNCTIONS
  //-----------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    // get value and save into entity
    let value = $( "#ID_CALLWAITING" + this.dynid ).choice( "option", "value" );

    // save only if a phone property set
    if( ( 'phone' in entity ) && ( 'callWaitingLines' in entity['phone'] ) ) {
      entity['phone']['callWaitingLines'] = ( value ? 10 : 0 );
      return true;
    }

    return false;
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  // no validation needed (true/false)
  OnValidate( param )
  {
    return 0;
  }

  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlCallWaiting.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlCallWaiting.Register( ControlCallWaiting.template );
﻿//=============================================================================
//
//  CONTROL CALL NUMBER IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlCallNumber extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/callNumber";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    this.entity = createOptions['entity'];
    let callnumber = "";
    
    if( ( 'phone' in this.entity ) && ( 'callnumber' in this.entity['phone'] ) )  {
      callnumber = this.entity['phone']['callnumber'];
    }

    $( "#ID_CALLNUMBER" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      value: callnumber,
      filter: { allow: "0123456789" },      
      validate: this.ValidateEdit.bind( this ),
      result: this.ControlCallnumberResult.bind( this )
    } );

  }

  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // send DEVRESTART_NEEDED
    if( eventName == "modified" && param > 0 )  {
      this.FireEvent( "restartNeeded" );
    }
        
    // save
    if( eventName == "option" && param == "save" ) {
      return this.FormSave( data );
    }

    return false;
  }


  //-----------------------------------------------------------------
  //  CONTROL
  //-----------------------------------------------------------------

  ControlCallnumberResult( result )
  {
    this.FireEvent( result );
  }

  //-----------------------------------------------------------------
  //  FORM FUNCTIONS
  //-----------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    // get value and save into entity
    let value = $( "#ID_CALLNUMBER" + this.dynid ).edit( "option", "value" );

    // save only if a phone property set
    if( ( 'phone' in entity ) && ( 'callnumber' in entity['phone'] ) ) {
      entity['phone']['callnumber'] = value;

      if( entity['class'] != "group" )  {
        if( !( 'authorize' in entity ) )        { entity['authorize'] = {};         }
        if( !( 'sip' in entity['authorize'] ) ) { entity['authorize']['sip'] = {};  }
        entity['authorize']['sip']['account'] = value;
      }
      else if( ( 'authorize' in entity ) 
        && ( 'sip' in entity['authorize'] ) 
        && ( 'account' in entity['authorize']['sip'] ) 
      ) {
        delete entity['authorize']['sip']['account'];
      }
      return true;
    }

    return false;
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    // switch off
    if( param === false ) {
      $( "#ID_CALLNUMBER" + this.dynid ).edit( "validate", false );
      return 0;
    }

    // validate
    return $( "#ID_CALLNUMBER" + this.dynid ).edit( "validate" );
  }

  //-----------------------------------------------------------------
  //  validation handler for edit
  //
  ValidateEdit( validateControl )
  {
    let value = $( "#ID_CALLNUMBER" + this.dynid ).edit( "option", "value" );
    if( typeof value != "string" || value.length == 0 )  {
      validateControl.error = "{$control_callnumber_error_empty}";
      return 1;
    }

    let callnumberInfo = Data.Instance().CalltargetDataset().CallnumberInfo( value );
    if( Array.isArray( callnumberInfo ) )  {
      for( var callnumberInfoRecord of callnumberInfo ) {
        if( ( callnumberInfoRecord['class'] != "addrBook" ) 
         && ( callnumberInfoRecord['entid'] != this.entity['entid'] ) 
        ) {
          validateControl.error = "{$control_callnumber_error_duplicated}";
          return 1;
        }
      }
    }
    else if( ( callnumberInfo )
     && ( callnumberInfo['class'] != "addrBook" ) 
     && ( callnumberInfo['entid'] != this.entity['entid'] ) 
      ) {
      //console.log( "collision on", callnumberInfo );
      validateControl.error = "{$control_callnumber_error_duplicated}";
      return 1;
    }

    let prefixInfo =  Data.Instance().CalltargetDataset().PrefixInfo( value );
    if( prefixInfo )  {
      validateControl.error = "{$control_callnumber_error_prefix}";
      return 1;
    }

    return 0;
  }


  //-----------------------------------------------------------------
  //  UNIT TEST
  //-----------------------------------------------------------------

  OnUnitTest( utControl, param )
  {
  }


  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlCallNumber.template;
    $element.viewable( createOptions );
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlCallNumber.Register( ControlCallNumber.template );
﻿//=============================================================================
//
//  CONTROL CALL NUMBER IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlCallPrefix extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/callPrefix";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    this.entity = createOptions['entity'];
    let prefix = "";
    
    if( ( 'phone' in this.entity ) && ( 'prefix' in this.entity['phone'] ) )  {
      prefix = this.entity['phone']['prefix'];
    }

    $( "#ID_CALLPREFIX" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      value: prefix,
      filter: { allow: "0123456789" },      
      validate: this.ValidateEdit.bind( this ),
      result: this.ControlCallprefixResult.bind( this )
    } );

  }

  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // save
    if( eventName == "option" && param == "save" ) {
      return this.FormSave( data );
    }

    return false;
  }


  //-----------------------------------------------------------------
  //  CONTROL
  //-----------------------------------------------------------------

  ControlCallprefixResult( result )
  {
    this.FireEvent( result );
  }

  //-----------------------------------------------------------------
  //  FORM FUNCTIONS
  //-----------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    // get value and save into entity
    let value = $( "#ID_CALLPREFIX" + this.dynid ).edit( "option", "value" );

    // save only if a phone property set
    if( ( 'phone' in entity ) && ( 'prefix' in entity['phone'] ) ) {
      entity['phone']['prefix'] = value;
      return true;
    }

    return false;
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    // switch off
    if( param === false ) {
      $( "#ID_CALLPREFIX" + this.dynid ).edit( "validate", false );
      return 0;
    }

    // validate
    return $( "#ID_CALLPREFIX" + this.dynid ).edit( "validate" );
  }

  //-----------------------------------------------------------------
  //  validation handler for edit
  //
  ValidateEdit( validateControl )
  {
    let value = $( "#ID_CALLPREFIX" + this.dynid ).edit( "option", "value" );
    if( typeof value != "string" || value.length == 0 )  {
      validateControl.error = "{$control_callprefix_error_empty}";
      return 1;
    }

    let callnumberInfo = Data.Instance().CalltargetDataset().CallnumberInfo( value, false );
    if( callnumberInfo && callnumberInfo[ 0 ]['class'] != "addrBook" )  {
      if( ( callnumberInfo.length > 1 )  
       || ( callnumberInfo[ 0 ]['entid'] != this.entity['entid'] )
        ) {
        validateControl.error = "{$control_callprefix_error_callnumber}";
        return 1;
      }
    }

    let prefixInfo =  Data.Instance().CalltargetDataset().PrefixInfo( value );
    if( prefixInfo )  {
      if( ( prefixInfo.length > 1 )  
       || ( prefixInfo[ 0 ]['entid'] != this.entity['entid'] ) 
        ) {
        validateControl.error = "{$control_callprefix_error_duplicated}";
        return 1;
      }
    }

    return 0;
  }


  //-----------------------------------------------------------------
  //  UNIT TEST
  //-----------------------------------------------------------------

  OnUnitTest( utControl, param )
  {
  }


  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlCallPrefix.template;
    $element.viewable( createOptions );
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlCallPrefix.Register( ControlCallPrefix.template );
﻿//=============================================================================
//
//  CONTROL CALL NUMBER IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlAddressBook extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/addressBook";
  static shownDeviceTemplates = [
    "AVP_870-0",
    "AHF_870-0",
    "AHFV_870-0",
    "AHT_870-0",
    "AHTV_870-0"
  ];

  static shownEntityClass = [
    "client",
    "asht",
    "jungpanel"
  ];

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  isInitialized = false;
  addrBook = null;
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    let entity = createOptions['entity'];
    this.addrBook = entity['addrBook'] || null;
    this.$container.empty();

    // check if we need to show address book
    if( this.addrBook == null ) {
      let deviceTemplate = entity['deviceTemplate'] || "";
      let entityClass = entity['class'];
      if( deviceTemplate.length > 0 && ControlAddressBook.shownDeviceTemplates.indexOf( deviceTemplate ) >= 0 )  {
        this.addrBook = "none";
      }
      else if( ControlAddressBook.shownEntityClass.indexOf( entityClass ) >= 0 ) {
        this.addrBook = "none";
      }
    }

    // show addressbook if needed
    if( this.addrBook )  {
      let $section = $( $.views.Html( ControlAddressBook.template + "/line", {}, this.dynid ) );
      this.$container.append( $section );

      // get selection 
      let selectedAddressBook = entity['addrBook'];
      if( selectedAddressBook == null || selectedAddressBook == "none" )  {
        selectedAddressBook = "none";
      }
      else {
        selectedAddressBook = Data.Instance().RootEntid() + "/" + selectedAddressBook + "/addrbook";
      }

      // create statepoint control
      this.$listCtrl = $section.find( ".addressBook" );
      ControlStatepointList.Build( this.$listCtrl, {
        inquirer: entity['entid'],
        selected: [ selectedAddressBook ],
        allowedTypes: ["ADDRBOOK"],
        noneRecord: true,
      } );

    }

  }

  //-----------------------------------------------------------------
  //  EVENT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // save
    if( eventName == "option" && param == "save" ) {
      return this.FormSave( data );
    }

    // references
    if( eventName == "option" && param == "references" ) {
      return this.FormReferences( data );
    }


    return false;
  }

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormReferences( entity )
  {
    // no addressbook needed
    if( !this.addrBook ) {
      return [];
    }

    // return refs
    let selected = this.$listCtrl.viewable( "option", "selected" );
    if( selected && selected[ 0 ] != "none" ) {
      return [ "sp:" + selected[ 0 ] ];
    }
    return [];
  }

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    // Prerequisites
    let result = "none";

    // no addressbook needed
    if( !this.addrBook )  {
      return true;
    }

    // get statepoint selected by list control, default is "none"
    result = this.$listCtrl.viewable( "option", "selected" );
    if( result == null )  {
      result = "none";
    }
    
    // default is "none", overwrite if something's selected
    if( result != "none" ) {
      result = result[ 0 ].split( "/" )[ 1 ];
    }

    // save into entity
    entity['addrBook'] = result;
    return true;
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    // no addressbook needed
    if( !this.addrBook )
      return 0;

    // switch off
    if( param === false ) {
      return this.$listCtrl.viewable( "validate", false );
    }

    // "no address book" is ok
    let selected = this.$listCtrl.viewable( "option", "selected" );
    if( ( selected == "none" )
     || ( selected == null )
      ) {
      return 0;
    }

    // validate ControlStatepointList
    return this.$listCtrl.viewable( "validate" );
  }

  
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlAddressBook.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlAddressBook.Register( ControlAddressBook.template );
﻿//=============================================================================
//
//  CONTROL CALLTARGET IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlDeviceVolume extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/deviceVolume";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  isInitialized = false;
  isAfsxMode = false;
  rangeAfsx = {
    valueRange: 6,
    valueMin: 1,
    valueMax: 6
  };
  rangeSsmp = {
    valueRange: 5,
    valueMin: 1,
    valueMax: 5
  };
  $lineRing = null;
  $controlRangeVoice = null;

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    let entity = createOptions['entity'];

    // defaults
    this.currentRange = {};

    // get device records
    this.deviceRecordRing  = entity['deviceConfig'].find( ( rec ) => rec['template'] == "volumeRing"  );
    this.deviceRecordVoice = entity['deviceConfig'].find( ( rec ) => rec['template'] == "volumeVoice" );
    this.deviceRecordMic   = entity['deviceConfig'].find( ( rec ) => rec['template'] == "volumeMic" );

    if( 'phone' in entity && 'ssmp' in entity['phone'] )  {
      this.isAfsxMode = ( entity['phone']['ssmp'] === false );
    }

    // range on ssmp we've got 5, on afs 6 values
    if( this.isAfsxMode )  {
      this.currentRange = this.rangeAfsx;
    }
    else {
      this.currentRange = this.rangeSsmp;
    }

    this.FormCreateControls();
    this.FormFillControls();

  }

  //-----------------------------------------------------------------
  //  VIEWABLE OVERRIDABLE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // send DEVRESTART_NEEDED
    if( eventName == "modified" && param > 0 )  {
      this.FireEvent( "restartNeeded" );
    }
        
    // save
    if( eventName == "option" && param == "save" ) {
      return this.FormSave( data );
    }
    return false;
  }

  //-----------------------------------------------------------------
  //  create controls (called from load)
  //
  FormCreateControls()
  {
    this.$container.empty();

    // ring volume range
    if( this.deviceRecordRing ) {

      // get html template for ring line & create
      this.$lineRing = $( $.views.Html( ControlDeviceVolume.template + "/ring", {}, this.dynid ) );
      this.$container.append( this.$lineRing );

      // create range slider
      this.$controlRing = this.$lineRing.find( ".range" );
      this.$controlRing.range( {
        size: CONST.size.L,
        valueRange: this.currentRange.valueRange,
        valueMin: this.currentRange.valueMin,
        valueMax: this.currentRange.valueMax,
        renderTooltip: this.ControlRenderTooltipOut.bind( this )
      } );
    }

    // voice volume range
    if( this.deviceRecordVoice ) {

      // get html template for ring line & create
      this.$lineVoice = $( $.views.Html( ControlDeviceVolume.template + "/call", {}, this.dynid ) );
      this.$container.append( this.$lineVoice );

      // create range slider
      this.$controlVoice = this.$lineVoice.find( ".range" );
      this.$controlVoice.range( {
        size: CONST.size.L,
        valueRange: this.currentRange.valueRange,
        valueMin: this.currentRange.valueMin,
        valueMax: this.currentRange.valueMax,
        renderTooltip: this.ControlRenderTooltipOut.bind( this )
      } );
    }

    // microphone volume range
    if( this.deviceRecordMic )  {

      // get html template for ring line & create
      this.$lineMic = $( $.views.Html( ControlDeviceVolume.template + "/mic", {}, this.dynid ) );
      this.$container.append( this.$lineMic );

      // create range slider
      this.$controlMic = this.$lineMic.find( ".range" );
      this.$controlMic.range( {
        size: CONST.size.L,
        valueRange: this.currentRange.valueRange,
        valueMin: this.currentRange.valueMin,
        valueMax: this.currentRange.valueMax,
        renderTooltip: this.ControlRenderTooltipIn.bind( this )
      } );
    }

  }


  //---------------------------------------------------------------------------
  //  FORM FUNCTIONS
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    // get ringtone value from control and save into entity
    if( this.deviceRecordRing ) {
      let value = this.$controlRing.range( "option", "value" );
      if( this.isAfsxMode ) {
        value = ( value - 1 );
      }
      else {
        value = ( value * 20 );
      }
      
      let saveDeviceRecordRing = entity['deviceConfig'].find( ( rec ) => rec['template'] == "volumeRing" );
      saveDeviceRecordRing['volumeRing'] = value;
    }

    // get voice value from control and save into entity
    if( this.deviceRecordVoice ) {
      let value = this.$controlVoice.range( "option", "value" );
      if( this.isAfsxMode ) {
        value = ( value - 1 );
      }
      else {
        value = ( value * 20 );
      }
      let saveDeviceRecordVoice = entity['deviceConfig'].find( ( rec ) => rec['template'] == "volumeVoice" );
      saveDeviceRecordVoice['volumeVoice'] = value;
    }

    // get microphone value from control and save into entity
    if( this.deviceRecordMic ) {
      let value = this.$controlMic.range( "option", "value" );
      if( this.isAfsxMode ) {
        value = ( value - 1 );
      }
      else {
        value = ( value * 20 );
      }
      let saveDeviceRecordMic = entity['deviceConfig'].find( ( rec ) => rec['template'] == "volumeMic" );
      saveDeviceRecordMic['volumeMic'] = value;
    }

    return true;
  }

  //-----------------------------------------------------------------
  // fill controls (called from load)
  //
  FormFillControls()
  {
    // RING volume
    if( this.$controlRing )  {
      if( this.isAfsxMode ) {
        let value = parseInt( this.deviceRecordRing['volumeRing'] ) + 1;
        // bug in template for AFSx after create
        if( value > 20 )  {
          value = Math.floor( value / 20 );
        }
        this.$controlRing.range( "option", "value", value );
      }
      else {
        let value = parseInt( this.deviceRecordRing['volumeRing'] ) / 20;
        this.$controlRing.range( "option", "value", value );
      }
    }
  
    // VOICE volume
    if( this.$controlVoice )  {
      if( this.isAfsxMode ) {
        let value = parseInt( this.deviceRecordVoice['volumeVoice'] ) + 1;
        // bug in template for AFSx after create
        if( value > 20 ) {
          value = Math.floor( value / 20 );
        }
        this.$controlVoice.range( "option", "value", value );
      }
      else {
        let value = parseInt( this.deviceRecordVoice['volumeVoice'] ) / 20;
        this.$controlVoice.range( "option", "value", value );
      }
    }


    // MICROPHONE volume
    if( this.$controlMic ) {
      if( this.isAfsxMode ) {
        let value = parseInt( this.deviceRecordMic['volumeMic'] ) + 1;
        // bug in template for AFSx after create
        if( value > 20 ) {
          value = Math.floor( value / 20 );
        }
        this.$controlMic.range( "option", "value", value );
      }
      else {
        let value = parseInt( this.deviceRecordMic['volumeMic'] ) / 20;
        this.$controlMic.range( "option", "value", value );
      }
    }
  }

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  tooltip while using slider (for speakers)
  //
  ControlRenderTooltipOut( value )
  {
    let tooltip = $.lang( "{$devicevolume_value_out}" );
    tooltip = tooltip.replace( "{value}", value );
    return tooltip;
  }

  //-----------------------------------------------------------------
  //  tooltip while using slider (for speakers)
  //
  ControlRenderTooltipIn( value )
  {
    let tooltip = $.lang( "{$devicevolume_value_in}" );
    tooltip = tooltip.replace( "{value}", value );
    return tooltip;
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  range must be in range from the beginning on
  //
  OnValidate( param )
  {
    return 0;
  }
  
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlDeviceVolume.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlDeviceVolume.Register( ControlDeviceVolume.template );
﻿//=============================================================================
//
//  CONTROL CALL NUMBER IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlShowClock extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/showClock";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  deviceRecord = null;
 
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    this.deviceRecord = createOptions['entity']['deviceConfig'].find( ( rec ) => rec['template'] == "showTime" );

    if( this.deviceRecord )  {
      this.$container.empty();      
      let $section = $( $.views.Html( ControlShowClock.template + "/line", {}, this.dynid ) );
      this.$container.append( $section );
      $( "#ID_SHOWCLOCK" + this.dynid ).choice( {
        size: CONST.size.L,
        text: "{$formbase_value_showclock}",
        type: "check",
        value: ( this.deviceRecord['showTime'] == "true" )
      } );
    }

  }

  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // send DEVRESTART_NEEDED
    if( eventName == "modified" && param > 0 )  {
      this.FireEvent( "restartNeeded" );
    }
        
    // save
    if( eventName == "option" && param == "save" ) {
      return this.FormSave( data );
    }

    return false;
  }

  //-----------------------------------------------------------------
  //  FORM FUNCTIONS
  //-----------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    // no addressbook needed
    if( !this.deviceRecord )
      return;

    // get value and save into entity
    let value = $( "#ID_SHOWCLOCK" + this.dynid ).choice( "option", "value" );
    let saveDeviceRecord = entity['deviceConfig'].find( ( rec ) => rec['template'] == "showTime" );
    if( saveDeviceRecord )  {
      saveDeviceRecord['showTime'] = ( value ? "true" : "false" );
    }

    return true;
  }

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------

  ControlCheckResult( value )
  {
    this.deviceRecord['showTime'] = value ? "true" : "false";
    this.FireEvent( "result", $.clone( this.entity ) );
  }

  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    return 0;
  }
  
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlShowClock.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlShowClock.Register( ControlShowClock.template );
﻿//=============================================================================
//
//  CONTROL MOUNT TYPE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlMountType extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/mountType";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  isInitialized = false;
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    let entity = createOptions['entity'];
    this.deviceRecord = entity['deviceConfig'].find( ( rec ) => rec['template'] == "mountType" );

    // correct bug in device template avp, aht + ahtv
    if( ( this.deviceRecord )
     && ( entity['deviceTemplate'] == "AVP_870-0" || entity['deviceTemplate'] == "AHT_870-0" || entity['deviceTemplate'] == "AHTV_870-0" )
      ) {
      this.deviceRecord = null;
    }

    if( this.deviceRecord )  {
      this.$container.empty();      
      let $section = $( $.views.Html( ControlMountType.template + "/line", {}, this.dynid ) );
      this.$container.append( $section );
      $( "#ID_MOUNTTYPE" + this.dynid ).choice( {
        size: CONST.size.L,
        type: "radio",
        value: this.deviceRecord['mountType'],
        data: [
          { id: "wall", text: "{$mounttype_value_wall}" },
          { id: "table", text: "{$mounttype_value_desk}" }
        ]
      } );
    }

    this.isInitialized = true;
  }

  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // send DEVRESTART_NEEDED
    if( eventName == "modified" && param > 0 )  {
      this.FireEvent( "restartNeeded" );
    }
        
    // save
    if( eventName == "option" && param == "save" ) {
      return this.FormSave( data );
    }

    return false;
  }

  //-----------------------------------------------------------------
  //  FORM FUNCTIONS
  //-----------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    // no addressbook needed
    if( !this.deviceRecord )
      return;

    // get value and save into entity
    let value = $( "#ID_MOUNTTYPE" + this.dynid ).choice( "option", "value" );
    let saveDeviceRecord = entity['deviceConfig'].find( ( rec ) => rec['template'] == "mountType" );
    if( saveDeviceRecord ) {
      saveDeviceRecord['mountType'] = value;
    }

    return true;
  }

  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    return 0;
  }

  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlMountType.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlMountType.Register( ControlMountType.template );
﻿//=============================================================================
//
//  CONTROL EXTERN CALL NUMBER IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          16.07.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlExternCallNumber extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/externCallNumber";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    let entity = createOptions['entity'];
    let externCallnumber = "";
    
    if( ( 'phone' in entity ) && ( 'externCallnumber' in entity['phone'] ) )  {
      externCallnumber = entity['phone']['externCallnumber'];
    }

    $( "#ID_EXTERN_CALLNUMBER" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      value: externCallnumber,
      validate: this.ValidateEdit.bind( this )
    } );

  }

  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // save
    if( eventName == "option" && param == "save" ) {
      return this.FormSave( data );
    }

    return false;
  }

  //-----------------------------------------------------------------
  //  FORM FUNCTIONS
  //-----------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    // get value and save into entity
    let value = $( "#ID_EXTERN_CALLNUMBER" + this.dynid ).edit( "option", "value" );

    // save only if a phone property set
    if( ( 'phone' in entity ) && ( 'externCallnumber' in entity['phone'] ) ) {
      entity['phone']['externCallnumber'] = value;
      return true;
    }

    return false;
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    if( param === false ) {
      $( "#ID_EXTERN_CALLNUMBER" + this.dynid ).edit( "validate", false );
      return 0;
    }

    // validate
    return $( "#ID_EXTERN_CALLNUMBER" + this.dynid ).edit( "validate" );
  }

  //-----------------------------------------------------------------
  //  validation handler for edit
  //
  ValidateEdit( validateControl )
  {
    let value = $( "#ID_EXTERN_CALLNUMBER" + this.dynid ).edit( "option", "value" );
    return ControlExternCallNumber.Validate( validateControl, value );
  }


  //-----------------------------------------------------------------
  //  UNIT TEST
  //-----------------------------------------------------------------

  OnUnitTest( utControl, param )
  {
  }


  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlExternCallNumber.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, value )
  {
  	// Extern call number is allowed to be empty. (?)
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlExternCallNumber.Register( ControlExternCallNumber.template );
﻿//=============================================================================
//
//  CONTROL CALL PROFILE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlCallProfile extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/callProfile";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  isInitialized = false;
  callprofile = null;
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    let entity = createOptions['entity'];

    // Ensure that callprofile isn't undefined if entity doesn't have phone object.
    // This may happen with non-autonomous ASC/ASHT! 
    this.callprofile = "prfstd";
    if( typeof entity["phone"] == "object" )  {
      this.callprofile = entity["phone"]["callprofile"];
    }

    this.$listCtrl = this.$container.find( ".listbox" );
    this.$listCtrl.listbox( {
      size: CONST.size.L,
      type: "droplist",
      selected: [ this.callprofile ],
      data: Data.Instance().CallprofileDataset(),
      validate: this.ValidateDroplist.bind( this )
      //result: this.ControlDroplistResult.bind( this )
    } );

    this.isInitialized = true;
  }


  //-----------------------------------------------------------------
  //  EVENT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // save
    if( eventName == "option" && param == "save" ) {
      return this.FormSave( data );
    }

    if( eventName == "option" && param == "references" ) {
      return this.FormReferences( data );
    }

    return false;
  }

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormReferences( entity )
  {
    let profile = "prfstd";

    // Get callprofile selected in list control.
    let selected = this.$listCtrl.listbox( "option", "selected" );
    if( Array.isArray( selected ) && selected.length > 0 ) {
      if( typeof selected[ 0 ] == "string" ) {
        profile = selected[ 0 ];
      }
    }

    if( profile != "prfstd" ) {
      return ["prf:admui.callprofile:" + profile ];
    }

    return [];
  }

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    // Get callprofile selected in list control.
    let selected = this.$listCtrl.listbox( "option", "selected" );
    if( Array.isArray( selected ) && selected.length > 0 ) {
      if( typeof selected[ 0 ] == "string" )  {
        entity["phone"]["callprofile"] = selected[ 0 ];
      }   
    }

    return true;
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    if( param === false ) {
      return this.$listCtrl.listbox( "validate", false );
    }
    return this.$listCtrl.listbox( "validate" );    
  }

  //-----------------------------------------------------------------
  //  validation handler for droplist
  //
  ValidateDroplist( validateControl )
  {
    let selected = this.$listCtrl.listbox( "option", "selected" );    
    return ControlCallProfile.Validate( validateControl, selected );
  }
  

  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlCallProfile.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    // Does selected call profile ID exist in current config?
    if( Array.isArray( selected ) && selected.length > 0 ) {    
      if( !( Data.Instance().CallprofileDataset().Has( selected[ 0 ] ) ) ) {      
        validateControl.error = "{$control_callprofile_error_deleted}";    
        return 1;
      }
    }
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlCallProfile.Register( ControlCallProfile.template );
﻿//=============================================================================
//
//  CONTROL DTMF DOOR-AND-LIGHT IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          16.07.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlDtmfDoorAndLight extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/dtmfDoorAndLight";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    let entity = createOptions['entity'];
    let dtmfDoor = "";
    let dtmfLight = "";

    $( "#ID_OPTIONAL" + this.dynid ).optional( { } );

    if( 'entrance' in entity )  {
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_DTMF_DOOR" );
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_DTMF_LIGHT" );
    }
    
    if( ( 'phone' in entity ) && ( 'dtmfDoor' in entity['phone'] ) )  {
      dtmfDoor = entity['phone']['dtmfDoor'];
    }
    if( ( 'phone' in entity ) && ( 'dtmfLight' in entity['phone'] ) )  {
      dtmfLight = entity['phone']['dtmfLight'];
    }    

    $( "#ID_DTMF_DOOR" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      value: dtmfDoor,
      filter: { allow: "0123456789*#", deny: null },
      validate: Validate.NotEmptyString   //this.ValidateEditDoor.bind( this )
    } );

    $( "#ID_DTMF_LIGHT" + this.dynid ).edit( {
      size: CONST.size.M,
      status: "normal",
      value: dtmfLight,
      filter: { allow: "0123456789*#", deny: null },
      validate: Validate.NotEmptyString //this.ValidateEditLight.bind( this )
    } );

  }

  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // save
    if( eventName == "option" && param == "save" ) {
      return this.FormSave( data );
    }

    return false;
  }

  //-----------------------------------------------------------------
  //  FORM FUNCTIONS
  //-----------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    // get values and save into entity
    let valueDoor  = $( "#ID_DTMF_DOOR" + this.dynid ).edit( "option", "value" );
    let valueLight = $( "#ID_DTMF_LIGHT" + this.dynid ).edit( "option", "value" );

    let isOk = true;

    // Save only if a phone property set
    if( ( 'phone' in entity ) && ( 'dtmfDoor' in entity['phone'] ) ) {
      entity['phone']['dtmfDoor'] = valueDoor;
    }
    else  {
      isOk = false;
    }

    if( ( 'phone' in entity ) && ( 'dtmfLight' in entity['phone'] ) ) {
      entity['phone']['dtmfLight'] = valueLight;
    }
    else  {
      isOk = false;
    }

    return isOk;
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    if( param === false ) {
      $( "#ID_DTMF_DOOR" + this.dynid ).edit( "validate", false );
      $( "#ID_DTMF_LIGHT" + this.dynid ).edit( "validate", false );      
      return 0;
    }

    // validate
    let numErr = 0;
    numErr += $( "#ID_DTMF_DOOR" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_DTMF_LIGHT" + this.dynid ).edit( "validate" );    
    return numErr;
  }

  //-----------------------------------------------------------------
  //  validation handler for dtmf/door edit
  //
  ValidateEditDoor( validateControl )
  {
    let value = $( "#ID_DTMF_DOOR" + this.dynid ).edit( "option", "value" );
    return ControlDtmfDoorAndLight.ValidateDtmfDoor( validateControl, value );
  }

  //-----------------------------------------------------------------
  //  validation handler for dtmf/light edit
  //
  ValidateEditLight( validateControl )
  {
    let value = $( "#ID_DTMF_LIGHT" + this.dynid ).edit( "option", "value" );
    return ControlDtmfDoorAndLight.ValidateDtmfLight( validateControl, value );
  }


  //-----------------------------------------------------------------
  //  UNIT TEST
  //-----------------------------------------------------------------

  OnUnitTest( utControl, param )
  {
  }


  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlDtmfDoorAndLight.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static ValidateDtmfDoor( validateControl, value )
  {
    // Empty DTMF edit is allowed!

    return 0;
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static ValidateDtmfLight( validateControl, value )
  {
    // Empty DTMF edit is allowed!

    return 0;
  }  

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlDtmfDoorAndLight.Register( ControlDtmfDoorAndLight.template );
﻿//=============================================================================
//
//  CONTROL CALL NUMBER IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlPassword extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/password";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    this.entity = createOptions['entity'];
    this.isEditMode = false;
    this.confirmedPassword = null;

    // check if password has been changed 
    if( ( 'authorize' in this.entity )
     && ( 'default' in this.entity['authorize'] )
     && ( this.entity['authorize']['default'] === false ) 
      ) {
      this.isEditMode = true;
    }

    // optional
    $( "#ID_OPTIONAL" + this.dynid ).optional( { } );

    $( "#ID_PASSWORD1" + this.dynid ).edit( {
      size: CONST.size.L,
      password: true,
      value: "",
      validateDepends: [ "ID_PASSWORD2" + this.dynid ],
      validate: this.ValidateEdit1.bind( this ),
      change: this.ControlEditPasswordChange.bind( this )
    });

    $( "#ID_PASSWORD2" + this.dynid ).edit( {
      size: CONST.size.L,
      password: true,
      value: "",
      validateDepends: [ "ID_PASSWORD1" + this.dynid ],
      validate: this.ValidateEdit2.bind( this ),
      change: this.ControlEditPasswordChange.bind( this )
    });

    $( "#ID_PASSWORD_EDIT" + this.dynid ).button( {
      size: 4,
      icon: "mdi mdi-pencil",
      click: this.ControlButtonPasswordClick.bind( this )
    });

    if( this.isEditMode ) {
      this.ModeEditPassword();
    }
    else {
      this.ModeNewPassword();
    }
  }

  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // save
    if( eventName == "option" ) {
      switch( param ) {
        case "status": 
          return this.FormStatus( data );

        case "save":
          return this.FormSave( data );

        case "value":
          return this.confirmedPassword;
  
      }      
    }

    return false;
  }

  //-----------------------------------------------------------------
  //  FORM FUNCTIONS
  //-----------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  status (disable)
  //
  FormStatus( status )
  {
    if( status == "disabled" ) {
      $( "#ID_PASSWORD_EDIT" + this.dynid ).button( "option", "status", "disabled" );
    }
    else {
      $( "#ID_PASSWORD_EDIT" + this.dynid ).button( "option", "status", "normal" );
    }
  }

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    if( ! this.isEditMode ) {
      let password = $( "#ID_PASSWORD2" + this.dynid ).edit( "option", "value" );
      Data.Instance().SetEntityPassword( entity['entid'], password, "password" );
    }
    if( ! ( 'authorize' in this.entity ) ) {
      entity['authorize'] = {};
    }
    entity['authorize']['default'] = false;
    return true;
  }


  //-----------------------------------------------------------------
  //  CONTROL
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  edit button clicked. show dialog
  //
  ControlButtonPasswordClick()
  {
    ChangePasswordDialog.Show( {
      result: ( password ) => {
        Data.Instance().SetEntityPassword( this.entity['entid'], password, "password" );
        this.FireEvent( 'result', password );
      }
    } );

  }

  //-----------------------------------------------------------------
  //  New password (not yet saved on server). If one of the two
  //  edit controls is changed. As soon passwords are the same fire 
  //  password event to upper layer. Needed for ipcam. 
  //
  ControlEditPasswordChange()
  {
    var password1 = $( "#ID_PASSWORD1" + this.dynid ).edit( "option", "value" );
    var password2 = $( "#ID_PASSWORD2" + this.dynid ).edit( "option", "value" );
    if( password1 == password2 )  {
      this.confirmedPassword = password1;
      this.FireEvent( "result", password1 );
    }
    else {
      this.confirmedPassword = null;
    }
  }

  //-----------------------------------------------------------------
  //  MODE
  //-----------------------------------------------------------------

  ModeEditPassword()
  {
    if( this.mode == "modeEditPassword" ) return;
    this.mode = "modeEditPassword";

    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPT_PASSWORD2" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPT_PASSWORD_EDIT" );

    // this code doesn't work, so re-create
    // $( "#ID_PASSWORD1" + this.dynid ).edit( "option", "password", false );
    // $( "#ID_PASSWORD1" + this.dynid ).edit( "option", "status", "readonly" );
    // $( "#ID_PASSWORD1" + this.dynid ).edit( "option", "value", "********" );
    $( "#ID_PASSWORD1" + this.dynid ).edit( {
      size: CONST.size.L,
      password: false,
      value: "********",
      status: "readonly",
      validateDepends: [ "ID_PASSWORD2" + this.dynid ],
      validate: this.ValidateEdit1.bind( this )
    });    

  }

  ModeNewPassword()
  {
    if( this.mode == "modeNewPassword" ) return;
    this.mode = "modeNewPassword";

    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPT_PASSWORD2" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPT_PASSWORD_EDIT" );
    $( "#ID_PASSWORD1" + this.dynid ).edit( "option", "status", "normal" );
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  upper layer calls validate
  //
  OnValidate( param )
  {
    if( this.mode == "modeEditPassword" ) {
      return 0;
    }

    // switch off
    if( param === false ) {
      $( "#ID_PASSWORD1" + this.dynid ).edit( "validate", false );
      $( "#ID_PASSWORD2" + this.dynid ).edit( "validate", false );
      return 0;
    }

    // validate
    let numErr = 0;
    numErr += $( "#ID_PASSWORD1" + this.dynid ).edit( "validate" );
    numErr += $( "#ID_PASSWORD2" + this.dynid ).edit( "validate" );


    // result
    return numErr;
  }

  //-----------------------------------------------------------------
  //  validate password repeat
  //
  ValidateEdit1( validateControl )
  {
    let password1 = $( "#ID_PASSWORD1" + this.dynid ).edit( "option", "value" );

    if( password1.length == 0 ) {
      validateControl.error = "{$change_password_error_empty}";
      return 1;
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  validate password repeat
  //
  ValidateEdit2( validateControl )
  {
    let password1 = $( "#ID_PASSWORD1" + this.dynid ).edit( "option", "value" );
    let password2 = $( "#ID_PASSWORD2" + this.dynid ).edit( "option", "value" );

    if( password1.length == 0 && password2.length == 0 ) {
      return 0;
    }

    if( password1 != password2 )  {
      validateControl.error = "{$change_password_error_different}";
      return 1;
    }

    return 0;
  }


  //-----------------------------------------------------------------
  //  UNIT TEST
  //-----------------------------------------------------------------

  OnUnitTest( utControl, param )
  {
  }


  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlPassword.template;
    $element.viewable( createOptions );
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlPassword.Register( ControlPassword.template );
﻿//=============================================================================
//
//  CONTROL CALL NUMBER IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlAccount extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/account";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    this.entity = createOptions['entity'];
    let account = "";
    let isReadOnly = createOptions['readonly'] || false;
    this.section = "";
    
    // read from authorize section
    if( ( 'authorize' in this.entity ) && ( 'admin' in this.entity['authorize'] ) )  {
      account = this.entity['authorize']['admin']['account'];
      this.section = "admin";
    }
    else if( ( 'authorize' in this.entity ) && ( 'websock' in this.entity['authorize'] ) ) {
      account = this.entity['authorize']['websock']['account'];
      this.section = "websock";
    }
    else {
      isReadOnly = true;
      account = "??";
    }

    // fill control
    $( "#ID_ACCOUNT" + this.dynid ).edit( {
      size: CONST.size.L,
      status: isReadOnly ? "readonly" : "normal",
      value: account,
      validate: this.ValidateEdit.bind( this )
    } );

  }

  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // save
    if( eventName == "option" && param == "save" ) {
      return this.FormSave( data );
    }

    return false;
  }

  //-----------------------------------------------------------------
  //  FORM FUNCTIONS
  //-----------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    // get value and save into entity
    let value = $( "#ID_ACCOUNT" + this.dynid ).edit( "option", "value" );

    if( this.section == "admin" )  {
      entity['authorize']['admin']['account'] = value;
      return true;
    }
    else if( this.section == "websock" ) {
      entity['authorize']['websock']['account'] = value;
      return true;
    }

    return false;
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    // switch off
    if( param === false ) {
      $( "#ID_ACCOUNT" + this.dynid ).edit( "validate", false );
      return 0;
    }

    // validate
    return $( "#ID_ACCOUNT" + this.dynid ).edit( "validate" );
  }

  //-----------------------------------------------------------------
  //  validation handler for edit
  //
  ValidateEdit( validateControl )
  {
    let value = $( "#ID_ACCOUNT" + this.dynid ).edit( "option", "value" );
    if( typeof value != "string" || value.length == 0 )  {
      validateControl.error = "{$control_account_error_empty}";
      return 1;
    }

    let accountInfo = Data.Instance().AccountInfo( value );
    if( ( accountInfo )
     && ( accountInfo['entid'] != this.entity['entid'] ) 
      ) {
      validateControl.error = "{$control_account_error_duplicated}";
      return 1;
    }

    return 0;
  }


  //-----------------------------------------------------------------
  //  UNIT TEST
  //-----------------------------------------------------------------

  OnUnitTest( utControl, param )
  {
  }


  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlAccount.template;
    $element.viewable( createOptions );
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlAccount.Register( ControlAccount.template );
﻿//=============================================================================
//
//  CONTROL CALL NUMBER IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlKeyTone extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/keytone";
  static shownDeviceTemplates = [
  ];
  static shownEntityClass = [
    "door"
  ];

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  isInitialized = false;
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    this.entity = createOptions['entity'];
    let entity = createOptions['entity'];
    this.deviceRecord = entity['deviceConfig'].find( ( rec ) => rec['template'] == "tlckeytone" );
    if( !this.deviceRecord )
      return;

    let enabled = this.deviceRecord['tlckeytone']['enabled'] || false;
    let mode = "none";
    this.isKeytoneListLoaded = false;
    if( enabled ) {
      mode = this.deviceRecord['tlckeytone']['mode'] || "none";
    }


    // build html
    this.$container.empty();    
    let $section = $( $.views.Html( ControlKeyTone.template + "/line", {}, this.dynid ) );
    this.$container.append( $section );

    // create droplist
    this.$listCtrl = $section.find( ".keytone" );

    //init optional
    this.$optionalCtrl = $section.find( ".optional" );
    $(this.$optionalCtrl[0]).optional( {} );

    //disable controls if mode not advanced
    if( mode !== "advanced" ){
      $( this.$optionalCtrl[0] ).optional( "hide", "ID_KEYTONE_CONTROL_KEYTONE_LIST" );
    }

    $(this.$listCtrl[0]).listbox( {
      size: CONST.size.L,
      type: "droplist",
      selected: [ mode ],
      data: [
        { id: "none", text: "{$keytone_value_none}" },
        { id: "tlc", text: "{$keytone_value_simple}" },
        { id: "advanced", text: "{$keytone_value_individual}", status: "normal" }
      ],
      result: ( selected ) => {
        if( selected[0] === "advanced" )  {
          $( this.$optionalCtrl[0] ).optional( "show", "ID_KEYTONE_CONTROL_KEYTONE_LIST" );
        }
        else  {
          $( this.$optionalCtrl[0] ).optional( "hide", "ID_KEYTONE_CONTROL_KEYTONE_LIST" );
          $( this.$listCtrl[1] ).listbox( "validate", false );
          delete this.deviceRecord['tlckeytone']['mapping'];
        }
      }
    });

    $( this.$listCtrl[1] ).listbox(
      {
        size: CONST.size.L + CONST.size.LBL,  //89.3, //24 + 65,
        height: 5,
        data: this.addressBookDataset,
        scrollbar: 'overlay', // 'overlay', 'static'
        sort: 'label',
        sortOrder: '+',
        placeholder: "",
        validate: this.ValidateKeytoneList.bind(this),
        result: ( selected ) => {
          if( selected && selected.length > 0 )  {
            $( "#ID_KEYTONE_EDIT" + this.dynid ).button( "option", "status", "normal" );
            $( "#ID_KEYTONE_DELETE" + this.dynid ).button( "option", "status", "normal" );
          }
          else {
            $( "#ID_KEYTONE_EDIT" + this.dynid ).button( "option", "status", "disabled" );
            $( "#ID_KEYTONE_DELETE" + this.dynid ).button( "option", "status", "disabled" );
          }
        }
      }
    );

    $( "#ID_KEYTONE_ADD" + this.dynid ).button( {
      size: CONST.size.M,
      text: "{$addrbook_button_add}",
      status: "normal",
      click: this.ControlAddClick.bind( this )
    } );

    $( "#ID_KEYTONE_EDIT" + this.dynid ).button( {
      size: CONST.size.M,
      text: "{$addrbook_button_edit}",
      status: "disabled",
      click: this.ControlEditClick.bind( this )
    } );

    $( "#ID_KEYTONE_DELETE" + this.dynid ).button( {
      size: 4,
      icon: "mdi mdi-delete",
      status: "disabled",
      click: this.ControlDeleteClick.bind( this )
    } );

    Data.Instance().Keytones().KeytoneRequest( ( param ) => {
      this.keytoneList = param;
      this.CreateListDataFromEntity( entity );
    } );
  }

  //-----------------------------------------------------------------
  //  EVENT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // send DEVRESTART_NEEDED
    if( eventName == "modified" && param > 0 )  {
      this.FireEvent( "restartNeeded" );
    }

    // save
    if( eventName == "option" && param == "save" ) {
      return this.FormSave( data );
    }

    // references
    if( eventName == "option" && param == "references" ) {
      return this.FormReferences( data );
    }


    return false;
  }

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormReferences( entity )
  {
    return [];
  }

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    // save into entity
    if( this.deviceRecord ) {
      let deviceRecord = entity['deviceConfig'].find( ( rec ) => rec['template'] == "tlckeytone" );
      let mode         = this.$listCtrl.listbox( "option", "selected" );
      if( Array.isArray( mode ) )  {
        mode = mode[0];
      }
      deviceRecord['tlckeytone']['mode'] = mode;

      if( mode != "none"  )  {
        deviceRecord['tlckeytone']['enabled'] = true;
      }
      else {
        deviceRecord['tlckeytone']['enabled'] = false;
      }

      if( mode == "advanced" )  {
        deviceRecord['tlckeytone']['mapping'] = this.deviceRecord['tlckeytone']['mapping'];
      }
    }
    return true;
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    let errorNo = 0;
    let $section = $( $.views.Html( ControlKeyTone.template + "/line", {}, this.dynid ) );
    let $keyToneLists = this.$listCtrl;

    let mode = $( $keyToneLists[0] ).listbox( "option", "selected" );
    if( mode[0] === "advanced" && this.isKeytoneListLoaded )  {
      errorNo += $( $keyToneLists[1] ).listbox( "validate" );
    }
    else {
      $( $keyToneLists[1] ).listbox( "validate", false );
    }

    return errorNo;
  }

  ValidateKeytoneList(validateControl)
  {
    let selected = $(this.$listCtrl[1]).listbox( "option", "data" );
    if( selected && selected.length === 0 )  {
      validateControl.error = "{$keytone_control_validate_empty_list}";
      return 1;
    }
    return 0;
  }

  OnValidateOff()
  {
    this.Option( "validate", false );
  }


  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlKeyTone.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    return 0;
  }

  //-----------------------------------------------------------------------------
  //  CreateListDataFromEntity(): description
  //
  CreateListDataFromEntity( entity )
  {
    let listboxData =  [];
    let keytones = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, "tlckeytone" );
    this.isKeytoneListLoaded = true;

    if( !keytones['tlckeytone']['mapping'] )  {
      return;
    }

    if( this.keytoneList.length > 0  )  {
      for( let callnumbner in keytones['tlckeytone']['mapping'] )  {
        let keytoneGuid = keytones['tlckeytone']['mapping'][ callnumbner ];
        let foundRecord = this.keytoneList.find( ( rec ) => rec['guid'] === keytoneGuid  );
        if( foundRecord )  {
          listboxData.push( {
            id: callnumbner,
            text: `${callnumbner} ("${foundRecord['label']}")`,
            icon: "mdi mdi-music"
          } )
        }
      }
      $( this.$listCtrl[1] ).listbox( "option", "data", listboxData );
    }
    else {
      let currentMode = $( this.$listCtrl[0] ).listbox( "option", "selected" );
      $( this.$optionalCtrl[0] ).optional( "hide", "ID_KEYTONE_CONTROL_KEYTONE_LIST" );
      $( this.$listCtrl[1] ).listbox( "validate", false );

      let data = [
        { id: "none", text: "{$keytone_value_none}" },
        { id: "tlc", text: "{$keytone_value_simple}" },
        { id: "advanced", text: "{$keytone_value_individual}", status: "disabled" }
      ];

      $( this.$listCtrl[0] ).listbox( "option", "data", data );

      if( currentMode[0] === "advanced" )  {

        $( this.$listCtrl[0] ).listbox( "option", "selected", [ "none" ] );
      }
    }



  }

  //=============================================================================
  //  BUTTON HANDLER
  //=============================================================================
  //-----------------------------------------------------------------------------
  //  ControlAddClick(): description
  //

  ControlAddClick()
  {
    ChangeKeytoneDialog.Show( {
      result: ( entry ) => {
        this.DataEntryAdd( entry );
      },
      param: {
        entity: this.entity,
      }
    } );
  }

  ControlEditClick()
  {
    let selected =  $(this.$listCtrl[1]).listbox("option","selected");
    let guid = this.GetGuidFromCallnumber( selected[0] );
    ChangeKeytoneDialog.Show( {
      result: ( entry ) => {
        this.DataEntryAdd( entry );
      },
      param: {
        entity: this.entity,
        callnumber: selected[0],
        guid: guid
      }
    } );
  }

  ControlDeleteClick()
  {
    let selected =  $(this.$listCtrl[1]).listbox("option","selected");

    this.DataEntryDelete( selected[0] );
  }

  //=============================================================================
  //  DIALOG RESULT HANDLER
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  DataEntryAdd( entry ): description
  //
  DataEntryAdd( entry )
  {
    // if an entry was edited delete old number and add new number
    if( !this.deviceRecord['tlckeytone']['mapping'] ) {
      this.deviceRecord['tlckeytone']['mapping'] = {};
    }

    if( entry[ 'oldCallNumber' ] )  {
      delete this.deviceRecord['tlckeytone']['mapping'][ entry['oldCallNumber'] ]
    }

    this.deviceRecord['tlckeytone']['mapping'][ entry['callnumber'] ] = entry['id'];
    this.CreateListDataFromEntity(this.entity);
    $(this.$listCtrl[1]).listbox( "option", "selected", entry['callnumber'] )
  }

  //-----------------------------------------------------------------------------
  //  DataEntryDelete( entry ): description
  //
  DataEntryDelete( callnumber )
  {
    delete this.deviceRecord['tlckeytone']['mapping'][ callnumber ]
    this.CreateListDataFromEntity( this.entity );


    let listData =  $(this.$listCtrl[1]).listbox("option","data");
    if( listData && listData.length > 0 )  {
      $(this.$listCtrl[1]).listbox("option","selected", listData[0]['id']);
      $( "#ID_KEYTONE_EDIT" + this.dynid ).button( "option", "status", "normal" );
      $( "#ID_KEYTONE_DELETE" + this.dynid ).button( "option", "status", "normal" );
    }
    else  {
      $( "#ID_KEYTONE_EDIT" + this.dynid ).button( "option", "status", "disabled" );
      $( "#ID_KEYTONE_DELETE" + this.dynid ).button( "option", "status", "disabled" );
    }

  }

  //=============================================================================
  //  HELPERS
  //=============================================================================


  //-----------------------------------------------------------------------------
  //  GetGuidFromCallnumber(): description
  //
  GetGuidFromCallnumber( callnumber )
  {
    let keytones = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, "tlckeytone" );
    return keytones['tlckeytone']['mapping'][ callnumber ] ?? false;
  }

}

//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlKeyTone.Register( ControlKeyTone.template );
﻿//=============================================================================
//
//  CONTROL CALL NUMBER IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlZam extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/zam";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  deviceRecord = null;
 
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    this.deviceRecord = createOptions['entity']['deviceConfig'].find( ( rec ) => rec['template'] == "zam" );
    this.$container.empty();      

    if( this.deviceRecord )  {
      let $section = $( $.views.Html( ControlZam.template + "/line", {}, this.dynid ) );
      this.$container.append( $section );
      $( "#ID_ZAM_OPTIONS" + this.dynid ).choice( {
        size: CONST.size.L,
        type: "check",
        data: [
          { id: "onSignalOpener", text: "{$zam_option_onopener}", value: this.deviceRecord['zam']['onSignalOpener'] },
          { id: "onSignalCall", text: "{$zam_option_oncall}", value: this.deviceRecord['zam']['onSignalCall'] },
          { id: "onSignalAcceptedCall", text: "{$zam_option_offhook}", value: this.deviceRecord['zam']['onSignalAcceptedCall'] },
          { id: "onSignalFailedConnection", text: "{$zam_option_failedcon}", value: this.deviceRecord['zam']['onSignalFailedConnection'] }
        ],
        value: this.deviceRecord['zam']
      } );
    }

  }

  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // send DEVRESTART_NEEDED
    if( eventName == "modified" && param > 0 )  {
      this.FireEvent( "restartNeeded" );
    }
        
    // save
    if( eventName == "option" && param == "save" ) {
      return this.FormSave( data );
    }

    return false;
  }

  //-----------------------------------------------------------------
  //  FORM FUNCTIONS
  //-----------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    // no addressbook needed
    if( !this.deviceRecord )
      return;

    // get value and save into entity
    let value = $( "#ID_ZAM_OPTIONS" + this.dynid ).choice( "option", "value" );
    let saveDeviceRecord = entity['deviceConfig'].find( ( rec ) => rec['template'] == "zam" );
    if( saveDeviceRecord )  {
      saveDeviceRecord['zam'] = value;
    }

    return true;
  }

  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    return 0;
  }
  
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlZam.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlZam.Register( ControlZam.template );
﻿//=============================================================================
//
//  CONTROL RINGTONE (inddoor devices)
//  (c) 2022 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          30.06.2022
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlRingtone extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/ringtone";
  static widgetId = "ringtone";
  static ringtonelist = [
    { id: 'Dreiklanggong', text: "Dreiklanggong" },
    { id: 'Glockenspiel', text: "Glockenspiel" },
    { id: 'Marimba', text: "Marimba" },
    { id: 'Suitcase_1', text: "Suitcase 1" },
    { id: 'Suitcase_2', text: "Suitcase 2" },
    { id: 'Vibraphone_1', text: "Vibraphone 1" },
    { id: 'Vibraphone_2', text: "Vibraphone 2" },
    { id: 'Vibraphone_3', text: "Vibraphone 3" }
  ];
  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    this.entity = createOptions['entity'];

    // optionals 
    $( '#ID_OPTIONAL' + this.dynid ).optional( { } );

    // get settings
    var isIndividualMode = false;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, ControlRingtone.widgetId );
    if( this.deviceRecord 
     && 'doorcalltone' in this.deviceRecord
     && 'storeycalltone' in this.deviceRecord
     && 'calltone' in this.deviceRecord
      ) {
      isIndividualMode = true;
    }

    // mode
    $( "#ID_RINGTONE" + this.dynid ).choice( {
      size: CONST.size.L,
      type: 'check',
      value: isIndividualMode,
      text: "{$control_ringtone_individual}",
      result: this.ControlRingtoneCheckResult.bind( this )
    });

    // fill lists
    $( "#ID_LIST_DOORCALL" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "droplist",
      data: ControlRingtone.ringtonelist
    } );

    $( "#ID_LIST_STOREYCALL" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "droplist",
      data: ControlRingtone.ringtonelist
    } );

    $( "#ID_LIST_DEFAULT" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "droplist",
      data: ControlRingtone.ringtonelist
    } );

    // set mode
    if( isIndividualMode )  {
      this.ModeIndividualTones();
    }
    else {
      this.ModeStandardTones();
    }

  }

  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // send DEVRESTART_NEEDED
    if( eventName == "modified" && param > 0 )  {
      this.FireEvent( "restartNeeded" );
    }

    // save
    if( eventName == "option" && param == "save" ) {
      return this.FormSave( data );
    }

    return false;
  }

  //-----------------------------------------------------------------
  //  FORM FUNCTIONS
  //-----------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    if( this.mode == "modeIndividualTones" ) {
      let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, ControlRingtone.widgetId );
      if( ! deviceRecord ) {
        deviceRecord = WidgetDeviceConfigBase.HelperAddDeviceFunction( entity, ControlRingtone.widgetId );
      }

      deviceRecord['doorcalltone'] = $( "#ID_LIST_DOORCALL" + this.dynid ).listbox( "option", "selected" )[ 0 ];
      deviceRecord['storeycalltone'] = $( "#ID_LIST_STOREYCALL" + this.dynid ).listbox( "option", "selected" )[ 0 ];
      deviceRecord['calltone'] = $( "#ID_LIST_DEFAULT" + this.dynid ).listbox( "option", "selected" )[ 0 ];
    }
    else {
      WidgetDeviceConfigBase.HelperRemoveDeviceFunction( entity, ControlRingtone.widgetId );
    }

    // remove old entity representation
    if( 'ringtone' in entity['sp'] ) {
      delete entity['sp']['ringtone'];
    }

    return true;
  }

  //-----------------------------------------------------------------
  //  CONTROL
  //-----------------------------------------------------------------

  ControlRingtoneCheckResult( value )
  {
    if( value ) {
      this.ModeIndividualTones();
    }
    else {
      this.ModeStandardTones();
    }
    this.Option( "modified", 1 );
  }

  //-----------------------------------------------------------------
  //  MODES
  //-----------------------------------------------------------------

  ModeStandardTones()
  {
    this.mode = "modeStandardTones";
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_DOORCALL" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_STOREYCALL" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_DEFAULT" );
    WidgetDeviceConfigBase.HelperRemoveDeviceFunction( this.entity, ControlRingtone.widgetId );
  }

  ModeIndividualTones()
  {
    this.mode = "modeIndividualTones";

    // get settings
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, ControlRingtone.widgetId );
    if( ! this.deviceRecord ) {
      this.deviceRecord = WidgetDeviceConfigBase.HelperAddDeviceFunction( this.entity, ControlRingtone.widgetId );
      this.deviceRecord['announcementtone'] = "Glockenspiel";
      this.deviceRecord['doorcalltone'] = "Suitcase_1";
      this.deviceRecord['storeycalltone'] = "Suitcase_1";
      this.deviceRecord['calltone'] = "Glockenspiel";
    }

    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_DOORCALL" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_STOREYCALL" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_DEFAULT" );    

    $( "#ID_LIST_DOORCALL" + this.dynid ).listbox( "option", "selected", [ this.deviceRecord['doorcalltone'] ] );
    $( "#ID_LIST_STOREYCALL" + this.dynid ).listbox( "option", "selected", [ this.deviceRecord['storeycalltone'] ] );
    $( "#ID_LIST_DEFAULT" + this.dynid ).listbox( "option", "selected", [ this.deviceRecord['calltone'] ] );

  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    // switch off
    if( param === false ) {
      return 0;
    }

    // validate
    return 0;
  }

  //-----------------------------------------------------------------
  //  UNIT TEST
  //-----------------------------------------------------------------

  OnUnitTest( utControl, param )
  {
  }


  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlRingtone.template;
    $element.viewable( createOptions );
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlRingtone.Register( ControlRingtone.template );
﻿//=============================================================================
//
//  CONTROL CALL PROFILE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlDoorCamera extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/control/doorCamera";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  isInitialized = false;
  internalCamera = null;
  selectedRecord = null;
  mode = null;

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    this.entity = createOptions['entity'];

    // interprete value in entity
    this.doorCamera = null;
    if( 'video' in this.entity && this.entity['video']['send'] === true ) {
      this.internalCamera = true;
    }
    else {
      this.internalCamera = false;
    }

    // build this.cameraList Dataset
    this.FormCreateCameraList( this.entity['entid'] );

    // fill controls
    $( "#ID_OPTIONAL" + this.dynid ).optional( { } );

    // build listbox control
    this.$listCtrl = this.$container.find( ".listbox" );
    this.$listCtrl.listbox( {
      size: CONST.size.L,
      type: "droplist",
      status: "normal",
      selected: [ this.selectedRecord['id'] ],
      data: this.cameraDataset,
      validate: this.ValidateDroplist.bind( this )
    } );

    // build edit control
    this.$editCtrl = this.$container.find( ".edit" );
    this.$editCtrl.edit( {
      size: CONST.size.L,
      status: "readonly",
      value: this.selectedRecord['text']
    } );


    if( this.cameraDataset.Size() == 1 ) {
      this.ModeReadonly();
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_LIST" );  
      $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_EDIT" );  
    }
    else {
      this.ModeSelectable();
      $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_LIST" );  
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_EDIT" );  
    }


    this.isInitialized = true;
  }


  //-----------------------------------------------------------------
  //  EVENT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  handler for .viewable( "option", ..., ... )
  //
  OnEvent( eventName, param, data )
  {
    // save
    if( eventName == "option" && param == "save" ) {
      return this.FormSave( data );
    }

    if( eventName == "option" && param == "references" ) {
      return this.FormReferences( data );
    }

    return false;
  }

  //---------------------------------------------------------------------------
  //  check all child cameras and put it into list
  //
  FormCreateCameraList( entityId ) 
  {
    this.cameraDataset = new Dataset( { id: 'id' } );
    var camList = [];

    // internal data 
    if( this.internalCamera )  {
      var record = {
        id: "internal",
        text: "{$control_doorcamera_value_internal}"
      };
      camList.push( record );
      this.doorCamera = "internal";
      this.selectedRecord = record;
    }
    else {
      var record = {
        id: "none",
        text: "{$control_doorcamera_value_none}"
      };
      camList.push( record );
      this.doorCamera = "none";
      this.selectedRecord = record;
    }

    // children
    let entInfoList = [];
    Data.Instance().TreeData().BranchEntInfoList( entityId, entInfoList );

    for( var entinfo of entInfoList )  {
      if( entinfo['class'] == "ipcam" )  {
        var record = {
          id: entinfo['entid'],
          text: entinfo['name']
        };
        camList.push( record );

        if( this.entity['video']['camera'] == entinfo['entid'] ) {
          this.selectedRecord = record;
        }
      }
    }
    
    this.cameraDataset.SetData( camList );
  }

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormReferences( entity )
  {
    return [];
  }

  //---------------------------------------------------------------------------
  //  save, called by form: .viewable( "option", "save", this.entity )
  //
  FormSave( entity )
  {
    if( this.mode == "modeSelectable" )  {
      let selected = this.$listCtrl.listbox( "option", "selected" );
      if( Array.isArray( selected ) && selected.length > 0 ) {
        var selectedRecord = this.cameraDataset.Get( selected[ 0 ] );
        switch( selectedRecord['id'] ) {

          case 'none':      
          case 'internal':  {  
            entity['video']['camera'] = null;   
          }  
          break;

          default: {
            entity['video']['camera'] = selectedRecord['id'];
          }
          break;
        }
      }
    }
    return true;
  }


  //-----------------------------------------------------------------
  //  MODES
  //-----------------------------------------------------------------

  ModeReadonly()
  {
    if( this.mode == "modeReadonly" ) 
      return;

    this.mode = "modeReadonly";
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_LIST" );  
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_EDIT" );  
  }

  ModeSelectable()
  {
    if( this.mode == "modeSelectable" ) 
      return;

    this.mode = "modeSelectable";
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_LIST" );  
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_EDIT" );
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    if( param === false ) {
      return this.$listCtrl.listbox( "validate", false );
    }
    return this.$listCtrl.listbox( "validate" );    
  }

  //-----------------------------------------------------------------
  //  validation handler for droplist
  //
  ValidateDroplist( validateControl )
  {
    let selected = this.$listCtrl.listbox( "option", "selected" );    
    return ControlDoorCamera.Validate( validateControl, selected );
  }
  

  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlDoorCamera.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    // Does selected call profile ID exist in current config?
    if( Array.isArray( selected ) && selected.length > 0 ) {    
    }
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlDoorCamera.Register( ControlDoorCamera.template );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigCall extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules  = [ "KBD", "IO", "TILE", "AZIO" ];
  static allowedTypes    = ["in"];
  static widgetIcon      = "mdi mdi-phone";
  static widgetNameShort = "{$widget_call_short}";
  static widgetNameLong  = "{$widget_call_long}";
  static widgetId        = "call";
  static widgetAvpIcon   = "kontakt_intern";
  static widgetFunctionLabel = "{$aui_widget_call}";

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/call";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    let targetEntityId = "";
    let targetNumber = "";
    let text = $.lang( "{$widget_call_error_descriptionline}" );

    if( ( 'param' in deviceRecord ) && ( 'targetEntityId' in deviceRecord['param'] ) ) {
      targetEntityId = deviceRecord['param']['targetEntityId'];
      targetNumber = deviceRecord['param']['target'];
    }
    let entinfo = Data.Instance().Entinfo( targetEntityId, entity );
    if( entinfo )  {
      text = $.lang( "{$widget_call_descriptionline}" );
      text = text.replace( "{target}", entinfo['name'] );
      text = text.replace( "{callnumber}", targetNumber );
    }

    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };

  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    // get device record
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    if( ( deviceRecord ) && ( 'param' in deviceRecord ) ) {
      let targetEntity = deviceRecord['param']['targetEntityId'];
      let targetCallnumber = deviceRecord['param']['target'];
      let reference = "call:" + targetEntity + ":" + targetCallnumber;
      return [ reference ];
    }

    return null;
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    if( ( deviceRecord ) && ( 'param' in deviceRecord ) ) {

      let targetEntityId   = deviceRecord['param']['targetEntityId'];
      let targetCallnumber = deviceRecord['param']['target'];
      let entinfo          = Data.Instance().Entinfo( targetEntityId, entity );

      if( !entinfo ) {
        validateControl.error = "{$const_error_device_deleted}";
        return 1;
      }

      if( targetCallnumber != entinfo['callnumber'] ) {
        validateControl.error = "{$const_error_callnumber_changed}";
        return 1;
      }
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {

    // set references to original data - DO NOT COPY!!!!
    // AVP common (method of base class)
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;

    // console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );
    // important: must be empty array if widget is just created to
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormLoadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord, WidgetDeviceConfigCall.widgetAvpIcon, WidgetDeviceConfigCall.widgetFunctionLabel );

    }

    // default value
    // distinguish between "deleted target" and "please select" in
    // calltarget control
    let currentSelection = [];

    let callnumber = null;
    if( this.deviceRecord['template'] == "call" ) {
      currentSelection.push( this.deviceRecord['param']['targetEntityId'] );
      callnumber = this.deviceRecord['param']['target'];
    }
    else  {
      this.preventFirstTimeValidate = true;
    }

    ControlCalltargetList.Build( $( "#ID_CALL_NUMBER" + this.dynid ), {
      //addressBook: this.entity['addrBook'],
      selected: currentSelection,
      forbiddenEntityId: this.entity['entid'],
      //result: this.ControlCalltargetResult.bind( this ),
      selectedCallnumber: callnumber
    } );

  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // ---- widget parameters ----
    let targetEntityId = $( "#ID_CALL_NUMBER" + this.dynid ).viewable( "option", "selected" )[ 0 ];
    let targetEntinfo = Data.Instance().Entinfo( targetEntityId );

    // standard: save param into device record
    this.deviceRecord['template']                = "call";
    this.deviceRecord['param']                   = {};
    this.deviceRecord['param']['targetEntityId'] = targetEntityId;  //this.targetEntity;
    this.deviceRecord['param']['target']         = targetEntinfo['callnumber']; //this.targetCallnumber;

    //if( this.targetEntity.includes( "door" ) ) {
    if( targetEntityId.includes( "door" ) ) {
      this.deviceRecord['param']['primType'] = "doorcall";
    }
    else {
      this.deviceRecord['param']['primType'] = "call";
    }


    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormReadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord );
    }

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate()
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }
    return $( "#ID_CALL_NUMBER" + this.dynid ).viewable( "validate" );
  }

  OnValidateOff()
  {
    $( "#ID_CALL_NUMBER" + this.dynid ).viewable( "validate", false );
  }

  //-----------------------------------------------------------------
  //  UNIT TEST
  //-----------------------------------------------------------------

  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Call" );

    this.TestCaseCallTarget( testControl );
    this.TestCaseCallTargetEntity( testControl );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //-----------------------------------------------------------------------------
  //  ResultHandlerCallTarget(): receives the result of call-target control
  //
  /*ControlCalltargetResult( result )
  {
    if( result )  {
      this.targetEntity     = result['id'];
      this.targetCallnumber = result['callnumber'];
    }
  }*/

  //-----------------------------------------------------------------------------
  //  TestCaseCallTarget(): test function for target ( call number)
  //
  TestCaseCallTarget( testController )
  {
    /*let testFunctionTimestamp = Date.now();
    if( this.deviceRecord['param']['target'] === this.targetCallnumber ) {
      testController.Success( `TestCaseCallTarget - pass [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
    else {
      testController.Error( `TestCaseCallTarget - failed [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }*/
  }

  //-----------------------------------------------------------------------------
  //  TestCaseCallTargetEntity(): test function for target entity
  //
  TestCaseCallTargetEntity( testController )
  {
    /*let testFunctionTimestamp = Date.now();
    if( this.deviceRecord['param']['targetEntityId'] === this.targetEntity ) {
      testController.Success( `TestCaseCallTargetEntity - pass [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
    else {
      testController.Error( `TestCaseCallTargetEntity - failed [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }*/
  }
}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigCall.Register( "/widget/deviceConfig/call" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigCall );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigDoorCall extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules  = [ "ATLC", "ATLM", "ATLCE1", "ATLCE2", "ATLCE3", "BTM", "BRMA" ];
  static allowedTypes    = ["in"];
  static widgetIcon      = "mdi mdi-phone";
  static widgetNameShort = "{$widget_doorcall_short}";
  static widgetNameLong  = "{$widget_doorcall_long}";
  static widgetId        = "doorcall";

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( "doorcall", entity );
    WidgetDeviceConfigBase.HelperRemovePermission( "doorcall", entity );
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/doorcall";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    // get device record
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    if( ( deviceRecord ) && ( 'param' in deviceRecord ) ) {
      let targetEntity = deviceRecord['param']['targetEntityId'];
      let targetCallnumber = deviceRecord['param']['target'];
      let reference = "call:" + targetEntity + ":" + targetCallnumber;
      return [ reference ];
    }

    return null;
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    if( ( deviceRecord ) && ( 'param' in deviceRecord ) ) {

      let targetEntityId   = deviceRecord['param']['targetEntityId'];
      let targetCallnumber = deviceRecord['param']['target'];
      let entinfo          = Data.Instance().Entinfo( targetEntityId, entity );

      if( !entinfo ) {
        validateControl.error = "{$const_error_device_deleted}";
        return 1;
      }

      if( targetCallnumber != entinfo['callnumber'] ) {
        validateControl.error = "{$const_error_callnumber_changed}";
        return 1;
      }
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    let targetEntityId = "";
    let targetNumber = "";
    let text = $.lang( "{$widget_doorcall_error_descriptionline}" );

    if( ( 'param' in deviceRecord ) && ( 'targetEntityId' in deviceRecord['param'] ) ) {
      targetEntityId = deviceRecord['param']['targetEntityId'];
      targetNumber = deviceRecord['param']['target'];
    }
    let entinfo = Data.Instance().Entinfo( targetEntityId, entity );
    if( entinfo ) {
      text = $.lang( "{$widget_doorcall_descriptionline}" );
      text = text.replace( "{target}", entinfo['name'] );
      text = text.replace( "{callnumber}", targetNumber );
    }

    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };

  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );

    // default value

    // important: must be empty array if widget is just created to
    // distinguish between "deleted target" and "please select" in
    // calltarget control
    let currentSelection = [];
    let callnumber = null;
    // previously saved: read from device record
    if( this.deviceRecord['template'] == "doorcall" ) {
      currentSelection.push( this.deviceRecord['param']['targetEntityId'] );
      callnumber = this.deviceRecord['param']['target'];

    }
    else {
      this.preventFirstTimeValidate = true;
    }

    // create control and set value
    ControlCalltargetList.Build( $( "#ID_DOORCALL_NUMBER" + this.dynid ), {
      addressBook: this.entity['addrBook'],
      selected: currentSelection,
      selectedCallnumber: callnumber,
      forbiddenEntityId: this.entity['entid'],
    } );

    // console.log( `${ this.constructor.name }( ${ this.uid }, ${ this.trigger } )`, this.deviceRecord );

  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // ---- widget parameters ----
    let targetEntityId = $( "#ID_DOORCALL_NUMBER" + this.dynid ).viewable( "option", "selected" )[ 0 ];
    let targetEntinfo = Data.Instance().Entinfo( targetEntityId );

    // standard: save param into device record
    this.deviceRecord['template']                = "doorcall";
    this.deviceRecord['param']                   = {};
    this.deviceRecord['param']['targetEntityId'] = targetEntityId;
    this.deviceRecord['param']['target']         = targetEntinfo['callnumber'];

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }

  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate()
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }
    return $( "#ID_DOORCALL_NUMBER" + this.dynid ).viewable( "validate" );
  }

  OnValidateOff()
  {
    $( "#ID_DOORCALL_NUMBER" + this.dynid ).viewable( "validate", false );
  }

  //-----------------------------------------------------------------
  //  UNITTEST
  //-----------------------------------------------------------------

  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Door Call" );

    this.TestCaseCallTarget( testControl );
    this.TestCaseCallTargetEntity( testControl );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }


  //-----------------------------------------------------------------------------
  //  TestCaseCallTarget(): test function for target ( call number)
  //
  TestCaseCallTarget( testController )
  {
    let testFunctionTimestamp = Date.now();
    if( this.deviceRecord['param']['target'] === this.targetCallnumber ) {
      testController.Success( `TestCaseCallTarget - pass [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
    else {
      testController.Error( `TestCaseCallTarget - failed [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
  }

  //-----------------------------------------------------------------------------
  //  TestCaseCallTargetEntity(): test function for target entity
  //
  TestCaseCallTargetEntity( testController )
  {
    let testFunctionTimestamp = Date.now();
    let targetEntityId = $( "#ID_DOORCALL_NUMBER" + this.dynid ).viewable( "option", "selected" )[ 0 ];
    if( this.deviceRecord['param']['targetEntityId'] === targetEntityId ) {
      testController.Success( `TestCaseCallTargetEntity - pass [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
    else {
      testController.Error( `TestCaseCallTargetEntity - failed [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
  }
}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigDoorCall.Register( "/widget/deviceConfig/doorcall" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigDoorCall );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigSetSp extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules  = [ "KBD", "TILE", "IO", "BTM", "BRMA", "ATLM", "ATLC", "ATLCE1", "ATLCE2", "ATLCE3", "AZIO", "TIMELINE", "SUS_APP", "TIMELINE" ];
  static allowedTypes    = ["in"];
  static widgetIcon      = "mdi mdi-flag";
  static widgetNameShort = "{$widget_set_sp_short}";
  static widgetNameLong  = "{$widget_set_sp_long}";
  static widgetId        = "setsp";
  static widgetAvpIcon       = "schalten";
  static widgetFunctionLabel = "{$aui_widget_setsp}";

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
    //WidgetDeviceConfigBase.HelperRemoveStatepoint( "setsp", entity );
    //WidgetDeviceConfigBase.HelperRemovePermission( "setsp", entity );
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/setsp";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // defaults
    let targetStatepoint = "";
    let text             = $.lang( "{$widget_setsp_descriptionline_error}" );

    // get target sp
    if( ( 'param' in deviceRecord ) && ( 'spaddr' in deviceRecord['param'] ) ) {
      targetStatepoint = deviceRecord['param']['spaddr'];
    }

    // target name: disinguish between own statepoint and foreign sp.
    let target = "";
    let spinfo = Data.Instance().StatepointInfo( targetStatepoint, entity );
    target     = spinfo['label'] || "";

    // action toggle/one/off
    if( deviceRecord['param']['value'] == "*" ) {
      text = $.lang( "{$widget_set_sp_descriptionline_toggle}" );
    }
    else if( deviceRecord['param']['value'] == "1" ) {
      text = $.lang( "{$widget_set_sp_descriptionline_on}" );
    }
    else if( deviceRecord['param']['value'] == "0" ) {
      text = $.lang( "{$widget_set_sp_descriptionline_off}" );
    }
    else {
      target = $.lang( "{$widget_set_sp_descriptionline}" );
    }
    text = text.replace( "{target}", target );

    // that's it
    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let spAddress = deviceRecord['param']['spaddr'];
    let spInfo = Data.Instance().StatepointInfo( spAddress, entity );

    // error statepoint does not exists
    if( spInfo['type'] == "ERROR" )  {
      validateControl.error = "{$const_error_statepoint_deleted}";
      return 1;
    }

    // error no permission
    if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddress, "w" ) ) {
      validateControl.error = "{$const_error_statepoint_permission}";
      return 1;
    }
    return 0;
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    if( ( deviceRecord ) && ( 'param' in deviceRecord ) ) {
      let reference = "sp:" + deviceRecord['param']['spaddr'];
      return [ reference ];
    }

    // no external ressources:
    return null;
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );
    this.isCommandMode = WidgetDeviceConfigBase.HelperIsCommandMode( this.deviceRecord );

    // console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormLoadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord, WidgetDeviceConfigSetSp.widgetAvpIcon, WidgetDeviceConfigSetSp.widgetFunctionLabel );
    }

    ControlStatepointList.Build( $( "#ID_SET_SP_TARGET" + this.dynid ), {
      size: CONST.size.L,
      inquirer: this.entity['entid'],
      entity: this.entity,
      pinnedEntityId: this.entity['entid'],
      allowedTypes: [ "BOOL" ],
      forbiddenNames: ["doormatic", "dnd"],

      result: this.ControlStatepointResult.bind( this )
    } );

    $( "#ID_SET_SP_VALUE" + this.dynid ).listbox( {
      type: 'droplist',
      selected: ['toggle'],
      size: CONST.size.L,
      data: [
        { id: "*", text: "{$widget_set_sp_toggle}" },
        { id: "1", text: "{$widget_set_sp_on}" },
        { id: "0", text: "{$widget_set_sp_off}" }
      ],
      text: "{$widget_empty_check_option1}"
    } );

    // fill data
    this.FormFillControls();
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // ---- widget parameters ----
    let targetAddress       = $( "#ID_SET_SP_TARGET" + this.dynid ).viewable( "option", "selected" )[0] ?? "";
    let value               = $( "#ID_SET_SP_VALUE" + this.dynid ).listbox( "option", "selected" )[0] ?? "";
    let icon                = $( "#ID_SET_SP_AVP_ICONS" + this.dynid ).listbox( "option", "value" );

    let targetEntityId      = targetAddress ? targetAddress.split( "/" )[1] : "";

    // standard: save param into device record
    this.deviceRecord['template']                = "setsp";
    this.deviceRecord['param']                   = {};
    this.deviceRecord['param']['targetEntityId'] = targetEntityId;
    this.deviceRecord['param']['spaddr']         = targetAddress;
    this.deviceRecord['param']['value']          = value;
    if( this.isCommandMode ) {
      this.deviceRecord['command'] = "SETSP(" + targetAddress + "," + value + ")";
    }

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormReadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord );
    }

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }


  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  FormFillControls(): fill the controls with already set widget values
  //
  FormFillControls()
  {
    // defaults (used on create)
    let selected                = [];
    let value                   = "*";
    let icon                    = "schalten";

    // previously saved: read from device record
    if( this.deviceRecord['template'] == "setsp" ) {

      // read from device record
      value = this.deviceRecord['param']['value'];
      selected.push( this.deviceRecord['param']['spaddr'] );
    }
    else  {
      this.preventFirstTimeValidate = true;
    }

    // update control
    $( "#ID_SET_SP_TARGET" + this.dynid ).viewable( "option", "selected", selected );
    $( "#ID_SET_SP_VALUE"  + this.dynid ).listbox ( "option", "selected", [value] );
  }

  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Set Sp" );

    this.TestCaseSetSpTarget( testControl, this.deviceRecord );
    this.TestCaseSetSpTargetEntityId( testControl, this.deviceRecord );
    this.TestCaseSetSpValue( testControl, this.deviceRecord );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //-----------------------------------------------------------------------------
  //  ResultHandlerCallTarget(): receives the result of call-target control
  //
  ControlStatepointResult( result )
  {
    if( result )  {
    }
  }

  //-----------------------------------------------------------------------------
  //  VALIDATION
  //-----------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  keys-dialog will call this
  //
  OnValidate()
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    let numError = 0;

    numError += $( '#ID_SET_SP_VALUE'  + this.dynid ).listbox( "validate" );
    numError += $( '#ID_SET_SP_TARGET' + this.dynid ).viewable( "validate" );

    return numError;
  }

  //-----------------------------------------------------------------------------
  //  keys-dialog will call this
  //
  OnValidateOff()
  {
    $( '#ID_SET_SP_VALUE' + this.dynid ).listbox( "validate", false );
    $( '#ID_SET_SP_TARGET' + this.dynid ).viewable( "validate", false );
  }


  //=============================================================================
  //  Test Cases
  //=============================================================================
  
  //-----------------------------------------------------------------------------
  //  TestCaseSetSpTarget(): description
  //
  TestCaseSetSpTarget( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = /\b.+\/[a-zA-Z]+\.\d+\/\w+\b/g;

    this.UnitTestCheckObject( "TestCaseSetSpTarget", testController, "param/spaddr", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseSetSpTargetEntityId(): test function for target entity
  //
  TestCaseSetSpTargetEntityId( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();
    let entityAddressAddressPattern = /\b(apartment|avp|door|jungpanel|client|group|sipaudio|susapp)\.\d+\b/g;

    this.UnitTestCheckObject( "TestCaseSetSpTargetEntityId", testController, "param/targetEntityId", deviceRecord, entityAddressAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseSetSpTargetEntityId(): test function for target entity
  //
  TestCaseSetSpValue( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();
    let possibleValues = ["*","0","1"];

    this.UnitTestCheckObject( "TestCaseSetSpValue", testController, "param/value", deviceRecord, possibleValues, testFunctionTimestamp );
  }
}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigSetSp.Register( "/widget/deviceConfig/setsp" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigSetSp );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigSetDnd extends WidgetDeviceConfigBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static allowedModules      = [ "KBD", "TILE", "IO", "BTM", "BRMA", "ATLM", "ATLC", "ATLCE1", "ATLCE2", "ATLCE3", "AZIO", "TIMELINE" ];
  static allowedTypes        = ["in"];
  static widgetIcon          = "mdi mdi-minus-circle";
  static widgetNameShort     = "{$widget_setdnd_short}";
  static widgetNameLong      = "{$widget_setdnd_long}";
  static widgetId            = "setdnd";
  static widgetAvpIcon       = "rufton_aus";
  static widgetFunctionLabel = "{$aui_widget_dnd}";  // for $FUNCTION in edit controls and avp default title

  //===========================================================================
  //  STATIC FUNCTIONS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( "setdnd", entity );
    WidgetDeviceConfigBase.HelperRemovePermission( "setdnd", entity );
  }

  //---------------------------------------------------------------------------
  //  build the widget
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/setdnd";
    $element.viewable( createData );
  }

  //---------------------------------------------------------------------------
  //  get references
  //
  static References( uid, trigger, entity )
  {
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    if( ( deviceRecord ) && ( 'param' in deviceRecord ) ) {
      let reference = "sp:" + deviceRecord['param']['spaddr'];
      return [ reference ];
    }

    // no references:
    return null;
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let spAddress = deviceRecord['param']['spaddr'];
    let spInfo = Data.Instance().StatepointInfo( spAddress, entity );
    
    // error statepoint does not exists
    if( spInfo['type'] == "ERROR" )  {
      validateControl.error = "{$const_error_statepoint_deleted}";
      return 1;
    }

    // error no permission
    if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddress, "w" ) ) {
      validateControl.error = "{$const_error_statepoint_permission}";
      return 1;
    }

    return 0;
  }

  //---------------------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // defaults
    let targetStatepoint = "";
    let text             = $.lang( "{$widget_setdnd_descriptionline_error}" );


    // get target sp
    if( ( 'param' in deviceRecord ) && ( 'spaddr' in deviceRecord['param'] ) ) {
      targetStatepoint = deviceRecord['param']['spaddr'];
    }
    // target name: disinguish between own statepoint and foreign sp.
    let target = "";
    let spinfo = Data.Instance().StatepointInfo( targetStatepoint, entity );
    target     = spinfo['label'] || "";

    // action toggle/one/off
    if( deviceRecord['param']['value'] == "*" ) {
      text = $.lang( "{$widget_setdnd_descriptionline_toggle}" );
    }
    else if( deviceRecord['param']['value'] == "1" ) {
      text = $.lang( "{$widget_setdnd_descriptionline_on}" );
    }
    else if( deviceRecord['param']['value'] == "0" ) {
      text = $.lang( "{$widget_setdnd_descriptionline_off}" );
    }
    else {
      target = $.lang( "{$widget_setdnd_descriptionline}" );
    }
    text = text.replace( "{target}", target );

    // that's it
    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //===========================================================================
  //  INSTANCE
  //===========================================================================

  //---------------------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );
    this.isCommandMode = WidgetDeviceConfigBase.HelperIsCommandMode( this.deviceRecord );

    // console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormLoadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord, WidgetDeviceConfigSetDnd.widgetAvpIcon, WidgetDeviceConfigSetDnd.widgetFunctionLabel );
    }

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );

    // dnd value
    $( "#ID_SET_DND_VALUE" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: 'droplist',
      selected: ["*"],
      data: [
        { id: "*", text: "{$widget_setdnd_value_toggle}" },
        { id: "1", text: "{$widget_setdnd_value_on}" },
        { id: "0", text: "{$widget_setdnd_value_off}" }
      ]
    } );

    // create statepoint list
    ControlStatepointList.Build( $( "#ID_SET_DND_TARGET" + this.dynid ), {
      entity: this.entity,
      inquirer: this.entity['entid'],
      pinnedEntityId: this.entity['entid'],
      allowedTypes:   ["BOOL"],
      allowedNames:   ["dnd"],
      result: this.ControlStatepointResult.bind( this ),
    } );

    // fill data
    this.FormFillControls();
  }

  //---------------------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    let targetStatepoint = null;
    targetStatepoint     = $("#ID_SET_DND_TARGET" + this.dynid ).viewable( "option", "selected" )[0];

    // standard: save param into device record
    this.deviceRecord['template']                = "setdnd";
    this.deviceRecord['param']                   = {};
    this.deviceRecord['param']['targetEntityId'] = targetStatepoint ? targetStatepoint.split("/")[1] : "";
    this.deviceRecord['param']['spaddr']         = targetStatepoint;
    this.deviceRecord['param']['value']          = $( "#ID_SET_DND_VALUE" + this.dynid ).listbox( "option", "selected" )[ 0 ];
    if( this.isCommandMode )  {
      this.deviceRecord['command'] = "SETSP(" + targetStatepoint + "," + this.deviceRecord['param']['value'] + ")";
    }

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormReadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord );
    }

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }


  //===========================================================================
  //  FORM
  //===========================================================================

  //---------------------------------------------------------------------------
  //  FormFillControls(): fill the controls with already set widget values
  //
  FormFillControls()
  {
    // defaults (used on create)
    let selected                = [];
    let value                   = "*";

    // previously saved: read from device record
    if( this.deviceRecord['template'] == "setdnd" ) {

      // read from device record
      value = this.deviceRecord['param']['value'];
      selected.push( this.deviceRecord['param']['spaddr'] );

    }
    else  {
      this.preventFirstTimeValidate = true;
      if( 'dnd' in this.entity['sp'] ) {
        selected.push( `${ Data.Instance().RootEntid() }/${ this.entity['entid'] }/dnd` );
      }
    }

    // update control
    $( "#ID_SET_DND_TARGET" + this.dynid ).viewable( "option", "selected", selected );
    $( "#ID_SET_DND_VALUE"  + this.dynid ).listbox ( "option", "selected", [ value ] );
  }


  //===========================================================================
  //  CONTROL HANDLER
  //===========================================================================

  //---------------------------------------------------------------------------
  //  ControlStatepointResult(): receives the result of call-target control
  //
  ControlStatepointResult( result )
  {
  }

  //===========================================================================
  //  VALIDATION (VIEWABLE OVERRIDE)
  //===========================================================================

  //---------------------------------------------------------------------------
  //  keys-dialog will call this at load & save
  //
  OnValidate( )
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    let numError = 0;
    numError += $( '#ID_SET_DND_VALUE' + this.dynid ).listbox( "validate" );
    numError += $( '#ID_SET_DND_TARGET' + this.dynid ).viewable( "validate" );
    return numError;
  }

  //---------------------------------------------------------------------------
  //  keys-dialog calls it when widget is removed
  //
  OnValidateOff( )
  {
    $( '#ID_SET_DND_VALUE' + this.dynid ).listbox( "validate", false );
    $( '#ID_SET_DND_TARGET' + this.dynid ).viewable( "validate", false );
  }


  //===========================================================================
  //  MODE
  //===========================================================================


  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  //---------------------------------------------------------------------------
  //  event from viewable, start unit test
  //
  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Set Dnd" );

    this.TestCaseSetDndTarget( testControl, this.deviceRecord );
    this.TestCaseSetDndTargetEntity( testControl, this.deviceRecord );
    this.TestCaseSetDndValue( testControl, this.deviceRecord );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //---------------------------------------------------------------------------
  //  TestCaseCallTarget(): test function for target ( call number)
  //
  TestCaseSetDndTarget( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = /\b.+\/[a-zA-Z]+\.\d+\/dnd\b/g;

    this.UnitTestCheckObject( "TestCaseSetDndTarget", testController, "param/spaddr", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }

  //---------------------------------------------------------------------------
  //  TestCaseCallTargetEntity(): test function for target entity
  //
  TestCaseSetDndTargetEntity( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();
    let entityPattern = /\b(apartment|avp|door|jungpanel|client|group|sipaudio|susapp)\.\d+\b/g;

    this.UnitTestCheckObject( "TestCaseSetDndTargetEntity", testController, "param/targetEntityId", deviceRecord, entityPattern, testFunctionTimestamp );
  }

  //---------------------------------------------------------------------------
  //  TestCaseSetDndValue(): test function for target entity
  //
  TestCaseSetDndValue( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();
    let possibleValues        = ["*", "0", "1"];

    this.UnitTestCheckObject( "TestCaseLastTrigger", testController, "param/value", deviceRecord, possibleValues, testFunctionTimestamp );
  }
}


//=============================================================================
//  REGISTER IN FACTORY
//=============================================================================

WidgetDeviceConfigSetDnd.Register( "/widget/deviceConfig/setdnd" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigSetDnd );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigSetRedir extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules  = [ "KBD", "TILE", "IO", "BTM", "BRMA", "ATLM", "ATLC", "ATLCE1", "ATLCE2", "ATLCE3", "AZIO", "TIMELINE" ];
  static allowedTypes    = ["in"];
  static widgetIcon      = "mdi mdi mdi-phone-missed";
  static widgetNameShort = "{$widget_set_redir_short}";
  static widgetNameLong  = "{$widget_set_redir_long}";
  static widgetId        = "setredir";
  static widgetAvpIcon       = "rufumleitung";
  static widgetFunctionLabel = "{$aui_widget_redir}";


  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( "setredir", entity );
    WidgetDeviceConfigBase.HelperRemovePermission( "setredir", entity );
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/setredir";
    $element.viewable( createData );
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord  = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let spAddress     = deviceRecord['param']['spaddr'];
    let entinfo       = Data.Instance().Entinfo( deviceRecord['param']['destentityId'], entity );

    // target device deleted
    if( !entinfo ) {
      validateControl.error = "{$const_error_device_deleted}";
      return 1;
    }

    // error no permission
    if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddress, "w" ) ) {
      validateControl.error = "{$const_error_statepoint_permission}";
      return 1;
    }

    // error no permission
    if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddress, "w" ) ) {
      validateControl.error = "{$const_error_statepoint_permission}";
      return 1;
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    // a statpoint in an external entity is a reference
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    if( deviceRecord['param']['spaddr'] ) {
      let destinationEntityId = deviceRecord['param']['destentityId'];
      return ["sp:" + deviceRecord['param']['spaddr'], `call:${destinationEntityId}`];
    }

    // no external ressources:
    return null;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // defaults
    let targetStatepoint = "";
    let text             = $.lang( "{$widget_set_redir_short}" );
    let time             = 0;

    // special handling: timeline
    if( deviceRecord['module'] == "TIMELINE" )  {
      if( 'param' in deviceRecord && 'value' in deviceRecord['param'] )  {
        switch( deviceRecord['param']['value'] )  {
          case '1': text = $.lang( "{$widget_set_redir_on}" )     + ": " + $.lang( "{$widget_set_redir_short}" ); break;
          case '0': text = $.lang( "{$widget_set_redir_off}" )    + ": " + $.lang( "{$widget_set_redir_short}" ); break;
          case '*': text = $.lang( "{$widget_set_redir_toggle}" ) + ": " + $.lang( "{$widget_set_redir_short}" ); break;
        }
      }
    }

    // get target sp
    if( ( 'param' in deviceRecord ) ) {

      if( 'spaddr' in deviceRecord['param'] )  {
        targetStatepoint = deviceRecord['param']['spaddr'];
      }

      if( 'time' in deviceRecord['param'] )  {
        time = deviceRecord['param']['time'];
      }

      if( 'time' in deviceRecord['param'] )  {
        switch( deviceRecord['param']['type'] )  {
          case "STATIC": text += ( " (" + $.lang( "{$widget_set_redir_type_static}" ) + ")" );           break;
          case "NAB":
            if( time === -1 )  {
              text += ( " (" + $.lang( `{$widget_set_redir_type_nab}` ) + ")" );
            }
            else {
              text += ( " (" + $.lang( `{$widget_set_redir_type_nab} ${time} Sek.` ) + ")" );
            }
            break;
          case "NA":     text += ( " (" + $.lang( `{$widget_set_redir_type_na} ${time} Sek.` ) + ")" );  break;
          case "BUSY":   text += ( " (" + $.lang( "{$widget_set_redir_type_busy}" ) + ")" );             break;
        }
      }
    }

    // exception on redir: comma not colon
    text += ", ";

    let spInfo = Data.Instance().StatepointInfo( targetStatepoint, entity );
    if( spInfo )  {
      text += ( "\"" + spInfo['label'] + "\"" );
    }

    // that's it
    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );
    this.isCommandMode = WidgetDeviceConfigBase.HelperIsCommandMode( this.deviceRecord );

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );


    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormLoadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord, WidgetDeviceConfigSetRedir.widgetAvpIcon, WidgetDeviceConfigSetRedir.widgetFunctionLabel );
    }

    ControlStatepointList.Build( $( "#ID_SET_REDIR_TARGET" + this.dynid ), {
      size: CONST.size.L,
      inquirer: this.entity['entid'],
      entity: this.entity,
      allowedTypes: [ "REDIR" ],
      pinnedEntityId: this.entity['entid'],
      result: this.ControlStatepointResult.bind( this )
    } );

    $( "#ID_SET_REDIR_TYPE" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: 'droplist',
      selected: ['STATIC'],
      data: [
        { id: "STATIC", text: "{$widget_set_redir_type_static}" },
        { id: "NA",     text: "{$widget_set_redir_type_na}" },
        { id: "NAB",    text: "{$widget_set_redir_type_nab}" }
      ],
      result: ( result ) => { this.ControlTypeResult( result ); }
    } );

    $( "#ID_SET_REDIR_TIME" + this.dynid ).edit( {
      size: CONST.size.L,
      value: "0",
      result: ( timeout ) => { this.ControlTimeResult( timeout ); },
      filter: { allow: "0123456789-" },
      validate: this.ValidateTime.bind( this ),
    } );

    $( "#ID_SET_REDIR_BUSY" + this.dynid ).choice( {
      type: "check",
      size: CONST.size.L,
      text: "{$widget_set_redir_on_busy_immediately}",
      value: "0",
      //result: ( result ) => { this.ControlTimeResult( result ) },
    } );

    $( "#ID_SET_REDIR_VALUE" + this.dynid ).listbox( {
      type: "droplist",
      size: CONST.size.L,
      selected: ["*"],
      data: [
        { id: "*", text: "{$widget_set_redir_toggle}" },
        { id: "1", text: "{$widget_set_redir_on}" },
        { id: "0", text: "{$widget_set_redir_off}" }
      ]
    } );
    if( ! this.isCommandMode ) {
      $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_VALUE" );
    }


    // fill data
    this.FormFillControls();
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // ---- widget parameters ----
    let targetAddress       = $( "#ID_SET_REDIR_TARGET" + this.dynid ).viewable( "option", "selected" )[0];
    let targetEntityId      = targetAddress.split( "/" )[1];
    let destEntityId        = $( "#ID_SET_REDIR_REDIRECT_TARGET" + this.dynid ).viewable( "option", "selected" )[0];
    let dest                = `${Data.Instance().RootEntid()}/${destEntityId}`;
    let time                = $( "#ID_SET_REDIR_TIME" + this.dynid ).edit( "option", "value" );
    let type                = $( "#ID_SET_REDIR_TYPE" + this.dynid ).listbox( "option", "selected" )[0];
    let isOnBusyImmediately = $( "#ID_SET_REDIR_BUSY" + this.dynid ).choice( "option", "value" );

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormReadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord );
    }

    switch( type )  {
      case "STATIC":
        time = 0;
        break;
      //  not necessary because it is default value
      // case "NA":
      //   time = time;
      //   break;
      case "NAB":
        time = -1;
        break;
    }

    if( type == "NA" && isOnBusyImmediately )  {
      type = "NAB";
    }

    // standard: save param into device record
    this.deviceRecord['template']                = "setredir";
    this.deviceRecord['param']                   = {};
    this.deviceRecord['param']['targetentityId'] = targetEntityId;
    this.deviceRecord['param']['spaddr']         = targetAddress;
    this.deviceRecord['param']['destentityId']   = destEntityId;
    this.deviceRecord['param']['dest']           = dest;
    this.deviceRecord['param']['time']           = time;
    this.deviceRecord['param']['type']           = type;
    this.deviceRecord['param']['busy']           = isOnBusyImmediately;
    this.deviceRecord['param']['prefix']         = "redir";

    if( this.isCommandMode ) {
      this.deviceRecord['param']['value'] = $( "#ID_SET_REDIR_VALUE" + this.dynid ).choice( "option", "value" )[ 0 ];
      if( this.deviceRecord['param']['value'] !== "0" )  {
        this.deviceRecord['command'] = "SETREDIR(" + targetAddress + "," + this.deviceRecord['param']['value'] + "," + dest +  ","  + time + "," + isOnBusyImmediately + ")";
      }
      else  {
        this.deviceRecord['command'] = "SETREDIR(" + targetAddress + "," + this.deviceRecord['param']['value'] + ")";
      }
    }

    //Unit Tests
    this.UnitTest();


    // return our copy of entity
    return this.entity;
  }


  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  FormFillControls(): fill the controls with already set widget values
  //
  FormFillControls()
  {
    // defaults (used on create)
    let selectedTarget          = [];
    let selectedDestination     = [];
    let time                    = 0;
    let type                    = "STATIC";
    let isOnBusyImmediately     = false;
    let destEntityId;
    let targetEntityId          = this.entity['entid'];
    let value                   = ["*"];

    // previously saved: read from device record
    if( this.deviceRecord['template'] == "setredir" ) {

      // read from device record
      selectedTarget.push( this.deviceRecord['param']['spaddr'] );
      selectedDestination.push( this.deviceRecord['param']['dest'].split( "/" )[1] );

      // update local saved data (needed for unit test)
      this.deviceRecord['param']['spaddr'].split( "/" )[1];
      destEntityId           = this.deviceRecord['param']['dest'].split( "/" )[1];
      targetEntityId         = this.deviceRecord['param']['targetentityId'] || this.entity['entid'];
      time                   = this.deviceRecord['param']['time'];
      isOnBusyImmediately    = this.deviceRecord['param']['busy'];
      type                   = this.deviceRecord['param']['type'];
      value                  = this.deviceRecord['param']['value'];
    }
    else {
      this.preventFirstTimeValidate = true;
      if( 'redir' in this.entity['sp'] )  {
        selectedTarget.push( `${Data.Instance().RootEntid()}/${this.entity['entid']}/redir` );
      }
    }

    if( type == "NAB" && time >= 0 )  {
      type = "NA";
    }

    ControlCalltargetList.Build( $( "#ID_SET_REDIR_REDIRECT_TARGET" + this.dynid ), {
      size: CONST.size.L,
      result: this.ControlCallTargetResult.bind( this ),
      forbiddenEntityId: ( targetEntityId || this.entity['entid'] ),
      addressBook: this.entity['addrBook'] || null,
      selected: [ destEntityId ],
    } );    

    // update control
    $( "#ID_SET_REDIR_TARGET" + this.dynid )         .viewable( "option", "selected", selectedTarget );
    $( "#ID_SET_REDIR_REDIRECT_TARGET" + this.dynid ).viewable( "option", "selected", selectedDestination );
    $( "#ID_SET_REDIR_TYPE"   + this.dynid )         .listbox ( "option", "selected", [ type ] );
    $( "#ID_SET_REDIR_TIME"   + this.dynid )         .edit    ( "option", "value",    time);

    switch( type )  {
      case "NA":
        this.ModeNa();
        break;
      case "NAB":
        this.ModeNab();
        break;
      case "STATIC":
        this.ModeStatic();
        break;
    }

    $( "#ID_SET_REDIR_TIME" + this.dynid ).edit( "option", "value", time );
    $( "#ID_SET_REDIR_BUSY" + this.dynid ).choice( "option", "value", isOnBusyImmediately );

    if( this.isCommandMode )  {
      $( "#ID_SET_REDIR_VALUE" + this.dynid ).listbox( "option", "selected", [ value ] ?? ["*"]);
    }

  }

  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Set Redir" );

    this.TestCaseSetRedirTarget( testControl, this.deviceRecord );
    this.TestCaseSetRedirDestination( testControl, this.deviceRecord );
    this.TestCaseSetRedirTargetEntityId( testControl, this.deviceRecord );
    this.TestCaseSetRedirDestinationEntityId( testControl, this.deviceRecord );
    this.TestCaseSetRedirType( testControl, this.deviceRecord );
    this.TestCaseSetRedirPrefix( testControl, this.deviceRecord );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //-----------------------------------------------------------------------------
  //  ResultHandlerCallTarget(): receives the result of call-target control
  //
  ControlStatepointResult( result )
  {
    if( result )  {
      let targetEntityId = result['entid'];

      ControlCalltargetList.Build( $( "#ID_SET_REDIR_REDIRECT_TARGET" + this.dynid ), {
        size: CONST.size.L,
        result: this.ControlCallTargetResult.bind( this ),
        forbiddenEntityId: targetEntityId,
        addressBook: this.entity['addrBook'] || null,
      } );
    }
  }

  //-----------------------------------------------------------------------------
  //  ControlTimeResult(): description
  //
  ControlTimeResult( timeout )
  {
    if( timeout > 120 ) {
      timeout = 120;
    }
    else if( isNaN( timeout ) || timeout <= 0 ) {
      timeout = 0;
    }

    $( "#ID_SET_REDIR_TIME" + this.dynid ).edit( "option", "value", timeout );
  }

  //-----------------------------------------------------------------------------
  //  ControlTypeResult(): description
  //
  ControlTypeResult( result )
  {
    switch( result[0].toUpperCase() )  {
      case "NA":
        this.ModeNa();
        break;
      case "NAB":
        this.ModeNab();
        break;
      case "STATIC":
        this.ModeStatic();
        break;
    }

  }

  //-----------------------------------------------------------------------------
  //  ResultHandlerCallTarget(): receives the result of call-target control
  //
  ControlCallTargetResult( result )
  {
    if( result )  {
      this.destEntity     = result['entid'];
      this.destCallnumber = result['callnumber'];
    }
  }

  //-----------------------------------------------------------------------------
  //  VALIDATION
  //-----------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  keys-dialog will call this
  //
  OnValidate()
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    let numError = 0;
    numError += $( '#ID_SET_REDIR_TARGET' + this.dynid ).viewable( "validate" );
    numError += $( '#ID_SET_REDIR_REDIRECT_TARGET' + this.dynid ).viewable( "validate" );
    if( this.mode == "ModeNa" )  {
      numError += $( '#ID_SET_REDIR_TIME' + this.dynid ).edit( "validate" );
    }

    return numError;
  }

  //-----------------------------------------------------------------------------
  //  keys-dialog will call this
  //
  OnValidateOff()
  {
    $( '#ID_SET_REDIR_TARGET' + this.dynid ).viewable( "validate", false );
    $( '#ID_SET_REDIR_REDIRECT_TARGET' + this.dynid ).viewable( "validate", false );
    $( '#ID_SET_REDIR_TIME' + this.dynid ).edit( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  name(): description
  //
  ValidateTime( data )
  {
    let time = parseInt( $( "#ID_SET_REDIR_TIME" + this.dynid ).edit( "option", "value")  );
    if( isNaN( time ) || time < -1 )  {
      data.error = "{$widget_set_redir_validate_time_error}";
      return 1;
    }
    if( time > 120 )  {
      data.error = "{$widget_set_redir_validate_time_error2}";
      return 1;
    }
    return 0;
  }

  //=============================================================================
  //  Modes
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  ModeNa(): description
  //
  ModeNa()
  {
    if( this.mode == "ModeNa" ) return;
    this.mode = "ModeNa";

    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_SET_REDIR_TIME" );
    $( '#ID_SET_REDIR_TIME' + this.dynid ).optional( "option", "value", "0" );
    $( '#ID_SET_REDIR_BUSY' + this.dynid ).optional( "option", "value", false );
  }

  //-----------------------------------------------------------------------------
  //  ModeNab(): description
  //
  ModeNab()
  {
    if( this.mode == "ModeNab" ) return;
    this.mode = "ModeNab";

    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_SET_REDIR_TIME" );
    $( "#ID_SET_REDIR_TIME" + this.dynid ).edit( "validate", false );
    $( '#ID_OPTIONAL_SET_REDIR_TIME' + this.dynid ).optional( "option", "value", "-1" );
  }

  //-----------------------------------------------------------------------------
  //  ModeStatic(): description
  //
  ModeStatic()
  {
    if( this.mode == "ModeStatic" ) return;
    this.mode = "ModeStatic";

    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_SET_REDIR_TIME" );
    $( "#ID_SET_REDIR_TIME" + this.dynid ).edit( "validate", false );
    $( '#ID_OPTIONAL_SET_REDIR_TIME' + this.dynid ).optional( "option", "value", "0" );
  }

  //=============================================================================
  //  TEST CASES
  //=============================================================================
  //-----------------------------------------------------------------------------
  //  TestCaseSetRedirTarget(): description
  //
  TestCaseSetRedirTarget( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = /\b.+\/[a-zA-Z]+\.\d+\/redir\b/g;

    this.UnitTestCheckObject( "TestCaseSetRedirTarget", testController, "param/spaddr", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseSetRedirDestination(): description
  //
  TestCaseSetRedirDestination( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let entityAddressAddressPattern = /\b.+\/[a-zA-Z]+\.\d+\b/g;

    this.UnitTestCheckObject( "TestCaseSetRedirDestination", testController, "param/dest", deviceRecord, entityAddressAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseSetRedirTargetEntityId(): description
  //
  TestCaseSetRedirTargetEntityId( testController, deviceRecord )
  {
    let testFunctionTimestamp       = Date.now();
    let entityAddressAddressPattern = /\b(apartment|avp|door|jungpanel|client|group|sipaudio|susapp)\.\d+\b/g;

    this.UnitTestCheckObject( "TestCaseSetRedirTargetEntityId", testController, "param/targetentityId", deviceRecord, entityAddressAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseSetRedirDestinationEntityId(): description
  //
  TestCaseSetRedirDestinationEntityId( testController, deviceRecord )
  {
    let testFunctionTimestamp       = Date.now();
    let entityAddressAddressPattern = /\b(apartment|avp|door|jungpanel|client|group|sipaudio|susapp)\.\d+\b/g;

    this.UnitTestCheckObject( "TestCaseSetRedirDestinationEntityId", testController, "param/destentityId", deviceRecord, entityAddressAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseSetRedirType(): description
  //
  TestCaseSetRedirType( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();
    let possibleValues        = ["STATIC", "NA", "NAB"];

    this.UnitTestCheckObject( "TestCaseRedir", testController, "param/type", deviceRecord, possibleValues, testFunctionTimestamp );

    switch( this.deviceRecord['param']['type'] ) {
      case "STATIC":
        this.UnitTestCheckObject( "TestCaseSetRedirType - time", testController, "param/time", deviceRecord, 0, testFunctionTimestamp );
        this.UnitTestCheckObject( "TestCaseSetRedirType - busy", testController, "param/busy", deviceRecord, false, testFunctionTimestamp );
        break;
      case "NA":
        this.UnitTestCheckObject( "TestCaseSetRedirType - time", testController, "param/time", deviceRecord, /[0-9]+/, testFunctionTimestamp );
        this.UnitTestCheckObject( "TestCaseSetRedirType - busy", testController, "param/busy", deviceRecord, [false,true], testFunctionTimestamp );
        break;
      case "NAB":
        this.UnitTestCheckObject( "TestCaseSetRedirType - time", testController, "param/time", deviceRecord, [-1], testFunctionTimestamp );
        this.UnitTestCheckObject( "TestCaseSetRedirType - busy", testController, "param/busy", deviceRecord, true, testFunctionTimestamp );
        break;
      default:
        testController.Error( `TestCaseSetRedirType - failed, Desired Parameter \" ${possibleValues.join("|")} \" is ''` );
        break;
    }
  }

  //-----------------------------------------------------------------------------
  //  TestCaseSetRedirPrefix(): description
  //
  TestCaseSetRedirPrefix( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();
    let possibleValues        = "redir";

    this.UnitTestCheckObject( "TestCaseSetRedirPrefix", testController, "param/prefix", deviceRecord, possibleValues, testFunctionTimestamp );
  }
}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigSetRedir.Register( "/widget/deviceConfig/setredir" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigSetRedir );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigSetSpKnx extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules      = [ "KBD", "TILE", "IO", "BTM", "BRMA", "ATLM", "ATLC", "ATLCE1", "ATLCE2", "ATLCE3", "AZIO", "TIMELINE", "SUS_APP" ];
  static allowedTypes        = ["in"];
  static widgetIcon          = "mdi mdi mdi-led-on";
  static widgetNameShort     = "{$widget_set_sp_knx_short}";
  static widgetNameLong      = "{$widget_set_sp_knx_long}";
  static widgetId            = "setspknx";
  static widgetAvpIcon       = "schalten";
  static widgetFunctionLabel = "{$aui_widget_set_sp_knx}";

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( deviceRecord['sp'], entity );
    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/setspknx";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  build description line, scheme:
  //  <action> (parameter): Target-SP
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // defaults
    let targetStatepoint = "";

    // get target sp
    targetStatepoint = `${ Data.Instance().RootEntid() }/${ entity['entid'] }/${ deviceRecord['sp'] }`;

    // target name: always use StatepointInfo for name handling
    let target   = deviceRecord['param']['knxGroupAddr'];
    let response = deviceRecord['param']['receiveKnxAddr'];

    // build everything
    let text;
    if( !response )  {
      text = $.lang( `{$widget_set_sp_knx_action} {$widget_set_sp_knx_group_address}: "${ target }"` );
    }
    else  {
      text = $.lang( `{$widget_set_sp_knx_action} {$widget_set_sp_knx_group_address}: "${ target }" {$widget_set_sp_knx_response}:"${response}"` );
    }

    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };

  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    // a statpoint in an external entity is a reference
    let deviceRecord            = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let isOwnStatepointSelected = ( deviceRecord['param']['spaddr'].indexOf( entity['entid'] ) >= 0 );
    if( deviceRecord['param']['spaddr'] ) {
      return ["sp:" + deviceRecord['param']['spaddr']];
    }

    // no external ressources:
    return null;
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let knxGw        = deviceRecord['param']['gateway'];
    let entinfo      = Data.Instance().Entinfo( knxGw, entity );


    if( !entinfo )  {
      validateControl.error = "{$const_error_device_deleted}";
      return 1;
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );
    this.isCommandMode = WidgetDeviceConfigBase.HelperIsCommandMode( this.deviceRecord );
    this.mode         = "receive_object_disabled";

    // console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormLoadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord, WidgetDeviceConfigSetSpKnx.widgetAvpIcon, WidgetDeviceConfigSetSpKnx.widgetFunctionLabel );
    }

    $( "#ID_SET_SP_KNX_VALUE" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: 'droplist',
      selected: ['*'],
      data: [
        { id: "*", text: "{$widget_set_sp_knx_toggle}" },
        { id: "1", text: "{$widget_set_sp_knx_on}" },
        { id: "0", text: "{$widget_set_sp_knx_off}" }
      ],
    } );

    $( "#ID_SET_SP_KNX_RECEIVE_OBJECT" + this.dynid ).choice( {
      size: CONST.size.L,
      type: 'check',
      value: false,
      result: ( result ) => {
        this.ControlReceiveObject( result );
      }
    } );

    $( "#ID_SET_SP_KNX_RECEIVE_OBJECT_VALUE" + this.dynid ).edit( {
      size: CONST.size.L,
      value: "",
      result: $.proxy( this.Modified, this ),
      validate: ( event ) => { 
        this.ValidateKnxAddress( event );
      },
    } );

    $( "#ID_SET_SP_KNX_GROUP_ADDRESS" + this.dynid ).edit( {
      size: CONST.size.L,
      value: "",
      result: $.proxy( this.Modified, this ),
      validate: ( event ) => { 
        this.ValidateKnxAddress( event );
      },
    } );

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );

    // fill data
    this.FormFillControls();
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // ---- widget parameters ----

    // {
    //   "template": "setspknx",
    //   "label": "knx7_7_7",
    //   "param": {
    //     "spaddr": "system.fj\/apartment.80\/knx_B710C7F0-E2F4-458A-963E-865BB60E7F69",
    //     "value": "*",
    //     "gateway": "knxgw.1",
    //     "door": false,
    //     "knxGroupAddr": "7\/7\/7",
    //     "knxresponse": true,
    //     "receiveKnxAddr": "1\/1\/1",
    //     "script": "setsp"
    //   },
    //   "sp": "knx_B710C7F0-E2F4-458A-963E-865BB60E7F69"
    // },
    let knxGroupAddress   = $( "#ID_SET_SP_KNX_GROUP_ADDRESS" + this.dynid ).edit( "option", "value" );
    let knxReceiveAddress = $( "#ID_SET_SP_KNX_RECEIVE_OBJECT_VALUE" + this.dynid ).edit( "option", "value" );
    let knxResponse       = $( "#ID_SET_SP_KNX_RECEIVE_OBJECT" + this.dynid ).choice( "option", "value" );
    let value             = $( "#ID_SET_SP_KNX_VALUE" + this.dynid ).listbox( "option", "selected" )[0];
    let spaddr            = "";
    let gateway           = $( "#ID_SET_SP_KNX_GATEWAY" + this.dynid ).viewable( "option", "selected" )[0];
    let door              = false;
    let script            = "setsp";
    let label             = "";
    let template          = "setspknx";
    let sp                = "";

    sp = this.CreateKnxSp( knxGroupAddress, knxResponse ? knxReceiveAddress : "", gateway );
    spaddr = sp['address'];
    if( !( "sp" in this.entity ) )  {
      this.entity['sp'] = {};
    }
    this.entity['sp'][ sp['name'] ] = sp;

    //=============================================================================
    //  basic settings
    //=============================================================================
    this.deviceRecord['label']                   = label;
    this.deviceRecord['template']                = template;
    this.deviceRecord['sp']                      = sp['name'];

    //=============================================================================
    //  params
    //=============================================================================
    if( !this.deviceRecord['param'] )  {
      this.deviceRecord['param'] = {};
    }

    this.deviceRecord['param']['knxresponse']    = knxResponse;
    this.deviceRecord['param']['knxGroupAddr']   = knxGroupAddress;
    this.deviceRecord['param']['receiveKnxAddr'] = knxResponse ? knxReceiveAddress : "";
    this.deviceRecord['param']['value']          = value;
    this.deviceRecord['param']['door']           = door;
    this.deviceRecord['param']['script']         = script;
    this.deviceRecord['param']['spaddr']         = spaddr;
    this.deviceRecord['param']['gateway']        = gateway;

    if( this.isCommandMode ) {
      this.deviceRecord['command'] = "SETSP(" + spaddr + "," + value + ")";
    }

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormReadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord );
    }

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }


  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  FormFillControls(): fill the controls with already set widget values
  //
  FormFillControls()
  {

    // defaults (used on create)
    let groupAddress         = "";
    let receiveObjectAddress = "";
    let knxresponse          = false;
    let gateway              = [];
    let value                = [ "*" ];

    ControlEntityList.Build( $( "#ID_SET_SP_KNX_GATEWAY" + this.dynid ), {
      allowedClass: "knxgw",
      //result: this.ControlCalltargetResult.bind( this )
    } );

    // previously saved: read from device record
    if( this.deviceRecord['template'] == "setspknx" ) {
      // read from device record
      knxresponse          = this.deviceRecord['param']['knxresponse'];
      groupAddress         = this.deviceRecord['param']['knxGroupAddr'];
      receiveObjectAddress = this.deviceRecord['param']['receiveKnxAddr'];
      value                = [ this.deviceRecord['param']['value'] ];
      gateway.push( this.deviceRecord['param']['gateway'] );

      //this.resultdata['deviceConfig']['param']['script']
      //this.resultdata['deviceConfig']['param']['spaddr']
    }
    else  {
      this.preventFirstTimeValidate = true;
    }


    $( "#ID_SET_SP_KNX_GATEWAY" + this.dynid ).viewable( "option", "selected", gateway );
    $( "#ID_SET_SP_KNX_VALUE" + this.dynid ).listbox( "option", "selected", value );
    $( "#ID_SET_SP_KNX_GROUP_ADDRESS" + this.dynid ).edit( "option", "value", groupAddress );
    $( "#ID_SET_SP_KNX_RECEIVE_OBJECT" + this.dynid ).choice( "option", "value", knxresponse );
    $( "#ID_SET_SP_KNX_RECEIVE_OBJECT_VALUE" + this.dynid ).edit( "option", "value", receiveObjectAddress );



    if( knxresponse )  {
      this.ModeReceiveObjectIsEnabled();
    }
    else {
      this.ModeReceiveObjectIsDisabled();
    }
  }

  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Set Sp Knx" );

    if( this.deviceRecord['module'] == "TILE" )  {
      this.TestCaseLabel( testControl, this.deviceRecord );
    }

    this.TestCaseWidgetTemplate( testControl, this.deviceRecord );
    this.TestCaseStatepoint( testControl, this.deviceRecord );

    this.TestCaseKnxGroupAddress( testControl, this.deviceRecord );
    this.TestCaseKnxReceiveAddress( testControl, this.deviceRecord );
    this.TestCaseValue( testControl, this.deviceRecord );
    this.TestCaseDoor( testControl, this.deviceRecord );
    this.TestCaseScript( testControl, this.deviceRecord );
    this.TestCaseSpaddr( testControl, this.deviceRecord );
    this.TestCaseGateway( testControl, this.deviceRecord );

    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //-----------------------------------------------------------------------------
  //  ControlAvailableMode(): description
  //
  ControlReceiveObject( result )
  {
    if( result === true ) {
      this.ModeReceiveObjectIsEnabled();
    }
    else {
      this.ModeReceiveObjectIsDisabled();
    }
  }

  //=============================================================================
  //  Modes
  //=============================================================================
  ModeReceiveObjectIsEnabled()
  {
    this.mode = "receive_object_enabled";
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_SET_SP_KNX_RECEIVE_OBJECT" );
  }

  ModeReceiveObjectIsDisabled()
  {
    this.mode = "receive_object_disabled";
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_SET_SP_KNX_RECEIVE_OBJECT" );
    $( '#ID_SET_SP_KNX_RECEIVE_OBJECT_VALUE' + this.dynid ).edit( "validate", false );
  }


//=============================================================================
//  Validation
//=============================================================================

  OnValidate()
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    let numError = 0;

    numError += $( "#ID_SET_SP_KNX_GATEWAY" + this.dynid ).viewable( "validate" );
    numError += $( "#ID_SET_SP_KNX_GROUP_ADDRESS" + this.dynid ).edit( "validate" );
    if( this.mode == "receive_object_enabled" )  {
      numError += $( "#ID_SET_SP_KNX_RECEIVE_OBJECT_VALUE" + this.dynid ).edit( "validate" );
    }

    return numError;
  }

  OnValidateOff()
  {
    $( "#ID_SET_SP_KNX_GATEWAY" + this.dynid ).viewable( "validate", false );
    $( "#ID_SET_SP_KNX_GROUP_ADDRESS" + this.dynid ).edit( "validate", false );
    $( "#ID_SET_SP_KNX_RECEIVE_OBJECT_VALUE" + this.dynid ).edit( "validate", false );
  }

  ValidateKnxAddress( event )
  {
    if( this.CheckKnxGroupAddress( event.value ) ) {
      event.error = null;
    }
    else {
      event.error = "{$widget_set_sp_knx_error_address}";
    }
  }


//=============================================================================
//  Validateion Helper
//=============================================================================

  /**
   * checkKnxGroupAddress( address )
   *
   * @param address address knx group address
   * checks if the entered knx group address is valid
   */
  CheckKnxGroupAddress( address )
  {
    if( !address ) {
      return false;
    }
    var splitAddress = address.split( "/" );

    var main   = parseInt( splitAddress[0] );
    var middle = parseInt( splitAddress[1] );
    var sub    = parseInt( splitAddress[2] );

    if( splitAddress.length == 3 ) {
      if( main == 0 && middle == 0 && sub == 0 ) {
        return false;
      }
      if( main < 0 || main > 31 ) {
        return false;
      }
      if( middle < 0 || middle > 7 ) {
        return false;
      }
      if( sub < 0 || sub > 255 ) {
        return false;
      }
    }
    else if( splitAddress.length == 2 ) {
      if( main == 0 && middle == 0 ) {
        return false;
      }
      if( main < 0 || main > 31 ) {
        return false;
      }
      if( middle < 0 || middle > 2047 ) {
        return false;
      }
    }
    else {
      if( isNaN( main ) ) {
        return false;
      }
      if( main <= 0 ) {
        return false;
      }
      if( main > 65535 ) {
        return false;
      }
    }
    return true;
  }

  //=============================================================================
  //  Statepoints
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  CreateKnxSp(): create special statepoint for knx
  //
  CreateKnxSp( knxSendAddr, knxReceiveAddr, knxgw )
  {
    var sp      = {};
    var time    = 0;
    var knxName = "";
    //try to get an active sp with same group address send
    knxName     = this.GetKnxSpName( knxSendAddr );

    sp[knxName]                           = {};
    sp[knxName]['name']                   = knxName;
    sp[knxName]['type']                   = "BOOL";
    //sp[ knxSpAddr ]['label']  = "KNX senden"; //TODO: languagefile;
    sp[knxName]['hidden']                 = true;
    sp[knxName]['HasDsr']                 = true;
    sp[knxName]['gateway']                = knxgw;
    sp[knxName]['knxdef']                 = [{
      "send": knxSendAddr,
      "receive": knxReceiveAddr,
      "dpt": "1.000"
    }];
    sp[knxName]['address']                = `${Data.Instance().RootEntid()}/${this.entity['entid']}/${knxName}`;
    sp[knxName]['default']                = false;
    sp[knxName]['template']               = "sendknx";



    // if( this.oldSendKnxAddr && this.oldSendKnxAddr != knxSendAddr ) {
    //   delete myEntity.sp["knx" + this.oldSendKnxAddr.replace( /\//g, "_" )];
    // }

    //AWFENTITY.entityUpdate( this.selfEntinfo.entid, myEntity, null);
    return sp[ knxName ];
  }

  //-----------------------------------------------------------------------------
  //  GetKnxSpName(): get current knx statepoint name or create a new one
  //
  GetKnxSpName( knxSendAddresse )
  {
    var statepoints = this.entity['sp'];
    var name        = "";
    for( var statepointNo in statepoints ) {
      var currentStatepoint = statepoints[statepointNo];
      if( currentStatepoint['knxdef'] && currentStatepoint['knxdef'][0]['send'] == knxSendAddresse ) {
        name = currentStatepoint['name'];
        break;
      }
    }

    if( name == "" ) {
      name = "knx_" + $.guidgen();
    }
    return name;
  }

  //-----------------------------------------------------------------------------
  //  TestCaseKnxGroupAddress():
  //
  TestCaseKnxGroupAddress( testController, deviceRecord )
  {
    let timestamp         = Date.now();
    let isValidKnxAddress = this.CheckKnxGroupAddress( deviceRecord['param']['knxGroupAddr'] );
    let testName          = "TestCaseKnxGroupAddress";
    let paramPathString   = "knxGroupAddr";
    let valueObject       = deviceRecord['param']['knxGroupAddr'];

    if( isValidKnxAddress ) {
      testController.Success( `${testName} - ok, Desired Parameter \"${paramPathString}\" [${( Date.now() - timestamp ) / 1000}s]` );
    }
    else {
      testController.Error( `${testName} - failed, Desired Parameter \"${paramPathString}\" is '${valueObject}' [${( Date.now() - timestamp ) / 1000}s]` );
    }
  }

  //-----------------------------------------------------------------------------
  //  TestCaseKnxReceiveAddress(): description
  //
  TestCaseKnxReceiveAddress( testController, deviceRecord )
  {
    let timestamp         = Date.now();
    let isValidKnxAddress = this.CheckKnxGroupAddress( deviceRecord['param']['knxGroupAddr'] );
    let testName          = "TestCaseKnxReceiveAddress";
    let paramPathString   = "knxGroupAddr";
    let valueObject       = deviceRecord['param']['knxGroupAddr'];

    if( isValidKnxAddress ) {
      testController.Success( `${testName} - ok, Desired Parameter \"${paramPathString}\" [${( Date.now() - timestamp ) / 1000}s]` );
    }
    else {
      testController.Error( `${testName} - failed, Desired Parameter \"${paramPathString}\" is '${valueObject}' [${( Date.now() - timestamp ) / 1000}s]` );
    }
  }

  //-----------------------------------------------------------------------------
  //  TestCaseDoor(): description
  //
  TestCaseDoor( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = [ false ];

    this.UnitTestCheckObject( "TestCaseDoor", testController, "param/door", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseScript(): description
  //
  TestCaseScript( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = [ "setsp" ];

    this.UnitTestCheckObject( "TestCaseScript", testController, "param/script", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseSpaddr(): description
  //
  TestCaseSpaddr( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = new RegExp( `${Data.Instance().RootEntid()}\/\\w+\\.\\d+\/knx_[A-Fa-f0-9\\-]`, "g" );

    this.UnitTestCheckObject( "TestCaseSpaddr", testController, "param/spaddr", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseGateway(): description
  //
  TestCaseGateway( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = /knxgw\.\d+/g;

    this.UnitTestCheckObject( "TestCaseGateway", testController, "param/gateway", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseLabel(): description
  //
  TestCaseLabel( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = /[\S ]+/g; // CAUTION: don't remove the space. ( e.g. "Max Musterman Door Backyard" )

    this.UnitTestCheckObject( "TestCaseLabel", testController, "label", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseWidgetTemplate(): description
  //
  TestCaseWidgetTemplate( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = "setspknx"; // CAUTION: don't remove the space. ( e.g. "Max Musterman Door Backyard" )

    this.UnitTestCheckObject( "TestCaseWidgetTemplate", testController, "template", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseStatepoint(): description
  //
  TestCaseStatepoint( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = /knx_[A-Za-e\d\-]+/g; // CAUTION: don't remove the space. ( e.g. "Max Musterman Door Backyard" )

    this.UnitTestCheckObject( "TestCaseStatepoint", testController, "sp", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseValue(): description
  //
  TestCaseValue( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = [ "*", "0", "1" ]; // CAUTION: don't remove the space. ( e.g. "Max Musterman Door Backyard" )

    this.UnitTestCheckObject( "TestCaseWidgetTemplate", testController, "param/value", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }
}
//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigSetSpKnx.Register( "/widget/deviceConfig/setspknx" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigSetSpKnx );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigSetDormatic extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules  = [ "KBD", "TILE", "IO", "BTM", "BRMA", "ATLM", "ATLC", "ATLCE1", "ATLCE2", "ATLCE3", "AZIO", "TIMELINE" ];
  static allowedTypes    = ["in"];
  static widgetIcon      = "mdi mdi-timer";
  static widgetNameShort = "{$widget_set_doormatic_short}";
  static widgetNameLong  = "{$widget_set_doormatic_long}";
  static widgetId        = "setdoormatic";
  static widgetAvpIcon   = "tuer";
  static widgetFunctionLabel = "{$aui_widget_doormatic}";

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( "setdoormatic", entity );
    WidgetDeviceConfigBase.HelperRemovePermission( "setdoormatic", entity );
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/setdoormatic";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // defaults
    let targetStatepoint = "";
    let text             = $.lang( "{$widget_set_doormatic_error_descriptionline}" );

    // get target sp
    if( ( 'param' in deviceRecord ) && ( 'spaddr' in deviceRecord['param'] ) ) {
      targetStatepoint = deviceRecord['param']['spaddr'];
    }

    // target name: disinguish between own statepoint and foreign sp.
    let target = "";
    let spinfo = Data.Instance().StatepointInfo( targetStatepoint, entity );
    target     = spinfo['label'] || "";

    // action toggle/one/off
    if( deviceRecord['param']['value'] == "*" ) {
      text = $.lang( "{$widget_set_doormatic_descriptionline_toggle}" );
    }
    else if( deviceRecord['param']['value'] == "1" ) {
      text = $.lang( "{$widget_set_doormatic_descriptionline_on}" );
    }
    else if( deviceRecord['param']['value'] == "0" ) {
      text = $.lang( "{$widget_set_doormatic_descriptionline_off}" );
    }
    else {
      target = $.lang( "{$widget_set_doormatic_descriptionline}" );
    }
    text = text.replace( "{target}", target );

    // that's it
    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    // a statpoint in an external entity is a reference
    let deviceRecord            = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let isOwnStatepointSelected = ( deviceRecord['param']['spaddr'].indexOf( entity['entid'] ) >= 0 );
    if( deviceRecord['param']['spaddr'] ) {
      return ["sp:" + deviceRecord['param']['spaddr']];
    }

    // no external ressources:
    return null;
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let spAddress = deviceRecord['param']['spaddr'];
    let spInfo = Data.Instance().StatepointInfo( spAddress, entity );

    // error statepoint does not exists
    if( spInfo['type'] == "ERROR" )  {
      validateControl.error = "{$const_error_statepoint_deleted}";
      return 1;
    }

    // error no permission
    if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddress, "w" ) ) {
      validateControl.error = "{$const_error_statepoint_permission}";
      return 1;
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );
    this.isCommandMode = WidgetDeviceConfigBase.HelperIsCommandMode( this.deviceRecord );

    // console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormLoadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord, WidgetDeviceConfigSetDormatic.widgetAvpIcon, WidgetDeviceConfigSetDormatic.widgetFunctionLabel );
    }

    ControlStatepointList.Build( $( "#ID_SET_DOORMATIC_TARGET" + this.dynid ), {
      size: CONST.size.L,
      inquirer: this.entity['entid'],
      entity: this.entity,
      pinnedEntityId: this.entity['entid'],
      allowedTypes: [ "BOOL" ],
      allowedNames: ["doormatic"],

      result: this.ControlStatepointResult.bind( this )
    } );

    $( "#ID_SET_DOORMATIC_VALUE" + this.dynid ).listbox( {
      type: 'droplist',
      size: CONST.size.L,
      data: [
        { id: "*", text: "{$widget_set_doormatic_toggle}" },
        { id: "1", text: "{$widget_set_doormatic_on}" },
        { id: "0", text: "{$widget_set_doormatic_off}" }
      ],
    } );

    // fill data
    this.FormFillControls();
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // ---- widget parameters ----

    // ---- widget parameters ----
    let targetAddress       = $( "#ID_SET_DOORMATIC_TARGET" + this.dynid ).viewable( "option", "selected" )[0] ?? "";
    let value               = $( "#ID_SET_DOORMATIC_VALUE" + this.dynid ).listbox( "option", "selected" )[0] ?? "";

    let targetEntityId      = targetAddress ? targetAddress.split( "/" )[1] : "";

    // standard: save param into device record
    this.deviceRecord['template']                = "setdoormatic";
    this.deviceRecord['param']                   = {};
    this.deviceRecord['param']['targetEntityId'] = targetEntityId;
    this.deviceRecord['param']['spaddr']         = targetAddress;
    this.deviceRecord['param']['value']          = value;
    if( this.isCommandMode ) {
      this.deviceRecord['command'] = "SETSP(" + targetAddress + "," + value + ")";
    }

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormReadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord );
    }

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }


  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  FormFillControls(): fill the controls with already set widget values
  //
  FormFillControls()
  {
    // defaults (used on create)
    let selected                = [];
    let value                   = "*";
    let icon                    = "tuer";
    this.target                 = null;


    if( this.entity['sp']['doormatic'] )  {
      selected = [ this.entity['sp']['doormatic']['address'] ];
    }

    // previously saved: read from device record
    if( this.deviceRecord['template'] == "setdoormatic" ) {

      // read from device record
      value = this.deviceRecord['param']['value'];
      selected = [ this.deviceRecord['param']['spaddr'] ];

      // update local saved data (needed for unit test)
    }
    else {
      this.preventFirstTimeValidate = true;
    }

    // update control
    $( "#ID_SET_DOORMATIC_TARGET" + this.dynid ).viewable( "option", "selected", selected );
    $( "#ID_SET_DOORMATIC_VALUE"  + this.dynid ).listbox ( "option", "selected", [value] );
  }

  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Set Doormatic" );

    this.TestCaseSetDoormaticTarget( testControl, this.deviceRecord );
    this.TestCaseSetDoormaticTargetEntityId( testControl, this.deviceRecord );
    this.TestCaseSetDoormaticValue( testControl, this.deviceRecord );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //-----------------------------------------------------------------------------
  //  VALIDATION
  //-----------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  keys-dialog will call this
  //
  OnValidate()
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    let numError = 0;
    numError += $( '#ID_SET_DOORMATIC_VALUE'  + this.dynid ).listbox( "validate" );
    numError += $( '#ID_SET_DOORMATIC_TARGET' + this.dynid ).viewable( "validate" );
    return numError;
  }

  //-----------------------------------------------------------------------------
  //  keys-dialog will call this
  //
  OnValidateOff()
  {
    $( '#ID_SET_DOORMATIC_VALUE' + this.dynid ).listbox( "validate", false );
    $( '#ID_SET_DOORMATIC_TARGET' + this.dynid ).viewable( "validate", false );
  }

  //=============================================================================
  //  Test Cases
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  TestCaseSetSpTarget(): description
  //
  TestCaseSetDoormaticTarget( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = /\b.+\/[a-zA-Z]+\.\d+\/\w+\b/g;

    this.UnitTestCheckObject( "TestCaseSetDoormaticTarget", testController, "param/spaddr", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseSetSpTargetEntityId(): test function for target entity
  //
  TestCaseSetDoormaticTargetEntityId( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();
    let entityAddressAddressPattern = /\b(apartment|avp|door|jungpanel|client|group|sipaudio|susapp)\.\d+\b/g;

    this.UnitTestCheckObject( "TestCaseSetDoormaticTargetEntityId", testController, "param/targetEntityId", deviceRecord, entityAddressAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseSetSpTargetEntityId(): test function for target entity
  //
  TestCaseSetDoormaticValue( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();
    let possibleValues = ["*","0","1"];

    this.UnitTestCheckObject( "TestCaseSetDoormaticValue", testController, "param/value", deviceRecord, possibleValues, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  ResultHandlerCallTarget(): receives the result of call-target control
  //
  ControlStatepointResult( result )
  {
  }


}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigSetDormatic.Register( "/widget/deviceConfig/setdoormatic" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigSetDormatic );
﻿ //=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigStoreyCall extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules      = ["KBD", "TILE", "IO", "BTM", "BRMA", "ATLM", "ATLC", "ATLCE1", "ATLCE2", "ATLCE3", "AZIO", "TIMELINE", "SUS_APP"];
  static allowedTypes        = ["in"];
  static widgetIcon          = "mdi mdi mdi-bell-ring-outline";
  static widgetNameShort     = "{$aui_widget_storeycall}";
  static widgetNameLong      = "{$widget_storey_call_long}";
  static widgetId            = "storeycall";
  static widgetAvpIcon       = "rufton_aus";
  static widgetFunctionLabel = "{$aui_widget_storeycall}";

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( deviceRecord['spname'], entity );
    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );

  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/storeycall";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    let reference = [];
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );


    if( ( deviceRecord ) && ( 'param' in deviceRecord ) ) {
      reference.push( "sp:" + deviceRecord['param']['spaddr'] );
    }

    if( deviceRecord['param']['addressDoor'] ) {
      reference.push( "sp:" + deviceRecord['param']['addressDoor'] );
    }

    if( deviceRecord['param']['addressLight'] ) {
      reference.push( "sp:" + deviceRecord['param']['addressLight'] );
    }

    if( deviceRecord['param']['target'] ) {
      let entinfoCalltraget = Data.Instance().Entinfo( deviceRecord['param']['targetEntityId'] );
      reference.push( "call:" + `${deviceRecord['param']['targetEntityId']}:${entinfoCalltraget['callnumber']}` );
    }

    if( reference.length > 0 )  {
      return reference;
    }
    // no references:
    return null;
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let spAddress = deviceRecord['param']['spaddr'];
    let spAddressDoor = deviceRecord['param']['addressDoor'];
    let spAddressLight = deviceRecord['param']['addressLight'];
    let targetEntityId = deviceRecord['param']['targetEntityId'];
    let targetCallnumber = deviceRecord['param']['target'];

    let spInfo = Data.Instance().StatepointInfo( spAddress, entity );
    let spInfoAddressDoor = Data.Instance().StatepointInfo( spAddressDoor, entity );
    let spInfoAddressLight = Data.Instance().StatepointInfo( spAddressLight, entity );
    let callTargetEntinfo = Data.Instance().Entinfo( targetEntityId, entity );

    if( !callTargetEntinfo ) {
      validateControl.error = "{$const_error_device_deleted}";
      return 1;
    }    

    if( targetCallnumber != callTargetEntinfo['callnumber'] ) {
      validateControl.error = "{$const_error_callnumber_changed}";
      return 1;
    }    

    // error statepoint does not exists
    if( spInfo['type'] == "ERROR" )  {
      validateControl.error = "{$const_error_statepoint_deleted}";
      return 1;
    }

    // error no permission
    if( !Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddress, "w" ) ) {
      validateControl.error = "{$const_error_statepoint_permission}";
      return 1;
    }

    // error statepoint does not exists or permission
    if( spInfoAddressDoor )  {
      if( spInfoAddressDoor['type'] == "ERROR" && spAddressDoor !== "" )  {
        validateControl.error = "{$const_error_statepoint_deleted}";
        return 1;
      }
      if( !Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddressDoor, "w" ) && spAddressDoor !== "" ) {
        validateControl.error = "{$const_error_statepoint_permission}";
        return 1;
      }
    }

    // error statepoint does not exists or no permission
    if( spInfoAddressLight ) {
      if( spInfoAddressLight['type'] == "ERROR" && spAddressLight !== "" )  {
        validateControl.error = "{$const_error_statepoint_deleted}";
        return 1;
      }
      if( !Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddressLight, "w" ) && spAddressLight !== "" ) {
        validateControl.error = "{$const_error_statepoint_permission}";
        return 1;
      }
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    //console.log( deviceRecord );
    let text = $.lang( "{$widget_storey_call_description}" );
    let targetEntity = "";

    // get target entity
    if( ( 'param' in deviceRecord ) && ( 'targetEntityId' in deviceRecord['param'] ) ) {
      targetEntity = deviceRecord['param']['targetEntityId'];
    }

    // get spinfo from server & set text
    let entinfo = Data.Instance().Entinfo( targetEntity, entity );
    if( entinfo ) {
      let paramText = $.lang( "{$widget_storey_call_description_target}" );
      paramText = paramText.replace( "{target}", entinfo['name'] );
      paramText = paramText.replace( "{callnumber}", entinfo['callnumber'] );
      text += ": " + paramText;
    }

    // that's it
    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }


  //-----------------------------------------------------------------
  //  update entity after local callnumber has changed
  //
  static UpdateEntity( deviceRecord, entity )
  {
    // is it a local statepoint?
    if( deviceRecord['param']['targetEntityId'] != entity['entid'] ) {
      return;
    }

    // update callnumber in device record
    if( ( 'param' in deviceRecord ) 
     && ( 'targetEntityId' in deviceRecord['param'] )
      ) {
      deviceRecord['param']['target'] = entity['phone']['callnumber'];
    }

    // update callnumber in statepoint
    if( ( 'spname' in deviceRecord )
     && ( deviceRecord['spname'] in entity['sp'] )
      ) {
      let sp = entity['sp'][ deviceRecord['spname'] ];
      sp['param']['target'] = entity['phone']['callnumber'];
    }

  }


  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );
    this.isCommandMode = WidgetDeviceConfigBase.HelperIsCommandMode( this.deviceRecord );

    // in createData always none, so change
    this.deviceRecord['template'] = WidgetDeviceConfigStoreyCall.widgetId;

    // create default if new (except dynamic statepoint will be created in save)
    if( !( 'param' in this.deviceRecord ) ) {
      this.deviceRecord['param'] = {};
      this.deviceRecord['param']['target'] = null;
      this.deviceRecord['param']['targetEntityId'] = null;
      this.deviceRecord['param']['addressDoor'] = null;
      this.deviceRecord['param']['addressLight'] = null;
    }

    // first statepoint in this entity, or no statepoint yet?
    if( !( 'sp' in this.entity ) )  {
      this.entity['sp'] = {};
    }


    if( !( this.deviceRecord['sp'] in this.entity['sp'] ) ) {
      let sp = this.HelperCreateStoreyCallSp();
      this.entity['sp'][ sp.name ] = sp;
      this.deviceRecord['sp'] = sp.name;
      this.preventFirstTimeValidate = true;
    }

    // console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormLoadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord, WidgetDeviceConfigStoreyCall.widgetAvpIcon, WidgetDeviceConfigStoreyCall.widgetFunctionLabel );
    }

    // build label
    ControlLabel.Build( $( "#ID_STOREY_CALL_LABEL" + this.dynid ), {
      label: this.entity['sp'][ this.deviceRecord['sp'] ]['label'],
      entityName: this.entity['name'],
      functionName: WidgetDeviceConfigStoreyCall.widgetNameShort
    } );

    // fill data
    this.FormFillControls();
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // ---- widget parameters ----
    // load sp
    let spName = this.deviceRecord['sp'] ?? this.deviceRecord['spname'];
    let sp;
    let spaddr;
    let label = $( "#ID_STOREY_CALL_LABEL" + this.dynid ).viewable( "option", "value" );

    if( !spName )  {
      sp = this.HelperCreateStoreyCallSp();
    }
    else  {
      sp = this.entity['sp'][ spName ];
    }

    spaddr = sp['address'];
    spName = sp['name'];

    // standard: save param into device record
    this.deviceRecord['template'] = "storeycall";
    this.deviceRecord['spname']   = spName;
    this.deviceRecord['sp']       = spName;

    // update device config param section
    let targetEntityId = $( "#ID_STOREY_CALL_TARGET" + this.dynid ).viewable( "option", "selected" )[0];
    let entinfo        = Data.Instance().Entinfo( targetEntityId );

    this.deviceRecord['param']                   = {};
    this.deviceRecord['param']['targetEntityId'] = targetEntityId;
    this.deviceRecord['param']['target']         = entinfo.callnumber;
    this.deviceRecord['param']['addressDoor']    = $( "#ID_STOREY_CALL_DOOR_OPENER" + this.dynid ).viewable( "option", "selected" )[0].replace( "none", "" );
    this.deviceRecord['param']['addressLight']   = $( "#ID_STOREY_CALL_DOOR_LIGHT" + this.dynid ).viewable( "option", "selected" )[0].replace( "none", "" );
    this.deviceRecord['param']['spaddr']         = spaddr;
    if( this.isCommandMode ) {
      this.deviceRecord['command'] = "SETSP(" + spaddr + ",1)";
    }
    
    //update sp param section
    sp['param']['addressDoor']  = this.deviceRecord['param']['addressDoor'];
    sp['param']['addressLight'] = this.deviceRecord['param']['addressLight'];
    sp['param']['target']       = this.deviceRecord['param']['target'];
    sp['label'] = label;
    //sp['label']                 = $( "#ID_STOREY_CALL_LABEL" + this.dynid ).viewable( "option", "value" );

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormReadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord );
    }
    // ---- widget statepoint ----

    // write safely into entity
    if( !( "sp" in this.entity ) )  {
      this.entity['sp'] = {};
    }
    this.entity['sp'][ spName ] = sp;

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }


  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  FormFillControls(): fill the controls with already set widget values
  //
  FormFillControls()
  {
    // previously saved: read from device record
    let target = [];
    let callnumber = this.deviceRecord['param']['target'];
    if( this.deviceRecord['template'] == "storeycall" ) {
      // arrays must be empty if no parameter is set
      if( this.deviceRecord['param']['targetEntityId'] ) {
        target.push( this.deviceRecord['param']['targetEntityId'] );
      }
      else if( 'phone' in this.entity ) {
        target.push( this.entity['entid'] );
      }
    }
    else {
      this.preventFirstTimeValidate = true;
    }

    ControlCalltargetList.Build( $( "#ID_STOREY_CALL_TARGET" + this.dynid ), {
      pinnedEntityId: this.entity['entid'],
      addressBook: this.entity['addrBook'] || null,
      selected: target,
      selectedCallnumber: callnumber,
      result: this.ControlCalltargetResult.bind( this )
    } );

    // fill (build) statepoint controls
    this.FormFillStatepointControls();
   
  }

  //-----------------------------------------------------------------------------
  //  fill doorlight & dooropener depending on call target
  //
  FormFillStatepointControls()
  {
    let selectedCallTarget = $( "#ID_STOREY_CALL_TARGET" + this.dynid ).viewable( "option", "selected" );



    // nothing selected, disable controls
    if( selectedCallTarget.length === 0 ) {

      ControlStatepointList.Build( $( "#ID_STOREY_CALL_DOOR_OPENER" + this.dynid ), {
        status: "normal",
        allowedTypes: ["BOOL"],
        selected: ['none'],
        noneRecord: true
      } );

      ControlStatepointList.Build( $( "#ID_STOREY_CALL_DOOR_LIGHT" + this.dynid ), {
        status: "normal",
        allowedTypes: ["BOOL"],
        selected: ['none'],
        noneRecord: true
      } );

    }

    // something selected, enable & set controls
    else {

      let addressDoor = [];
      let addressLight = [];
      addressDoor.push( this.deviceRecord['param']['addressDoor'] ? this.deviceRecord['param']['addressDoor'] : "none" );
      addressLight.push( this.deviceRecord['param']['addressLight'] ? this.deviceRecord['param']['addressLight'] : "none" );

      ControlStatepointList.Build( $( "#ID_STOREY_CALL_DOOR_OPENER" + this.dynid ), {
        inquirer: this.entity['entid'],
        entity: this.entity,
        allowedTypes: ["BOOL", "NONE"],
        noneRecord: true,
        selected: addressDoor
      } );

      ControlStatepointList.Build( $( "#ID_STOREY_CALL_DOOR_LIGHT" + this.dynid ), {
        inquirer: this.entity['entid'],
        entity: this.entity,
        allowedTypes: ["BOOL", "NONE"],
        selected: addressLight,
        noneRecord: true
      } );
    }
  }

  //-----------------------------------------------------------------
  //  unit test
  //-----------------------------------------------------------------

  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Set Storey Call" );

    this.TestCaseCallTarget( testControl );
    this.TestCaseCallTargetEntity( testControl );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );
    console.warn( "Missing Unit Tests, delayed to redesign test framework" );
    console.groupEnd();
  }

  //-----------------------------------------------------------------------------
  //  control
  //-----------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  ResultHandlerCallTarget(): receives the result of call-target control
  //
  ControlCalltargetResult( result )
  {
    if( result ) {
      this.deviceRecord['param']['addressDoor']  = null;
      this.deviceRecord['param']['addressLight'] = null;
    }

    this.FormFillStatepointControls();
  }

  //===========================================================================
  //  VALIDATION (VIEWABLE OVERRIDE)
  //===========================================================================

  //---------------------------------------------------------------------------
  //  keys-dialog will call this at load & save
  //
  OnValidate( )
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    let numError = 0;
    numError += $( '#ID_STOREY_CALL_TARGET' + this.dynid ).viewable( "validate" );
    numError += $( '#ID_STOREY_CALL_DOOR_OPENER' + this.dynid ).viewable( "validate" );
    numError += $( '#ID_STOREY_CALL_DOOR_LIGHT' + this.dynid ).viewable( "validate" );
    return numError;
  }

  //---------------------------------------------------------------------------
  //  keys-dialog calls it when widget is removed
  //
  OnValidateOff( )
  {
    $( '#ID_STOREY_CALL_TARGET' + this.dynid ).viewable( "validate", false );
    $( '#ID_STOREY_CALL_DOOR_OPENER' + this.dynid ).viewable( "validate", false );
    $( '#ID_STOREY_CALL_DOOR_LIGHT' + this.dynid ).viewable( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  unit test
  //-----------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  TestCaseCallTarget(): test function for target ( call number)
  //
  TestCaseCallTarget( testController )
  {
    let testFunctionTimestamp = Date.now();
    if( this.deviceRecord['param']['target'].match( /\d+/ ) !== null ) {
      testController.Success( `TestCaseCallTarget - pass [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
    else {
      testController.Error( `TestCaseCallTarget - failed [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
  }

  //-----------------------------------------------------------------------------
  //  TestCaseCallTargetEntity(): test function for target entity
  //
  TestCaseCallTargetEntity( testController )
  {
    let testFunctionTimestamp = Date.now();
    if( this.deviceRecord['param']['targetEntityId'].match( /\w*\.\d*/ ) !== null ) {
      testController.Success( `TestCaseCallTargetEntity - pass [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
    else {
      testController.Error( `TestCaseCallTargetEntity - failed [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
  }

  //-----------------------------------------------------------------------------
  //  helper
  //-----------------------------------------------------------------------------

  HelperCreateStoreyCallSp( callnumber, addressDoor, addressLight )
  {
    let statePointName = this.GetStoreyCallSpName();
    return  {
      "name": statePointName,
      "address": `${Data.Instance().RootEntid() }/${this.entity['entid']}/${statePointName}`,
      "type": "BOOL",
      "template": "storeycall",
      "default": false,
      "hidden": true,
      "datapoint": true,
      "exception": "storeycall",
      "label": `{{name}} (${ WidgetDeviceConfigStoreyCall.widgetNameShort })`,
      "param": {
        "target": callnumber,
        "addressDoor": addressDoor,
        "addressLight": addressLight
      }
    };
  }

  GetStoreyCallSpName()
  {
    return "stc_" + $.guidgen();
  }
}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigStoreyCall.Register( "/widget/deviceConfig/storeycall" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigStoreyCall );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigMuteMic extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules      = [ "KBD", "IO", "TILE", "AZIO" ];
  static allowedTypes        = ["in"];
  static widgetIcon          = "mdi mdi-microphone-off";
  static widgetNameShort     = "{$widget_mute_mic_short}";
  static widgetNameLong      = "{$widget_mute_mic_long}";
  static widgetId            = "mute_mic";
  static widgetAvpIcon       = "mikrofon_aus";
  static widgetFunctionLabel = "{$aui_widget_mute_mic}";

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything from entity
  //
  static Remove( uid, trigger, entity )
  {
    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( "mute_mic", entity );
    WidgetDeviceConfigBase.HelperRemovePermission( "mute_mic", entity );
    return true;
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/mute_mic";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    // no external ressources
    return null;
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    return 0;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    let text = $.lang( "{$widget_mute_mic_descriptionline}" );


    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };

  }

  //-----------------------------------------------------------------
  //  special: check if this widget is allowed in this entity
  //
  static IsUseable( entity, isFunction, isLed, module, type )
  {
    // this widget is for key
    if( isFunction || isLed )
      return false;

    // check if in allowed modules
    if( !( WidgetDeviceConfigMuteMic.allowedModules.includes( module ) ) )  {
      return false;
    }

    // check if in allowed modules
    if( !( WidgetDeviceConfigMuteMic.allowedTypes.includes( type ) ) ) {
      return false;
    }

    return ( entity['deviceTemplate'] !== "AFSV" && entity['deviceTemplate'] !== "AFS" ); //&& entity['class'] != "timeline" );
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );


    // console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    if( this.deviceRecord['template'] !== "mute_mic" ) {
      this.preventFirstTimeValidate = true;
    }

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormLoadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord, WidgetDeviceConfigMuteMic.widgetAvpIcon, WidgetDeviceConfigMuteMic.widgetFunctionLabel );
      $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_MUTE_MIC" );
    }
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    console.error( "device record different to old admin, old admin is not working!" );

    // ---- widget parameters ----
    let targetAddress       = `local\\mutemic`;

    // standard: save param into device record
    this.deviceRecord['template']        = "mute_mic";
    this.deviceRecord['param']           = {};
    this.deviceRecord['param']['spaddr'] = targetAddress;
    this.deviceRecord['param']['target'] = "";
    this.deviceRecord['param']['targetEntityId'] = "";

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormReadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord );
    }

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }

  //===========================================================================
  //  VALIDATION (VIEWABLE OVERRIDE)
  //===========================================================================

  //---------------------------------------------------------------------------
  //  keys-dialog will call this at load & save
  //
  OnValidate( )
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }
    return 0;
  }

  //---------------------------------------------------------------------------
  //  keys-dialog calls it when widget is removed
  //
  OnValidateOff( )
  {
  }

  //-----------------------------------------------------------------------------
  //  OnUnitTest(): description
  //
  OnUnitTest( testControl )
  {
    console.error( "Unit test does not cover all parameters, save was possible with a non working deviceRecord!" );

    let testTimestamp = Date.now();

    console.group( "Widget Grab Pic Menu" );

    this.TestCaseMuteMicSpAddr( testControl, this.deviceRecord );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //=============================================================================
  //  Test Cases
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  TestCaseMuteMicTargetEntityId(): description
  //
  TestCaseMuteMicSpAddr( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = /local\\mutemic/g;

    this.UnitTestCheckObject( "TestCaseMuteMicTargetEntityId", testController, "param/spaddr", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }
}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigMuteMic.Register( "/widget/deviceConfig/mute_mic" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigMuteMic );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigMuteSpeaker extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules = [ "KBD", "IO", "TILE", "AZIO" ];
  static allowedTypes = ["in"];
  static widgetIcon = "mdi mdi-music-note-off";
  static widgetNameShort = "{$widget_mute_speaker_short}";
  static widgetNameLong = "{$widget_mute_speaker_long}";
  static widgetId = "mute_speaker";
  static widgetAvpIcon       = "rufton_aus";
  static widgetFunctionLabel = "{$aui_widget_mute_speaker}";

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything from entity
  //
  static Remove( uid, trigger, entity )
  {
    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( "mute_speaker", entity );
    WidgetDeviceConfigBase.HelperRemovePermission( "mute_speaker", entity );

    return true;
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/mute_speaker";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    // no external ressources
    return null;
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    return 0;
  }

  //-----------------------------------------------------------------
  //  special: check if this widget is allowed in this entity
  //
  /*static IsUseable( entity, isFunction, isLed )
  {
    // this widget is for key
    if( isFunction || isLed )
      return false;

    return ( entity['deviceTemplate'] !== "AFSV" && entity['deviceTemplate'] !== "AFS" );
  }*/

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );


    // console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    if( this.deviceRecord['template'] !== "mute_speaker" ) {
      this.preventFirstTimeValidate = true;
    }

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormLoadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord, WidgetDeviceConfigMuteSpeaker.widgetAvpIcon, WidgetDeviceConfigMuteSpeaker.widgetFunctionLabel );
      $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_MUTE_SPEAKER" );
    }
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // ---- widget parameters ----
    let targetAddress       = `local\\muteringtone`;

    // standard: save param into device record
    this.deviceRecord['template']        = "mute_speaker";
    this.deviceRecord['param']           = {};
    this.deviceRecord['param']['spaddr'] = targetAddress;
    this.deviceRecord['param']['target'] = "";
    this.deviceRecord['param']['targetEntityId'] = "";

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormReadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord );
    }

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }

  //===========================================================================
  //  VALIDATION (VIEWABLE OVERRIDE)
  //===========================================================================

  //---------------------------------------------------------------------------
  //  keys-dialog will call this at load & save
  //
  OnValidate( )
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    return 0;
  }

  //---------------------------------------------------------------------------
  //  keys-dialog calls it when widget is removed
  //
  OnValidateOff( )
  {
  }


  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Mute Speaker / Ringtone" );

    this.TestCaseMuteSpeakerSpAddr( testControl, this.deviceRecord );
    this.TestCaseMuteSpeakerTarget( testControl, this.deviceRecord );
    this.TestCaseMuteSpeakerTargetEntityId( testControl, this.deviceRecord );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //=============================================================================
  //  Test Cases
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  TestCaseMuteSpeakerTargetEntityId(): description
  //
  TestCaseMuteSpeakerSpAddr( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = /local\\muteringtone/g;

    this.UnitTestCheckObject( "TestCaseMuteSpeakerTargetEntityId", testController, "param/spaddr", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseMuteSpeakerTargetEntityId(): description
  //
  TestCaseMuteSpeakerTarget( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    if( deviceRecord['param']['target'] !== "" )  {
      testController.Error( `TestCaseMuteSpeakerTarget - failed, Missing Parameter \"param/target\" [${ ( Date.now() - testFunctionTimestamp ) / 1000 }s]` );
    }
    else  {
      testController.Success( `TestCaseMuteSpeakerTarget - ok, Desired Parameter \"param/target\" [${ ( Date.now() - testFunctionTimestamp ) / 1000 }s]` );
    }
  }

  //-----------------------------------------------------------------------------
  //  TestCaseMuteSpeakerTargetEntityId(): description
  //
  TestCaseMuteSpeakerTargetEntityId( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    if( deviceRecord['param']['targetEntityId'] !== "" )  {
      testController.Error( `TestCaseMuteSpeakerTargetEntityId - failed, Missing Parameter \"param/targetEntityId\" [${ ( Date.now() - testFunctionTimestamp ) / 1000 }s]` );
    }
    else  {
      testController.Success( `TestCaseMuteSpeakerTarget - ok, Desired Parameter \"param/target\" [${ ( Date.now() - testFunctionTimestamp ) / 1000 }s]` );
    }
  }

}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigMuteSpeaker.Register( "/widget/deviceConfig/mute_speaker" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigMuteSpeaker );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigGrabPicMenu extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules = [];   // -> IsUseable
  static allowedTypes = [];     // -> IsUseable
  static widgetIcon = "mdi mdi-checkbox-multiple-blank-outline";
  static widgetNameShort = "{$widget_grabpicmenu_short}";
  static widgetNameLong = "{$widget_grabpicmenu_long}";
  static widgetId = "grabpicmenu";
  static widgetAvpIcon       = "tuerrufbilder";
  static widgetFunctionLabel = "{$aui_widget_grabpic}";

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything from entity
  //
  static Remove( uid, trigger, entity )
  {
    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( "grabpicmenu", entity );
    WidgetDeviceConfigBase.HelperRemovePermission( "grabpicmenu", entity );

    return true;
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/grabpicmenu";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    let text = $.lang( "{$widget_grabpicmenu_descriptionline}" );
    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };

  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    // no external ressources
    return null;
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    return 0;
  }

  //-----------------------------------------------------------------
  //  special: check if this widget is allowed in this entity
  //
  static IsUseable( entity, isFunction, isLed )
  {
    // this widget is for key
    if( isFunction || isLed )
      return false;

    return ( ( entity['class'] == "apartment" )
          && ( entity['deviceTemplate'] == "AFSV" ) 
           );
  }


  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );


    // console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    // previously saved: read from device record
    if( this.deviceRecord['template'] !== "grabpicmenu" ) {
      this.preventFirstTimeValidate = true;
    }

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormLoadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord, WidgetDeviceConfigGrabPicMenu.widgetAvpIcon, WidgetDeviceConfigGrabPicMenu.widgetFunctionLabel );
      $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_GRAB_PIC_MENU" );
    }
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // ---- widget parameters ----
    let targetAddress       = `${Data.Instance().RootEntid()}/${this.entity['entid']}/newpic`;
    let targetEntityId      = this.entity['entid'];

    // standard: save param into device record
    this.deviceRecord['template']                = "grabpicmenu";
    this.deviceRecord['param'] = {};
    this.deviceRecord['param']['targetEntityId'] = targetEntityId;
    this.deviceRecord['param']['spaddr']         = targetAddress;
    this.deviceRecord['param']['target']         = targetAddress;

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormReadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord );
    }

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }

  //===========================================================================
  //  VALIDATION (VIEWABLE OVERRIDE)
  //===========================================================================

  //---------------------------------------------------------------------------
  //  keys-dialog will call this at load & save
  //
  OnValidate( )
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    let numError = 0;
    return numError;
  }

  //---------------------------------------------------------------------------
  //  keys-dialog calls it when widget is removed
  //
  OnValidateOff( )
  {

  }

  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  //---------------------------------------------------------------------------
  //  event from viewable, start unit test
  //
  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Grab Pic Menu" );

    this.TestCaseGrabPicMenuTarget( testControl, this.deviceRecord );
    this.TestCaseGrabPicMenuTargetEntityId( testControl, this.deviceRecord );
    this.TestCaseGrabPicSpAddr( testControl, this.deviceRecord );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //=============================================================================
  //  Test Cases
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  TestCaseGrabPicMenuTarget(): description
  //
  TestCaseGrabPicMenuTarget( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = /\b.+\/[a-zA-Z]+\.\d+\/\w+\b/g;

    this.UnitTestCheckObject( "TestCaseGrabPicMenuTarget", testController, "param/spaddr", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseSetSpTargetEntityId(): test function for target entity
  //
  TestCaseGrabPicMenuTargetEntityId( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();
    let entityAddressAddressPattern = /\b(apartment|avp|door|jungpanel|client|group|sipaudio|susapp)\.\d+\b/g;

    this.UnitTestCheckObject( "TestCaseGrabPicMenuTargetEntityId", testController, "param/targetEntityId", deviceRecord, entityAddressAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseSetSpTargetEntityId(): test function for target entity
  //
  TestCaseGrabPicSpAddr( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();
    let statePointeAddressPattern = /\b.+\/[a-zA-Z]+\.\d+\/\w+\b/g;

    this.UnitTestCheckObject( "TestCaseGrabPicSpAddr", testController, "param/spaddr", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }
}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigGrabPicMenu.Register( "/widget/deviceConfig/grabpicmenu" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigGrabPicMenu );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigExternCall extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules  = [ "KBD", "IO", "TILE", "AZIO", "ATLC", "ATLCE1", "ATLCE2", "ATLCE3", "BRMA", "BTM", "ATLM" ];
  static allowedTypes    = ["in"];
  static widgetIcon      = "mdi mdi-phone-outgoing";
  static widgetNameShort = "{$aui_widget_externcall}";
  static widgetNameLong  = "{$widget_externcall_long}";
  static widgetId        = "externcall";
  static widgetAvpIcon   = "kontakt_extern";
  static widgetFunctionLabel = "{$aui_widget_externcall}";

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( "externcall", entity );
    WidgetDeviceConfigBase.HelperRemovePermission( "externcall", entity );
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/externcall";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    let targetNumber = deviceRecord['param']['target'];
    let text = $.lang( "{$widget_extern_call_descriptionline}" );
    text     = text.replace( "{callnumber}", targetNumber );


    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };

  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    //TODO: validate prefix?
    return 0;
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    //TODO: no references for externcall or may the corresponding gw ?
    return null;
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );
    this.isSaved      = false;
    this.addrBook     = this.entity['addrBook'] || null;

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormLoadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord, WidgetDeviceConfigExternCall.widgetAvpIcon, WidgetDeviceConfigExternCall.widgetFunctionLabel );
    }

    // important: must be empty array if widget is just created to
    // distinguish between "deleted target" and "please select" in
    // externcalltarget control
    let currentSelection = [];
    if( ( 'param' in this.deviceRecord ) && ( 'targetEntityId' in this.deviceRecord['param'] ) ) {
      currentSelection.push( this.deviceRecord['param']['targetEntityId'] );
    }
    
    // console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    $( "#ID_CALL_NUMBER" + this.dynid ).edit( {
      size: CONST.size.L,
      value: "",
      filter: { allow: "0123456789-()*#" }
    } );

    var addrEntinfo = Data.Instance().Entinfo( this.addrBook );
    if( $.isPlainObject( addrEntinfo ) )  {
      ControlAddressbookChoose.Build( $( "#ID_ADDRBK_POPUP" + this.dynid ), {
        anchor: ( "#ID_ADDRBK_BUTTON" + this.dynid ),
        addrBook: this.addrBook,
        result: ( record ) => { 
          $( "#ID_CALL_NUMBER" + this.dynid ).edit( "option", "value", record['callnumber'] );
        }
      } );
    }

    // fill data
    this.FormFillControls();
  }

  //-----------------------------------------------------------------
  //  key-dialog is externcalling to save the data into the entity
  //
  Save()
  {
    // ---- status for validation ----
    this.isSaved = true;

    // ---- widget parameters ----

    // standard: save param into device record
    this.deviceRecord['template']                = "externcall";
    //this.deviceRecord['sp'] = "externcall";
    this.deviceRecord['param']                   = {};
    this.deviceRecord['param']['targetEntityId'] = false;
    this.deviceRecord['param']['target']         = $("#ID_CALL_NUMBER" + this.dynid ).edit( "option", "value" );

    // ---- widget statepoint ----

    // (re-)create statepoint "empty"
    // The statepoint should always be recreated to remove 
    // params that are not longer used. 
    // let spCall = {
    //   name: "empty",
    //   address: `/${ Data.Instance().RootEntid() }/${ this.entity['entid'] }/externcall`,
    //   type: "BOOL",
    //   template: "externcall",
    //   default: false,
    //   persistent: false,
    //   hidden: false,
    //   label: "{$aui_widget_externcall}"
    // };

    // write safely into entity
    // if( !( "sp" in this.entity ) )  {
    //   this.entity['sp'] = {};
    // }
    // this.entity['sp']["empty"] = spEmpty;

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormReadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord );
    }

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }


  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  FormFillControls(): fill the controls with already set widget values
  //
  FormFillControls()
  {
    // if( this.deviceRecord['template'] === "externcall"
    // && this.deviceRecord['targetEntityId'] != false ) {
    if( this.deviceRecord['template'] === "externcall" )  {
      $( "#ID_CALL_NUMBER" + this.dynid ).edit( {
        size: CONST.size.L,
        value: this.deviceRecord['param']['target'],
        filter: { allow: "0123456789-()*#" },
        validate: this.ValidateCallNumber.bind( this )
        /*( data ) =>
        {
          data.error = null;
          if( !data.value ) {
            data.error = "{$widget_extern_call_error_validate_empty}";
            return 1;
          }
          else if( isNaN( data.value ) )  {
            data.error = "{$widget_extern_call_error_validate_not_a_number}";
            return 1;
          }
          else {
            return 0;
          }
        },*/
      } );
    }
    else  {
      this.preventFirstTimeValidate = true;
    }

    // }
  }

  OnValidate()
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    let num = 0;
    num +=  $( "#ID_CALL_NUMBER" + this.dynid ).edit( "validate", true );
    return num;
  }

  OnValidateOff()
  {
    $( "#ID_CALL_NUMBER" + this.dynid ).edit( "validate", false );
  }

  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Extern Call" );

    this.TestCaseCallTarget( testControl, this.deviceRecord );
    this.TestCaseCallTargetEntity( testControl, this.deviceRecord );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //-----------------------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------------------

  ValidateCallNumber( validateControl )
  {
    if( !validateControl.value ) {
      validateControl.error = "{$widget_extern_call_error_validate_empty}";
      return 1;
    }
    else {
      return 0;
    }
  }

  //-----------------------------------------------------------------------------
  //  TestCaseCallTarget(): test function for target ( externcall number)
  //
  TestCaseCallTarget( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = /\d+/g;

    this.UnitTestCheckObject( "TestCaseSetRedirTarget", testController, "param/target", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------------------
  //  TestCaseCallTargetEntity(): test function for target entity
  //
  TestCaseCallTargetEntity( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let possibleValues = [false];

    this.UnitTestCheckObject( "TestCaseCallTargetEntity", testController, "param/targetEntityId", deviceRecord, possibleValues, testFunctionTimestamp );
  }
}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigExternCall.Register( "/widget/deviceConfig/externcall" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigExternCall );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigLdOpener extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules      = [ "KBD", "TILE", "SUS_APP" ];
  static allowedTypes        = ["in"];
  static widgetIcon          = "mdi  mdi-key-plus";
  static widgetNameShort     = "{$widget_ldopener_short}";
  static widgetNameLong      = "{$widget_ldopener_long}";
  static widgetId            = "ldopener";
  static widgetAvpIcon       = "tuer_oeffnen";
  static widgetFunctionLabel = "{$aui_widget_ldopener}";

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    let foundAnotherDoorOpener = false;

    for( let keyConfig of entity['deviceConfig'] ) {
      if( keyConfig['template'] === "ldopener" && keyConfig['uid'] !== uid ) {
        foundAnotherDoorOpener = true;
        break;
      }
    }

    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
    if( foundAnotherDoorOpener === false ) {
      WidgetDeviceConfigBase.HelperRemoveStatepoint( "lastdoor", entity );
      WidgetDeviceConfigBase.HelperRemovePermission( "lastdoor", entity );
    }
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/ldopener";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    return [`sp:${Data.Instance().RootEntid()}/${entity['entid']}/lastdoor`];
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let spAddress = `${Data.Instance().RootEntid()}/${entity['entid']}/lastdoor`;
    let spInfo = Data.Instance().StatepointInfo( spAddress, entity );

    // error statepoint does not exists
    if( spInfo['type'] == "ERROR" )  {
      validateControl.error = "{$const_error_statepoint_deleted}";
      return 1;
    }

    // error no permission
    if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddress, "w" ) ) {
      validateControl.error = "{$const_error_statepoint_permission}";
      return 1;
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    let text = $.lang( "{$widget_ldopener_descriptionline}" );

    // get statepoint to check timeout parameter
    let statepoint = entity['sp']['lastdoor'];
    if( statepoint && ( 'alias' in statepoint ) )  {
      let timeout = statepoint['alias']['timeout'] || 0;
      if( timeout )  {
        let paramText = $.lang( "{$widget_ldopener_descriptionline_timeout}" );
        paramText = paramText.replace( "{timeout}", timeout / 60 );
        text += ", " + paramText;
      }
    }

    // that's it
    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );

    // console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormLoadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord, WidgetDeviceConfigLdOpener.widgetAvpIcon, WidgetDeviceConfigLdOpener.widgetFunctionLabel );
      $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_MUTE_SPEAKER" );
    }

    // first statepoint in this entity, or no statepoint yet?
    if( !( 'sp' in this.entity ) ) {
      let sp = this.HelperCreateLdOpenerSp();
      this.entity['sp'] = {};
      this.entity['sp'][ sp.name ] = sp;
      this.deviceRecord['sp'] = sp.name;
    }
    else if( !( 'lastdoor' in this.entity['sp'] ) ) {
      let sp = this.HelperCreateLdOpenerSp();
      this.entity['sp'][ sp.name ] = sp;
      this.deviceRecord['sp'] = sp.name;
    }

    $( "#ID_LDOPENER_IS_TIME_BASED" + this.dynid ).choice( {
      type: 'check',
      value: false,
      text: "{$widget_ldlight_works_until_time}",
      result: ( result ) => this.ControlIsTimeBased( result )
    } );

    $( "#ID_LDOPENER_TIME" + this.dynid ).edit( {
      size: CONST.size.M,
      value: 0,
      status: "normal",
      result: ( timeout )  => {
        if( timeout > 120 ) {
          timeout = 120;
        }
        else if( isNaN( timeout ) || timeout <= 0 ) {
          timeout = 0;
        }

        $( "#ID_LDOPENER_TIME" + this.dynid ).edit( "option", "value", timeout );
      },
      filter: { allow: "0123456789" }
    } );

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );

    // fill data
    this.FormFillControls();
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // ---- widget parameters ----
    let timeout     = parseInt( $( "#ID_LDOPENER_TIME" + this.dynid ).edit( "option", "value" ) );
    let isTimeBased = $( "#ID_LDOPENER_IS_TIME_BASED" + this.dynid ).choice( "option", "value" );

    if( timeout > 120 ) {
      timeout = 120;
    }
    else if( isTimeBased
      && ( isNaN( timeout ) || timeout <= 0 ) ) {
      timeout = 0;
    }

    // standard: save param into device record
    this.deviceRecord['template']         = "ldopener";
    this.deviceRecord['opener']           = true;
    this.deviceRecord['param']            = {};
    this.deviceRecord['param']['dynssmp'] = false;

    // ---- widget statepoint ----

    // (re-)create statepoint "empty"
    // The statepoint should always be recreated to remove
    // params that are not longer used.
    let spLastDoor = {
      name: "lastdoor",
      address: `/${Data.Instance().RootEntid()}/${this.entity['entid']}/lastdoor`,
      type: "BOOL",
      template: "lastdoor",
      default: false,
      persistent: false,
      hidden: true,
      label: "{$aui_widget_lastdoor}",
      alias: {
        target: null,
        timeout: timeout * 60
      }
    };

    // write safely into entity
    if( !( "sp" in this.entity ) )  {
      this.entity['sp'] = {};
    }
    this.entity['sp']["lastdoor"] = spLastDoor;

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormReadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord );
    }

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }


  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  FormFillControls(): fill the controls with already set widget values
  //
  FormFillControls()
  {
    let time = 0;

    if( this.deviceRecord['template'] == "ldopener" ) {
      time = parseInt( this.entity['sp']['lastdoor']['alias']['timeout'] );
    }
    else  {
      this.preventFirstTimeValidate = true;
    }

    if( time > 0 )  {
      this.ModeTimeBased();
      $( "#ID_LDOPENER_TIME" + this.dynid ).edit( "option", "value", time / 60 );
      $( "#ID_LDOPENER_IS_TIME_BASED" + this.dynid ).choice( "option", "value", true );
    }
    else {
      this.ModeWorksForEver();
      $( "#ID_LDOPENER_TIME" + this.dynid ).edit( "option", "value", 0 );
      $( "#ID_LDOPENER_IS_TIME_BASED" + this.dynid ).choice( "option", "value", false );
    }
  }

  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Last Door Light" );

    this.TestCaseLdOpenerTemplate( testControl );
    this.TestCaseLdOpenerDynssmp( testControl );
    this.TestCaseLdOpenerSp( testControl );

    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //-----------------------------------------------------------------------------
  //  TestCaseCallTarget(): test function for dynssmp
  //
  TestCaseLdOpenerDynssmp( testController )
  {
    let testFunctionTimestamp = Date.now();
    if( this.deviceRecord['param']['dynssmp'] === false ) {
      testController.Success( `TestCaseLdOpener - pass [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
    else {
      testController.Error( `TestCaseLdOpener - failed [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
  }

  //-----------------------------------------------------------------------------
  //  TestCaseLdOpenerTemplate(): test template is set correctly
  //
  TestCaseLdOpenerTemplate( testController )
  {
    let testFunctionTimestamp = Date.now();
    if( this.deviceRecord['template'] === "ldopener" ) {
      testController.Success( `TestCaseLdOpenerTemplate - pass [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
    else {
      testController.Error( `TestCaseLdOpenerTemplate - failed [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
  }

  //-----------------------------------------------------------------------------
  //  TestCaseLdOpenerSp(): test template is set correctly
  //
  TestCaseLdOpenerSp( testController )
  {
    let testFunctionTimestamp = Date.now();
    let lastdoorSp  = this.entity['sp']['lastdoor'];
    let regexStatepointgPattern = new RegExp( `${Data.Instance().RootEntid()}\\/\\w+\\.\\d+\\/lastlight`, "g");

    if( lastdoorSp )  {
      if( lastdoorSp['alias']
        && lastdoorSp['alias']['target'] === null
        && lastdoorSp['alias']['timeout']  >= 0  ) {
        testController.Success( `TestCaseLdOpenerTemplate - alias - pass [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
      }
      else {
        testController.Error( `TestCaseLdOpenerTemplate - alias - failed [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
      }

      if( lastdoorSp
        && lastdoorSp['hidden'] === true
        && lastdoorSp['persistent'] === false
        && lastdoorSp['type'] === "BOOL"
        && lastdoorSp['name'] === "lastdoor"
        && lastdoorSp['address'].match( regexStatepointgPattern ) !== null ) {

        testController.Success( `TestCaseLdOpenerTemplate - basic sp settings - pass [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
      }
      else  {
        testController.Error( `TestCaseLdOpenerTemplate - basic sp settings - failed [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
      }

    }
  }

  //-----------------------------------------------------------------------------
  //  ControlAvailableMode(): description
  //
  ControlIsTimeBased( result )
  {
    if( result === true )  {
      this.ModeTimeBased();
    }
    else {
      this.ModeWorksForEver();
    }
  }


  //-----------------------------------------------------------------------------
  //  VALIDATION
  //-----------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  keys-dialog will call this
  //
  OnValidate()
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    let numError = 0;
    numError += $( "#ID_LDLIGHT_TIME" + this.dynid ).edit( "validate" );
    return numError;
  }

  //-----------------------------------------------------------------------------
  //  keys-dialog will call this
  //
  OnValidateOff()
  {
    $( "#ID_LDLIGHT_TIME" + this.dynid ).edit( "validate", false );
  }

  //=============================================================================
  //  MODES
  //=============================================================================
  //-----------------------------------------------------------------------------
  //  ModeTimeBased(): description
  //
  ModeTimeBased()
  {
    this.mode = "modeTimeBase";
    $( "#ID_LDOPENER_TIME" + this.dynid ).edit( "option", "value", 10 );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_IS_TIME_BASED" );
  }

  //-----------------------------------------------------------------------------
  //  ModeWorksForEver(): description
  //
  ModeWorksForEver()
  {
    this.mode = "modeWorksForEver";
    $( "#ID_LDOPENER_TIME" + this.dynid ).edit( "option", "value", 0 );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_IS_TIME_BASED" );
  }

  //-----------------------------------------------------------------------------
  //  HelperCreateLdOpenerSp(): description
  //
  HelperCreateLdOpenerSp()
  {
    let sp = {
      name: "lastdoor",
      address: `/${Data.Instance().RootEntid()}/${this.entity['entid']}/lastdoor`,
      type: "BOOL",
      template: "lastdoor",
      default: false,
      persistent: false,
      hidden: true,
      label: "{$aui_widget_lastdoor}",
      alias: {
        target: null,
        timeout: 0
      }
    };

    return sp;
  }
}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigLdOpener.Register( "/widget/deviceConfig/ldopener" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigLdOpener );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigLdLight extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules      = [ "KBD", "TILE", "SUS_APP" ];
  static allowedTypes        = ["in"];
  static widgetIcon          = "mdi  mdi-lightbulb-outline";
  static widgetNameShort     = "{$widget_ldlight_short}";
  static widgetNameLong      = "{$widget_ldlight_long}";
  static widgetId            = "ldlight";
  static widgetAvpIcon       = "licht";
  static widgetFunctionLabel = "{$aui_widget_ldlight}";

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    let foundAnotherDoorLight = false;

    for( let keyConfig of entity['deviceConfig'] ) {
      if( keyConfig['template'] === "ldlight" && keyConfig['uid'] !== uid ) {
        foundAnotherDoorLight = true;
        break;
      }
    }

    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );

    if( foundAnotherDoorLight === false )  {
      WidgetDeviceConfigBase.HelperRemoveStatepoint( "lastlight", entity );
      WidgetDeviceConfigBase.HelperRemovePermission( "lastlight", entity );
    }
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/ldlight";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    return [`sp:${Data.Instance().RootEntid()}/${entity['entid']}/lastlight`];
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let spAddress = `${Data.Instance().RootEntid()}/${entity['entid']}/lastlight`;
    let spInfo = Data.Instance().StatepointInfo( spAddress, entity );

    // error statepoint does not exists
    if( spInfo['type'] == "ERROR" )  {
      validateControl.error = "{$const_error_statepoint_deleted}";
      return 1;
    }

    // error no permission
    if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddress, "w" ) ) {
      validateControl.error = "{$const_error_statepoint_permission}";
      return 1;
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    let text = $.lang( "{$widget_ldlight_descriptionline}" );

    // get statepoint to check timeout parameter
    let statepoint = entity['sp']['lastlight'];
    if( statepoint && ( 'alias' in statepoint ) ) {
      let timeout = statepoint['alias']['timeout'] || 0;
      if( timeout ) {
        let paramText = $.lang( "{$widget_ldlight_descriptionline_timeout}" );
        paramText = paramText.replace( "{timeout}", timeout / 60 );
        text += ", " + paramText;
      }
    }

    // that's it
    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );

    // ---- status for validation ----
    this.isSaved = false;

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormLoadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord, WidgetDeviceConfigLdLight.widgetAvpIcon, WidgetDeviceConfigLdLight.widgetFunctionLabel );
      $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_MUTE_SPEAKER" );
    }

    // first statepoint in this entity, or no statepoint yet?
    if( !( 'sp' in this.entity ) ) {
      let sp = this.HelperCreateLdLightSp();
      this.entity['sp'] = {};
      this.entity['sp'][ sp.name ] = sp;
      this.deviceRecord['sp'] = sp.name;
    }
    else if( !( 'lastlight' in this.entity['sp'] ) ) {
      let sp = this.HelperCreateLdLightSp();
      this.entity['sp'][ sp.name ] = sp;
      this.deviceRecord['sp'] = sp.name;
    }

    $( "#ID_LDLIGHT_IS_TIME_BASED" + this.dynid ).choice( {
      type: 'check',
      value: false,
      text: "{$widget_ldlight_works_until_time}",
      result: ( result ) => this.ControlIsTimeBased( result )
    } );

    $( "#ID_LDLIGHT_TIME" + this.dynid ).edit( {
      size: CONST.size.M,
      value: 0,
      status: "normal",
      result: ( timeout )  => {
        if( timeout > 120 ) {
          timeout = 120;
        }
        else if( isNaN( timeout ) || timeout <= 0 ) {
          timeout = 0;
        }

        $( "#ID_LDLIGHT_TIME" + this.dynid ).edit( "option", "value", timeout );
      },
      filter: { allow: "0123456789" },
     } );

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );

    // fill data
    this.FormFillControls();
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // ---- status for validation ----
    this.isSaved = true;

    // ---- widget parameters ----
    let timeout     = parseInt( $( "#ID_LDLIGHT_TIME" + this.dynid ).edit( "option", "value" ) );
    let isTimeBased = $( "#ID_LDLIGHT_IS_TIME_BASED" + this.dynid ).choice( "option", "value" );

    if( timeout > 120 ) {
      timeout = 120;
    }
    else if( isTimeBased
      && ( isNaN( timeout ) || timeout <= 0 ) ) {
      timeout = 0;
    }

    // standard: save param into device record
    this.deviceRecord['template']         = "ldlight";
    this.deviceRecord['opener']           = true;
    this.deviceRecord['param']            = {};
    this.deviceRecord['param']['dynssmp'] = false;

    // ---- widget statepoint ----

    // (re-)create statepoint "empty"
    // The statepoint should always be recreated to remove
    // params that are not longer used.
    let spLastLight = {
      name: "lastlight",
      address: `${Data.Instance().RootEntid()}/${this.entity['entid']}/lastlight`,
      type: "BOOL",
      template: "lastlight",
      default: false,
      persistent: false,
      hidden: true,
      label: "{$aui_widget_lastlight}",
      alias: {
        target: null,
        timeout: timeout * 60
      }
    };

    // write safely into entity
    if( !( "sp" in this.entity ) )  {
      this.entity['sp'] = {};
    }
    this.entity['sp']["lastlight"] = spLastLight;


    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormReadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord );
    }

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }

  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //
  //  FormFillControls(): fill the controls with already set widget values
  FormFillControls()
  {
    let time = 0;
    if( this.deviceRecord['template'] == "ldlight" ) {
      time = parseInt( this.entity['sp']['lastlight']['alias']['timeout'] );
    }
    else  {
      this.preventFirstTimeValidate = true;
    }

    if( time > 0 )  {
      this.ModeTimeBased();
      $( "#ID_LDLIGHT_TIME" + this.dynid ).edit( "option", "value", time / 60 );
      $( "#ID_LDLIGHT_IS_TIME_BASED" + this.dynid ).choice( "option", "value", true );
    }
    else {
      this.ModeWorksForEver();
      $( "#ID_LDLIGHT_TIME" + this.dynid ).edit( "option", "value", 0 );
      $( "#ID_LDLIGHT_IS_TIME_BASED" + this.dynid ).choice( "option", "value", false );
    }
  }

  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Last Door Light" );

    this.TestCaseLdLightTemplate( testControl );
    this.TestCaseLdLightDynssmp( testControl );
    this.TestCaseLdLightSp( testControl );

    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //-----------------------------------------------------------------------------
  //  TestCaseCallTarget(): test function for dynssmp
  //
  TestCaseLdLightDynssmp( testController )
  {
    let testFunctionTimestamp = Date.now();
    if( this.deviceRecord['param']['dynssmp'] === false ) {
      testController.Success( `TestCaseLdLight - pass [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
    else {
      testController.Error( `TestCaseLdLight - failed [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
  }

  //-----------------------------------------------------------------------------
  //  TestCaseLdLightTemplate(): test template is set correctly
  //
  TestCaseLdLightTemplate( testController )
  {
    let testFunctionTimestamp = Date.now();
    if( this.deviceRecord['template'] === "ldlight" ) {
      testController.Success( `TestCaseLdLightTemplate - pass [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
    else {
      testController.Error( `TestCaseLdLightTemplate - failed [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
    }
  }

  //-----------------------------------------------------------------------------
  //  TestCaseLdLightSp(): test template is set correctly
  //
  TestCaseLdLightSp( testController )
  {

    let testFunctionTimestamp = Date.now();
    let lastlightSp = this.entity['sp']['lastlight'];
    let regexStatepointgPattern = new RegExp( `${Data.Instance().RootEntid()}\\/\\w+\\.\\d+\\/lastdoor`, "g");

    if( lastlightSp )  {
      if( lastlightSp['alias']
        && lastlightSp['alias']['target'] === null
        && lastlightSp['alias']['timeout']  >= 0  ) {
        testController.Success( `TestCaseLdLightTemplate - alias - pass [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
      }
      else {
        testController.Error( `TestCaseLdLightTemplate - alias - failed [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
      }

      if( lastlightSp
        && lastlightSp['hidden'] === true
        && lastlightSp['persistent'] === false
        && lastlightSp['type'] === "BOOL"
        && lastlightSp['name'] === "lastlight"
        && lastlightSp['address'].match( regexStatepointgPattern ) !== null ) {

        testController.Success( `TestCaseLdLightTemplate - basic sp settings - pass [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
      }
      else  {
        testController.Error( `TestCaseLdLightTemplate - basic sp settings - failed [${( Date.now() - testFunctionTimestamp ) / 1000}s]` );
      }

    }
  }

  //-----------------------------------------------------------------------------
  //  ControlAvailableMode(): description
  //
  ControlIsTimeBased( result )
  {
    if( result === true )  {
      this.ModeTimeBased();
    }
    else {
      this.ModeWorksForEver();
    }
  }


  //-----------------------------------------------------------------------------
  //  VALIDATION
  //-----------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  keys-dialog will call this
  //
  OnValidate()
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    let numError = 0;
    numError += $( "#ID_LDLIGHT_TIME" + this.dynid ).edit( "validate" );
    return numError;
  }

  //-----------------------------------------------------------------------------
  //  keys-dialog will call this
  //
  OnValidateOff()
  {
    $( "#ID_LDLIGHT_TIME" + this.dynid ).edit( "validate", false );
  }

  //=============================================================================
  //  MODES
  //=============================================================================
  //-----------------------------------------------------------------------------
  //  ModeTimeBased(): description
  //
  ModeTimeBased()
  {
    this.mode = "modeTimeBase";
    $( "#ID_LDLIGHT_TIME" + this.dynid ).edit( "option", "value", 0 );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_IS_TIME_BASED" );
  }

  //-----------------------------------------------------------------------------
  //  ModeWorksForEver(): description
  //
  ModeWorksForEver()
  {
    this.mode = "modeWorksForEver";
    $( "#ID_LDLIGHT_TIME" + this.dynid ).edit( "option", "value", 0 );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_IS_TIME_BASED" );
  }

  //-----------------------------------------------------------------------------
  //  HelperCreateLdLightSp(): description
  //
  HelperCreateLdLightSp()
  {
    let sp = {
      name: "lastlight",
      address: `${Data.Instance().RootEntid()}/${this.entity['entid']}/lastlight`,
      type: "BOOL",
      template: "lastlight",
      persistent: false,
      hidden: true,
      label: "{$aui_widget_lastlight}",
      alias: {
        target: null,
        timeout: 0
      }
    };

    return sp;
  }
}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigLdLight.Register( "/widget/deviceConfig/ldlight" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigLdLight );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigLed extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules = [ "LED" ];
  static allowedTypes = [ "*" ];
  static widgetIcon = "mdi mdi-led-on";
  static widgetNameShort = "{$widget_led_short}";
  static widgetNameLong = "{$widget_led_long}";
  static widgetId = "led";
  static allowedTriggers = [ "setdnd", "setredir", "setsp", "mute_speaker", "mute_mic", "setdoormatic", "storeycall", "grabpicmenu", "setspknx" ];
  static allowedTriggerMap = {
    "setdnd": "dnd", 
    "setredir": "redir", 
    "setsp": "bool", 
    "mute_speaker": "mute_speaker", 
    "mute_mic": "mute_mic", 
    "setdoormatic": "doormatic",
    "storeycall": "storeycall",
    "grabpicmenu": "grabpicmenu",
    "setspknx": "bool"
  };


  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/led";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    // check if a statepoint is set in target
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    if( ( deviceRecord ) && ( 'param' in deviceRecord ) ) {

      // match regex for statepoint
      let spAddress = deviceRecord['param']['target'];
      if( spAddress.match( /.+\/.+\/.+/ig ) )  {
        return [ "sp:" + spAddress + ":r" ];
      }
    }

    // no external ressources:
    return null;
  }

  //-----------------------------------------------------------------
  //  validate the widget
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    if( ( deviceRecord ) && ( 'param' in deviceRecord ) ) {

      // match regex for statepoint
      let spAddress = deviceRecord['param']['target'];
      if( spAddress.match( /.+\/.+\/.+/ig ) ) {

        // get statepoint
        let spInfo = Data.Instance().StatepointInfo( spAddress, entity );

        // error statepoint does not exists
        if( spInfo['type'] == "ERROR" ) {
          validateControl.error = "{$const_error_statepoint_deleted}";
          return 1;
        }

        // error no permission
        if( !Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddress, "r" ) ) {
          validateControl.error = "{$const_error_statepoint_permission}";
          return 1;
        }

      }
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // defaults
    let targetStatepoint = deviceRecord['param']['target'];
    let text = $.lang( "{$widget_led_desciption_action}" );
    let ledColor = "default";

    // target
    let linkedtrigger = deviceRecord['param']['lastTrigger'];
    if( linkedtrigger == "single" )  {
      text += ": " + $.lang( "{$widget_led_desciption_triggersingle}" );
    }
    else if( linkedtrigger == "double" ) {
      text += ": " + $.lang( "{$widget_led_desciption_triggerdouble}" );
    }
    else if( linkedtrigger == "pos" ) {
      text += ": " + $.lang( "{$widget_led_desciption_triggerpos}" );
    }    
    else if( linkedtrigger == "neg" ) {
      text += ": " + $.lang( "{$widget_led_desciption_triggerneg}" );
    }    
    else if( linkedtrigger == "individually" ) {
      let spinfo = Data.Instance().StatepointInfo( targetStatepoint, entity );
      text += ": \"" + spinfo['label'] + "\"";
    }
    else {
      text = $.lang( "{$widget_led_desciption_default}" );
    }

    // color
    let availableColorCodes = WidgetDeviceConfigLed.HelperAvailableColorCodes( deviceRecord );
    let selectedColor = availableColorCodes[ 0 ]['id'];  // default
    if( ( 'param' in deviceRecord ) && ( 'color' in deviceRecord['param'] ) ) {
      if( deviceRecord['param']['color']['1'] === true ) {
        selectedColor = "1";
      }
      else if( deviceRecord['param']['color']['2'] === true ) {
        selectedColor = "2";
      }
    }
    ledColor = availableColorCodes.find( ( code ) => code['id'] == selectedColor );
    ledColor = ledColor['color'];

    // that's it
    let result = {
      text: text,
      icon: "mdi mdi-circle-medium",
      color: ledColor
    };
    if( linkedtrigger != "individually")  {
      result['linked'] = linkedtrigger;
    }
    return result;
  }

  //-----------------------------------------------------------------
  //  update led to linked trigger single or double
  //
  static UpdateLinked( deviceRecord, entity )
  {
    let isLedReLinked = false;

    // trigger
    let trigger = deviceRecord['param']['lastTrigger'];

    // trigger "single", "double", "pos", "neg": read foreign device record
    if( trigger == "single" || trigger == "double" || trigger == "pos" || trigger == "neg" ) {

      // seems to be an unused parameter, but it exists always 
      deviceRecord['param']['targetEntityId'] = "";
      deviceRecord['param']['target'] = "";

      // get linked deviceRecord and get needed informations from it
      let linkedDeviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, deviceRecord['uid'], trigger );
      if( linkedDeviceRecord ) {
        let linkedSpTarget = null;
        if( 'param' in linkedDeviceRecord ) {
          linkedSpTarget = linkedDeviceRecord['param']['spaddr'] || null;
        }

        // convert widgetId to the in led used "script"
        let triggerScript = WidgetDeviceConfigLed.allowedTriggerMap[ linkedDeviceRecord['template'] ] || null;

        // depending on script in linked deviceRecord set the parameters
        if( ( triggerScript == "bool" || triggerScript == "dnd" || triggerScript == "doormatic" || triggerScript == "storeycall" || triggerScript == "grabpicmenu" )
         && ( linkedSpTarget )
          ) {
          WidgetDeviceConfigLed.HelperSetScript( deviceRecord, triggerScript );
          delete deviceRecord['param']['prefix'];
          deviceRecord['param']['dest'] = -1;
          deviceRecord['param']['target'] = linkedSpTarget;
          deviceRecord['param']['targetEntityId'] = "";
          isLedReLinked = true;
        }
        else if( triggerScript == "redir" && linkedSpTarget ) {
          WidgetDeviceConfigLed.HelperSetScript( deviceRecord, triggerScript );
          deviceRecord['param']['prefix'] = "redir";
          deviceRecord['param']['target'] = linkedSpTarget;
          deviceRecord['param']['targetEntityId'] = "";
          deviceRecord['param']['dest'] = linkedDeviceRecord['param']['dest'];
          isLedReLinked = true;
        }
        else if( triggerScript == "mute_speaker" ) {
          WidgetDeviceConfigLed.HelperSetScript( deviceRecord, triggerScript );
          deviceRecord['param']['target'] = "local\\muteringtone";          
          deviceRecord['param']['targetEntityId'] = "";
          isLedReLinked = true;
        }
        else if( triggerScript == "mute_mic" ) {
          WidgetDeviceConfigLed.HelperSetScript( deviceRecord, triggerScript );
          deviceRecord['param']['target'] = "local\\mutemic";          
          deviceRecord['param']['targetEntityId'] = "";
          isLedReLinked = true;
        }
      }
    }
    // else: Trigger 'indiviually' means: LED is perfect, don't delete. 
    else  {
      isLedReLinked = true;
    }
    
    // result decide if "LED" is deleted
    return isLedReLinked;
  }

  //-----------------------------------------------------------------
  //  HELPER
  //-----------------------------------------------------------------

  // determine the colors used in this led (list for control)
  static HelperAvailableColorCodes( deviceRecord )
  {
    let availableColorCodes = [];

    let index = 0;
    for( let led of deviceRecord['led'] ) {
      if( led['sysname'].indexOf( "spare" ) >= 0 ) {
        index++;
      }
      if( led['sysname'].indexOf( "white" ) >= 0 ) {
        index++;
        availableColorCodes.push( { id: String( index ), text: "{$widget_led_color_white}", color: "blue" } );
      }
      if( led['sysname'].indexOf( "amber" ) >= 0 ) {
        index++;
        availableColorCodes.push( { id: String( index ), text: "{$widget_led_color_yellow}", color: "yellow" } );
      }
      if( led['sysname'].indexOf( "red" ) >= 0 ) {
        index++;
        availableColorCodes.push( { id: String( index ), text: "{$widget_led_color_red}", color: "red" } );
      }
    }
    return availableColorCodes;
  }

  // determine the triggers available and usable for this key
  static HelperAvailableTriggers( entity, uid )
  {
    let availableTriggers = [];
    for( let deviceRecord of entity['deviceConfig'] ) {
      if( deviceRecord['uid'] == uid ) {
        if( ( deviceRecord['trigger'] == "single" )
         && ( WidgetDeviceConfigLed.allowedTriggers.indexOf( deviceRecord['template'] ) >= 0 )
          ) {
          availableTriggers.push( { id: "single", text: "{$widget_led_trigger_single}" } );
        }
        if( ( deviceRecord['trigger'] == "double" )
         && ( WidgetDeviceConfigLed.allowedTriggers.indexOf( deviceRecord['template'] ) >= 0 )
          ) {
          availableTriggers.push( { id: "double", text: "{$widget_led_trigger_double}" } );
        }
        if( ( deviceRecord['trigger'] == "pos" )
         && ( WidgetDeviceConfigLed.allowedTriggers.indexOf( deviceRecord['template'] ) >= 0 )
          ) {
          availableTriggers.push( { id: "pos", text: "{$widget_led_trigger_pos}" } );
        }
        if( ( deviceRecord['trigger'] == "neg" )
         && ( WidgetDeviceConfigLed.allowedTriggers.indexOf( deviceRecord['template'] ) >= 0 )
          ) {
          availableTriggers.push( { id: "neg", text: "{$widget_led_trigger_neg}" } );
        }
      }
    }
    availableTriggers.push( { id: "individually", text: "{$widget_led_trigger_individually}" } );
    return availableTriggers;
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data 
    this.preventFirstTimeValidate = false;
    this.entity = createData['entity'];
    this.uid = createData['options']['uid'];
    this.trigger = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );
   
    console.log( `${ this.constructor.name }( ${ this.uid }, ${ this.trigger } )`, this.deviceRecord );

    // create controls
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );

    // color
    this.availableColorCodes = WidgetDeviceConfigLed.HelperAvailableColorCodes( this.deviceRecord ); 
    $( "#ID_COLOR" + this.dynid ).choice( {
      size: CONST.size.XL,
      data: this.availableColorCodes
    });

    // trigger
    this.availableTriggers = WidgetDeviceConfigLed.HelperAvailableTriggers( this.entity, this.uid );
    $( "#ID_TRIGGER" + this.dynid ).choice( {
      size: CONST.size.XL,
      data: this.availableTriggers,
      result: this.ControlTriggerResult.bind( this )
    } );

    // individual categories 
    this.individualTypes = [
      { id: "redir",      text: WidgetDeviceConfigFactory.WidgetName( "setredir" ),     icon: WidgetDeviceConfigFactory.WidgetIcon( "setredir" ) },
      { id: "doormatic",  text: WidgetDeviceConfigFactory.WidgetName( "setdoormatic" ), icon: WidgetDeviceConfigFactory.WidgetIcon( "setdoormatic" ) },
      { id: "bool",       text: WidgetDeviceConfigFactory.WidgetName( "setsp" ),        icon: WidgetDeviceConfigFactory.WidgetIcon( "setsp" ) },
      { id: "dnd",        text: WidgetDeviceConfigFactory.WidgetName( "setdnd" ),       icon: WidgetDeviceConfigFactory.WidgetIcon( "setdnd" ) },
    ];
    $( "#ID_INDTYPE" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "droplist",
      data: this.individualTypes,
      validate: this.ValidateIndvividualType.bind( this ),
      result: this.ControlIndividualTypeSelect.bind( this )
    } );

    // statepoint control will be create in mode handlers

    // redir target control
    ControlCalltargetList.Build( $( "#ID_INDDESTINATION" + this.dynid ), {
      addressBook: this.entity['addrBook'],
      forbiddenEntityId: [ this.entity['entid'] ]
    } );


    // fill data
    this.FormFillControls();

    //this.UnitTest();

  }

  OnEvent( eventName, data )
  {
    console.log( eventName, data );
  }

  //---------------------------------------------------------------------------
  //  VALIDATION
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  viewable override: validate
  //
  OnValidate()
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    let numErr = 0;
    if( this.mode == "modeTriggerSingle" 
     || this.mode == "modeTriggerDouble" 
     || this.mode == "modeTriggerPos" 
     || this.mode == "modeTriggerNeg"  
      ) {
      return 0;
    }
    else if( this.mode == "modeTriggerIndividual" ) {
      return $( "#ID_INDTYPE" + this.dynid ).listbox( "validate" );
    }
    else if( this.mode == "modeTriggerIndividualRedir" ) {
      numErr += $( "#ID_INDTYPE" + this.dynid ).listbox( "validate" );
      numErr += $( "#ID_INDSTATEPOINT" + this.dynid ).viewable( "validate" );
      numErr += $( "#ID_INDDESTINATION" + this.dynid ).viewable( "validate" );
      return numErr;
    }
    else if( this.mode == "modeTriggerIndividualDnd"
          || this.mode == "modeTriggerIndividualDoormatic" 
          || this.mode == "modeTriggerIndividualBool" 
           ) {
      numErr += $( "#ID_INDTYPE" + this.dynid ).listbox( "validate" );
      numErr += $( "#ID_INDSTATEPOINT" + this.dynid ).viewable( "validate" );
      numErr += $( "#ID_INDDESTINATION" + this.dynid ).viewable( "validate", false );
      return numErr;
    }
  }

  //---------------------------------------------------------------------------
  //  viewable override: validate
  //
  OnValidateOff()
  {
    $( "#ID_INDTYPE" + this.dynid ).listbox( "validate", false );

    // on linked keys individual controls does not exist
    if( this.mode != "modeTriggerSingle" 
     && this.mode != "modeTriggerDouble" 
     && this.mode != "modeTriggerPos" 
     && this.mode != "modeTriggerNeg"  
     && this.mode != "modeTriggerIndividual"
      ) {
        $( "#ID_INDSTATEPOINT" + this.dynid ).viewable( "validate", false );
        $( "#ID_INDDESTINATION" + this.dynid ).viewable( "validate", false );
    }    
  }

  //---------------------------------------------------------------------------
  //  viewable override: validate
  //
  ValidateIndvividualType( validateControl )
  {
    let selected = $( "#ID_INDTYPE" + this.dynid ).listbox( "option", "selected" );
    if( !( Array.isArray( selected ) ) || ( selected.length == 0 ) )  {
      validateControl.error = "{$widget_led_type_error_select}";
      return 1;
    }

    validateControl.error = null;
    return 0;
  }

  //---------------------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {

    // read controls
    this.FormReadControls();

    // unit test
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }

  //---------------------------------------------------------------------------
  //  FORM
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  set controls from currect deviceRecord
  //
  FormFillControls()
  {
    // set color code
    let selectedColor = this.availableColorCodes[ 0 ]['id'];  // default
    if( ( 'param' in this.deviceRecord ) && ( 'color' in this.deviceRecord['param'] ) ) {
      if( this.deviceRecord['param']['color']['1'] === true ) {
        selectedColor = "1";
      }
      else if( this.deviceRecord['param']['color']['2'] === true ) {
        selectedColor = "2";
      }
    }
    $( "#ID_COLOR" + this.dynid ).choice( "option", "value", selectedColor );

    // trigger
    let selectedTrigger = this.availableTriggers[ 0 ]['id']; // default
    if( ( 'param' in this.deviceRecord ) && ( 'lastTrigger' in this.deviceRecord['param'] ) ) {
      selectedTrigger = this.deviceRecord['param']['lastTrigger'];
    }
    $( "#ID_TRIGGER" + this.dynid ).choice( "option", "value", selectedTrigger );

    // set the modes regrading the selected trigger
    if( selectedTrigger == "single" )  {
      this.ModeTriggerSingle();
    }
    else if( selectedTrigger == "double" ) {
      this.ModeTriggerDouble();
    }
    if( selectedTrigger == "pos" )  {
      this.ModeTriggerPos();
    }
    else if( selectedTrigger == "neg" ) {
      this.ModeTriggerNeg();
    }    
    else if( selectedTrigger == "individually" ) {

      // select mode
      let script = WidgetDeviceConfigLed.HelperGetScript( this.deviceRecord );
      if( script == "redir" ) {
        $( "#ID_INDTYPE" + this.dynid ).listbox( "option", "selected", [ script ] );
        this.ModeTriggerIndividualRedir();
      }
      else if( script == "dnd" ) {
        $( "#ID_INDTYPE" + this.dynid ).listbox( "option", "selected", [script] );
        this.ModeTriggerIndividualDnd();
      }
      else if( script == "bool" ) {
        $( "#ID_INDTYPE" + this.dynid ).listbox( "option", "selected", [script] );
        this.ModeTriggerIndividualBool();
      }
      else if( script == "doormatic" ) {
        $( "#ID_INDTYPE" + this.dynid ).listbox( "option", "selected", [script] );
        this.ModeTriggerIndividualDoormatic();
      }
      else {
        this.ModeTriggerIndividual();
        this.preventFirstTimeValidate = true;
      }

      // set statpoint value 
      if( script == "dnd" || script == "bool" || script == "doormatic" ) {

        // target statepoint
        let selectedTarget = [];
        if( this.deviceRecord['param']['target'] ) selectedTarget = [this.deviceRecord['param']['target']];
        $( "#ID_INDSTATEPOINT" + this.dynid ).viewable( "option", "selected", selectedTarget );

      }

      // set redirection values
      else if( script == "redir" ) {

        // target statepoint
        let selectedTarget = [];
        if( this.deviceRecord['param']['target'] ) selectedTarget = [ this.deviceRecord['param']['target'] ];
        $( "#ID_INDSTATEPOINT" + this.dynid ).viewable( "option", "selected", selectedTarget );

        // redir dest entity
        let resultSelectedDest = [];
        let selectedDest = this.deviceRecord['param']['dest'] || "";
        if( selectedDest.length )  {
          resultSelectedDest.push( selectedDest.split( "/" )[ 1 ] );
        }
        $( "#ID_INDDESTINATION" + this.dynid ).viewable( "option", "selected", resultSelectedDest );
      }

    }

  }

  //---------------------------------------------------------------------------
  //  set currect deviceRecord from controls 
  //
  FormReadControls()
  {
    // default sections
    this.deviceRecord['template'] = WidgetDeviceConfigLed.widgetId;
    if( !( 'param' in this.deviceRecord ) )  {
      this.deviceRecord['param'] = {};
    }
    if( !( 'color' in this.deviceRecord['param'] ) ) {
      this.deviceRecord['param']['color'] = {
        "1": false,
        "2": false
      };
    }
    if( !( 'script' in this.deviceRecord['param'] ) ) {
      this.deviceRecord['param']['script'] = {
        "dnd": null,
        "redir": true,
        "bool": null,
        "mute_speaker": null,
        "mute_mic": null,
        "doormatic": null,
        "storeycall": null,
        "grabpicmenu": null,

      };
    }
    if( 'storeycall' in this.deviceRecord['param']['script'] ) {
      this.deviceRecord['param']['script']['storeycall'] = null;
    }

    // color code
    this.deviceRecord['param']['color']['1'] = false;
    this.deviceRecord['param']['color']['2'] = false;
    let selectedColor = $( "#ID_COLOR" + this.dynid ).choice( "option", "value" );
    this.deviceRecord['param']['color'][ selectedColor ] = true;

    // trigger
    let trigger = this.deviceRecord['param']['lastTrigger'] = $( "#ID_TRIGGER" + this.dynid ).choice( "option", "value" );

    // trigger "single", "double", "pos", "neg": read foreign device record
    if( trigger == "single" || trigger == "double" || trigger == "pos" || trigger == "neg" )  {
      WidgetDeviceConfigLed.UpdateLinked( this.deviceRecord, this.entity );
    }

    // trigger "individually", read controls
    else {

      let script = $( "#ID_INDTYPE" + this.dynid ).listbox( "option", "selected" )[ 0 ];
      WidgetDeviceConfigLed.HelperSetScript( this.deviceRecord, script );

      if( script == "bool" || script == "dnd" || script == "doormatic" || script == "storeycall" )  {
        delete this.deviceRecord['param']['prefix'];
        this.deviceRecord['param']['dest'] = -1;
        this.deviceRecord['param']['target'] = $( "#ID_INDSTATEPOINT" + this.dynid ).viewable( "option", "selected" )[ 0 ];
        this.deviceRecord['param']['targetEntityId'] = "";
      }
      else if( script == "redir" )  {
        this.deviceRecord['param']['prefix'] = "redir";
        this.deviceRecord['param']['target'] = $( "#ID_INDSTATEPOINT" + this.dynid ).viewable( "option", "selected" )[ 0 ];
        this.deviceRecord['param']['targetEntityId'] = "";
        let dest = $( "#ID_INDDESTINATION" + this.dynid ).viewable( "option", "selected" )[ 0 ];
        if( dest.indexOf( "/" ) < 0 ) dest = Data.Instance().RootEntid() + "/" + dest;
        this.deviceRecord['param']['dest'] = dest;
      }
      else if( script == "mute_speaker" )  {
        deviceRecord['param']['target'] = "local\\muteringtone";          
        deviceRecord['param']['targetEntityId'] = "";
      }
      else if( script == "mute_mic" )  {
        deviceRecord['param']['target'] = "local\\mutemic";          
        deviceRecord['param']['targetEntityId'] = "";
      }
    }

  }

  
  //---------------------------------------------------------------------------
  //  CONTROL HANDLERS
  //---------------------------------------------------------------------------

  ControlTriggerResult( result )
  {
    switch( result ) {
      case "single":        this.ModeTriggerSingle();     break;
      case "double":        this.ModeTriggerDouble();     break;
      case "pos":           this.ModeTriggerPos();        break;
      case "neg":           this.ModeTriggerNeg();        break;
      case "individually":  this.ModeTriggerIndividual(); break;
    }
  }

  ControlIndividualTypeSelect( selected )
  {
    switch( selected[ 0 ] )  {
      case "redir":     this.ModeTriggerIndividualRedir();      break;
      case "doormatic": this.ModeTriggerIndividualDoormatic();  break;
      case "dnd":       this.ModeTriggerIndividualDnd();        break;
      case "grabpicmenu":
      case "storeycall":
      case "bool":      this.ModeTriggerIndividualBool();       break;
    }
  }

  //---------------------------------------------------------------------------
  //  MODES
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  linked to "key single click"
  //
  ModeTriggerSingle()
  {
    if( this.mode == "modeTriggerSingle" )  return;
    this.mode = "modeTriggerSingle";

    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDTYPE" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDSTATEPOINT" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDTARGET" );
    $( "#ID_INDTYPE" + this.dynid ).listbox( "option", "selected", [] );
  }

  //---------------------------------------------------------------------------
  //  linked to "key double click"
  //
  ModeTriggerDouble()
  {
    if( this.mode == "modeTriggerDouble" ) return;
    this.mode = "modeTriggerDouble";

    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDTYPE" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDSTATEPOINT" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDTARGET" );
    $( "#ID_INDTYPE" + this.dynid ).listbox( "option", "selected", [] );
  }

  //---------------------------------------------------------------------------
  //  linked to "key press"
  //
  ModeTriggerPos()
  {
    if( this.mode == "modeTriggerPos" )  return;
    this.mode = "modeTriggerPos";

    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDTYPE" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDSTATEPOINT" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDTARGET" );
    $( "#ID_INDTYPE" + this.dynid ).listbox( "option", "selected", [] );
  }

  //---------------------------------------------------------------------------
  //  linked to "key double click"
  //
  ModeTriggerNeg()
  {
    if( this.mode == "modeTriggerNeg" ) return;
    this.mode = "modeTriggerNeg";

    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDTYPE" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDSTATEPOINT" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDTARGET" );
    $( "#ID_INDTYPE" + this.dynid ).listbox( "option", "selected", [] );
  }  

  //---------------------------------------------------------------------------
  //  individual trigger, not choosed
  //
  ModeTriggerIndividual()
  {
    if( this.mode == "modeTriggerIndividual" ) return;
    this.mode = "modeTriggerIndividual";
  
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_INDTYPE" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDSTATEPOINT" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDTARGET" );
  }

  //---------------------------------------------------------------------------
  //  individual trigger, redir choosed
  //
  ModeTriggerIndividualRedir()
  {
    if( this.mode == "modeTriggerIndividualRedir" ) return;
    this.mode = "modeTriggerIndividualRedir";

    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_INDTYPE" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_INDSTATEPOINT" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_INDTARGET" );

    ControlStatepointList.Build( $( "#ID_INDSTATEPOINT" + this.dynid ), {
      inquirer: this.entity['entid'],
      entity: this.entity,
      forbiddenEntityId: [ this.entity['entid'] ],
      pinnedEntityId: this.entity['entid'],
      allowedTypes: ["REDIR"],
      permission: "r"
    } );
    $( "#ID_INDTYPE" + this.dynid ).listbox( "validate", false );
    $( "#ID_INDSTATEPOINT" + this.dynid ).viewable( "validate", false );
    $( "#ID_INDDESTINATION" + this.dynid ).viewable( "validate", false );

    $( "#ID_INDDESTINATION" + this.dynid ).viewable( "option", "selected", [] );
  }

  //---------------------------------------------------------------------------
  //  individual trigger, doormatic choosed
  //
  ModeTriggerIndividualDoormatic()
  {
    if( this.mode == "modeTriggerIndividualDoormatic" ) return;
    this.mode = "modeTriggerIndividualDoormatic";

    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_INDTYPE" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_INDSTATEPOINT" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDTARGET" );

    ControlStatepointList.Build( $( "#ID_INDSTATEPOINT" + this.dynid ), {
      inquirer: this.entity['entid'],
      entity: this.entity,
      forbiddenEntityId: [ this.entity['entid'] ],
      pinnedEntityId: this.entity['entid'],
      allowedTypes: ["BOOL"],
      allowedNames: ["doormatic"],
      permission: "r"
    } );

    $( "#ID_INDTYPE" + this.dynid ).listbox( "validate", false );
    $( "#ID_INDSTATEPOINT" + this.dynid ).viewable( "validate", false );
    $( "#ID_INDDESTINATION" + this.dynid ).viewable( "validate", false );

  }

  //---------------------------------------------------------------------------
  //  individual trigger, bool choosed
  //
  ModeTriggerIndividualBool()
  {
    if( this.mode == "modeTriggerIndividualBool" ) return;
    this.mode = "modeTriggerIndividualBool";

    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_INDTYPE" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_INDSTATEPOINT" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDTARGET" );

    ControlStatepointList.Build( $( "#ID_INDSTATEPOINT" + this.dynid ), {
      inquirer: this.entity['entid'],
      entity: this.entity,
      forbiddenEntityId: [ this.entity['entid'] ],
      pinnedEntityId: this.entity['entid'],
      allowedTypes: ["BOOL"],
      permission: "r"
    } );

    $( "#ID_INDTYPE" + this.dynid ).listbox( "validate", false );
    $( "#ID_INDSTATEPOINT" + this.dynid ).viewable( "validate", false );
    $( "#ID_INDDESTINATION" + this.dynid ).viewable( "validate", false );
  }

  //---------------------------------------------------------------------------
  //  individual trigger, dnd choosed
  //
  ModeTriggerIndividualDnd()
  {
    if( this.mode == "modeTriggerIndividualDnd" ) return;
    this.mode = "modeTriggerIndividualDnd";

    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_INDTYPE" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_INDSTATEPOINT" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_INDTARGET" );

    ControlStatepointList.Build( $( "#ID_INDSTATEPOINT" + this.dynid ), {
      inquirer: this.entity['entid'],
      entity: this.entity,
      forbiddenEntityId: [ this.entity['entid'] ],
      pinnedEntityId: this.entity['entid'],
      allowedTypes: ["BOOL"],
      allowedNames: ["dnd"],
      permission: "r"
    } );

    $( "#ID_INDTYPE" + this.dynid ).listbox( "validate", false );
    $( "#ID_INDSTATEPOINT" + this.dynid ).viewable( "validate", false );
    $( "#ID_INDDESTINATION" + this.dynid ).viewable( "validate", false );
  }


  //---------------------------------------------------------------------------
  //  OVERRIDABLES
  //---------------------------------------------------------------------------

  

  //---------------------------------------------------------------------------
  //  HELPER
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  search in sect "script" for the first option that is "true"
  //
  static HelperGetScript( deviceRecord )
  {
    if( !( 'param' in deviceRecord ) || !( 'script' in deviceRecord['param'] ) ) {
      return null;
    }

    for( let key in deviceRecord['param']['script'] )  {
      if( deviceRecord['param']['script'][ key ] === true ) {
        return key;
      }
    }
    return null;
  }

  //---------------------------------------------------------------------------
  //  reset "script" in deviceRecord and set an option
  //
  static HelperSetScript( deviceRecord, activeScript )
  {
    for( let key in deviceRecord['param']['script'] ) {
      deviceRecord['param']['script'][ key ] = false;
    }

    if( activeScript in deviceRecord['param']['script'] )  {
      deviceRecord['param']['script'][ activeScript ] = true;
    }
  }

  //---------------------------------------------------------------------------
  //  UNITTEST
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  event from viewable, start unit test
  //
  OnUnitTest( testController )
  {
    // parepare
    let testTimestamp = Date.now();
    console.group( "Widget Led" );

    // test cases
    this.TestCaseTemplate( testController, this.deviceRecord );
    this.TestCaseParamTarget( testController, this.deviceRecord );
    this.TestCaseParamScript( testController, this.deviceRecord );
    this.TestCaseParamColor( testController, this.deviceRecord );
    this.TestCaseLastTrigger( testController, this.deviceRecord );
    this.TestCaseRedir( testController, this.deviceRecord );

    // finish
    console.log( `took: ${ ( Date.now() - testTimestamp ) / 1000 }s` );
    console.groupEnd();
  }

  //---------------------------------------------------------------------------
  //  check for correct template
  //
  TestCaseTemplate( testController, deviceRecord )
  {
    let testTimestamp = Date.now();
    this.UnitTestCheckObject( "TestCaseTemplate", testController, "template", deviceRecord, "led", testTimestamp );
  }

  //---------------------------------------------------------------------------
  //  check that target is a statepoint
  //
  TestCaseParamTarget( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();
    let statePointeAddressPattern = /\b.+\/[a-zA-Z]+\.\d+\/[-_a-zA-Z0-9]+\b/g;
    this.UnitTestCheckObject( "TestCaseTemplate", testController, "param/target", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }

  //---------------------------------------------------------------------------
  //  check integrity of section script
  //
  TestCaseParamScript( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();

    // check existance and desired values
    this.UnitTestCheckObject( "TestCaseParamScript", testController, "param/script/dnd", deviceRecord, [true, null, false], testFunctionTimestamp );
    this.UnitTestCheckObject( "TestCaseParamScript", testController, "param/script/redir", deviceRecord, [true, null, false], testFunctionTimestamp );
    this.UnitTestCheckObject( "TestCaseParamScript", testController, "param/script/bool", deviceRecord, [true, null, false], testFunctionTimestamp );
    this.UnitTestCheckObject( "TestCaseParamScript", testController, "param/script/mute_speaker", deviceRecord, [true, null, false], testFunctionTimestamp );
    this.UnitTestCheckObject( "TestCaseParamScript", testController, "param/script/mute_mic", deviceRecord, [true, null, false], testFunctionTimestamp );
    this.UnitTestCheckObject( "TestCaseParamScript", testController, "param/script/doormatic", deviceRecord, [true, null, false], testFunctionTimestamp );

    // ensure that one of script is true
    if( 'param' in deviceRecord && 'script' in deviceRecord['param'] )  {
      let isOneSCriptActivated = false;
      for( let key in deviceRecord['param']['script'] )  {
        if( deviceRecord['param']['script'][ key ] === true ) {
          isOneSCriptActivated = true;
          break;
        }
      }
      if( !isOneSCriptActivated ) {
        testController.Error( `TestCaseParamScript - failed: One script need to active [${ ( Date.now() - testFunctionTimestamp ) / 1000 }s]` );
      }
    }

    // check if statepoint name and script fits
    let script = WidgetDeviceConfigLed.HelperGetScript( deviceRecord );
    let spName = deviceRecord['param']['target'].split( "/" )[ 2 ];
    testController.error = 0;
    switch( script )  {
      case 'dnd': if( spName != "dnd" ) testController.Error( `TestCaseParamScript - failed, SP not 'dnd' \"${ deviceRecord['param']['target'] }\" [${ ( Date.now() - timestamp ) / 1000 }s]` ); break;
      case 'redir': if( spName != "redir" ) testController.Error( `TestCaseParamScript - failed, SP not 'redir' \"${ deviceRecord['param']['target'] }\" [${ ( Date.now() - timestamp ) / 1000 }s]` ); break;
      case 'doormatic': if( spName != "doormatic" ) testController.Error( `TestCaseParamScript - failed, SP not 'doormatic' \"${ deviceRecord['param']['target'] }\" [${ ( Date.now() - timestamp ) / 1000 }s]` ); break;
      case 'bool': {
        let spInfo = Data.Instance().StatepointInfo( deviceRecord['param']['target'], this.entity );
        if( spInfo['type'] != "BOOL" )  {
          testController.Error( `TestCaseParamScript - failed, SP not 'bool' \"${ deviceRecord['param']['target'] }\" [${ ( Date.now() - testFunctionTimestamp ) / 1000 }s]` ); break;
        }
      }
      break;
      case 'storeycall':
      case 'mute_speaker':
      case 'mic':
        break;

      default: {
        testController.Error( `TestCaseParamScript - failed, unknown script \"${ script }\" [${ ( Date.now() - testFunctionTimestamp ) / 1000 }s]` );
      }
      break;
    }
    if( testController.error == 0 ) {
      testController.Success( `TestCaseParamScript - ok, Statepoint is \"dnd/redir/bool/doormatic\" [${ ( Date.now() - testFunctionTimestamp ) / 1000 }s]` );
    }


  }//test script

  //---------------------------------------------------------------------------
  //  check for color
  //
  TestCaseParamColor( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();

    this.UnitTestCheckObject( "TestCaseParamColor", testController, "param/color/1", deviceRecord, [true, null, false], testFunctionTimestamp );
    this.UnitTestCheckObject( "TestCaseParamColor", testController, "param/color/2", deviceRecord, [true, null, false], testFunctionTimestamp );

    // ensure that one of color is true
    if( 'param' in deviceRecord && 'color' in deviceRecord['param'] ) {
      let isOneColorActivated = false;
      for( let key in deviceRecord['param']['color'] ) {
        if( deviceRecord['param']['color'][ key ] === true ) {
          isOneColorActivated = true;
          break;
        }
      }
      if( !isOneColorActivated ) {
        testController.Error( `TestCaseParamScript - failed: One color need to active [${ ( Date.now() - testFunctionTimestamp ) / 1000 }s]` );
        isSuccess = false;
      }
    }
  
  }// test color

  //---------------------------------------------------------------------------
  //  check for lastTrigger
  //
  TestCaseLastTrigger( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();
    this.UnitTestCheckObject( "TestCaseLastTrigger", testController, "param/lastTrigger", deviceRecord, ["individually", "single", "double", "pos", "neg"], testFunctionTimestamp );
  }// test trigger

  //---------------------------------------------------------------------------
  //  check for lastTrigger
  //
  TestCaseRedir( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();
    let targetPattern = /\b.+\/[a-zA-Z]+\.\d+\b/g;

    if( deviceRecord['param']['script']['redir'] == true ) {
      this.UnitTestCheckObject( "TestCaseRedir", testController, "param/dest", deviceRecord, targetPattern, testFunctionTimestamp );
      this.UnitTestCheckObject( "TestCaseRedir", testController, "param/prefix", deviceRecord, "redir", testFunctionTimestamp );
    }

  }// test trigger


}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigLed.Register( "/widget/deviceConfig/led" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigLed );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          23.07.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigSwitchContact extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules      = [ "ATLC", "ATLCE1", "ATLCE2", "ATLCE3", "IO", "AZIO" ];
  static allowedTypes        = ["out"];
  static widgetIcon          = "mdi mdi-power-socket";
  static widgetNameLong      = "{$widget_switchcontact_long}";
  static widgetNameShort     = "{$aui_widget_switchcontact}";
  static widgetId            = "switchcontact";


  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  local statepoint for permission check
  //
  static LocalStatepointName( uid, trigger, entity )
  {
    //console.log( "P-Check, switchcontact" );
    // get device record to perform permission check
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    if( deviceRecord )  {
      return deviceRecord['sp'];
    }
    return null;
  }

  //-----------------------------------------------------------------
  //  remove everything from entity
  //
  static Remove( uid, trigger, entity )
  {
    // get device record to get the statepoint name
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( deviceRecord['sp'], entity );
    WidgetDeviceConfigBase.HelperRemovePermission( deviceRecord['sp'], entity );

    // finally remote device record
    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );

    return true;
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/switchcontact";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    // no external ressources
    return null;
  }

  //-----------------------------------------------------------------
  //  build description line, scheme:
  //  <action> (parameter): Target-SP
  //  
  static DescriptionLine( deviceRecord, entity )
  {
    // defaults
    let targetStatepoint = "";

    // get target sp
    targetStatepoint = `${ Data.Instance().RootEntid() }/${ entity['entid'] }/${ deviceRecord['uid'] }`;

    // target name: always use StatepointInfo for name handling
    let target = "";
    let spinfo = Data.Instance().StatepointInfo( targetStatepoint, entity );
    target = spinfo['label'] || "";

    // build everything
    let text = $.lang( `{$widget_switchcontact_action}: "${ target }"` );

    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };

  }


  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    let reference = `sp:${Data.Instance().RootEntid()}/${entity['entid']}/${uid}`;

    return [ reference ];
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let spName       = deviceRecord['uid'];
    let spAddress    = entity['sp'][ spName ]['address'];
    let spInfo       = Data.Instance().StatepointInfo( spAddress, entity );

    // error statepoint does not exists
    if( spInfo['type'] == "ERROR" )  {
      validateControl.error = "{$const_error_statepoint_deleted}";
      return 1;
    }

    // error no permission
    if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddress, "w" ) ) {
      validateControl.error = "{$const_error_statepoint_permission}";
      return 1;
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // $.clone not need, "createData" is allready cloned!
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );

    // first statepoint in this entity, or no statepoint yet?
    if( !( 'sp' in this.entity ) ) {
      this.entity['sp'] = {};
      this.entity['sp'][ this.uid ] = this.HelperCreateSwitchContactSp();
    }
    else if( !( this.uid in this.entity['sp'] ) ) {
      this.entity['sp'][ this.uid ] = this.HelperCreateSwitchContactSp();
    }

    // build label
    ControlLabel.Build( $( "#ID_LABEL" + this.dynid ), {
      label: this.entity['sp'][ this.uid ]['label'],
      entityName: this.entity['name'],
      functionName: WidgetDeviceConfigSwitchContact.widgetNameShort
    } );


    $( "#ID_SWITCH_CONTACT_INVERT" + this.dynid ).choice( {
      size: CONST.size.L,
      type: "check",
      selection: "single"
    } );

    // console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    // fill data
    this.FormFillControls();
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // ---- widget parameters ----
    // load sp
    let spName = this.deviceRecord['uid'];
    //let sp = "";
    let sp     = this.entity['sp'][ spName ];
    let spaddr = "";
    let label  = "";

    // this is not longer needed, since it is created by default in Load
    /*if( !this.entity['sp'][ spName ] )  {
      sp = this.HelperCreateSwitchContactSp();
    }
    else  {
      sp = this.entity['sp'][ spName ];
    }*/

    spaddr = sp['address'];
    spName = sp['name'];
    label  = sp['label'];

    // standard: save param into device record
    this.deviceRecord['template'] = "switchcontact";
    this.deviceRecord['sp']       = spName;
    this.entity['sp'][ spName ]   = sp;

    this.deviceRecord['param']                   = {};
    this.deviceRecord['param']['posttrigger']    = true;
    this.entity['sp'][ spName ]['label']         = $( "#ID_LABEL" + this.dynid ).viewable( "option", "value" );
    this.deviceRecord['param']['invert']         = $( "#ID_SWITCH_CONTACT_INVERT" + this.dynid ).choice( "option", "value" );

    // ---- widget statepoint ----

    // write safely into entity
    if( !( "sp" in this.entity ) )  {
      this.entity['sp'] = {};
    }
    this.entity['sp'][ spName ] = sp;

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }
  //-----------------------------------------------------------------------------
  //  FormFillControls(): fill the controls with already set widget values
  //
  FormFillControls()
  {
    let invert      = false;
    let posttrigger = true;
    let spName      = this.deviceRecord['uid'];

    // previously saved: read from device record
    if( this.deviceRecord['template'] == "switchcontact" ) {
      invert      = this.deviceRecord['param']['invert'];
      posttrigger = this.deviceRecord['param']['posttrigger'];
    }
    else {
      this.preventFirstTimeValidate = true;
    }

    $( "#ID_SWITCH_CONTACT_INVERT" + this.dynid ).edit( "option", "value", invert );

  }

  //===========================================================================
  //  VALIDATION (VIEWABLE OVERRIDE)
  //===========================================================================

  //---------------------------------------------------------------------------
  //  keys-dialog will call this at load & save
  //
  OnValidate( )
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    let numError = 0;
    return numError;
  }

  //---------------------------------------------------------------------------
  //  keys-dialog calls it when widget is removed
  //
  OnValidateOff( )
  {
    $( '#ID_SET_DND_VALUE' + this.dynid ).listbox( "validate", false );
  }

  //=============================================================================
  //  Unit Test
  //=============================================================================

  //---------------------------------------------------------------------------
  //  event from viewable, start unit test
  //
  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Switch Contact" );
    // this.TestCaseMuteMicSpAddr( testControl, this.deviceRecord );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );
    console.groupEnd();
  }


  // //-----------------------------------------------------------------------------
  // //  TestCaseMuteMicTargetEntityId(): description
  // //
  // TestCaseMuteMicSpAddr( testController, deviceRecord )
  // {
  //   let testFunctionTimestamp     = Date.now();
  //   let statePointeAddressPattern = /local\\mute_mic/g;
  //
  //   this.UnitTestCheckObject( "TestCaseMuteMicTargetEntityId", testController, "param/spaddr", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  // }

  //=============================================================================
  //  HELPER
  //=============================================================================

  HelperCreateSwitchContactSp()
  {
    let sp = {
      "name": this.deviceRecord['uid'],
      "type": "BOOL",
      "address": `${ Data.Instance().RootEntid() }/${ this.entity['entid'] }/${ this.deviceRecord['uid'] }`,
      "label": `{{name}} (${ WidgetDeviceConfigSwitchContact.widgetNameShort })`,
      "template": "switchcontact",
      "default": false,
      "resdef": [
        {
          "id": this.deviceRecord['out']['sysid'],
          "action": "switch",
          "periods": 1
        }
      ]
    };
    return sp;
  }
}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigSwitchContact.Register( "/widget/deviceConfig/switchcontact" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigSwitchContact );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          23.07.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigTimeSwitchContact extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules      = [ "ATLC", "ATLCE1", "ATLCE2", "ATLCE3", "IO", "AZIO" ];
  static allowedTypes        = ["out"];
  static widgetIcon          = "mdi mdi-power-socket";
  static widgetNameLong      = "{$widget_timeswitchcontact_long}";
  static widgetNameShort     = "{$aui_widget_switchcontact}";
  static widgetId            = "timeswitchcontact";


  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  local statepoint for permission check
  //
  static LocalStatepointName( uid, trigger, entity )
  {
    //console.log( "P-Check, switchcontact" );
    // get device record to perform permission check
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    if( deviceRecord ) {
      return deviceRecord['sp'];
    }
    return null;
  }

  //-----------------------------------------------------------------
  //  remove everything from entity
  //
  static Remove( uid, trigger, entity )
  {
    // get device record to get the statepoint name
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( deviceRecord['sp'], entity );
    WidgetDeviceConfigBase.HelperRemovePermission( deviceRecord['sp'], entity );

    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );

    return true;
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/timeswitchcontact";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    let reference = `sp:${Data.Instance().RootEntid()}/${entity['entid']}/${uid}`;

    return [ reference ];
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let spName       = deviceRecord['uid'];
    let spAddress    = entity['sp'][ spName ]['address'];
    let spInfo       = Data.Instance().StatepointInfo( spAddress, entity );

    // error statepoint does not exists
    if( spInfo['type'] == "ERROR" )  {
      validateControl.error = "{$const_error_statepoint_deleted}";
      return 1;
    }

    // error no permission
    if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddress, "w" ) ) {
      validateControl.error = "{$const_error_statepoint_permission}";
      return 1;
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  build description line
  //  <action> (parameter): Target-SP
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // defaults
    let targetStatepoint = "";
    let text = "-";
    // get target sp
    targetStatepoint = `${ Data.Instance().RootEntid() }/${ entity['entid'] }/${ deviceRecord['uid'] }`;

    // target name: always use StatepointInfo for name handling
    let target = "";
    let spinfo = Data.Instance().StatepointInfo( targetStatepoint, entity );
    target = spinfo['label'] || "";

    // parameter, time
    let time = CONST.switchTime.find( ( element ) => ( element.id === String( deviceRecord['param']['time'] ) ) );

    // build everything
    if( time ) {
      text = $.lang( `{$widget_timeswitchcontact_action} (${ time.text }): "${ target }"` );
    }
    else  {
      text = $.lang( `{$widget_timeswitchcontact_action} ( - ): "${ target }"` );
    }

    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }


  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // $.clone not need, "createData" is cloned!
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );

    // first statepoint in this entity, or no statepoint yet?
    if( !( 'sp' in this.entity ) ) {
      this.entity['sp'] = {};
      this.entity['sp'][ this.uid ] = this.HelperCreateSwitchContactSp();
    }
    else if( !( this.uid in this.entity['sp'] ) ) {
      this.entity['sp'][ this.uid ] = this.HelperCreateSwitchContactSp();
    }

    // build label
    ControlLabel.Build( $( "#ID_LABEL" + this.dynid ), {
      label: this.entity['sp'][ this.uid ]['label'],
      entityName: this.entity['name'],
      functionName: WidgetDeviceConfigSwitchContact.widgetNameShort
    } );

    $( "#ID_TIME_SWITCH_CONTACT_TIME" + this.dynid ).listbox( {
      size: CONST.size.L,
      height: 7,
      type: "droplist",
      search: true,
      searchPlaceholder: "{$const_common_search_placeholder}",
      placeholder: "{$const_common_empty_placeholder}",

      selection: "single",
      data: CONST.switchTime,
    } );

    $( "#ID_TIME_SWITCH_CONTACT_INVERT" + this.dynid ).choice( {
      size: CONST.size.L,
      type: "check"
    } );

    $( "#ID_TIME_SWITCH_CONTACT_POSTTRIGGER" + this.dynid ).choice( {
      size: CONST.size.L,
      type: "check"
    } );

    // console.log( `${ this.constructor.name }( ${ this.uid }, ${ this.trigger } )`, this.deviceRecord );

    // fill data
    this.FormFillControls();
  }


  //-----------------------------------------------------------------------------
  //  FormFillControls(): fill the controls with already set widget values
  //
  FormFillControls()
  {
    let invert      = false;
    let posttrigger = true;
    let time        = ['3000'];

    // previously saved: read from device record
    if( this.deviceRecord['template'] == "timeswitchcontact" ) {
      invert      = this.deviceRecord['param']['invert'];
      posttrigger = this.deviceRecord['param']['posttrigger'];
      time        = [ this.deviceRecord['param']['time'] ];
    }
    else  {
      this.preventFirstTimeValidate = true;
    }

    //$( "#ID_TIME_SWITCH_CONTACT_LABEL" + this.dynid ).edit( "option", "value", label );
    $( "#ID_TIME_SWITCH_CONTACT_INVERT" + this.dynid ).edit( "option", "value", invert );
    $( "#ID_TIME_SWITCH_CONTACT_POSTTRIGGER" + this.dynid ).edit( "option", "value", posttrigger );
    $( "#ID_TIME_SWITCH_CONTACT_TIME" + this.dynid ).listbox( "option", "selected", time );
  }


  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // ---- widget parameters ----
    // load sp
    let spName = this.deviceRecord['uid'];

    // standard: save param into device record
    this.deviceRecord['template'] = "timeswitchcontact";
    this.deviceRecord['sp'] = spName;

    this.deviceRecord['param'] = {};
    this.deviceRecord['param']['invert'] = $( "#ID_TIME_SWITCH_CONTACT_INVERT" + this.dynid ).choice( "option", "value" );
    this.deviceRecord['param']['posttrigger'] = $( "#ID_TIME_SWITCH_CONTACT_POSTTRIGGER" + this.dynid ).choice( "option", "value" );
    this.deviceRecord['param']['time'] = $( "#ID_TIME_SWITCH_CONTACT_TIME" + this.dynid ).listbox( "option", "selected" )[0];

    this.entity['sp'][ spName ]['label'] = $( "#ID_LABEL" + this.dynid ).viewable( "option", "value" );
    this.entity['sp'][ spName ]['retrigger'] = this.deviceRecord['param']['posttrigger'];
    this.entity['sp'][ spName ]['signaltime'] = parseInt( this.deviceRecord['param']['time'] );
    this.entity['sp'][ spName ]['invert'] = this.deviceRecord['param']['invert']

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }


  //===========================================================================
  //  VALIDATION (VIEWABLE OVERRIDE)
  //===========================================================================

  //---------------------------------------------------------------------------
  //  keys-dialog will call this at load & save
  //
  OnValidate( event )
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    let numError = 0;
    return numError;
  }

  //---------------------------------------------------------------------------
  //  keys-dialog calls it when widget is removed
  //
  OnValidateOff( )
  {
    $("#ID_TIME_SWITCH_CONTACT_TIME" + this.dynid ).listbox( "validate", false );
  }


  //=============================================================================
  //  unit test
  //=============================================================================

  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Switch Contact" );

    // this.TestCaseMuteMicSpAddr( testControl, this.deviceRecord );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //=============================================================================
  //  Test Cases
  //=============================================================================

  // //-----------------------------------------------------------------------------
  // //  TestCaseMuteMicTargetEntityId(): description
  // //
  // TestCaseMuteMicSpAddr( testController, deviceRecord )
  // {
  //   let testFunctionTimestamp     = Date.now();
  //   let statePointeAddressPattern = /local\\mute_mic/g;
  //
  //   this.UnitTestCheckObject( "TestCaseMuteMicTargetEntityId", testController, "param/spaddr", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  // }


  //=============================================================================
  //  helper
  //=============================================================================

  HelperCreateSwitchContactSp()
  {
    let sp = {
      "name": this.deviceRecord['uid'],
      "type": "BOOL",
      "address": `${ Data.Instance().RootEntid() }/${ this.entity['entid'] }/${ this.deviceRecord['uid'] }`,
      "label": `{{name}} (${ WidgetDeviceConfigTimeSwitchContact.widgetNameShort })`,
      "template": "timeswitchcontact",
      "default": false,
      "resdef": [
        {
          "id": this.deviceRecord['out']['sysid'],
          "action": "switch",
          "periods": 1
        }
      ]
    };
    return sp;
  }
}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigTimeSwitchContact.Register( "/widget/deviceConfig/timeswitchcontact" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigTimeSwitchContact );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          23.07.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigLightcontact extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules      = [ "ATLC", "ATLCE1", "ATLCE2", "ATLCE3" ];
  static allowedTypes        = ["out"];
  static widgetIcon          = "mdi mdi-lightbulb-outline";
  static widgetNameShort     = "{$widget_lightcontact_short}";
  static widgetNameLong      = "{$widget_lightcontact_long}";
  static widgetId            = "lightcontact";


  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  // this contact is used by function
  static IsReadOnly( )
  {
    return true;
  }

  //-----------------------------------------------------------------
  //  remove everything from entity
  //
  static Remove( uid, trigger, entity )
  {
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, "light" );

    if( deviceRecord ) {
      deviceRecord['sp'] = null;
    }

    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( "lightcontact", entity );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( "light", entity );
    WidgetDeviceConfigBase.HelperRemovePermission( "lightcontact", entity );
    return true;
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/lightcontact";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    // no external ressources
    let reference = `sp:${Data.Instance().RootEntid()}/${entity['entid']}/light`;
    return [ reference ];
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let spName       = "light";
    let spAddress    = entity['sp'][ spName ]['address'];
    let spInfo       = Data.Instance().StatepointInfo( spAddress, entity );

    // error statepoint does not exists
    if( spInfo['type'] == "ERROR" )  {
      validateControl.error = "{$const_error_statepoint_deleted}";
      return 1;
    }

    // error no permission
    if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddress, "w" ) ) {
      validateControl.error = "{$const_error_statepoint_permission}";
      return 1;
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    let time = deviceRecord['param']['time'] / 1000;
    let suffix = $.lang("{$widget_lightcontact_abbr_second}");

    while ( time >= 60 )  {
      time = time / 60;
      if( suffix === $.lang("{$widget_lightcontact_abbr_second}") )  {
        suffix = $.lang("{$widget_lightcontact_abbr_minute}");
      } else {
        suffix = $.lang("{$widget_lightcontact_abbr_hour}");
      }
    }

    if( time == 0 )  {
      time = $.lang( "{$widget_lightcontact_duration_manual}" );
      suffix = "";
    }    

    let text = $.lang( "{$aui_widget_lightcontact}" + " (" + time + suffix + ")" );

    if( deviceRecord['param']['invert'] )  {
      text = text + $.lang("{$widget_lightcontact_invert}");
    }

    if( deviceRecord['param']['posttrigger'] )  {
      text = text + $.lang("{$widget_lightcontact_posttrigger}");
    }

    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };

  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-------------------------------------------------------------- ---
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );

    // previously saved: read from device record
    if( this.deviceRecord['template'] !== "lightcontact" ) {
      this.preventFirstTimeValidate = true;
    }

    switch( this.deviceRecord['sp'] )  {
      case "light":
        $( "#ID_LIGHTCONTACT_HINT" + this.dynid ).text( $.lang( "{$widget_lightcontact_used_from_light}" ) );
        break;
      default:
        $( "#ID_LIGHTCONTACT_HINT" + this.dynid ).text( $.lang( "{$widget_lightcontact_used_from_no_one}" ) );
        break;
    }



    // console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );


  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // this widget is just a hint !!!
    return this.entity;
  }

  //===========================================================================
  //  VALIDATION (VIEWABLE OVERRIDE)
  //===========================================================================

  //---------------------------------------------------------------------------
  //  keys-dialog will call this at load & save
  //
  OnValidate( )
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    let numError = 0;

    return numError;
  }

  //---------------------------------------------------------------------------
  //  keys-dialog calls it when widget is removed
  //
  OnValidateOff( )
  {

  }

  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  //---------------------------------------------------------------------------
  //  event from viewable, start unit test
  //
  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Light Contact" );

    // this.TestCaseMuteMicSpAddr( testControl, this.deviceRecord );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //=============================================================================
  //  Test Cases
  //=============================================================================

  // //-----------------------------------------------------------------------------
  // //  TestCaseMuteMicTargetEntityId(): description
  // //
  // TestCaseMuteMicSpAddr( testController, deviceRecord )
  // {
  //   let testFunctionTimestamp     = Date.now();
  //   let statePointeAddressPattern = /local\\mute_mic/g;
  //
  //   this.UnitTestCheckObject( "TestCaseMuteMicTargetEntityId", testController, "param/spaddr", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  // }
}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigLightcontact.Register( "/widget/deviceConfig/lightcontact" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigLightcontact );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigOpenercontact extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules      = [ "ATLC", "ATLCE1", "ATLCE2", "ATLCE3" ];
  static allowedTypes        = ["out"];
  static widgetIcon          = "mdi mdi-key-plus";
  static widgetNameShort     = "{$widget_openercontact_short}";
  static widgetNameLong      = "{$widget_openercontact_long}";
  static widgetId            = "openercontact";


  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  // this contact is used by function
  static IsReadOnly( )
  {
    return true;
  }

  //-----------------------------------------------------------------
  //  remove everything from entity
  //
  static Remove( uid, trigger, entity )
  {

    let deviceRecord   = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid );
    let functionRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, deviceRecord['sp'] );

    if( deviceRecord ) {
      WidgetDeviceConfigBase.HelperRemoveStatepoint( deviceRecord['sp'], entity );
      deviceRecord['sp'] = null;
    }

    if( functionRecord )  {
      functionRecord['sp'] = null;
    }

    if( entity['sp'][ deviceRecord['sp'] ] )  {
      delete entity['sp'][ deviceRecord['sp'] ];
    }

    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
    return true;
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/openercontact";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity,uid, trigger );
    let spRecord     = entity['sp'][ deviceRecord['sp'] ];
    let reference    = [];
    let output       = spRecord['param']['output'];

    if( output.indexOf( "/" ) === -1 )  {
      reference.push(`sp:${Data.Instance().RootEntid()}/${entity['entid']}/${output}`);
    }
    else {
      reference.push(`sp:${output}`);
    }
    reference.push(`sp:${spRecord['address']}`);
    return reference;
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let spName       = "door";
    let spAddress    = `${Data.Instance().RootEntid}/${entity['entid']}/${deviceRecord['sp']}`;
    let spInfo       = Data.Instance().StatepointInfo( spAddress, entity );


    // error statepoint does not exists
    if( spInfo['type'] == "ERROR" )  {
      validateControl.error = "{$const_error_statepoint_deleted}";
      return 1;
    }

    // error no permission
    if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddress, "w" ) ) {
      validateControl.error = "{$const_error_statepoint_permission}";
      return 1;
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    let time = deviceRecord['param']['time'] / 1000;
    let suffix = "s";

    while ( time >= 60 )  {
      time = time / 60;
      if( suffix === $.lang("{$widget_openercontact_abbr_second}") )  {
        suffix = $.lang("{$widget_openercontact_abbr_minute}");
      } else {
        suffix = $.lang("{$widget_openercontact_abbr_hour}");
      }
    }

    if( time == 0 )  {
      time = $.lang( "{$widget_openercontact_duration_manual}" );
      suffix = "";
    }

    let text = $.lang( "{$widget_openercontact_short}" ) + " (" + time + suffix + ")";

    if( 'sp' in deviceRecord && deviceRecord['sp'] == "door2" ) {
      text = $.lang( "{$widget_openercontact_short2}" ) + " (" + time + suffix + ")" ;
    }

    if( deviceRecord['param']['invert'] )  {
      text = text + $.lang("{$widget_openercontact_invert}");
    }

    if( deviceRecord['param']['posttrigger'] )  {
      text = text + $.lang("{$widget_openercontact_posttrigger}");
    }


    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };

  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );

    // previously saved: read from device record
    if( this.deviceRecord['template'] !== "setdnd" ) {
      this.preventFirstTimeValidate = true;
    }


    switch( this.deviceRecord['sp'] )  {
      case "door":
        $( "#ID_OPENERCONTACT_HINT" + this.dynid ).text( $.lang( "{$widget_openercontact_used_from_door}" ) );
        break;
      case "door2":
        $( "#ID_OPENERCONTACT_HINT" + this.dynid ).text( $.lang( "{$widget_openercontact_used_from_door2}" ) );
        break;
      default:
        $( "#ID_OPENERCONTACT_HINT" + this.dynid ).text( $.lang( "{$widget_openercontact_used_from_no_one}" ) );
        break;
    }

    // console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );


  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // this widget is just a hint !!!
    return this.entity;
  }

  //===========================================================================
  //  VALIDATION (VIEWABLE OVERRIDE)
  //===========================================================================

  //---------------------------------------------------------------------------
  //  keys-dialog will call this at load & save
  //
  OnValidate( )
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    let numError = 0;
    return numError;
  }

  //---------------------------------------------------------------------------
  //  keys-dialog calls it when widget is removed
  //
  OnValidateOff( )
  {
  }

  //=============================================================================
  //  Unit Test
  //=============================================================================
  //---------------------------------------------------------------------------
  //  event from viewable, start unit test
  //
  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Opener Contact" );

    // this.TestCaseMuteMicSpAddr( testControl, this.deviceRecord );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //=============================================================================
  //  Test Cases
  //=============================================================================

  // //-----------------------------------------------------------------------------
  // //  TestCaseMuteMicTargetEntityId(): description
  // //
  // TestCaseMuteMicSpAddr( testController, deviceRecord )
  // {
  //   let testFunctionTimestamp     = Date.now();
  //   let statePointeAddressPattern = /local\\mute_mic/g;
  //
  //   this.UnitTestCheckObject( "TestCaseMuteMicTargetEntityId", testController, "param/spaddr", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  // }
}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigOpenercontact.Register( "/widget/deviceConfig/openercontact" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigOpenercontact );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          23.07.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceStateNotifyingContact extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules      = [ "ATLC", "ATLCE1", "ATLCE2", "ATLCE3", "AZIO", "IO" ];
  static allowedTypes        = ["in"];
  static widgetIcon          = "mdi mdi-eye";
  static widgetNameShort     = "{$aui_widget_state_notifying_contact}";
  static widgetNameLong      = "{$widget_state_notifying_contact_short}";
  static widgetId            = "state_notifying_contact";


  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  local statepoint for permission check
  //
  static LocalStatepointName( uid, trigger, entity )
  {
    // get device record to perform permission check
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    if( deviceRecord ) {
      return deviceRecord['sp'];
    }
    return null;
  }

  //-----------------------------------------------------------------
  //  remove everything from entity
  //
  static Remove( uid, trigger, entity )
  {
    // get device record to get the statepoint name
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( deviceRecord['sp'], entity );
    WidgetDeviceConfigBase.HelperRemovePermission( deviceRecord['sp'], entity );

    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
    return true;
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/state_notifying_contact";
    $element.viewable( createData );
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let spName       = deviceRecord['uid'];
    let spAddress    = entity['sp'][ spName ]['address'];
    let spInfo       = Data.Instance().StatepointInfo( spAddress, entity );

    // error statepoint does not exists
    if( spInfo['type'] == "ERROR" )  {
      validateControl.error = "{$const_error_statepoint_deleted}";
      return 1;
    }

    // error no permission
    if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddress, "w" ) ) {
      validateControl.error = "{$const_error_statepoint_permission}";
      return 1;
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // defaults
    let targetStatepoint = "";

    // get target sp
    targetStatepoint = `${ Data.Instance().RootEntid() }/${ entity['entid'] }/${ deviceRecord['uid'] }`;

    // target name: always use StatepointInfo for name handling
    let target = "";
    let spinfo = Data.Instance().StatepointInfo( targetStatepoint, entity );
    target = spinfo['label'] || "";

    // build everything
    let text = $.lang( `{$widget_switchcontact_action}: "${ target }"` );

    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };

  }


  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    let reference = `sp:${Data.Instance().RootEntid()}/${entity['entid']}/${uid}`;
    return [ reference ];
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let spName       = deviceRecord['uid'];
    let spAddress    = entity['sp'][ spName ]['address'];
    let spInfo       = Data.Instance().StatepointInfo( spAddress, entity );

    // error statepoint does not exists
    if( spInfo['type'] == "ERROR" )  {
      validateControl.error = "{$const_error_statepoint_deleted}";
      return 1;
    }

    // error no permission
    if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddress, "w" ) ) {
      validateControl.error = "{$const_error_statepoint_permission}";
      return 1;
    }

    return 0;
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );

    // first statepoint in this entity, or no statepoint yet?
    if( !( 'sp' in this.entity ) ) {
      this.entity['sp'] = {};
    }
    if( !( this.uid in this.entity['sp'] ) ) {
      this.entity['sp'][ this.uid ] = this.CreateStateNotifyingContactSp();
    }

    // build label
    ControlLabel.Build( $( "#ID_STATE_NOTIFYING_CONTACT_LABEL" + this.dynid ), {
      label: this.entity['sp'][ this.uid ]['label'],
      entityName: this.entity['name'],
      functionName: WidgetDeviceStateNotifyingContact.widgetNameShort
    } );

    // console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );

  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // ---- widget parameters ----
    // load sp
    let spName = this.deviceRecord['uid'];
    let sp     = "";
    let spaddr = "";
    let label  = "";

    if( !this.entity['sp'][ spName ] )  {
      sp = this.CreateStateNotifyingContactSp();
    }
    else  {
      sp = this.entity['sp'][ spName ];
    }

    spaddr = sp['address'];
    spName = sp['name'];
    label  = sp['label'];

    // standard: save param into device record
    this.deviceRecord['template'] = "state_notifying_contact";
    this.deviceRecord['sp'] = spName;
    this.deviceRecord['param']                = {};
    this.deviceRecord['param']['posttrigger'] = true;
    this.deviceRecord['param']['invert']      = false;

    // ---- widget statepoint ----

    // write safely into entity
    if( !( "sp" in this.entity ) )  {
      this.entity['sp'] = {};
    }
    this.entity['sp'][ spName ]          = sp;
    this.entity['sp'][ spName ]['label'] = $( "#ID_STATE_NOTIFYING_CONTACT_LABEL" + this.dynid ).viewable( "option", "value" );

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }

  //===========================================================================
  //  VALIDATION (VIEWABLE OVERRIDE)
  //===========================================================================

  //---------------------------------------------------------------------------
  //  keys-dialog will call this at load & save
  //
  OnValidate( )
  {
    return 0;
  }

  //---------------------------------------------------------------------------
  //  keys-dialog calls it when widget is removed
  //
  OnValidateOff( )
  {
  }

  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  //---------------------------------------------------------------------------
  //  event from viewable, start unit test
  //
  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget State Notifying Contact" );

    // this.TestCaseMuteMicSpAddr( testControl, this.deviceRecord );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //=============================================================================
  //  Test Cases
  //=============================================================================

  // //-----------------------------------------------------------------------------
  // //  TestCaseMuteMicTargetEntityId(): description
  // //
  // TestCaseMuteMicSpAddr( testController, deviceRecord )
  // {
  //   let testFunctionTimestamp     = Date.now();
  //   let statePointeAddressPattern = /local\\mute_mic/g;
  //
  //   this.UnitTestCheckObject( "TestCaseMuteMicTargetEntityId", testController, "param/spaddr", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  // }
  CreateStateNotifyingContactSp()
  {
    let sp = {
        "name": this.deviceRecord['uid'],
        "type": "BOOL",
        "address": `${Data.Instance().RootEntid()}/${this.entity['entid']}/${this.deviceRecord['uid']}`,
        "label": `{{name}} (${ WidgetDeviceStateNotifyingContact.widgetNameShort })`,
        "template": "state_notifying_contact",
        "default": false,
        "resdef": [
          {
            "id": this.deviceRecord['in']['sysid'],
            "action": "input",
          }
        ]
      };
    return sp;
  }
}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceStateNotifyingContact.Register( "/widget/deviceConfig/state_notifying_contact" );
WidgetDeviceConfigFactory.Register( WidgetDeviceStateNotifyingContact );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigLocalDoorLight extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules      = [ "ATLC", "ATLM", "ATLCE1", "ATLCE2", "ATLCE3", "BTM", "BRMA" ];
  static allowedTypes        = ["in"];
  static widgetIcon          = "mdi mdi-lightbulb-outline";
  static widgetNameShort     = "{$widget_local_door_light_short}";
  static widgetNameLong      = "{$widget_local_door_light_long}";
  static widgetId            = "localdoorlight";
  static widgetAvpIcon       = "mikrofon_aus";
  static widgetFunctionLabel = "{$aui_widget_local_door_light}";

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything from entity
  //
  static Remove( uid, trigger, entity )
  {
    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( "localdoorlight", entity );
    WidgetDeviceConfigBase.HelperRemovePermission( "localdoorlight", entity );
    return true;
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/local_door_light";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    // no external ressources
    return null;
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    return 0;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    let text = $.lang( "{$widget_local_door_light_description_line}" );


    return {
      text: text,
      icon: null,     // just for LED
      color: null     // just for LED
    };

  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );


    // console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    if( this.deviceRecord['template'] !== "localdoorlight" ) {
      this.preventFirstTimeValidate = true;
    }

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormLoadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord, WidgetDeviceConfigLocalDoorLight.widgetAvpIcon, WidgetDeviceConfigLocalDoorLight.widgetFunctionLabel );
      $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_LOCAL_DOOR_LIGHT" );
    }
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    // ---- widget parameters ----

    // standard: save param into device record
    this.deviceRecord['template'] = "localdoorlight";
    delete this.deviceRecord['param'];

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormReadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord );
    }

    // UnitTests
    this.UnitTest();

    // return our copy of entity
    return this.entity;
  }

  //===========================================================================
  //  VALIDATION (VIEWABLE OVERRIDE)
  //===========================================================================

  //---------------------------------------------------------------------------
  //  keys-dialog will call this at load & save
  //
  OnValidate( )
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }
    return 0;
  }

  //---------------------------------------------------------------------------
  //  keys-dialog calls it when widget is removed
  //
  OnValidateOff( )
  {
  }

  //-----------------------------------------------------------------------------
  //  OnUnitTest(): description
  //
  OnUnitTest( testControl )
  {

    let testTimestamp = Date.now();

    console.group( "Widget Local Door Light" );

    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //=============================================================================
  //  Test Cases
  //=============================================================================

}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigLocalDoorLight.Register( "/widget/deviceConfig/local_door_light" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigLocalDoorLight );
//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigObserverLedKnx extends WidgetDeviceConfigBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static allowedModules      = [ "LED" ];
  static allowedTypes        = ["*"];
  static widgetIcon          = "mdi mdi-led-on";
  static widgetNameShort     = "{$widget_observe_led_knx_short}";
  static widgetNameLong      = "{$widget_observe_led_knx_long}";
  static widgetId            = "observerledknx";
  static widgetAvpIcon       = "rufton_aus";
  static widgetFunctionLabel = "{$aui_widget_dnd}";  // for $FUNCTION in edit controls and avp default title

  //===========================================================================
  //  STATIC FUNCTIONS
  //===========================================================================

  //---------------------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );

    WidgetDeviceConfigBase.HelperRemove( uid, trigger, entity );
    WidgetDeviceConfigBase.HelperRemoveStatepoint( deviceRecord['sp'], entity );
    //WidgetDeviceConfigBase.HelperRemovePermission( deviceRecord['sp'], entity );
  }

  //---------------------------------------------------------------------------
  //  build the widget
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/observerledknx";
    $element.viewable( createData );
  }

  //---------------------------------------------------------------------------
  //  get references
  //
  static References( uid, trigger, entity )
  {
    // let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    // if( ( deviceRecord ) && ( 'param' in deviceRecord ) ) {
    //   let reference = "sp:" + deviceRecord['param']['spaddr'];
    //   return [ reference ];
    // }

    // no references:
    return null;
  }

  //---------------------------------------------------------------------------
  //  validation without seen on screen
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    // get device record to check if statepoint is accessible
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( entity, uid, trigger );
    let knxGw        = deviceRecord['param']['knxgw'];
    let entinfo      = Data.Instance().Entinfo( knxGw, entity );

    if( !entinfo )  {
      validateControl.error = "{$const_error_device_deleted}";
      return 1;
    }

    return 0;
  }

  //---------------------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // defaults
    let targetStatepoint = deviceRecord['param']['target'];
    let text = $.lang( "{$widget_led_description_action}" );
    let ledColor = "default";

    // target
    text = $.lang( "{$widget_led_description_action}" ) + " " ;

    // color
    let availableColorCodes = WidgetDeviceConfigLed.HelperAvailableColorCodes( deviceRecord );
    let selectedColor = availableColorCodes[ 0 ]['id'];  // default
    if( ( 'param' in deviceRecord ) && ( 'color' in deviceRecord['param'] ) ) {
      if( deviceRecord['param']['color']['1'] === true ) {
        selectedColor = "1";
      }
      else if( deviceRecord['param']['color']['2'] === true ) {
        selectedColor = "2";
      }

      text = text.replace("%name%", entity['name']);
      text = text.replace("%groupAddress%", deviceRecord['param']['knxGroupAddress']);
    }
    ledColor = availableColorCodes.find( ( code ) => code['id'] == selectedColor );
    ledColor = ledColor['color'];

    // that's it
    let result = {
      text: text,
      icon: "mdi mdi-circle-medium",
      color: ledColor
    };
    return result;
  }


  //-----------------------------------------------------------------
  //  update led to linked trigger single or double
  //
  static UpdateLinked( deviceRecord, entity )
  {
    return true;
  }
  //===========================================================================
  //  INSTANCE
  //===========================================================================

  //---------------------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // set references to original data - DO NOT COPY!!!!
    // a widget should save any change directly to the data
    this.entity       = createData['entity'];
    this.uid          = createData['options']['uid'];
    this.trigger      = createData['options']['trigger'] || null;
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, this.uid, this.trigger );
    this.isCommandMode = WidgetDeviceConfigBase.HelperIsCommandMode( this.deviceRecord );

    console.log( `${this.constructor.name}( ${this.uid}, ${this.trigger} )`, this.deviceRecord );

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormLoadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord, WidgetDeviceConfigObserverLedKnx.widgetAvpIcon, WidgetDeviceConfigObserverLedKnx.widgetFunctionLabel );
    }

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );

    // dnd value
    // $( "#ID_OBSERVE_LED_KNX_VALUE" + this.dynid ).listbox( {
    //   size: CONST.size.L,
    //   type: 'droplist',
    //   selected: ["*"],
    //   data: [
    //     { id: "*", text: "{$widget_observe_led_knx_value_toggle}" },
    //     { id: "1", text: "{$widget_observe_led_knx_value_on}" },
    //     { id: "0", text: "{$widget_observe_led_knx_value_off}" }
    //   ]
    // } );

    // create statepoint list
    ControlEntityList.Build( $( "#ID_OBSERVE_LED_KNX_GATEWAY" + this.dynid ), {
      entity: this.entity,
      inquirer: this.entity['entid'],
      allowedClass: ['knxgw'],
      result: this.ControlStatepointResult.bind( this ),
    } );


    $( "#ID_OBSERVE_LED_GROUP_ADDRESS" + this.dynid ).edit( {
      size: CONST.size.L,
      value: "",
      result: $.proxy( this.Modified, this ),
      validate: ( event ) => { this.ValidateKnxAddress( event ) }
    } );

    this.availableColorCodes = WidgetDeviceConfigLed.HelperAvailableColorCodes( this.deviceRecord );
    $( "#ID_OBSERVE_LED_COLOR" + this.dynid ).choice( {
      size: CONST.size.XL,
      data: this.availableColorCodes,
      value: this.availableColorCodes[0]['id']
    } );

    // fill data
    this.FormFillControls();
  }

  //---------------------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    let knxGw = $("#ID_OBSERVE_LED_KNX_GATEWAY" + this.dynid ).viewable( "option", "selected" )[0];
    let ledColor = $("#ID_OBSERVE_LED_COLOR" + this.dynid ).choice( "option", "value" );
    let knxGroupAddress = $("#ID_OBSERVE_LED_GROUP_ADDRESS" + this.dynid ).edit( "option", "value" );
    //let oldKnxGroupAddress = this.entity['sp'][ this.deviceRecord['sp'] ] ?? false;
    let sp = this.entity['sp'][ this.deviceRecord['sp'] ] ?? false;
    let newSp = this.CreateKnxSp( knxGroupAddress, knxGw );

    // standard: save param into device record
    this.deviceRecord['sp']                      = newSp['name'];
    this.deviceRecord['template']                = "observerledknx";
    this.deviceRecord['param']                   = {};
    //this.deviceRecord['param']['target']         = `${Data.Instance().RootEntid()}/${this.entity['entid']}/${knxGw.replace( ".", "_" )}_knxr${knxGroupAddress.replaceAll( "/", "_" )}`;
    this.deviceRecord['param']['target']         = `${Data.Instance().RootEntid()}/${this.entity['entid']}/${newSp['name']}`;
    this.deviceRecord['param']['targetEntityId'] = "";
    this.deviceRecord['param']['knxgw']          = knxGw;
    this.deviceRecord['param']['knxGroupAddress'] = knxGroupAddress;
    this.deviceRecord['param']['script'] = {
      "dnd": false,
      "redir": false,
      "bool": true,
      "presence": false
    };

    this.deviceRecord['param']['color'] = {
      "1": ledColor === "1",
      "2": ledColor !== "1",
    }

    if( this.isCommandMode )  {
      this.deviceRecord['command'] = "SETSP(" + targetStatepoint + "," + this.deviceRecord['param']['value'] + ")";
    }

    // AVP common (method of base class)
    if( this.deviceRecord['module'] == "TILE" ) {
      this.FormReadTileControls( "#ID_AVP_LABEL" + this.dynid, this.deviceRecord );
    }

    if( newSp['name'] !== sp['name'] ) {
      delete this.entity['sp'][ sp['name'] ];
    }

    this.entity['sp'][ newSp['name'] ] = newSp;

    // UnitTests
    this.UnitTest();

    // return our copy of entity

    return this.entity;
  }


  //===========================================================================
  //  FORM
  //===========================================================================

  //---------------------------------------------------------------------------
  //  FormFillControls(): fill the controls with already set widget values
  //
  FormFillControls()
  {
    // defaults (used on create)
    let selectedKnxGateway      = [];
    let knxGroupAddress         = "";
    let ledColor                = "1";
    let sp;

    // previously saved: read from device record
    if( this.deviceRecord['template'] == "observerledknx" ) {

      // read from device record
      knxGroupAddress = this.deviceRecord['param']['knxGroupAddress'];
      selectedKnxGateway.push( this.deviceRecord['param']['knxgw'] );
      sp = this.deviceRecord['param']['sp'];

      // getting active color
      ledColor = this.deviceRecord['param']['color']['1'] ? "1" : "2";
    }
    else  {
      ledColor = this.availableColorCodes[ 0 ]['id'];
      this.preventFirstTimeValidate = true;
    }

    // update control
    $( "#ID_OBSERVE_LED_COLOR" + this.dynid ).choice( "option", "value", ledColor );
    $( "#ID_OBSERVE_LED_KNX_GATEWAY" + this.dynid ).viewable( "option", "selected", selectedKnxGateway );
    $( "#ID_OBSERVE_LED_GROUP_ADDRESS"  + this.dynid ).edit( "option", "value", knxGroupAddress );
  }


  //===========================================================================
  //  CONTROL HANDLER
  //===========================================================================

  //---------------------------------------------------------------------------
  //  ControlStatepointResult(): receives the result of call-target control
  //
  ControlStatepointResult( result )
  {
  }

  //===========================================================================
  //  VALIDATION (VIEWABLE OVERRIDE)
  //===========================================================================

  //---------------------------------------------------------------------------
  //  keys-dialog will call this at load & save
  //
  OnValidate( )
  {
    if( this.preventFirstTimeValidate == true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    let numError = 0;
    numError += $( '#ID_OBSERVE_LED_KNX_GATEWAY' + this.dynid ).viewable( "validate" );
    numError += $( '#ID_OBSERVE_LED_GROUP_ADDRESS' + this.dynid ).edit( "validate" );
    return numError;
  }

  //---------------------------------------------------------------------------
  //  keys-dialog calls it when widget is removed
  //
  OnValidateOff( )
  {
    $( '#ID_OBSERVE_LED_KNX_GATEWAY' + this.dynid ).viewable( "validate", false );
    $( '#ID_OBSERVE_LED_GROUP_ADDRESS' + this.dynid ).edit( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  ValidateKnxAddress(): check the string format of the knx address
  //
  ValidateKnxAddress( event )
  {
    if( this.CheckKnxGroupAddress( event.value ) ) {
      event.error = null;
    }
    else {
      event.error = "{$widget_set_sp_knx_error_address}";
    }
  }

  //===========================================================================
  //  MODE
  //===========================================================================


  //===========================================================================
  //  UNIT TEST
  //===========================================================================

  //---------------------------------------------------------------------------
  //  event from viewable, start unit test
  //
  OnUnitTest( testControl )
  {
    let testTimestamp = Date.now();

    console.group( "Widget Observe LED KNX" );

    this.TestCaseSetDndTarget( testControl, this.deviceRecord );
    this.TestCaseSetDndTargetEntity( testControl, this.deviceRecord );
    this.TestCaseSetDndValue( testControl, this.deviceRecord );
    console.log( `took: ${( Date.now() - testTimestamp ) / 1000}s` );

    console.groupEnd();
  }

  //---------------------------------------------------------------------------
  //  TestCaseCallTarget(): test function for target ( call number)
  //
  TestCaseSetDndTarget( testController, deviceRecord )
  {
    let testFunctionTimestamp     = Date.now();
    let statePointeAddressPattern = /\b.+\/[a-zA-Z]+\.\d+\/dnd\b/g;

    this.UnitTestCheckObject( "TestCaseSetDndTarget", testController, "param/spaddr", deviceRecord, statePointeAddressPattern, testFunctionTimestamp );
  }

  //---------------------------------------------------------------------------
  //  TestCaseCallTargetEntity(): test function for target entity
  //
  TestCaseSetDndTargetEntity( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();
    let entityPattern = /\b(apartment|avp|door|jungpanel|client|group|sipaudio|susapp)\.\d+\b/g;

    this.UnitTestCheckObject( "TestCaseSetDndTargetEntity", testController, "param/targetEntityId", deviceRecord, entityPattern, testFunctionTimestamp );
  }

  //---------------------------------------------------------------------------
  //  TestCaseSetDndValue(): test function for target entity
  //
  TestCaseSetDndValue( testController, deviceRecord )
  {
    let testFunctionTimestamp = Date.now();
    let possibleValues        = ["*", "0", "1"];

    this.UnitTestCheckObject( "TestCaseLastTrigger", testController, "param/value", deviceRecord, possibleValues, testFunctionTimestamp );
  }

  //-----------------------------------------------------------------
  //  HELPER
  //-----------------------------------------------------------------

  // determine the colors used in this led (list for control)
  static HelperAvailableColorCodes( deviceRecord )
  {
    let availableColorCodes = [];

    let index = 0;
    for( let led of deviceRecord['led'] ) {
      if( led['sysname'].indexOf( "spare" ) >= 0 ) {
        index++;
      }
      if( led['sysname'].indexOf( "white" ) >= 0 ) {
        index++;
        availableColorCodes.push( { id: String( index ), text: "{$widget_led_color_white}", color: "blue" } );
      }
      if( led['sysname'].indexOf( "amber" ) >= 0 ) {
        index++;
        availableColorCodes.push( { id: String( index ), text: "{$widget_led_color_yellow}", color: "yellow" } );
      }
      if( led['sysname'].indexOf( "red" ) >= 0 ) {
        index++;
        availableColorCodes.push( { id: String( index ), text: "{$widget_led_color_red}", color: "red" } );
      }
    }
    return availableColorCodes;
  }

  CreateKnxSp( knxGroupAddress, knxGateway )
  {
    let sp = {};
    //let knxGateway = "";
    //let knxStatePointName = `${knxGateway.replace( /\./g,  "_" )}_knxr${knxGroupAddress.replace( /\//g, "_" )}`;
    var knxStatePointName = "knxr_" + $.guidgen();

    sp['name'] = knxStatePointName;
    sp['type'] = "BOOL";
    sp['label'] =  "{$widget_led_observe}";
    sp['default'] = false;
    sp['HasDsr'] = true;
    sp['knxdef'] = [{
      "send": null,
      "receive": knxGroupAddress,
      "dpt": "1.000"
    }];
    sp['address'] = `${Data.Instance().RootEntid()}/${this.entity['entid']}/${knxStatePointName}`;
    sp['default'] = false;
    sp['template'] = "observerledknx";
    sp['gateway'] = knxGateway;
    sp['hidden'] = true;



    return sp;
  }

  /**
   * checkKnxGroupAddress( address )
   *
   * @param address address knx group address
   * checks if the entered knx group address is valid
   */
  CheckKnxGroupAddress( address )
  {
    if( !address ) {
      return false;
    }
    var splitAddress = address.split( "/" );

    var main   = parseInt( splitAddress[0] );
    var middle = parseInt( splitAddress[1] );
    var sub    = parseInt( splitAddress[2] );

    if( splitAddress.length == 3 ) {
      if( main == 0 && middle == 0 && sub == 0 ) {
        return false;
      }
      if( main < 0 || main > 31 ) {
        return false;
      }
      if( middle < 0 || middle > 7 ) {
        return false;
      }
      if( sub < 0 || sub > 255 ) {
        return false;
      }
    }
    else if( splitAddress.length == 2 ) {
      if( main == 0 && middle == 0 ) {
        return false;
      }
      if( main < 0 || main > 31 ) {
        return false;
      }
      if( middle < 0 || middle > 2047 ) {
        return false;
      }
    }
    else {
      if( isNaN( main ) ) {
        return false;
      }
      if( main <= 0 ) {
        return false;
      }
      if( main > 65535 ) {
        return false;
      }
    }
    return true;
  }
}


//=============================================================================
//  REGISTER IN FACTORY
//=============================================================================

WidgetDeviceConfigObserverLedKnx.Register( "/widget/deviceConfig/observerledknx" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigObserverLedKnx );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceConfigToemode extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static allowedModules = [ "ATLC-toemode" ];
  static allowedTypes = "";
  static widgetIcon = "mdi mdi-help";
  static widgetNameShort = "{$widget_toemode_short}";
  static widgetNameLong = "{$widget_toemode_long}";
  static widgetId = "toemode";
  static widgetIsLineControl = true;

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  remove everything from entity
  //
  static Remove( uid, trigger, entity )
  {
    return true;
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = "/widget/deviceConfig/toemode";
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    // no external ressources
    return null;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: $.lang( "{$widget_toemode_descrline}" ),
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // $.clone not need, "createData" is cloned!
    this.entity = createData['entity'];
    this.resultHandler = createData['options']['result'];
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, WidgetDeviceConfigToemode.widgetId );
    $( "#ID_TOEMODE" + this.dynid ).choice( {
      size: CONST.size.XL,
      type: "radio",
      layout: 'horizontal',      
      value: this.deviceRecord['toemode'],
      data: [
        { id: "ac", text: "{$widget_toemode_ac}" },
        { id: "dc", text: "{$widget_toemode_dc}" }
      ],
      result: ( value ) => {
        this.deviceRecord['toemode'] = value;
        this.resultHandler( this.deviceRecord );
      }
    } );
  }

  //-----------------------------------------------------------------
  //  save value to entity
  //
  Save(  )
  {
  }

}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceConfigToemode.Register( "/widget/deviceConfig/toemode" );
WidgetDeviceConfigFactory.Register( WidgetDeviceConfigToemode );
﻿//=============================================================================
//
//  FUNCTION WIDGET DND IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WidgetDeviceFunctionDnd extends WidgetDeviceConfigBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template = "/widget/deviceFunctions/dnd";
  static isFunction = true;
  static allowedModules = ["function"];             // module is function..
  static allowedTypes = "*";                        // not used by functions
  static allowedClasses = ["apartment", "avp", "sipaudio", "door", "group", "client", "asht", "jungpanel"];    // used by functions
  static allowedOnEntrance = true;
  static functionMustBePresent = false;             // don't show widget of no deviceRecord is present
  static widgetIcon = "mdi mdi-minus-circle"; 
  static widgetNameLong = "{$widget_dnd_long}";
  static widgetNameShort = "{$aui_widget_dnd}";     // label: must be "aui_" (a dict.json macro) 
  static widgetId = "dnd";
  static widgetUiHeight = 4;                        // we need 4rem in height

  //===========================================================================
  //  STATIC INFORMATION OVERRIDABLES
  //===========================================================================

  //-----------------------------------------------------------------
  //  local statepoint for permission check
  //
  static LocalStatepointName( uid, trigger, entity )
  {
    return "dnd";
  }

  //---------------------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    // switch off function in device record
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, "dnd" );
    if( deviceRecord ) {
      deviceRecord['sp'] = null;
    }

    // remove statepoint
    if( 'dnd' in entity['sp'] )  {
      delete entity['sp']['dnd'];
    }

    // remove permissions
    WidgetDeviceConfigBase.HelperRemovePermission( "dnd", entity );

  }

  //---------------------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = WidgetDeviceFunctionDnd.template;
    $element.viewable( createData );
  }

  //---------------------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    if( ( 'sp' in entity ) && ( 'dnd' in entity['sp'] ) )  {
      let reference = `sp:${ Data.Instance().RootEntid() }/${ entity['entid'] }/dnd`;
      return [ reference ];
    }
    return null;
  }

  //---------------------------------------------------------------------------
  //  validate the widget
  //
  static Validate( entity )
  {
    return 0;
  }

  //---------------------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: WidgetDeviceFunctionDnd.widgetNameLong,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //===========================================================================
  //  INSTANCE
  //===========================================================================

  //---------------------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // copy not needed
    this.entity = createData['entity'];
    this.statepoint = this.entity['sp']['dnd'];
    if( Array.isArray( this.entity['sp'] ) )  {
      this.entity['sp'] = {};
    }


    // get device record create if not exists
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, WidgetDeviceFunctionDnd.widgetId );
    if( !this.deviceRecord )  {
      this.deviceRecord = WidgetDeviceConfigBase.HelperAddDeviceFunction( this.entity, WidgetDeviceFunctionDnd.widgetId );
    }

    // create default
    if( !( 'dnd' in this.entity['sp'] ) ) {
      this.statepoint = {
        "label": "{{name}} (" + WidgetDeviceFunctionDnd.widgetNameShort + ")",
        "address": ( Data.Instance().RootEntid() + "/" + this.entity['entid'] + "/" + WidgetDeviceFunctionDnd.widgetId ),
        "type": "BOOL",
        "name": WidgetDeviceFunctionDnd.widgetId,
        "persistent": true,
        "template": WidgetDeviceFunctionDnd.widgetId
      };
      this.entity['sp']['dnd'] = this.statepoint;
      this.deviceRecord['sp'] = WidgetDeviceFunctionDnd.widgetId;
      this.Option( "modified", 1 );
    }

    // ensure default permission record exists
    WidgetDeviceConfigBase.HelperAddDefaultPermission( WidgetDeviceFunctionDnd.widgetId, this.entity );


    // build label
    ControlLabel.Build( $( "#ID_LABEL" + this.dynid ), {
      label:        this.statepoint['label'],
      entityName:   this.entity['name'],
      functionName: WidgetDeviceFunctionDnd.widgetNameShort
    } );

  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidate()
  {
    let numError = 0;
    $( "#ID_LABEL" + this.dynid ).viewable( "validate" );
    return numError;
  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidateOff()
  {
    $( "#ID_LABEL" + this.dynid ).viewable( "validate", false );
  }

  //---------------------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save( )
  {
    // read label and return our entity-copy
    this.entity['sp']['dnd']['label'] = $( "#ID_LABEL" + this.dynid ).viewable( "option", "value" );
    return this.entity;
  }

  

}


//=============================================================================
//  REGISTER IN FACTORY
//=============================================================================

WidgetDeviceFunctionDnd.Register( WidgetDeviceFunctionDnd.template );
WidgetDeviceConfigFactory.Register( WidgetDeviceFunctionDnd );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceFunctionPresence extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/deviceFunctions/presence";
  static isFunction = true;
  static allowedModules = ["function"];             // module is function..
  static allowedTypes = "*";                        // not used by functions
  static allowedClasses = ["apartment", "avp", "tkphone", "sipaudio", "door", "group", "susapp", "client", "asht", "jungpanel"];    // used by functions
  static allowedOnEntrance = true;
  static functionMustBePresent = false;             // don't show widget of no deviceRecord is present
  static widgetIcon = "mdi mdi-account-check";
  static widgetNameLong = "{$widget_presence_long}";
  static widgetNameShort = "{$aui_widget_presence}";     // label: must be "aui_" (a dict.json macro)
  static widgetId = "presence";
  static widgetUiHeight = 4;                        // we need 4rem in height

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  local statepoint for permission check
  //
  static LocalStatepointName( uid, trigger, entity )
  {
    return "presence";
  }

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    // switch off function in device record
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, "presence" );
    if( deviceRecord ) {
      deviceRecord['sp'] = null;
    }

    // remove statepoint
    if( 'presence' in entity['sp'] )  {
      delete entity['sp']['presence'];
    }

    // remove permissions
    WidgetDeviceConfigBase.HelperRemovePermission( "presence", entity );

  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = WidgetDeviceFunctionPresence.template;
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    if( ( 'sp' in entity ) && ( 'presence' in entity['sp'] ) ) {
      let reference = `sp:${ Data.Instance().RootEntid() }/${ entity['entid'] }/presence`;
      return [ reference ];
    }
    return null;
  }

  //-----------------------------------------------------------------
  //  validate the widget
  //
  static Validate( uid, trigger, entity )
  {
    return 0;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: WidgetDeviceFunctionPresence.widgetNameLong,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // copy not needed
    this.entity = createData['entity'];
    this.statepoint = this.entity['sp']['presence'];
    if( Array.isArray( this.entity['sp'] ) )  {
      this.entity['sp'] = {};
    }

    // get device record create if not exists
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, WidgetDeviceFunctionPresence.widgetId );
    if( !this.deviceRecord )  {
      this.deviceRecord = WidgetDeviceConfigBase.HelperAddDeviceFunction( this.entity, WidgetDeviceFunctionPresence.widgetId );
    }

    // create default
    if( !( 'presence' in this.entity['sp'] ) ) {
      this.statepoint = {
        "label": "{{name}}",
        "address": ( Data.Instance().RootEntid() + "/" + this.entity['entid'] + "/" + WidgetDeviceFunctionPresence.widgetId ),
        "type": "PRESENCE",
        "name": WidgetDeviceFunctionPresence.widgetId,
        "persistent": false,
        "template": WidgetDeviceFunctionPresence.widgetId
      };
      this.entity['sp']['presence'] = this.statepoint;
      this.deviceRecord['sp'] = WidgetDeviceFunctionPresence.widgetId;
      this.Option( "modified", 1 );
    }

    // ensure default permission record exists
    WidgetDeviceConfigBase.HelperAddDefaultPermission( WidgetDeviceFunctionPresence.widgetId, this.entity );


    // build label
    ControlLabel.Build( $( "#ID_LABEL" + this.dynid ), {
      label:        this.statepoint['label'],
      entityName:   this.entity['name'],
      functionName: WidgetDeviceFunctionPresence.widgetNameShort
    } );

  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidate()
  {
    let numError = 0;
    $( "#ID_LABEL" + this.dynid ).viewable( "validate" );
    return numError;
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save( )
  {
    // read label and return our entity-copy
    this.entity['sp']['presence']['label'] = $( "#ID_LABEL" + this.dynid ).viewable( "option", "value" );
    return this.entity;
  }

  

}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceFunctionPresence.Register( WidgetDeviceFunctionPresence.template );
WidgetDeviceConfigFactory.Register( WidgetDeviceFunctionPresence );
﻿//=============================================================================
//
//  FUNCTION WIDGET DND IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WidgetDeviceFunctionGrabpic extends WidgetDeviceConfigBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template = "/widget/deviceFunctions/grabpic";
  static isFunction = true;
  static allowedModules = ["function"];             // module is function..
  static allowedTypes = "*";                        // not used by functions
  static allowedClasses = ["apartment", "avp", "tkphone", "sipaudio", "door", "susapp", "client", "asht", "jungpanel", "ipcam"];    // used by functions
  static functionMustBePresent = false;             // don't show widget of no deviceRecord is present
  static widgetIcon = "mdi mdi-image-multiple";
  static widgetNameLong = "{$widget_grabpic_long}";
  static widgetNameShort = "{$aui_widget_grabpic}";     // label: must be "aui_" (a dict.json macro)
  static widgetId = "grabpic";
  static widgetUiHeight = 10;                        // we need 12rem in height

  //===========================================================================
  //  STATIC INFORMATION OVERRIDABLES
  //===========================================================================

  //-----------------------------------------------------------------
  //  special: check if this widget is allowed in this entity
  //
  static IsUseable( entity, isFunction, isLed )
  {
    // this widget is for function
    if( !isFunction ) {
      return false;
    }

    // check class list
    let isClassAllowed = ( WidgetDeviceFunctionGrabpic.allowedClasses.indexOf( entity['class'] ) >= 0 );
    
    // special handling for audio only indoor devices
    if( entity['deviceTemplate'] == "AFS" 
     || entity['deviceTemplate'] == "AHF_870-0"
     || entity['deviceTemplate'] == "AHT_870-0"
      ) {
      return false;
    }

    if( ( entity['class'] == "door" )
     && ( 'video' in entity )
     && ( 'send' in entity['video'] )
     && ( entity['video']['send'] == false )
      ) {
      return false;
    }
    
    return isClassAllowed;
  }

  //---------------------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    // switch off function in device record
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, "grabpic" );
    if( deviceRecord ) {
      deviceRecord['sp'] = null;
    }

    // remove statepoint
    if( 'grabpic' in entity['sp'] )  {
      delete entity['sp']['grabpic'];
    }

    // remove permissions
    WidgetDeviceConfigBase.HelperRemovePermission( "grabpic", entity );

  }

  //---------------------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = WidgetDeviceFunctionGrabpic.template;
    $element.viewable( createData );
  }

  //---------------------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    if( ( 'sp' in entity ) && ( 'grabpic' in entity['sp'] ) ) {
      let reference = `sp:${ Data.Instance().RootEntid() }/${ entity['entid'] }/grabpic`;
      return [ reference ];
    }

    return null;
  }

  //---------------------------------------------------------------------------
  //  validate the widget
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    return 0;
  }

  //---------------------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: WidgetDeviceFunctionGrabpic.widgetNameLong,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //===========================================================================
  //  INSTANCE
  //===========================================================================

  //---------------------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // copy not needed
    this.entity = createData['entity'];
    this.statepoint = this.entity['sp']['grabpic'];
    if( Array.isArray( this.entity['sp'] ) )  {
      this.entity['sp'] = {};
    }
    this.isAutomaticOnly = ( this.entity['deviceTemplate'] == "AFSV" || this.entity['class'] == "susapp" );
    this.isManuallyOnly = ( this.entity['class'] == "ipcam" );

    // get device record create if not exists
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, WidgetDeviceFunctionGrabpic.widgetId );
    if( !this.deviceRecord )  {
      this.deviceRecord = WidgetDeviceConfigBase.HelperAddDeviceFunction( this.entity, WidgetDeviceFunctionGrabpic.widgetId );
    }

    // if previously off, turn on
    if( this.deviceRecord['sp'] != WidgetDeviceFunctionGrabpic.widgetId )  {
      this.deviceRecord['sp'] = WidgetDeviceFunctionGrabpic.widgetId;
      this.Option( "modified", 1 );
    }

    // create default
    if( !( 'grabpic' in this.entity['sp'] ) ) {
      this.statepoint = {
        "address": ( Data.Instance().RootEntid() + "/" + this.entity['entid'] + "/" + WidgetDeviceFunctionGrabpic.widgetId ),
        "type": "BOOL",
        "name": WidgetDeviceFunctionGrabpic.widgetId,
        "persistent": false,
        "hidden": true,
        "template": WidgetDeviceFunctionGrabpic.widgetId,
        "param": {
          "manuallyOnly": this.isAutomaticOnly ? false : true,
          "missedCalls": this.isAutomaticOnly ? true : false,
          "calls": false,
          "time": 5
        }
      };
      this.entity['sp']['grabpic'] = this.statepoint;
    }

    // correct if param does not exist
    if( !( 'param' in this.statepoint ) ) {
      this.statepoint['param'] = {};
      this.statepoint['param']['manuallyOnly'] = this.isAutomaticOnly ? false : true;
      this.statepoint['param']['missedCalls'] = this.isAutomaticOnly ? true : false;
      this.statepoint['param']['calls'] = false;
      this.statepoint['param']['time'] = 5;
    }

    // correct manual on AFSV
    if( this.isAutomaticOnly )  {
      this.statepoint['param']['manuallyOnly'] = false;
      //this.statepoint['param']['missedCalls'] = true;
    }

    // optional control is needed by mode
    $( "#ID_OPTIONAL" + this.dynid ).optional( {} );

    //if( this.entity['class'] == "door" ) {
    if( 'entrance' in this.entity && this.entity['entrance']['enabled'] ) {
      this.ModeTime();
    }
    else if( ( this.statepoint['param']['manuallyOnly'] == true )
     || ( this.statepoint['param']['missedCalls'] == false && this.statepoint['param']['calls'] == false ) 
      ) {
      this.ModeManually();
    }
    else {
      this.ModeAutomatic();
    }

    // fill data
    this.FormFillControls();
  }

  //---------------------------------------------------------------------------
  //  create controls and fill data
  //
  FormFillControls()
  {
    let mode = ( this.mode == "modeAutomatic" ) ? "automatic" : "manual";
    let isAutoOnly = ( this.entity['class'] == "susapp" ) || ( this.entity['deviceTemplate'] == "AFSV" );
    let isManualOnly = ( this.entity['class'] == "ipcam" );

    $( "#ID_TIME" + this.dynid ).edit( {
      size: CONST.size.M,
      value: this.statepoint['param']['time'],
      //validate: Validate.NotEmptyString
      validate: this.ValidateTime.bind( this )
    } );

    $( "#ID_MODE" + this.dynid ).choice( {
      size: CONST.size.XL,
      type: 'radio',
      data: [
        { id: 'manual', text: "{$widget_grabpic_mode_manual}", status: isAutoOnly ? "disabled" : "normal" },
        { id: 'automatic', text: "{$widget_grabpic_mode_auto}", status: isManualOnly ? "disabled" : "normal" },
      ],
      value: mode,
      result: ( id ) => {
        if( id == "manual" ) {
          this.ModeManually();
        } 
        else {
          this.ModeAutomatic();
        }
      }
    } );

    $( "#ID_CALLS" + this.dynid ).choice( {
      size: CONST.size.XL,
      type: 'check',
      validate: this.ValidateCalls.bind( this ),
      data: [
        { id: 'calls', text: '{$widget_grabpic_event_calls}', value: this.statepoint['param']['calls'] },
        { id: 'missedcalls', text: '{$widget_grabpic_event_missedcalls}', value: this.statepoint['param']['missedCalls'] }
      ],
    } );
  }

  //---------------------------------------------------------------------------
  // validate time (4...30 sec)
  //
  ValidateTime( validateControl )  
  {
    let value = parseInt( validateControl.value );
    if( value < 2 || value > 30 )  {
      validateControl.error = "{$widget_grabpic_time_error}";
      return 1;
    }

    return 0;
  }

  //---------------------------------------------------------------------------
  // validate calls option field
  //
  ValidateCalls( validateControl )  
  {
    if( validateControl.value['calls'] == false && validateControl.value['missedcalls'] == false ) {
      validateControl.error = "{$widget_grabpic_option_error}";
      return 1;
    }
    return 0;
  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidate()
  {
    let numError = 0;
    if( this.mode == "modeTime" ) {
      $( "#ID_TIME" + this.dynid ).edit( "validate" );
    }
    if( this.mode == "modeAutomatic" ) {
      $( "#ID_CALLS" + this.dynid ).choice( "validate" );
    }
    return numError;
  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidateOff()
  {
  }

  //---------------------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save( )
  {
    // read controls and return
    if( this.mode == "modeTime" ) {
      this.statepoint['param']['manuallyOnly'] = true;
      this.statepoint['param']['missedCalls'] = false;
      this.statepoint['param']['calls'] = false;
      this.statepoint['param']['isImageStorageEnabled'] = true;
      this.statepoint['param']['time'] = parseInt( $( "#ID_TIME" + this.dynid ).edit( "option", "value" ) );
    }
    else if( this.mode == "modeManually" ) {
      this.statepoint['param']['manuallyOnly'] = true;
      this.statepoint['param']['missedCalls'] = false;
      this.statepoint['param']['calls'] = false;
      this.statepoint['param']['time'] = 0;
    }
    else {
      let callsResult = $( "#ID_CALLS" + this.dynid ).choice( "option", "value" );
      this.statepoint['param']['manuallyOnly'] = false;
      this.statepoint['param']['missedCalls'] = callsResult['missedcalls'];
      this.statepoint['param']['calls'] = callsResult['calls'];
      this.statepoint['param']['time'] = 0;
    }

    return this.entity;
  }

  //===========================================================================
  //  MODES
  //===========================================================================

  ModeTime()
  {
    if( this.mode == "modeTime" )
      return;

    this.mode = "modeTime";
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_TIME" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_MODE" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_CALLS" );
  }

  ModeAutomatic()
  {
    if( this.mode == "modeAutomatic" )
      return;

    this.mode = "modeAutomatic";
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_TIME" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_CALLS" );
  }

  ModeManually()
  {
    if( this.mode == "modeManually" )
      return;

    this.mode = "modeManually";
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_TIME" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_CALLS" );

  }

}


//=============================================================================
//  REGISTER IN FACTORY
//=============================================================================

WidgetDeviceFunctionGrabpic.Register( WidgetDeviceFunctionGrabpic.template );
WidgetDeviceConfigFactory.Register( WidgetDeviceFunctionGrabpic );
//=============================================================================
//
//  FUNCTION WIDGET REDIR IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WidgetDeviceFunctionRedir extends WidgetDeviceConfigBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template = "/widget/deviceFunctions/redir";
  static isFunction = true;
  static allowedModules = ["function"];             // module is function..
  static allowedTypes = "*";                        // not used by functions
  static allowedClasses = ["apartment", "avp", "sipaudio", "door", "group", "client", "asht", "jungpanel"];    // used by functions
  static allowedOnEntrance = true;
  static functionMustBePresent = false;             // don't show widget of no deviceRecord is present
  static widgetIcon = "mdi mdi-phone-missed";
  static widgetNameLong = "{$widget_redir_long}";
  static widgetNameShort = "{$aui_widget_redir}";     // label: must be "aui_" (a dict.json macro)
  static widgetId = "redir";
  static widgetUiHeight = 4;                        // we need 4rem in height

  //===========================================================================
  //  STATIC INFORMATION OVERRIDABLES
  //===========================================================================

  //-----------------------------------------------------------------
  //  local statepoint for permission check
  //
  static LocalStatepointName( uid, trigger, entity )
  {
    return "redir";
  }

  //---------------------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    // switch off function in device record
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, "redir" );
    if( deviceRecord ) {
      deviceRecord['sp'] = null;
    }

    // remove statepoint
    if( 'redir' in entity['sp'] )  {
      delete entity['sp']['redir'];
    }

    // remove permissions
    WidgetDeviceConfigBase.HelperRemovePermission( "redir", entity );

  }

  //---------------------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = WidgetDeviceFunctionRedir.template;
    $element.viewable( createData );
  }

  //---------------------------------------------------------------------------
  //  get used external ressources
  //
  static References( entity )
  {
    return null;
  }

  //---------------------------------------------------------------------------
  //  validate the widget
  //
  static Validate( entity )
  {
    return 0;
  }

  //---------------------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: WidgetDeviceFunctionRedir.widgetNameLong,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //===========================================================================
  //  INSTANCE
  //===========================================================================

  //---------------------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // copy not needed
    this.entity = createData['entity'];
    this.statepoint = this.entity['sp']['redir'];
    if( Array.isArray( this.entity['sp'] ) )  {
      this.entity['sp'] = {};
    }


    // get device record create if not exists
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, WidgetDeviceFunctionRedir.widgetId );
    if( !this.deviceRecord )  {
      this.deviceRecord = WidgetDeviceConfigBase.HelperAddDeviceFunction( this.entity, WidgetDeviceFunctionRedir.widgetId );
    }

    // create default
    if( !( 'redir' in this.entity['sp'] ) ) {
      this.statepoint = {
        "label": "{{name}} (" + WidgetDeviceFunctionRedir.widgetNameShort + ")",
        "address": ( Data.Instance().RootEntid() + "/" + this.entity['entid'] + "/" + WidgetDeviceFunctionRedir.widgetId ),
        "type": "REDIR",
        "name": WidgetDeviceFunctionRedir.widgetId,
        "persistent": true,
        "template": WidgetDeviceFunctionRedir.widgetId
      };
      this.entity['sp']['redir'] = this.statepoint;
      this.deviceRecord['sp'] = WidgetDeviceFunctionRedir.widgetId;
      this.Option( "modified", 1 );
    }

    // ensure default permission record exists
    WidgetDeviceConfigBase.HelperAddDefaultPermission( WidgetDeviceFunctionRedir.widgetId, this.entity );


    // build label
    ControlLabel.Build( $( "#ID_LABEL" + this.dynid ), {
      label:        this.statepoint['label'],
      entityName:   this.entity['name'],
      functionName: WidgetDeviceFunctionRedir.widgetNameShort
    } );

  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidate()
  {
    let numError = 0;
    $( "#ID_LABEL" + this.dynid ).viewable( "validate" );
    return numError;
  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidateOff()
  {
    $( "#ID_LABEL" + this.dynid ).viewable( "validate", false );
  }

  //---------------------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save( )
  {
    // read label and return our entity-copy
    this.entity['sp']['redir']['label'] = $( "#ID_LABEL" + this.dynid ).viewable( "option", "value" );
    return this.entity;
  }

  

}


//=============================================================================
//  REGISTER IN FACTORY
//=============================================================================

WidgetDeviceFunctionRedir.Register( WidgetDeviceFunctionRedir.template );
WidgetDeviceConfigFactory.Register( WidgetDeviceFunctionRedir );
﻿//=============================================================================
//
//  FUNCTION WIDGET DND IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WidgetDeviceFunctionDoormatic extends WidgetDeviceConfigBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template = "/widget/deviceFunctions/doormatic";
  static isFunction = true;
  static allowedModules = ["function"];             // module is function..
  static allowedTypes = "*";                        // not used by functions
  static allowedClasses = ["apartment", "avp", "sipaudio", "door", "client", "asht", "jungpanel"];    // used by functions
  static functionMustBePresent = false;             // don't show widget of no deviceRecord is present
  static widgetIcon = "mdi mdi-timer";
  static widgetNameLong = "{$widget_doormatic_long}";
  static widgetNameShort = "{$aui_widget_doormatic}";     // label: must be "aui_" (a dict.json macro)
  static widgetId = "doormatic";
  static widgetUiHeight = 18;                        // we need 4rem in height
  static modeAllDoors = "modeAllDoors";
  static modeSelectedDoors = "modeSelectedDoors";
  static defaultTimer = 10;

  //===========================================================================
  //  STATIC INFORMATION OVERRIDABLES
  //===========================================================================


  //-----------------------------------------------------------------
  //  special: check if this widget is allowed in this entity
  //
  static IsUseable( entity, isFunction, isLed, module, configRecordType )
  {
    // this widget is for function
    if( !isFunction ) {
      return false;
    }

    if( ( 'entrance' in entity )
     && ( 'enabled' in entity['entrance'] )
     && ( entity['entrance']['enabled'] == true )
      ) {
      return true;
    }

    var isUsableByType = ( 
        ( WidgetDeviceFunctionDoormatic['allowedTypes'] == "*" )
     || ( WidgetDeviceFunctionDoormatic['allowedTypes'].indexOf( configRecordType ) >= 0 )
      );
    if( !isUsableByType )
      return false;

    var isUsableByClass = ( 
        ( WidgetDeviceFunctionDoormatic == "*" )
     || !( 'allowedClasses' in WidgetDeviceFunctionDoormatic )
     || ( WidgetDeviceFunctionDoormatic['allowedClasses'] == "*" )
     || ( WidgetDeviceFunctionDoormatic['allowedClasses'].indexOf( entity['class'] ) >= 0 )
      );
    if( !isUsableByClass )
      return false;

    var isUsableByModule = ( 
        ( WidgetDeviceFunctionDoormatic['allowedTypes'] == "*" )
     || ( WidgetDeviceFunctionDoormatic['allowedTypes'].indexOf( configRecordType ) >= 0 )
      );      
    if( ! isUsableByModule )
      return false;


    return true;
  }

  //-----------------------------------------------------------------
  //  local statepoint for permission check
  //
  static LocalStatepointName( uid, trigger, entity )
  {
    return "doormatic";
  }

  //---------------------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    // switch off function in device record
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, "doormatic" );
    if( deviceRecord ) {
      deviceRecord['sp'] = null;
    }

    // remove statepoint
    if( 'doormatic' in entity['sp'] )  {
      delete entity['sp']['doormatic'];
    }

    // remove permissions
    WidgetDeviceConfigBase.HelperRemovePermission( "doormatic", entity );

  }

  //---------------------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = WidgetDeviceFunctionDoormatic.template;
    $element.viewable( createData );
  }

  //---------------------------------------------------------------------------
  //  get used external resources
  //
  static References( uid, trigger, entity )
  {
    if( ( 'sp' in entity ) && ( 'doormatic' in entity['sp'] ) ) {
      let refList = [];

      // own statepoint
      refList.push( `sp:${ Data.Instance().RootEntid() }/${ entity['entid'] }/doormatic` );

      // on doormatic no permission check, so use call reference
      let sp = entity['sp']['doormatic'];
      if( ( 'param' in sp ) && Array.isArray( sp['param']['targets'] ) ) {
        for( let target of sp['param']['targets'] ) {
          refList.push( "call:" + target );
        }
      }
      return refList;
    }

    return null;
  }

  //---------------------------------------------------------------------------
  //  get used external resources
  //
  /*static References( uid, trigger, entity )
  {
    console.log( "references of doormatic" );
    let statepointDataset = Data.Instance().StatepointDataset();
    let reference         = [];

    let statepointListDataview = statepointDataset.BuildView( 'default', {
      rebuildAlways: true,
      entity: entity,
      sortOrder: "+",
      sortKey: "text",
      filter: this.FilterDoorOpener.bind( this ),
    } );

    for( let index = 0; index < statepointListDataview.length; index++ )  {
      reference.push( `sp:${statepointListDataview.GetByIndex( index )['address']}` )
    }
    return reference.length > 0 ? reference : null;
  }*/

  //---------------------------------------------------------------------------
  //  validate the widget
  //
  static Validate( entity )
  {
    return 0;
  }

  //---------------------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: WidgetDeviceFunctionDoormatic.widgetNameLong,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //===========================================================================
  //  INSTANCE
  //===========================================================================

  //---------------------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // copy not needed
    this.entity       = createData['entity'];
    this.statepoint   = this.entity['sp']['doormatic'];
    if( Array.isArray( this.entity['sp'] ) )  {
      this.entity['sp'] = {};
    }
    this.selectedList = [];

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );

    // get device record create if not exists
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, WidgetDeviceFunctionDoormatic.widgetId );
    if( !this.deviceRecord )  {
      this.deviceRecord = WidgetDeviceConfigBase.HelperAddDeviceFunction( this.entity, WidgetDeviceFunctionDoormatic.widgetId );
    }

    // create default
    if( !( 'doormatic' in this.entity['sp'] ) ) {
      this.statepoint = this.HelperCreateDoormaticSp( WidgetDeviceFunctionDoormatic.defaultTimer, [] );
      this.entity['sp']['doormatic'] = this.statepoint;
      this.deviceRecord['sp'] = WidgetDeviceFunctionDoormatic.widgetId;
      this.Option( "modified", 1 );
    }

    // ensure default permission record exists
    WidgetDeviceConfigBase.HelperAddDefaultPermission( WidgetDeviceFunctionDoormatic.widgetId, this.entity );


    // build label
    ControlLabel.Build( $( "#ID_LABEL" + this.dynid ), {
      label:        this.statepoint['label'],
      entityName:   this.entity['name'],
      functionName: WidgetDeviceFunctionDoormatic.widgetNameShort
    } );


    // dataset/-view used by <add> button
    this.entityDataset = Data.Instance().EntityDataset();
    this.statepointDataset = Data.Instance().StatepointDataset();

    this.selectedListDataview = this.entityDataview = this.entityDataset.BuildView( 'default', {
      rebuildAlways: true,
      sortOrder: "+",
      sortKey: "text",
      filter: this.ControlEntitySelectedDataviewFilter.bind( this )
    } );

    this.entityDataview = this.entityDataset.BuildView( 'default', {
      rebuildAlways: true,
      sortOrder: "+",
      sortKey: "text",
      filter: this.ControlEntityDataviewFilter.bind( this )
    } );

    this.FormFillControls();
  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidate()
  {
    let numError = 0;
    $( "#ID_LABEL" + this.dynid ).viewable( "validate" );

    if( this.mode === WidgetDeviceFunctionDoormatic.modeSelectedDoors )  {
      $( "#ID_DOORMATIC_LIST" + this.dynid ).listbox( "validate" );
    }
    return numError;
  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidateOff()
  {
    if( this.preventFirstTimeValidate === true ) {
      this.preventFirstTimeValidate = false;
      return;
    }

    $( "#ID_LABEL" + this.dynid ).viewable( "validate", false );
    if( this.deviceMode != "door" ) {
      $( "#ID_DOORMATIC_LIST" + this.dynid ).listbox( "validate", false );
    }
  }

  //---------------------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save( )
  {
    var targets = [];
    var timer;
    var opener = null;
    var isStoreycall = false;
    var isRingOnly = false;

    // read label and return our entity-copy
    if( this.mode === WidgetDeviceFunctionDoormatic.modeSelectedDoors )  {
      targets = this.selectedList;
    }

    if( this.deviceMode === "door" )  {
      timer = parseInt( $( "#ID_DOORMATIC_TIME_DELAY" + this.dynid ).edit( "option", "value" ) );
      opener = $( "#ID_DOORMATIC_OPENER" + this.dynid ).choice( "option", "value" )
    }
    else  {
      timer = parseInt( $( "#ID_DOORMATIC_OPEN_AFTER" + this.dynid ).edit( "option", "value" ) );
      var options = $( "#ID_DOORMATIC_OPTIONS" + this.dynid ).choice( "option", "value" );
      isStoreycall = options['storeycall'];
      isRingOnly = options['ringonly'];
    }


    if( !this.entity['sp'] )  {
      this.entity['sp'] = {};
    }

    this.entity['sp'][ WidgetDeviceFunctionDoormatic.widgetId ] = this.HelperCreateDoormaticSp( timer, targets );
    this.entity['sp'][ WidgetDeviceFunctionDoormatic.widgetId ]['label'] = $( "#ID_LABEL" + this.dynid ).viewable( "option", "value" );
    this.entity['sp'][ WidgetDeviceFunctionDoormatic.widgetId ]['param']['ringonly'] = isRingOnly;
    this.entity['sp'][ WidgetDeviceFunctionDoormatic.widgetId ]['param']['storeycall'] = isStoreycall;
    this.entity['sp'][ WidgetDeviceFunctionDoormatic.widgetId ]['param']['opener'] = opener;

    return this.entity;
  }

  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  FormFillControls(): fill the controls with already set widget values
  //
  FormFillControls()
  {
    this.DeviceMode();
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_SELECTED_DOORS" );

    switch( this.deviceMode ) {
      case "door":
        this.FormFillDoorControls();
        break;
      default:
        this.FormFillApartmentControls();
        break;
    }
  }


  //=============================================================================
  //  DEVICE MODES
  //=============================================================================
  //-----------------------------------------------------------------------------
  //  DeviceMode(): set the device mode based on the entity class and show the correct optionals
  //
  DeviceMode()
  {
    this.deviceMode = this.entity['class'];
    if( this.deviceMode === "door" )  {
      $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_DEVICE_TYPE_DOOR" );
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_DEVICE_TYPE_APARTMENT" );
    }
    else  {
      $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_OPTIONAL_DEVICE_TYPE_DOOR" );
      $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_OPTIONAL_DEVICE_TYPE_APARTMENT" );
    }
  }

  //=============================================================================
  //  MODES
  //=============================================================================

  //=============================================================================
  //  CONTROLS
  //=============================================================================
  //-----------------------------------------------------------------------------
  //  FormFillDoorControls(): initialise door controls
  //
  FormFillDoorControls()
  {
    let timer = 10;

    // previously saved: read from device record
    if( this.deviceRecord['template'] == "doormatic" ) {
      timer = this.entity['sp']['doormatic']['param']['timer'];
    }
    else  {
      this.preventFirstTimeValidate = true;
    }
    $( "#ID_DOORMATIC_TIME_DELAY" + this.dynid ).edit( {
      size: CONST.size.S,
      value: timer,
      filter: { allow: "0123456789" },
      result: ( time ) => {
        $( "#ID_DOORMATIC_TIME_DELAY" + this.dynid ).edit( "option", "value", this.HelperInRange( time, 0, 10, 10 ) );
      }
    } );

    $( "#ID_DOORMATIC_OPENER" + this.dynid ).choice( {
      size: CONST.size.L,
      type: "radio",
      value: this.entity['sp']['doormatic']['param']['opener'] || "opener",
      data: [
        { id: "opener", text: "{$widget_doormatic_opener}" },
        { id: "opener2", text: "{$widget_doormatic_opener2}" }
      ]
    } );
  }

  //-----------------------------------------------------------------------------
  //  FormFillApartmentControls(): initialise apartment controls
  //
  FormFillApartmentControls()
  {
    // defaults (used on create)
    let selected                = [];
    let timer                   = 10;

    // previously saved: read from device record
    if( this.deviceRecord['template'] == "doormatic" ) {
      selected = this.entity['sp']['doormatic']['param']['targets'];
      timer   = this.entity['sp']['doormatic']['param']['timer'];
    }
    else  {
      this.preventFirstTimeValidate = true;
    }

    this.selectedList = selected;

    // initialize controls
    $( "#ID_DOORMATIC_OPEN_AFTER" + this.dynid ).edit( {
      size: CONST.size.S,
      value: timer,
      filter: { allow: "0123456789" },
      result: ( time ) =>
      {
        $( "#ID_DOORMATIC_OPEN_AFTER" + this.dynid ).edit( "option", "value", this.HelperInRange( time, 0, 10, 10 ) );
      }
    } );

    // options checker
    var isStoreycall = ( 'storeycall' in this.entity['sp']['doormatic']['param'] ) ? this.entity['sp']['doormatic']['param']['storeycall'] : false;
    var isRingOnly = ( 'ringonly' in this.entity['sp']['doormatic']['param'] ) ? this.entity['sp']['doormatic']['param']['ringonly'] : true;
    $( "#ID_DOORMATIC_OPTIONS" + this.dynid ).choice( {
      size: CONST.size.L,
      type: "check",
      data: [
        { id: "storeycall", text: "{$widget_doormatic_option_storeycall}", value: isStoreycall },
        { id: "ringonly", text: "{$widget_doormatic_option_online}", value: isRingOnly }
      ]
    } );     

    $( "#ID_DOORMATIC_OPEN_OF" + this.dynid ).choice( {
      size: CONST.size.L,
      value: "all",
      data: [
        { id: "all", text:"{$widget_doormatic_all_doors}" },
        { id: "selected", text:"{$widget_doormatic_selected_doors}" },
      ],
      filter: { allow: "0123456789" },
      result: ( value ) => { 
        value === "all" ? this.ModeAllDoors() : this.ModeSelectedDoors(); 
      }
    } );

    $( "#ID_DOORMATIC_LIST" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "listbox",
      data: this.selectedListDataview,
    } );

    $( "#ID_DOORMATIC_BUTTON_ADD" + this.dynid ).button(  {
      size: CONST.size.XS,
      status: 'normal',
      icon: "mdi mdi-plus",
      click: () => {
        $( '#ID_DOORMATIC_CHOOSE_LIST' + this.dynid ).listbox( "show" );
        //this.Option( "modified", 1 );
      }
    }  );

    $( "#ID_DOORMATIC_BUTTON_DELETE" + this.dynid ).button(  {
      size: CONST.size.XS,
      status: 'normal',
      icon: 'mdi mdi-minus',
      click: () => {
        let selectedDoors = $( "#ID_DOORMATIC_LIST" + this.dynid ).listbox( "option", "value" );
        if( Array.isArray( selected ) && selected.length > 0 ) {
          this.selectedList = this.selectedList.filter( entityId => entityId !== selectedDoors[0] );
          this.selectedListDataview.Update();
        }
      }
    }  );

    $( '#ID_DOORMATIC_CHOOSE_LIST' + this.dynid ).listbox( {
      size: 40,
      height: 5,
      type: 'listbox',
      search: true,
      status: "normal",
      placeholder: "{$const_common_empty_placeholder}",
      searchPlaceholder: "{$const_common_search_placeholder}",
      anchor: "#ID_DOORMATIC_BUTTON_ADD" + this.dynid,
      position: "bottomright",
      data: this.entityDataview,
      result:  ( value ) =>  {
        let selectedDoors = value;
        this.selectedList.push( selectedDoors[0] );
        this.ReloadList( value );
      }
    } );

    //set default modes
    if( this.selectedList && this.selectedList.length > 0 )  {
      this.ModeSelectedDoors();
    }
    else  {
      this.ModeAllDoors();
    }
  }

  ReloadList( selected )
  {
    this.selectedListDataview.Update();
    $( "#ID_DOORMATIC_LIST" + this.dynid ).listbox( "option", "selected", selected );
  }


  //=============================================================================
  //  FILTER
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  ControlEntityDataviewFilter(): description
  //
  ControlEntityDataviewFilter( id, record, options, dataviewClass )
  {
    let isAllowed = true;

    // check for allowed classes
    if( record['class'] !== "door" )  {
      isAllowed = false;
    }

    // already in selected list?
    if( isAllowed ) {
      isAllowed = !( this.selectedList.includes( id ) );
    }

    // check if child of folder "project"
    // if( isAllowed ) {
    //   isAllowed = Data.Instance().IsChild( id, "folder.confdev" );
    // }

    return isAllowed;
  }

  //-----------------------------------------------------------------------------
  //  ControlEntitySelectedDataviewFilter(): description
  //
  ControlEntitySelectedDataviewFilter( id, record, options, dataviewClass )
  {
    let isAllowed = false;

    // check for allowed classes
    if( this.selectedList.includes( id ) )  {
      isAllowed = true;
    }

    return isAllowed;
  }


  //=============================================================================
  //  Mode
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  ModeAllDoors(): description
  //
  ModeAllDoors()
  {
    this.mode = WidgetDeviceFunctionDoormatic.modeAllDoors;
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_SELECTED_DOORS" );
    $( '#ID_DOORMATIC_OPEN_OF' + this.dynid ).choice( "option", "value", "all" );
  }

  //-----------------------------------------------------------------------------
  //  ModeSelectedDoors(): description
  //
  ModeSelectedDoors()
  {
    this.mode = WidgetDeviceFunctionDoormatic.modeSelectedDoors;
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_SELECTED_DOORS" );
    $( '#ID_DOORMATIC_OPEN_OF' + this.dynid ).choice( "option", "value", "selected" );
  }

  //-----------------------------------------------------------------------------
  //  helper
  //-----------------------------------------------------------------------------

  HelperCreateDoormaticSp( timer, targets )
  {
    let statePointName = WidgetDeviceFunctionDoormatic.widgetId;
    return {
      "name": statePointName,
      "address": `${Data.Instance().RootEntid()}/${this.entity['entid']}/${statePointName}`,
      "type": "BOOL",
      "template": statePointName,
      "default": false,
      "label": "{{name}} (" + WidgetDeviceFunctionDoormatic.widgetNameShort + ")",
      "param": {
        "targets": targets,
        "timer": timer,
        "ringonly": true,     // device only
        "storeycall": false   // device only
      }
    };
  }

  //-----------------------------------------------------------------------------
  //  HelperInRange(): helper function to set result values in a range
  //
  HelperInRange( value, min, max, defaultValue )
  {
    if( !value )  {
      return defaultValue;
    }

    if( value > max )  {
      return max;
    }

    if( value < min )  {
      return min;
    }

    return value;

  }

  //=============================================================================
  // FILTERS
  //=============================================================================
  static FilterDoorOpener( id, record, options, dataviewClass )
  {
    let entityId = id.split( '/' )[1];
    let spName   = id.split( '/' )[2];
    let sp       = options.entity['sp']['doormatic'];
    let targets  = sp['param']['targets'] || [];


    // filter out all non door sp's
    if( record['class'] !== "door" || spName.match( /\bdoor\b|\bdoor2\b/ ) === null ) {
      return false;
    }

    // check the permissions
    let isAllowed = Data.Instance().PermissionList().IsAllowed( options.entity['entid'], id, "w" );

    // filter out non selected doors
    if( targets.length > 0 ) {
      if( targets.includes( entityId ) === false ) {
        return false;
      }
    }

    // if the selection list (targets) is empty then get all allowed door openers
    else if( !isAllowed ) {
      return false;
    }

    return true;
  }
}


//=============================================================================
//  REGISTER IN FACTORY
//=============================================================================

WidgetDeviceFunctionDoormatic.Register( WidgetDeviceFunctionDoormatic.template );
WidgetDeviceConfigFactory.Register( WidgetDeviceFunctionDoormatic );
//=============================================================================
//
//  FUNCTION WIDGET BUILDING AUTOMATION URL IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WidgetDeviceFunctionBuildingAutomationUrl extends WidgetDeviceConfigBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template = "/widget/deviceFunctions/building_automation_url";
  static isFunction = true;
  static allowedModules = ["function"];             // module is function..
  static allowedTypes = "*";                        // not used by functions
  static allowedClasses = ["avp",];                 // used by functions
  static functionMustBePresent = false;             // don't show widget of no deviceRecord is present
  static widgetIcon = "mdi mdi-home-automation";
  static widgetNameLong = "{$widget_building_automation_url_long}";
  static widgetNameShort = "{$aui_widget_building_automation_url}";     // label: must be "aui_" (a dict.json macro)
  static widgetId = "building_automation_url";
  static widgetUiHeight = 4;                        // we need 4rem in height

  //===========================================================================
  //  STATIC INFORMATION OVERRIDABLES
  //===========================================================================

  //---------------------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    // switch off function in device record
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, "building_automation_url" );
    if( deviceRecord ) {
      deviceRecord['sp'] = null;
    }

    // remove statepoint
    if( 'building_automation_url' in entity['sp'] )  {
      delete entity['sp']['building_automation_url'];
    }

    // remove permissions
    WidgetDeviceConfigBase.HelperRemovePermission( "building_automation_url", entity );

  }

  //---------------------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = WidgetDeviceFunctionBuildingAutomationUrl.template;
    $element.viewable( createData );
  }

  //---------------------------------------------------------------------------
  //  get used external ressources
  //
  static References( entity )
  {
    return null;
  }

  //---------------------------------------------------------------------------
  //  validate the widget
  //
  static Validate( entity )
  {
    return 0;
  }

  //---------------------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: WidgetDeviceFunctionBuildingAutomationUrl.widgetNameLong,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //===========================================================================
  //  INSTANCE
  //===========================================================================

  //---------------------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    let url = "";
    // copy not needed
    this.entity = createData['entity'];
    if( Array.isArray( this.entity['sp'] ) )  {
      this.entity['sp'] = {};
    }
    this.statepoint = this.entity['sp']['building_automation_url'];

    // get device record create if not exists
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, WidgetDeviceFunctionBuildingAutomationUrl.widgetId );
    if( !this.deviceRecord )  {
      this.deviceRecord = WidgetDeviceConfigBase.HelperAddDeviceFunction( this.entity, WidgetDeviceFunctionBuildingAutomationUrl.widgetId );
    }

    // create default
    if( !( 'building_automation_url' in this.entity['sp'] ) ) {
      this.statepoint = {
        "label": "{{name}} (" + WidgetDeviceFunctionBuildingAutomationUrl.widgetNameShort + ")",
        "address": ( Data.Instance().RootEntid() + "/" + this.entity['entid'] + "/" + WidgetDeviceFunctionBuildingAutomationUrl.widgetId ),
        "type": "BOOL",
        "name": WidgetDeviceFunctionBuildingAutomationUrl.widgetId,
        "persistent": true,
        "template": WidgetDeviceFunctionBuildingAutomationUrl.widgetId,
        "param": {
          "url": url,
        }
      };
      this.entity['sp']['building_automation_url'] = this.statepoint;
      this.deviceRecord['sp']                      = WidgetDeviceFunctionBuildingAutomationUrl.widgetId;
      this.Option( "modified", 1 );
    }
    else {
      url = this.entity['sp'][WidgetDeviceFunctionBuildingAutomationUrl.widgetId]['param']['url'];
    }

    // ensure default permission record exists
    WidgetDeviceConfigBase.HelperAddDefaultPermission( WidgetDeviceFunctionBuildingAutomationUrl.widgetId, this.entity );

    $( "#ID_URL" + this.dynid ).edit( {
      size: CONST.size.L,
      value: url,
    } );

  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidate()
  {
    let numError = 0;
    $( "#ID_URL" + this.dynid ).edit( "validate" );
    return numError;
  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidateOff()
  {
    $( "#ID_URL" + this.dynid ).viewable( "validate", false );
  }

  //---------------------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save( )
  {
    let url = $( "#ID_URL" + this.dynid ).edit( "option", "value" );
    // read label and return our entity-copy
    this.entity['sp'][ WidgetDeviceFunctionBuildingAutomationUrl.widgetId ] = this.HelperCreateBuildingAutomationUrlSp( url );
    return this.entity;
  }

  //-----------------------------------------------------------------------------
  //  helper
  //-----------------------------------------------------------------------------

  HelperCreateBuildingAutomationUrlSp( url )
  {
    let statePointName = WidgetDeviceFunctionBuildingAutomationUrl.widgetId;
    return {
      "name": statePointName,
      "address": `${Data.Instance().RootEntid()}/${this.entity['entid']}/${statePointName}`,
      "type": "BOOL",
      "template": statePointName,
      "persistent": false,
      "hidden": true,
      "default": false,
      "label": "{{name}} (" + WidgetDeviceFunctionBuildingAutomationUrl.widgetNameShort + ")",
      "param": {
        "url": url,
      }
    };
  }
  

}


//=============================================================================
//  REGISTER IN FACTORY
//=============================================================================

WidgetDeviceFunctionBuildingAutomationUrl.Register( WidgetDeviceFunctionBuildingAutomationUrl.template );
WidgetDeviceConfigFactory.Register( WidgetDeviceFunctionBuildingAutomationUrl );
//=============================================================================
//
//  FUNCTION WIDGET VIDEOURL IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WidgetDeviceFunctionVideoUrl extends WidgetDeviceConfigBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template = "/widget/deviceFunctions/videourl";
  static isFunction = true;
  static allowedModules = ["function"];             // module is function..
  static allowedTypes = "*";                        // not used by functions
  static allowedClasses = ["tkphone", "sipaudio" ];    // used by functions
  static allowedOnEntrance = false;
  static functionMustBePresent = false;             // don't show widget of no deviceRecord is present
  static widgetIcon = "mdi mdi-camcorder";
  static widgetNameLong = "{$widget_videourl_long}";
  static widgetNameShort = "{$aui_widget_video}";     // label: must be "aui_" (a dict.json macro)
  static widgetId = "videourl";
  static widgetUiHeight = 20;                        // we need 4rem in height
  static modeMjpeg = "MJPEG";
  static modeSingleShot = "singleShot";
  static modeSecure = "secure";
  static modeUnSecure = "unsecure";
  static modePasswordChange = "passwordChange";

  //===========================================================================
  //  STATIC INFORMATION OVERRIDABLES
  //===========================================================================

  //-----------------------------------------------------------------
  //  special: check if this widget is allowed in this entity
  //
  static IsUseable( entity, isFunction, isLed )
  {
    // this widget is for function
    if( !isFunction ) {
      return false;
    }

    // check class list
    let isClassAllowed = ( WidgetDeviceFunctionVideoUrl.allowedClasses.indexOf( entity['class'] ) >= 0 );
  
    if( 'entrance' in entity ) {
      return false;
    }

    return isClassAllowed;
  }

  //---------------------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    // switch off function in device record
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, "videourl" );
    if( deviceRecord ) {
      deviceRecord['sp'] = null;
    }

    // remove statepoint
    if( 'videourl' in entity['sp'] )  {
      delete entity['sp']['videourl'];
    }

    //prepare video settings of the device
    //video settings
    if( 'video' in entity )  {
      delete entity['video']['http'];
      delete entity['video']['httpGuid'];
      delete entity['video']['httpFormat'];
      delete entity['video']['httpAuth'];
      delete entity['video']['httpUser'];
      delete entity['video']['httpFps'];
    }

    // remove permissions
    WidgetDeviceConfigBase.HelperRemovePermission( "videourl", entity );

  }

  //---------------------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = WidgetDeviceFunctionVideoUrl.template;
    $element.viewable( createData );
  }

  //---------------------------------------------------------------------------
  //  get used external ressources
  //
  static References( entity )
  {
    return null;
  }

  //---------------------------------------------------------------------------
  //  validate the widget
  //
  static Validate( entity )
  {
    return 0;
  }

  //---------------------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: WidgetDeviceFunctionVideoUrl.widgetNameLong,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //===========================================================================
  //  INSTANCE
  //===========================================================================

  //---------------------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // copy not needed
    this.entity = createData['entity'];
    this.statepoint = this.entity['sp']['videourl'];
    if( Array.isArray( this.entity['sp'] ) )  {
      this.entity['sp'] = {};
    }


    // get device record create if not exists
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, WidgetDeviceFunctionVideoUrl.widgetId );
    if( !this.deviceRecord )  {
      this.deviceRecord = WidgetDeviceConfigBase.HelperAddDeviceFunction( this.entity, WidgetDeviceFunctionVideoUrl.widgetId );
    }

    // create default
    if( !( 'videourl' in this.entity['sp'] ) ) {
      this.statepoint = {
        "label": "{{name}} (" + WidgetDeviceFunctionVideoUrl.widgetNameShort + ")",
        "address": ( Data.Instance().RootEntid() + "/" + this.entity['entid'] + "/" + WidgetDeviceFunctionVideoUrl.widgetId ),
        "type": "BOOL",
        "name": WidgetDeviceFunctionVideoUrl.widgetId,
        "template": WidgetDeviceFunctionVideoUrl.widgetId,
        "hidden": true,
      };
      this.entity['sp']['videourl'] = this.statepoint;
      this.deviceRecord['sp'] = WidgetDeviceFunctionVideoUrl.widgetId;

      //prepare video settings of the device
      //video settings
      this.entity['video']['http']       = true;
      this.entity['video']['httpGuid']   = this.entity['video']['httpGuid'] ?? $.guidgen();
      this.entity['video']['httpFormat'] = "mjpeg";
      this.entity['video']['httpAuth']   = false;
      this.entity['video']['httpUser']   = "root";
      this.entity['video']['httpFps']    = 12;

      this.Option( "modified", 1 );
    }

    // correct missing values (where server uses default)
    if( !( 'httpFps' in this.entity['video'])
     || !( this.entity['video']['httpFps'] ) 
      ) {
      this.entity['video']['httpFps'] = 12;
    }

    // ensure default permission record exists
    WidgetDeviceConfigBase.HelperAddDefaultPermission( WidgetDeviceFunctionVideoUrl.widgetId, this.entity );


    // build label
    /*ControlLabel.Build( $( "#ID_LABEL" + this.dynid ), {
      label:        this.statepoint['label'],
      entityName:   this.entity['name'],
      functionName: WidgetDeviceFunctionVideoUrl.widgetNameShort
    } );*/

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );

    Data.Instance().On( "COSCONF.READ", this.etag, this.OnNetworkList.bind( this ) );
    Data.Instance().XhrRequest( "COSCONF.READ" );

    this.FormFillControls();
  }

  //=============================================================================
  //  Validate
  //=============================================================================
  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidate()
  {
    let numError = 0;
    //$( "#ID_LABEL" + this.dynid ).viewable( "validate" );
    if( this.passwordMode && this.passwordMode === WidgetDeviceFunctionVideoUrl.modePasswordChange )  {
      $( "ID_PASSWORD_REPEAT" ).edit( "validate", false );
    }
    return numError;
  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidateOff()
  {
    //$( "#ID_LABEL" + this.dynid ).viewable( "validate", false );
    if( this.passwordMode && this.passwordMode === WidgetDeviceFunctionVideoUrl.modePasswordChange )  {
      $( "ID_PASSWORD_REPEAT" ).edit( "validate", false );
    }
  }

  //-----------------------------------------------------------------
  //  validate password repeat
  //
  ValidatePassword( validateControl )
  {
    let password1 = $( "#ID_PASSWORD" + this.dynid ).edit( "option", "value" );
    let password2 = $( "#ID_PASSWORD_REPEAT" + this.dynid ).edit( "option", "value" );

    if( password1.length == 0 )  {
      validateControl.error = "{$change_password_error_empty}";
      return 1;
    }

    if( password1 != password2 )  {
      validateControl.error = "{$change_password_error_different}";
      return 1;
    }

    return 0;
  }

  //---------------------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save( )
  {
    let password = $( "#ID_PASSWORD" + this.dynid ).edit( "option", "value" );
    let format   = $( "#ID_FORMAT" + this.dynid ).choice( "option", "value" );
    let isAuth   = $( "#ID_SECURITY" + this.dynid ).choice( "option", "value" );
    let user     = $( "#ID_PARTICIPANT" + this.dynid ).edit( "option", "value" );
    let value = $( "#ID_FRAMES" + this.dynid ).edit( "option", "selected" );
    let fps      = parseInt( value[ 0 ] ) ?? 12;


    // read label and return our entity-copy
    //this.entity['sp']['videourl']['label'] = $( "#ID_LABEL" + this.dynid ).viewable( "option", "value" );

    //video settings
    this.entity['video']['http']       = true;// TODO what is the use of this?
    this.entity['video']['httpGuid']   = this.entity['video']['httpGuid'];
    this.entity['video']['httpFormat'] = format;
    this.entity['video']['httpAuth']   = isAuth;
    this.entity['video']['httpUser']   = user;
    this.entity['video']['httpFps']    = fps;


    if( this.passwordMode && this.passwordMode ===  WidgetDeviceFunctionVideoUrl.modePasswordChange )  {
      Data.Instance().SetEntityPassword( this.entity['entid'], password, "videoUrl" );
    }

    return this.entity;
  }

  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------------------
  //  FormFillControls(): fill the controls with already set widget values
  //
  FormFillControls()
  {
    //choice
    $( "#ID_FORMAT" + this.dynid ).choice( {
        data: [
          { text: "{$widget_video_url_jpeg}", id: "jpeg" },
          { text: "{$widget_video_url_mjpeg}", id: "mjpeg" },
        ],
        value: this.entity['video']['httpFormat'],
        result: this.ControlFormatResult.bind( this ),
      } );

    $( "#ID_SECURITY" + this.dynid ).choice( {
      type: "check",
      text: "{$widget_video_url_security_description}",
      value: this.entity['video']['httpAuth'],
      result: this.ControlSecurityResult.bind( this ),
    } );

    // correct bugs of older admins (AP-1825)
    if( parseInt( this.entity['video']['httpFps'] ) == 0 )  {
      this.entity['video']['httpFps'] = 12;
    }

    //listbox
    $( "#ID_FRAMES" + this.dynid ).listbox( {
        type: "droplist",
        size: CONST.size.S,
        data: this.HelperCreateFpsList(),
        selected: [ this.entity['video']['httpFps'] ],
      }
    );

    //edit
    $( "#ID_PARTICIPANT" + this.dynid ).edit( {
      size: CONST.size.L,
      value: this.entity['video']['httpUser'],
      result: ( time ) => {}
    } );

    $( "#ID_PASSWORD" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly",
      password: true,
      value: "******",//TODO: set default value
      validateDepends: [ "ID_PASSWORD_REPEAT" + this.dynid ],
      result: () => {}
    } );

    $( "#ID_PASSWORD_REPEAT" + this.dynid ).edit( {
      size: CONST.size.L,
      status: "readonly",
      password: true,
      value: "******",//TODO: set default value
      validateDepends: [ "ID_PASSWORD" + this.dynid ],
      validate: this.ValidatePassword.bind( this ),
      result: () => {}
    } );

    // button
    $( "#ID_PASSWORD_CHANGE" + this.dynid ).button( {
      icon: "mdi mdi-pencil",
      click: () => { this.ModePasswordChange(); },
    } );

    //url control is defined after the cosconf network list is arrived

    //optionals
    if( this.entity['video']['httpFormat'] === "mjpeg" )  {
      this.ModeMJPEG();
    }
    else  {
      this.ModeSingleShot();
    }

    if( this.entity['video']['httpAuth'] )  {
      this.ModeSecure();
    }
    else  {
      this.ModeUnsecure();
    }
  }
  
  //=============================================================================
  //  HELPER
  //=============================================================================


  //-----------------------------------------------------------------------------
  //  HelperCreateFpsList(): generate fps list data
  //

  HelperCreateFpsList()
  {
    let fps = [];

    for( let index = 1; index <= 12; index++ )  {
      fps.push( { id: index, text: index } );
    }

    return fps;
  }

  //=============================================================================
  //  CONTROL RESULTS
  //=============================================================================
  //-----------------------------------------------------------------------------
  //  ControlFormatResult(): description
  //
  ControlFormatResult( value )
  {
    if( value === "mjpeg" )  {
      this.ModeMJPEG();
    }
    else  {
      this.ModeSingleShot();
    }
  }
  
  //-----------------------------------------------------------------------------
  //  ControlSecurityResult(): description
  //
  ControlSecurityResult( value )
  {
    if( value === true )  {
      this.ModeSecure();
    }
    else {
      this.ModeUnsecure();
    }
  }

  //=============================================================================
  //  MODES
  //=============================================================================

  //-----------------------------------------------------------------------------
  //  ModeMJPEG(): description
  //
  ModeMJPEG()
  {
    this.mode = WidgetDeviceFunctionVideoUrl.modeMjpeg;
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_VIDEO_URL_FRAMES" );
    this.entity['video']['httpFormat'] = "mjpeg";
    ControlVideoUrlAddress.Build( $( "#ID_URL" + this.dynid ), {
      data: this.ipPoolV4,
      entity: this.entity,
    } );
  }

  //-----------------------------------------------------------------------------
  //  ModeSingleShot(): description
  //
  ModeSingleShot()
  {
    this.mode = WidgetDeviceFunctionVideoUrl.modeSingleShot;
    this.entity['video']['httpFormat'] = "jpeg";
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_VIDEO_URL_FRAMES" );
    ControlVideoUrlAddress.Build( $( "#ID_URL" + this.dynid ), {
      data: this.ipPoolV4,
      entity: this.entity,
    } );
  }

  //-----------------------------------------------------------------------------
  //  ModeSecure(): description
  //
  ModeSecure()
  {
    this.mode = WidgetDeviceFunctionVideoUrl.modeSecure;
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_VIDEO_URL_BASIC_AUTH" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_VIDEO_URL_PASSWORD_REPEAT" );
  }

  //-----------------------------------------------------------------------------
  //  ModeUnsecure(): description
  //
  ModeUnsecure()
  {
    this.mode = WidgetDeviceFunctionVideoUrl.modeUnSecure;
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_VIDEO_URL_BASIC_AUTH" );
    $( '#ID_OPTIONAL' + this.dynid ).optional( "hide", "ID_OPTIONAL_VIDEO_URL_PASSWORD_REPEAT" );
  }

  //-----------------------------------------------------------------------------
  //  ModePasswordChange(): description
  //
  ModePasswordChange()
  {
    this.passwordMode = WidgetDeviceFunctionVideoUrl.modePasswordChange;
    $( '#ID_OPTIONAL' + this.dynid ).optional( "show", "ID_OPTIONAL_VIDEO_URL_PASSWORD_REPEAT" );

    // enable controls
    $( "#ID_PASSWORD" + this.dynid ).edit( "option", "status", "normal" );
    $( "#ID_PASSWORD_REPEAT" + this.dynid ).edit( "option", "status", "normal" );
  }

  //=============================================================================
  //  EVENTS
  //=============================================================================
  OnNetworkList( response )
  {
    if( $.isPlainObject( response['data'] ) ) {
      let ipPoolv4     = [];
      let ipPoolv6     = [];
      // copy data
      this.networkList = $.clone( response['data'] );

      for( let macAddress in this.networkList['interfaces'] ) {
        let ipv4 = this.networkList['interfaces'][macAddress]['ipv4'];
        let ipv6 = this.networkList['interfaces'][macAddress]['ipv6'];
        ipPoolv4.push( ipv4 );
        ipPoolv6.push( ipv6 );
      }

      this.ipPoolV4  = ipPoolv4;
      ControlVideoUrlAddress.Build( $( "#ID_URL" + this.dynid ), {
        data: this.ipPoolV4,
        entity: this.entity,
      } );

    }
  }
}


//=============================================================================
//  REGISTER IN FACTORY
//=============================================================================

WidgetDeviceFunctionVideoUrl.Register( WidgetDeviceFunctionVideoUrl.template );
WidgetDeviceConfigFactory.Register( WidgetDeviceFunctionVideoUrl );
//=============================================================================
//
//  FUNCTION WIDGET LIGHT IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WidgetDeviceFunctionLight extends WidgetDeviceConfigBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template              = "/widget/deviceFunctions/light";
  static isFunction            = true;
  static allowedModules        = ["function"];             // module is function..
  static allowedTypes          = "*";                        // not used by functions
  static allowedClasses        = ["door"];    // used by functions
  static functionMustBePresent = false;             // don't show widget of no deviceRecord is present
  static widgetIcon            = "mdi mdi-lightbulb-outline";
  static widgetNameLong        = "{$widget_light_long}";
  static widgetNameShort       = "{$aui_widget_light}";     // label: must be "aui_" (a dict.json macro)
  static widgetId              = "light";
  static widgetUiHeight        = 17;                        // we need 4rem in height
  static modeLocal             = "local";
  static modeExtern            = "extern";

  //===========================================================================
  //  STATIC INFORMATION OVERRIDABLES
  //===========================================================================


  //-----------------------------------------------------------------
  //  special: check if this widget is allowed in this entity
  //
  static IsUseable( entity, isFunction, isLed )
  {
    // this widget is for function
    if( !isFunction ) {
      return false;
    }

    if( ( 'entrance' in entity )
     && ( 'enabled' in entity['entrance'] )
     && ( entity['entrance']['enabled'] == true )
      ) {
      return true;
    }

    return false;
  }

  //-----------------------------------------------------------------
  //  local statepoint for permission check
  //
  static LocalStatepointName( uid, trigger, entity )
  {
    return "light";
  }

  //---------------------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    // switch off function in device record
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, "light" );

    if( deviceRecord ) {
      deviceRecord['sp'] = null;
    }

    // remove statepoint
    if( 'light' in entity['sp'] ) {
      delete entity['sp']['light'];
    }
    WidgetDeviceConfigBase.HelperRemoveStatepoint( "light", entity );
    WidgetDeviceFunctionLight.HelperRemoveLocalOutputConfig( entity );

    // entrance
    if( ( 'entrance' in entity ) && ( 'light' in entity['entrance'] ) ) {
      delete entity['entrance']['light'];
    }

    // remove permissions
    WidgetDeviceConfigBase.HelperRemovePermission( "light", entity );

  }

  //---------------------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = WidgetDeviceFunctionLight.template;
    $element.viewable( createData );
  }

  //---------------------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    let reference = [];
    let output = "altc_a3";

    if( ( 'sp' in entity ) && ( 'light' in entity['sp'] ) ) {
      reference.push(`sp:${Data.Instance().RootEntid()}/${entity['entid']}/light`);
      output = entity.sp['light']['param']['output'];
    }

    if( output.indexOf( "/" ) !== -1 )  {
      reference.push( `sp:${output}` );
    }
    else  {
      reference.push( `sp:${Data.Instance().RootEntid()}/${entity['entid']}/${output}` );
    }

    if( reference.length > 0 )  {
      return reference;
    }

    return null;
  }

  //---------------------------------------------------------------------------
  //  validate the widget
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    if( ! ( 'light' in entity.sp ) )  {
      return 0;
    }

    if( !( 'param' in entity.sp['light'] ) || !( 'output' in entity.sp['light']['param'] ) ) {
      validateControl.error = "{$const_error_statepoint_deleted}";
      return 1;
    }

    let spOutput = entity.sp['light']['param']['output'];

    if( spOutput.indexOf( "/" ) >= 0 ) {

      // error no permission
      let spInfo = Data.Instance().StatepointInfo( spOutput, entity );
      if( spInfo['type'] === "ERROR" )  {
        return 1;
      }

      // error no permission
      if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spOutput, "w" ) ) {
        validateControl.error = "{$const_error_statepoint_permission}";
        return 1;
      }
    }

  }

  //---------------------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: WidgetDeviceFunctionLight.widgetNameLong,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //===========================================================================
  //  INSTANCE
  //===========================================================================

  //---------------------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // copy not needed
    this.entity     = createData['entity'];
    this.statepoint = this.entity['sp']['light'];
    if( Array.isArray( this.entity['sp'] ) )  {
      this.entity['sp'] = {};
    }
    this.isNew      = false;
    this.isAtlc     = this.entity['class'] == "door";


    // get device record create if not exists
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, WidgetDeviceFunctionLight.widgetId );
    if( !this.deviceRecord ) {
      this.deviceRecord = WidgetDeviceConfigBase.HelperAddDeviceFunction( this.entity, WidgetDeviceFunctionLight.widgetId );
    }

    if( this.deviceRecord && !this.deviceRecord['sp'] )  {
      this.deviceRecord['sp'] = WidgetDeviceFunctionLight.widgetId;
    }

    // create default
    if( !( 'light' in this.entity['sp'] ) ) {
      this.statepoint = {
        "label": "{{name}} (" + WidgetDeviceFunctionLight.widgetNameShort + ")",
        "address": ( Data.Instance().RootEntid() + "/" + this.entity['entid'] + "/" + WidgetDeviceFunctionLight.widgetId ),
        "type": "BOOL",
        "name": WidgetDeviceFunctionLight.widgetId,
        "template": WidgetDeviceFunctionLight.widgetId,
        "signaltime": 60000,
        "retrigger": true,
        "invert": false,
        "resdef": [{
          "id": 3,
          "action": "switch",
          "periods": 1
        }],
        "param": {
          "output": this.isAtlc ? "atlc_a3" : null,
        }
      };
      this.isNew = true;
      this.entity['sp']['light'] = this.statepoint;
      this.deviceRecord['sp']    = WidgetDeviceFunctionLight.widgetId;
      this.Option( "modified", 1 );
    }

    // correct false entitiy
    if( !( 'param' in this.entity['sp']['light'] ) ) {
      this.entity['sp']['light']['param'] = {
        "output": ""
      };
    }    

    // correct use of dict-macros
    if( 'label' in this.entity['sp']['light'] ) {
      this.entity['sp']['light']['label'] = ( this.entity['sp']['light']['label'] ).replace( "{$aui_widget_lightcontact}", WidgetDeviceFunctionLight.widgetNameShort );
    }    

    // ensure default permission record exists
    WidgetDeviceConfigBase.HelperAddDefaultPermission( WidgetDeviceFunctionLight.widgetId, this.entity );

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );

    // build label
    ControlLabel.Build( $( "#ID_LABEL" + this.dynid ), {
      label: this.statepoint['label'],
      entityName: this.entity['name'],
      functionName: WidgetDeviceFunctionLight.widgetNameShort
    } );

    this.FormFillControls();
  }

  //=============================================================================
  //  VALIDATION
  //=============================================================================

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidate()
  {
    // prevent 1st validate wile create
    if( this.isNew )  {
      this.isNew = false;
      return 0;
    }

    let numError = 0;
    $( "#ID_LABEL" + this.dynid ).viewable( "validate" );

    if( this.mode === WidgetDeviceFunctionLight.modeExtern )  {
      numError += $( "#ID_LIGHT_OUTPUT_EXTERN" + this.dynid ).viewable( "validate" );
    }
    else  {
      numError += $( "#ID_LIGHT_OUTPUT_LOCAL" + this.dynid ).listbox( "validate" );
    }

    return numError;
  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidateOff()
  {
    $( "#ID_LABEL" + this.dynid ).viewable( "validate", false );
    $( "#ID_LIGHT_OUTPUT_EXTERN" + this.dynid ).viewable( "validate", false );
    $( "#ID_LIGHT_OUTPUT_LOCAL" + this.dynid ).listbox( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  name(): description
  //
  ValidateEmpty( validateControl )
  {
    let selected = $( "#ID_LIGHT_OUTPUT_LOCAL" + this.dynid ).listbox( "option", "selected" );
    if( Array.isArray( selected ) && selected.length === 0 )  {
      validateControl.error = "{$widget_light_error_is_empty}";
      return 1;
    }
    return 0;
  }

  //---------------------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    let time;
    let statepoint;
    let isPostTriggerSet = $( "#ID_LIGHT_POST_TRIGGER" + this.dynid ).choice( "option", "value" );
    let isInverted = $( "#ID_LIGHT_INVERT" + this.dynid ).choice( "option", "value" );
    WidgetDeviceFunctionLight.HelperRemoveLocalOutputConfig( this.entity );

    let options = $( "#ID_LIGHT_OPTIONS" + this.dynid ).choice( "option", "value" );

    if( this.mode === WidgetDeviceFunctionLight.modeLocal ) {
      statepoint = $( "#ID_LIGHT_OUTPUT_LOCAL" + this.dynid ).listbox( "option", "selected" )[0];
      time       = $( "#ID_LIGHT_TIMEOUT" + this.dynid ).listbox( "option", "selected" )[0];
      this.HelperSetDeviceConfig();
      if( this.entity['sp']['light']['alias'] ) {
        delete this.entity['sp']['light']['alias'];
      }
    }
    else {
      statepoint = $( "#ID_LIGHT_OUTPUT_EXTERN" + this.dynid ).viewable( "option", "selected" )[0];
      if( !this.entity['sp']['light']['alias'] )  {
        this.entity['sp']['light']['alias'] = {
          target: statepoint
        };
      }
      else {
        this.entity['sp']['light']['alias']['target'] = statepoint;
      }
    }
    // read label and return our entity-copy
    this.entity['sp']['light']['label'] = $( "#ID_LABEL" + this.dynid ).viewable( "option", "value" );
    this.entity['sp']['light']['retrigger'] = options['posttrigger'];
    this.entity['sp']['light']['signaltime'] = parseInt( time );
    this.entity['sp']['light']['param']['output'] = statepoint;

    if( options['invert'] === false )  {
      delete this.entity['sp']['light']['invert'];
    }
    else {
      this.entity['sp']['light']['invert'] = options['invert'];
    }

    if( this.mode == WidgetDeviceFunctionLight.modeExtern )  {
      delete this.entity['sp']['light']['resdef'];
      delete this.entity['sp']['light']['signaltime'];
      delete this.entity['sp']['light']['retrigger'];
    }

    // entrance
    if( !( 'entrance' in this.entity ) ) {
      this.entity['entrance'] = {};
    }
    this.entity['entrance']['light'] = this.entity['sp']['light']['address'];
    
    
    return this.entity;
  }


  //-----------------------------------------------------------------------------
  //  FillControls(): description
  //
  FormFillControls()
  {
    let isLocalOutput = this.entity['sp']['light']['param']['output'] ? ( this.entity['sp']['light']['param']['output'].indexOf( "/" ) === -1 ) : false;
    let isInverted   = false;
    let isPosttrigger = true;
    let options = [
      { id: "invert",      text: "{$widget_light_invert_check}",           value: false },
      { id: "posttrigger", text: "{$widget_light_trigger_check}",          value: true },
    ];

    let timeout  = 60000;

    let localDeviceConfigRecord;
    let localOutput;

    if( this.deviceRecord['template'] == "light" ) {
      //isLocalOutput   = this.entity['sp'][ WidgetDeviceFunctionLight.widgetId ]['param']['output'].indexOf( "/" ) === -1;
      isPosttrigger   = this.entity['sp'][ WidgetDeviceFunctionLight.widgetId ]['retrigger'];

      timeout         = this.entity['sp'][ WidgetDeviceFunctionLight.widgetId ]['signaltime'];
      localOutput     = this.entity['sp'][ WidgetDeviceFunctionLight.widgetId ]['param']['output'];

      if( isLocalOutput ) {
        let local = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, localOutput );
        if( local && local['param'] )  {
          isInverted = local['param']['invert'] ?? false;
        }
      }

      options = [
        { id: "invert",      text: "{$widget_door2_invert_check}",  value: isInverted },
        { id: "posttrigger", text: "{$widget_door2_trigger_check}", value: isPosttrigger },
      ];
    }
    else  {
      this.isNew = true;
    }

    // prevent local output on non atlc
    if( ! this.isAtlc ) {
      isLocalOutput = false;
    }


    // hint: since we've got the old UI pattern "internal/external" we 
    // must not add local statepoints here.
    ControlStatepointList.Build( $( "#ID_LIGHT_OUTPUT_EXTERN" + this.dynid ), {
      size: CONST.size.L,
      inquirer: this.entity['entid'],
      forbiddenEntityId: this.entity['entid'],
      allowedTypes: ["BOOL"]
    } );

    $( "#ID_LIGHT_OUTPUT" + this.dynid ).choice( {
      size: CONST.size.L,
      data: [
        { id: WidgetDeviceFunctionLight.modeLocal, text: "{$widget_light_local_outputs}", status: this.isAtlc ? "normal" : "disabled" },
        { id: WidgetDeviceFunctionLight.modeExtern, text:"{$widget_light_extern_outputs}"},
      ],
      value: isLocalOutput ? WidgetDeviceFunctionLight.modeLocal : WidgetDeviceFunctionLight.modeExtern,
      result: ( value ) => {  
        this.ControlOutputResult( value === WidgetDeviceFunctionLight.modeLocal );
      }
    } );

    $( "#ID_LIGHT_OUTPUT_LOCAL" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "droplist",
      search: true,
      selected: [], // this.isNew ? [ localOutput ] : [],
      validate: this.ValidateEmpty.bind(this),
    } );

    $( "#ID_LIGHT_TIMEOUT" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "droplist",
      selected: [`${timeout}`],
      data: CONST.switchTime,
      validate: this.ValidateEmpty.bind(this),
    } );

    $( "#ID_LIGHT_OPTIONS" + this.dynid ).choice( {
      size: CONST.size.L,
      type: "check",
      data: options,
    } );

    this.HelperLocalOutputs();

    if( isLocalOutput )  {
      this.ModeLocalOutput();
      $( "#ID_LIGHT_OUTPUT_LOCAL" + this.dynid ).listbox( "option", "selected", [ this.entity['sp']['light']['param']['output'] ] );
    }
    else  {
      this.ModeExternalOutput();
      if( this.entity['sp']['light']['param']['output'] ) {
        $( "#ID_LIGHT_OUTPUT_EXTERN" + this.dynid ).viewable( "option", "selected", [this.entity['sp']['light']['param']['output']] );
      }
    }
  }

  //-----------------------------------------------------------------------------
  //  ValidateTime(): description
  //
  ValidateTime( data )
  {
    let time = parseInt( $( "#ID_LIGHT_TIMEOUT" + this.dynid ).edit( "option", "value" ) );
    if( isNaN( time ) || time < -1 ) {
      data.error = "{$widget_set_redir_validate_time_error}";
      return 1;
    }
    return 0;
  }

  //=============================================================================
  //  HELPER
  //=============================================================================
  //-----------------------------------------------------------------------------
  //  HelperSetDeviceConfig(): write the light configuration to the correct output
  //                           on the same the device. e.g. if altce1_a2 is selected
  //                           then the function writes the configuration in the
  //                           device section with the uid of atlce1_2
  //
  HelperSetDeviceConfig()
  {
    let statepointAddress;
    if( this.mode === WidgetDeviceFunctionLight.modeLocal )  {
      statepointAddress   = $( "#ID_LIGHT_OUTPUT_LOCAL" + this.dynid ).listbox( "option", "selected" )[0];
    }
    else {
      return;
    }

    let uid              = statepointAddress;
    let deviceConfig     = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, uid, null );
    let time             = $( "#ID_LIGHT_TIMEOUT" + this.dynid ).listbox( "option", "selected" )[0];
    let isInvertSet      = $( "#ID_LIGHT_INVERT" + this.dynid ).choice( "option", "value" );
    let isPostTriggerSet = $( "#ID_LIGHT_POST_TRIGGER" + this.dynid ).choice( "option", "value" );
    let sysid            = deviceConfig['out']['sysid'];
    let options = $( "#ID_LIGHT_OPTIONS" + this.dynid ).choice( "option", "value" );

    //clean up old param
    deviceConfig['param']       = {};
    deviceConfig['sp']          = WidgetDeviceFunctionLight.widgetId;
    deviceConfig['template']    = 'lightcontact';
    deviceConfig['param']['time'] = parseInt( time );
    deviceConfig['param']['invert'] = options['invert'];
    deviceConfig['param']['posttrigger'] = options['posttrigger'];

    this.entity['sp']['light']['resdef'][0]['id'] = parseInt( sysid );

    this.entity['sp']['light']['signaltime'] = parseInt( time );
  }
  
  //-----------------------------------------------------------------------------
  //  HelperLocalOutputs(): description
  //
  HelperLocalOutputs()
  {
    let outputs = [];
    let validTemlates = ['light', 'none'];

    for( let deviceConfigRecord of this.entity['deviceConfig'] )  {
      if( "out" in deviceConfigRecord
        && ( deviceConfigRecord['sp'] == "light" || deviceConfigRecord['template'] === "none" ) )  {
        outputs.push( { 
          id: deviceConfigRecord['uid'], 
          text: `{$widget_light_output} ${deviceConfigRecord['abbr']}`, 
          icon: "mdi mdi-key" 
        } );
      }
    }

    $( "#ID_LIGHT_OUTPUT_LOCAL" + this.dynid ).listbox( "option", "data", outputs );
  }

  //-----------------------------------------------------------------------------
  //  HelperRemoveLocalOutputConfig(): description
  //
  static HelperRemoveLocalOutputConfig( entity )
  {
    for( let deviceConfigRecord of entity['deviceConfig'] )  {
      if( "out" in deviceConfigRecord
        && ( deviceConfigRecord['sp'] == "light" || deviceConfigRecord['template'] === "lightcontact" ) )  {
        deviceConfigRecord['template'] = "none";
        delete deviceConfigRecord['sp'];
        delete deviceConfigRecord['param'];
      }
    }
  }

  //-----------------------------------------------------------------------------
  //  ControlOutputResult(): description
  //
  ControlOutputResult( isLocal )
  {
    if( isLocal )  {
      this.ModeLocalOutput();
    }
    else  {
      this.ModeExternalOutput();
    }
  }

  //=============================================================================
  //  MODES
  //=============================================================================
  //-----------------------------------------------------------------------------
  //  ModeLocalOutput(): description
  //
  ModeLocalOutput()
  {
    this.mode = WidgetDeviceFunctionLight.modeLocal;
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_LIGHT_OUTPUT_EXTERN" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_LIGHT_OUTPUT_LOCAL" );

    //validateoff
    $( "#ID_LIGHT_OUTPUT_EXTERN" + this.dynid ).viewable( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  ModeExternalOutput(): description
  //
  ModeExternalOutput()
  {
    this.mode = WidgetDeviceFunctionLight.modeExtern;
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_LIGHT_OUTPUT_LOCAL" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_LIGHT_OUTPUT_EXTERN" );

    //validateoff
    $( '#ID_LIGHT_OUTPUT_LOCAL' + this.dynid ).listbox( "validate", false );
  }
}

//=============================================================================
//  REGISTER IN FACTORY
//=============================================================================

WidgetDeviceFunctionLight.Register( WidgetDeviceFunctionLight.template );
WidgetDeviceConfigFactory.Register( WidgetDeviceFunctionLight );
//=============================================================================
//
//  FUNCTION WIDGET DOOR IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WidgetDeviceFunctionDoor extends WidgetDeviceConfigBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template              = "/widget/deviceFunctions/door";
  static isFunction            = true;
  static allowedModules        = ["function"];             // module is function..
  static allowedTypes          = "*";                        // not used by functions
  static allowedClasses        = ["door"];    // used by functions
  static allowedOnEntrance     = true;
  static functionMustBePresent = false;             // don't show widget of no deviceRecord is present
  static widgetIcon            = "mdi mdi-key-plus";
  static widgetNameLong        = "{$widget_door_long}";
  static widgetNameShort       = "{$aui_widget_door}";     // label: must be "aui_" (a dict.json macro)
  static widgetId              = "door";
  static widgetUiHeight        = 20;                        // we need 4rem in height
  static modeLocal             = "local";
  static modeExtern            = "extern";

  //===========================================================================
  //  STATIC INFORMATION OVERRIDABLES
  //===========================================================================

  //-----------------------------------------------------------------
  //  special: check if this widget is allowed in this entity
  //
  static IsUseable( entity, isFunction, isLed )
  {
    // this widget is for function
    if( !isFunction ) {
      return false;
    }

    if( ( 'entrance' in entity )
     && ( 'enabled' in entity['entrance'] )
     && ( entity['entrance']['enabled'] == true )
      ) {
      return true;
    }

    return false;
  }

  //-----------------------------------------------------------------
  //  local statepoint for permission check
  //
  static LocalStatepointName( uid, trigger, entity )
  {
    return "door";
  }

  //---------------------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    // switch off function in device record
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, "door" );

    if( deviceRecord ) {
      deviceRecord['sp'] = null;
    }

    // remove statepoint
    if( 'door' in entity['sp'] ) {
      delete entity['sp']['door'];
    }
    WidgetDeviceConfigBase.HelperRemoveStatepoint( "door", entity );
    WidgetDeviceFunctionDoor.HelperRemoveLocalOutputConfig( entity );

    // entrance
    if( ( 'entrance' in entity ) && ( 'opener' in entity['entrance'] ) ) {
      delete entity['entrance']['opener'];
    }

    // remove permissions
    WidgetDeviceConfigBase.HelperRemovePermission( "door", entity );

  }

  //---------------------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = WidgetDeviceFunctionDoor.template;
    $element.viewable( createData );
  }

  //---------------------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    let reference = [];
    let output;

    if( ( 'sp' in entity ) && ( 'door' in entity['sp'] ) ) {
      reference.push(`sp:${Data.Instance().RootEntid()}/${entity['entid']}/door`);
      output = entity.sp['door']['param']['output'];
    }

    if( output && output.indexOf( "/" ) !== -1 )  {
      reference.push( `sp:${output}` );
    }
    else  {
      reference.push( `sp:${Data.Instance().RootEntid()}/${entity['entid']}/${output}` );
    }

    if( reference.length > 0 )  {
      return reference;
    }

    return null;
  }

  //---------------------------------------------------------------------------
  //  validate the widget
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    if( ! ( 'door' in entity.sp ) ) {
      return 0;
    }
    
    if( !( 'param' in entity.sp['door'] ) || !( 'output' in entity.sp['door']['param'] ) ) {
      validateControl.error = "{$const_error_statepoint_deleted}";
      return 1;
    }

    let spOutput = entity.sp['door']['param']['output'];
    
    if( spOutput.indexOf( "/" ) >= 0 ) {

      // error no permission     
      let spInfo = Data.Instance().StatepointInfo( spOutput, entity );
      if( spInfo['type'] === "ERROR" )  {
        return 1;
      }

      // error no permission
      if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spOutput, "w" ) ) {
        validateControl.error = "{$const_error_statepoint_permission}";
        return 1;
      }
    }
    return 0;
  }

  //---------------------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: WidgetDeviceFunctionDoor.widgetNameLong,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //===========================================================================
  //  INSTANCE
  //===========================================================================

  //---------------------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // copy not needed
    this.entity     = createData['entity'];
    this.statepoint = this.entity['sp']['door'];
    this.isAtlc = this.entity['class'] == "door";
    if( Array.isArray( this.entity['sp'] ) )  {
      this.entity['sp'] = {};
    }


    // get device record create if not exists
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, WidgetDeviceFunctionDoor.widgetId );
    if( !this.deviceRecord ) {
      this.deviceRecord = WidgetDeviceConfigBase.HelperAddDeviceFunction( this.entity, WidgetDeviceFunctionDoor.widgetId );
    }

    if( this.deviceRecord && !this.deviceRecord['sp'] )  {
      this.deviceRecord['sp'] = WidgetDeviceFunctionDoor.widgetId;
    }

    // create default
    if( !( 'door' in this.entity['sp'] ) ) {
      this.statepoint            = {
        "label": "{{name}} (" + WidgetDeviceFunctionDoor.widgetNameShort + ")",
        "address": ( Data.Instance().RootEntid() + "/" + this.entity['entid'] + "/" + WidgetDeviceFunctionDoor.widgetId ),
        "type": "BOOL",
        "name": WidgetDeviceFunctionDoor.widgetId,
        "default": false,
        "template": WidgetDeviceConfigOpenercontact.widgetId,
        "signaltime": 3000,
        "retrigger": true,
        "invert": false,
        "resdef": [{
          "id": 1,
          "action": "switch",
          "periods": 1
        }],
        "param": {
          "output": this.isAtlc ? "atlc_a1" : null,
          "disconnectCalls": false,
        }
      };

      this.entity['sp']['door'] = this.statepoint;
      this.deviceRecord['sp']   = WidgetDeviceFunctionDoor.widgetId;
      this.Option( "modified", 1 );
    }

    // correct false entitiy
    if( !( 'param' in this.entity['sp']['door'] ) ) {
      this.entity['sp']['door']['param'] = {
        "output": "",
        "disconnectCalls": false
      };
    }    

    // correct use of dict-macros
    if( 'label' in this.entity['sp']['door'] ) {
      this.entity['sp']['door']['label'] = ( this.entity['sp']['door']['label'] ).replace( "{$aui_widget_openercontact}", WidgetDeviceFunctionDoor.widgetNameShort );
    }

    // ensure default permission record exists
    WidgetDeviceConfigBase.HelperAddDefaultPermission( WidgetDeviceFunctionDoor.widgetId, this.entity );

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );

    // build label
    ControlLabel.Build( $( "#ID_LABEL" + this.dynid ), {
      label: this.statepoint['label'],
      entityName: this.entity['name'],
      functionName: WidgetDeviceFunctionDoor.widgetNameShort
    } );

    $( "#ID_CHECK_DOOR_OPTIONS" + this.dynid ).choice( {
      size: CONST.size.L,
      type: "check",
      data: [
        { id: "invert",      text: "{$widget_door2_invert_check}", value: false },
        { id: "posttrigger", text: "{$widget_door2_trigger_check}", value: true },
        //{ id: "cancelCall",  text: "{$widget_door2_canceling_calls_suffix}", value: false },
      ]
    } );

    this.FormFillControls();
  }

  //=============================================================================
  //  VALIDATION
  //=============================================================================

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidate()
  {
    if( this.preventFirstTimeValidate )  {
      this.preventFirstTimeValidate = false;
      return 0;
    }
    let numError = 0;
    $( "#ID_LABEL" + this.dynid ).viewable( "validate" );

    if( this.mode === WidgetDeviceFunctionDoor.modeExtern )  {
      numError += $( "#ID_DOOR_OUTPUT_EXTERN" + this.dynid ).viewable( "validate" );
    }
    else  {
      numError += $( "#ID_DOOR_OUTPUT_LOCAL" + this.dynid ).listbox( "validate" );
    }

    return numError;
  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidateOff()
  {
    $( "#ID_LABEL" + this.dynid ).viewable( "validate", false );
    $( "#ID_DOOR_OUTPUT_EXTERN" + this.dynid ).viewable( "validate", false );
    $( "#ID_DOOR_OUTPUT_LOCAL" + this.dynid ).listbox( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  name(): description
  //
  ValidateEmpty( validateControl )
  {
    let selected = $( "#ID_DOOR_OUTPUT_LOCAL" + this.dynid ).listbox( "option", "selected" );
    if( Array.isArray( selected ) && selected.length === 0 )  {
      validateControl.error = "{$widget_door_error_is_empty}";
      return 1;
    }
    return 0;
  }

  //---------------------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    let time;
    let statepoint;
    let isCancelingCall = $( "#ID_DOOR_IS_CANCELING_CALL" + this.dynid ).choice( "option", "value" );

    let options = $( "#ID_CHECK_DOOR_OPTIONS" + this.dynid ).choice( "option", "value" );


    WidgetDeviceFunctionDoor.HelperRemoveLocalOutputConfig( this.entity );

    if( this.mode === WidgetDeviceFunctionDoor.modeLocal ) {
      statepoint = $( "#ID_DOOR_OUTPUT_LOCAL" + this.dynid ).listbox( "option", "selected" )[0];
      time       = $( "#ID_DOOR_TIMEOUT" + this.dynid ).listbox( "option", "selected" )[0];
      this.HelperSetDeviceConfig();
      if( this.entity['sp']['door']['alias'] ) {
        delete this.entity['sp']['door']['alias'];
      }
    }
    else {
      statepoint = $( "#ID_DOOR_OUTPUT_EXTERN" + this.dynid ).viewable( "option", "selected" )[0];
      if( !this.entity['sp']['door']['alias'] )  {
        this.entity['sp']['door']['alias'] = {
          target: statepoint
        };
      }
      else {
        this.entity['sp']['door']['alias']['target'] = statepoint;
      }
    }
    // read label and return our entity-copy
    this.entity['sp']['door']['label'] = $( "#ID_LABEL" + this.dynid ).viewable( "option", "value" );
    this.entity['sp']['door']['retrigger'] = options['posttrigger'];
    this.entity['sp']['door']['signaltime'] = parseInt( time );
    this.entity['sp']['door']['default'] = false;
    this.entity['sp']['door']['param']['output'] = statepoint;
    this.entity['sp']['door']['param']['disconnectCalls'] = isCancelingCall;

    if( options['invert'] === false )  {
      delete this.entity['sp']['door']['invert'];
    }
    else {
      this.entity['sp']['door']['invert'] = options['invert'];
    }

    if( this.mode == WidgetDeviceFunctionDoor.modeExtern )  {
      delete this.entity['sp']['door']['resdef'];
      delete this.entity['sp']['door']['signaltime'];
      delete this.entity['sp']['door']['retrigger'];
    }

    // entrance
    if( !( 'entrance' in this.entity ) ) {
      this.entity['entrance'] = {};
    }
    this.entity['entrance']['opener'] = this.entity['sp']['door']['address'];
   

    return this.entity;
  }


  //-----------------------------------------------------------------------------
  //  FillControls(): description
  //
  FormFillControls()
  {
    let isLocalOutput = this.entity['sp'][WidgetDeviceFunctionDoor.widgetId]['param']['output'] ? ( this.entity['sp'][ WidgetDeviceFunctionDoor.widgetId ]['param']['output'].indexOf( "/" ) === -1 ) : false;
    let isCancelingCall = this.entity['sp'][ WidgetDeviceFunctionDoor.widgetId ]['param']['disconnectCalls'] || false;

    let isInverted   = false;
    let isPosttrigger = true;

    let timeout  = 3000;

    let localOutput;
    let options = [
      { id: "invert",      text: "{$widget_door2_invert_check}",           value: false },
      { id: "posttrigger", text: "{$widget_door2_trigger_check}",          value: true },
    ];

    if( this.deviceRecord['template'] == WidgetDeviceFunctionDoor.widgetId ) {
      this.preventFirstTimeValidate = true;
      //isLocalOutput   = this.entity['sp'][ WidgetDeviceFunctionDoor.widgetId ]['param']['output'].indexOf( "/" ) === -1;
      isPosttrigger   = this.entity['sp'][ WidgetDeviceFunctionDoor.widgetId ]['retrigger'];

      timeout         = this.entity['sp'][ WidgetDeviceFunctionDoor.widgetId ]['signaltime'];
      localOutput     = this.entity['sp'][ WidgetDeviceFunctionDoor.widgetId ]['param']['output'];

      if( isLocalOutput ) {
        let local = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, localOutput );
        if( local && local['param'] )  {
          isInverted = local['param']['invert'] ?? false;
        }
      }

      options = [
        { id: "invert",      text: "{$widget_door2_invert_check}",  value: isInverted },
        { id: "posttrigger", text: "{$widget_door2_trigger_check}", value: isPosttrigger },
      ];
    }
    else  {
      this.preventFirstTimeValidate = true;
    }

    // prevent local output on non atlc
    if( !this.isAtlc ) {
      isLocalOutput = false;
    }    

    // hint: since we've got the old UI pattern "internal/external" we 
    // must not add local statepoints here.
    ControlStatepointList.Build( $( "#ID_DOOR_OUTPUT_EXTERN" + this.dynid ), {
      size: CONST.size.L,
      inquirer: this.entity['entid'],
      forbiddenEntityId: this.entity['entid'],
      allowedTypes: ["BOOL"],
    } );

    $( "#ID_DOOR_OUTPUT" + this.dynid ).choice( {
      size: CONST.size.L,
      data: [
        { id: WidgetDeviceFunctionDoor.modeLocal, text: "{$widget_door_local_outputs}", status: this.isAtlc ? "normal" : "disabled" },
        { id: WidgetDeviceFunctionDoor.modeExtern, text:"{$widget_door_extern_outputs}"},
      ],
      value: isLocalOutput ? WidgetDeviceFunctionDoor.modeLocal : WidgetDeviceFunctionDoor.modeExtern,
      result: ( value ) => {  
        this.ControlOutputResult( value === WidgetDeviceFunctionDoor.modeLocal );  
      }
    } );

    $( "#ID_DOOR_OUTPUT_LOCAL" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "droplist",
      placeholder: "{$const_common_empty_placeholder}",
      validate: this.ValidateEmpty.bind(this),
    } );

    $( "#ID_DOOR_TIMEOUT" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "droplist",
      selected: [ timeout ],
      data: CONST.switchTime,
      validate: this.ValidateEmpty.bind(this),
    } );

    $( "#ID_DOOR_IS_CANCELING_CALL" + this.dynid ).choice( {
      size: CONST.size.L,
      type: "check",
      value: isCancelingCall,
      text: "{$widget_door_canceling_calls_suffix}"
      // result isn't needed because the decision doesn't cause ui changes
    } );

    //checkboxes fill values into checkboxes
    $( "#ID_CHECK_DOOR_OPTIONS" + this.dynid ).choice( "option", "data", options );

    this.HelperLocalOutputs();

    if( isLocalOutput )  {
      this.ModeLocalOutput();
      $( "#ID_DOOR_OUTPUT_LOCAL" + this.dynid ).listbox( "option", "selected", [ this.entity['sp']['door']['param']['output'] ] );
    }
    else  {
      this.ModeExternalOutput();
      if( this.entity['sp']['door']['param']['output'] ) {
        $( "#ID_DOOR_OUTPUT_EXTERN" + this.dynid ).viewable( "option", "selected", [ this.entity['sp']['door']['param']['output'] ] );
      }
    }
  }

  //-----------------------------------------------------------------------------
  //  ValidateTime(): description
  //
  ValidateTime( data )
  {
    let time = parseInt( $( "#ID_DOOR_TIMEOUT" + this.dynid ).edit( "option", "value" ) );
    if( isNaN( time ) || time < -1 ) {
      data.error = "{$widget_set_redir_validate_time_error}";
      return 1;
    }
    return 0;
  }

  //=============================================================================

  //  HELPER
  //=============================================================================
  //-----------------------------------------------------------------------------
  //  HelperSetDeviceConfig(): write the door configuration to the correct output
  //                           on the same the device. e.g. if altce1_a2 is selected
  //                           then the function writes the configuration in the
  //                           device section with the uid of atlce1_2
  //
  HelperSetDeviceConfig()
  {
    let statepointAddress;
    if( this.mode === WidgetDeviceFunctionDoor.modeLocal )  {
      statepointAddress   = $( "#ID_DOOR_OUTPUT_LOCAL" + this.dynid ).listbox( "option", "selected" )[0];
    }
    else {
      return;
    }

    let uid                  = statepointAddress;
    let deviceConfig         = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, uid, null );
    let time                 = $( "#ID_DOOR_TIMEOUT" + this.dynid ).listbox( "option", "selected" )[0];
    // let isInvertSet          = $( "#ID_DOOR_INVERT" + this.dynid ).choice( "option", "value" );
    // let isPostTriggerSet     = $( "#ID_DOOR_POST_TRIGGER" + this.dynid ).choice( "option", "value" );
    let options              = $( "#ID_CHECK_DOOR_OPTIONS" + this.dynid ).choice( "option", "value" );
    let isDisconnectingCalls = $( "#ID_DOOR_IS_CANCELING_CALL" + this.dynid ).choice( "option", "value" );
    let sysid                = deviceConfig['out']['sysid'];

    //clean up old param
    deviceConfig['param']                    = {};
    deviceConfig['sp']                       = WidgetDeviceFunctionDoor.widgetId;
    deviceConfig['template']                 = 'openercontact';
    deviceConfig['param']['time']            = parseInt( time );
    deviceConfig['param']['invert']          = options['invert'];
    deviceConfig['param']['posttrigger']     = options['posttrigger'];
    deviceConfig['param']['output']          = uid;
    deviceConfig['param']['disconnectCalls'] = isDisconnectingCalls;

    this.entity['sp']['door']['signaltime'] = parseInt( time );
    this.entity['sp']['door']['resdef'][0]['id'] = parseInt( sysid );

  }
  
  //-----------------------------------------------------------------------------
  //  HelperLocalOutputs(): description
  //
  HelperLocalOutputs()
  {
    let outputs = [];

    for( let deviceConfigRecord of this.entity['deviceConfig'] )  {
      if( ( "out" in deviceConfigRecord )
       && ( deviceConfigRecord['sp'] == "door" || deviceConfigRecord['template'] === "none" ) 
        ) {
        outputs.push( { 
          id: deviceConfigRecord['uid'], 
          text: `{$widget_door_output} ${deviceConfigRecord['abbr']}`, 
          icon: "mdi mdi-key" 
        } );
      }
    }

    $( "#ID_DOOR_OUTPUT_LOCAL" + this.dynid ).listbox( "option", "data", outputs );
  }

  //-----------------------------------------------------------------------------
  //  HelperRemoveLocalOutputConfig(): description
  //
  static HelperRemoveLocalOutputConfig( entity )
  {
    for( let deviceConfigRecord of entity['deviceConfig'] )  {
      if( "out" in deviceConfigRecord
        && ( deviceConfigRecord['sp'] == "door" && deviceConfigRecord['template'] === "openercontact" ) )  {
        deviceConfigRecord['template'] = "none";
        delete deviceConfigRecord['sp'];
        delete deviceConfigRecord['param'];
      }
    }
  }

  //-----------------------------------------------------------------------------
  //  ControlOutputResult(): description
  //
  ControlOutputResult( isLocal )
  {
    if( isLocal )  {
      this.ModeLocalOutput();
    }
    else  {
      this.ModeExternalOutput();
    }
  }

  //=============================================================================
  //  MODES
  //=============================================================================
  //-----------------------------------------------------------------------------
  //  ModeLocalOutput(): description
  //
  ModeLocalOutput()
  {
    this.mode = WidgetDeviceFunctionDoor.modeLocal;
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_DOOR_OUTPUT_EXTERN" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_DOOR_OUTPUT_LOCAL" );
  }

  //-----------------------------------------------------------------------------
  //  ModeExternalOutput(): description
  //
  ModeExternalOutput()
  {
    this.mode = WidgetDeviceFunctionDoor.modeExtern;
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_DOOR_OUTPUT_LOCAL" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_DOOR_OUTPUT_EXTERN" );
  }
}

//=============================================================================
//  REGISTER IN FACTORY
//=============================================================================

WidgetDeviceFunctionDoor.Register( WidgetDeviceFunctionDoor.template );
WidgetDeviceConfigFactory.Register( WidgetDeviceFunctionDoor );
//=============================================================================
//
//  FUNCTION WIDGET DOOR2 IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WidgetDeviceFunctionDoor2 extends WidgetDeviceConfigBase
{
  //===========================================================================
  //  STATIC
  //===========================================================================

  static template              = "/widget/deviceFunctions/door2";
  static isFunction            = true;
  static allowedModules        = ["function"];             // module is function..
  static allowedTypes          = "*";                        // not used by functions
  static allowedClasses        = ["door"];    // used by functions
  static functionMustBePresent = false;             // don't show widget of no deviceRecord is present
  static widgetIcon            = "mdi mdi-key-plus";
  static widgetNameLong        = "{$widget_door2_long}";
  static widgetNameShort       = "{$aui_widget_door2}";     // label: must be "aui_" (a dict.json macro)
  static widgetId              = "door2";
  static widgetUiHeight        = 26;                        // we need 4rem in height
  static modeLocal             = "local";
  static modeExtern            = "extern";

  //===========================================================================
  //  STATIC INFORMATION OVERRIDABLES
  //===========================================================================


  //-----------------------------------------------------------------
  //  special: check if this widget is allowed in this entity
  //
  static IsUseable( entity, isFunction, isLed )
  {
    // this widget is for function
    if( !isFunction ) {
      return false;
    }

    if( ( 'entrance' in entity )
     && ( 'enabled' in entity['entrance'] )
     && ( entity['entrance']['enabled'] == true )
      ) {
      return true;
    }

    return false;
  }

  //-----------------------------------------------------------------
  //  local statepoint for permission check
  //
  static LocalStatepointName( uid, trigger, entity )
  {
    return "door2";
  }

  //---------------------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    // switch off function in device record
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, "door2" );

    if( deviceRecord ) {
      deviceRecord['sp'] = null;
    }

    // remove statepoint
    if( WidgetDeviceFunctionDoor2.widgetId in entity['sp'] ) {
      delete entity['sp'][WidgetDeviceFunctionDoor2.widgetId];
    }
    WidgetDeviceConfigBase.HelperRemoveStatepoint( "door2", entity );
    WidgetDeviceFunctionDoor2.HelperRemoveLocalOutputConfig( entity );

    // entrance
    if( ( 'entrance' in entity ) && ( 'opener2' in entity['entrance'] ) ) {
      delete entity['entrance']['opener2'];
    }

    // remove permissions
    WidgetDeviceConfigBase.HelperRemovePermission( "door2", entity );

  }

  //---------------------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = WidgetDeviceFunctionDoor2.template;
    $element.viewable( createData );
  }

  //---------------------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    let reference = [];
    let output;

    if( ( 'sp' in entity ) && ( WidgetDeviceFunctionDoor2.widgetId in entity['sp'] ) ) {
      reference.push(`sp:${Data.Instance().RootEntid()}/${entity['entid']}/door2`);
      output = entity.sp[ WidgetDeviceFunctionDoor2.widgetId ]['param']['output'];
    }

    if( output && output.indexOf( "/" ) !== -1 )  {
      reference.push( `sp:${output}` );
    }
    else  {
      reference.push( `sp:${Data.Instance().RootEntid()}/${entity['entid']}/${output}` );
    }

    if( reference.length > 0 )  {
      return reference;
    }

    return null;
  }

  //---------------------------------------------------------------------------
  //  validate the widget
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    if( !( 'door2' in entity.sp ) ) {
      return 0;
    }

    if( !( 'param' in entity.sp['door2'] ) || !( 'output' in entity.sp['door2']['param'] ) ) {
      validateControl.error = "{$const_error_statepoint_deleted}";
      return 1;
    }
    
    let spOutput = entity.sp['door2']['param']['output'];

    if( spOutput.indexOf( "/" ) >= 0 ) {

      // error no permission
      let spInfo = Data.Instance().StatepointInfo( spOutput, entity );
      if( spInfo['type'] === "ERROR" )  {
        return 1;
      }

      // error no permission
      if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spOutput, "w" ) ) {
        validateControl.error = "{$const_error_statepoint_permission}";
        return 1;
      }
    }

    return 0;
  }

  //---------------------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: WidgetDeviceFunctionDoor2.widgetNameLong,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //===========================================================================
  //  INSTANCE
  //===========================================================================

  //---------------------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // copy not needed
    this.entity     = createData['entity'];
    this.statepoint = this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId];
    if( Array.isArray( this.entity['sp'] ) )  {
      this.entity['sp'] = {};
    }
    this.selectedList = [];
    this.isNew = false;
    this.isAtlc = this.entity['class'] == "door";


    // get device record create if not exists
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, WidgetDeviceFunctionDoor2.widgetId );
    if( !this.deviceRecord ) {
      this.deviceRecord = WidgetDeviceConfigBase.HelperAddDeviceFunction( this.entity, WidgetDeviceFunctionDoor2.widgetId );
    }
    if( this.deviceRecord && !this.deviceRecord['sp'] )  {
      this.deviceRecord['sp'] = WidgetDeviceFunctionDoor2.widgetId;
    }



    // create default
    if( !( WidgetDeviceFunctionDoor2.widgetId in this.entity['sp'] ) ) {
      this.statepoint            = {
        "name": WidgetDeviceFunctionDoor2.widgetId,
        "type": "BOOL",
        "label": "{{name}} (" + WidgetDeviceFunctionDoor2.widgetNameShort + ")",
        "address": ( Data.Instance().RootEntid() + "/" + this.entity['entid'] + "/" + WidgetDeviceFunctionDoor2.widgetId ),
        "default": false,
        "template": WidgetDeviceFunctionDoor2.widgetId,
        "signaltime": 3000,
        "retrigger": true,
        "invert": false,
        "resdef": [{
          "id": 2,
          "action": "switch",
          "periods": 1
        }],
        "param": {
          "output": this.isAtlc ? "atlc_a2" : null,
          "disconnectCalls": false,
          "recipients": [],
        }
      };
      this.entity['sp'][ WidgetDeviceFunctionDoor2.widgetId ] = this.statepoint;
      this.deviceRecord['sp']    = WidgetDeviceFunctionDoor2.widgetId;
      this.Option( "modified", 1 );
      this.isNew = true;
    }

    // correct false entitiy
    if( !( 'param' in this.entity['sp']['door2'] ) ) {
      this.entity['sp']['door2']['param'] = {
        "output": "",
        "disconnectCalls": false
      };
    }    

    // correct use of dict-macros
    if( 'label' in this.entity['sp']['door2'] ) {
      this.entity['sp']['door2']['label'] = ( this.entity['sp']['door2']['label'] ).replace( "{$aui_widget_openercontact}", WidgetDeviceFunctionDoor2.widgetNameShort );
    }


    // ensure default permission record exists
    WidgetDeviceConfigBase.HelperAddDefaultPermission( WidgetDeviceFunctionDoor2.widgetId, this.entity );

    // init optional
    $( '#ID_OPTIONAL' + this.dynid ).optional( {} );

    // build label
    ControlLabel.Build( $( "#ID_LABEL" + this.dynid ), {
      label: this.statepoint['label'],
      entityName: this.entity['name'],
      functionName: WidgetDeviceFunctionDoor2.widgetNameShort
    } );


    // dataset/-view used by <add> button
    this.entityDataset = Data.Instance().EntityDataset();
    this.statepointDataset = Data.Instance().StatepointDataset();

    this.selectedListDataview = this.entityDataset.BuildView( 'default', {
      rebuildAlways: true,
      sortOrder: "+",
      sortKey: "text",
      filter: this.ControlEntitySelectedDataviewFilter.bind( this )
    } );

    this.entityDataview = this.entityDataset.BuildView( 'default', {
      rebuildAlways: true,
      sortOrder: "+",
      sortKey: "text",
      filter: this.ControlEntityDataviewFilter.bind( this )
    } );

    $( "#ID_CHECK_DOOR2_OPTIONS" + this.dynid ).choice( {
      size: CONST.size.L,
      type: "check",
      data: [
        { id: "invert",      text: "{$widget_door2_invert_check}", value: false },
        { id: "posttrigger", text: "{$widget_door2_trigger_check}", value: true },
        { id: "cancelCall",  text: "{$widget_door2_canceling_calls_suffix}", value: false },
      ]
    } );

    $( "#ID_DOOR2_IS_CANCELING_CALL" + this.dynid ).choice( {
      size: CONST.size.L,
      type: "check",
      value: false,
      text: "{$widget_door2_canceling_calls_suffix}"
    } );

    this.FormFillControls();
  }

  //=============================================================================
  //  VALIDATION
  //=============================================================================

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidate()
  {
    // prevent 1st validate wile create
    if( this.isNew || this.preventFirstTimeValidate )  {
      this.isNew = false;
      this.preventFirstTimeValidate = false;
      return 0;
    }

    let numError = 0;
    $( "#ID_LABEL" + this.dynid ).viewable( "validate" );

    if( this.mode === WidgetDeviceFunctionDoor2.modeExtern )  {
      numError += $( "#ID_DOOR2_OUTPUT_EXTERN" + this.dynid ).viewable( "validate" );
    }
    else  {
      numError += $( "#ID_DOOR2_OUTPUT_LOCAL" + this.dynid ).listbox( "validate" );
    }

    $( "#ID_DOOR2_USE_FOR_LIST" + this.dynid ).listbox( "validate" );

    return numError;
  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidateOff()
  {
    $( "#ID_LABEL" + this.dynid ).viewable( "validate", false );
    $( "#ID_DOOR2_OUTPUT_EXTERN" + this.dynid ).viewable( "validate", false );
    $( "#ID_DOOR2_OUTPUT_LOCAL" + this.dynid ).listbox( "validate", false );
    $( "#ID_DOOR2_USE_FOR_LIST" + this.dynid ).listbox( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  ValidateEmpty(): description
  //
  ValidateEmpty( validateControl )
  {
    let selected = $( "#ID_DOOR2_OUTPUT_LOCAL" + this.dynid ).listbox( "option", "selected" );
    if( Array.isArray( selected ) && selected.length === 0 )  {
      validateControl.error = "{$widget_door2_error_is_empty}";
      return 1;
    }
    return 0;
  }

  ValidateEmptySelected( validateControl )
  {

    if( Array.isArray( this.selectedList ) && this.selectedList.length === 0 )  {
      validateControl.error = "{$widget_door2_error_list_is_empty}";
      return 1;
    }
    return 0;
  }
  //---------------------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save()
  {
    let time;
    let statepoint;
    let localDeviceRecord;
    let isCancelingCall = $( "#ID_DOOR2_IS_CANCELING_CALL" + this.dynid ).choice( "option", "value" );
    let options = $( "#ID_CHECK_DOOR2_OPTIONS" + this.dynid ).choice( "option", "value" );

    WidgetDeviceFunctionDoor2.HelperRemoveLocalOutputConfig( this.entity );

    if( this.mode === WidgetDeviceFunctionDoor2.modeLocal ) {
      statepoint = $( "#ID_DOOR2_OUTPUT_LOCAL" + this.dynid ).listbox( "option", "selected" )[0];
      time       = $( "#ID_DOOR2_TIMEOUT" + this.dynid ).listbox( "option", "selected" )[0];
      localDeviceRecord = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, statepoint );

      this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['resdef'] = [ {
        id: parseInt( localDeviceRecord['out']['sysid'] ),
        action: "switch",
        periods: "1"
      } ];

      this.HelperSetDeviceConfig();
      if( this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['alias'] ) {
        delete this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['alias'];
      }
    }
    else {
      statepoint = $( "#ID_DOOR2_OUTPUT_EXTERN" + this.dynid ).viewable( "option", "selected" )[0];
      if( !this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['alias'] )  {
        this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['alias'] = {
          target: statepoint
        };
      }
      else {
        this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['alias']['target'] = statepoint;
      }
    }
    // read label and return our entity-copy
    this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['label']                    = $( "#ID_LABEL" + this.dynid ).viewable( "option", "value" );
    this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['retrigger']                = options['posttrigger'];
    this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['signaltime']               = parseInt(time);
    this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['param']['output']          = statepoint;
    this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['param']['disconnectCalls'] = isCancelingCall;
    this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['param']['recipients']      = this.selectedList;
    this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['default']                  = false;

    if( options['invert'] === false )  {
      delete this.entity['sp']['door2']['invert'];
    }
    else {
      this.entity['sp']['door2']['invert'] = options['invert'];
    }

    if( this.mode == WidgetDeviceFunctionDoor2.modeExtern )  {
      delete this.entity['sp']['door2']['resdef'];
      delete this.entity['sp']['door2']['signaltime'];
      delete this.entity['sp']['door2']['retrigger'];
    }

    // entrance
    if( !( 'entrance' in this.entity ) ) {
      this.entity['entrance'] = {};
    }
    this.entity['entrance']['opener2'] = this.entity['sp']['door2']['address'];    

    return this.entity;
  }


  //-----------------------------------------------------------------------------
  //  FormFillControls(): description
  //
  FormFillControls()
  {
    let isLocalOutput = this.entity['sp'][ WidgetDeviceFunctionDoor2.widgetId ]['param']['output'] ? ( this.entity['sp'][ WidgetDeviceFunctionDoor2.widgetId ]['param']['output'].indexOf( "/" ) === -1 ) : false;

    let isCancelingCall = false;
    let isInverted      = false;
    let isPosttrigger   = true;

    let timeout         = 3000;
    let selected = [];


    let localOutput;
    let options = [
      { id: "invert",      text: "{$widget_door2_invert_check}",           value: false },
      { id: "posttrigger", text: "{$widget_door2_trigger_check}",          value: true },
    ];

    // previously saved: read from device record
    if( this.deviceRecord['template'] == WidgetDeviceFunctionDoor2.widgetId ) {
      this.preventFirstTimeValidate = true;
      selected        = this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['param']['recipients'] || [];
      isCancelingCall = this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['param']['disconnectCalls'];
      //isLocalOutput   = this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['param']['output'].indexOf( "/" ) === -1;
      isPosttrigger   = this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['retrigger'];
      timeout         = this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['signaltime'];
      localOutput     = this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['param']['output'];

      if( isLocalOutput ) {
        let local = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, localOutput );
        if( local && local['param'] )  {
          isInverted = local['param']['invert'] ?? false;
        }
      }
      $( "#ID_DOOR2_IS_CANCELING_CALL" + this.dynid ).choice( "option", "value", isCancelingCall );
      options = [
        { id: "invert",      text: "{$widget_door2_invert_check}",           value: isInverted },
        { id: "posttrigger", text: "{$widget_door2_trigger_check}",          value: isPosttrigger },
      ];
    }
    else  {
      this.preventFirstTimeValidate = true;
    }

    this.selectedList = selected;

    // hint: since we've got the old UI pattern "internal/external" we 
    // must not add local statepoints here.
    ControlStatepointList.Build( $( "#ID_DOOR2_OUTPUT_EXTERN" + this.dynid ), {
      size: CONST.size.L,
      inquirer: this.entity['entid'],
      forbiddenEntityId: this.entity['entid'],
      allowedTypes: ["BOOL"],
    } );

    $( "#ID_DOOR2_OUTPUT" + this.dynid ).choice( {
      size: CONST.size.L,
      data: [
        { id: WidgetDeviceFunctionDoor2.modeLocal, text: "{$widget_door2_local_outputs}", status: this.isAtlc ? "normal" : "disabled" },
        { id: WidgetDeviceFunctionDoor2.modeExtern, text:"{$widget_door2_extern_outputs}"},
      ],
      value: isLocalOutput ? WidgetDeviceFunctionDoor2.modeLocal : WidgetDeviceFunctionDoor2.modeExtern,
      result: ( value ) => {  
        this.ControlOutputResult( value === WidgetDeviceFunctionDoor2.modeLocal );  
      }
    } );

    $( "#ID_DOOR2_OUTPUT_LOCAL" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "droplist",
      search: true,
      placeholder: "{$const_common_empty_placeholder}",
      searchPlaceholder: "{$const_common_search_placeholder}",
      validate: this.ValidateEmpty.bind(this),
    } );

    $( "#ID_DOOR2_TIMEOUT" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "droplist",
      selected: [ `${timeout}` ],
      data: CONST.switchTime,
      placeholder: "{$const_common_empty_placeholder}",
      searchPlaceholder: "{$const_common_search_placeholder}",
      validate: this.ValidateEmpty.bind(this),
    } );

    //checkboxes fill values into checkboxes
    $( "#ID_CHECK_DOOR2_OPTIONS" + this.dynid ).choice( "option", "data", options );


    // let allowedClasses = ["apartment", "avp", "client", "sipaudio"].join( "," );
    // ControlEntityList.Build( $( "#ID_DOOR2_USE_FOR_LIST" + this.dynid ), {
    //   allowedClass: allowedClasses,
    // });

    $( "#ID_DOOR2_USE_FOR_LIST" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "listbox",
      data: this.selectedListDataview,
      validate: this.ValidateEmptySelected.bind(this),
      errorAnchor: ".dummy",
      placeholder: "{$const_common_empty_placeholder}",
      searchPlaceholder: "{$const_common_search_placeholder}",
      result: () => {
        if ( Array.isArray( this.selectedList ) && this.selectedList.length > 0 )  {
          $( "#ID_DOOR2_BUTTON_DELETE" + this.dynid ).button("option", "status", "normal");
        }
      }
    } );

    $( "#ID_DOOR2_BUTTON_ADD" + this.dynid ).button(  {
      size: CONST.size.XS,
      status: 'normal',
      icon: "mdi mdi-plus",
      click: () => {
        $( '#ID_DOOR2_CHOOSE_LIST' + this.dynid ).listbox( "show" );
        //this.Option( "modified", 1 );
      }
    }  );

    $( "#ID_DOOR2_BUTTON_DELETE" + this.dynid ).button(  {
      size: CONST.size.XS,
      status: 'disabled',
      icon: 'mdi mdi-minus',
      click: () => {
        let selectedDoors = $( "#ID_DOOR2_USE_FOR_LIST" + this.dynid ).listbox( "option", "value" );
        if( Array.isArray( selected ) && selected.length > 0 ) {
          this.selectedList = this.selectedList.filter( entityId => entityId !== selectedDoors[0] );
          this.selectedListDataview.Update();
        }

        $( "#ID_DOOR2_BUTTON_DELETE" + this.dynid ).button( "option", "status", "disabled" );
      }
    }  );

    $( '#ID_DOOR2_CHOOSE_LIST' + this.dynid ).listbox( {
      size: 40,
      height: 5,
      type: 'listbox',
      search: true,
      status: "normal",
      placeholder: "",
      searchPlaceholder: "{$const_common_search_placeholder}",
      anchor: "#ID_DOOR2_BUTTON_ADD" + this.dynid,
      position: "bottomright",
      data: this.entityDataview,
      result: ( selectedDoors ) =>  {
        this.selectedList.push( selectedDoors[0] );
        this.ReloadList( selectedDoors );
        if( $( "#ID_DOOR2_USE_FOR_LIST" + this.dynid ).listbox( "isValidationEnabled" ) )  {
          $( "#ID_DOOR2_USE_FOR_LIST" + this.dynid ).listbox( "validate" );
        }
        $( "#ID_DOOR2_BUTTON_DELETE" + this.dynid ).button( "option", "status", "normal" );
      }
    } );

    this.HelperLocalOutputs();

    if( isLocalOutput )  {
      this.ModeLocalOutput();
      $( "#ID_DOOR2_OUTPUT_LOCAL" + this.dynid ).listbox( "option", "selected", [ this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['param']['output'] ] );
    }
    else  {
      this.ModeExternalOutput();
      if( this.entity['sp'][ WidgetDeviceFunctionDoor2.widgetId ]['param']['output'] ) {
        $( "#ID_DOOR2_OUTPUT_EXTERN" + this.dynid ).viewable( "option", "selected", [ this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['param']['output'] ] );
      }
    }
  }

  //-----------------------------------------------------------------------------
  //  ValidateTime(): description
  //
  ValidateTime( data )
  {
    let time = parseInt( $( "#ID_DOOR2_TIMEOUT" + this.dynid ).edit( "option", "value" ) );
    if( isNaN( time ) || time < -1 ) {
      data.error = "{$widget_set_redir_validate_time_error}";
      return 1;
    }
    return 0;
  }

  //=============================================================================
  //  HELPER
  //=============================================================================
  //-----------------------------------------------------------------------------
  //  HelperSetDeviceConfig(): write the door2 configuration to the correct output
  //                           on the same the device. e.g. if altce1_a2 is selected
  //                           then the function writes the configuration in the
  //                           device section with the uid of atlce1_2
  //
  HelperSetDeviceConfig()
  {
    let statepointAddress;
    if( this.mode === WidgetDeviceFunctionDoor2.modeLocal )  {
      statepointAddress   = $( "#ID_DOOR2_OUTPUT_LOCAL" + this.dynid ).listbox( "option", "selected" )[0];
    }
    else {
      return;
    }

    let uid                 = statepointAddress;
    let deviceConfig        = WidgetDeviceConfigBase.HelperGetDeviceRecord( this.entity, uid, null );
    let time                = $( "#ID_DOOR2_TIMEOUT" + this.dynid ).listbox( "option", "selected" )[0];
    //let isInvertSet         = $( "#ID_DOOR2_INVERT" + this.dynid ).choice( "option", "value" );
    //let isPostTriggerSet    = $( "#ID_DOOR2_POST_TRIGGER" + this.dynid ).choice( "option", "value" );
    let isCancelingCalls    = $( "#ID_DOOR2_IS_CANCELING_CALL" + this.dynid ).choice( "option", "value" );
    let options = $( "#ID_CHECK_DOOR2_OPTIONS" + this.dynid ).choice( "option", "value" );


    //clean up old param
    deviceConfig['param']                = {};
    deviceConfig['sp']                   = WidgetDeviceFunctionDoor2.widgetId;
    deviceConfig['template']             = 'openercontact';
    deviceConfig['param']['time']        = parseInt( time );
    deviceConfig['param']['invert']      = options['invert'];
    deviceConfig['param']['posttrigger'] = options['posttrigger'];
    deviceConfig['param']['output']      = uid;
    deviceConfig['param']['disconnectCalls'] = isCancelingCalls;
    
    //this.entity['sp'][WidgetDeviceFunctionDoor2.widgetId]['signaltime'] = parseInt( time );
  }

  //-----------------------------------------------------------------------------
  //  HelperLocalOutputs(): description
  //
  HelperLocalOutputs()
  {
    let outputs = [];

    for( let deviceConfigRecord of this.entity['deviceConfig'] )  {
      if( "out" in deviceConfigRecord
        && ( deviceConfigRecord['sp'] == "door2" || deviceConfigRecord['template'] === "none" ) )  {
        outputs.push( { 
          id: deviceConfigRecord['uid'], 
          text: `{$widget_door2_output} ${deviceConfigRecord['abbr']}`, 
          icon: "mdi mdi-key" 
        } );
      }
    }

    $( "#ID_DOOR2_OUTPUT_LOCAL" + this.dynid ).listbox( "option", "data", outputs );
  }

  //-----------------------------------------------------------------------------
  //  HelperRemoveLocalOutputConfig(): description
  //
  static HelperRemoveLocalOutputConfig( entity )
  {
    for( let deviceConfigRecord of entity['deviceConfig'] )  {
      if( "out" in deviceConfigRecord
        && ( deviceConfigRecord['sp'] == "door2" && deviceConfigRecord['template'] === "openercontact" ) )  {
        deviceConfigRecord['template'] = "none";
        delete deviceConfigRecord['sp'];
        delete deviceConfigRecord['param'];
      }
    }
  }

  //-----------------------------------------------------------------------------
  //  ControlOutputResult(): description
  //
  ControlOutputResult( isLocal )
  {
    if( isLocal )  {
      this.ModeLocalOutput();
    }
    else  {
      this.ModeExternalOutput();
    }
  }

  //=============================================================================
  //  MODES
  //=============================================================================
  //-----------------------------------------------------------------------------
  //  ModeLocalOutput(): description
  //
  ModeLocalOutput()
  {
    this.mode = WidgetDeviceFunctionDoor2.modeLocal;
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_DOOR2_OUTPUT_EXTERN" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_DOOR2_OUTPUT_LOCAL" );
    //validateoff
    $( '#ID_DOOR2_OUTPUT_EXTERN' + this.dynid ).viewable( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  ModeExternalOutput(): description
  //
  ModeExternalOutput()
  {
    this.mode = WidgetDeviceFunctionDoor2.modeExtern;
    $( "#ID_OPTIONAL" + this.dynid ).optional( "hide", "ID_DOOR2_OUTPUT_LOCAL" );
    $( "#ID_OPTIONAL" + this.dynid ).optional( "show", "ID_DOOR2_OUTPUT_EXTERN" );

    //validateoff
    $( '#ID_DOOR2_OUTPUT_LOCAL' + this.dynid ).listbox( "validate", false );
  }

  //-----------------------------------------------------------------------------
  //  ReloadList(): description
  //
  ReloadList( selected )
  {
    this.selectedListDataview.Update();
    $( "#ID_DOOR2_USE_FOR_LIST" + this.dynid ).listbox( "option", "selected", selected );
  }

  //=============================================================================
  // FILTERS
  //=============================================================================
  //-----------------------------------------------------------------------------
  //  ControlEntityDataviewFilter(): description
  //
  ControlEntityDataviewFilter( id, record, options, dataviewClass )
  {
    let isAllowed = false;
    let allowedClasses = [ 'apartment', "avp", "client", "asht", "sipaudio", "susapp" ];

    // check for allowed classes
    // if( record['class'] !== "door" )  {
    //   isAllowed = false;
    // }

    // already in selected list?
    if( allowedClasses.includes( record.class )
      && !( this.selectedList.includes( id ) ) ) {
      isAllowed = true;
    }

    // check if child of folder "project"
    // if( isAllowed ) {
    //   isAllowed = Data.Instance().IsChild( id, "folder.confdev" );
    // }  r

    return isAllowed;
  }

  //-----------------------------------------------------------------------------
  //  ControlEntitySelectedDataviewFilter(): description
  //
  ControlEntitySelectedDataviewFilter( id, record, options, dataviewClass )
  {
    let isAllowed = false;
    let allowedClasses = [ 'apartment', "avp", "client", "sipaudio", "susapp", "asht" ];

    // check for allowed classes
    if( allowedClasses.includes( record.class )
     && this.selectedList.includes( id ) 
      ) {
      isAllowed = true;
    }

    return isAllowed;
  }
}

//=============================================================================
//  REGISTER IN FACTORY
//=============================================================================

WidgetDeviceFunctionDoor2.Register( WidgetDeviceFunctionDoor2.template );
WidgetDeviceConfigFactory.Register( WidgetDeviceFunctionDoor2 );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceFunctionCamera extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/deviceFunctions/camera";
  static isFunction = true;
  static allowedModules = ["function"];             // module is function..
  static allowedTypes = "*";                        // not used by functions
  static allowedClasses = [ "door", "ipcam" ];               // used by functions
  static functionMustBePresent = false;             // don't show widget of no deviceRecord is present
  static widgetIcon = "mdi mdi-video";
  static widgetNameLong = "{$widget_camera_long}";
  static widgetNameShort = "{$aui_widget_camera}";     // label: must be "aui_" (a dict.json macro)
  static widgetId = "camera";
  static widgetUiHeight = 4;                        // we need 4rem in height

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  local statepoint for permission check
  //
  static LocalStatepointName( uid, trigger, entity )
  {
    return "camera";
  }

  //-----------------------------------------------------------------
  //  special: check if this widget is allowed in this entity
  //
  static IsUseable( entity, isFunction, isLed )
  {
    // this widget is for function
    if( !isFunction ) {
      return false;
    }

    // check class list
    let isClassAllowed = ( WidgetDeviceFunctionCamera.allowedClasses.indexOf( entity['class'] ) >= 0 );
  
    if( ( 'video' in entity )
     && ( 'send' in entity['video'] )
     && ( entity['video']['send'] == false )
      ) {
      return false;
    }

    return isClassAllowed;
  }

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    // switch off function in device record
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, "camera" );
    if( deviceRecord ) {
      deviceRecord['sp'] = null;
    }

    // remove statepoint
    if( 'camera' in entity['sp'] )  {
      delete entity['sp']['camera'];
    }

    // remove permissions
    WidgetDeviceConfigBase.HelperRemovePermission( "camera", entity );

  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = WidgetDeviceFunctionCamera.template;
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    if( ( 'sp' in entity ) && ( 'camera' in entity['sp'] ) ) {
      let reference = `sp:${ Data.Instance().RootEntid() }/${ entity['entid'] }/camera`;
      return [ reference ];
    }
    return null;
  }

  //-----------------------------------------------------------------
  //  validate the widget
  //
  static Validate( uid, trigger, entity )
  {
    return 0;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: WidgetDeviceFunctionCamera.widgetNameLong,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // copy not needed
    this.entity = createData['entity'];
    this.statepoint = this.entity['sp']['camera'];
    if( Array.isArray( this.entity['sp'] ) )  {
      this.entity['sp'] = {};
    }

    // get device record create if not exists
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, WidgetDeviceFunctionCamera.widgetId );
    if( !this.deviceRecord )  {
      this.deviceRecord = WidgetDeviceConfigBase.HelperAddDeviceFunction( this.entity, WidgetDeviceFunctionCamera.widgetId );
    }

    // create default
    if( !( 'camera' in this.entity['sp'] ) ) {
      this.statepoint = {
        "label": "{{name}} ({$aui_widget_camera})",
        "address": ( Data.Instance().RootEntid() + "/" + this.entity['entid'] + "/" + WidgetDeviceFunctionCamera.widgetId ),
        "type": "CAMERA",
        "name": WidgetDeviceFunctionCamera.widgetId,
        "persistent": false,
        "template": WidgetDeviceFunctionCamera.widgetId
      };
      this.entity['sp']['camera'] = this.statepoint;
      this.deviceRecord['sp'] = WidgetDeviceFunctionCamera.widgetId;
      this.Option( "modified", 1 );
    }

    // ensure default permission record exists
    WidgetDeviceConfigBase.HelperAddDefaultPermission( WidgetDeviceFunctionCamera.widgetId, this.entity );


    // build label
    ControlLabel.Build( $( "#ID_LABEL" + this.dynid ), {
      label:        this.statepoint['label'],
      entityName:   this.entity['name'],
      functionName: WidgetDeviceFunctionCamera.widgetNameShort
    } );

  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidate()
  {
    let numError = 0;
    $( "#ID_LABEL" + this.dynid ).viewable( "validate" );
    return numError;
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save( )
  {
    // read label and return our entity-copy
    this.entity['sp']['camera']['label'] = $( "#ID_LABEL" + this.dynid ).viewable( "option", "value" );
    return this.entity;
  }

  

}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceFunctionCamera.Register( WidgetDeviceFunctionCamera.template );
WidgetDeviceConfigFactory.Register( WidgetDeviceFunctionCamera );
//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          21.08.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceFunctionCallContact extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC (COMMON)
  //-----------------------------------------------------------------

  static template = "/widget/deviceFunctions/callcontact";
  static isFunction = true;
  static allowedModules = ["function"];             // module is function..
  static allowedTypes = "*";                        // not used by functions
  static allowedClasses = ["apartment", "avp", "tkphone", "sipaudio", "door", "client", "asht", "jungpanel"];
  static allowedOnEntrance = true;
  static functionMustBePresent = false;             // don't show widget of no deviceRecord is present
  static widgetIcon = "mdi mdi-bell-plus";
  static widgetNameLong = "{$widget_call_contact_long}";
  static widgetNameShort = "{$widget_call_contact_short}";
  static widgetId = "callcontact";
  static widgetUiHeight = 16;                        // we need 4rem in height

  //-----------------------------------------------------------------
  //  STATIC (this)
  //-----------------------------------------------------------------

  static allowedEventList = [ "INVITE", "RING", "ONLINE", "DISCONNECTING", "HOLD", "MANUAL" ];
  static eventList = [
    { "id": "INVITE", "text": "{$widget_call_contact_invite}" },
    { "id": "RING", "text": "{$widget_call_contact_ring}" },
    { "id": "ONLINE", "text": "{$widget_call_contact_online}" },
    { "id": "DISCONNECTING", "text": "{$widget_call_contact_offline}" },
    { "id": "HOLD", "text": "{$widget_call_contact_hold}" },
    { "id": "MANUAL", "text": "{$widget_call_contact_manual}" },
  ];

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  local statepoint for permission check
  //
  static LocalStatepointName( uid, trigger, entity )
  {
    return "callcontact";
  }

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    // switch off function in device record
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, "callcontact" );
    if( deviceRecord ) {
      deviceRecord['sp'] = null;
    }

    // remove statepoint (old admin was creating a sp)
    if( 'callcontact' in entity['sp'] )  {
      delete entity['sp']['callcontact'];
    }

    // remove permissions
    WidgetDeviceConfigBase.HelperRemovePermission( "callcontact", entity );

  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = WidgetDeviceFunctionCallContact.template;
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    let spAddress = entity['sp'][ WidgetDeviceFunctionCallContact.widgetId ]['param']['targetsp'];

    if( spAddress )  {
      return [ `sp:${spAddress}` ];
    }

    return null;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: WidgetDeviceFunctionCallContact.widgetNameLong,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    //TODO Load callcontact
    // copy not needed
    this.entity = createData['entity'];
    if( Array.isArray( this.entity['sp'] ) )  {
      this.entity['sp'] = {};
    }
    this.isDoor = ( this.entity['class'] == "door" );
    this.isNew  = false;

    // get device record create if not exists
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, WidgetDeviceFunctionCallContact.widgetId );
    if( !this.deviceRecord || this.deviceRecord['sp'] == null ) {
      this.preventFirstTimeValidate = true;
      this.deviceRecord       = WidgetDeviceConfigBase.HelperAddDeviceFunction( this.entity, WidgetDeviceFunctionCallContact.widgetId );
      this.deviceRecord['sp'] = "callcontact";
      this.Option( "modified", 1 );
      if( !this.entity['sp'][ WidgetDeviceFunctionCallContact.widgetId ] ) {
        this.entity['sp'][ WidgetDeviceFunctionCallContact.widgetId ] = {
          "type":     "BOOL",
          "name":     WidgetDeviceFunctionCallContact.widgetNameShort,
          "template": WidgetDeviceFunctionCallContact.widgetId,
          "address":  "{{system.root}}/{{this.entid}}/" + WidgetDeviceFunctionCallContact.widgetId,
          "default":  false,
          "hidden":   true,
          "driver":   WidgetDeviceFunctionCallContact.widgetId.toUpperCase(),
          "param":    {
            "triggeron": "RING",
            "triggeroff": "DISCONNECTING",
            "offtime":    0,
            "invert":     false,
            "justvideo":  false,
            "isOutgoing": true,
            "isIncoming": true
          }
        };
        this.isNew = true;
      }
    }

    // usable defaults, even if not set
    var currentTurnOnEvent = this.entity['sp'][ WidgetDeviceFunctionCallContact.widgetId ]['param']['triggeron'];
    var currentTurnOffEvent = this.entity['sp'][ WidgetDeviceFunctionCallContact.widgetId ]['param']['triggeroff'];

    ControlStatepointList.Build( $( "#ID_LIST_CONTACT" + this.dynid ), {
      size:           CONST.size.L,
      inquirer:       this.entity['entid'],
      entity:         this.entity,
      pinnedEntityId: this.entity['entid'],
      allowedTypes:   ['BOOL'],
    } );

    $( "#ID_LIST_TURN_ON" + this.dynid ).listbox( {
      size:     CONST.size.L,
      height: 6,
      type:     "droplist",
      selected: [ currentTurnOnEvent ],
      placeholder: "{$const_common_empty_placeholder}",
      data:     WidgetDeviceFunctionCallContact.eventList,
      result:   ( value ) => {this.RenderTurnOffList();},
    } );

    $( "#ID_LIST_TURN_OFF" + this.dynid ).listbox( {
      size: CONST.size.L,
      height: 6,
      type: "droplist",
      selected: [ currentTurnOffEvent ],
      placeholder: "{$const_common_empty_placeholder}",
      data: WidgetDeviceFunctionCallContact.eventList,
    } );


    $( "#ID_CHECK_CALL_CONTACT_OPTIONS" + this.dynid ).choice( {
      size: CONST.size.L,
      type: "check",
      data: [] 
    } );

    this.RenderTurnOffList();
    this.FormFillControls();
  }

  //=============================================================================
  //  Validation
  //=============================================================================

  //-----------------------------------------------------------------
  //  validate the widget
  //
  static Validate( validateControl, uid, trigger, entity )
  {
    if( !( WidgetDeviceFunctionCallContact.widgetId in entity.sp ) ) {
      return 0;
    }

    let spAddress = entity['sp'][ WidgetDeviceFunctionCallContact.widgetId ]['param']['targetsp'];
    let spInfo = Data.Instance().StatepointInfo( spAddress, entity );


    // error statepoint does not exists
    if( !spInfo || spInfo['type'] == "ERROR" )  {
      validateControl.error = "{$const_error_statepoint_deleted}";
      return 1;
    }

    // error no permission
    if( ! Data.Instance().PermissionList().IsAllowed( entity['entid'], spAddress, "w" ) ) {
      validateControl.error = "{$const_error_statepoint_permission}";
      return 1;
    }

    return 0;
  }

  //-----------------------------------------------------------------------------
  //  OnValidate(): description
  //
  OnValidate()
  {
    // prevent 1st validate wile create
    if( this.isNew || this.preventFirstTimeValidate )  {
      this.isNew = false;
      this.preventFirstTimeValidate = false;
      return 0;
    }
    let numError = 0;
    numError += $( "#ID_LIST_CONTACT" + this.dynid ).viewable( "validate" );
    return numError;
  }

  //-----------------------------------------------------------------------------
  //  OffValidate(): description
  //
  OnValidateOff()
  {
    $( "#ID_LIST_CONTACT" + this.dynid ).viewable( "validate", false );
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save( )
  {
    let contact          = $( "#ID_LIST_CONTACT" + this.dynid ).viewable( "option", "selected" )[0];
    let triggerOn        = $( "#ID_LIST_TURN_ON" + this.dynid ).listbox( "option", "selected" )[0];
    let triggerOff       = $( "#ID_LIST_TURN_OFF" + this.dynid ).listbox( "option", "selected" )[0];
    let options           = $( "#ID_CHECK_CALL_CONTACT_OPTIONS" + this.dynid ).choice( "option", "value" );
    let time             = isNaN( parseInt( triggerOff ) ) ? 0 : parseInt( triggerOff );

    if( WidgetDeviceFunctionCallContact.allowedEventList.includes( triggerOff ) === false )  {
      triggerOff = "TIME";
    }

    let param = {
      "targetsp":   contact,
      "triggeron":  triggerOn,
      "triggeroff": triggerOff,
      "offtime":    time,
      "invert":     options['invert'],
      "justvideo":  options['onlyVideo'],
      "isOutgoing": options['out'],
      "isIncoming": options['in']
    };

    this.entity['sp'][ WidgetDeviceFunctionCallContact.widgetId ]['param'] = param;
    return this.entity;
  }


  //=============================================================================
  //  HELPERS
  //=============================================================================

  RenderTurnOffList()
  {
    let selectedTurnOn = $( "#ID_LIST_TURN_ON" + this.dynid ).listbox( "option", "selected" )[0];
    let selectedTurnOff = $( "#ID_LIST_TURN_OFF" + this.dynid ).listbox( "option", "selected" )[0];
    let eventList = WidgetDeviceFunctionCallContact.eventList;
    eventList = eventList.filter( ( event ) => event.id != selectedTurnOn );

    if( selectedTurnOn != "MANUAL" )  {
      eventList = eventList.concat( CONST.switchTime.filter( ( time ) => time.id != "0" ) );
    }

    $( "#ID_LIST_TURN_OFF" + this.dynid ).listbox( "option", "data", eventList );
    if( selectedTurnOn == selectedTurnOff )  {
      $( "#ID_LIST_TURN_OFF" + this.dynid ).listbox( "option", "selected", eventList[ 0 ]['id'] );
    }
  }

  FormFillControls()
  {
    let sp = this.entity['sp'][ WidgetDeviceFunctionCallContact.widgetId ];
    let triggerOn    = sp['param']['triggeron'];
    let triggerOff   = sp['param']['triggeroff'] == "TIME" ? `${sp['param']['offtime']}` : sp['param']['triggeroff'];
    let offTimeout     = sp['param']['offtime'];
    let contact        = sp['param']['targetsp'];

    var isInvert    = sp['param']['invert'] || false;
    var isOnlyVideo = sp['param']['justvideo'] || false;
    var isOut       = sp['param']['isOutgoing'] || false;
    var isIn        = sp['param']['isIncoming'] || false;
    

    var options = [];
    if( this.isDoor ) {
      options = [
        { id: "in",        text: "{$widget_call_contact_option_incoming_call}", value: isIn },
        { id: "out",       text: "{$widget_call_contact_option_outgoing_call}", value: isOut },
        { id: "invert",    text: "{$widget_call_contact_option_invert}",        value: isInvert }
      ];
    }
    else {
      options = [
        { id: "in",        text: "{$widget_call_contact_option_incoming_call}", value: isIn },
        { id: "out",       text: "{$widget_call_contact_option_outgoing_call}", value: isOut },
        { id: "onlyVideo", text: "{$widget_call_contact_option_video_call}",    value: isOnlyVideo },
        { id: "invert",    text: "{$widget_call_contact_option_invert}",        value: isInvert }
      ];
    }


    //checkboxes fill values into checkboxes
    $( "#ID_CHECK_CALL_CONTACT_OPTIONS" + this.dynid ).choice( "option", "data", options );

    $( "#ID_LIST_CONTACT" + this.dynid ).viewable( "option", "selected", [ contact ] );
    $( "#ID_LIST_TURN_ON" + this.dynid ).listbox( "option", "selected", [ triggerOn ] );
    this.RenderTurnOffList();
    $( "#ID_LIST_TURN_OFF" + this.dynid ).listbox( "option", "selected", [ triggerOff ] );

  }
}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceFunctionCallContact.Register( WidgetDeviceFunctionCallContact.template );
WidgetDeviceConfigFactory.Register( WidgetDeviceFunctionCallContact );

﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceFunctionVideoprivacy extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/deviceFunctions/videoprivacy";
  static isFunction = true;
  static isModule = true;           // no not list in function list of basic systems
  static allowedModules = ["function"];             // module is function..
  static allowedTypes = "*";                        // not used by functions
  static allowedClasses = [ "door" ];    // used by functions
  static functionMustBePresent = false;             // don't show widget of no deviceRecord is present
  static widgetIcon = "mdi mdi-video-off";
  static widgetNameLong = "{$widget_videoprivacy_long}";
  static widgetNameShort = "{$aui_widget_videoprivacy}";     // label: must be "aui_" (a dict.json macro)
  static widgetId = "videoprivacy";
  static widgetUiHeight = 4;                        // we need 4rem in height

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  local statepoint for permission check
  //
  static LocalStatepointName( uid, trigger, entity )
  {
    return "videoprivacy";
  }

  //-----------------------------------------------------------------
  //  special: check if this widget is allowed in this entity
  //
  static IsUseable( entity, isFunction, isLed )
  {
    // this widget is for function
    if( !isFunction ) {
      return false;
    }

    // check class list
    let isClassAllowed = ( WidgetDeviceFunctionVideoprivacy.allowedClasses.indexOf( entity['class'] ) >= 0 );
  
    // check allowed classes
    if( ! Data.IsFeatureEnabled( "module.preview-vprivacy" ) ) {
      return false;
    }

    if( ( 'video' in entity )
     && ( 'send' in entity['video'] )
     && ( entity['video']['send'] == false )
      ) {
      return false;
    }

    return isClassAllowed;
  }

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    // switch off function in device record
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, "videoprivacy" );
    if( deviceRecord ) {
      deviceRecord['sp'] = null;
    }

    // remove statepoint
    if( 'videoprivacy' in entity['sp'] )  {
      delete entity['sp']['videoprivacy'];
    }

    // remove permissions
    WidgetDeviceConfigBase.HelperRemovePermission( "videoprivacy", entity );

  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = WidgetDeviceFunctionVideoprivacy.template;
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    if( ( 'sp' in entity ) && ( 'videoprivacy' in entity['sp'] ) ) {
      let reference = `sp:${ Data.Instance().RootEntid() }/${ entity['entid'] }/videoprivacy`;
      return [ reference ];
    }
    return null;
  }

  //-----------------------------------------------------------------
  //  validate the widget
  //
  static Validate( uid, trigger, entity )
  {
    return 0;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: WidgetDeviceFunctionVideoprivacy.widgetNameLong,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // copy not needed
    this.entity = createData['entity'];
    this.statepoint = this.entity['sp']['videoprivacy'];
    if( Array.isArray( this.entity['sp'] ) )  {
      this.entity['sp'] = {};
    }

    // get device record create if not exists
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, WidgetDeviceFunctionVideoprivacy.widgetId );
    if( !this.deviceRecord )  {
      this.deviceRecord = WidgetDeviceConfigBase.HelperAddDeviceFunction( this.entity, WidgetDeviceFunctionVideoprivacy.widgetId );
    }

    // create default
    if( !( 'videoprivacy' in this.entity['sp'] ) ) {
      this.statepoint = {
        "label": "{{name}} ({$aui_widget_videoprivacy})",
        "address": ( Data.Instance().RootEntid() + "/" + this.entity['entid'] + "/" + WidgetDeviceFunctionVideoprivacy.widgetId ),
        "type": "VIDEOPRIVACY",
        "name": WidgetDeviceFunctionVideoprivacy.widgetId,
        "persistent": false,
        "template": WidgetDeviceFunctionVideoprivacy.widgetId
      };
      this.entity['sp']['videoprivacy'] = this.statepoint;
      this.deviceRecord['sp'] = WidgetDeviceFunctionVideoprivacy.widgetId;
      this.Option( "modified", 1 );
    }

    // ensure default permission record exists
    WidgetDeviceConfigBase.HelperAddDefaultPermission( WidgetDeviceFunctionVideoprivacy.widgetId, this.entity );


    // build label
    ControlLabel.Build( $( "#ID_LABEL" + this.dynid ), {
      label:        this.statepoint['label'],
      entityName:   this.entity['name'],
      functionName: WidgetDeviceFunctionVideoprivacy.widgetNameShort
    } );

  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidate()
  {
    return 0;
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save( )
  {
    // read label and return our entity-copy
    return this.entity;
  }

  

}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceFunctionVideoprivacy.Register( WidgetDeviceFunctionVideoprivacy.template );
WidgetDeviceConfigFactory.Register( WidgetDeviceFunctionVideoprivacy );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceFunctionOutage extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/deviceFunctions/deviceoutage";
  static isFunction = true;
  static isModule = true;           // no not list in function list of basic systems
  static allowedModules = ["function"];             // module is function..
  static allowedTypes = "*";                        // not used by functions
  static allowedClasses = [ "door", "apartment", "avp", "jungpanel", "sipaudio" ];    // used by functions
  static functionMustBePresent = false;             // don't show widget of no deviceRecord is present
  static widgetIcon = "mdi mdi-alarm-light";
  static widgetNameLong = "{$widget_deviceoutage_long}";
  //static widgetNameShort = "{$aui_widget_videoprivacy}";     // label: must be "aui_" (a dict.json macro)
  static widgetId = "deviceoutage";
  //static widgetUiHeight = 4;                        // we need 4rem in height

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  special: check if this widget is allowed in this entity
  //
  static IsUseable( entity, isFunction, isLed )
  {
    // this widget is for function
    if( !isFunction ) {
      return false;
    }

    // check allowed classes
    if( Data.IsFeatureEnabled( "module.deviceoutage" ) 
     && WidgetDeviceFunctionOutage.allowedClasses.includes( entity['class'] ) 
      ) {
      return true;
    }

    return false;
  }

  //-----------------------------------------------------------------
  //  local statepoint for permission check
  //
  static LocalStatepointName( uid, trigger, entity )
  {
    return null;
  }


  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    // switch off function in device record
    WidgetDeviceConfigBase.HelperRemoveDeviceFunction( entity, "deviceoutage" );

  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = WidgetDeviceFunctionOutage.template;
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    return null;
  }

  //-----------------------------------------------------------------
  //  validate the widget
  //
  static Validate( uid, trigger, entity )
  {
    return 0;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: WidgetDeviceFunctionOutage.widgetNameLong,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // copy not needed
    this.entity = createData['entity'];

    // get device record create if not exists
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, WidgetDeviceFunctionOutage.widgetId );
    if( !this.deviceRecord )  {
      this.deviceRecord = WidgetDeviceConfigBase.HelperAddDeviceFunction( this.entity, WidgetDeviceFunctionOutage.widgetId );
    }
    this.deviceRecord["sp"] = "dummy";
    this.Option( "modified", 1 );


  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidate()
  {
    return 0;
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save( )
  {
    // read label and return our entity-copy
    return this.entity;
  }

  

}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceFunctionOutage.Register( WidgetDeviceFunctionOutage.template );
WidgetDeviceConfigFactory.Register( WidgetDeviceFunctionOutage );
﻿//=============================================================================
//
//  WIDGET NONE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class WidgetDeviceFunctionOnvif extends WidgetDeviceConfigBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/deviceFunctions/onvif";
  static isFunction = true;
  static isModule = true;                   // no not list in function list of basic systems
  static allowedModules = ["function"];             // module is function..
  static allowedTypes = "*";                        // not used by functions
  static allowedClasses = [ "door" ];    // used by functions
  static functionMustBePresent = false;             // don't show widget of no deviceRecord is present
  static widgetIcon = "mdi mdi-video-wireless-outline";
  static widgetNameLong = "{$widget_onvif_long}";
  static widgetNameShort = "{$widget_onvif_long}";     // label: must be "aui_" (a dict.json macro)
  static widgetId = "onvif";
  static widgetUiHeight = 4;                        // we need 4rem in height

  //-----------------------------------------------------------------
  //  STATIC INFORMATION OVERRIDABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  local statepoint for permission check
  //
  static LocalStatepointName( uid, trigger, entity )
  {
    return null;//"videoprivacy";
  }

  //-----------------------------------------------------------------
  //  special: check if this widget is allowed in this entity
  //
  static IsUseable( entity, isFunction, isLed )
  {
    // this widget is for function
    if( !isFunction ) {
      return false;
    }

    // check class list
    let isClassAllowed = ( WidgetDeviceFunctionOnvif.allowedClasses.indexOf( entity['class'] ) >= 0 );
  
    // check allowed classes
    if( ! Data.IsFeatureEnabled( "module.onvif" ) ) {
      return false;
    }

    if( ( 'video' in entity )
     && ( 'send' in entity['video'] )
     && ( entity['video']['send'] == false )
      ) {
      return false;
    }

    return isClassAllowed;
  }

  //-----------------------------------------------------------------
  //  remove everything widget special from entity
  //
  static Remove( uid, trigger, entity )
  {
    // switch off function in device record
    let deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( entity, WidgetDeviceFunctionOnvif.widgetId );
    if( deviceRecord ) {
      WidgetDeviceConfigBase.HelperRemoveDeviceFunction( entity, WidgetDeviceFunctionOnvif.widgetId );
    }
    if( 'video' in entity && 'onvif' in entity['video'] ) {
      delete entity['video']['onvif'];
    }
  }

  //-----------------------------------------------------------------
  //  create user interface
  //
  static Build( $element, createData )
  {
    createData['view'] = WidgetDeviceFunctionOnvif.template;
    $element.viewable( createData );
  }

  //-----------------------------------------------------------------
  //  get used external ressources
  //
  static References( uid, trigger, entity )
  {
    return null;
  }

  //-----------------------------------------------------------------
  //  validate the widget
  //
  static Validate( uid, trigger, entity )
  {
    return 0;
  }

  //-----------------------------------------------------------------
  //  build description line
  //
  static DescriptionLine( deviceRecord, entity )
  {
    // that's it
    return {
      text: WidgetDeviceFunctionOnvif.widgetNameLong,
      icon: null,     // just for LED
      color: null     // just for LED
    };
  }

  //-----------------------------------------------------------------
  //  INSTANCE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    // copy not needed
    this.entity = createData['entity'];

    // get device record create if not exists
    this.deviceRecord = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, WidgetDeviceFunctionOnvif.widgetId );
    if( !this.deviceRecord )  {
      this.deviceRecord = WidgetDeviceConfigBase.HelperAddDeviceFunction( this.entity, WidgetDeviceFunctionOnvif.widgetId );
      this.deviceRecord['enabled'] = true;
      if( ! ('video' in this.entity ) ) {
        this.entity["video"] ={};
      }
      this.entity["video"]['onvif'] = {
        enabled: true,
        profileName: StringTools.CreateLimitedAsciiString( this.entity['name'] ),
        encoding: "h264",
        resolution: {
          height: 288,
          width: 352
        }
      };

      this.Option( "modified", 1 );
    }

    // init controls
    $( "#ID_PROFILE_NAME" + this.dynid ).edit( {
      size: CONST.size.L,
      filter: { allow: "0123456789.abcdefghijklmnopqrstuvwxyz_ABCDEFGHIJKLMNOPQRSTUVWXYZ" },
      value: this.entity["video"]['onvif']['profileName']
    });

    $( "#ID_NAME_REFRESH" + this.dynid ).button( {
      size: CONST.size.XS,
      icon: "mdi mdi-refresh",
      tooltip: "{$widget_onvif_refresh_tooltip}",
      click: this.ControlRefreshClick.bind( this )
    } );

    $( "#ID_RESOLUTION" + this.dynid ).listbox( {
      size: CONST.size.L,
      type: "droplist",
      data: [
        { id: "h264_352x288", text: "352 x 288 (H.264)" }
      ],
      selected: [ "h264_352x288" ]
    });


  }

  //---------------------------------------------------------------------------
  // button: refresh profile name 
  //
  ControlRefreshClick()
  {
    this.entity["video"]['onvif']['profileName'] = StringTools.CreateLimitedAsciiString( this.entity['name'] );
    $( "#ID_PROFILE_NAME" + this.dynid ).edit( "option", "value", this.entity["video"]['onvif']['profileName'] );
    this.Option( "modified", 1 );
  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidate()
  {
    return 0;
  }

  //-----------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  Save( )
  {
    // read label and return our entity-copy
    this.entity["video"]['onvif']['profileName'] = $( "#ID_PROFILE_NAME" + this.dynid ).edit( "option", "value" );
    return this.entity;
  }

  

}


//-----------------------------------------------------------------------------
//  REGISTER IN FACTORY
//-----------------------------------------------------------------------------

WidgetDeviceFunctionOnvif.Register( WidgetDeviceFunctionOnvif.template );
WidgetDeviceConfigFactory.Register( WidgetDeviceFunctionOnvif );
﻿//=============================================================================
//
//  CONTROL DEVICE GROUP IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//  Data:
//    filteredList [
//      [
//        { index: 5, keyIndex: 1, uid: "key1", config: { ... }, 
//          label: "Taste Türöffner",  $line': <user-if>  
//        },
//        { index: 6, keyIndex: 1, uid: "key1", config: { ... } },
//        { index: 7, keyIndex: 1, uid: "key1", config: { ... } }
//      ],
//      [ ... ]
//    ]
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlDeviceConfigGroup extends Viewable
{
  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  static template = "/widget/deviceConfigGroup/devGroup";

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    this.entity = createOptions['entity'];
    this.modules = createOptions['module'];
    this.filteredList = [];
    this.isTileNewMode = false;
    this.svgControls = null;
    this.svgActive = null;

    // filter list and build controls
    this.DataBuildFilteredList();
    this.FormRebuild();

    // subscribe startup data
    Data.Instance().On( "DATA/STARTUP_FINISHED/", this.etag, this.DataStartupFinished.bind( this ) );
  }

  //-----------------------------------------------------------------
  //
  //
  OnEvent( eventName, param, data )
  {
    // send DEVRESTART_NEEDED
    if( eventName == "modified" && param > 0 )  {
      this.FireEvent( "restartNeeded" );
    }

    // last update before save entity (do last chance update)
    if( eventName == "option" && param == "save" ) {
      this.DataSaveEntity( data );
      if( this.Option( "error" ) ) {
        this.entity = $.clone( data );
        this.DataBuildFilteredList();
        this.FormRebuild();
        this.OnValidate();
      }
    }

    // rebuild complete UI
    if( eventName == "option" && param == "rebuild" ) {
      this.entity = $.clone( data );
      let numErr = this.Option( "error" );
      this.DataBuildFilteredList();
      this.FormRebuild();
      this.FormPermissionCheck( this.entity );
      if( numErr )  {
        this.OnValidate();
      }
    }

    // get references
    if( eventName == "option" && param == "references" ) {
      return this.FormReferences();
    }

    // permission checker
    if( eventName == "permission-check" ) {
      return this.FormPermissionCheck( data );
    }
  }



  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  create controls after form creation (once)
  //
  FormRebuild(  )
  {
    this.$container.empty();

    this.$hlayout = $( $.views.Html( ControlDeviceConfigGroup.template + "/hlayout", {}, this.dynid ) );
    this.$lineContainer = this.$hlayout.find( ".h-layout-fix" );
    this.$previewContainer = this.$hlayout.find( ".h-layout-stretch" );
    this.$container.append( this.$hlayout );

    for( let keyGroup of this.filteredList )  {

      let isThisGroupTile = ( keyGroup[ 0 ]['config']['module'] == "TILE" );

      // special for AVP: skip empty keys
      if( isThisGroupTile && ( keyGroup[ 0 ]['config']['template'] == "none" ) ) {
        continue;
      }

      // create label + preview for this key based on first trigger 
      keyGroup[ 0 ]['label']   = DeviceKeyInfo.GetTitle     ( keyGroup[ 0 ]['uid'], keyGroup[ 0 ]['keyIndex'], this.entity['deviceTemplate'], keyGroup[ 0 ]['config'] );
      keyGroup[ 0 ]['preview'] = DeviceKeyInfo.GetSvgPreview( keyGroup[ 0 ]['uid'], keyGroup[ 0 ]['keyIndex'], this.entity['deviceTemplate'], keyGroup[ 0 ]['config'], this.entity['deviceConfig'] );

      // build the line and store 
      let $line = null;
      if( isThisGroupTile )  {
        $line = $( $.views.Html( ControlDeviceConfigGroup.template + "/lineTile", keyGroup[0], this.dynid ) );
      }
      else if( keyGroup[0]['config']['template'] == "toemode" )  {
        $line = $( $.views.Html( ControlDeviceConfigGroup.template + "/toemode", keyGroup[0], this.dynid ) );
      }
      else  {
        $line = $( $.views.Html( ControlDeviceConfigGroup.template + "/line", keyGroup[0], this.dynid ) );
      }
      keyGroup[ 0 ]['$line'] = $line;
      keyGroup[ 0 ]['$permCheck'] = $line.find( ".permission-check" );

      // add mouseover event handler for preview svg
      if( keyGroup[ 0 ]['preview'] ) {
        $line.on( "mouseenter", ( ev ) => {
          this.ControlMouseEnter( keyGroup[ 0 ] );
        } );
        $line.on( "mouseleave", ( ev ) => {
          this.ControlMouseLeave( keyGroup[ 0 ] );
        } );
      }

      // we've got more informations in keyGroupRecord. The actual deviceKeyRecord
      // is stored in "config". To keep the interface clean we create a new array
      // without our internal data. This will be given to WidgetDeviceConfigFactory.
      let keyConfigRecords = [];
      for( let rec of keyGroup )  {
        keyConfigRecords.push( rec['config'] );
      }

      // placeholder will show a message "click for more info"
      let placeholder = "{$devkeygrp_collapsed}";
      if( Data.Instance().IsLoading() ) {
        placeholder = "{$const_common_loading}";
      }

      // no description line: place a widget directly
      let $control = $line.find( ".control" );
      if( $control.length )  {
        if( WidgetDeviceConfigFactory.WidgetIsLineControl( keyGroup[ 0 ]['config']['template'] ) ) {
          WidgetDeviceConfigFactory.BuildUI( $control, keyGroup[ 0 ]['config']['template'], null, null, this.entity, ( deviceRecord ) => {
            this.ControlDirectResult( deviceRecord );
          } );
        }
      }

      // description line
      let descriptionData = WidgetDeviceConfigFactory.DescriptionLine( keyConfigRecords, this.entity );
      let $descr = $line.find( ".descriptionline" );
      if( $descr.length )  {
        $descr.descriptionline( {
          size: CONST.size.XL ,
          placeholder: placeholder,
          data: descriptionData,
          status: "expanded",
          click: ( ev ) => {
            this.ControlDescriptionClick( keyGroup[ 0 ] );
          }
        } );
      }

      // edit button 
      let $button = $line.find( ".button" );
      if( $button.length ) {
        $button.button( {
          size: 4,
          icon: "mdi mdi-pencil",
          click: ( ev ) => {
            this.ControlEditClick( keyGroup[ 0 ] );
          }
        } );
      }

      // delete button 
      if( isThisGroupTile ) {
        let $button2 = $line.find( ".button2" );
        if( $button2.length )  {
          $button2.button( {
            size: 4,
            icon: "mdi mdi-delete",
            click: ( ev ) =>{
              this.ControlDeleteClick( keyGroup );
            }
          } );
        }
      }

      // append the line to container
      this.$lineContainer.append( $line );
    }

    // if we skipped some tiles display <add> button
    if( this.isTileNewMode ) {
      this.$buttonLine = $( $.views.Html( ControlDeviceConfigGroup.template + "/add", {}, this.dynid ) );
      this.$lineContainer.append( this.$buttonLine );

      let $button = this.$buttonLine.find( ".button" );
      $button.button( {
        size: CONST.size.M,
        text: "{$devkeygrp_button_add}",
        click: this.ControlAddTileClick.bind( this )
      } );

    }

    this.svgControls = null;
    this.FormRebuildSvgPreview();
  }

  //-----------------------------------------------------------------
  //  update control content after update data
  //
  FormRebuildSvgPreview()
  {
    if( this.svgControls != null )
      return;

    // url for image loading
    let getUrl = window.location;
    let baseUrl = getUrl.protocol + "//" + getUrl.host + "/" + getUrl.pathname.split( '/' )[1];

    // we do this once!
    this.svgControls = {};
    this.svgActive = null;

    for( let keyGroup of this.filteredList ) {
      let preview = keyGroup[ 0 ]['preview'];
      if( preview )  {

        if( ! ( preview['file'] in this.svgControls ) ) {

          let $svg = $( "<div>" );
          this.$previewContainer.append( $svg );
          $svg.svg( {
            size: -1,
            url: baseUrl + "/img/" + preview['file']
          } );
          this.svgControls[ preview['file'] ] = $svg;

          // show the first preview image as default, all others are hidden
          if( this.svgActive == null )  {
            this.svgActive = preview['file'];
          }
          else {
            $svg.css( "display", "none" );
          }
        }
      }
    }


  }

  //-----------------------------------------------------------------
  //  update control content after update data
  //
  FormFillUpdatedControls()
  {
    for( let keyGroup of this.filteredList ) {

      // if this keygroup isn't displayed (on avp): skip.
      if( !( '$line' in keyGroup[ 0 ] ) ) {
        continue;
      }

      // we've got more informations in keyGroupRecord. The actual deviceKeyRecord
      // is stored in "config". To keep the interface clean we create a new array
      // without our internal data. This will be given to WidgetDeviceConfigFactory.
      let keyConfigRecords = [];
      for( let rec of keyGroup ) {
        keyConfigRecords.push( rec['config'] );
      }

      // description line
      let descriptionData = WidgetDeviceConfigFactory.DescriptionLine( keyConfigRecords, this.entity );
      let $descr = keyGroup[0]['$line'].find( ".descriptionline" );
      $descr.descriptionline( "option", "data", descriptionData );
      $descr.descriptionline( "option", "placeholder", "{$devkeygrp_collapsed}" );

    }
  }

  //-----------------------------------------------------------------
  //  inform upper layer about changed entity
  //
  FormFireResult()
  {
    this.FireEvent( "result", $.clone( this.entity ) );
  }

  //-----------------------------------------------------------------
  //  get references list from widgets
  //
  FormReferences()
  {
    let referenceList = [];

    for( let keyGroup of this.filteredList ) {
      let result = WidgetDeviceConfigFactory.ReferencesByKey( keyGroup[ 0 ]['uid'], this.entity );
      if( Array.isArray( result ) ) {
        $.array.Join( referenceList, result );
      }
    }

    return referenceList;
  }

  //-----------------------------------------------------------------
  //  get references list from widgets
  //
  FormPermissionCheck( entinfo )
  {
    for( let keyGroup of this.filteredList ) {

      if( !( '$line' in keyGroup[ 0 ] ) )  {
        continue;
      }

      let $permCheck = keyGroup[ 0 ]['$line'].find( ".permission-check" );
      let hasPermissionWarning = false;

      for( let keyRecord of keyGroup )  {        
        let spname = WidgetDeviceConfigFactory.LocalStatepointName( keyRecord['config']['template'], keyRecord['uid'], keyRecord['config']['trigger'], this.entity );
        if( spname ) {
          hasPermissionWarning = ( hasPermissionWarning ? hasPermissionWarning : ControlPermissionCheck.Check( spname, this.entity ) );
        }
      }
    
      if( hasPermissionWarning ) {
        $permCheck.removeClass( "hidden" );
      }
      else {
        $permCheck.addClass( "hidden" );
      }

    }
  }


  //-----------------------------------------------------------------
  //  CONTROL
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  mouse enters a line, show preview
  //
  ControlMouseEnter( keyRecord )
  {
    if( ! this.svgControls )  {
      return;
    }

    if( ( 'preview' in keyRecord ) && ( keyRecord['preview']['file'] in this.svgControls ) )  {

      if( this.svgActive != keyRecord['preview']['file'] ) {
        this.svgControls[ keyRecord['preview']['file'] ].css( "display", "" );
        this.svgActive = keyRecord['preview']['file'];

        for( let key in this.svgControls ) {
          if( key != this.svgActive ) {
            this.svgControls[key].css( "display", "none" );
          }
        }

      }

      this.svgControls[ keyRecord['preview']['file'] ].svg( "option", "selected", keyRecord['preview']['key'] );
    }

  }

  //-----------------------------------------------------------------
  //  mouse leaves a line
  //
  ControlMouseLeave( keyRecord )
  {
    if( !this.svgControls ) {
      return;
    }

    if( ( 'preview' in keyRecord ) && ( keyRecord['preview']['file'] in this.svgControls ) )  {
      this.svgControls[ keyRecord['preview']['file'] ].svg( "option", "selected", null );
    }
  }

  //-----------------------------------------------------------------
  //  click on edit button beside description line
  //
  ControlEditClick( keyRecord0 )
  {
    // start device keys dialog
    DeviceKeysDialog.Show( {
      data: this.entity,
      options: {
        uid: keyRecord0['uid'],       
        keyNumberIndex: keyRecord0['keyIndex'],
      },
      result: ( entity ) => {
        if( $.isPlainObject( entity ) )  {
          this.Option( "modified", 1 );
          this.entity = entity;
          this.DataBuildFilteredList();
          this.FormRebuild();
          this.FormFireResult();
          this.OnValidate();

        }
      }
    } );

  }

  //-----------------------------------------------------------------
  //  a direkt line control posts it's result
  //
  ControlDirectResult( deviceRecord )
  {
    var index = WidgetDeviceConfigBase.HelperGetDeviceFunction( this.entity, deviceRecord['template'] );
    this.entity['deviceConfig'][ index ] = deviceRecord;
    this.Option( "modified", 1 );
    this.FormFireResult();
    /*
          this.Option( "modified", 1 );
          this.entity = entity;
          this.DataBuildFilteredList();
          this.FormRebuild();
          this.FormFireResult();
          this.OnValidate();
    */
  }  

  //-----------------------------------------------------------------
  //  click on description line
  //
  ControlDescriptionClick( keyRecord0 )
  {
    this.ControlEditClick( keyRecord0 );
  }

  //-----------------------------------------------------------------
  //  click on delete button beside description line
  //
  ControlDeleteClick( keyGroup )
  {
    for( let keyRecord of keyGroup )  {
      WidgetDeviceConfigFactory.RemoveFromEntity( keyRecord['uid'], keyRecord['config']['trigger'], this.entity );
    }

    this.DataBuildFilteredList();
    this.FormRebuild();
    this.FormFireResult();
    this.Option( "modified", 1 );

    let numErr = this.Option( "error" );
    if( numErr )  {
      this.OnValidate();
    }


  }

  //-----------------------------------------------------------------
  //  click on add-tile button at end of form
  //
  ControlAddTileClick()
  {
    let firstUnusedKeyGroup = false;
    for( let keyGroup of this.filteredList ) {
      if( keyGroup[ 0 ]['config']['template'] == "none" ) {
        firstUnusedKeyGroup = keyGroup;
        break;
      }
    }

    if( Array.isArray( firstUnusedKeyGroup ) ) {

      // start device keys dialog
      DeviceKeysDialog.Show( {
        data: this.entity,
        options: {
          uid: firstUnusedKeyGroup[ 0 ]['uid'],
          keyNumberIndex: firstUnusedKeyGroup[ 0 ]['keyIndex'],
        },
        result: ( entity ) => {
          if( $.isPlainObject( entity ) ) {
            this.entity = entity;
            this.DataBuildFilteredList();
            this.FormRebuild();
            this.Option( "modified", 1 );
            this.FormFireResult();
            this.OnValidate();
  
          }
        }
      } );

    }
  }


  //-----------------------------------------------------------------
  //  DATA
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  event data layer completed
  //
  DataStartupFinished()
  {
    this.FormFillUpdatedControls();
  }

  //-----------------------------------------------------------------
  //  helper to build our internal data
  //
  DataSaveEntity( entity )
  {
    for( let keyGroup of this.filteredList ) {
      // got through all device records
      for( let deviceRecord of keyGroup ) {
        if( deviceRecord['config']['template'] != "none" ) {
          let entityDeviceRecord = entity['deviceConfig'][ deviceRecord['index'] ];
          WidgetDeviceConfigFactory.UpdateEntity( entityDeviceRecord, entity );
        }
      }
    }
  }

  //-----------------------------------------------------------------
  //  helper to build our internal data
  //
  DataBuildFilteredList()
  {
    // prequisites
    if( !( 'deviceConfig' in this.entity ) ) {
      return;
    }

    // functions or keys & io?
    if( this.modules[ 0 ] == "function" )  {
      this.DataBuildFilteredListFunctions();
    }
    else  {
      this.DataBuildFilteredListKeys();
    }
  }

  //-----------------------------------------------------------------
  //  build data for functions
  //
  DataBuildFilteredListFunctions()
  {
    this.filteredList = [];
    let keyIndex = 0;
    let overallIndex = 0;

    for( let configRecord of this.entity['deviceConfig'] ) {
      if( this.modules.indexOf( configRecord['module'] ) >= 0 ) {

        let record = {
          index: overallIndex,
          keyIndex: keyIndex,
          config: configRecord
        };
        this.filteredList.push( record );
      }

      if( configRecord['module'] != "ATLM" )  {
        overallIndex++;
      }
    }
    
  }//func

  //-----------------------------------------------------------------
  //  build data for keys & io
  //
  DataBuildFilteredListKeys()
  {
    this.filteredList = [];
    let keyIndex = 0;
    let overallIndex = 0;
    var isAtlcToeModePossible = ( this.modules.indexOf( "ATLC" ) >= 0 );

    for( let configRecord of this.entity['deviceConfig'] )  {
      
      // special for AC/DC mode on ATLC-NG
      if( isAtlcToeModePossible && configRecord['template'] == "toemode" )  {
        let record = {
          index: overallIndex,
          keyIndex: 0,
          uid: "atlc_a1-toemode",
          config: configRecord
        };
        this.filteredList.push( [
          record
        ] );
      }
      
      // build keys or IOs
      if( this.modules.indexOf( configRecord['module'] ) >= 0 )  {

        // on avp we should display an <add> button below
        // if there's an empty tile
        if( configRecord['module'] == "TILE" && configRecord['template'] == "none" )  {
          this.isTileNewMode = true;
        }

        // group all triggers of one "uid"
        let foundIndex = this.filteredList.findIndex( ( seek ) => seek[ 0 ]['uid'] == configRecord['uid'] );
        if( foundIndex < 0 )  {
          foundIndex = this.filteredList.push( [ ] ) - 1;

          // atlm key does not count
          if( configRecord['module'] != "ATLM" ) {
            keyIndex++;
          }
        }

        // get the group and add a trigger
        let filteredRecord = this.filteredList[ foundIndex ];
        let record = {
          index: overallIndex,
          keyIndex: keyIndex,
          uid: configRecord['uid'],
          config: configRecord
        };
        filteredRecord.push( record );

      }

      overallIndex++;
      
    }

  }//func


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate()
  {
    let globalErr = 0;
    for( let keyGroup of this.filteredList ) {
      
      // if this keygroup isn't displayed (on avp): skip.
      //if( !( '$line' in keyGroup[0] ) ) {
      //  continue;
      //}

      let numErr = 0;

      // got through all device records
      for( let deviceRecord of keyGroup )  {
        if( deviceRecord['config']['template'] != "none" && deviceRecord['config']['template'] != "toemode")  {
          let trigger = deviceRecord['config']['trigger'] || null;
          let validateResult = WidgetDeviceConfigFactory.Validate( deviceRecord['config']['template'], deviceRecord['config']['uid'], trigger, this.entity );
          numErr += validateResult;
        }
      }

      // one description line has only 1 error
      if( numErr > 0 )  {
        globalErr += 1;
      }

      // display error status, if needed 
      if( '$line' in keyGroup[ 0 ] ) {
        let $descr = keyGroup[ 0 ]['$line'].find( ".descriptionline" );
        if( $descr.length )  {
          if( numErr > 0 )  {
            $descr.descriptionline( "status", "add", "error" );
          }
          else {
            $descr.descriptionline( "status", "remove", "error" );
          }
        }
      }

    }

    this.Option( "error", globalErr );
    return globalErr;
  }
  
  
  //-----------------------------------------------------------------
  //  HELPERS
  //-----------------------------------------------------------------


  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlDeviceConfigGroup.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    if( Array.isArray( selected ) && selected.length > 0 ) {
      if( !( Data.Instance().CalltargetDataset().Has( selected[ 0 ] ) ) ) {
        validateControl.error = "{$control_calltarget_error_deleted}";
        return 1;
      }
    }
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlDeviceConfigGroup.Register( ControlDeviceConfigGroup.template );
﻿//=============================================================================
//
//  CONTROL DEVICE GROUP IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//  Data:
//
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlDeviceFunctionGroup extends Viewable
{
  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  static template = "/widget/deviceConfigGroup/devFunction";

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    this.entity = createOptions['entity'];
    this.modules = createOptions['module'];

    // create grid
    $( '#ID_FUNCTIONS' + this.dynid ).grid( {
      size: CONST.size.XL,
      height: "fit",
      heightMin: 6,
      heightMax: 12,
      selection: 'single',
      data: [],
      dataId: 'id',
      scrollbar: 'overlay', // 'overlay', 'static'
      //focus: "cell",
      sort: 'text',
      sortOrder: '+',
      headerHidden: true,
      header: {
        'text': { text: "{$devfunc_gridheader_description}", size: -53, sortable: true, align: "left" },
        'active': { text: "{$devfunc_gridheader_status}", size: -8, inplace: "onoff", align: "left", sortable: true },
        'permission': { text: "", size: -4, align: "center", status: "no-hover" },
      },
      renderCell: this.ControlGridRenderCell.bind( this ),
      renderStatus: this.ControlGridRenderStatus.bind( this ),
      inplaceEdit: this.ControlGridInplaceActivate.bind( this ),
      dblclick: this.ControlGridDblClick.bind( this ),
      result: this.ControlGridSelected.bind( this )
    } );

    $( '#ID_EDIT' + this.dynid ).button( {
      size: CONST.size.M,
      text: "{$devfunc_button_edit}",
      click: this.ControlButtonEditClick.bind( this )
    } );
    $( '#ID_ON' + this.dynid ).button( {
      size: CONST.size.XS,
      icon: "mdi mdi-check-bold",
      click: this.ControlButtonOnClick.bind( this )
    } );
    $( '#ID_OFF' + this.dynid ).button( {
      size: CONST.size.XS,
      icon: "mdi mdi-close-thick",
      click: this.ControlButtonOffClick.bind( this )
    } );    


    // filter list and build controls
    this.DataBuildList();
    this.FormRebuild();
    this.ControlGridSelected( [] );

  }

  //-----------------------------------------------------------------
  //
  //
  OnEvent( eventName, param, data )
  {
    // send DEVRESTART_NEEDED
    if( eventName == "modified" && param > 0 )  {
      this.FireEvent( "restartNeeded" );
    }

    // rebuild complete UI
    if( eventName == "option" && param == "rebuild" ) {
      this.entity = $.clone( data );
      this.DataBuildList();
      this.FormRebuild();
      this.FormPermissionCheck( this.entity );
    }

    // get references
    if( eventName == "option" && param == "references" ) {
      return this.FormReferences();
    }

    // permission checker
    if( eventName == "permission-check" ) {
      return this.FormPermissionCheck( data );
    }

  }

  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  create controls after form creation (once)
  //
  FormRebuild(  )
  {
    $( '#ID_FUNCTIONS' + this.dynid ).grid( "option", "data", this.filteredList );
  }

  //-----------------------------------------------------------------
  //  inform upper layer about changed entity
  //
  FormFireResult()
  {
    console.log( "Function-Fire", this.entity );
    this.FireEvent( "result", $.clone( this.entity ) );
  }

  //-----------------------------------------------------------------
  //  get references list from widgets
  //
  FormReferences()
  {
    let referenceList = [];

    for( let record of this.filteredList )  {
      if( record['active'] ) {
        let result = [];
        if( record['type'] == "rule" ) {
          result = WidgetRuleConfigFactory.References( record['id'], this.entity );
        }
        else {
          result = WidgetDeviceConfigFactory.ReferencesByFunction( record['id'], this.entity );
        }
        if( Array.isArray( result ) ) {
          $.array.Join( referenceList, result );
        }
      }
    }
    return referenceList;
  }

  //-----------------------------------------------------------------
  //  get references list from widgets
  //
  FormPermissionCheck( entinfo )
  {
    let isChanged = false;
    for( let record of this.filteredList )  {
      if( record['active'] ) {
        let spname;
        if( record['type'] == "rule" ) {
          spname = WidgetRuleConfigFactory.LocalStatepointName( record['id'], this.entity );
        }
        else {
          spname = WidgetDeviceConfigFactory.LocalStatepointName( record['id'], null, null, this.entity );
        }
        
        let isPermissionWarning = false;
        if( Array.isArray( spname ) ) {
          if( ControlPermissionCheck.Check( spname, this.entity ) ) {
            isPermissionWarning = true;
          };
        }
        else {
          isPermissionWarning = ControlPermissionCheck.Check( spname, this.entity );
        }

        if( isPermissionWarning != record['permission'] )  {
          isChanged = true;
          record['permission'] = isPermissionWarning;
        }
      }
    }

    if( isChanged )  {
      this.FormRebuild();
    }
  }

  //-----------------------------------------------------------------
  //  click on edit button beside description line
  //  
  FormUpdateButtonStatus( )
  {
    let selected = $( '#ID_FUNCTIONS' + this.dynid ).grid( "option", "selected" );    
    if( Array.isArray( selected ) && selected.length > 0 )  {
      let id = selected[ 0 ];
      let record = this.filteredList.find( ( rec => rec['id'] == id ) );
      $( '#ID_EDIT' + this.dynid ).button( "option", "status", "normal" ); 
      if( record['active'] == true )  {
        $( '#ID_ON' + this.dynid ).button( "option", "status", "disabled" ); 
        $( '#ID_OFF' + this.dynid ).button( "option", "status", "normal" ); 
      }
      else {
        $( '#ID_ON' + this.dynid ).button( "option", "status", "normal" ); 
        $( '#ID_OFF' + this.dynid ).button( "option", "status", "disabled" );   
      }
    }
    else {
      $( '#ID_EDIT' + this.dynid ).button( "option", "status", "disabled" ); 
      $( '#ID_ON' + this.dynid ).button( "option", "status", "disabled" ); 
      $( '#ID_OFF' + this.dynid ).button( "option", "status", "disabled" ); 
    }
  }

  //-----------------------------------------------------------------
  //  CONTROL
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build cell (icon + description)
  //
  ControlGridRenderCell( columnId, record )
  {
    let icon = "";
    let text = record[ columnId ];
    if( columnId == 'text' ) {
      icon = record['icon'];
      text = record['text'];
    }
    else if( columnId == "permission" ) {
      return {
        icon: record['permission'] ? "mdi mdi-security yellow-text" : "",
        tooltip: record['permission'] ? $.lang( "{$permcheck_tooltip}" ) : ""
      };     
    }
    return {
      'text': text,
      'icon': icon
    };
  }

  //-----------------------------------------------------------------
  //  set status for a grid line (error or normal)
  //
  ControlGridRenderStatus( recordId, record )
  {
    return ( record['error'] === true ) ? "red" : "normal";
  }

  //-----------------------------------------------------------------
  //  click on grid check/cross icon
  //
  ControlGridInplaceActivate( columnId, record )
  {
    this.DataToggleActivation( record );
    this.FormUpdateButtonStatus();
  }

  //-----------------------------------------------------------------
  //  click on edit button beside description line
  //
  ControlDeviceFunctionResult( entity )
  {
    this.entity = entity;
    this.Option( "modified", 1 );
    this.OnValidate();
    this.DataBuildList();
    this.FormRebuild();
    this.FormFireResult();
  }

  //-----------------------------------------------------------------
  //  double click on grid line
  //
  ControlGridDblClick( id )
  {
    var functionRecord = this.filteredList.find( ( rec ) => rec['id'] == id );

    // -- rule dialog --
    if( functionRecord['type'] == "rule" )  {
      DeviceRuleDialog.Show( {
        data: this.entity,
        options: {
          ruleId: id
        },
        result: this.ControlDeviceFunctionResult.bind( this )
      } );
    }
    // -- widget dialog --
    else {
      DeviceFunctionDialog.Show( {
        data: this.entity,
        options: {
          functionId: id
        },
        result: this.ControlDeviceFunctionResult.bind( this )
      } );
    }

  }

  //-----------------------------------------------------------------
  //  click on edit button beside description line
  //  
  ControlGridSelected( selected )
  {
    this.FormUpdateButtonStatus();
  }

  //-----------------------------------------------------------------
  //  click on edit button below description line
  //
  ControlButtonEditClick( )
  {
    let selected = $( '#ID_FUNCTIONS' + this.dynid ).grid( "option", "selected" );
    if( Array.isArray( selected ) && selected.length > 0 )  {

      var functionRecord = this.filteredList.find( ( rec ) => rec['id'] == selected[ 0 ] );
      if( functionRecord['type'] == "rule" ) {
        DeviceRuleDialog.Show( {
          data: this.entity,
          options: {
            ruleId: selected[0]
          },
          result: this.ControlDeviceFunctionResult.bind( this )
        } );
      }
      else {
        DeviceFunctionDialog.Show( {
          data: this.entity,
          options: {
            functionId: selected[0]
          },
          result: this.ControlDeviceFunctionResult.bind( this )
        } );
      }

    }
  }

  //-----------------------------------------------------------------
  //  click "on" button below grid
  //
  ControlButtonOnClick()
  {
    let selected = $( '#ID_FUNCTIONS' + this.dynid ).grid( "option", "selected" );
    if( Array.isArray( selected ) && selected.length > 0 )  {
      let id = selected[ 0 ];
      let record = this.filteredList.find( ( rec => rec['id'] == id ) );
      if( record )  {
        this.DataToggleActivation( record );
        this.FormUpdateButtonStatus();
      }
    }
  }

  //-----------------------------------------------------------------
  //  click "on" button below grid
  //
  ControlButtonOffClick()
  {
    let selected = $( '#ID_FUNCTIONS' + this.dynid ).grid( "option", "selected" );
    if( Array.isArray( selected ) && selected.length > 0 )  {
      let id = selected[ 0 ];
      let record = this.filteredList.find( ( rec => rec['id'] == id ) );
      if( record )  {
        this.DataToggleActivation( record );
        this.FormUpdateButtonStatus();
      }
    }
  }

  
  //-----------------------------------------------------------------
  //  DATA
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  helper to build our internal data
  //
  DataBuildList()
  {
    var errorPermStatusList = {};
    if( Array.isArray( this.filteredList ) ) {
      for( var item of this.filteredList ) {
        errorPermStatusList[ item['id'] ] = {
          error: item['error'],
          permission: item['permission']
        };
      }
    }

    // get the list of functions allowed for this device
    this.filteredList = this.DataBuildListWidget( errorPermStatusList );
    if( AccessApplication.Instance().isRuleEdit ) {
      var filteredRuleList = this.DataBuildListRules( errorPermStatusList );
      this.filteredList = this.filteredList.concat( filteredRuleList );
    }
    
  }

  //-----------------------------------------------------------------
  //  create allowed widgets list
  //
  DataBuildListWidget( errorPermStatusList )
  {
    // get the list of functions allowed for this device
    var filteredList = WidgetDeviceConfigFactory.AllowedWidgetList( "function", this.entity );

    // throw away "none"
    let indexNone = filteredList.findIndex( ( rec ) => rec['id'] == "none" );
    if( indexNone >= 0 ) {
      filteredList.splice( indexNone, 1 );
    }

    // check active status
    let deviceConfig = this.entity['deviceConfig'] || null;
    if( deviceConfig ) {
      for( let listItem of filteredList ) {
        let deviceRecord = deviceConfig.find( ( rec ) => rec['template'] == listItem['id'] );
        var error = ( listItem['id'] in errorPermStatusList ) ? errorPermStatusList[listItem['id']]['error'] : false;
        var permission = ( listItem['id'] in errorPermStatusList ) ? errorPermStatusList[listItem['id']]['permission'] : false;
        if( deviceRecord && ( typeof deviceRecord['sp'] == "string" || deviceRecord['enabled'] === true ) ) {
          listItem['active'] = true;
          listItem['error'] = error;
          listItem['permission'] = permission;
        }
        else {
          listItem['active'] = false;
          listItem['error'] = error;
          listItem['permission'] = permission;
        }
      }
    }

    return filteredList;
  }

  //-----------------------------------------------------------------
  //  create allowed widgets list
  //
  DataBuildListRules( errorPermStatusList )
  {
    // get the list of functions allowed for this device
    var filteredList = WidgetRuleConfigFactory.AllowedRuleList( this.entity );
  
    // check if this rule is used
    let ruleConfig = this.entity['rules'] || null;
    for( let listItem of filteredList ) {
      let ruleRecord = ruleConfig ? ( ruleConfig[ listItem['id'] ] || null ) : null;
      var error = ( listItem['id'] in errorPermStatusList ) ? errorPermStatusList[ listItem['id'] ]['error'] : false;
      var permission = ( listItem['id'] in errorPermStatusList ) ? errorPermStatusList[ listItem['id'] ]['permission'] : false;
      if( ruleRecord ) {
        listItem['active'] = true;
        listItem['error'] = error;
        listItem['permission'] = permission;
      }
      else {
        listItem['active'] = false;
        listItem['error'] = error;
        listItem['permission'] = permission;
      }
    }

    return filteredList;
  }

  //-----------------------------------------------------------------
  //  click on grid check/cross icon
  //
  DataToggleActivation( record )
  {
    if( record['type'] == "widget" )  {
      this.DataToggleActivationWidget( record );
    }
    else {
      this.DataToggleActivationRule( record );
    }

  }  

  //-----------------------------------------------------------------
  //  toggle widget
  //
  DataToggleActivationWidget( record )
  {
    // if active remove it
    if( record['active'] ) {
      WidgetDeviceConfigFactory.RemoveFunctionFromEntity( record['id'], this.entity );
      this.ControlDeviceFunctionResult( this.entity );
    }

    // if not active create new fucntion through dialog
    else {
      DeviceFunctionDialog.Show( {
        data: this.entity,
        options: {
          functionId: record['id']
        },
        result: this.ControlDeviceFunctionResult.bind( this )
      } );
    }

    // re-validate if control had an error
    if( this.Option( "error" ) > 0 ) {
      this.OnValidate();
    }
  }

  //-----------------------------------------------------------------
  //  toggle rule
  //
  DataToggleActivationRule( record )
  {
    // if active remove it
    if( record['active'] ) {
      WidgetRuleConfigFactory.RemoveFromEntity( record['id'], this.entity );
      this.ControlDeviceFunctionResult( this.entity );
    }

    // if not active create new fucntion through dialog
    else {
      DeviceRuleDialog.Show( {
        data: this.entity,
        options: {
          ruleId: record['id']
        },
        result: this.ControlDeviceFunctionResult.bind( this )
      } );
    }

    // re-validate if control had an error
    if( this.Option( "error" ) > 0 ) {
      this.OnValidate();
    }
  }

  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate()
  {
    let numErr = 0;

    // rebuild all error status of list
    for( let deviceFunction of this.filteredList ) {
      if( deviceFunction['active'] ) {

        // validate
        var validateResult = 0;
        if( deviceFunction['type'] == "rule" )  {
          validateResult = WidgetRuleConfigFactory.Validate( deviceFunction['id'], this.entity );
        }
        else {
          validateResult = WidgetDeviceConfigFactory.Validate( deviceFunction['id'], null, null, this.entity );
        }

        if( validateResult > 0 ) {
          numErr++;
          deviceFunction['error'] = true;
        }
        else {
          deviceFunction['error'] = false;
        }
      }
    }

    // set new list to control
    $( '#ID_FUNCTIONS' + this.dynid ).grid( "option", "data", this.filteredList );

    // and set error status of form
    this.Option( "error", numErr );
    return numErr;
  }
  
  OnValidateOff()
  {
    return 0;
  }
  
  //-----------------------------------------------------------------
  //  HELPERS
  //-----------------------------------------------------------------


  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlDeviceFunctionGroup.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlDeviceFunctionGroup.Register( ControlDeviceFunctionGroup.template );
﻿//=============================================================================
//
//  CONTROL DEVICE GROUP IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//  Data:
//
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class ControlCommandGroup extends Viewable
{
  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  static template = "/widget/commandGroup/function";

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createOptions )
  {
    // no clone needed (done in viewable)
    this.entity = createOptions['entity'];
    this.modules = createOptions['module'];
    this.commandPath = createOptions['commandPath'];
    this.uidPrefix = createOptions['uidPrefix'];
    this.commandList = $.object.GetPathValue( this.entity, this.commandPath ); //this.ObjectGetPathValue( this.entity, this.commandPath );

    // set label text 
    let $label = this.$container.find( "label" ).first();
    $label.text( $.lang( createOptions['text'] ) );

    // create grid
    $( '#ID_FUNCTIONS' + this.dynid ).grid( {
      size: CONST.size.XL,
      height: 8,
      selection: 'single',
      data: [],
      dataId: 'id',
      scrollbar: 'overlay', // 'overlay', 'static'
      result: this.ControlListSelected.bind( this ),
      dblclick: this.ControlListDblClick.bind( this ),
      headerHidden: true,
      header: {
        text: { text: "", size: CONST.size.XL - 4 },
        warning: { text: "", size: 4 }
      },      
      renderCell: ( id, rec ) => { 
        if( id == "text" )  return rec; 
        return {};  
        // future: { icon: "mdi mdi-security yellow-text" };
      },
      renderStatus: this.ControlGridRenderStatus.bind( this )
    } );

    $( '#ID_NEW' + this.dynid ).button( {
      size: CONST.size.M,
      text: "{$commandgrp_button_new}",
      click: this.ControlButtonNewClick.bind( this )
    } );
    $( '#ID_EDIT' + this.dynid ).button( {
      size: CONST.size.M,
      text: "{$commandgrp_button_edit}",
      click: this.ControlButtonEditClick.bind( this )
    } );
    $( '#ID_DELETE' + this.dynid ).button( {
      size: CONST.size.XS,
      icon: "mdi mdi-delete",
      click: this.ControlButtonDeleteClick.bind( this )
    } );
    


    // filter list and build controls
    this.DataBuildList();
    this.FormRebuild();
    this.ControlListSelected( [] );

  }

  //-----------------------------------------------------------------
  //
  //
  OnEvent( eventName, param, data )
  {
    // rebuild complete UI
    if( eventName == "option" && param == "rebuild" ) {
      this.entity = $.clone( data );
      this.DataBuildList();
      this.FormRebuild();
    }

    // get references
    if( eventName == "option" && param == "references" ) {
      return this.FormReferences();
    }
  }

  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  create controls after form creation (once)
  //
  FormRebuild(  )
  {
    $( '#ID_FUNCTIONS' + this.dynid ).grid( "option", "data", this.filteredList );
  }

  //-----------------------------------------------------------------
  //  inform upper layer about changed entity
  //
  FormFireResult()
  {
    this.FireEvent( "result", $.clone( this.entity ) );
  }

  //-----------------------------------------------------------------
  //  get references list from widgets
  //
  FormReferences()
  {
    let referenceList = [];

    for( let record of this.filteredList )  {
      let result = WidgetDeviceConfigFactory.ReferencesByKey( record['id'], this.entity, this.commandPath );
      if( Array.isArray( result ) ) {
        $.array.Join( referenceList, result );
      }
    }
    return referenceList;
  }

  //-----------------------------------------------------------------
  //  CONTROL
  //-----------------------------------------------------------------


  //-----------------------------------------------------------------
  //  set status for a grid line (error or normal)
  //
  ControlGridRenderStatus( recordId, record )
  {
    return ( record['error'] === true ) ? "red" : "normal";
  }

  //-----------------------------------------------------------------
  //  click on edit button beside description line
  //
  ControlDeviceCommandResult( entity )
  {
    if( $.isPlainObject( entity ) ) {
      this.Option( "modified", 1 );
      this.entity = entity;
      this.DataBuildList();
      this.FormRebuild();
      this.FormFireResult();
    }
  }

  //-----------------------------------------------------------------
  //  double click on grid line
  //
  ControlListDblClick( id )
  {
    this.ControlButtonEditClick();
  }

  //-----------------------------------------------------------------
  //  click on edit button beside description line
  //  
  ControlListSelected( selected )
  {
    let isEnabled = ( Array.isArray( selected ) && selected.length > 0 );
    $( '#ID_EDIT' + this.dynid ).button( "option", "status", isEnabled ? "normal" : "disabled" ); 
    $( '#ID_DELETE' + this.dynid ).button( "option", "status", isEnabled ? "normal" : "disabled"  ); 
  }

  //-----------------------------------------------------------------
  //  click on edit button below description line
  //
  ControlButtonEditClick( )
  {
    let selected = $( '#ID_FUNCTIONS' + this.dynid ).grid( "option", "selected" );
    if( Array.isArray( selected ) && selected.length > 0 )  {

      let commandRecord = this.commandList.find( ( rec ) => rec['uid'] == selected[ 0 ] );
      let tempEntity = $.clone( this.entity );
      let tempCommandList = $.object.GetPathValue( tempEntity, this.commandPath );  //this.ObjectGetPathValue( tempEntity, this.commandPath );

      // start device keys dialog
      DeviceKeysDialog.Show( {
        data: tempEntity,  //this.entity,
        options: {
          uid: commandRecord['uid'],
          deviceConfig: tempCommandList
        },
        result: this.ControlDeviceCommandResult.bind( this )
      } );      
    }
  }

  //-----------------------------------------------------------------
  //  click "on" button below grid
  //
  ControlButtonNewClick()
  {
    // create a copy and add an empty record
    let tempEntity = $.clone( this.entity );
    let tempCommandList = $.object.GetPathValue( tempEntity, this.commandPath ); //this.ObjectGetPathValue( tempEntity, this.commandPath );
    let newUid = this.uidPrefix + $.uuid();
    tempCommandList.push( {
      'module': "TIMELINE",
      'template': "none",
      'in': { sysid: 1 },
      'uid': newUid,
      'param': {
      }
    } );

    // start device keys dialog
    DeviceKeysDialog.Show( {
      data: tempEntity,  //this.entity,
      options: {
        uid: newUid,
        deviceConfig: tempCommandList
      },
      result: this.ControlDeviceCommandResult.bind( this )
    } );


  }

  //-----------------------------------------------------------------
  //  click "on" button below grid
  //
  ControlButtonDeleteClick()
  {
    let selected = $( '#ID_FUNCTIONS' + this.dynid ).grid( "option", "selected" );
    if( Array.isArray( selected ) && selected.length > 0 )  {
      let commandIndex = this.commandList.findIndex( ( rec ) => rec['uid'] == selected[ 0 ] );
      let commandRecord = this.commandList[ commandIndex ];
      if( commandIndex >= 0 )  {
        WidgetDeviceConfigFactory.RemoveFromEntity( commandRecord['uid'], null, this.entity );
        this.commandList.splice( commandIndex, 1 );
        this.ControlDeviceCommandResult( this.entity );
      }
    }
  }

  
  //-----------------------------------------------------------------
  //  DATA
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  helper to build our internal data
  //
  DataBuildList()
  {
    // get command list from path and remove action "none"
    this.commandList = $.object.GetPathValue( this.entity, this.commandPath );

    // "commandList" need to be a reference into entity, so use special function ObjectSetPathValue
    let newCommandList = this.commandList.filter( ( rec ) => rec['template'] != "none" );
    $.object.SetPathValue( this.entity, this.commandPath, newCommandList );

    // build grid list
    this.filteredList = [];
    for( let command of this.commandList ) {
      if( command['module'] == "TIMELINE" )  {
        let descriptionLine = WidgetDeviceConfigFactory.SingleDescriptionLine( command, this.entity );
        let text = descriptionLine['text'];
        let icon = WidgetDeviceConfigFactory.WidgetIcon( command['template'] );
        this.filteredList.push( {
          id: command['uid'],
          template: command['template'],
          text: text,
          icon: icon
        });
      }
    }
  }

  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate()
  {
    let numErr = 0;

    for( let record of this.filteredList ) {
      let validateResult = WidgetDeviceConfigFactory.Validate( record['template'], record['id'], null, this.entity );
      if( validateResult > 0 ) {
        numErr++;
        record['error'] = true;
      }
      else {
        record['error'] = false;
      }
    }

    $( '#ID_FUNCTIONS' + this.dynid ).grid( "option", "data", this.filteredList );

    this.Option( "error", numErr );
    return numErr;
  }
  
  OnValidateOff()
  {
    return 0;
  }
  
  //-----------------------------------------------------------------
  //  HELPERS
  //-----------------------------------------------------------------

  ObjectSetPathValue( object, path, newValue ) 
  {
    let result = null;
    let pathList = path.split( "." );
    let currentObject = object;

    for( let key of pathList ) {
      if( key in currentObject ) {
        result = currentObject = currentObject[key];
      }
    }

    if( result ) {
      result = newValue;
    }

  }

  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Build( $element, createOptions )
  {
    createOptions['view'] = ControlCommandGroup.template;
    $element.viewable( createOptions );
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

ControlCommandGroup.Register( ControlCommandGroup.template );
//=============================================================================
//
//  FUNCTION WIDGET BUILDING AUTOMATION URL IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          10.06.2021
//=============================================================================


//=============================================================================
//  CLASS
//=============================================================================

class WidgetRuleGenericUI extends Viewable
{
  //===========================================================================
  //  INSTANCE
  //===========================================================================

  //---------------------------------------------------------------------------
  //  standard viewable load
  //
  Load( createData )
  {
    this.ruleId = createData['ruleId'];
    this.rule = Data.Instance().Rule( this.ruleId );
    this.langId = AccessApplication.Instance().Language();
    this.entity = createData['entity'];
    this.ruleConfig = ( 'rules' in this.entity ) ? this.entity['rules'][ this.ruleId ] : null;
    this.preventFirstTimeValidate = false;
    if( !this.ruleConfig )  {
      this.preventFirstTimeValidate = true;
    }
    this.controlList = {};
    this.isStatepoint = false;
    this.status = "statusIdle";

    if( 'required' in this.rule && this.rule['required'] )  {
      var requiredWidget = this.rule['required'].split( ":" );
      this.isRequiredError = ! this.DataCheckRequiredStatepoint( requiredWidget[ 1 ] );
    }

    if( 'statepoint' in this.rule && this.rule['statepoint'] ) {
      this.isStatepoint = true;
      this.DataCreateStatepoint( );
      this.FormCreateStatepointLabel();
    }

    this.FormCreateControls( this.rule['parameter'] );

    if( this.isRequiredError ) {
      setTimeout( () => {
        this.ShowToast( requiredWidget[2] );
        this.StatusError();
      }, 50 )
    }

  }

  //---------------------------------------------------------------------------
  //  viewable event function
  //
  OnEvent( eventName, param, data )
  {
    //console.log( eventName, param, data );
    
    if( eventName == "option" && param == "save" ) {
      return this.DataSave( data );
    }
    else if( eventName == "modified" )  {
      this.preventFirstTimeValidate = false;
    }
  }


  //===========================================================================
  //  FORM
  //===========================================================================

  //---------------------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  DataCheckRequiredStatepoint( requiredStatepoint )
  {
    if( ( 'sp' in this.entity )
     && ( requiredStatepoint in this.entity['sp'] ) 
      ) {
      return true;
    }
    
    return false;
  }

  //---------------------------------------------------------------------------
  //  key-dialog is calling to save the data into the entity
  //
  DataSave( entity )
  {
    if( !( 'rules' in entity ) ) {
      entity['rules'] = {}  ;
    }

    entity['rules'][ this.ruleId ] = { 
      config: { 
        parameter: { } 
      }, 
      run: this.rule['run'] 
    };    

    var ruleConfig = {};
    for( var paramName in this.controlList ) {
      var $ctrl = this.controlList[ paramName ];
      var result = $ctrl.viewable( "option", "value" );
      ruleConfig[ paramName ] = result;
    }

    entity['rules'][ this.ruleId ]['config']['parameter'] = ruleConfig;

    // sp
    if( this.isStatepoint ) {

      this.statepoint['label'] = $( "#ID_LABEL" + this.dynid ).viewable( "option", "value" );
      entity['rules'][ this.ruleId ]['config']['sp'] = this.rule['statepoint']['name'];
      entity['sp'][ this.rule['statepoint']['name'] ] = this.statepoint;
      entity['permission'] = this.entity['permission'];
    }

    return entity;
  }

  //---------------------------------------------------------------------------
  //  create local statepoint if needed
  //
  DataCreateStatepoint()
  {
    var spTemplate = this.rule['statepoint'];
    if( !( spTemplate['name'] in this.entity['sp'] ) )  {
      this.entity['sp'][ spTemplate['name'] ] = {
        label: spTemplate['label'][ this.langId ],
        address: ( Data.Instance().RootEntid() + "/" + this.entity['entid'] + "/" + spTemplate['name'] ), 
        type: spTemplate['type'] || "BOOL",
        name: spTemplate['name'],
        default: spTemplate['default'] || false,
        persistent: spTemplate['persistent'] || false,
        hidden: spTemplate['hidden'] || false,
        template: this.ruleId
      };
    }

    this.statepoint = this.entity['sp'][ this.rule['statepoint']['name'] ];

    // ensure default permission record exists
    WidgetDeviceConfigBase.HelperAddDefaultPermission( spTemplate['name'], this.entity );

  }

  //===========================================================================
  //  FORM
  //===========================================================================

  //---------------------------------------------------------------------------
  //  in case of a statepoint, create label
  //
  FormCreateStatepointLabel()
  {
    var $line = $( $.views.Html( "/widget/ruleConfig/ruleGenericUi/label", {}, this.dynid ) );
    $( "#ID_CONTROLS" + this.dynid ).append( $line );

    // build label
    ControlLabel.Build( $( "#ID_LABEL" + this.dynid ), {
      label: this.statepoint['label'],
      entityName: this.entity['name'],
      functionName: ""
    } );
    $( "#ID_CONTROLS" + this.dynid ).append( $line );

  }

  //---------------------------------------------------------------------------
  //  create controls from parameter list
  //
  FormCreateControls( ruleList )
  {
    for( var paramName in ruleList ) {
      var parameter = ruleList[ paramName ];

      switch( parameter['type'] )  {
        case "statepoint":  
          this.controlList[ paramName ] = this.FormCreateControlStatepoint( paramName, parameter );
          break;

        case "integer":
          this.controlList[ paramName ] = this.FormCreateControlInteger( paramName, parameter );
          break;

        case "calltarget":
          this.controlList[ paramName ] = this.FormCreateControlCalltarget( paramName, parameter );
          break;

        case "calltarget-list":
          this.controlList[ paramName ] = this.FormCreateControlCalltargetList( paramName, parameter );
          break;
      }
    }
  }

  //---------------------------------------------------------------------------
  //  create parameter (statepoint)
  //
  FormCreateControlStatepoint( paramName, parameter )
  {
    var createOptions = {
      paramName: paramName, 
      parameter: parameter,
      entity: this.entity,
      ruleId: this.ruleId,
      ruleConfig: this.ruleConfig
    };
    return RuleControlStatepoint.Append( $( "#ID_CONTROLS" + this.dynid ), createOptions );
  }

  //---------------------------------------------------------------------------
  //  create parameter (integer)
  //
  FormCreateControlInteger( paramName, parameter )
  {
    var createOptions = {
      paramName: paramName,
      parameter: parameter,
      entity: this.entity,
      ruleId: this.ruleId,
      ruleConfig: this.ruleConfig
    };
    return RuleControlInteger.Append( $( "#ID_CONTROLS" + this.dynid ), createOptions );    
  }


  //---------------------------------------------------------------------------
  //  create parameter (call target)
  //
  FormCreateControlCalltarget( paramName, parameter )
  {
    var createOptions = {
      paramName: paramName,
      parameter: parameter,
      entity: this.entity,
      ruleId: this.ruleId,
      ruleConfig: this.ruleConfig
    };
    return RuleControlCalltarget.Append( $( "#ID_CONTROLS" + this.dynid ), createOptions );    
  }

  //---------------------------------------------------------------------------
  //  create parameter (call target list)
  //
  FormCreateControlCalltargetList( paramName, parameter )
  {
    var createOptions = {
      paramName: paramName,
      parameter: parameter,
      entity: this.entity,
      ruleId: this.ruleId,
      ruleConfig: this.ruleConfig
    };
    return RuleControlCalltargetList.Append( $( "#ID_CONTROLS" + this.dynid ), createOptions );        
  }


  //===========================================================================
  //  VALIDATE
  //===========================================================================

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidate()
  {
    if( this.preventFirstTimeValidate ) {
      return 0;
    }

    var numErr = 0;
    for( var paramName in this.controlList ) {
      var $ctrl = this.controlList[ paramName ];
      numErr += $ctrl.viewable( "validate" );
    }
    return numErr;
  }

  //---------------------------------------------------------------------------
  // viewable override: validate
  //
  OnValidateOff()
  {
    for( var paramName in this.controlList ) {
      var $ctrl = this.controlList[ paramName ];
      $ctrl.viewable( "validate", false );
    }
  }

  //-----------------------------------------------------------------------------
  //  STATUS
  //-----------------------------------------------------------------------------

  StatusError()
  {
    this.status = "statusError";
    this.Option( "error", 1 );
  }

  //-----------------------------------------------------------------------------
  //  TOAST
  //-----------------------------------------------------------------------------

  ShowToast( requiredWidget )
  {
    var widgetName = WidgetDeviceConfigFactory.WidgetName( requiredWidget );
    var message = $.lang( "{$rule_required_error_notice}" );
    message = message.replace( "{{widgetName}}", widgetName );
    $( "#ID_TOAST" + this.dynid ).toast( {
      text: message,
      icon: "mdi mdi-alert-octagon",
      status: "yellow",
      height: 3,
      size: -1,
      time: -1,
    } );
  }

  //===========================================================================
  //  STATICS
  //===========================================================================

  static LocalStatepointName( ruleId, entity )
  {
    if( ( 'rules' in entity ) && ( ruleId in entity['rules'] ) && ( 'sp' in entity['rules'][ ruleId ]['config'] ) ) {
      return entity['rules'][ ruleId ]['config']['sp'];
    }
    return null;
  }

  static Validate( ruleId, entity )
  {
    // get current configuration
    var config = null;
    if( ( 'rules' in entity ) && ( ruleId in entity['rules'] ) && ( 'parameter' in entity['rules'][ ruleId ]['config'] ) ) {
      config = entity['rules'][ ruleId ]['config']['parameter'];
    }

    // no configuration => no error
    if( ! config ) {
      return 0;
    }

    // get rule parameter definition
    var rule = Data.Instance().Rule( ruleId );
    var paramDefinition = rule['parameter'];
    var numErr = 0;

    for( var paramName in paramDefinition ) {
      var parameter = paramDefinition[ paramName ];

      switch( parameter['type'] ) {
        case "statepoint": 
          numErr += RuleControlStatepoint.Validate( ruleId, config[ paramName ], entity ); 
          break;

        case "integer":
          numErr += RuleControlInteger.Validate( ruleId, config[ paramName ], entity ); 
          break;

        case "calltarget":
          numErr += RuleControlCalltarget.Validate( ruleId, config[ paramName ], entity ); 
          break;

        case "calltarget-list":
          numErr += RuleControlCalltargetList.Validate( ruleId, config[ paramName ], entity ); 
          break;

      }
    }

    return numErr;
  }

  //-----------------------------------------------------------------
  //  get the references used by this rule
  //
  static References( ruleId, entity )
  {
    // get current configuration
    var config = null;
    if( ( 'rules' in entity ) && ( ruleId in entity['rules'] ) && ( 'parameter' in entity['rules'][ ruleId ]['config'] ) ) {
      config = entity['rules'][ ruleId ]['config'];
    }

    // no configuration => no refs
    if( !config ) {
      return 0;
    }

    // get rule parameter definition
    var rule = Data.Instance().Rule( ruleId );
    var paramDefinition = rule['parameter'];
    var referenceList = [
      "rule:" + rule['run']
    ];

    // local statepoints need to be added
    if( 'sp' in config ) {
      referenceList.push( "sp:" + Data.Instance().RootEntid() + "/" + entity['entid'] + "/" + config['sp'] )
    }

    // get references from all parameters
    for( var paramName in paramDefinition ) {
      var parameter = paramDefinition[paramName];

      switch( parameter['type'] ) {
        case "statepoint":
          var result = RuleControlStatepoint.References( ruleId, config['parameter'][ paramName ], entity );
          if( result ) $.array.Join( referenceList, result );
          break;

        case "integer":
          var result = RuleControlInteger.References( ruleId, config['parameter'][ paramName ], entity );
          if( result ) $.array.Join( referenceList, result );
          break;

        case "calltarget":
          var result = RuleControlCalltarget.References( ruleId, config['parameter'][ paramName ], entity );
          if( result ) $.array.Join( referenceList, result );
          break;

        case "calltarget-list":
          var result = RuleControlCalltargetList.References( ruleId, config['parameter'][ paramName ], entity );
          if( result ) $.array.Join( referenceList, result );
          break;

      }
    }

    return referenceList;
    
  }  

  //-----------------------------------------------------------------
  //  remove rule from entity
  //
  static Remove( ruleId, entity )
  {
    // get current configuration
    var config = null;
    if( ( 'rules' in entity ) && ( ruleId in entity['rules'] ) && ( 'parameter' in entity['rules'][ruleId]['config'] ) ) {
      config = entity['rules'][ruleId]['config'];
    }

    delete entity['rules'][ ruleId ];

    // no configuration => no refs
    if( config ) {
      var spName = config['sp'] || null;
      if( spName ) {
        WidgetDeviceConfigBase.HelperRemoveStatepoint( spName, entity );
        WidgetDeviceConfigBase.HelperRemovePermission( spName, entity );
      }
   }    

  }

  //===========================================================================
  //  BUILDER
  //===========================================================================

  static Build( $element, options )
  {
    options['view'] = '/widget/ruleConfig/ruleGenericUi';
    $element.viewable( options );
  }


}

//=============================================================================
//  REGISTER
//=============================================================================

WidgetRuleGenericUI.Register( '/widget/ruleConfig/ruleGenericUi' );
//=============================================================================
//
//  CONTROL CALL PROFILE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class RuleControlBase extends Viewable
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/ruleConfig/ruleControlBase";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  isInitialized = false;

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createData )
  {
  }

  //-----------------------------------------------------------------
  //  FORM
  //-----------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  basic layout help/no help
  //
  FormCreateBasicLayout( paramName, parameter )
  {
    var $line = null;
    var createOptions = {};

    // create line with or w/o help topic
    if( 'help' in parameter ) {
      var topicId = "rule/" + this.rule['id'] + "/" + paramName;
      createOptions = {
        label: parameter['label'][this.langId],
        helpTopicId: topicId
      };
      $line = $( $.views.Html( RuleControlBase.template + "/help", createOptions, this.dynid ) );
    }
    else {
      createOptions = {
        label: parameter['label'][this.langId]
      };
      $line = $( $.views.Html( RuleControlBase.template + "/line", createOptions, this.dynid ) );
    }

    // add line
    $( this.$parent ).append( $line );

    return $line;
  }    

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( validateControl, selected )
  {
    return 0;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

//RuleControlBase.Register( RuleControlBase.template );
﻿//=============================================================================
//
//  CONTROL CALL PROFILE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class RuleControlStatepoint extends RuleControlBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/ruleConfig/ruleControlStatepoint";

  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createData )
  {
    this.ruleId = createData['ruleId'];
    this.rule = Data.Instance().Rule( this.ruleId );
    this.ruleConfig = createData['ruleConfig']; 
    this.langId = AccessApplication.Instance().Language();
    this.entity = createData['entity'];
    this.parameter = createData['parameter'];
    this.paramName = createData['paramName'];
   
    this.FormCreateBasicLayout( this.paramName, this.parameter );
    this.FormFill( );

  }

  //-----------------------------------------------------------------
  //  viewable event handler
  //
  OnEvent( eventName, param, data )  
  {
    // implement standard ( "option", "value" .. ) handler
    if( eventName == "option" && param == "value" ) {
      if( data ) {
        console.warn( "RuleControlInteger: option/value no setter implemented!" );
        return;
      }
      return this.FormRead( );
    }
  }


  //---------------------------------------------------------------------------
  //  FORM
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  statepoint droplist
  //
  FormFill( )
  {
    // allowed names parameter
    var allowedNames = null;
    if( 'allowedNames' in this.parameter['param'] ) {
      allowedNames = this.parameter['param']['allowedNames'];
      if( allowedNames[ 0 ] == "*" ) {
        allowedNames = null;
      }
    }

    // value
    var value = undefined;
    if( this.ruleConfig && ( 'config' in this.ruleConfig ) && ( 'parameter' in this.ruleConfig['config'] ) ) {
      value = this.ruleConfig['config']['parameter'][ this.paramName ] || undefined;
    }
    else if( 'default' in this.parameter['param'] ) {
      value = this.parameter['param']['default'];
      value = value.replace( "{server.id}", Data.Instance().RootEntid() );
      value = value.replace( "{entity.id}", this.entity['entid'] );
    }

    // build the control
    ControlStatepointList.Build( $( "#ID_RULE_CONTROL" + this.dynid ), {
      inquirer: this.entity['entid'],
      entity: this.entity,
      selected: [ value ],
      forbiddenEntityId: [ this.entity['entid'] ],
      pinnedEntityId: this.entity['entid'],
      allowedTypes: this.parameter['param']['allowedTypes'],
      allowedNames: allowedNames || undefined,
      permission: this.parameter['param']['permission'] || "rw"
    } );

  }

  //---------------------------------------------------------------------------
  //  get form data
  //
  FormRead()
  {
    var selected = $( "#ID_RULE_CONTROL" + this.dynid ).viewable( "option", "selected" );
    if( Array.isArray( selected ) && selected.length > 0 ) {
      return selected[ 0 ];
    }
    return "";
  }


  //-----------------------------------------------------------------
  //  FORM HELPER
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    if( ( this.parameter['param']['mandantory'] || false ) !== true ) {
      return 0;
    }

    if( param === false ) {
      return $( "#ID_RULE_CONTROL" + this.dynid ).viewable( "validate", false );
    }
    return $( "#ID_RULE_CONTROL" + this.dynid ).viewable( "validate" );
  }

  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Append( $parent, createOptions )
  {
    // build new parent element
    var $element = $( "<div>" );
    $parent.append( $element );

    // create viewable inside
    createOptions['view'] = RuleControlStatepoint.template;
    $element.viewable( createOptions );

    // return new element
    return $element;
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( ruleId, ruleConfig, entity )
  {
    var spinfo = Data.Instance().StatepointInfo( ruleConfig, entity );
    return ( spinfo ) ? 0 : 1;
  }

  //-----------------------------------------------------------------
  //  get list of used references
  //
  static References( ruleId, param, entity )
  {
    var ref = "sp:" + param;
    return [ ref ];
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

RuleControlStatepoint.Register( RuleControlStatepoint.template );
﻿//=============================================================================
//
//  CONTROL CALL PROFILE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class RuleControlCalltarget extends RuleControlBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/ruleConfig/ruleControlCalltarget";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  isInitialized = false;
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createData )
  {
    this.ruleId = createData['ruleId'];
    this.rule = Data.Instance().Rule( this.ruleId );
    this.ruleConfig = createData['ruleConfig']; 
    this.langId = AccessApplication.Instance().Language();
    this.entity = createData['entity'];
    this.parameter = createData['parameter'];
    this.paramName = createData['paramName'];
   
    this.FormCreateBasicLayout( this.paramName, this.parameter );
    this.FormLoad( this.ruleConfig );

  }

  //-----------------------------------------------------------------
  //  viewable event handler
  //
  OnEvent( eventName, param, data )  
  {
    // implement standard ( "option", "value" .. ) handler
    if( eventName == "option" && param == "value" ) {
      if( data ) {
        console.warn( "RuleControlCalltarget: option/value no setter implemented!" );
        return;
      }
      return this.FormRead( );
    }
  }


  //---------------------------------------------------------------------------
  //  FORM
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  fill data
  //
  FormLoad( )
  {
    var value = undefined;
    if( this.ruleConfig && ( 'config' in this.ruleConfig ) && ( 'parameter' in this.ruleConfig['config'] ) ) {
      value = this.ruleConfig['config']['parameter'][ this.paramName ] || undefined;
    }

    // build the control
    ControlCalltargetList.Build( $( "#ID_RULE_CONTROL" + this.dynid ), {
      size: CONST.size.L,
      //result: this.ControlForwardTargetResult.bind( this ),
      forbiddenEntityId: this.entity['entid'],
      addressBook: null,
      selected: [ value ]
    } ); 
  }

  //---------------------------------------------------------------------------
  //  read data
  //
  FormRead()
  {
    var selected = $( "#ID_RULE_CONTROL" + this.dynid ).viewable( "option", "selected" );
    if( Array.isArray( selected ) && selected.length > 0 )  {
      return selected[ 0 ];
    }
    return "";
  }


  //-----------------------------------------------------------------
  //  FORM HELPER
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  OnValidate( param )
  {
    if( ( this.parameter['param']['mandantory'] || false ) !== true ) {
      return 0;
    }

    if( param === false ) {
      return $( "#ID_RULE_CONTROL" + this.dynid ).viewable( "validate", false );
    }
    return $( "#ID_RULE_CONTROL" + this.dynid ).viewable( "validate" );
  }

  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Append( $parent, createOptions )
  {
    // build new parent element
    var $element = $( "<div>" );
    $parent.append( $element );

    // create viewable inside
    createOptions['view'] = RuleControlCalltarget.template;
    $element.viewable( createOptions );

    // return new element
    return $element;
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( ruleId, ruleConfig, entity )
  {
    var entinfo = Data.Instance().Entinfo( ruleConfig );
    return ( entinfo ) ? 0 : 1;
  }

  //-----------------------------------------------------------------
  //  get list of used references
  //
  static References( ruleId, param, entity )
  {
    var result = [];
    if( Array.isArray( param ) ) {
      for( var value of param ) {
        result.push( "call:" + value );
      }
    }
    else {
      result.push( "call:" + param );
    }

    return result;
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

RuleControlCalltarget.Register( RuleControlCalltarget.template );
﻿//=============================================================================
//
//  CONTROL CALL PROFILE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class RuleControlCalltargetList extends RuleControlBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/ruleConfig/ruleControlCalltargetList";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  isInitialized = false;
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createData )
  {
    this.ruleId = createData['ruleId'];
    this.rule = Data.Instance().Rule( this.ruleId );
    this.ruleConfig = createData['ruleConfig']; 
    this.langId = AccessApplication.Instance().Language();
    this.entity = createData['entity'];
    this.parameter = createData['parameter'];
    this.paramName = createData['paramName'];

    // member dataset
    this.memberDataset = new Dataset( { id: "entid" } );

    // call target dataset
    this.callTargetDataset = Data.Instance().CalltargetDataset();
    this.callTargetDataview = this.callTargetDataset.BuildView( 'default', {
      rebuildAlways: true,
      sortOrder: "+",
      sortKey: "text",
      filter: this.ControlCalltargetDataviewFilter.bind( this )
    } );

   
    this.FormCreateBasicLayout( this.paramName, this.parameter );
    this.FormLoad( );

  }

  //-----------------------------------------------------------------
  //  viewable event handler
  //
  OnEvent( eventName, data, param  )  
  {
    // implement standard ( "option", "value" .. ) handler
    if( eventName == "option" && data == "value" ) {
      if( param ) {
        console.warn( "RuleControlCalltarget: option/value no setter implemented!" );
        return;
      }
      return this.FormRead( );
    }
  }


  //---------------------------------------------------------------------------
  //  FORM
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  set form data
  //
  FormLoad( )
  {
    var $ctrlGroup = $( $.views.Html( "/widget/ruleConfig/ruleControlCalltarget/control", {}, this.dynid ) );
    $( "#ID_RULE_CONTROL" + this.dynid ).append( $ctrlGroup );

    var data = [];
    if( this.ruleConfig && ( 'config' in this.ruleConfig ) && ( 'parameter' in this.ruleConfig['config'] ) ) {

      var value = this.ruleConfig['config']['parameter'][ this.paramName ] || null;
      if( Array.isArray( value ) ) {
        for( var entityId of value ) {
          var entinfo = Data.Instance().Entinfo( entityId );
          if( !entinfo )  {
            entinfo = { _preview: { status: "red", id: entityId, icon: "mdi mdi-help", text: "{$const_deleted_item}" } };
          }
          var record = {
            entid: entinfo['_preview']['id'],
            text: entinfo['_preview']['text'],
            icon: entinfo['_preview']['icon'],
            status: entinfo['_preview']['status']
          };
          data.push( record );
        }
      }
      else {
        var entinfo = Data.Instance().Entinfo( value );
        if( !entinfo ) {
          entinfo = { _preview: { status: "red", id: entityId, icon: "mdi mdi-help", text: "{$const_deleted_item}" } };
        }
        var record = {
          entid: entinfo['_preview']['id'],
          text: entinfo['_preview']['text'],
          icon: entinfo['_preview']['icon'],
          status: entinfo['_preview']['status']
        };
        data.push( record );
      }

    }
    this.memberDataset.SetData( data );

    // build the control
    $( "#ID_CALLTARGET_GRID" + this.dynid ).grid( {
      size: CONST.size.L,
      height: 4,
      data: this.memberDataset,
      headerHidden: true,
      scrollbar: "overlay",
      validate: this.ValidateCalltargetList.bind( this ),
      header: {
        'text': { text: "", size: CONST.size.L, align: "left", resizeable: false, sortable: false },
      },
      renderCell: ( columnId, record ) => {
        if( columnId == "text" ) {
          return {
            text: record['text'],
            icon: record['icon']
          };
        }
        return { text: record[ columnId ] };
      }, 
      result: this.ControlListboxSelect.bind( this )
    } );  

    // build the control
    $( "#ID_CALLTARGET_ADD" + this.dynid ).button( {
      size: CONST.size.M,
      text: "{$rule_calltarget_add}",
      click: () => {
        this.callTargetDataview.Update();
        $( '#ID_CALLTARGET_POPUP' + this.dynid ).listbox( "show" );
      }
    } );  

    $( '#ID_CALLTARGET_POPUP' + this.dynid ).listbox( {
      size: 35,
      height: 5,
      type: 'listbox',
      search: true,
      focus: true,
      status: "normal",
      searchPlaceholder: "{$const_common_search_placeholder}",
      anchor: "#ID_CALLTARGET_ADD" + this.dynid,
      position: "bottomleft",
      data: this.callTargetDataview,
      result: this.ControlCalltargetPopupResult.bind( this )
    } );    

    // build the control
    $( "#ID_CALLTARGET_DELETE" + this.dynid ).button( {
      size: CONST.size.XS,
      status: "disabled",
      icon: "mdi mdi-delete",
      click: this.ControlCalltargetDeleteClick.bind( this )
    } );  
  }

  //---------------------------------------------------------------------------
  //  get form data
  //
  FormRead()
  {
    var result = [];
    this.memberDataset.Each( ( id, record ) => {
      result.push( id );
    } );
    if( result.length == 1 ) {
      result = result[ 0 ];
    }

    return result;
  }

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------

  ControlListboxSelect( selected )
  {
    if( selected.length > 0 ) {
      $( "#ID_CALLTARGET_DELETE" + this.dynid ).button( "option", "status", "normal" );
    }
    else {
      $( "#ID_CALLTARGET_DELETE" + this.dynid ).button( "option", "status", "disabled" );
    }
  }

  //---------------------------------------------------------------------------
  //  filter in add member dataview the already existing entries
  //
  ControlCalltargetDataviewFilter( entid, record )
  {
    let isAllowed = true;

    if( isAllowed && entid == this.entity['entid'] ) {
      isAllowed = false;
    }
    if( isAllowed && this.memberDataset.Has( entid ) ) {
      isAllowed = false;
    }
    if( isAllowed && record['class'] == "addrBook" ) {
      isAllowed = false;
    }
    if( isAllowed && record['class'] == "group" ) {
      isAllowed = false;
    }

    return isAllowed;
  }  

  //---------------------------------------------------------------------------
  //  popup returned
  //
  ControlCalltargetPopupResult( resultList )
  {
    for( let entid of resultList ) {

      let addrEntinfo = Data.Instance().Entinfo( entid );

      let record = {};
      record['entid'] = entid;
      record['text'] = addrEntinfo['name'];
      record['icon'] = CONST_CLASS_ICO[ addrEntinfo['class'] ];
      this.memberDataset.Add( record );
    }
  } 

  //---------------------------------------------------------------------------
  //  delete button clicked
  //
  ControlCalltargetDeleteClick()
  {
    let selected = $( `#ID_CALLTARGET_GRID${ this.dynid }` ).grid( "option", "selected" );
    this.memberDataset.Delete( selected[ 0 ] );
    this.ControlListboxSelect( [] );
    this.Option( "modified", 1 );
    if( this.Option( "error" ) > 0 ) {
      this.OnValidate( true );
    }
  }


  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  validate
  //
  OnValidate( param )
  {
    if( ( this.parameter['param']['mandantory'] || false ) !== true ) {
      return 0;
    }

    if( param === false ) {
      return $( "#ID_CALLTARGET_GRID" + this.dynid ).grid( "validate", false );
    }
    return $( "#ID_CALLTARGET_GRID" + this.dynid ).grid( "validate" );
  }

  //-----------------------------------------------------------------
  //  check if list is empty
  //
  ValidateCalltargetList( validateControl )
  {
    // check for empty list
    if( this.memberDataset.Size() == 0 ) {
      validateControl.error = "{$rule_calltarget_error_empty}";
      return 1;
    }

    // check for deleted items in list
    var numErr = 0;
    this.memberDataset.Each( ( id, record ) =>  {
      if( ! Data.Instance().Entinfo( id ) ) {
        numErr++;
      }
    } );
    if( numErr ) {
      validateControl.error = "{$rule_calltarget_error_deleted}";
      return 1;
    }

    // no error
    validateControl.error = null;
    return 0;
  }

  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Append( $parent, createOptions )
  {
    // build new parent element
    var $element = $( "<div>" );
    $parent.append( $element );

    // create viewable inside
    createOptions['view'] = RuleControlCalltargetList.template;
    $element.viewable( createOptions );

    // return new element
    return $element;
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( ruleId, ruleConfig, entity )
  {
    if( Array.isArray( ruleConfig ) )  {
      for( var entid of ruleConfig ) {
        if( ! Data.Instance().Entinfo( entid ) ) {
          return 1;
        }
      }
    }
    else {
      var entinfo = Data.Instance().Entinfo( ruleConfig );
      return ( entinfo ) ? 0 : 1;
    }
  }

  //-----------------------------------------------------------------
  //  get list of used references
  //
  static References( ruleId, param, entity )
  {
    var result = [];
    if( Array.isArray( param ) ) {
      for( var value of param ) {
        result.push( "call:" + value );
      }
    }
    else {
      result.push( "call:" + param );
    }

    //console.log( "ref-ctl", result );
    return result;
  }


}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

RuleControlCalltargetList.Register( RuleControlCalltargetList.template );
﻿//=============================================================================
//
//  CONTROL CALL PROFILE IMPLEMENTATION
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class RuleControlInteger extends RuleControlBase
{
  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  static template = "/widget/ruleConfig/ruleControlInteger";

  //-----------------------------------------------------------------
  //  MEMBER VARIABLES
  //-----------------------------------------------------------------

  isInitialized = false;
  
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable initialization
  //
  Load( createData )
  {
    this.ruleId = createData['ruleId'];
    this.rule = Data.Instance().Rule( this.ruleId );
    this.ruleConfig = createData['ruleConfig']; 
    this.langId = AccessApplication.Instance().Language();
    this.entity = createData['entity'];
    this.parameter = createData['parameter'];
    this.paramName = createData['paramName'];
   
    this.FormCreateBasicLayout( this.paramName, this.parameter );
    this.FormLoad( );

  }

  //-----------------------------------------------------------------
  //  viewable event handler
  //
  OnEvent( eventName, param, data  )  
  {
    // implement standard ( "option", "value" .. ) handler
    if( eventName == "option" && param == "value" ) {
      if( data ) {
        console.warn( "RuleControlInteger: option/value no setter implemented!" );
        return;
      }
      return this.FormRead();
    }
  }


  //---------------------------------------------------------------------------
  //  FORM
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  statepoint droplist
  //
  FormLoad( )
  {
    // value
    var value = "0";
    if( this.ruleConfig && ( 'config' in this.ruleConfig ) && ( 'parameter' in this.ruleConfig['config'] ) ) {
      value = this.ruleConfig['config']['parameter'][ this.paramName ] || "0";
    }
    else if( 'default' in this.parameter['param'] ) {
      value = this.parameter['param']['default'];
    }

    // create inside tags
    var langId = AccessApplication.Instance().Language();
    var postLabel = ( 'post-label' in this.parameter ) ? this.parameter['post-label'][ langId ] : ""
    var $test = $( $.views.Html( "/widget/ruleConfig/ruleControlInteger/postlabel", { postLabel: postLabel }, this.dynid ) );
    $( "#ID_RULE_CONTROL" + this.dynid ).append( $test );

    // create edit
    $( "#ID_RULE_EDIT" + this.dynid ).edit( {
      size: CONST.size.M,
      value: value, 
      filter: { allow: "0123456789", deny: null },
      validate: this.ValidateEdit.bind( this )
    } );

  }

  //---------------------------------------------------------------------------
  //  get form data
  //
  FormRead()
  {
    return parseInt( $( "#ID_RULE_EDIT" + this.dynid ).edit( "option", "value" ) );
  }

  //-----------------------------------------------------------------
  //  FORM HELPER
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  CONTROL EVENTS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  VALIDATE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  validate requested from upper layer
  //
  OnValidate( param )
  {
    if( ( this.parameter['param']['mandantory'] || false ) !== true ) {
      return 0;
    }

    if( param === false ) {
      return $( "#ID_RULE_EDIT" + this.dynid ).edit( "validate", false );
    }
    var result = $( "#ID_RULE_EDIT" + this.dynid ).edit( "validate" );
    return result;
  }

  //-----------------------------------------------------------------
  //  do validate
  //
  ValidateEdit( validateControl )
  {
    if( validateControl.value == "" )  {
      validateControl.error = "{$const_error_empty}";
      return 1;
    }

    var intValue = parseInt( validateControl.value );
    var range = this.parameter['param']['range'] || "0-0";
    range = range.split( "-" );
    range[ 0 ] = parseInt( range[ 0 ] );
    range[ 1 ] = parseInt( range[ 1 ] );
    if( range[ 1 ] && ( intValue < range[ 0 ] || intValue > range[ 1 ] ) ) {
      validateControl.error = "{$rule_integer_error_range}";
      return 1;
    }

    validateControl.error = null;
    return 0;

  }

  //-----------------------------------------------------------------
  //  STATIC
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build interface to outside
  //
  static Append( $parent, createOptions )
  {
    // build new parent element
    var $element = $( "<div>" );
    $parent.append( $element );

    // create viewable inside
    createOptions['view'] = RuleControlInteger.template;
    $element.viewable( createOptions );

    // return new element
    return $element;
  }

  //-----------------------------------------------------------------
  //  validate interface to outside
  //
  static Validate( ruleId, ruleConfig, entity )
  {
    return 0;
  }

  //-----------------------------------------------------------------
  //  get list of used references
  //
  static References( ruleId, param, entity )
  {
    return [];
  }

}


//-----------------------------------------------------------------------------
//  REGISTER
//-----------------------------------------------------------------------------

RuleControlInteger.Register( RuleControlInteger.template );
﻿//=============================================================================
//
//  Access Data Controller
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jäkel
//          07.07.2020
//
//  Events:
//
//    "DATA/UPDATE/"  - entity loaded (param := entity)
//    "ENTINFO/UPDATE/{entid}" - entinfo updated
//    "ENTINFO/DELETE/{entid}" - entity deleted
//    "DATA/STARTUP_PROGRESS/" - load startupdata progress with percentage
//    "DATA/STARTUP_FINISHED/" - load startupdata finished
//
//  Interface:
//
//    RootEntid() - ...
//    TreeData()  - get treeset
//    PresenceData()  - get presence dataset
//    Entinfo() - get a entity info
//    IsChild() - check if entid is child of another
//    StatepointInfo()  - get a sp info
//    CalltargetDataset() - self explainable
//    StatepointDataset() - self explainable
//    EntityDataset() - dataset with all entities
//    PermissionList() - permissions module
//
//    RequestEntity() - start load entity
//    SaveEntity()  - start saving entity
//    MoveEntity()  - move
//    SetEntityPassword() -
//    MarkEntityForReboot( entityId )
//
//=============================================================================

class Data extends EventSource
{
  //-------------------------------------------------------
  //  CLASS DATA
  //-------------------------------------------------------

  static _instance = null;
  static _timeDifference = 0;
  static _adminConfigFile = "agw";
  static _featureMap = {
    // enabled through entinfo (module entity in tree)
    "module.preview-vdoor": false,
    "module.preview-ipcamdoor": false,
    "module.onvif": false,
    "module.asolis": false,   
    "module.deviceoutage": false
  }; 

  //-------------------------------------------------------
  //  INIT
  //-------------------------------------------------------

  constructor()
  {
    super( );
    this.status = {
      isAuthorized: false,
      isConnected: false
    };
    this._adminConfig = { };
    this._treeData = new TreeData();
    this._presenceData = new PresenceData();
    this._entinfoManager = new EntinfoManager();
    this._logfile = new LogfileData();
    this._reboot = new Reboot();
    this._addressbook = new Addressbook();
    this._keytones = new KeytonesModel();
    this._licenses = new LicenceModel();
    this._listsContainer = new TemporaryListsContainer();
    this._initialDataLoader = new InitialDataLoader();
    this._susStatus = "NOTUSED";
    this._modules = new ModuleData;
    this._version = "4.7.11 (Test)";
   

    // Note: From submodule constructors or other functions called from Data.constructor()
    // it is not possible to call Data.Instance() (stack overflow). For this, use Data.Start().

    this.createEntityCallback = null;
  }

  //-----------------------------------------------------------------
  //  SINGLETON 
  //  
  static Instance()
  {
    if( ! Data._instance ) {
      Data._instance = new Data;
    }
    return Data._instance;
  }

  //-----------------------------------------------------------------
  //  DEBUG CODE - REMOVE
  //  
  DebugMessageUpdateFlood()
  {
    return;
  }

  //-----------------------------------------------------------------
  //  Start all submodules, then begin initial data download.
  //  
  Start()
  {
    console.log( "Start Data-Layer" );
    this.isLoadFinished = false;

    // Let submodules do their subscriptions.
    this._treeData.Start();
    this._presenceData.Start();
    // EntinfoManager doesn't have Start function!
    this._listsContainer.Start(); // temporary!!

    // Start data download sequence.
    this._initialDataLoader.StartSequence( ( progressPercent ) => {

      if( progressPercent < 100 )  {
        this.Publish( "DATA/STARTUP_PROGRESS/", progressPercent );
      }
      else  {
        this.isLoadFinished = true;
        this.Publish( "DATA/STARTUP_FINISHED/" );

        // modules
        this._modules.Load();
      }
    } );

    // $FJ
    this._entinfoManager.On( "update", this.EntinfoUpdate.bind( this ) );
    this._entinfoManager.On( "delete", this.EntinfoDelete.bind( this ) );    

  }

  //-----------------------------------------------------------------
  // Clear data.
  // Avoid malfunctions by overwriting existing data at Start().  
  //  
  Stop()
  {
    console.log( "Stop Data-Layer" );
    this._treeData.Stop();
    this._presenceData.Stop();
  }  

  //-------------------------------------------------------
  //  FEATURE MAP
  //-------------------------------------------------------

  //-------------------------------------------------------
  //  query if a feature is enabled
  //
  static IsFeatureEnabled( feature )
  {
    return ( Data._featureMap[ feature ] === true );
  }

  
  //-------------------------------------------------------
  //  SUS
  //-------------------------------------------------------

  // set by dashboard (ADMIN.STARTUP_INFO) or in mailbox (sus-mailbox)
  SetSusStatus( status )
  {
    this._susStatus = status;
  }

  // used by form susapp to show toast
  GetSusStatus( )
  {
    return this._susStatus;
  }

  //-------------------------------------------------------
  //  SCANDEVICE
  //-------------------------------------------------------

  ScanDevice( type )
  {
    new XhrRequest( "ADMIN.SCAN_DEVICES", { 'type': type }, ( response ) => {
      //console.log( "scan-dev", response );
    } );
  }

  CameraInfo( onvifURL, onvifUser, onvifPass, entityId )
  {
    new XhrRequest( "ADMIN.CAMERA_INFO", { 'onvifURL': onvifURL, 'onvifUser': onvifUser, 'onvifPass': onvifPass, 'entityId': entityId }, ( response ) => {
      //console.log( "cam-info", response );
    } );
  }

  //-------------------------------------------------------
  //  GETTERS
  //-------------------------------------------------------

  IsLoading()
  {
    return ! ( this.isLoadFinished );
  }

  TreeData()
  { 
    return this._treeData;
  }

  PresenceData()
  { 
    return this._presenceData;
  }

  Reboot()
  { 
    return this._reboot;
  }

  Addressbook()
  {
    return this._addressbook;
  }

  Licences( )
  {
    return this._licenses.Get();
  }

  Keytones()
  {
    return this._keytones;
  }

  Logfile()
  {
    return this._logfile;
  }

  Entinfo( entityId, localUnsavedEntity = null )
  {
    let entinfo = this._treeData.Get( entityId );
    if( localUnsavedEntity && localUnsavedEntity['entid'] == entityId )  {
      
      // build entinfo from the (unsaved) local entity
      entinfo = {};
      entinfo['name']     = localUnsavedEntity['name']    || null;
      entinfo['class']    = localUnsavedEntity['class']   || null;
      entinfo['comment']  = localUnsavedEntity['comment'] || null;
      entinfo['entid']    = localUnsavedEntity['entid']   || null;
      entinfo['parent']   = localUnsavedEntity['parent']  || null;
      if( ( 'phone' in localUnsavedEntity ) && ( 'callnumber' in localUnsavedEntity['phone'] ) ) {
        entinfo['callnumber'] = localUnsavedEntity['phone']['callnumber'];
      }
      if( ( 'phone' in localUnsavedEntity ) && ( 'externCallnumber' in localUnsavedEntity['phone'] ) ) {
        entinfo['externCallnumber'] = localUnsavedEntity['phone']['externCallnumber'];
      }
      if( ( 'phone' in localUnsavedEntity ) && ( 'prefix' in localUnsavedEntity['phone'] ) ) {
        entinfo['prefix'] = localUnsavedEntity['phone']['prefix'];
      }
      if( ( 'authorize' in localUnsavedEntity ) && ( 'websocket' in localUnsavedEntity['authorize'] ) && ( 'account' in localUnsavedEntity['authorize']['websocket'] ) ) {
        entinfo['account'] = localUnsavedEntity['authorize']['websocket']['account'];
      }
      if( ( 'adminUi' in localUnsavedEntity ) && ( 'macaddr' in localUnsavedEntity['adminUi'] ) ) {
        entinfo['macaddr'] = localUnsavedEntity['adminUi']['macaddr'];
      }
      if( ( 'sp' in localUnsavedEntity ) ) {
        entinfo['sp'] = localUnsavedEntity['sp'];
      }
      if( ( 'permission' in localUnsavedEntity ) ) {
        entinfo['permission'] = localUnsavedEntity['permission'];
      }

      return entinfo;
    }

    // correct bug, that entinfo for basicparam does not exist
    var result = this._treeData.Get( entityId );
    if( entityId == "admui.basicparam" )  {
      result['name'] = "{$aui_lang_basicparameters}";
    }

    return result;
  }

  EntityIdFromIp( ipport )
  {
    return this._entinfoManager.EntityIdFromIp( ipport );
  }

  IsChild( entityId, entityIdParent )
  {
    let currentEntityId = entityId;
    while( currentEntityId )  {
      if( currentEntityId == entityIdParent )  {
        return true;
      }
      let entinfo = this._treeData.Get( currentEntityId );
      currentEntityId = entinfo['_parent'];
    }
    
    return false;
  }

  //-----------------------------------------------------------------
  //  information about a statepoint
  //
  StatepointInfo( statepointAddress, localUnsavedEntity = null )
  {
    return this._listsContainer.StatepointDataset().StatepointInfo( statepointAddress, localUnsavedEntity );
  }

  //-----------------------------------------------------------------
  //  information about a admin/client login account
  //
  AccountInfo( account )
  {
    let entid = this._listsContainer.EntityDataset().FindAccount( account );
    if( entid ) {
      return this.Entinfo( entid );
    }

    return null;
  }

    //-----------------------------------------------------------------
  //  information about a admin/client login account
  //
  MacAddressInfo( macAddress )
  {
    let entid = this._listsContainer.EntityDataset().FindMacAddress( macAddress );
    if( entid ) {
      return this.Entinfo( entid );
    }

    return null;
  }

  //-----------------------------------------------------------------
  //  interface to call target control
  //
  CalltargetDataset()
  {
    return this._listsContainer.CalltargetDataset();
  }

  //-----------------------------------------------------------------
  //  interface to statepoint control
  //
  StatepointDataset()
  {
    return this._listsContainer.StatepointDataset();
  }

  //-----------------------------------------------------------------
  //  interface to entity control
  //
  EntityDataset()
  {
    return this._listsContainer.EntityDataset();
  }

  //-----------------------------------------------------------------
  //  interface to statepoint control (permission are needed)
  //
  PermissionList()
  {
    return this._listsContainer.PermissionList();
  }

  //-----------------------------------------------------------------
  //  interface to call profile control
  //
  CallprofileDataset()
  {
    return this._listsContainer.CallprofileDataset();
  }

  //-----------------------------------------------------------------
  //  Get server root node.
  //
  RootEntid()
  {
    return this._treeData.RootEntid( );
  }

  //-----------------------------------------------------------------
  //  get time difference between server and client
  //
  TimeOffset()
  {
    return Data._timeDifference;
  }

  //-----------------------------------------------------------------
  //  get list of modules
  //
  RuleList()
  {
    return this._modules.GetRuleList();
  }

  //-----------------------------------------------------------------
  //  get list of modules
  //
  Rule( id )
  {
    return this._modules.GetRule( id );
  }

  //-----------------------------------------------------------------
  //  get list of modules
  //
  ModuleByEntid( entid )
  {
    return this._modules.GetModuleByEntid( entid );
  }

  //-----------------------------------------------------------------
  //  update list of modules
  //
  UpdateModules( )
  {
    this._modules.Load( () => {
      console.log( "module load refresh finished", this._modules.GetRuleList() );
      this.Publish( "MODULES/UPDATED/" );
    } );
  }
  

  //-------------------------------------------------------
  //  VERSION 
  //-------------------------------------------------------

  // set by dashboard (ADMIN.STARTUP_INFO) or in mailbox (sus-mailbox)
  SetVersion( version )
  {
    this._version = version;
  }

  // used by form susapp to show toast
  GetVersion( isShort = false )
  {
    if( isShort ) {
      return this._version.split( " " )[ 0 ];
    }
    return this._version;
  }


  // $FJ
  //-----------------------------------------------------------------
  //  STATIC EVENTS
  //-----------------------------------------------------------------

  EntinfoUpdate( entinfo )
  {
    if( entinfo['entid'] == "menu.licence" )  {
      this.RequestEntity( "menu.licence" );
    }

    // build feature map
    if( entinfo['class'] == "module" ) {
      Data._featureMap[ entinfo['entid'] ] = true;
    }
  }

  EntinfoDelete( entityId )
  {
    delete Data._featureMap[ entityId ];
  }

  //-----------------------------------------------------------------
  //  ENTITY HANDLING on server
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Request full entity by ID, wait for XHR response.
  //
  RequestEntity( entityId, fnResponse )
  {
    let xhrRequest = new XhrRequest( "ADMIN.GET_ENTITY", { 'entityId': entityId }, ( response ) => {      
      this.XhrResponse( response );
      if( $.isFunction( fnResponse ) && 'data' in response ) {
        fnResponse( response['data'] );
      }
    } ); 
    return true;
  }

  //-----------------------------------------------------------------
  //  Get entities directly without calling XhrReponse
  //  Used by moving entities as child to another
  //
  GetEntityListDirect( entityIdList, fnResponse )
  {
    if( !Array.isArray( entityIdList ) ) {
      console.warn( "GetEntityDirect - entityIdList need to be an array!" );
      return;
    }

    // load the entities and 
    let numEntitiesToLoad = entityIdList.length;
    for( var entityId of entityIdList )  {
      new XhrRequest( "ADMIN.GET_ENTITY", { 'entityId': entityId }, ( response ) => { 
        numEntitiesToLoad--;
        fnResponse( response['data'], numEntitiesToLoad == 0 );
      } );       
    }
  }

  //-----------------------------------------------------------------
  //  Write full entity back to server.
  //
  //  Unlike SavePartialEntity this function doesn't have an isMoved flag
  //  because of old-admin compatibility reasons! So SaveEntity can be used to change
  //  an entity's position on the tree.
  //
  SaveEntity( entity, isLogfileUsed = true )
  {
    let xhrRequest = new XhrRequest( "ADMIN.UPDATE_ENTITY", { 'entity': entity }, ( response ) => {
      this.XhrResponseUpdateEntity( response, entity );
    } );

    // Add Logfile record.
    let adminEntityId = SimpleCookies.Instance().Get( 'user' );
    let logfileRecord = {
        'type':     'info',
        'admin':    adminEntityId,
        'entid':    entity['entid'],
        'message':  "{$STORE_UPDATE}",
        'param':  {     
          'name': entity['name']
        }
    };
    if( isLogfileUsed ) {
      this._logfile.AddRecordRequest( logfileRecord );
    }
  }

  //-----------------------------------------------------------------
  //  Add full entity 
  //
  //  this function should only be used for invisible entities
  //  like rules or other modules!
  //
  AddEntity( entity )
  {
    let xhrRequest = new XhrRequest( "ADMIN.ADD_ENTITY", { 'entity': entity }, ( response ) => {
      this.XhrResponseUpdateEntity( response, entity );
    } );    
  }  

  //-----------------------------------------------------------------
  //  Update entity on server, but only those sub-elements given; 
  //  all other sub-object remain untouched.
  //
  //  Set isMoved = true if entity's parent is to be updated. Otherwise 
  //  server will ignore the parent and the entity's position on the tree
  //  will remain unchanged!
  //  
  SavePartialEntity( partialEntity, isMoved = false, isLogFile = true )
  {
    // Precondition.  
    if( typeof partialEntity['entid'] != "string" ) { 
      console.warn( "Partial entity minimum requirement is entityId!" );
      return;
    }

    let xhrRequest = new XhrRequest( "ADMIN.UPDATE_PARTIAL_ENTITY", { 'partialEntity': partialEntity, 'isMoved': isMoved }, ( response ) => {
      this.XhrResponse( response );
    } );

    // Add Logfile record.
    if( isLogFile )  {

      var entinfo = this.Entinfo( partialEntity['entid'] );
      let adminEntityId = SimpleCookies.Instance().Get( 'user' );
      let logfileRecord = {
          'type':     'info',
          'admin':    adminEntityId,
          'entid':    partialEntity['entid'],
          'message':  "{$STORE_UPDATE}",
          'param':  {     
            'name': entinfo['name']
          }
      };
      this._logfile.AddRecordRequest( logfileRecord );

    }
  }

  //-----------------------------------------------------------------
  //  Update entity on server, but only those sub-elements given; 
  //  all other sub-object remain untouched.
  //
  //  Set isMoved = true if entity's parent is to be updated. Otherwise 
  //  server will ignore the parent and the entity's position on the tree
  //  will remain unchanged!
  //  
  SavePartialEntityList( partialEntityList, isMoved = false )
  {
    this.SavePartialEntityListRecurse( partialEntityList, isMoved );
  }  

  SavePartialEntityListRecurse( partialEntityList, isMoved )
  {
    // get item to be updated
    let partialEntity = partialEntityList.pop();
    //console.log( "Update Partial Entity:", partialEntity['entid'] );

    // create xhr request
    new XhrRequest( "ADMIN.UPDATE_PARTIAL_ENTITY", { 'partialEntity': partialEntity, 'isMoved': isMoved }, ( response ) => {
      this.XhrResponse( response );
      if( partialEntityList.length )  {
        this.SavePartialEntityListRecurse( partialEntityList, isMoved );
      }
    } );

  }

  
  //-----------------------------------------------------------------
  //  Move an entity given by ID to another parent entity given by ID.
  //  
  MoveEntity( moveEntityId, toEntityId )
  {
    // Note: We can't use SaveEntity() for moving an entity, because then we would need to have
    // the complete entity downloaded.

    let xhrRequest = new XhrRequest( "ADMIN.MOVE_ENTITY", { 'moveEntityId': moveEntityId, 'toEntityId': toEntityId }, ( response ) => {
      this.XhrResponseMoveEntity( response, moveEntityId, toEntityId );
    } );
  }


  //-----------------------------------------------------------------
  //  Create an entity of given class in given folder.
  //  After server's confirmation, createCallback is performed 
  //  with the new entityId.
  // 
  CreateEntity( entityClass, deviceTemplate, parentEntityId, createCallback = null )
  {
    let createParam = {
      'class':          entityClass,
      'parent':         parentEntityId,
      'deviceTemplate': deviceTemplate,
      'macaddr':        null,
      'password':       null
    };

    this.createEntityCallback = createCallback;

    let xhrRequest = new XhrRequest( "ADMIN.ENTITY_CREATE", { 'createParam': createParam, 'lang': 'de' }, ( response ) => {
      this.XhrResponse( response );
    } );
  }

  //-----------------------------------------------------------------
  //  Delete entity with given ID.
  // 
  DeleteEntity( entityId, isLogfileUsed = true )
  {
    // If a tree branch is to be deleted, all subitems need to be deleted recursively!

    // Get an array of all entInfos to be deleted.
    let entInfoList = [];
    this._treeData.BranchEntInfoList( entityId, entInfoList );

    // Make an array of all entityIds to be deleted.
    let entityIdList = [];
    for( let info of entInfoList ) {

      // Protect admin.1 against being deleted together with a folder!
      if( info["entid"] == "admin.1" )  {
        return;
      }

      entityIdList.push( info["entid"] );
    } // for

    // Delete them all.  
    new XhrRequest( "ADMIN.DELETE_ENTITIES", { 'entityIdList': entityIdList }, ( response ) => {

      this.XhrResponseDeleteEntities( response, entInfoList, isLogfileUsed );
      
      // remove all references to deleted entities in permission records
      new PermissionDeleter( entityIdList, ( response ) => { } );

    } );


  }

  //-----------------------------------------------------------------
  //  Set password for a given entity.
  //
  SetEntityPassword( entityId, password, passwordKey )
  {
    let xhrRequest = new XhrRequest( "ADMIN.SET_PASSWORD", { 'entityId': entityId, 'password': password, 'passwordKey': passwordKey }, ( response ) => {
      this.XhrResponse( response );
    } );
  }      


  //-----------------------------------------------------------------
  //  ADMIN MESSAGES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // - Add "device restart needed" record to messages database.
  // - Set device's pending_reboot flag in IP DB.
  // 
  MarkEntityForReboot( entityId )
  {
    new XhrRequest( "ADMIN.MARK_REBOOT_DEVICE", { 'entityId': entityId }, ( response ) => {
      this.XhrResponse( response );
      }
    );
  }

  //-----------------------------------------------------------------
  // Delete a device's pairing and/or certain tables.
  //
  ResetDeviceParameters( entityId, unpairing, calldetails, grabpic )
  {
    let params = { 
      'unpairing': unpairing, 
      'calldetails': calldetails,
      'grabpic': grabpic
    };

    new XhrRequest( "ADMIN.RESET_DEVICE_PARAMS", { 'entityId': entityId, 'params': params }, ( response ) => {
      this.XhrResponse( response );
      } 
    );
  }


  //-----------------------------------------------------------------
  //  XHR REQUEST/RESPONSE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Single XHR function
  //
  XhrRequest( command, data = null )
  {
    new XhrRequest( command, data, ( response ) => {      
      this.XhrResponse( response );
    } );
  }

  //-----------------------------------------------------------------
  // General XHR response handling
  //    
  XhrResponse( response )
  {
    //console.log( 'Data::XhrResponse for ' + response['meta']['command'] ); 

    let command      = response['meta']['command'];
    let responseCode = parseInt( response['meta']['status'] );    

    // If error, we are done here.
    if( responseCode != 200 ) {
      return;
    }

    //-- GET-------------------------------------------------
    if( command == "ADMIN.GET_ENTITY" )  {

      // special for license
      if( response['data']['entid'] == "menu.licence" )  {
        this._licenses.Put( response['data'] );
      }

      // Store entity & publish event.
      let entityId = response['data']['entid'];
      this.Publish( "DATA/UPDATE/" + entityId, response['data'] );

    } // GET_ENTITY
    //-- CREATE----------------------------------------------
    else if( command == "ADMIN.ENTITY_CREATE" )  {

      // Store entity.
      let entityId = response['data']['entid'];

      // Perform callback.
      if( this.createEntityCallback instanceof Function )  {
        this.createEntityCallback( entityId );
      }

      // Add Logfile record.
      let adminEntityId = SimpleCookies.Instance().Get( 'user' );
      let logfileRecord = {
          'type':     'info',
          'admin':    adminEntityId,
          'entid':    entityId,
          'message':  "{$STORE_CREATE}",      
          'param':  {     
            'name': response['data']['name']
          }
      };
      this._logfile.AddRecordRequest( logfileRecord );
     
    } // ENTITY_CREATE

    //-- ADMINCONFIG----------------------------------------------
    else if( command == "ADMINCONFIG.GET" ) {
      if( 'param' in response['meta'] ) {
        this._adminConfig[ response['meta']['param'] ] = response['data'];
      }
    }

    //-- Default --------------------------------------------
    // Publish result.
    else  {
      this.Publish( response['meta']['command'], response );
    }

  }


  //-----------------------------------------------------------------
  // XHR response handling: ADMIN.UPDATE_ENTITY
  //    
  XhrResponseUpdateEntity( response, entity )
  {
    let command      = response['meta']['command'];
    let responseCode = parseInt( response['meta']['status'] );

    // If error, we are done here. Server sends 'isOk' flag as data.
    if( responseCode != 200 || !response['data'] ) {
      return;
    }

    //-- UPDATE_ENTITY --------------------------------------
    if( command == "ADMIN.UPDATE_ENTITY" )  {
      // Basic update on tree without waiting for mailbox message.
      //
      // NOTE: Doing this for UPDATE_ENTITY, but not UPDATE_PARTIAL_ENTITY,
      // because only the first.mentioned command is used for moving entities
      // (ASC, ASHT).
      this._treeData.ProcessEntInfoBascis( entity['entid'], entity['parent'] );      
    }
  }

  //-----------------------------------------------------------------
  // XHR response handling: ADMIN.UPDATE_ENTITY
  //   
  XhrResponseMoveEntity( response, moveEntityId, toEntityId )
  {
    let command      = response['meta']['command'];
    let responseCode = parseInt( response['meta']['status'] );

    // If error, we are done here. Server sends 'isOk' flag as data.
    if( responseCode != 200 || !response['data'] ) {
      return;
    }

    //-- MOVE_ENTITY ----------------------------------------
    if( command == "ADMIN.MOVE_ENTITY" )  {
      // Basic update on tree without waiting for mailbox message.
      this._treeData.ProcessEntInfoBascis( moveEntityId, toEntityId );
    } // MOVE_ENTITY
  }

  //-----------------------------------------------------------------
  // XHR response handling: ADMIN.DELETE_ENTITIES
  //    
  XhrResponseDeleteEntities( response, entInfoList, isLogfileUsed )
  {
    let command      = response['meta']['command'];
    let responseCode = parseInt( response['meta']['status'] );

    // If error, we are done here.
    if( responseCode != 200 ) {
      return;
    }

    if( !isLogfileUsed )
      return;

    //-- DELETE_ENTITIES ------------------------------------
    if( command == "ADMIN.DELETE_ENTITIES" )  {

      let adminEntityId = SimpleCookies.Instance().Get( 'user' );      

      // For each of the deleted entities, add an "entity deleted" logfile record.
      for( let entityId of response['data'] ) {

        // Get entity's name.        
        let index = entInfoList.findIndex( ( rec ) => rec["entid"] == entityId );
        let name = entInfoList[ index ]["name"];
        if( index >= 0 )  {
          entInfoList.splice( index, 1 );
        }

        // Compose & add log record.
        if( typeof name == "string" )  {
          let logfileRecord = {
            'type':     'info',
            'admin':    adminEntityId,
            'entid':    entityId,
            'message':  "{$STORE_DELETE}",      
            'param':  {     
              'name': name
            }
          };
          this._logfile.AddRecordRequest( logfileRecord );
        }
      } // for

    } // DELETE_ENTITIES
  }


  //-------------------------------------------------------
  //  CONFIG
  //-------------------------------------------------------  

  LoadAdminConfig()
  {
    Data.Instance().XhrRequest( "ADMINCONFIG.GET", "dhcp" );
    Data.Instance().XhrRequest( "ADMINCONFIG.GET", "agw" );
  }

  AdminConfig( type )
  {
    if( type in this._adminConfig )  {
      return this._adminConfig[ type ];
    }
    return null;
  }

  AdminParam( key )
  {
    return this._adminConfig[ Data._adminConfigFile ][ key ];
  }
}
﻿//=============================================================================
//
//  Initial Data Loader
//  (c) 2021 CYTEL Technology AG
//
//  Description: After Data.Start(), send a sequence of XHR requests to server,
//  in order to get all basic data:
//  - tree
//  - entinfos
//  - presence list
//
//  Editor: Peter Aidam
//          21.06.2021
//=============================================================================

class InitialDataLoader
{
  //-------------------------------------------------------
  //  INIT / START
  //-------------------------------------------------------

  constructor()
  {
    this._isFinished = false;
    this._progressFunction = null;
  }


  //-----------------------------------------------------------------
  //  Start with first request of sequence: tree.
  //  
  StartSequence( progressCallback )
  {
    this._isFinished = false;
    this._progressFunction = progressCallback;
    this._isLastEntinfoRequest = false;
    this._isLastAdressBookRequest = false;

    // For percentage calculation: Ajust here if additional data load is required!
    this._awaitedResponses = 5;   // <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

    // Start data download sequence.
    this.RequestTree( );
  }


  //-------------------------------------------------------
  //  SEQUENCE REQUESTS
  //-------------------------------------------------------

  //-----------------------------------------------------------------
  //  Request complete tree from server.
  //  
  RequestTree( )
  {
    new XhrRequest( "ADMIN.DEVTREE", null, ( response ) => {      
      this.XhrResponse( response );
    } );
  }  

  //-----------------------------------------------------------------
  // After tree has been received, request all entInfos from server.
  //    
  RequestEntInfoList( )
  {
    // Prerequisites.
    let requestEntityIdList = Object();    
    let requestCount = 0;

    // The response from server must not exceed 1MB in size!
    let requestLimit = 100; // <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

    // Get an array of all entityIds in tree. 
    let treeKeys = Data.Instance()._treeData.Keys();

    // Request entInfos in packages of requestLimit size.
    for( let index in treeKeys )  {
      let entityId = treeKeys[ index ];

      requestCount++;
      requestEntityIdList[ entityId ] = true;

      // If limit is reached or list is completed, send a request.
      if( requestCount >= requestLimit || index >= treeKeys.length-1 )  {

        // Will this request complete entInfoList?
        if( index >= treeKeys.length-1 )  {
          this._isLastEntinfoRequest = true;
        }

        // Do the request.
        new XhrRequest( "ADMIN.GET_ENTINFO_LIST", { 'entityIdList': requestEntityIdList, 'isLastRequest': this._isLastEntinfoRequest }, ( response ) => {      
          this.XhrResponse( response );
        } );           

        // Prepare next loop.
        requestEntityIdList = Object();
        requestCount = 0;
      }    
    } // for

  }

  //-----------------------------------------------------------------
  // After all of entinfoList has been received, request presenceList.
  //
  RequestPresenceList( )
  {
    // Preliminarily not "ADMIN.PRESENCE"!
    // AGW doesn't compile if admin envelope uses presence.
    new XhrRequest( "PRESENCE.LIST", null, ( response ) => {      
      this.XhrResponse( response );
    } );    
  }  

  //-----------------------------------------------------------------
  // After presenceList has been received, request all address books from server.
  //    
  RequestAddressBooks( )
  {
    let entidList = Data.Instance()._treeData.AddressBookEntidList( );

    // Any address book to be requested?
    if( entidList.length == 0 )  {
      return false;
    }

    for( let addressBookEntid of entidList ) {

      // Will this request complete loading address books?      
      if( addressBookEntid == entidList[ entidList.length-1 ] )  {
        this._isLastAdressBookRequest = true;
      }

      // Do the request.      
      new XhrRequest( "ADMIN.ADDRESSBOOK", { 'entityId': addressBookEntid, 'isLastRequest': this._isLastAdressBookRequest }, ( response ) => {      
        this.XhrResponse( response );
      } );     

    }
    return true;
  }


  //-------------------------------------------------------
  //  SINGLE XHR FUNCTION
  //-------------------------------------------------------

  XhrRequest( command, data = null )
  {
    new XhrRequest( command, data, ( response ) => {      
      this.XhrResponse( response );
    } );
  }


  //-----------------------------------------------------------------
  //  XHR RESPONSE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // After each response,
  // - put result data info Data object
  // - look what step of sequence is to be done next, or
  // - look if sequence is finished
  //    
  XhrResponse( response )
  {
    //console.log( 'initialDataLoader::XhrResponse for ' + response['meta']['command'] ); 

    let command      = response['meta']['command'];
    let responseCode = parseInt( response['meta']['status'] );    

    // If error, we are done here.
    if( responseCode != 200 ) {
      return;
    }

    //-- ADMIN.DEVTREE --------------------------------------
    if( command == "ADMIN.DEVTREE" )  {    
      // Store tree & publish event.
      Data.Instance()._treeData.StartupData( response['data'] );

      // Progress. 
      this.CallProgressFunction( 1 );      

      // After tree was downloaded, next step: entinfoList.
      this.RequestEntInfoList( );      
    }

    //-- ADMIN.GET_ENTINFO_LIST -----------------------------
    else if( command == "ADMIN.GET_ENTINFO_LIST" )  {    
      Data.Instance()._treeData.ProcessEntInfoList( response['data']['entInfoList'], true );

      // After all of entinfos was downloaded, next step: presenceList.
      if( response['data']['isLastRequest'] )  {

        // Progress. 
        this.CallProgressFunction( 3 ); // take the entinfos as two progress steps!

        this.RequestPresenceList( );
      }
    }

    //-- ADMIN.PRESENCE -------------------------------------
    else if( command == "PRESENCE.LIST" )  {
      // Store & publish event.
      Data.Instance()._presenceData.StartupData( response['data'] );

      // Progress. 
      this.CallProgressFunction( 4 );

      if( !this.RequestAddressBooks( ) )  {
        // If no address book to be requested: we are already done!
        this._isFinished = true;

        // Progress. 
        this.CallProgressFunction( this._awaitedResponses );
      }
    }

    //-- ADMIN.ADDRESSBOOK ----------------------------------
    else if( command == "ADMIN.ADDRESSBOOK" )  {

      Data.Instance()._listsContainer.calltargetDataset.ProcessAddressbook( response["data"]["entityId"], response['data']['addressBook'] );

      // After all address books have been downloaded...
      if( response["data"]["isLastRequest"] )  {
        // This was the last step of sequence: we are done!
        this._isFinished = true;

        // Progress. 
        this.CallProgressFunction( this._awaitedResponses );
      }

    }

  }

  //-----------------------------------------------------------------
  //  HELPERS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // After each response, calculate which percentage of all data was
  // loaded after the time, then call into Data object.
  //    
  CallProgressFunction( responseNumber )
  {
    // Calculate percentage.
    let percent = responseNumber*100 / this._awaitedResponses;

    // Callback.
    this._progressFunction( percent );    

  }

}
﻿//=============================================================================
//
//  AGW Data Controller
//  (c) 2020 CYTEL Technology AG
//
//  Editor: Fraynk Jäkel
//          07.07.2020
//=============================================================================

class SupportInfo 
{
  //-------------------------------------------------------
  //  INIT
  //-------------------------------------------------------

  constructor()
  {
  }

  //-----------------------------------------------------------------
  //  SINGLETON 
  //  
  static Instance()
  {
    if( !( $.app._theAgwSupportinfo instanceof SupportInfo ) ) {
      $.app._theAgwSupportinfo = new SupportInfo;
    }
    return $.app._theAgwSupportinfo;
  }

  Get( )
  {
    console.log( "start run SupportInfo" );
    // build request data
    let request = new XMLHttpRequest();
    let envelope = {
      "meta": {
        "guid": $.guidgen(),
        "command": "SUPPORT.GET_INFORMATION",
        "session": null
      },
      "data": null
    };

    // perform xhr-request
    request.addEventListener( "load", this.EventXhrRequestLoad.bind( this ) );
    request.open( "POST", "xhr.php" );
    request.setRequestHeader( "Content-Type", "application/json" );
    request.send( JSON.stringify( envelope ) );

  }

  EventXhrRequestLoad( event )
  {
    // prequistes
    var self = this;
    console.log( "received SupportInfo" );

    // decode response
    let response = JSON.parse( event.target.response );
    let command = response['meta']['command'];
    let responseCode = parseInt( response['meta']['status'] );
    if( responseCode == 200 ) {
      this.OnWorkSupportInformation( response );
    }
    else  {
      console.log( "error SupportInfo" );
    }
  }

  OnWorkSupportInformation( response )
  {
    console.log( "SI:", response );
    var fileName = response['data']['file'];
    var filePath = "download.php?file=" + fileName + "&u"; 
    Application.Instance().DownloadFile( filePath );

  }  


}//=============================================================================
//
//  Tree Data
//  (c) 2021 CYTEL Technology AG
//
//  Description: Merge devTree and entInfo into viewable tree.
//
//  Editor: Peter Aidam
//          03.05.2021
//=============================================================================


class TreeData extends Treeset
{
  //-----------------------------------------------------------------
  //  CONSTRUCTOR / START
  //-----------------------------------------------------------------
  
  constructor( )
  {
    // call base class constructor
    super( { id: 'entid' } );

    // set sorter to us
    this._options.sorter = this.Sorter.bind( this );

    this._partialElementKeys = {};
  }

  //-----------------------------------------------------------------
  // Access singletons.
  //    
  Start( )
  {
    // Subscribe mailbox event.
    this.eventId = Mailbox.Instance().On( "ENTINFO", this.MailboxEventEntInfo.bind( this ) );
  }

  //-----------------------------------------------------------------
  // Clear data.
  //    
  Stop( )
  {
    // Unsubscribe mailbox event.
    Mailbox.Instance().Off( this.eventId );

    // Clear data.
    let empty = {};
    this.SetData( empty );
  }


  //-----------------------------------------------------------------
  //  GETTERS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Root.
  //
  RootEntid( )
  {
    return this.rootEntid;
  }

  //-----------------------------------------------------------------
  //  A list of all entityIds of entities of class "addrBook".
  //
  AddressBookEntidList( )
  {
    let entidList = [];

    // TODO: Recurse all subfolders of RootEntid(). Collect this of class
    // "addrBook".
    this.Each( "folder.confdev", ( id, record ) =>  {
      if( record["class"] == "addrBook" )  {
        entidList.push( id );
      }
    });

    return entidList;
  }

  //-----------------------------------------------------------------
  //  A list of all entInfos inside a given branch, including 
  //  the branch's root.
  //
  BranchEntInfoList( branchEntityId, branchEntInfoList )
  {
    if( !Array.isArray( branchEntInfoList ) )  {
      branchEntInfoList = [];
    }
  
    // Get data.
    let entInfo = this.Get( branchEntityId );

    // Recurse all children. Since the resulting list is used to delete
    // branches and their subitems, each subitem's children must be added 
    // to result BEFORE the subitem itself is added!
    if( 'children' in entInfo ) {
      for( let child of entInfo['children'] ) {
        this.BranchEntInfoList( child["entid"], branchEntInfoList );
      }
    }

    // Do this AFTER recursion!
    branchEntInfoList.push( entInfo );
  }


  //-----------------------------------------------------------------
  //  INTERFACE FOR DATA MODEL
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Used to initially set devTree after it was received from server.
  //  
  StartupData( devTree )
  {
    this.SetData( devTree );
    //console.log( devTree );
     
    // Root ID (tree root is an object with one sub-object).
    for( let id in devTree )  {
      this.rootEntid = id;
    }

  }

  //-----------------------------------------------------------------
  //  - Merge a list of entInfos into existing tree items.
  //  - Publish static events using EntinfoManager
  //  - Publish UI events
  //
  ProcessEntInfoList( entInfoList, isInitialDataLoad=false )
  {
    //console.log( "Process Entinfo-List", entInfoList );

    // Go thru all of entInfoList (object).
    for( let entityId in entInfoList )  {
      let newItem = entInfoList[ entityId ];
      let oldItem = this.Get( entityId );

      if( typeof oldItem == "object" )  {
        // Item is already there, so update it.
  
        let oldParent = this.GetParent( entityId );

        // Note: Update doesn't touch the existing item's children.
        this.Update( entityId, newItem );

        // If parent has changed, move it also.
        //
        // To avoid crashes in tree ctrl due to duplicated entity items on devtree, 
        // don't move anything during initial data load!
        if( !isInitialDataLoad && typeof oldParent == "string" && typeof newItem["parent"] == "string" )  {        
          if( newItem["parent"] != oldParent )  {

            // Note: Treeset.Move may crash if called while treeview is initialised!
            this.Move( entityId, newItem["parent"] );
          }
        }
      }
      else  {
        // Item doesn't exist yet, so add it.
        this.Add( newItem["parent"], newItem );
      }

      // Distribute events.
      Data.Instance()._entinfoManager.ProcessUpdate( newItem, this._partialElementKeys[ entityId ] );

      // Updated partial elements to be passed only once.
      delete this._partialElementKeys[ entityId ];
    }
  }

  //-----------------------------------------------------------------
  //  Merge basic info on an entinfo into tree, right after server has confirmed
  //  and Update or Move. So this helps if websocket doesn't work and mailbox
  //  messages arrive late.
  //
  //  At this time, doing this for parent only. This is extraordinarily important,
  //  because otherwise Drag & Drop wouldn't work w/o Websocket!
  //
  ProcessEntInfoBascis( entityId, parent )
  {
    let oldItem = this.Get( entityId );

    if( typeof oldItem == "object" )  {
      // Item is already there, so update it.
  
      let oldParent = this.GetParent( entityId );

      // Create a preliminary new Item out of oldItem plus new name & parent.
      let newItem = $.clone( oldItem );
      if ( typeof parent == "string"  )  {
        newItem["parent"] = parent;
      }      

      // If parent has changed, move it also.
      if( typeof oldParent == "string" && typeof newItem["parent"] == "string" )  {        
        if( newItem["parent"] != oldParent )  {

          // Note: Update doesn't touch the existing item's children.
          this.Update( entityId, newItem );

          this.Move( entityId, newItem["parent"] );
        }
      }
    }

  }


  //-----------------------------------------------------------------
  //  MAILBOX EVENT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // Event: A list of mailbox elements of type "ENTINFO" was received
  // from server.
  // - Collect entIds of elements of term "U"pdate. For these, Request 
  //   full entInfos from server.
  // - For elements of type "D"elete, delete corresponding tree items. 
  //    
  MailboxEventEntInfo( entInfoElementList )  
  {
    // Prerequisites
    let updateEntidList = Object();

    // Iterate thru mailbox element list.
    for( let id in entInfoElementList )  {

      let entInfoElement = entInfoElementList[ id ];
      let entityId = entInfoElement["entid"];

      if( entInfoElement["term"] == "U" )  {  // "U"pdate
        updateEntidList[ entityId ] = true;

        // When Data.SavePartialEntity is called, server sends list of keys of updated entity elements.
        // Save this key list so they can be passed with "ENTINFO" events when full entInfo has 
        // arrived after xhr request.
        this._partialElementKeys[ entityId ] = entInfoElement["partialElements"];
      }
      else if( entInfoElement["term"] == "D" )  {  // "D"elete
        this.Delete( entityId );
        delete this._partialElementKeys[ entityId ];

        // Distribute events.
        Data.Instance()._entinfoManager.ProcessDelete( entityId );
      }
    } // for

    // Request updated entInfos from server.
    if( Object.keys( updateEntidList ).length > 0 )  {
      let xhrRequest = new XhrRequest( "ADMIN.GET_ENTINFO_LIST", { 'entityIdList': updateEntidList, 'isLastRequest': true }, ( response ) => {      
        this.XhrEntInfoListResponse( response );
      } );
    }
  }
  

  //-----------------------------------------------------------------
  //  XHR
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // Server's response on request for a list of entInfos.
  // - Enter these entInfos into tree.
  // - Generate update events for each of these entInfos (TODO).
  //    
  XhrEntInfoListResponse( response )
  {
    // Prerequisites.
    let command      = response['meta']['command'];
    let responseCode = parseInt( response['meta']['status'] );    

    // If error, we are done here.
    if( responseCode != 200 ) {
      return;
    }

    //-- GET_ENTINFO_LIST -----------------------------------
    if( command == "ADMIN.GET_ENTINFO_LIST" )  {    
      this.ProcessEntInfoList( response['data']['entInfoList'] );
    }
  }


  //-----------------------------------------------------------------
  //  DRAG & DROP INTERFACE
  //-----------------------------------------------------------------

  IsDropAllowed( dragEntityId, dropEntityId ) 
  {
    // Prerequisites.
    let dragClass = this.Get( dragEntityId )["class"];
    let dropClass = this.Get( dropEntityId )["class"];

    // Note: Make sure that verification in server's Config::Move 
    // follows the same rules as this verification!

    // unconfigured can't be dragged
    if( dragEntityId == "folder.unconfdev" ) {
      return false;
    }

    // Everything can be dropped on any folder - except folder.unconfdev.
    if( dropClass == "folder" && dropEntityId != "folder.unconfdev" )  {
      return true;
    }

    // SW client ASC/ASHT can be dropped on phones.
    // Note: Old admin doesn't allow to drop on "jungpanel".
    if( ( dropClass == "apartment" || dropClass == "sipaudio" || dropClass == "tkphone" || dropClass == "avp" ) 
     && ( dragClass == "client" || dragClass == "asht" ) 
      )  {

      // Except the phone already has a SW client.
      // -> To be clarified; old admin allows multiple ASC/ASHT per phone, too!
      let dropEntInfo = this.Get( dropEntityId );
      if( !Array.isArray( dropEntInfo['children'] ) || dropEntInfo['children'].length == 0 )  {
        return true;
      }
    }

    // Porters camera: allow to drag a camera to ASC
    if( ( dropClass == "client" ) && ( dragClass == "ipcam" ) ) {

      // Except the phone already has a SW client.
      // -> To be clarified; old admin allows multiple ASC/ASHT per phone, too!
      let dropEntInfo = this.Get( dropEntityId );
      if( !Array.isArray( dropEntInfo['children'] ) || dropEntInfo['children'].length == 0 )  {
        return true;
      }
    }    

    // ipcameras can be droped on sip phones. They 
    // become a virtual door. 
    if( Data.IsFeatureEnabled( "module.preview-vdoor" ) )  {
      if( ( dropClass == "sipaudio" ) && ( dragClass == "ipcam" ) ) {
        // Except the phone already has a SW client.
        // -> To be clarified; old admin allows multiple ASC/ASHT per phone, too!
        let dropEntInfo = this.Get( dropEntityId );
        if( !Array.isArray( dropEntInfo['children'] ) || dropEntInfo['children'].length == 0 )  {
          return true;
        }
      }    
    }

    // ipcameras can be droped on old siedle doors
    // so a audio only door station can be a video door
    if( Data.IsFeatureEnabled( "module.preview-ipcamdoor" ) )  {
      if( ( dropClass == "door" ) && ( dragClass == "ipcam" ) ) {
        //console.log( "Drop allowed: module.dooripcam" );
        return true;
      }    
    }
    

    return false;
  }


  //-----------------------------------------------------------------
  //  SORTER
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // Compare to items to figure out their sort order inside a tree folder.
  //      
  Sorter( recordA, recordB )
  {  
    // Note that record['_preview'] contents is provided in OnCreatePreview!    

    // Special sort criteria: folders at top.
    let isFolderA =  ( recordA['_preview']['class'] == "folder" );
    let isFolderB =  ( recordB['_preview']['class'] == "folder" );    
    if( isFolderA && !isFolderB )  {
      return -1;
    }
    else if( !isFolderA && isFolderB ) {
      return 1;
    }
    else if( isFolderA && isFolderB ) {
      // "Unconfigured devices" is topmost item.
      if( recordA['_preview']['id'] == "folder.unconfdev" )  {
        return -1;
      }
      else if( recordB['_preview']['id'] == "folder.unconfdev" )  {
        return 1;
      }
    }

    // sort by text
    let textA = recordA['_preview']['text'];
    let textB = recordB['_preview']['text'];
    return textA.localeCompare( textB, navigator.language, { numeric: true, sensitivity: 'base' } );

  }


  //-----------------------------------------------------------------
  //  OVERRIDABLES
  //-----------------------------------------------------------------

  OnCreatePreview( record )
  {
    // some entities doesn't have a name
    if( !( 'name' in record ) || record['name'] == null )  {
      record['name'] = "";
    }

    // correct bugs in tree-data (version 5.0.0 - 6.0.1)
    if( record['name'] == "" )  {
      if( record['entid'] == "admui.basicparam" )  {
        record['name'] = "{$aui_lang_basicparameters}";
      }
    }

    // Icon
    let icon = CONST_CLASS_ICO[ record["class"] ];
    if( typeof icon != "string" )  {
      icon = CONST_CLASS_ICO["none"]; 
    }

    // create inplace buttons for tree control
    let inplaceButton = null;
    let isContainer = FormFactory.IsContainer( record['class'] );
    let isLocked = FormFactory.IsLockedForm( record['entid'] );
    let isPermission = FormFactory.IsPermission( record['class'] );
    if( isContainer || !isLocked )  {
      inplaceButton = [];
    }
    if( inplaceButton && isPermission && record['entid'] != "folder.unconfdev" ) {
      inplaceButton.push( { id: "permission", icon: "mdi mdi-security", tooltip: $.lang( "{$const_form_tooltip_permission}" ) } );
    }
    if( isContainer && record['entid'] != "folder.unconfdev" ) {
      inplaceButton.push( { id: "add", icon: "mdi mdi-plus", tooltip: $.lang( "{$const_form_tooltip_add}" ) } );
    }
    if( !isLocked ) {
      inplaceButton.push( { id: "delete", icon: "mdi mdi-delete", tooltip: $.lang( "{$const_form_tooltip_delete}" ) } );
    }

    // return preview
    return {
      id: record['entid'],
      class: record['class'],
      text: $.lang( record['name'] ),
      icon: icon,
      inplaceButton: inplaceButton
    };
  }

  OnCreateStatus( record )
  {
    return null;
  }

  OnSortCompare( record1, record2 )
  {
    return 0;
  }

}//=============================================================================
//
//  Presence Data
//  (c) 2021 CYTEL Technology AG
//
//  Description: Data for presence list control.
//  record :=  {
//    'entid':           device entityId
//    'devapi':          BOOL: devapi device or not
//    'presenceValue':   OFFLINE, IDLE etc.
//    'name':            device name
//    'callnumber':      ["phone"]["callnumber"]
//    'idAddr':          like "192.168.0.100"
//    'lastAction':      time of last change
//    'remotePhoneNo':   if BUSY device's remote
//    'info'
//  }
//
//  Editor: Peter Aidam
//          14.06.2021
//=============================================================================


class PresenceData extends Dataset
{
  //-----------------------------------------------------------------
  //  CONSTRUCTOR / START
  //-----------------------------------------------------------------
  
  constructor( )
  {
    // call base class constructor
    super( { id: 'entid' } );
  }

  //-----------------------------------------------------------------
  // Access singletons.
  //    
  Start( )
  {
    // Subscribe mailbox event.
    this.eventId = Mailbox.Instance().On( "PRESENCE", this.MailboxEventPresence.bind( this ) );

    // Subscribe entinfo update/create message.
    Data.Instance()._entinfoManager.On( "update", this.ProcessEntinfoUpdate.bind( this ) );    
  }

  //-----------------------------------------------------------------
  // Clear data.
  //    
  Stop( )
  {
    // Unsubscribe mailbox event.
    Mailbox.Instance().Off( this.eventId );    

    // Clear data.
    let empty = {};
    this.SetData( empty );
  }


  //-----------------------------------------------------------------
  //  INTERFACE FOR DATA MODEL
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Used to initially set presenceList after it was received from server.
  //  
  StartupData( presenceList )
  {
    // Prerequisites.
    let list = [];

    // Translate server's presence list (which is an object with one item
    // for each device entityId) into dataset array.
    for( let entityId in presenceList )  {

      // Copy object item to array record, and add column "entid".
      let arrayRecord = presenceList[ entityId ];

      // Get some additional info from the entity's entinfo.
      let entinfo = Data.Instance().Entinfo( entityId );
      arrayRecord["entid"]      = entityId;
      arrayRecord["name"]       = entinfo["name"];
      arrayRecord["callnumber"] = entinfo["callnumber"];

      arrayRecord["status"] = "green";


      list.push( arrayRecord );
    } // for


    this.SetData( list );
    //console.log( `Presence-Data: ${ this.Size() } items` );    
  }


  //-----------------------------------------------------------------
  //  MAILBOX EVENTS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // Event: A list of mailbox elements of type "PRESENCE" was received
  // from server.
  // - For elements of type "U"pdate, update corresponding presence list 
  //   items, or add a new item.
  // - For elements of type "D"elete, delete corresponding presence list 
  //   items. 
  //    
  MailboxEventPresence( presenceElementList )
  {
    // Iterate thru mailbox element list.
    for( let id in presenceElementList )  {

      let presenceElement = presenceElementList[ id ];

      if( presenceElement["term"] == "U" )  {  // "U"pdate

        let listRecord = {};

        // Mailbox presence element is equivalent to a presence record, plus the
        // mailbox fields "type" and "term". So create new record for presence list
        // by shifting all fields except those two.
        for( let fieldId in presenceElement )  {
          if( fieldId != "type" && fieldId != "term" )  {
            listRecord[ fieldId ] = presenceElement[ fieldId ];
          }
        } // for

        // Get some additional info from the entity's entinfo.
        let entinfo = Data.Instance().Entinfo( presenceElement["entid"] );
        if( typeof entinfo == "object" )  {        
          listRecord["name"]       = entinfo["name"];
          listRecord["callnumber"] = entinfo["callnumber"];
        }
        else  {
          // Right after a device was created, entinfo is not available immediately!
          // Will then later be updated by ProcessEntinfoUpdate.
          listRecord["name"]       = "-";
          listRecord["callnumber"] = "-";
        }

        // Update or add.
        if( this.Has( presenceElement["entid"] ) )  {
          // Overwrite old record with new record.
          this.Update( listRecord );
        }
        else  {
          this.Add( listRecord );
          //console.log( `Add: Presence-Data: ${ this.Size() } items` );            
        }

      }
      else if( presenceElement["term"] == "D" )  {  // "D"elete
        this.Delete( presenceElement["entid"] );
        //console.log( `Delete: Presence-Data: ${ this.Size() } items` );            
      }      

    } // for

  }


  //-----------------------------------------------------------------
  //  ENTINFO EVENT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Update: called by entInfoManager after mailbox or xhr events
  //
  ProcessEntinfoUpdate( entinfo )
  {
    // Get entity's existing presence record.
    let listRecord = this.Get( entinfo["entid"] );
    if( typeof listRecord != "object" )  {
      return;
    }

    // Update that record if name or callnumber has changed,
    // OR
    // if entinfo was not available when presence record was created.
    if( listRecord["name"] != entinfo["name"] || listRecord["callnumber"] != entinfo["callnumber"] )  {
      listRecord["name"]       = entinfo["name"];
      listRecord["callnumber"] = entinfo["callnumber"];
      this.Update( listRecord );
    }

  }


  //-----------------------------------------------------------------
  //  OVERRIDABLES
  //-----------------------------------------------------------------


}﻿//=============================================================================
//
//  Mailbox Controller
//  (c) 2021 CYTEL Technology AG
//
//  Description: Mailbox + websocket connection.
//  - Maintain websocket channel to server; receive mailbox messages on this channel.
//  - Periodically poll for mailbox messages from server
//    (for the case websocket doesn't work.)
//  - Publish mailbox messages to subscribers.
//
//  Editor: Peter Aidam
//          21.05.2021
//=============================================================================

class Mailbox extends StaticEventSource
{
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  constructor()
  {
    super( );
    this.status = {
      isStarted: false,
      isWebsocketConnected: false,
      websocketRetries: 0,
      isPolling: false
    };

    // remove after impl
    this.debugIsError = false;
  }

  //-----------------------------------------------------------------
  //  SINGLETON 
  //  
  static Instance()
  {
    if( ! ( $.app._theMailbox instanceof Mailbox ) )  {
      $.app._theMailbox = new Mailbox;
    }
    return $.app._theMailbox;
  }

  //-----------------------------------------------------------------
  //  remove this after implementation (called by Logoclick)!
  //  
  DebugSimulateSusMessage()
  {
    this.debugIsError = !this.debugIsError;
    let simMsg = [
      { type: "SUS", term: "U", susStatus: this.debugIsError ? "ERROR" : "OK" }
    ];
    this.DistributeMailboxElements( simMsg );
  }

  //-----------------------------------------------------------------
  //  Open websocket.
  //  
  Start()
  {
    // First try websocket. If it fails, polling will be used instead.
    this.status.websocketRetries = 1; // <<<<
    console.log( 'Try establish websocket...' );
    this.StartWebsocket();

    this.status.isStarted = true;
  }

  //-----------------------------------------------------------------
  //  Open websocket.
  //  
  StartWebsocket()
  {
    Data.Instance().Publish( "WEBSOCKET/STATUS/", "connect" );

    // Websocket connect
    if( location.protocol == "https:" )  {
      // Firefox doesn't add subpath to exceptions
      this.websocket = new WebSocket( "wss://" + location.host + ":443/a5/", "admin" );
    }
    else {
      this.websocket = new WebSocket( "ws://" + location.host + ":13122/a5/", "admin" );
    }

    // Websocket events.
    this.websocket.addEventListener( 'open', this.EventWebsocketOpen.bind( this ) );
    this.websocket.addEventListener( 'close', this.EventWebsocketClose.bind( this ) );
    this.websocket.addEventListener( 'error', this.EventWebsocketError.bind( this ) );
    this.websocket.addEventListener( 'message', this.EventWebsocketMessage.bind( this ) );
  }

  //-----------------------------------------------------------------
  //  Close websocket or stop polling.
  //  
  Stop()
  {
    console.log('Websocket Stop()');
    this.status.isStarted = false;
    this.status.isPolling = false;
    if( typeof this.websocket == "object" )  {
      this.websocket.close( );
    }
  }
  

  //-----------------------------------------------------------------
  //  Websocket events
  //-----------------------------------------------------------------

  EventWebsocketOpen( event )
  {
    console.log('Websocket connection has been opened');
    Data.Instance().Publish( "WEBSOCKET/STATUS/", "open" );
    this.status.isWebsocketConnected = true;
    this.status.websocketRetries = 0;    
    this.status.isPolling = false;
  }

  EventWebsocketClose( event )
  {
    // Caution: A call to close event will follow after error event!
    // The log line may be confusing because it may occur after a connect retry
    // has been started!
    Data.Instance().Publish( "WEBSOCKET/STATUS/", "close" );

    if( event.wasClean )  {
      console.log('Websocket connection or connect attempt has been closed (clean)' );
      this.status.isWebsocketConnected = false;
      return;      
    }

    console.log('Websocket connection or connect attempt has been closed (!clean)' );

    // If connected before and Stop() was not called, try to re-establish websocket.
    // So schedule retry.
    if ( this.status.isStarted && this.status.isWebsocketConnected )  {
      this.status.websocketRetries = 1; // <<<<
    }

    this.status.isWebsocketConnected = false;

    // Retry connect?
    if ( this.status.isStarted && this.status.websocketRetries > 0 )  {
      console.log( 'Retry establish websocket (' + this.status.websocketRetries + ' left)...' );
      this.status.websocketRetries = this.status.websocketRetries - 1;
      this.StartWebsocket();
      return;
    }

    // After all retries, use the alternative.
    if ( this.status.isStarted && !this.status.isPolling )  {
      console.log( 'Mailbox polling...' );
      this.status.isPolling = true;
      let xhrRequest = new XhrRequest( "MAILBOX.LONG_POLLING", null, ( response ) => {
        this.XhrResponseLongPolling( response );
      } );
    }

  }

  EventWebsocketError( event )
  {
    console.log('Websocket error: ', event );
    Data.Instance().Publish( "WEBSOCKET/STATUS/", "error" );

    // A call to close event with event.wasClean=false will follow after error event!
  }

/*
  EventWebsocketClose( event )
  {
    // Caution: A call to close event will follow after error event!
    // The log line may be confusing because it may occur after a connect retry
    // has been started!
    console.log('Websocket connection or connect attempt has been closed, event=', event );
    Data.Instance().Publish( "WEBSOCKET/STATUS/", "close" );
    this.status.isWebsocketConnected = false;

    if( event.wasClean )  {
      console.log('Websocket close CLEAN' );
    }
    else  {
      console.log('Websocket close DIRTY' );
    }

  }

  EventWebsocketError( event )
  {
    console.log('Websocket error: ', event );
    Data.Instance().Publish( "WEBSOCKET/STATUS/", "error" );

    // If connected before, try to re-establish websocket.
    // So schedule retry.
    if ( this.status.isStarted && this.status.isWebsocketConnected )  {
      this.status.websocketRetries = 1; // <<<<
    }

    this.status.isWebsocketConnected = false;

    // Retry connect?
    if ( this.status.isStarted && this.status.websocketRetries > 0 )  {
      console.log( 'Retry establish websocket (' + this.status.websocketRetries + ' left)...' );
      this.status.websocketRetries = this.status.websocketRetries - 1;
      this.StartWebsocket();
      return;
    }

    // Use the alternative.
    if ( this.status.isStarted && !this.status.isPolling )  {
      console.log( 'Mailbox polling...' );
      this.status.isPolling = true;
      let xhrRequest = new XhrRequest( "MAILBOX.LONG_POLLING", null, ( response ) => {
        this.XhrResponseLongPolling( response );
      } );
    }
  }
*/



  EventWebsocketMessage( event )
  {
    let message = JSON.parse( event.data );

    //-------------------------------------------------------    
    // Server's first message after open connection.
    if( message['meta']['command'] == "ADMIN.EVENT.HELLO" )  {
      this.EventWebsocketMessageHello( message );
    } // .HELLO

    //-------------------------------------------------------
    else if( message['meta']['command'] == "ADMIN.EVENT.MAILBOX" )  {
      for( let msg of message.data )  {

        if( msg['type'] == "ADDRBOOK" ) {
          Data.Instance().Publish( "ADDRESSBOOK/UPDATE/" + msg['entid'], msg );
        }

        if( msg['type'] == "PRESENCE" ) {
          Data.Instance().Publish( "PRESENCE/UPDATE/" + msg['entid'], msg );
        }

        if( msg['type'] == "SCANDEVICE" ) {
          Data.Instance().Publish( "SCANDEVICE/UPDATE", msg );
        }

        if( msg['type'] == "VIDEO_INFO" ) {
          Data.Instance().Publish( "VIDEO_INFO/UPDATE", msg );
        }

      }
      this.EventWebsocketMessageMailbox( message );
    } // .MAILBOX
  }      


  //-----------------------------------------------------------------
  //  WEBSOCKET MESSAGES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // Server's first message after open connection.
  //
  EventWebsocketMessageHello( websocketMessage )
  {
    this._sessionId = websocketMessage['meta']['session'];
   
    // store time difference between server and client
    Data._timeDifference = moment( new Date() ).unix() - websocketMessage['meta']['serverTime'];
    //if( Math.abs( Data._timeDifference ) > 10 )  {
    //  console.log( "Time difference =", Data._timeDifference );
    //}

    // -> Admin's first message to server informs of session - authorizedSession relation.
    let authorizedSession = SimpleCookies.Instance().Get("session");

    let envelope = {
      "meta": {
        "session": this._sessionId,
        "authorizedSession": authorizedSession,
        "guid": $.guidgen(),
        "command": "ADMIN.HELLO",
      },
      "data": {
      }
    };

    this.websocket.send( JSON.stringify( envelope ) );    
  }

  //-----------------------------------------------------------------
  // Server sent one or multiple mailbox elements. Distribute them
  // to their subscribers.
  //  
  EventWebsocketMessageMailbox( websocketMessage )
  {
    this.DistributeMailboxElements( websocketMessage['data'] );
    return;
  }


  //-----------------------------------------------------------------
  //  MAILBOX ELEMENTS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // Distribute an array of mailbox elements to their subscribers.
  //  
  DistributeMailboxElements( mailboxElementList )
  {
    // Prerequisites
    let entInfoElementList       = []; // list of mailbox elements of type "ENTINFO"
    let presenceElementList      = []; // list of mailbox elements of type "PRESENCE"
    let addressBookElementList   = []; // list of mailbox elements of type "ADDRBOOK"
    let adminMessageElementList  = []; // list of mailbox elements of type "ADMIN_MESSAGE"
    let deviceRestartElementList = []; // list of mailbox elements of type "DEVICE_RESTART"
    let susElementList           = []; // list of mailbox elements of type "SUS"
    let ruleElementList          = []; // list of mailbox elements of type "RULE"

    // Go thru all elements in mailbox. Collect them in type-specific lists.
    for( let index in mailboxElementList )  {
      let mailboxElement = mailboxElementList[ index ];

      if( mailboxElement["type"] == "ENTINFO" )  {
        // this is just made for server internal 
        if( mailboxElement['entid'] == "sus.internalparam" )  {
          continue;
        }
        entInfoElementList.push( mailboxElement );
      }
      else if( mailboxElement["type"] == "PRESENCE" )  {
        presenceElementList.push( mailboxElement );
      }
      else if( mailboxElement["type"] == "ADDRBOOK" )  {
        addressBookElementList.push( mailboxElement );
      }
      else if( mailboxElement["type"] == "ADMIN_MESSAGE" )  {
        adminMessageElementList.push( mailboxElement );
      }
      else if( mailboxElement["type"] == "DEVICE_RESTART" )  {
        deviceRestartElementList.push( mailboxElement );
      }
      else if( mailboxElement["type"] == "SUS" ) {
        susElementList.push( mailboxElement );
      }    
      else if( mailboxElement["type"] == "RULE" ) {
        ruleElementList.push( mailboxElement );
      }          
    } // for

    // Distribute the lists to subscribers.
    if( entInfoElementList.length > 0 )  {
      this.Publish( "ENTINFO", entInfoElementList );
    }
    if( presenceElementList.length > 0 )  {
      if( presenceElementList.length > 200 )  {
        console.warn( "Slow network detected, skip presence updates!" );
      }
      else {
        this.Publish( "PRESENCE", presenceElementList );
      }
    }
    if( addressBookElementList.length > 0 )  {
      this.Publish( "ADDRBOOK", addressBookElementList );
    }
    if( adminMessageElementList.length > 0 )  {
      this.Publish( "ADMIN_MESSAGE", adminMessageElementList );
    }
    if( deviceRestartElementList.length > 0 )  {
      this.Publish( "DEVICE_RESTART", deviceRestartElementList );
    }
    if( susElementList.length > 0 ) {
      // this.Publish( "SUS", susElementList ); currently not used
      this.DistributeSusMessage( susElementList );
    }    
    if( ruleElementList.length > 0 ) {
      this.Publish( "RULE", ruleElementList );
    }     

  }

  DistributeSusMessage( susElementList )
  {
    for( var message of susElementList ) {
      if( 'susStatus' in message ) {
        Data.Instance().Publish( "SUS/STATUS", message['susStatus'] );
        Data.Instance().SetSusStatus( message['susStatus'] );
      }
    }
  }


  //-----------------------------------------------------------------
  //  XHR RESPONSES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // XHR response on "ADMIN.FETCH_MAILBOX". Distribute the mailbox
  // messsages to their observers.
  //
  XhrResponseFetchMailbox( response )
  {
    // Prerequisites.
    let responseCode = parseInt( response['meta']['status'] );

    // If error, we are done here.
    if( responseCode != 200 ) {
      return;
    }

    //-- FETCH_MAILBOX --------------------------------------
    if( response['meta']['command'] == "ADMIN.FETCH_MAILBOX" )  {

      // Generate events for all messages in the array.
      this.DistributeMailboxElements( response['data']['mailbox'] );
    }

  }

  //-----------------------------------------------------------------
  // XHR response on "MAILBOX.LONG_POLLING". Distribute the mailbox
  // messsages to their observers.
  //
  XhrResponseLongPolling( response )
  {
    // Prerequisites.
    let responseCode = parseInt( response['meta']['status'] );

    if( !this.status.isStarted )  {
      return;
    }

    if( responseCode == 408 ) { // "408 Request Timeout";
      console.log( "Mailbox polling timeout (not an error!) -> restart loop" );
    }

    // If error, we are done here.
    else if( responseCode != 200 ) {
      return;
    }

    // Generate events for all messages in the array.
    if( response['data'] != null )  {
      this.DistributeMailboxElements( response['data']['mailbox'] );
    }

    // Do the same again.
    if( this.status.isPolling )  {
      let xhrRequest = new XhrRequest( "MAILBOX.LONG_POLLING", null, ( response ) => {
        this.XhrResponseLongPolling( response );
      } );
    }
  }

}
﻿//=============================================================================
//
//  XHR Request
//  (c) 2021 CYTEL Technology AG
//
//  Description: Send a single XHR request and wait for response.
//
//  Editor: Peter Aidam
//          07.06.2021
//=============================================================================

class XhrRequest
{
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  constructor( command, data, callback, finishedCallback = null )
  {
    this.command = command;
    this.callback = callback;
    this.finishedCallback = finishedCallback;

    // Do the request.
    this.DoRequest( command, data );

    // Keep this object alive for a time.
    setTimeout( () => {     
    }, 60000 );

  }


  //-----------------------------------------------------------------
  //  SINGLE XHR FUNCTION
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // Prepare & perform XHR request.  
  //
  DoRequest( command, data = null )
  {
    // build request data
    let request = new XMLHttpRequest();
    let envelope = {
      "meta": { 
        "guid": $.guidgen( ),
        "command": command,
        "session": null
      },
      "data": data
    };

    // Perform xhr-request
    request.addEventListener( "load", this.EventXhrRequestLoad.bind( this ) );
    request.addEventListener( "error", this.EventXhrRequestError.bind( this ) );
    request.addEventListener( "abort", this.EventXhrRequestError.bind( this ) );
    request.open( "POST", "xhr.php" );
    request.setRequestHeader( "Content-Type", "application/json" );
    request.send( JSON.stringify( envelope ) );

  }

  //-----------------------------------------------------------------
  // "The load event is fired when an XMLHttpRequest transaction 
  // completes successfully."
  //
  EventXhrRequestLoad( event, data )
  {
    // Decode response.
    let response = null;
    if( event.target.status == 200 )  {
      response = JSON.parse( event.target.response );
    }
    else {
      // Connect status: Draw conclusions from XHR error.
      ConnectStatus.Instance().WorkXhrError( event );      
    }

    // Avoid subsequent interpreter errors.
    if( response == null )  {
      console.log( "XhrRequest::EventXhrRequestLoad response on " + this.command + " = " + event.target.response );
      if( this.finishedCallback ) {
        this.finishedCallback();
      }
      return;
    }

    let responseCode = parseInt( response['meta']['status'] );

    // Connect status: Draw conclusions from XHR response.
    ConnectStatus.Instance().WorkXhrResponse( response );

    // Log error.
    if ( responseCode != 200 ) {
      // 401 & 503 are common when admin is logged off or server is down. So only show the other potential errors.
      // Addendum: "408 Request Timeout" is common with "MAILBOX.LONG_POLLING".
      if( responseCode != 401 && responseCode != 503 && responseCode != 408 )  {
        console.log( "XhrRequest::EventXhrRequestLoad error-response on " + this.command + " = " + event.target.response );     
      }
    }

    // Callback with json response.
    this.callback( response );

    // finished callback for xhrRequestInterval
    if( this.finishedCallback ) {
      this.finishedCallback();
    }

  }

  //-----------------------------------------------------------------  
  // The error event is fired when the request encountered an error.
  //
  EventXhrRequestError( event )
  {
    // do not supress browser warning NET::ERR_CERT_AUTHORITY_INVALID
    // in this case: reload complete page
    // credits to:
    // https://stackoverflow.com/questions/48127436/how-to-catch-chrome-error-neterr-file-not-found-in-xmlhttprequest

    if( event.type == "error" && event.status == 0 && event.readyState == 4 )  {
      location.reload( true );
    }

    // Connect status: Draw conclusions from XHR error.
    ConnectStatus.Instance().WorkXhrError( event );

    // finished callback for xhrRequestInterval
    if( this.finishedCallback ) {
      this.finishedCallback();
    }

  }

}
﻿//=============================================================================
//
//  XHR Request Interval
//  (c) 2021 CYTEL Technology AG
//
//  Description: Send XHR requests periodically
//
//  Editor: Peter Aidam
//          16.06.2021
//=============================================================================

class XhrRequestInterval
{
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  constructor( )
  {
    this._timerList = { };    
  }


  //-----------------------------------------------------------------
  //  SINGLETON 
  //  
  static Instance()
  {
    if( ! ( $.app._theAgwXhrRequestInterval instanceof XhrRequestInterval ) )  {
      $.app._theAgwXhrRequestInterval = new XhrRequestInterval;
    }
    return $.app._theAgwXhrRequestInterval;
  }


  //-----------------------------------------------------------------
  //  Interface
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // Start first timeout; save timeout for restart after 1st response.
  //
  Set( timeout, command, data, callback = null )
  {
    this.Clear( command );

    // Note: We intenionally use setTimeout, not setInterval. Timeout will be restarted after response.
    // setInterval caused trouble when response didn't arrive - requests then formed a stack.

    this._timerList[ command ] = {
      'timeout' : timeout,
      'data'    : data,
      'timerId' : null,
      'callback': callback
    };

    // we need to do the first request directly
    this.IntervalTimer( command );

    // start timer
    this._timerList[ command ]['timerId'] = setTimeout( this.IntervalTimer, timeout, command );
  }

  //-----------------------------------------------------------------
  // Stop restarting timeout after next response.
  //  
  Clear( command )
  {
    if( command in this._timerList )  {
      clearTimeout( this._timerList[ command ]['timerId'] );
      delete this._timerList[ command ];
    }
  }


  //-----------------------------------------------------------------
  //  Timer
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // Callback function for setTimeout().
  //
  IntervalTimer( command )
  {
    var self = XhrRequestInterval.Instance( );
    let xhrRequest = new XhrRequest( 
      command, 
      self._timerList[ command ]['data'], 
      self.Response.bind( self ),
      () => {
        self.XhrRequestFinished( command ) 
      }
    );
  }


  //-------------------------------------------------------
  //  RESPONSE
  //-------------------------------------------------------  

  //-----------------------------------------------------------------
  // On response
  // - Restart timeout
  // - Perform callback function
  //   
  Response( response )
  {
    // Prerequisites.
    let command = response['meta']['command'];
    let callback = this._timerList[ command ]['callback'];

    // Start new timeout.
    /*if( command in this._timerList ) {
      clearTimeout( this._timerList[ command ]['timerId'] );
      this._timerList[ command ]['timerId'] = setTimeout( this.IntervalTimer, this._timerList[ command ]['timeout'], command );
    }*/

    // Callback with json response.
    if( callback instanceof Function )  {
      callback( response );
    }
  }

  XhrRequestFinished( command )
  {
    // Start new timeout.
    if( command in this._timerList ) {
      clearTimeout( this._timerList[ command ]['timerId'] );
      this._timerList[ command ]['timerId'] = setTimeout( this.IntervalTimer, this._timerList[ command ]['timeout'], command );
    }
  }

}
﻿//=============================================================================
//
//  Status
//  (c) 2021 CYTEL Technology AG
//
//  Description: Connection status handled by cookies.
//
//  Editor: Peter Aidam
//          08.06.2021
//=============================================================================

class ConnectStatus extends EventSource
{
  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  constructor()
  {
    super( );
    this.status = {
      isAuthorized: false,
      isConnected: false
    };
  }

  //-----------------------------------------------------------------
  //  SINGLETON 
  //  
  static Instance()
  {
    if( ! ( $.app._theConnectStatus instanceof ConnectStatus ) )  {
      $.app._theConnectStatus = new ConnectStatus;
    }
    return $.app._theConnectStatus;
  }


  //-----------------------------------------------------------------
  //  CONNECTION/AUTHORIZATION
  //-----------------------------------------------------------------

  static IsAuthorized( )
  {
    let user = SimpleCookies.Instance().Get( 'user' );
    return ( typeof user === 'string' && user != "anonymous" );
  }

  static AuthorizedEntid( )
  {
    return SimpleCookies.Instance().Get( 'user' );
  }

  static ClearAuthorized()
  {
    SimpleCookies.Instance().Remove( 'user' );
    ConnectStatus.Instance().status.isAuthorized = false;
  }

  static IsConnected( )
  {
    return ( SimpleCookies.Instance().Get( 'connected' ) == "true" );
  }

  static ClearConnected()
  {
    SimpleCookies.Instance().Remove( 'connected' );
    ConnectStatus.Instance().status.isConnected = false;
  }

  static IsFirstInstall()
  {
    let cookie = SimpleCookies.Instance().Get( 'firstinstall' );
    return ( typeof cookie === 'string' && cookie == "true" );
  }


  //-----------------------------------------------------------------
  //  STATUS FROM XHR
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // Draw conclusions from XHR response (JSON).
  //  
  WorkXhrResponse( response )
  {
    // Prerequisites.
    let newStatus = {
      isAuthorized: false,
      isConnected: false
    };

    // Look at response code.
    let responseCode = parseInt( response['meta']['status'] );
    if( responseCode == 503 ) {
      newStatus.isAuthorized = false;
      newStatus.isConnected  = false;
      SimpleCookies.Instance().Remove( 'user' );
      SimpleCookies.Instance().Remove( 'connected' );
    }
    else  {
      let user = SimpleCookies.Instance().Get( 'user' );
      let isConnected = ( SimpleCookies.Instance().Get( 'connected' ) == "true" );
      newStatus.isConnected  = isConnected;
      newStatus.isAuthorized = ( typeof user === 'string' && user != "anonymous" );
    }

    
    // Check if server status has changed. If so, send event.
    if( JSON.stringify( newStatus ) !== JSON.stringify( this.status ) )  {
      this.status = newStatus;
      this.Publish( "STATUS/STATUS", this.status );
    }
  }

  //-----------------------------------------------------------------
  // Draw conclusions from XHR error.
  //    
  WorkXhrError( event )
  {
    // Prerequisites.
    let newStatus = {
      isAuthorized: false,
      isConnected: false
    };

    SimpleCookies.Instance().Remove( 'connected' );
    SimpleCookies.Instance().Remove( 'user' );

    // check if server status has changed (probably yes)
    if( JSON.stringify( newStatus ) !== JSON.stringify( this.status ) ) {
      this.status = newStatus;
      this.Publish( "STATUS/STATUS", this.status );
    }
  }

}
﻿//=============================================================================
//
//  Lists Container
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//
//  Description: Event Receiver for mailbox (with complete entinfo inside).
//  In future this container is not longer needed.
//
//=============================================================================

class TemporaryListsContainer extends StaticEventSource
{
  //-----------------------------------------------------------------
  //  CLASS VARIABLES
  //-----------------------------------------------------------------

  calltargetDataset = null;
  statepointDataset = null;
  permissionList = null;
  entityDataset = null;
  callprofileDataset = null;

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  Start()
  {
    // call target dataset
    this.calltargetDataset = new CalltargetDataset( { } );
    
    // statepoint dataset
    this.statepointDataset = new StatepointDataset( { } );
    this.statepointDataset.Start( );

    // entity dataset
    this.entityDataset = new EntityDataset( {} );
    this.entityDataset.Start( );

    // permission list
    this.permissionList = new PermissionList();
    this.permissionList.Start( );

    // callprofile dataset
    this.callprofileDataset = new CallprofileDataset( { } );

  }

  Stop()
  {
    this.statepointDataset.Stop();
    this.permissionList.Stop();
  }


  //-----------------------------------------------------------------
  //  INTERFACE
  //-----------------------------------------------------------------

  CalltargetDataset()
  {
    return this.calltargetDataset;
  }

  StatepointDataset()
  {
    return this.statepointDataset;
  }

  EntityDataset()
  {
    return this.entityDataset;
  }

  PermissionList()
  {
    return this.permissionList;
  }

  CallprofileDataset()
  {
    return this.callprofileDataset;
  }

}﻿//=============================================================================
//
//  Calltarget List
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//
//  Description: Dataset for calltargets
//  record :=  {
//    'id':     entity-id, or addressbook RECORD id
//    'entid':  entity-id, or addressbook's entid
//    'class':  entinfo entityId or "addrBook"
//    'text':   entity[name] / addrBkRecord[label]
//    'callnumber': entity's callnumber of addrRecord's callnumber
//  }
//  
//=============================================================================

class CalltargetDataset extends Dataset 
{
  //-----------------------------------------------------------------
  //  CLASS VARIABLES
  //-----------------------------------------------------------------

  prefixList = [];

  //-----------------------------------------------------------------
  //  CONSTRUCTOR
  //-----------------------------------------------------------------

  constructor( options )
  {
    super( options );

    // subscribe entinfo update/create messages
    Data.Instance()._entinfoManager.On( "update", this.ProcessEntinfoUpdate.bind( this ) );
    Data.Instance()._entinfoManager.On( "delete", this.ProcessEntinfoDelete.bind( this ) );

    Mailbox.Instance().On( "ADDRBOOK", this.MailboxEventAddressbook.bind( this ) );

  }

  //-----------------------------------------------------------------
  //  OUTSIDE INTERFACE
  //-----------------------------------------------------------------

  CallnumberInfo( callnumber, isExactSearch = true )
  {
    // search exact for callnumber
    if( isExactSearch )  {
      let info = this.Filter( callnumber, "callnumber" );
      if( info.length == 1 ) {
        return this.Get( info[ 0 ] );
      }
      else if( info.length > 1 ){
        let result = [];
        for( let id of info ) {
          result.push( this.Get( id ) );
        }
        return result;
      }
    }
    // search just the first digits of callnumber
    else {
      let result = [];
      this.Each( ( id, record ) => {
        if( record['callnumber'].indexOf( callnumber ) == 0 )  {
          result.push( record );
        }
      } );
      if( result.length > 0 ) {
        return result;
      }
    }

    return null;
  }

  PrefixInfo( prefix )
  {
    let result = [];
    for( let record of this.prefixList )  {
      if( prefix.indexOf( record['prefix'] ) == 0 )  {
        result.push( record );
      }
    }
    return ( result.length > 0 ) ? result : null;
  }


  //-----------------------------------------------------------------
  //  ENTINFO INTERFACE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Update: called by entInfoManager after mailbox or xhr events
  //
  ProcessEntinfoUpdate( entinfo )
  {
    if( ( 'callnumber' in entinfo )
     && ( typeof entinfo['callnumber'] == "string" )
     && ( entinfo['callnumber'].length > 0 )
      ) {
      let record = {
        'id':         entinfo['entid'],   //  entity-id
        'entid':      entinfo['entid'],   //  entity-id
        'class':      entinfo['class'],   //  entity-class
        'text':       entinfo['name'],
        'icon':       CONST_CLASS_ICO[ entinfo['class'] ],
        'callnumber': entinfo['callnumber'],
        'status':     "normal"
      };
      if( this.Has( entinfo['entid'] ) )  {
        this.Update( record );
      }
      else {
        this.Add( record );
      }
    }

    if( ( 'prefix' in entinfo )
     && ( typeof entinfo['prefix'] == "string" )
     && ( entinfo['prefix'].length > 0 )
      ) {

      // change existing 
      let index = this.prefixList.findIndex( ( rec ) => ( rec['entid'] == entinfo['entid'] ) );
      if( index >= 0 )  {
        this.prefixList[ index ]['prefix'] =  entinfo['prefix'];
        this.prefixList[ index ]['name'] =  entinfo['name'];
      }
      // add new
      else {
        let record = {
          'id':         entinfo['entid'],   //  entity-id
          'entid':      entinfo['entid'],   //  entity-id
          'class':      entinfo['class'],   //  entity-class
          'text':       entinfo['name'],
          'icon':       CONST_CLASS_ICO[ entinfo['class'] ],
          'prefix':     entinfo['prefix'],
          'status':     "normal"
        };
        this.prefixList.push( record );
      }
    }
  }

  //-----------------------------------------------------------------
  //  Delete: called by entInfoManager after mailbox or xhr events.
  //
  ProcessEntinfoDelete( entityId )
  {
    // Entity class not available at this point -> so go the ugly way to find out.
    let entityIdClassString = entityId.substr( 0, 8 );

    // Is it an addressbook?
    if( entityIdClassString == "addrBook" ) {

      let deleteIdList = [];

      // 1. Collect all IDs of records originating from the deleted addressbook.
      this.Each( function( id, record )  {
        if( record['entid'] == entityId )  {
          deleteIdList.push( id );
        }
      } );   

      // 2. Delete all those records.
      for( let deleteId in deleteIdList )  {
        this.Delete( deleteId );
      }
    }
    
    // else: entinfo.
    else {
      // There's only one record to be deleted.
      if( this.Has( entityId ) )  {
        this.Delete( entityId );
      }

      // Eventually remove entity's record from prefixList.
      let index = this.prefixList.findIndex( ( rec ) => ( rec['entid'] == entityId ) );
      if( index >= 0 )  {
        this.prefixList.splice( index, 1 );
      }
    } // entinfo
  }


  //-----------------------------------------------------------------
  //  ADDRESSBOOK INTERFACE
  //-----------------------------------------------------------------  

  //-----------------------------------------------------------------
  //  Merge an adressbook into callTarget list.
  //  
  ProcessAddressbook( addressbookEntid, addressbook )
  {
    for( let addressbookRecord of addressbook ) {

      // Compose calltarget record.
      let record = {
        'entid':      addressbookEntid,
        'id':         addressbookRecord['id'],
        'class':      "addrBook",
        'text':       addressbookRecord['label'],
        'icon':       CONST_CLASS_ICO['addrBook'],
        'callnumber': addressbookRecord['callnumber'],
        'status': "normal"
      };

      // Add calltarget record.      
      this.Add( record );
      //console.log( `----- addressbook-merge Add: calltarget-Data: ${ this.Size() } items` );           
    } // for
  }

  //-----------------------------------------------------------------
  //  Event handler for addressbook record update/delete from mailbox.
  //
  MailboxEventAddressbook( addrbookElementList )
  {
    // Iterate thru mailbox element list.
    for( let id in addrbookElementList )  {

      let addrbookElement = addrbookElementList[ id ];      
    
      if( addrbookElement["term"] == "U" ) {

        // Compose record.
        let record = {
          'entid':      addrbookElement['entid'],  // addressbook entid
          'id':         addrbookElement['id'],
          'class':      "addrBook",
          'text':       addrbookElement['label'],
          'icon':       CONST_CLASS_ICO['addrBook'],
          'callnumber': addrbookElement['callnumber'],
          'status': "normal"
        };

        // Update or add record.
        if( this.Has( record['id'] ) ) {
          this.Update( record );
        }
        else {
          this.Add( record );
          //console.log( `----- addressbook Add: calltarget-Data: ${ this.Size() } items` );           
        }
      }
      else if( addrbookElement["term"] == "D" ) {
        this.Delete( addrbookElement["id"] );
        //console.log( `----- addressbook Delete: calltarget-Data: ${ this.Size() } items` );         
      }

    } // for
  }
 
}﻿//=============================================================================
//
//  entinfo Manager
//  (c) 2021 CYTEL Technology AG
//
//  Description: Distribute entInfo update/delete events.
//  No data is held by this object, it is all inside treeData.
//
//  Class name is as suggested by FJ.
//
//  Editor: Peter Aidam
//          18.06.2021
//
//=============================================================================

class EntinfoManager extends StaticEventSource
{
  //-----------------------------------------------------------------
  //  CONSTRUCTOR
  //-----------------------------------------------------------------

  constructor()
  {
    super();

    this.indexIp = {};
  }


  //-----------------------------------------------------------------
  //  INTERFACE
  //-----------------------------------------------------------------

  EntityIdFromIp( ipport )
  {
    if( ipport in this.indexIp ) {
      return this.indexIp[ ipport ];
    }
    return null;
  }  

  //-----------------------------------------------------------------
  //  PROCESS UPDATE / DELETE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // Used by treeData to send static event (treeData is not derived
  // from StaticEventSource).
  //
  ProcessUpdate( entinfo, partialElementKeyList )
  {
    // index
    if( 'ip' in entinfo ) {
      this.HelperRemoveEntityIdFromIndexIp( entinfo['entid'] );
      this.indexIp[ entinfo['ip'] ] = entinfo['entid'];
    }

    this.Publish( "update", entinfo, partialElementKeyList );

    // Also send wildcard event, because there's no wildcard subscription available.
    // TODO: remove these events if not used!
    Data.Instance().Publish( "ENTINFO/UPDATE/" + entinfo['entid'], entinfo, partialElementKeyList );
    Data.Instance().Publish( "ENTINFO/UPDATE/*", entinfo, partialElementKeyList );
  }

  //-----------------------------------------------------------------
  // Used by treeData to send static event (treeData is not derived
  // from StaticEventSource).
  //  
  ProcessDelete( entityId )
  {
    this.HelperRemoveEntityIdFromIndexIp( entityId );
    
    this.Publish( "delete", entityId );

    // Also send wildcard event, because there's no wildcard subscription available.
    // TODO: remove these events if not used!    
    Data.Instance().Publish( "ENTINFO/DELETE/" + entityId, entityId );
    Data.Instance().Publish( "ENTINFO/DELETE/*", entityId );
  }

  //-----------------------------------------------------------------
  //  helper removes an entityId from an index
  //-----------------------------------------------------------------

  HelperRemoveEntityIdFromIndexIp( entityId )
  {
    for( var key in this.indexIp ) {
      if( this.indexIp[ key ] == entityId ) {
        delete this.indexIp[ key ];
      }
    }
  }


}﻿//=============================================================================
//
//  Calltarget List
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//
//  Description: Dataset for calltargets
//  record :=
//    address:  statepoint's address
//    type:     type BOOL, REDIR, etc
//    entid:    entity id this statepoint is attached
//    class:    entity class of this statepoint
//    label:    statepoint's label
//    icon:     icon of entity
//  
//=============================================================================

class StatepointDataset extends Dataset 
{
  //-----------------------------------------------------------------
  //  CLASS VARIABLES
  //-----------------------------------------------------------------

  eventId1 = 0;
  eventId2 = 0;
  eventId3 = 0;
  entityStatepointIndex = {};

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  constructor( options )
  {
    super( );

    options['id'] = "address";
    options['preview'] = this.OnCreatePreview.bind( this );
    this.Initialize( options );
  }


  //-----------------------------------------------------------------
  //  DATASET OVERRIDABLES
  //-----------------------------------------------------------------

  OnCreatePreview( record )
  {
    // return preview
    return {
      id: record['address'],
      text: record['label'],
      icon: record['icon'],
      status: "normal"
    };
  }

  //-----------------------------------------------------------------
  //  data will call Start after basic initialization
  //    
  Start()
  {
    // subscribe entinfo update/create messages
    this.eventId1 = Data.Instance()._entinfoManager.On( "update", this.EntinfoUpdate.bind( this ) );
    this.eventId2 = Data.Instance()._entinfoManager.On( "create", this.EntinfoUpdate.bind( this ) );
    this.eventId3 = Data.Instance()._entinfoManager.On( "delete", this.EntinfoDelete.bind( this ) );
  }

  //-----------------------------------------------------------------
  //  data will call stop if logoff
  //    
  Stop()
  {
    // Unsubscribe mailbox event.
    Data.Instance()._entinfoManager.Off( this.eventId1 );
    Data.Instance()._entinfoManager.Off( this.eventId2 );
    Data.Instance()._entinfoManager.Off( this.eventId3 );

  }

  //-----------------------------------------------------------------
  //  INTERFACE TO OUTSIDE
  //-----------------------------------------------------------------

  StatepointInfo( spAddress, localUnsavedEntity )
  {
    if( typeof spAddress != "string" || spAddress.length < 5 ) {
      return null;
    }

    if( $.isPlainObject( localUnsavedEntity ) ) {
      return this._LocalStatepointInfo( spAddress, localUnsavedEntity );
    }

    if( this.Has( spAddress ) ) {
      let spRecord = this.Get( spAddress );
      return $.clone( spRecord );
    }
    return null;
  }

  _LocalStatepointInfo( spAddress, localUnsavedEntity )
  {
    // default result
    let resultSpInfo = {
      'address': "",
      'name': "none",
      'type': "ERROR",
      'label': $.lang( "{$const_deleted_item}" ),
      'entid': localUnsavedEntity['entid'],
      'class': localUnsavedEntity['class'],
      'icon': CONST_CLASS_ICO["none"]
    };

    // prequisites
    let spInfo = this.Get( spAddress );
    let isLocalStatepoint = ( spAddress.indexOf( ( localUnsavedEntity['entid'] + "/" ) ) >= 0 );
    if( !isLocalStatepoint && spInfo ) {
      return $.clone( spInfo );
    }
    else if( !isLocalStatepoint ) {
      return resultSpInfo;
    }

    // build entinfo from local statpoint
    let localEntid = spAddress.split( "/" )[ 1 ];
    let localSpName = spAddress.split( "/" )[ 2 ];
    let localStatepoint = localUnsavedEntity['sp'][ localSpName ];
    if( localStatepoint )  {
      resultSpInfo = {
        'address': Data.Instance().RootEntid() + "/" + localUnsavedEntity['entid'] + "/" + localSpName,
        'name': localSpName,
        'type': localStatepoint['type'],
        'label': $.lang( localStatepoint['label'] || "" ),
        'entid': localUnsavedEntity['entid'],
        'class': localUnsavedEntity['class'],
        'icon': CONST_CLASS_ICO[ localUnsavedEntity['class'] ]
      };

      // check if name has been changed, but not yet saved
      let entinfo = Data.Instance().Entinfo( localEntid );
      resultSpInfo['label'] = resultSpInfo['label'].replace( "{{name}}", localUnsavedEntity['name'] );
    }
    
    return resultSpInfo;
  }

  //-----------------------------------------------------------------
  //  INTERFACE TO LIST CONTAINER
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Update: called by listsContainer after mailbox or xhr events
  //
  EntinfoUpdate( entinfo )
  {
    // temporary 
    let entityAddress = Data.Instance().RootEntid() + "/" + entinfo['entid'];

    // if this entinfo has statepoints
    if( ( 'sp' in entinfo ) && ( $.isPlainObject( entinfo['sp'] ) ) ) {

      // first time create section for this entity
      if( !( entinfo['entid'] in this.entityStatepointIndex ) ) {
        this.entityStatepointIndex[ entinfo['entid'] ] = {};
      }

      // a uuid to detected deleted SP's
      let updateUuid = $.uuid();

      // loop through it and add
      for( let spName in entinfo['sp'] )  {
        let statepoint = entinfo['sp'][ spName ];
        let spAddress = entityAddress + "/" + spName;

        // skip hidden statepoints
        if( 'hidden' in statepoint && statepoint['hidden'] === true ) {
          continue;
        }

        // add to index
        this.entityStatepointIndex[ entinfo['entid'] ][ spAddress ] = updateUuid;

        // build + add record
        let record = {
          'address': spAddress,
          'name': spName,
          'type': statepoint['type'],
          'label': $.lang( statepoint['label'] ),
          'entid': entinfo['entid'], 
          'class': entinfo['class'],   
          'icon': CONST_CLASS_ICO[ entinfo['class'] ]
        };
        if( this.Has( record['address'] ) ) {
          this.Update( record );
        }
        else {
          this.Add( record );
        }
      }//for

      // remove all deleted statepoints
      for( let spAddress in this.entityStatepointIndex[ entinfo['entid'] ] )  {
        if( this.entityStatepointIndex[ entinfo['entid'] ][ spAddress ] !=  updateUuid ) {
          this.Delete( spAddress );
        }
      }

    }
  }

  //-----------------------------------------------------------------
  //  Delete: called by listsContainer after mailbox or xhr events
  //
  EntinfoDelete( entityId )
  {
    // remove all deleted statepoints
    for( let spAddress in this.entityStatepointIndex[ entityId ] )  {
      this.Delete( spAddress );
    }   
    delete this.entityStatepointIndex[ entityId ]; 
  }



 
}﻿//=============================================================================
//
//  Entity Dataset
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          17.06.2021
//
//  Description: Dataset for choose entity 
//  record :=  {
//    'id':       entity-id, or addressbook RECORD id
//    'class':    entinfo entityId or "addrBook"
//    'text':     entity[name] / addrBkRecord[label]
//    'icon':     icon from const
//    'account':  login for admin or client
//  }
//  
//=============================================================================

class EntityDataset extends Dataset 
{
  //-----------------------------------------------------------------
  //  CLASS VARIABLES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  data will call Start after basic initialization
  //    
  Start()
  {
    // subscribe entinfo update/create messages
    this.eventId1 = Data.Instance()._entinfoManager.On( "update", this.EntinfoUpdate.bind( this ) );
    this.eventId2 = Data.Instance()._entinfoManager.On( "create", this.EntinfoUpdate.bind( this ) );
    this.eventId3 = Data.Instance()._entinfoManager.On( "delete", this.EntinfoDelete.bind( this ) );
  }

  //-----------------------------------------------------------------
  //  data will call stop if logoff
  //    
  Stop()
  {
    // Unsubscribe mailbox event.
    Data.Instance()._entinfoManager.Off( this.eventId1 );
    Data.Instance()._entinfoManager.Off( this.eventId2 );
    Data.Instance()._entinfoManager.Off( this.eventId3 );

  }

  //-----------------------------------------------------------------
  //  data will call stop if logoff
  //    
  FindAccount( account )
  {
    let info = this.Filter( account, "account" );
    if( info.length > 0 ) {
      return this.Get( info[ 0 ] )['id'];
    }
    return null;
  }

  //-----------------------------------------------------------------
  //  data will call stop if logoff
  //    
  FindMacAddress( macAddr )
  {
    let info = this.Filter( macAddr, "macaddr" );
    if( info.length > 0 ) {
      return this.Get( info[ 0 ] )['id'];
    }
    return null;
  }

  

  //-----------------------------------------------------------------
  //  ENTINFO INTERFACE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Update: called by listsContainer after mailbox or xhr events
  //
  EntinfoUpdate( entinfo )
  {
    // filter internal entites
    if( ! ( 'name' in entinfo ) || entinfo['name'] == null )  {
      return;
    }
    if( entinfo['class'] == "admui" || entinfo['class'] == "userstatus" ) {
      return;
    }
    if( entinfo['entid'].substr( 0, 4) == "menu")  {
      return;
    }
    
    let record = {
      'id':         entinfo['entid'],   //  entity-id
      'class':      entinfo['class'],   //  entity-class
      'text':       $.lang( entinfo['name'] ) || "",
      'icon':       CONST_CLASS_ICO[ entinfo['class'] ],
      'account':    ( 'account' in entinfo ) ? entinfo['account'] : null,
      'macaddr':    ( 'macaddr' in entinfo ) ? entinfo['macaddr'] : null
    };
    if( this.Has( entinfo['entid'] ) )  {
      this.Update( record );
    }
    else {
      this.Add( record );
    }
  }

  //-----------------------------------------------------------------
  //  Delete: called by listsContainer after mailbox or xhr events
  //
  EntinfoDelete( entityId )
  {
    this.Delete( entityId );
  }


 
}﻿//=============================================================================
//
//  Permission List
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          22.06.2021
//
//  Description: Permission module
//  permissionList :=  {
//    "admin.1": [
//      { "p": "rw",  "w": "*",  "f": "redir"  },
//      { "p": "rw",  "w": "*",  "f": "dnd"    }
//    ],
//    ...more 
//  }  
//
//  debugging code: 
//  dbgPerm :=  {
//    "apartment.1": {
//      "system.fj/door.1/camera": { i: "rw", s: "rw" },
//      "system.fj/door.2/redir":  { i: "rw", s: "rw" }
//    }
//  }
//=============================================================================

class PermissionList 
{
  //-----------------------------------------------------------------
  //  CLASS VARIABLES
  //-----------------------------------------------------------------

  entinfoList = {};
  dbgPerm = {};   // debugging code
  eventId1 = 0;
  eventId2 = 0;
  eventId3 = 0;
  ownPermission = {
    "w": "*",
    "f": "*",
    "p": "rw",
    "cr": "",
    "inherit": false
  };

  //-----------------------------------------------------------------
  //  INITIALIZE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  data will call Start after basic initialization
  //    
  Start()
  {
    // subscribe entinfo update/create messages
    this.eventId1 = Data.Instance()._entinfoManager.On( "update", this.EntinfoUpdate.bind( this ) );
    this.eventId2 = Data.Instance()._entinfoManager.On( "create", this.EntinfoUpdate.bind( this ) );
    this.eventId3 = Data.Instance()._entinfoManager.On( "delete", this.EntinfoDelete.bind( this ) );
  }

  //-----------------------------------------------------------------
  //  data will call stop if logoff
  //    
  Stop()
  {
    // Unsubscribe mailbox event.
    Data.Instance()._entinfoManager.Off( this.eventId1 );
    Data.Instance()._entinfoManager.Off( this.eventId2 );
    Data.Instance()._entinfoManager.Off( this.eventId3 );

    this.entinfoList = {};
  }

  //-----------------------------------------------------------------
  //  ENTINFO EVENTS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Update: called by entInfoManager after mailbox or xhr events
  //
  EntinfoUpdate( entinfo )
  {
    this.entinfoList[ entinfo['entid'] ] = {};
    this.entinfoList[ entinfo['entid'] ]['permission'] = $.clone( entinfo['permission'] );
    this.entinfoList[ entinfo['entid'] ]['entid'] = entinfo['entid'];
    if( 'parent' in entinfo && typeof entinfo['parent'] == "string" )  {
      this.entinfoList[ entinfo['entid'] ]['parent'] = entinfo['parent'];
    }
  }

  //-----------------------------------------------------------------
  //  Delete: called by listsContainer after mailbox or xhr events
  //
  EntinfoDelete( entityId )
  {
    delete this.entinfoList[ entityId ];
  }


  //-----------------------------------------------------------------
  //  INTERFACE
  //-----------------------------------------------------------------

  IsAllowed( inquirerEntityId, targetStatepointAddr, right )
  {
    let permission = this.Get( inquirerEntityId, targetStatepointAddr );
    if( permission ) return ( permission["p"].indexOf( right ) >= 0 );
    return false;
  }


  //-----------------------------------------------------------------
  //  INTERNAL CALCULATION
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  build path of entities starting with inquirer
  //
  _CalcEntityPath( inquirerEntityId )
  {
    // prequisites
    let entityPath = inquirerEntityId;
    let parent = "";

    // build inquirer parent list
    let entinfo = this.entinfoList[ inquirerEntityId ];
    if( !entinfo ) {
      return entityPath;
    }
    parent = entinfo['parent'];

    // loop to root
    while( typeof parent == "string" ) {
      
      entinfo = this.entinfoList[ parent ];
      if( !entinfo ) {
        return null;
      }
      // change against c++ implementation: entinfo does not contain entid
      // bug => entityPath = entinfo["entid"] + "\\" + entityPath;
      entityPath = parent + "\\" + entityPath;
      parent = entinfo['parent'];
    }

    entityPath += "\\";
    return entityPath;
  }

  //-----------------------------------------------------------------
  //  build path of entities starting with inquirer
  //
  _CalcApplicablePermRecord( entinfo, inquirerPath, targetStatepointName )
  {
    // prepare params
    let permissionList = entinfo['permission'] || [];
    let maxIdx = -1;
    let maxLevel = -1;
    let cr = false;

    // loop through the list of permission records (of this entity)
    for( var i = 0; i < permissionList.length; i++ ) {

      // get the current perm record 
      let currentPermission = permissionList[ i ];
      let level = -1;
      let crFlag = false;

      //-------------------------------------------------------------------
      // Phase 1: check 'who', exlicit (e.g. 'w':'user.2') has top-priority

      //only rights with the wildcast or the correct inquirerEntityId are possibles rights for the dialog 
      if( currentPermission['w'] != '*' ) {
        // check if the inquirer (or it's parents is 'w') 
        var pos = inquirerPath.search( RegExp( "\\b" + currentPermission['w'] + "\\b", "g" ) );
        if( pos < 0 ) {
          continue;   // skip 
        }
        else {
          level = pos * 1000000;
        }
      }
      // *: check if the inquirer is inherited from target, else skip
      else if( inquirerPath.indexOf( entinfo['entid'] + "\\" ) < 0 ) { 
        continue;
      }

      //-------------------------------------------------------------------
      // Phase 2: check 'filter', without wildcard better prio
      if( currentPermission["f"].indexOf( "*" ) < 0 ) {   // if last character isn't '*'
        if( targetStatepointName == currentPermission['f'] ) {
          level = level + targetStatepointName.length * 1000;
        }
        else {
          continue;
        }
      }
      // if the $target matches the first chars of 'f' e.g.: strcmp( 'pres*', 'presence', 4 )
      else if( ( targetStatepointName + '*' ).indexOf( currentPermission['f'] ) >= 0 ) {
        crFlag = true;
        if( level < 0 )
          level = 0;

        level = level + currentPermission['f'].length - 1;
      }
      else {
        continue;
      }

      //-------------------------------------------------------------------
      // Phase 3: check what is best
      if( level > maxLevel ) {
        maxLevel = level;
        maxIdx = i;
        cr = crFlag;
      }

    }//for  

    if( maxIdx >= 0 ) {
      let res = $.clone( permissionList[ maxIdx ] );
      res['cr'] = cr;
      return res;
    }

    return null;
  }

  //-----------------------------------------------------------------
  //  INTERFACE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  calcluate permissions for permission dialog
  //
  GetAttributes( inquirerEntityId, targetStatepointAddr )
  {
    
    // prepare variables
    let result                  = [];    //BVar::Array();
    let resultPermissionRecord0 = null;  //BVar::Null();
    let resultPermissionRecord1 = null;  //BVar::Null();
    let inquirerPath            = this._CalcEntityPath( inquirerEntityId );
    targetStatepointAddr        = targetStatepointAddr.Split( "/" );    // convert string to array
    let targetEntityId          = targetStatepointAddr[ 1 ];
    let targetStatepointName    = targetStatepointAddr[ 2 ];
  
    //--[ Attribute record #0 ]---------------------------------------
    // 
    // iterate the entities upwards using the parent field
    let currentEntity = this.entinfoList[ targetEntityId ]; //m_entityList.GetAt( targetEntityId );
    let inherit = false;
    while( currentEntity && ( typeof currentEntity == "object" ) )  {
  
      // if the entity has permissions..
      if( 'permission' in currentEntity )  {
        resultPermissionRecord0 = this._CalcApplicablePermRecord( currentEntity, inquirerPath, targetStatepointName );
        if( resultPermissionRecord0 && ( typeof resultPermissionRecord0 == "object" ) )  {
          resultPermissionRecord0["holder"] = currentEntity["entid"];
          resultPermissionRecord0["inherit"] = inherit ? true : false;
          result.Push( resultPermissionRecord0 );
          break;
        }
      }
      
      // get parent and next
      inherit = true;
      if( 'parent' in currentEntity ) {
        currentEntity = this.entinfoList[ currentEntity['parent'] ];  //m_entityList.GetAt( currentEntity["parent"] );
      }
      else  {
        currentEntity = null;
      }
        
    }//while
  
    //--[ Attribute record #1 ]---------------------------------------
    // 
    // if the right is not inherited, go upwards one level 
    if( !inherit )  {
  
      // get parent
      if( 'parent' in currentEntity )  {
        currentEntity = this.entinfoList[ currentEntity['parent'] ];  //m_entityList.GetAt( currentEntity["parent"] );
      }
      else  {
        currentEntity = null;
      }
  
      while( currentEntity && ( typeof currentEntity == "object" ) )  {
        // if the entity has permissions..
        if( 'permission' in currentEntity )  {
          resultPermissionRecord1 = this._CalcApplicablePermRecord( currentEntity, inquirerPath, targetStatepointName );
          if( resultPermissionRecord1 && ( typeof resultPermissionRecord1 == "object" ) )  {
            resultPermissionRecord1["holder"] = currentEntity["entid"];
            resultPermissionRecord1["inherit"] = 1;
            result.Push( resultPermissionRecord1 );
            break;
          }
        }
      
        // get parent and next
        if( 'parent' in currentEntity )  {
          currentEntity = this.entinfoList[ currentEntity['parent'] ];  //m_entityList.GetAt( currentEntity["parent"] );
        }
        else  {
          currentEntity = null;
        }
        
      }//while
  
    }
    // calc #1: in case of inherit just double the record
    else  {
      result.Push( resultPermissionRecord0 );
    }
  
    return $.clone( result );
  }

  //-----------------------------------------------------------------
  //  calcluate permissions given a statepoint and inquirer
  //
  Get( inquirerEntityId, targetStatepointAddr )
  {
    // debugging code
    let dbgTargetSpAddr = targetStatepointAddr;

    // prepare variables
    let resultPermissionRecord = null;
    let inquirerPath = this._CalcEntityPath( inquirerEntityId );
    if( inquirerPath == null )  {
      return null;
    }    
    targetStatepointAddr      = targetStatepointAddr.split( "/" );    // convert string to array
    let targetEntityId        = targetStatepointAddr[ 1 ];
    let targetStatepointName  = targetStatepointAddr[ 2 ];
  
    // Make sure there are NO permissions for an empty enquirer.
    if( inquirerEntityId.length <= 0 || targetEntityId.length <= 0 )  {
      return resultPermissionRecord;
    }
  
    // exception on self, regardless what the permissions in 
    // entity say we always return [ "rw", "*", "*" ]
    if( inquirerEntityId == targetEntityId )  {
      resultPermissionRecord = $.clone( this.ownPermission );
      return resultPermissionRecord;
    }
  
    // iterate the entities upwards using the parent field
    let currentEntityId = targetEntityId;
    let inherit = false;
  
    while( typeof currentEntityId == "string" ) {
  
      // get reference to current entity
      let currentEntity = this.entinfoList[ currentEntityId ];
  
      // if the entity has permissions..
      if( currentEntity && ( 'permission' in currentEntity ) )  {
        
        resultPermissionRecord = this._CalcApplicablePermRecord( currentEntity, inquirerPath, targetStatepointName );
        if( resultPermissionRecord && ( typeof resultPermissionRecord == "object" ) ) {
          resultPermissionRecord["holder"] = currentEntity["entid"];
          resultPermissionRecord["inherit"] = inherit ? true : false;
          break;
        }              
      }
  
      // get parent id and next
      inherit = true;
      if( currentEntity && ( 'parent' in currentEntity ) )  {
        currentEntityId = currentEntity["parent"];
      }
      else  {
        currentEntityId = null;
      }
        
    }
  
    //----- debugging code -----
    /*
    if( ! Data.Instance().IsLoading( ) )  {

      if( !( inquirerEntityId in this.dbgPerm ) ) {
        this.dbgPerm[ inquirerEntityId ] = {};
      }
      if( !( dbgTargetSpAddr in this.dbgPerm[ inquirerEntityId ] ) ) {
        this.dbgPerm[ inquirerEntityId ][ dbgTargetSpAddr ] = {
          i: resultPermissionRecord['p'] || null,
          ih: resultPermissionRecord['holder'] || null,
          s: null,
          sh: null
        };
        
        //new XhrRequest( "PERMISSION.GET_ATTRIBUTES", { 'inquirerEntityId': inquirerEntityId, 'targetStatepointAddr': dbgTargetSpAddr  }, ( response ) => {      
        //  console.log( response );
        //} );

        new XhrRequest( "PERMISSION.GET", { 'inquirerEntityId': inquirerEntityId, 'targetStatepointAddr': dbgTargetSpAddr  }, ( response ) => {      
          this.dbgPerm[ inquirerEntityId ][ dbgTargetSpAddr ]['s'] = response['data']['p'];
          this.dbgPerm[ inquirerEntityId ][ dbgTargetSpAddr ]['sh'] = response['data']['holder'];
          if( response['data']['p'] != this.dbgPerm[ inquirerEntityId ][ dbgTargetSpAddr ]['i'] )  {
            console.log( "i:", inquirerEntityId, "->", dbgTargetSpAddr, this.dbgPerm[ inquirerEntityId ][ dbgTargetSpAddr ] );
          }
        } );    
        
      }
    }
    */
    //------ end debug ------

    return resultPermissionRecord;
  }

}﻿//=============================================================================
//
//  logfile data
//  (c) 2021 CYTEL Technology AG
//
//  Description: All logfile XHR requests.
//  
//  Events only. Logfile data is not stored here.
//
//  This is not a dataset!
//
//  Editor: Peter Aidam
//          20.07.2021
//
//=============================================================================

class LogfileData
{

  //-----------------------------------------------------------------
  //  XHR requests
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // Get complete logfile from server.
  //
  ListRequest( ) {
    let xhrRequest = new XhrRequest( "ADMIN.LOGFILE", null, ( response ) => {
      this.XhrResponse( response );
    } );
    return true;
  }

  //-----------------------------------------------------------------
  // Delete complete logfile from server.
  //
  DeleteRequest( ) {
    let xhrRequest = new XhrRequest( "ADMIN.LOGFILE_DELETE", null, ( response ) => {
      this.XhrResponse( response );
    } );
    return true;
  }

  //-----------------------------------------------------------------
  // Add a record to logfile on server.
  //
  // Example of logfileRecord:
  // let logRecord = {
  //   'type':     'info',
  //   'admin':    'admin.1',
  //   'entid':    'timeline.33',
  //   'message':  "{$STORE_UPDATE}",      
  //   'param':  {     
  //    'name': "Zeitprofil 33"
  //   }
  // };
  //
  AddRecordRequest( logfileRecord ) {
    let xhrRequest = new XhrRequest( "ADMIN.LOGFILE_ADDRECORD", { 'logfileRecord': logfileRecord }, ( response ) => {
      this.XhrResponse( response );
    } );
  }


  //-----------------------------------------------------------------
  // XHR response handling
  //    
  XhrResponse( response )
  {
    //console.log( 'Logfile::XhrResponse for ' + response['meta']['command'] ); 

    let command      = response['meta']['command'];
    let responseCode = parseInt( response['meta']['status'] );    

    // We are done if server was not available.
    if( responseCode == 503 ) {
      return;
    }

    //-- GET ----------------------------------------------------
    if( command == "ADMIN.LOGFILE" )  {

      // Prerequisites.
      let logArray = [];

      // Translate server's logfile array into a displayable array (custom data).
      if( Array.isArray( response['data']['logfile'] ) )  {
        for( let arrayRecord of response['data']['logfile'] ) {

          let displayableRecord = {};
          displayableRecord["guid"]    = arrayRecord["guid"];
          displayableRecord["type"]    = arrayRecord["type"];
          displayableRecord["created"] = arrayRecord["created"];
          displayableRecord["entid"]   = arrayRecord["entid"];

          // Get some additional info from the entity's entinfo.
          let entinfo = Data.Instance().Entinfo( arrayRecord["entid"] );
          if( typeof entinfo == "object" )  {
            displayableRecord["name"] = entinfo["name"];
          }
          else  {
            displayableRecord["name"] = "{$const_deleted_item_short}";
          }

          // Get some additional info from the admin's entinfo.        
          let adminEntinfo = Data.Instance().Entinfo( arrayRecord["admin"] );
          if( typeof adminEntinfo == "object" )  {
            displayableRecord["admin"] = adminEntinfo["name"];
          }
          else  {
            displayableRecord["admin"] = "{$const_deleted_item_short}";
          }

          // Displayable message.
          displayableRecord["message"] = this._renderMessage( arrayRecord );

          logArray.push( displayableRecord );
        } // for
      } // if array

      // Publish event.
      Data.Instance().Publish( "LOGFILE/LIST/", logArray );
    }

    //-- DELETE -------------------------------------------------
    else if( command == "ADMIN.LOGFILE_DELETE" )  {
      // Publish event.
      Data.Instance().Publish( "LOGFILE/LIST/DELETE/" );
    }
  }


  //-----------------------------------------------------------------
  //  HELPERS
  //-----------------------------------------------------------------    

  //-----------------------------------------------------------------
  //  Convert the message into a displayable string 
  //
  _renderMessage( record )
  {
    let msg = $.lang( record.message );
    if( record.param )  {
      msg = Mustache.render( msg, record.param );    
    }  
    msg = $.lang( msg );  
    return msg;
  }  

}﻿//=============================================================================
//
//  Callprofile Dataset
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Peter Aidam
//          21.07.2021
//
//  Description: 
//  - Fills callprofiles from entity "admui.callprofile" into dataset.
//  - Updates dataset if "admui.callprofile" is updated.
//
//  Used to fill dropdowns. So nothing more than id + name is required.
//
//  record :=  {
//    'id':     callprofile-id
//    'icon':   "mdi mdi-phone-lock", 
//    'text':   callprofile-name
//  }
//  
//=============================================================================

class CallprofileDataset extends Dataset 
{
  //-----------------------------------------------------------------
  //  CONSTRUCTOR
  //-----------------------------------------------------------------

  constructor( options )
  {
    super( options );

    // Subscribe entinfo update/create message.
    Data.Instance()._entinfoManager.On( "update", this.ProcessEntinfoUpdate.bind( this ) );
  }

  //-----------------------------------------------------------------
  //  PUBLIC INTERFACE
  //-----------------------------------------------------------------

  ProfileInfo( profileId )
  {
    return this.Get( profileId );
  }

  //-----------------------------------------------------------------
  //  ENTINFO INTERFACE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Update: called by entInfoManager after mailbox or xhr events
  //
  ProcessEntinfoUpdate( entinfo )
  {
    // Only interested in one special entity.
    if( entinfo['entid'] == "admui.callprofile" )  {
      this._ProcessProfileList( entinfo['callprofile'] );
    }
  }

  //-----------------------------------------------------------------
  //  HELPER
  //-----------------------------------------------------------------    

  //-----------------------------------------------------------------
  //  Merge profiles from "admui.callprofile" into dataset. 
  //
  _ProcessProfileList( callprofileList )
  {
    // reset database
    let newData = [];
    for( let rec of callprofileList )  {
      let newRecord = {};
      newRecord['id'] = rec['id'];
      newRecord['text'] = rec['name'];
      newRecord['icon'] = "mdi mdi-phone-lock";
      newData.push( newRecord );
    }
    this.SetData( newData );

  }

}﻿//=============================================================================
//
//  Reboot Devices
//  (c) 2021 CYTEL Technology AG
//
//  Description: Device Reboot requests and their responses from server.
//
//  Editor: Peter Aidam
//          11.10.2021
//
//=============================================================================

class Reboot
{
  //-----------------------------------------------------------------
  //  CONSTRUCTOR
  //-----------------------------------------------------------------

  constructor()
  {
  }


  //-----------------------------------------------------------------
  //  REBOOT requests
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // Reboot a rebootable (SSMP, devapi) device.
  //
  RebootDevice( entityId )
  {
    let xhrRequest = new XhrRequest( "ADMIN.REBOOT_DEVICE", { 'entityId': entityId }, ( response ) => {      
      this.XhrResponse( response );
    } );

    // Add Logfile record.
    let adminEntityId = SimpleCookies.Instance().Get( 'user' );
    let entinfo = Data.Instance().Entinfo( entityId );
    let logfileRecord = {
      'type':     'info',
      'admin':    adminEntityId,
      'entid':    entityId,
      'message':  "{$REBOOT_DEVICE}",
      'param': {
        'name': entinfo['name']
      }
    }
    Data.Instance().Logfile().AddRecordRequest( logfileRecord ); 
  }

  //-----------------------------------------------------------------
  // Reboot all rebootable (SSMP, devapi) devices which are marked 
  // for reboot.
  //  
  RebootDeviceNeeded( )
  {
    let xhrRequest = new XhrRequest( "ADMIN.REBOOT_DEVICE_NEEDED", null, ( response ) => {      
      this.XhrResponse( response );
    } );

    // Add Logfile record.
    let adminEntityId = SimpleCookies.Instance().Get( 'user' );
    let logfileRecord = {
      'type':     'info',
      'admin':    adminEntityId,
      'entid':    Data.Instance().RootEntid(),
      'message':  "{$REBOOT_NEEDED}"
    }
    Data.Instance().Logfile().AddRecordRequest( logfileRecord );     
  }

  //-----------------------------------------------------------------
  // Reboot all rebootable (SSMP, devapi) devices.
  //  
  RebootDeviceAll( )
  {
    let xhrRequest = new XhrRequest( "ADMIN.REBOOT_DEVICE_ALL", null, ( response ) => {      
      this.XhrResponse( response );
    } );

    // Add Logfile record.
    let adminEntityId = SimpleCookies.Instance().Get( 'user' );
    let logfileRecord = {
      'type':     'info',
      'admin':    adminEntityId,
      'entid':    Data.Instance().RootEntid(),
      'message':  "{$REBOOT_ALL}"
    }
    Data.Instance().Logfile().AddRecordRequest( logfileRecord );         
  }


  //-----------------------------------------------------------------
  //  XHR RESPONSE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // General XHR response handling
  //    
  XhrResponse( response )
  {
    let command      = response['meta']['command'];
    let responseCode = parseInt( response['meta']['status'] );

    // If error, we are done here.
    if( responseCode != 200 ) {
      return;
    }

    //-------------------------------------------------------
    if( command == "ADMIN.REBOOT_DEVICE_NEEDED" || command == "ADMIN.REBOOT_DEVICE_ALL" )  {

      // Server sends 'isOk' flag as response['data'].

      // Event for Dashboard.
      if ( response['data'] )  {
        // status == "started" is not used. Use Mailbox event DEVICE_RESTART instead.
      }
      else  {
        // status == "nothing_to_do".
        Data.Instance().Publish( "REBOOT_DEVICES/STARTED/", "nothing_to_do" );
      }
    }

  }

}﻿//=============================================================================
//
//  Addressbook
//  (c) 2021 CYTEL Technology AG
//
//  Description: Addressbook related requests and their responses from server.
//
//  Editor: Peter Aidam
//          12.10.2021
//
//=============================================================================

class Addressbook
{
  //-----------------------------------------------------------------
  //  CONSTRUCTOR
  //-----------------------------------------------------------------

  constructor()
  {
    this.addressbookCallback = null;    
  }


  //-----------------------------------------------------------------
  //  ADDRESSES (ADDRESSBOOKS)
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Get an address book, given by entity ID.
  //
  AddressbookRequest( addrBookEntityId, requestCallback )
  {
    // Save callback function for XhrResponse.
    this.addressbookCallback = requestCallback;

    // Do the request.
    new XhrRequest( "ADMIN.ADDRESSBOOK", { 'entityId': addrBookEntityId, 'isLastRequest': true }, ( response ) => {
      this.XhrResponse( response );
    } );
  }

  //-----------------------------------------------------------------
  //  Add a new address record to an addressbook given by its entityId.
  //
  AddRecord( addrBookEntityId, addressRecord )
  {
    let xhrRequest = new XhrRequest( "ADMIN.ADDRESSBOOK_ADDRECORD", { 'addrBookEntityId': addrBookEntityId, 'addressRecord': addressRecord }, ( response ) => {
      // Don't eveluate xhr response: After address book modification server sends a mailbox event.
    } );
  }

  //-----------------------------------------------------------------
  //  Update an address record in an addressbook given by its entityId.
  //    
  UpdateRecord( addrBookEntityId, addressRecord )
  {
    let xhrRequest = new XhrRequest( "ADMIN.ADDRESSBOOK_UPDATERECORD", { 'addrBookEntityId': addrBookEntityId, 'addressRecord': addressRecord }, ( response ) => {
     // Don't eveluate xhr response: After address book modification server sends a mailbox event.
    } );
  }

  //-----------------------------------------------------------------
  //  Update an address record in an addressbook given by its entityId.
  //    
  DeleteRecord( addrBookEntityId, addressRecordId )
  {
    let xhrRequest = new XhrRequest( "ADMIN.ADDRESSBOOK_DELETERECORD", { 'addrBookEntityId': addrBookEntityId, 'addressRecordId': addressRecordId }, ( response ) => {
     // Don't eveluate xhr response: After address book modification server sends a mailbox event.
    } );
  }


  //-----------------------------------------------------------------
  //  XHR RESPONSE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // XHR response handling
  //    
  XhrResponse( response )
  {
    let command      = response['meta']['command'];
    let responseCode = parseInt( response['meta']['status'] );    

    // If error, we are done here.
    if( responseCode != 200 ) {
      return;
    }

    //-- ADDRESSBOOK----------------------------------------------
    if( command == "ADMIN.ADDRESSBOOK" ) {
      // Perform callback.
      if( this.addressbookCallback instanceof Function )  {
        this.addressbookCallback(  response['data']['addressBook'] );
      }
    }

  }  

}//=============================================================================
//
//  Keytones
//  (c) 2021 CYTEL Technology AG
//
//  Description: Keytones related requests and their responses from server.
//
//  Editor: Tobias Stumpf
//          16.11.2021
//
//=============================================================================

class KeytonesModel
{
  //-----------------------------------------------------------------
  //  CONSTRUCTOR
  //-----------------------------------------------------------------

  constructor()
  {
    this.keytoneCallback = null;
  }

  //=============================================================================
  //  STATICS
  //=============================================================================

 static KEY_TONE_TABLE = "folder.admin.keytones";

  //-----------------------------------------------------------------
  //  ADDRESSES (ADDRESSBOOKS)
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Get an address book, given by entity ID.
  //
  KeytoneRequest( requestCallback )
  {
    // Save callback function for XhrResponse.
     this.keytoneCallback = requestCallback;

    // Do the request.
    new XhrRequest( "ADMIN.KEYTONES", { 'entityId': KeytonesModel.KEY_TONE_TABLE, 'isLastRequest': true }, ( response ) => {
      this.XhrResponse( response );
    } );
  }

  //-----------------------------------------------------------------
  //  Add a new address record to an addressbook given by its entityId.
  //
  AddKeytone( keytoneRecord )
  {
    let xhrRequest = new XhrRequest( "ADMIN.KEYTONES_ADDRECORD", { 'addrBookEntityId': addrBookEntityId, 'addressRecord': addressRecord }, ( response ) => {
      // Don't eveluate xhr response: After address book modification server sends a mailbox event.
    } );
  }

  //-----------------------------------------------------------------
  //  Update an address record in an addressbook given by its entityId.
  //
  UpdateRecord( addrBookEntityId, addressRecord )
  {
    let xhrRequest = new XhrRequest( "ADMIN.KEYTONES_UPDATERECORD", { 'addrBookEntityId': addrBookEntityId, 'addressRecord': addressRecord }, ( response ) => {
      // Don't eveluate xhr response: After address book modification server sends a mailbox event.
    } );
  }

  //-----------------------------------------------------------------
  //  Update an address record in an addressbook given by its entityId.
  //
  DeleteRecord( addrBookEntityId, addressRecordId )
  {
    let xhrRequest = new XhrRequest( "ADMIN.KEYTONES_DELETERECORD", { 'addrBookEntityId': addrBookEntityId, 'addressRecordId': addressRecordId }, ( response ) => {
      // Don't eveluate xhr response: After address book modification server sends a mailbox event.
    } );
  }


  //-----------------------------------------------------------------
  //  XHR RESPONSE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  // XHR response handling
  //
  XhrResponse( response )
  {
    let command      = response['meta']['command'];
    let responseCode = parseInt( response['meta']['status'] );

    // If error, we are done here.
    if( responseCode != 200 ) {
      return;
    }

    //-- ADDRESSBOOK----------------------------------------------
    if( command == "ADMIN.KEYTONE" ) {
      // Perform callback.
      if( this.keytoneCallback instanceof Function )  {
        this.keytoneCallback(  response['data']['keytones'] );
      }
    }

  }
}//=============================================================================
//
//  Permission Deleter
//  (c) 2022 CYTEL Technology AG
//
//  Description: remove permissions holders that are deleted
//
//  Editor: Frank Jaekel
//          18.01.2022
//=============================================================================

class PermissionDeleter
{
  //---------------------------------------------------------------------------
  //  class variables
  //---------------------------------------------------------------------------

  timerId = null;


  //---------------------------------------------------------------------------
  //  constructor
  //---------------------------------------------------------------------------

  constructor( deleteEntityList, fnResult )
  {
    // get an array of all entInfos to be analysed
    let entInfoList = [];
    Data.Instance().TreeData().BranchEntInfoList( "folder.confdev", entInfoList );

    // build change list
    let changeList = [];
    for( let entinfo of entInfoList ) {
      var result = this.CheckPermissionsForDeleted( entinfo, deleteEntityList );
      if( result )  {
        changeList.push( result );
      }
    } // for

    // give list to data layer
    if( changeList.length > 0 )  {
      Data.Instance().SavePartialEntityList( changeList );
    }
    
  }


  //---------------------------------------------------------------------------
  //  implementation
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  check permission for a deleted record
  //
  CheckPermissionsForDeleted( entinfo, deleteEntityList )
  {
    if( 'permission' in entinfo && Array.isArray( entinfo['permission'] ) )  {
      if( entinfo['permission'].find( rec => deleteEntityList.includes( rec['w'] ) ) ) {

        let newPermissionList = entinfo['permission'].filter( ( rec ) => !( deleteEntityList.includes( rec['w'] ) ) );
        //console.log( "Found in", entinfo['entid'], newPermissionList, entinfo['permission'] );

        // save the entity
        let entity = {};
        entity['entid'] = entinfo['entid'];
        entity['permission'] = newPermissionList;
        return entity;

        //Data.Instance().SavePartialEntity( entity );

      }
    }

    return null;
  }

}//=============================================================================
//
//  Licenses
//  (c) 2022 CYTEL Technology AG
//
//  Description: Hold content of entity menu.licence
//
//  Editor: Frank Jaekel
//          03.02.2022
//
//=============================================================================

/*
  License         APR       other       class
  ------------------------------------------------------------------
  Administrator	  -                     admin
  Adressbuch	    -                     addrBook
  ATLC 670-0	    APR: 1                door
  Jung-Panel	    APR: 1    ALFP:  1    jungpanel
  Knx-Gateway	    -         ALKNX: 1    knxgw
  Ordner	        -                     folder
  Siedle AFS	    APR: 1                apartment
  Siedle AFSV	    APR: 1                apartment
  Siedle AHF	    APR: 1                apartment
  Siedle AHFV	    APR: 1                apartment
  Siedle AHT	    APR: 1                apartment
  Siedle AHTV	    APR: 1                apartment
  Siedle APP	    APR: 1    ALSA: 1     susapp
  Siedle ASC	    APR: 1    ASC:  1     client
  Siedle ASHT	    APR: 1    ASHT: 1     asht
  Siedle AVP	    APR: 1                avp
  SIP-Telefon	    APR: 1    ALFA: 1     sipphone
  SIP-Trunk	      -         ALT:  1-n   trunk
  TK-Gateway	    -         ALT:  1     tkgw
  TK-Telefon	    APR: 1                tkphone
  Zeitprofil	    -                     timeline
  AGW	            -                     agw
*/

class LicenceModel
{
  //-----------------------------------------------------------------
  //  CLASS VARIABLES
  //-----------------------------------------------------------------

  licenseEntity = null;

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  Start()
  {
  }

  Stop()
  {
  }

  //-----------------------------------------------------------------
  //  INTERFACE
  //-----------------------------------------------------------------

  Put( entity )
  {
    this.licenseEntity = entity;
  }

  Get( )
  {
    return this.licenseEntity['licence'];
  }

}

//=============================================================================
//
//  Modules
//  (c) 2021 CYTEL Technology AG
//
//  Description: Container for rules and other modules
//
//  Editor: Frank Jaekel
//          01.04.2022
//
//  rule: {
//    active: true
//    error: false
//    icon: "mdi mdi-account-check"
//    id: "presence"
//    permission: false
//    text: "Anzeige in Kontaktliste"
//    type: "rule"
//  }
//=============================================================================

class ModuleData
{
  //-----------------------------------------------------------------
  //  DATA
  //-----------------------------------------------------------------

  ruleList = [];
  ruleScript = {};
  entityRuleList = {};
  installedRuleList = [];
  entitiesToLoad = -1;

  newRuleList = null;


  //-----------------------------------------------------------------
  //  METHODS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  LOAD
  //
  Load( fnFinishedCallback )
  {
    // reset internal data
    this.ruleList = [];
    this.entityRuleList = {};

    this.ruleDataset = new Dataset( {
      id: "id"
    });

    var langId = Application.Instance().Language();

    // load the installed rules list
    new XhrRequest( "ADMINCONFIG.MODULE_LIST", { }, ( response ) => {
      this.installedRuleList = response['data'];
      this.LoadEntities( fnFinishedCallback );
    } );



  }//load


  //-----------------------------------------------------------------
  //  LOAD entities for further data
  //
  LoadEntities( fnFinishedCallback )
  {
    // all modules can be found as direct child of folder.rules
    var ruleList = Data.Instance()._treeData.GetBranch( "folder.rules" );
    this.entitiesToLoad = ruleList.length;

    // delete last item
    if( this.entitiesToLoad == 0 ) {
      if( $.isFunction( fnFinishedCallback ) ) {
        fnFinishedCallback();
      }
      return;
    }

    for( var rule of ruleList )  {
      new XhrRequest( "ADMIN.GET_ENTITY", { 'entityId': rule }, ( response ) => {

        this.entitiesToLoad--;

        if( 'data' in response )  {
          this.entityRuleList[ response['data']['entid'] ] = response['data'];
        }

        if( this.entitiesToLoad == 0 ) {
          this.AnalyseLoaded();
          if( $.isFunction( fnFinishedCallback ) )  {
            fnFinishedCallback();
          }
        }
      } );

    }
  }

  //-----------------------------------------------------------------
  //  get list of loaded rules
  //
  GetRuleList()
  {
    return this.ruleList;
  }


  //-----------------------------------------------------------------
  //  get list of loaded rules
  //
  GetRule( id )
  {
    return this.ruleList.find( ( rec ) => rec['id'] == id );
  }

  //-----------------------------------------------------------------
  //  get list of loaded rules
  //
  GetModuleByEntid( entityId )
  {
    return this.ruleList.find( ( rec ) => rec['entid'] == entityId );
  }
  
  //-----------------------------------------------------------------
  //  INTERNALS
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Prepare list for UI
  //
  AnalyseLoaded()
  {
    var langId = Application.Instance().Language();

    // first add all modules that are installed (analyse loaded entities)
    for( var entityId in this.entityRuleList )  {
      var entity = this.entityRuleList[ entityId ];

      for( var ruleId in entity['rules'] ) {
        
        var rule = entity['rules'][ ruleId ];
        if( rule['disabled'] ) {
          continue;
        }

        // check if packet needs a newer server
        var isNewerServerNeeded = false;
        if( entity['packet']['packet-minversion'] )  {
          var neededBuild = parseInt( entity['packet']['packet-minversion'].split( "-" )[ 1 ] );
          var installedVersion = Data.Instance().GetVersion();
          const regex = /\(.* (\d*)/;
          let matches = regex.exec( installedVersion );
          if( matches ) {
            var installedBuild = parseInt( matches[ 1 ] );
            isNewerServerNeeded = ( neededBuild > installedBuild );
          }
        }

        var record = {};
        record['id'] = ruleId;
        record['packet-id'] = entity['packet-id'];
        record['packet-minversion'] = entity['packet']['packet-minversion'] ? entity['packet']['packet-minversion'] : "";
        record['license'] = entity['license'];
        record['entid'] = entityId;
        record['run'] = entityId + "/" + ruleId;
        record['label'] = rule['meta']['label'][ langId ];
        record['icon'] = rule['meta']['icon'];
        record['version'] = rule['meta']['version'];
        record['created'] = rule['meta']['created'];
        record['allowed'] = rule['meta']['allowed'];
        record['required'] = rule['meta']['required'];
        record['statepoint'] = rule['meta']['statepoint'];
        record['parameter'] = rule['meta']['parameter'];
        record['error'] = false;
        if( isNewerServerNeeded ) {
          record['error'] = "version";
        }
        else if( 'license' in entity && 'error' in entity['license'] ) {
          record['error'] = "license";
        }
        if( record['error'] == false && 'sidebar-general' in rule['meta'] ) {
          record['sidebar-general'] = rule['meta']['sidebar-general'];
          Data.Instance().Publish( "MODULES/UPDATE-SIDEBAR", rule['meta']['sidebar-general'] );
        }

        for( var key in record['parameter'] ) {
          var parameter = record['parameter'][ key ];
          if( parameter && 'help' in parameter ) {
            var topicId = "rule/" + record['id'] + "/" + key;
            AccessApplication.Instance().HelpAddTopic( topicId, parameter['help'] );
          }
        }

        this.ruleList.push( record );
        this.ruleScript[ ruleId ] = rule['run-template'];
      }
    }

  }

}//=============================================================================
//
//  Modules
//  (c) 2021 CYTEL Technology AG
//
//  Description: Add an already uploaded module
//
//  Editor: Frank Jaekel
//          19.04.2022
//
//=============================================================================

class ModuleLoader
{
  //-----------------------------------------------------------------
  //  DATA
  //-----------------------------------------------------------------

  moduleId = null;
  moduleInfo = null;
  steps = null;
  fnProgress = null;
  restart = null;
  entityId = null;

  //-----------------------------------------------------------------
  //  METHODS
  //-----------------------------------------------------------------

  constructor( fnProgress = null )
  {
    // now start state machine
    this.statemachine = new Statemachine( [
        { id: "start",    text: "{$modload_info_starting}" },
        { id: "modinfo",  text: "{$modload_info_modinfo}"  },
        { id: "install",  text: "{$modload_info_install}"  },
        { id: "update",   text: "{$modload_info_update}"   },
        { id: "finished", text: "{$modload_info_finished}" }
      ],
      this.StatemachineStart.bind( this ),
      this.StatemachineNext.bind( this ),
      this.StatemachineFinished.bind( this )
    );

    this.entinfoEventId = Data.Instance()._entinfoManager.On( "update", this.EntinfoUpdate.bind( this ) );

    if( typeof fnProgress === "function" ) {
      this.fnProgress = fnProgress;
    }
  }
 
  //-----------------------------------------------------------------
  //  start installing
  //
  Start( uploadedModuleId )
  {
    // store data
    this.moduleId = uploadedModuleId;
    this.statemachine.Start();
  }

  //-----------------------------------------------------------------
  //  stop installing
  //
  Stop( status )
  {
    Data.Instance()._entinfoManager.Off( this.entinfoEventId );

    // hide progress dialog (after some time)
    var dialogTimeout = 1000;
    if( status ) {
      ProgressDialog.Message( "finished", true, "{$modload_info_finished}" );
    }
    else {
      ProgressDialog.Message( "error", -1, "{$modload_info_error}" );
      dialogTimeout = 2000;
    }
    setTimeout( () => {
      ProgressDialog.Hide();

      // send event to upper layer
      if( this.fnProgress ) {
        this.fnProgress( status, this.restart );
      }
    }, dialogTimeout );

  }


  //-----------------------------------------------------------------
  //  DATA-EVENTS
  //-----------------------------------------------------------------

  EntinfoUpdate( entinfo )
  {
    //console.log( `-> ENTINFO, ${entinfo.entid}` );

    if( 'steps' in this.moduleInfo )  {
      var index = this.moduleInfo['steps'].findIndex( ( rec ) => rec['type'] == "entity" && rec['data'] == entinfo['entid'] );
      if( index >= 0 )  {
        this.moduleInfo['steps'][ index ]['status'] = "finished";
      }
      this.StateInstallHelperCheckFinished();
    }
  }


  //-----------------------------------------------------------------
  //  STATES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  load module info 
  //
  StateModinfo()
  {
    // get module info
    new XhrRequest( "ADMINCONFIG.MODULE_INFO", this.moduleId, ( response ) => {
      this.moduleInfo = response['data'];
      //console.log( `  -> MODULEINFO:`, this.moduleInfo );

      var neededLicenses = this.moduleInfo['license'];
      var licenses = Data.Instance().Licences();
      for( var licenseKey in neededLicenses ) {
        var freeLicenses = licenses[ licenseKey ]['availabeLicenses'] - licenses[ licenseKey ]['engagedLicenses'];
        if( neededLicenses[ licenseKey ] > freeLicenses ) {
          console.warn( `${ licenseKey }: Not enough licenses: ${ freeLicenses } available, ${ neededLicenses[ licenseKey ] } needed!` );
          this.Stop( false );
          MessageBox.Show( {
            text: "{$modload_info_license}",
            layout: "fatal"
          } );          
          return;
        }
      }

      if( Array.isArray( this.moduleInfo['install'] ) && 'packet-id' in this.moduleInfo ) {
        this.moduleInfo['steps'] = [];
        for( var step of this.moduleInfo['install'] ) {
          step = step.split( ":" );
          this.moduleInfo['steps'].push( {
            type: step[ 0 ],
            data: step[ 1 ],
            status: "created"
          } );
        }
        ProgressDialog.Message( "modinfo", true, "{$modload_info_modinfo}" );
        this.statemachine.Next();
      }
      else {
        this.Stop( false );
        console.warn( `ModuleLoader.StateModinfo: Packet "${ this.moduleInfo['packet-id'] }" no install section!` );
      }      

    } );

  }

  //-----------------------------------------------------------------
  //  install all entities
  //
  StateInstall()
  {
    for( var step of this.moduleInfo['steps'] ) {
      if( step['type'] == "entity" ) {
        this.StateInstallEntity( step );
      }
      else if( step['type'] == "restart" ) {
        this.StateInstallRestart( step );
      }
      else {
        console.warn( `   -> Install: Unknown step type "${ step['type'] }"` );
        step['state'] = "finished";
        this.StateInstallHelperCheckFinished();
      }
    }
  }

  //-----------------------------------------------------------------
  //  get the new entity template from server
  //
  StateInstallEntity( step )
  {
    var entityId = step['data'];
    new XhrRequest( "ADMINCONFIG.MODULE_GET_ENTITY", { module: this.moduleInfo['packet-id'], entid: entityId }, ( response ) => {
      var entity = response['data'];
      if( entity['entid'] ) {
        this.entityId = entity['entid'];
        if( Data.Instance().Entinfo( entity['entid'] ) ) {
          Data.Instance().SaveEntity( entity );
        }
        else {
          Data.Instance().AddEntity( entity );
        }
      }
      else {
        this.Stop( false );
        console.warn( `ModuleLoader.StateInstallEntity: Entity "${ entityId }" not loaded!` );
      }
    } );    
  }

  //-----------------------------------------------------------------
  //  show restart toast
  //
  StateInstallRestart( step )
  {
    //console.log( step );
    this.restart = "restart:service";
    //AccessApplication.Instance().Command( 'reboot' );
  }


  //-----------------------------------------------------------------
  //  check if all steps are finished
  //
  StateInstallHelperCheckFinished()
  {
    var isFinished = this.moduleInfo['steps'].findIndex( ( rec ) => rec['status'] != "finished" );
    if( isFinished ) {
      ProgressDialog.Message( "install", true, "{$modload_info_install}" );
      this.statemachine.Next();
    }
  }

  //-----------------------------------------------------------------
  //  last step: update the module infos in data layer
  //
  StateUpdate()
  {
    Data.Instance().UpdateModules();
    
    ProgressDialog.Message( "update", true, "{$modload_info_update}" );
    this.statemachine.Next();
  }


  //-----------------------------------------------------------------
  //  STATE MACHINE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Start 
  //
  StatemachineStart( rec, current, numSteps )
  {
    //console.log( `-> START STATE: "${rec['id']}"` );
    ProgressDialog.Show();
    ProgressDialog.Message( rec['id'], true, rec['text'] );

    this.statemachine.Next();
  }

  //-----------------------------------------------------------------
  //  Next step
  //
  StatemachineNext( rec, current, numSteps )
  {
    //console.log( `-> NEXT STATE: "${ rec['id'] }"` );
    ProgressDialog.Message( rec['id'], false, rec['text'] );

    if( rec['id'] == "modinfo" )  {
      this.StateModinfo();
    }
    else if( rec['id'] == "install" ) {
      this.StateInstall();
    }
    else if( rec['id'] == "update" ) {
      this.StateUpdate();
    }

  }
  
  //-----------------------------------------------------------------
  //  finished
  //
  StatemachineFinished( rec, current, numSteps )
  {
    //console.log( this.moduleInfo );
    //console.log( `-> FINISHED STATE: "${ rec['id'] }"` );
    this.Stop( true );

    // Add Logfile record.
    let langId = AccessApplication.Instance().Language();
    let adminEntityId = SimpleCookies.Instance().Get( 'user' );
    let logfileRecord = {
        'type':     'info',
        'admin':    adminEntityId,
        'entid':    this.entityId ? this.entityId : adminEntityId,
        'message':  "{$STORE_CREATE}",      
        'param':  {     
          'name': this.moduleInfo['ui']['label'][ langId ]
        }
    };
    //Data.Instance().Logfile().AddRecordRequest( logfileRecord );    

  }

}//=============================================================================
//
//  Modules
//  (c) 2022 CYTEL Technology AG
//
//  Description: Remove an module
//
//  Editor: Frank Jaekel
//          20.04.2022
//
//=============================================================================

class ModuleDeleter
{
  //-----------------------------------------------------------------
  //  DATA
  //-----------------------------------------------------------------

  moduleEntityId = null;
  updateList = [];
  fnProgress = null;


  //-----------------------------------------------------------------
  //  METHODS
  //-----------------------------------------------------------------

  constructor( fnProgress = null )
  {
    // now start state machine
    this.statemachine = new Statemachine( [
      { id: "start",    text: "{$moddelete_info_starting}" },
      { id: "modinfo",  text: "{$moddelete_info_modinfo}"  },
      { id: "delete",   text: "{$moddelete_info_delete}"   },
      { id: "update",   text: "{$moddelete_info_update}"   },
      { id: "configs",  text: "{$moddelete_info_configs}"  },
      { id: "finished", text: "{$moddelete_info_finished}" }
      ],
      this.StatemachineStart.bind( this ),
      this.StatemachineNext.bind( this ),
      this.StatemachineFinished.bind( this )
    );

    this.eventDeleteId = Data.Instance()._entinfoManager.On( "delete", this.EntinfoDelete.bind( this ) );
    this.eventUpdateId = Data.Instance()._entinfoManager.On( "update", this.EntinfoUpdate.bind( this ) );

    if( typeof fnProgress === "function" ) {
      this.fnProgress = fnProgress;
    }
  }
 
  //-----------------------------------------------------------------
  //  start installing
  //
  Start( moduleEntityId )
  {
    // store data
    this.moduleEntityId = moduleEntityId;
    this.module = Data.Instance().ModuleByEntid( moduleEntityId );

    var refs = Verify.Instance().GetReferencedEntityIds( moduleEntityId );
    if( refs )  {
      for( var entid of refs ) {
        this.updateList.push( {
          type: "entity",
          data: entid,
          status: "created"
        } );
      }
    }
    else {
      this.updateList = [];
    }

    this.statemachine.Start();
  }

  //-----------------------------------------------------------------
  //  stop installing
  //
  Stop( status )
  {
    Data.Instance()._entinfoManager.Off( this.eventDeleteId );
    Data.Instance()._entinfoManager.Off( this.eventUpdateId );

    // hide progress dialog (after some time)
    var dialogTimeout = 1000;
    if( status ) {
      ProgressDialog.Message( "finished", true, "{$moddelete_info_finished}" );
    }
    else {
      ProgressDialog.Message( "error", -1, "{$moddelete_info_error}" );
      dialogTimeout = 2000;
    }
    setTimeout( () => {
      // hide dialog + send event to upper layer
      ProgressDialog.Hide();
      if( this.fnProgress ) {
        this.fnProgress( status, this.moduleInfo['remove'] );
      }
    }, dialogTimeout );

    // inform UI about modules change
    Data.Instance().Publish( "MODULES/UPDATED/" );

  }


  //-----------------------------------------------------------------
  //  DATA-EVENTS
  //-----------------------------------------------------------------

  EntinfoDelete( entityId )
  {
    //console.log( `-> ENTINFO-DEL, ${entityId}`, entityId );
    if( entityId == this.moduleEntityId ) {
      ProgressDialog.Message( "delete", true, "{$moddelete_info_delete}" );
      this.statemachine.Next();
    }
  }

  EntinfoUpdate( entinfo )
  {
    //console.log( `-> ENTINFO-UPD, ${ entinfo.entid }` );

    // check if entity is in our update list
    var index = this.updateList.findIndex( ( rec ) => rec['type'] == "entity" && rec['data'] == entinfo['entid'] && rec['status'] == "modified" );
    if( index < 0 )  {
      return;
    }

    this.updateList[ index ]['status'] = "finished";
    this.StateConfigsHelperCheckFinished( entinfo['entid'] );

  }


  //-----------------------------------------------------------------
  //  STATES
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  delete desired rule entity
  //
  StateStart( rec )
  {
    //console.log( `-> START STATE: "${ this.moduleEntityId }"` );
    ProgressDialog.Show();
    ProgressDialog.Message( rec['id'], true, rec['text'] );

    this.statemachine.Next();    
  }  

  //-----------------------------------------------------------------
  //  delete desired rule entity
  //
  StateDelete()
  {
    //console.log( `-> DELETE STATE: "${ this.moduleEntityId }":` );
    Data.Instance().DeleteEntity( this.moduleEntityId, false ); // false: no log-record
  }

  //-----------------------------------------------------------------
  //  remove configs in entities
  //
  StateConfigs()
  {
    console.log( `-> CONFIGS STATE:`, this.updateList );
    if( this.updateList.length == 0 )  {
      ProgressDialog.Message( "configs", true, "{$moddelete_info_configs}" );
      this.statemachine.Next();    
    }
    else {
      for( var rec of this.updateList ) {
        this.StateConfigsHelperUpdateEntity( rec['data'] );
      }
      setTimeout( () => {
        this.statemachine.Next();
      }, 1000 );
    }
  }

  //-----------------------------------------------------------------
  //  helper remove a config of an entity
  //
  StateConfigsHelperUpdateEntity( entitiyId )
  {
    //console.log( `  -> REQ ENTITY ${ entitiyId }` );

    // get entity from server
    Data.Instance().RequestEntity( entitiyId, ( entity ) => {
      
      //console.log( `  -> RESP ENTITY ${ entitiyId }`, entity );

      // get all installed rule configs for this module
      var removeKeyList = [];
      for( var ruleId in entity['rules'] ) {
        var ruleRun = entity['rules'][ ruleId ]['run'];
        if( ruleRun && ruleRun.indexOf( this.moduleEntityId ) >= 0 )  {
          //console.log( `  -> REMOVE ENTITY RULE ${ ruleId }`, ruleRun );
          removeKeyList.push( ruleId );
        }
      }

      // now remove them all
      for( var key of removeKeyList ) {
        var statepointName = entity['rules'][ key ]['config']['sp'] || null;
        if( statepointName )  {
          //delete entity['sp'][ statepointName ];
          WidgetDeviceConfigBase.HelperRemoveStatepoint( statepointName, entity );
          WidgetDeviceConfigBase.HelperRemovePermission( statepointName, entity );
        }
        delete entity['rules'][ key ];
      }

      // remove the references
      for( var index in entity['references'] ) {
        if( entity['references'][ index ].indexOf( this.moduleEntityId ) >= 0 ) {
          entity['references'].splice( index, 1 );
        }
      }

      // rules size parameter
      delete entity['rules']['size'];
      entity['rules']['size'] = Object.keys( entity['rules'] ).length;
      //console.log( `  -> REMOVE NEW ENTITY: `, entity['rules'] );

      // set status 
      var index = this.updateList.findIndex( ( rec ) => rec['type'] == "entity" && rec['data'] == entity['entid'] );
      this.updateList[ index ]['status'] = "modified";

      // prepare (userparam should not be updated) + save entity 
      delete entity['userparam'];
      Data.Instance().SavePartialEntity( entity );

    } );
  }

  //-----------------------------------------------------------------
  //  helper: check if all configs are removed
  //
  StateConfigsHelperCheckFinished()
  {
    var isFinished = this.updateList.find( ( rec ) => rec['status'] != "finished" );
    if( isFinished ) {
      ProgressDialog.Message( "configs", true, "{$moddelete_info_configs}" );
      //console.log( "CHECK HELPER -> Finished")
      this.statemachine.Next();
    }
  }

  //-----------------------------------------------------------------
  //  update rule list
  //
  StateUpdate()
  {
    //console.log( `-> UPDATE STATE` );

    Data.Instance()._modules.Load( () => {
      ProgressDialog.Message( "update", true, "{$moddelete_info_update}" );
      //console.log( "  -> UPDATE STATE", Data.Instance().RuleList() );
      this.statemachine.Next();    
    } );

  }

  //-----------------------------------------------------------------
  //  STATE MACHINE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  Start 
  //
  StatemachineStart( rec, current, numSteps )
  {
    this.StateStart( rec );
  }

//-----------------------------------------------------------------
  //  load module info 
  //
  StateModinfo()
  {
    // get module info
    new XhrRequest( "ADMINCONFIG.MODULE_INFO", this.module['packet-id'], ( response ) => {
      this.moduleInfo = response['data'];
      console.log( `  -> MODULEINFO:`, this.moduleInfo );

      ProgressDialog.Message( "modinfo", true, "{$moddelete_info_modinfo}" );
      this.statemachine.Next();

    } );

  }  
  //-----------------------------------------------------------------
  //  Next step
  //
  StatemachineNext( rec, current, numSteps )
  {
    //console.log( `-> NEXT STATE: "${ rec['id'] }"` );
    ProgressDialog.Message( rec['id'], false, rec['text'] );

    if( rec['id'] == "modinfo" ) {
      this.StateModinfo();
    }
    else if( rec['id'] == "delete" )  {
      this.StateDelete();
    }
    else if( rec['id'] == "configs" ) {
      this.StateConfigs();
    }
    else if( rec['id'] == "update" ) {
      this.StateUpdate();
    }

  }
  
  //-----------------------------------------------------------------
  //  finished
  //
  StatemachineFinished( rec, current, numSteps )
  {
    //console.log( `-> FINISHED STATE: "${ rec['id'] }"` );
    this.Stop( true );
  }

}//=============================================================================
//
//  Rule debugger
//  (c) 2022 CYTEL Technology AG
//
//  Description: get some informations about rules
//
//  Editor: Frank Jaekel
//          22.04.2022
//
//=============================================================================

class RuleDebugger extends EventSource
{
  //-----------------------------------------------------------------
  //  CLASS DATA
  //-----------------------------------------------------------------

  static _instance = null;
  ruleInstanceList = null;

  //-----------------------------------------------------------------
  //  SINGLETON 
  //  
  static Instance()
  {
    if( !RuleDebugger._instance ) {
      RuleDebugger._instance = new RuleDebugger;
    }
    return RuleDebugger._instance;
  }

  //-----------------------------------------------------------------
  //  INTERFACE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  start xhr req to get the list
  //
  RequestRuleInstanceList( fnResult )
  {
    new XhrRequest( "ADMIN.RULE_LIST", { }, ( response ) => {
      if( 'data' in response ) {
        this.ruleInstanceList = response['data'];
        if( $.isFunction( fnResult ) ) {
          fnResult( this.ruleInstanceList );
        }
      }
      else {
        fnResult( null );
      }
    } );

    Mailbox.Instance().On( "RULE", this.OnRuleDebugger.bind( this ) );
  }

  //-----------------------------------------------------------------
  //  get last requested list
  //
  RuleInstanceList( )
  {
    return this.ruleInstanceList;
  }

  //-----------------------------------------------------------------
  //  start xhr req to get the list
  //
  RequestRuleSource( ruleId, fnResult )
  {
    new XhrRequest( "ADMIN.RULE_SOURCE", { ruleId: ruleId }, ( response ) => {
      if( 'data' in response ) {
        this.ruleInstanceList = response['data'];
        if( $.isFunction( fnResult ) ) {
          fnResult( this.ruleInstanceList );
        }
      }
      else {
        fnResult( null );
      }
    } );
  }

  //-----------------------------------------------------------------
  //  start xhr req to get the list
  //
  RequestRuleDebug( ruleId, isEnabled )
  {
    new XhrRequest( "ADMIN.RULE_DEBUG", { ruleId: ruleId, isDebugEnabled: isEnabled }, ( response ) => { } );
  }  

  //-----------------------------------------------------------------
  //  mailbox event: rule debugger
  //
  OnRuleDebugger( ruleDebugList )
  {
    if( Array.isArray( ruleDebugList ) ) {
      for( var sequence of ruleDebugList ) {
        if( sequence['term'] == "seq" ) {
          this.Publish( "RULE/SEQUENCE", sequence );
        }
        else if( sequence['term'] == "stop" ) {
          this.Publish( "RULE/STOP_DEBUG", sequence );
        }
        else {
          console.log( "unknown rule event:", sequence );
        }
      }
    }
  }
  
}
﻿//=============================================================================
//
//  Access Verify Controller
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jäkel
//          24.08.2021
//
//  Description:
//    runs in background and verifies all references
//
//=============================================================================

class Verify extends EventSource
{
  //---------------------------------------------------------------------------
  //  STATIC CODE
  //---------------------------------------------------------------------------

  static _instance = null;
  static Instance()
  {
    if( !Verify._instance ) {
      Verify._instance = new Verify;
    }
    return Verify._instance;
  }

  static registeredClasses = [];

  //---------------------------------------------------------------------------
  //  build new object
  //
  static Register( classPointer )
  {
    let className = classPointer.toString().split( '(' || /s+/ )[ 0 ].match( /\b\w+\b/g )[ 1 ];
    if( !( 'Verify' in classPointer ) ) {
      console.warn( `${ className }: missing static function "Verify"` );
      return;
    }
    Verify.registeredClasses.push( classPointer );
  }

  //---------------------------------------------------------------------------
  //  INSTANCE VARIABLES
  //---------------------------------------------------------------------------

  isStarted = false;

  errorDataset = new Dataset( { id: "id" } );
  errorIndex = {};
  references = {};
  referenceReverse = {};
  
  eventId1 = null;
  eventId2 = null;
  eventId3 = null;

  xhrRequest = null;


  //---------------------------------------------------------------------------
  //  INTERFACE
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  Starts the verification
  //
  Start()
  {
    //console.log( "Verifier: Start." );
   
    // subscribe entinfo update/create messages
    this.eventId1 = Data.Instance()._entinfoManager.On( "update", this.EntinfoUpdate.bind( this ) );
    this.eventId2 = Data.Instance()._entinfoManager.On( "create", this.EntinfoUpdate.bind( this ) );
    this.eventId3 = Data.Instance()._entinfoManager.On( "delete", this.EntinfoDelete.bind( this ) );

    // Subscribe admin message mailbox events.
    this.messageEventId = Mailbox.Instance().On( "ADMIN_MESSAGE", this.MailboxEventAdminMessage.bind( this ) );

    // Initial request for admin messages.
    this.xhrRequest = new XhrRequest( "ADMIN.ADMIN_MESSAGES", null, ( response ) => {
      this.DataAdminMessagesResponse( response );
    } );
  }

  //---------------------------------------------------------------------------
  //  Stops the verification
  //
  Stop()
  {
    //console.log( "Verifier: Stop." );

    // Unsubscribe mailbox event.
    if( this.eventId1 ) {
      Data.Instance()._entinfoManager.Off( this.eventId1 );
      Data.Instance()._entinfoManager.Off( this.eventId2 );
      Data.Instance()._entinfoManager.Off( this.eventId3 );
    }

    this.xhrRequest = null;
    this.errorDataset.SetData( [] );

    // Unsubscribe admin message mailbox event.
    Mailbox.Instance().Off( this.messageEventId );
  }


  //---------------------------------------------------------------------------
  //  verify all objects
  //
  All( )
  {
    let treeset = Data.Instance().TreeData();
    this._VerifyRecursive( treeset, "folder.devices" );

    //console.log( "Verify:All" );
    //console.log( this.references );
    //console.log( this.referenceReverse );
  }

  //---------------------------------------------------------------------------
  //  get error list for dashboard
  //
  ErrorDataset()
  {
    return this.errorDataset;
  }

  //---------------------------------------------------------------------------
  //  Does device with given entityId need a restart?
  //
  IsDeviceRestartNeeded( entid )
  {
    return this.errorDataset.Has( ( rec ) => ( rec['entid'] == entid && rec['msgid'] == "DEVRESTART_NEEDED" ) );
  }

  //---------------------------------------------------------------------------
  //  Does at least one device need a restart?
  //
  IsAnyDeviceRestartNeeded( )
  {
    let result = false;

    this.errorDataset.Each( ( id, record ) => {
      if( record['msgid'] == "DEVRESTART_NEEDED" )  {
        result = true;
      }
    });

    return result;
  }  

  //---------------------------------------------------------------------------
  //  get entities that references to another entity
  //
  GetReferencedEntityIds( entid )
  {
    if( entid in this.referenceReverse )  {
      return this.referenceReverse[ entid ];
    }
    return null;
  }


  //---------------------------------------------------------------------------
  //  EVENTS
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  entinfo is created / updated
  //
  EntinfoUpdate( entinfo )
  {
    // todo: partial update on permissions 
    let isPermissionChanged = true;

    // update are reference indexes
    this._UpdateReferenceLists( entinfo );

    let treeset = Data.Instance().TreeData();
    this._VerifyItem( treeset, entinfo );
    this.Publish( "VERIFY/REVALIDATE/" + entinfo['entid'] );

    if( entinfo['entid'] in this.referenceReverse )  {
      let reverseList = this.referenceReverse[ entinfo['entid'] ];
      for( let id of reverseList )  {
        let reverseEntinfo = Data.Instance().Entinfo( id );
        this._VerifyItem( treeset, reverseEntinfo );
        this.Publish( "VERIFY/REVALIDATE/" + id );
      }
    }

    // if permission has been changed go through all children
    entinfo = treeset.Get( entinfo['entid'] );
    if( isPermissionChanged && ( 'children' in entinfo ) ) {
      this._RevalidateChildren( treeset, entinfo );
    }


  }

  //---------------------------------------------------------------------------
  //  helper for entinfo update
  //
  _RevalidateChildren( treeset, entinfo )
  {
    for( let childEntinfo of entinfo['children'] ) {

      this._VerifyItem( treeset, childEntinfo );
      this.Publish( "VERIFY/REVALIDATE/" + childEntinfo['entid'] );

      if( 'children' in childEntinfo )  {
        this._RevalidateChildren( treeset, childEntinfo );
      }

    }
  }

  //---------------------------------------------------------------------------
  //  entinfo is deleted
  //
  EntinfoDelete( entid )
  {
    let isPermissionChanged = true;

    // first revalidate all references of the device that is deleted
    if( entid in this.referenceReverse ) {

      // get data
      let treeset = Data.Instance().TreeData();
      let reverseList = this.referenceReverse[ entid ];

      // verify all (former) references
      for( let id of reverseList ) {
        let reverseEntinfo = Data.Instance().Entinfo( id );
        this._VerifyItem( treeset, reverseEntinfo );
        this.Publish( "VERIFY/REVALIDATE/" + id );
      }

    }

    // update are reference indexes
    delete this.referenceReverse[ entid ];
  }

  //---------------------------------------------------------------------------
  //  Process messages.
  //
  MailboxEventAdminMessage( messageElementList )
  {
    // Iterate thru mailbox element list.
    for( let id in messageElementList )  {
      var messageElement = messageElementList[ id ];
      if( !( 'term' in messageElement ) || messageElement["term"] == "U" )  {  // "U"pdate
        delete messageElement["term"];
        if( ! this.errorDataset.Has( ( rec ) => rec['entid'] == messageElement['param']['entid'] ) ) {
          this.errorDataset.Add( messageElement['param'] );
        }
      }
      else if( messageElement["term"] == "D" )  {  // "D"elete
        // Some times server deletes duplicated messages w/o sending single msgs. 
        // In this case it will send a "D" message with msgid
        // then we need to remove all messages with entid/msgid as filter
        if( 'msgid' in messageElement ) {
          this._HelperErrorDsRemoveRecords( messageElement['entid'], messageElement['msgid'] );
        }
        else {
          if( this.errorDataset.Has( messageElement['id'] ) ) {
            this.errorDataset.Delete( messageElement['id'] );
          }
        }
      }

    } // for

    // Hier werden alle Elemente zu errorDataset hinzugefügt, bei XHR DataAdminMessagesResponse
    // dagegen nur DEVRESTART_NEEDED!?

    // Publish event if there is at least one DEVRESTART_NEEDED.    
    this.Publish( "VERIFY/DEVRESTART_NEEDED", this.IsAnyDeviceRestartNeeded( ) );
  }

  //---------------------------------------------------------------------------
  //  XHR
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  Response on initial request for admin messages.
  //  
  DataAdminMessagesResponse( response )
  {
    let command      = response['meta']['command'];
    let responseCode = parseInt( response['meta']['status'] );    

    this.xhrRequest = null;

    // If error, we are done here.
    if( responseCode != 200 ) {
      return;
    }

    //-- ADMIN.MESSAGES --------------------------------------
    if( command == "ADMIN.ADMIN_MESSAGES" )  {

      let isDeviceRestartNeeded = false;
      for ( let record of response['data']['messages'] ) {
        if( record['msgid'] == "DEVRESTART_NEEDED" )  {
          this.errorDataset.Add( record );
          isDeviceRestartNeeded = true;
        }
      } // for

      // Publish event if there is at least one DEVRESTART_NEEDED.
      this.Publish( "VERIFY/DEVRESTART_NEEDED", isDeviceRestartNeeded ); 
    } // "ADMIN.ADMIN_MESSAGES"
  }

  //---------------------------------------------------------------------------
  //  INTERN
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  verify a folder (recursive)
  //
  _VerifyRecursive( treeset, folderEntid )
  {
    // get data
    let entinfo = treeset.Get( folderEntid );

    // update are reference indexes
    this._UpdateReferenceLists( entinfo );
    // verify THIS
    this._VerifyItem( treeset, entinfo );

    // loop all children
    if( 'children' in entinfo )  {
      for( let child of entinfo['children'] )  {
        this._VerifyRecursive( treeset, child['entid'] );
      }
    }
  }



  //---------------------------------------------------------------------------
  //  verify am item
  //
  _VerifyItem( treeset, entinfo )
  {
    setTimeout( () => {
      // verify-chain for this entinfo
      for( let classPointer of Verify.registeredClasses ) {
        let result = classPointer['Verify']( treeset, entinfo );
        if( result.length > 0 ) {
          for( let rec of result ) {
            this._HelperErrorDsAddRecordUnique( rec );
          }
          let numErrorsOverall = this.errorDataset.Size();
          this.Publish( "VERIFY/ERRORSTATUS/*", { entid: entinfo['entid'], error: true, numErr: numErrorsOverall } );
        }
        else {
          let numErrorsOverall = this.errorDataset.Size();
          this._HelperErrorDsRemoveRecords( entinfo['entid'], "ENTITY_ERROR" );
          this.Publish( "VERIFY/ERRORSTATUS/*", { entid: entinfo['entid'], error: false, numErr: numErrorsOverall } );
        }
      }

    }, 10 );
  }


  //---------------------------------------------------------------------------
  //  update the reference und reverse-refence lists
  //
  _UpdateReferenceLists( entinfo )
  {
    if( 'references' in entinfo ) {

      // build new entid list
      let deleteList = [];
      let entidList = this._HelperBuildEntidListFromReferences( entinfo['references'] );
      let addList = entidList;

      if( entinfo['entid'] in this.references ) {
        let oldEntidList = this._HelperBuildEntidListFromReferences( this.references[entinfo['entid']] );
        deleteList = $.array.NotIn( oldEntidList, entidList );
        addList = $.array.NotIn( entidList, oldEntidList );
      }

      // add all actual refs into global list
      for( let id of addList ) {
        if( !( id in this.referenceReverse ) ) {
          this.referenceReverse[id] = [];
        }
        this.referenceReverse[id].push( entinfo['entid'] );
      }

      // delete refs that are not longer needed
      for( let id of deleteList ) {
        if( !( id in this.referenceReverse ) ) {
          continue;
        }
        $.array.Remove( this.referenceReverse[id], id );
      }


      // keep version for later use
      this.references[entinfo['entid']] = entinfo['references'];
    }
    else {
      delete this.references[entinfo['entid']];
    }

    //console.log( "Verify:_UpdateReferenceLists" );
    //console.log( this.references );
    //console.log( this.referenceReverse );

  }

  //---------------------------------------------------------------------------
  //  record-dataset helpers
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  Remove a record (unique) to error database
  //
  _HelperErrorDsRemoveRecords( entid, msgid )
  {
    let errorList = this.errorDataset.Filter( entid, "entid" );
    for( let id of errorList ) {
      let record = this.errorDataset.Get( id );
      if( record['msgid'] == msgid ) {
        this.errorDataset.Delete( id );
      }
    }

  }

  //---------------------------------------------------------------------------
  //  Add a record (unique) to error database
  //
  _HelperErrorDsAddRecordUnique( record )
  {
    let hasErrorRecord = false;

    // get all record for a specific entid and check if an error rec exists
    let errorList = this.errorDataset.Filter( record['entid'], "entid" );
    for( let id of errorList )  {
      let entry = this.errorDataset.Get( id );
      if( entry['msgid'] != "DEVRESTART_NEEDED" ) {
        hasErrorRecord = true;
      }
    }

    // if no previously added record...
    if( !hasErrorRecord ) {
      this.errorDataset.Add( record );
    }
  }

  //---------------------------------------------------------------------------
  //  convert a reference list into array of entid's
  //
  _HelperBuildEntidListFromReferences( referenceList )
  {
    let result = {};

    for( let item of referenceList ) {

      // normalize reference, extract entid-id
      let reference = item.split( ":" );
      let refEntid = null;
      if( reference[ 0 ] == "sp" ) {
        refEntid = reference[ 1 ].split( "/" )[ 1 ];
      }
      else if( reference[ 0 ] == "call" ) {
        refEntid = reference[ 1 ];
      }
      else if( reference[ 0 ] == "prf" ) {
        refEntid = reference[ 1 ];
      }
      else if( reference[ 0 ] == "rule" ) {
        refEntid = reference[ 1 ].split( "/" )[ 0 ];
      }

      result[ refEntid ] = true;
    }

    return Object.keys( result );
  }

}
﻿//=============================================================================
//
//  Access Verify Item
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jäkel
//          24.08.2021
//
//  Description:
//    verify extern references (deleted, or permissions)
//
//=============================================================================

class VerifyReferences
{
  //---------------------------------------------------------------------------
  //  do verify (single object)
  //
  static Verify( treeset, entinfo )
  {
    let result = [];
    let isError = false;

    if( ! ( 'references' in entinfo ) ) {
      return result;
    }

    // loop through all references and check them, break on first error
    for( let ref of entinfo['references'] )  {
      let reference = ref.split( ":" );
      if( reference[ 0 ] == "sp" )  {
        isError = VerifyReferences.VerifySp( treeset, entinfo, reference );
        if( isError ) {
          //console.log( "VerifySp failed:", entinfo, reference );
          break;
        }
      }
      else if( reference[ 0 ] == "call" ) {
        isError = VerifyReferences.VerifyCall( treeset, entinfo, reference );
        if( isError ) {
          //console.log( "VerifyCall failed:", entinfo, reference );
          break;
        }
      }
      else if( reference[0] == "prf" ) {
        isError = VerifyReferences.VerifyProfile( treeset, entinfo, reference );
        if( isError ) {
          //console.log( "VerifyPrf failed:", entinfo, reference );
          break;
        }
      }
    }

    // if an error has detected, add a general error list
    if( isError ) {
      result.push( {
        "id": $.guidgen(),
        "type": "warn",
        "entid": entinfo['entid'],
        "msgid": "ENTITY_ERROR",
        "param": null,
        "created": new Date().getTime(),
        "message": "{$ENTITY_ERROR}",
        "updated": null
      } );
    }

    return result;
  }

  //---------------------------------------------------------------------------
  //  verify statepoint
  //
  static VerifySp( treeset, entinfo, reference )
  {
    // SP existance
    let spInfo = Data.Instance().StatepointInfo( reference[ 1 ] );
    if( !spInfo || spInfo['type'] == "ERROR" )  {
      // error statepoint does not exist
      console.log( "Verify-Error: SP deleted", entinfo['entid'], reference[ 1 ] );
      return true;
    }

    // SP permissions

    // is right in reference? No: We assume write permission
    let right = "w";
    if( reference.length > 2 )  {
      right = reference[ 2 ];
    }

    // no right checking required (doormatic)
    if( right == "-" ) {
      return false;
    }

    // calc permission
    if( ! Data.Instance().PermissionList().IsAllowed( entinfo['entid'], reference[ 1 ], right ) )  {
      // error statepoint permission
       console.log( "Verify-Error: SP permission", entinfo['entid'], reference[ 1 ] );
      return true;
    }

    // no error
    return false;
  }

  //---------------------------------------------------------------------------
  //  verify statepoint
  //
  static VerifyCall( treeset, entinfo, reference )
  {
    let calltargetEntinfo = Data.Instance().Entinfo( reference[ 1 ] );
    if( ( ! calltargetEntinfo )
     || ( ( reference.length > 2 ) && ( reference[ 2 ] != calltargetEntinfo['callnumber'] ) )
      ) {
      console.log( "Verify-Error: call permission", entinfo['entid'], reference[1] );
      return true;
    }

    // no error
    return false;
  }

  //---------------------------------------------------------------------------
  //  verify call profile
  //
  static VerifyProfile( treeset, entinfo, reference )
  {
    let profileInfo = Data.Instance().CallprofileDataset().ProfileInfo( reference[ 2 ] );
    if( !profileInfo ) {
      console.log( "Verify-Error: call permission", entinfo['entid'], reference[1] );
      return true;
    }

    // no error
    return false;
  }

}

Verify.Register( VerifyReferences );
﻿//=============================================================================
//
//  Search
//  (c) 2021 CYTEL Technology AG
//
//  Editor: Frank Jäkel
//          24.08.2021
//
//  Description:
//    runs in background and searches all tree items (entities)
//
//=============================================================================

class Search extends EventSource
{
  //---------------------------------------------------------------------------
  //  STATIC CODE
  //---------------------------------------------------------------------------

  static _instance = null;
  static Instance()
  {
    if( !Search._instance ) {
      Search._instance = new Search;
    }
    return Search._instance;
  }

  //---------------------------------------------------------------------------
  //  INSTANCE
  //---------------------------------------------------------------------------

  resultDataset = null;
  currentSearch = null;

  constructor()
  {
    super();
    this.resultDataset = new Dataset( { id: "entid" } );
  }

  //---------------------------------------------------------------------------
  //  INTERFACE
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  start a new search (stops previous)
  //
  Start( searchString, parent = "folder.devices" )
  {
    // stop and clear for a new search
    this.Stop();
    this.Clear();

    // inform UI
    this.Publish( "SEARCH/START", { search: searchString } );
    this.currentSearch = searchString;

    // allow wildcards (although this is always a wildcard search)
    searchString = searchString.replace( "*", "" );

    // excape special characters
    searchString = searchString.replace( "\\", "\\\\" );
    searchString = searchString.replace( ".", "\\." );
    searchString = searchString.replace( "^", "\\^" );
    searchString = searchString.replace( "$", "\\$" );
    searchString = searchString.replace( "+", "\\+" );
    searchString = searchString.replace( "-", "\\-" );
    searchString = searchString.replace( "?", "\\?" );
    searchString = searchString.replace( "(", "\\(" );
    searchString = searchString.replace( ")", "\\)" );
    searchString = searchString.replace( "[", "\\[" );
    searchString = searchString.replace( "]", "\\]" );
    searchString = searchString.replace( "{", "\\{" );
    searchString = searchString.replace( "}", "\\}" );
    searchString = searchString.replace( "|", "\\|" );

    this.regex = new RegExp( searchString, "im" );
    this._searchList = [];
    this._timeout = 10; // <<<<<<<<<<<<<<<

    // Prepare list for new search. 
    let treeset = Data.Instance().TreeData();
    this._SearchRecursive( treeset, parent );

    // Start search interval timer.
    this._timerId = setTimeout( this.IntervalTimer, this._timeout, 0 );
  }

  //---------------------------------------------------------------------------
  //  stops a running search
  //
  Stop( )
  {
    if ( this._timerId != null)  {
      clearTimeout( this._timerId );
      this._timerId = null;
      this._Finish( )
    }
  }

  //---------------------------------------------------------------------------
  //  clears search result
  //
  Clear()
  {
    this.resultDataset.SetData( [] );
  }

  //---------------------------------------------------------------------------
  //  gets search result dataset
  //
  Dataset( )
  {
    return this.resultDataset;
  }

  CurrentSearch()
  {
    return this.currentSearch;
  }


  //---------------------------------------------------------------------------
  //  TIMER
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  // Callback function for setTimeout().
  // - search one item
  // - start new timeout for next item
  //
  IntervalTimer( index )
  {
    var self = Search.Instance( );

    if( self._timerId != null )  {

      // Search one item.
      let treeset = Data.Instance().TreeData();
      let entid = self._searchList[ index ];
      let entinfo = treeset.Get( entid );
      if( typeof entinfo == "object" )  {
        self._SearchItem( treeset, entinfo, index );
      }

      ++index;

      // Have we reached end of the list?
      if( index >= self._searchList.length )  {
        self._searchList = [];
        self._timerId = null;
        self._Finish( );
        return;
      }

      // Start new timeout.
      clearTimeout( self._timerId );
      this._timerId = setTimeout( self.IntervalTimer, self._timeout, index );
    }
  }


  //---------------------------------------------------------------------------
  //  INTERN
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  To be called after searched is finished - no matter if it was stopped or
  //  end of list was reached.
  //
  _Finish( )
  {
    this.Publish( "SEARCH/FINISHED" );
  }

  //---------------------------------------------------------------------------
  //  search a folder (recursive)
  //
  _SearchRecursive( treeset, folderEntid )
  {
    // get data
    let entinfo = treeset.Get( folderEntid );
    this._searchList.push( entinfo['entid'] );

    // loop all children
    if( 'children' in entinfo ) {
      for( let child of entinfo['children'] ) {
        this._SearchRecursive( treeset, child['entid'] );
      }
    }
  }

  //---------------------------------------------------------------------------
  //  perform search within an item
  //
  _SearchItem( treeset, entinfo, searchCount )
  {
    //searchCount++;
    //console.log("--SearchItem", searchCount );

    this._SearchItem_Name( treeset, entinfo );
    this._SearchItem_Callnumber( treeset, entinfo );
    this._SearchItem_Prefix( treeset, entinfo );
    this._SearchItem_ExternCallnr( treeset, entinfo );
    this._SearchItem_Account( treeset, entinfo );
    this._SearchItem_Macaddr( treeset, entinfo );
    this._SearchItem_Statepoint( treeset, entinfo );
    this._SearchItem_References( treeset, entinfo );
  }

  //---------------------------------------------------------------------------
  //  search in name
  //
  _SearchItem_Name( treeset, entinfo )
  {
    if( ( 'name' in entinfo )
     && ( entinfo['name'].match( this.regex ) )
      ) {
      this._HelperUpdateOrCreateRecord( entinfo['entid'], "{$search_result_name}" );
    }

    if( ( 'comment' in entinfo )
     && ( entinfo['comment'].match( this.regex ) )
      ) {
      this._HelperUpdateOrCreateRecord( entinfo['entid'], "{$search_result_comment}" );
    }
  }

  //---------------------------------------------------------------------------
  //  search in callnumber
  //
  _SearchItem_Callnumber( treeset, entinfo )
  {
    if( ( 'callnumber' in entinfo )
     && ( entinfo['callnumber'].match( this.regex ) )
      ) {
      this._HelperUpdateOrCreateRecord( entinfo['entid'], "{$search_result_callnumber}" );
    }
  }

  //---------------------------------------------------------------------------
  //  search in mac
  //
  _SearchItem_Macaddr( treeset, entinfo )
  {
    if( ( 'macaddr' in entinfo )
     && ( entinfo['macaddr'].match( this.regex ) )
      ) {
      this._HelperUpdateOrCreateRecord( entinfo['entid'], "{$search_result_macaddr}" );
    }
  }

  //---------------------------------------------------------------------------
  //  search in mac
  //
  _SearchItem_Prefix( treeset, entinfo )
  {
    if( ( 'prefix' in entinfo )
     && ( entinfo['prefix'].match( this.regex ) )
      ) {
      this._HelperUpdateOrCreateRecord( entinfo['entid'], "{$search_result_prefix}" );
    }
  }

  //---------------------------------------------------------------------------
  //  search in externCallnumber
  //
  _SearchItem_ExternCallnr( treeset, entinfo )
  {
    if( ( 'externCallnumber' in entinfo )
     && ( entinfo['externCallnumber'].match( this.regex ) )
      ) {
      this._HelperUpdateOrCreateRecord( entinfo['entid'], "{$search_result_externcallnr}" );
    }
  }

  //---------------------------------------------------------------------------
  //  search in account
  //
  //  entinfo['account'] -> entity["authorize"]["websock"]["account"]
  //
  //  susapp and jungpanel also have authorize/websock/account, although there is
  //  no edit for these in their forms!
  //
  _SearchItem_Account( treeset, entinfo )
  {
    if( ( 'account' in entinfo )
     && ( entinfo['account'].match( this.regex ) )
      ) {
      this._HelperUpdateOrCreateRecord( entinfo['entid'], "{$search_result_account}" );
    }
  }


  //---------------------------------------------------------------------------
  //  search in name
  //
  _SearchItem_Statepoint( treeset, entinfo )
  {
    if( 'sp' in entinfo ) {
      for( let spName in entinfo['sp'] ) {
        
        let sp = entinfo['sp'][ spName ];
        if( ( 'hidden' in sp ) && ( sp['hidden'] == true ) ) {
          continue;
        }

        let label = $.lang( sp['label'] );
        if( label.match( this.regex ) ) {
          this._HelperUpdateOrCreateRecord( entinfo['entid'], "{$search_result_statepoint}" );
        }
      }
    }    
  }

  //---------------------------------------------------------------------------
  //  search in references
  //
  _SearchItem_References( treeset, entinfo )
  {
    if( 'references' in entinfo ) {

      for( let ref of entinfo['references'] ) {
      
        let reference = ref.split( ":" );

        // this entity has a key/i/o configured to a foreign sp
        if( reference[ 0 ] == "sp" )  {
          let spInfo = Data.Instance().StatepointInfo( reference[ 1 ] );
          if( spInfo && spInfo['type'] != "ERROR" ) {
            let label = $.lang( spInfo['label'] );
            if( label.match( this.regex ) ) {
              this._HelperUpdateOrCreateRecord( entinfo['entid'], "{$search_result_keys}" );
            }
          }
        }
        // this entity calls another entity
        else if( reference[ 0 ] == "call" ) {
          let refEntinfo = Data.Instance().Entinfo( reference[ 1 ] );
          if( refEntinfo && ( 'name' in refEntinfo ) ) {
            if( refEntinfo['name'].match( this.regex ) ) {
              this._HelperUpdateOrCreateRecord( entinfo['entid'], "{$search_result_keys}" );
            }

          }
        }
      }
    }

  }

  
  //---------------------------------------------------------------------------
  //  HELPER
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  record easy interface
  //
  _HelperUpdateOrCreateRecord( entid, msgid )
  {
    if( this.resultDataset.Has( entid ) ) {
      let record = this.resultDataset.Get( entid );
      if( record['message'].indexOf( msgid ) < 0 ) {
        record['message'].push( msgid );
      }
    }
    else {
      let record = {
        entid: entid,
        message: [ msgid ]
      };
      this.resultDataset.Add( record );
    }
  }


}//=============================================================================
//
//  Help Loader
//  (c) 2021 CYTEL Technology AG
//
//  Description: Returns help topic for <help-topic> web component
//
//  Editor: Frank Jaekel
//          26.10.2021
//
//=============================================================================

class HelpLoader extends Help 
{
  //---------------------------------------------------------------------------
  //  BUILD 
  //---------------------------------------------------------------------------
  constructor( langId )
  {
    super();
    this.langId = langId;
    this.dynamicTopics = {};
  }

  //---------------------------------------------------------------------------
  //  INTERFACE
  //---------------------------------------------------------------------------

  AddTopic( topicId, topic )
  {
    this.dynamicTopics[ topicId ] = topic;
  }

  //---------------------------------------------------------------------------
  //  OVERRIDABLE
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  //  load topic from server
  //
  OnLoadTopic( topicId, resultCallback )
  {
    if( topicId in this.dynamicTopics )  {
      var result = this.dynamicTopics[ topicId ][ this.langId ];
      resultCallback( marked( result ) );
    }
    else {
      $.ajax( {
        url: "help.php?topic=" + topicId + "&lang=" + this.langId,
        success: ( result ) => {
          resultCallback( marked( result ) );
        }
      } );
    }
  
  }//loadTopic
}

//=============================================================================
//
//  File: time.js
//  (c) 2021 by CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          02.02.2021
//
//  Description:
//          Helper for formating date & time from unix time
//
//=============================================================================

//=============================================================================
//  INIT
//=============================================================================

class Time
{
  static FormatFromUnix( timestamp )
  {
    let zone = moment.tz.guess();
    let m = moment.tz( timestamp * 1000, "utc" );
    return m.tz( zone ).format( "L LT" );
  }
  static FormatDateFromUnix( timestamp )
  {
    let zone = moment.tz.guess();
    let m = moment.tz( timestamp * 1000, "utc" );
    return m.tz( zone ).format( "L" );
  }
  static FormatTimeFromUnix( timestamp )
  {
    let zone = moment.tz.guess();
    let m = moment.tz( timestamp * 1000, "utc" );
    return m.tz( zone ).format( "LT" );
  }

  static FormatFromEndOfSupport( endofsupport )
  {
    let zone = moment.tz.guess();
    let m = moment.tz( endofsupport, "utc" );
    return m.tz( zone ).format( "L" );
  }

}﻿//=============================================================================
//
//  File: ipaddress.js
//  (c) 2021 by CYTEL Technology AG
//
//  Editor: Frank Jaekel
//          16.02.2021
//
//  Description:
//          Ecapsultion of ip address
//
//=============================================================================

//-----------------------------------------------------------------------------
//  CLASS
//-----------------------------------------------------------------------------

class IpAddress
{
  //-----------------------------------------------------------------
  //  constructor: (opt) initialize with value
  //
  constructor( initValue )
  {
    this.value = initValue;
  }

  //-----------------------------------------------------------------
  //  setter for value
  // 
  set value( initValue )
  {
    this.ipValue = null;
    let ip = initValue.match( /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/ );
    if( ip ) {
      this.ipValue = ( +ip[ 1 ] << 24 ) + ( +ip[ 2 ] << 16 ) + ( +ip[ 3 ] << 8 ) + ( +ip[ 4 ] );
    }
  }

  //-----------------------------------------------------------------
  //  getter for value
  // 
  get value( )
  {
    return this.AsString( );
  }

  //-----------------------------------------------------------------
  //  bool: Is a valid ip?
  // 
  IsValid()
  {
    return ( this.ipValue !== null );
  }

  //-----------------------------------------------------------------
  //  return as long (e.g for masking)
  // 
  AsNumber()
  {
    return this.ipValue;
  }

  //-----------------------------------------------------------------
  //  return as string (default)
  // 
  AsString()
  {
    return ( ( this.ipValue >>> 24 ) + '.' + ( this.ipValue >> 16 & 255 ) + '.' + ( this.ipValue >> 8 & 255 ) + '.' + ( this.ipValue & 255 ) );
  }


  //-----------------------------------------------------------------
  //  return as string (default)
  // 
  static MaskToNetwork( mask )
  {
    switch( mask.trim() )  {
      case '128.0.0.0': return 1;
      case '192.0.0.0': return 2;
      case '224.0.0.0': return 3;
      case '240.0.0.0': return 4;
      case '248.0.0.0': return 5;
      case '252.0.0.0': return 6;
      case '254.0.0.0': return 7;
      case '255.0.0.0': return 8;
      case '255.128.0.0': return 9;
      case '255.192.0.0': return 10;
      case '255.224.0.0': return 11;
      case '255.240.0.0': return 12;
      case '255.248.0.0': return 13;
      case '255.252.0.0': return 14;
      case '255.254.0.0': return 15;
      case '255.255.0.0': return 16;
      case '255.255.128.0': return 17;
      case '255.255.192.0': return 18;
      case '255.255.224.0': return 19;
      case '255.255.240.0': return 20;
      case '255.255.248.0': return 21;
      case '255.255.252.0': return 22;
      case '255.255.254.0': return 23;
      case '255.255.255.0': return 24;
      case '255.255.255.128': return 25;
      case '255.255.255.192': return 26;
      case '255.255.255.224': return 27;
      case '255.255.255.240': return 28;
      case '255.255.255.248': return 29;
      case '255.255.255.252': return 30;
      case '255.255.255.254': return 31;
      case '255.255.255.255': return 32;
      default:  return 0;
    }
  }
}
//=============================================================================
//  
//  STRING TOOLS
//  for special uses
//
//=============================================================================

class StringTools
{
  //---------------------------------------------------------------------------
  //  INTERFACE
  //---------------------------------------------------------------------------

  static CreateLimitedAsciiString( input )
  {
    var newString = "";
    for( var i = 0; i < input.length; i++ ) {
      var char = input.charCodeAt( i );
      if( StringTools._IsInDeleteList( char ) ) {
        continue;
      }
      var newChar = StringTools._GetReplacementCharacter( char );
      if( newChar !== null ) {
        newString += newChar;
      }
      else {
        newString += input[ i ];
      }
    }
    return newString;
  }

  //---------------------------------------------------------------------------
  //  DATA
  //---------------------------------------------------------------------------

  static deletementList = [
    { from: 0, to: 31 },      // 
    { from: 33, to: 47 },     // ! ... /
    { from: 58, to: 63 },     // : ... ?  
    { from: 91, to: 94 },     // [ ... ^
    { from: 123, to: 191 },   // { ... ¿
    { from: 255, to: 65536 }  // else
  ];

  static replacementList = [
    { code:  32, r: '_' },   // blank
    { code:  64, r: 'a' },   // @
    { code:  95, r: '_' },   // _
    { code:  96, r: '' },    // `
    
    { code: 192, r: 'A' },   // À
    { code: 193, r: 'A' },   // Á
    { code: 194, r: 'A' },   // Â
    { code: 195, r: 'A' },   // Ã
    { code: 196, r: 'Ae' },   // Ä
    { code: 197, r: 'A' },   // Å
    { code: 198, r: 'AE' },  // Æ
    
    { code: 199, r: 'C' },   // Ç
    
    { code: 200, r: 'E' },   // È
    { code: 201, r: 'E' },   // É
    { code: 202, r: 'E' },   // Ê
    { code: 203, r: 'E' },   // Ë
    
    { code: 204, r: 'I' },   // Ì
    { code: 205, r: 'I' },   // Í
    { code: 206, r: 'I' },   // Î
    { code: 207, r: 'I' },   // Ï

    { code: 208, r: 'D' },   // Ð

    { code: 209, r: 'N' },   // Ñ
    
    { code: 210, r: 'O' },   // Ò
    { code: 211, r: 'O' },   // Ó
    { code: 212, r: 'O' },   // Ô
    { code: 213, r: 'O' },   // Õ
    { code: 214, r: 'Oe' },   // Ö
    { code: 215, r: 'x' },   // ×
    { code: 216, r: 'O' },   // Ø

    { code: 217, r: 'U' },   // Ù
    { code: 218, r: 'U' },   // Ú
    { code: 219, r: 'U' },   // Û
    { code: 220, r: 'Ue' },   // Ü
    
    { code: 221, r: 'Y' },   // Ý
    { code: 222, r: 'p' },   // Þ

    { code: 223, r: 'ss' },   // ß

    { code: 224, r: 'a' },   // à
    { code: 225, r: 'a' },   // á
    { code: 226, r: 'a' },   // â
    { code: 227, r: 'a' },   // ã
    { code: 228, r: 'ae' },   // ä
    { code: 229, r: 'a' },   // å
    { code: 230, r: 'ae' },   // æ

    { code: 231, r: 'c' },   // ç
    
    { code: 232, r: 'e' },   // è
    { code: 233, r: 'e' },   // é
    { code: 234, r: 'e' },   // ê
    { code: 235, r: 'e' },   // ë

    { code: 236, r: 'i' },   // ì
    { code: 237, r: 'i' },   // í
    { code: 238, r: 'i' },   // î
    { code: 239, r: 'i' },   // ï

    { code: 240, r: 'o' },   // ð
    { code: 241, r: 'n' },   // ñ

    { code: 242, r: 'o' },   // ò
    { code: 243, r: 'o' },   // ó
    { code: 244, r: 'o' },   // ô
    { code: 245, r: 'o' },   // õ
    { code: 246, r: 'oe' },   // ö
    { code: 248, r: 'o' },   // ø

    { code: 249, r: 'u' },   // ù
    { code: 250, r: 'u' },   // ú
    { code: 251, r: 'u' },   // û
    { code: 252, r: 'ue' },   // ü
    
    { code: 253, r: 'y' },   // ý
    { code: 254, r: 'p' },   // þ
    { code: 255, r: 'y' },   // ÿ

  ];

  //---------------------------------------------------------------------------
  //  HELPERS
  //---------------------------------------------------------------------------

  static _GetReplacementCharacter( char )
  {
    for( var replacement of StringTools.replacementList )  {
      if( char == replacement.code ) {
        return replacement.r;
      }
    }
    return null;
  }

  static _IsInDeleteList( char )
  {
    for( var deletement of StringTools.deletementList )  {
      if( char >= deletement.from && char <= deletement.to )  {
        return true;
      }
    }
    return false;
  }

}//=============================================================================
//
//  Access Main Application 
//  (c) 2020 CYTEL Technology AG
//
//  Editor: Tobias Stumpf
//          07.07.2020
//=============================================================================

class AccessApplication extends Application
{
  //-----------------------------------------------------------------
  //  CLASS MEMBERS
  //-----------------------------------------------------------------

  static isVirtualDoorAllowed = true;   // sipphone + ipcam = virtual door
  static isIpCamOnDoorAllowed = true;   // attach ip cam on door 
  static isWizzardDebug = false;

  //-----------------------------------------------------------------
  //  INIT
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  viewable load
  //
  Load( data )
  {
    console.log( "running on", window.location.pathname, "-", window.location.href );

    this.isRuleEdit = true;

    // must be first command
    this._CheckLoadLanguage( );

    // load all admin configurations
    Data.Instance().LoadAdminConfig();

    // initialize help
    this.helpLoader = new HelpLoader( this.langId );

    // browser title
    Application.Instance().SetApplicationTitle( "{$application_window_title}" );

    // load content 
    this.currentContent = null;
    this.ReloadPage( );

    // Check server status.
    // Response on SESSION.STATUS triggers ConnectStatus and its events ...
    setTimeout( () =>  {
      XhrRequestInterval.Instance().Set( 15000, "SESSION.STATUS", null );
    }, 1000 );

    // ... and here we catch these events and look if content is to be reloaded.
    ConnectStatus.Instance().On( "STATUS/STATUS", this.etag, ( response ) =>  {
      this.ReloadPage( );
    } );

  }//load


  //-----------------------------------------------------------------
  //  check page that needs to be shown 
  //
  ReloadPage( )
  {
    // stop verify (will be started in dashboard)
    Verify.Instance().Stop();

    // ----- install wizzard -----
    if( AccessApplication.isWizzardDebug || ConnectStatus.IsFirstInstall() ) {
      if( this.currentContent != "/pages/install" ) {
        this.currentContent = "/pages/install";
        $( '#ID_BODY' ).viewable( { view: "/pages/install" } );

        // Unload previous data.
        Data.Instance().Stop();

        // Stop websocket.        
        Mailbox.Instance().Stop();

        // standard polling interval
        XhrRequestInterval.Instance().Set( 15000, "SESSION.STATUS", null );
      }
    }
    // ----- main admin interface  -----
    else if( ConnectStatus.IsAuthorized() ) {
      if( this.currentContent != "/pages/main" ) {

        // show default (tabs) page
        this.currentContent = "/pages/main";
        $( '#ID_BODY' ).viewable( { view: "/pages/main" } );

        // Start websocket.        
        Mailbox.Instance().Start( );

        // Load data.
        Data.Instance().Start( );

        // standard polling interval
        XhrRequestInterval.Instance().Set( 15000, "SESSION.STATUS", null );
      }
    }
    // ----- login page -----
    else if( ConnectStatus.IsConnected() ) {
      if( this.currentContent != "/pages/login" ) {
        this.currentContent = "/pages/login";
        $( '#ID_BODY' ).viewable( { view: "/pages/login" } );

        // Unload previous data.
        Data.Instance().Stop( );

        // Stop websocket.
        Mailbox.Instance().Stop( );        

        // standard polling interval
        XhrRequestInterval.Instance().Set( 15000, "SESSION.STATUS", null );
      }
    }
    // ----- restarting page -----
    else {
      if( this.currentContent != "/pages/restart" ) {
        this.currentContent = "/pages/restart";
        $( '#ID_BODY' ).viewable( { view: "/pages/restart" } );

        // Unload previous data.
        Data.Instance().Stop( );

        // Stop websocket.        
        Mailbox.Instance().Stop( );       
        
        // better polling interval while wait for server restarted
        XhrRequestInterval.Instance().Set( 5000, "SESSION.STATUS", null );
      }
    }
  }

  //-----------------------------------------------------------------
  //  INTERFACE
  //-----------------------------------------------------------------

  //-----------------------------------------------------------------
  //  global commands
  //
  Command( id )
  {
    switch( id )  {

      // --- download support info ---
      case "support-info":
        SupportInfo.Instance().Get();
        break;

      case "search":
        console.warn( "ToDo: search-command" );
        break;

      // --- reboot access ---
      case 'reboot': {
        RebootAccessDialog.Show( {
          layout: "reboot",
          result: this.CommandRebootSystem.bind( this )
        } );
      }
      break;

      // --- reboot access service ---
      case 'restart-service': {
        RebootAccessDialog.Show( {
          result: this.CommandRestartService.bind( this )
        } );
      }
      break;

      // --- logoff ---
      case 'exit':  {
        Data.Instance().XhrRequest( "SESSION.LOGOFF" );
      }
      break;

    }
  }

  //-----------------------------------------------------------------
  //  reboot system after connected page has been shown
  //
  CommandRebootSystem( )
  {
    // Add Logfile record.
    let adminEntityId = SimpleCookies.Instance().Get( 'user' );
    let logfileRecord = {
      'type':     'info',
      'admin':    adminEntityId,
      'entid':    Data.Instance().RootEntid(),
      'message':  "{$SYSTEM_REBOOT}",
      'param': null
    }
    Data.Instance().Logfile().AddRecordRequest( logfileRecord ); 

    // give some time to the system to write the record
    // then request restart
    setTimeout( () => {
      ConnectStatus.ClearAuthorized();
      ConnectStatus.ClearConnected();
      this.currentContent = null;
      this.ReloadPage();
      Data.Instance().XhrRequest( "BACKUP.REBOOT" );
    }, 500 );

  }

  //-----------------------------------------------------------------
  //  restart services after connected page has been shown
  //
  CommandRestartService( )
  {
    // Add Logfile record.
    let adminEntityId = SimpleCookies.Instance().Get( 'user' );
    let logfileRecord = {
      'type':     'info',
      'admin':    adminEntityId,
      'entid':    Data.Instance().RootEntid(),
      'message':  "{$SYSTEM_RESTARTED}",
      'param': null
    }
    Data.Instance().Logfile().AddRecordRequest( logfileRecord ); 

    // give some time to the system to write the record
    // then request restart
    setTimeout( () => {
      ConnectStatus.ClearAuthorized( );
      ConnectStatus.ClearConnected( );
      this.currentContent = null;
      this.ReloadPage( );
      Data.Instance().XhrRequest( "BACKUP.RESTART", { "uid": $.guidgen() } );  
    }, 500 );
  }

  //-------------------------------------------------------
  //  set modified 
  //
  Modified( id, isModified )
  {
    this.Publish( "APP/MODIFIED", { id: id, isModified: isModified } );
  }

  //-------------------------------------------------------
  //  add help topic that is inside of a rule entity
  //
  HelpAddTopic( topicId, topic )
  {
    this.helpLoader.AddTopic( topicId, topic );
  }

  //-------------------------------------------------------
  //  wich language is selected in login?
  //
  Language()
  {
    return this.langId;
  }



  //-------------------------------------------------------
  //  INTERNALS
  //-------------------------------------------------------

  //-------------------------------------------------------
  //  check language on startup
  //
  _CheckLoadLanguage( )
  {
    // default language
    this.langId = 'en';

    // check language cookie
    var reqLang = Cookies.get( 'lang' );
    if( typeof reqLang == 'string' ) {
      if( reqLang == 'de' || reqLang == 'en' ) {
        this.langId = reqLang;
      }
    }
    // check browser language
    else {
      var browserLang = $.lang.Get( );
      if( browserLang == 'de' || browserLang == 'en' ) {
        this.langId = browserLang;
      }
    }

    // set language 
    $.lang.Set( this.langId );
    moment.locale( this.langId );
  }




};


//=============================================================================
//  BUILDER
//=============================================================================

AccessApplication.Register( '/application/access_main' );
$.extend( true, $.lang._data, { 'de':{'document_title_login':"Anmelden",'login_title_login':"Siedle Access",'login_subtitle_login':"Systemverwaltung",'login_label_password':"Kennwort",'login_label_user':"Benutzername",'login_label_language':"Sprache",'login_lang_german':"Deutsch",'login_lang_english':"Englisch",'login_label_enter':"Anmelden",'login_link_download':"Downloads / Lizenz",'restart_title':"Neustart",'restart_headline':"Siedle Access",'restart_message':"Access Professional wird neu gestartet. Sie werden automatisch auf die Anmeldeseite umgeleitet, sobald das System wieder verfügbar ist. Bitte warten...",'install_header_title':"Siedle Access - Ersteinrichtung",'install_button_next':"Weiter",'install_button_previous':"Zurück",'wizpage_welcome_sidebar':"Willkommen",'wizpage_welcome_title':"Willkommen bei der Ersteinrichtung!",'wizpage_welcome_language':"Show this in english",'wizpage_welcome_line1':"Vielen Dank, dass Sie Siedle Access Professional verwenden. Sie bereiten Access Professional für die Inbetriebnahme vor.",'wizpage_welcome_line2':"Wir führen Sie Schritt für Schritt durch die Ersteinrichtung, um grundlegende Systemeinstellungen vorzunehmen. Nachträgliche Änderungen sind in der Systemverwaltung jederzeit möglich.",'wizpage_welcome_line3':"Mit Abschluss der Ersteinrichtung werden alle vorgenommenen Einstellungen gespeichert und das System neu gestartet. Nach dem Neustart ist eine sofortige Inbetriebnahme der Endgeräte und Schnittstellen über die Systemverwaltung möglich. Die Systemverwaltung erreichen Sie per Webbrowser unter der IP-Adresse und den Anmeldedaten, die Sie im Verlauf der Ersteinrichtung vergeben werden. Weiterführende Informationen finden Sie in der Inbetriebnahmeanleitung zu Access Professional.",'wizpage_welcome_line4':"Kunden, die Access Professional für Testzwecke installieren, haben mit Abschluss der Ersteinrichtung 30 Tage Zeit, um sich aus dem Leistungsspektrum von Access Professional ihr Wunschsystem zu konfigurieren und vollumfänglich zu testen. Bitte wenden Sie sich bei Fragen zu Access Professional an Ihren Access Certified Partner (ACP).",'wizpage_welcome_line5':"Um mit der Ersteinrichtung zu beginnen, klicken Sie bitte auf \„Los gehts!\“. Um die Ersteinrichtung abzubrechen, schließen Sie einfach das Browserfenster.",'wizpage_welcome_button':"Los gehts!",'wizpage_license_sidebar':"Lizenz",'wizpage_license_title':"Lizenzvereinbarungen",'wizpage_license_help':"Endbenutzer-Lizenzvertrag (EULA)",'wizpage_license_check':"Ich (wir) akzeptiere(n) diese Lizenzvereinbarungen. Ich bin (wir sind) berechtigt, diese Lizenzvereinbarungen auch im Namen der nutzenden Organisation zu akzeptieren.",'wizpage_network_sidebar':"Netzwerk",'wizpage_network_title':"Netzwerk",'wizpage_network_help':"Hier konfigurieren Sie die Netzwerkanbindung des Access-Systems. Nach Abschluss der Ersteinrichtung finden Sie erweiterte Konfigurationsmöglichkeiten in der Systemverwaltung im Bereich „Systemwartung“ - „Netzwerk“.",'wizpage_nic_name':"Netzwerkadapter \„{{name}}\“",'wizpage_nic_steps':"(Schritt {{step}}/{{numsteps}})",'wizpage_dns_title':"DNS-Server",'netcard_label_dns_server':"DNS-Server",'netcard_hint_dns_server_dhcp':"(Automatisch durch DHCP)",'netcard_hint_dns_server_supersede':"(Manuell überschrieben)",'netcard_hint_dns_server_opt':"(Optional)",'wizpage_timezone_sidebar':"Zeitzone",'wizpage_timezone_title':"Zeitzone",'wizpage_timezone_help':"Hier konfigurieren Sie die Zeitzone des Access-Systems.",'wizpage_timezone_errorempty':"Sie müssen eine Zeitzone auswählen!",'wizpage_password_sidebar':"Zugangsdaten",'wizpage_password_title':"Zugangsdaten",'wizpage_password_help':"Hier legen Sie das Kennwort für das Benutzerkonto \„admin\“ fest. Mit diesen Zugangsdaten erfolgt die Anmeldung an der Access-Systemverwaltung.",'wizpage_password_admin':"Hauptadministrator",'wizpage_password1':"Kennwort",'wizpage_password2':"Wiederholung",'wizpage_result_sidebar':"Abschluss",'wizpage_result_title':"Abschluss",'wizpage_result_help1':"Sie haben alle erforderlichen Einstellungen für die Ersteinrichtung vorgenommen. Mit \„Speichern und Neustart\“ werden alle vorgenommen Einstellungen gespeichert und das System neu gestartet.",'wizpage_result_help2':"Nach dem Neustart ist Access Professional bereit für die Inbetriebnahme.",'wizpage_result_help3':"Der Zugriff auf die Access-Systemverwaltung ist dann ausschließlich über die von Ihnen konfigurierte IP-Adresse möglich.",'wizpage_result_button_finish':"Speichern und Neustart",'formbase_modify_conflict_notice':"Diese Seite wurde inzwischen von einem anderen Benutzer geändert, oder das Gerät wurde neu gestartet. Ihre Änderungen werden nicht übernommen. Drücken Sie [\„Verwerfen\“](ID_UNDO), um die Seite zu aktualisieren!",'formbase_reloaded_notice':"Das Dokument wurde neu geladen.",'formbase_savefirst_notice':"Bitte speichern Sie zuerst das Dokument!",'formbase_incomplete_notice':"Die Konfiguration ist noch nicht vollständig. Bitte füllen Sie die [fehlenden Felder](ID_VALIDATE) aus und speichern das Dokument!",'formbase_unconfdev_notice':"Die Bearbeitung ist gesperrt. Bitte verschieben Sie dieses Gerät zuerst in den Projektordner!",'formbase_label_name':"Name",'formbase_label_description':"Beschreibung",'formbase_label_device_type':"Gerätetyp",'formbase_label_ipaddr':"IP-Adresse",'formbase_label_port':"Port",'formbase_value_dtmf_from_door_only':"DTMF nur für Rufe von der Tür zum Gateway zulassen",'formbase_group_devicekeys':"Tastenkonfiguration",'formbase_group_deviceio':"Eingänge und Ausgänge",'formbase_group_functions':"Schalten und Steuern",'dashboard_group_messages':"Meldungen",'dashboard_group_functions':"Funktionen",'dashboard_group_information':"Informationen",'dashboard_info_hardwareid':"Hardware-Kennung",'dashboard_info_version':"Systemversion",'dashboard_info_numberdevices':"Anzahl Geräte",'dashboard_info_demotime':"Ende Demozeitraum",'dashboard_info_supportcontract':"Ende Wartungsvertrag",'dashboard_info_warning_expired':"Abgelaufen am",'dashboard_info_susstatus':"Siedle App Dienst",'dashboard_susinfo_notused':"Nicht verwendet",'dashboard_susinfo_connecting':"Verbindung wird aufgebaut",'dashboard_susinfo_online':"Verbunden",'dashboard_susinfo_error':"Verbindungsfehler",'dashboard_grid_column_entid':"Objekt",'dashboard_grid_column_msgid':"Meldung",'dashboard_grid_placeholder':"Keine Meldungen vorhanden",'dashboard_menu_general':"Allgemein",'dashboard_menu_siedledevices':"Siedle-Geräte",'dashboard_menu_server':"Server",'dashboard_menu_supportinfo':"Support-Informationen",'dashboard_menu_search':"Suchen",'dashboard_menu_exit':"Abmelden",'dashboard_menu_device_restartall':"Geräte-Neustart",'dashboard_menu_device_restartneeded':"Geräte-Neustart (wenn nötig)",'dashboard_menu_restart_service':"Dienste-Neustart",'dashboard_menu_reboot':"Hardware-Neustart",'dashboard_toast_nonetworkconfigured':"Sie müssen in den Grundparametern ein Netzwerkinterface auswählen! [Hier klicken um die Grundparameter zu ändern.](ID_BASICPARAM)",'presence_document_title':"Teilnehmerstatus",'presence_filter_label':"Filter",'presence_filter_check_online':"Online",'presence_filter_check_offline':"Offline",'presence_header_object_name':"Objekt",'presence_header_callnumber':"Rufnummer",'presence_header_status':"Status",'presence_header_ip':"IP-Adresse",'presence_header_last_action':"Letzte Meldung",'presence_header_remote_number':"Zielrufnummer",'presence_status_offline':"Offline",'presence_status_devstart':"Gerät startet",'presence_status_devready':"Warte auf Sip",'presence_status_idle':"Bereit",'presence_status_dnd':"Nicht stören",'presence_status_call':"Ruft an",'presence_status_ring':"Wird gerufen",'presence_status_disc':"Verbindungsabbau",'presence_status_busy':"Besetzt",'presence_status_trying':"Wartend",'presence_status_error':"Fehler",'basicparam_document_title':"Grundparameter",'basicparam_label_syslang':"Systemsprache",'basicparam_label_netcardgrid':"Netzwerkadapter",'basicparam_netcardgrid_header_name':"Name",'basicparam_netcardgrid_header_ipv4':"IPv4",'basicparam_netcardgrid_header_ipv6':"IPv6",'basicparam_netcardgrid_header_client':"Client",'basicparam_netcardgrid_header_phone':"Telefonie",'basicparam_box_title_times_door':"Zeiten (Tür)",'basicparam_label_door_ring_timeout':"Max. Klingeldauer",'basicparam_label_door_call_timeout':"Max. Gesprächsdauer",'basicparam_label_latevideo_timeout':"Videonachlauf",'basicparam_box_title_times_general':"Zeiten (Allgemein)",'basicparam_label_ring_timeout':"Max. Klingeldauer",'basicparam_label_call_timeout':"Max. Gesprächsdauer",'basicparam_box_title_times_lists':"Zeiten (Listen)",'basicparam_label_lists_time_grabpic':"Bildspeicher, max.",'basicparam_label_lists_time_cdr':"Rufliste, max.",'basicparam_label_lists_time_voicemail':"Voicemail, max.",'basicparam_postlabel_seconds':"Sek.",'basicparam_postlabel_minutes':"Min.",'basicparam_postlabel_days':"Tage",'basicparam_box_deviceoutage':"Geräte-Ausfallerkennung",'basicparam_label_deviceoutage_time':"Störung melden nach",'basicparam_postlabel_deviceoutage_time':"Min.",'basicparam_label_deviceoutage_contact':"Schaltkontakt",'basicparam_error_outage_time':"Die Erkennungszeit für die Ausfallerkennung muss zwischen 1 und 10 Minuten sein!",'basicparam_error_door_ring_timeout':"Die Klingeldauer muss zwischen 5 und 60 Sekunden sein!",'basicparam_error_door_call_timeout':"Die Gesprächsdauer muss zwischen 1 und 1440 Minuten sein!",'basicparam_error_lateviedeo_timeout':"Der Videonachlauf muss zwischen 0 und 15 Sekunden sein!",'basicparam_error_ring_timeout':"Die Klingeldauer muss zwischen 5 und 120 Sekunden sein!",'basicparam_error_call_timeout':"Die Gesprächsdauer muss zwischen 10 und 1440 Minuten sein!",'basicparam_error_grabpic_time':"Die Zeit für den Bildspeicher muss zwischen 1 und 120 Tagen liegen!",'basicparam_error_cdr_time':"Der Zeit für die Ruflisten muss zwischen 1 und 120 Tagen liegen!",'basicparam_error_voicemail_time':"Die Zeit für die Voicemail muss zwischen 1 und 120 Tagen liegen!",'basicparam_toast_unsaved_notice':"Sie müssen die Dienste neu starten, damit die Änderungen aktiv werden. [Hier klicken um Dienste neu zu starten.](ID_SAVE_UNSAVED)!",'logfile_document_title':"Protokoll",'logfile_header_created':"Zeit",'logfile_header_entid':"Objekt",'logfile_header_message':"Meldung",'logfile_header_admin':"Benutzer",'callnumberplan_document_title':"Rufnummernpläne",'callnumberplan_label_profiles':"Rufprofile",'callnumberplan_label_permissions':"Wahlregeln",'callnumberplan_profile_header_name':"Profile",'callnumberplan_permission_header_rule':"Regel",'callnumberplan_permission_header_allowed':"Erlaubt",'callnumberplan_buttonlabel_add':"Neu",'callnumberplan_buttonlabel_edit':"Bearbeiten",'callnumberplan_buttonlabel_delete':"Löschen",'backup_document_title':"Sichern / Wiederherstellen",'backup_group_backup':"Sichern",'backup_group_restore':"Wiederherstellen",'backup_backup_label':"Sicherungsdatei",'backup_backup_button':"Erstellen",'backup_restore_label':"Datei hochladen",'backup_restore_upload':"Sicherungsdatei hier ablegen oder klicken für Auswahl",'backup_restore_button':"Wiederherstellen",'backup_restore_button_exec':"Wird ausgeführt...",'backup_restore_help':"Das System wird heruntergefahren und die hochgeladene Sicherung wird aktiviert. Ihre bestehende Installation geht dabei unwiederbringlich verloren!",'backup_restore_file':"Hochgeladene Datei",'backup_restore_file_placeholder':"Keine Datei vorhanden",'backup_restore_file_failed':"Fehler: Die Datei \„{file}\“ ist keine Access Backupdatei!",'backup_restore_file_corrupted':"Fehler: Die Datei \„{file}\“ enhält keine Backupdaten!",'search_document_title':"Suchergebnis",'search_header_entid':"Objekt",'search_header_message':"Meldung",'search_result_message':"Gefunden in:",'search_result_name':"Name",'search_result_comment':"Beschreibung",'search_result_statepoint':"Funktionen",'search_result_keys':"Tasten/Ein-Ausgänge",'search_result_callnumber':"Rufnummer",'search_result_macaddr':"MAC-Adresse",'search_result_prefix':"Präfix",'search_result_externcallnr':"Rufnummer (extern)",'search_result_account':"Benutzername",'licenses_document_title':"Lizenzen",'licenses_download_tooltip':"Export Hardware-Kennung",'licenses_download_label':"Exportieren",'license_demoperiod_label':"Ende Demozeitraum",'license_supportcontract_label':"Ende Wartungsvertrag",'license_grid_label':"Lizenzen",'license_gridheader_type':"Lizenztyp",'license_gridheader_description':"Beschreibung",'license_gridheader_available':"Verfügbar",'license_gridheader_engaged':"Verwendet",'license_description_apr':"Access Nutzerlizenz",'license_description_alfp':"Access Lizenz Fremdgerät Panel",'license_description_alsa':"Access Lizenz Siedle APP",'license_description_alt':"Access Lizenz Telefonie-Anbindung",'license_description_asc':"Access Software Concierge",'license_description_asht':"Access Software Haustelefon",'license_description_knx':"Access Lizenz KNX-Anbindung",'license_description_sipaudio':"Access Lizenz Fremdgeräte-Anbindung",'license_description_alfc':"Access Lizenz Fremdgerät IP-Kamera",'license_description_alfm':"Access Lizenz Funktionsmodul",'license_box_upload':"Lizenz",'license_upload_label':"Upload",'license_upload':"Lizenzdatei hier ablegen oder klicken für Auswahl",'license_toast_expired':"Ihre Demoversion ist abgelaufen. Bitte spielen Sie eine gültige Lizenz ein.",'license_toast_invalid':"Ihre Lizenz ist ungültig. Bitte spielen Sie eine gültige Lizenz ein.",'license_toast_invalid_version':"Ihre Lizenz gehört zu einer früheren Version. Bitte spielen Sie eine aktuelle Lizenz ein.",'license_file_upload_succeeded':"\„{file}\“ wurde erfolgreich hochgeladen und aktiviert!",'license_file_upload_failed':"\„{file}\“ konnte nicht hochgeladen werden!",'license_file_upload_invalid':"\„{file}\“ kann nicht eingespielt werden. Es handelt sich nicht um eine gültige Lizenzdatei!",'license_file_upload_invalidversion':"\„{file}\“ kann nicht eingespielt werden. Das ist eine Lizenz einer früheren Version!",'rules_document_title':"Funktionsmodule",'rules_box_upload':"Modul",'rule_upload_label':"Datei hochladen",'rule_upload_text':"Moduldatei hier ablegen oder klicken für Auswahl",'rule_grid_label':"Aktive Funktionen",'rule_gridheader_description':"Beschreibung",'rule_gridheader_version':"Version",'rule_gridheader_type':"Typ",'rule_type_core':"Grundsystem",'rule_type_module':"Modul",'rule_button_delete':"Löschen",'rule_error_update':"Für das Modul ist ein Update erforderlich!",'rule_error_version':"Für das Modul ist ein Server ab der Version {{version}} nötig!",'systemtime_document_title':"Systemzeit",'systemtime_srvtime_label':"Serverzeit",'systemtime_clienttime_label':"Browserzeit",'systemtime_region_label':"Region",'systemtime_zone_label':"Zeitzone",'systemtime_timemode_label':"Modus",'systemtime_timemode_automatic':"Automatische Updates über Zeitserver",'systemtime_timemode_manual':"Hardware-Uhr verwenden",'systemtime_timeserver_box':"Zeitsynchronisation",'systemtime_ntplist_label':"Zeitserver",'systemtime_button_add':"Neu",'systemtime_button_edit':"Bearbeiten",'systemtime_ntpstat_label':"NTP-Status",'systemtime_ntpstat_status_sync':"Synchronisiert mit {{ntpip}}, Stratum {{stratum}}.",'systemtime_ntpstat_status_unsync':"Nicht synchronisiert",'systemtime_ntpstat_status_unknown':"Keine Verbindung zum NTP-Server",'systemtime_ntpstat_info_updates':"Updates alle {{polling}} Sekunden, Zeitkorrektur genau {{tmcorrect}}ms.",'systemtime_timesend_label':"Uhrzeit setzen",'systemtime_timesend_button':"Aktuelle Browserzeit setzen",'systemtime_timesend_help':"Die Uhrzeit wird unabhängig von der Zeitzone gesetzt. Die Änderungen sind sofort wirksam.",'systemtime_toast_unsaved_notice':"Sie müssen die Hardware neu starten, damit die Änderungen aktiv werden. [Hier klicken um Hardware neu zu starten.](ID_SAVE_UNSAVED)!",'network_dns_box':"DNS (Domain Name System)",'network_netcard_title_prefix':"Netzwerkadapter",'network_toast_expert_mode_notice':"Expertenmodus: Ihre Systemdateien wurde manuell verändert. Sie können weiterarbeiten, oder die Änderungen verwerfen. [Hier klicken um Änderungen zu verwerfen.](ID_RESYNC)",'network_toast_unsaved_notice':"Sie müssen die Dienste neu starten, damit die Änderungen aktiv werden. [Hier klicken um Dienste neu zu starten.](ID_SAVE_UNSAVED)!",'dns_config_server':"DNS-Server",'dns_config_add':"Neu",'dns_config_edit':"Bearbeiten",'dns_config_delete':"Löschen",'dns_config_modus':"DNS-Serveradresse automatisch beziehen",'dns_config_label_modus':"Modus",'dns_config_supersede_list_empty':"Es muss eine DNS-Serveradresse angegeben werden!",'dns_config_dhcp_dns_unknown':"Unbekannt",'dns_config_dns_noconfig':"Kein DNS-Server konfiguriert",'netcard_label_name':"Name",'netcard_label_system_name':"Systemname",'netcard_label_type':"IP-Konfiguration",'netcard_label_ip':"IP-Adresse",'netcard_label_net_mask':"Subnetzmaske",'netcard_label_dhcp':"DHCP-Server",'netcard_label_default_gateway':"Standardgateway",'netcard_label_dns_mode':"DNS-Server",'netcard_label_button_options':"DHCP",'netcard_static':"Feste IP",'netcard_dhcp':"IP vom DHCP",'netcard_no_config':"Nicht konfiguriert",'netcard_dhcp_server':"DHCP-Server",'netcard_is_dhcp_active':"Für diesen Netzwerkadapter",'netcard_dhcp_pool_start':"IP-Pool, Start",'netcard_dhcp_pool_end':"IP-Pool, Ende",'netcard_pool_error_range':"Der Start des IP-Pools muss kleiner sein als das Ende!",'netcard_pool_error_ip':"Die IP-Adresse darf nicht im Bereich des DHCP IP-Pools sein!",'netcard_ip_as_access':"Die IP-Adresse ist als Access Server IP gesetzt!",'netcard_button_options':"Optionen",'netcard_warning_options_changed':"Ihre DHCP-Optionen wurden automatisch angepasst. Bitte prüfen Sie die Einstellungen!",'change_ip_dialog_title_edit':"DNS-Serveradresse ändern",'change_ip_dialog_title_new':"Neue DNS-Serveradresse",'change_ip_dialog_label_ip':"IP-Adresse",'ip_change_error_empty':"DNS-Serveradresse darf nicht leer sein!",'ip_change_error_not_valid':"Ungültige IP-Adresse",'dhcp_menu_general':"Allgemein",'dhcp_menu_options':"Optionen",'dhcp_menu_leases':"Leases",'dhcp_menu_reservations':"Reservierungen",'dhcp_ipaddr':"IP-Adresse",'dhcp_subnetmask':"Subnetzmaske",'dhcp_scope_start':"IP-Pool, Start",'dhcp_scope_end':"IP-Pool, Ende",'dhcp_scope_error_range':"IP-Adresse nicht im Bereich des Netzwerks",'dhcp_optionheader_option':"Optionsname (ID)",'dhcp_optionheader_value':"Wert",'dhcp_optionheader_active':"Aktiv",'dhcp_option_enabled':"Status",'dhcp_option_enabled_check':"Option ist aktiv",'dhcp_option_name':"Option",'dhcp_option_value':"Wert",'dhcp_option_autovalue_tooltip':"Standardwert setzen",'dhcp_option_active_yes':"Ja",'dhcp_option_active_no':"Nein",'dhcp_leaseheader_macaddr':"MAC-Adresse",'dhcp_leaseheader_ipaddr':"IP-Adresse",'dhcp_leaseheader_hostname':"Hostname",'dhcp_leaseheader_date':"Ablaufdatum",'dhcp_lease_action_label':"Lease",'dhcp_lease_delete':"Freigeben",'dhcp_lease_delete_undo':"Rückgängig",'dhcp_lease_reserve':"Reservieren",'dhcp_lease_status_active':"Aktiv",'dhcp_lease_status_deleted':"Gelöscht",'dhcp_lease_status_new':"Neu",'dhcp_lease_status_edit':"Verändert",'dhcp_reserve_action_label':"Reservierung",'dhcp_reserve_mac_label':"MAC-Adresse",'dhcp_reserve_ip_label':"IP-Adresse",'dhcp_reserve_new':"Neu",'dhcp_reserve_delete':"Löschen",'dhcp_reserve_delete_undo':"Rückgängig",'dhcp_reserve_edit':"Bearbeiten",'dhcp_reserve_error_duplicatemac':"MAC-Adresse existiert bereits!",'dhcp_reserve_error_duplicateip':"IP-Adresse existiert bereits!",'dhcp_reserve_error_ipnotinrange':"IP-Adresse nicht im Scope!",'dhcp_reserve_error_ipnetcard':"IP-Adresse ist für das Interface vergeben!",'dhcp_reserve_cancel_tooltip':"Abbrechen",'dhcp_reserve_apply_tooltip':"Übernehmen",'ruledbg_title':"Rule Event Viewer",'ruledbg_label_instances':"Active Rule Instances",'ruledbg_label_currentinst':"Current Rule Instance",'ruledbg_label_details':"Details",'ruledbg_label_events':"Events",'ruledbg_label_watch':"Watch",'api_document_title':"Schnittstellen",'api_group_onvif':"ONVIF-Kamerafreigabe",'api_label_onvif_enable':"Video",'api_label_onvif_enable_text':"Türkameras über ONVIF freigeben",'api_label_onvif_user':"Benutzername",'folder_document_title':"Ordner",'folder_document_changed_notice':"Das Dokument wurde auf einem anderen Computer geändert. Drücken Sie [\„Verwerfen\“](ID_UNDO) um  diese Änderungen zu übernehmen.",'folder_create_type':"Ordner",'apartment_document_title':"Innengerät",'apartment_create_type_afs':"Siedle AFS",'apartment_create_type_afsv':"Siedle AFSV",'apartment_create_type_ahf':"Siedle AHF 870-0",'apartment_create_type_ahfv':"Siedle AHFV 870-0",'apartment_create_type_aht':"Siedle AHT 870-0",'apartment_create_type_ahtv':"Siedle AHTV 870-0",'tkphone_document_title':"TK-Telefon",'tkphone_create_type':"TK-Telefon",'tkphone_label_gateway':"TK-Gateway",'avp_document_title':"Innengerät",'avp_create_type':"Siedle AVP 870-0",'door_document_title':"Tür",'door_create_type':"ATLC 670-0 (ATLC V1)",'tkgw_document_title':"TK-Gateway",'tkgw_create_type':"TK-Gateway",'tkgw_label_prefix':"Wählpräfix",'tkgw_label_incoming':"Rufe (eingehend)",'tkgw_incoming_forward':"Weiterleiten zu",'tkgw_box_title_gateway':"TK-Gateway",'tkgw_label_registrar':"Registrar",'tkgw_label_account':"Account",'tkgw_label_password':"Kennwort",'tkgw_label_framesize':"Frame-Size",'tkgw_label_transport':"Transport",'tkgw_label_port':"Port",'tkgw_framesize_auto':"Automatisch",'tkgw_framesize_ms20':"20ms",'tkgw_framesize_ms40':"40ms",'tkgw_transport_udp':"UDP",'tkgw_transport_tcp':"TCP",'tkgw_transport_tls':"TLS",'tkgw_error_port_empty':"Darf nicht leer oder Null sein!",'trunk_document_title':"SIP-Trunk",'trunk_create_type':"SIP-Trunk",'trunk_label_prefix':"Wählpräfix",'trunk_label_remote_prefix':"Entfernter Präfix",'trunk_label_lines':"Leitungen",'trunk_box_title_trunk':"SIP-Trunk",'trunk_label_ipaddress':"IP-Adresse",'trunk_label_framesize':"Frame-Size",'trunk_label_port':"Port",'trunk_error_license':"Nicht ausreichend Lizenzen vorhanden!",'group_document_title':"Gruppe",'group_create_type':"Gruppe",'group_callreject_label':"Ruf verweigern",'group_callreject_check':"Ablehnen führt zum Ende des Gruppenrufs",'group_missedcall_label':"Angenommene Rufe",'group_missedcall_check':"Für die anderen Teilnehmer als verpassten Ruf eintragen",'group_voicemail_label':"Voicemail",'group_voicemail_check':"Von Gruppenteilnehmern bei Gruppenruf zulassen",'group_member_box':"Gruppe",'group_member_label':"Mitglieder",'group_member_placeholder':"Keine Mitglieder",'group_member_header_name':"Name",'group_member_header_callnumber':"Rufnummer",'group_member_button_add':"Hinzufügen",'group_member_button_tooltip':"Fügt ein neues Mitglied hinzu",'sipaudio_document_title':"SIP Telefon",'sipaudio_create_type':"SIP Telefon",'susapp_document_title':"Siedle APP",'susapp_create_type':"Siedle APP",'susapp_qrcode_label':"Registrierung",'susapp_qrvalid_label':"Gültig bis",'susapp_susstatus_label':"Status",'susapp_susstatus_scanned':"Registriert",'susapp_susstatus_request_qr':"QR-Code wird abgerufen",'susapp_susstatus_expired':"QR-Code abgelaufen",'susapp_refresh_expired_tooltip':"Neuen QR-Code abrufen",'client_document_title':"ASC",'client_create_type':"Siedle ASC 170",'client_device_type':"ASC_170",'asht_document_title':"ASHT",'asht_create_type':"Siedle ASHT 170",'asht_device_type':"ASHT_170",'jungpanel_document_title':"JUNG Panel",'jungpanel_create_type':"JUNG Panel",'addrbook_document_title':"Adressbuch",'addrbook_create_type':"Adressbuch",'addrbook_box_grid':"Adressbuch",'addrbook_header_name':"Name",'addrbook_header_callnr':"Rufnummer",'addrbook_header_comment':"Kommentar",'addrbook_button_add':"Neu",'addrbook_button_edit':"Bearbeiten",'admin_document_title':"Admin",'admin_create_type':"Administrator",'knxgw_document_title':"KNX Gateway",'knxgw_create_type':"KNX Gateway",'knxgw_box_grid':"KNX",'knxgw_label_grid':"Gruppenadressen",'knxgw_header_name':"Label",'knxgw_header_addr_send':"Gruppenadresse",'knxgw_header_addr_receive':"Rückmeldeobjekt",'knxgw_button_add':"Neu",'knxgw_button_edit':"Bearbeiten",'timeline_document_title':"Zeitprofil",'timeline_create_type':"Zeitprofil",'timeline_status_label':"Label (Status)",'timeline_automatic_label':"Label (Automatik-Betrieb)",'timeline_box_time':"Zeitprofil",'timeline_monday':"Montag",'timeline_tuesday':"Dienstag",'timeline_wednesday':"Mittwoch",'timeline_thursday':"Donnerstag",'timeline_friday':"Freitag",'timeline_staturday':"Samstag",'timeline_sunday':"Sonntag",'timeline_detail_start':"Start",'timeline_detail_end':"Stop",'timeline_box_action':"Aktionen",'timeline_action_on':"Aktionen bei \„Ein\“",'timeline_action_off':"Aktionen bei \„Aus\“",'agw_document_title':"Access Gateway",'ipcam_document_title':"IP Kamera",'ipcam_create_type':"IP Kamera",'ipcam_label_account':"Benutzername",'ipcam_label_auth':"Autorisierung",'ipcam_check_auth':"Kamera erfordert Benutzername und Kennwort",'ipcam_streams_placeholder':"Keine Daten ermittelt",'ipcam_grid_name':"Name",'ipcam_grid_resultion':"Auflösung",'ipcam_grid_codec':"Codec",'ipcam_group_streams':"Kamerainformationen",'ipcam_label_streams':"Profile",'const_common_loading':"Daten werden geladen...",'const_common_loading_short':"Laden...",'const_common_search_placeholder':"Suchen",'const_common_empty_placeholder':"Bitte auswählen",'const_switchtype_toggle':"Umschalten",'const_switchtype_on':"Ein",'const_switchtype_off':"Aus",'const_switchtype_trigger':"Flush",'const_category_new_door':"Türstation",'const_category_new_indoor':"Innengeräte",'const_category_new_hardware':"Fremdgerät",'const_category_new_software':"Software-Client",'const_form_dashboard':"Dashboard",'const_form_basicparam':"Grundparameter",'const_form_presence':"Teilnehmerstatus",'const_form_backup':"Sichern / Wiederherstellen",'const_form_systemtime':"Systemzeit",'const_form_network':"Netzwerk",'const_form_logfile':"Protokoll",'const_form_callnumberplan':"Rufnummernpläne",'const_form_licenses':"Lizenzen",'const_form_test_page':"Test_Page",'const_form_rules':"Funktionsmodule",'const_form_label_help':"Hilfe",'const_form_tooltip_help':"Zeigt Hilfeseite",'const_form_tooltip_delete':"Löschen",'const_form_tooltip_permission':"Rechte einstellen",'const_form_label_add':"Neu",'const_form_tooltip_add':"Erstellt ein neues Objekt",'const_form_label_save':"Speichern",'const_form_tooltip_save':"Speichert das aktuelle Dokument",'const_form_label_undo':"Verwerfen",'const_form_tooltip_undo':"Verwirft gemachte Änderungen",'const_form_tooltip_lists':"Zeigt die Listen des Geräts",'const_form_tooltip_devicerestart':"Startet das Gerät neu",'const_form_label_refresh':"Aktualisieren",'const_form_tooltip_refresh':"Erneut laden",'const_messagebox_delete_text':"Dieses Objekt wird endgültig gelöscht. Möchten Sie fortfahren?",'const_messagebox_delete_label':"Löschen",'const_messagebox_delete_refused':"Dieser Ordner enthält den Haupt-Administrator, der nicht gelöscht werden darf!",'const_time_manual':"Manuell",'const_framesize_auto':"Auto",'const_framesize_ms20':"20ms",'const_framesize_ms40':"40ms",'aui_lang_dashboard':"Dashboard",'aui_lang_basicparameters':"Grundparameter",'aui_lang_callprofiles':"Rufnummernpläne",'aui_lang_userstatus':"Teilnehmerstatus",'aui_lang_search':"Suche",'aui_lang_log':"Logfile",'aui_lang_unconfdev':"Unkonfigurierte Geräte",'aui_lang_confdev':"Projekt",'aui_lang_confdevcomment':"Projektordner für das Access-System",'aui_lang_licence':"Lizenzen",'aui_lang_networksettings':"Netzwerk",'aui_lang_tile':"Funktion",'const_menu_network':"Netzwerk",'const_menu_pabx':"TK Gateway",'const_menu_password':"Kennwort",'const_menu_supportinfo':"Support-Informationen",'const_menu_exit':"Abmelden",'const_menu_device_restart':"Geräte-Neustart",'const_menu_devices':"Geräteliste",'const_menu_restart_service':"Dienste-Neustart",'const_menu_restart_service_short':"Neustart",'const_menu_reboot':"Hardware-Neustart",'const_menu_help':"Hilfe",'const_menu_save':"Speichern",'const_menu_undo':"Verwerfen",'const_menu_construct':"Under Construction",'const_box_title_general':"Allgemein",'const_dialog_button_ok':"Ok",'const_dialog_button_save':"Speichern",'const_dialog_button_apply':"Übernehmen",'const_dialog_button_cancel':"Abbrechen",'const_dialog_button_close':"Schließen",'const_dialog_button_reload':"Aktualisieren",'const_toast_savefirst':"Bitte speichern Sie zuerst die Form!",'const_toast_suserror':"Die Verbindung zum Siedle Server für die Kommunikation mit den Apps ist fehlgeschlagen. Überprüfen Sie Ihre Internetverbindung!",'const_error_novalidmac':"Keine gültige MAC-Adresse!",'const_error_empty':"Es muss ein Wert gesetzt werden!",'const_error_statepoint_deleted':"Das Objekt existiert nicht mehr!",'const_error_statepoint_permission':"Keine Rechte auf das Objekt zuzugreifen!",'const_error_device_deleted':"Das Gerät ist gelöscht!",'const_error_callnumber_changed':"Die verwendete Rufnummer hat sich geändert!",'const_deleted_item':"Gelöschter Eintrag",'const_deleted_item_short':"Gelöscht",'aui_widget_buildingAutomationUrl':"Gebäudeautomation",'aui_widget_callcontact':"Ruf-Kontakt",'aui_widget_camera':"Video",'aui_widget_dnd':"Nicht stören",'aui_widget_door':"Türöffner",'aui_widget_door2':"Türöffner 2",'aui_widget_doormatic':"Türmatik",'aui_widget_externcall':"Ruf, extern",'aui_widget_grabpic':"Bildspeicher",'aui_widget_grabpicmenu':"Bildspeicher Menü",'aui_widget_ldlight':"Türlicht",'aui_widget_ldopener':"Türöffner",'aui_widget_light':"Türlicht",'aui_widget_lightcontact':"Türlicht",'aui_widget_openercontact':"Türöffner",'aui_widget_presence':"Onlinestatus",'aui_widget_redir':"Rufumleitung",'aui_widget_ringtone':"Klingeltöne",'aui_lang_ringdoor':"Türruf",'aui_widget_call':"Ruf",'aui_widget_setredir':"Rufumleitung",'aui_widget_setsp':"Schalten",'aui_widget_setspknx':"KNX senden",'aui_widget_state_notifying_contact':"Zustand melden",'aui_widget_storeycall':"Etagenruf",'aui_widget_switchcontact':"Ausgang",'aui_widget_timeline':"Status",'aui_widget_automatic':"Automatik",'aui_widget_timeswitchcontact':"Ausgang",'aui_widget_video':"Video-URL",'STORE_CREATE':"Neues Objekt '{{name}}' wurde angelegt.",'STORE_UPDATE':"Objekt '{{name}}' wurde verändert.",'STORE_DELETE':"Objekt '{{name}}' wurde gelöscht.",'SYSTEM_RESTARTED':"Neustart der Access Dienste wurde angefordert.",'SYSTEM_REBOOT':"Neustart der Access Hardware wurde angefordert.",'REBOOT_ALL':"Für alle Geräte wurde ein Neustart angefordert.",'REBOOT_DEVICE':"Für das Gerät '{{name}}' wurde ein Neustart angefordert.",'REBOOT_NEEDED':"Ein Neustart aller Geräte, falls notwendig, wurden angefordert.",'ENTITY_ERROR':"In diesem Objekt sind Fehler, bitte öffnen Sie die Form für Details!",'DEVRESTART_NEEDED':"Geräte-Neustart erforderlich!",'messagebox_cancel':"Abbrechen",'messagebox_save':"Speichern",'messagebox_ok':"Ok",'modinstall_title':"Modulinstallation",'modinstall_info_starting':"Lade Modulinformationen.",'modinstall_info_install':"Installiere Moduldateien.",'modinstall_info_update':"Update bestehende Daten.",'modinstall_info_finished':"Installation abgeschlossen.",'password_dialog_title':"Kennwort ändern",'password_label_password':"Kennwort",'password_label_repeated':"Wiederholung",'password_placeholder_password':"Neues Kennwort",'password_placeholder_repeated':"Wiederholung",'change_password_error_empty':"Das Kennwort darf nicht leer sein!",'change_password_error_different':"Kennwörter nicht identisch!",'reboot_agw_really_restart_agw':"Ihre Dienste werden jetzt neu gestartet. Möchten Sie den Neustart durchführen?",'reboot_agw_reboot':"Ihre Hardware wird jetzt neu gestartet. Möchten Sie den Neustart durchführen?",'reboot_agw_cancel':"Abbrechen",'reboot_agw_ok':"Neustart",'formnew_please_select':"Bitte wählen Sie das zu erzeugende Objekt",'formnew_no_license':"(Keine Lizenz verfügbar)",'formnew_title':"Neues Objekt",'devicelist_title':"Gerätelisten",'devicelist_category_contact':"Kontakte",'devicelist_category_camera':"Kameras",'devicelist_category_switch':"Schaltliste",'permissions_dialog_title':"Rechte für \„{{name}}\“",'permissions_button_add':"Hinzufügen",'permissions_button_delete':"Löschen",'permissions_who_placeholder':"Fügen Sie hier Objekte hinzu um spezielle Rechte zu vergeben.",'permissions_inherit_objects':"Zu vererbende Rechte (Kindobjekte)",'permissions_label_permissionlist':"Rechte",'permissions_label_who':"Wer",'permissions_rightgrid_text':"Beschreibung",'permissions_rightgrid_read':"Lesen",'permissions_rightgrid_write':"Ausführen",'permissions_rightgrid_cr':"Geerbt",'permissions_rightgrid_cr_tooltip':"Geerbt von \„{{name}}\“",'name_callprofile_dialog_title':"Rufprofil",'name_callprofile_label_name':"Rufprofil Name",'name_callprofile_error_empty':"Das Rufprofil darf nicht leer sein!",'permission_callprofile_dialog_title':"Wahlregel",'permission_callprofile_label_rule':"Rechtemaske",'permission_callprofile_label_forbid':"Verbieten",'permission_callprofile_error_empty':"Die Rechtemaske darf nicht leer sein!",'permission_callprofile_error_wildcard_pos':"* kann nur am Ende der Maske verwendet werden!",'address_title':"Adressbucheintrag",'address_cancel':"Abbrechen",'address_save':"Speichern",'address_label_label':"Name",'address_label_callnumber':"Rufnummer",'address_label_comment':"Kommentar",'address_error_nointernal':"Im Adressbuch sind keine internen Teilnehmer erlaubt!",'knxentry_title':"KNX-Schaltpunkt",'knxentry_label_label':"Label",'knxentry_label_send':"Gruppenadresse",'knxentry_label_receive':"Rückmeldeobjekt",'knxentry_postfix_receive':"(optional)",'ntpentry_title':"Ntp-Server",'ntpentry_cancel':"Abbrechen",'ntpentry_save':"Speichern",'ntpentry_ipaddress_label':"IP-Addresse / Host",'macdialog_title':"Geräte- und Benutzerwechsel",'macdialog_help':"Geben Sie die neue MAC-Adresse des Geräts an, welches Sie als Austauschgerät bekommen haben.",'macdialog_warning_online':"Das Gerät darf nicht am Server angemeldet sein!",'macdialog_label':"MAC-Adresse",'macdialog_susreg_label':"Registrierung",'macdialog_susreg_check':"Mobiltelefon des Benutzers abmelden",'macdialog_userparam_label':"Zurücksetzen",'macdialog_userparam_check':"Einstellungen des Benutzers",'macdialog_calllist_check':"Ruflisten des Benutzers",'macdialog_grabpic_check':"Bildspeicher des Benutzers",'devkey_tooltip_keysingle':"Einfach-Click",'devkey_tooltip_keydouble':"Doppel-Click",'devkey_tooltip_keypos':"Taste drücken",'devkey_tooltip_keyneg':"Taste loslassen",'devkey_tooltip_led':"LED",'change_keytone_dialogtitle':"Rufquittierung",'change_keytone_dialogcancel':"Abbrechen",'change_keytone_dialogsave':"Speichern",'change_keytone_dialog_call_target':"Rufziel",'change_keytone_dialog_callnumber':"Ton",'change_keytone_dialog_tone_must_selected':"Ein Rufquittungston muss ausgewählt sein",'scandevice_title':"Gerätesuche",'scandevice_check_showunconfigured':"Nur auswählbare Geräte zeigen",'scandevice_status_search_unknown':"Suche nach Unbekannten Geräten...",'scandevice_status_searchknx':"Suche nach KNX IP-Gateways...",'scandevice_status_searchipcam':"Suche nach IP-Kameras...",'scandevice_status_notfound':"Keine unkonfigurierten Geräte gefunden!",'scandevice_grid_name':"Name",'scandevice_grid_ip':"IP Adresse",'ipcam_title':"IP Kamera Prüfung",'ipcam_button_login':"Prüfen",'ipcam_label_ipaddress':"IP Adresse",'page_header_title':"Siedle Access - Administration",'page_header_search_placeholder':"Suchen",'page_header_warning_websocket':"Die Updategeschwindigkeit vom Server ist herabgesetzt!",'page_menu_dashboard':"Meldungen anzeigen",'page_menu_settings':"Einstellungen",'page_menu_exit':"Abmelden",'accordion_section_general':"Systemwartung",'accordion_section_devtree':"Teilnehmer",'const_error_novalidip':"Keine gültige IP-Adresse!",'const_error_ipempty':"IP-Adresse darf nicht leer sein!",'const_error_novalidmask':"Keine gültige Subnetzmaske!",'const_error_nostring':"Interner Fehler",'const_error_notemptystring':"Es muss ein Wert eingetragen werden!",'const_error_novalidport':"Keine gültiger Port!",'const_error_portempty':"Der Port darf nicht leer sein!",'const_error_novalidiporhost':"Keine gültige IP-Adresse oder Hostname!",'widget_base_avptile_label':"Kachelbeschriftung",'widget_base_avptile_icon':"Kachelsymbol",'device_config_description_linkedled':"mit LED",'keytitle_apartment_key':"Taste {key}",'keytitle_afsv_key1':"Türöffner",'keytitle_afsv_key2':"Taste I",'keytitle_afsv_key3':"Taste II",'keytitle_afsv_key4':"Taste Rufton aus",'keytitle_afs_key1':"Türöffner",'keytitle_afs_key2':"Taste I",'keytitle_afs_key3':"Taste Rufton aus",'keytitle_ahtf_a1':"Ausgang A1",'keytitle_ahtf_e1':"Eingang E1",'keytitle_ahtf_a2':"Ausgang A2",'keytitle_ahtf_e2':"Eingang E2",'keytitle_timeline_on':"Aktion für \„Ein\“",'keytitle_timeline_off':"Aktion für \„Aus\“",'triggername_input_generic':"Eingang",'triggername_output_generic':"Ausgang",'triggername_tile':"Antippen",'triggername_callkey':"Klick",'triggername_single':"Einfachklick",'triggername_double':"Doppelklick",'triggername_pos':"Taste drücken",'triggername_neg':"Taste loslassen",'triggername_led':"LED",'keytitle_brma':"Modul {module}, Taste {key}",'keytitle_btm':"Modul {module}, Taste {key}",'keytitle_atlc_a1_toemode':"Spannungsmodus A1",'keytitle_atlc_e':"Eingang E",'keytitle_atlc_a1':"Ausgang A1",'keytitle_atlc_a2':"Ausgang A2",'keytitle_atlc_a3':"Ausgang A3",'keytitle_atlce1_e1':"Eingang Modul 1/E1",'keytitle_atlce1_e2':"Eingang Modul 1/E2",'keytitle_atlce1_a1':"Ausgang Modul 1/A1",'keytitle_atlce1_a2':"Ausgang Modul 1/A2",'keytitle_atlce1_a3':"Ausgang Modul 1/A3",'keytitle_atlce1_a4':"Ausgang Modul 1/A4",'keytitle_atlce2_e1':"Eingang Modul 2/E1",'keytitle_atlce2_e2':"Eingang Modul 2/E2",'keytitle_atlce2_a1':"Ausgang Modul 2/A1",'keytitle_atlce2_a2':"Ausgang Modul 2/A2",'keytitle_atlce2_a3':"Ausgang Modul 2/A3",'keytitle_atlce2_a4':"Ausgang Modul 2/A4",'keytitle_atlce3_e1':"Eingang Modul 3/E1",'keytitle_atlce3_e2':"Eingang Modul 3/E2",'keytitle_atlce3_a1':"Ausgang Modul 3/A1",'keytitle_atlce3_a2':"Ausgang Modul 3/A2",'keytitle_atlce3_a3':"Ausgang Modul 3/A3",'keytitle_atlce3_a4':"Ausgang Modul 3/A4",'keytitle_avp_key1':"Taste Türöffner",'keytitle_avp_tile':"Kachel \„{name}\“",'keytitle_atlm_l':"Taste Türlicht",'permission_label_camera':"Video-Überwachung",'permission_label_addrbook':"Adressbuch",'permission_label_dnd':"Nicht stören",'permission_label_door':"Türöffner",'permission_label_door2':"Türöffner 2",'permission_label_doormatic':"Türmatik",'permission_label_light':"Türlicht",'permission_label_presence':"Anzeige in Kontaktliste",'permission_label_redir':"Rufumleitung",'permission_label_videoprivacy':"Video-Mitsehsperre",'widget_empty_long':"Vorlage (für Widget)",'widget_empty_short':"Vorlage",'widget_empty_label_option1':"Option 1",'widget_empty_label_option2':"Option 2",'widget_empty_check_option1':"Soll jetzt und immer aktiv sein.",'widget_empty_check_option2':"Ist auch eine Möglichkeit.",'widget_none_short':"Keine Funktion",'widget_none_long':"Keine Funktion",'widget_none_noparam':"Keine Parameter",'widget_none_description':"Keine Aktion",'control_calltarget_error_deleted':"Das ausgewählte Rufziel ist gelöscht!",'control_calltarget_error_empty':"Es muss ein Rufziel ausgewählt werden!",'control_calltarget_callnumber_changed':"Die Rufnummer hat sich geändert!",'control_statepoint_error_empty':"Es muss ein Schaltpunkt ausgewählt werden!",'control_entitylist_error_deleted':"Der ausgewählte Eintrag ist gelöscht!",'control_entitylist_error_empty':"Es muss ein Eintrag ausgewählt werden!",'labelctrl_label':"Label",'permcheck_tooltip':"Hinweis: Für dieses Objekt sind noch keine externen Rechte vergeben.",'macaddr_error_exists':"Diese MAC-Adresse ist schon vergeben!",'macaddr_label':"MAC-Adresse",'callwaiting_label':"Anklopfen",'callwaiting_value':"Anrufe auch während aktivem Gespräch signalisieren",'control_callnumber_error_empty':"Rufnummer darf nicht leer sein",'control_callnumber_error_duplicated':"Die Rufnummer ist schon vergeben",'control_callnumber_error_prefix':"Die Rufnummer darf nicht mit einem Gateway- oder Trunkpräfix beginnen!",'control_callnumber_label':"Rufnummer",'control_callprefix_label':"Präfix",'control_callprefix_error_empty':"Der Präfix darf nicht leer sein!",'control_callprefix_error_duplicated':"Der Präfix ist schon vergeben!",'control_callprefix_error_callnumber':"Die Präfix überschneidet sich mit einer Rufnummer!",'addrbk_label_addressbook':"Adressbuch",'devicevolume_value_out':"Lautstärke {value}",'devicevolume_value_in':"Empfindlichkeit {value}",'devicevolume_label_ringvolume':"Ruftonlautstärke",'devicevolume_label_callvolume':"Sprachlautstärke",'devicevolume_label_micvolume':"Mikrofon",'formbase_label_showclock':"Uhrzeit",'formbase_value_showclock':"Uhrzeit am Endgerät anzeigen",'mounttype_label':"Montageart",'mounttype_value_wall':"Wandmontage",'mounttype_value_desk':"Tischmontage",'extern_callnumber_label':"Externe Rufnummer",'control_callprofile_error_deleted':"Das ausgewählte Rufprofil ist gelöscht!",'control_callprofile_label':"Rufprofil",'dtmf_gateway_outgoing_only':"DTMF nur für Rufe von der Tür zum Gateway zulassen",'dtmf_label_door':"Türöffner (DTMF)",'dtmf_label_light':"Türlicht (DTMF)",'control_password_label':"Kennwort",'control_password_label2':"Kennwort, Wiederholung",'control_account_error_empty':"Der Benutzername darf nicht leer sein",'control_account_error_duplicated':"Der Benutzername ist schon vergeben",'control_account_label_account':"Benutzername",'keytone_droplist_label':"Rufquittierung",'keytone_value_none':"Keine",'keytone_value_simple':"Einfach",'keytone_value_individual':"Individuell",'keytone_control_validate_empty_list':"Die Liste darf nicht leer sein",'zam_options_label':"ZAM-Quittierung bei",'zam_option_oncall':"Ruf",'zam_option_onopener':"Türöffner",'zam_option_offhook':"Gesprächsannahme",'zam_option_failedcon':"fehlgeschlagener Verbindung",'control_ringtone_label':"Ruftöne",'control_ringtone_individual':"Individuelle Ruftöne einstellen",'control_ringtone_label_doorcall':"Rufton Türruf",'control_ringtone_label_storeycall':"Rufton Etagenruf",'control_ringtone_label_default':"Rufton Standard",'control_doorcamera_label':"Türkamera",'control_doorcamera_value_none':"Keine Kamera installiert",'control_doorcamera_value_internal':"Interne Kamera",'widget_call_short':"Ruf",'widget_call_long':"Ruf",'widget_call_label':"Rufziel",'widget_call_descriptionline':"Ruf: \„{target}\“ ({callnumber})",'widget_call_error_descriptionline':"Ruf zu gelöschtem Rufziel",'widget_doorcall_long':"Türruf",'widget_doorcall_short':"Türruf",'widget_doorcall_label':"Rufziel",'widget_doorcall_descriptionline':"Ruf: \„{target}\“ ({callnumber})",'widget_doorcall_error_descriptionline':"Ruf zu gelöschtem Rufziel",'widget_set_sp_long':"Schalten",'widget_set_sp_short':"Vorlage",'widget_set_sp_label':"Schaltpunkt",'widget_set_sp_value':"Wert",'widget_set_sp_toggle':"Umschalten",'widget_set_sp_on':"Ein",'widget_set_sp_off':"Aus",'widget_set_sp_error_descriptionline':"Schalten von gelöschtem Schaltpunkt",'widget_set_sp_descriptionline_toggle':"Umschalten: \„{target}\“",'widget_set_sp_descriptionline_on':"Ein: \„{target}\“",'widget_set_sp_descriptionline_off':"Aus: \„{target}\“",'widget_set_sp_descriptionline':"Schalten: \„{target}\“",'widget_setdnd_long':"Nicht stören",'widget_setdnd_short':"Nicht stören",'widget_setdnd_value_label':"Wert",'widget_setdnd_value_toggle':"Umschalten",'widget_setdnd_value_on':"Ein",'widget_setdnd_value_off':"Aus",'widget_setdnd_drop_list_label':"Nicht stören",'widget_setdnd_descriptionline_error':"Schalten von gelöschtem Schaltpunkt",'widget_setdnd_descriptionline_toggle':"Umschalten: \„{target}\“",'widget_setdnd_descriptionline_on':"Ein: \„{target}\“",'widget_setdnd_descriptionline_off':"Aus: \„{target}\“",'widget_setdnd_descriptionline':"Schalten: \„{target}\“",'widget_set_redir_long':"Rufumleitung",'widget_set_redir_short':"Rufumleitung",'widget_set_redir_label':"Schaltpunkt",'widget_set_redir_value':"Wert",'widget_set_redir_label_option2':"Option 2",'widget_set_redir_check_option1':"Soll jetzt und immer aktiv sein.",'widget_set_redir_check_option2':"Ist auch eine Möglichkeit.",'widget_set_redir_type':"Typ",'widget_set_redir_time':"Zeit",'widget_set_redir_type_static':"Sofort",'widget_set_redir_type_na':"Nach Zeit",'widget_set_redir_type_nab':"Wenn Besetzt",'widget_set_redir_on_busy_immediately':"Wenn besetzt sofort",'widget_set_redir_toggle':"Umschalten",'widget_set_redir_on':"Ein",'widget_set_redir_off':"Aus",'widget_set_redir_owner':"Umleiten",'widget_set_redir_owner_own':"dieses Gerät",'widget_set_redir_owner_others':"anderes Gerät",'widget_set_redir_redirect_target':"Ziel",'widget_set_redir_validate_time_error':"Zeit muss gesetzt sein! ( Zeit >= -1 )",'widget_set_redir_validate_time_error2':"Zeit für die Rufmleitung darf max. 120 Sekunden sein!",'widget_set_sp_knx_long':"KNX Telegramm senden",'widget_set_sp_knx_short':"Vorlage",'widget_set_sp_knx_label':"Schaltpunkt",'widget_set_sp_knx_value':"Wert",'widget_set_sp_knx_toggle':"Umschalten",'widget_set_sp_knx_yes':"Ja",'widget_set_sp_knx_no':"Nein",'widget_set_sp_knx_gateway':"Gateway",'widget_set_sp_knx_group_address':"Gruppenadresse",'widget_set_sp_knx_receive_object':"Rückmeldeobjekt",'widget_set_sp_knx_on':"Ein",'widget_set_sp_knx_off':"Aus",'widget_set_sp_knx_error_address':"KNX Adresse ist falsch! ( 0/0/1 - 31/7/255, 0/1 - 31/2047, 1 - 65535  )",'aui_widget_set_sp_knx':"Schalten über KNX",'widget_set_sp_knx_action':"Telegramm senden",'widget_set_sp_knx_response':"Rückmeldeobjekt",'widget_set_doormatic_long':"Türmatik",'widget_set_doormatic_label':"Türmatik",'widget_set_doormatic_value':"Wert",'widget_set_doormatic_toggle':"Umschalten",'widget_set_doormatic_on':"Ein",'widget_set_doormatic_off':"Aus",'widget_set_doormatic_owner':"Türmatik",'widget_set_doormatic_owner_own':"dieses Gerät",'widget_set_doormatic_owner_others':"anderes Gerät",'widget_set_doormatic_error_descriptionline':"Schalten von gelöschtem Schaltpunkt",'widget_set_doormatic_descriptionline_toggle':"Umschalten: \„{target}\“",'widget_set_doormatic_descriptionline_on':"Ein: \„{target}\“",'widget_set_doormatic_descriptionline_off':"Aus: \„{target}\“",'widget_set_doormatic_descriptionline':"Schalten: \„{target}\“",'widget_storey_call_long':"Etagenruf",'widget_storey_call_short':"Etagenruf",'widget_storey_call_description':"Etagenruf",'widget_storey_call_description_target':"\„{target}\“ ({callnumber})",'widget_storey_call_label':"Label",'widget_storey_call_value':"Wert",'widget_storey_call_door_light':"Türlicht",'widget_storey_call_door_opener':"Türöffner",'widget_storey_call_target':"Rufziel",'widget_mute_mic_short':"Mikrofon aus",'widget_mute_mic_long':"Mikrofon aus",'widget_mute_mic_no_parameter':"Keine Parameter",'widget_mute_mic_descriptionline':"Mikrofon aus",'aui_widget_mute_mic':"Mikrofon aus",'widget_mute_speaker_short':"Rufton aus",'widget_mute_speaker_long':"Rufton aus",'widget_mute_speaker_no_parameter':"Keine Parameter",'aui_widget_mute_speaker':"Rufton aus",'widget_grabpicmenu_short':"Bildspeichermenü",'widget_grabpicmenu_long':"Bildspeichermenü",'widget_grabpicmenu_no_parameter':"Keine Parameter",'widget_grabpicmenu_descriptionline':"Bildspeichermenu",'widget_grabpicmenu_error_descriptionline':"Kein Bildspeicher",'widget_externcall_long':"Ruf, extern",'widget_externcall_label':"Rufnummer",'widget_externcall_prefix_hint':"(inklusive Präfix)",'widget_extern_call_descriptionline':"Externruf: ({callnumber})",'widget_extern_call_error_descriptionline':"Ruf zu gelöschtem Rufziel",'widget_extern_call_error_validate_empty':"Rufnummer darf nicht leer sein",'widget_extern_call_error_validate_not_a_number':"Rufnummer muss eine Zahl sein",'widget_ldopener_long':"Türöffner (letzte Tür)",'widget_ldopener_short':"Vorlage",'widget_ldopener_descriptionline':"Türöffner (letzte Tür)",'widget_ldopener_descriptionline_timeout':"Timeout {timeout} Min.",'widget_ldopener_owner':"Türöffner",'widget_ldopener_works_always':"funktioniert immer",'widget_ldopener_works_until_time':"zeitlich begrenzen auf",'widget_ldopener_prefix_time':"Minuten nach Türruf",'widget_ldlight_long':"Türlicht (letzte Tür)",'widget_ldlight_short':"Vorlage",'widget_ldlight_descriptionline':"Türlicht (letzte Tür)",'widget_ldlight_descriptionline_timeout':"Timeout {timeout} Min.",'widget_ldlight_owner':"Türlicht",'widget_ldlight_works_always':"funktioniert immer",'widget_ldlight_works_until_time':"zeitlich begrenzen auf",'widget_ldlight_prefix_time':"Minuten nach Türruf",'widget_led_long':"LED einstellen",'widget_led_short':"LED",'widget_led_color_label':"Farbe",'widget_led_color_white':"Weiß",'widget_led_color_yellow':"Gelb",'widget_led_color_red':"Rot",'widget_led_trigger_label':"LED",'widget_led_trigger_single':"Von Taste \„Einfachklick\“ übernehmen",'widget_led_trigger_double':"Von Taste \„Doppelklick\“ übernehmen",'widget_led_trigger_pos':"Von \„Taste drücken\“ übernehmen",'widget_led_trigger_neg':"Von \„Taste loslassen\“ übernehmen",'widget_led_trigger_individually':"Individuell",'widget_led_type_label':"Typ",'widget_led_type_redir':"Rufumleitung",'widget_led_type_dnd':"Nicht stören",'widget_led_type_doormatic':"Türmatik",'widget_led_type_bool':"Schalten",'widget_led_type_error_select':"Es muss ein Typ ausgewählt werden!",'widget_led_device_label':"Gerät",'widget_led_redirtarget_label':"Umleitungsziel",'widget_led_desciption_action':"LED",'widget_led_desciption_default':"LED: Komplexe Konfiguation",'widget_led_desciption_triggersingle':"Verbunden mit \„Einfachklick\“",'widget_led_desciption_triggerdouble':"Verbunden mit \„Doppelklick\“",'widget_led_desciption_triggerpos':"Verbunden mit \„Taste drücken\“",'widget_led_desciption_triggerneg':"Verbunden mit \„Taste loslassen\“",'widget_led_desciption_triggerindividually':"-",'widget_switchcontact_short':"Schaltkontakt",'widget_switchcontact_long':"Schaltkontakt",'widget_switchcontact_label':"Label",'widget_switchcontact_invert':"Invertieren",'widget_switchcontact_name':"Ausgang",'widget_switchcontact_action':"Schaltausgang",'widget_timeswitchcontact_short':"Zeitschaltkontakt",'widget_timeswitchcontact_long':"Schaltkontakt (Zeitschalter)",'widget_timeswitchcontact_label':"Label",'widget_timeswitchcontact_invert':"Invertieren",'widget_timeswitchcontact_posttrigger':"Nachtriggern",'widget_timeswitchcontact_name':"Ausgang",'widget_timeswitchcontact':"Schaltausgang",'widget_timeswitchcontact_time':"Schaltdauer",'widget_timeswitchcontact_action':"Schaltausgang",'widget_lightcontact_short':"Türlicht",'widget_lightcontact_long':"Funktion (Türlicht)",'widget_lightcontact_used_from_light':"Dieser Ausgang wird von der Funktion Türlicht verwendet.",'widget_lightcontact_used_from_no_one':"Dieser Ausgang wird von keiner Funktion verwendet!",'widget_lightcontact_posttrigger':", Nachtriggern",'widget_lightcontact_invert':", Invertiert",'widget_lightcontact_duration_manual':"manuell",'widget_openercontact_short':"Türöffner",'widget_openercontact_short2':"Türöffner 2",'widget_openercontact_long':"Funktion (Türöffner)",'widget_openercontact_used_from_door':"Dieser Ausgang wird von der Funktion Türöffner verwendet.",'widget_openercontact_used_from_door2':"Dieser Ausgang wird von der Funktion Türöffner 2 verwendet.",'widget_openercontact_used_from_no_one':"Dieser Ausgang wird von keiner Funktion verwendet!",'widget_openercontact_duration_manual':"manuell",'widget_openercontact_abbr_second':"s",'widget_openercontact_abbr_minute':"min",'widget_openercontact_abbr_hour':"h",'widget_openercontact_posttrigger':", Nachtriggern",'widget_openercontact_invert':", Invertiert",'widget_state_notifying_contact_short':"Zustand melden",'widget_state_notifying_contact_long':"Zustand melden",'widget_state_notifying_contact_label':"Label",'widget_state_notifying_contact_invert':"Invertieren",'widget_state_notifying_contact_name':"Zustand melden",'widget_state_notifying_contact':"Schaltausgang",'widget_state_notifying_contact_action':"Zustand melden",'widget_local_door_light_short':"Türlicht",'widget_local_door_light_long':"Türlicht",'widget_local_door_light_no_parameter':"Keine Parameter",'widget_local_door_light_description_line':"Türlicht",'widget_observe_led_knx_long':"KNX empfangen",'widget_observe_led_knx_short':"KNX empfangen",'widget_observe_led_gateway':"KNX-Gateway",'widget_observe_led_knx_group_address':"Gruppenadresse",'widget_observe_led_knx_color_yellow':"Gelb",'widget_observe_led_knx_color_red':"Rot",'widget_observe_led_knx_color':"Farbe",'widget_observe_led_knx_drop_list_label':"Nicht stören",'widget_led_description_action':"LED: \„%name% (KNX %groupAddress%)\“",'widget_toemode_short':"Mode A1",'widget_toemode_long':"Spannungsmodus A1",'widget_toemode_descrline':"Spannungsmodus A1",'widget_toemode_ac':"AC",'widget_toemode_dc':"DC",'widget_dnd_long':"Nicht stören",'widget_presence_long':"Anzeige in Kontaktliste",'widget_grabpic_long':"Bildspeicher",'widget_grabpic_mode_label':"Modus",'widget_grabpic_mode_auto':"Bilder automatisch aufzeichnen",'widget_grabpic_mode_manual':"Bilder nur manuell aufzeichnen",'widget_grabpic_event_label':"Ereignis",'widget_grabpic_event_calls':"Bei angenommenen Rufen",'widget_grabpic_event_missedcalls':"Bei verpassten Rufen",'widget_grabpic_time_label':"Bild nach",'widget_grabpic_time_postfix':"Sekunden aufzeichnen",'widget_grabpic_time_error':"Die Zeit muss zwischen 2 und 30 Sekunden liegen!",'widget_grabpic_option_error':"Eine der Optionen muss ausgewählt werden!",'widget_redir_long':"Rufumleitung",'widget_doormatic_long':"Türmatik",'widget_doormatic_time':"Verzögert öffnen",'widget_doormatic_time_delay_description':"Sekunden nach einem Ruf die Tür öffnen",'widget_doormatic_open_after':"Öffnen nach",'widget_doormatic_open_after_suffix':"Sekunden",'widget_doormatic_open_of':"Öffnen von",'widget_doormatic_all_doors':"Alle Türen",'widget_doormatic_selected_doors':"Bestimmte Türen",'widget_doormatic_doorlist':"Bestimmte Türen",'widget_label_prevent_online':"Gesprächsannahme",'widget_doormatic_options_label':"Türmatik ausführen",'widget_doormatic_option_storeycall':"Bei Etagenrufen",'widget_doormatic_option_online':"Nur während Rufsignalisierung",'widget_doormatic_opener_label':"Schalten",'widget_doormatic_opener':"Türöffner",'widget_doormatic_opener2':"Türöffner 2",'widget_building_automation_url_long':"Gebäudeautomation-URL",'widget_building_automation_url_label':"URL",'widget_videourl_long':"Video-URL",'widget_video_url_label':"Label",'widget_video_url_format':"Format",'widget_video_url_security':"Sicherheit",'widget_video_url_security_description':"HTTP-Basic-Authentifizierung",'widget_video_url_url':"URL",'widget_video_url_participant':"Benutzername",'widget_video_url_password':"Kennwort",'widget_video_url_password_repeat':"Kennwort wiederholen",'widget_video_url_frames':"Frames",'widget_video_url_fps':"HTTP-Basic-Authentifizierung verwenden",'widget_video_url_single_pictures':"Einzelbilder",'widget_video_url_jpeg':"Einzelbild",'widget_video_url_mjpeg':"MJPEG",'widget_light_long':"Türlicht",'widget_light_switch_contact':"Schaltkontakt",'widget_light_switch_timeout':"Zeit",'widget_light_invert':"Invertiert",'widget_light_post_trigger':"Nachtriggern",'widget_light_local_outputs':"Lokaler Schaltkontakt",'widget_light_extern_outputs':"Anderer Schaltkontakt",'widget_light_output':"Ausgang",'widget_light_error_is_empty':"Es muss ein Ausgang ausgewählt sein",'widget_light_options':"Optionen",'widget_light_invert_check':"Schaltausgang invertiert",'widget_light_trigger_check':"Schaltausgang Nachtriggern",'widget_lightcontact_abbr_second':"s",'widget_lightcontact_abbr_minute':"min",'widget_lightcontact_abbr_hour':"h",'widget_door_long':"Türöffner",'widget_door_switch_contact':"Schaltkontakt",'widget_door_switch_timeout':"Zeit",'widget_door_invert':"Invertiert",'widget_door_post_trigger':"Nachtriggern",'widget_door_local_outputs':"lokaler Schaltkontakt",'widget_door_extern_outputs':"anderer Schaltkontakt",'widget_door_output':"Ausgang",'widget_door_canceling_calls':"Türruf",'widget_door_canceling_calls_suffix':"Türöffner beendet Ruf",'widget_door_error_is_empty':"Es muss ein Ausgang ausgewählt sein",'widget_door_options':"Optionen",'widget_door2_long':"Türöffner 2",'widget_door2_switch_contact':"Schaltkontakt",'widget_door2_switch_timeout':"Zeit",'widget_door2_invert':"Invertiert",'widget_door2_invert_check':"Schaltausgang invertiert",'widget_door2_post_trigger':"Nachtriggern",'widget_door2_trigger_check':"Schaltausgang Nachtriggern",'widget_door2_trigger_check_new':"Schaltausgang Nachtriggern",'widget_door2_local_outputs':"lokaler Schaltkontakt",'widget_door2_extern_outputs':"anderer Schaltkontakt",'widget_door2_output':"Ausgang",'widget_door2_canceling_calls':"Türruf",'widget_door2_canceling_calls_suffix':"Türöffner beendet Ruf",'widget_door2_use_for':"Türöffner gilt für",'widget_door2_options':"Optionen",'widget_door2_error_is_empty':"Es muss ein Ausgang ausgewählt sein",'widget_door2_error_list_is_empty':"Die Liste darf nicht leer sein",'widget_camera_long':"Video-Überwachung",'widget_call_contact_long':"Rufkontakt",'widget_call_contact_short':"Rufkontakt",'widget_call_contact_contact':"Schaltpunkt",'widget_call_contact_turn_on':"Einschalten",'widget_call_contact_turn_off':"Ausschalten",'widget_call_contact_options':"Optionen",'widget_call_contact_option_incoming_call':"Eingehende Anrufe",'widget_call_contact_option_outgoing_call':"Ausgehende Anrufe",'widget_call_contact_option_video_call':"Nur bei Türrufen",'widget_call_contact_option_invert':"Invertieren",'widget_call_contact_invite':"Rufbeginn",'widget_call_contact_ring':"Gegenstelle signalisiert",'widget_call_contact_online':"Gespräch beginnt",'widget_call_contact_offline':"Gespräch endet",'widget_call_contact_hold':"Gespräch wird gehalten",'widget_call_contact_manual':"Manuell",'widget_videoprivacy_long':"Video-Mitsehsperre",'widget_videoprivacy_text1':"Während eines Rufs ist das Video für nicht am Ruf beteiligte Personen gesperrt.",'widget_videoprivacy_text2':"Sie können Teilnehmer von der Mitsehsperre ausnehmen, indem Sie über das Rechtesystem Rechte vergeben.",'widget_deviceoutage_long':"Ausfallerkennung",'widget_deviceoutage_text1':"Dieses Gerät wird auf Störung überwacht. Im Falle einer anhaltenden Störung wird ein Alarm ausgelöst.",'widget_deviceoutage_text2':"In den Grundparametern konfigurieren Sie den Alarmkontakt und die Dauer bis zum Auslösen des Alarms.",'widget_onvif_long':"ONVIF-Kamerafreigabe",'widget_onvif_profile_label':"Profilname",'widget_onvif_refresh_tooltip':"Name aktualisieren",'widget_onvif_resolution_label':"Auflösung",'devkeygrp_collapsed':"Mehr Informationen...",'devkeygrp_collapsed_loading':"Laden...",'devkeygrp_no_action':"Keine Aktion",'devkeygrp_button_add_label':"Kachel",'devkeygrp_button_add':"Hinzufügen",'devfunc_label_function':"Funktionen",'devfunc_gridheader_description':"Beschreibung",'devfunc_gridheader_status':"Status",'devfunc_button_edit':"Bearbeiten",'commandgrp_button_new':"Neu",'commandgrp_button_edit':"Bearbeiten",'rule_required_error_notice':"Fehler: Diese Funktion benötigt \"{{widgetName}}\"! Bitte konfigurieren Sie die Funktion zuerst.",'rule_calltarget_add':"Hinzufügen",'rule_calltarget_error_empty':"Es muss ein Eintrag ausgewählt werden!",'rule_calltarget_error_deleted':"Rufziele sind gelöscht!",'rule_integer_error_range':"Wert ist außerhalb des erlaubten Bereichs!",'modload_title':"Modulinstallation",'modload_info_starting':"Starte Modulinstallation",'modload_info_modinfo':"Lade Modulinformationen.",'modload_info_install':"Installiere Moduldateien.",'modload_info_update':"Update bestehende Daten.",'modload_info_finished':"Installation abgeschlossen.",'modload_info_error':"Während der Modulinstallation ist ein Fehler aufgetreten!",'modload_info_license':"Für dieses Modul stehen nicht genügend Lizenzen zur Verfügung!",'moddelete_title':"Modul löschen.",'moddelete_info_starting':"Starte Löschen des Moduls.",'moddelete_info_modinfo':"Lade Modulinformationen.",'moddelete_info_delete':"Entferne Modul.",'moddelete_info_configs':"Entferne konfigurierte Modulkonfiguration.",'moddelete_info_update':"Update bestehende Daten.",'moddelete_info_finished':"Löschen abgeschlossen.",'moddelete_info_error':"Während des löschen des Moduls ist ein Fehler aufgetreten!",'application_window_title':"Siedle Access",'application_edit_adminpass':"Login Kennwort ändern"},'en':{'document_title_login':"Login",'login_title_login':"Siedle Access",'login_subtitle_login':"Settings",'login_label_password':"Password",'login_label_user':"User",'login_label_language':"Language",'login_lang_german':"German",'login_lang_english':"English",'login_label_enter':"Login",'login_link_download':"Downloads / License",'restart_title':"Restart",'restart_headline':"Siedle Access",'restart_message':"The device is restarting, please wait. You will be automatically redirected as soon as the device is available again.",'install_header_title':"Siedle Access - Initial setup",'install_button_next':"Next",'install_button_previous':"Previous",'wizpage_welcome_language':"Dies Seite auf Deutsch zeigen",'wizpage_welcome_button':"Let's go!",'wizpage_welcome_line1':"Thank you for using Siedle Access Professional. You are preparing Access Professional for commissioning.",'wizpage_welcome_line2':"We guide you step by step through the initial setup to make basic system settings. Subsequent changes are possible at any time in the system administration.",'wizpage_welcome_line3':"When the initial setup is completed, all settings made are saved and the system is restarted. After the restart, immediate commissioning of the terminal devices and interfaces is possible via the system administration. You can access the system management via web browser under the IP address and the login data that you will be assigned during the initial setup. Further information can be found in the commissioning instructions for Access Professional.",'wizpage_welcome_line4':"Customers installing Access Professional for trial purposes have 30 days from the completion of the initial setup to configure and fully test their desired system from the range of Access Professional features. Please contact your Access Certified Partner (ACP) if you have any questions about Access Professional.",'wizpage_welcome_line5':"To start the initial setup, please click on \"Let's go!\". To cancel the initial setup, simply close the browser window.",'wizpage_welcome_sidebar':"Welcome",'wizpage_welcome_title':"Welcome to the initial setup!",'wizpage_license_sidebar':"Licence agreements",'wizpage_license_title':"License",'wizpage_license_help':"End User License Agreement (EULA)",'wizpage_license_check':"I (we) accept these licence agreements. I (we) am (are) authorised to accept these licence agreements also on behalf of the using organisation.",'wizpage_network_sidebar':"Network",'wizpage_network_title':"Network",'wizpage_network_help':"Here you configure the network connection of the Access system. Extended configuration options can be found in the System Maintenance area after the initial installation has been completed.",'wizpage_nic_name':"Network interface \"{{name}}\"",'wizpage_nic_steps':"(step {{step}}/{{numsteps}})",'wizpage_dns_title':"DNS-Server",'netcard_label_dns_server':"DNS-Server",'netcard_hint_dns_server_dhcp':"(Automatically by DHCP)",'netcard_hint_dns_server_supersede':"(Manually overwritten)",'netcard_hint_dns_server_opt':"(Optional)",'wizpage_timezone_sidebar':"Time zone",'wizpage_timezone_title':"Time zone",'wizpage_timezone_help':"Here you configure the time zone of the Access system.",'wizpage_timezone_errorempty':"You must select a time zone!",'wizpage_password_sidebar':"Access data",'wizpage_password_title':"Access data",'wizpage_password_help':"Here you set the password for the user account \"admin\". This access data is used to log on to the Access system administration.",'wizpage_password_admin':"Main administrator",'wizpage_password1':"Password",'wizpage_password2':"Repeat",'wizpage_result_sidebar':"Closing",'wizpage_result_title':"Closing",'wizpage_result_help1':"You have made all the necessary settings for the initial setup. Click \"Save and restart\" to save all settings and restart the system.",'wizpage_result_help2':"After the restart, Access Professional is ready for commissioning.",'wizpage_result_help3':"Access to the Access system management is then only possible via the IP address you have configured.",'wizpage_result_button_finish':"Save and restart",'formbase_modify_conflict_notice':"The document has been changed on another computer, or the device has been restartet. Press [\„Undo\“](ID_UNDO) to apply these changes.",'formbase_group_deviceio':"Inputs and outputs",'formbase_group_functions':"Switching and controlling",'formbase_unconfdev_notice':"Editing is locked. Please move this device to the project folder first!",'formbase_reloaded_notice':"The document has been reloaded.",'formbase_savefirst_notice':"Please save the document first!",'formbase_incomplete_notice':"The configuration is not yet complete. Please fill in the [missing fields](ID_VALIDATE) and save the document!",'formbase_label_name':"Name",'formbase_label_description':"Description",'formbase_label_device_type':"Device type",'formbase_label_ipaddr':"IP address",'formbase_label_port':"Port",'formbase_value_dtmf_from_door_only':"Allow DTMF only for calls from the door to the gateway",'formbase_group_devicekeys':"Key configuration",'dashboard_info_hardwareid':"Hardware ID",'dashboard_info_version':"System version",'dashboard_info_numberdevices':"Device count",'dashboard_info_demotime':"End of demo period",'dashboard_info_supportcontract':"End of maintenance contract",'dashboard_info_warning_expired':"Expired on",'dashboard_info_susstatus':"Siedle App Service",'dashboard_susinfo_notused':"Not used",'dashboard_susinfo_connecting':"Connection is being established",'dashboard_susinfo_online':"Connected",'dashboard_susinfo_error':"Connection error",'dashboard_toast_nonetworkconfigured':"You must select a network interface in the basic parameters! [Click here to change basic parameters.](ID_BASICPARAM)",'dashboard_group_messages':"Messages",'dashboard_group_functions':"Functions",'dashboard_group_information':"Information",'dashboard_grid_column_entid':"Object",'dashboard_grid_column_msgid':"Message",'dashboard_grid_placeholder':"No messages available",'dashboard_menu_general':"General",'dashboard_menu_siedledevices':"Siedle devices",'dashboard_menu_server':"Server",'dashboard_menu_supportinfo':"Support informations",'dashboard_menu_search':"Search",'dashboard_menu_exit':"Logoff",'dashboard_menu_device_restartall':"Device restart",'dashboard_menu_device_restartneeded':"Device restart (if necessary)",'dashboard_menu_restart_service':"Restart services",'dashboard_menu_reboot':"Restart hardware",'presence_status_offline':"Offline",'presence_status_devstart':"Device starts",'presence_status_devready':"Waiting for Sip",'presence_status_idle':"Ready",'presence_status_dnd':"Do not disturb",'presence_status_call':"Calling",'presence_status_ring':"Ringing",'presence_status_disc':"Disconnecting",'presence_status_busy':"Busy",'presence_status_trying':"Waiting",'presence_status_error':"Error",'presence_header_last_action':"Last message",'presence_document_title':"User status",'presence_filter_label':"Filter",'presence_filter_check_online':"Online",'presence_filter_check_offline':"Offline",'presence_header_object_name':"Object",'presence_header_callnumber':"Call number",'presence_header_status':"Status",'presence_header_ip':"IP address",'presence_header_remote_number':"Destination number",'basicparam_box_title_times_door':"Durations (door)",'basicparam_box_title_times_general':"Durations (General)",'basicparam_box_title_times_lists':"Durations (lists)",'basicparam_label_lists_time_grabpic':"Video memory, max.",'basicparam_box_deviceoutage':"Device outage detection",'basicparam_label_deviceoutage_time':"Report fault after",'basicparam_postlabel_deviceoutage_time':"Min.",'basicparam_label_deviceoutage_contact':"Switching contact",'basicparam_error_outage_time':"The outage detection time must be between 1 and 10 minutes!",'basicparam_document_title':"Basic parameters",'basicparam_label_syslang':"System language",'basicparam_label_netcardgrid':"Network adapter",'basicparam_netcardgrid_header_name':"Name",'basicparam_netcardgrid_header_ipv4':"IPv4",'basicparam_netcardgrid_header_ipv6':"IPv6",'basicparam_netcardgrid_header_client':"Client",'basicparam_netcardgrid_header_phone':"Telephony",'basicparam_label_door_ring_timeout':"Max. ring duration",'basicparam_label_door_call_timeout':"Max. call duration",'basicparam_label_latevideo_timeout':"Video follow up",'basicparam_label_ring_timeout':"Max. ring duration",'basicparam_label_call_timeout':"Max. call duration",'basicparam_label_lists_time_cdr':"Call list, max.",'basicparam_label_lists_time_voicemail':"Voicemail, max.",'basicparam_postlabel_seconds':"Sec.",'basicparam_postlabel_minutes':"Min.",'basicparam_postlabel_days':"Days",'basicparam_error_door_ring_timeout':"The ring duration must be between 5 and 60 seconds!",'basicparam_error_door_call_timeout':"The call duration must be between 1 and 1440 minutes!",'basicparam_error_lateviedeo_timeout':"The video lag must be between 0 and 15 seconds!",'basicparam_error_ring_timeout':"The ring duration must be between 5 and 120 seconds!",'basicparam_error_call_timeout':"The call duration must be between 10 and 1440 minutes!",'basicparam_error_grabpic_time':"The time for image storage must be between 1 and 120 days!",'basicparam_error_cdr_time':"The time for the call lists must be between 1 and 120 days!",'basicparam_error_voicemail_time':"The time for the voicemail must be between 1 and 120 days!",'basicparam_toast_unsaved_notice':"You have to restart the services for the changes to become active. [Click here to restart services](ID_SAVE_UNSAVED)!",'logfile_document_title':"Log",'logfile_header_created':"Time",'logfile_header_entid':"Object",'logfile_header_message':"Message",'logfile_header_admin':"Admin",'callnumberplan_document_title':"Call number plans",'callnumberplan_label_permissions':"Call rules",'callnumberplan_label_profiles':"Call profiles",'callnumberplan_profile_header_name':"Profiles",'callnumberplan_permission_header_rule':"Rule",'callnumberplan_permission_header_allowed':"Permitted",'callnumberplan_buttonlabel_add':"New",'callnumberplan_buttonlabel_edit':"Edit",'callnumberplan_buttonlabel_delete':"Delete",'backup_group_backup':"Backup",'backup_restore_file_failed':"Error: The file \"{file}\" is not an Access backup file!",'backup_restore_file_corrupted':"Error: The file \"{file}\" does not contain backup data!",'backup_restore_upload':"Drag your backup file here or click for selection",'backup_document_title':"Backup / Restore",'backup_group_restore':"Restore",'backup_backup_label':"Backup file",'backup_backup_button':"Create",'backup_restore_label':"Upload file",'backup_restore_button':"Restore",'backup_restore_button_exec':"Will be executed...",'backup_restore_help':"The system is shut down and the uploaded backup is activated. Your existing installation will be irretrievably lost!",'backup_restore_file':"Uploaded file",'backup_restore_file_placeholder':"No file available",'search_document_title':"Search result",'search_header_entid':"Object",'search_header_message':"Message",'search_result_message':"Found in:",'search_result_name':"Name",'search_result_comment':"Description",'search_result_statepoint':"Functions",'search_result_keys':"Buttons/Outputs",'search_result_callnumber':"Call number",'search_result_macaddr':"MAC address",'search_result_prefix':"Prefix",'search_result_externcallnr':"Phone number (external)",'search_result_account':"User",'license_file_upload_succeeded':"\"{file}\" has been successfully uploaded and activated!",'license_file_upload_failed':"\"{file}\" could not be uploaded!",'license_file_upload_invalid':"\"{file}\" cannot be imported. This is not a valid licence file!",'license_file_upload_invalidversion':"\"{file}\" cannot be imported. This is a licence from an earlier version!",'license_demoperiod_label':"End of demo period",'license_supportcontract_label':"End of maintenance contract",'license_gridheader_engaged':"Used",'license_toast_expired':"Your demo version has expired. Please upload a valid licence.",'license_toast_invalid':"Your licence is invalid. Please upload a valid licence.",'license_toast_invalid_version':"Your licence belongs to an earlier version. Please install a more recent licence.",'license_upload':"Drop licence file here or click for selection",'license_description_apr':"Access user licence",'license_description_knx':"Access licence KNX connection",'license_description_alfp':"Access licence external device panel",'license_description_alsa':"Access licence Siedle APP",'license_description_alt':"Access licence telephony connection",'license_description_asc':"Access Software Concierge",'license_description_asht':"Access Software Inhouse Telephone",'license_description_sipaudio':"Access licence for a non-Siedle device",'license_description_alfc':"Access license IP camera",'license_description_alfm':"Access license function module",'licenses_document_title':"Licences",'licenses_download_tooltip':"Export Server ID",'licenses_download_label':"Export",'license_grid_label':"Licences",'license_gridheader_type':"Licence type",'license_gridheader_description':"Description",'license_gridheader_available':"Available",'license_box_upload':"Licence",'license_upload_label':"Upload",'rules_document_title':"Function modules",'rules_box_upload':"Modules",'rule_upload_label':"Upload",'rule_upload_text':"Drop rule file here or click for selection",'rule_grid_label':"Installed",'rule_gridheader_description':"Description",'rule_gridheader_version':"Version",'rule_gridheader_type':"Type",'rule_type_core':"Basic system",'rule_type_module':"Module",'rule_button_delete':"Delete",'rule_error_update':"An update is required for the module!",'rule_error_version':"The module requires a server with version {{version}} or higher!",'systemtime_ntpstat_status_sync':"Synchronized to {{ntpip}} at stratum {{stratum}}.",'systemtime_ntpstat_info_updates':"Polling server every {{polling}} seconds. Time correct to within {{tmcorrect}}ms.",'systemtime_timemode_manual':"Use hardware clock",'systemtime_button_add':"New",'systemtime_document_title':"System time",'systemtime_srvtime_label':"Server time",'systemtime_clienttime_label':"Browser time",'systemtime_region_label':"Region",'systemtime_zone_label':"Time zone",'systemtime_timemode_label':"Mode",'systemtime_timemode_automatic':"Automatic updates with time server",'systemtime_timeserver_box':"Time synchronization",'systemtime_ntplist_label':"Time server",'systemtime_button_edit':"Edit",'systemtime_ntpstat_label':"NTP status",'systemtime_ntpstat_status_unsync':"Not synchronized",'systemtime_ntpstat_status_unknown':"No connection to the NTP server",'systemtime_timesend_label':"Set time",'systemtime_timesend_button':"Set current browser time",'systemtime_timesend_help':"The time is set regardless of the time zone. The changes are effective immediately.",'systemtime_toast_unsaved_notice':"You must restart the hardware for the changes to become active. [Click here to restart hardware](ID_SAVE_UNSAVED)!",'network_dns_box':"DNS (Domain Name System)",'network_netcard_title_prefix':"Network adapter",'network_toast_expert_mode_notice':"Your system files have been changed manually. You can continue working or discard the changes. [Click here to discard changes](ID_RESYNC)",'network_toast_unsaved_notice':"You have to restart the services for the changes to become active. [Click here to restart services](ID_SAVE_UNSAVED)!",'dns_config_server':"DNS-Server",'dns_config_add':"New",'dns_config_edit':"Edit",'dns_config_delete':"Delete",'dns_config_modus':"Obtain DNS server address automatically",'dns_config_label_modus':"Mode",'dns_config_supersede_list_empty':"A DNS server address must be specified!",'dns_config_dhcp_dns_unknown':"Unknown",'dns_config_dns_noconfig':"No DNS server configured",'netcard_label_name':"Name",'netcard_label_system_name':"System name",'netcard_label_type':"IP configuration",'netcard_label_ip':"IP address",'netcard_label_net_mask':"Subnet mask",'netcard_label_dhcp':"DHCP server",'netcard_label_default_gateway':"Default gateway",'netcard_label_dns_mode':"DNS-Server",'netcard_label_button_options':"DHCP",'netcard_static':"Static IP",'netcard_dhcp':"IP from DHCP",'netcard_no_config':"Not configured",'netcard_dhcp_server':"DHCP server",'netcard_is_dhcp_active':"For this network adapter",'netcard_dhcp_pool_start':"IP pool, start",'netcard_dhcp_pool_end':"IP pool, end",'netcard_button_options':"Options",'netcard_warning_options_changed':"Your DHCP options have been adjusted automatically. Please check the settings!",'netcard_pool_error_range':"The start of the IP pool must be smaller than the end!",'netcard_pool_error_ip':"The IP address must not be in the range of the DHCP IP pool!",'netcard_ip_as_access':"The IP address is set as Access Server IP!",'change_ip_dialog_title_edit':"Change DNS address",'change_ip_dialog_title_new':"New DNS addresss",'change_ip_dialog_label_ip':"IP address",'ip_change_error_empty':"DNS address must not be empty!",'ip_change_error_not_valid':"Invalid IP address!",'dhcp_lease_delete':"Delete",'dhcp_reserve_new':"New",'dhcp_lease_status_new':"New",'dhcp_option_active_yes':"Yes",'dhcp_option_active_no':"No",'dhcp_optionheader_option':"Option name (ID)",'dhcp_menu_general':"General",'dhcp_menu_options':"Options",'dhcp_menu_leases':"Leases",'dhcp_menu_reservations':"Reservations",'dhcp_ipaddr':"IP address",'dhcp_subnetmask':"Subnet mask",'dhcp_scope_start':"IP pool, start",'dhcp_scope_end':"IP pool, end",'dhcp_scope_error_range':"IP address not in network range!",'dhcp_optionheader_value':"Value",'dhcp_optionheader_active':"Active",'dhcp_option_enabled':"Status",'dhcp_option_enabled_check':"Activate option",'dhcp_option_name':"Option",'dhcp_option_value':"Value",'dhcp_option_autovalue_tooltip':"Standard value",'dhcp_leaseheader_macaddr':"MAC address",'dhcp_leaseheader_ipaddr':"IP address",'dhcp_leaseheader_hostname':"Hostname",'dhcp_leaseheader_date':"Expiry date",'dhcp_lease_action_label':"Lease",'dhcp_lease_delete_undo':"Undo",'dhcp_lease_reserve':"Reserve",'dhcp_lease_status_active':"Active",'dhcp_lease_status_deleted':"Deleted",'dhcp_lease_status_edit':"Modified",'dhcp_reserve_action_label':"Reservation",'dhcp_reserve_mac_label':"MAC address",'dhcp_reserve_ip_label':"IP address",'dhcp_reserve_delete':"Delete",'dhcp_reserve_delete_undo':"Undo",'dhcp_reserve_edit':"Edit",'dhcp_reserve_error_duplicatemac':"MAC address already exists!",'dhcp_reserve_error_duplicateip':"IP address already exists!",'dhcp_reserve_error_ipnotinrange':"IP address not in scope!",'dhcp_reserve_error_ipnetcard':"This IP address is used by the interface!",'dhcp_reserve_cancel_tooltip':"Cancel",'dhcp_reserve_apply_tooltip':"Apply",'ruledbg_title':"Rules Event Viewer",'ruledbg_label_instances':"Active Rule Instances",'ruledbg_label_currentinst':"Current Rule Instance",'ruledbg_label_details':"Details",'ruledbg_label_events':"Events",'ruledbg_label_watch':"Watch",'api_document_title':"Interfaces",'api_group_onvif':"ONVIF camera share",'api_label_onvif_enable':"Video",'api_label_onvif_enable_text':"Share door cameras via ONVIF",'api_label_onvif_user':"User name",'folder_document_title':"Access Gateway",'folder_document_changed_notice':"The document has been changed on another computer. Press [\"Discard\"](ID_UNDO) to apply these changes.",'folder_create_type':"Folder",'apartment_document_title':"Indoor device",'apartment_create_type_afs':"Siedle AFS",'apartment_create_type_afsv':"Siedle AFSV",'apartment_create_type_ahf':"Siedle AHF 870-0",'apartment_create_type_ahfv':"Siedle AHFV 870-0",'apartment_create_type_aht':"Siedle AHT 870-0",'apartment_create_type_ahtv':"Siedle AHTV 870-0",'tkphone_document_title':"PABX phone",'tkphone_create_type':"PABX phone",'tkphone_label_gateway':"PABX gateway",'avp_document_title':"Indoor device",'avp_create_type':"Siedle AVP 870-0",'door_document_title':"Door",'door_create_type':"ATLC 670-0 (ATLC V1)",'tkgw_label_incoming':"Calls (incoming)",'tkgw_document_title':"PABX gateway",'tkgw_create_type':"PABX gateway",'tkgw_incoming_forward':"Forward to",'tkgw_label_prefix':"Dial prefix",'tkgw_box_title_gateway':"PABX gateway",'tkgw_label_registrar':"Registrar",'tkgw_label_account':"Account",'tkgw_label_password':"Password",'tkgw_label_framesize':"Frame size",'tkgw_label_transport':"Transport",'tkgw_label_port':"Port",'tkgw_framesize_ms20':"20ms",'tkgw_framesize_ms40':"40ms",'tkgw_transport_udp':"UDP",'tkgw_transport_tcp':"TCP",'tkgw_transport_tls':"TLS",'tkgw_error_port_empty':"Must not be empty or zero!",'tkgw_framesize_auto':"Automatic",'trunk_label_remote_prefix':"Remote prefix",'trunk_document_title':"SIP trunk",'trunk_create_type':"SIP trunk",'trunk_box_title_trunk':"SIP trunk",'trunk_error_license':"Not enough licences available!",'trunk_label_prefix':"Dial prefix",'trunk_label_lines':"Lines",'trunk_label_ipaddress':"IP address",'trunk_label_framesize':"Frame size",'group_callreject_label':"Reject a call",'group_voicemail_check':"Allow from group participants on group call",'group_document_title':"Group",'group_create_type':"Group",'group_callreject_check':"Reject leads to the end of the group call",'group_missedcall_label':"Answered calls",'group_voicemail_label':"Voicemail",'group_missedcall_check':"Enter as missed call for the other participants",'group_member_box':"Group",'group_member_label':"Members",'group_member_placeholder':"No members",'group_member_header_name':"Name",'group_member_header_callnumber':"Call number",'group_member_button_add':"Add",'group_member_button_tooltip':"Adds a new member",'sipaudio_document_title':"SIP phone",'sipaudio_create_type':"SIP phone",'susapp_document_title':"Siedle APP",'susapp_create_type':"Siedle APP",'susapp_qrcode_label':"Registration",'susapp_qrvalid_label':"Valid until",'susapp_susstatus_label':"Status",'susapp_susstatus_scanned':"Registered",'susapp_susstatus_request_qr':"QR code is retrieved",'susapp_susstatus_expired':"QR code expired",'susapp_refresh_expired_tooltip':"Retrieve new QR code",'client_document_title':"ASC",'client_create_type':"Siedle ASC 170",'client_device_type':"ASC_170",'asht_document_title':"ASHT",'asht_create_type':"Siedle ASHT 170",'asht_device_type':"ASHT_170",'jungpanel_document_title':"JUNG Panel",'jungpanel_create_type':"JUNG Panel",'addrbook_document_title':"Directory",'addrbook_create_type':"Directory",'addrbook_box_grid':"Directory",'addrbook_header_name':"Name",'addrbook_header_callnr':"Call number",'addrbook_header_comment':"Comment",'addrbook_button_add':"New",'addrbook_button_edit':"Edit",'admin_document_title':"Admin",'admin_create_type':"Administrator",'knxgw_header_addr_receive':"Feedback object",'knxgw_document_title':"KNX Gateway",'knxgw_create_type':"KNX Gateway",'knxgw_box_grid':"KNX",'knxgw_label_grid':"Group addresses",'knxgw_header_name':"Label",'knxgw_header_addr_send':"Group address",'knxgw_button_add':"New",'knxgw_button_edit':"Edit",'timeline_action_on':"Actions for \"On\"",'timeline_action_off':"Actions for \"Off\"",'timeline_document_title':"Time profile",'timeline_create_type':"Time profile",'timeline_status_label':"Label (Status)",'timeline_automatic_label':"Label (automatic mode)",'timeline_box_time':"Time profile",'timeline_monday':"Monday",'timeline_tuesday':"Tuesday",'timeline_wednesday':"Wednesday",'timeline_thursday':"Thursday",'timeline_friday':"Friday",'timeline_staturday':"Saturday",'timeline_sunday':"Sunday",'timeline_detail_start':"Start",'timeline_detail_end':"Stop",'timeline_box_action':"Actions",'ipcam_document_title':"IP camera",'ipcam_create_type':"IP camera",'ipcam_label_account':"User name",'ipcam_label_auth':"Authorization",'ipcam_check_auth':"Camera requires user name and password",'ipcam_streams_placeholder':"No data obtained",'ipcam_grid_name':"Name",'ipcam_grid_resultion':"Resolution",'ipcam_grid_codec':"Codec",'ipcam_group_streams':"Camera information",'ipcam_label_streams':"Profiles",'const_form_callnumberplan':"Call numbering plans",'const_form_tooltip_permission':"Set permissions",'const_error_statepoint_permission':"No permissions to access the object!",'const_form_licenses':"Licences",'const_switchtype_toggle':"Toggle",'const_switchtype_on':"On",'const_switchtype_off':"Off",'const_menu_dashboard':"",'const_menu_basicparam':"",'const_menu_network':"Network",'const_menu_pabx':"PBX gateway",'const_menu_password':"Password",'const_menu_supportinfo':"Support informations",'const_menu_exit':"Logoff",'const_menu_device_restart':"Device restart",'const_menu_devices':"Device list",'const_menu_restart_service':"Restart services",'const_menu_restart_service_short':"Restart",'const_menu_reboot':"Restart hardware",'const_menu_help':"Help",'const_menu_save':"Save",'const_menu_undo':"Discard",'const_menu_construct':"Under Construction",'const_box_title_general':"General",'const_dialog_button_save':"Save",'const_dialog_button_apply':"Apply",'const_dialog_button_cancel':"Cancel",'const_toast_savefirst':"Please save the form first!",'const_error_novalidmac':"Not a valid Mac address!",'const_error_empty':"A value must be set!",'const_common_loading':"Loading data...",'const_common_loading_short':"Loading...",'const_common_search_placeholder':"Search",'const_common_empty_placeholder':"Please select",'const_switchtype_trigger':"Flush",'const_category_new_door':"Door station",'const_category_new_indoor':"Indoor units",'const_category_new_hardware':"External device",'const_category_new_software':"Software client",'const_form_dashboard':"Dashboard",'const_form_basicparam':"Basic parameters",'const_form_presence':"User status",'const_form_backup':"Backup / Restore",'const_form_systemtime':"System time",'const_form_network':"Network",'const_form_logfile':"Log",'const_form_test_page':"Test_Page",'const_form_rules':"Function modules",'const_form_label_help':"Help",'const_form_tooltip_help':"Shows help page",'const_form_tooltip_delete':"Delete",'const_form_label_add':"New",'const_form_tooltip_add':"Creates a new object",'const_form_label_save':"Save",'const_form_tooltip_save':"Saves the current document",'const_form_label_undo':"Discard",'const_form_tooltip_undo':"Discards changes made",'const_form_tooltip_lists':"Shows the lists of the device",'const_form_tooltip_devicerestart':"Restarts the device",'const_form_label_refresh':"Refresh",'const_form_tooltip_refresh':"Reload",'const_messagebox_delete_text':"This object will be permanently deleted. Would you like to continue?",'const_messagebox_delete_label':"Delete",'const_messagebox_delete_refused':"This folder contains the main administrator, which must not be deleted!",'const_time_manual':"Manual",'const_framesize_auto':"Automatic",'const_framesize_ms20':"20ms",'const_framesize_ms40':"40ms",'aui_lang_dashboard':"Dashboard",'aui_lang_basicparameters':"Basic parameters",'aui_lang_callprofiles':"Numbering plans",'aui_lang_userstatus':"User status",'aui_lang_search':"Search",'aui_lang_log':"Log",'aui_lang_unconfdev':"Unconfigured devices",'aui_lang_confdev':"Project",'aui_lang_confdevcomment':"Project folder for the Access system",'aui_lang_licence':"Licenses",'aui_lang_networksettings':"Network",'aui_lang_tile':"Function",'const_dialog_button_ok':"Ok",'const_dialog_button_close':"Close",'const_dialog_button_reload':"Refresh",'const_error_statepoint_deleted':"The object no longer exists!",'const_error_device_deleted':"The device is deleted!",'const_error_callnumber_changed':"The phone number used has changed!",'const_deleted_item':"Deleted entry",'const_deleted_item_short':"Deleted",'aui_widget_grabpic':"Video Memory",'aui_widget_grabpicmenu':"Video Memory Menu",'aui_widget_buildingAutomationUrl':"Building automation",'aui_widget_callcontact':"Call contact",'aui_widget_camera':"Video",'aui_widget_dnd':"Do not disturb",'aui_widget_door':"Door opener",'aui_widget_door2':"Door opener 2",'aui_widget_doormatic':"Doormatic",'aui_widget_externcall':"Call, external",'aui_widget_ldlight':"Door light",'aui_widget_ldopener':"Door opener",'aui_widget_light':"Door light",'aui_widget_lightcontact':"Door light",'aui_widget_openercontact':"Door opener",'aui_widget_presence':"Presence",'aui_widget_redir':"Call redirection",'aui_widget_ringtone':"Ring tones",'aui_lang_ringdoor':"Door call",'aui_widget_call':"Call",'aui_widget_setredir':"Call redirection",'aui_widget_setsp':"Switching",'aui_widget_setspknx':"Send KNX",'aui_widget_state_notifying_contact':"Notifying state",'aui_widget_storeycall':"Storey call",'aui_widget_switchcontact':"Output",'aui_widget_timeline':"Status",'aui_widget_automatic':"Automatic",'aui_widget_timeswitchcontact':"Output",'aui_widget_video':"Video URL",'STORE_CREATE':"New object '{{name}}' was created.",'STORE_UPDATE':"Object '{{name}}' was edited.",'STORE_DELETE':"Object '{{name}}' was deleted.",'SYSTEM_RESTARTED':"Restart of the Access services was requested.",'SYSTEM_REBOOT':"Restart of the Access hardware was requested.",'REBOOT_ALL':"A restart was requested for all devices.",'REBOOT_DEVICE':"A restart was requested for the device '{{name}}'.",'REBOOT_NEEDED':"A restart of all devices, if necessary, was requested.",'ENTITY_ERROR':"There are errors in this object, please open the form for details!",'DEVRESTART_NEEDED':"Device restart required!",'messagebox_cancel':"Cancel",'messagebox_save':"Save",'messagebox_ok':"Ok",'password_dialog_title':"Change password",'password_label_password':"Password",'password_label_repeated':"Repeat",'password_placeholder_password':"New password",'password_placeholder_repeated':"Repeat",'change_password_error_empty':"Password must not be empty!",'change_password_error_different':"Passwords are not identical!",'reboot_agw_really_restart_agw':"Service restart writes all operating system settings and restarts the services. Do you like to restart?",'reboot_agw_cancel':"Cancel",'reboot_agw_ok':"Restart",'reboot_agw_reboot':"Your hardware will now reboot. Do you want to perform the reboot?",'formnew_please_select':"Please select the object to be created",'formnew_no_license':"(no licence available)",'formnew_title':"New object",'devicelist_title':"Device lists",'devicelist_category_contact':"Contacts",'devicelist_category_camera':"Cameras",'devicelist_category_switch':"Switching list",'permissions_dialog_title':"Permissions for \"{{name}}\"",'permissions_label_permissionlist':"Permissions",'permissions_label_who':"Who",'permissions_who_placeholder':"Add objects here to assign special permissions.",'permissions_inherit_objects':"Permissions to be inherited (child objects)",'permissions_button_add':"Add",'permissions_button_delete':"Delete",'permissions_rightgrid_text':"Description",'permissions_rightgrid_read':"Read",'permissions_rightgrid_write':"Execute",'permissions_rightgrid_cr':"Inherited",'permissions_rightgrid_cr_tooltip':"Inherited from \"{name}\"",'name_callprofile_dialog_title':"Call profile",'name_callprofile_label_name':"Call profile name",'name_callprofile_error_empty':"The call profile must not be empty!",'permission_callprofile_dialog_title':"Call rule",'permission_callprofile_label_rule':"Permission mask",'permission_callprofile_label_forbid':"Deny",'permission_callprofile_error_empty':"The permission mask must not be empty!",'permission_callprofile_error_wildcard_pos':"* can only be used at the end of the mask!",'address_cancel':"Cancel",'address_save':"Save",'address_title':"Address book entry",'address_label_label':"Name",'address_label_callnumber':"Call number",'address_label_comment':"Comment",'address_error_nointernal':"No internal participants are allowed in the address book!",'knxentry_title':"KNX switching point",'knxentry_label_label':"Label",'knxentry_label_send':"Group address",'knxentry_label_receive':"Feedback object",'knxentry_postfix_receive':"(optional)",'ntpentry_title':"NTP server",'ntpentry_cancel':"Cancel",'ntpentry_save':"Save",'ntpentry_ipaddress_label':"IP address / host",'macdialog_grabpic_check':"Video memory of the user",'macdialog_title':"Changing devices and users",'macdialog_help':"Enter the new MAC address of the device you received as a replacement device.",'macdialog_warning_online':"The device must not be logged on to the server!",'macdialog_label':"MAC address",'macdialog_susreg_label':"Registration",'macdialog_susreg_check':"Logging off the user's mobile phone",'macdialog_userparam_label':"Reset",'macdialog_userparam_check':"User settings",'macdialog_calllist_check':"Call lists of the user",'devkey_tooltip_keysingle':"Single-click",'devkey_tooltip_keydouble':"Double-click",'devkey_tooltip_keypos':"Press key",'devkey_tooltip_keyneg':"Release key",'devkey_tooltip_led':"LED",'change_keytone_dialogcancel':"Cancel",'change_keytone_dialogsave':"Save",'change_keytone_dialogtitle':"Call acknowledgement tone",'change_keytone_dialog_call_target':"Call destination",'change_keytone_dialog_callnumber':"Audio",'change_keytone_dialog_tone_must_selected':"A call acknowledgement tone must be selected",'scandevice_title':"Device search",'scandevice_check_showunconfigured':"Show only selectable devices",'scandevice_status_search_unknown':"Search for Unknown Devices...",'scandevice_status_searchknx':"Search for KNX IP gateways...",'scandevice_status_searchipcam':"Search for IP Cameras...",'scandevice_status_notfound':"No unconfigured units found!",'scandevice_grid_name':"Name",'scandevice_grid_ip':"IP address",'ipcam_title':"IP camera test",'ipcam_button_login':"Test",'ipcam_label_ipaddress':"IP address",'page_header_title':"Siedle Access - Administration",'page_header_search_placeholder':"Search",'page_header_warning_websocket':"The update speed from the server is reduced!",'page_menu_dashboard':"Show messages",'page_menu_settings':"Settings",'page_menu_exit':"Logoff",'accordion_section_general':"System maintenance",'accordion_section_devtree':"Users",'const_error_novalidip':"Not a valid IP address!",'const_error_ipempty':"IP address must not be empty!",'const_error_novalidmask':"No valid subnet mask!",'const_error_nostring':"Internal error",'const_error_notemptystring':"A value must be entered!",'const_error_novalidport':"No valid port!",'const_error_portempty':"The port must not be empty!",'const_error_novalidiporhost':"Not a valid IP address or host name!",'widget_base_avptile_label':"Tile label",'widget_base_avptile_icon':"Tile icon",'device_config_description_linkedled':"with LED",'keytitle_apartment_key':"Button {key}",'keytitle_afsv_key1':"Button door opener",'keytitle_afsv_key2':"Button I",'keytitle_afsv_key3':"Button II",'keytitle_afsv_key4':"Button ringtone off",'keytitle_afs_key1':"Button door opener",'keytitle_afs_key2':"Button I",'keytitle_afs_key3':"Button ringtone off",'keytitle_ahtf_a1':"Output A1",'keytitle_ahtf_e1':"Input E1",'keytitle_ahtf_a2':"Output A2",'keytitle_ahtf_e2':"Input E2",'keytitle_timeline_on':"Actions for \"On\"",'keytitle_timeline_off':"Actions for \"Off\"",'triggername_input_generic':"Input",'triggername_output_generic':"Output",'triggername_tile':"Touch",'triggername_callkey':"Click",'triggername_single':"Single click",'triggername_double':"Double click",'triggername_pos':"Press key",'triggername_neg':"Release key",'triggername_led':"LED",'keytitle_brma':"Module {module}, button {key}",'keytitle_btm':"Module {module}, button {key}",'keytitle_atlc_a1_toemode':"Voltage mode A1",'keytitle_atlc_e':"Input E",'keytitle_atlc_a1':"Output A1",'keytitle_atlc_a2':"Output A2",'keytitle_atlc_a3':"Output A3",'keytitle_atlce1_e1':"Input module 1/E1",'keytitle_atlce1_e2':"Input module 1/E2",'keytitle_atlce1_a1':"Output module 1/A1",'keytitle_atlce1_a2':"Output module 1/A2",'keytitle_atlce1_a3':"Output module 1/A3",'keytitle_atlce1_a4':"Output module 1/A4",'keytitle_atlce2_e1':"Input module 2/E1",'keytitle_atlce2_e2':"Input module 2/E2",'keytitle_atlce2_a1':"Output module 2/A1",'keytitle_atlce2_a2':"Output module 2/A2",'keytitle_atlce2_a3':"Output module 2/A3",'keytitle_atlce2_a4':"Output module 2/A4",'keytitle_atlce3_e1':"Input module 3/E1",'keytitle_atlce3_e2':"Input module 3/E2",'keytitle_atlce3_a1':"Output module 3/A1",'keytitle_atlce3_a2':"Output module 3/A2",'keytitle_atlce3_a3':"Output module 3/A3",'keytitle_atlce3_a4':"Output module 3/A4",'keytitle_avp_key1':"Button door opener",'keytitle_avp_tile':"Tile \"{name}\"",'keytitle_atlm_l':"Button door light",'permission_label_camera':"Show camera",'permission_label_addrbook':"Directory",'permission_label_dnd':"Do not disturb",'permission_label_door':"Door opener",'permission_label_door2':"Door opener 2",'permission_label_doormatic':"Doormatic",'permission_label_light':"Door light",'permission_label_presence':"User status",'permission_label_redir':"Call redirection",'permission_label_videoprivacy':"Video privacy",'widget_empty_long':"Template (for widget)",'widget_empty_short':"Template",'widget_empty_label_option1':"Option 1",'widget_empty_label_option2':"Option 2",'widget_empty_check_option1':"Shall be active now and always.",'widget_empty_check_option2':"It's also a possibility.",'widget_none_short':"No function",'widget_none_long':"No function",'widget_none_description':"No action",'widget_none_noparam':"No parameters",'control_calltarget_error_deleted':"The selected call destination has been deleted!",'control_calltarget_error_empty':"A call destination must be selected!",'control_calltarget_callnumber_changed':"The phone number has changed!",'control_statepoint_error_empty':"A switching point must be selected!",'control_entitylist_error_deleted':"The selected entry is deleted!",'control_entitylist_error_empty':"An entry must be selected!",'labelctrl_label':"Label",'permcheck_tooltip':"Note: No external permissions have been set for this object yet.",'macaddr_error_exists':"This MAC address is already assigned!",'macaddr_label':"MAC address",'callwaiting_label':"Call waiting",'callwaiting_value':"Signal calls even during an active call",'control_callnumber_error_empty':"Call number must not be empty",'control_callnumber_error_duplicated':"The number is already in use",'control_callnumber_error_prefix':"The call number must not begin with a gateway or trunk prefix!",'control_callnumber_label':"Call number",'control_callprefix_label':"Prefix",'control_callprefix_error_empty':"The prefix must not be empty!",'control_callprefix_error_duplicated':"The prefix is already taken!",'control_callprefix_error_callnumber':"The prefix overlaps with a phone number!",'addrbk_label_addressbook':"Directory",'devicevolume_label_ringvolume':"Ringtone volume",'devicevolume_label_callvolume':"Speech volume",'devicevolume_value_out':"Volume {value}",'devicevolume_value_in':"Sensitivity {value}",'devicevolume_label_micvolume':"Microphone",'formbase_label_showclock':"Time",'formbase_value_showclock':"Display time on the end device",'mounttype_label':"Installation type",'mounttype_value_wall':"Wall mounted",'mounttype_value_desk':"Table mounted",'extern_callnumber_label':"External call number",'control_callprofile_error_deleted':"The selected call profile has been deleted!",'control_callprofile_label':"Call profile",'dtmf_gateway_outgoing_only':"Allow DTMF only for calls from the door to the gateway",'dtmf_label_door':"Door opener (DTMF)",'dtmf_label_light':"Door light (DTMF)",'control_password_label':"Password",'control_password_label2':"Password, repetition",'control_account_label_account':"Username",'control_account_error_empty':"The username must not be empty",'control_account_error_duplicated':"The username is already taken",'keytone_value_simple':"Simple",'keytone_droplist_label':"Call acknowledgement",'keytone_value_none':"None",'keytone_value_individual':"Individual",'keytone_control_validate_empty_list':"The list must not be empty",'zam_options_label':"ZAM acknowledgement tone at",'zam_option_oncall':"Call",'zam_option_onopener':"Door opener",'zam_option_offhook':"Call acceptance",'zam_option_failedcon':"Failed connection",'control_ringtone_label':"Ringtones",'control_ringtone_individual':"Set individual ringtones",'control_ringtone_label_doorcall':"Ringtone door call",'control_ringtone_label_storeycall':"Ringtone storey call",'control_ringtone_label_default':"Ringtone standard",'control_doorcamera_label':"Door camera",'control_doorcamera_value_none':"No camera installed",'control_doorcamera_value_internal':"Internal camera",'widget_call_short':"Call",'widget_call_long':"Call",'widget_call_label':"Call destination",'widget_call_descriptionline':"Call: \"{target}\" ({callnumber})",'widget_call_error_descriptionline':"Call to deleted call destination",'widget_doorcall_long':"Door call",'widget_doorcall_short':"Door call",'widget_doorcall_label':"Call destination",'widget_doorcall_descriptionline':"Call: \"{target}\" ({callnumber})",'widget_doorcall_error_descriptionline':"Call to deleted call destination",'widget_set_sp_descriptionline_toggle':"Toggle: \"{target}\"",'widget_set_sp_descriptionline_on':"On: \"{target}\"",'widget_set_sp_descriptionline_off':"Off: \"{target}\"",'widget_set_sp_descriptionline':"Switching: \"{target}\"",'widget_set_sp_toggle':"Toggle",'widget_set_sp_on':"On",'widget_set_sp_off':"Off",'widget_set_sp_long':"Switching",'widget_set_sp_short':"Template",'widget_set_sp_label':"Switching point",'widget_set_sp_value':"Value",'widget_set_sp_error_descriptionline':"Switching from deleted switching point",'widget_setdnd_descriptionline_on':"On: \"{target}\"",'widget_setdnd_descriptionline_off':"Off: \"{target}\"",'widget_setdnd_value_toggle':"Toggle",'widget_setdnd_value_on':"On",'widget_setdnd_value_off':"Off",'widget_setdnd_descriptionline_toggle':"Toggle: \"{target}\"",'widget_setdnd_long':"Do not disturb",'widget_setdnd_short':"Do not disturb",'widget_setdnd_value_label':"Value",'widget_setdnd_drop_list_label':"Do not disturb",'widget_setdnd_descriptionline_error':"Switching from deleted switching point",'widget_setdnd_descriptionline':"Switching: \"{target}\"",'widget_set_redir_on_busy_immediately':"Immediately if busy",'widget_set_redir_owner_own':"this device",'widget_set_redir_owner_others':"other device",'widget_set_redir_toggle':"Toggle",'widget_set_redir_on':"On",'widget_set_redir_off':"Off",'widget_set_redir_long':"Call redirection",'widget_set_redir_short':"Call redirection",'widget_set_redir_label':"Switching point",'widget_set_redir_value':"Value",'widget_set_redir_label_option2':"Option 2",'widget_set_redir_check_option1':"Shall be active now and always.",'widget_set_redir_check_option2':"It's also a possibility.",'widget_set_redir_type':"Type",'widget_set_redir_time':"Time",'widget_set_redir_type_static':"Immediately",'widget_set_redir_type_na':"After time",'widget_set_redir_type_nab':"When busy",'widget_set_redir_owner':"Redirect",'widget_set_redir_redirect_target':"Destination",'widget_set_redir_validate_time_error':"Time must be set! ( Time >= -1 )",'widget_set_redir_validate_time_error2':"Time for call redirection must not exceed 120 seconds!",'widget_set_sp_knx_toggle':"Toggle",'widget_set_sp_knx_yes':"Yes",'widget_set_sp_knx_no':"No",'widget_set_sp_knx_on':"On",'widget_set_sp_knx_off':"Off",'widget_set_sp_knx_receive_object':"Feedback object",'widget_set_sp_knx_response':"Feedback object",'widget_set_sp_knx_long':"Send KNX telegram",'widget_set_sp_knx_short':"Template",'widget_set_sp_knx_label':"Switching point",'widget_set_sp_knx_value':"Value",'widget_set_sp_knx_gateway':"Gateway",'widget_set_sp_knx_group_address':"Group address",'widget_set_sp_knx_error_address':"KNX address is wrong! ( 0/0/1 - 31/7/255, 0/1 - 31/2047, 1 - 65535 )",'aui_widget_set_sp_knx':"Switching via KNX",'widget_set_sp_knx_action':"Send Telegram",'widget_set_doormatic_toggle':"Toggle",'widget_set_doormatic_on':"On",'widget_set_doormatic_off':"Off",'widget_set_doormatic_descriptionline_toggle':"Toggle: \"{target}\"",'widget_set_doormatic_descriptionline_on':"On: \"{target}\"",'widget_set_doormatic_descriptionline_off':"Off: \"{target}\"",'widget_set_doormatic_descriptionline':"Switching: \"{target}\"",'widget_set_doormatic_long':"Doormatic",'widget_set_doormatic_label':"Doormatic",'widget_set_doormatic_value':"Value",'widget_set_doormatic_owner':"Doormatic",'widget_set_doormatic_owner_own':"this equipment",'widget_set_doormatic_owner_others':"other device",'widget_set_doormatic_error_descriptionline':"Switching from deleted switching point",'widget_storey_call_door_light':"Door light",'widget_storey_call_door_opener':"Door opener",'widget_storey_call_long':"Storey call",'widget_storey_call_short':"Storey call",'widget_storey_call_description':"Storey call",'widget_storey_call_description_target':"\"{target}\" ({callnumber})",'widget_storey_call_label':"Label",'widget_storey_call_value':"Value",'widget_storey_call_target':"Call destination",'widget_mute_mic_short':"Microphone off",'widget_mute_mic_long':"Microphone off",'widget_mute_mic_no_parameter':"No parameters",'widget_mute_mic_descriptionline':"Microphone off",'aui_widget_mute_mic':"Microphone off",'widget_mute_speaker_short':"Ringtone off",'widget_mute_speaker_long':"Ringtone off",'widget_mute_speaker_no_parameter':"No parameters",'aui_widget_mute_speaker':"Ringtone off",'widget_grabpicmenu_short':"Video memory menu",'widget_grabpicmenu_long':"Video memory menu",'widget_grabpicmenu_descriptionline':"Video memory menu",'widget_grabpicmenu_error_descriptionline':"No video memory",'widget_grabpicmenu_no_parameter':"No parameters",'widget_extern_call_error_validate_not_a_number':"Call number must be a number",'widget_externcall_long':"Call, external",'widget_externcall_label':"Call number",'widget_externcall_prefix_hint':"(including prefix)",'widget_extern_call_descriptionline':"External call: ({callnumber})",'widget_extern_call_error_descriptionline':"Call to deleted call destination",'widget_extern_call_error_validate_empty':"Call number must not be empty",'widget_ldopener_long':"Door opener (last door)",'widget_ldopener_short':"Template",'widget_ldopener_descriptionline':"Door opener (last door)",'widget_ldopener_descriptionline_timeout':"Timeout {timeout} Min.",'widget_ldopener_owner':"Door opener",'widget_ldopener_works_always':"always works",'widget_ldopener_works_until_time':"limit the duration to",'widget_ldopener_prefix_time':"Minutes after door call",'widget_ldlight_long':"Door light (last door)",'widget_ldlight_short':"Template",'widget_ldlight_descriptionline':"Door light (last door)",'widget_ldlight_descriptionline_timeout':"Timeout {timeout} Min.",'widget_ldlight_owner':"Door Light",'widget_ldlight_works_always':"always works",'widget_ldlight_works_until_time':"limit the duration to",'widget_ldlight_prefix_time':"Minutes after door call",'widget_led_trigger_single':"Connected to \"Single click\"",'widget_led_trigger_double':"Connected to \"Double click\".",'widget_led_trigger_pos':"Connected to \"Press key\"",'widget_led_trigger_neg':"Connected to \"Release key\".",'widget_led_desciption_triggersingle':"Connected to \"Single click\"",'widget_led_desciption_triggerdouble':"Connected to \"Double click\"",'widget_led_desciption_triggerpos':"Connected to \"Press key\"",'widget_led_desciption_triggerneg':"Connected to \"Release key\"",'widget_led_redirtarget_label':"Destination",'widget_led_color_label':"Colour",'widget_led_long':"Set LED",'widget_led_short':"LED",'widget_led_color_white':"White",'widget_led_color_yellow':"Yellow",'widget_led_color_red':"Red",'widget_led_trigger_label':"LED",'widget_led_trigger_individually':"Individual",'widget_led_type_label':"Type",'widget_led_type_redir':"Call redirection",'widget_led_type_dnd':"Do not disturb",'widget_led_type_doormatic':"Doormatic",'widget_led_type_bool':"Switching",'widget_led_type_error_select':"A type must be selected!",'widget_led_device_label':"Device",'widget_led_desciption_action':"LED",'widget_led_desciption_default':"LED: Complex configuration",'widget_led_desciption_triggerindividually':"-",'widget_switchcontact_short':"Switching contact",'widget_switchcontact_long':"Switching contact",'widget_switchcontact_action':"Switching contact",'widget_switchcontact_label':"Label",'widget_switchcontact_invert':"Invert",'widget_switchcontact_name':"Output",'widget_timeswitchcontact_short':"Time switching contact",'widget_timeswitchcontact_long':"Switching contact (timed)",'widget_timeswitchcontact_action':"Switching contact",'widget_timeswitchcontact':"Switching contact",'widget_timeswitchcontact_label':"Label",'widget_timeswitchcontact_invert':"Invert",'widget_timeswitchcontact_posttrigger':"Retrigger",'widget_timeswitchcontact_name':"Output",'widget_timeswitchcontact_time':"Switching time",'widget_lightcontact_duration_manual':"manual",'widget_lightcontact_invert':", inverted",'widget_lightcontact_posttrigger':", retriggered",'widget_lightcontact_long':"Function (door light)",'widget_lightcontact_used_from_light':"This contact is used by the door light function.",'widget_lightcontact_used_from_no_one':"This contact is not used by any function!",'widget_lightcontact_short':"Door light",'widget_openercontact_duration_manual':"manual",'widget_openercontact_posttrigger':", retriggered",'widget_openercontact_invert':", inverted",'widget_openercontact_long':"Function (door opener)",'widget_openercontact_used_from_door':"This contact is used by the door opener function.",'widget_openercontact_used_from_door2':"This contact is used by the door opener 2 function.",'widget_openercontact_used_from_no_one':"This contact is not used by any function!",'widget_openercontact_short':"Door opener",'widget_openercontact_short2':"Door opener 2",'widget_openercontact_abbr_second':"s",'widget_openercontact_abbr_minute':"min",'widget_openercontact_abbr_hour':"h",'widget_state_notifying_contact_short':"Notifying state",'widget_state_notifying_contact_long':"Notifying state",'widget_state_notifying_contact_label':"Label",'widget_state_notifying_contact_invert':"Invert",'widget_state_notifying_contact_name':"Notifying state",'widget_state_notifying_contact':"Switching contact",'widget_state_notifying_contact_action':"Notifying state",'widget_local_door_light_short':"Door light",'widget_local_door_light_long':"Door light",'widget_local_door_light_no_parameter':"No parameters",'widget_local_door_light_description_line':"Door light",'widget_observe_led_knx_color':"Colour",'widget_observe_led_knx_long':"Receive KNX",'widget_observe_led_knx_short':"Receive KNX",'widget_observe_led_gateway':"KNX gateway",'widget_observe_led_knx_group_address':"Group address",'widget_observe_led_knx_color_yellow':"Yellow",'widget_observe_led_knx_color_red':"Red",'widget_observe_led_knx_drop_list_label':"Do not disturb",'widget_led_description_action':"LED: \"%name% (KNX %groupAddress%)\"",'widget_toemode_short':"Mode A1",'widget_toemode_long':"Voltage mode A1",'widget_toemode_descrline':"Voltage mode A1",'widget_toemode_ac':"AC",'widget_toemode_dc':"DC",'widget_dnd_long':"Do not disturb",'widget_presence_long':"User Status",'widget_grabpic_time_label':"Record image after",'widget_grabpic_time_postfix':"seconds",'widget_grabpic_long':"Video Memory",'widget_grabpic_mode_label':"Mode",'widget_grabpic_mode_auto':"Automatically record images",'widget_grabpic_mode_manual':"Record images manually only",'widget_grabpic_event_label':"Event",'widget_grabpic_event_calls':"For answered calls",'widget_grabpic_event_missedcalls':"For missed calls",'widget_grabpic_time_error':"The time must be between 2 and 30 seconds!",'widget_redir_long':"Call redirection",'widget_doormatic_time_delay_description':"seconds after the call open the door",'widget_doormatic_options_label':"Execute doormatic",'widget_doormatic_option_storeycall':"For storey calls",'widget_doormatic_option_online':"During call signalling only",'widget_doormatic_opener_label':"Switch",'widget_doormatic_opener':"Door opener",'widget_doormatic_opener2':"Door opener 2",'widget_doormatic_long':"Doormatic",'widget_doormatic_time':"Delayed opening",'widget_doormatic_open_after':"Open after",'widget_doormatic_open_after_suffix':"Seconds",'widget_doormatic_open_of':"Open from",'widget_doormatic_all_doors':"All doors",'widget_doormatic_selected_doors':"Certain doors",'widget_doormatic_doorlist':"Certain doors",'widget_building_automation_url_long':"Building Automation URL",'widget_building_automation_url_label':"URL",'widget_videourl_long':"Video URL",'widget_video_url_label':"Label",'widget_video_url_format':"Format",'widget_video_url_security':"Security",'widget_video_url_security_description':"HTTP basic authentication",'widget_video_url_url':"URL",'widget_video_url_participant':"User name",'widget_video_url_password':"Password",'widget_video_url_password_repeat':"Repeat password",'widget_video_url_frames':"Frames",'widget_video_url_fps':"Use HTTP basic authentication",'widget_video_url_single_pictures':"Single images",'widget_video_url_jpeg':"Single image",'widget_video_url_mjpeg':"MJPEG",'widget_light_long':"Door light",'widget_light_switch_contact':"Switching contact",'widget_light_switch_timeout':"Time",'widget_light_invert':"Inverted",'widget_light_post_trigger':"Retrigger",'widget_light_local_outputs':"Local switching contact",'widget_light_extern_outputs':"Other switching contact",'widget_light_output':"Output",'widget_light_error_is_empty':"An output must be selected",'widget_light_options':"Options",'widget_light_invert_check':"Switching output inverted",'widget_light_trigger_check':"Switching output retriggering",'widget_lightcontact_abbr_second':"s",'widget_lightcontact_abbr_minute':"min",'widget_lightcontact_abbr_hour':"h",'widget_door_long':"Door opener",'widget_door_switch_contact':"Switching contact",'widget_door_switch_timeout':"Time",'widget_door_invert':"Inverted",'widget_door_post_trigger':"Retrigger",'widget_door_local_outputs':"Local switching contact",'widget_door_extern_outputs':"Other switching contact",'widget_door_output':"Output",'widget_door_canceling_calls':"Door call",'widget_door_canceling_calls_suffix':"Door opener terminates call",'widget_door_error_is_empty':"An output must be selected",'widget_door_options':"Options",'widget_door2_long':"Door opener 2",'widget_door2_switch_contact':"Switching contact",'widget_door2_switch_timeout':"Time",'widget_door2_invert':"Inverted",'widget_door2_invert_check':"Switching output inverted",'widget_door2_post_trigger':"Retrigger",'widget_door2_trigger_check':"Switching output retriggering",'widget_door2_trigger_check_new':"Switching output retriggering",'widget_door2_local_outputs':"Local switching contact",'widget_door2_extern_outputs':"Other switching contact",'widget_door2_output':"Output",'widget_door2_canceling_calls':"Door Call",'widget_door2_canceling_calls_suffix':"Door opener terminates call",'widget_door2_use_for':"Door opener applies to",'widget_door2_options':"Options",'widget_door2_error_is_empty':"An output must be selected",'widget_door2_error_list_is_empty':"The list must not be empty",'widget_camera_long':"Video surveillance",'widget_call_contact_long':"Call contact",'widget_call_contact_short':"Call contact",'widget_call_contact_contact':"Switching point",'widget_call_contact_turn_on':"Switch on",'widget_call_contact_turn_off':"Switch off",'widget_call_contact_options':"Options",'widget_call_contact_option_incoming_call':"Incoming calls",'widget_call_contact_option_outgoing_call':"Outgoing calls",'widget_call_contact_option_video_call':"Only for door calls",'widget_call_contact_option_invert':"Invert",'widget_call_contact_invite':"Start of call",'widget_call_contact_ring':"Remote signals",'widget_call_contact_online':"Conversation begins",'widget_call_contact_offline':"Conversation ends",'widget_call_contact_hold':"Call waiting",'widget_call_contact_manual':"Manual",'widget_videoprivacy_long':"Video privacy",'widget_videoprivacy_text1':"During a call, the video is blocked for persons not involved in the call.",'widget_videoprivacy_text2':"You can exclude participants from the viewing block by assigning rights via the rights system.",'widget_deviceoutage_long':"Device outage detection",'widget_deviceoutage_text1':"This unit is monitored for malfunction. In the event of a persistent malfunction, an alarm is triggered.",'widget_deviceoutage_text2':"In the basic parameters, you configure the alarm contact and the duration until the alarm is triggered.",'widget_onvif_long':"ONVIF camera share",'widget_onvif_profile_label':"Profile name",'widget_onvif_refresh_tooltip':"Refresh name",'widget_onvif_resolution_label':"Resolution",'devkeygrp_collapsed':"More information...",'devkeygrp_collapsed_loading':"Loading...",'devkeygrp_no_action':"No action",'devkeygrp_button_add_label':"Tile",'devkeygrp_button_add':"Add",'devfunc_label_function':"Functions",'devfunc_gridheader_description':"Description",'devfunc_gridheader_status':"Status",'devfunc_button_edit':"Edit",'commandgrp_button_new':"New",'commandgrp_button_edit':"Edit",'rule_required_error_notice':"Error: This function requires \"{{widgetName}}\"! Please configure the function first.",'rule_calltarget_add':"Add",'rule_calltarget_error_empty':"An entry must be selected!",'rule_calltarget_error_deleted':"Call destinations are deleted!",'rule_integer_error_range':"Value out of range!",'modload_title':"Module installation",'modload_info_starting':"Start module installation",'modload_info_modinfo':"Load module information.",'modload_info_install':"Install module files.",'modload_info_update':"Update existing data.",'modload_info_finished':"Installation completed.",'modload_info_error':"An error occurred during the module installation!",'modload_info_license':"There are not enough licences available for this module!",'moddelete_title':"Delete module.",'moddelete_info_starting':"Starts deleting the module.",'moddelete_info_modinfo':"Load module information.",'moddelete_info_delete':"Remove module.",'moddelete_info_configs':"Remove configured module configuration.",'moddelete_info_update':"Update existing data.",'moddelete_info_finished':"Delete completed.",'moddelete_info_error':"An error occurred while deleting the module!",'application_window_title':"Siedle Access",'application_edit_adminpass':"Change login password"} } );
$.extend( true, $.views._tpl, { '/lib/viewable/dialog/simple':'<div class=\"dialog overlay overlay-dark\"><div class=\'themeDialog pageDialog dialogSimple\' tabindex=\'0\'><div class=\"dialog-content\"></div></div></div>','/lib/viewable/dialog/classic':'<div class=\"dialog overlay overlay-dark\"><div class=\'themeDialog pageDialog dialogClassic v-layout\' tabindex=\'0\'><!-- title bar --><div class=\"dialog-title v-layout-fix hilight\"><div class=\'text\'><span>{{title}}</span></div><div class=\'close\'><i class=\'mdi mdi-close\'></i></div></div><!-- content container --><div class=\"dialog-container v-layout-stretch v-layout\"><!-- scrollable content --><div class=\"scroll-container v-layout-stretch\"><div class=\'dialog-content scrollable\'></div></div><!-- button footer --><div class=\"dialog-footer v-layout-fix v-align-middle h-align-right\"></div></div></div></div>','/lib/viewable/dialog/classic2':'<div class=\"dialog overlay overlay-dark\"><div class=\'themeDialog pageDialog dialogClassic2 v-layout-no-height\' tabindex=\'0\'><!-- title bar --><div class=\"dialog-title v-layout-fix hilight\"><div class=\'text\'><span>{{title}}</span></div><div class=\'close\'><i class=\'mdi mdi-close\'></i></div></div><!-- content container --><div class=\"dialog-container v-layout-stretch v-layout\"><!-- content --><div class=\'dialog-content v-layout-stretch\'></div><!-- button footer --><div class=\"dialog-footer v-layout-fix v-align-middle h-align-right\"></div></div></div></div>','/lib/viewable/popup/simple':'<div class=\"dialog overlay overlay-dark\"><div class=\'themeDialog pageDialog popupSimple\' tabindex=\'0\'><!-- insert dialog style at element with \'.modal-layer\' --></div></div>','/lib/treemenu/accordion/head':'<div data-id=\"{{id}}\" class=\"head hilight\" title=\"{{tooltip}}\"><i class=\"{{icon}}\"></i><span>{{text}}</span></div>','/lib/treemenu/accordion/item':'<div data-id=\"{{id}}\" class=\"item normal\" title=\"{{tooltip}}\"><i class=\"{{icon}}\"></i><span>{{text}}</span></div>','/lib/treemenu/accordion/group':'<div data-id=\"{{id}}\" class=\"item normal\" title=\"{{tooltip}}\"><i class=\"{{icon}}\"></i><span>{{text}}</span></div>','/lib/treemenu/accordion/subitem':'<div data-id=\"{{id}}\" class=\"subitem normal\" title=\"{{tooltip}}\"><i class=\"mdi mdi-blank\"></i><span>{{text}}</span></div>','/lib/treemenu/accordion/group-container':'<div data-group-id=\"{{id}}\"></div>','/lib/treemenu/accordion/separator':'<div class=\"separator disabled\"></div>','/lib/breadcrumps/container':'<div class=\"breadcrumps normal h-layout-fix v-layout-fix v-align-middle\"></div>','/lib/breadcrumps/home':'<div class=\"home\" data-id=\"ID_HOME\"><i class=\"{{icon}}\"></i></div>','/lib/breadcrumps/chevron':'<div class=\"chevron normal\"><svg width=\"100%\" viewBox=\"0 0 55 100\" preserveAspectRatio=\"none\"><path d=\"M 0,0 L 50,50 L 0,100\" vector-effect=\"non-scaling-stroke\"></path></svg></div>','/lib/breadcrumps/button':'<div class=\"button normal\" data-id=\"{{id}}\"><span>{{text}}</span></div>','/lib/button/textonly':'<div class=\"container\"><span>{{text}}</span></div>','/lib/button/textonly-dd':'<div class=\"container\"><i class=\"mdi mdi-spacer\"></i><span>{{text}}</span><i class=\"mdi mdi-menu-down\"></i></div>','/lib/button/icononly':'<div class=\"container\"><i class=\"{{icon}}\"></i></div>','/lib/button/icononly-dd':'<div class=\"container\"><i class=\"{{icon}}\"></i><i class=\"mdi mdi-menu-down\"></i></div>','/lib/button/iconText':'<div class=\"container\"><i class=\'{{icon}}\'></i><s class=\"xs\"></s><span>{{text}}</span></div>','/lib/button/iconText-dd':'<div class=\"container\"><i class=\'{{icon}}\'></i><s class=\"xs\"></s><span>{{text}}</span><i class=\"mdi mdi-menu-down\"></i></div>','/lib/button/icon':'<div class=\"icon\"><i class=\"{{icon}}\"></i></div>','/lib/button/icon-dd':'<div class=\"icon\"><i class=\"{{icon}}\"></i></div><div class=\"icon-dd\"><i class=\"mdi mdi-menu-down\"></i></div>','/lib/button/indicator':'<div class=\"container\"><i class=\"{{icon}}\"></i></div><div class=\"status\"><div class=\"indicator\"></div></div>','/lib/choice/radiogroup':'<fieldset style=\'display: inline-block\'></fieldset><div class=\'message-text\'></div>','/lib/choice/radioitem-v':'<input type=\'radio\' name=\'{{groupid}}\' id=\'{{ctrlid}}\' value=\'{{id}}\' class=\'no-hover\'/><label for=\'{{ctrlid}}\' class=\'no-hover\'>{{text}}</label>','/lib/choice/radioitem':'<input type=\'radio\' name=\'{{groupid}}\' id=\'{{ctrlid}}\' value=\'{{id}}\' class=\'no-hover\'/><label for=\'{{ctrlid}}\' class=\'no-hover radiogroup-bottom-spacer\' style=\'display: block;\'>{{text}}</label>','/lib/choice/checkbox':'<input type=\'checkbox\' name=\'{{id}}\' id=\'{{ctrlid}}\' class=\'no-hover\'/><label for=\'{{ctrlid}}\' class=\'no-hover\' style=\'display: block;\'>{{text}}</label>','/lib/choice/checkgroup':'<fieldset style=\'display: inline-block\'></fieldset><div class=\'message-text\'></div>','/lib/listbox/container':'<div class=\'scroll-container\'><div class=\'scrollable\'></div></div><div class=\"message-text\"></div>','/lib/listbox/droplist/container':'<div class=\'head hover\'><i class=\'icon\'></i><span class=\'text\' style=\"width: 100%;\"></span><i class=\"chevron mdi mdi-menu-down\"></i></div><div class=\'popup normal\'><div class=\'scroll-container\'><div class=\'scrollable\'></div></div></div>','/lib/listbox/droplist/popup':'<div class=\'scroll-container\'><div class=\'scrollable\'></div></div>','/lib/listbox/item':'<div class=\"item\" data-id=\"{{id}}\"><i class=\"{{icon}}\"></i><span>{{text}}</span></div>','/lib/listbox/customitem':'<div class=\"item\" data-id=\"{{id}}\"></div>','/lib/listbox/search':'<div class=\"search normal\"><input type=\"text\" spellcheck=\"false\" placeholder=\"{{searchPlaceholder}}\" tabindex=\'0\'/><i class=\"{{seachIcon}}\"></i></div>','/lib/listbox/placeholder':'<div class=\"placeholder disabled\"><span>{{placeholder}}</span></div>','/lib/toast/container':'<div class=\"toast toast-transition green h-layout\"><div class=\"h-layout-fix h-align-center v-align-middle\" style=\"width:3em\"><span class=\"xl\"><i class=\"{{icon}}\"></i></span></div><div class=\"h-layout-stretch v-align-middle\"><span class=\"\">{{text}}</span></div><div class=\"close-button\"><i class=\"mdi mdi-close\"></i></div></div>','/lib/tabs/container':'<div class=\"tabs no-hover\"></div>','/lib/tabs/header':'<div class=\"header\"><div class=\"h-layout\"><div class=\"content h-layout-stretch\"><span class=\"{{icon}}\"></span><span class=\"tab-text\">{{text}}</span><sub><i class=\"mdi mdi-circle\"></i></sub></div><div class=\"close-button h-layout-fix\"><sup><i class=\"mdi mdi-close-thick\"></i></sup></div></div></div>','/lib/tabs/header-sticky':'<div class=\"header\"><div class=\"h-layout\"><div class=\"content h-layout-stretch\"><span>{{icon}}</span><span class=\"tab-text\">{{text}}</span><sub><i class=\"mdi mdi-circle\"></i></sub></div></div></div>','/lib/tabs/scroller-left':'<div class=\"tabs-scroller-left normal hover\" style=\"display:none\"><i class=\"mdi mdi-chevron-left\"></i></div>','/lib/tabs/scroller-right':'<div class=\"tabs-scroller-right normal hover\" style=\"display:none\"><i class=\"mdi mdi-chevron-right\"></i></div>','/lib/multi/container':'<div class=\"container\"></div>','/lib/multi/blank':'<div class=\"blank viewable-container pageContent themeContent\"></div>','/lib/multi/scroll-static':'<div class=\"scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"></div></div>','/lib/multi/scroll':'<div class=\"scroll-container pageContent themeContent viewable-container\"><div class=\"scrollable form\"></div></div>','/lib/groupbox/label':'<h3><div class=\"groupbox-action\"><i class=\"mdi mdi-chevron-up mdi-rotate-180\"></i><s class=\"s\"></s><span>{{text}}</span></div></h3>','/lib/popupmenu':'<div tabindex=\"0\" class=\"themeNormal popupmenu\"></div>','/lib/popupmenu/item':'<div class=\"item\"><i class=\"{{icon}}\"></i><span>{{text}}</span></div>','/lib/popupmenu/separator':'<div class=\"separator\"></div>','/lib/toolbar':'<div class=\"container h-layout\"><div class=\"title h-layout-stretch v-align-middle\"><i class=\'{{icon}}\'></i><s></s><span class=\"toolbar-title\">{{text}}</span><s class=\"s\"></s><div class=\"icon-status\"><i class=\"{{iconStatus}}\"></i></div></div><div class=\"buttons h-layout-fix v-align-middle\"></div></div>','/lib/toolbar-closeable':'<div class=\"container h-layout\"><div class=\"title h-layout-stretch v-align-middle\"><i class=\'{{icon}}\'></i><s></s><span class=\"toolbar-title\">{{text}}</span><s class=\"s\"></s><div class=\"icon-status\"><i class=\"{{iconStatus}}\"></i></div></div><div class=\"buttons h-layout-fix v-align-middle\"></div><div class=\"close-button\"><i class=\"mdi mdi-close\"></i></div></div>','/lib/toolbar/button':'<div class=\'toolbar-button\'><i class=\"icon {{icon}}\"></i><s class=\"s\"></s><div class=\"text\">{{text}}</div></div>','/lib/toolbar/button-icon':'<div class=\'toolbar-button\'><i class=\"icon {{icon}}\"></i></div>','/lib/toolbar/button-text':'<div class=\'toolbar-button\' title=\"{{tooltip}}\"><div class=\"text\">{{text}}</div></div>','/lib/toolbar/icon':'<div class=\'toolbar-icon no-hover\'><i class=\"tb-icon {{icon}}\"></i></div>','/lib/grid/container':'<div class=\"grid-container v-layout\"><style></style><div class=\"header v-layout-fix\"></div><div class=\"v-layout-stretch scroll-container\"><div class=\"scrollable data\"></div></div></div>','/lib/grid/test':'<div class=\"row\"><div class=\"cell cell-test\"><i class=\"{{icon}}\"></i><span>{{text}}</span></div></div>','/lib/grid/header-item':'<div class=\"cell {{cssColumnId}}\"><i class=\"{{icon}}\"></i><s class=\"s\"></s><span class=\"text\">{{text}}</span><div class=\"cell-sorter\"><i class=\"mdi mdi-none\"></i></div></div>','/lib/grid/header-item-icon':'<div class=\"cell {{cssColumnId}}\"><i class=\"{{icon}}\"></i><div class=\"cell-sorter\"><i class=\"mdi mdi-none\"></i></div></div>','/lib/grid/header-item-text':'<div class=\"cell {{cssColumnId}}\"><span class=\"text\">{{text}}</span><div class=\"cell-sorter\"><i class=\"mdi mdi-none\"></i></div></div>','/lib/grid/header-scrollbar':'<div class=\"scroll-cell\"></div>','/lib/grid/header-splitter':'<div id=\"{{id}}\" class=\"cell-splitter\"></div>','/lib/grid/data-row':'<div class=\"row\"></div>','/lib/grid/data-placeholder':'<div class=\"placeholder disabled\"><span>{{placeholder}}</span></div>','/lib/grid/data-placeholder-icon-only':'<div class=\"placeholder\"><span class=\"xxxl\"><i class=\"{{placeholderIcon}}\"></i></span></div>','/lib/grid/data-placeholder-icon':'<div class=\"placeholder\"><span class=\"l\"><i class=\"l {{placeholderIcon}}\"></i></span><s class=\"s\"></s><span class=\"disabled\">{{placeholder}}</span></div>','/lib/grid/data-cell':'<div class=\"cell {{cssColumnId}}\"><i class=\"{{icon}}\"></i><s class=\"s\"></s><span>{{text}}</span><s class=\"s\"></s><i class=\"{{posticon}}\"></i></div>','/lib/grid/data-cell-icon':'<div class=\"cell {{cssColumnId}}\"><i class=\"{{icon}}\"></i><s class=\"s\"></s><i class=\"{{posticon}}\"></i></div>','/lib/grid/data-cell-text':'<div class=\"cell {{cssColumnId}}\"><span>{{text}}</span><s class=\"s\"></s><i class=\"{{posticon}}\"></i></div>','/lib/grid/data-cell-empty':'<div class=\"cell {{cssColumnId}}\"><span></span></div>','/lib/grid/inplace-onoff':'<div class=\"cell grid-inplace-onoff {{cssColumnId}}\"><i class=\"{{icon}}\"></i><s class=\"s\"></s><i class=\"{{icon2}}\"></i></div>','/lib/range':'<div class=\"range-trackbar range-nonactive\" style=\"width:100%\"></div><div class=\"range-trackbar range-active\"></div><div class=\"range-handle\"></div><div class=\"range-tooltip\">{{tooltip}}</div>','/lib/optional':'','/lib/accordion/item-container':'<div class=\"head default hover\"><i class=\"mdi mdi-chevron-down\"></i><span>{{text}}</span></div><div class=\"content\"></div><div class=\"splitter\"></div>','/lib/badge/text':'<div class=\"badge badge-red\">{{text}}</div>','/lib/badge/icon':'<div class=\"badge-icon\"><i class=\"{{icon}}\"></i></div>','/lib/tree/test':'<div class=\"row\"><div class=\"level\"></div><i></i><i class=\"mdi mdi-user\"></i><span>Test-Item</span></div><div class=\"children\"></div>','/lib/tree/container':'<div class=\"tree-container v-layout\"><div class=\"data v-layout-stretch scroll-container\"><div class=\"scrollable\"></div></div></div>','/lib/tree/item':'<div class=\"row hover\"><div class=\"level\"></div><i class=\"chevron mdi mdi-spacer\"></i><i class=\"{{icon}}\"></i><span>{{text}}</span><s></s><i class=\"posticon mdi mdi-lock\"></i><div class=\"inplace-button\"></div></div><div class=\"children\"><div class=\"child-line\"></div></div>','/lib/tree/item-content':'<div class=\"level\"></div><i class=\"chevron mdi mdi-spacer\"></i><i class=\"{{icon}}\"></i><span>{{text}}</span><s></s><i class=\"posticon mdi mdi-lock\"></i><div class=\"inplace-button\"></div>','/lib/tree/children-content':'<div class=\"child-line\"></div>','/lib/tree/item-test-menu':'<div class=\"row hover\"><div class=\"level\"></div><i class=\"chevron\"></i><i class=\"{{icon}}\"></i><span>{{text}}</span><s></s><i class=\"posticon mdi mdi-lock\"></i><div class=\"menu\"><i class=\"mdi mdi-plus-circle-outline\"></i></div></div><div class=\"children\"><div class=\"child-line\"></div></div>','/lib/qrcode':'<div class=\"code\"></div>','/lib/descriptionline/container-single':'<div class=\"container no-hover\"></div>','/lib/descriptionline/container-multi':'<div class=\"chevron\"><i class=\"hover {{chevronIcon}}\"></i></div><div class=\"container no-hover\"></div>','/lib/descriptionline/line':'<div class=\"line-content\" title=\"{{tooltip}}\"><i class=\"icon-{{iconcolor}} {{icon}}\"></i><s class=\"s\"></s><span class=\"frame\">{{text}}</span><i class=\"posticon-{{posticoncolor}} {{posticon}}\"></i></div>','/lib/svg/container':'<div class=\"svg-container\"><div class=\"svg-content\"></div></div>','/lib/fileupload/container':'<div class=\"container\"><input type=\'file\' name=\'{{id}}\' id=\'{{ctrlid}}\' class=\'no-hover\' style=\"display: none\" /><label for=\'{{ctrlid}}\' class=\'no-hover\'><i class=\"{{icon}}\"></i><s></s><span>{{text}}</span></label></div>','/lib/timeline/legend-de':'<div class=\"tl-legend\"><span>0h</span><span>2h</span><span>4h</span><span>6h</span><span>8h</span><span>10h</span><span>12h</span><span>14h</span><span>16h</span><span>18h</span><span>20h</span><span>22h</span><!-- <span>24</span>--></div>','/lib/timeline/legend-en':'<div class=\"tl-legend\"><span>12am</span><span>2am</span><span>4am</span><span>6am</span><span>8am</span><span>10am</span><span>12pm</span><span>2pm</span><span>4pm</span><span>6pm</span><span>8pm</span><span>10pm</span><!-- <span>12am</span>--></div>','/lib/timeline/popdown':'<div class=\'tl_popdown\' tabindex=\'0\'><table><tbody><tr><td><label>{{label_start}}</label></td><td><input id=\'{{id_start}}\' class=\"timeline-edit\" type=\'text\'></td></tr><tr><td><label>{{label_stop}}</label></td><td><input id=\'{{id_stop}}\' type=\'text\'></td></tr></tbody></table></div>','/lib/wizzard/head':'<div class=\"wizzard-sidebar\"></div>','/lib/wizzard/item':'<div class=\"wizzard-item v-align-middle\"><i class=\"{{icon}}\"></i><s></s><span>{{text}}</span></div>','/lib/wizzard/item-checked':'<div class=\"wizzard-item v-align-middle\"><i class=\"{{icon}}\"></i><s></s><span>{{text}}</span><s class=\"s\"></s><span><i class=\"mdi mdi-check\"></i></span></div>','/pages/main':'<!-- V-LAYOUT --><div class=\"v-layout\" style=\"overflow-x: hidden;\"><!-- HEADER (fix vertical)--><div id=\"ID_HEADER\" class=\"pageHeader themePageHeader v-layout-fix\"></div><!-- <div id=\"ID_HEADER\" class=\"pageHeader themePageHeader v-layout-fix\"></div>--><div class=\"v-layout-stretch h-layout\"><!-- left space --><div id=\"ID_SIDEBAR\" class=\"pageSidebar themeSidebar h-layout-fix splitter-parent no-scroll\" style=\"height: 100%\"></div><!-- CONTENT --><div id=\"ID_CONTENT\" class=\"h-layout-stretch v-layout\"><!-- NAVIGATION --><div id=\"ID_TABS\" class=\"pageNavigation themeNavigation v-layout-fix\"></div><!-- MULTI TABS --><div class=\"v-layout-stretch pageContent themeContent\" id=\"ID_MULTI\"></div></div></div></div>','/pages/login':'<!-- H-LAYOUT (full page) --><div class=\"page-login h-layout\" style=\"height:100%\"><!-- SIDE-HEADER (fix vertical)--><div id=\"ID_LOGINSIDEBAR\" class=\"themeSidebar sidebar h-layout-fix\"><div class=\"icon-container hilight\"><div class=\"h-align-center\"><i class=\"mdi mdi-account-circle\"></i></div></div><div class=\"login-download\"><a class=\"login-link\" href=\"/downloads/\">{$login_link_download}</a></div></div><!-- CONTENT (stretch vertical)--><div class=\"h-layout-stretch themeNormal\"><div class=\"ctrl-container readonly\"><div><span class=\"headline readonly\">{$login_title_login}</span></div><div><span class=\"subline disabled\">{$login_subtitle_login}</span></div><br /><div class=\"label\"><span class=\"disabled\">{$login_label_user}</span></div><div id=\"ID_LOGIN_USER\" tabindex=\"1\"></div><div class=\"label\"><span class=\"disabled\">{$login_label_password}</span></div><div id=\"ID_LOGIN_PASSWORD\" tabindex=\"1\"></div><div class=\"label\"><span class=\"disabled\">{$login_label_language}</span></div><div id=\"ID_LOGIN_LANG\" class=\"normal\" tabindex=\"2\"></div><br /><div id=\"ID_LOGIN\" class=\"login-button v-align-middle\"><i class=\"mdi mdi-arrow-right-bold-circle-outline\"></i>&nbsp;<span>{$login_label_enter}</span></div></div></div></div>','/pages/restart':'<style>#ID_LOGINSIDEBAR .icon-container{ background-image: url( img/a5grid.svg ); }</style><!-- H-LAYOUT (full page) --><div class=\"pages-restart h-layout\" style=\"height:100%\"><!-- SIDE-HEADER (fix vertical)--><div id=\"ID_LOGINSIDEBAR\" class=\"themeSidebar sidebar h-layout-fix\"><div class=\"icon-container hilight\"><div class=\"h-align-center\"><i class=\"mdi  mdi-account-circle\"></i></div></div></div><!-- CONTENT (stretch vertical)--><div class=\"h-layout-stretch themeNormal\"><div class=\"ctrl-container readonly\"><div><span class=\"headline readonly\">{$restart_headline}</span></div><br/><br/><div><span>{$restart_message}</span></div></div></div></div>','/pages/install':'<!-- V-LAYOUT --><div class=\"v-layout\" style=\"overflow-x: hidden; height: 100%\"><!-- HEADER (fix vertical)--><div id=\"ID_HEADER\" class=\"pageHeader themePageHeader v-layout-fix\"><div class=\"h-layout\" style=\"height: 100%\"><!-- title --><div class=\"v-layout-stretch v-align-middle\"><s class=\"l\"></s><span class=\"caption\">{$install_header_title}</span></div><!-- logo --><div id=\"ID_SSS\" class=\"v-align-middle logo\"><svg xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns=\"http://www.w3.org/2000/svg\" width=\"8.75em\" height=\"1.2rem\" viewBox=\"0 0 1000 176.12419\" version=\"1.1\" xmlns:cc=\"http://creativecommons.org/ns#\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\"><g transform=\"translate(-22.348 58.484)\"><path d=\"m411.41 117.09c-3.2688-.29698-8.6852-1.0428-12.037-1.6572-5.5788-1.0228-7.6145-1.4599-16.124-3.4616l-2.8508-.67042v-12.842c0-8.039.15354-12.842.41088-12.842.52811 0 5.2234 2.1908 5.5019 2.5672.2029.27402 6.7429 3.1689 12.036 5.3277 13.302 5.4243 26.715 7.5531 36.954 5.8649 4.8197-.79466 8.5421-2.2104 12.188-4.6357 13.608-9.0509 13.607-28.418-.003-40.241-5.15-4.465-10.06-7.595-31.41-20.009-13.4-7.796-15.38-9.07-20.69-13.32-12.86-10.292-17.53-19.82-17.53-35.765 0-9.591 1.923-16.466 6.5129-23.283 2.657-3.9433 7.3014-9.1797 8.1442-9.1797.25224 0 .56801-.28523.70181-.63380.13339-.34860.53409-.63383.88989-.63383.3556 0 .64667-.19025.64667-.42254 0-.23246.28496-.42253.6335-.42253.34846 0 .63357-.15957.63357-.35457 0-.19508.75789-.71792 1.6845-1.162.92642-.44409 1.6866-.95002 1.6893-1.1243.003-.17427.18957-.31692.41566-.31692.22579 0 1.4137-.46568 2.6395-1.0348 5.0336-2.3371 11.303-3.9495 19.088-4.9093 5.36-.66082 23.332-.28764 30.653.63630 12.475 1.5745 17.963 2.6312 19.322 3.7203.39839.31942.52786 3.1697.52786 11.618 0 10.466-.0482 11.18-.73901 10.969-.40656-.12448-2.4496-.97224-4.5402-1.884-14.24-6.2109-33.424-10.049-42.94-8.5919-10.774 1.6501-17.756 7.2751-20.035 16.141-1.4417 5.6084.0442 13.49 3.1785 16.858 2.3524 2.5276 2.5703 2.8041 2.5703 3.261 0 .24423.52258.7861 1.1614 1.204.63877.41796 1.3722.96272 1.6301 1.2106 1.036.99636 3.4235 3.0112 4.0389 3.4085.36009.23246 2.1359 1.3733 3.9462 2.5353s3.5972 2.398 3.9708 2.7466c.37365.34857 1.1519.87388 1.7295 1.1673 2.2639 1.1503 4.8262 2.6748 5.0441 3.0013.21074.31566 1.6849 1.1976 6.7755 4.0538 1.0452.58646 2.7714 1.5908 3.8356 2.232 3.6812 2.2175 12.644 7.2926 12.878 7.2926.12923 0 .85886.47536 1.6201 1.0564.76142.58101 1.5289 1.0564 1.7055 1.0564.17672 0 1.1661.57042 2.1986 1.2676 1.0327.69719 1.9656 1.2677 2.0734 1.2677.18848 0 3.2514 2.3852 6.6804 5.2024 1.6318 1.3407 7.0184 6.6281 7.0184 6.8892 0 .07538.93466 1.5212 2.0772 3.213 2.4546 3.6352 4.7102 8.9413 5.7981 13.64 1.1212 4.8431 1.1165 17.587-.008 22.395-1.7057 7.2903-3.6783 12.022-7.0392 16.886-10.898 15.773-35.102 23.455-65.264 20.714zm-361.49-1.33c-7.4242-.47315-18.582-1.9176-24.39-3.1576l-2.0061-.42847v-8.4158c0-9.7964-.51036-9.1103 5.3056-7.1318 11.851 4.0313 27.607 6.7084 36.295 6.1665 5.1908-.3237 13.06-1.9308 16.7-3.4106 5.2288-2.1256 10.881-6.5832 13.86-10.931 3.6653-5.3496 5.2216-10.583 5.2383-17.616.0211-8.9199-2.1779-14.334-8.3348-20.521-5.644-5.676-11.566-9.382-31.687-19.837-22.845-11.869-29.445-17.18-34.658-27.888-2.7123-5.5712-3.6177-9.4379-3.8584-16.479-.29939-8.7578 1.2864-15.478 5.2795-22.374 3.6891-6.3706 10.602-12.917 16.713-15.827 4.0847-1.9451 6.9313-3.0812 8.7109-3.4766 9.9095-2.2019 9.7888-2.1906 23.44-2.202 13.37-.01214 16.943.28312 25.34 2.0887 1.626.34960 3.574.74581 4.329.88050l1.3726.24482v8.6069c0 7.0229-.10708 8.6069-.58096 8.6069-.31953 0-1.618-.45887-2.8856-1.0197-5.7026-2.5232-16.172-5.2858-24.17-6.378-15.258-2.0835-27.12.72073-35.065 8.2895-5.4876 5.2275-7.4903 9.9861-7.5381 17.912-.03137 5.1676.58909 7.9203 2.7562 12.232 3.5437 7.0503 8.583 11.074 23.983 19.147 2.2067 1.1569 5.4376 2.8965 7.1797 3.8658 1.7421.96929 3.8327 2.0874 4.6457 2.4847s1.5806.85879 1.7058 1.0256c.12515.1668 1.6456.95954 3.3787 1.7618 1.7331.80221 3.8987 1.9672 4.8125 2.589.91379.62174 2.0541 1.3021 2.534 1.5119.47991.20984 1.2484.61966 1.7078.91071.45944.29094 2.2649 1.4272 4.0122 2.5247 6.6123 4.1534 9.4786 6.5071 12.75 10.47 2.8147 3.4097 4.5755 5.7723 4.5755 6.1391 0 .15388.34519.78459.76712 1.4015 1.2212 1.7855 3.4466 7.8849 4.3225 11.847 1.0112 4.5748 1.1 13.285.18453 18.113-4.0095 21.147-21.291 35.332-46.452 38.128-3.4765.38638-15.156.46832-20.272.14215zm90.169-.34601c-6.5471-.65029-13.525-1.6494-17.795-2.5482-1.625-.34193-3.0371-.62181-3.1381-.62181-.10097 0-.18361-3.8029-.18361-8.4509v-8.4509h1.4696c.80833 0 2.1862.26962 3.062.59937 8.2497 3.1052 21.443 5.7433 31.789 6.3567 4.5019.26668 12.837-.57952 16.742-1.6999 2.5518-.73226 8.6464-3.1351 9.0205-3.5565.11569-.13062.68633-.4464 1.2671-.70145 1.9146-.84046 5.7845-4.3035 7.827-7.0041 3.9035-5.161 5.5719-9.8636 5.9643-16.812.58-10.285-2.73-17.892-10.69-24.536-5.95-4.972-10.74-7.832-29.28-17.497-15.48-8.065-18.48-9.898-24.84-15.191-5.81-4.841-10.28-11.836-12.42-19.467-.82996-2.957-.94751-4.3595-.93861-11.197.0123-8.9685.5826-11.858 3.584-18.139 2.3332-4.8829 4.6601-7.9882 9.5447-12.738 7.267-7.0661 13.211-9.7491 25.607-11.558 11.873-1.7329 39.101.05676 44.666 2.9356l1.2505.64704v8.4401c0 4.642-.13138 8.44-.29294 8.44-.16098 0-2.2043-.74702-4.5402-1.6601-4.7507-1.8571-11.329-3.7463-17.973-5.1616-5.9853-1.2751-20.616-1.426-24.845-.25628-11.734 3.2453-19.426 10.018-21.788 19.182-1.2311 4.7773-.59014 13.708 1.2137 16.91 1.2936 2.2962 4.036 5.8619 6.2427 8.1166 3.092 3.1593 8.8834 6.9168 18.482 11.991 2.2069 1.1666 4.2029 2.2698 4.436 2.4517.23297.18180 2.5135 1.4109 5.068 2.7313 25.236 13.044 33.746 19.709 39.855 31.215.72686 1.369 1.3229 2.8902 1.3245 3.3804.001.4902.4632 2.2477 1.0257 3.9057 3.2182 9.4864 2.3374 23.128-2.0967 32.477-6.7318 14.193-19.298 23.075-37.998 26.858-3.885.78582-6.3055.95917-14.782 1.0583-6.2663.0729-12.313-.0974-15.838-.44762zm100.94.35838c-.81299-.0763-3.8539-.34038-6.7575-.58644-5.4078-.458-14.102-1.6774-18.055-2.5324l-2.2173-.47957v-8.6812-8.681l1.7579.28122c.96693.15432 3.2.74722 4.9625 1.3167 11.047 3.5691 24.771 6.116 32.959 6.116 7.7308 0 19.212-2.8429 23.274-5.763 1.0868-.78123 2.1434-1.4211 2.348-1.4222.79803-.0031 5.7739-4.8736 7.3444-7.1873 3.5774-5.2705 5.0641-10.446 5.0641-17.63 0-12.113-5.286-19.964-19.326-28.706-2.3039-1.4344-10.444-5.8737-18.088-9.8652-19.588-10.227-24.572-13.388-30.551-19.37-12.78-12.787-14.623-35.679-4.1205-51.187 3.553-5.2466 12.007-12.543 17.458-15.068 6.1654-2.8555 17.413-4.7926 27.81-4.7893 9.2884.0028 23.562 1.5841 29.347 3.251 4.0658 1.1714 3.8061.50723 3.8061 9.7357 0 4.6532-.16531 8.138-.38734 8.138-.21314 0-1.291-.38753-2.3955-.86120-5.0393-2.1612-13.58-4.617-21.29-6.1216-5.5149-1.0762-16.666-1.3091-21.249-.4437-7.7732 1.4679-13.16 4.1488-17.817 8.8679-3.2382 3.2807-5.3235 6.7833-6.322 10.619-1.0339 3.9718-.69192 13.307.51622 14.084.22708.14597.55894.74081.73774 1.3218.17884 0.581.44183 1.1514.58488 1.2676.14246.11603.82992 1.333 1.5268 2.7039 2.2931 4.5113 6.1861 8.0782 14.36 13.157.69689 0.433 3.9277 2.2419 7.1796 4.0196 3.2521 1.7778 6.5779 3.6084 7.3909 4.068.81297.45956 3.5359 1.9618 6.0507 3.3384s5.8406 3.2087 7.3909 4.0714c1.5501.86268 3.1036 1.7045 3.4519 1.8706.34852.16617 1.3937.78505 2.3229 1.3753 1.9745 1.2544 3.2624 2.0476 5.2183 3.2139 3.239 1.9315 9.0562 6.7659 12.246 10.177 3.8568 4.1242 6.0149 8.0719 8.7586 16.021l1.6043 4.648.0123 8.6622c.0123 8.2456-.0446 8.8451-1.1439 12.465-5.8525 19.272-20.756 30.752-44.433 34.225-3.8128.5592-16.874.77995-21.328.36042zm269.45-85.502v-85.354h13.515 13.515v85.354 85.354h-13.515-13.515v-85.354zm64.195.21136v-85.143h44.557 44.556v8.4509 8.4509h-31.464-31.464v28.099 28.099h29.986 29.986v8.6622 8.6622h-29.775-29.775v31.268 31.268h32.098 32.098v8.662 8.6623h-45.401-45.401v-85.143zm114.88-.25236v-85.395l25.446.20985c25.599.21105 28.279.35382 37.69 2.0086 16.221 2.8522 29.743 11.333 37.917 23.782 3.3872 5.1586 4.68 7.6785 6.826 13.305 6.4594 16.935 8.5775 36.954 6.5348 61.765-2.548 30.951-14.538 52.59-34.306 61.912-13.013 6.1367-26.174 7.7737-62.687 7.7971l-17.421.0123v-85.395zm51.288 67.602c6.1864-1.1332 10.892-2.9408 15.915-6.1134 2.1614-1.3651 7.9641-7.1892 8.817-8.8496.34852-.67815.95958-1.565 1.358-1.9707.99159-1.0096 4.7349-8.836 5.7727-12.069 1.2427-3.8716 2.457-9.822 3.0831-15.107.72322-6.1042.72232-35.405 0-43.522-1.2448-13.965-4.7125-26.074-9.5591-33.379-3.2805-4.9447-9.5975-9.6297-16.269-12.066-6.5417-2.3886-8.085-2.5891-21.656-2.813l-12.566-.20744v68.283 68.283h11.272c6.4378 0 12.37-.20109 13.832-.46892zm85.126-67.561v-85.354h13.304 13.304v75.847 75.847h31.675 31.675v9.5073 9.5072h-44.979-44.979v-85.354zm104.74-.21136v-85.566h44.768 44.768v8.6622 8.6622h-31.464-31.464v28.099 28.099h29.986 29.986v8.6622 8.6622h-29.986-29.986v31.268 31.268h32.52 32.52v8.8735 8.8735h-45.823-45.823v-85.566z\" /></g></svg></div><s class=\"l\"></s></div></div><!-- H-LAYOUT (full page) --><div class=\"pages-install v-layout-stretch h-layout\"><!-- SIDE-HEADER (fix vertical)--><div id=\"ID_STEPS\" class=\"themeSidebar sidebar h-layout-fix\"><div class=\"top\"></div><div id=\"ID_WIZZARD_SIDEBAR\"></div></div><!-- CONTENT (stretch vertical)--><div class=\"h-layout-stretch themeNormal scroll-container-static\" style=\"width:100%;height: 100%\"><div class=\"scrollable\"><div class=\"wizzard-container\"><div id=\"ID_WIZZARD_PAGE\"></div><div style=\"min-height: 2em;\"></div></div></div></div></div></div>','/pages/install/sidebar':'<div class=\"wizzard-sidebar\">HEY YA</div>','/pages/install/sidebar/item':'<div class=\"wizzard-item v-align-middle\"><i class=\"{{icon}}\"></i><s></s><span>{{text}}</span></div>','/pages/install/sidebar/item-checked':'<div class=\"wizzard-item v-align-middle\"><i class=\"{{icon}}\"></i><s></s><span>{{text}}</span><s class=\"s\"></s><span><i class=\"mdi mdi-check\"></i></span></div>','/pages/install/welcome':'<div class=\"welcome\" style=\"height: 26.5em\"><div class=\"mouth\"><img src=\"img/mouth.svg\"></img></div><div class=\"fg\"><div class=\"h-layout\"><div class=\"h-layout-stretch\"><h1>{$wizpage_welcome_title}</h1></div><div id=\"ID_LANGUAGE{##}\" class=\"h-layout-fix v-align-bottom link\"><span>{$wizpage_welcome_language}</span></div></div><hr><br/><p>{$wizpage_welcome_line1}</p><p>{$wizpage_welcome_line2}</p><p>{$wizpage_welcome_line3}</p><p>{$wizpage_welcome_line4}</p><p>{$wizpage_welcome_line5}</p><br/><br/><div id=\"ID_NEXT{##}\" class=\"button default hover\">{$wizpage_welcome_button}</div></div></div>','/pages/install/license':'<div class=\"license\"><h1>{$wizpage_license_title}</h1><hr><p>{$wizpage_license_help}</p><br /><div class=\"scroll-container eula-container\" style=\"width:100%; height: 20em\"><div class=\"scrollable eula\"><span id=\"ID_LICENSE_TEXT{##}\" class=\"pre-text\"></span></div></div><div id=\"ID_CHECK{##}\"></div><br /><div id=\"ID_PREVIOUS{##}\" class=\"button normal hover\">{$install_button_previous}</div><s class=\"s\"></s><div id=\"ID_NEXT{##}\" class=\"button\">{$install_button_next}</div></div>','/pages/install/network':'<div class=\"form\"><h1>{$wizpage_network_title}</h1><hr><p>{$wizpage_network_help}</p><br /><div id=\"ID_WIDGET{##}\"></div><br /><br /><br /><div id=\"ID_PREVIOUS{##}\" class=\"button normal hover\">{$install_button_previous}</div><s class=\"s\"></s><div id=\"ID_NEXT{##}\" class=\"button\">{$install_button_next}</div></div>','/pages/install/networkInterface':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"><h3><div id=\"ID_NETCARD_NAME{##}\" style=\"display: inline;\">{$wizpage_nic_name}</div><s></s><span id=\"ID_NETCARD_STEP{##}\"></span></h3><hr class=\"light\"/><br><!-- input type name --><help-topic data-help=\"/pages/install/networkInterface/type\"><div class=\"line\"><label>{$netcard_label_type}</label><div id=\'ID_NETCARD_TYPE{##}\'></div></div></help-topic><!-- ip address + mask + gw --><div data-id=\"ID_OPTIONAL_IP_SETTINGS\" class=\"optional-item\"><!-- input ip name --><help-topic data-help=\"/pages/install/networkInterface/ip\"><div class=\"line\"><label>{$netcard_label_ip}</label><div id=\'ID_NETCARD_IP{##}\'></div></div></help-topic><!-- input net mask name --><help-topic data-help=\"/pages/install/networkInterface/mask\"><div class=\"line\"><label>{$netcard_label_net_mask}</label><div id=\'ID_NETCARD_NET_MASK{##}\'></div></div></help-topic><!-- input gateway --><help-topic data-help=\"/pages/install/networkInterface/gateway\"><div class=\"line\"><label>{$netcard_label_default_gateway}</label><div id=\'ID_NETCARD_GATEWAY{##}\'></div></div></help-topic></div><!-- DCHP options form container --><div data-id=\"ID_OPTIONAL_DHCP_ACTIVE_SETTINGS\" class=\"optional-item\"><!-- input net mask name --><help-topic data-help=\"/pages/install/networkInterface/dhcpserver\"><div class=\"line\"><label>{$netcard_label_dhcp}</label><div id=\'ID_NETCARD_DHCP_ACTIV{##}\'></div></div></help-topic></div><!-- DCHP options form container --><div data-id=\"ID_OPTIONAL_DHCP_SETTINGS\" class=\"optional-item\"><!-- input dhcp pool start --><help-topic data-help=\"/pages/install/networkInterface/poolstart\"><div class=\"line\"><label>{$netcard_dhcp_pool_start}</label><div id=\'ID_NETCARD_DHCP_POOL_START{##}\'></div></div></help-topic><!-- input dhcp pool stop --><help-topic data-help=\"/pages/install/networkInterface/poolend\"><div class=\"line\"><label>{$netcard_dhcp_pool_end}</label><div id=\'ID_NETCARD_DHCP_POOL_END{##}\'></div></div></help-topic></div></div>','/pages/install/networkDns':'<h3><div style=\"display: inline;\">{$wizpage_dns_title}</div><s></s><span id=\"ID_NETCARD_STEP{##}\"></span></h3><hr class=\"light\" /><br><!-- input dns server address --><help-topic data-help=\"/pages/install/networkDns\"><div class=\"line\"><label>{$netcard_label_dns_server}</label><div id=\'ID_DNS_SERVER{##}\'></div><s class=\"s\"><span id=\"ID_DNS_SERVER_HINT{##}\"></span></s></div></help-topic>','/pages/install/timezone':'<div class=\"form\"><h1>{$wizpage_timezone_title}</h1><hr><p>{$wizpage_timezone_help}</p><br /><help-topic data-help=\"/pages/install/timezone/zone\"><div class=\"line\"><label>{$systemtime_zone_label}</label><div id=\"ID_ZONE_NEW{##}\"></div></div></help-topic><br /><br /><br /><div id=\"ID_PREVIOUS{##}\" class=\"button normal hover\">{$install_button_previous}</div><s class=\"s\"></s><div id=\"ID_NEXT{##}\" class=\"button\">{$install_button_next}</div></div>','/pages/install/password':'<div class=\"form\"><h1>{$wizpage_password_title}</h1><hr><p>{$wizpage_password_help}</p><br /><help-topic data-help=\"/pages/install/password/admin\"><div class=\"line\"><label>{$wizpage_password_admin}</label><div id=\"ID_ACCOUNT{##}\"></div></div></help-topic><help-topic data-help=\"/pages/install/password/password\"><div class=\"line\"><label>{$wizpage_password1}</label><div id=\"ID_PASSWORD1{##}\"></div></div></help-topic><help-topic data-help=\"/pages/install/password/password2\"><div class=\"line\"><label>{$wizpage_password2}</label><div id=\"ID_PASSWORD2{##}\"></div></div></help-topic><br /><br /><br /><div id=\"ID_PREVIOUS{##}\" class=\"button normal hover\">{$install_button_previous}</div><s class=\"s\"></s><div id=\"ID_NEXT{##}\" class=\"button\">{$install_button_next}</div></div>','/pages/install/result':'<div class=\"form\"><h1>{$wizpage_result_title}</h1><hr><br /><p>{$wizpage_result_help1}</p><p>{$wizpage_result_help2}</p><p>{$wizpage_result_help3}</p><br /><br /><br /><div id=\"ID_FINISH{##}\" class=\"button default hover\">{$wizpage_result_button_finish}</div></div>','/forms/system/dashboard':'<div class=\"v-layout\"><!-- TOOLBAR --><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form dashboard\"><div id=\"ID_OPTIONAL{##}\"><div id=\"ID_TOAST{##}\"></div><!-- FORM --><div class=\"h-layout\"><div class=\"h-layout-stretch\"><div class=\'groupbox\'><h3>{$dashboard_group_messages}</h3><div class=\"line\"><div id=\"ID_MESSAGE_GRID{##}\"></div></div></div><div class=\'groupbox\'><h3>{$dashboard_group_information}</h3><help-topic data-help=\"/forms/system/dashboard/hwid\"><div class=\"line\"><label>{$dashboard_info_hardwareid}</label><div id=\'ID_HARDWAREID{##}\'></div><s class=\"s\"></s></div></help-topic><help-topic data-help=\"/forms/system/dashboard/version\"><div class=\"line\"><label>{$dashboard_info_version}</label><div id=\'ID_VERSION{##}\'></div></div></help-topic><help-topic data-help=\"/forms/system/dashboard/numdev\"><div class=\"line\"><label>{$dashboard_info_numberdevices}</label><div id=\'ID_NUMBERDEVICES{##}\'></div></div></help-topic><help-topic data-help=\"/forms/system/supportcontract\"><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_SUPPORTCONTRACT\"><label>{$dashboard_info_supportcontract}</label><div id=\'ID_SUPPORTCONTRACT{##}\'></div></div></help-topic><help-topic data-help=\"/forms/system/dashboard/demotime\"><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_DEMOTIME\"><label>{$dashboard_info_demotime}</label><div id=\'ID_DEMOTIME{##}\'></div></div></help-topic><help-topic data-help=\"/forms/system/dashboard/susstatus\"><div class=\"line\"><label>{$dashboard_info_susstatus}</label><div id=\'ID_SUSSTATUS{##}\'></div></div></help-topic></div></div><!-- function bar --><div class=\"h-layout-fix\" style=\"padding-left:2em\"><div class=\'groupbox\' style=\"width: 17em;height: 33.5em\"><h3>{$dashboard_group_functions}</h3><div class=\"dashboard-header readonly\"><span>{$dashboard_menu_general}</span></div><div id=\"ID_SUPPORTINFO{##}\" class=\"dashboard-link hover\"><i class=\"mdi mdi-pill\"></i><s class=\"s\"></s><span>{$dashboard_menu_supportinfo}</span></div><div id=\"ID_EXIT{##}\" class=\"dashboard-link hover\"><i class=\"mdi mdi-exit-to-app\"></i><s class=\"s\"></s><span>{$dashboard_menu_exit}</span></div><br><div class=\"dashboard-header readonly\"><span>{$dashboard_menu_siedledevices}</span></div><div id=\"ID_DEVICE_RESTART_ALL{##}\" class=\"dashboard-link hover\"><i class=\"mdi mdi-restart\"></i><s class=\"s\"></s><span>{$dashboard_menu_device_restartall}</span></div><div id=\"ID_DEVICE_RESTART_NEEDED{##}\" class=\"dashboard-link hover\"><i class=\"mdi mdi-restart\"></i><s class=\"s\"></s><span>{$dashboard_menu_device_restartneeded}</span></div><br><div class=\"dashboard-header readonly\"><span>{$dashboard_menu_server}</span></div><div id=\"ID_RESTART_SERVICE{##}\" class=\"dashboard-link hover\"><i class=\"mdi mdi-power\"></i><s class=\"s\"></s><span>{$dashboard_menu_restart_service}</span></div><div id=\"ID_REBOOT{##}\" class=\"dashboard-link hover\"><i class=\"mdi mdi-desktop-tower-monitor\"></i><s class=\"s\"></s><span>{$dashboard_menu_reboot}</span></div></div></div></div><!-- h-layout--></div><!-- optional --></div><!-- scrollable --></div><!-- scroll-container-->','/forms/system/presence':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><!-- search/filter controls --><div class=\"line\"><label class=\"presence-span\">{$presence_filter_label}</label><div id=\"ID_FILTER_CHECK_ONLINE{##}\"></div><s></s><div id=\"ID_FILTER_CHECK_OFFLINE{##}\"></div><s class=\"l\"></s><s class=\"l\"></s><s class=\"l\"></s><label class=\"presence-span\">{$const_common_search_placeholder}</label><div id=\"ID_FILTER_SEARCH{##}\"></div></div><!-- grid --><div class=\"line\"><div id=\"ID_PRESENCE_GRID{##}\"></div></div></div></div></div></div>','/forms/system/basicparam':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><div id=\"ID_OPTIONAL{##}\" class=\"optional\"><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><help-topic data-help=\"/forms/system/basicparam/syslang\"><div class=\"line\"><label>{$basicparam_label_syslang}</label><div id=\'ID_SYSLANG{##}\'></div></div></help-topic><help-topic data-help=\"/forms/system/basicparam/netcard\"><div class=\"line\"><label>{$basicparam_label_netcardgrid}</label><div id=\'ID_NETCARD_GRID{##}\'></div></div></help-topic></div><!-- device outage alarm --><div class=\'groupbox optional-item\' data-id=\"ID_OPTIONAL_DEVICEOUTAGE\"><h3>{$basicparam_box_deviceoutage}</h3><help-topic data-help=\"/forms/system/basicparam/deviceoutage_time\"><div class=\"line\"><label>{$basicparam_label_deviceoutage_time}</label><div id=\'ID_DEVICEOUTAGE_TIME{##}\'></div><s class=\"s\"></s><span>{$basicparam_postlabel_deviceoutage_time}</span><s class=\"s\"></s><span class=\"message-text\"></span></div></help-topic><help-topic data-help=\"/forms/system/basicparam/deviceoutage_contact\"><div class=\"line\"><label>{$basicparam_label_deviceoutage_contact}</label><div id=\'ID_DEVICEOUTAGE_CONTACT{##}\'></div></div></help-topic></div><!-- times (door) --><div class=\'groupbox\'><h3>{$basicparam_box_title_times_door}</h3><help-topic data-help=\"/forms/system/basicparam/doorringtimeout\"><div class=\"line\"><label>{$basicparam_label_door_ring_timeout}</label><div id=\'ID_DOOR_RING_TIMEOUT{##}\'></div><s class=\"s\"></s><span>{$basicparam_postlabel_seconds}</span><s class=\"s\"></s><span class=\"message-text\"></span></div></help-topic><help-topic data-help=\"/forms/system/basicparam/doorcalltimeout\"><div class=\"line\"><label>{$basicparam_label_door_call_timeout}</label><div id=\'ID_DOOR_CALL_TIMEOUT{##}\'></div><s class=\"s\"></s><span>{$basicparam_postlabel_minutes}</span><s class=\"s\"></s><span class=\"message-text\"></span></div></help-topic><help-topic data-help=\"/forms/system/basicparam/latevideotimeout\"><div class=\"line\"><label>{$basicparam_label_latevideo_timeout}</label><div id=\'ID_LATEVIDEO_TIMEOUT{##}\'></div><s class=\"s\"></s><span>{$basicparam_postlabel_seconds}</span><s class=\"s\"></s><span class=\"message-text\"></span></div></help-topic></div><!-- times (general) --><div class=\'groupbox\'><h3>{$basicparam_box_title_times_general}</h3><help-topic data-help=\"/forms/system/basicparam/ringtimeout\"><div class=\"line\"><label>{$basicparam_label_ring_timeout}</label><div id=\'ID_RING_TIMEOUT{##}\'></div><s class=\"s\"></s><span>{$basicparam_postlabel_seconds}</span><s class=\"s\"></s><span class=\"message-text\"></span></div></help-topic><help-topic data-help=\"/forms/system/basicparam/calltimeout\"><div class=\"line\"><label>{$basicparam_label_call_timeout}</label><div id=\'ID_CALL_TIMEOUT{##}\'></div><s class=\"s\"></s><span>{$basicparam_postlabel_minutes}</span><s class=\"s\"></s><span class=\"message-text\"></span></div></help-topic></div><!-- times (lists) --><div class=\'groupbox\'><h3>{$basicparam_box_title_times_lists}</h3><help-topic data-help=\"/forms/system/basicparam/grabpic\"><div class=\"line\"><label>{$basicparam_label_lists_time_grabpic}</label><div id=\'ID_LISTS_TIME_GRABPIC{##}\'></div><s class=\"s\"></s><span>{$basicparam_postlabel_days}</span><s class=\"s\"></s><span class=\"message-text\"></span></div></help-topic><help-topic data-help=\"/forms/system/basicparam/timecdr\"><div class=\"line\"><label>{$basicparam_label_lists_time_cdr}</label><div id=\'ID_LISTS_TIME_CDR{##}\'></div><s class=\"s\"></s><span>{$basicparam_postlabel_days}</span><s class=\"s\"></s><span class=\"message-text\"></span></div></help-topic><help-topic data-help=\"/forms/system/basicparam/timevoicemail\"><div class=\"line\"><label>{$basicparam_label_lists_time_voicemail}</label><div id=\'ID_LISTS_TIME_VOICEMAIL{##}\'></div><s class=\"s\"></s><span>{$basicparam_postlabel_days}</span><s class=\"s\"></s><span class=\"message-text\"></span></div></help-topic></div><br><br><br></div><!-- OPTIONAL --></div></div></div>','/forms/system/logfile':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><!-- grid --><div class=\"line\"><div id=\"ID_LOGFILE_GRID{##}\"></div></div></div></div></div></div>','/forms/system/callnumberplan':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><!-- PROFILES --><help-topic data-help=\"/forms/system/callnumberplan/profiles\"><div class=\'line\'><label>{$callnumberplan_label_profiles}</label><div id=\'ID_PROFILES{##}\' class=\'normal\'></div></div><div class=\"line\"><label class=\"none\"></label><div class=\"button\" id=\"ID_PROFILE_ADD{##}\"></div><s class=\"s\"></s><div class=\"button\" id=\"ID_PROFILE_EDIT{##}\"></div><s class=\"s\"></s><div class=\"button\" id=\"ID_PROFILE_DELETE{##}\"></div><s class=\"s\"></s></div></help-topic><div class=\"line\"></div><!-- PERMISSIONS --><help-topic data-help=\"/forms/system/callnumberplan/permissions\"><div class=\"line\"><label>{$callnumberplan_label_permissions}</label><div id=\"ID_PERMISSIONS{##}\"></div></div><div class=\"line\"><label class=\"none\"></label><div class=\"button\" id=\"ID_PERMISSION_ADD{##}\"></div><s class=\"s\"></s><div class=\"button\" id=\"ID_PERMISSION_EDIT{##}\"></div><s class=\"s\"></s><div class=\"button\" id=\"ID_PERMISSION_DELETE{##}\"></div><s class=\"s\"></s></div></help-topic></div></div></div></div>','/forms/system/backup':'<div class=\"v-layout\"><!-- TOOLBAR --><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><div class=\'groupbox\'><h3>{$backup_group_backup}</h3><help-topic data-help=\"/forms/system/backup/create\"><div class=\"line\"><label>{$backup_backup_label}</label><div id=\"ID_BACKUP{##}\"></div></div></help-topic></div><div class=\'groupbox\'><h3>{$backup_group_restore}</h3><help-topic data-help=\"/forms/system/backup/upload\"><div class=\"line\"><label>{$backup_restore_label}</label><div id=\"ID_RESTORE_UPLOAD{##}\"></div></div></help-topic><help-topic data-help=\"/forms/system/backup/file\"><div class=\"line\"><label>{$backup_restore_file}</label><div id=\"ID_RESTORE_FILE{##}\"></div></div></help-topic><br/><br/><div class=\"line\"><label></label><div id=\"ID_RESTORE_BUTTON{##}\"></div></div><div class=\"line\"><label></label><div style=\"width:520px\"><em>{$backup_restore_help}</em></div></div></div></div></div></div>','/forms/system/searchForm':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><!-- grid --><div class=\"line\"><div id=\"ID_SEARCH_GRID{##}\"></div></div></div></div></div></div>','/forms/system/licenses':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_OPTIONAL{##}\"><div id=\"ID_TOAST{##}\"></div><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><!-- demo period --><help-topic data-help=\"/forms/system/licenses/demopriod\"><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_DEMOPERIOD\"><label>{$license_demoperiod_label}</label><div id=\"ID_DEMOPERIOD{##}\"></div></div></help-topic><!-- end of support contract --><help-topic data-help=\"/forms/system/supportcontract\"><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_SUPPORTCONTRACT\"><label>{$license_supportcontract_label}</label><div id=\"ID_SUPPORTCONTRACT{##}\"></div></div></help-topic><!-- grid --><help-topic data-help=\"/forms/system/licenses/grid\"><div class=\"line\"><label>{$license_grid_label}</label><div id=\"ID_LICENSES_GRID{##}\"></div></div></help-topic></div><div class=\'groupbox\'><h3>{$license_box_upload}</h3><!-- demo period --><help-topic data-help=\"/forms/system/licenses/upload\"><div class=\"line\"><label>{$license_upload_label}</label><div id=\"ID_UPLOAD{##}\"></div></div></help-topic></div></div><!-- optional --></div><!-- scrollable --></div></div>','/forms/system/rules':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><!-- grid --><help-topic data-help=\"/forms/system/rules/grid\"><div class=\"line\"><label>{$rule_grid_label}</label><div id=\"ID_RULES_GRID{##}\"></div></div></help-topic><div class=\"line\"><label></label><div id=\"ID_RULE_DELETE{##}\">Delete</div></div></div><div class=\'groupbox\'><h3>{$rules_box_upload}</h3><!-- upload control --><help-topic data-help=\"/forms/system/rules/upload\"><div class=\"line\"><label>{$rule_upload_label}</label><div id=\"ID_UPLOAD{##}\"></div></div></help-topic></div></div></div></div>','/forms/system/systemtime':'<div class=\"v-layout\"><!-- TOOLBAR --><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_OPTIONAL{##}\"><div id=\"ID_TOAST{##}\"></div><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><help-topic data-help=\"/forms/system/systemtime/srvtime\"><div class=\"line\"><label>{$systemtime_srvtime_label}</label><div id=\"ID_SRVTIME{##}\"></div><s class=\"s\"></s><span id=\"ID_SRVTIME_POST{##}\"></span></div></help-topic><help-topic data-help=\"/forms/system/systemtime/clitime\"><div class=\"line\"><label>{$systemtime_clienttime_label}</label><div id=\"ID_CLIENTTIME{##}\"></div><s class=\"s\"></s><span id=\"ID_CLIENTTIME_POST{##}\"></span></div></help-topic><help-topic data-help=\"/forms/system/systemtime/zone\"><div class=\"line\"><label>{$systemtime_zone_label}</label><div id=\"ID_ZONE_NEW{##}\"></div></div></help-topic></div><div class=\'groupbox\'><h3>{$systemtime_timeserver_box}</h3><help-topic data-help=\"/forms/system/systemtime/timemode\"><div class=\"line\"><label>{$systemtime_timemode_label}</label><div id=\"ID_TIMEMODE{##}\"></div></div></help-topic><div class=\"optional-item\" data-id=\"ID_OPTION_NTP\"><help-topic data-help=\"/forms/system/systemtime/ntp\"><div class=\"line\"><label>{$systemtime_ntplist_label}</label><div id=\"ID_NTP_LIST{##}\"></div></div><div class=\"line\"><label></label><div id=\"ID_NTP_ADD{##}\"></div><s class=\"s\"></s><div id=\"ID_NTP_EDIT{##}\"></div><s class=\"s\"></s><div id=\"ID_NTP_DELETE{##}\"></div><s class=\"s\"></s><div id=\"ID_NTP_UP{##}\"></div><s class=\"s\"></s><div id=\"ID_NTP_DOWN{##}\"></div><s class=\"s\"></s></div></help-topic><br/><help-topic data-help=\"/forms/system/systemtime/ntpstat\"><div class=\"line\"><label>{$systemtime_ntpstat_label}</label><div class=\"v-layout\"><span class=\"v-layout-fix\" id=\"ID_NTP_STATISTIC{##}\"></span><span class=\"v-layout-fix\" id=\"ID_NTP_STATISTIC2{##}\"></span><span class=\"v-layout-fix\" id=\"ID_NTP_STATISTIC3{##}\"></span></div></div></help-topic></div><div class=\"optional-item\" data-id=\"ID_OPTION_MANUAL\"><br/><div class=\"line\"><label>{$systemtime_timesend_label}</label><div id=\"ID_TIME_SEND{##}\"></div></div><div class=\"line\"><label></label><span><em>{$systemtime_timesend_help}</em></span></div></div></div></div></div></div></div>','/forms/network/network':'<div class=\"v-layout\"><!-- TOOLBAR --><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><!-- TOAST --><div id=\"ID_TOAST{##}\"></div><!-- Common DNS Server --><div id=\"ID_DNS_ANCHOR{##}\"></div><!-- ANCHOR FOR GROUPS PER NETCARD --><div id=\"ID_NETCARD_ANCHOR{##}\"></div><br><br><br></div><!-- scrollable --></div><!-- scroll-container--></div>','/forms/system/network/widgets/controls/dns_config':'<!-- modus --><help-topic data-help=\"/forms/system/network/widgets/controls/dns_config/mode\"><div class=\"line\"><label for=\"ID_DNS_CONFIG_MODUS{##}\">{$dns_config_label_modus}</label><div id=\"ID_DNS_CONFIG_MODUS{##}\"></div></div></help-topic><!-- dns box --><help-topic data-help=\"/forms/system/network/widgets/controls/dns_config/list\"><div class=\"multiline\"><label for=\"ID_DNS_CONFIG_BOX{##}\">{$dns_config_server}</label><div id=\'ID_DNS_CONFIG_BOX{##}\'></div><s class=\"s\"></s><div class=\"\"></div></div><div class=\"line\"><label class=\"none\"></label><div class=\"button\" id=\"ID_DNS_CONFIG_ADD{##}\"></div><s class=\"s\"></s><div class=\"button\" id=\"ID_DNS_CONFIG_EDIT{##}\"></div><s class=\"s\"></s><div class=\"button\" id=\"ID_DNS_CONFIG_DELETE{##}\"></div><s class=\"s\"></s><div class=\"button\" id=\"ID_DNS_CONFIG_UP{##}\"></div><s class=\"s\"></s><div class=\"button\" id=\"ID_DNS_CONFIG_DOWN{##}\"></div><s class=\"s\"></s></div></help-topic>','/forms/system/network/widgets/controls/netcard':'<h3><i id=\"ID_HEADLINE_STATUS{##}\" class=\"mdi mdi-circle\"></i>&nbsp;&nbsp;{$network_netcard_title_prefix}&nbsp;&quot;<span id=\"ID_HEADLINE_NAME{##}\"></span>&quot;</h3><!-- input netcard name --><help-topic data-help=\"/forms/system/network/widgets/controls/netcard/name\"><div class=\"line\"><label for=\"ID_NETCARD_NAME{##}\">{$netcard_label_name}</label><div id=\'ID_NETCARD_NAME{##}\'></div><s></s><div><span>(\"</span><span id=\"ID_NETCARD_SYSTEM_NAME{##}\"></span><span>\")</span></div></div></help-topic><!-- input type name --><help-topic data-help=\"/forms/system/network/widgets/controls/netcard/type\"><div class=\"multiline\"><label for=\"ID_NETCARD_TYPE{##}\">{$netcard_label_type}</label><div id=\'ID_NETCARD_TYPE{##}\'></div></div></help-topic><div id=\"FORM_STATUS_GROUP_DHCP{##}\" class=\"\"><!-- input ip name --><help-topic data-help=\"/forms/system/network/widgets/controls/netcard/ip\"><div class=\"line\"><label for=\"ID_NETCARD_IP{##}\">{$netcard_label_ip}</label><div id=\'ID_NETCARD_IP{##}\'></div></div></help-topic><!-- input net mask name --><help-topic data-help=\"/forms/system/network/widgets/controls/netcard/mask\"><div class=\"line\"><label for=\"ID_NETCARD_NET_MASK{##}\">{$netcard_label_net_mask}</label><div id=\'ID_NETCARD_NET_MASK{##}\'></div></div></help-topic><!-- input gateway --><help-topic data-help=\"/forms/system/network/widgets/controls/netcard/gw\"><div class=\"line\"><label for=\"ID_NETCARD_GATEWAY{##}\">{$netcard_label_default_gateway}</label><div id=\'ID_NETCARD_GATEWAY{##}\'></div></div></help-topic></div><!-- DCHP options form container --><div id=\"ID_OPTIONAL_DHCP_ACTIVE{##}\" class=\"optional\"><div id=\"ID_OPTIONAL_DHCP_ACTIVE_SETTINGS{##}\" data-id=\"ID_OPTIONAL_DHCP_ACTIVE_SETTINGS\" class=\"optional-item\"><!-- input net mask name --><help-topic data-help=\"/forms/system/network/widgets/controls/netcard/dhcp\"><div class=\"line\"><label for=\"ID_NETCARD_DHCP_ACTIV{##}\">{$netcard_label_dhcp}</label><div id=\'ID_NETCARD_DHCP_ACTIV{##}\'></div></div></help-topic></div></div><!-- DCHP options form container --><div id=\"ID_OPTIONAL_DHCP{##}\" class=\"optional\"><div id=\"ID_OPTIONAL_DHCP_SETTINGS{##}\" data-id=\"ID_OPTIONAL_DHCP_SETTINGS\" class=\"optional-item\"><!-- input dhcp pool start --><help-topic data-help=\"/forms/system/network/widgets/controls/netcard/poolstart\"><div class=\"line\"><label for=\"ID_NETCARD_DHCP_POOL_START{##}\">{$netcard_dhcp_pool_start}</label><div id=\'ID_NETCARD_DHCP_POOL_START{##}\'></div></div></help-topic><!-- input dhcp pool stop --><help-topic data-help=\"/forms/system/network/widgets/controls/netcard/poolend\"><div class=\"line\"><label for=\"ID_NETCARD_DHCP_POOL_END{##}\">{$netcard_dhcp_pool_end}</label><div id=\'ID_NETCARD_DHCP_POOL_END{##}\'></div></div></help-topic><!-- button options --><help-topic data-help=\"/forms/system/network/widgets/controls/netcard/dhcpopt\"><div class=\"line\"><label for=\"ID_NETCARD_BUTTON_OPTIONS{##}\">{$netcard_label_button_options}</label><div class=\"button\" id=\"ID_NETCARD_BUTTON_OPTIONS{##}\">{$netcard_button_options}</div><s class=\"s\"></s><span class=\"s red-text\" style=\"display:none; font-style:italic\" id=\"ID_NETCARD_BUTTON_OPTIONS_HINT{##}\">{$netcard_warning_options_changed}</span></div></help-topic></div></div>','/forms/system/network/dialog/changeIpAddress':'<div class=\"dialogform\"><!-- header --><h2><i class=\"mdi mdi-lan\"></i><s class=\"s\"></s><span id=\"ID_TITLE{##}\" class=\"xl\">{$change_ip_dialog_title_new}</span></h2><hr/><div class=\"group\"><!-- row 1 --><div class=\"line\"><label>{$change_ip_dialog_label_ip}</label><div id=\"ID_DNS_IP_ADDRESS{##}\"></div></div><label class=\"none\"></label><div class=\"dialog-error error-text\"></div></div></div>','/dialog/dhcpOptions':'<div class=\"dialog-header\"><h2 class=\"v-align-middle\"><i class=\"mdi mdi-check-network-outline\"></i><s class=\"s\"></s>DHCP-Server: \"Access\"</h2></div><div class=\"h-layout\" style=\"height:100%\"><div class=\"dhcp-navigation h-layout-fix\"><div id=\"ID_SIDEBAR{##}\"></div></div><div class=\"h-layout-stretch scroll-container\"><div id=\"ID_CONTENT{##}\" class=\"dhcp-content scrollable dialogform optional\"><!-- form general --><div class=\"optional-item\" data-id=\"ID_GENERAL\"><help-topic data-help=\"/dialog/dhcpOptions/ipaddr\"><div class=\"line\"><label>{$dhcp_ipaddr}</label><div id=\"ID_IPADDRESS{##}\"></div></div></help-topic><help-topic data-help=\"/dialog/dhcpOptions/mask\"><div class=\"line\"><label>{$dhcp_subnetmask}</label><div id=\"ID_SUBNETMASK{##}\"></div></div></help-topic><div class=\"group\"><help-topic data-help=\"/dialog/dhcpOptions/scopestart\"><div class=\"line\"><label>{$dhcp_scope_start}</label><div id=\"ID_SCOPE_START{##}\"></div></div></help-topic><help-topic data-help=\"/dialog/dhcpOptions/scopeend\"><div class=\"line\"><label>{$dhcp_scope_end}</label><div id=\"ID_SCOPE_END{##}\"></div></div></help-topic></div></div><!-- form options --><div class=\"optional-item\" data-id=\"ID_OPTIONS\"><!-- options list (grid) --><div class=\"line\"><div id=\"ID_DHCP_OPTIONLIST{##}\"></div></div><!-- options edit controls --><div id=\"ID_OPTIONS{##}\" class=\"group\"><help-topic data-help=\"/dialog/dhcpOptions/enabled\"><div class=\"line\"><label>{$dhcp_option_enabled}</label><div id=\"ID_DHCP_OPTIONENABLE{##}\"></div></div></help-topic><help-topic data-help=\"/dialog/dhcpOptions/optname\"><div class=\"line\"><label>{$dhcp_option_name}</label><div id=\"ID_DHCP_OPTIONNAME{##}\"></div></div></help-topic><help-topic data-help=\"/dialog/dhcpOptions/value\"><div class=\"line\"><label>{$dhcp_option_value}</label><div id=\"ID_DHCP_OPTIONVALUE{##}\"></div><s class=\"s\"></s><div id=\"ID_DHCP_OPTIONVALUE_AUTO{##}\"></div><s class=\"s\"></s><div class=\"message-text\"></div></div><div class=\"line\"><label class=\"none\"></label><div id=\"ID_DHCP_OPTIONEDIT_CANCEL{##}\"></div><s class=\"s\"></s><div id=\"ID_DHCP_OPTIONEDIT_APPLY{##}\"></div></div></help-topic></div></div><!-- form leases --><div class=\"optional-item\" data-id=\"ID_LEASES\"><div class=\"line\"><div id=\"ID_DHCP_LEASELIST{##}\"></div></div><help-topic data-help=\"/dialog/dhcpOptions/leases\"><div class=\"line\"><label>{$dhcp_lease_action_label}</label><div id=\"ID_DHCP_LEASE_DELETE{##}\"></div><s class=\"s\"></s><div id=\"ID_DHCP_LEASE_RESERVE{##}\"></div></div></help-topic></div><!-- form reservations --><div class=\"optional-item\" data-id=\"ID_RESERVATIONS\"><div class=\"line\"><div id=\"ID_DHCP_RESERVATIONLIST{##}\"></div></div><help-topic data-help=\"/dialog/dhcpOptions/reservations\"><div class=\"line\"><label>{$dhcp_reserve_action_label}</label><div id=\"ID_DHCP_RESERVATION_NEW{##}\"></div><s class=\"s\"></s><div id=\"ID_DHCP_RESERVATION_DELETE{##}\"></div></div></help-topic><div><help-topic data-help=\"/dialog/dhcpOptions/reservationmac\"><div class=\"line\"><label>{$dhcp_reserve_mac_label}</label><div id=\"ID_RESERVATION_MAC{##}\"></div></div></help-topic><help-topic data-help=\"/dialog/dhcpOptions/reservationip\"><div class=\"line\"><label>{$dhcp_reserve_ip_label}</label><div id=\"ID_RESERVATION_IP{##}\"></div></div><div class=\"line\"><label class=\"none\"></label><div id=\"ID_RESERVATION_CANCEL{##}\"></div><s class=\"s\"></s><div id=\"ID_RESERVATION_APPLY{##}\"></div></div></help-topic></div></div></div></div></div>','/forms/system/ruleDebug':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><div class=\"v-layout-stretch\"><div class=\"h-layout ruleDebug\"><div class=\"h-layout-stretch \"><!-- SCROLLABLE --><div class=\"scroll-container-static pageContent themeContent\"><div class=\"scrollable form rule-main form\"><div class=\"line\"><label>{$ruledbg_label_instances}</label><div id=\"ID_RULESLIST{##}\" class=\"no-focus\"></div></div><div class=\"line\"><label></label><div id=\"ID_RULEWATCH{##}\"></div><s class=\"s\"></s><div id=\"ID_RULE_NOWATCH{##}\"></div></div><div class=\"line\"></div><div><span>{$ruledbg_label_currentinst}</span></div><br><div id=\"ID_RULESSOURCE{##}\"></div></div></div></div><div class=\"h-layout-fix rule-sidebar\"><!-- <div class=\"rule-sidebar-header\"><i class=\"mdi mdi-sitemap\"></i><s class=\"s\"></s><span>{$ruledbg_label_details}</span></div><div class=\"rule-sidebar-details\"></div>--><div class=\"rule-sidebar-header\"><i class=\"mdi mdi-list-status\"></i><s class=\"s\"></s><span>{$ruledbg_label_events}</span><span style=\"width:100%\"><span class=\"h-align-right\"><i id=\"ID_SEQUENCES_DELETE{##}\" class=\"rule-sidebar-button mdi mdi-delete\"></i></span></span></div><div class=\"rule-sidebar-sequences\" style=\"width: 100%; height: 100%\"><div id=\"ID_RULE_SEQUENCES{##}\"></div></div></div></div></div></div>','/forms/system/ruleDebugSource':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"><div class=\"ruleDebugSource optional-item\" data-id=\"ID_OPTIONAL_SOURCE\"><div class=\"rule-src-container\"></div></div><div class=\"line\"></div><div class=\"parameter-container optional-item\" data-id=\"ID_OPTIONAL_PARAMETER\"><div><span>Parameter</span></div><div id=\"ID_JSONCONTROL{##}\">JSON</div></div></div>','/forms/system/api':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_OPTIONAL{##}\"><div id=\"ID_TOAST{##}\"></div><div class=\'groupbox optional-item\' data-id=\"ID_OPTIONAL_ONVIF\"><h3>{$api_group_onvif}</h3><help-topic data-help=\"/forms/system/api/enable\"><div class=\"line\"><label>{$api_label_onvif_enable}</label><div id=\"ID_ONVIF_ENABLE{##}\"></div></div></help-topic><help-topic data-help=\"/forms/system/api/onvifuser\"><div class=\"line\"><label>{$api_label_onvif_user}</label><div id=\"ID_ONVIF_USER{##}\"></div></div></help-topic><div id=\"ID_ONVIF_PASSWORD{##}\"></div></div></div><!-- optional --></div><!-- scrollable --></div><!-- scroll-container --></div>','/forms/general/folder':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/folder/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic></div></div></div></div>','/forms/general/apartment':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><!-- group general --><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/apartment/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/apartment/devtype\"><div class=\"line\"><label>{$formbase_label_device_type}</label><div id=\'ID_DEVICE_TYPE{##}\'></div></div></help-topic><div id=\"ID_CALLNUMBER{##}\"></div><div id=\"ID_MACADDR{##}\"></div><div id=\'ID_ADDRBOOK{##}\'></div><div id=\'ID_CALLPROFILE{##}\'></div><div id=\'ID_MOUNTTYPE{##}\'></div><div id=\"ID_VOLUME{##}\"></div><div id=\'ID_SHOWCLOCK{##}\'></div><div id=\'ID_CALLWAITING{##}\'></div><div id=\'ID_RINGTONE{##}\'></div></div><!-- group keys --><div class=\'groupbox\'><h3>{$formbase_group_devicekeys}</h3><div id=\"ID_DEVGROUP_KBD{##}\"></div></div><!-- group io --><div class=\'groupbox\'><h3>{$formbase_group_deviceio}</h3><div id=\"ID_DEVGROUP_IO{##}\"></div></div><!-- group functions --><div class=\'groupbox\' style=\"min-height: 12em\"><h3>{$formbase_group_functions}</h3><div id=\"ID_DEVGROUP_FUNCTION{##}\"></div></div></div></div></div>','/forms/general/tkphone':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><!-- group general --><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/tkphone/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/tkphone/devtype\"><div class=\"line\"><label>{$formbase_label_device_type}</label><div id=\'ID_DEVICE_TYPE{##}\'></div></div></help-topic><div id=\"ID_CALLNUMBER{##}\"></div><div id=\"ID_EXTERN_CALLNUMBER{##}\"></div><div id=\'ID_ADDRBOOK{##}\'></div><help-topic data-help=\"/forms/general/tkphone/gw\"><div class=\"line\"><label>{$tkphone_label_gateway}</label><div id=\'ID_GATEWAY{##}\'></div></div></help-topic><div id=\'ID_CALLPROFILE{##}\'></div><div id=\"ID_DTMF{##}\"></div></div><!-- group functions --><div class=\'groupbox\' style=\"min-height: 12em\"><h3>{$formbase_group_functions}</h3><div id=\"ID_DEVGROUP_FUNCTION{##}\"></div></div></div></div></div>','/forms/general/avp':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><!-- group general --><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/avp/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/avp/devtype\"><div class=\"line\"><label>{$formbase_label_device_type}</label><div id=\'ID_DEVICE_TYPE{##}\'></div></div></help-topic><div id=\"ID_CALLNUMBER{##}\"></div><div id=\"ID_MACADDR{##}\"></div><div id=\'ID_ADDRBOOK{##}\'></div><div id=\'ID_CALLPROFILE{##}\'></div><div id=\'ID_MOUNTTYPE{##}\'></div><div id=\"ID_VOLUME{##}\"></div><div id=\'ID_SHOWCLOCK{##}\'></div><div id=\'ID_CALLWAITING{##}\'></div><div id=\'ID_RINGTONE{##}\'></div></div><!-- group keys --><div class=\'groupbox\'><h3>{$formbase_group_devicekeys}</h3><div id=\"ID_DEVGROUP_KBD{##}\"></div></div><!-- group io --><div class=\'groupbox\'><h3>{$formbase_group_deviceio}</h3><div id=\"ID_DEVGROUP_IO{##}\"></div></div><!-- group functions --><div class=\'groupbox\' style=\"min-height: 12em\"><h3>{$formbase_group_functions}</h3><div id=\"ID_DEVGROUP_FUNCTION{##}\"></div></div></div></div></div>','/forms/general/door':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><!-- group general --><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/door/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/door/devtype\"><div class=\"line\"><label>{$formbase_label_device_type}</label><div id=\'ID_DEVICE_TYPE{##}\'></div></div></help-topic><div id=\"ID_CALLNUMBER{##}\"></div><div id=\"ID_MACADDR{##}\"></div><div id=\'ID_CALLPROFILE{##}\'></div><div id=\'ID_DOORCAMERA{##}\'></div><div id=\"ID_VOLUME{##}\"></div><div id=\"ID_KEYTONES{##}\"></div><div id=\"ID_ZAM{##}\"></div></div><!-- group keys --><div class=\'groupbox\'><h3>{$formbase_group_devicekeys}</h3><div id=\"ID_DEVGROUP_KBD{##}\"></div></div><!-- group io --><div class=\'groupbox\'><h3>{$formbase_group_deviceio}</h3><div id=\"ID_DEVGROUP_IO{##}\"></div></div><!-- group functions --><div class=\'groupbox\' style=\"min-height: 12em\"><h3>{$formbase_group_functions}</h3><div id=\"ID_DEVGROUP_FUNCTION{##}\"></div></div></div></div></div>','/forms/general/tkgw':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><div id=\"ID_OPTIONAL{##}\" class=\"optional\"><!-- group general --><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/tkgw/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/tkgw/devtype\"><div class=\"line\"><label>{$formbase_label_device_type}</label><div id=\'ID_DEVICE_TYPE{##}\'></div></div></help-topic><div id=\'ID_PREFIX{##}\'></div><help-topic data-help=\"/forms/general/tkgw/incoming\"><div class=\"line\"><label>{$tkgw_label_incoming}</label><div id=\'ID_INCOMING{##}\'></div></div></help-topic><!-- Dropdown for \'incoming call forward\' --><div class=\"optional-item\" data-id=\"ID_OPTIONAL_INCOMING_FORWARD_TO\"><help-topic data-help=\"/forms/general/tkgw/callfwd\"><div class=\"line\"><label></label><div id=\"ID_INCOMING_FORWARD_TO{##}\"></div></div></help-topic></div><div id=\"ID_DTMF{##}\"></div><div class=\"line\"><label></label><div id=\"ID_DTMF_FROM_DOOR{##}\"></div></div></div><!-- group gateway --><div class=\'groupbox\' style=\"min-height: 12em\"><h3>{$tkgw_box_title_gateway}</h3><help-topic data-help=\"/forms/general/tkgw/registrar\"><div class=\"line\"><label>{$tkgw_label_registrar}</label><div id=\'ID_REGISTRAR{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/tkgw/account\"><div class=\"line\"><label>{$tkgw_label_account}</label><div id=\'ID_ACCOUNT{##}\'></div></div></help-topic><div id=\"ID_PASSWORD{##}\"></div><help-topic data-help=\"/forms/general/tkgw/framesize\"><div class=\"line\"><label>{$tkgw_label_framesize}</label><div id=\'ID_FRAMESIZE{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/tkgw/transport\"><div class=\"line\"><label>{$tkgw_label_transport}</label><div id=\'ID_TRANSPORT{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/tkgw/port\"><div class=\"line\"><label>{$tkgw_label_port}</label><div id=\'ID_PORT{##}\'></div></div></help-topic></div></div><!-- ID_OPTIONAL --></div></div></div>','/forms/general/trunk':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><!-- group general --><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/trunk/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/trunk/devtype\"><div class=\"line\"><label>{$formbase_label_device_type}</label><div id=\'ID_DEVICE_TYPE{##}\'></div></div></help-topic><div id=\'ID_PREFIX{##}\'></div><help-topic data-help=\"/forms/general/trunk/remoteprefix\"><div class=\"line\"><label>{$trunk_label_remote_prefix}</label><div id=\'ID_REMOTE_PREFIX{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/trunk/lines\"><div class=\"line\"><label>{$trunk_label_lines}</label><div id=\'ID_LINES{##}\'></div></div></help-topic><div id=\"ID_DTMF{##}\"></div><div class=\"line\"><label></label><div id=\"ID_DTMF_FROM_DOOR{##}\"></div></div></div><!-- group trunk --><div class=\'groupbox\' style=\"min-height: 12em\"><h3>{$trunk_box_title_trunk}</h3><help-topic data-help=\"/forms/general/trunk/ip\"><div class=\"line\"><label>{$trunk_label_ipaddress}</label><div id=\'ID_IPADDRESS{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/trunk/framesize\"><div class=\"line\"><label>{$trunk_label_framesize}</label><div id=\'ID_FRAMESIZE{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/trunk/port\"><div class=\"line\"><label>{$trunk_label_port}</label><div id=\'ID_PORT{##}\'></div></div></help-topic></div></div></div></div>','/forms/general/group':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><!-- group general --><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/group/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic><div id=\"ID_CALLNUMBER{##}\"></div><help-topic data-help=\"/forms/general/group/reject\"><div class=\"line\"><label>{$group_callreject_label}</label><div id=\'ID_CALLREJECT{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/group/missed\"><div class=\"line\"><label>{$group_missedcall_label}</label><div id=\'ID_MISSEDCALL{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/group/voicemail\"><div class=\"line\"><label>{$group_voicemail_label}</label><div id=\'ID_VOICEMAIL{##}\'></div></div></help-topic></div><!-- group member --><div class=\'groupbox\'><h3>{$group_member_box}</h3><help-topic data-help=\"/forms/general/group/members\"><div class=\"line\"><label>{$group_member_label}</label><div id=\'ID_MEMBERLIST{##}\'></div></div><div class=\"line\"><label></label><div id=\'ID_MEMBERLIST_ADD{##}\'></div><div id=\'ID_MEMBERLIST_POPUP{##}\'></div><s class=\"s\"></s><div id=\'ID_MEMBERLIST_DELETE{##}\'></div></div></help-topic></div><!-- group functions --><div class=\'groupbox\' style=\"min-height: 12em\"><h3>{$formbase_group_functions}</h3><div id=\"ID_DEVGROUP_FUNCTION{##}\"></div></div></div></div></div>','/forms/general/sipaudio':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><!-- group general --><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/sipaudio/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/sipaudio/devtype\"><div class=\"line\"><label>{$formbase_label_device_type}</label><div id=\'ID_DEVICE_TYPE{##}\'></div></div></help-topic><div id=\"ID_CALLNUMBER{##}\"></div><div id=\'ID_ADDRBOOK{##}\'></div><div id=\'ID_CALLPROFILE{##}\'></div><div id=\"ID_PASSWORD{##}\"></div><div id=\'ID_CALLWAITING{##}\'></div><div id=\"ID_DTMF{##}\"></div></div><!-- group functions --><div class=\'groupbox\' style=\"min-height: 12em\"><h3>{$formbase_group_functions}</h3><div id=\"ID_DEVGROUP_FUNCTION{##}\"></div></div></div></div></div>','/forms/general/susapp':'<!-- SCREEN --><div class=\"v-layout sus-screen\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_OPTIONAL{##}\"><div id=\"ID_TOAST{##}\"></div><!-- group general --><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/susapp/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/susapp/devtype\"><div class=\"line\"><label>{$formbase_label_device_type}</label><div id=\'ID_DEVICE_TYPE{##}\'></div></div></help-topic><div id=\"ID_CALLNUMBER{##}\"></div><div id=\'ID_CALLPROFILE{##}\'></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_QRCODE\"><help-topic data-help=\"/forms/general/susapp/qrcode\"><div class=\"line\"><label>{$susapp_qrcode_label}</label><div id=\"ID_QRCODE{##}\"></div><s class=\"s\"></s><div class=\"v-layout\"><div id=\"ID_RESET_PARAM{##}\"></div><br/><div id=\"ID_QRCODE_PRINT{##}\"></div></div></div></help-topic><help-topic data-help=\"/forms/general/susapp/qrstatus\"><div class=\"line\"><label>{$susapp_qrvalid_label}</label><div id=\"ID_QRVALID{##}\"></div></div></help-topic></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_STATUS\"><help-topic data-help=\"/forms/general/susapp/status\"><div class=\"line\"><label>{$susapp_susstatus_label}</label><div id=\"ID_SUS_STATUS{##}\"></div><s class=\"s\"></s><div id=\"ID_RESET_PARAM2{##}\"></div><s class=\"s\"></s></div></help-topic></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_STATUS_EXPIRED\"><help-topic data-help=\"/forms/general/susapp/status\"><div class=\"line\"><label>{$susapp_susstatus_label}</label><div id=\"ID_SUS_STATUS_EXPIRED{##}\"></div><s class=\"s\"></s><div id=\"ID_RESET_PARAM_EXPIRED{##}\"></div><s class=\"s\"></s></div></help-topic></div></div><!-- group functions --><div class=\'groupbox\' style=\"min-height: 12em\"><h3>{$formbase_group_functions}</h3><div id=\"ID_DEVGROUP_FUNCTION{##}\"></div></div></div></div></div></div><!-- PRINT --><div class=\"hidden sus-print\"><style>@page { size: auto; /* auto is the initial value */ margin: 0mm; /* this affects the margin in the printer settings */ }</style><div class=\"h-layout\" style=\"width: 20cm\"><div id=\"ID_PRINT_NAME{##}\" class=\"h-layout-stretch\"></div><div class=\"h-layout-fix\"><img src=\"img/qr2.png\" style=\"height:0.7cm\"></div></div><br style=\"height: 0.5cm\"><div class=\"background\"><img src=\"img/qr1.png\" style=\"width:20cm\"><div class=\"link-appstore\" id=\"ID_PRINT_QRLINK{##}\"></div><div class=\"sus-qrcode\" id=\"ID_PRINT_QRCODE{##}\"></div><div class=\"sus-expiry\" id=\"ID_PRINT_EXPIRY{##}\">2021-12-24</div></div></div>','/forms/general/client':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><div id=\"ID_OPTIONAL{##}\" class=\"optional\"><!-- group \"general\" --><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/client/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/client/devtype\"><div class=\"line\"><label>{$formbase_label_device_type}</label><div id=\'ID_DEVICE_TYPE{##}\'></div></div></help-topic><div class=\"optional-item\" data-id=\"ID_OPTIONAL_CALLNUMBER\"><div id=\"ID_CALLNUMBER{##}\"></div></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_ADDRBOOK\"><div id=\"ID_ADDRBOOK{##}\"></div></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_CALLPROFILE\"><div id=\"ID_CALLPROFILE{##}\"></div></div><div id=\'ID_USERNAME{##}\'></div><div id=\"ID_PASSWORD{##}\"></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_CALLWAITING\"><div id=\"ID_CALLWAITING{##}\"></div></div></div><!-- group \"functions\" --><div class=\"optional-item\" data-id=\"ID_OPTIONAL_DEVGROUP_FUNCTION\"><div class=\'groupbox\' style=\"min-height: 12em\"><h3>{$formbase_group_functions}</h3><div id=\"ID_DEVGROUP_FUNCTION{##}\"></div></div></div></div><!-- ID_OPTIONAL --></div></div></div>','/forms/general/asht':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><div id=\"ID_OPTIONAL{##}\" class=\"optional\"><!-- group \"general\" --><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/asht/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/asht/devtype\"><div class=\"line\"><label>{$formbase_label_device_type}</label><div id=\'ID_DEVICE_TYPE{##}\'></div></div></help-topic><div class=\"optional-item\" data-id=\"ID_OPTIONAL_CALLNUMBER\"><div id=\"ID_CALLNUMBER{##}\"></div></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_ADDRBOOK\"><div id=\"ID_ADDRBOOK{##}\"></div></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_CALLPROFILE\"><div id=\"ID_CALLPROFILE{##}\"></div></div><div id=\'ID_USERNAME{##}\'></div><div id=\"ID_PASSWORD{##}\"></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_CALLWAITING\"><div id=\"ID_CALLWAITING{##}\"></div></div></div><!-- group \"functions\" --><div class=\"optional-item\" data-id=\"ID_OPTIONAL_DEVGROUP_FUNCTION\"><div class=\'groupbox\' style=\"min-height: 12em\"><h3>{$formbase_group_functions}</h3><div id=\"ID_DEVGROUP_FUNCTION{##}\"></div></div></div></div><!-- ID_OPTIONAL --></div></div></div>','/forms/general/jungpanel':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><!-- group general --><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/jungpanel/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/jungpanel/devtype\"><div class=\"line\"><label>{$formbase_label_device_type}</label><div id=\'ID_DEVICE_TYPE{##}\'></div></div></help-topic><div id=\"ID_CALLNUMBER{##}\"></div><div id=\"ID_MACADDR{##}\"></div><div id=\'ID_ADDRBOOK{##}\'></div><div id=\'ID_CALLPROFILE{##}\'></div><div id=\'ID_CALLWAITING{##}\'></div></div><!-- group functions --><div class=\'groupbox\' style=\"min-height: 12em\"><h3>{$formbase_group_functions}</h3><div id=\"ID_DEVGROUP_FUNCTION{##}\"></div></div></div></div></div>','/forms/general/addrbook':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></div><div class=\'groupbox\'><h3>{$addrbook_box_grid}</h3><div class=\"line\"><div id=\'ID_ADDRBOOK_GRID{##}\'></div></div><div class=\"line\"><div id=\"ID_ADDRBOOK_ADD{##}\"></div><s class=\"s\"></s><div id=\"ID_ADDRBOOK_EDIT{##}\"></div><s class=\"s\"></s><div id=\"ID_ADDRBOOK_DELETE{##}\"></div></div></div></div></div></div>','/forms/general/admin':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/admin/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic><div id=\'ID_USERNAME{##}\'></div><div id=\'ID_PASSWORD{##}\'></div></div></div></div></div>','/forms/general/knxgw':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/knxgw/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/door/devtype\"><div class=\"line\"><label>{$formbase_label_device_type}</label><div id=\'ID_DEVICE_TYPE{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/knxgw/ip\"><div class=\"line\"><label>{$formbase_label_ipaddr}</label><div id=\'ID_IPADDR{##}\'></div><s class=\"s\"></s><div id=\"ID_IPADDR_FIND{##}\"></div><div class=\"message-text\" style=\"margin-top: 0.375em;\"></div></div></help-topic><help-topic data-help=\"/forms/general/knxgw/port\"><div class=\"line\"><label>{$formbase_label_port}</label><div id=\'ID_PORT{##}\'></div></div></help-topic></div><div class=\'groupbox\'><h3>{$knxgw_box_grid}</h3><help-topic data-help=\"/forms/general/knxgw/grid\"><div class=\"line\"><label>{$knxgw_label_grid}</label><div id=\'ID_KNX_GRID{##}\'></div></div><div class=\"line\"><label></label><div id=\"ID_KNX_ADD{##}\"></div><s class=\"s\"></s><div id=\"ID_KNX_EDIT{##}\"></div><s class=\"s\"></s><div id=\"ID_KNX_DELETE{##}\"></div></div></help-topic></div></div></div></div>','/forms/general/timeline':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/timeline/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic><div id=\'ID_LABEL_STATUS{##}\'></div><div id=\'ID_LABEL_AUTOMATIC{##}\'></div></div><div class=\'groupbox\'><h3>{$timeline_box_time}</h3><div class=\"line\"><label></label><div id=\'ID_TL_LEGEND{##}\'></div></div><div class=\"line\"><label>{$timeline_monday}</label><div id=\'ID_TL_MONDAY{##}\'></div></div><div class=\"line\"><label>{$timeline_tuesday}</label><div id=\'ID_TL_TUESDAY{##}\'></div></div><div class=\"line\"><label>{$timeline_wednesday}</label><div id=\'ID_TL_WEDNESDAY{##}\'></div></div><div class=\"line\"><label>{$timeline_thursday}</label><div id=\'ID_TL_THURSDAY{##}\'></div></div><div class=\"line\"><label>{$timeline_friday}</label><div id=\'ID_TL_FRIDAY{##}\'></div></div><div class=\"line\"><label>{$timeline_staturday}</label><div id=\'ID_TL_SATURDAY{##}\'></div></div><div class=\"line\"><label>{$timeline_sunday}</label><div id=\'ID_TL_SUNDAY{##}\'></div></div></div><div class=\'groupbox\'><h3>{$timeline_box_action}</h3><div id=\'ID_COMMAND_ON{##}\'></div><div id=\'ID_COMMAND_OFF{##}\'></div></div></div></div></div>','/forms/general/agw':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_TOAST{##}\"></div><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/folder/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic></div></div></div></div>','/forms/general/ipcam':'<div class=\"v-layout\"><div class=\"v-layout-fix\" id=\'ID_TOOLBAR{##}\'></div><!-- SCROLLABLE --><div class=\"v-layout-stretch scroll-container-static viewable-container pageContent themeContent\"><div class=\"scrollable form\"><div id=\"ID_OPTIONAL{##}\"><div id=\"ID_TOAST{##}\"></div><div class=\'groupbox\'><h3>{$const_box_title_general}</h3><div id=\'ID_NAME{##}\'></div><help-topic data-help=\"/forms/general/ipcam/description\"><div class=\"line\"><label>{$formbase_label_description}</label><div id=\'ID_DESCRIPTION{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/door/devtype\"><div class=\"line\"><label>{$formbase_label_device_type}</label><div id=\'ID_DEVICE_TYPE{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/ipcam/ip\"><div class=\"line\"><label>{$formbase_label_ipaddr}</label><div id=\'ID_IPADDR{##}\'></div><s class=\"s\"></s><div id=\"ID_IPADDR_FIND{##}\"></div><div class=\"message-text\" style=\"margin-top: 0.375em;\"></div></div></help-topic><help-topic data-help=\"/forms/general/ipcam/port\"><div class=\"line\"><label>{$formbase_label_port}</label><div id=\'ID_PORT{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/ipcam/useauth\"><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_USE_AUTH\"><label>{$ipcam_label_auth}</label><div id=\'ID_USEAUTH{##}\'></div></div></help-topic><help-topic data-help=\"/forms/general/ipcam/username\"><div class=\"line\"><label>{$ipcam_label_account}</label><div id=\'ID_USERNAME{##}\'></div></div></help-topic><div id=\"ID_PASSWORD{##}\"></div></div><div class=\'groupbox\'><h3>{$ipcam_group_streams}</h3><help-topic data-help=\"/forms/general/ipcam/streams\"><div class=\"line\"><label>{$ipcam_label_streams}</label><div id=\'ID_STREAMS{##}\'></div><s class=\"s\"></s><div id=\"ID_CAMERA_INFO{##}\"></div></div><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_CAMERAINFO_ERROR\"><label></label><div><span id=\"ID_CAMERAINFO_ERROR{##}\" class=\"s red-text\" style=\"font-style: italic; line-height: 1em;\">asdasd</span></div></div></help-topic></div><!-- group functions --><div class=\'groupbox optional-item\' style=\"min-height: 12em\" data-id=\"ID_OPTIONAL_FUNCTIONS\"><h3>{$formbase_group_functions}</h3><div id=\"ID_DEVGROUP_FUNCTION{##}\"></div></div></div><!-- optional --></div><!-- scrollable --></div><!-- scroll container --></div><!-- v-layout -->','/dialog/messageBox':'<div class=\"dialogform\"><br><div class=\"h-layout\"><div class=\"h-layout-fix\" style=\"width: 3em;margin-top: 0.5em\"><span class=\"xxxl\"><i id=\"ID_ICON{##}\" class=\"mdi mdi-alert blue-text\"></i></span></div><div class=\"h-layout-stretch\" style=\"margin-top: 0.5em\"><span id=\"ID_TEXT{##}\" class=\'m\'>{$reboot_device_question}</span></div></div></div>','/dialog/progressDialog':'<div class=\"dialogform\" style=\"min-height: 8em\"><div id=\"ID_CONTENT{##}\" style=\"padding-top: 1.25rem\"></div>','/dialog/progressDialog/line':'<div><i class=\"{{icon}}\"></i><s class=\"s\"></s><span>{{text}}</span></div>','/dialog/changePassword':'<div class=\"dialogform\"><!-- header --><h2><i class=\"mdi mdi-account\"></i><s class=\"s\"></s><span id=\"ID_TITLE{##}\" class=\"xl\">{$password_dialog_title}</span></h2><hr/><div class=\"group error-below\"><!-- row 1 --><help-topic data-help=\"/dialog/changePassword/password\"><div class=\"line\"><label>{$password_label_password}</label><div id=\"ID_PASSWORD{##}\"></div></div></help-topic><!-- row2 --><help-topic data-help=\"/dialog/changePassword/repeat\"><div class=\"line\"><label>{$password_label_repeated}</label><div id=\"ID_PASSWORD_REPEATED{##}\"></div></div></help-topic><label class=\"none\"></label><div class=\"password-error error-text\"></div></div></div>','/dialog/rebootAccess':'<div class=\"dialogform\"><br><div class=\"h-layout\"><div class=\"h-layout-fix\" style=\"width: 3em;margin-top: 0.5em\"><span class=\"xxxl\"><i class=\"mdi mdi-alert blue-text\"></i></span></div><div class=\"h-layout-stretch\" style=\"margin-top: 0.5em\"><span id=\"ID_REBOOT_TEXT{##}\" class=\'m\'>{$reboot_agw_really_restart_agw}</span></div></div></div>','/dialog/formNew':'<div class=\"dialog-header\"><h2 class=\"v-align-middle\"><i class=\"mdi mdi-plus-circle\"></i><s class=\"s\"></s><span class=\"xl\" id=\"ID_TITLE{##}\">{$formnew_title}</span></h2></div><div class=\"dialogform\"><div class=\"line\" style=\"padding-top: 1em\"><div id=\"ID_LIST{##}\"></div></div><br></div>','/dialog/deviceLists':'<div class=\"dialog-header\"><h2 class=\"v-align-middle\"><i class=\"mdi mdi-format-list-bulleted\"></i><s class=\"s\"></s><span class=\"xl\" id=\"ID_TITLE{##}\">{$devicelist_title}</span></h2></div><div class=\"dialogform\"><div class=\"line\" style=\"padding-top: 1em\"><div id=\"ID_CATEGORY{##}\"></div></div><div class=\"line\"><div id=\"ID_LIST{##}\"></div></div><br></div>','/dialog/permissions':'<div class=\"dialog-header\"><h2 class=\"v-align-middle\"><i class=\"mdi mdi-security\"></i><s class=\"s\"></s><span class=\"xl\" id=\"ID_TITLE{##}\"></span></h2></div><div class=\"dialogform\"><help-topic data-help=\"/dialog/permissions/who\"><div class=\"line\" style=\"padding-top: 1em\"><label>{$permissions_label_who}</label><div id=\"ID_RIGHTHOLDER{##}\"></div></div><div class=\"line\"><label></label><div style=\"width: 100%; display: inline-flex; justify-content: flex-end;\"><div id=\"ID_DELETE{##}\"></div><s class=\"s\"></s><div id=\"ID_ADD{##}\"></div><div id=\"ID_POPUP{##}\"></div></div></div></help-topic><help-topic data-help=\"/dialog/permissions/what\"><div class=\"line\" style=\"padding-top: 1em\"><label>{$permissions_label_permissionlist}</label><div id=\"ID_RIGHTS{##}\"></div></div></help-topic><br><br></div>','/dialog/nameCallProfile':'<div class=\"dialogform\"><!-- header --><h2><i class=\"mdi mdi-phone-lock\"></i><s class=\"s\"></s>{$name_callprofile_dialog_title}</h2><hr/><!-- form --><div class=\'group error-below\'><help-topic data-help=\"/dialog/nameCallProfile/name\"><div class=\'line\'><label>{$name_callprofile_label_name}</label><div id=\'ID_EDIT_NAME{##}\'></div></div></help-topic></div><br /><br /><br /></div>','/dialog/permissionCallProfile':'<div class=\"dialogform\"><!-- header --><h2><i class=\"mdi mdi-phone-lock\"></i><s class=\"s\"></s>{$permission_callprofile_dialog_title}</h2><hr/><!-- form --><div class=\'group error-below\'><help-topic data-help=\"/dialog/nameCallProfile/value\"><div class=\'line\'><label>{$permission_callprofile_label_rule}</label><div id=\'ID_EDIT_RULE{##}\'></div></div><div class=\'line\'><label></label><div id=\'ID_CHECK_FORBID{##}\'></div></div></help-topic></div><br /><br /><br /></div>','/dialog/addressEntry':'<div class=\"dialogform\"><h2><i class=\"mdi mdi-card-account-details-outline\"></i><s></s><span class=\"xl\" id=\"ID_TITLE{##}\">{$address_title}</span></h2><hr /><div class=\"group error-below\"><!-- row 1 --><help-topic data-help=\"/dialog/addressEntry/label\"><div class=\"line\"><label>{$address_label_label}</label><div id=\"ID_LABEL{##}\"></div></div></help-topic><!-- row2 --><help-topic data-help=\"/dialog/addressEntry/callnumber\"><div class=\"line\"><label>{$address_label_callnumber}</label><div id=\"ID_CALLNUMBER{##}\"></div></div></help-topic><!-- row2 --><help-topic data-help=\"/dialog/addressEntry/comment\"><div class=\"line\"><label>{$address_label_comment}</label><div id=\"ID_COMMENT{##}\"></div></div></help-topic><br /></div></div>','/dialog/knxEntry':'<div class=\"dialogform\"><h2><i class=\"mdi mdi-lock-pattern\"></i><s></s><span class=\"xl\" id=\"ID_TITLE{##}\">{$knxentry_title}</span></h2><hr /><div class=\"group error-below\"><!-- row 1 --><help-topic data-help=\"/dialog/knxEntry/label\"><div class=\"line\"><label>{$knxentry_label_label}</label><div id=\"ID_LABEL{##}\"></div></div></help-topic><!-- row2 --><help-topic data-help=\"/dialog/knxEntry/send\"><div class=\"line\"><label>{$knxentry_label_send}</label><div id=\"ID_GRPADDR_SEND{##}\"></div></div></help-topic><!-- row2 --><help-topic data-help=\"/dialog/knxEntry/recv\"><div class=\"line\"><label>{$knxentry_label_receive}</label><div id=\"ID_GRPADDR_RECV{##}\"></div><s class=\"s\"></s><span>{$knxentry_postfix_receive}</span></div></help-topic></div><br /></div>','/dialog/ntpEntry':'<div class=\"dialogform\"><h2><i class=\"mdi mdi-clock-outline\"></i><s></s><span class=\"xl\" id=\"ID_TITLE{##}\">{$ntpentry_title}</span></h2><hr /><div class=\"group error-below\"><!-- row 1 --><help-topic data-help=\"/dialog/ntpEntry/value\"><div class=\"line\"><label>{$ntpentry_ipaddress_label}</label><div id=\"ID_IPADDRESS{##}\"></div></div></help-topic></div><br /><br /><br /></div>','/dialog/macAddr':'<div class=\"dialogform\" style=\"min-height: 9em\"><h2><i class=\"mdi mdi-wrench-outline\"></i><s></s><span class=\"xl\" id=\"ID_TITLE{##}\">{$macdialog_title}</span></h2><hr /><div class=\"group error-below\"><div id=\"ID_OPTIONAL{##}\"><help-topic data-help=\"/dialog/macAddr/macaddr\"><div class=\"optional-item\" data-id=\"ID_OPTIONAL_MACADDR\"><div class=\"line\"><label>{$macdialog_label}</label><div id=\"ID_MACADDRESS{##}\"></div></div></div></help-topic><help-topic data-help=\"/dialog/macAddr/susreg\"><div class=\"optional-item\" data-id=\"ID_OPTIONAL_SUSREG\"><div class=\"line\"><label>{$macdialog_susreg_label}</label><div id=\"ID_SUSREG{##}\"></div></div></div></help-topic><help-topic data-help=\"/dialog/macAddr/userparam\"><div class=\"optional-item\" data-id=\"ID_OPTIONAL_USERPARAM\"><div class=\"line\"><label>{$macdialog_userparam_label}</label><div id=\"ID_USERPARAM{##}\"></div></div></div></help-topic></div></div><br /></div>','/dialog/deviceKeys':'<div class=\"v-layout\"><div class=\"v-layout-fix dialog-header\"><div class=\"h-layout\" style=\"height:100%\"><div class=\"h-layout-fix\"><h2 class=\"v-align-middle\"><i class=\"mdi mdi-gesture-tap-button\"></i><s class=\"s\"></s><div id=\"ID_TITLE{##}\"></div></h2></div><div id=\"ID_OPTIONAL{##}\" class=\"h-layout-stretch h-align-right v-align-middle optional\"><div class=\"optional-item\" data-id=\"ID_OPTION_KEYSINGLE\"><div id=\'ID_BUTTON_KEYSINGLE{##}\'></div><s class=\"s\"></s></div><div class=\"optional-item\" data-id=\"ID_OPTION_KEYDOUBLE\"><div id=\'ID_BUTTON_KEYDOUBLE{##}\'></div><s class=\"l\"></s></div><div class=\"optional-item\" data-id=\"ID_OPTION_KEYPOS\"><div id=\'ID_BUTTON_KEYPOS{##}\'></div><s class=\"s\"></s></div><div class=\"optional-item\" data-id=\"ID_OPTION_KEYNEG\"><div id=\'ID_BUTTON_KEYNEG{##}\'></div><s class=\"l\"></s></div><div class=\"optional-item\" data-id=\"ID_OPTION_LED\"><div id=\'ID_BUTTON_LED{##}\'></div><s class=\"s\"></s></div></div></div></div><div class=\"v-layout-stretch h-layout error-below\" style=\"position: relative\"><div class=\"devicekeys-navigation h-layout-fix\"><div id=\"ID_SIDEBAR{##}\"></div></div><div class=\"h-layout-stretch scroll-container\"><div id=\"ID_WIDGET_CONTENT{##}\" class=\"devicekeys-content scrollable dialogform optional\"><!-- row2 --><div class=\"line\"><label>{$password_label_repeated}</label><div id=\"ID_PASSWORD_REPEATED{##}\"></div></div></div></div></div></div>','/dialog/deviceFunction':'<div class=\"dialog-header\"><div class=\"h-layout\" style=\"height:100%\"><div class=\"h-layout-fix\"><h2 class=\"v-align-middle\"><i id=\"ID_ICON{##}\" class=\"mdi mdi-code-braces\"></i><s class=\"s\"></s><div id=\"ID_TITLE{##}\"></div></h2></div></div></div><div class=\"dialogform error-below\" style=\"height:100%; min-height: 8em\"><div id=\"ID_WIDGET_CONTENT{##}\" class=\"devicefunction-content \"></div></div>','/dialog/deviceRule':'<div class=\"dialog-header\"><div class=\"h-layout\" style=\"height:100%\"><div class=\"h-layout-fix\"><h2 class=\"v-align-middle\"><i id=\"ID_ICON{##}\" class=\"mdi mdi-code-braces\"></i><s class=\"s\"></s><div id=\"ID_TITLE{##}\"></div></h2></div></div></div><div class=\"dialogform error-below\" style=\"height:100%; min-height: 8em\"><div id=\"ID_WIDGET_CONTENT{##}\" class=\"devicefunction-content error-below\"></div></div>','/dialog/changeKeytoneDialog':'<div class=\"dialogform\"><h2><i class=\"mdi mdi-music\"></i><s></s><span class=\"xl\" id=\"ID_TITLE{##}\">{$change_keytone_dialogtitle}</span></h2><hr /><div class=\"group error-below\"><!-- row 1 --><help-topic data-help=\"/dialog/changeKeytoneDialog/label\"><div class=\"line\"><label>{$change_keytone_dialog_call_target}</label><div id=\"ID_CALL_TARGET{##}\"></div></div></help-topic><!-- row2 --><help-topic data-help=\"/dialog/changeKeytoneDialog/callnumber\"><div class=\"line\"><label>{$change_keytone_dialog_callnumber}</label><div id=\"ID_KEYTONE_LIST{##}\"></div></div></help-topic><br /></div></div>','/dialog/scandevice':'<div class=\"dialog-header\"><h2 class=\"v-align-middle\"><i class=\"mdi mdi-layers-search-outline\"></i><s class=\"s\"></s><span class=\"xl\" id=\"ID_TITLE{##}\">{$scandevice_title}</span></h2></div><div class=\"dialogform\"><div class=\"line\"><div id=\"ID_CHECK{##}\"></div></div><div class=\"line\"><div id=\"ID_LIST{##}\"></div></div><br></div>','/dialog/ipcamCheck':'<div class=\"dialog-header\"><h2 class=\"v-align-middle\"><i class=\"mdi mdi-video-outline\"></i><s class=\"s\"></s><span class=\"xl\" id=\"ID_TITLE{##}\">{$ipcam_title}</span></h2></div><div class=\"dialogform error-below\"><div style=\"height: 1em\"></div><div class=\"line\"><label>{$ipcam_label_ipaddress}</label><div id=\'ID_IPADDRESS{##}\'></div></div><help-topic data-help=\"/forms/general/ipcam/username\"><div class=\"line\"><label>{$ipcam_label_account}</label><div id=\'ID_USERNAME{##}\'></div></div></help-topic><help-topic data-help=\"/dialog/changePassword/password\"><div class=\"line\"><label>{$password_label_password}</label><div id=\'ID_PASSWORD{##}\'></div></div></help-topic><div class=\"line\"><label></label><div><span id=\'ID_RESPONSE{##}\' class=\"s red-text\" style=\"font-style: italic; line-height: 1em;\"></span></div></div></div>','/part/header':'<div class=\"h-layout\" style=\"height: 100%\"><!-- title --><div class=\"v-layout-stretch v-align-middle\"><s class=\"l\"></s><span class=\"caption\">{$page_header_title}</span></div><!-- search --><div class=\"v-layout-fix v-align-middle\"><div id=\"ID_HEADER_SEARCH\"></div><s class=\"l\"></s><!-- <s class=\"l\"></s>--><!-- <s class=\"l\"></s>--></div><!-- status --><div class=\"v-layout-fix v-align-middle\"><div id=\"ID_HEADER_STATUS\"></div></div><!-- account button --><div class=\"v-layout-fix v-align-middle\"><div id=\"ID_HEADER_ACCOUNT\"></div><div id=\"ID_HEADER_ACCOUNT_BADGE\" style=\"right:-4px\"></div><div id=\"ID_HEADER_ACCOUNT_MENU\" class=\"themeNormal\"></div><s class=\"s\"></s></div><!-- logo --><div id=\"ID_SSS\" class=\"v-align-middle logo\"><svg xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns=\"http://www.w3.org/2000/svg\" width=\"8.75em\" height=\"1.2rem\" viewBox=\"0 0 1000 176.12419\" version=\"1.1\" xmlns:cc=\"http://creativecommons.org/ns#\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\"><g transform=\"translate(-22.348 58.484)\"><path d=\"m411.41 117.09c-3.2688-.29698-8.6852-1.0428-12.037-1.6572-5.5788-1.0228-7.6145-1.4599-16.124-3.4616l-2.8508-.67042v-12.842c0-8.039.15354-12.842.41088-12.842.52811 0 5.2234 2.1908 5.5019 2.5672.2029.27402 6.7429 3.1689 12.036 5.3277 13.302 5.4243 26.715 7.5531 36.954 5.8649 4.8197-.79466 8.5421-2.2104 12.188-4.6357 13.608-9.0509 13.607-28.418-.003-40.241-5.15-4.465-10.06-7.595-31.41-20.009-13.4-7.796-15.38-9.07-20.69-13.32-12.86-10.292-17.53-19.82-17.53-35.765 0-9.591 1.923-16.466 6.5129-23.283 2.657-3.9433 7.3014-9.1797 8.1442-9.1797.25224 0 .56801-.28523.70181-.63380.13339-.34860.53409-.63383.88989-.63383.3556 0 .64667-.19025.64667-.42254 0-.23246.28496-.42253.6335-.42253.34846 0 .63357-.15957.63357-.35457 0-.19508.75789-.71792 1.6845-1.162.92642-.44409 1.6866-.95002 1.6893-1.1243.003-.17427.18957-.31692.41566-.31692.22579 0 1.4137-.46568 2.6395-1.0348 5.0336-2.3371 11.303-3.9495 19.088-4.9093 5.36-.66082 23.332-.28764 30.653.63630 12.475 1.5745 17.963 2.6312 19.322 3.7203.39839.31942.52786 3.1697.52786 11.618 0 10.466-.0482 11.18-.73901 10.969-.40656-.12448-2.4496-.97224-4.5402-1.884-14.24-6.2109-33.424-10.049-42.94-8.5919-10.774 1.6501-17.756 7.2751-20.035 16.141-1.4417 5.6084.0442 13.49 3.1785 16.858 2.3524 2.5276 2.5703 2.8041 2.5703 3.261 0 .24423.52258.7861 1.1614 1.204.63877.41796 1.3722.96272 1.6301 1.2106 1.036.99636 3.4235 3.0112 4.0389 3.4085.36009.23246 2.1359 1.3733 3.9462 2.5353s3.5972 2.398 3.9708 2.7466c.37365.34857 1.1519.87388 1.7295 1.1673 2.2639 1.1503 4.8262 2.6748 5.0441 3.0013.21074.31566 1.6849 1.1976 6.7755 4.0538 1.0452.58646 2.7714 1.5908 3.8356 2.232 3.6812 2.2175 12.644 7.2926 12.878 7.2926.12923 0 .85886.47536 1.6201 1.0564.76142.58101 1.5289 1.0564 1.7055 1.0564.17672 0 1.1661.57042 2.1986 1.2676 1.0327.69719 1.9656 1.2677 2.0734 1.2677.18848 0 3.2514 2.3852 6.6804 5.2024 1.6318 1.3407 7.0184 6.6281 7.0184 6.8892 0 .07538.93466 1.5212 2.0772 3.213 2.4546 3.6352 4.7102 8.9413 5.7981 13.64 1.1212 4.8431 1.1165 17.587-.008 22.395-1.7057 7.2903-3.6783 12.022-7.0392 16.886-10.898 15.773-35.102 23.455-65.264 20.714zm-361.49-1.33c-7.4242-.47315-18.582-1.9176-24.39-3.1576l-2.0061-.42847v-8.4158c0-9.7964-.51036-9.1103 5.3056-7.1318 11.851 4.0313 27.607 6.7084 36.295 6.1665 5.1908-.3237 13.06-1.9308 16.7-3.4106 5.2288-2.1256 10.881-6.5832 13.86-10.931 3.6653-5.3496 5.2216-10.583 5.2383-17.616.0211-8.9199-2.1779-14.334-8.3348-20.521-5.644-5.676-11.566-9.382-31.687-19.837-22.845-11.869-29.445-17.18-34.658-27.888-2.7123-5.5712-3.6177-9.4379-3.8584-16.479-.29939-8.7578 1.2864-15.478 5.2795-22.374 3.6891-6.3706 10.602-12.917 16.713-15.827 4.0847-1.9451 6.9313-3.0812 8.7109-3.4766 9.9095-2.2019 9.7888-2.1906 23.44-2.202 13.37-.01214 16.943.28312 25.34 2.0887 1.626.34960 3.574.74581 4.329.88050l1.3726.24482v8.6069c0 7.0229-.10708 8.6069-.58096 8.6069-.31953 0-1.618-.45887-2.8856-1.0197-5.7026-2.5232-16.172-5.2858-24.17-6.378-15.258-2.0835-27.12.72073-35.065 8.2895-5.4876 5.2275-7.4903 9.9861-7.5381 17.912-.03137 5.1676.58909 7.9203 2.7562 12.232 3.5437 7.0503 8.583 11.074 23.983 19.147 2.2067 1.1569 5.4376 2.8965 7.1797 3.8658 1.7421.96929 3.8327 2.0874 4.6457 2.4847s1.5806.85879 1.7058 1.0256c.12515.1668 1.6456.95954 3.3787 1.7618 1.7331.80221 3.8987 1.9672 4.8125 2.589.91379.62174 2.0541 1.3021 2.534 1.5119.47991.20984 1.2484.61966 1.7078.91071.45944.29094 2.2649 1.4272 4.0122 2.5247 6.6123 4.1534 9.4786 6.5071 12.75 10.47 2.8147 3.4097 4.5755 5.7723 4.5755 6.1391 0 .15388.34519.78459.76712 1.4015 1.2212 1.7855 3.4466 7.8849 4.3225 11.847 1.0112 4.5748 1.1 13.285.18453 18.113-4.0095 21.147-21.291 35.332-46.452 38.128-3.4765.38638-15.156.46832-20.272.14215zm90.169-.34601c-6.5471-.65029-13.525-1.6494-17.795-2.5482-1.625-.34193-3.0371-.62181-3.1381-.62181-.10097 0-.18361-3.8029-.18361-8.4509v-8.4509h1.4696c.80833 0 2.1862.26962 3.062.59937 8.2497 3.1052 21.443 5.7433 31.789 6.3567 4.5019.26668 12.837-.57952 16.742-1.6999 2.5518-.73226 8.6464-3.1351 9.0205-3.5565.11569-.13062.68633-.4464 1.2671-.70145 1.9146-.84046 5.7845-4.3035 7.827-7.0041 3.9035-5.161 5.5719-9.8636 5.9643-16.812.58-10.285-2.73-17.892-10.69-24.536-5.95-4.972-10.74-7.832-29.28-17.497-15.48-8.065-18.48-9.898-24.84-15.191-5.81-4.841-10.28-11.836-12.42-19.467-.82996-2.957-.94751-4.3595-.93861-11.197.0123-8.9685.5826-11.858 3.584-18.139 2.3332-4.8829 4.6601-7.9882 9.5447-12.738 7.267-7.0661 13.211-9.7491 25.607-11.558 11.873-1.7329 39.101.05676 44.666 2.9356l1.2505.64704v8.4401c0 4.642-.13138 8.44-.29294 8.44-.16098 0-2.2043-.74702-4.5402-1.6601-4.7507-1.8571-11.329-3.7463-17.973-5.1616-5.9853-1.2751-20.616-1.426-24.845-.25628-11.734 3.2453-19.426 10.018-21.788 19.182-1.2311 4.7773-.59014 13.708 1.2137 16.91 1.2936 2.2962 4.036 5.8619 6.2427 8.1166 3.092 3.1593 8.8834 6.9168 18.482 11.991 2.2069 1.1666 4.2029 2.2698 4.436 2.4517.23297.18180 2.5135 1.4109 5.068 2.7313 25.236 13.044 33.746 19.709 39.855 31.215.72686 1.369 1.3229 2.8902 1.3245 3.3804.001.4902.4632 2.2477 1.0257 3.9057 3.2182 9.4864 2.3374 23.128-2.0967 32.477-6.7318 14.193-19.298 23.075-37.998 26.858-3.885.78582-6.3055.95917-14.782 1.0583-6.2663.0729-12.313-.0974-15.838-.44762zm100.94.35838c-.81299-.0763-3.8539-.34038-6.7575-.58644-5.4078-.458-14.102-1.6774-18.055-2.5324l-2.2173-.47957v-8.6812-8.681l1.7579.28122c.96693.15432 3.2.74722 4.9625 1.3167 11.047 3.5691 24.771 6.116 32.959 6.116 7.7308 0 19.212-2.8429 23.274-5.763 1.0868-.78123 2.1434-1.4211 2.348-1.4222.79803-.0031 5.7739-4.8736 7.3444-7.1873 3.5774-5.2705 5.0641-10.446 5.0641-17.63 0-12.113-5.286-19.964-19.326-28.706-2.3039-1.4344-10.444-5.8737-18.088-9.8652-19.588-10.227-24.572-13.388-30.551-19.37-12.78-12.787-14.623-35.679-4.1205-51.187 3.553-5.2466 12.007-12.543 17.458-15.068 6.1654-2.8555 17.413-4.7926 27.81-4.7893 9.2884.0028 23.562 1.5841 29.347 3.251 4.0658 1.1714 3.8061.50723 3.8061 9.7357 0 4.6532-.16531 8.138-.38734 8.138-.21314 0-1.291-.38753-2.3955-.86120-5.0393-2.1612-13.58-4.617-21.29-6.1216-5.5149-1.0762-16.666-1.3091-21.249-.4437-7.7732 1.4679-13.16 4.1488-17.817 8.8679-3.2382 3.2807-5.3235 6.7833-6.322 10.619-1.0339 3.9718-.69192 13.307.51622 14.084.22708.14597.55894.74081.73774 1.3218.17884 0.581.44183 1.1514.58488 1.2676.14246.11603.82992 1.333 1.5268 2.7039 2.2931 4.5113 6.1861 8.0782 14.36 13.157.69689 0.433 3.9277 2.2419 7.1796 4.0196 3.2521 1.7778 6.5779 3.6084 7.3909 4.068.81297.45956 3.5359 1.9618 6.0507 3.3384s5.8406 3.2087 7.3909 4.0714c1.5501.86268 3.1036 1.7045 3.4519 1.8706.34852.16617 1.3937.78505 2.3229 1.3753 1.9745 1.2544 3.2624 2.0476 5.2183 3.2139 3.239 1.9315 9.0562 6.7659 12.246 10.177 3.8568 4.1242 6.0149 8.0719 8.7586 16.021l1.6043 4.648.0123 8.6622c.0123 8.2456-.0446 8.8451-1.1439 12.465-5.8525 19.272-20.756 30.752-44.433 34.225-3.8128.5592-16.874.77995-21.328.36042zm269.45-85.502v-85.354h13.515 13.515v85.354 85.354h-13.515-13.515v-85.354zm64.195.21136v-85.143h44.557 44.556v8.4509 8.4509h-31.464-31.464v28.099 28.099h29.986 29.986v8.6622 8.6622h-29.775-29.775v31.268 31.268h32.098 32.098v8.662 8.6623h-45.401-45.401v-85.143zm114.88-.25236v-85.395l25.446.20985c25.599.21105 28.279.35382 37.69 2.0086 16.221 2.8522 29.743 11.333 37.917 23.782 3.3872 5.1586 4.68 7.6785 6.826 13.305 6.4594 16.935 8.5775 36.954 6.5348 61.765-2.548 30.951-14.538 52.59-34.306 61.912-13.013 6.1367-26.174 7.7737-62.687 7.7971l-17.421.0123v-85.395zm51.288 67.602c6.1864-1.1332 10.892-2.9408 15.915-6.1134 2.1614-1.3651 7.9641-7.1892 8.817-8.8496.34852-.67815.95958-1.565 1.358-1.9707.99159-1.0096 4.7349-8.836 5.7727-12.069 1.2427-3.8716 2.457-9.822 3.0831-15.107.72322-6.1042.72232-35.405 0-43.522-1.2448-13.965-4.7125-26.074-9.5591-33.379-3.2805-4.9447-9.5975-9.6297-16.269-12.066-6.5417-2.3886-8.085-2.5891-21.656-2.813l-12.566-.20744v68.283 68.283h11.272c6.4378 0 12.37-.20109 13.832-.46892zm85.126-67.561v-85.354h13.304 13.304v75.847 75.847h31.675 31.675v9.5073 9.5072h-44.979-44.979v-85.354zm104.74-.21136v-85.566h44.768 44.768v8.6622 8.6622h-31.464-31.464v28.099 28.099h29.986 29.986v8.6622 8.6622h-29.986-29.986v31.268 31.268h32.52 32.52v8.8735 8.8735h-45.823-45.823v-85.566z\"/></g></svg></div><s class=\"l\"></s></div>','/part/sidebar':'<div id=\"ID_ACCORDION{##}\"><!-- tree #1 --><div class=\"accordion-item collapsed\" data-id=\"ACCORDION_GENERAL\" data-type=\"fix\"><h3>{$accordion_section_general}</h3><div id=\"ID_GENERAL_TREE{##}\"></div></div><!-- tree #2 --><div class=\"accordion-item expanded\" data-id=\"ACCORDION_DEVTREE\" data-type=\"flex\"><h3>{$accordion_section_devtree}</h3><div id=\"ID_DEVTREE{##}\"></div></div></div>','/widget/deviceConfigBase/avpTile':'<div class=\"line\"><label>{$widget_base_avptile_icon}</label><div id=\"ID_AVP_TILE_ICON{##}\"></div><s class=\"s\"></s></div><div class=\"line\"><label>{$widget_base_avptile_label}</label><div id=\"ID_AVP_TILE_EDIT{##}\"></div><s class=\"s\"></s></div>','/widget/deviceConfig/empty':'<div class=\"line\"><label>{$widget_empty_label_option1}</label><div id=\"ID_CHECK_OPTION1{##}\"></div></div><div class=\"line\"><label>{$widget_empty_label_option2}</label><div id=\"ID_CHECK_OPTION2{##}\"></div></div>','/widget/deviceConfig/none':'<div class=\"line\"><span>{$widget_none_noparam}</span></div>','/widget/control/calltargetList':'<div id=\"ID_DROPLIST{##}\" class=\"normal\"></div>','/widget/control/statepointList':'<div id=\"ID_DROPLIST{##}\" class=\"normal\"></div>','/widget/control/entityList':'<div id=\"ID_DROPLIST{##}\" class=\"normal\"></div>','/widget/control/entityName':'<help-topic data-help=\"/widget/control/entityName/name\"><div class=\"line\"><label>{$formbase_label_name}</label><div class=\"edit\"></div></div></help-topic>','/widget/control/label':'<help-topic data-help=\"/widget/control/label\"><div class=\"line\"><label>{$labelctrl_label}</label><div class=\"edit\"></div><s class=\"s\"></s><div class=\"permissionCheck\"></div></div></help-topic>','/widget/control/videoUrlAddress':'<div class=\"videoUrl\"></div>','/widget/control/permissionCheck':'<span title=\"{$permcheck_tooltip}\" class=\"hidden permIcon\"><i class=\"mdi mdi-security yellow-text\"></i></span>','/widget/control/addrbkchoose':'<div id=\"ID_DROPLIST{##}\" class=\"normal\"></div>','/widget/jsoncontrol':'<div class=\"scroll-container normal\"><div class=\"scrollable json-control\"><div class=\"container\"><pre id=\"ID_JSON{##}\"></pre></div></div></div>','/widget/control/macAddress':'<help-topic data-help=\"/widget/control/macAddress\"><div class=\"line\"><label>{$macaddr_label}</label><div id=\'ID_MACADDR{##}\'></div><s class=\"s\"></s><div id=\"ID_MACADDR_EDIT{##}\"></div></div></help-topic>','/widget/control/callWaiting':'<help-topic data-help=\"/widget/control/callWaiting\"><div class=\"line\"><label>{$callwaiting_label}</label><div id=\"ID_CALLWAITING{##}\"></div></div></help-topic>','/widget/control/callNumber':'<help-topic data-help=\"/widget/control/callNumber\"><div class=\"line\"><label>{$control_callnumber_label}</label><div id=\"ID_CALLNUMBER{##}\"></div></div></help-topic>','/widget/control/callPrefix':'<help-topic data-help=\"/widget/control/callPrefix\"><div class=\"line\"><label>{$control_callprefix_label}</label><div id=\"ID_CALLPREFIX{##}\"></div></div></help-topic>','/widget/control/addressBook':'','/widget/control/addressBook/line':'<help-topic data-help=\"/widget/control/addressBook\"><div class=\"line\"><label>{$addrbk_label_addressbook}</label><div class=\"addressBook\" id=\'ID_ADDRBOOK{##}\'></div></div></help-topic>','/widget/control/deviceVolume':'','/widget/control/deviceVolume/ring':'<help-topic data-help=\"/widget/control/deviceVolume/ring\"><div class=\"line\"><label>{$devicevolume_label_ringvolume}</label><div class=\"range\"></div></div></help-topic>','/widget/control/deviceVolume/call':'<help-topic data-help=\"/widget/control/deviceVolume/call\"><div class=\"line\"><label>{$devicevolume_label_callvolume}</label><div class=\"range\"></div></div></help-topic>','/widget/control/deviceVolume/mic':'<help-topic data-help=\"/widget/control/deviceVolume/mic\"><div class=\"line\"><label>{$devicevolume_label_micvolume}</label><div class=\"range\"></div></div></help-topic>','/widget/control/showClock':'','/widget/control/showClock/line':'<help-topic data-help=\"/widget/control/showClock\"><div class=\"line\"><label>{$formbase_label_showclock}</label><div id=\'ID_SHOWCLOCK{##}\'></div></div></help-topic>','/widget/control/mountType':'','/widget/control/mountType/line':'<help-topic data-help=\"/widget/control/mountType\"><div class=\"line\"><label>{$mounttype_label}</label><div id=\'ID_MOUNTTYPE{##}\'></div></div></help-topic>','/widget/control/externCallNumber':'<help-topic data-help=\"/widget/control/externCallNumber\"><div class=\"line\"><label>{$extern_callnumber_label}</label><div id=\"ID_EXTERN_CALLNUMBER{##}\"></div></div></help-topic>','/widget/control/callProfile':'<help-topic data-help=\"/widget/control/callProfile\"><div class=\"line\"><label>{$control_callprofile_label}</label><div class=\"listbox\"></div></div></help-topic>','/widget/control/dtmfDoorAndLight':'<div id=\"ID_OPTIONAL{##}\"><help-topic data-help=\"/widget/control/dtmfDoorAndLight/door\"><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_DTMF_DOOR\"><label>{$dtmf_label_door}</label><div id=\'ID_DTMF_DOOR{##}\'></div></div></help-topic><help-topic data-help=\"/widget/control/dtmfDoorAndLight/light\"><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_DTMF_LIGHT\"><label>{$dtmf_label_light}</label><div id=\'ID_DTMF_LIGHT{##}\'></div></div></help-topic></div>','/widget/control/password':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"><help-topic data-help=\"/widget/control/password/password\"><div class=\"line\"><label>{$control_password_label}</label><div id=\"ID_PASSWORD1{##}\"></div><div class=\"optional-item\" data-id=\"ID_OPT_PASSWORD_EDIT\"><s class=\"s\"></s><div id=\"ID_PASSWORD_EDIT{##}\"></div></div></div></help-topic><help-topic data-help=\"/widget/control/password/password2\"><div class=\"line optional-item\" data-id=\"ID_OPT_PASSWORD2\"><label>{$control_password_label2}</label><div id=\"ID_PASSWORD2{##}\"></div></div></help-topic></div>','/widget/control/account':'<help-topic data-help=\"/widget/control/account\"><div class=\"line\"><label>{$control_account_label_account}</label><div id=\"ID_ACCOUNT{##}\"></div></div></help-topic>','/widget/control/keytone':'','/widget/control/keytone/line':'<help-topic data-help=\"/widget/control/keytone\"><div class=\"line\"><label>{$keytone_droplist_label}</label><div class=\"keytone\" id=\'ID_KEYTONE\'></div></div></help-topic><help-topic data-help=\"/widget/control/keytone\"><div id=\"ID_OPTIONAL{##}\" class=\"optional\"><div class=\"optional-item\" data-id=\"ID_KEYTONE_CONTROL_KEYTONE_LIST\"><div class=\"line\"><label></label><div class=\"keytone\" id=\'ID_KEYTONE_LIST\'></div></div><div class=\"line\"><label></label><div><div id=\"ID_KEYTONE_ADD{##}\"></div><s class=\"s\"></s><div id=\"ID_KEYTONE_EDIT{##}\"></div><s class=\"s\"></s><div id=\"ID_KEYTONE_DELETE{##}\"></div></div></div></div></div></help-topic>','/widget/control/zam':'','/widget/control/zam/line':'<help-topic data-help=\"/widget/control/zam\"><div class=\"line\"><label>{$zam_options_label}</label><div id=\'ID_ZAM_OPTIONS{##}\'></div></div></help-topic>','/widget/control/ringtone':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"><help-topic data-help=\"/widget/control/ringtone\"><div class=\"line\"><label>{$control_ringtone_label}</label><div id=\"ID_RINGTONE{##}\"></div></div></help-topic><help-topic data-help=\"/widget/control/ringtone/doorcall\"><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_DOORCALL\"><label>{$control_ringtone_label_doorcall}</label><div id=\"ID_LIST_DOORCALL{##}\"></div></div></help-topic><help-topic data-help=\"/widget/control/ringtone/storeycall\"><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_STOREYCALL\"><label>{$control_ringtone_label_storeycall}</label><div id=\"ID_LIST_STOREYCALL{##}\"></div></div></help-topic><help-topic data-help=\"/widget/control/ringtone/default\"><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_DEFAULT\"><label>{$control_ringtone_label_default}</label><div id=\"ID_LIST_DEFAULT{##}\"></div></div></help-topic></div><!-- optional -->','/widget/control/doorCamera':'<div id=\"ID_OPTIONAL{##}\"><help-topic data-help=\"/widget/control/doorCamera\"><div class=\"line optional-item\" data-id=\"ID_LIST\"><label>{$control_doorcamera_label}</label><div class=\"listbox\"></div></div></help-topic><help-topic data-help=\"/widget/control/doorCamera\"><div class=\"line optional-item\" data-id=\"ID_EDIT\"><label>{$control_doorcamera_label}</label><div class=\"edit\"></div></div></help-topic></div>','/widget/deviceConfig/call':'<div id=\"ID_AVP_LABEL{##}\"></div><help-topic data-help=\"/widget/deviceConfig/call/number\"><div class=\"line\"><label>{$widget_call_label}</label><div id=\"ID_CALL_NUMBER{##}\"></div></div></help-topic>','/widget/deviceConfig/doorcall':'<help-topic data-help=\"/widget/deviceConfig/doorcall/number\"><div class=\"line\"><label>{$widget_doorcall_label}</label><div id=\"ID_DOORCALL_NUMBER{##}\"></div></div></help-topic>','/widget/deviceConfig/setsp':'<div id=\"ID_AVP_LABEL{##}\"></div><help-topic data-help=\"/widget/deviceConfig/setsp/target\"><div class=\"line\"><label>{$widget_set_sp_label}</label><div id=\"ID_SET_SP_TARGET{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/setsp/value\"><div class=\"line\"><label>{$widget_set_sp_value}</label><div id=\"ID_SET_SP_VALUE{##}\"></div></div></help-topic>','/widget/deviceConfig/setdnd':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"><div id=\"ID_AVP_LABEL{##}\"></div><help-topic data-help=\"/widget/deviceConfig/setdnd/target\"><div class=\"line\"><label>{$widget_setdnd_drop_list_label}</label><div id=\"ID_SET_DND_TARGET{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/setdnd/value\"><div class=\"line\"><label>{$widget_setdnd_value_label}</label><div id=\"ID_SET_DND_VALUE{##}\"></div></div></help-topic></div>','/widget/deviceConfig/setredir':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"><div id=\"ID_AVP_LABEL{##}\"></div><help-topic data-help=\"/widget/deviceConfig/setredir/target\"><div class=\"line\"><label>{$widget_set_redir_owner}</label><div id=\"ID_SET_REDIR_TARGET{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/setredir/target2\"><div class=\"line\"><label>{$widget_set_redir_redirect_target}</label><div id=\"ID_SET_REDIR_REDIRECT_TARGET{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/setredir/type\"><div class=\"line\"><label>{$widget_set_redir_type}</label><div id=\"ID_SET_REDIR_TYPE{##}\"></div></div></help-topic><div class=\"optional-item\" data-id=\"ID_OPTIONAL_SET_REDIR_TIME\"><help-topic data-help=\"/widget/deviceConfig/setredir/time\"><div class=\"line\"><label>{$widget_set_redir_time}</label><div id=\"ID_SET_REDIR_TIME{##}\"></div></div></help-topic><div class=\"line\"><label></label><div id=\"ID_SET_REDIR_BUSY{##}\"></div></div></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_VALUE\"><help-topic data-help=\"/widget/deviceConfig/setredir/timelinevalue\"><div class=\"line\"><label>{$widget_set_redir_value}</label><div id=\"ID_SET_REDIR_VALUE{##}\"></div></div></help-topic></div></div>','/widget/deviceConfig/setspknx':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"><div id=\"ID_AVP_LABEL{##}\"></div><help-topic data-help=\"/widget/deviceConfig/setspknx/gw\"><div class=\"line\"><label>{$widget_set_sp_knx_gateway}</label><div id=\"ID_SET_SP_KNX_GATEWAY{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/setspknx/grpaddr\"><div class=\"line\"><label>{$widget_set_sp_knx_group_address}</label><div id=\"ID_SET_SP_KNX_GROUP_ADDRESS{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/setspknx/recv\"><div class=\"line\"><label>{$widget_set_sp_knx_receive_object}</label><div id=\"ID_SET_SP_KNX_RECEIVE_OBJECT{##}\"></div></div></help-topic><div class=\"optional-item\" data-id=\"ID_OPTIONAL_SET_SP_KNX_RECEIVE_OBJECT\"><div class=\"line\"><label></label><div id=\"ID_SET_SP_KNX_RECEIVE_OBJECT_VALUE{##}\"></div></div></div><help-topic data-help=\"/widget/deviceConfig/setspknx/value\"><div class=\"line\"><label>{$widget_set_sp_knx_value}</label><div id=\"ID_SET_SP_KNX_VALUE{##}\"></div></div></help-topic></div>','/widget/deviceConfig/setdoormatic':'<div id=\"ID_AVP_LABEL{##}\"></div><help-topic data-help=\"/widget/deviceConfig/setdoormatic/target\"><div class=\"line\"><label>{$widget_set_doormatic_label}</label><div id=\"ID_SET_DOORMATIC_TARGET{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/setdoormatic/value\"><div class=\"line\"><label>{$widget_set_doormatic_value}</label><div id=\"ID_SET_DOORMATIC_VALUE{##}\"></div></div></help-topic>','/widget/deviceConfig/storeycall':'<div id=\"ID_AVP_LABEL{##}\"></div><div class=\"line\"><div id=\"ID_STOREY_CALL_LABEL{##}\"><span>class \'ControlLabel\'</span></div></div><help-topic data-help=\"/widget/deviceConfig/storeycall/target\"><div class=\"line\"><label>{$widget_storey_call_target}</label><div id=\"ID_STOREY_CALL_TARGET{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/storeycall/opener\"><div class=\"line\"><label>{$widget_storey_call_door_opener}</label><div id=\"ID_STOREY_CALL_DOOR_OPENER{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/storeycall/light\"><div class=\"line\"><label>{$widget_storey_call_door_light}</label><div id=\"ID_STOREY_CALL_DOOR_LIGHT{##}\"></div></div></help-topic>','/widget/deviceConfig/mute_mic':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"/><div id=\"ID_AVP_LABEL{##}\"></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_MUTE_MIC\"><div class=\"line\"><span>{$widget_mute_mic_no_parameter}</span></div></div>','/widget/deviceConfig/mute_speaker':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"/><div id=\"ID_AVP_LABEL{##}\"></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_MUTE_SPEAKER\"><div class=\"line\"><span>{$widget_mute_speaker_no_parameter}</span></div></div>','/widget/deviceConfig/grabpicmenu':'<div id=\"ID_AVP_LABEL{##}\"></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_GRAB_PIC_MENU\"><div class=\"line\"><span>{$widget_grabpicmenu_no_parameter}</span></div></div>','/widget/deviceConfig/externcall':'<div id=\"ID_AVP_LABEL{##}\"></div><help-topic data-help=\"/widget/deviceConfig/externcall/number\"><div class=\"line\"><label>{$widget_externcall_label}</label><div id=\"ID_CALL_NUMBER{##}\"></div><div><s class=\"s\"></s><!-- <span>{$widget_externcall_prefix_hint}</span></div>--><div id=\"ID_ADDRBK_BUTTON{##}\"></div><div id=\"ID_ADDRBK_POPUP{##}\"></div></div></help-topic>','/widget/deviceConfig/ldopener':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"><div id=\"ID_AVP_LABEL{##}\"></div><help-topic data-help=\"/widget/deviceConfig/ldopener/timebased\"><div class=\"multiline\"><label>{$widget_ldopener_owner}</label><div id=\"ID_LDOPENER_IS_TIME_BASED{##}\"></div></div></help-topic><div class=\"optional-item\" data-id=\"ID_OPTIONAL_IS_TIME_BASED\"><div class=\"line\"><label></label><div><div id=\"ID_LDOPENER_TIME{##}\" class=\"ldopener_time\"></div><s></s><span>{$widget_ldopener_prefix_time}</span></div></div></div></div>','/widget/deviceConfig/ldlight':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"><div id=\"ID_AVP_LABEL{##}\"></div><help-topic data-help=\"/widget/deviceConfig/ldlight/timebased\"><div class=\"line\"><label>{$widget_ldlight_owner}</label><div id=\"ID_LDLIGHT_IS_TIME_BASED{##}\"></div></div></help-topic><div class=\"optional-item\" data-id=\"ID_OPTIONAL_IS_TIME_BASED\"><div class=\"line\"><label></label><div><div id=\"ID_LDLIGHT_TIME{##}\" class=\"ldlight_time\"></div><s></s><span>{$widget_ldlight_prefix_time}</span></div></div></div></div>','/widget/deviceConfig/led':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"><help-topic data-help=\"/widget/deviceConfig/led/color\"><div class=\"line\"><label>{$widget_led_color_label}</label><div id=\"ID_COLOR{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/led/trigger\"><div class=\"line\"><label>{$widget_led_trigger_label}</label><div id=\"ID_TRIGGER{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/led/type\"><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_INDTYPE\"><label>{$widget_led_type_label}</label><div id=\"ID_INDTYPE{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/led/statepoint\"><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_INDSTATEPOINT\"><label>{$widget_led_device_label}</label><div id=\"ID_INDSTATEPOINT{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/led/target\"><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_INDTARGET\"><label>{$widget_led_redirtarget_label}</label><div id=\"ID_INDDESTINATION{##}\"></div></div></help-topic></div>','/widget/deviceConfig/switchcontact':'<div id=\"ID_LABEL{##}\"></div><help-topic data-help=\"/widget/deviceConfig/switchcontact/invert\"><div class=\"line\"><label>{$widget_switchcontact_invert}</label><div id=\"ID_SWITCH_CONTACT_INVERT{##}\"></div></div></help-topic>','/widget/deviceConfig/timeswitchcontact':'<div id=\"ID_LABEL{##}\"></div><help-topic data-help=\"/widget/deviceConfig/timeswitchcontact/time\"><div class=\"line\"><label>{$widget_timeswitchcontact_time}</label><div id=\"ID_TIME_SWITCH_CONTACT_TIME{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/timeswitchcontact/invert\"><div class=\"line\"><label>{$widget_switchcontact_invert}</label><div id=\"ID_TIME_SWITCH_CONTACT_INVERT{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/timeswitchcontact/posttrigger\"><div class=\"line\"><label>{$widget_timeswitchcontact_posttrigger}</label><div id=\"ID_TIME_SWITCH_CONTACT_POSTTRIGGER{##}\"></div></div></help-topic>','/widget/deviceConfig/lightcontact':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"/><div class=\"line\"><span id=\"ID_LIGHTCONTACT_HINT{##}\"></span></div>','/widget/deviceConfig/openercontact':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"/><div class=\"line\"><span id=\"ID_OPENERCONTACT_HINT{##}\"></span></div>','/widget/deviceConfig/state_notifying_contact':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"/><div id=\"ID_STATE_NOTIFYING_CONTACT_LABEL{##}\"></div>','/widget/deviceConfig/local_door_light':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"/><div id=\"ID_AVP_LABEL{##}\"></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_LOCAL_DOOR_LIGHT\"><div class=\"line\"><span>{$widget_local_door_light_no_parameter}</span></div></div>','/widget/deviceConfig/observerledknx':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"><div id=\"ID_AVP_LABEL{##}\"></div><help-topic data-help=\"/widget/deviceConfig/observerledknx/ledcolor\"><div class=\"line\"><label>{$widget_observe_led_knx_color}</label><div id=\"ID_OBSERVE_LED_COLOR{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/observerledknx/gateway\"><div class=\"line\"><label>{$widget_observe_led_gateway}</label><div id=\"ID_OBSERVE_LED_KNX_GATEWAY{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceConfig/observerledknx/grpaddr\"><div class=\"line\"><label>{$widget_observe_led_knx_group_address}</label><div id=\"ID_OBSERVE_LED_GROUP_ADDRESS{##}\"></div></div></help-topic></div>','/widget/deviceConfig/toemode':'<div><div id=\"ID_TOEMODE{##}\"></div></div>','/widget/deviceFunctions/dnd':'<div class=\"line\"><div id=\"ID_LABEL{##}\"></div></div>','/widget/deviceFunctions/presence':'<div class=\"line\"><div id=\"ID_LABEL{##}\"></div></div>','/widget/deviceFunctions/grabpic':'<div id=\"ID_OPTIONAL{##}\"><div class=\"optional-item\" data-id=\"ID_OPTIONAL_TIME\"><help-topic data-help=\"/widget/deviceFunctions/grabpic/time\"><div class=\"line\"><label>{$widget_grabpic_time_label}</label><div id=\"ID_TIME{##}\"></div><s class=\"s\"></s><span>{$widget_grabpic_time_postfix}</span></div></help-topic></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_MODE\"><help-topic data-help=\"/widget/deviceFunctions/grabpic/mode\"><div class=\"line\"><label>{$widget_grabpic_mode_label}</label><div id=\"ID_MODE{##}\"></div></div></help-topic></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_CALLS\"><help-topic data-help=\"/widget/deviceFunctions/grabpic/calls\"><div class=\"line\"><label>{$widget_grabpic_event_label}</label><div id=\"ID_CALLS{##}\"></div></div></help-topic></div></div>','/widget/deviceFunctions/redir':'<div class=\"line\"><div id=\"ID_LABEL{##}\"></div></div>','/widget/deviceFunctions/doormatic':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"/><div class=\"line\"><div id=\"ID_LABEL{##}\" class=\"\"></div></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_DEVICE_TYPE_DOOR\"><help-topic data-help=\"/widget/deviceFunctions/doormatic/time\"><div class=\"line \"><label>{$widget_doormatic_open_after}</label><div id=\"ID_DOORMATIC_TIME_DELAY{##}\"></div><s></s><span>{$widget_doormatic_open_after_suffix}</span></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/doormatic/opener\"><div class=\"line \"><label>{$widget_doormatic_opener_label}</label><div id=\"ID_DOORMATIC_OPENER{##}\"></div></div></help-topic></div><div class=\"optional-item\" data-id=\"ID_OPTIONAL_DEVICE_TYPE_APARTMENT\"><!-- delay after that a door or doors will be opened --><help-topic data-help=\"/widget/deviceFunctions/doormatic/openafter\"><div class=\"line\"><label>{$widget_doormatic_open_after}</label><div id=\"ID_DOORMATIC_OPEN_AFTER{##}\"></div><s></s><span>{$widget_doormatic_open_after_suffix}</span></div></help-topic><!-- doormatic options --><help-topic data-help=\"/widget/deviceFunctions/doormatic/options\"><div class=\"line\"><label>{$widget_doormatic_options_label}</label><div id=\"ID_DOORMATIC_OPTIONS{##}\"></div></div></help-topic><!-- call dorrmatic if call is peicked up? --><!-- choice of all door will be opened or selected doors --><help-topic data-help=\"/widget/deviceFunctions/doormatic/openof\"><div class=\"line\"><label>{$widget_doormatic_open_of}</label><div id=\"ID_DOORMATIC_OPEN_OF{##}\"></div></div></help-topic><!-- Doormatic List of door which will be opened --><help-topic data-help=\"/widget/deviceFunctions/doormatic/doorlist\"><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_SELECTED_DOORS\"><label>{$widget_doormatic_doorlist}</label><div id=\"ID_DOORMATIC_LIST_CONTAINER{##}\" class=\"h-layout\"><div id=\"ID_DOORMATIC_LIST{##}\"></div><s class=\"s\"></s><div class=\"v-layout\"><div id=\"ID_DOORMATIC_BUTTON_ADD{##}\"></div><br style=\"margin: 0\"><div id=\"ID_DOORMATIC_BUTTON_DELETE{##}\"></div><div id=\"ID_DOORMATIC_CHOOSE_LIST{##}\"></div></div></div></div></help-topic></div>','/widget/deviceFunctions/building_automation_url':'<help-topic data-help=\"/widget/deviceFunctions/building_automation_url/url\"><div class=\"line\"><label>{$widget_building_automation_url_label}</label><div id=\"ID_URL{##}\"></div></div></help-topic>','/widget/deviceFunctions/videourl':'<div id=\"ID_TOAST{##}\"></div><div id=\"ID_OPTIONAL{##}\" class=\"optional\"/><!-- <div class=\"line\"><div id=\"ID_LABEL{##}\"></div></div>--><help-topic data-help=\"/widget/deviceFunctions/videourl/format\"><div class=\"line\"><label>{$widget_video_url_format}</label><div id=\"ID_FORMAT{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/videourl/frames\"><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_VIDEO_URL_FRAMES\"><label>{$widget_video_url_frames}</label><div id=\"ID_FRAMES{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/videourl/security\"><div class=\"line\"><label>{$widget_video_url_security}</label><div id=\"ID_SECURITY{##}\"></div></div></help-topic><!-- Basic Auth optional --><div class=\"optional-item\" data-id=\"ID_OPTIONAL_VIDEO_URL_BASIC_AUTH\"><help-topic data-help=\"/widget/deviceFunctions/videourl/participant\"><div class=\"line\"><label>{$widget_video_url_participant}</label><div id=\"ID_PARTICIPANT{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/videourl/password\"><div class=\"line\"><label>{$widget_video_url_password}</label><div id=\"ID_PASSWORD{##}\"></div><s class=\"s\"></s><div id=\"ID_PASSWORD_CHANGE{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/videourl/password2\"><div class=\"line optional-item\" data-id=\"ID_OPTIONAL_VIDEO_URL_PASSWORD_REPEAT\"><label>{$widget_video_url_password_repeat}</label><div id=\"ID_PASSWORD_REPEAT{##}\"></div></div></help-topic></div><help-topic data-help=\"/widget/deviceFunctions/videourl/url\"><div class=\"line\"><label>{$widget_video_url_url}</label><div id=\"ID_URL{##}\"></div></div></help-topic>','/widget/deviceFunctions/light':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"/><div class=\"line\"><div id=\"ID_LABEL{##}\" class=\"\"></div></div><help-topic data-help=\"/widget/deviceFunctions/light/output\"><div class=\"line\"><label>{$widget_light_switch_contact}</label><div id=\"ID_LIGHT_OUTPUT{##}\" class=\"\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/light/outputextern\"><div class=\"line optional-item\" data-id=\"ID_LIGHT_OUTPUT_EXTERN\"><label>{$widget_light_output}</label><div id=\"ID_LIGHT_OUTPUT_EXTERN{##}\" class=\"\"></div></div></help-topic><div class=\"optional-item\" data-id=\"ID_LIGHT_OUTPUT_LOCAL\"><help-topic data-help=\"/widget/deviceFunctions/light/outputlocal\"><div class=\"line\"><label>{$widget_light_output}</label><div id=\"ID_LIGHT_OUTPUT_LOCAL{##}\" class=\"\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/light/timeout\"><div class=\"line\"><label>{$widget_light_switch_timeout}</label><div id=\"ID_LIGHT_TIMEOUT{##}\" class=\"\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/light/options\"><div class=\"line\"><label>{$widget_light_options}</label><div id=\"ID_LIGHT_OPTIONS{##}\" class=\"\"></div></div></help-topic></div>','/widget/deviceFunctions/door':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"/><div class=\"line\"><div id=\"ID_LABEL{##}\" class=\"\"></div></div><help-topic data-help=\"/widget/deviceFunctions/door/output\"><div class=\"line\"><label>{$widget_door_switch_contact}</label><div id=\"ID_DOOR_OUTPUT{##}\" class=\"\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/door/outputextern\"><div class=\"line optional-item\" data-id=\"ID_DOOR_OUTPUT_EXTERN\"><label>{$widget_door_output}</label><div id=\"ID_DOOR_OUTPUT_EXTERN{##}\" class=\"\"></div></div></help-topic><div class=\"optional-item\" data-id=\"ID_DOOR_OUTPUT_LOCAL\"><help-topic data-help=\"/widget/deviceFunctions/door/outputlocal\"><div class=\"line\"><label>{$widget_door_output}</label><div id=\"ID_DOOR_OUTPUT_LOCAL{##}\" class=\"\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/door/timeout\"><div class=\"line\"><label>{$widget_door_switch_timeout}</label><div id=\"ID_DOOR_TIMEOUT{##}\" class=\"\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/door/options\"><div class=\"line\"><label>{$widget_door_options}</label><div id=\"ID_CHECK_DOOR_OPTIONS{##}\" class=\"\"></div></div></help-topic></div><help-topic data-help=\"/widget/deviceFunctions/door/cancelcall\"><div class=\"line\"><label>{$widget_door_canceling_calls}</label><div id=\"ID_DOOR_IS_CANCELING_CALL{##}\" class=\"\"></div></div></help-topic>','/widget/deviceFunctions/door2':'<div id=\"ID_OPTIONAL{##}\" class=\"optional\"/><div class=\"line\"><div id=\"ID_LABEL{##}\" class=\"\"></div></div><help-topic data-help=\"/widget/deviceFunctions/door/output\"><div class=\"line\"><label>{$widget_door_switch_contact}</label><div id=\"ID_DOOR2_OUTPUT{##}\" class=\"\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/door/outputextern\"><div class=\"line optional-item\" data-id=\"ID_DOOR2_OUTPUT_EXTERN\"><label>{$widget_door_output}</label><div id=\"ID_DOOR2_OUTPUT_EXTERN{##}\" class=\"\"></div></div></help-topic><div class=\"optional-item\" data-id=\"ID_DOOR2_OUTPUT_LOCAL\"><help-topic data-help=\"/widget/deviceFunctions/door/outputlocal\"><div class=\"line\"><label>{$widget_door_output}</label><div id=\"ID_DOOR2_OUTPUT_LOCAL{##}\" class=\"\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/door/timeout\"><div class=\"line\"><label>{$widget_door_switch_timeout}</label><div id=\"ID_DOOR2_TIMEOUT{##}\" class=\"\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/door/options\"><div class=\"line\"><label>{$widget_door2_options}</label><div id=\"ID_CHECK_DOOR2_OPTIONS{##}\" class=\"\"></div></div></help-topic></div><help-topic data-help=\"/widget/deviceFunctions/door/cancelcall\"><div class=\"line\"><label>{$widget_door_canceling_calls}</label><div id=\"ID_DOOR2_IS_CANCELING_CALL{##}\" class=\"\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/door2/usefor\"><div class=\"line\"><label>{$widget_door2_use_for}</label><div class=\"h-layout\"><div id=\"ID_DOOR2_USE_FOR_LIST{##}\" class=\"\"></div><s class=\"s\"></s><div class=\"v-layout\"><div id=\"ID_DOOR2_BUTTON_ADD{##}\"></div><br style=\"margin: 0\"><div id=\"ID_DOOR2_BUTTON_DELETE{##}\"></div><div id=\"ID_DOOR2_CHOOSE_LIST{##}\"></div></div></div></div></help-topic>','/widget/deviceFunctions/camera':'<div class=\"line\"><div id=\"ID_LABEL{##}\"></div></div>','/widget/deviceFunctions/callcontact':'<help-topic data-help=\"/widget/deviceFunctions/callcontact/contact\"><div class=\"line\"><label>{$widget_call_contact_contact}</label><div id=\"ID_LIST_CONTACT{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/callcontact/turnon\"><div class=\"line\"><label>{$widget_call_contact_turn_on}</label><div id=\"ID_LIST_TURN_ON{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/callcontact/turnoff\"><div class=\"line\"><label>{$widget_call_contact_turn_off}</label><div id=\"ID_LIST_TURN_OFF{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/callcontact/options\"><div class=\"line\"><label>{$widget_call_contact_options}</label><div id=\"ID_CHECK_CALL_CONTACT_OPTIONS{##}\"></div></div></help-topic>','/widget/deviceFunctions/videoprivacy':'<div class=\"line\"><span>{$widget_videoprivacy_text1}<br>{$widget_videoprivacy_text2}</span></div><div class=\"line\"><span></span></div>','/widget/deviceFunctions/deviceoutage':'<div class=\"line\"><span>{$widget_deviceoutage_text1}<br>{$widget_deviceoutage_text2}</span></div><div class=\"line\"><span></span></div>','/widget/deviceFunctions/onvif':'<help-topic data-help=\"/widget/deviceFunctions/onvif/profile\"><div class=\"line\"><label>{$widget_onvif_profile_label}</label><div id=\"ID_PROFILE_NAME{##}\"></div><s class=\"s\"></s><div id=\"ID_NAME_REFRESH{##}\"></div></div></help-topic><help-topic data-help=\"/widget/deviceFunctions/onvif/resolution\"><div class=\"line\"><label>{$widget_onvif_resolution_label}</label><div id=\"ID_RESOLUTION{##}\"></div></div></help-topic><!-- <div class=\"line\"><span>{$widget_videoprivacy_text1}<br>{$widget_videoprivacy_text2}</span></div>-->','/widget/deviceConfigGroup/devGroup':'','/widget/deviceConfigGroup/devGroup/hlayout':'<div class=\"h-layout\"><div class=\"h-layout-fix\"></div><div class=\"h-layout-stretch\"></div></div>','/widget/deviceConfigGroup/devGroup/toemode':'<div class=\"line\"><label>{{label}}</label><div class=\"control\"></div></div>','/widget/deviceConfigGroup/devGroup/line':'<div class=\"line\"><label>{{label}}</label><div class=\"descriptionline\"></div><s class=\"s\"></s><div class=\"button\"></div><span class=\"permission-check hidden \"><s class=\"s\"></s><i class=\"mdi mdi-security yellow-text\" title=\"{$permcheck_tooltip}\"></i></span></div>','/widget/deviceConfigGroup/devGroup/lineTile':'<div class=\"line\"><label>{{label}}</label><div class=\"descriptionline\"></div><s class=\"s\"></s><div class=\"button\"></div><s class=\"s\"></s><div class=\"button2\"></div><span class=\"permission-check hidden\" title=\"{$permcheck_tooltip}\"><s class=\"s\"></s><i class=\"mdi mdi-security yellow-text\"></i></span></div>','/widget/deviceConfigGroup/devGroup/add':'<div class=\"line\"><label>{$devkeygrp_button_add_label}</label><div class=\"button\"></div></div>','/widget/deviceConfigGroup/devFunction':'<div class=\"line\"><label>{$devfunc_label_function}</label><div id=\"ID_FUNCTIONS{##}\"></div></div><div class=\"line\"><label></label><div id=\"ID_EDIT{##}\"></div><s class=\"s\"></s><div id=\"ID_ON{##}\"></div><s class=\"s\"></s><div id=\"ID_OFF{##}\"></div></div><div class=\"line\"></div>','/widget/commandGroup/function':'<help-topic data-help=\"/widget/commandGroup/function\"><div class=\"line\"><label>{$devfunc_label_function}</label><div id=\"ID_FUNCTIONS{##}\"></div></div><div class=\"line\"><label></label><div id=\"ID_NEW{##}\"></div><s class=\"s\"></s><div id=\"ID_EDIT{##}\"></div><s class=\"s\"></s><div id=\"ID_DELETE{##}\"></div></div></help-topic><div class=\"line\"></div>','/widget/ruleConfig/ruleGenericUi':'<div id=\"ID_TOAST{##}\"></div><div id=\"ID_CONTROLS{##}\"></div>','/widget/ruleConfig/ruleGenericUi/label':'<div id=\"ID_LABEL{##}\"></div>','/widget/ruleConfig/ruleGenericUi/control':'<div class=\"line\"><label>{{label}}</label><div id=\"{{id}}{##}\"></div></div>','/widget/ruleConfig/ruleGenericUi/controlHelp':'<help-topic data-help=\"{{helpTopicId}}\"><div class=\"line\"><label>{{label}}</label><div id=\"{{id}}{##}\"></div></div></help-topic>','/widget/ruleConfig/ruleControlBase':'','/widget/ruleConfig/ruleControlBase/help':'<help-topic data-help=\"{{helpTopicId}}\"><div class=\"line\"><label>{{label}}</label><div id=\"ID_RULE_CONTROL{##}\"></div></div></help-topic>','/widget/ruleConfig/ruleControlBase/line':'<div class=\"line\"><label>{{label}}</label><div id=\"ID_RULE_CONTROL{##}\"></div></div>','/widget/ruleConfig/ruleControlStatepoint':'','/widget/ruleConfig/ruleControlCalltargetList':'','/widget/ruleConfig/ruleControlCalltarget':'','/widget/ruleConfig/ruleControlCalltarget/control':'<div><div id=\"ID_CALLTARGET_GRID{##}\"></div></div><br><div id=\"ID_CALLTARGET_ADD{##}\"></div><div id=\'ID_CALLTARGET_POPUP{##}\'></div><s class=\"s\"></s><div id=\"ID_CALLTARGET_DELETE{##}\"></div>','/widget/ruleConfig/ruleControlInteger':'','/widget/ruleConfig/ruleControlInteger/postlabel':'<div id=\"ID_RULE_EDIT{##}\"></div><s class=\"s\"></s><span>{{postLabel}}</span>' } );
$.extend( true, $.views._md, { } );
